from ..value_set import ValueSet

class BloodTransfusion(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group procedures of blood transfusions.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a blood transfusion.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Blood Transfusion"
    OID = "2.16.840.1.113762.1.4.1029.213"
    DEFINITION_VERSION = "20220127"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "30233H0",  # Transfusion of Autologous Whole Blood into Peripheral Vein, Percutaneous Approach
        "30233H1",  # Transfusion of Nonautologous Whole Blood into Peripheral Vein, Percutaneous Approach
        "30233K0",  # Transfusion of Autologous Frozen Plasma into Peripheral Vein, Percutaneous Approach
        "30233K1",  # Transfusion of Nonautologous Frozen Plasma into Peripheral Vein, Percutaneous Approach
        "30233L0",  # Transfusion of Autologous Fresh Plasma into Peripheral Vein, Percutaneous Approach
        "30233L1",  # Transfusion of Nonautologous Fresh Plasma into Peripheral Vein, Percutaneous Approach
        "30233M0",  # Transfusion of Autologous Plasma Cryoprecipitate into Peripheral Vein, Percutaneous Approach
        "30233M1",  # Transfusion of Nonautologous Plasma Cryoprecipitate into Peripheral Vein, Percutaneous Approach
        "30233N0",  # Transfusion of Autologous Red Blood Cells into Peripheral Vein, Percutaneous Approach
        "30233N1",  # Transfusion of Nonautologous Red Blood Cells into Peripheral Vein, Percutaneous Approach
        "30233P0",  # Transfusion of Autologous Frozen Red Cells into Peripheral Vein, Percutaneous Approach
        "30233P1",  # Transfusion of Nonautologous Frozen Red Cells into Peripheral Vein, Percutaneous Approach
        "30233R0",  # Transfusion of Autologous Platelets into Peripheral Vein, Percutaneous Approach
        "30233R1",  # Transfusion of Nonautologous Platelets into Peripheral Vein, Percutaneous Approach
        "30233T0",  # Transfusion of Autologous Fibrinogen into Peripheral Vein, Percutaneous Approach
        "30233T1",  # Transfusion of Nonautologous Fibrinogen into Peripheral Vein, Percutaneous Approach
        "30243H0",  # Transfusion of Autologous Whole Blood into Central Vein, Percutaneous Approach
        "30243H1",  # Transfusion of Nonautologous Whole Blood into Central Vein, Percutaneous Approach
        "30243K0",  # Transfusion of Autologous Frozen Plasma into Central Vein, Percutaneous Approach
        "30243K1",  # Transfusion of Nonautologous Frozen Plasma into Central Vein, Percutaneous Approach
        "30243L0",  # Transfusion of Autologous Fresh Plasma into Central Vein, Percutaneous Approach
        "30243L1",  # Transfusion of Nonautologous Fresh Plasma into Central Vein, Percutaneous Approach
        "30243M0",  # Transfusion of Autologous Plasma Cryoprecipitate into Central Vein, Percutaneous Approach
        "30243M1",  # Transfusion of Nonautologous Plasma Cryoprecipitate into Central Vein, Percutaneous Approach
        "30243N0",  # Transfusion of Autologous Red Blood Cells into Central Vein, Percutaneous Approach
        "30243N1",  # Transfusion of Nonautologous Red Blood Cells into Central Vein, Percutaneous Approach
        "30243P0",  # Transfusion of Autologous Frozen Red Cells into Central Vein, Percutaneous Approach
        "30243P1",  # Transfusion of Nonautologous Frozen Red Cells into Central Vein, Percutaneous Approach
        "30243R0",  # Transfusion of Autologous Platelets into Central Vein, Percutaneous Approach
        "30243R1",  # Transfusion of Nonautologous Platelets into Central Vein, Percutaneous Approach
        "30243T0",  # Transfusion of Autologous Fibrinogen into Central Vein, Percutaneous Approach
        "30243T1",  # Transfusion of Nonautologous Fibrinogen into Central Vein, Percutaneous Approach
    }

    SNOMEDCT = {
        "11397000",  # Autotransfusion of whole blood (procedure)
        "116795008",  # Transfusion of cryoprecipitate (procedure)
        "116861002",  # Transfusion of fresh frozen plasma (procedure)
        "116863004",  # Transfusion of red blood cells (procedure)
        "117078000",  # Transfusion of platelet concentrate (procedure)
        "12719002",  # Platelet transfusion (procedure)
        "13569004",  # Transfusion of plasma (procedure)
        "180206004",  # Intra-arterial blood transfusion (procedure)
        "180207008",  # Intravenous blood transfusion of packed cells (procedure)
        "180208003",  # Intravenous blood transfusion of platelets (procedure)
        "225284006",  # Transfusing whole blood under pressure (procedure)
        "288170000",  # Packed blood cell transfusion (procedure)
        "33389009",  # Transfusion of whole blood (procedure)
        "425513008",  # Transfusion of leucoreduced red blood cells (procedure)
        "426290002",  # Transfusion of washed red blood cells (procedure)
        "54790000",  # Transfusion of blood component (procedure)
        "71493000",  # Transfusion of packed red blood cells (procedure)
    }

class ConversionOfCardiacRhythm(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent a grouping of procedures of conversion of cardiac rhythm.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a conversion of cardiac rhythm.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Conversion of Cardiac Rhythm"
    OID = "2.16.840.1.113762.1.4.1029.357"
    DEFINITION_VERSION = "20210121"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "5A12012",  # Performance of Cardiac Output, Single, Manual
        "5A2204Z",  # Restoration of Cardiac Rhythm, Single
    }

    SNOMEDCT = {
        "250041003",  # Conversion of cardiac rhythm to sinus rhythm (procedure)
    }

class DeliveryProcedures(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group concepts of procedures for delivery.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure of delivery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Delivery Procedures"
    OID = "2.16.840.1.113762.1.4.1045.59"
    DEFINITION_VERSION = "20210611"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "10D00Z0",  # Extraction of Products of Conception, High, Open Approach
        "10D00Z1",  # Extraction of Products of Conception, Low, Open Approach
        "10D00Z2",  # Extraction of Products of Conception, Extraperitoneal, Open Approach
        "10D07Z3",  # Extraction of Products of Conception, Low Forceps, Via Natural or Artificial Opening
        "10D07Z4",  # Extraction of Products of Conception, Mid Forceps, Via Natural or Artificial Opening
        "10D07Z5",  # Extraction of Products of Conception, High Forceps, Via Natural or Artificial Opening
        "10D07Z6",  # Extraction of Products of Conception, Vacuum, Via Natural or Artificial Opening
        "10D07Z7",  # Extraction of Products of Conception, Internal Version, Via Natural or Artificial Opening
        "10D07Z8",  # Extraction of Products of Conception, Other, Via Natural or Artificial Opening
        "10E0XZZ",  # Delivery of Products of Conception, External Approach
    }

    SNOMEDCT = {
        "10745001",  # Delivery of transverse presentation (procedure)
        "11466000",  # Cesarean section (procedure)
        "1197219005",  # Elective lower segment cesarean section through inverted T shaped incision of uterus (procedure)
        "14119008",  # Braxton Hicks obstetrical version with extraction (procedure)
        "15413009",  # High forceps delivery with episiotomy (procedure)
        "16819009",  # Delivery of face presentation (procedure)
        "177141003",  # Elective cesarean section (procedure)
        "177142005",  # Elective upper segment cesarean section (procedure)
        "177143000",  # Elective lower segment cesarean section (procedure)
        "177152009",  # Breech extraction delivery with version (procedure)
        "177157003",  # Spontaneous breech delivery (procedure)
        "177158008",  # Assisted breech delivery (procedure)
        "177161009",  # Forceps cephalic delivery (procedure)
        "177162002",  # High forceps cephalic delivery with rotation (procedure)
        "177164001",  # Midforceps cephalic delivery with rotation (procedure)
        "177167008",  # Barton forceps cephalic delivery with rotation (procedure)
        "177168003",  # DeLee forceps cephalic delivery with rotation (procedure)
        "177170007",  # Piper forceps delivery (procedure)
        "177173009",  # High vacuum delivery (procedure)
        "177174003",  # Low vacuum delivery (procedure)
        "177175002",  # Vacuum delivery before full dilation of cervix (procedure)
        "177176001",  # Trial of vacuum delivery (procedure)
        "177179008",  # Cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177180006",  # Manipulative cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177181005",  # Non-manipulative cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177184002",  # Normal delivery procedure (procedure)
        "177185001",  # Water birth delivery (procedure)
        "17744000",  # Subtotal hysterectomy after cesarean delivery (procedure)
        "17860005",  # Low forceps delivery with episiotomy (procedure)
        "18625004",  # Low forceps delivery (procedure)
        "19390001",  # Partial breech delivery with forceps to aftercoming head (procedure)
        "199771001",  # Piper forceps delivery by application to aftercoming head (procedure)
        "22633006",  # Vaginal delivery, medical personnel present (procedure)
        "2321005",  # Delivery by Ritgen maneuver (procedure)
        "236973005",  # Delivery procedure (procedure)
        "236974004",  # Instrumental delivery (procedure)
        "236975003",  # Nonrotational forceps delivery (procedure)
        "236976002",  # Outlet forceps delivery (procedure)
        "236977006",  # Forceps delivery, face to pubes (procedure)
        "236978001",  # Forceps delivery to the aftercoming head (procedure)
        "236980007",  # Groin traction at breech delivery (procedure)
        "236981006",  # Lovset's maneuver (procedure)
        "236982004",  # Delivery of the after coming head (procedure)
        "236983009",  # Burns Marshall maneuver (procedure)
        "236984003",  # Mauriceau Smellie Veit maneuver (procedure)
        "236985002",  # Emergency lower segment cesarean section (procedure)
        "236986001",  # Emergency upper segment cesarean section (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "236988000",  # Elective cesarean hysterectomy (procedure)
        "236989008",  # Abdominal delivery for shoulder dystocia (procedure)
        "237311001",  # Breech delivery (procedure)
        "25296001",  # Delivery by Scanzoni maneuver (procedure)
        "25828002",  # Mid forceps delivery with episiotomy (procedure)
        "26313002",  # Delivery by vacuum extraction with episiotomy (procedure)
        "265639000",  # Midforceps delivery without rotation (procedure)
        "274130007",  # Emergency cesarean section (procedure)
        "275168001",  # Neville-Barnes forceps delivery (procedure)
        "275169009",  # Simpson's forceps delivery (procedure)
        "28542003",  # Wigand-Martin maneuver (procedure)
        "288039005",  # Internal cephalic version and extraction (procedure)
        "288042004",  # Hysterectomy and fetus removal (procedure)
        "288193006",  # Supervision - normal delivery (procedure)
        "28860009",  # Prague maneuver (procedure)
        "29613008",  # Delivery by double application of forceps (procedure)
        "302382009",  # Breech extraction with internal podalic version (procedure)
        "302383004",  # Forceps delivery (procedure)
        "30476003",  # Barton's forceps delivery (procedure)
        "306727001",  # Breech presentation, delivery, no version (procedure)
        "33807004",  # Internal and combined version with extraction (procedure)
        "359940006",  # Partial breech extraction (procedure)
        "359943008",  # Partial breech delivery (procedure)
        "384729004",  # Delivery of vertex presentation (procedure)
        "384730009",  # Delivery of cephalic presentation (procedure)
        "38479009",  # Frank breech delivery (procedure)
        "40219000",  # Delivery by Malstrom's extraction with episiotomy (procedure)
        "40704000",  # Wright's obstetrical version with extraction (procedure)
        "40792007",  # Kristeller maneuver (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "416055001",  # Total breech extraction (procedure)
        "417121007",  # Breech extraction (procedure)
        "4504004",  # Potter's obstetrical version with extraction (procedure)
        "450483001",  # Cesarean section through inverted T shaped incision of uterus (procedure)
        "450484007",  # Cesarean section through J shaped incision of uterus (procedure)
        "450798003",  # Delivery of occipitoposterior presentation (procedure)
        "45718005",  # Vaginal delivery with forceps including postpartum care (procedure)
        "48204000",  # Spontaneous unassisted delivery, medical personnel present (procedure)
        "54973000",  # Total breech delivery with forceps to aftercoming head (procedure)
        "5556001",  # Manually assisted spontaneous delivery (procedure)
        "57271003",  # Extraperitoneal cesarean section (procedure)
        "58705005",  # Bracht maneuver (procedure)
        "61586001",  # Delivery by vacuum extraction (procedure)
        "62508004",  # Mid forceps delivery (procedure)
        "65243006",  # Delivery by midwife (procedure)
        "69422002",  # Trial forceps delivery (procedure)
        "699999008",  # Obstetrical version with extraction (procedure)
        "700000006",  # Vaginal delivery of fetus (procedure)
        "709004006",  # Emergency lower segment cesarean section with inverted T incision (procedure)
        "71166009",  # Forceps delivery with rotation of fetal head (procedure)
        "72492007",  # Footling breech delivery (procedure)
        "734275002",  # Delivery by outlet vacuum extraction (procedure)
        "734276001",  # Delivery by mid-vacuum extraction (procedure)
        "736018001",  # Elective upper segment cesarean section with bilateral tubal ligation (procedure)
        "736020003",  # Emergency upper segment cesarean section with bilateral tubal ligation (procedure)
        "736026009",  # Elective lower segment cesarean section with bilateral tubal ligation (procedure)
        "736118004",  # Emergency lower segment cesarean section with bilateral tubal ligation (procedure)
        "75928003",  # Pinard maneuver (procedure)
        "788180009",  # Lower uterine segment cesarean section (procedure)
        "84195007",  # Classical cesarean section (procedure)
        "85403009",  # Delivery, medical personnel present (procedure)
        "89053004",  # Vaginal cesarean section (procedure)
        "89346004",  # Delivery by Kielland rotation (procedure)
        "89849000",  # High forceps delivery (procedure)
        "90438006",  # Delivery by Malstrom's extraction (procedure)
    }

class Hysterectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent a grouping of procedures of a hysterectomy.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a hysterectomy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hysterectomy"
    OID = "2.16.840.1.113762.1.4.1029.358"
    DEFINITION_VERSION = "20210121"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0UT90ZL",  # Resection of Uterus, Supracervical, Open Approach
        "0UT90ZZ",  # Resection of Uterus, Open Approach
        "0UT97ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening
        "0UT97ZZ",  # Resection of Uterus, Via Natural or Artificial Opening
    }

    SNOMEDCT = {
        "11050006",  # Closure of vesicouterine fistula with hysterectomy (procedure)
        "112918004",  # Vaginal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "116140006",  # Total hysterectomy (procedure)
        "116141005",  # Abdominal hysterectomy (procedure)
        "116142003",  # Radical hysterectomy (procedure)
        "116143008",  # Total abdominal hysterectomy (procedure)
        "116144002",  # Total abdominal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "1187630003",  # Subtotal hysterectomy via vaginal approach (procedure)
        "1303414001",  # Laparoscopic vaginal hysterectomy (procedure)
        "1304071003",  # Endoscopic vaginal natural orifice transluminal hysterectomy (procedure)
        "13254001",  # Abdominal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "176795006",  # Subtotal abdominal hysterectomy (procedure)
        "176873000",  # Transcervical resection endometrium (procedure)
        "17744000",  # Subtotal hysterectomy after cesarean delivery (procedure)
        "236886002",  # Hysterectomy (procedure)
        "236887006",  # Laparoscopic hysterectomy (procedure)
        "236888001",  # Laparoscopic total hysterectomy (procedure)
        "236889009",  # Laparoscopic supracervical hysterectomy (procedure)
        "236891001",  # Laparoscopic radical hysterectomy (procedure)
        "236897002",  # Endometrial resection (procedure)
        "236906003",  # Excision of rudimentary uterine horn (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "265056007",  # Vaginal hysterectomy (procedure)
        "27185000",  # Vaginal hysterectomy with partial colpectomy and repair of enterocele (procedure)
        "288042004",  # Hysterectomy and fetus removal (procedure)
        "288043009",  # Hysterectomy in pregnancy (procedure)
        "29529008",  # Supracervical hysterectomy with removal of both tubes and ovaries (procedure)
        "30160001",  # Vaginal hysterectomy with total colpectomy and repair of enterocele (procedure)
        "302190000",  # Abdominal hysterectomy and right salpingo-oophorectomy (procedure)
        "302191001",  # Abdominal hysterectomy and left salpingo-oophorectomy (procedure)
        "307771009",  # Radical abdominal hysterectomy (procedure)
        "309879006",  # Abdominal hysterocolpectomy (procedure)
        "309880009",  # Vaginal hysterocolpectomy (procedure)
        "31191000",  # Excision of septum of uterus (procedure)
        "35955002",  # Radical vaginal hysterectomy (procedure)
        "359971002",  # Ward-Mayo operation for vaginal hysterectomy (procedure)
        "359974005",  # Tuffier operation for vaginal hysterectomy (procedure)
        "359977003",  # Mayo operation for vaginal hysterectomy (procedure)
        "359980002",  # Vaginal panhysterectomy (procedure)
        "359983000",  # Heaney operation for vaginal hysterectomy (procedure)
        "387643005",  # Partial hysterectomy (procedure)
        "387644004",  # Supracervical hysterectomy (procedure)
        "387646002",  # Uterine fundectomy (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "413144006",  # Abdominal hysterectomy with conservation of ovaries (procedure)
        "413145007",  # Vaginal hysterectomy with conservation of ovaries (procedure)
        "414575003",  # Laparoscopic total abdominal hysterectomy and bilateral salpingo-oophorectomy (procedure)
        "431316002",  # Laparoscopic subtotal hysterectomy (procedure)
        "43791001",  # Vaginal hysterectomy with total colpectomy (procedure)
        "441820006",  # Laparoscopic-assisted vaginal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "446446002",  # Total abdominal hysterectomy and removal of vaginal cuff (procedure)
        "446679008",  # Total laparoscopic excision of uterus by abdominal approach (procedure)
        "447771005",  # Abdominal hysterectomy and excision of periuterine tissue (procedure)
        "448539002",  # Vaginal hysterectomy and excision of periuterine tissue (procedure)
        "49493009",  # Resection of cornua of bilateral fallopian tubes (procedure)
        "54130005",  # Hysteroscopy with resection of intrauterine septum (procedure)
        "54490004",  # Vaginal hysterectomy with repair of enterocele (procedure)
        "608805000",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele (procedure)
        "608806004",  # Laparoscopic assisted vaginal hysterectomy with repair of rectocele (procedure)
        "608807008",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "63516002",  # Vaginal hysterectomy with colpo-urethrocystopexy, Pereyra type (procedure)
        "699789005",  # Vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "708877008",  # Laparoscopic total hysterectomy using robotic assistance (procedure)
        "708878003",  # Laparoscopic radical hysterectomy using robotic assistance (procedure)
        "708985003",  # Laparoscopic vaginal hysterectomy using robotic assistance (procedure)
        "713157005",  # Laparoscopic resection of uterine cornua (procedure)
        "737099004",  # Abdominal hysterectomy with sacrocolpopexy using mesh (procedure)
        "739669004",  # Supracervical hysterectomy with left salpingo-oophorectomy (procedure)
        "739670003",  # Supracervical hysterectomy with right salpingo-oophorectomy (procedure)
        "739671004",  # Total hysterectomy with left oophorectomy (procedure)
        "739672006",  # Total hysterectomy with right oophorectomy (procedure)
        "739673001",  # Total hysterectomy with left salpingo-oophorectomy (procedure)
        "739674007",  # Total hysterectomy with right salpingo-oophorectomy (procedure)
        "740514001",  # Total hysterectomy with right salpingectomy (procedure)
        "740515000",  # Total hysterectomy with left salpingectomy (procedure)
        "75835007",  # Laparoscopic-assisted vaginal hysterectomy (procedure)
        "762625001",  # Vaginal hysterectomy and pelvic floor repair (procedure)
        "767610009",  # Total hysterectomy via vaginal approach (procedure)
        "767611008",  # Total abdominal hysterectomy using intrafascial technique (procedure)
        "767612001",  # Total hysterectomy via vaginal approach using intrafascial technique (procedure)
        "77902002",  # Vaginal hysterectomy with partial colpectomy (procedure)
        "860602007",  # Postpartum excision of uterus (procedure)
        "86477000",  # Total hysterectomy with removal of both tubes and ovaries (procedure)
    }

class SevereMaternalMorbidityProcedures(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group concepts of procedures for severe maternal morbidities.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a severe maternal morbidity.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Severe Maternal Morbidity Procedures"
    OID = "2.16.840.1.113762.1.4.1029.256"
    DEFINITION_VERSION = "20230218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0B110F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach
        "0B113F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach
        "0B114F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach
        "0UT90ZL",  # Resection of Uterus, Supracervical, Open Approach
        "0UT90ZZ",  # Resection of Uterus, Open Approach
        "0UT97ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening
        "0UT97ZZ",  # Resection of Uterus, Via Natural or Artificial Opening
        "5A12012",  # Performance of Cardiac Output, Single, Manual
        "5A1935Z",  # Respiratory Ventilation, Less than 24 Consecutive Hours
        "5A1945Z",  # Respiratory Ventilation, 24-96 Consecutive Hours
        "5A1955Z",  # Respiratory Ventilation, Greater than 96 Consecutive Hours
        "5A2204Z",  # Restoration of Cardiac Rhythm, Single
    }

    SNOMEDCT = {
        "11050006",  # Closure of vesicouterine fistula with hysterectomy (procedure)
        "112918004",  # Vaginal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "116140006",  # Total hysterectomy (procedure)
        "116141005",  # Abdominal hysterectomy (procedure)
        "116142003",  # Radical hysterectomy (procedure)
        "116143008",  # Total abdominal hysterectomy (procedure)
        "116144002",  # Total abdominal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "1186678006",  # Continuous positive airway pressure with an assured constant airway pressure adjunct (regime/therapy)
        "1187630003",  # Subtotal hysterectomy via vaginal approach (procedure)
        "1303414001",  # Laparoscopic vaginal hysterectomy (procedure)
        "1304071003",  # Endoscopic vaginal natural orifice transluminal hysterectomy (procedure)
        "13254001",  # Abdominal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "176795006",  # Subtotal abdominal hysterectomy (procedure)
        "176873000",  # Transcervical resection endometrium (procedure)
        "17744000",  # Subtotal hysterectomy after cesarean delivery (procedure)
        "236886002",  # Hysterectomy (procedure)
        "236887006",  # Laparoscopic hysterectomy (procedure)
        "236888001",  # Laparoscopic total hysterectomy (procedure)
        "236889009",  # Laparoscopic supracervical hysterectomy (procedure)
        "236891001",  # Laparoscopic radical hysterectomy (procedure)
        "236897002",  # Endometrial resection (procedure)
        "236906003",  # Excision of rudimentary uterine horn (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "250041003",  # Conversion of cardiac rhythm to sinus rhythm (procedure)
        "265056007",  # Vaginal hysterectomy (procedure)
        "27185000",  # Vaginal hysterectomy with partial colpectomy and repair of enterocele (procedure)
        "288042004",  # Hysterectomy and fetus removal (procedure)
        "288043009",  # Hysterectomy in pregnancy (procedure)
        "29529008",  # Supracervical hysterectomy with removal of both tubes and ovaries (procedure)
        "30160001",  # Vaginal hysterectomy with total colpectomy and repair of enterocele (procedure)
        "302190000",  # Abdominal hysterectomy and right salpingo-oophorectomy (procedure)
        "302191001",  # Abdominal hysterectomy and left salpingo-oophorectomy (procedure)
        "307771009",  # Radical abdominal hysterectomy (procedure)
        "309879006",  # Abdominal hysterocolpectomy (procedure)
        "309880009",  # Vaginal hysterocolpectomy (procedure)
        "31191000",  # Excision of septum of uterus (procedure)
        "35955002",  # Radical vaginal hysterectomy (procedure)
        "359971002",  # Ward-Mayo operation for vaginal hysterectomy (procedure)
        "359974005",  # Tuffier operation for vaginal hysterectomy (procedure)
        "359977003",  # Mayo operation for vaginal hysterectomy (procedure)
        "359980002",  # Vaginal panhysterectomy (procedure)
        "359983000",  # Heaney operation for vaginal hysterectomy (procedure)
        "387643005",  # Partial hysterectomy (procedure)
        "387644004",  # Supracervical hysterectomy (procedure)
        "387646002",  # Uterine fundectomy (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "413144006",  # Abdominal hysterectomy with conservation of ovaries (procedure)
        "413145007",  # Vaginal hysterectomy with conservation of ovaries (procedure)
        "414575003",  # Laparoscopic total abdominal hysterectomy and bilateral salpingo-oophorectomy (procedure)
        "431316002",  # Laparoscopic subtotal hysterectomy (procedure)
        "43791001",  # Vaginal hysterectomy with total colpectomy (procedure)
        "43988005",  # Temporary tracheostomy (procedure)
        "441820006",  # Laparoscopic-assisted vaginal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "446446002",  # Total abdominal hysterectomy and removal of vaginal cuff (procedure)
        "446679008",  # Total laparoscopic excision of uterus by abdominal approach (procedure)
        "447771005",  # Abdominal hysterectomy and excision of periuterine tissue (procedure)
        "448134000",  # Continuous positive airway pressure to nonventilated lung (regime/therapy)
        "448539002",  # Vaginal hysterectomy and excision of periuterine tissue (procedure)
        "47545007",  # Continuous positive airway pressure ventilation treatment (regime/therapy)
        "49493009",  # Resection of cornua of bilateral fallopian tubes (procedure)
        "54130005",  # Hysteroscopy with resection of intrauterine septum (procedure)
        "54490004",  # Vaginal hysterectomy with repair of enterocele (procedure)
        "608805000",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele (procedure)
        "608806004",  # Laparoscopic assisted vaginal hysterectomy with repair of rectocele (procedure)
        "608807008",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "63516002",  # Vaginal hysterectomy with colpo-urethrocystopexy, Pereyra type (procedure)
        "699789005",  # Vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "708877008",  # Laparoscopic total hysterectomy using robotic assistance (procedure)
        "708878003",  # Laparoscopic radical hysterectomy using robotic assistance (procedure)
        "708985003",  # Laparoscopic vaginal hysterectomy using robotic assistance (procedure)
        "713157005",  # Laparoscopic resection of uterine cornua (procedure)
        "737099004",  # Abdominal hysterectomy with sacrocolpopexy using mesh (procedure)
        "739669004",  # Supracervical hysterectomy with left salpingo-oophorectomy (procedure)
        "739670003",  # Supracervical hysterectomy with right salpingo-oophorectomy (procedure)
        "739671004",  # Total hysterectomy with left oophorectomy (procedure)
        "739672006",  # Total hysterectomy with right oophorectomy (procedure)
        "739673001",  # Total hysterectomy with left salpingo-oophorectomy (procedure)
        "739674007",  # Total hysterectomy with right salpingo-oophorectomy (procedure)
        "740514001",  # Total hysterectomy with right salpingectomy (procedure)
        "740515000",  # Total hysterectomy with left salpingectomy (procedure)
        "75835007",  # Laparoscopic-assisted vaginal hysterectomy (procedure)
        "762625001",  # Vaginal hysterectomy and pelvic floor repair (procedure)
        "767610009",  # Total hysterectomy via vaginal approach (procedure)
        "767611008",  # Total abdominal hysterectomy using intrafascial technique (procedure)
        "767612001",  # Total hysterectomy via vaginal approach using intrafascial technique (procedure)
        "77902002",  # Vaginal hysterectomy with partial colpectomy (procedure)
        "860602007",  # Postpartum excision of uterus (procedure)
        "86477000",  # Total hysterectomy with removal of both tubes and ovaries (procedure)
    }

class Tracheostomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent a grouping of procedures for tracheostomy.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a tracheostomy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Tracheostomy"
    OID = "2.16.840.1.113762.1.4.1029.359"
    DEFINITION_VERSION = "20210121"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0B110F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach
        "0B113F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach
        "0B114F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "43988005",  # Temporary tracheostomy (procedure)
    }

class Ventilation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent a grouping of procedures for ventilation.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures of a ventilation.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Ventilation"
    OID = "2.16.840.1.113762.1.4.1029.360"
    DEFINITION_VERSION = "20210121"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "5A1935Z",  # Respiratory Ventilation, Less than 24 Consecutive Hours
        "5A1945Z",  # Respiratory Ventilation, 24-96 Consecutive Hours
        "5A1955Z",  # Respiratory Ventilation, Greater than 96 Consecutive Hours
    }

    SNOMEDCT = {
        "1186678006",  # Continuous positive airway pressure with an assured constant airway pressure adjunct (regime/therapy)
        "448134000",  # Continuous positive airway pressure to nonventilated lung (regime/therapy)
        "47545007",  # Continuous positive airway pressure ventilation treatment (regime/therapy)
    }

class ApplicationOfGraduatedCompressionStockings(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts of the procedure for the application of graduated compression stockings (GCS) for venous thromboembolism (VTE) prophylaxis.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent the procedure of the application of a device used for venous thromboembolism (VTE) prophylaxis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Application of Graduated Compression Stockings"
    OID = "2.16.840.1.113762.1.4.1110.66"
    DEFINITION_VERSION = "20220212"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "229511001",  # Application of graduated compression garment (procedure)
    }

class ApplicationOfIntermittentPneumaticCompressionDevices(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts of the procedure for the application of Intermittent Pneumatic Compression Devices (IPC) for venous thromboembolism (VTE) prophylaxis.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent the procedure of the application of a device used for venous thromboembolism (VTE) prophylaxis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Application of Intermittent Pneumatic Compression Devices"
    OID = "2.16.840.1.113762.1.4.1110.65"
    DEFINITION_VERSION = "20220212"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "443448006",  # Application of intermittent pneumatic compression device (procedure)
    }

class ApplicationOfVenousFootPumps(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts of the procedure for the application of venous foot pump for venous thromboembolism (VTE) prophylaxis.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent the procedure of the application of a device used for venous thromboembolism (VTE) prophylaxis.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Application of Venous Foot Pumps"
    OID = "2.16.840.1.113762.1.4.1110.64"
    DEFINITION_VERSION = "20220212"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "424831005",  # Application of foot pump (procedure)
        "442410008",  # Application of venous foot pump (procedure)
    }

class GeneralSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for thoracic and abdominal general surgery procedures, except for cardiac procedures.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for thoracic and abdominal general surgery, except for cardiac procedures.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "General Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.255"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "008Q0ZZ",  # Division of Vagus Nerve, Open Approach
        "06L20ZZ",  # Occlusion of Gastric Vein, Open Approach
        "07JP0ZZ",  # Inspection of Spleen, Open Approach
        "07TP0ZZ",  # Resection of Spleen, Open Approach
        "0BDN0ZX",  # Extraction of Right Pleura, Open Approach, Diagnostic
        "0BDN0ZZ",  # Extraction of Right Pleura, Open Approach
        "0BDP0ZX",  # Extraction of Left Pleura, Open Approach, Diagnostic
        "0BDP0ZZ",  # Extraction of Left Pleura, Open Approach
        "0BPT00Z",  # Removal of Drainage Device from Diaphragm, Open Approach
        "0BPT02Z",  # Removal of Monitoring Device from Diaphragm, Open Approach
        "0BPT07Z",  # Removal of Autologous Tissue Substitute from Diaphragm, Open Approach
        "0BPT0JZ",  # Removal of Synthetic Substitute from Diaphragm, Open Approach
        "0BPT0KZ",  # Removal of Nonautologous Tissue Substitute from Diaphragm, Open Approach
        "0BPT0MZ",  # Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Open Approach
        "0BPT70Z",  # Removal of Drainage Device from Diaphragm, Via Natural or Artificial Opening
        "0BPT72Z",  # Removal of Monitoring Device from Diaphragm, Via Natural or Artificial Opening
        "0BPT77Z",  # Removal of Autologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening
        "0BPT7JZ",  # Removal of Synthetic Substitute from Diaphragm, Via Natural or Artificial Opening
        "0BPT7KZ",  # Removal of Nonautologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening
        "0BPT7MZ",  # Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Via Natural or Artificial Opening
        "0BPT80Z",  # Removal of Drainage Device from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BPT82Z",  # Removal of Monitoring Device from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BPT87Z",  # Removal of Autologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BPT8JZ",  # Removal of Synthetic Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BPT8KZ",  # Removal of Nonautologous Tissue Substitute from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BPT8MZ",  # Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BTC0ZZ",  # Resection of Right Upper Lung Lobe, Open Approach
        "0BTD0ZZ",  # Resection of Right Middle Lung Lobe, Open Approach
        "0BTF0ZZ",  # Resection of Right Lower Lung Lobe, Open Approach
        "0BTG0ZZ",  # Resection of Left Upper Lung Lobe, Open Approach
        "0BTH0ZZ",  # Resection of Lung Lingula, Open Approach
        "0BTJ0ZZ",  # Resection of Left Lower Lung Lobe, Open Approach
        "0BTK0ZZ",  # Resection of Right Lung, Open Approach
        "0BTL0ZZ",  # Resection of Left Lung, Open Approach
        "0BTM0ZZ",  # Resection of Bilateral Lungs, Open Approach
        "0BWT00Z",  # Revision of Drainage Device in Diaphragm, Open Approach
        "0BWT02Z",  # Revision of Monitoring Device in Diaphragm, Open Approach
        "0BWT07Z",  # Revision of Autologous Tissue Substitute in Diaphragm, Open Approach
        "0BWT0JZ",  # Revision of Synthetic Substitute in Diaphragm, Open Approach
        "0BWT0KZ",  # Revision of Nonautologous Tissue Substitute in Diaphragm, Open Approach
        "0BWT0MZ",  # Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Open Approach
        "0BWT70Z",  # Revision of Drainage Device in Diaphragm, Via Natural or Artificial Opening
        "0BWT72Z",  # Revision of Monitoring Device in Diaphragm, Via Natural or Artificial Opening
        "0BWT77Z",  # Revision of Autologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening
        "0BWT7JZ",  # Revision of Synthetic Substitute in Diaphragm, Via Natural or Artificial Opening
        "0BWT7KZ",  # Revision of Nonautologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening
        "0BWT7MZ",  # Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Via Natural or Artificial Opening
        "0BWT80Z",  # Revision of Drainage Device in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BWT82Z",  # Revision of Monitoring Device in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BWT87Z",  # Revision of Autologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BWT8JZ",  # Revision of Synthetic Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BWT8KZ",  # Revision of Nonautologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0BWT8MZ",  # Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Via Natural or Artificial Opening Endoscopic
        "0D11074",  # Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D11076",  # Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D11079",  # Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1107A",  # Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1107B",  # Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D110J4",  # Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D110J6",  # Bypass Upper Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D110J9",  # Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D110JA",  # Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D110JB",  # Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D110K4",  # Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D110K6",  # Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D110K9",  # Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D110KA",  # Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D110KB",  # Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D110Z4",  # Bypass Upper Esophagus to Cutaneous, Open Approach
        "0D110Z6",  # Bypass Upper Esophagus to Stomach, Open Approach
        "0D110Z9",  # Bypass Upper Esophagus to Duodenum, Open Approach
        "0D110ZA",  # Bypass Upper Esophagus to Jejunum, Open Approach
        "0D110ZB",  # Bypass Upper Esophagus to Ileum, Open Approach
        "0D11874",  # Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D11876",  # Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D11879",  # Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1187A",  # Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1187B",  # Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118J4",  # Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118J6",  # Bypass Upper Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118J9",  # Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118JA",  # Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118JB",  # Bypass Upper Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118K4",  # Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118K6",  # Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118K9",  # Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118KA",  # Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118KB",  # Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D118Z4",  # Bypass Upper Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D118Z6",  # Bypass Upper Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic
        "0D118Z9",  # Bypass Upper Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D118ZA",  # Bypass Upper Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D118ZB",  # Bypass Upper Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D12074",  # Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D12076",  # Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D12079",  # Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1207A",  # Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1207B",  # Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D120J4",  # Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D120J6",  # Bypass Middle Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D120J9",  # Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D120JA",  # Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D120JB",  # Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D120K4",  # Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D120K6",  # Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D120K9",  # Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D120KA",  # Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D120KB",  # Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D120Z4",  # Bypass Middle Esophagus to Cutaneous, Open Approach
        "0D120Z6",  # Bypass Middle Esophagus to Stomach, Open Approach
        "0D120Z9",  # Bypass Middle Esophagus to Duodenum, Open Approach
        "0D120ZA",  # Bypass Middle Esophagus to Jejunum, Open Approach
        "0D120ZB",  # Bypass Middle Esophagus to Ileum, Open Approach
        "0D12874",  # Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D12876",  # Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D12879",  # Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1287A",  # Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1287B",  # Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128J4",  # Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128J6",  # Bypass Middle Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128J9",  # Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128JA",  # Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128JB",  # Bypass Middle Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128K4",  # Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128K6",  # Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128K9",  # Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128KA",  # Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128KB",  # Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D128Z4",  # Bypass Middle Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D128Z6",  # Bypass Middle Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic
        "0D128Z9",  # Bypass Middle Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D128ZA",  # Bypass Middle Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D128ZB",  # Bypass Middle Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D13074",  # Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D13076",  # Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D13079",  # Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1307A",  # Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1307B",  # Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D130J4",  # Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D130J6",  # Bypass Lower Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D130J9",  # Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D130JA",  # Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D130JB",  # Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D130K4",  # Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D130K6",  # Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D130K9",  # Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D130KA",  # Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D130KB",  # Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D130Z4",  # Bypass Lower Esophagus to Cutaneous, Open Approach
        "0D130Z6",  # Bypass Lower Esophagus to Stomach, Open Approach
        "0D130Z9",  # Bypass Lower Esophagus to Duodenum, Open Approach
        "0D130ZA",  # Bypass Lower Esophagus to Jejunum, Open Approach
        "0D130ZB",  # Bypass Lower Esophagus to Ileum, Open Approach
        "0D13874",  # Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D13876",  # Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D13879",  # Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1387A",  # Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1387B",  # Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138J4",  # Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138J6",  # Bypass Lower Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138J9",  # Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138JA",  # Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138JB",  # Bypass Lower Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138K4",  # Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138K6",  # Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138K9",  # Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138KA",  # Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138KB",  # Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D138Z4",  # Bypass Lower Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D138Z6",  # Bypass Lower Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic
        "0D138Z9",  # Bypass Lower Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D138ZA",  # Bypass Lower Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D138ZB",  # Bypass Lower Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D15074",  # Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D15076",  # Bypass Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D15079",  # Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1507A",  # Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1507B",  # Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D150J4",  # Bypass Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D150J6",  # Bypass Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D150J9",  # Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D150JA",  # Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D150JB",  # Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D150K4",  # Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D150K6",  # Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D150K9",  # Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D150KA",  # Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D150KB",  # Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D150Z4",  # Bypass Esophagus to Cutaneous, Open Approach
        "0D150Z6",  # Bypass Esophagus to Stomach, Open Approach
        "0D150Z9",  # Bypass Esophagus to Duodenum, Open Approach
        "0D150ZA",  # Bypass Esophagus to Jejunum, Open Approach
        "0D150ZB",  # Bypass Esophagus to Ileum, Open Approach
        "0D15874",  # Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D15876",  # Bypass Esophagus to Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D15879",  # Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1587A",  # Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1587B",  # Bypass Esophagus to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158J4",  # Bypass Esophagus to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158J6",  # Bypass Esophagus to Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158J9",  # Bypass Esophagus to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158JA",  # Bypass Esophagus to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158JB",  # Bypass Esophagus to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158K4",  # Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158K6",  # Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158K9",  # Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158KA",  # Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158KB",  # Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D158Z4",  # Bypass Esophagus to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D158Z6",  # Bypass Esophagus to Stomach, Via Natural or Artificial Opening Endoscopic
        "0D158Z9",  # Bypass Esophagus to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D158ZA",  # Bypass Esophagus to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D158ZB",  # Bypass Esophagus to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D16074",  # Bypass Stomach to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D16079",  # Bypass Stomach to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1607A",  # Bypass Stomach to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1607B",  # Bypass Stomach to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1607L",  # Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D160J4",  # Bypass Stomach to Cutaneous with Synthetic Substitute, Open Approach
        "0D160J9",  # Bypass Stomach to Duodenum with Synthetic Substitute, Open Approach
        "0D160JA",  # Bypass Stomach to Jejunum with Synthetic Substitute, Open Approach
        "0D160JB",  # Bypass Stomach to Ileum with Synthetic Substitute, Open Approach
        "0D160JL",  # Bypass Stomach to Transverse Colon with Synthetic Substitute, Open Approach
        "0D160K4",  # Bypass Stomach to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D160K9",  # Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D160KA",  # Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D160KB",  # Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D160KL",  # Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D160Z4",  # Bypass Stomach to Cutaneous, Open Approach
        "0D160Z9",  # Bypass Stomach to Duodenum, Open Approach
        "0D160ZA",  # Bypass Stomach to Jejunum, Open Approach
        "0D160ZB",  # Bypass Stomach to Ileum, Open Approach
        "0D160ZL",  # Bypass Stomach to Transverse Colon, Open Approach
        "0D16874",  # Bypass Stomach to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D16879",  # Bypass Stomach to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1687A",  # Bypass Stomach to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1687B",  # Bypass Stomach to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1687L",  # Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168J9",  # Bypass Stomach to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168JA",  # Bypass Stomach to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168JB",  # Bypass Stomach to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168JL",  # Bypass Stomach to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168K9",  # Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168KA",  # Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168KB",  # Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168KL",  # Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D168Z9",  # Bypass Stomach to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D168ZA",  # Bypass Stomach to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D168ZB",  # Bypass Stomach to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D168ZL",  # Bypass Stomach to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D19074",  # Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D19079",  # Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1907A",  # Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1907B",  # Bypass Duodenum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1907L",  # Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D190J4",  # Bypass Duodenum to Cutaneous with Synthetic Substitute, Open Approach
        "0D190J9",  # Bypass Duodenum to Duodenum with Synthetic Substitute, Open Approach
        "0D190JA",  # Bypass Duodenum to Jejunum with Synthetic Substitute, Open Approach
        "0D190JB",  # Bypass Duodenum to Ileum with Synthetic Substitute, Open Approach
        "0D190JL",  # Bypass Duodenum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D190K4",  # Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D190K9",  # Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D190KA",  # Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D190KB",  # Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D190KL",  # Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D190Z4",  # Bypass Duodenum to Cutaneous, Open Approach
        "0D190Z9",  # Bypass Duodenum to Duodenum, Open Approach
        "0D190ZA",  # Bypass Duodenum to Jejunum, Open Approach
        "0D190ZB",  # Bypass Duodenum to Ileum, Open Approach
        "0D190ZL",  # Bypass Duodenum to Transverse Colon, Open Approach
        "0D19874",  # Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D19879",  # Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1987A",  # Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1987B",  # Bypass Duodenum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1987L",  # Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198J4",  # Bypass Duodenum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198J9",  # Bypass Duodenum to Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198JA",  # Bypass Duodenum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198JB",  # Bypass Duodenum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198JL",  # Bypass Duodenum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198K4",  # Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198K9",  # Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198KA",  # Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198KB",  # Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198KL",  # Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D198Z4",  # Bypass Duodenum to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D198Z9",  # Bypass Duodenum to Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D198ZA",  # Bypass Duodenum to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D198ZB",  # Bypass Duodenum to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D198ZL",  # Bypass Duodenum to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1A074",  # Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1A07A",  # Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1A07B",  # Bypass Jejunum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1A07H",  # Bypass Jejunum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1A07K",  # Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07L",  # Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07M",  # Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07N",  # Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07P",  # Bypass Jejunum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1A07Q",  # Bypass Jejunum to Anus with Autologous Tissue Substitute, Open Approach
        "0D1A0J4",  # Bypass Jejunum to Cutaneous with Synthetic Substitute, Open Approach
        "0D1A0JA",  # Bypass Jejunum to Jejunum with Synthetic Substitute, Open Approach
        "0D1A0JB",  # Bypass Jejunum to Ileum with Synthetic Substitute, Open Approach
        "0D1A0JH",  # Bypass Jejunum to Cecum with Synthetic Substitute, Open Approach
        "0D1A0JK",  # Bypass Jejunum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1A0JL",  # Bypass Jejunum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1A0JM",  # Bypass Jejunum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1A0JN",  # Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1A0JP",  # Bypass Jejunum to Rectum with Synthetic Substitute, Open Approach
        "0D1A0JQ",  # Bypass Jejunum to Anus with Synthetic Substitute, Open Approach
        "0D1A0K4",  # Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KA",  # Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KB",  # Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KH",  # Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KK",  # Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KL",  # Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KM",  # Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KN",  # Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KP",  # Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KQ",  # Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Open Approach
        "0D1A0Z4",  # Bypass Jejunum to Cutaneous, Open Approach
        "0D1A0ZA",  # Bypass Jejunum to Jejunum, Open Approach
        "0D1A0ZB",  # Bypass Jejunum to Ileum, Open Approach
        "0D1A0ZH",  # Bypass Jejunum to Cecum, Open Approach
        "0D1A0ZK",  # Bypass Jejunum to Ascending Colon, Open Approach
        "0D1A0ZL",  # Bypass Jejunum to Transverse Colon, Open Approach
        "0D1A0ZM",  # Bypass Jejunum to Descending Colon, Open Approach
        "0D1A0ZN",  # Bypass Jejunum to Sigmoid Colon, Open Approach
        "0D1A0ZP",  # Bypass Jejunum to Rectum, Open Approach
        "0D1A0ZQ",  # Bypass Jejunum to Anus, Open Approach
        "0D1A874",  # Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87A",  # Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87B",  # Bypass Jejunum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87H",  # Bypass Jejunum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87K",  # Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87L",  # Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87M",  # Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87N",  # Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87P",  # Bypass Jejunum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A87Q",  # Bypass Jejunum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8J4",  # Bypass Jejunum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JA",  # Bypass Jejunum to Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JB",  # Bypass Jejunum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JH",  # Bypass Jejunum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JK",  # Bypass Jejunum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JL",  # Bypass Jejunum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JM",  # Bypass Jejunum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JN",  # Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JP",  # Bypass Jejunum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8JQ",  # Bypass Jejunum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8K4",  # Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KA",  # Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KB",  # Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KH",  # Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KK",  # Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KL",  # Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KM",  # Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KN",  # Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KP",  # Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8KQ",  # Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1A8Z4",  # Bypass Jejunum to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZA",  # Bypass Jejunum to Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZB",  # Bypass Jejunum to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZH",  # Bypass Jejunum to Cecum, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZK",  # Bypass Jejunum to Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZL",  # Bypass Jejunum to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZM",  # Bypass Jejunum to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZN",  # Bypass Jejunum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZP",  # Bypass Jejunum to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1A8ZQ",  # Bypass Jejunum to Anus, Via Natural or Artificial Opening Endoscopic
        "0D1B074",  # Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1B07B",  # Bypass Ileum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1B07H",  # Bypass Ileum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1B07K",  # Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07L",  # Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07M",  # Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07N",  # Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07P",  # Bypass Ileum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1B07Q",  # Bypass Ileum to Anus with Autologous Tissue Substitute, Open Approach
        "0D1B0J4",  # Bypass Ileum to Cutaneous with Synthetic Substitute, Open Approach
        "0D1B0JB",  # Bypass Ileum to Ileum with Synthetic Substitute, Open Approach
        "0D1B0JH",  # Bypass Ileum to Cecum with Synthetic Substitute, Open Approach
        "0D1B0JK",  # Bypass Ileum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1B0JL",  # Bypass Ileum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1B0JM",  # Bypass Ileum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1B0JN",  # Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1B0JP",  # Bypass Ileum to Rectum with Synthetic Substitute, Open Approach
        "0D1B0JQ",  # Bypass Ileum to Anus with Synthetic Substitute, Open Approach
        "0D1B0K4",  # Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KB",  # Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KH",  # Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KK",  # Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KL",  # Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KM",  # Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KN",  # Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KP",  # Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KQ",  # Bypass Ileum to Anus with Nonautologous Tissue Substitute, Open Approach
        "0D1B0Z4",  # Bypass Ileum to Cutaneous, Open Approach
        "0D1B0ZB",  # Bypass Ileum to Ileum, Open Approach
        "0D1B0ZH",  # Bypass Ileum to Cecum, Open Approach
        "0D1B0ZK",  # Bypass Ileum to Ascending Colon, Open Approach
        "0D1B0ZL",  # Bypass Ileum to Transverse Colon, Open Approach
        "0D1B0ZM",  # Bypass Ileum to Descending Colon, Open Approach
        "0D1B0ZN",  # Bypass Ileum to Sigmoid Colon, Open Approach
        "0D1B0ZP",  # Bypass Ileum to Rectum, Open Approach
        "0D1B0ZQ",  # Bypass Ileum to Anus, Open Approach
        "0D1B874",  # Bypass Ileum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87B",  # Bypass Ileum to Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87H",  # Bypass Ileum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87K",  # Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87L",  # Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87M",  # Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87N",  # Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87P",  # Bypass Ileum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B87Q",  # Bypass Ileum to Anus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8J4",  # Bypass Ileum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JB",  # Bypass Ileum to Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JH",  # Bypass Ileum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JK",  # Bypass Ileum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JL",  # Bypass Ileum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JM",  # Bypass Ileum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JN",  # Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JP",  # Bypass Ileum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8JQ",  # Bypass Ileum to Anus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8K4",  # Bypass Ileum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KB",  # Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KH",  # Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KK",  # Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KL",  # Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KM",  # Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KN",  # Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KP",  # Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8KQ",  # Bypass Ileum to Anus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1B8Z4",  # Bypass Ileum to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZB",  # Bypass Ileum to Ileum, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZH",  # Bypass Ileum to Cecum, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZK",  # Bypass Ileum to Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZL",  # Bypass Ileum to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZM",  # Bypass Ileum to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZN",  # Bypass Ileum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZP",  # Bypass Ileum to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1B8ZQ",  # Bypass Ileum to Anus, Via Natural or Artificial Opening Endoscopic
        "0D1H074",  # Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1H07H",  # Bypass Cecum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1H07K",  # Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07L",  # Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07M",  # Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07N",  # Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07P",  # Bypass Cecum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1H0J4",  # Bypass Cecum to Cutaneous with Synthetic Substitute, Open Approach
        "0D1H0JH",  # Bypass Cecum to Cecum with Synthetic Substitute, Open Approach
        "0D1H0JK",  # Bypass Cecum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1H0JL",  # Bypass Cecum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1H0JM",  # Bypass Cecum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1H0JN",  # Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1H0JP",  # Bypass Cecum to Rectum with Synthetic Substitute, Open Approach
        "0D1H0K4",  # Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KH",  # Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KK",  # Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KL",  # Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KM",  # Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KN",  # Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KP",  # Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1H0Z4",  # Bypass Cecum to Cutaneous, Open Approach
        "0D1H0ZH",  # Bypass Cecum to Cecum, Open Approach
        "0D1H0ZK",  # Bypass Cecum to Ascending Colon, Open Approach
        "0D1H0ZL",  # Bypass Cecum to Transverse Colon, Open Approach
        "0D1H0ZM",  # Bypass Cecum to Descending Colon, Open Approach
        "0D1H0ZN",  # Bypass Cecum to Sigmoid Colon, Open Approach
        "0D1H0ZP",  # Bypass Cecum to Rectum, Open Approach
        "0D1H874",  # Bypass Cecum to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87H",  # Bypass Cecum to Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87K",  # Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87L",  # Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87M",  # Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87N",  # Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H87P",  # Bypass Cecum to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8J4",  # Bypass Cecum to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JH",  # Bypass Cecum to Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JK",  # Bypass Cecum to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JL",  # Bypass Cecum to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JM",  # Bypass Cecum to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JN",  # Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8JP",  # Bypass Cecum to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8K4",  # Bypass Cecum to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KH",  # Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KK",  # Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KL",  # Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KM",  # Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KN",  # Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8KP",  # Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1H8Z4",  # Bypass Cecum to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZH",  # Bypass Cecum to Cecum, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZK",  # Bypass Cecum to Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZL",  # Bypass Cecum to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZM",  # Bypass Cecum to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZN",  # Bypass Cecum to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1H8ZP",  # Bypass Cecum to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1K074",  # Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1K07K",  # Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07L",  # Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07M",  # Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07N",  # Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07P",  # Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1K0J4",  # Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Open Approach
        "0D1K0JK",  # Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1K0JL",  # Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1K0JM",  # Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1K0JN",  # Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1K0JP",  # Bypass Ascending Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1K0K4",  # Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KK",  # Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KL",  # Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KM",  # Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KN",  # Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KP",  # Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1K0Z4",  # Bypass Ascending Colon to Cutaneous, Open Approach
        "0D1K0ZK",  # Bypass Ascending Colon to Ascending Colon, Open Approach
        "0D1K0ZL",  # Bypass Ascending Colon to Transverse Colon, Open Approach
        "0D1K0ZM",  # Bypass Ascending Colon to Descending Colon, Open Approach
        "0D1K0ZN",  # Bypass Ascending Colon to Sigmoid Colon, Open Approach
        "0D1K0ZP",  # Bypass Ascending Colon to Rectum, Open Approach
        "0D1K874",  # Bypass Ascending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K87K",  # Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K87L",  # Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K87M",  # Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K87N",  # Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K87P",  # Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8J4",  # Bypass Ascending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8JK",  # Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8JL",  # Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8JM",  # Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8JN",  # Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8JP",  # Bypass Ascending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8K4",  # Bypass Ascending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8KK",  # Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8KL",  # Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8KM",  # Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8KN",  # Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8KP",  # Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1K8Z4",  # Bypass Ascending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1K8ZK",  # Bypass Ascending Colon to Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1K8ZL",  # Bypass Ascending Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1K8ZM",  # Bypass Ascending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1K8ZN",  # Bypass Ascending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1K8ZP",  # Bypass Ascending Colon to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1L074",  # Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1L07L",  # Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07M",  # Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07N",  # Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07P",  # Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1L0J4",  # Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Open Approach
        "0D1L0JL",  # Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1L0JM",  # Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1L0JN",  # Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1L0JP",  # Bypass Transverse Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1L0K4",  # Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KL",  # Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KM",  # Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KN",  # Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KP",  # Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1L0Z4",  # Bypass Transverse Colon to Cutaneous, Open Approach
        "0D1L0ZL",  # Bypass Transverse Colon to Transverse Colon, Open Approach
        "0D1L0ZM",  # Bypass Transverse Colon to Descending Colon, Open Approach
        "0D1L0ZN",  # Bypass Transverse Colon to Sigmoid Colon, Open Approach
        "0D1L0ZP",  # Bypass Transverse Colon to Rectum, Open Approach
        "0D1L874",  # Bypass Transverse Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L87L",  # Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L87M",  # Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L87N",  # Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L87P",  # Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8J4",  # Bypass Transverse Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8JL",  # Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8JM",  # Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8JN",  # Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8JP",  # Bypass Transverse Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8K4",  # Bypass Transverse Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8KL",  # Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8KM",  # Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8KN",  # Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8KP",  # Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1L8Z4",  # Bypass Transverse Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1L8ZL",  # Bypass Transverse Colon to Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D1L8ZM",  # Bypass Transverse Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1L8ZN",  # Bypass Transverse Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1L8ZP",  # Bypass Transverse Colon to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1M074",  # Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1M07M",  # Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1M07N",  # Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1M07P",  # Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1M0J4",  # Bypass Descending Colon to Cutaneous with Synthetic Substitute, Open Approach
        "0D1M0JM",  # Bypass Descending Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1M0JN",  # Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1M0JP",  # Bypass Descending Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1M0K4",  # Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1M0KM",  # Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1M0KN",  # Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1M0KP",  # Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1M0Z4",  # Bypass Descending Colon to Cutaneous, Open Approach
        "0D1M0ZM",  # Bypass Descending Colon to Descending Colon, Open Approach
        "0D1M0ZN",  # Bypass Descending Colon to Sigmoid Colon, Open Approach
        "0D1M0ZP",  # Bypass Descending Colon to Rectum, Open Approach
        "0D1M874",  # Bypass Descending Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M87M",  # Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M87N",  # Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M87P",  # Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8J4",  # Bypass Descending Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8JM",  # Bypass Descending Colon to Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8JN",  # Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8JP",  # Bypass Descending Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8K4",  # Bypass Descending Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8KM",  # Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8KN",  # Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8KP",  # Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1M8Z4",  # Bypass Descending Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1M8ZM",  # Bypass Descending Colon to Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D1M8ZN",  # Bypass Descending Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1M8ZP",  # Bypass Descending Colon to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D1N074",  # Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1N07N",  # Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1N07P",  # Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1N0J4",  # Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Open Approach
        "0D1N0JN",  # Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1N0JP",  # Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1N0K4",  # Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1N0KN",  # Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1N0KP",  # Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1N0Z4",  # Bypass Sigmoid Colon to Cutaneous, Open Approach
        "0D1N0ZN",  # Bypass Sigmoid Colon to Sigmoid Colon, Open Approach
        "0D1N0ZP",  # Bypass Sigmoid Colon to Rectum, Open Approach
        "0D1N874",  # Bypass Sigmoid Colon to Cutaneous with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N87N",  # Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N87P",  # Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8J4",  # Bypass Sigmoid Colon to Cutaneous with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8JN",  # Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8JP",  # Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8K4",  # Bypass Sigmoid Colon to Cutaneous with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8KN",  # Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8KP",  # Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0D1N8Z4",  # Bypass Sigmoid Colon to Cutaneous, Via Natural or Artificial Opening Endoscopic
        "0D1N8ZN",  # Bypass Sigmoid Colon to Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D1N8ZP",  # Bypass Sigmoid Colon to Rectum, Via Natural or Artificial Opening Endoscopic
        "0D590ZZ",  # Destruction of Duodenum, Open Approach
        "0D597ZZ",  # Destruction of Duodenum, Via Natural or Artificial Opening
        "0D5E0ZZ",  # Destruction of Large Intestine, Open Approach
        "0D5E7ZZ",  # Destruction of Large Intestine, Via Natural or Artificial Opening
        "0D5F0ZZ",  # Destruction of Right Large Intestine, Open Approach
        "0D5F7ZZ",  # Destruction of Right Large Intestine, Via Natural or Artificial Opening
        "0D5G0ZZ",  # Destruction of Left Large Intestine, Open Approach
        "0D5G7ZZ",  # Destruction of Left Large Intestine, Via Natural or Artificial Opening
        "0D5H0ZZ",  # Destruction of Cecum, Open Approach
        "0D5H7ZZ",  # Destruction of Cecum, Via Natural or Artificial Opening
        "0D5K0ZZ",  # Destruction of Ascending Colon, Open Approach
        "0D5K7ZZ",  # Destruction of Ascending Colon, Via Natural or Artificial Opening
        "0D5L0ZZ",  # Destruction of Transverse Colon, Open Approach
        "0D5L7ZZ",  # Destruction of Transverse Colon, Via Natural or Artificial Opening
        "0D5M0ZZ",  # Destruction of Descending Colon, Open Approach
        "0D5M7ZZ",  # Destruction of Descending Colon, Via Natural or Artificial Opening
        "0D5N0ZZ",  # Destruction of Sigmoid Colon, Open Approach
        "0D5N7ZZ",  # Destruction of Sigmoid Colon, Via Natural or Artificial Opening
        "0D710DZ",  # Dilation of Upper Esophagus with Intraluminal Device, Open Approach
        "0D710ZZ",  # Dilation of Upper Esophagus, Open Approach
        "0D720DZ",  # Dilation of Middle Esophagus with Intraluminal Device, Open Approach
        "0D720ZZ",  # Dilation of Middle Esophagus, Open Approach
        "0D730DZ",  # Dilation of Lower Esophagus with Intraluminal Device, Open Approach
        "0D730ZZ",  # Dilation of Lower Esophagus, Open Approach
        "0D740DZ",  # Dilation of Esophagogastric Junction with Intraluminal Device, Open Approach
        "0D740ZZ",  # Dilation of Esophagogastric Junction, Open Approach
        "0D750DZ",  # Dilation of Esophagus with Intraluminal Device, Open Approach
        "0D750ZZ",  # Dilation of Esophagus, Open Approach
        "0D760DZ",  # Dilation of Stomach with Intraluminal Device, Open Approach
        "0D760ZZ",  # Dilation of Stomach, Open Approach
        "0D767DZ",  # Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening
        "0D767ZZ",  # Dilation of Stomach, Via Natural or Artificial Opening
        "0D768DZ",  # Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0D768ZZ",  # Dilation of Stomach, Via Natural or Artificial Opening Endoscopic
        "0D770DZ",  # Dilation of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0D770ZZ",  # Dilation of Stomach, Pylorus, Open Approach
        "0D777DZ",  # Dilation of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0D777ZZ",  # Dilation of Stomach, Pylorus, Via Natural or Artificial Opening
        "0D840ZZ",  # Division of Esophagogastric Junction, Open Approach
        "0D847ZZ",  # Division of Esophagogastric Junction, Via Natural or Artificial Opening
        "0D848ZZ",  # Division of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0D9100Z",  # Drainage of Upper Esophagus with Drainage Device, Open Approach
        "0D910ZZ",  # Drainage of Upper Esophagus, Open Approach
        "0D9170Z",  # Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening
        "0D917ZZ",  # Drainage of Upper Esophagus, Via Natural or Artificial Opening
        "0D9180Z",  # Drainage of Upper Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D918ZZ",  # Drainage of Upper Esophagus, Via Natural or Artificial Opening Endoscopic
        "0D9200Z",  # Drainage of Middle Esophagus with Drainage Device, Open Approach
        "0D920ZZ",  # Drainage of Middle Esophagus, Open Approach
        "0D9270Z",  # Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening
        "0D927ZZ",  # Drainage of Middle Esophagus, Via Natural or Artificial Opening
        "0D9280Z",  # Drainage of Middle Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D928ZZ",  # Drainage of Middle Esophagus, Via Natural or Artificial Opening Endoscopic
        "0D9300Z",  # Drainage of Lower Esophagus with Drainage Device, Open Approach
        "0D930ZZ",  # Drainage of Lower Esophagus, Open Approach
        "0D9370Z",  # Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening
        "0D937ZZ",  # Drainage of Lower Esophagus, Via Natural or Artificial Opening
        "0D9380Z",  # Drainage of Lower Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D938ZZ",  # Drainage of Lower Esophagus, Via Natural or Artificial Opening Endoscopic
        "0D9400Z",  # Drainage of Esophagogastric Junction with Drainage Device, Open Approach
        "0D940ZZ",  # Drainage of Esophagogastric Junction, Open Approach
        "0D9470Z",  # Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening
        "0D947ZZ",  # Drainage of Esophagogastric Junction, Via Natural or Artificial Opening
        "0D9480Z",  # Drainage of Esophagogastric Junction with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D948ZZ",  # Drainage of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0D9500Z",  # Drainage of Esophagus with Drainage Device, Open Approach
        "0D950ZZ",  # Drainage of Esophagus, Open Approach
        "0D9570Z",  # Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening
        "0D957ZZ",  # Drainage of Esophagus, Via Natural or Artificial Opening
        "0D9580Z",  # Drainage of Esophagus with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D958ZZ",  # Drainage of Esophagus, Via Natural or Artificial Opening Endoscopic
        "0D9600Z",  # Drainage of Stomach with Drainage Device, Open Approach
        "0D960ZZ",  # Drainage of Stomach, Open Approach
        "0D967ZZ",  # Drainage of Stomach, Via Natural or Artificial Opening
        "0D968ZZ",  # Drainage of Stomach, Via Natural or Artificial Opening Endoscopic
        "0D9700Z",  # Drainage of Stomach, Pylorus with Drainage Device, Open Approach
        "0D970ZZ",  # Drainage of Stomach, Pylorus, Open Approach
        "0D977ZZ",  # Drainage of Stomach, Pylorus, Via Natural or Artificial Opening
        "0D978ZZ",  # Drainage of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0D9800Z",  # Drainage of Small Intestine with Drainage Device, Open Approach
        "0D980ZZ",  # Drainage of Small Intestine, Open Approach
        "0D987ZZ",  # Drainage of Small Intestine, Via Natural or Artificial Opening
        "0D988ZZ",  # Drainage of Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0D9900Z",  # Drainage of Duodenum with Drainage Device, Open Approach
        "0D990ZZ",  # Drainage of Duodenum, Open Approach
        "0D997ZZ",  # Drainage of Duodenum, Via Natural or Artificial Opening
        "0D998ZZ",  # Drainage of Duodenum, Via Natural or Artificial Opening Endoscopic
        "0D9A00Z",  # Drainage of Jejunum with Drainage Device, Open Approach
        "0D9A0ZZ",  # Drainage of Jejunum, Open Approach
        "0D9A7ZZ",  # Drainage of Jejunum, Via Natural or Artificial Opening
        "0D9A8ZZ",  # Drainage of Jejunum, Via Natural or Artificial Opening Endoscopic
        "0D9B00Z",  # Drainage of Ileum with Drainage Device, Open Approach
        "0D9B0ZZ",  # Drainage of Ileum, Open Approach
        "0D9B7ZZ",  # Drainage of Ileum, Via Natural or Artificial Opening
        "0D9B8ZZ",  # Drainage of Ileum, Via Natural or Artificial Opening Endoscopic
        "0D9C00Z",  # Drainage of Ileocecal Valve with Drainage Device, Open Approach
        "0D9C0ZZ",  # Drainage of Ileocecal Valve, Open Approach
        "0D9C70Z",  # Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening
        "0D9C7ZZ",  # Drainage of Ileocecal Valve, Via Natural or Artificial Opening
        "0D9C80Z",  # Drainage of Ileocecal Valve with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D9C8ZZ",  # Drainage of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0D9E00Z",  # Drainage of Large Intestine with Drainage Device, Open Approach
        "0D9E0ZZ",  # Drainage of Large Intestine, Open Approach
        "0D9E7ZZ",  # Drainage of Large Intestine, Via Natural or Artificial Opening
        "0D9E8ZZ",  # Drainage of Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0D9F00Z",  # Drainage of Right Large Intestine with Drainage Device, Open Approach
        "0D9F0ZZ",  # Drainage of Right Large Intestine, Open Approach
        "0D9F70Z",  # Drainage of Right Large Intestine with Drainage Device, Via Natural or Artificial Opening
        "0D9F7ZZ",  # Drainage of Right Large Intestine, Via Natural or Artificial Opening
        "0D9F8ZZ",  # Drainage of Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0D9G00Z",  # Drainage of Left Large Intestine with Drainage Device, Open Approach
        "0D9G0ZZ",  # Drainage of Left Large Intestine, Open Approach
        "0D9G7ZZ",  # Drainage of Left Large Intestine, Via Natural or Artificial Opening
        "0D9G8ZZ",  # Drainage of Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0D9H00Z",  # Drainage of Cecum with Drainage Device, Open Approach
        "0D9H0ZZ",  # Drainage of Cecum, Open Approach
        "0D9H7ZZ",  # Drainage of Cecum, Via Natural or Artificial Opening
        "0D9H8ZZ",  # Drainage of Cecum, Via Natural or Artificial Opening Endoscopic
        "0D9J00Z",  # Drainage of Appendix with Drainage Device, Open Approach
        "0D9J0ZZ",  # Drainage of Appendix, Open Approach
        "0D9J70Z",  # Drainage of Appendix with Drainage Device, Via Natural or Artificial Opening
        "0D9J7ZZ",  # Drainage of Appendix, Via Natural or Artificial Opening
        "0D9J80Z",  # Drainage of Appendix with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0D9J8ZZ",  # Drainage of Appendix, Via Natural or Artificial Opening Endoscopic
        "0D9K00Z",  # Drainage of Ascending Colon with Drainage Device, Open Approach
        "0D9K0ZZ",  # Drainage of Ascending Colon, Open Approach
        "0D9K7ZZ",  # Drainage of Ascending Colon, Via Natural or Artificial Opening
        "0D9K8ZZ",  # Drainage of Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0D9L00Z",  # Drainage of Transverse Colon with Drainage Device, Open Approach
        "0D9L0ZZ",  # Drainage of Transverse Colon, Open Approach
        "0D9L7ZZ",  # Drainage of Transverse Colon, Via Natural or Artificial Opening
        "0D9L8ZZ",  # Drainage of Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0D9M00Z",  # Drainage of Descending Colon with Drainage Device, Open Approach
        "0D9M0ZZ",  # Drainage of Descending Colon, Open Approach
        "0D9M7ZZ",  # Drainage of Descending Colon, Via Natural or Artificial Opening
        "0D9M8ZZ",  # Drainage of Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0D9N00Z",  # Drainage of Sigmoid Colon with Drainage Device, Open Approach
        "0D9N0ZZ",  # Drainage of Sigmoid Colon, Open Approach
        "0D9N7ZZ",  # Drainage of Sigmoid Colon, Via Natural or Artificial Opening
        "0D9N8ZZ",  # Drainage of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0D9P00Z",  # Drainage of Rectum with Drainage Device, Open Approach
        "0D9P0ZZ",  # Drainage of Rectum, Open Approach
        "0D9P7ZZ",  # Drainage of Rectum, Via Natural or Artificial Opening
        "0D9P8ZZ",  # Drainage of Rectum, Via Natural or Artificial Opening Endoscopic
        "0D9V00Z",  # Drainage of Mesentery with Drainage Device, Open Approach
        "0D9V0ZZ",  # Drainage of Mesentery, Open Approach
        "0D9W00Z",  # Drainage of Peritoneum with Drainage Device, Open Approach
        "0D9W0ZZ",  # Drainage of Peritoneum, Open Approach
        "0DB10ZZ",  # Excision of Upper Esophagus, Open Approach
        "0DB17ZZ",  # Excision of Upper Esophagus, Via Natural or Artificial Opening
        "0DB20ZZ",  # Excision of Middle Esophagus, Open Approach
        "0DB27ZZ",  # Excision of Middle Esophagus, Via Natural or Artificial Opening
        "0DB30ZZ",  # Excision of Lower Esophagus, Open Approach
        "0DB37ZZ",  # Excision of Lower Esophagus, Via Natural or Artificial Opening
        "0DB40ZZ",  # Excision of Esophagogastric Junction, Open Approach
        "0DB47ZZ",  # Excision of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DB50ZZ",  # Excision of Esophagus, Open Approach
        "0DB57ZZ",  # Excision of Esophagus, Via Natural or Artificial Opening
        "0DB60Z3",  # Excision of Stomach, Open Approach, Vertical
        "0DB60ZZ",  # Excision of Stomach, Open Approach
        "0DB67Z3",  # Excision of Stomach, Via Natural or Artificial Opening, Vertical
        "0DB67ZZ",  # Excision of Stomach, Via Natural or Artificial Opening
        "0DB68Z3",  # Excision of Stomach, Via Natural or Artificial Opening Endoscopic, Vertical
        "0DB68ZZ",  # Excision of Stomach, Via Natural or Artificial Opening Endoscopic
        "0DB80ZZ",  # Excision of Small Intestine, Open Approach
        "0DB87ZZ",  # Excision of Small Intestine, Via Natural or Artificial Opening
        "0DB88ZZ",  # Excision of Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DB90ZZ",  # Excision of Duodenum, Open Approach
        "0DB97ZZ",  # Excision of Duodenum, Via Natural or Artificial Opening
        "0DBA0ZZ",  # Excision of Jejunum, Open Approach
        "0DBA7ZZ",  # Excision of Jejunum, Via Natural or Artificial Opening
        "0DBA8ZZ",  # Excision of Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DBB0ZZ",  # Excision of Ileum, Open Approach
        "0DBB7ZZ",  # Excision of Ileum, Via Natural or Artificial Opening
        "0DBB8ZZ",  # Excision of Ileum, Via Natural or Artificial Opening Endoscopic
        "0DBC0ZZ",  # Excision of Ileocecal Valve, Open Approach
        "0DBC7ZZ",  # Excision of Ileocecal Valve, Via Natural or Artificial Opening
        "0DBC8ZZ",  # Excision of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0DBE0ZZ",  # Excision of Large Intestine, Open Approach
        "0DBE7ZZ",  # Excision of Large Intestine, Via Natural or Artificial Opening
        "0DBE8ZZ",  # Excision of Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DBF0ZZ",  # Excision of Right Large Intestine, Open Approach
        "0DBF7ZZ",  # Excision of Right Large Intestine, Via Natural or Artificial Opening
        "0DBF8ZZ",  # Excision of Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DBG0ZZ",  # Excision of Left Large Intestine, Open Approach
        "0DBG7ZZ",  # Excision of Left Large Intestine, Via Natural or Artificial Opening
        "0DBG8ZZ",  # Excision of Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DBH0ZZ",  # Excision of Cecum, Open Approach
        "0DBH7ZZ",  # Excision of Cecum, Via Natural or Artificial Opening
        "0DBH8ZZ",  # Excision of Cecum, Via Natural or Artificial Opening Endoscopic
        "0DBK0ZZ",  # Excision of Ascending Colon, Open Approach
        "0DBK7ZZ",  # Excision of Ascending Colon, Via Natural or Artificial Opening
        "0DBK8ZZ",  # Excision of Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DBL0ZZ",  # Excision of Transverse Colon, Open Approach
        "0DBL7ZZ",  # Excision of Transverse Colon, Via Natural or Artificial Opening
        "0DBL8ZZ",  # Excision of Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DBM0ZZ",  # Excision of Descending Colon, Open Approach
        "0DBM7ZZ",  # Excision of Descending Colon, Via Natural or Artificial Opening
        "0DBM8ZZ",  # Excision of Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DBN0ZZ",  # Excision of Sigmoid Colon, Open Approach
        "0DBN7ZZ",  # Excision of Sigmoid Colon, Via Natural or Artificial Opening
        "0DBN8ZZ",  # Excision of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DBP0ZZ",  # Excision of Rectum, Open Approach
        "0DBP7ZZ",  # Excision of Rectum, Via Natural or Artificial Opening
        "0DBP8ZZ",  # Excision of Rectum, Via Natural or Artificial Opening Endoscopic
        "0DC10ZZ",  # Extirpation of Matter from Upper Esophagus, Open Approach
        "0DC20ZZ",  # Extirpation of Matter from Middle Esophagus, Open Approach
        "0DC30ZZ",  # Extirpation of Matter from Lower Esophagus, Open Approach
        "0DC40ZZ",  # Extirpation of Matter from Esophagogastric Junction, Open Approach
        "0DC50ZZ",  # Extirpation of Matter from Esophagus, Open Approach
        "0DC80ZZ",  # Extirpation of Matter from Small Intestine, Open Approach
        "0DC90ZZ",  # Extirpation of Matter from Duodenum, Open Approach
        "0DCA0ZZ",  # Extirpation of Matter from Jejunum, Open Approach
        "0DCB0ZZ",  # Extirpation of Matter from Ileum, Open Approach
        "0DCC0ZZ",  # Extirpation of Matter from Ileocecal Valve, Open Approach
        "0DCE0ZZ",  # Extirpation of Matter from Large Intestine, Open Approach
        "0DCF0ZZ",  # Extirpation of Matter from Right Large Intestine, Open Approach
        "0DCG0ZZ",  # Extirpation of Matter from Left Large Intestine, Open Approach
        "0DCH0ZZ",  # Extirpation of Matter from Cecum, Open Approach
        "0DCK0ZZ",  # Extirpation of Matter from Ascending Colon, Open Approach
        "0DCL0ZZ",  # Extirpation of Matter from Transverse Colon, Open Approach
        "0DCM0ZZ",  # Extirpation of Matter from Descending Colon, Open Approach
        "0DCN0ZZ",  # Extirpation of Matter from Sigmoid Colon, Open Approach
        "0DCP0ZZ",  # Extirpation of Matter from Rectum, Open Approach
        "0DCV0ZZ",  # Extirpation of Matter from Mesentery, Open Approach
        "0DCW0ZZ",  # Extirpation of Matter from Peritoneum, Open Approach
        "0DF50ZZ",  # Fragmentation in Esophagus, Open Approach
        "0DF57ZZ",  # Fragmentation in Esophagus, Via Natural or Artificial Opening
        "0DF58ZZ",  # Fragmentation in Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DF60ZZ",  # Fragmentation in Stomach, Open Approach
        "0DF67ZZ",  # Fragmentation in Stomach, Via Natural or Artificial Opening
        "0DF68ZZ",  # Fragmentation in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DF80ZZ",  # Fragmentation in Small Intestine, Open Approach
        "0DF87ZZ",  # Fragmentation in Small Intestine, Via Natural or Artificial Opening
        "0DF88ZZ",  # Fragmentation in Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DF90ZZ",  # Fragmentation in Duodenum, Open Approach
        "0DF97ZZ",  # Fragmentation in Duodenum, Via Natural or Artificial Opening
        "0DF98ZZ",  # Fragmentation in Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DFA0ZZ",  # Fragmentation in Jejunum, Open Approach
        "0DFA7ZZ",  # Fragmentation in Jejunum, Via Natural or Artificial Opening
        "0DFA8ZZ",  # Fragmentation in Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DFB0ZZ",  # Fragmentation in Ileum, Open Approach
        "0DFB7ZZ",  # Fragmentation in Ileum, Via Natural or Artificial Opening
        "0DFB8ZZ",  # Fragmentation in Ileum, Via Natural or Artificial Opening Endoscopic
        "0DFE0ZZ",  # Fragmentation in Large Intestine, Open Approach
        "0DFE7ZZ",  # Fragmentation in Large Intestine, Via Natural or Artificial Opening
        "0DFE8ZZ",  # Fragmentation in Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DFF0ZZ",  # Fragmentation in Right Large Intestine, Open Approach
        "0DFF7ZZ",  # Fragmentation in Right Large Intestine, Via Natural or Artificial Opening
        "0DFF8ZZ",  # Fragmentation in Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DFG0ZZ",  # Fragmentation in Left Large Intestine, Open Approach
        "0DFG7ZZ",  # Fragmentation in Left Large Intestine, Via Natural or Artificial Opening
        "0DFG8ZZ",  # Fragmentation in Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DFH0ZZ",  # Fragmentation in Cecum, Open Approach
        "0DFH7ZZ",  # Fragmentation in Cecum, Via Natural or Artificial Opening
        "0DFH8ZZ",  # Fragmentation in Cecum, Via Natural or Artificial Opening Endoscopic
        "0DFK0ZZ",  # Fragmentation in Ascending Colon, Open Approach
        "0DFK7ZZ",  # Fragmentation in Ascending Colon, Via Natural or Artificial Opening
        "0DFK8ZZ",  # Fragmentation in Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DFL0ZZ",  # Fragmentation in Transverse Colon, Open Approach
        "0DFL7ZZ",  # Fragmentation in Transverse Colon, Via Natural or Artificial Opening
        "0DFL8ZZ",  # Fragmentation in Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DFM0ZZ",  # Fragmentation in Descending Colon, Open Approach
        "0DFM7ZZ",  # Fragmentation in Descending Colon, Via Natural or Artificial Opening
        "0DFM8ZZ",  # Fragmentation in Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DFN0ZZ",  # Fragmentation in Sigmoid Colon, Open Approach
        "0DFN7ZZ",  # Fragmentation in Sigmoid Colon, Via Natural or Artificial Opening
        "0DFN8ZZ",  # Fragmentation in Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DH502Z",  # Insertion of Monitoring Device into Esophagus, Open Approach
        "0DH503Z",  # Insertion of Infusion Device into Esophagus, Open Approach
        "0DH572Z",  # Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening
        "0DH573Z",  # Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening
        "0DH582Z",  # Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DH583Z",  # Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DH60DZ",  # Insertion of Intraluminal Device into Stomach, Open Approach
        "0DH67DZ",  # Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening
        "0DH68DZ",  # Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening Endoscopic
        "0DH802Z",  # Insertion of Monitoring Device into Small Intestine, Open Approach
        "0DH803Z",  # Insertion of Infusion Device into Small Intestine, Open Approach
        "0DH872Z",  # Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening
        "0DH873Z",  # Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening
        "0DH882Z",  # Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DH883Z",  # Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DH902Z",  # Insertion of Monitoring Device into Duodenum, Open Approach
        "0DH903Z",  # Insertion of Infusion Device into Duodenum, Open Approach
        "0DH972Z",  # Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening
        "0DH973Z",  # Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening
        "0DH982Z",  # Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DH983Z",  # Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DHA02Z",  # Insertion of Monitoring Device into Jejunum, Open Approach
        "0DHA03Z",  # Insertion of Infusion Device into Jejunum, Open Approach
        "0DHA72Z",  # Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening
        "0DHA73Z",  # Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening
        "0DHA82Z",  # Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DHA83Z",  # Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DHB02Z",  # Insertion of Monitoring Device into Ileum, Open Approach
        "0DHB03Z",  # Insertion of Infusion Device into Ileum, Open Approach
        "0DHB72Z",  # Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening
        "0DHB73Z",  # Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening
        "0DHB82Z",  # Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening Endoscopic
        "0DHB83Z",  # Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening Endoscopic
        "0DJ00ZZ",  # Inspection of Upper Intestinal Tract, Open Approach
        "0DJ60ZZ",  # Inspection of Stomach, Open Approach
        "0DJD0ZZ",  # Inspection of Lower Intestinal Tract, Open Approach
        "0DJU0ZZ",  # Inspection of Omentum, Open Approach
        "0DJV0ZZ",  # Inspection of Mesentery, Open Approach
        "0DJW0ZZ",  # Inspection of Peritoneum, Open Approach
        "0DL60CZ",  # Occlusion of Stomach with Extraluminal Device, Open Approach
        "0DL60DZ",  # Occlusion of Stomach with Intraluminal Device, Open Approach
        "0DL60ZZ",  # Occlusion of Stomach, Open Approach
        "0DL67DZ",  # Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening
        "0DL67ZZ",  # Occlusion of Stomach, Via Natural or Artificial Opening
        "0DL68DZ",  # Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DL68ZZ",  # Occlusion of Stomach, Via Natural or Artificial Opening Endoscopic
        "0DL70CZ",  # Occlusion of Stomach, Pylorus with Extraluminal Device, Open Approach
        "0DL70DZ",  # Occlusion of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0DL70ZZ",  # Occlusion of Stomach, Pylorus, Open Approach
        "0DL77DZ",  # Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0DL77ZZ",  # Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DL78DZ",  # Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DL78ZZ",  # Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0DM50ZZ",  # Reattachment of Esophagus, Open Approach
        "0DM60ZZ",  # Reattachment of Stomach, Open Approach
        "0DM80ZZ",  # Reattachment of Small Intestine, Open Approach
        "0DM90ZZ",  # Reattachment of Duodenum, Open Approach
        "0DMA0ZZ",  # Reattachment of Jejunum, Open Approach
        "0DMB0ZZ",  # Reattachment of Ileum, Open Approach
        "0DME0ZZ",  # Reattachment of Large Intestine, Open Approach
        "0DMF0ZZ",  # Reattachment of Right Large Intestine, Open Approach
        "0DMG0ZZ",  # Reattachment of Left Large Intestine, Open Approach
        "0DMH0ZZ",  # Reattachment of Cecum, Open Approach
        "0DMK0ZZ",  # Reattachment of Ascending Colon, Open Approach
        "0DML0ZZ",  # Reattachment of Transverse Colon, Open Approach
        "0DMM0ZZ",  # Reattachment of Descending Colon, Open Approach
        "0DMN0ZZ",  # Reattachment of Sigmoid Colon, Open Approach
        "0DMP0ZZ",  # Reattachment of Rectum, Open Approach
        "0DN10ZZ",  # Release Upper Esophagus, Open Approach
        "0DN17ZZ",  # Release Upper Esophagus, Via Natural or Artificial Opening
        "0DN18ZZ",  # Release Upper Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DN20ZZ",  # Release Middle Esophagus, Open Approach
        "0DN27ZZ",  # Release Middle Esophagus, Via Natural or Artificial Opening
        "0DN28ZZ",  # Release Middle Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DN30ZZ",  # Release Lower Esophagus, Open Approach
        "0DN37ZZ",  # Release Lower Esophagus, Via Natural or Artificial Opening
        "0DN38ZZ",  # Release Lower Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DN40ZZ",  # Release Esophagogastric Junction, Open Approach
        "0DN47ZZ",  # Release Esophagogastric Junction, Via Natural or Artificial Opening
        "0DN48ZZ",  # Release Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0DN50ZZ",  # Release Esophagus, Open Approach
        "0DN57ZZ",  # Release Esophagus, Via Natural or Artificial Opening
        "0DN58ZZ",  # Release Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DN60ZZ",  # Release Stomach, Open Approach
        "0DN67ZZ",  # Release Stomach, Via Natural or Artificial Opening
        "0DN68ZZ",  # Release Stomach, Via Natural or Artificial Opening Endoscopic
        "0DN70ZZ",  # Release Stomach, Pylorus, Open Approach
        "0DN77ZZ",  # Release Stomach, Pylorus, Via Natural or Artificial Opening
        "0DN78ZZ",  # Release Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0DN80ZZ",  # Release Small Intestine, Open Approach
        "0DN90ZZ",  # Release Duodenum, Open Approach
        "0DNA0ZZ",  # Release Jejunum, Open Approach
        "0DNB0ZZ",  # Release Ileum, Open Approach
        "0DNC0ZZ",  # Release Ileocecal Valve, Open Approach
        "0DNC7ZZ",  # Release Ileocecal Valve, Via Natural or Artificial Opening
        "0DNC8ZZ",  # Release Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0DNE0ZZ",  # Release Large Intestine, Open Approach
        "0DNF0ZZ",  # Release Right Large Intestine, Open Approach
        "0DNG0ZZ",  # Release Left Large Intestine, Open Approach
        "0DNH0ZZ",  # Release Cecum, Open Approach
        "0DNJ0ZZ",  # Release Appendix, Open Approach
        "0DNK0ZZ",  # Release Ascending Colon, Open Approach
        "0DNL0ZZ",  # Release Transverse Colon, Open Approach
        "0DNM0ZZ",  # Release Descending Colon, Open Approach
        "0DNN0ZZ",  # Release Sigmoid Colon, Open Approach
        "0DNV0ZZ",  # Release Mesentery, Open Approach
        "0DNW0ZZ",  # Release Peritoneum, Open Approach
        "0DP000Z",  # Removal of Drainage Device from Upper Intestinal Tract, Open Approach
        "0DP002Z",  # Removal of Monitoring Device from Upper Intestinal Tract, Open Approach
        "0DP003Z",  # Removal of Infusion Device from Upper Intestinal Tract, Open Approach
        "0DP007Z",  # Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Open Approach
        "0DP00CZ",  # Removal of Extraluminal Device from Upper Intestinal Tract, Open Approach
        "0DP00DZ",  # Removal of Intraluminal Device from Upper Intestinal Tract, Open Approach
        "0DP00JZ",  # Removal of Synthetic Substitute from Upper Intestinal Tract, Open Approach
        "0DP00KZ",  # Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Open Approach
        "0DP00UZ",  # Removal of Feeding Device from Upper Intestinal Tract, Open Approach
        "0DP070Z",  # Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP072Z",  # Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP073Z",  # Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP077Z",  # Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07CZ",  # Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07DZ",  # Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07JZ",  # Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07KZ",  # Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07UZ",  # Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP080Z",  # Removal of Drainage Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP082Z",  # Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP083Z",  # Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP087Z",  # Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP08CZ",  # Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP08DZ",  # Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP08JZ",  # Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP08KZ",  # Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP08UZ",  # Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DP501Z",  # Removal of Radioactive Element from Esophagus, Open Approach
        "0DP502Z",  # Removal of Monitoring Device from Esophagus, Open Approach
        "0DP503Z",  # Removal of Infusion Device from Esophagus, Open Approach
        "0DP50UZ",  # Removal of Feeding Device from Esophagus, Open Approach
        "0DP57DZ",  # Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening
        "0DP58DZ",  # Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DP600Z",  # Removal of Drainage Device from Stomach, Open Approach
        "0DP602Z",  # Removal of Monitoring Device from Stomach, Open Approach
        "0DP603Z",  # Removal of Infusion Device from Stomach, Open Approach
        "0DP607Z",  # Removal of Autologous Tissue Substitute from Stomach, Open Approach
        "0DP60CZ",  # Removal of Extraluminal Device from Stomach, Open Approach
        "0DP60DZ",  # Removal of Intraluminal Device from Stomach, Open Approach
        "0DP60JZ",  # Removal of Synthetic Substitute from Stomach, Open Approach
        "0DP60KZ",  # Removal of Nonautologous Tissue Substitute from Stomach, Open Approach
        "0DP60MZ",  # Removal of Stimulator Lead from Stomach, Open Approach
        "0DP60UZ",  # Removal of Feeding Device from Stomach, Open Approach
        "0DP670Z",  # Removal of Drainage Device from Stomach, Via Natural or Artificial Opening
        "0DP672Z",  # Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening
        "0DP673Z",  # Removal of Infusion Device from Stomach, Via Natural or Artificial Opening
        "0DP677Z",  # Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening
        "0DP67CZ",  # Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening
        "0DP67JZ",  # Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening
        "0DP67KZ",  # Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening
        "0DP67UZ",  # Removal of Feeding Device from Stomach, Via Natural or Artificial Opening
        "0DP680Z",  # Removal of Drainage Device from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP682Z",  # Removal of Monitoring Device from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP683Z",  # Removal of Infusion Device from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP687Z",  # Removal of Autologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP68CZ",  # Removal of Extraluminal Device from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP68JZ",  # Removal of Synthetic Substitute from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP68KZ",  # Removal of Nonautologous Tissue Substitute from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DP68UZ",  # Removal of Feeding Device from Stomach, Via Natural or Artificial Opening Endoscopic
        "0DPD00Z",  # Removal of Drainage Device from Lower Intestinal Tract, Open Approach
        "0DPD02Z",  # Removal of Monitoring Device from Lower Intestinal Tract, Open Approach
        "0DPD03Z",  # Removal of Infusion Device from Lower Intestinal Tract, Open Approach
        "0DPD07Z",  # Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0CZ",  # Removal of Extraluminal Device from Lower Intestinal Tract, Open Approach
        "0DPD0DZ",  # Removal of Intraluminal Device from Lower Intestinal Tract, Open Approach
        "0DPD0JZ",  # Removal of Synthetic Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0KZ",  # Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0UZ",  # Removal of Feeding Device from Lower Intestinal Tract, Open Approach
        "0DPD70Z",  # Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD72Z",  # Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD73Z",  # Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD77Z",  # Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7CZ",  # Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7DZ",  # Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7JZ",  # Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7KZ",  # Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7UZ",  # Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD80Z",  # Removal of Drainage Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD82Z",  # Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD83Z",  # Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD87Z",  # Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD8CZ",  # Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD8DZ",  # Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD8JZ",  # Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD8KZ",  # Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPD8UZ",  # Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DPP01Z",  # Removal of Radioactive Element from Rectum, Open Approach
        "0DPU00Z",  # Removal of Drainage Device from Omentum, Open Approach
        "0DPU01Z",  # Removal of Radioactive Element from Omentum, Open Approach
        "0DPU07Z",  # Removal of Autologous Tissue Substitute from Omentum, Open Approach
        "0DPU0JZ",  # Removal of Synthetic Substitute from Omentum, Open Approach
        "0DPU0KZ",  # Removal of Nonautologous Tissue Substitute from Omentum, Open Approach
        "0DPV00Z",  # Removal of Drainage Device from Mesentery, Open Approach
        "0DPV01Z",  # Removal of Radioactive Element from Mesentery, Open Approach
        "0DPV07Z",  # Removal of Autologous Tissue Substitute from Mesentery, Open Approach
        "0DPV0JZ",  # Removal of Synthetic Substitute from Mesentery, Open Approach
        "0DPV0KZ",  # Removal of Nonautologous Tissue Substitute from Mesentery, Open Approach
        "0DPW00Z",  # Removal of Drainage Device from Peritoneum, Open Approach
        "0DPW01Z",  # Removal of Radioactive Element from Peritoneum, Open Approach
        "0DPW07Z",  # Removal of Autologous Tissue Substitute from Peritoneum, Open Approach
        "0DPW0JZ",  # Removal of Synthetic Substitute from Peritoneum, Open Approach
        "0DPW0KZ",  # Removal of Nonautologous Tissue Substitute from Peritoneum, Open Approach
        "0DQ10ZZ",  # Repair Upper Esophagus, Open Approach
        "0DQ17ZZ",  # Repair Upper Esophagus, Via Natural or Artificial Opening
        "0DQ18ZZ",  # Repair Upper Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DQ20ZZ",  # Repair Middle Esophagus, Open Approach
        "0DQ27ZZ",  # Repair Middle Esophagus, Via Natural or Artificial Opening
        "0DQ28ZZ",  # Repair Middle Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DQ30ZZ",  # Repair Lower Esophagus, Open Approach
        "0DQ37ZZ",  # Repair Lower Esophagus, Via Natural or Artificial Opening
        "0DQ38ZZ",  # Repair Lower Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DQ40ZZ",  # Repair Esophagogastric Junction, Open Approach
        "0DQ47ZZ",  # Repair Esophagogastric Junction, Via Natural or Artificial Opening
        "0DQ48ZZ",  # Repair Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0DQ50ZZ",  # Repair Esophagus, Open Approach
        "0DQ57ZZ",  # Repair Esophagus, Via Natural or Artificial Opening
        "0DQ58ZZ",  # Repair Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DQ60ZZ",  # Repair Stomach, Open Approach
        "0DQ67ZZ",  # Repair Stomach, Via Natural or Artificial Opening
        "0DQ68ZZ",  # Repair Stomach, Via Natural or Artificial Opening Endoscopic
        "0DQ70ZZ",  # Repair Stomach, Pylorus, Open Approach
        "0DQ77ZZ",  # Repair Stomach, Pylorus, Via Natural or Artificial Opening
        "0DQ78ZZ",  # Repair Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0DQ80ZZ",  # Repair Small Intestine, Open Approach
        "0DQ87ZZ",  # Repair Small Intestine, Via Natural or Artificial Opening
        "0DQ88ZZ",  # Repair Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DQ90ZZ",  # Repair Duodenum, Open Approach
        "0DQ97ZZ",  # Repair Duodenum, Via Natural or Artificial Opening
        "0DQ98ZZ",  # Repair Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DQA0ZZ",  # Repair Jejunum, Open Approach
        "0DQA7ZZ",  # Repair Jejunum, Via Natural or Artificial Opening
        "0DQA8ZZ",  # Repair Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DQB0ZZ",  # Repair Ileum, Open Approach
        "0DQB7ZZ",  # Repair Ileum, Via Natural or Artificial Opening
        "0DQB8ZZ",  # Repair Ileum, Via Natural or Artificial Opening Endoscopic
        "0DQC0ZZ",  # Repair Ileocecal Valve, Open Approach
        "0DQC7ZZ",  # Repair Ileocecal Valve, Via Natural or Artificial Opening
        "0DQC8ZZ",  # Repair Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0DQE0ZZ",  # Repair Large Intestine, Open Approach
        "0DQE7ZZ",  # Repair Large Intestine, Via Natural or Artificial Opening
        "0DQE8ZZ",  # Repair Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DQF0ZZ",  # Repair Right Large Intestine, Open Approach
        "0DQF7ZZ",  # Repair Right Large Intestine, Via Natural or Artificial Opening
        "0DQF8ZZ",  # Repair Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DQG0ZZ",  # Repair Left Large Intestine, Open Approach
        "0DQG7ZZ",  # Repair Left Large Intestine, Via Natural or Artificial Opening
        "0DQG8ZZ",  # Repair Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DQH0ZZ",  # Repair Cecum, Open Approach
        "0DQH7ZZ",  # Repair Cecum, Via Natural or Artificial Opening
        "0DQH8ZZ",  # Repair Cecum, Via Natural or Artificial Opening Endoscopic
        "0DQK0ZZ",  # Repair Ascending Colon, Open Approach
        "0DQK7ZZ",  # Repair Ascending Colon, Via Natural or Artificial Opening
        "0DQK8ZZ",  # Repair Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DQL0ZZ",  # Repair Transverse Colon, Open Approach
        "0DQL7ZZ",  # Repair Transverse Colon, Via Natural or Artificial Opening
        "0DQL8ZZ",  # Repair Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DQM0ZZ",  # Repair Descending Colon, Open Approach
        "0DQM7ZZ",  # Repair Descending Colon, Via Natural or Artificial Opening
        "0DQM8ZZ",  # Repair Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DQN0ZZ",  # Repair Sigmoid Colon, Open Approach
        "0DQN7ZZ",  # Repair Sigmoid Colon, Via Natural or Artificial Opening
        "0DQN8ZZ",  # Repair Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DQV0ZZ",  # Repair Mesentery, Open Approach
        "0DQW0ZZ",  # Repair Peritoneum, Open Approach
        "0DR507Z",  # Replacement of Esophagus with Autologous Tissue Substitute, Open Approach
        "0DR50JZ",  # Replacement of Esophagus with Synthetic Substitute, Open Approach
        "0DR50KZ",  # Replacement of Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DR577Z",  # Replacement of Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DR57JZ",  # Replacement of Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DR57KZ",  # Replacement of Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DR587Z",  # Replacement of Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DR58JZ",  # Replacement of Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DR58KZ",  # Replacement of Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DRV07Z",  # Replacement of Mesentery with Autologous Tissue Substitute, Open Approach
        "0DRV0JZ",  # Replacement of Mesentery with Synthetic Substitute, Open Approach
        "0DRV0KZ",  # Replacement of Mesentery with Nonautologous Tissue Substitute, Open Approach
        "0DRW07Z",  # Replacement of Peritoneum with Autologous Tissue Substitute, Open Approach
        "0DRW0JZ",  # Replacement of Peritoneum with Synthetic Substitute, Open Approach
        "0DRW0KZ",  # Replacement of Peritoneum with Nonautologous Tissue Substitute, Open Approach
        "0DS50ZZ",  # Reposition Esophagus, Open Approach
        "0DS57ZZ",  # Reposition Esophagus, Via Natural or Artificial Opening
        "0DS58ZZ",  # Reposition Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DS5XZZ",  # Reposition Esophagus, External Approach
        "0DS60ZZ",  # Reposition Stomach, Open Approach
        "0DS67ZZ",  # Reposition Stomach, Via Natural or Artificial Opening
        "0DS68ZZ",  # Reposition Stomach, Via Natural or Artificial Opening Endoscopic
        "0DS6XZZ",  # Reposition Stomach, External Approach
        "0DS90ZZ",  # Reposition Duodenum, Open Approach
        "0DS97ZZ",  # Reposition Duodenum, Via Natural or Artificial Opening
        "0DS98ZZ",  # Reposition Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DSA0ZZ",  # Reposition Jejunum, Open Approach
        "0DSA7ZZ",  # Reposition Jejunum, Via Natural or Artificial Opening
        "0DSA8ZZ",  # Reposition Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DSB0ZZ",  # Reposition Ileum, Open Approach
        "0DSB7ZZ",  # Reposition Ileum, Via Natural or Artificial Opening
        "0DSB8ZZ",  # Reposition Ileum, Via Natural or Artificial Opening Endoscopic
        "0DSH0ZZ",  # Reposition Cecum, Open Approach
        "0DSH7ZZ",  # Reposition Cecum, Via Natural or Artificial Opening
        "0DSH8ZZ",  # Reposition Cecum, Via Natural or Artificial Opening Endoscopic
        "0DSK0ZZ",  # Reposition Ascending Colon, Open Approach
        "0DSK7ZZ",  # Reposition Ascending Colon, Via Natural or Artificial Opening
        "0DSK8ZZ",  # Reposition Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DSL0ZZ",  # Reposition Transverse Colon, Open Approach
        "0DSL7ZZ",  # Reposition Transverse Colon, Via Natural or Artificial Opening
        "0DSL8ZZ",  # Reposition Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DSM0ZZ",  # Reposition Descending Colon, Open Approach
        "0DSM7ZZ",  # Reposition Descending Colon, Via Natural or Artificial Opening
        "0DSM8ZZ",  # Reposition Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DSN0ZZ",  # Reposition Sigmoid Colon, Open Approach
        "0DSN7ZZ",  # Reposition Sigmoid Colon, Via Natural or Artificial Opening
        "0DSN8ZZ",  # Reposition Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DSP0ZZ",  # Reposition Rectum, Open Approach
        "0DSP7ZZ",  # Reposition Rectum, Via Natural or Artificial Opening
        "0DSP8ZZ",  # Reposition Rectum, Via Natural or Artificial Opening Endoscopic
        "0DT10ZZ",  # Resection of Upper Esophagus, Open Approach
        "0DT17ZZ",  # Resection of Upper Esophagus, Via Natural or Artificial Opening
        "0DT18ZZ",  # Resection of Upper Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DT20ZZ",  # Resection of Middle Esophagus, Open Approach
        "0DT27ZZ",  # Resection of Middle Esophagus, Via Natural or Artificial Opening
        "0DT28ZZ",  # Resection of Middle Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DT30ZZ",  # Resection of Lower Esophagus, Open Approach
        "0DT37ZZ",  # Resection of Lower Esophagus, Via Natural or Artificial Opening
        "0DT38ZZ",  # Resection of Lower Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DT40ZZ",  # Resection of Esophagogastric Junction, Open Approach
        "0DT47ZZ",  # Resection of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DT48ZZ",  # Resection of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0DT50ZZ",  # Resection of Esophagus, Open Approach
        "0DT57ZZ",  # Resection of Esophagus, Via Natural or Artificial Opening
        "0DT58ZZ",  # Resection of Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DT60ZZ",  # Resection of Stomach, Open Approach
        "0DT67ZZ",  # Resection of Stomach, Via Natural or Artificial Opening
        "0DT68ZZ",  # Resection of Stomach, Via Natural or Artificial Opening Endoscopic
        "0DT70ZZ",  # Resection of Stomach, Pylorus, Open Approach
        "0DT77ZZ",  # Resection of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DT78ZZ",  # Resection of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0DT80ZZ",  # Resection of Small Intestine, Open Approach
        "0DT87ZZ",  # Resection of Small Intestine, Via Natural or Artificial Opening
        "0DT88ZZ",  # Resection of Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DT90ZZ",  # Resection of Duodenum, Open Approach
        "0DT97ZZ",  # Resection of Duodenum, Via Natural or Artificial Opening
        "0DT98ZZ",  # Resection of Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DTA0ZZ",  # Resection of Jejunum, Open Approach
        "0DTA7ZZ",  # Resection of Jejunum, Via Natural or Artificial Opening
        "0DTA8ZZ",  # Resection of Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DTB0ZZ",  # Resection of Ileum, Open Approach
        "0DTB7ZZ",  # Resection of Ileum, Via Natural or Artificial Opening
        "0DTB8ZZ",  # Resection of Ileum, Via Natural or Artificial Opening Endoscopic
        "0DTC0ZZ",  # Resection of Ileocecal Valve, Open Approach
        "0DTC7ZZ",  # Resection of Ileocecal Valve, Via Natural or Artificial Opening
        "0DTC8ZZ",  # Resection of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0DTE0ZZ",  # Resection of Large Intestine, Open Approach
        "0DTE7ZZ",  # Resection of Large Intestine, Via Natural or Artificial Opening
        "0DTE8ZZ",  # Resection of Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DTF0ZZ",  # Resection of Right Large Intestine, Open Approach
        "0DTF7ZZ",  # Resection of Right Large Intestine, Via Natural or Artificial Opening
        "0DTF8ZZ",  # Resection of Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DTG0ZZ",  # Resection of Left Large Intestine, Open Approach
        "0DTG7ZZ",  # Resection of Left Large Intestine, Via Natural or Artificial Opening
        "0DTG8ZZ",  # Resection of Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DTH0ZZ",  # Resection of Cecum, Open Approach
        "0DTH7ZZ",  # Resection of Cecum, Via Natural or Artificial Opening
        "0DTH8ZZ",  # Resection of Cecum, Via Natural or Artificial Opening Endoscopic
        "0DTK0ZZ",  # Resection of Ascending Colon, Open Approach
        "0DTK7ZZ",  # Resection of Ascending Colon, Via Natural or Artificial Opening
        "0DTK8ZZ",  # Resection of Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DTL0ZZ",  # Resection of Transverse Colon, Open Approach
        "0DTL7ZZ",  # Resection of Transverse Colon, Via Natural or Artificial Opening
        "0DTL8ZZ",  # Resection of Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DTM0ZZ",  # Resection of Descending Colon, Open Approach
        "0DTM7ZZ",  # Resection of Descending Colon, Via Natural or Artificial Opening
        "0DTM8ZZ",  # Resection of Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DTN0ZZ",  # Resection of Sigmoid Colon, Open Approach
        "0DTN7ZZ",  # Resection of Sigmoid Colon, Via Natural or Artificial Opening
        "0DTN8ZZ",  # Resection of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DTP0ZZ",  # Resection of Rectum, Open Approach
        "0DTP7ZZ",  # Resection of Rectum, Via Natural or Artificial Opening
        "0DTP8ZZ",  # Resection of Rectum, Via Natural or Artificial Opening Endoscopic
        "0DU107Z",  # Supplement Upper Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU10JZ",  # Supplement Upper Esophagus with Synthetic Substitute, Open Approach
        "0DU10KZ",  # Supplement Upper Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU177Z",  # Supplement Upper Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU17JZ",  # Supplement Upper Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU17KZ",  # Supplement Upper Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU187Z",  # Supplement Upper Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU18JZ",  # Supplement Upper Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU18KZ",  # Supplement Upper Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU207Z",  # Supplement Middle Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU20JZ",  # Supplement Middle Esophagus with Synthetic Substitute, Open Approach
        "0DU20KZ",  # Supplement Middle Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU277Z",  # Supplement Middle Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU27JZ",  # Supplement Middle Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU27KZ",  # Supplement Middle Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU287Z",  # Supplement Middle Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU28JZ",  # Supplement Middle Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU28KZ",  # Supplement Middle Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU307Z",  # Supplement Lower Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU30JZ",  # Supplement Lower Esophagus with Synthetic Substitute, Open Approach
        "0DU30KZ",  # Supplement Lower Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU377Z",  # Supplement Lower Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU37JZ",  # Supplement Lower Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU37KZ",  # Supplement Lower Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU387Z",  # Supplement Lower Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU38JZ",  # Supplement Lower Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU38KZ",  # Supplement Lower Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU407Z",  # Supplement Esophagogastric Junction with Autologous Tissue Substitute, Open Approach
        "0DU40JZ",  # Supplement Esophagogastric Junction with Synthetic Substitute, Open Approach
        "0DU40KZ",  # Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Open Approach
        "0DU477Z",  # Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU47JZ",  # Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU47KZ",  # Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU487Z",  # Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU48JZ",  # Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU48KZ",  # Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU507Z",  # Supplement Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU50JZ",  # Supplement Esophagus with Synthetic Substitute, Open Approach
        "0DU50KZ",  # Supplement Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU577Z",  # Supplement Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU57JZ",  # Supplement Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU57KZ",  # Supplement Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU587Z",  # Supplement Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU58JZ",  # Supplement Esophagus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU58KZ",  # Supplement Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU607Z",  # Supplement Stomach with Autologous Tissue Substitute, Open Approach
        "0DU60JZ",  # Supplement Stomach with Synthetic Substitute, Open Approach
        "0DU60KZ",  # Supplement Stomach with Nonautologous Tissue Substitute, Open Approach
        "0DU677Z",  # Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU67JZ",  # Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU67KZ",  # Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU687Z",  # Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU68JZ",  # Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU68KZ",  # Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU707Z",  # Supplement Stomach, Pylorus with Autologous Tissue Substitute, Open Approach
        "0DU70JZ",  # Supplement Stomach, Pylorus with Synthetic Substitute, Open Approach
        "0DU70KZ",  # Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Open Approach
        "0DU777Z",  # Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU77JZ",  # Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU77KZ",  # Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU787Z",  # Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU78JZ",  # Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU78KZ",  # Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU807Z",  # Supplement Small Intestine with Autologous Tissue Substitute, Open Approach
        "0DU80JZ",  # Supplement Small Intestine with Synthetic Substitute, Open Approach
        "0DU80KZ",  # Supplement Small Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DU877Z",  # Supplement Small Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU87JZ",  # Supplement Small Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU87KZ",  # Supplement Small Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU887Z",  # Supplement Small Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU88JZ",  # Supplement Small Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU88KZ",  # Supplement Small Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU907Z",  # Supplement Duodenum with Autologous Tissue Substitute, Open Approach
        "0DU90JZ",  # Supplement Duodenum with Synthetic Substitute, Open Approach
        "0DU90KZ",  # Supplement Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0DU977Z",  # Supplement Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU97JZ",  # Supplement Duodenum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU97KZ",  # Supplement Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU987Z",  # Supplement Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU98JZ",  # Supplement Duodenum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DU98KZ",  # Supplement Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUA07Z",  # Supplement Jejunum with Autologous Tissue Substitute, Open Approach
        "0DUA0JZ",  # Supplement Jejunum with Synthetic Substitute, Open Approach
        "0DUA0KZ",  # Supplement Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0DUA77Z",  # Supplement Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUA7JZ",  # Supplement Jejunum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUA7KZ",  # Supplement Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUA87Z",  # Supplement Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUA8JZ",  # Supplement Jejunum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUA8KZ",  # Supplement Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUB07Z",  # Supplement Ileum with Autologous Tissue Substitute, Open Approach
        "0DUB0JZ",  # Supplement Ileum with Synthetic Substitute, Open Approach
        "0DUB0KZ",  # Supplement Ileum with Nonautologous Tissue Substitute, Open Approach
        "0DUB77Z",  # Supplement Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUB7JZ",  # Supplement Ileum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUB7KZ",  # Supplement Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUB87Z",  # Supplement Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUB8JZ",  # Supplement Ileum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUB8KZ",  # Supplement Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUC07Z",  # Supplement Ileocecal Valve with Autologous Tissue Substitute, Open Approach
        "0DUC0JZ",  # Supplement Ileocecal Valve with Synthetic Substitute, Open Approach
        "0DUC0KZ",  # Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Open Approach
        "0DUC77Z",  # Supplement Ileocecal Valve with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUC7JZ",  # Supplement Ileocecal Valve with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUC7KZ",  # Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUC87Z",  # Supplement Ileocecal Valve with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUC8JZ",  # Supplement Ileocecal Valve with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUC8KZ",  # Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUE07Z",  # Supplement Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUE0JZ",  # Supplement Large Intestine with Synthetic Substitute, Open Approach
        "0DUE0KZ",  # Supplement Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUE77Z",  # Supplement Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUE7JZ",  # Supplement Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUE7KZ",  # Supplement Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUE87Z",  # Supplement Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUE8JZ",  # Supplement Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUE8KZ",  # Supplement Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUF07Z",  # Supplement Right Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUF0JZ",  # Supplement Right Large Intestine with Synthetic Substitute, Open Approach
        "0DUF0KZ",  # Supplement Right Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUF77Z",  # Supplement Right Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUF7JZ",  # Supplement Right Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUF7KZ",  # Supplement Right Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUF87Z",  # Supplement Right Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUF8JZ",  # Supplement Right Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUF8KZ",  # Supplement Right Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUG07Z",  # Supplement Left Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUG0JZ",  # Supplement Left Large Intestine with Synthetic Substitute, Open Approach
        "0DUG0KZ",  # Supplement Left Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUG77Z",  # Supplement Left Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUG7JZ",  # Supplement Left Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUG7KZ",  # Supplement Left Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUG87Z",  # Supplement Left Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUG8JZ",  # Supplement Left Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUG8KZ",  # Supplement Left Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUH07Z",  # Supplement Cecum with Autologous Tissue Substitute, Open Approach
        "0DUH0JZ",  # Supplement Cecum with Synthetic Substitute, Open Approach
        "0DUH0KZ",  # Supplement Cecum with Nonautologous Tissue Substitute, Open Approach
        "0DUH77Z",  # Supplement Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUH7JZ",  # Supplement Cecum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUH7KZ",  # Supplement Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUH87Z",  # Supplement Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUH8JZ",  # Supplement Cecum with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUH8KZ",  # Supplement Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUK07Z",  # Supplement Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0DUK0JZ",  # Supplement Ascending Colon with Synthetic Substitute, Open Approach
        "0DUK0KZ",  # Supplement Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUK77Z",  # Supplement Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUK7JZ",  # Supplement Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUK7KZ",  # Supplement Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUK87Z",  # Supplement Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUK8JZ",  # Supplement Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUK8KZ",  # Supplement Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUL07Z",  # Supplement Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0DUL0JZ",  # Supplement Transverse Colon with Synthetic Substitute, Open Approach
        "0DUL0KZ",  # Supplement Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUL77Z",  # Supplement Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUL7JZ",  # Supplement Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUL7KZ",  # Supplement Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUL87Z",  # Supplement Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUL8JZ",  # Supplement Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUL8KZ",  # Supplement Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUM07Z",  # Supplement Descending Colon with Autologous Tissue Substitute, Open Approach
        "0DUM0JZ",  # Supplement Descending Colon with Synthetic Substitute, Open Approach
        "0DUM0KZ",  # Supplement Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUM77Z",  # Supplement Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUM7JZ",  # Supplement Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUM7KZ",  # Supplement Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUM87Z",  # Supplement Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUM8JZ",  # Supplement Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUM8KZ",  # Supplement Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUN07Z",  # Supplement Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0DUN0JZ",  # Supplement Sigmoid Colon with Synthetic Substitute, Open Approach
        "0DUN0KZ",  # Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUN77Z",  # Supplement Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUN7JZ",  # Supplement Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUN7KZ",  # Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUN87Z",  # Supplement Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUN8JZ",  # Supplement Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUN8KZ",  # Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0DUV07Z",  # Supplement Mesentery with Autologous Tissue Substitute, Open Approach
        "0DUV0JZ",  # Supplement Mesentery with Synthetic Substitute, Open Approach
        "0DUV0KZ",  # Supplement Mesentery with Nonautologous Tissue Substitute, Open Approach
        "0DUW07Z",  # Supplement Peritoneum with Autologous Tissue Substitute, Open Approach
        "0DUW0JZ",  # Supplement Peritoneum with Synthetic Substitute, Open Approach
        "0DUW0KZ",  # Supplement Peritoneum with Nonautologous Tissue Substitute, Open Approach
        "0DV10CZ",  # Restriction of Upper Esophagus with Extraluminal Device, Open Approach
        "0DV10DZ",  # Restriction of Upper Esophagus with Intraluminal Device, Open Approach
        "0DV10ZZ",  # Restriction of Upper Esophagus, Open Approach
        "0DV17DZ",  # Restriction of Upper Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV17ZZ",  # Restriction of Upper Esophagus, Via Natural or Artificial Opening
        "0DV18DZ",  # Restriction of Upper Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV18ZZ",  # Restriction of Upper Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DV20CZ",  # Restriction of Middle Esophagus with Extraluminal Device, Open Approach
        "0DV20DZ",  # Restriction of Middle Esophagus with Intraluminal Device, Open Approach
        "0DV20ZZ",  # Restriction of Middle Esophagus, Open Approach
        "0DV27DZ",  # Restriction of Middle Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV27ZZ",  # Restriction of Middle Esophagus, Via Natural or Artificial Opening
        "0DV28DZ",  # Restriction of Middle Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV28ZZ",  # Restriction of Middle Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DV30CZ",  # Restriction of Lower Esophagus with Extraluminal Device, Open Approach
        "0DV30DZ",  # Restriction of Lower Esophagus with Intraluminal Device, Open Approach
        "0DV30ZZ",  # Restriction of Lower Esophagus, Open Approach
        "0DV37DZ",  # Restriction of Lower Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV37ZZ",  # Restriction of Lower Esophagus, Via Natural or Artificial Opening
        "0DV38DZ",  # Restriction of Lower Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV38ZZ",  # Restriction of Lower Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DV40CZ",  # Restriction of Esophagogastric Junction with Extraluminal Device, Open Approach
        "0DV40DZ",  # Restriction of Esophagogastric Junction with Intraluminal Device, Open Approach
        "0DV40ZZ",  # Restriction of Esophagogastric Junction, Open Approach
        "0DV47DZ",  # Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening
        "0DV47ZZ",  # Restriction of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DV48DZ",  # Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV48ZZ",  # Restriction of Esophagogastric Junction, Via Natural or Artificial Opening Endoscopic
        "0DV50CZ",  # Restriction of Esophagus with Extraluminal Device, Open Approach
        "0DV50DZ",  # Restriction of Esophagus with Intraluminal Device, Open Approach
        "0DV50ZZ",  # Restriction of Esophagus, Open Approach
        "0DV57DZ",  # Restriction of Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV57ZZ",  # Restriction of Esophagus, Via Natural or Artificial Opening
        "0DV58DZ",  # Restriction of Esophagus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV58ZZ",  # Restriction of Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DV60CZ",  # Restriction of Stomach with Extraluminal Device, Open Approach
        "0DV60DZ",  # Restriction of Stomach with Intraluminal Device, Open Approach
        "0DV60ZZ",  # Restriction of Stomach, Open Approach
        "0DV67ZZ",  # Restriction of Stomach, Via Natural or Artificial Opening
        "0DV68ZZ",  # Restriction of Stomach, Via Natural or Artificial Opening Endoscopic
        "0DV70CZ",  # Restriction of Stomach, Pylorus with Extraluminal Device, Open Approach
        "0DV70DZ",  # Restriction of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0DV70ZZ",  # Restriction of Stomach, Pylorus, Open Approach
        "0DV77DZ",  # Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV77ZZ",  # Restriction of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DV78DZ",  # Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV78ZZ",  # Restriction of Stomach, Pylorus, Via Natural or Artificial Opening Endoscopic
        "0DV80CZ",  # Restriction of Small Intestine with Extraluminal Device, Open Approach
        "0DV80DZ",  # Restriction of Small Intestine with Intraluminal Device, Open Approach
        "0DV80ZZ",  # Restriction of Small Intestine, Open Approach
        "0DV87DZ",  # Restriction of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DV87ZZ",  # Restriction of Small Intestine, Via Natural or Artificial Opening
        "0DV88DZ",  # Restriction of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV88ZZ",  # Restriction of Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DV90CZ",  # Restriction of Duodenum with Extraluminal Device, Open Approach
        "0DV90DZ",  # Restriction of Duodenum with Intraluminal Device, Open Approach
        "0DV90ZZ",  # Restriction of Duodenum, Open Approach
        "0DV97DZ",  # Restriction of Duodenum with Intraluminal Device, Via Natural or Artificial Opening
        "0DV97ZZ",  # Restriction of Duodenum, Via Natural or Artificial Opening
        "0DV98DZ",  # Restriction of Duodenum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DV98ZZ",  # Restriction of Duodenum, Via Natural or Artificial Opening Endoscopic
        "0DVA0CZ",  # Restriction of Jejunum with Extraluminal Device, Open Approach
        "0DVA0DZ",  # Restriction of Jejunum with Intraluminal Device, Open Approach
        "0DVA0ZZ",  # Restriction of Jejunum, Open Approach
        "0DVA7DZ",  # Restriction of Jejunum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVA7ZZ",  # Restriction of Jejunum, Via Natural or Artificial Opening
        "0DVA8DZ",  # Restriction of Jejunum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVA8ZZ",  # Restriction of Jejunum, Via Natural or Artificial Opening Endoscopic
        "0DVB0CZ",  # Restriction of Ileum with Extraluminal Device, Open Approach
        "0DVB0DZ",  # Restriction of Ileum with Intraluminal Device, Open Approach
        "0DVB0ZZ",  # Restriction of Ileum, Open Approach
        "0DVB7DZ",  # Restriction of Ileum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVB7ZZ",  # Restriction of Ileum, Via Natural or Artificial Opening
        "0DVB8DZ",  # Restriction of Ileum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVB8ZZ",  # Restriction of Ileum, Via Natural or Artificial Opening Endoscopic
        "0DVC0CZ",  # Restriction of Ileocecal Valve with Extraluminal Device, Open Approach
        "0DVC0DZ",  # Restriction of Ileocecal Valve with Intraluminal Device, Open Approach
        "0DVC0ZZ",  # Restriction of Ileocecal Valve, Open Approach
        "0DVC7DZ",  # Restriction of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening
        "0DVC7ZZ",  # Restriction of Ileocecal Valve, Via Natural or Artificial Opening
        "0DVC8DZ",  # Restriction of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVC8ZZ",  # Restriction of Ileocecal Valve, Via Natural or Artificial Opening Endoscopic
        "0DVE0CZ",  # Restriction of Large Intestine with Extraluminal Device, Open Approach
        "0DVE0DZ",  # Restriction of Large Intestine with Intraluminal Device, Open Approach
        "0DVE0ZZ",  # Restriction of Large Intestine, Open Approach
        "0DVE7DZ",  # Restriction of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVE7ZZ",  # Restriction of Large Intestine, Via Natural or Artificial Opening
        "0DVE8DZ",  # Restriction of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVE8ZZ",  # Restriction of Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DVF0CZ",  # Restriction of Right Large Intestine with Extraluminal Device, Open Approach
        "0DVF0DZ",  # Restriction of Right Large Intestine with Intraluminal Device, Open Approach
        "0DVF0ZZ",  # Restriction of Right Large Intestine, Open Approach
        "0DVF7DZ",  # Restriction of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVF7ZZ",  # Restriction of Right Large Intestine, Via Natural or Artificial Opening
        "0DVF8DZ",  # Restriction of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVF8ZZ",  # Restriction of Right Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DVG0CZ",  # Restriction of Left Large Intestine with Extraluminal Device, Open Approach
        "0DVG0DZ",  # Restriction of Left Large Intestine with Intraluminal Device, Open Approach
        "0DVG0ZZ",  # Restriction of Left Large Intestine, Open Approach
        "0DVG7DZ",  # Restriction of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVG7ZZ",  # Restriction of Left Large Intestine, Via Natural or Artificial Opening
        "0DVG8DZ",  # Restriction of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVG8ZZ",  # Restriction of Left Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DVH0CZ",  # Restriction of Cecum with Extraluminal Device, Open Approach
        "0DVH0DZ",  # Restriction of Cecum with Intraluminal Device, Open Approach
        "0DVH0ZZ",  # Restriction of Cecum, Open Approach
        "0DVH7DZ",  # Restriction of Cecum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVH7ZZ",  # Restriction of Cecum, Via Natural or Artificial Opening
        "0DVH8DZ",  # Restriction of Cecum with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVH8ZZ",  # Restriction of Cecum, Via Natural or Artificial Opening Endoscopic
        "0DVK0CZ",  # Restriction of Ascending Colon with Extraluminal Device, Open Approach
        "0DVK0DZ",  # Restriction of Ascending Colon with Intraluminal Device, Open Approach
        "0DVK0ZZ",  # Restriction of Ascending Colon, Open Approach
        "0DVK7DZ",  # Restriction of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVK7ZZ",  # Restriction of Ascending Colon, Via Natural or Artificial Opening
        "0DVK8DZ",  # Restriction of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVK8ZZ",  # Restriction of Ascending Colon, Via Natural or Artificial Opening Endoscopic
        "0DVL0CZ",  # Restriction of Transverse Colon with Extraluminal Device, Open Approach
        "0DVL0DZ",  # Restriction of Transverse Colon with Intraluminal Device, Open Approach
        "0DVL0ZZ",  # Restriction of Transverse Colon, Open Approach
        "0DVL7DZ",  # Restriction of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVL7ZZ",  # Restriction of Transverse Colon, Via Natural or Artificial Opening
        "0DVL8DZ",  # Restriction of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVL8ZZ",  # Restriction of Transverse Colon, Via Natural or Artificial Opening Endoscopic
        "0DVM0CZ",  # Restriction of Descending Colon with Extraluminal Device, Open Approach
        "0DVM0DZ",  # Restriction of Descending Colon with Intraluminal Device, Open Approach
        "0DVM0ZZ",  # Restriction of Descending Colon, Open Approach
        "0DVM7DZ",  # Restriction of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVM7ZZ",  # Restriction of Descending Colon, Via Natural or Artificial Opening
        "0DVM8DZ",  # Restriction of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVM8ZZ",  # Restriction of Descending Colon, Via Natural or Artificial Opening Endoscopic
        "0DVN0CZ",  # Restriction of Sigmoid Colon with Extraluminal Device, Open Approach
        "0DVN0DZ",  # Restriction of Sigmoid Colon with Intraluminal Device, Open Approach
        "0DVN0ZZ",  # Restriction of Sigmoid Colon, Open Approach
        "0DVN7DZ",  # Restriction of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVN7ZZ",  # Restriction of Sigmoid Colon, Via Natural or Artificial Opening
        "0DVN8DZ",  # Restriction of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0DVN8ZZ",  # Restriction of Sigmoid Colon, Via Natural or Artificial Opening Endoscopic
        "0DW000Z",  # Revision of Drainage Device in Upper Intestinal Tract, Open Approach
        "0DW002Z",  # Revision of Monitoring Device in Upper Intestinal Tract, Open Approach
        "0DW003Z",  # Revision of Infusion Device in Upper Intestinal Tract, Open Approach
        "0DW007Z",  # Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Open Approach
        "0DW00CZ",  # Revision of Extraluminal Device in Upper Intestinal Tract, Open Approach
        "0DW00DZ",  # Revision of Intraluminal Device in Upper Intestinal Tract, Open Approach
        "0DW00JZ",  # Revision of Synthetic Substitute in Upper Intestinal Tract, Open Approach
        "0DW00KZ",  # Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Open Approach
        "0DW00UZ",  # Revision of Feeding Device in Upper Intestinal Tract, Open Approach
        "0DW070Z",  # Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW072Z",  # Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW073Z",  # Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW077Z",  # Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07CZ",  # Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07DZ",  # Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07JZ",  # Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07KZ",  # Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07UZ",  # Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW080Z",  # Revision of Drainage Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW082Z",  # Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW083Z",  # Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW087Z",  # Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW08CZ",  # Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW08DZ",  # Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW08JZ",  # Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW08KZ",  # Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW08UZ",  # Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DW57DZ",  # Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening
        "0DW58DZ",  # Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening Endoscopic
        "0DW600Z",  # Revision of Drainage Device in Stomach, Open Approach
        "0DW602Z",  # Revision of Monitoring Device in Stomach, Open Approach
        "0DW603Z",  # Revision of Infusion Device in Stomach, Open Approach
        "0DW607Z",  # Revision of Autologous Tissue Substitute in Stomach, Open Approach
        "0DW60CZ",  # Revision of Extraluminal Device in Stomach, Open Approach
        "0DW60DZ",  # Revision of Intraluminal Device in Stomach, Open Approach
        "0DW60JZ",  # Revision of Synthetic Substitute in Stomach, Open Approach
        "0DW60KZ",  # Revision of Nonautologous Tissue Substitute in Stomach, Open Approach
        "0DW60MZ",  # Revision of Stimulator Lead in Stomach, Open Approach
        "0DW60UZ",  # Revision of Feeding Device in Stomach, Open Approach
        "0DW670Z",  # Revision of Drainage Device in Stomach, Via Natural or Artificial Opening
        "0DW672Z",  # Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening
        "0DW673Z",  # Revision of Infusion Device in Stomach, Via Natural or Artificial Opening
        "0DW677Z",  # Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening
        "0DW67CZ",  # Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening
        "0DW67DZ",  # Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening
        "0DW67JZ",  # Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening
        "0DW67KZ",  # Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening
        "0DW67UZ",  # Revision of Feeding Device in Stomach, Via Natural or Artificial Opening
        "0DW680Z",  # Revision of Drainage Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW682Z",  # Revision of Monitoring Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW683Z",  # Revision of Infusion Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW687Z",  # Revision of Autologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW68CZ",  # Revision of Extraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW68DZ",  # Revision of Intraluminal Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW68JZ",  # Revision of Synthetic Substitute in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW68KZ",  # Revision of Nonautologous Tissue Substitute in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW68UZ",  # Revision of Feeding Device in Stomach, Via Natural or Artificial Opening Endoscopic
        "0DW807Z",  # Revision of Autologous Tissue Substitute in Small Intestine, Open Approach
        "0DW80JZ",  # Revision of Synthetic Substitute in Small Intestine, Open Approach
        "0DW80KZ",  # Revision of Nonautologous Tissue Substitute in Small Intestine, Open Approach
        "0DW877Z",  # Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening
        "0DW87JZ",  # Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening
        "0DW87KZ",  # Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening
        "0DW887Z",  # Revision of Autologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DW88JZ",  # Revision of Synthetic Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DW88KZ",  # Revision of Nonautologous Tissue Substitute in Small Intestine, Via Natural or Artificial Opening Endoscopic
        "0DWD00Z",  # Revision of Drainage Device in Lower Intestinal Tract, Open Approach
        "0DWD02Z",  # Revision of Monitoring Device in Lower Intestinal Tract, Open Approach
        "0DWD03Z",  # Revision of Infusion Device in Lower Intestinal Tract, Open Approach
        "0DWD07Z",  # Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0CZ",  # Revision of Extraluminal Device in Lower Intestinal Tract, Open Approach
        "0DWD0DZ",  # Revision of Intraluminal Device in Lower Intestinal Tract, Open Approach
        "0DWD0JZ",  # Revision of Synthetic Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0KZ",  # Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0UZ",  # Revision of Feeding Device in Lower Intestinal Tract, Open Approach
        "0DWD70Z",  # Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD72Z",  # Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD73Z",  # Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD77Z",  # Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7CZ",  # Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7DZ",  # Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7JZ",  # Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7KZ",  # Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7UZ",  # Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD80Z",  # Revision of Drainage Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD82Z",  # Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD83Z",  # Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD87Z",  # Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD8CZ",  # Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD8DZ",  # Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD8JZ",  # Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD8KZ",  # Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWD8UZ",  # Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening Endoscopic
        "0DWE07Z",  # Revision of Autologous Tissue Substitute in Large Intestine, Open Approach
        "0DWE0JZ",  # Revision of Synthetic Substitute in Large Intestine, Open Approach
        "0DWE0KZ",  # Revision of Nonautologous Tissue Substitute in Large Intestine, Open Approach
        "0DWE77Z",  # Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWE7JZ",  # Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWE7KZ",  # Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWE87Z",  # Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DWE8JZ",  # Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DWE8KZ",  # Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening Endoscopic
        "0DWU07Z",  # Revision of Autologous Tissue Substitute in Omentum, Open Approach
        "0DWU0JZ",  # Revision of Synthetic Substitute in Omentum, Open Approach
        "0DWU0KZ",  # Revision of Nonautologous Tissue Substitute in Omentum, Open Approach
        "0DWV07Z",  # Revision of Autologous Tissue Substitute in Mesentery, Open Approach
        "0DWV0JZ",  # Revision of Synthetic Substitute in Mesentery, Open Approach
        "0DWV0KZ",  # Revision of Nonautologous Tissue Substitute in Mesentery, Open Approach
        "0DWW00Z",  # Revision of Drainage Device in Peritoneum, Open Approach
        "0DWW07Z",  # Revision of Autologous Tissue Substitute in Peritoneum, Open Approach
        "0DWW0JZ",  # Revision of Synthetic Substitute in Peritoneum, Open Approach
        "0DWW0KZ",  # Revision of Nonautologous Tissue Substitute in Peritoneum, Open Approach
        "0DX60Z5",  # Transfer Stomach to Esophagus, Open Approach
        "0DX80Z5",  # Transfer Small Intestine to Esophagus, Open Approach
        "0DXE0Z5",  # Transfer Large Intestine to Esophagus, Open Approach
        "0DY60Z0",  # Transplantation of Stomach, Allogeneic, Open Approach
        "0DY60Z1",  # Transplantation of Stomach, Syngeneic, Open Approach
        "0DY60Z2",  # Transplantation of Stomach, Zooplastic, Open Approach
        "0F140D3",  # Bypass Gallbladder to Duodenum with Intraluminal Device, Open Approach
        "0F140D4",  # Bypass Gallbladder to Stomach with Intraluminal Device, Open Approach
        "0F140D5",  # Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D6",  # Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D7",  # Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D8",  # Bypass Gallbladder to Cystic Duct with Intraluminal Device, Open Approach
        "0F140D9",  # Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Open Approach
        "0F140DB",  # Bypass Gallbladder to Small Intestine with Intraluminal Device, Open Approach
        "0F140Z3",  # Bypass Gallbladder to Duodenum, Open Approach
        "0F140Z4",  # Bypass Gallbladder to Stomach, Open Approach
        "0F140Z5",  # Bypass Gallbladder to Right Hepatic Duct, Open Approach
        "0F140Z6",  # Bypass Gallbladder to Left Hepatic Duct, Open Approach
        "0F140Z7",  # Bypass Gallbladder to Caudate Hepatic Duct, Open Approach
        "0F140Z8",  # Bypass Gallbladder to Cystic Duct, Open Approach
        "0F140Z9",  # Bypass Gallbladder to Common Bile Duct, Open Approach
        "0F140ZB",  # Bypass Gallbladder to Small Intestine, Open Approach
        "0F150D3",  # Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F150D4",  # Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Open Approach
        "0F150D5",  # Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D6",  # Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D7",  # Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D8",  # Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F150D9",  # Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F150DB",  # Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F150Z3",  # Bypass Right Hepatic Duct to Duodenum, Open Approach
        "0F150Z4",  # Bypass Right Hepatic Duct to Stomach, Open Approach
        "0F150Z5",  # Bypass Right Hepatic Duct to Right Hepatic Duct, Open Approach
        "0F150Z6",  # Bypass Right Hepatic Duct to Left Hepatic Duct, Open Approach
        "0F150Z7",  # Bypass Right Hepatic Duct to Caudate Hepatic Duct, Open Approach
        "0F150Z8",  # Bypass Right Hepatic Duct to Cystic Duct, Open Approach
        "0F150Z9",  # Bypass Right Hepatic Duct to Common Bile Duct, Open Approach
        "0F150ZB",  # Bypass Right Hepatic Duct to Small Intestine, Open Approach
        "0F160D3",  # Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F160D4",  # Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Open Approach
        "0F160D5",  # Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D6",  # Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D7",  # Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D8",  # Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F160D9",  # Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F160DB",  # Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F160Z3",  # Bypass Left Hepatic Duct to Duodenum, Open Approach
        "0F160Z4",  # Bypass Left Hepatic Duct to Stomach, Open Approach
        "0F160Z5",  # Bypass Left Hepatic Duct to Right Hepatic Duct, Open Approach
        "0F160Z6",  # Bypass Left Hepatic Duct to Left Hepatic Duct, Open Approach
        "0F160Z7",  # Bypass Left Hepatic Duct to Caudate Hepatic Duct, Open Approach
        "0F160Z8",  # Bypass Left Hepatic Duct to Cystic Duct, Open Approach
        "0F160Z9",  # Bypass Left Hepatic Duct to Common Bile Duct, Open Approach
        "0F160ZB",  # Bypass Left Hepatic Duct to Small Intestine, Open Approach
        "0F180D3",  # Bypass Cystic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F180D4",  # Bypass Cystic Duct to Stomach with Intraluminal Device, Open Approach
        "0F180D5",  # Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D6",  # Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D7",  # Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D8",  # Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F180D9",  # Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F180DB",  # Bypass Cystic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F180Z3",  # Bypass Cystic Duct to Duodenum, Open Approach
        "0F180Z4",  # Bypass Cystic Duct to Stomach, Open Approach
        "0F180Z5",  # Bypass Cystic Duct to Right Hepatic Duct, Open Approach
        "0F180Z6",  # Bypass Cystic Duct to Left Hepatic Duct, Open Approach
        "0F180Z7",  # Bypass Cystic Duct to Caudate Hepatic Duct, Open Approach
        "0F180Z8",  # Bypass Cystic Duct to Cystic Duct, Open Approach
        "0F180Z9",  # Bypass Cystic Duct to Common Bile Duct, Open Approach
        "0F180ZB",  # Bypass Cystic Duct to Small Intestine, Open Approach
        "0F190D3",  # Bypass Common Bile Duct to Duodenum with Intraluminal Device, Open Approach
        "0F190D4",  # Bypass Common Bile Duct to Stomach with Intraluminal Device, Open Approach
        "0F190D5",  # Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D6",  # Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D7",  # Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D8",  # Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F190D9",  # Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F190DB",  # Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F190Z3",  # Bypass Common Bile Duct to Duodenum, Open Approach
        "0F190Z4",  # Bypass Common Bile Duct to Stomach, Open Approach
        "0F190Z5",  # Bypass Common Bile Duct to Right Hepatic Duct, Open Approach
        "0F190Z6",  # Bypass Common Bile Duct to Left Hepatic Duct, Open Approach
        "0F190Z7",  # Bypass Common Bile Duct to Caudate Hepatic Duct, Open Approach
        "0F190Z8",  # Bypass Common Bile Duct to Cystic Duct, Open Approach
        "0F190Z9",  # Bypass Common Bile Duct to Common Bile Duct, Open Approach
        "0F190ZB",  # Bypass Common Bile Duct to Small Intestine, Open Approach
        "0F1D0D3",  # Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F1D0DB",  # Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F1D0DC",  # Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach
        "0F1D0Z3",  # Bypass Pancreatic Duct to Duodenum, Open Approach
        "0F1D0ZB",  # Bypass Pancreatic Duct to Small Intestine, Open Approach
        "0F1D0ZC",  # Bypass Pancreatic Duct to Large Intestine, Open Approach
        "0F1F0D3",  # Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F1F0DB",  # Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F1F0DC",  # Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach
        "0F1F0Z3",  # Bypass Accessory Pancreatic Duct to Duodenum, Open Approach
        "0F1F0ZB",  # Bypass Accessory Pancreatic Duct to Small Intestine, Open Approach
        "0F1F0ZC",  # Bypass Accessory Pancreatic Duct to Large Intestine, Open Approach
        "0F1G0D3",  # Bypass Pancreas to Duodenum with Intraluminal Device, Open Approach
        "0F1G0DB",  # Bypass Pancreas to Small Intestine with Intraluminal Device, Open Approach
        "0F1G0DC",  # Bypass Pancreas to Large Intestine with Intraluminal Device, Open Approach
        "0F1G0Z3",  # Bypass Pancreas to Duodenum, Open Approach
        "0F1G0ZB",  # Bypass Pancreas to Small Intestine, Open Approach
        "0F1G0ZC",  # Bypass Pancreas to Large Intestine, Open Approach
        "0F500ZZ",  # Destruction of Liver, Open Approach
        "0F510ZZ",  # Destruction of Right Lobe Liver, Open Approach
        "0F520ZZ",  # Destruction of Left Lobe Liver, Open Approach
        "0F550ZZ",  # Destruction of Right Hepatic Duct, Open Approach
        "0F557ZZ",  # Destruction of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F560ZZ",  # Destruction of Left Hepatic Duct, Open Approach
        "0F567ZZ",  # Destruction of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F580ZZ",  # Destruction of Cystic Duct, Open Approach
        "0F587ZZ",  # Destruction of Cystic Duct, Via Natural or Artificial Opening
        "0F590ZZ",  # Destruction of Common Bile Duct, Open Approach
        "0F597ZZ",  # Destruction of Common Bile Duct, Via Natural or Artificial Opening
        "0F5C0ZZ",  # Destruction of Ampulla of Vater, Open Approach
        "0F5C7ZZ",  # Destruction of Ampulla of Vater, Via Natural or Artificial Opening
        "0F5D0ZZ",  # Destruction of Pancreatic Duct, Open Approach
        "0F5D7ZZ",  # Destruction of Pancreatic Duct, Via Natural or Artificial Opening
        "0F5F0ZZ",  # Destruction of Accessory Pancreatic Duct, Open Approach
        "0F5F7ZZ",  # Destruction of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F5G0ZZ",  # Destruction of Pancreas, Open Approach
        "0F750DZ",  # Dilation of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F750ZZ",  # Dilation of Right Hepatic Duct, Open Approach
        "0F757DZ",  # Dilation of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F757ZZ",  # Dilation of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F760DZ",  # Dilation of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F760ZZ",  # Dilation of Left Hepatic Duct, Open Approach
        "0F767DZ",  # Dilation of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F767ZZ",  # Dilation of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F780DZ",  # Dilation of Cystic Duct with Intraluminal Device, Open Approach
        "0F780ZZ",  # Dilation of Cystic Duct, Open Approach
        "0F787DZ",  # Dilation of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F787ZZ",  # Dilation of Cystic Duct, Via Natural or Artificial Opening
        "0F790DZ",  # Dilation of Common Bile Duct with Intraluminal Device, Open Approach
        "0F790ZZ",  # Dilation of Common Bile Duct, Open Approach
        "0F797DZ",  # Dilation of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F797ZZ",  # Dilation of Common Bile Duct, Via Natural or Artificial Opening
        "0F7C0DZ",  # Dilation of Ampulla of Vater with Intraluminal Device, Open Approach
        "0F7C0ZZ",  # Dilation of Ampulla of Vater, Open Approach
        "0F7C7DZ",  # Dilation of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0F7C7ZZ",  # Dilation of Ampulla of Vater, Via Natural or Artificial Opening
        "0F7D0DZ",  # Dilation of Pancreatic Duct with Intraluminal Device, Open Approach
        "0F7D0ZZ",  # Dilation of Pancreatic Duct, Open Approach
        "0F7D7DZ",  # Dilation of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F7D7ZZ",  # Dilation of Pancreatic Duct, Via Natural or Artificial Opening
        "0F7F0DZ",  # Dilation of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0F7F0ZZ",  # Dilation of Accessory Pancreatic Duct, Open Approach
        "0F7F7DZ",  # Dilation of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F7F7ZZ",  # Dilation of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F8G0ZZ",  # Division of Pancreas, Open Approach
        "0F9000Z",  # Drainage of Liver with Drainage Device, Open Approach
        "0F900ZZ",  # Drainage of Liver, Open Approach
        "0F9100Z",  # Drainage of Right Lobe Liver with Drainage Device, Open Approach
        "0F910ZZ",  # Drainage of Right Lobe Liver, Open Approach
        "0F9200Z",  # Drainage of Left Lobe Liver with Drainage Device, Open Approach
        "0F920ZZ",  # Drainage of Left Lobe Liver, Open Approach
        "0F9500Z",  # Drainage of Right Hepatic Duct with Drainage Device, Open Approach
        "0F950ZZ",  # Drainage of Right Hepatic Duct, Open Approach
        "0F9570Z",  # Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F957ZZ",  # Drainage of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F9580Z",  # Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0F958ZZ",  # Drainage of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0F9600Z",  # Drainage of Left Hepatic Duct with Drainage Device, Open Approach
        "0F960ZZ",  # Drainage of Left Hepatic Duct, Open Approach
        "0F9670Z",  # Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F967ZZ",  # Drainage of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F9680Z",  # Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0F968ZZ",  # Drainage of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0F9800Z",  # Drainage of Cystic Duct with Drainage Device, Open Approach
        "0F980ZZ",  # Drainage of Cystic Duct, Open Approach
        "0F9870Z",  # Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F987ZZ",  # Drainage of Cystic Duct, Via Natural or Artificial Opening
        "0F9880Z",  # Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0F988ZZ",  # Drainage of Cystic Duct, Via Natural or Artificial Opening Endoscopic
        "0F9900Z",  # Drainage of Common Bile Duct with Drainage Device, Open Approach
        "0F990ZZ",  # Drainage of Common Bile Duct, Open Approach
        "0F9C00Z",  # Drainage of Ampulla of Vater with Drainage Device, Open Approach
        "0F9C0ZZ",  # Drainage of Ampulla of Vater, Open Approach
        "0F9C70Z",  # Drainage of Ampulla of Vater with Drainage Device, Via Natural or Artificial Opening
        "0F9C7ZZ",  # Drainage of Ampulla of Vater, Via Natural or Artificial Opening
        "0F9D0ZZ",  # Drainage of Pancreatic Duct, Open Approach
        "0F9D7ZZ",  # Drainage of Pancreatic Duct, Via Natural or Artificial Opening
        "0F9D8ZZ",  # Drainage of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0F9F0ZZ",  # Drainage of Accessory Pancreatic Duct, Open Approach
        "0F9F7ZZ",  # Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F9F8ZZ",  # Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0F9G0ZZ",  # Drainage of Pancreas, Open Approach
        "0FB00ZZ",  # Excision of Liver, Open Approach
        "0FB10ZZ",  # Excision of Right Lobe Liver, Open Approach
        "0FB20ZZ",  # Excision of Left Lobe Liver, Open Approach
        "0FB50ZZ",  # Excision of Right Hepatic Duct, Open Approach
        "0FB57ZZ",  # Excision of Right Hepatic Duct, Via Natural or Artificial Opening
        "0FB60ZZ",  # Excision of Left Hepatic Duct, Open Approach
        "0FB67ZZ",  # Excision of Left Hepatic Duct, Via Natural or Artificial Opening
        "0FB80ZZ",  # Excision of Cystic Duct, Open Approach
        "0FB87ZZ",  # Excision of Cystic Duct, Via Natural or Artificial Opening
        "0FB90ZZ",  # Excision of Common Bile Duct, Open Approach
        "0FB97ZZ",  # Excision of Common Bile Duct, Via Natural or Artificial Opening
        "0FBC0ZZ",  # Excision of Ampulla of Vater, Open Approach
        "0FBC7ZZ",  # Excision of Ampulla of Vater, Via Natural or Artificial Opening
        "0FBD0ZZ",  # Excision of Pancreatic Duct, Open Approach
        "0FBD7ZZ",  # Excision of Pancreatic Duct, Via Natural or Artificial Opening
        "0FBF0ZZ",  # Excision of Accessory Pancreatic Duct, Open Approach
        "0FBF7ZZ",  # Excision of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FBG0ZZ",  # Excision of Pancreas, Open Approach
        "0FC00ZZ",  # Extirpation of Matter from Liver, Open Approach
        "0FC10ZZ",  # Extirpation of Matter from Right Lobe Liver, Open Approach
        "0FC20ZZ",  # Extirpation of Matter from Left Lobe Liver, Open Approach
        "0FC50ZZ",  # Extirpation of Matter from Right Hepatic Duct, Open Approach
        "0FC60ZZ",  # Extirpation of Matter from Left Hepatic Duct, Open Approach
        "0FC80ZZ",  # Extirpation of Matter from Cystic Duct, Open Approach
        "0FC90ZZ",  # Extirpation of Matter from Common Bile Duct, Open Approach
        "0FCC0ZZ",  # Extirpation of Matter from Ampulla of Vater, Open Approach
        "0FCC7ZZ",  # Extirpation of Matter from Ampulla of Vater, Via Natural or Artificial Opening
        "0FCD0ZZ",  # Extirpation of Matter from Pancreatic Duct, Open Approach
        "0FCD7ZZ",  # Extirpation of Matter from Pancreatic Duct, Via Natural or Artificial Opening
        "0FCF0ZZ",  # Extirpation of Matter from Accessory Pancreatic Duct, Open Approach
        "0FCF7ZZ",  # Extirpation of Matter from Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FCG0ZZ",  # Extirpation of Matter from Pancreas, Open Approach
        "0FF50ZZ",  # Fragmentation in Right Hepatic Duct, Open Approach
        "0FF57ZZ",  # Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening
        "0FF60ZZ",  # Fragmentation in Left Hepatic Duct, Open Approach
        "0FF67ZZ",  # Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening
        "0FF80ZZ",  # Fragmentation in Cystic Duct, Open Approach
        "0FF87ZZ",  # Fragmentation in Cystic Duct, Via Natural or Artificial Opening
        "0FF90ZZ",  # Fragmentation in Common Bile Duct, Open Approach
        "0FF97ZZ",  # Fragmentation in Common Bile Duct, Via Natural or Artificial Opening
        "0FFC0ZZ",  # Fragmentation in Ampulla of Vater, Open Approach
        "0FFC7ZZ",  # Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening
        "0FFD0ZZ",  # Fragmentation in Pancreatic Duct, Open Approach
        "0FFD7ZZ",  # Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening
        "0FFD8ZZ",  # Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FFF0ZZ",  # Fragmentation in Accessory Pancreatic Duct, Open Approach
        "0FFF7ZZ",  # Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FFF8ZZ",  # Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FH002Z",  # Insertion of Monitoring Device into Liver, Open Approach
        "0FH102Z",  # Insertion of Monitoring Device into Right Lobe Liver, Open Approach
        "0FH202Z",  # Insertion of Monitoring Device into Left Lobe Liver, Open Approach
        "0FHB02Z",  # Insertion of Monitoring Device into Hepatobiliary Duct, Open Approach
        "0FHB0DZ",  # Insertion of Intraluminal Device into Hepatobiliary Duct, Open Approach
        "0FHB72Z",  # Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FHB7DZ",  # Insertion of Intraluminal Device into Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FHB82Z",  # Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FHD02Z",  # Insertion of Monitoring Device into Pancreatic Duct, Open Approach
        "0FHD0DZ",  # Insertion of Intraluminal Device into Pancreatic Duct, Open Approach
        "0FHD72Z",  # Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening
        "0FHD7DZ",  # Insertion of Intraluminal Device into Pancreatic Duct, Via Natural or Artificial Opening
        "0FHD82Z",  # Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FHG02Z",  # Insertion of Monitoring Device into Pancreas, Open Approach
        "0FJ00ZZ",  # Inspection of Liver, Open Approach
        "0FJB0ZZ",  # Inspection of Hepatobiliary Duct, Open Approach
        "0FJB7ZZ",  # Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FJB8ZZ",  # Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FL50CZ",  # Occlusion of Right Hepatic Duct with Extraluminal Device, Open Approach
        "0FL50DZ",  # Occlusion of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0FL50ZZ",  # Occlusion of Right Hepatic Duct, Open Approach
        "0FL60CZ",  # Occlusion of Left Hepatic Duct with Extraluminal Device, Open Approach
        "0FL60DZ",  # Occlusion of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0FL60ZZ",  # Occlusion of Left Hepatic Duct, Open Approach
        "0FL80CZ",  # Occlusion of Cystic Duct with Extraluminal Device, Open Approach
        "0FL80DZ",  # Occlusion of Cystic Duct with Intraluminal Device, Open Approach
        "0FL80ZZ",  # Occlusion of Cystic Duct, Open Approach
        "0FL90CZ",  # Occlusion of Common Bile Duct with Extraluminal Device, Open Approach
        "0FL90DZ",  # Occlusion of Common Bile Duct with Intraluminal Device, Open Approach
        "0FL90ZZ",  # Occlusion of Common Bile Duct, Open Approach
        "0FLC0CZ",  # Occlusion of Ampulla of Vater with Extraluminal Device, Open Approach
        "0FLC0DZ",  # Occlusion of Ampulla of Vater with Intraluminal Device, Open Approach
        "0FLC0ZZ",  # Occlusion of Ampulla of Vater, Open Approach
        "0FLC7DZ",  # Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0FLC7ZZ",  # Occlusion of Ampulla of Vater, Via Natural or Artificial Opening
        "0FLC8DZ",  # Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FLC8ZZ",  # Occlusion of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic
        "0FLD0CZ",  # Occlusion of Pancreatic Duct with Extraluminal Device, Open Approach
        "0FLD0DZ",  # Occlusion of Pancreatic Duct with Intraluminal Device, Open Approach
        "0FLD0ZZ",  # Occlusion of Pancreatic Duct, Open Approach
        "0FLD7DZ",  # Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FLD7ZZ",  # Occlusion of Pancreatic Duct, Via Natural or Artificial Opening
        "0FLD8DZ",  # Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FLD8ZZ",  # Occlusion of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FLF0CZ",  # Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Open Approach
        "0FLF0DZ",  # Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0FLF0ZZ",  # Occlusion of Accessory Pancreatic Duct, Open Approach
        "0FLF7DZ",  # Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FLF7ZZ",  # Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FLF8DZ",  # Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FLF8ZZ",  # Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FM40ZZ",  # Reattachment of Gallbladder, Open Approach
        "0FM50ZZ",  # Reattachment of Right Hepatic Duct, Open Approach
        "0FM60ZZ",  # Reattachment of Left Hepatic Duct, Open Approach
        "0FM80ZZ",  # Reattachment of Cystic Duct, Open Approach
        "0FM90ZZ",  # Reattachment of Common Bile Duct, Open Approach
        "0FMC0ZZ",  # Reattachment of Ampulla of Vater, Open Approach
        "0FMD0ZZ",  # Reattachment of Pancreatic Duct, Open Approach
        "0FMF0ZZ",  # Reattachment of Accessory Pancreatic Duct, Open Approach
        "0FMG0ZZ",  # Reattachment of Pancreas, Open Approach
        "0FN00ZZ",  # Release Liver, Open Approach
        "0FN10ZZ",  # Release Right Lobe Liver, Open Approach
        "0FN20ZZ",  # Release Left Lobe Liver, Open Approach
        "0FN40ZZ",  # Release Gallbladder, Open Approach
        "0FN50ZZ",  # Release Right Hepatic Duct, Open Approach
        "0FN57ZZ",  # Release Right Hepatic Duct, Via Natural or Artificial Opening
        "0FN58ZZ",  # Release Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FN60ZZ",  # Release Left Hepatic Duct, Open Approach
        "0FN67ZZ",  # Release Left Hepatic Duct, Via Natural or Artificial Opening
        "0FN68ZZ",  # Release Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FN80ZZ",  # Release Cystic Duct, Open Approach
        "0FN87ZZ",  # Release Cystic Duct, Via Natural or Artificial Opening
        "0FN88ZZ",  # Release Cystic Duct, Via Natural or Artificial Opening Endoscopic
        "0FN90ZZ",  # Release Common Bile Duct, Open Approach
        "0FN97ZZ",  # Release Common Bile Duct, Via Natural or Artificial Opening
        "0FN98ZZ",  # Release Common Bile Duct, Via Natural or Artificial Opening Endoscopic
        "0FNC0ZZ",  # Release Ampulla of Vater, Open Approach
        "0FNC7ZZ",  # Release Ampulla of Vater, Via Natural or Artificial Opening
        "0FNC8ZZ",  # Release Ampulla of Vater, Via Natural or Artificial Opening Endoscopic
        "0FND0ZZ",  # Release Pancreatic Duct, Open Approach
        "0FND7ZZ",  # Release Pancreatic Duct, Via Natural or Artificial Opening
        "0FND8ZZ",  # Release Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FNF0ZZ",  # Release Accessory Pancreatic Duct, Open Approach
        "0FNF7ZZ",  # Release Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FNF8ZZ",  # Release Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FNG0ZZ",  # Release Pancreas, Open Approach
        "0FP000Z",  # Removal of Drainage Device from Liver, Open Approach
        "0FP002Z",  # Removal of Monitoring Device from Liver, Open Approach
        "0FP003Z",  # Removal of Infusion Device from Liver, Open Approach
        "0FP40DZ",  # Removal of Intraluminal Device from Gallbladder, Open Approach
        "0FPB00Z",  # Removal of Drainage Device from Hepatobiliary Duct, Open Approach
        "0FPB01Z",  # Removal of Radioactive Element from Hepatobiliary Duct, Open Approach
        "0FPB02Z",  # Removal of Monitoring Device from Hepatobiliary Duct, Open Approach
        "0FPB03Z",  # Removal of Infusion Device from Hepatobiliary Duct, Open Approach
        "0FPB07Z",  # Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Open Approach
        "0FPB0CZ",  # Removal of Extraluminal Device from Hepatobiliary Duct, Open Approach
        "0FPB0DZ",  # Removal of Intraluminal Device from Hepatobiliary Duct, Open Approach
        "0FPB0JZ",  # Removal of Synthetic Substitute from Hepatobiliary Duct, Open Approach
        "0FPB0KZ",  # Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Open Approach
        "0FPB70Z",  # Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB71Z",  # Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB72Z",  # Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB73Z",  # Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB77Z",  # Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7CZ",  # Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7DZ",  # Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7JZ",  # Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7KZ",  # Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB80Z",  # Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB81Z",  # Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB82Z",  # Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB83Z",  # Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB87Z",  # Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB8CZ",  # Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB8DZ",  # Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB8JZ",  # Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPB8KZ",  # Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD00Z",  # Removal of Drainage Device from Pancreatic Duct, Open Approach
        "0FPD01Z",  # Removal of Radioactive Element from Pancreatic Duct, Open Approach
        "0FPD02Z",  # Removal of Monitoring Device from Pancreatic Duct, Open Approach
        "0FPD03Z",  # Removal of Infusion Device from Pancreatic Duct, Open Approach
        "0FPD07Z",  # Removal of Autologous Tissue Substitute from Pancreatic Duct, Open Approach
        "0FPD0CZ",  # Removal of Extraluminal Device from Pancreatic Duct, Open Approach
        "0FPD0DZ",  # Removal of Intraluminal Device from Pancreatic Duct, Open Approach
        "0FPD0JZ",  # Removal of Synthetic Substitute from Pancreatic Duct, Open Approach
        "0FPD0KZ",  # Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Open Approach
        "0FPD70Z",  # Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD71Z",  # Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD72Z",  # Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD73Z",  # Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD77Z",  # Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7CZ",  # Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7DZ",  # Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7JZ",  # Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7KZ",  # Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD80Z",  # Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD81Z",  # Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD82Z",  # Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD83Z",  # Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD87Z",  # Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD8CZ",  # Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD8DZ",  # Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD8JZ",  # Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPD8KZ",  # Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FPG00Z",  # Removal of Drainage Device from Pancreas, Open Approach
        "0FPG02Z",  # Removal of Monitoring Device from Pancreas, Open Approach
        "0FPG03Z",  # Removal of Infusion Device from Pancreas, Open Approach
        "0FPG0DZ",  # Removal of Intraluminal Device from Pancreas, Open Approach
        "0FPGXDZ",  # Removal of Intraluminal Device from Pancreas, External Approach
        "0FQ40ZZ",  # Repair Gallbladder, Open Approach
        "0FQ50ZZ",  # Repair Right Hepatic Duct, Open Approach
        "0FQ57ZZ",  # Repair Right Hepatic Duct, Via Natural or Artificial Opening
        "0FQ58ZZ",  # Repair Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FQ60ZZ",  # Repair Left Hepatic Duct, Open Approach
        "0FQ67ZZ",  # Repair Left Hepatic Duct, Via Natural or Artificial Opening
        "0FQ68ZZ",  # Repair Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FQ80ZZ",  # Repair Cystic Duct, Open Approach
        "0FQ87ZZ",  # Repair Cystic Duct, Via Natural or Artificial Opening
        "0FQ88ZZ",  # Repair Cystic Duct, Via Natural or Artificial Opening Endoscopic
        "0FQ90ZZ",  # Repair Common Bile Duct, Open Approach
        "0FQ97ZZ",  # Repair Common Bile Duct, Via Natural or Artificial Opening
        "0FQ98ZZ",  # Repair Common Bile Duct, Via Natural or Artificial Opening Endoscopic
        "0FQC0ZZ",  # Repair Ampulla of Vater, Open Approach
        "0FQC7ZZ",  # Repair Ampulla of Vater, Via Natural or Artificial Opening
        "0FQC8ZZ",  # Repair Ampulla of Vater, Via Natural or Artificial Opening Endoscopic
        "0FQD0ZZ",  # Repair Pancreatic Duct, Open Approach
        "0FQD7ZZ",  # Repair Pancreatic Duct, Via Natural or Artificial Opening
        "0FQD8ZZ",  # Repair Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FQF0ZZ",  # Repair Accessory Pancreatic Duct, Open Approach
        "0FQF7ZZ",  # Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FQF8ZZ",  # Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FQG0ZZ",  # Repair Pancreas, Open Approach
        "0FR507Z",  # Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FR50JZ",  # Replacement of Right Hepatic Duct with Synthetic Substitute, Open Approach
        "0FR50KZ",  # Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR607Z",  # Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FR60JZ",  # Replacement of Left Hepatic Duct with Synthetic Substitute, Open Approach
        "0FR60KZ",  # Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR807Z",  # Replacement of Cystic Duct with Autologous Tissue Substitute, Open Approach
        "0FR80JZ",  # Replacement of Cystic Duct with Synthetic Substitute, Open Approach
        "0FR80KZ",  # Replacement of Cystic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR907Z",  # Replacement of Common Bile Duct with Autologous Tissue Substitute, Open Approach
        "0FR90JZ",  # Replacement of Common Bile Duct with Synthetic Substitute, Open Approach
        "0FR90KZ",  # Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Open Approach
        "0FRC07Z",  # Replacement of Ampulla of Vater with Autologous Tissue Substitute, Open Approach
        "0FRC0JZ",  # Replacement of Ampulla of Vater with Synthetic Substitute, Open Approach
        "0FRC0KZ",  # Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach
        "0FRD07Z",  # Replacement of Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FRD0JZ",  # Replacement of Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FRD0KZ",  # Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FRF07Z",  # Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FRF0JZ",  # Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FRF0KZ",  # Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FS40ZZ",  # Reposition Gallbladder, Open Approach
        "0FS50ZZ",  # Reposition Right Hepatic Duct, Open Approach
        "0FS60ZZ",  # Reposition Left Hepatic Duct, Open Approach
        "0FS80ZZ",  # Reposition Cystic Duct, Open Approach
        "0FS90ZZ",  # Reposition Common Bile Duct, Open Approach
        "0FSC0ZZ",  # Reposition Ampulla of Vater, Open Approach
        "0FSD0ZZ",  # Reposition Pancreatic Duct, Open Approach
        "0FSF0ZZ",  # Reposition Accessory Pancreatic Duct, Open Approach
        "0FT10ZZ",  # Resection of Right Lobe Liver, Open Approach
        "0FT20ZZ",  # Resection of Left Lobe Liver, Open Approach
        "0FT50ZZ",  # Resection of Right Hepatic Duct, Open Approach
        "0FT57ZZ",  # Resection of Right Hepatic Duct, Via Natural or Artificial Opening
        "0FT58ZZ",  # Resection of Right Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FT60ZZ",  # Resection of Left Hepatic Duct, Open Approach
        "0FT67ZZ",  # Resection of Left Hepatic Duct, Via Natural or Artificial Opening
        "0FT68ZZ",  # Resection of Left Hepatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FT80ZZ",  # Resection of Cystic Duct, Open Approach
        "0FT87ZZ",  # Resection of Cystic Duct, Via Natural or Artificial Opening
        "0FT88ZZ",  # Resection of Cystic Duct, Via Natural or Artificial Opening Endoscopic
        "0FT90ZZ",  # Resection of Common Bile Duct, Open Approach
        "0FT97ZZ",  # Resection of Common Bile Duct, Via Natural or Artificial Opening
        "0FT98ZZ",  # Resection of Common Bile Duct, Via Natural or Artificial Opening Endoscopic
        "0FTC0ZZ",  # Resection of Ampulla of Vater, Open Approach
        "0FTC7ZZ",  # Resection of Ampulla of Vater, Via Natural or Artificial Opening
        "0FTC8ZZ",  # Resection of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic
        "0FTD0ZZ",  # Resection of Pancreatic Duct, Open Approach
        "0FTD7ZZ",  # Resection of Pancreatic Duct, Via Natural or Artificial Opening
        "0FTF0ZZ",  # Resection of Accessory Pancreatic Duct, Open Approach
        "0FTF7ZZ",  # Resection of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FTG0ZZ",  # Resection of Pancreas, Open Approach
        "0FU507Z",  # Supplement Right Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FU50JZ",  # Supplement Right Hepatic Duct with Synthetic Substitute, Open Approach
        "0FU50KZ",  # Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU607Z",  # Supplement Left Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FU60JZ",  # Supplement Left Hepatic Duct with Synthetic Substitute, Open Approach
        "0FU60KZ",  # Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU807Z",  # Supplement Cystic Duct with Autologous Tissue Substitute, Open Approach
        "0FU80JZ",  # Supplement Cystic Duct with Synthetic Substitute, Open Approach
        "0FU80KZ",  # Supplement Cystic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU907Z",  # Supplement Common Bile Duct with Autologous Tissue Substitute, Open Approach
        "0FU90JZ",  # Supplement Common Bile Duct with Synthetic Substitute, Open Approach
        "0FU90KZ",  # Supplement Common Bile Duct with Nonautologous Tissue Substitute, Open Approach
        "0FUC07Z",  # Supplement Ampulla of Vater with Autologous Tissue Substitute, Open Approach
        "0FUC0JZ",  # Supplement Ampulla of Vater with Synthetic Substitute, Open Approach
        "0FUC0KZ",  # Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach
        "0FUD07Z",  # Supplement Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FUD0JZ",  # Supplement Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FUD0KZ",  # Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FUF07Z",  # Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FUF0JZ",  # Supplement Accessory Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FUF0KZ",  # Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FV50CZ",  # Restriction of Right Hepatic Duct with Extraluminal Device, Open Approach
        "0FV50DZ",  # Restriction of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0FV50ZZ",  # Restriction of Right Hepatic Duct, Open Approach
        "0FV60CZ",  # Restriction of Left Hepatic Duct with Extraluminal Device, Open Approach
        "0FV60DZ",  # Restriction of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0FV60ZZ",  # Restriction of Left Hepatic Duct, Open Approach
        "0FV80CZ",  # Restriction of Cystic Duct with Extraluminal Device, Open Approach
        "0FV80DZ",  # Restriction of Cystic Duct with Intraluminal Device, Open Approach
        "0FV80ZZ",  # Restriction of Cystic Duct, Open Approach
        "0FV90CZ",  # Restriction of Common Bile Duct with Extraluminal Device, Open Approach
        "0FV90DZ",  # Restriction of Common Bile Duct with Intraluminal Device, Open Approach
        "0FV90ZZ",  # Restriction of Common Bile Duct, Open Approach
        "0FVC0CZ",  # Restriction of Ampulla of Vater with Extraluminal Device, Open Approach
        "0FVC0DZ",  # Restriction of Ampulla of Vater with Intraluminal Device, Open Approach
        "0FVC0ZZ",  # Restriction of Ampulla of Vater, Open Approach
        "0FVC7DZ",  # Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0FVC7ZZ",  # Restriction of Ampulla of Vater, Via Natural or Artificial Opening
        "0FVC8DZ",  # Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FVC8ZZ",  # Restriction of Ampulla of Vater, Via Natural or Artificial Opening Endoscopic
        "0FVD0CZ",  # Restriction of Pancreatic Duct with Extraluminal Device, Open Approach
        "0FVD0DZ",  # Restriction of Pancreatic Duct with Intraluminal Device, Open Approach
        "0FVD0ZZ",  # Restriction of Pancreatic Duct, Open Approach
        "0FVD7DZ",  # Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FVD7ZZ",  # Restriction of Pancreatic Duct, Via Natural or Artificial Opening
        "0FVD8DZ",  # Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FVD8ZZ",  # Restriction of Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FVF0CZ",  # Restriction of Accessory Pancreatic Duct with Extraluminal Device, Open Approach
        "0FVF0DZ",  # Restriction of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0FVF0ZZ",  # Restriction of Accessory Pancreatic Duct, Open Approach
        "0FVF7DZ",  # Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FVF7ZZ",  # Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FVF8DZ",  # Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0FVF8ZZ",  # Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FW000Z",  # Revision of Drainage Device in Liver, Open Approach
        "0FW002Z",  # Revision of Monitoring Device in Liver, Open Approach
        "0FW003Z",  # Revision of Infusion Device in Liver, Open Approach
        "0FWB00Z",  # Revision of Drainage Device in Hepatobiliary Duct, Open Approach
        "0FWB02Z",  # Revision of Monitoring Device in Hepatobiliary Duct, Open Approach
        "0FWB03Z",  # Revision of Infusion Device in Hepatobiliary Duct, Open Approach
        "0FWB07Z",  # Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Open Approach
        "0FWB0CZ",  # Revision of Extraluminal Device in Hepatobiliary Duct, Open Approach
        "0FWB0DZ",  # Revision of Intraluminal Device in Hepatobiliary Duct, Open Approach
        "0FWB0JZ",  # Revision of Synthetic Substitute in Hepatobiliary Duct, Open Approach
        "0FWB0KZ",  # Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Open Approach
        "0FWB70Z",  # Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB72Z",  # Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB73Z",  # Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB77Z",  # Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7CZ",  # Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7DZ",  # Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7JZ",  # Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7KZ",  # Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB80Z",  # Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB82Z",  # Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB83Z",  # Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB87Z",  # Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB8CZ",  # Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB8DZ",  # Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB8JZ",  # Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWB8KZ",  # Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD00Z",  # Revision of Drainage Device in Pancreatic Duct, Open Approach
        "0FWD02Z",  # Revision of Monitoring Device in Pancreatic Duct, Open Approach
        "0FWD03Z",  # Revision of Infusion Device in Pancreatic Duct, Open Approach
        "0FWD07Z",  # Revision of Autologous Tissue Substitute in Pancreatic Duct, Open Approach
        "0FWD0CZ",  # Revision of Extraluminal Device in Pancreatic Duct, Open Approach
        "0FWD0DZ",  # Revision of Intraluminal Device in Pancreatic Duct, Open Approach
        "0FWD0JZ",  # Revision of Synthetic Substitute in Pancreatic Duct, Open Approach
        "0FWD0KZ",  # Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Open Approach
        "0FWD70Z",  # Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD72Z",  # Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD73Z",  # Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD77Z",  # Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7CZ",  # Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7DZ",  # Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7JZ",  # Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7KZ",  # Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD80Z",  # Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD82Z",  # Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD83Z",  # Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD87Z",  # Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD8CZ",  # Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD8DZ",  # Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD8JZ",  # Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWD8KZ",  # Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening Endoscopic
        "0FWG00Z",  # Revision of Drainage Device in Pancreas, Open Approach
        "0FWG02Z",  # Revision of Monitoring Device in Pancreas, Open Approach
        "0FWG03Z",  # Revision of Infusion Device in Pancreas, Open Approach
        "0FWG0DZ",  # Revision of Intraluminal Device in Pancreas, Open Approach
        "0HQ4XZZ",  # Repair Neck Skin, External Approach
        "0HQ5XZZ",  # Repair Chest Skin, External Approach
        "0TRB07Z",  # Replacement of Bladder with Autologous Tissue Substitute, Open Approach
        "0W1G0JY",  # Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Open Approach
        "0W3G0ZZ",  # Control Bleeding in Peritoneal Cavity, Open Approach
        "0W3H0ZZ",  # Control Bleeding in Retroperitoneum, Open Approach
        "0W3P0ZZ",  # Control Bleeding in Gastrointestinal Tract, Open Approach
        "0W9G00Z",  # Drainage of Peritoneal Cavity with Drainage Device, Open Approach
        "0W9G0ZZ",  # Drainage of Peritoneal Cavity, Open Approach
        "0W9J00Z",  # Drainage of Pelvic Cavity with Drainage Device, Open Approach
        "0W9J0ZZ",  # Drainage of Pelvic Cavity, Open Approach
        "0WCG0ZZ",  # Extirpation of Matter from Peritoneal Cavity, Open Approach
        "0WCJ0ZZ",  # Extirpation of Matter from Pelvic Cavity, Open Approach
        "0WCP0ZZ",  # Extirpation of Matter from Gastrointestinal Tract, Open Approach
        "0WCR0ZZ",  # Extirpation of Matter from Genitourinary Tract, Open Approach
        "0WFG0ZZ",  # Fragmentation in Peritoneal Cavity, Open Approach
        "0WHG03Z",  # Insertion of Infusion Device into Peritoneal Cavity, Open Approach
        "0WHG0YZ",  # Insertion of Other Device into Peritoneal Cavity, Open Approach
        "0WHH03Z",  # Insertion of Infusion Device into Retroperitoneum, Open Approach
        "0WHH0YZ",  # Insertion of Other Device into Retroperitoneum, Open Approach
        "0WHJ03Z",  # Insertion of Infusion Device into Pelvic Cavity, Open Approach
        "0WHJ0YZ",  # Insertion of Other Device into Pelvic Cavity, Open Approach
        "0WHP03Z",  # Insertion of Infusion Device into Gastrointestinal Tract, Open Approach
        "0WJG0ZZ",  # Inspection of Peritoneal Cavity, Open Approach
        "0WJJ0ZZ",  # Inspection of Pelvic Cavity, Open Approach
        "0WJP0ZZ",  # Inspection of Gastrointestinal Tract, Open Approach
        "0WJR0ZZ",  # Inspection of Genitourinary Tract, Open Approach
        "0WMF0ZZ",  # Reattachment of Abdominal Wall, Open Approach
        "0WPG00Z",  # Removal of Drainage Device from Peritoneal Cavity, Open Approach
        "0WPG01Z",  # Removal of Radioactive Element from Peritoneal Cavity, Open Approach
        "0WPG03Z",  # Removal of Infusion Device from Peritoneal Cavity, Open Approach
        "0WPG0JZ",  # Removal of Synthetic Substitute from Peritoneal Cavity, Open Approach
        "0WPG0YZ",  # Removal of Other Device from Peritoneal Cavity, Open Approach
        "0WPH00Z",  # Removal of Drainage Device from Retroperitoneum, Open Approach
        "0WPH01Z",  # Removal of Radioactive Element from Retroperitoneum, Open Approach
        "0WPH03Z",  # Removal of Infusion Device from Retroperitoneum, Open Approach
        "0WPH0YZ",  # Removal of Other Device from Retroperitoneum, Open Approach
        "0WPP01Z",  # Removal of Radioactive Element from Gastrointestinal Tract, Open Approach
        "0WPP03Z",  # Removal of Infusion Device from Gastrointestinal Tract, Open Approach
        "0WPP0YZ",  # Removal of Other Device from Gastrointestinal Tract, Open Approach
        "0WQ6XZ2",  # Repair Neck, Stoma, External Approach
        "0WQF0ZZ",  # Repair Abdominal Wall, Open Approach
        "0WQFXZ2",  # Repair Abdominal Wall, Stoma, External Approach
        "0WQFXZZ",  # Repair Abdominal Wall, External Approach
        "0WWG00Z",  # Revision of Drainage Device in Peritoneal Cavity, Open Approach
        "0WWG01Z",  # Revision of Radioactive Element in Peritoneal Cavity, Open Approach
        "0WWG03Z",  # Revision of Infusion Device in Peritoneal Cavity, Open Approach
        "0WWG0JZ",  # Revision of Synthetic Substitute in Peritoneal Cavity, Open Approach
        "0WWG0YZ",  # Revision of Other Device in Peritoneal Cavity, Open Approach
        "0WWH00Z",  # Revision of Drainage Device in Retroperitoneum, Open Approach
        "0WWH01Z",  # Revision of Radioactive Element in Retroperitoneum, Open Approach
        "0WWH03Z",  # Revision of Infusion Device in Retroperitoneum, Open Approach
        "0WWH0YZ",  # Revision of Other Device in Retroperitoneum, Open Approach
        "0WWJ00Z",  # Revision of Drainage Device in Pelvic Cavity, Open Approach
        "0WWJ01Z",  # Revision of Radioactive Element in Pelvic Cavity, Open Approach
        "0WWJ03Z",  # Revision of Infusion Device in Pelvic Cavity, Open Approach
        "0WWJ0JZ",  # Revision of Synthetic Substitute in Pelvic Cavity, Open Approach
        "0WWJ0YZ",  # Revision of Other Device in Pelvic Cavity, Open Approach
        "0WWP01Z",  # Revision of Radioactive Element in Gastrointestinal Tract, Open Approach
        "0WWP03Z",  # Revision of Infusion Device in Gastrointestinal Tract, Open Approach
        "0WWP0YZ",  # Revision of Other Device in Gastrointestinal Tract, Open Approach
    }

    SNOMEDCT = {
        "10002003",  # Resection of stomach fundus (procedure)
        "10107006",  # Duodenoduodenostomy (procedure)
        "10254002",  # Excision of diverticulum of small intestine (procedure)
        "10259007",  # Proctopexy (procedure)
        "10272008",  # Closure of fistula of diaphragm (procedure)
        "10388006",  # Manipulation of tissue of diaphragm (procedure)
        "1054002",  # Incision of sphincter of Oddi (procedure)
        "10590008",  # Fulguration of lesion of large intestine (procedure)
        "107938000",  # Small intestine excision (procedure)
        "107944001",  # Large intestine excision (procedure)
        "108184008",  # Diaphragm excision (procedure)
        "10869008",  # Hemigastrectomy with vagotomy (procedure)
        "10985002",  # Gallbladder to intestine anastomosis (procedure)
        "11127003",  # Esophagogastrostomy, antesternal or antethoracic (procedure)
        "11236009",  # Near-total pancreatectomy with preservation of duodenum (procedure)
        "112860004",  # Hemigastrectomy by abdominal approach (procedure)
        "112864008",  # Incision and exploration of jejunum (procedure)
        "112871003",  # Anastomosis of large intestine to large intestine (procedure)
        "112873000",  # Closure of perineosigmoidal fistula (procedure)
        "112875007",  # Removal of foreign body of rectum by incision (procedure)
        "112877004",  # Frickman operation, abdominal proctopexy (procedure)
        "112883001",  # Removal of foreign body from liver (procedure)
        "112884007",  # Cholecystoduodenostomy (procedure)
        "112977000",  # Repair of diaphragmatic hernia (procedure)
        "112982007",  # Lysis of adhesions of stomach (procedure)
        "11574002",  # Creation of endorectal ileal H-pouch with anastomosis to anus (procedure)
        "116026007",  # Total jejunectomy (procedure)
        "116027003",  # Partial jejunectomy (procedure)
        "116165008",  # Bronchial sleeve resection with pulmonary resection (procedure)
        "116175006",  # Proximal subtotal gastrectomy (procedure)
        "116215005",  # Proximal subtotal pancreatectomy (procedure)
        "116241004",  # Pancreaticoduodenectomy (procedure)
        "116242006",  # Pylorus-sparing pancreaticoduodenectomy (procedure)
        "11626008",  # Miles operation, complete proctectomy (procedure)
        "11682008",  # Jejunoileostomy (procedure)
        "119288004",  # Excision of ampulla of Vater (procedure)
        "119746007",  # Lung excision (procedure)
        "119879002",  # Esophagus closure (procedure)
        "119880004",  # Esophagus reconstruction (procedure)
        "119883002",  # Stomach closure (procedure)
        "119887001",  # Small intestine closure (procedure)
        "119888006",  # Small intestine reconstruction (procedure)
        "119892004",  # Large intestine closure (procedure)
        "119915002",  # Biliary tract incision (procedure)
        "119916001",  # Biliary tract excision (procedure)
        "119925007",  # Biliary tract closure (procedure)
        "119926008",  # Biliary tract reconstruction (procedure)
        "119927004",  # Gallbladder reconstruction (procedure)
        "119928009",  # Biliary tract repair (procedure)
        "119936000",  # Pancreas closure (procedure)
        "119937009",  # Pancreas reconstruction (procedure)
        "120167008",  # Diaphragm closure (procedure)
        "120168003",  # Diaphragm transposition (procedure)
        "120183008",  # Peritoneum closure (procedure)
        "120184002",  # Mesentery closure (procedure)
        "120186000",  # Peritoneum reconstruction (procedure)
        "120188004",  # Mesentery reconstruction (procedure)
        "12074006",  # Ligation of gastric varices (procedure)
        "12092003",  # Repair of rectal prolapse (procedure)
        "12102007",  # Inversion of diverticulum of large intestine (procedure)
        "12324002",  # Exploratory laparotomy with biopsy (procedure)
        "12421006",  # Closure of esophagostomy by thoracic approach (procedure)
        "12666009",  # Cholecystocolostomy (procedure)
        "12827003",  # Excision of part of rectum (procedure)
        "128399009",  # Anastomosis of gall bladder (procedure)
        "128400002",  # Anastomosis of stomach (procedure)
        "128401003",  # Anastomosis of duodenum (procedure)
        "128406008",  # Anastomosis of jejunum (procedure)
        "12959006",  # Creation of subcutaneous tunnel with esophageal anastomosis (procedure)
        "1299162005",  # Destruction of lesion of large intestine using laser (procedure)
        "13327002",  # Partial colectomy by transanal approach (procedure)
        "1337002",  # Colosigmoidostomy (procedure)
        "133871003",  # Cecosigmoidostomy (procedure)
        "13403003",  # Pharyngoesophageal repair (procedure)
        "13811001",  # Take-down of gastric anastomosis (procedure)
        "1390003",  # Laparoscopic-assisted sigmoid colectomy (procedure)
        "14072009",  # Total esophagectomy (procedure)
        "14335006",  # Closure of esophageal fistula by thoracic approach (procedure)
        "14352005",  # Incision and drainage of pseudocyst of pancreas (procedure)
        "14403007",  # Ileotransversostomy (procedure)
        "1457004",  # Truncal vagotomy with pyloroplasty and gastrostomy (procedure)
        "14599002",  # Esophagojejunostomy by abdominal approach (procedure)
        "14622006",  # Esophagoplasty by cervical approach (procedure)
        "14687003",  # Choledochoenterostomy (procedure)
        "14690009",  # Anastomosis of large intestine to rectum (procedure)
        "14759003",  # Lysis of adhesions of gallbladder (procedure)
        "1501006",  # Altemeier operation, perineal rectal pull-through (procedure)
        "15051002",  # Repair of pancreas (procedure)
        "15213007",  # Truncal vagotomy (procedure)
        "15257006",  # Lobectomy of liver with partial excision of adjacent lobes (procedure)
        "15457008",  # Closure of cecosigmoidal fistula (procedure)
        "16079003",  # Sauer-Bacon operation, abdominoperineal resection (procedure)
        "16224002",  # Closure of hepatic duct fistula (procedure)
        "16280008",  # Closure of gastric fistula (procedure)
        "1645004",  # Transduodenal sphincterotomy (procedure)
        "16564004",  # Hartmann operation, rectal resection (procedure)
        "17254000",  # Hepaticolithotomy (procedure)
        "172938001",  # Pharyngolaryngoesophagectomy (procedure)
        "173170008",  # Bilobectomy of lung (procedure)
        "173171007",  # Lobectomy of lung (procedure)
        "173172000",  # Excision of segment of lung (procedure)
        "17319005",  # Take-down of stoma of common duct (procedure)
        "173560002",  # Esophagogastrectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173566008",  # Total esophagectomy and interposition of microvascularly attached colon (procedure)
        "173571001",  # Partial esophagectomy and end-to-end anastomosis of esophagus (procedure)
        "173572008",  # Partial esophagectomy and interposition of microvascularly attached jejunum (procedure)
        "173573003",  # Partial esophagectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173576006",  # Partial esophagectomy and interposition of microvascularly attached colon (procedure)
        "173586007",  # Bypass of esophagus by anastomosis of esophagus to esophagus (procedure)
        "173587003",  # Bypass of esophagus using stomach (procedure)
        "173588008",  # Bypass of esophagus by interposition of microvascularly attached jejunum (procedure)
        "173590009",  # Bypass of esophagus by interposition of microvascularly attached colon (procedure)
        "173600008",  # Conversion from previous direct anastomosis of esophagus (procedure)
        "173605003",  # Repair of rupture of esophagus (procedure)
        "173619003",  # Transection of esophagus using staple gun (procedure)
        "173685001",  # Repair of esophageal hiatus using abdominal approach (procedure)
        "173705006",  # Revision of fundoplication of stomach (procedure)
        "173714001",  # Total gastrectomy and excision of surrounding tissue (procedure)
        "173715000",  # Total gastrectomy and interposition of jejunum (procedure)
        "173716004",  # Total gastrectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173720000",  # Partial gastrectomy and anastomosis of stomach to transposed jejunum (procedure)
        "173722008",  # Maki's pylorus-preserving gastrectomy (procedure)
        "173734000",  # Plastic operations on stomach (procedure)
        "173740007",  # Bypass of stomach by anastomosis of esophagus to duodenum (procedure)
        "173747005",  # Roux-en-Y gastrojejunostomy (procedure)
        "173748000",  # Bypass of stomach by anastomosis of stomach to transposed jejunum (procedure)
        "173750008",  # Conversion to anastomosis of stomach to transposed jejunum (procedure)
        "173751007",  # Conversion from previous anastomosis of stomach to transposed jejunum (procedure)
        "173778009",  # Gastrotomy and ligation of bleeding point of stomach (procedure)
        "173788005",  # Pyloromyotomy (procedure)
        "173793008",  # Closure of pyloroplasty (procedure)
        "173794002",  # Pyloromyotomy and wedge resection (procedure)
        "173795001",  # Open dilation of pylorus (procedure)
        "173800006",  # Repair of perforation of pylorus (procedure)
        "173842008",  # Closure of perforated gastric ulcer (procedure)
        "173848007",  # Total excision of duodenum (procedure)
        "173849004",  # Partial excision of duodenum (procedure)
        "173853002",  # Open destruction of lesion of duodenum (procedure)
        "173856005",  # Bypass of duodenum (procedure)
        "173857001",  # Bypass of duodenum by anastomosis of stomach to jejunum (procedure)
        "173858006",  # Bypass of duodenum by anastomosis of duodenum to duodenum (procedure)
        "173860008",  # Anastomosis of duodenum to colon (procedure)
        "173889005",  # Closure of perforated duodenal ulcer (procedure)
        "173895006",  # Total jejunectomy and anastomosis of stomach to ileum (procedure)
        "173896007",  # Total jejunectomy and anastomosis of duodenum to ileum (procedure)
        "173897003",  # Total jejunectomy and anastomosis of duodenum to colon (procedure)
        "173898008",  # Partial jejunectomy and anastomosis of jejunum to ileum (procedure)
        "173899000",  # Partial jejunectomy and anastomosis of duodenum to colon (procedure)
        "173903007",  # Excision of lesion of jejunum (procedure)
        "173904001",  # Open destruction of lesion of jejunum (procedure)
        "173911002",  # Bypass of jejunum (procedure)
        "173913004",  # Bypass of jejunum by anastomosis of jejunum to ileum (procedure)
        "173923008",  # Closure of perforation of jejunum (procedure)
        "173947001",  # Ileectomy and anastomosis of stomach to ileum (procedure)
        "173948006",  # Ileectomy and anastomosis of duodenum to ileum (procedure)
        "173949003",  # Ileectomy and anastomosis of ileum to ileum (procedure)
        "173950003",  # Ileectomy and anastomosis of ileum to colon (procedure)
        "173955008",  # Open destruction of lesion of ileum (procedure)
        "173958005",  # Bypass of ileum (procedure)
        "173959002",  # Bypass of ileum by anastomosis of jejunum to ileum (procedure)
        "173960007",  # Bypass of ileum by anastomosis of ileum to ileum (procedure)
        "173961006",  # Bypass of ileum by anastomosis of ileum to cecum (procedure)
        "173962004",  # Ileotransverse colonic bypass (procedure)
        "173967005",  # Anastomosis of ileum to cecum (procedure)
        "173973006",  # Revision of anastomosis of ileum (procedure)
        "173977007",  # Creation of ileostomy (procedure)
        "173980008",  # Creation of temporary ileostomy (procedure)
        "173994009",  # Plication of ileum (procedure)
        "174000002",  # Strictureplasty of ileum (procedure)
        "174002005",  # Closure of perforation of ileum (procedure)
        "174003000",  # Exclusion of segment of ileum (procedure)
        "174005007",  # Oversewing of small intestine (procedure)
        "174018004",  # Enterotomy and endoscopic examination of ileum (procedure)
        "174059005",  # Excision of colon and rectum (procedure)
        "174071004",  # Extended right hemicolectomy (procedure)
        "174072006",  # Extended right hemicolectomy and end-to-end anastomosis (procedure)
        "174073001",  # Extended right hemicolectomy and anastomosis of ileum to colon (procedure)
        "174080004",  # Right hemicolectomy and end to end anastomosis of ileum to colon (procedure)
        "174081000",  # Right hemicolectomy and side-to-side anastomosis of ileum to transverse colon (procedure)
        "174086005",  # Transverse colectomy and end-to-end anastomosis (procedure)
        "174087001",  # Transverse colectomy and anastomosis of ileum to colon (procedure)
        "174093009",  # Left hemicolectomy and end to end anastomosis of colon to rectum (procedure)
        "174094003",  # Left hemicolectomy and end-to-end anastomosis of colon to colon (procedure)
        "174100000",  # Sigmoid colectomy and end-to-end anastomosis of ileum to rectum (procedure)
        "174101001",  # Sigmoid colectomy and anastomosis of colon to rectum (procedure)
        "174121000",  # Bypass of colon (procedure)
        "174123002",  # Bypass of colon by anastomosis of ileum to colon (procedure)
        "174124008",  # Bypass of colon by anastomosis of cecum to sigmoid colon (procedure)
        "174125009",  # Bypass of colon by anastomosis of transverse colon to sigmoid colon (procedure)
        "174126005",  # Bypass of colon by anastomosis of transverse colon to rectum (procedure)
        "174132000",  # Refashioning of cecostomy (procedure)
        "174164007",  # Enterorrhaphy of colon (procedure)
        "174165008",  # Open removal of foreign body from colon (procedure)
        "174240003",  # Proctectomy and anastomosis of colon to anus (procedure)
        "174241004",  # Anterior resection of rectum and anastomosis of colon to rectum using staples (procedure)
        "174273005",  # Transsphincteric anastomosis of colon to anus (procedure)
        "174278001",  # Rectosigmoidectomy and peranal anastomosis (procedure)
        "174286001",  # Perineal operations for rectal prolapse (procedure)
        "174349004",  # Reanastomosis of rectum to anal canal for correction of congenital atresia of rectum (procedure)
        "174416009",  # Closure of bowel fistula (procedure)
        "174430004",  # Resection of segment of liver (procedure)
        "174431000",  # Wedge resection of liver (procedure)
        "174432007",  # Marsupialization of liver lesion (procedure)
        "174495007",  # Total cholecystectomy and exploration of common bile duct (procedure)
        "174503007",  # Revision of anastomosis of gallbladder (procedure)
        "174504001",  # Closure of gallbladder anastomosis (procedure)
        "174507008",  # Closure of fistula of gallbladder (procedure)
        "174508003",  # Closure of gallbladder perforation (procedure)
        "174542007",  # Excision of ampulla of Vater and replantation of common bile duct into duodenum (procedure)
        "174543002",  # Partial excision of bile duct and anastomosis of bile duct to duodenum (procedure)
        "174545009",  # Partial excision of bile duct and anastomosis of bile duct to jejunum (procedure)
        "174546005",  # Partial excision of bile duct and end-to-end anastomosis of bile duct (procedure)
        "174549003",  # Extirpation of lesion of bile duct (procedure)
        "174550003",  # Excision of lesion of bile duct (procedure)
        "174551004",  # Destruction lesion of bile duct (procedure)
        "174557000",  # Revision of anastomosis of hepatic duct (procedure)
        "174558005",  # Open dilation of anastomosis of hepatic duct (procedure)
        "174561006",  # Anastomosis main bile duct to duodenum (procedure)
        "174562004",  # Anastomosis of common bile duct to transposed jejunum (procedure)
        "174564003",  # Revision of anastomosis of common bile duct (procedure)
        "174565002",  # Open dilation of anastomosis of common bile duct (procedure)
        "174578002",  # Reanastomosis of bile duct (procedure)
        "174582000",  # Open removal of calculus from bile duct and drainage of bile duct (procedure)
        "174588001",  # Plastic repair of sphincter of Oddi using duodenal approach (procedure)
        "174589009",  # Sphincteroplasty of bile duct and pancreatic duct using duodenal approach (procedure)
        "174595005",  # Revision of stoma of small intestine (procedure)
        "174596006",  # Sphincterotomy of bile duct and pancreatic duct using duodenal approach (procedure)
        "174602009",  # Transduodenal excision of ampulla of Vater (procedure)
        "174697009",  # Total pancreatectomy and excision of surrounding tissue (procedure)
        "174702000",  # Proximal pancreatectomy (procedure)
        "174710004",  # Left pancreatectomy and drainage of pancreatic duct (procedure)
        "174712007",  # Excision of tail of pancreas and drainage of pancreatic duct (procedure)
        "174717001",  # Extirpation of lesion of pancreas (procedure)
        "174718006",  # Excision of lesion of islet of Langerhans (procedure)
        "174720009",  # Destruction of lesion of pancreas (procedure)
        "17473003",  # Cecotomy (procedure)
        "174734009",  # Open dilation of pancreatic duct (procedure)
        "174735005",  # Repair of pancreatic duct (procedure)
        "174740002",  # Open cystogastrostomy of pancreas (procedure)
        "174758008",  # Suture of pancreas (procedure)
        "174776001",  # Total splenectomy (procedure)
        "174778000",  # Total splenectomy and reimplantation of fragments (procedure)
        "17478007",  # Closure of pancreatic fistula (procedure)
        "176003",  # Anastomosis of rectum (procedure)
        "176006006",  # Construction of colonic conduit (procedure)
        "17671008",  # Laparoscopic-assisted anterior resection of rectum with colostomy (procedure)
        "177784006",  # Decortication of lung (procedure)
        "177835007",  # Repair of rupture of the diaphragm (procedure)
        "177836008",  # Repair of traumatic rupture of the diaphragm (procedure)
        "177837004",  # Repair of postoperative rupture of the diaphragm (procedure)
        "177846005",  # Destruction of lesion of diaphragm (procedure)
        "177916006",  # Resuture of burst abdomen (procedure)
        "177919004",  # Repair of anterior abdominal wall with distant flap (procedure)
        "177920005",  # Reconstruction of skin of anterior abdominal wall using free skin flap with microvascular anastomosis (procedure)
        "177970008",  # Repair of mesentery of small intestine (procedure)
        "177977006",  # Repair of mesentery of colon (procedure)
        "178732003",  # Pyloroduodenotomy (procedure)
        "17988003",  # Partial ileectomy (procedure)
        "18047003",  # Closure of small intestine fistula (procedure)
        "18133004",  # Child operation, radical subtotal pancreatectomy (procedure)
        "184687006",  # Hepaticostomy with removal of calculus (procedure)
        "18561003",  # Closure of enterocolic fistula (procedure)
        "186648008",  # Repair of intestine (procedure)
        "18692006",  # Bypass gastroenterostomy (procedure)
        "18878003",  # Hepatic portoenterostomy (procedure)
        "191532000",  # Dilation of ampulla of Vater (procedure)
        "1924009",  # Hepaticotomy with drainage (procedure)
        "192461000",  # Dilation of sphincter of Oddi (procedure)
        "193289008",  # Dilation of pancreatic sphincter (procedure)
        "195115001",  # Duodenocholedochotomy (procedure)
        "195767005",  # Cholangiostomy (procedure)
        "195969003",  # Choledochoduodenostomy (procedure)
        "19612002",  # Hepatocholangiocystoduodenostomy (procedure)
        "19639005",  # Sigmoidopexy (procedure)
        "196505006",  # Hepaticostomy (procedure)
        "19725004",  # Correction of esophageal atresia (procedure)
        "198144000",  # Hepaticodochotomy (procedure)
        "20241005",  # Closure of gastroduodenostomy (procedure)
        "20246000",  # Fixation of intestine (procedure)
        "20357008",  # Repair of peritoneum (procedure)
        "20460000",  # Closure of gastroenterocolic fistula (procedure)
        "21118000",  # Destructive procedure on pancreas (procedure)
        "21400005",  # Transection of vagus nerve by abdominal approach (procedure)
        "21482000",  # Closure of enterovesical fistula with bowel resection (procedure)
        "21538007",  # Distal subtotal gastrectomy with vagotomy (procedure)
        "21832001",  # Suture of laceration of diaphragm (procedure)
        "21991006",  # Belsey operation, esophagogastric sphincter (procedure)
        "22094003",  # Linear proctotomy (procedure)
        "22216006",  # Closure of cholecystoduodenal fistula (procedure)
        "22324003",  # Jejunectomy, resection of a segment or all of jejunum (procedure)
        "2234009",  # Proctosigmoidopexy (procedure)
        "22853007",  # Transposition of biliary fistulous tract into intestine (procedure)
        "22997007",  # Repair of esophagus (procedure)
        "23167008",  # Esophagotomy by thoracic approach (procedure)
        "232635006",  # Excision of segment of right upper lobe (procedure)
        "232636007",  # Excision of segment of middle lobe of right lung (procedure)
        "232637003",  # Excision of segment of right lower lobe (procedure)
        "232638008",  # Excision of segment of left upper lobe (procedure)
        "232639000",  # Excision of segment of left lower lobe (procedure)
        "232640003",  # Lobectomy with bronchoplastic procedure (procedure)
        "232641004",  # Lobectomy of lung with excision of part of adjacent lobe (procedure)
        "232642006",  # Extrapericardial pneumonectomy (procedure)
        "232643001",  # Intrapericardial pneumonectomy (procedure)
        "232644007",  # Extrapleural pneumonectomy (procedure)
        "232645008",  # Pleuropneumonectomy (procedure)
        "232646009",  # Completion pneumonectomy (procedure)
        "232647000",  # Pneumonectomy with tracheobronchoplasty (procedure)
        "23281002",  # Anastomosis of cystic duct (procedure)
        "234319005",  # Splenectomy (procedure)
        "235152003",  # Enterorrhaphy of small intestine (procedure)
        "235159007",  # Percutaneous endoscopic insertion of gastrostomy tube (procedure)
        "235160002",  # McKeown esophagectomy (procedure)
        "235161003",  # Ivor Lewis subtotal esophagectomy (procedure)
        "235162005",  # Trans-hiatal esophagectomy (procedure)
        "235163000",  # Partial esophagectomy with supra-aortic reconstruction (procedure)
        "235164006",  # Partial esophagectomy with subaortic reconstruction (procedure)
        "235165007",  # Partial esophagectomy and total gastrectomy (procedure)
        "235166008",  # Partial segmental esophagectomy (procedure)
        "235168009",  # Direct reconstruction of esophagus (procedure)
        "235169001",  # Gastric pull-up (procedure)
        "235170000",  # Reconstruction of esophagus - gastric tube (procedure)
        "235171001",  # Jejunal interposition (procedure)
        "235173003",  # Primary repair of esophageal atresia (procedure)
        "235174009",  # Secondary repair for esophageal atresia (procedure)
        "235175005",  # Closure of recurrent tracheoesophageal fistula (procedure)
        "235176006",  # Bypass of esophagus using colon (procedure)
        "235177002",  # Creation of cervical tube esophagostomy (procedure)
        "235178007",  # Creation of transthoracic tube esophagostomy (procedure)
        "235179004",  # Creation of end esophagostomy (procedure)
        "235186007",  # Total fundoplication using abdominal approach (procedure)
        "235187003",  # Partial fundoplication (procedure)
        "235188008",  # Partial fundoplication using abdominal approach (procedure)
        "235189000",  # Anterior gastropexy (procedure)
        "235190009",  # Posterior gastropexy (procedure)
        "235191008",  # Longitudinal gastroplasty (procedure)
        "235192001",  # Collis-Belsey procedure (procedure)
        "235193006",  # V-Y gastroplasty (procedure)
        "235214008",  # Billroth I partial gastrectomy - Horsley modification (procedure)
        "235215009",  # Billroth I partial gastrectomy - von Haberer-Finney modification (procedure)
        "235216005",  # Billroth I partial gastrectomy - von Haberer modification (procedure)
        "235217001",  # Billroth I partial gastrectomy - Schoemaker modification (procedure)
        "235218006",  # Billroth II partial gastrectomy - Moynihan modification (procedure)
        "235219003",  # Billroth II partial gastrectomy - Balfour modification (procedure)
        "235225004",  # Heineke-Mikulicz pyloroplasty (procedure)
        "235226003",  # Finney pyloroplasty (procedure)
        "235227007",  # Dilation of pyloric stricture (procedure)
        "235228002",  # Balloon dilation of pyloric stricture (procedure)
        "235229005",  # Truncal vagotomy with drainage (procedure)
        "235230000",  # Highly selective vagotomy with drainage (procedure)
        "235231001",  # Closure of perforated stomach (procedure)
        "235232008",  # Oversewing perforated gastric ulcer (procedure)
        "235245007",  # Stapling of stomach (procedure)
        "235253004",  # Excision of cyst of duodenum (procedure)
        "235254005",  # Excision of duodenal atresia (procedure)
        "235255006",  # Excision of duodenal atresia with tapering (procedure)
        "235259000",  # Closure of perforated duodenum (procedure)
        "235260005",  # Closure of perforated duodenum using omental patch (procedure)
        "235265000",  # Snare resection of duodenum (procedure)
        "235266004",  # Cauterization of duodenal lesion (procedure)
        "235267008",  # Laser treatment of duodenal lesion (procedure)
        "235270007",  # Heater probe treatment of duodenal lesion (procedure)
        "235271006",  # Application of hemoclips onto duodenal lesion (procedure)
        "235281005",  # Anterior gastrojejunostomy (procedure)
        "235282003",  # Anastomosis of stomach to ileum (procedure)
        "235287009",  # Transmesenteric plication (procedure)
        "235289007",  # Enterotomy and washout (procedure)
        "235291004",  # Bishop-Koop anastomosis (procedure)
        "235298005",  # Polypectomy - small intestine (procedure)
        "235299002",  # Cauterization of lesion - small intestine (procedure)
        "235300005",  # Laser treatment of lesion of small intestine (procedure)
        "235302002",  # Destruction of lesion of small intestine using thermal probe (procedure)
        "235305000",  # Interposition of small intestine (procedure)
        "235306004",  # Ileal transposition (procedure)
        "235307008",  # Bypass of small intestine (procedure)
        "235325001",  # Extended left hemicolectomy (procedure)
        "235332005",  # Creation of ileal pouch (procedure)
        "235333000",  # Oversewing of distal bowel (procedure)
        "235340004",  # Excision of polyp of large intestine (procedure)
        "235345009",  # Cauterization of lesion - large intestine (procedure)
        "235348006",  # Destruction of lesion of large intestine using thermal probe (procedure)
        "235355008",  # Closure of perforated large intestine (procedure)
        "235360007",  # Dunphy repair for rectal prolapse (procedure)
        "235364003",  # Total proctectomy (procedure)
        "235366001",  # Duhamel procedure (procedure)
        "235367005",  # Duhamel procedure with Lester Martin modification (procedure)
        "235371008",  # Abdominosacroperineal pull-through (procedure)
        "235372001",  # Rehbein procedure (procedure)
        "235373006",  # Sacroperineal pull-through (procedure)
        "235374000",  # Posterior sagittal anorectoplasty (procedure)
        "235376003",  # Anterior fixation of rectum (procedure)
        "235377007",  # Posterior fixation of rectum (procedure)
        "235378002",  # Mesh rectopexy (procedure)
        "235407009",  # Gastrointestinal tract anastomosis - intestine (procedure)
        "235408004",  # Anastomosis of jejunum to colon (procedure)
        "235409007",  # Ileocolic anastomosis (procedure)
        "235460008",  # Marsupialization of liver abscess (procedure)
        "235466002",  # Supraradical pancreatectomy (procedure)
        "235467006",  # Duodenal preserving pancreatic head resection (procedure)
        "235468001",  # Distal pancreatic tail resection (procedure)
        "235469009",  # 30 to 50 percent distal pancreatectomy (procedure)
        "235470005",  # 50 to 70 percent distal pancreatectomy (procedure)
        "235472002",  # Local excision of lesion of pancreas (procedure)
        "235476004",  # Operation on pancreatic duct (procedure)
        "235477008",  # Pancreaticojejunostomy (procedure)
        "235479006",  # Proximal pancreaticojejunostomy (procedure)
        "235480009",  # Caudal pancreaticojejunostomy (procedure)
        "235481008",  # Lateral pancreaticojejunostomy (procedure)
        "235482001",  # Closure of pancreaticogastrostomy (procedure)
        "235483006",  # Closure of pancreaticoduodenostomy (procedure)
        "235484000",  # Closure of pancreaticojejunostomy (procedure)
        "235485004",  # Pancreatic duct prosthesis operations (procedure)
        "235489005",  # Incision of pancreatic duct (procedure)
        "235490001",  # Suturing of pancreatic duct (procedure)
        "235491002",  # Ligation of pancreatic duct (procedure)
        "235493004",  # Dilation of pancreatic duct (procedure)
        "235498008",  # Operation for pancreatic fistula (procedure)
        "235499000",  # Excision of pancreatic fistula (procedure)
        "235501008",  # Occlusion of pancreatic fistula (procedure)
        "235509005",  # Triple bypass of pancreas (procedure)
        "235510000",  # Stapling of pancreas (procedure)
        "235511001",  # Banding of pancreas (procedure)
        "235520005",  # Closure of cholecystogastrostomy (procedure)
        "235521009",  # Closure of cholecystoduodenostomy (procedure)
        "235522002",  # Closure of cholecystojejunostomy (procedure)
        "235528003",  # Anastomosis main bile duct to stomach (procedure)
        "235529006",  # Closure of main bile duct anastomosis (procedure)
        "235530001",  # Closure of choledochoduodenostomy (procedure)
        "235531002",  # Closure of choledochoenterostomy (procedure)
        "235532009",  # Closure of choledochogastrostomy (procedure)
        "235533004",  # Suturing of choledochotomy (procedure)
        "235534005",  # Suturing of bile duct (procedure)
        "235536007",  # Closure of hepatic duct anastomosis (procedure)
        "235537003",  # Closure of hepatico-duodenostomy (procedure)
        "235538008",  # Closure of hepatico-enterostomy (procedure)
        "235540003",  # Suture of hepatic duct (procedure)
        "235570008",  # Dilation of bile duct (procedure)
        "235571007",  # Dilation of anastomosis of bile duct (procedure)
        "235573005",  # Excision of biliary fistula (procedure)
        "235574004",  # Occlusion of biliary fistula (procedure)
        "235581006",  # Sphincteroplasty of duodenal papilla (procedure)
        "235582004",  # Sphincterotomy of duodenal papilla (procedure)
        "235584003",  # Sphincterotomy of accessory papilla (procedure)
        "235585002",  # Dilation of duodenal papilla (procedure)
        "235586001",  # Pancreatic duct septotomy (procedure)
        "235588000",  # Excision of duodenal papilla (procedure)
        "235589008",  # Destruction of lesion of duodenal papilla (procedure)
        "235590004",  # Destruction lesion of ampulla Vater (procedure)
        "235591000",  # Laser therapy of ampullary tumor (procedure)
        "235860000",  # Choledochostomy with removal of calculus (procedure)
        "23797008",  # Esophagojejunostomy (procedure)
        "238203006",  # Repair of paracolostomy hernia (procedure)
        "238210000",  # Resuture of skin dehiscence abdominal incision (procedure)
        "238211001",  # Resuture of deep dehiscence abdominal incision (procedure)
        "238213003",  # Reconstruction of abdominal wall using flap (procedure)
        "238214009",  # Reconstruction of abdominal wall using local skin flap (procedure)
        "238217002",  # Reconstruction of abdominal wall with free skin flap (procedure)
        "238218007",  # Reconstruction of abdominal wall with local muscle flap (procedure)
        "238219004",  # Reconstruction of abdominal wall using synthetic patch (procedure)
        "238221009",  # Rives prosthetic abdominal wall repair (procedure)
        "238236007",  # Excision of vitelline fistula (procedure)
        "238240003",  # Excision of vitelline sinus (procedure)
        "238255007",  # Second look laparotomy (procedure)
        "238335008",  # Removal of plomb with decortication (procedure)
        "238358006",  # Reconstruction of diaphragm with mesh (procedure)
        "239005002",  # Incision of bile duct, except common (procedure)
        "23968004",  # Excision of colon (procedure)
        "239879006",  # Incision and exploration of hepatic ducts (procedure)
        "240294007",  # Open operation for fetal congenital diaphragmatic hernia (procedure)
        "240962009",  # Biliary dilation procedure (procedure)
        "241560003",  # Portoenterostomy (procedure)
        "242519008",  # Kasai procedure (procedure)
        "243287005",  # Repair of hepatic duct (procedure)
        "24431004",  # Esophagoduodenostomy with complete gastrectomy (procedure)
        "24506003",  # Esophagogastrectomy (procedure)
        "245667003",  # Paul operation, temporary ileostomy (procedure)
        "247501005",  # Creation of Brooke ileostomy (procedure)
        "24792005",  # Choledochotomy with removal of calculus with cholecystotomy (procedure)
        "248352005",  # Permanent continent ileostomy (procedure)
        "24883002",  # Biliopancreatic bypass to ileum with partial gastrectomy (procedure)
        "249210007",  # Creation of continent ileostomy (procedure)
        "25187007",  # Incision and drainage of omental abscess (procedure)
        "25196007",  # Proctectomy by abdominoperineal resection with colostomy, two stages (procedure)
        "25321000",  # Thoracoscopic pneumonectomy (procedure)
        "257909005",  # Repair of congenital diaphragmatic hernia using muscle flap (procedure)
        "260645006",  # Minilaparotomy (procedure)
        "261502004",  # Choledochojejunostomy to Roux loop (procedure)
        "261766001",  # Construction of divided colostomy (procedure)
        "26187001",  # Kader operation, temporary gastrostomy (procedure)
        "26268001",  # Anastomosis of hepatic ducts (procedure)
        "262794006",  # Surgical transection of esophagus (procedure)
        "26390003",  # Total colectomy (procedure)
        "26452005",  # Total gastrectomy (procedure)
        "264963003",  # Suture of anterior abdominal wall (procedure)
        "264969004",  # Laparotomy and removal of foreign body from abdominal cavity (procedure)
        "265340005",  # Partial esophagectomy and proximal gastrectomy (procedure)
        "265341009",  # Total esophagectomy and interposition of microvascularly attached jejunum (procedure)
        "265344001",  # Bypass of esophagus (procedure)
        "265345000",  # Creation of tube esophagostomy (procedure)
        "265356000",  # Repair of esophageal hiatus using thoracic approach (procedure)
        "265358004",  # Antireflux fundoplication using thoracic approach (procedure)
        "265359007",  # Antireflux fundoplication using abdominal approach (procedure)
        "265360002",  # Antireflux gastropexy (procedure)
        "265364006",  # Partitioning of stomach (procedure)
        "265365007",  # Bypass of stomach by anastomosis of stomach to duodenum (procedure)
        "265380001",  # Jejunojejunal bypass (procedure)
        "265381002",  # Bypass of jejunum by anastomosis of jejunum to colon (procedure)
        "265411006",  # Partial excision of rectum and sigmoid colon for prolapse (procedure)
        "265412004",  # Rectosigmoidectomy and rectopexy (procedure)
        "265414003",  # Abdominoperineal resection of rectum (procedure)
        "265417005",  # Posterior fixation of rectum using prosthetic material (procedure)
        "265418000",  # Fixation of rectum using fascia lata (procedure)
        "265420002",  # Sponge rectopexy (procedure)
        "265429001",  # Removal of colorectal septum (procedure)
        "265458003",  # Pancreaticoduodenectomy and excision of surrounding tissue (procedure)
        "265459006",  # Pancreaticoduodenectomy with distal gastrectomy (procedure)
        "265461002",  # 95 percent distal pancreatectomy (procedure)
        "265462009",  # Roux-en-Y pancreaticojejunostomy (procedure)
        "26565004",  # Polya operation, gastrectomy (procedure)
        "26619008",  # Closure of ileosigmoidal fistula (procedure)
        "26732005",  # Esophagogastrostomy by abdominal approach (procedure)
        "26925005",  # Transverse colectomy (procedure)
        "26993008",  # Abdominal esophagostomy (procedure)
        "27010001",  # Partial excision of large intestine (procedure)
        "27041000",  # Cholecystoileostomy (procedure)
        "27063009",  # Excision of lesion of gallbladder (procedure)
        "271564001",  # Total fundoplication using thoracic approach (procedure)
        "271565000",  # Anastomosis of divided bile duct (procedure)
        "27191003",  # Repair of laceration of gallbladder (procedure)
        "271996001",  # Excision of duodenal web (procedure)
        "27210007",  # Laryngopharyngoesophagectomy with radical neck dissection and synchronous thyroidectomy (procedure)
        "274025005",  # Colonic polypectomy (procedure)
        "274026006",  # Gastrointestinal anastomosis procedure (procedure)
        "274448007",  # Biliary lithotripsy (procedure)
        "275007007",  # Ivor Lewis partial esophagogastrectomy (procedure)
        "275011001",  # Ripstein fixation of rectum using polytetrafluoroethylene mesh (procedure)
        "275014009",  # Soave endorectal pull-through operation for Hirschsprung's disease (procedure)
        "275017002",  # Duhamel removal of colorectal septum (procedure)
        "275019004",  # Roux-en-Y cholecystojejunostomy (procedure)
        "275024001",  # Supraduodenal choledochotomy (procedure)
        "275162000",  # Hofmeister valved gastrectomy (procedure)
        "275194005",  # Bancroft partial gastrectomy and esophagogastrostomy (procedure)
        "275230005",  # Repair of duodenum (procedure)
        "275231009",  # Incision and drainage of anorectal abscess (procedure)
        "276190007",  # Ileocolic resection (procedure)
        "278500000",  # Surgical transposition of stomach (procedure)
        "27892007",  # Closure of fecal fistula (procedure)
        "281413005",  # Duodenoplasty (procedure)
        "281567001",  # Reconstruction of bile duct (procedure)
        "282400002",  # Excision of lesion of hepatic ducts (procedure)
        "283377009",  # Incision of bile duct (procedure)
        "28345000",  # Delayed closure of abdominal wall (procedure)
        "28467003",  # Intrathoracic esophagoileostomy (procedure)
        "286079005",  # Choledochostomy (procedure)
        "28658009",  # Plication of diaphragm for hernia repair by thoracoabdominal approach (procedure)
        "287315009",  # Suture of diaphragm (procedure)
        "287334009",  # Esophagus to antesternal anastomosis (procedure)
        "287783005",  # Bowel stoma repair (procedure)
        "287788001",  # Bowel stoma closure (procedure)
        "287793003",  # Intestine bypass shunt (procedure)
        "287809004",  # Total gastroduodenectomy (procedure)
        "287810009",  # Gastric anastomosis revision (procedure)
        "287815004",  # Excision of excess colostomy mucosa (procedure)
        "287816003",  # Radical gastrectomy (procedure)
        "287818002",  # Partial gastroduodenectomy (procedure)
        "287819005",  # Pylorectomy (procedure)
        "287824008",  # Enterotomy for exploration (procedure)
        "287827001",  # Excision of redundant mucosa from ileostomy (procedure)
        "287828006",  # Duodenal ulcer excision (procedure)
        "287829003",  # Enterotomy for drainage (procedure)
        "287834004",  # Gastroenterostomy - no gastrectomy (procedure)
        "287844002",  # Bile duct to intestine anastomosis (procedure)
        "287846000",  # Radical pancreaticoduodenectomy (procedure)
        "287847009",  # Partial pancreatectomy (procedure)
        "287848004",  # Pancreas local excision (procedure)
        "287850007",  # Bile duct to pancreas anastomosis (procedure)
        "287853009",  # Laparotomy and division of peritoneal adhesions (procedure)
        "287866009",  # Choledochotomy and endoscopy (procedure)
        "287867000",  # Choledochotomy and drainage (procedure)
        "287994002",  # Choledochotomy and radiography (procedure)
        "287995001",  # Bile duct stoma closure (procedure)
        "288019006",  # Retroperitoneal exploration operation (procedure)
        "28851001",  # Esophagojejunoplasty (procedure)
        "28890002",  # Repair of traumatic diaphragmatic hernia, chronic (procedure)
        "289060008",  # Anterior resection of rectal prolapse (procedure)
        "28981008",  # Closure of esophagostomy by cervical approach (procedure)
        "290017007",  # Excision of anterior mucosal prolapse of rectum (procedure)
        "29065006",  # Sigmoidotomy (procedure)
        "29205000",  # Proctopexy for prolapse by perineal approach (procedure)
        "294034007",  # Incision of common bile duct (procedure)
        "29453004",  # Jejunotomy (procedure)
        "29630005",  # Operation on pancreas (procedure)
        "298799004",  # Pancreaticojejunostomy, side-to-side anastomosis (procedure)
        "29952004",  # Percutaneous placement of gastrostomy tube (procedure)
        "299681007",  # Construction of transverse loop colostomy (procedure)
        "299682000",  # Construction of sigmoid loop colostomy (procedure)
        "3001009",  # Total lobectomy with bronchoplasty (procedure)
        "301087006",  # Dilation of pylorus (procedure)
        "301425009",  # Partial excision of bile duct (procedure)
        "30161002",  # Incision and exploration of duodenum (procedure)
        "301698008",  # Snare resection of lesion - large intestine (procedure)
        "301699000",  # Snare resection of lesion of colon (procedure)
        "301704008",  # Roux-en-Y hepaticojejunostomy (procedure)
        "30215008",  # Closure of esophageal fistula (procedure)
        "302354002",  # Small intestinal strictureplasty (procedure)
        "302619004",  # Cholecystectomy and exploration of bile duct (procedure)
        "30273008",  # Repair of neonatal diaphragmatic hernia (procedure)
        "303395008",  # Coagulation of duodenal lesion using electrical energy (procedure)
        "303398005",  # Coagulation of lesion of small intestine using electrical energy (procedure)
        "303401008",  # Parks panproctocolectomy, anastomosis of ileum to anus and creation of pouch (procedure)
        "303565004",  # Anastomosis of pharynx to esophagus (procedure)
        "303566003",  # Anastomosis of pharynx to stomach (procedure)
        "303582002",  # Coagulation of lesion of large intestine using electrical energy (procedure)
        "303584001",  # Sphincter-saving proctectomy (procedure)
        "30402002",  # Ripstein operation, repair of prolapsed rectum (procedure)
        "304060006",  # Anastomosis of esophagus to transposed jejunum (procedure)
        "3041000",  # Repair of stomach (procedure)
        "304583001",  # Extended right hemicolectomy and ileostomy (procedure)
        "304585008",  # Creation of colonic pouch (procedure)
        "304587000",  # Excision of colonic pouch (procedure)
        "305621000",  # Bypass of duodenum by anastomosis of duodenum to jejunum (procedure)
        "307009004",  # Wedge resection of rectum (procedure)
        "307010009",  # Extended right hemihepatectomy (procedure)
        "307011008",  # Caudate lobectomy of liver (procedure)
        "30706000",  # Incision of small intestine (procedure)
        "307183003",  # Creation of ileo-anal pouch (procedure)
        "307185005",  # Creation of ileo-anal W-shaped pouch (procedure)
        "307190008",  # Laparoscopic closure of perforated gastric ulcer (procedure)
        "307192000",  # Revision gastropexy (procedure)
        "307193005",  # Resection of terminal ileum (procedure)
        "307196002",  # Excision of cyst of pancreas (procedure)
        "307197006",  # Excision of common hepatic duct (procedure)
        "307259004",  # Suture of gastric ulcer (procedure)
        "307260009",  # Suture of duodenal ulcer (procedure)
        "307261008",  # Closure of perforated bowel ulcer (procedure)
        "307281009",  # Transection of vagus (procedure)
        "307302001",  # Devine exclusion of stomach (procedure)
        "307303006",  # Billroth I gastric antrectomy (procedure)
        "307304000",  # Polya gastric antrectomy (procedure)
        "307330001",  # Esophagectomy and interposition reconstruction (procedure)
        "307331002",  # Esophagectomy and jejunal interposition (procedure)
        "307332009",  # Esophagectomy and colonic interposition (procedure)
        "307364002",  # Repair of jejunum (procedure)
        "307367009",  # Repair of small intestine (procedure)
        "307368004",  # Repair of ileum (procedure)
        "307510004",  # Oversewing of bleeding duodenal ulcer (procedure)
        "307511000",  # Under-running of bleeding duodenal ulcer (procedure)
        "307512007",  # Oversewing of bleeding gastric ulcer (procedure)
        "307555006",  # Enterotomy and removal of gallstone (procedure)
        "307561009",  # Ileectomy and ileostomy (procedure)
        "307562002",  # Closure of gastroenterostomy (procedure)
        "307568003",  # Excision of neck of pancreas (procedure)
        "307569006",  # Partial excision of head of pancreas (procedure)
        "307583008",  # Excision of cecum (procedure)
        "307584002",  # Cecoplasty (procedure)
        "307586000",  # Laparoscopic transverse colectomy (procedure)
        "307615003",  # Extended left hemihepatectomy (procedure)
        "307620003",  # Marsupialization of liver cyst (procedure)
        "307621004",  # Laparoscopic marsupialization of liver cyst (procedure)
        "307626009",  # Destruction of liver metastasis (procedure)
        "307638000",  # Anastomosis of ascending colon to rectum (procedure)
        "307639008",  # Anastomosis of transverse colon to rectum (procedure)
        "307640005",  # Anastomosis of descending colon to rectum (procedure)
        "307642002",  # Anastomosis of ascending to sigmoid colon (procedure)
        "307643007",  # Anastomosis of transverse to sigmoid colon (procedure)
        "307644001",  # Transverse colectomy and ileostomy (procedure)
        "307645000",  # Transverse colectomy and colostomy (procedure)
        "307648003",  # Sigmoid colectomy and ileostomy (procedure)
        "307654002",  # Subtotal colectomy with cecorectal anastomosis (procedure)
        "307655001",  # Anastomosis of cecum to rectum (procedure)
        "307656000",  # Subtotal colectomy with ileosigmoid anastomosis (procedure)
        "307657009",  # Left hemicolectomy and colostomy (procedure)
        "307658004",  # Sigmoid colectomy and colostomy (procedure)
        "307659007",  # Subtotal colectomy with ileorectal anastomosis (procedure)
        "307662005",  # Excision of colocutaneous fistula (procedure)
        "307665007",  # Closure of defunctioning colostomy (procedure)
        "307666008",  # Total colectomy and ileostomy (procedure)
        "307667004",  # Total colectomy, ileostomy and rectal mucous fistula (procedure)
        "307669001",  # Total colectomy, ileostomy and closure of rectal stump (procedure)
        "307709007",  # Repair of perforated colon (procedure)
        "30803004",  # Printen and Mason operation, high gastric bypass (procedure)
        "30947006",  # Repair of bile duct laceration (procedure)
        "310223003",  # Repair of congenital diaphragmatic hernia using sutures (procedure)
        "310224009",  # Repair of congenital diaphragmatic hernia using patch (procedure)
        "310420002",  # Patch repair of traumatic rupture of diaphragm (procedure)
        "310421003",  # Patch repair of postoperative rupture of diaphragm (procedure)
        "31048008",  # Gastrotomy with suture repair of esophagogastric laceration (procedure)
        "31078004",  # Closure of esophagocutaneous fistula (procedure)
        "310803009",  # Interposition of microvascularly attached colon (procedure)
        "310804003",  # Interposition of microvascularly attached jejunum (procedure)
        "31130001",  # Total abdominal colectomy with proctectomy and ileostomy (procedure)
        "31158009",  # Enterectomy with double-barrel enterostomy (procedure)
        "312340001",  # Temporary percutaneous endoscopic gastrostomy (procedure)
        "312341002",  # Permanent percutaneous endoscopic gastrostomy (procedure)
        "312406002",  # Anterior resection of rectum with stapled anastomosis of sigmoid colon to anus with creation of J pouch (procedure)
        "312407006",  # Anastomosis of sigmoid colon to anal canal (procedure)
        "312497000",  # Salvage abdominoperineal resection of rectum (procedure)
        "31256001",  # Ligation of gastric ulcer (procedure)
        "312694003",  # Excision of necrotic patch of small intestine (procedure)
        "314118002",  # Laparoscopic total splenectomy (procedure)
        "314511002",  # Left hepatic trisegmentectomy (procedure)
        "314592008",  # Low anterior resection of rectum (procedure)
        "314593003",  # Posterior gastrojejunostomy (procedure)
        "314594009",  # Creation of loop ileostomy (procedure)
        "31461001",  # Transection of vagus nerve by transthoracic approach (procedure)
        "314617008",  # Chemoembolization of liver metastases (procedure)
        "314632001",  # Under-running of bleeding gastric ulcer (procedure)
        "314636003",  # Insertion of reversed intestinal segment (procedure)
        "314638002",  # Excision of reversed intestinal segment (procedure)
        "31521004",  # Resection of intestine for interposition (procedure)
        "315319006",  # Anastomosis of intestine to anus (procedure)
        "315321001",  # Right hemicolectomy and anastomosis of ileum to colon (procedure)
        "315322008",  # Transverse colectomy with anastomosis (procedure)
        "315324009",  # Left hemicolectomy with anastomosis (procedure)
        "315325005",  # Left hemicolectomy with stoma (procedure)
        "315326006",  # Sigmoid colectomy with anastomosis (procedure)
        "315327002",  # Sigmoid colectomy with stoma (procedure)
        "315328007",  # Anterior resection of rectum with anastomosis (procedure)
        "31738002",  # Jejunocecostomy for obesity (procedure)
        "31774006",  # Steinberg operation, revision of gastric anastomosis (procedure)
        "3183007",  # Closure of colostomy (procedure)
        "32044008",  # Partial colectomy by abdominal approach (procedure)
        "32091001",  # Repair of large intestine (procedure)
        "32325005",  # Inversion of diverticulum of small intestine (procedure)
        "328895009",  # Decompression of common bile duct (procedure)
        "32918006",  # Anastomosis of esophagus by intrathoracic approach (procedure)
        "329884007",  # Incision and exploration of common bile duct for relief of obstruction (procedure)
        "330857008",  # Repair of common bile duct laceration (procedure)
        "33149006",  # Pancreatectomy (procedure)
        "331865000",  # Suture of common bile duct (procedure)
        "33187003",  # Incision of esophagus (procedure)
        "33479006",  # Distal subtotal pancreatectomy with splenectomy and pancreaticojejunostomy (procedure)
        "3360004",  # Anastomosis of bile ducts (procedure)
        "33639009",  # Segmental lobectomy with concomitant decortication (procedure)
        "33649007",  # Fixation of large intestine to abdominal wall (procedure)
        "34130000",  # Cholecystocecostomy (procedure)
        "34414001",  # Incision of rectum (procedure)
        "35083003",  # Suture of mesentery (procedure)
        "35129004",  # Intrathoracic esophageal anastomosis with interposition of colon (procedure)
        "35535008",  # Transverse colostomy (procedure)
        "355610001",  # Anastomosis of esophagus to small bowel (procedure)
        "3562008",  # Rodney Smith operation, radical subtotal pancreatectomy (procedure)
        "357579002",  # Anastomosis of esophagus to intestinal segment (procedure)
        "358575006",  # Gastroduodenostomy (procedure)
        "35892001",  # Oversewing of perforated duodenal ulcer (procedure)
        "359529008",  # Jaboulay operation, gastroduodenostomy (procedure)
        "359567006",  # Proximal subtotal pancreatectomy with pancreaticoduodenectomy and pancreatic jejunostomy (procedure)
        "359571009",  # Right colectomy (procedure)
        "359572002",  # Enterectomy with anastomosis (procedure)
        "359574001",  # Closure of ulcer of duodenum (procedure)
        "359575000",  # Gastrectomy with jejunal transposition (procedure)
        "359579006",  # Partial gastrectomy with jejunal transposition (procedure)
        "359581008",  # Jejunoileostomy bypass shunt for obesity (procedure)
        "359584000",  # Fixation of duodenum to abdominal wall (procedure)
        "359585004",  # Ileojejunal bypass (procedure)
        "359587007",  # Resection of cecum and terminal ileum (procedure)
        "359588002",  # Fixation of duodenum (procedure)
        "359591002",  # Suture of laceration of duodenum (procedure)
        "359594005",  # Duodenorrhaphy (procedure)
        "359596007",  # Fixation of jejunum to abdominal wall (procedure)
        "359605007",  # Fixation of ileum to abdominal wall (procedure)
        "359615001",  # Partial lobectomy of lung (procedure)
        "359618004",  # Segmental lobectomy (procedure)
        "359623004",  # Total lobectomy of lung (procedure)
        "359887003",  # Nissen fundoplication (procedure)
        "359890009",  # Fundoplication (procedure)
        "359893006",  # Total fundoplication (procedure)
        "359902008",  # Repair of abdominal sinus (procedure)
        "359908007",  # Repair of paraesophageal diaphragmatic hernia (procedure)
        "359911008",  # Repair of parahiatal diaphragmatic hernia (procedure)
        "360363002",  # Ureterocolostomy (procedure)
        "360368006",  # Gastrorrhaphy for suture of wound (procedure)
        "360370002",  # Repair of stomach laceration by suture (procedure)
        "360372005",  # Gastrorrhaphy for perforated gastric ulcer (procedure)
        "360377004",  # Oversewing of ulcer crater of stomach (procedure)
        "361152000",  # Revision of stoma of large intestine (procedure)
        "361153005",  # Repair of stoma of large intestine (procedure)
        "36192008",  # Total abdominal colectomy with ileoproctostomy (procedure)
        "363742001",  # Pyloroplasty (procedure)
        "36432003",  # Anastomosis of urinary bladder to intestine (procedure)
        "364682008",  # Excision of bile duct (procedure)
        "36482008",  # Incision and exploration of intestine (procedure)
        "36508007",  # Fulguration of lesion of duodenum (procedure)
        "3654008",  # Lingulectomy of lung (procedure)
        "366741003",  # Repair of bile duct (procedure)
        "367365007",  # Anastomosis main bile duct to small intestine (procedure)
        "367405008",  # Excision of tissue of diaphragm (procedure)
        "367436000",  # Permanent ileostomy (procedure)
        "367453007",  # Partial resection of intestine with anastomosis (procedure)
        "367509004",  # Incision of hepatic ducts (procedure)
        "36788006",  # Cecoplication (procedure)
        "36889006",  # Gastropexy (procedure)
        "36917004",  # Incision of esophageal web (procedure)
        "37179000",  # Esophagoduodenal anastomosis with interposition of small bowel (procedure)
        "37412006",  # Excision of large intestine for interposition (procedure)
        "37593003",  # Total ileectomy (procedure)
        "37903002",  # Incision and drainage of perirectal abscess (procedure)
        "3796003",  # Excision of redundant mucosa from jejunostomy (procedure)
        "38213000",  # Closure of intestinal cutaneous fistula (procedure)
        "38507005",  # Cholecystopexy (procedure)
        "386196007",  # Excision of diverticulum from large intestine (procedure)
        "386197003",  # Excision of diverticula of colon (procedure)
        "386578009",  # Creation of split ileostomy (procedure)
        "386695005",  # Repair of stoma of esophagus (procedure)
        "386697002",  # Repair AND revision of stoma of esophagus (procedure)
        "386828002",  # Creation of defunctioning colostomy (procedure)
        "386829005",  # Temporary colostomy (procedure)
        "387606008",  # Construction of end colostomy (procedure)
        "387867004",  # Incision of stomach (procedure)
        "38841005",  # Pyloric antrotomy (procedure)
        "38971003",  # Esophagoileostomy, antesternal or antethoracic, with interposition of small bowel (procedure)
        "39027008",  # Pantaloon operation, revision of gastric anastomosis (procedure)
        "39083009",  # Thoracic esophagostomy (procedure)
        "39166002",  # Closure of duodenal fistula (procedure)
        "395136002",  # Total mesorectal excision (procedure)
        "395140006",  # Primary intestinal anastomosis (procedure)
        "395151006",  # Multi-layer hand sewn intestinal anastomosis (procedure)
        "395152004",  # Single layer hand sewn intestinal anastomosis (procedure)
        "39536003",  # Duodenotomy for foreign body removal (procedure)
        "395724000",  # Choledocholithotomy (procedure)
        "39620006",  # Witzel operation, temporary gastrostomy (procedure)
        "39655008",  # Esophagoplasty by thoracic approach (procedure)
        "397108006",  # Esophagostomy operation (procedure)
        "397159007",  # Exteriorization of intestine (procedure)
        "397160002",  # Paul-Mikulicz operation (procedure)
        "397161003",  # Rankin operation, exteriorization of intestine (procedure)
        "397162005",  # Mikulicz operation, exteriorization of intestine, first stage (procedure)
        "39738004",  # Incision of bile duct with T or Y tube insertion (procedure)
        "3980006",  # Subtotal resection of esophagus (procedure)
        "398246005",  # Anastomosis of pancreatic duct to stomach (procedure)
        "398261006",  # Pancreas to stomach anastomosis (procedure)
        "398740003",  # Colostomy (procedure)
        "399472007",  # Resection of extrahepatic bile duct (procedure)
        "40053007",  # Closure of biliary tract fistula (procedure)
        "400988008",  # High anterior resection of rectum (procedure)
        "401004",  # Distal subtotal pancreatectomy (procedure)
        "40234006",  # Hemigastrectomy (procedure)
        "4044002",  # Permanent colostomy (procedure)
        "40543003",  # Repair of diaphragmatic hiatal hernia by thoracoabdominal approach (procedure)
        "40667002",  # Fixation of small intestine (procedure)
        "40801000",  # Exploration of pelvic cavity by laparotomy (procedure)
        "40875005",  # Anastomosis of common bile duct (procedure)
        "40902008",  # Segmental lobectomy with bronchoplasty (procedure)
        "40995000",  # Pancreaticocystojejunostomy (procedure)
        "41200003",  # Excision of rectal procidentia with anastomosis by perineal approach (procedure)
        "41214003",  # Excision of bile duct tumor with repair (procedure)
        "41276005",  # Creation of endorectal ileal S-pouch with anastomosis to anus (procedure)
        "41330009",  # Laparoscopic-assisted left colectomy (procedure)
        "41436002",  # Plication of intestine (procedure)
        "41457005",  # Incision of peritoneum (procedure)
        "41536004",  # Revision of anastomosis of small intestine (procedure)
        "42246001",  # Hepatojejunostomy (procedure)
        "42268006",  # Ileosigmoidostomy (procedure)
        "42301000",  # Temporary tube ileostomy (procedure)
        "42332004",  # Fistulization of appendix (procedure)
        "425904004",  # Closure of connection of stomach to transposed jejunum (procedure)
        "425964001",  # Endoscopic retrograde lithotripsy of calculus of bile duct (procedure)
        "426150005",  # Freeing of extensive adhesions of peritoneum (procedure)
        "426344007",  # Imaging guided transluminal occlusion of colorectal fistula (procedure)
        "42645008",  # Closure of gastroesophageal fistula (procedure)
        "426738005",  # Duodenal switch (procedure)
        "427029008",  # Partitioning of stomach using staples (procedure)
        "427158008",  # Imaging guided percutaneous occlusion of colorectal fistula (procedure)
        "427997001",  # Subtotal excision of colon and rectum and creation of colonic pouch and anastomosis of colon to anus (procedure)
        "43075005",  # Partial resection of colon (procedure)
        "430798009",  # Transanal resection of rectum and anastomosis using staples (procedure)
        "43082009",  # Closure of gastrojejunal fistula (procedure)
        "431836005",  # Dilation of bile duct using fluoroscopic guidance (procedure)
        "43196001",  # Ileocolotomy (procedure)
        "43232006",  # Extrapleural pneumonectomy with empyemectomy (procedure)
        "43344006",  # Hofmeister operation, gastrectomy (procedure)
        "4336007",  # Lloyd-Davies operation, abdominoperineal resection (procedure)
        "43519002",  # Destruction of lesion of small intestine (procedure)
        "438338008",  # Total gastrectomy with extended lymphadenectomy (procedure)
        "43868005",  # Creation of endorectal ileal J-pouch with anastomosis to anus (procedure)
        "438836003",  # Anastomosis of choledochal cyst (procedure)
        "439008003",  # Transfer of free jejunum with microvascular anastomosis (procedure)
        "43907003",  # Pancreas to intestine anastomosis (procedure)
        "439171005",  # Laparoscopic selective transection of vagus nerve (procedure)
        "439712000",  # Cholecystectomy with exploration of common bile duct and choledochoenterostomy (procedure)
        "439878001",  # Completion gastrectomy (procedure)
        "440032004",  # Laparoscopic fixation of prolapse of rectum (procedure)
        "440033009",  # Laparoscopic fixation of prolapse of rectum with resection of sigmoid (procedure)
        "440070000",  # Repair of ileoanal pouch (procedure)
        "440118004",  # Laparoscopic insertion of gastrostomy tube (procedure)
        "440549000",  # Insertion of peripancreatic drain with jejunostomy (procedure)
        "440581009",  # Laparoscopic anastomosis of gallbladder to intestine (procedure)
        "440715006",  # Stapled transanal resection of rectum (procedure)
        "44076002",  # Esophagoenteric anastomosis, antesternal or antethoracic (procedure)
        "44119001",  # Anastomosis of small intestine to small intestine (procedure)
        "44192009",  # Closure of cholecystogastroenteric fistula (procedure)
        "442338001",  # Bypass of stomach (procedure)
        "442805003",  # Incision of stomach and suture of bleeding gastric ulcer (procedure)
        "442806002",  # Excision of rectum with anastomosis of colon to anus and creation of colonic reservoir (procedure)
        "442912006",  # Revision of repair of stomach (procedure)
        "443480008",  # Laparoscopic excision of part of colon (procedure)
        "443665007",  # Transanal disk excision of rectum (procedure)
        "44378008",  # Partial colectomy with anastomosis (procedure)
        "443906008",  # Bypass of stomach with short limb Roux-en-Y gastroenterostomy (procedure)
        "443908009",  # Excision of part of rectum by perineal approach (procedure)
        "443909001",  # Excision of part of colon with mobilization of splenic flexure (procedure)
        "444165004",  # Excision of part of colon and excision of terminal ileum with ileocolic anastomosis (procedure)
        "444593009",  # Marsupialization of cyst of pancreas (procedure)
        "445831005",  # Partial excision of stomach and jejunal transposition and excision of intestine (procedure)
        "445884009",  # Wedge resection of colon (procedure)
        "445934003",  # Argon plasma coagulation of lesion of small intestine (procedure)
        "445935002",  # Argon plasma coagulation of lesion of sigmoid colon (procedure)
        "445938000",  # Argon plasma coagulation of lesion of duodenum (procedure)
        "445971001",  # Laparotomy and incision of small intestine (procedure)
        "445975005",  # Excision of polyp of duodenum (procedure)
        "446222007",  # Hand-assisted laparoscopic left colectomy (procedure)
        "446223002",  # Hand-assisted laparoscopic right colectomy (procedure)
        "446228006",  # Hand-assisted laparoscopic extended left colectomy (procedure)
        "44642007",  # Partial colectomy with colostomy and creation of mucofistula (procedure)
        "446472006",  # Incision of colon and excision of polyp (procedure)
        "446652009",  # Laparotomy and decompression of intestine (procedure)
        "446733006",  # Suturing of laceration of esophagus by transabdominal approach (procedure)
        "446734000",  # Suturing of laceration of esophagus by transthoracic approach (procedure)
        "446740007",  # Banding of varix of stomach (procedure)
        "446746001",  # Excision of mucosa of duodenum (procedure)
        "447166008",  # Hand-assisted laparoscopic extended right colectomy (procedure)
        "447281005",  # Video assisted thoracoscopic excision of esophagus (procedure)
        "44751009",  # Total abdominal colectomy with proctectomy and continent ileostomy (procedure)
        "447517002",  # Video assisted thoracoscopic excision of esophagus by abdominal approach (procedure)
        "447787008",  # Conversion from previous anastomosis of stomach to jejunum (procedure)
        "447936000",  # Excision of intrahepatic bile duct (procedure)
        "447946003",  # Right hemicolectomy and ileostomy (procedure)
        "447978006",  # Argon plasma coagulation of lesion of bile duct (procedure)
        "447982008",  # Low anterior resection of rectum with colostomy (procedure)
        "448013009",  # Incision of pancreatic sphincter (procedure)
        "448050005",  # Colectomy and side to side anastomosis of ileum to colon (procedure)
        "448053007",  # Resuturing of previous incision of anterior abdominal wall (procedure)
        "448125005",  # Ablation of neoplasm of bile duct using laser (procedure)
        "448143009",  # Anterior resection of rectum and total excision of mesorectum (procedure)
        "448156002",  # Excision of rectum by transanal approach (procedure)
        "448190006",  # Partial excision of esophagus and anastomosis of esophagus to jejunum (procedure)
        "448352008",  # Sphincteroplasty of pancreatic duct using duodenal approach (procedure)
        "448369005",  # Pharyngolaryngoesophagectomy and radical neck dissection (procedure)
        "448474003",  # Low anterior resection of rectum and total excision of mesorectum (procedure)
        "448596007",  # Excision of rectal stump (procedure)
        "448653006",  # Anterior resection of rectum and total mesorectal excision with colostomy (procedure)
        "449396003",  # Dilation of mature T tube tract (procedure)
        "4520009",  # Anastomosis of esophagus, antesternal or antethoracic, with insertion of prosthesis (procedure)
        "45220009",  # Posterior resection of rectum (procedure)
        "45228002",  # Duodenoileostomy (procedure)
        "45274003",  # Closure of enterostomy of small intestine with resection and anastomosis (procedure)
        "45314005",  # Closure of cholecystogastric fistula (procedure)
        "45464004",  # Excision of lesion of ampulla of Vater (procedure)
        "4558008",  # Anterior resection of rectum (procedure)
        "45584003",  # Repair of gallbladder (procedure)
        "456004",  # Total abdominal colectomy with ileostomy (procedure)
        "45602008",  # Lysis of adhesions of peritoneum (procedure)
        "45692002",  # Anastomosis of esophagus, antesternal or antethoracic, with interposition of jejunal loop (procedure)
        "45753003",  # Cholecystenterostomy with gastroenterostomy (procedure)
        "45900003",  # Esophagectomy (procedure)
        "45993005",  # Cholecystectomy with exploration of common duct (procedure)
        "46070005",  # Revision of colostomy (procedure)
        "46295004",  # Proctectomy by abdominoperineal resection with colostomy, one stage (procedure)
        "46381000",  # Truncal vagotomy with pyloroplasty (procedure)
        "46412002",  # Apicectomy lung (procedure)
        "46599009",  # Closure of fistula of common bile duct (procedure)
        "46936007",  # Hemigastrectomy by thoracic approach (procedure)
        "47041005",  # Incision and exploration of pancreas (procedure)
        "47202004",  # Revision of pyloroplasty (procedure)
        "47208000",  # Repair of omentum (procedure)
        "47302001",  # Ileotomy (procedure)
        "47353000",  # Reopening of laparotomy site (procedure)
        "474008",  # Esophagoenteric anastomosis, intrathoracic (procedure)
        "47696003",  # Anastomosis of intestine, large-to-anus (procedure)
        "47705001",  # Incision of rectal stricture (procedure)
        "47789005",  # Excision of lesion of pancreas (procedure)
        "47934001",  # Plastic repair with reconstruction of cardia of stomach (procedure)
        "4811009",  # Reconstruction of diaphragm (procedure)
        "4829004",  # Excision of small intestine for interposition (procedure)
        "48371009",  # Revision of stoma of common bile duct (procedure)
        "4862007",  # Revision of anastomosis of large intestine (procedure)
        "48630009",  # Incision and exploration of rectum (procedure)
        "48783008",  # Choledochopancreatostomy (procedure)
        "48835001",  # Esophagogastric fundoplasty (procedure)
        "48855000",  # Incision and exploration of ileum (procedure)
        "48859006",  # Creation of esophagogastric sphincteric competence (procedure)
        "4891005",  # Proctostomy (procedure)
        "48920003",  # Hepatotomy with packing (procedure)
        "49209004",  # Subtotal gastrectomy (procedure)
        "49245001",  # Bypass gastrojejunostomy (procedure)
        "4930005",  # Repair of thoracogastric fistula (procedure)
        "49440008",  # Revision of colostomy, complicated (procedure)
        "49795001",  # Total pneumonectomy (procedure)
        "49826001",  # Highly selective vagotomy with pyloroplasty (procedure)
        "49942007",  # Take-down of esophagostomy (procedure)
        "50012009",  # Cecopexy (procedure)
        "50774009",  # Laparoscopic-assisted right colectomy (procedure)
        "50824001",  # Resection of hepatic ducts (procedure)
        "51233000",  # Sigmoidostomy (procedure)
        "51346007",  # Ileectomy (procedure)
        "51396008",  # Anastomosis of small intestine to rectal stump (procedure)
        "51452009",  # Closure of pharyngoesophageal fistula (procedure)
        "51582001",  # Transposition of tissue of diaphragm (procedure)
        "51684008",  # Radical subtotal resection of pancreas (procedure)
        "51977003",  # Repair of rectourethral fistula with colostomy (procedure)
        "52667004",  # Sigmoidosigmoidostomy (procedure)
        "52917009",  # Repair of diaphragmatic hernia by abdominal approach (procedure)
        "53003006",  # Resection of splenic flexure of colon (procedure)
        "53087005",  # Intrathoracic esophagocolostomy (procedure)
        "53133001",  # Cecocoloplicopexy (procedure)
        "53150004",  # Devascularization of stomach (procedure)
        "53266006",  # Operation on diaphragm (procedure)
        "53442002",  # Excision of stomach structure (procedure)
        "5415002",  # Destruction of lesion of liver (procedure)
        "54153006",  # Incision of liver (procedure)
        "54154000",  # Resection of sphincter of Oddi (procedure)
        "54164009",  # Rectosigmoidectomy (procedure)
        "543008",  # Choledochostomy with transduodenal sphincteroplasty (procedure)
        "54357003",  # Partial excision of small intestine (procedure)
        "54629001",  # Permanent magnetic colostomy (procedure)
        "54747001",  # Multiple segmental resections of large intestine (procedure)
        "54987000",  # Repair of common bile duct (procedure)
        "55286008",  # Janeway operation, permanent gastrostomy (procedure)
        "55517006",  # Esophagogastric fundoplasty with fundic patch (procedure)
        "55586007",  # Anastomosis of esophagus, antesternal or antethoracic, with insertion of rubber tube (procedure)
        "55966001",  # Rankin operation, complete proctectomy (procedure)
        "56614001",  # Repair of fistula involving urinary bladder and intestine (procedure)
        "5722009",  # Surgical reanastomosis of colon (procedure)
        "57236003",  # Closure of cholecystenteric fistula (procedure)
        "5745003",  # Excision ampulla of Vater with reimplantation of common duct (procedure)
        "57632009",  # Plication of stomach (procedure)
        "58090001",  # Pancreaticocystoenterostomy (procedure)
        "5809008",  # Selective vagotomy with pyloroplasty and gastrostomy (procedure)
        "58303000",  # Anastomosis of gallbladder to hepatic ducts (procedure)
        "58365005",  # Fulguration of lesion of small intestine (procedure)
        "58685000",  # Closure of cholecystocolic fistula (procedure)
        "5870006",  # Tertiary closure of abdominal wall (procedure)
        "58709004",  # Destruction of lesion of duodenum (procedure)
        "59164006",  # Repair of gastroschisis with prosthesis or graft (procedure)
        "59372001",  # Internal drainage of pancreatic cyst (procedure)
        "59411003",  # Destruction of tissue of diaphragm (procedure)
        "59567009",  # Imbrication of diaphragm (procedure)
        "59656001",  # Incision of pancreas (procedure)
        "59657005",  # Intestinal bypass for morbid obesity (procedure)
        "59912004",  # Pancreaticoduodenostomy (procedure)
        "60055006",  # Closure of cholecystojejunal fistula (procedure)
        "60194009",  # Distal subtotal pancreatectomy with splenectomy (procedure)
        "60228007",  # Cholecystectomy with exploration of common duct with cholangiography (procedure)
        "60305009",  # Repair of mesentery (procedure)
        "60465006",  # Permanent gastrostomy (procedure)
        "60523002",  # Closure of cecostomy (procedure)
        "60581003",  # Intrathoracic esophageal anastomosis with interposition of small bowel (procedure)
        "60606003",  # Duodenocholedochotomy with transduodenal choledocholithotomy (procedure)
        "60892005",  # Jejunojejunostomy (procedure)
        "60954008",  # Cervical esophagostomy (procedure)
        "61218006",  # Excision of Meckel's diverticulum (procedure)
        "61326000",  # Hepatectomy, total right lobectomy (procedure)
        "61477008",  # Incision and exploration of liver (procedure)
        "61524001",  # Resection of exteriorized segment of small intestine (procedure)
        "61667000",  # Highly selective vagotomy (procedure)
        "6190006",  # Colopexy (procedure)
        "62069000",  # Delayed opening colostomy (procedure)
        "62572002",  # Longmire operation, bile duct anastomosis (procedure)
        "62755007",  # Esophagotomy with removal of foreign body by cervical approach (procedure)
        "62807001",  # Incision and exploration of colon (procedure)
        "63003009",  # Removal of foreign body from large intestine by incision (procedure)
        "63322005",  # Excision of cystic bile duct remnant (procedure)
        "63499008",  # Repair of tracheoesophageal fistula by cervical approach (procedure)
        "63816008",  # Hepatectomy, total left lobectomy (procedure)
        "6402000",  # Excision of common bile duct (procedure)
        "64271007",  # Revision of stoma of gallbladder (procedure)
        "64492005",  # Revision of anastomosis of biliary tract (procedure)
        "64906005",  # Endoscopic dilation of sphincter of pylorus by incision (procedure)
        "65291007",  # Total resection of large intestine (procedure)
        "65310005",  # Vagotomy (procedure)
        "65657001",  # Take-down of stoma of gallbladder (procedure)
        "65666002",  # Manipulation of stomach (procedure)
        "65788008",  # Closure of gastrocolic fistula (procedure)
        "65954005",  # Hepaticocystoduodenostomy (procedure)
        "65960005",  # Closure of gastrojejunocolic fistula (procedure)
        "66343001",  # Excision of lesion of small intestine (procedure)
        "66361005",  # Excision of choledochal cyst (procedure)
        "66444008",  # Irrigation cholecystostomy (procedure)
        "66513002",  # Incision of omentum (procedure)
        "66545007",  # Repair of tracheoesophageal fistula by thoracic approach (procedure)
        "66740000",  # Coloplication (procedure)
        "66896000",  # Esophagoileostomy with interposition of small bowel (procedure)
        "6704000",  # Repair of parasternal diaphragmatic hernia (procedure)
        "67075000",  # Transplantation of tissue of stomach (procedure)
        "67160005",  # Empyemectomy with lobectomy (procedure)
        "67389003",  # Diverticulectomy of duodenum (procedure)
        "67557008",  # Laparoscopic cholecystectomy with exploration of common duct (procedure)
        "6759001",  # Esophagojejunostomy by thoracic approach (procedure)
        "67706006",  # Pancreatolithotomy (procedure)
        "67997001",  # Removal of foreign body from esophagus by incision (procedure)
        "68141001",  # Plication of mesentery (procedure)
        "68188002",  # Incision of large intestine (procedure)
        "68342003",  # Ramstedt operation, pyloromyotomy with wedge resection (procedure)
        "68911004",  # Duodenojejunostomy (procedure)
        "68960009",  # Closure of enterovesical fistula with bowel and urinary bladder resection (procedure)
        "69036001",  # Total pancreaticoduodenectomy (procedure)
        "69191006",  # Incision of intestine (procedure)
        "69455007",  # Repair of diaphragm (procedure)
        "69538006",  # Excision of diverticulum of ampulla of Vater (procedure)
        "70110002",  # Plication of small intestine (procedure)
        "70172002",  # Ileoanal anastomosis (procedure)
        "70243005",  # Anastomosis of colon to rectum (procedure)
        "70263003",  # Closure of perforation of esophagus (procedure)
        "70377003",  # Closure of gastrojejunostomy (procedure)
        "70553005",  # Fistulization for ileus (procedure)
        "70676003",  # Esophagogastrostomy (procedure)
        "71102008",  # Revision of gastric anastomosis with jejunal interposition (procedure)
        "71145001",  # Suture of esophagus (procedure)
        "71273005",  # Partial hepatectomy (procedure)
        "71309007",  # Closure of enterostomy (procedure)
        "71827004",  # Choledochojejunostomy (procedure)
        "7223004",  # Take-down of stoma of large intestine (procedure)
        "72261007",  # Resection of rectum with pelvic exenteration (procedure)
        "72583007",  # Repair of diaphragmatic hernia by thoracic approach (procedure)
        "72811008",  # Closure of sigmoidostomy (procedure)
        "72971003",  # Gastrojejunostomy with vagotomy (procedure)
        "73509004",  # Partial colectomy with skin level colostomy (procedure)
        "73554003",  # Cardioplasty of stomach and esophagus (procedure)
        "73713008",  # Repair of gastroschisis (procedure)
        "73759005",  # Duodenectomy (procedure)
        "73886004",  # Roux-en-Y cholecystojejunostomy with jejunojejunostomy (procedure)
        "73921006",  # Repair of rectal prolapse by abdominal approach (procedure)
        "73978003",  # Multiple segmental resections of small intestine (procedure)
        "73996007",  # Proctopexy combined with sigmoid resection by abdominal approach (procedure)
        "74180003",  # Hepatectomy, trisegmentectomy (procedure)
        "74770008",  # Exploratory laparotomy (procedure)
        "7495004",  # Stromeyer-Little operation hepatotomy (procedure)
        "75142005",  # Choledochostomy with cholecystotomy (procedure)
        "75190003",  # Transposition of tissue of stomach (procedure)
        "75312003",  # Partial colectomy with ileostomy and creation of mucofistula (procedure)
        "75389003",  # Cystogastrostomy of pancreas (procedure)
        "75420000",  # Excision of redundant mucosa from duodenostomy (procedure)
        "75479001",  # Transplantation of tissue of diaphragm (procedure)
        "75639003",  # Removal of tube cholecystostomy (procedure)
        "75894009",  # Delorme operation, proctopexy (procedure)
        "75905004",  # Sigmoidoproctostomy (procedure)
        "76062007",  # Duhamel operation, abdominoperineal pull-through (procedure)
        "76077004",  # Total pancreatectomy with transplantation (procedure)
        "76192001",  # Mikulicz operation, resection of large intestine, second stage (procedure)
        "76443003",  # Total esophagectomy with gastropharyngostomy (procedure)
        "764677008",  # Selective internal radiotherapy of liver using yttrium (90-Y) labeled microspheres (procedure)
        "76967003",  # Ileopancreatostomy (procedure)
        "77071005",  # Revision of stoma of stomach (procedure)
        "77539008",  # Pancreaticocystoduodenostomy (procedure)
        "77878003",  # Von Kraske operation, proctectomy (procedure)
        "77959002",  # Suture of large intestine with colostomy (procedure)
        "78017003",  # Filleting of pancreas (procedure)
        "7813002",  # Esophagojejunostomy with interposition of small bowel (procedure)
        "78233008",  # Suture of peritoneum (procedure)
        "78824001",  # Repair of traumatic diaphragmatic hernia, acute (procedure)
        "78905003",  # Removal of foreign body from pancreas (procedure)
        "78925004",  # Ileoileostomy (procedure)
        "79170004",  # Ileoduodenotomy (procedure)
        "79433000",  # Closure of colon fistula (procedure)
        "79572004",  # Choledocholithotripsy (procedure)
        "79695004",  # Incision and exploration of common bile duct (procedure)
        "80028005",  # Closure of choledochoduodenal fistula (procedure)
        "80173001",  # Closure of abdominal wall dehiscence (procedure)
        "80200006",  # Transposition of biliary fistulous tract into stomach (procedure)
        "80285004",  # Hill-Allison operation, diaphragmatic hernia repair (procedure)
        "80294005",  # Total abdominal colectomy with rectal mucosectomy and ileoanal anastomosis (procedure)
        "80319003",  # Cholecystojejunostomy (procedure)
        "80443001",  # Excision of lesion of diaphragm (procedure)
        "80444007",  # Stapling of gastric varices (procedure)
        "80772001",  # Hepatogastrostomy (procedure)
        "81092009",  # Transsacral rectosigmoidectomy (procedure)
        "81561002",  # Suture of omentum (procedure)
        "81569000",  # Selective vagotomy with pyloroplasty (procedure)
        "81714000",  # Ileorectostomy (procedure)
        "8174004",  # Roux-en-Y cholecystenterostomy (procedure)
        "81913003",  # Plication of diaphragm (procedure)
        "82293009",  # Repair of esophageal stricture (procedure)
        "82619000",  # Left colectomy (procedure)
        "82744000",  # Esophagogastrostomy by thoracic approach (procedure)
        "82780000",  # Incision and exploration of esophagus (procedure)
        "82900007",  # Partial colectomy with coloproctostomy (procedure)
        "82967007",  # Gastric stapling for obesity (procedure)
        "83101004",  # Laryngoesophagectomy (procedure)
        "8322005",  # Closure of stomach ulcer (procedure)
        "8324006",  # Incision and drainage of rectal abscess (procedure)
        "83371007",  # Proximal subtotal gastrectomy by abdominal approach (procedure)
        "83599002",  # Incision of duodenum (procedure)
        "83840007",  # Esophagotomy by cervical approach (procedure)
        "83857006",  # Gastroduodenectomy (procedure)
        "83985009",  # Resection of stomach with gastrojejunal anastomosis (procedure)
        "84262009",  # Scott operation, intestinal bypass for obesity (procedure)
        "84354004",  # Closure of tracheoesophageal fistula (procedure)
        "84477003",  # Roux-en-Y anastomosis of extrahepatic biliary ducts and gastrointestinal tract (procedure)
        "84510006",  # Temporary gastrostomy (procedure)
        "84604002",  # Sigmoid colectomy (procedure)
        "84824001",  # Pemberton operation, rectal prolapse repair (procedure)
        "84840003",  # Total lobectomy of lung with concomitant decortication (procedure)
        "84952009",  # Resection of colon for interposition (procedure)
        "85217002",  # Distal subtotal gastrectomy (procedure)
        "85759000",  # Esophagotomy with removal of foreign body by thoracic approach (procedure)
        "85849005",  # Exteriorization of large intestine (procedure)
        "85946004",  # Lobectomy of liver (procedure)
        "86369000",  # Hepatotomy with drainage (procedure)
        "86481000",  # Laparotomy (procedure)
        "86620009",  # Inversion of diverticulum of stomach (procedure)
        "86789002",  # Abdominoperineal pull-through procedure (procedure)
        "87231001",  # Anastomosis of hepatic duct to gastrointestinal tract (procedure)
        "87260005",  # Closure of esophageal fistula by cervical approach (procedure)
        "87279008",  # Excision of lesion of colon (procedure)
        "87289007",  # Anastomosis of small-to-large intestine (procedure)
        "87602008",  # Closure of enterostomy of large intestine with resection and anastomosis (procedure)
        "87604009",  # Sleeve resection of stomach (procedure)
        "87659008",  # Incision and drainage of duodenum (procedure)
        "87677003",  # Resection of rectum (procedure)
        "8777004",  # Incision and exploration of large intestine (procedure)
        "87879002",  # Cholangiocholangiostomy (procedure)
        "88187000",  # Collis-Nissen operation for hiatal hernia repair with esophagogastroplasty (procedure)
        "88314007",  # Wookey excision of esophagus with radical neck dissection (procedure)
        "88354009",  # Removal of foreign body from intestine by incision (procedure)
        "88849006",  # Esophagesophagostomy (procedure)
        "88865007",  # Destructive procedure of small intestine (procedure)
        "88888008",  # Excision of lesion of duodenum (procedure)
        "8903007",  # Excision of omphalomesenteric duct (procedure)
        "8906004",  # Cecocolostomy (procedure)
        "8929007",  # Esophagogastropexy (procedure)
        "89300004",  # Incision and exploration of small intestine (procedure)
        "89339008",  # Partial colectomy with coloproctostomy and colostomy (procedure)
        "89441008",  # Jejunocolostomy for obesity (procedure)
        "89522007",  # Colocolostomy (procedure)
        "89642006",  # Anterior resection of rectum with colostomy (procedure)
        "89951000",  # U-tube hepaticoenterostomy (procedure)
        "90021002",  # Esophagoduodenostomy (procedure)
        "90063003",  # Pyloric antrectomy (procedure)
        "90297001",  # Closure of thoracointestinal fistula (procedure)
        "90410003",  # Wedge resection of stomach (procedure)
        "90524002",  # Closure of esophagopleurocutaneous fistula (procedure)
        "90838004",  # Removal of gastrostomy tube (procedure)
        "90856004",  # Total excision of small intestine (procedure)
        "9102008",  # Proximal subtotal gastrectomy by thoracic approach (procedure)
        "9121000",  # Excision of lesion of large intestine (procedure)
        "91516004",  # Distal subtotal pancreatectomy with pancreaticojejunostomy (procedure)
        "91596000",  # Complete excision of lung with mediastinal dissection (procedure)
        "91621005",  # Total gastrectomy with intestinal interposition (procedure)
        "91625001",  # Selective vagotomy (procedure)
        "91679005",  # Plication of jejunum (procedure)
        "9175006",  # Parietal cell vagotomy with pyloroplasty and gastrostomy (procedure)
        "9215005",  # Colotomy (procedure)
        "9238008",  # Suture of stomach (procedure)
        "9292007",  # Esophageal hiatus hernia repair (procedure)
        "9429009",  # Plastic repair with reconstruction of stomach (procedure)
        "9524002",  # Total pancreatectomy (procedure)
        "9666005",  # Esophagocolic anastomosis, antesternal or antethoracic (procedure)
        "9735005",  # Repair of colon (procedure)
        "9895003",  # Destruction of lesion of large intestine (procedure)
        "9905009",  # Loop colostomy (procedure)
    }

class GeneralOrNeuraxialAnesthesia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of a procedure for surgery where a general and/or neuraxial anesthesia is used.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for general and/or neuraxial anesthesia.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "General or Neuraxial Anesthesia"
    OID = "2.16.840.1.113883.3.666.5.1743"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "3E030FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Open Approach
        "3E033FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Percutaneous Approach
        "3E040FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Open Approach
        "3E043FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Percutaneous Approach
        "3E050FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Open Approach
        "3E053FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Percutaneous Approach
        "3E060FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Open Approach
        "3E063FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Percutaneous Approach
    }

    SNOMEDCT = {
        "112943005",  # Epidural injection of anesthetic substance, diagnostic, caudal, continuous (procedure)
        "112987001",  # Inhalation anesthesia, machine system, closed, rebreathing of primary agent (procedure)
        "180878007",  # Inhalation anesthetic using muscle relaxant (procedure)
        "180885006",  # Local anesthetic lumbar epidural block (procedure)
        "180886007",  # Local anesthetic sacral epidural block (procedure)
        "18946005",  # Epidural anesthesia (procedure)
        "231249005",  # Local anesthetic intrathecal block (procedure)
        "231251009",  # Local anesthetic cervical intrathecal block (procedure)
        "231252002",  # Local anesthetic thoracic intrathecal block (procedure)
        "231253007",  # Local anesthetic lumbar intrathecal block (procedure)
        "231255000",  # Spinal subdural local anesthetic block (procedure)
        "231256004",  # Cervical subdural block (procedure)
        "231257008",  # Thoracic subdural block (procedure)
        "231258003",  # Lumbar subdural block (procedure)
        "231261002",  # Combined spinal/epidural local anesthetic block (procedure)
        "231262009",  # Combined spinal/epidural technique with needles at separate spaces (procedure)
        "231263004",  # Combined spinal/epidural technique - spinal needle through Tuohy needle (procedure)
        "241696005",  # Total intravenous anesthesia (procedure)
        "241717009",  # Inhalational analgesia (procedure)
        "24277005",  # Inhalation anesthesia, machine system, closed, circulation of primary agent (procedure)
        "266802007",  # Endotracheal anesthesia (procedure)
        "27653002",  # Inhalation anesthesia, machine system, semi-closed, rebreathing of primary agent (procedure)
        "288185006",  # Inhalation general anesthesia (procedure)
        "30050007",  # Inhalation anesthesia, machine system, semi-closed, no rebreathing of primary agent (procedure)
        "302335002",  # Local anesthetic thoracic epidural block (procedure)
        "302336001",  # Local anesthetic cervical epidural block (procedure)
        "303356007",  # Local anesthetic nerve block around spinal cord meninges (procedure)
        "303358008",  # Neurolytic nerve block around spinal cord meninges (procedure)
        "35597003",  # Epidural injection of anesthetic substance, diagnostic, lumbar, continuous (procedure)
        "398044000",  # Low dose epidural (procedure)
        "405650006",  # Neuraxial nerve block (procedure)
        "426309005",  # Epidural injection of anesthetic substance, therapeutic, continuous, cervical (procedure)
        "427060008",  # Epidural injection of anesthetic substance, therapeutic, thoracic, continuous (procedure)
        "44812007",  # Inhalation anesthesia, machine system, closed, no rebreathing of primary agent (procedure)
        "448585004",  # Target controlled total intravenous anesthesia (procedure)
        "449126000",  # Administration of inhalation anesthetic using endotracheal tube (procedure)
        "50697003",  # Administration of general anesthetic (procedure)
        "58611004",  # Epidural injection of anesthetic substance, therapeutic, lumbar, continuous (procedure)
        "64287005",  # Inhalation anesthesia, machine system, semi-closed, circulation of primary agent and gases (procedure)
        "67716003",  # Epidural injection of anesthetic substance, therapeutic, caudal, continuous (procedure)
        "74527005",  # Injection of anesthetic substance, therapeutic, subdural, differential (procedure)
        "82776005",  # Injection of anesthetic substance, therapeutic, subdural, continuous (procedure)
    }

class GynecologicalSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures of gynecologic surgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for gynecologic surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Gynecological Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.257"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0U500ZZ",  # Destruction of Right Ovary, Open Approach
        "0U508ZZ",  # Destruction of Right Ovary, Via Natural or Artificial Opening Endoscopic
        "0U510ZZ",  # Destruction of Left Ovary, Open Approach
        "0U518ZZ",  # Destruction of Left Ovary, Via Natural or Artificial Opening Endoscopic
        "0U520ZZ",  # Destruction of Bilateral Ovaries, Open Approach
        "0U528ZZ",  # Destruction of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic
        "0U540ZZ",  # Destruction of Uterine Supporting Structure, Open Approach
        "0U548ZZ",  # Destruction of Uterine Supporting Structure, Via Natural or Artificial Opening Endoscopic
        "0U550ZZ",  # Destruction of Right Fallopian Tube, Open Approach
        "0U557ZZ",  # Destruction of Right Fallopian Tube, Via Natural or Artificial Opening
        "0U558ZZ",  # Destruction of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0U560ZZ",  # Destruction of Left Fallopian Tube, Open Approach
        "0U567ZZ",  # Destruction of Left Fallopian Tube, Via Natural or Artificial Opening
        "0U568ZZ",  # Destruction of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0U570ZZ",  # Destruction of Bilateral Fallopian Tubes, Open Approach
        "0U577ZZ",  # Destruction of Bilateral Fallopian Tubes, Via Natural or Artificial Opening
        "0U578ZZ",  # Destruction of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic
        "0U800ZZ",  # Division of Right Ovary, Open Approach
        "0U810ZZ",  # Division of Left Ovary, Open Approach
        "0U820ZZ",  # Division of Bilateral Ovaries, Open Approach
        "0U840ZZ",  # Division of Uterine Supporting Structure, Open Approach
        "0UB00ZZ",  # Excision of Right Ovary, Open Approach
        "0UB07ZZ",  # Excision of Right Ovary, Via Natural or Artificial Opening
        "0UB08ZZ",  # Excision of Right Ovary, Via Natural or Artificial Opening Endoscopic
        "0UB10ZZ",  # Excision of Left Ovary, Open Approach
        "0UB17ZZ",  # Excision of Left Ovary, Via Natural or Artificial Opening
        "0UB18ZZ",  # Excision of Left Ovary, Via Natural or Artificial Opening Endoscopic
        "0UB20ZZ",  # Excision of Bilateral Ovaries, Open Approach
        "0UB27ZZ",  # Excision of Bilateral Ovaries, Via Natural or Artificial Opening
        "0UB28ZZ",  # Excision of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic
        "0UB40ZZ",  # Excision of Uterine Supporting Structure, Open Approach
        "0UB47ZZ",  # Excision of Uterine Supporting Structure, Via Natural or Artificial Opening
        "0UB48ZZ",  # Excision of Uterine Supporting Structure, Via Natural or Artificial Opening Endoscopic
        "0UB50ZZ",  # Excision of Right Fallopian Tube, Open Approach
        "0UB57ZZ",  # Excision of Right Fallopian Tube, Via Natural or Artificial Opening
        "0UB58ZZ",  # Excision of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UB60ZZ",  # Excision of Left Fallopian Tube, Open Approach
        "0UB67ZZ",  # Excision of Left Fallopian Tube, Via Natural or Artificial Opening
        "0UB68ZZ",  # Excision of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UB70ZZ",  # Excision of Bilateral Fallopian Tubes, Open Approach
        "0UB77ZZ",  # Excision of Bilateral Fallopian Tubes, Via Natural or Artificial Opening
        "0UB78ZZ",  # Excision of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic
        "0UT00ZZ",  # Resection of Right Ovary, Open Approach
        "0UT07ZZ",  # Resection of Right Ovary, Via Natural or Artificial Opening
        "0UT08ZZ",  # Resection of Right Ovary, Via Natural or Artificial Opening Endoscopic
        "0UT10ZZ",  # Resection of Left Ovary, Open Approach
        "0UT17ZZ",  # Resection of Left Ovary, Via Natural or Artificial Opening
        "0UT18ZZ",  # Resection of Left Ovary, Via Natural or Artificial Opening Endoscopic
        "0UT20ZZ",  # Resection of Bilateral Ovaries, Open Approach
        "0UT27ZZ",  # Resection of Bilateral Ovaries, Via Natural or Artificial Opening
        "0UT28ZZ",  # Resection of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic
        "0UT40ZZ",  # Resection of Uterine Supporting Structure, Open Approach
        "0UT47ZZ",  # Resection of Uterine Supporting Structure, Via Natural or Artificial Opening
        "0UT48ZZ",  # Resection of Uterine Supporting Structure, Via Natural or Artificial Opening Endoscopic
        "0UT50ZZ",  # Resection of Right Fallopian Tube, Open Approach
        "0UT57ZZ",  # Resection of Right Fallopian Tube, Via Natural or Artificial Opening
        "0UT58ZZ",  # Resection of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UT60ZZ",  # Resection of Left Fallopian Tube, Open Approach
        "0UT67ZZ",  # Resection of Left Fallopian Tube, Via Natural or Artificial Opening
        "0UT68ZZ",  # Resection of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UT70ZZ",  # Resection of Bilateral Fallopian Tubes, Open Approach
        "0UT77ZZ",  # Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening
        "0UT78ZZ",  # Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic
        "0UT90ZL",  # Resection of Uterus, Supracervical, Open Approach
        "0UT90ZZ",  # Resection of Uterus, Open Approach
        "0UT97ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening
        "0UT97ZZ",  # Resection of Uterus, Via Natural or Artificial Opening
        "0UT98ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening Endoscopic
        "0UT98ZZ",  # Resection of Uterus, Via Natural or Artificial Opening Endoscopic
        "0UTC0ZZ",  # Resection of Cervix, Open Approach
        "0UTC7ZZ",  # Resection of Cervix, Via Natural or Artificial Opening
        "0UTC8ZZ",  # Resection of Cervix, Via Natural or Artificial Opening Endoscopic
        "10T20ZZ",  # Resection of Products of Conception, Ectopic, Open Approach
        "10T27ZZ",  # Resection of Products of Conception, Ectopic, Via Natural or Artificial Opening
        "10T28ZZ",  # Resection of Products of Conception, Ectopic, Via Natural or Artificial Opening Endoscopic
    }

    SNOMEDCT = {
        "112918004",  # Vaginal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "116028008",  # Salpingo-oophorectomy (procedure)
        "116141005",  # Abdominal hysterectomy (procedure)
        "116142003",  # Radical hysterectomy (procedure)
        "116143008",  # Total abdominal hysterectomy (procedure)
        "116144002",  # Total abdominal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "116167000",  # Excision of lesion of ovary in fragments (procedure)
        "116173004",  # Wedge resection of ovary (procedure)
        "118865002",  # Procedure on round ligament (procedure)
        "120053002",  # Fallopian tube excision (procedure)
        "120064004",  # Ovary destructive procedure (procedure)
        "13254001",  # Abdominal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "15306005",  # Coagulation of ovary using electrical energy (procedure)
        "1689004",  # Local excision of ovary (procedure)
        "176914002",  # Right salpingo-oophorectomy (procedure)
        "176915001",  # Left salpingo-oophorectomy (procedure)
        "176916000",  # Right salpingectomy (procedure)
        "176917009",  # Left salpingectomy (procedure)
        "177018009",  # Open destruction of lesion of ovary (procedure)
        "177019001",  # Open cauterization of lesion of ovary (procedure)
        "177046006",  # Endoscopic extirpation of lesion of ovary (procedure)
        "177055009",  # Excision of lesion of broad ligament of uterus (procedure)
        "177056005",  # Destruction of lesion of broad ligament of uterus (procedure)
        "214173000",  # Excision of lesion of uterine ligament (procedure)
        "223306006",  # Coagulation of broad ligament using electrical energy (procedure)
        "22699007",  # Partial salpingo-oophorectomy (procedure)
        "23575009",  # Local destruction of ovary (procedure)
        "236889009",  # Laparoscopic supracervical hysterectomy (procedure)
        "236932009",  # Multicauterization of ovary (procedure)
        "236935006",  # Evacuation of broad ligament hematoma (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "236988000",  # Elective cesarean hysterectomy (procedure)
        "25344002",  # Fimbriectomy of fallopian tube (procedure)
        "25811000",  # Total excision of bilateral fallopian tubes (procedure)
        "265056007",  # Vaginal hysterectomy (procedure)
        "27185000",  # Vaginal hysterectomy with partial colpectomy and repair of enterocele (procedure)
        "276189003",  # Excision of broad ligament (procedure)
        "287931008",  # Fallopian tube cornual resection (procedure)
        "287935004",  # Partial excision of bilateral fallopian tubes (procedure)
        "287945002",  # Resection of bilateral fallopian tubes (procedure)
        "29529008",  # Supracervical hysterectomy with removal of both tubes and ovaries (procedure)
        "29672006",  # Partial oophorectomy (procedure)
        "29827000",  # Excision of bilateral fallopian tubes and ovaries (procedure)
        "300897007",  # Total salpingectomy (procedure)
        "30160001",  # Vaginal hysterectomy with total colpectomy and repair of enterocele (procedure)
        "30172007",  # Coagulation of round ligament of uterus using electrical energy (procedure)
        "302190000",  # Abdominal hysterectomy and right salpingo-oophorectomy (procedure)
        "302191001",  # Abdominal hysterectomy and left salpingo-oophorectomy (procedure)
        "302376006",  # Right oophorectomy (procedure)
        "302377002",  # Left oophorectomy (procedure)
        "307771009",  # Radical abdominal hysterectomy (procedure)
        "309879006",  # Abdominal hysterocolpectomy (procedure)
        "309880009",  # Vaginal hysterocolpectomy (procedure)
        "315059002",  # Laparoscopic salpingectomy (procedure)
        "315060007",  # Open salpingectomy (procedure)
        "33015000",  # Laparoscopy with fulguration of fallopian tube using electrical energy (procedure)
        "35887003",  # Excision of lesion of ovary (procedure)
        "35955002",  # Radical vaginal hysterectomy (procedure)
        "359971002",  # Ward-Mayo operation for vaginal hysterectomy (procedure)
        "359974005",  # Tuffier operation for vaginal hysterectomy (procedure)
        "359977003",  # Mayo operation for vaginal hysterectomy (procedure)
        "359980002",  # Vaginal panhysterectomy (procedure)
        "359983000",  # Heaney operation for vaginal hysterectomy (procedure)
        "361222003",  # Wertheim-Meigs abdominal hysterectomy (procedure)
        "361223008",  # Wertheim operation (procedure)
        "387739000",  # Enucleation of ovarian cyst (procedure)
        "387740003",  # Removal of ovarian cyst (procedure)
        "387741004",  # Excision of cyst of ovary (procedure)
        "41058005",  # Salpingectomy for tubal ectopic pregnancy by abdominal approach (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "413144006",  # Abdominal hysterectomy with conservation of ovaries (procedure)
        "413145007",  # Vaginal hysterectomy with conservation of ovaries (procedure)
        "41393008",  # Excision of cyst of fallopian tube (procedure)
        "414575003",  # Laparoscopic total abdominal hysterectomy and bilateral salpingo-oophorectomy (procedure)
        "42349006",  # Excision of cyst of broad ligament (procedure)
        "431316002",  # Laparoscopic subtotal hysterectomy (procedure)
        "43426004",  # Excision of hydrocele of canal of Nuck (procedure)
        "43791001",  # Vaginal hysterectomy with total colpectomy (procedure)
        "441820006",  # Laparoscopic-assisted vaginal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "4443002",  # Decortication of ovary (procedure)
        "445912000",  # Excision of fallopian tube and surgical removal of ectopic pregnancy (procedure)
        "44619000",  # Wedge resection of bilateral ovaries (procedure)
        "446446002",  # Total abdominal hysterectomy and removal of vaginal cuff (procedure)
        "446526009",  # Debulking of neoplasm of ovary (procedure)
        "447771005",  # Abdominal hysterectomy and excision of periuterine tissue (procedure)
        "44793005",  # Excision of lesion of fallopian tube (procedure)
        "448539002",  # Vaginal hysterectomy and excision of periuterine tissue (procedure)
        "47045001",  # Excision of lesion of round ligament (procedure)
        "49493009",  # Resection of cornua of bilateral fallopian tubes (procedure)
        "54080001",  # Cauterization of broad ligament of uterus (procedure)
        "54490004",  # Vaginal hysterectomy with repair of enterocele (procedure)
        "60158005",  # Bisection of bilateral ovaries (procedure)
        "62692004",  # Partial salpingectomy (procedure)
        "63516002",  # Vaginal hysterectomy with colpo-urethrocystopexy, Pereyra type (procedure)
        "63596003",  # Laparoscopic treatment of ectopic pregnancy with salpingectomy (procedure)
        "71225000",  # Excision of uterine ligament (procedure)
        "75835007",  # Laparoscopic-assisted vaginal hysterectomy (procedure)
        "76876009",  # Excision of bilateral ovaries (procedure)
        "77902002",  # Vaginal hysterectomy with partial colpectomy (procedure)
        "784191009",  # Classic intrafascial supracervical hysterectomy (procedure)
        "82269004",  # Cauterization of fallopian tube (procedure)
        "83152002",  # Oophorectomy (procedure)
        "83416007",  # Cauterization of round ligament of uterus (procedure)
        "84093003",  # Coagulation of fallopian tube using electrical energy (procedure)
        "8550006",  # Partial bilateral salpingectomy for sterilization (procedure)
        "85855000",  # Partial bilateral salpingectomy for sterilization by endoscopy (procedure)
        "86477000",  # Total hysterectomy with removal of both tubes and ovaries (procedure)
        "87102007",  # Excision of round ligament (procedure)
        "89325008",  # Cauterization of ovary (procedure)
    }

class HipFractureSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of hip fracture surgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for hip fracture surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hip Fracture Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.258"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0SQ90ZZ",  # Repair Right Hip Joint, Open Approach
        "0SQ9XZZ",  # Repair Right Hip Joint, External Approach
        "0SQB0ZZ",  # Repair Left Hip Joint, Open Approach
        "0SQBXZZ",  # Repair Left Hip Joint, External Approach
        "0SS904Z",  # Reposition Right Hip Joint with Internal Fixation Device, Open Approach
        "0SS905Z",  # Reposition Right Hip Joint with External Fixation Device, Open Approach
        "0SS90ZZ",  # Reposition Right Hip Joint, Open Approach
        "0SS934Z",  # Reposition Right Hip Joint with Internal Fixation Device, Percutaneous Approach
        "0SS935Z",  # Reposition Right Hip Joint with External Fixation Device, Percutaneous Approach
        "0SS93ZZ",  # Reposition Right Hip Joint, Percutaneous Approach
        "0SS944Z",  # Reposition Right Hip Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SS945Z",  # Reposition Right Hip Joint with External Fixation Device, Percutaneous Endoscopic Approach
        "0SS94ZZ",  # Reposition Right Hip Joint, Percutaneous Endoscopic Approach
        "0SS9X4Z",  # Reposition Right Hip Joint with Internal Fixation Device, External Approach
        "0SS9X5Z",  # Reposition Right Hip Joint with External Fixation Device, External Approach
        "0SS9XZZ",  # Reposition Right Hip Joint, External Approach
        "0SSB04Z",  # Reposition Left Hip Joint with Internal Fixation Device, Open Approach
        "0SSB05Z",  # Reposition Left Hip Joint with External Fixation Device, Open Approach
        "0SSB0ZZ",  # Reposition Left Hip Joint, Open Approach
        "0SSB34Z",  # Reposition Left Hip Joint with Internal Fixation Device, Percutaneous Approach
        "0SSB35Z",  # Reposition Left Hip Joint with External Fixation Device, Percutaneous Approach
        "0SSB3ZZ",  # Reposition Left Hip Joint, Percutaneous Approach
        "0SSB44Z",  # Reposition Left Hip Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SSB45Z",  # Reposition Left Hip Joint with External Fixation Device, Percutaneous Endoscopic Approach
        "0SSB4ZZ",  # Reposition Left Hip Joint, Percutaneous Endoscopic Approach
        "0SSBX4Z",  # Reposition Left Hip Joint with Internal Fixation Device, External Approach
        "0SSBX5Z",  # Reposition Left Hip Joint with External Fixation Device, External Approach
        "0SSBXZZ",  # Reposition Left Hip Joint, External Approach
    }

    SNOMEDCT = {
        "119614000",  # Hip joint reconstruction (procedure)
        "16174006",  # Acetabuloplasty with use of methyl methacrylate (procedure)
        "239499008",  # Excision arthroplasty of joint of hip (procedure)
        "239500004",  # Interposition arthroplasty of the hip (procedure)
        "32972004",  # Acetabuloplasty, Whitman type procedure (procedure)
        "44167006",  # Shelf operation, arthroplasty of hip (procedure)
        "47458005",  # Repair of hip joint (procedure)
        "61509005",  # Acetabuloplasty, Colonna type procedure (procedure)
        "73764009",  # Repair of acetabulum (procedure)
        "83381006",  # Arthroplasty of hip with bone graft (procedure)
        "87618002",  # Acetabuloplasty with resection of femoral head (procedure)
    }

class HipReplacementSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for total hip replacement procedures.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for a total hip replacement.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hip Replacement Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.259"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0SP907Z",  # Removal of Autologous Tissue Substitute from Right Hip Joint, Open Approach
        "0SP908Z",  # Removal of Spacer from Right Hip Joint, Open Approach
        "0SP909Z",  # Removal of Liner from Right Hip Joint, Open Approach
        "0SP90EZ",  # Removal of Articulating Spacer from Right Hip Joint, Open Approach
        "0SP90JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Open Approach
        "0SP90KZ",  # Removal of Nonautologous Tissue Substitute from Right Hip Joint, Open Approach
        "0SPA0JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Acetabular Surface, Open Approach
        "0SPB07Z",  # Removal of Autologous Tissue Substitute from Left Hip Joint, Open Approach
        "0SPB08Z",  # Removal of Spacer from Left Hip Joint, Open Approach
        "0SPB09Z",  # Removal of Liner from Left Hip Joint, Open Approach
        "0SPB0EZ",  # Removal of Articulating Spacer from Left Hip Joint, Open Approach
        "0SPB0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Open Approach
        "0SPB0KZ",  # Removal of Nonautologous Tissue Substitute from Left Hip Joint, Open Approach
        "0SPE0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Acetabular Surface, Open Approach
        "0SPR0JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Femoral Surface, Open Approach
        "0SPS0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Femoral Surface, Open Approach
        "0SR9019",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SR901A",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SR901Z",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SR9029",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR902A",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR902Z",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SR9039",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SR903A",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SR903Z",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SR9049",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR904A",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR904Z",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SR9069",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR906A",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR906Z",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SR907Z",  # Replacement of Right Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SR90EZ",  # Replacement of Right Hip Joint with Articulating Spacer, Open Approach
        "0SR90J9",  # Replacement of Right Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SR90JA",  # Replacement of Right Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SR90JZ",  # Replacement of Right Hip Joint with Synthetic Substitute, Open Approach
        "0SR90KZ",  # Replacement of Right Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRA009",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRA00A",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRA00Z",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRA019",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRA01A",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRA01Z",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRA039",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRA03A",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRA03Z",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRA07Z",  # Replacement of Right Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRA0J9",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRA0JA",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRA0JZ",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRA0KZ",  # Replacement of Right Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRB019",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRB01A",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRB01Z",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SRB029",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB02A",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB02Z",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SRB039",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRB03A",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRB03Z",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SRB049",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB04A",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB04Z",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SRB069",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB06A",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB06Z",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRB07Z",  # Replacement of Left Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SRB0EZ",  # Replacement of Left Hip Joint with Articulating Spacer, Open Approach
        "0SRB0J9",  # Replacement of Left Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRB0JA",  # Replacement of Left Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRB0JZ",  # Replacement of Left Hip Joint with Synthetic Substitute, Open Approach
        "0SRB0KZ",  # Replacement of Left Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRE009",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRE00A",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRE00Z",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRE019",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRE01A",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRE01Z",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRE039",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRE03A",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRE03Z",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRE07Z",  # Replacement of Left Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRE0J9",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRE0JA",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRE0JZ",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRE0KZ",  # Replacement of Left Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRR019",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRR01A",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRR01Z",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRR039",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRR03A",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRR03Z",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRR07Z",  # Replacement of Right Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRR0J9",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRR0JA",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRR0JZ",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRR0KZ",  # Replacement of Right Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRS019",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRS01A",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRS01Z",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRS039",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRS03A",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRS03Z",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRS07Z",  # Replacement of Left Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRS0J9",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRS0JA",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRS0JZ",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRS0KZ",  # Replacement of Left Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SU907Z",  # Supplement Right Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SU909Z",  # Supplement Right Hip Joint with Liner, Open Approach
        "0SU90BZ",  # Supplement Right Hip Joint with Resurfacing Device, Open Approach
        "0SU90JZ",  # Supplement Right Hip Joint with Synthetic Substitute, Open Approach
        "0SU90KZ",  # Supplement Right Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SUA09Z",  # Supplement Right Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUA0BZ",  # Supplement Right Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUB07Z",  # Supplement Left Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SUB09Z",  # Supplement Left Hip Joint with Liner, Open Approach
        "0SUB0BZ",  # Supplement Left Hip Joint with Resurfacing Device, Open Approach
        "0SUB0JZ",  # Supplement Left Hip Joint with Synthetic Substitute, Open Approach
        "0SUB0KZ",  # Supplement Left Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SUE09Z",  # Supplement Left Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUE0BZ",  # Supplement Left Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUR09Z",  # Supplement Right Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUR0BZ",  # Supplement Right Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SUS09Z",  # Supplement Left Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUS0BZ",  # Supplement Left Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SW907Z",  # Revision of Autologous Tissue Substitute in Right Hip Joint, Open Approach
        "0SW908Z",  # Revision of Spacer in Right Hip Joint, Open Approach
        "0SW909Z",  # Revision of Liner in Right Hip Joint, Open Approach
        "0SW90JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Open Approach
        "0SW90KZ",  # Revision of Nonautologous Tissue Substitute in Right Hip Joint, Open Approach
        "0SWA0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Open Approach
        "0SWB07Z",  # Revision of Autologous Tissue Substitute in Left Hip Joint, Open Approach
        "0SWB08Z",  # Revision of Spacer in Left Hip Joint, Open Approach
        "0SWB09Z",  # Revision of Liner in Left Hip Joint, Open Approach
        "0SWB0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Open Approach
        "0SWB0KZ",  # Revision of Nonautologous Tissue Substitute in Left Hip Joint, Open Approach
        "0SWE0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Open Approach
        "0SWR0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Open Approach
        "0SWS0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Open Approach
    }

    SNOMEDCT = {
        "112727005",  # Revision of prosthetic replacement of hip joint (procedure)
        "112728000",  # Total revision of hip replacement with use of methyl methacrylate (procedure)
        "1230048008",  # Total replacement of hip joint using metal augmentation (procedure)
        "1231410001",  # Total replacement of hip joint using impaction bone graft (procedure)
        "1231411002",  # Total replacement of hip joint using mesh (procedure)
        "1231412009",  # Total replacement of hip joint using ring (procedure)
        "1231413004",  # Total replacement of hip joint using allograft of bone (procedure)
        "1231414005",  # Total replacement of hip joint using autogenous bone graft (procedure)
        "1231415006",  # Total replacement of hip joint using cage (procedure)
        "1290059002",  # Conversion from total prosthetic replacement of hip joint (procedure)
        "15036006",  # Implantation of prosthesis or prosthetic device of acetabulum of hip joint, Aufranc-Turner (procedure)
        "15163009",  # Total arthroplasty of hip, low friction (procedure)
        "179294005",  # Conversion to cemented total hip replacement (procedure)
        "179295006",  # Revision of cemented total prosthetic replacement of hip joint (procedure)
        "179305003",  # Conversion to uncemented total hip replacement (procedure)
        "179306002",  # Revision of uncemented total prosthetic replacement of hip joint (procedure)
        "19954002",  # Reconstruction of hip with use of methyl methacrylate (procedure)
        "265106004",  # Insertion of Austin-Moore prosthesis (procedure)
        "265157000",  # Cemented prosthetic total arthroplasty of hip joint (procedure)
        "265160007",  # Uncemented prosthetic total arthroplasty of hip joint (procedure)
        "265161006",  # Conversion from uncemented total hip replacement (procedure)
        "276178005",  # Uncemented prosthetic hemiarthroplasty of hip joint (procedure)
        "307814001",  # Cemented arthroplasty of hip joint (procedure)
        "307815000",  # Uncemented prosthetic arthroplasty of hip joint (procedure)
        "309853002",  # Removal of previous uncemented total prosthetic replacement of hip joint (procedure)
        "314491003",  # Prosthetic hybrid total replacement of hip joint (procedure)
        "32581000",  # Partial hip replacement by cup with acetabuloplasty (procedure)
        "340922009",  # Partial hip replacement by cup (procedure)
        "386649003",  # Partial hip replacement by prosthesis (procedure)
        "398010007",  # Prosthetic arthroplasty of hip (procedure)
        "426618001",  # Prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "426904006",  # Prosthetic hybrid total replacement of hip joint using cemented acetabular component (procedure)
        "428622003",  # Revision of hybrid total prosthetic replacement of hip joint using cemented femoral component (procedure)
        "430688009",  # Prosthetic arthroplasty of left hip (procedure)
        "430694001",  # Prosthetic arthroplasty of right hip (procedure)
        "43660001",  # Replacement of femoral head by prosthesis with use of methyl methacrylate (procedure)
        "443435007",  # Total replacement of right hip joint (procedure)
        "444051004",  # Replacement of bilateral hip joints (procedure)
        "445185007",  # Resurfacing of head of femur (procedure)
        "447992000",  # Conversion from hybrid total prosthetic hip replacement (procedure)
        "449039004",  # Conversion to total prosthetic replacement of hip joint (procedure)
        "47596009",  # Cup arthroplasty of hip with use of methyl methacrylate (procedure)
        "52734007",  # Total replacement of hip (procedure)
        "57589001",  # Arthroplasty, acetabular and proximal femoral prosthetic replacement, with allograft (procedure)
        "62402000",  # Revision of total hip replacement (procedure)
        "68491008",  # Implantation of prosthesis or prosthetic device of acetabulum of hip joint with use of methyl methacrylate (procedure)
        "69625007",  # Replacement of femoral head by prosthesis (procedure)
        "711202009",  # Conversion to prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "76116002",  # Implantation of prosthesis or prosthetic device of acetabulum of hip joint (procedure)
        "768879004",  # Revision of hybrid prosthetic replacement of hip joint using cemented acetabular component (procedure)
        "770606008",  # Total replacement of left hip joint (procedure)
        "7989006",  # Removal of prosthesis of joint of hip (procedure)
    }

class IntracranialNeurosurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of intracranial neurosurgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for intracranial neurosurgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intracranial Neurosurgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.260"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "00500ZZ",  # Destruction of Brain, Open Approach
        "00510ZZ",  # Destruction of Cerebral Meninges, Open Approach
        "00520ZZ",  # Destruction of Dura Mater, Open Approach
        "00560ZZ",  # Destruction of Cerebral Ventricle, Open Approach
        "00570ZZ",  # Destruction of Cerebral Hemisphere, Open Approach
        "00580ZZ",  # Destruction of Basal Ganglia, Open Approach
        "00590ZZ",  # Destruction of Thalamus, Open Approach
        "005A0ZZ",  # Destruction of Hypothalamus, Open Approach
        "005B0ZZ",  # Destruction of Pons, Open Approach
        "005C0ZZ",  # Destruction of Cerebellum, Open Approach
        "005D0ZZ",  # Destruction of Medulla Oblongata, Open Approach
        "00800ZZ",  # Division of Brain, Open Approach
        "00870ZZ",  # Division of Cerebral Hemisphere, Open Approach
        "00880ZZ",  # Division of Basal Ganglia, Open Approach
        "009000Z",  # Drainage of Brain with Drainage Device, Open Approach
        "00900ZZ",  # Drainage of Brain, Open Approach
        "009100Z",  # Drainage of Cerebral Meninges with Drainage Device, Open Approach
        "00910ZZ",  # Drainage of Cerebral Meninges, Open Approach
        "009200Z",  # Drainage of Dura Mater with Drainage Device, Open Approach
        "00920ZZ",  # Drainage of Dura Mater, Open Approach
        "009300Z",  # Drainage of Intracranial Epidural Space with Drainage Device, Open Approach
        "00930ZZ",  # Drainage of Intracranial Epidural Space, Open Approach
        "009400Z",  # Drainage of Intracranial Subdural Space with Drainage Device, Open Approach
        "00940ZZ",  # Drainage of Intracranial Subdural Space, Open Approach
        "009500Z",  # Drainage of Intracranial Subarachnoid Space with Drainage Device, Open Approach
        "00950ZZ",  # Drainage of Intracranial Subarachnoid Space, Open Approach
        "009600Z",  # Drainage of Cerebral Ventricle with Drainage Device, Open Approach
        "00960ZZ",  # Drainage of Cerebral Ventricle, Open Approach
        "009700Z",  # Drainage of Cerebral Hemisphere with Drainage Device, Open Approach
        "00970ZZ",  # Drainage of Cerebral Hemisphere, Open Approach
        "009800Z",  # Drainage of Basal Ganglia with Drainage Device, Open Approach
        "00980ZZ",  # Drainage of Basal Ganglia, Open Approach
        "009900Z",  # Drainage of Thalamus with Drainage Device, Open Approach
        "00990ZZ",  # Drainage of Thalamus, Open Approach
        "009A00Z",  # Drainage of Hypothalamus with Drainage Device, Open Approach
        "009A0ZZ",  # Drainage of Hypothalamus, Open Approach
        "009B00Z",  # Drainage of Pons with Drainage Device, Open Approach
        "009B0ZZ",  # Drainage of Pons, Open Approach
        "009C00Z",  # Drainage of Cerebellum with Drainage Device, Open Approach
        "009C0ZZ",  # Drainage of Cerebellum, Open Approach
        "009D00Z",  # Drainage of Medulla Oblongata with Drainage Device, Open Approach
        "009D0ZZ",  # Drainage of Medulla Oblongata, Open Approach
        "00B00ZZ",  # Excision of Brain, Open Approach
        "00B10ZZ",  # Excision of Cerebral Meninges, Open Approach
        "00B20ZZ",  # Excision of Dura Mater, Open Approach
        "00B60ZZ",  # Excision of Cerebral Ventricle, Open Approach
        "00B70ZZ",  # Excision of Cerebral Hemisphere, Open Approach
        "00B80ZZ",  # Excision of Basal Ganglia, Open Approach
        "00B90ZZ",  # Excision of Thalamus, Open Approach
        "00BA0ZZ",  # Excision of Hypothalamus, Open Approach
        "00BB0ZZ",  # Excision of Pons, Open Approach
        "00BC0ZZ",  # Excision of Cerebellum, Open Approach
        "00BD0ZZ",  # Excision of Medulla Oblongata, Open Approach
        "00C00ZZ",  # Extirpation of Matter from Brain, Open Approach
        "00C10ZZ",  # Extirpation of Matter from Cerebral Meninges, Open Approach
        "00C20ZZ",  # Extirpation of Matter from Dura Mater, Open Approach
        "00C30ZZ",  # Extirpation of Matter from Intracranial Epidural Space, Open Approach
        "00C40ZZ",  # Extirpation of Matter from Intracranial Subdural Space, Open Approach
        "00C50ZZ",  # Extirpation of Matter from Intracranial Subarachnoid Space, Open Approach
        "00C60ZZ",  # Extirpation of Matter from Cerebral Ventricle, Open Approach
        "00C70ZZ",  # Extirpation of Matter from Cerebral Hemisphere, Open Approach
        "00C80ZZ",  # Extirpation of Matter from Basal Ganglia, Open Approach
        "00C90ZZ",  # Extirpation of Matter from Thalamus, Open Approach
        "00CA0ZZ",  # Extirpation of Matter from Hypothalamus, Open Approach
        "00CB0ZZ",  # Extirpation of Matter from Pons, Open Approach
        "00CC0ZZ",  # Extirpation of Matter from Cerebellum, Open Approach
        "00CD0ZZ",  # Extirpation of Matter from Medulla Oblongata, Open Approach
        "00D10ZZ",  # Extraction of Cerebral Meninges, Open Approach
        "00D20ZZ",  # Extraction of Dura Mater, Open Approach
        "00H002Z",  # Insertion of Monitoring Device into Brain, Open Approach
        "00H003Z",  # Insertion of Infusion Device into Brain, Open Approach
        "00H602Z",  # Insertion of Monitoring Device into Cerebral Ventricle, Open Approach
        "00H603Z",  # Insertion of Infusion Device into Cerebral Ventricle, Open Approach
        "00J00ZZ",  # Inspection of Brain, Open Approach
        "00N90ZZ",  # Release Thalamus, Open Approach
        "00NA0ZZ",  # Release Hypothalamus, Open Approach
        "00P000Z",  # Removal of Drainage Device from Brain, Open Approach
        "00P002Z",  # Removal of Monitoring Device from Brain, Open Approach
        "00P003Z",  # Removal of Infusion Device from Brain, Open Approach
        "00P007Z",  # Removal of Autologous Tissue Substitute from Brain, Open Approach
        "00P00JZ",  # Removal of Synthetic Substitute from Brain, Open Approach
        "00P00KZ",  # Removal of Nonautologous Tissue Substitute from Brain, Open Approach
        "00P600Z",  # Removal of Drainage Device from Cerebral Ventricle, Open Approach
        "00P602Z",  # Removal of Monitoring Device from Cerebral Ventricle, Open Approach
        "00P603Z",  # Removal of Infusion Device from Cerebral Ventricle, Open Approach
        "00P6X2Z",  # Removal of Monitoring Device from Cerebral Ventricle, External Approach
        "00Q90ZZ",  # Repair Thalamus, Open Approach
        "00QA0ZZ",  # Repair Hypothalamus, Open Approach
        "00T70ZZ",  # Resection of Cerebral Hemisphere, Open Approach
        "00W000Z",  # Revision of Drainage Device in Brain, Open Approach
        "00W002Z",  # Revision of Monitoring Device in Brain, Open Approach
        "00W003Z",  # Revision of Infusion Device in Brain, Open Approach
        "00W007Z",  # Revision of Autologous Tissue Substitute in Brain, Open Approach
        "00W00JZ",  # Revision of Synthetic Substitute in Brain, Open Approach
        "00W00KZ",  # Revision of Nonautologous Tissue Substitute in Brain, Open Approach
        "00W00MZ",  # Revision of Neurostimulator Lead in Brain, Open Approach
        "00W600Z",  # Revision of Drainage Device in Cerebral Ventricle, Open Approach
        "00W602Z",  # Revision of Monitoring Device in Cerebral Ventricle, Open Approach
        "00W603Z",  # Revision of Infusion Device in Cerebral Ventricle, Open Approach
        "00W60MZ",  # Revision of Neurostimulator Lead in Cerebral Ventricle, Open Approach
        "0N9000Z",  # Drainage of Skull with Drainage Device, Open Approach
        "0N900ZZ",  # Drainage of Skull, Open Approach
        "0N9100Z",  # Drainage of Frontal Bone with Drainage Device, Open Approach
        "0N910ZZ",  # Drainage of Frontal Bone, Open Approach
        "0N9300Z",  # Drainage of Right Parietal Bone with Drainage Device, Open Approach
        "0N930ZZ",  # Drainage of Right Parietal Bone, Open Approach
        "0N9400Z",  # Drainage of Left Parietal Bone with Drainage Device, Open Approach
        "0N940ZZ",  # Drainage of Left Parietal Bone, Open Approach
        "0N9500Z",  # Drainage of Right Temporal Bone with Drainage Device, Open Approach
        "0N950ZZ",  # Drainage of Right Temporal Bone, Open Approach
        "0N9600Z",  # Drainage of Left Temporal Bone with Drainage Device, Open Approach
        "0N960ZZ",  # Drainage of Left Temporal Bone, Open Approach
        "0N9700Z",  # Drainage of Occipital Bone with Drainage Device, Open Approach
        "0N970ZZ",  # Drainage of Occipital Bone, Open Approach
        "0W9100Z",  # Drainage of Cranial Cavity with Drainage Device, Open Approach
        "0W910ZZ",  # Drainage of Cranial Cavity, Open Approach
        "0WC10ZZ",  # Extirpation of Matter from Cranial Cavity, Open Approach
        "0WH10YZ",  # Insertion of Other Device into Cranial Cavity, Open Approach
        "0WJ10ZZ",  # Inspection of Cranial Cavity, Open Approach
        "0WP100Z",  # Removal of Drainage Device from Cranial Cavity, Open Approach
        "0WP101Z",  # Removal of Radioactive Element from Cranial Cavity, Open Approach
        "0WP10JZ",  # Removal of Synthetic Substitute from Cranial Cavity, Open Approach
        "0WP10YZ",  # Removal of Other Device from Cranial Cavity, Open Approach
        "0WW100Z",  # Revision of Drainage Device in Cranial Cavity, Open Approach
        "0WW101Z",  # Revision of Radioactive Element in Cranial Cavity, Open Approach
        "0WW103Z",  # Revision of Infusion Device in Cranial Cavity, Open Approach
        "0WW10JZ",  # Revision of Synthetic Substitute in Cranial Cavity, Open Approach
        "0WW10YZ",  # Revision of Other Device in Cranial Cavity, Open Approach
    }

    SNOMEDCT = {
        "10458001",  # Evacuation of intracerebral hematoma (procedure)
        "112929000",  # Exploratory craniotomy, supratentorial (procedure)
        "1201979001",  # Excision of lesion of brain (procedure)
        "1287541008",  # Coagulation of brain tissue (procedure)
        "1296603004",  # Drainage of cerebral epidural space by incision (procedure)
        "1296604005",  # Drainage of cerebral epidural space by trephination (procedure)
        "1296605006",  # Drainage of cerebral subdural space by incision (procedure)
        "1296606007",  # Drainage of cerebral subdural space by trephination (procedure)
        "1296607003",  # Drainage of cerebral subarachnoid space by trephination (procedure)
        "1296608008",  # Drainage of cerebral subarachnoid space by incision (procedure)
        "1296626006",  # Division of tract of brain (procedure)
        "13431007",  # Prefrontal lobotomy (procedure)
        "13433005",  # Excision of lesion of brain by transtemporal approach (procedure)
        "13666002",  # Leukotomy (procedure)
        "14247003",  # Cerebral hemispherectomy (procedure)
        "14554005",  # Cryotherapy of brain (procedure)
        "1583007",  # Tractotomy of mesencephalon (procedure)
        "171442008",  # Major excision of brain tissue (procedure)
        "171443003",  # Total lobectomy of brain (procedure)
        "171447002",  # Excision of lesion of frontal lobe of brain (procedure)
        "171448007",  # Excision of lesion of temporal lobe of brain (procedure)
        "171449004",  # Excision of lesion of parietal lobe of brain (procedure)
        "171450004",  # Excision of lesion of occipital lobe of brain (procedure)
        "171451000",  # Excision of lesion of cerebellum (procedure)
        "171455009",  # Stereotactic leucotomy (procedure)
        "171471009",  # Drainage of lesion of brain tissue (procedure)
        "171472002",  # Drainage of abscess of brain tissue (procedure)
        "171473007",  # Evacuation of hematoma from temporal lobe of brain (procedure)
        "171474001",  # Evacuation of hematoma from cerebellum (procedure)
        "171479006",  # Open division of brain tissue (procedure)
        "171495006",  # Maintenance of neurostimulator in brain (procedure)
        "171540003",  # Endoscopic extirpation of lesion of ventricle of brain (procedure)
        "171554006",  # Drainage of subarachnoid space of brain (procedure)
        "171692002",  # Removal of lesion of brain meninges (procedure)
        "171693007",  # Extirpation of lesion of meninges of brain cortex (procedure)
        "171694001",  # Extirpation of lesion of meninges of sphenoidal ridge of cranium (procedure)
        "171695000",  # Extirpation of lesion of meninges of subfrontal region of brain (procedure)
        "171696004",  # Extirpation of lesion of meninges of parasagittal region of brain (procedure)
        "171697008",  # Extirpation of lesion of falx cerebri (procedure)
        "171698003",  # Extirpation of lesion of tentorium cerebelli (procedure)
        "171722000",  # Evacuation of intracranial subdural abscess (procedure)
        "178373004",  # Exploratory open craniotomy (procedure)
        "178375006",  # Exploratory burrhole of cranium (procedure)
        "178376007",  # Reopening of cranium and re-exploration of intracranial operation site and surgical arrest of postoperative bleeding (procedure)
        "178391009",  # Cranial morcellization (procedure)
        "1868005",  # Removal of foreign body from brain (procedure)
        "1995001",  # Stripping of cerebral meninges (procedure)
        "21551002",  # Division of brain tissue (procedure)
        "22480005",  # Intracranial microdissection (procedure)
        "230810008",  # Excision of tumor of brain meninges (procedure)
        "230811007",  # Excision of cyst of brain meninges (procedure)
        "230814004",  # Removal of intracranial subdural tumor (procedure)
        "230837005",  # Focal excision of brain tissue tumor (procedure)
        "230838000",  # Focal excision of brain abscess (procedure)
        "230839008",  # Focal excision of brain hematoma (procedure)
        "230840005",  # Focal excision of brain cyst (procedure)
        "230841009",  # Focal excision of swollen brain (procedure)
        "230859007",  # Destruction of brain tumor (procedure)
        "230886007",  # Exploration of ventricle of brain (procedure)
        "230888008",  # Open exploration of ventricle of brain (procedure)
        "230890009",  # Excision of lesion of ventricle of brain (procedure)
        "230893006",  # Removal of foreign body from ventricle of brain (procedure)
        "234064002",  # Marsupialization of cerebral lesion (procedure)
        "234915008",  # Marsupialization of lesion of brain (procedure)
        "25353009",  # Craniotomy (procedure)
        "2552005",  # Incision of cerebral subarachnoid space (procedure)
        "260643004",  # Middle cranial fossa craniectomy (procedure)
        "260644005",  # Posterior cranial fossa craniectomy (procedure)
        "26096003",  # Stripping of cerebral subdural membrane (procedure)
        "27181009",  # Removal of foreign body from brain by incision (procedure)
        "27190002",  # Excision of lesion of cerebral meninges (procedure)
        "2737001",  # Exploratory craniotomy, infratentorial (procedure)
        "273990004",  # Coagulation of brain tissue using electrical energy (procedure)
        "273991000",  # Lobotomy of brain (procedure)
        "28264008",  # Amygdalotomy (procedure)
        "287205001",  # Brain decortication (procedure)
        "287208004",  # Operation on thalamus (procedure)
        "29063004",  # Incision of cerebrum (procedure)
        "29114001",  # Incision and drainage of cranial sinus (procedure)
        "2968008",  # Craniectomy with treatment of penetrating wound of brain (procedure)
        "30195003",  # Excision of lesion of cerebral cortex (procedure)
        "304848009",  # Callosotomy (procedure)
        "304849001",  # Subpial transection of cortex (procedure)
        "307299001",  # Removal of lesion from intracranial subdural space (procedure)
        "31746001",  # Debridement of cerebral meninges (procedure)
        "32606000",  # Marsupialization of cyst of brain (procedure)
        "34097003",  # Craniectomy for excision of meningioma, infratentorial (procedure)
        "36910002",  # Craniectomy (procedure)
        "37380009",  # Destruction of cerebral lesion (procedure)
        "38512006",  # Resection of cerebral meninges (procedure)
        "39667005",  # Amygdalohippocampotomy (procedure)
        "39704007",  # Exploration of brain (procedure)
        "4045001",  # Drainage of cerebral ventricle by incision (procedure)
        "42699003",  # Incision of brain (procedure)
        "427037000",  # Complete callosotomy (procedure)
        "427599009",  # Excision of lesion of tissue of brain stem (procedure)
        "429314008",  # Partial callosotomy (procedure)
        "429510003",  # Multiple subpial transections of cerebral cortex (procedure)
        "438369007",  # Incision of bone of cranium and transection of tentorium cerebelli (procedure)
        "438614004",  # Excision of bone of cranium and excision of neoplasm of cerebellopontine angle (procedure)
        "439493007",  # Incision of bone of cranium with elevation of bone flap and excision of epileptogenic focus (procedure)
        "439499006",  # Excision of bone of cranium and excision of infratentorial neoplasm of brain (procedure)
        "440062005",  # Excision of bone of cranium with excision of abscess of brain (procedure)
        "441832005",  # Extirpation of lesion of meninges of base of skull (procedure)
        "442616007",  # Extirpation of lesion of meninges of clivus of skull (procedure)
        "445746008",  # Craniotomy and removal of hematoma from extradural space (procedure)
        "44589002",  # Debridement of brain (procedure)
        "445926009",  # Excision of intradural lesion of base of middle cranial fossa (procedure)
        "445942002",  # Excision of temporal lobe (procedure)
        "446103006",  # Partial excision of lesion of brain (procedure)
        "446115005",  # Elevation of depressed fracture of skull and debridement of brain (procedure)
        "446209004",  # Excision of intradural lesion of base of anterior cranial fossa (procedure)
        "446210009",  # Excision of intradural lesion of base of posterior cranial fossa (procedure)
        "446440008",  # Curettage of lesion of brain (procedure)
        "446570000",  # Craniectomy by retromastoid approach (procedure)
        "446571001",  # Craniotomy of posterior fossa (procedure)
        "446683008",  # Craniotomy and decompression of brain (procedure)
        "446705006",  # Trephination of cranium and drainage of chronic hematoma from subdural space (procedure)
        "446816008",  # Craniotomy and evacuation of traumatic hematoma (procedure)
        "446848007",  # Craniotomy and evacuation of blood clot (procedure)
        "446850004",  # Trephination of cranium and external drainage of effusion from subdural space (procedure)
        "446865003",  # Craniotomy and removal of foreign body (procedure)
        "446868001",  # Excision of meningioma by supratentorial approach (procedure)
        "447107001",  # Craniotomy and excision of neoplasm of brain (procedure)
        "447115003",  # Total excision of lesion of brain tissue (procedure)
        "447156000",  # Trephination of cranium and drainage of empyema of subdural space (procedure)
        "447362004",  # Excision of meningioma by infratentorial approach (procedure)
        "447363009",  # Craniotomy by retromastoid approach (procedure)
        "447469009",  # Partial excision of meningioma by infratentorial approach (procedure)
        "447518007",  # Partial excision of meningioma by supratentorial approach (procedure)
        "447529006",  # Total excision of meningioma by infratentorial approach (procedure)
        "447594008",  # Total excision of meningioma by supratentorial approach (procedure)
        "45322003",  # Decompression of brain (procedure)
        "4570007",  # Chemopallidectomy (procedure)
        "53423001",  # Lobectomy of brain (procedure)
        "5384005",  # Excision of part of frontal cortex (procedure)
        "54872009",  # Destruction of lesion of cerebral meninges (procedure)
        "54891009",  # Removal of foreign body from cerebral meninges (procedure)
        "56143008",  # Division of cerebral meninges (procedure)
        "56959007",  # Operation on globus pallidus (procedure)
        "58337008",  # Transorbital leukotomy (procedure)
        "58515002",  # Cryoleucotomy (procedure)
        "59696004",  # Curettage of cerebral meninges (procedure)
        "59712006",  # Evacuation of subdural hematoma (procedure)
        "61327009",  # Puncture and drainage of craniopharyngioma (procedure)
        "61965006",  # Incision of cranial sinus (procedure)
        "67402009",  # Partial lobectomy of brain (procedure)
        "67864003",  # Trephination of cranium (procedure)
        "68346000",  # Drainage of cerebrum (procedure)
        "68731005",  # Decortication of cerebral meninges (procedure)
        "69086007",  # Curettage of brain (procedure)
        "69788001",  # Tractotomy of medulla oblongata (procedure)
        "71722001",  # Thalamotomy (procedure)
        "73006002",  # Cranial decompression, posterior fossa (procedure)
        "75626002",  # Pallidoamygdalotomy (procedure)
        "76961002",  # Lobotomy and tractotomy (procedure)
        "77697000",  # Cryothalamectomy (procedure)
        "789003",  # Cranial decompression, subtemporal, supratentorial (procedure)
        "79093007",  # Craniotomy with treatment of penetrating wound of brain (procedure)
        "807005",  # Excision of brain (procedure)
        "81259007",  # Incision of cerebral meninges (procedure)
        "8155007",  # Excision of basal ganglion (procedure)
        "81849003",  # Pallidoansotomy (procedure)
        "82678002",  # Incision of cerebral epidural space (procedure)
        "82917000",  # Reopening of craniotomy site (procedure)
        "87060004",  # Pallidotomy (procedure)
        "87192001",  # Pallidectomy (procedure)
        "90287005",  # Thalamectomy (procedure)
        "91652006",  # Incision of cerebral subdural space (procedure)
        "9257004",  # Revision of intracranial neurostimulator receiver (procedure)
        "9372002",  # Drainage of cranial sinus by trephination (procedure)
        "9580008",  # Chemothalamectomy (procedure)
        "9888007",  # Excision of intracranial lesion (procedure)
    }

class KneeReplacementSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the procedure of a total knee replacement.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for total knee replacement.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Knee Replacement Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.261"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0QPD0JZ",  # Removal of Synthetic Substitute from Right Patella, Open Approach
        "0QPF0JZ",  # Removal of Synthetic Substitute from Left Patella, Open Approach
        "0QRD0JZ",  # Replacement of Right Patella with Synthetic Substitute, Open Approach
        "0QRF0JZ",  # Replacement of Left Patella with Synthetic Substitute, Open Approach
        "0QUD0JZ",  # Supplement Right Patella with Synthetic Substitute, Open Approach
        "0QUF0JZ",  # Supplement Left Patella with Synthetic Substitute, Open Approach
        "0SPC09Z",  # Removal of Liner from Right Knee Joint, Open Approach
        "0SPC0JC",  # Removal of Synthetic Substitute from Right Knee Joint, Patellar Surface, Open Approach
        "0SPC0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPD09Z",  # Removal of Liner from Left Knee Joint, Open Approach
        "0SPD0JC",  # Removal of Synthetic Substitute from Left Knee Joint, Patellar Surface, Open Approach
        "0SPD0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Open Approach
        "0SPT0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Femoral Surface, Open Approach
        "0SPU0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Femoral Surface, Open Approach
        "0SPV0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Tibial Surface, Open Approach
        "0SPW0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Tibial Surface, Open Approach
        "0SRC069",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRC06A",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRC06Z",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRC07Z",  # Replacement of Right Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SRC0EZ",  # Replacement of Right Knee Joint with Articulating Spacer, Open Approach
        "0SRC0J9",  # Replacement of Right Knee Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRC0JA",  # Replacement of Right Knee Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRC0JZ",  # Replacement of Right Knee Joint with Synthetic Substitute, Open Approach
        "0SRC0KZ",  # Replacement of Right Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRC0L9",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRC0LA",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRC0LZ",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Open Approach
        "0SRC0M9",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRC0MA",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRC0MZ",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Open Approach
        "0SRC0N9",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Cemented, Open Approach
        "0SRC0NA",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Uncemented, Open Approach
        "0SRC0NZ",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Open Approach
        "0SRD069",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRD06A",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRD06Z",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRD07Z",  # Replacement of Left Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SRD0EZ",  # Replacement of Left Knee Joint with Articulating Spacer, Open Approach
        "0SRD0J9",  # Replacement of Left Knee Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRD0JA",  # Replacement of Left Knee Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRD0JZ",  # Replacement of Left Knee Joint with Synthetic Substitute, Open Approach
        "0SRD0KZ",  # Replacement of Left Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRD0L9",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRD0LA",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRD0LZ",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Open Approach
        "0SRD0M9",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRD0MA",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRD0MZ",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Open Approach
        "0SRD0N9",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Cemented, Open Approach
        "0SRD0NA",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Uncemented, Open Approach
        "0SRD0NZ",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Open Approach
        "0SRT07Z",  # Replacement of Right Knee Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRT0J9",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRT0JA",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRT0JZ",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRT0KZ",  # Replacement of Right Knee Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRU07Z",  # Replacement of Left Knee Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRU0J9",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRU0JA",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRU0JZ",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRU0KZ",  # Replacement of Left Knee Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRV07Z",  # Replacement of Right Knee Joint, Tibial Surface with Autologous Tissue Substitute, Open Approach
        "0SRV0J9",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRV0JA",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRV0JZ",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Open Approach
        "0SRV0KZ",  # Replacement of Right Knee Joint, Tibial Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRW07Z",  # Replacement of Left Knee Joint, Tibial Surface with Autologous Tissue Substitute, Open Approach
        "0SRW0J9",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRW0JA",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRW0JZ",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Open Approach
        "0SRW0KZ",  # Replacement of Left Knee Joint, Tibial Surface with Nonautologous Tissue Substitute, Open Approach
        "0SUC07Z",  # Supplement Right Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SUC09C",  # Supplement Right Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUC09Z",  # Supplement Right Knee Joint with Liner, Open Approach
        "0SUC0JZ",  # Supplement Right Knee Joint with Synthetic Substitute, Open Approach
        "0SUC0KZ",  # Supplement Right Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SUD07Z",  # Supplement Left Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SUD09C",  # Supplement Left Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUD09Z",  # Supplement Left Knee Joint with Liner, Open Approach
        "0SUD0JZ",  # Supplement Left Knee Joint with Synthetic Substitute, Open Approach
        "0SUD0KZ",  # Supplement Left Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SUT09Z",  # Supplement Right Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUU09Z",  # Supplement Left Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUV09Z",  # Supplement Right Knee Joint, Tibial Surface with Liner, Open Approach
        "0SUW09Z",  # Supplement Left Knee Joint, Tibial Surface with Liner, Open Approach
        "0SWC09Z",  # Revision of Liner in Right Knee Joint, Open Approach
        "0SWC0JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Open Approach
        "0SWC0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Open Approach
        "0SWD09Z",  # Revision of Liner in Left Knee Joint, Open Approach
        "0SWD0JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Open Approach
        "0SWD0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Open Approach
        "0SWT0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Open Approach
        "0SWU0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Open Approach
        "0SWV0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Open Approach
        "0SWW0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Open Approach
    }

    SNOMEDCT = {
        "1201734006",  # Total replacement of knee with impaction bone graft (procedure)
        "1217702001",  # Total replacement of knee joint using cone (procedure)
        "1220622006",  # Total replacement of knee joint using knee stem (procedure)
        "1222566006",  # Total replacement of knee joint with bone allograft (procedure)
        "1222567002",  # Total replacement of knee joint with bone autograft (procedure)
        "1222568007",  # Total replacement of knee joint using modular metal augmentation (procedure)
        "1222569004",  # Total replacement of knee joint using rotating hinge prosthesis (procedure)
        "1222570003",  # Total knee replacement using valgus-varus constrained prosthesis (procedure)
        "1231396005",  # Repair of joint of knee using robotic assistance (procedure)
        "1251509000",  # Repair of joint of right knee (procedure)
        "1251510005",  # Repair of joint of left knee (procedure)
        "179345007",  # Conversion to cemented total knee replacement (procedure)
        "179352009",  # Conversion to uncemented total knee replacement (procedure)
        "179406003",  # Conversion from cemented unicompartmental knee replacement (procedure)
        "179411001",  # Conversion from uncemented unicompartmental knee replacement (procedure)
        "179416006",  # Conversion from hybrid unicompartmental knee replacement (procedure)
        "19063003",  # Arthroplasty of knee (procedure)
        "265170009",  # Cemented prosthetic total arthroplasty of knee joint (procedure)
        "265172001",  # Uncemented total prosthetic arthroplasty of knee joint (procedure)
        "27748003",  # Arthroplasty of knee, condyle and plateau, lateral compartment (procedure)
        "29712008",  # Revision of prosthetic replacement of knee joint (procedure)
        "307817008",  # Cemented arthroplasty of knee joint (procedure)
        "307819006",  # Uncemented prosthetic arthroplasty of knee joint (procedure)
        "443681002",  # Total replacement of left knee joint (procedure)
        "443682009",  # Total replacement of right knee joint (procedure)
        "444463001",  # Replacement of bilateral knee joints (procedure)
        "445091006",  # Revision of hybrid knee replacement (procedure)
        "53386008",  # Arthroplasty of knee, tibial plateau (procedure)
        "609588000",  # Prosthetic total arthroplasty of knee joint (procedure)
        "64295009",  # Arthroplasty of knee, femoral condyles (procedure)
        "713687000",  # Total replacement of joint of knee with constrained condylar prosthesis (procedure)
        "88056004",  # Arthroplasty of knee with constrained prosthesis (procedure)
        "90827004",  # Arthroplasty of knee, condyle and plateau, medial compartment (procedure)
    }

class UrologicalSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of urological surgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for urologic surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Urological Surgery"
    OID = "2.16.840.1.113883.3.117.1.7.1.272"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0T13073",  # Bypass Right Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0T13074",  # Bypass Right Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0T13076",  # Bypass Right Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T13077",  # Bypass Right Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T13078",  # Bypass Right Kidney Pelvis to Colon with Autologous Tissue Substitute, Open Approach
        "0T13079",  # Bypass Right Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1307A",  # Bypass Right Kidney Pelvis to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1307B",  # Bypass Right Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1307C",  # Bypass Right Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1307D",  # Bypass Right Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T130J3",  # Bypass Right Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Open Approach
        "0T130J4",  # Bypass Right Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Open Approach
        "0T130J6",  # Bypass Right Kidney Pelvis to Right Ureter with Synthetic Substitute, Open Approach
        "0T130J7",  # Bypass Right Kidney Pelvis to Left Ureter with Synthetic Substitute, Open Approach
        "0T130J8",  # Bypass Right Kidney Pelvis to Colon with Synthetic Substitute, Open Approach
        "0T130J9",  # Bypass Right Kidney Pelvis to Colocutaneous with Synthetic Substitute, Open Approach
        "0T130JA",  # Bypass Right Kidney Pelvis to Ileum with Synthetic Substitute, Open Approach
        "0T130JB",  # Bypass Right Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach
        "0T130JC",  # Bypass Right Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T130JD",  # Bypass Right Kidney Pelvis to Cutaneous with Synthetic Substitute, Open Approach
        "0T130K3",  # Bypass Right Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0T130K4",  # Bypass Right Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0T130K6",  # Bypass Right Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T130K7",  # Bypass Right Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T130K8",  # Bypass Right Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T130K9",  # Bypass Right Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T130KA",  # Bypass Right Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T130KB",  # Bypass Right Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T130KC",  # Bypass Right Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T130KD",  # Bypass Right Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T130Z3",  # Bypass Right Kidney Pelvis to Right Kidney Pelvis, Open Approach
        "0T130Z4",  # Bypass Right Kidney Pelvis to Left Kidney Pelvis, Open Approach
        "0T130Z6",  # Bypass Right Kidney Pelvis to Right Ureter, Open Approach
        "0T130Z7",  # Bypass Right Kidney Pelvis to Left Ureter, Open Approach
        "0T130Z8",  # Bypass Right Kidney Pelvis to Colon, Open Approach
        "0T130Z9",  # Bypass Right Kidney Pelvis to Colocutaneous, Open Approach
        "0T130ZA",  # Bypass Right Kidney Pelvis to Ileum, Open Approach
        "0T130ZB",  # Bypass Right Kidney Pelvis to Bladder, Open Approach
        "0T130ZC",  # Bypass Right Kidney Pelvis to Ileocutaneous, Open Approach
        "0T130ZD",  # Bypass Right Kidney Pelvis to Cutaneous, Open Approach
        "0T14073",  # Bypass Left Kidney Pelvis to Right Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0T14074",  # Bypass Left Kidney Pelvis to Left Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0T14076",  # Bypass Left Kidney Pelvis to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T14077",  # Bypass Left Kidney Pelvis to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T14078",  # Bypass Left Kidney Pelvis to Colon with Autologous Tissue Substitute, Open Approach
        "0T14079",  # Bypass Left Kidney Pelvis to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1407A",  # Bypass Left Kidney Pelvis to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1407B",  # Bypass Left Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1407C",  # Bypass Left Kidney Pelvis to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1407D",  # Bypass Left Kidney Pelvis to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T140J3",  # Bypass Left Kidney Pelvis to Right Kidney Pelvis with Synthetic Substitute, Open Approach
        "0T140J4",  # Bypass Left Kidney Pelvis to Left Kidney Pelvis with Synthetic Substitute, Open Approach
        "0T140J6",  # Bypass Left Kidney Pelvis to Right Ureter with Synthetic Substitute, Open Approach
        "0T140J7",  # Bypass Left Kidney Pelvis to Left Ureter with Synthetic Substitute, Open Approach
        "0T140J8",  # Bypass Left Kidney Pelvis to Colon with Synthetic Substitute, Open Approach
        "0T140J9",  # Bypass Left Kidney Pelvis to Colocutaneous with Synthetic Substitute, Open Approach
        "0T140JA",  # Bypass Left Kidney Pelvis to Ileum with Synthetic Substitute, Open Approach
        "0T140JB",  # Bypass Left Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach
        "0T140JC",  # Bypass Left Kidney Pelvis to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T140JD",  # Bypass Left Kidney Pelvis to Cutaneous with Synthetic Substitute, Open Approach
        "0T140K3",  # Bypass Left Kidney Pelvis to Right Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0T140K4",  # Bypass Left Kidney Pelvis to Left Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0T140K6",  # Bypass Left Kidney Pelvis to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T140K7",  # Bypass Left Kidney Pelvis to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T140K8",  # Bypass Left Kidney Pelvis to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T140K9",  # Bypass Left Kidney Pelvis to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T140KA",  # Bypass Left Kidney Pelvis to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T140KB",  # Bypass Left Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T140KC",  # Bypass Left Kidney Pelvis to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T140KD",  # Bypass Left Kidney Pelvis to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T140Z3",  # Bypass Left Kidney Pelvis to Right Kidney Pelvis, Open Approach
        "0T140Z4",  # Bypass Left Kidney Pelvis to Left Kidney Pelvis, Open Approach
        "0T140Z6",  # Bypass Left Kidney Pelvis to Right Ureter, Open Approach
        "0T140Z7",  # Bypass Left Kidney Pelvis to Left Ureter, Open Approach
        "0T140Z8",  # Bypass Left Kidney Pelvis to Colon, Open Approach
        "0T140Z9",  # Bypass Left Kidney Pelvis to Colocutaneous, Open Approach
        "0T140ZA",  # Bypass Left Kidney Pelvis to Ileum, Open Approach
        "0T140ZB",  # Bypass Left Kidney Pelvis to Bladder, Open Approach
        "0T140ZC",  # Bypass Left Kidney Pelvis to Ileocutaneous, Open Approach
        "0T140ZD",  # Bypass Left Kidney Pelvis to Cutaneous, Open Approach
        "0T16076",  # Bypass Right Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T16077",  # Bypass Right Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T16078",  # Bypass Right Ureter to Colon with Autologous Tissue Substitute, Open Approach
        "0T16079",  # Bypass Right Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1607A",  # Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1607B",  # Bypass Right Ureter to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1607C",  # Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1607D",  # Bypass Right Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T160J6",  # Bypass Right Ureter to Right Ureter with Synthetic Substitute, Open Approach
        "0T160J7",  # Bypass Right Ureter to Left Ureter with Synthetic Substitute, Open Approach
        "0T160J8",  # Bypass Right Ureter to Colon with Synthetic Substitute, Open Approach
        "0T160J9",  # Bypass Right Ureter to Colocutaneous with Synthetic Substitute, Open Approach
        "0T160JA",  # Bypass Right Ureter to Ileum with Synthetic Substitute, Open Approach
        "0T160JB",  # Bypass Right Ureter to Bladder with Synthetic Substitute, Open Approach
        "0T160JC",  # Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T160JD",  # Bypass Right Ureter to Cutaneous with Synthetic Substitute, Open Approach
        "0T160K6",  # Bypass Right Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T160K7",  # Bypass Right Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T160K8",  # Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T160K9",  # Bypass Right Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T160KA",  # Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T160KB",  # Bypass Right Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T160KC",  # Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T160KD",  # Bypass Right Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T160Z6",  # Bypass Right Ureter to Right Ureter, Open Approach
        "0T160Z7",  # Bypass Right Ureter to Left Ureter, Open Approach
        "0T160Z8",  # Bypass Right Ureter to Colon, Open Approach
        "0T160Z9",  # Bypass Right Ureter to Colocutaneous, Open Approach
        "0T160ZA",  # Bypass Right Ureter to Ileum, Open Approach
        "0T160ZB",  # Bypass Right Ureter to Bladder, Open Approach
        "0T160ZC",  # Bypass Right Ureter to Ileocutaneous, Open Approach
        "0T160ZD",  # Bypass Right Ureter to Cutaneous, Open Approach
        "0T17076",  # Bypass Left Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T17077",  # Bypass Left Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T17078",  # Bypass Left Ureter to Colon with Autologous Tissue Substitute, Open Approach
        "0T17079",  # Bypass Left Ureter to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1707A",  # Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1707B",  # Bypass Left Ureter to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1707C",  # Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1707D",  # Bypass Left Ureter to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T170J6",  # Bypass Left Ureter to Right Ureter with Synthetic Substitute, Open Approach
        "0T170J7",  # Bypass Left Ureter to Left Ureter with Synthetic Substitute, Open Approach
        "0T170J8",  # Bypass Left Ureter to Colon with Synthetic Substitute, Open Approach
        "0T170J9",  # Bypass Left Ureter to Colocutaneous with Synthetic Substitute, Open Approach
        "0T170JA",  # Bypass Left Ureter to Ileum with Synthetic Substitute, Open Approach
        "0T170JB",  # Bypass Left Ureter to Bladder with Synthetic Substitute, Open Approach
        "0T170JC",  # Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T170JD",  # Bypass Left Ureter to Cutaneous with Synthetic Substitute, Open Approach
        "0T170K6",  # Bypass Left Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T170K7",  # Bypass Left Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T170K8",  # Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T170K9",  # Bypass Left Ureter to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T170KA",  # Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T170KB",  # Bypass Left Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T170KC",  # Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T170KD",  # Bypass Left Ureter to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T170Z6",  # Bypass Left Ureter to Right Ureter, Open Approach
        "0T170Z7",  # Bypass Left Ureter to Left Ureter, Open Approach
        "0T170Z8",  # Bypass Left Ureter to Colon, Open Approach
        "0T170Z9",  # Bypass Left Ureter to Colocutaneous, Open Approach
        "0T170ZA",  # Bypass Left Ureter to Ileum, Open Approach
        "0T170ZB",  # Bypass Left Ureter to Bladder, Open Approach
        "0T170ZC",  # Bypass Left Ureter to Ileocutaneous, Open Approach
        "0T170ZD",  # Bypass Left Ureter to Cutaneous, Open Approach
        "0T18076",  # Bypass Bilateral Ureters to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T18077",  # Bypass Bilateral Ureters to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T18078",  # Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Open Approach
        "0T18079",  # Bypass Bilateral Ureters to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1807A",  # Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1807B",  # Bypass Bilateral Ureters to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1807C",  # Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1807D",  # Bypass Bilateral Ureters to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T180J6",  # Bypass Bilateral Ureters to Right Ureter with Synthetic Substitute, Open Approach
        "0T180J7",  # Bypass Bilateral Ureters to Left Ureter with Synthetic Substitute, Open Approach
        "0T180J8",  # Bypass Bilateral Ureters to Colon with Synthetic Substitute, Open Approach
        "0T180J9",  # Bypass Bilateral Ureters to Colocutaneous with Synthetic Substitute, Open Approach
        "0T180JA",  # Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Open Approach
        "0T180JB",  # Bypass Bilateral Ureters to Bladder with Synthetic Substitute, Open Approach
        "0T180JC",  # Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T180JD",  # Bypass Bilateral Ureters to Cutaneous with Synthetic Substitute, Open Approach
        "0T180K6",  # Bypass Bilateral Ureters to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T180K7",  # Bypass Bilateral Ureters to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T180K8",  # Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T180K9",  # Bypass Bilateral Ureters to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T180KA",  # Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T180KB",  # Bypass Bilateral Ureters to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T180KC",  # Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T180KD",  # Bypass Bilateral Ureters to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T180Z6",  # Bypass Bilateral Ureters to Right Ureter, Open Approach
        "0T180Z7",  # Bypass Bilateral Ureters to Left Ureter, Open Approach
        "0T180Z8",  # Bypass Bilateral Ureters to Colon, Open Approach
        "0T180Z9",  # Bypass Bilateral Ureters to Colocutaneous, Open Approach
        "0T180ZA",  # Bypass Bilateral Ureters to Ileum, Open Approach
        "0T180ZB",  # Bypass Bilateral Ureters to Bladder, Open Approach
        "0T180ZC",  # Bypass Bilateral Ureters to Ileocutaneous, Open Approach
        "0T180ZD",  # Bypass Bilateral Ureters to Cutaneous, Open Approach
        "0T1B079",  # Bypass Bladder to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B07C",  # Bypass Bladder to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B07D",  # Bypass Bladder to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B0J9",  # Bypass Bladder to Colocutaneous with Synthetic Substitute, Open Approach
        "0T1B0JC",  # Bypass Bladder to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T1B0JD",  # Bypass Bladder to Cutaneous with Synthetic Substitute, Open Approach
        "0T1B0K9",  # Bypass Bladder to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0KC",  # Bypass Bladder to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0KD",  # Bypass Bladder to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0Z9",  # Bypass Bladder to Colocutaneous, Open Approach
        "0T1B0ZC",  # Bypass Bladder to Ileocutaneous, Open Approach
        "0T1B0ZD",  # Bypass Bladder to Cutaneous, Open Approach
        "0T730DZ",  # Dilation of Right Kidney Pelvis with Intraluminal Device, Open Approach
        "0T730ZZ",  # Dilation of Right Kidney Pelvis, Open Approach
        "0T737DZ",  # Dilation of Right Kidney Pelvis with Intraluminal Device, Via Natural or Artificial Opening
        "0T737ZZ",  # Dilation of Right Kidney Pelvis, Via Natural or Artificial Opening
        "0T738DZ",  # Dilation of Right Kidney Pelvis with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0T738ZZ",  # Dilation of Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0T740DZ",  # Dilation of Left Kidney Pelvis with Intraluminal Device, Open Approach
        "0T740ZZ",  # Dilation of Left Kidney Pelvis, Open Approach
        "0T747DZ",  # Dilation of Left Kidney Pelvis with Intraluminal Device, Via Natural or Artificial Opening
        "0T747ZZ",  # Dilation of Left Kidney Pelvis, Via Natural or Artificial Opening
        "0T748DZ",  # Dilation of Left Kidney Pelvis with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0T748ZZ",  # Dilation of Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0T780ZZ",  # Dilation of Bilateral Ureters, Open Approach
        "0T7B0DZ",  # Dilation of Bladder with Intraluminal Device, Open Approach
        "0T7B0ZZ",  # Dilation of Bladder, Open Approach
        "0T7B7DZ",  # Dilation of Bladder with Intraluminal Device, Via Natural or Artificial Opening
        "0T7B7ZZ",  # Dilation of Bladder, Via Natural or Artificial Opening
        "0T7B8DZ",  # Dilation of Bladder with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0T7B8ZZ",  # Dilation of Bladder, Via Natural or Artificial Opening Endoscopic
        "0T9300Z",  # Drainage of Right Kidney Pelvis with Drainage Device, Open Approach
        "0T930ZZ",  # Drainage of Right Kidney Pelvis, Open Approach
        "0T9400Z",  # Drainage of Left Kidney Pelvis with Drainage Device, Open Approach
        "0T940ZZ",  # Drainage of Left Kidney Pelvis, Open Approach
        "0TB00ZZ",  # Excision of Right Kidney, Open Approach
        "0TB07ZZ",  # Excision of Right Kidney, Via Natural or Artificial Opening
        "0TB08ZZ",  # Excision of Right Kidney, Via Natural or Artificial Opening Endoscopic
        "0TB10ZZ",  # Excision of Left Kidney, Open Approach
        "0TB17ZZ",  # Excision of Left Kidney, Via Natural or Artificial Opening
        "0TB18ZZ",  # Excision of Left Kidney, Via Natural or Artificial Opening Endoscopic
        "0TB30ZZ",  # Excision of Right Kidney Pelvis, Open Approach
        "0TB37ZZ",  # Excision of Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TB38ZZ",  # Excision of Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TB40ZZ",  # Excision of Left Kidney Pelvis, Open Approach
        "0TB47ZZ",  # Excision of Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TB48ZZ",  # Excision of Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TB60ZZ",  # Excision of Right Ureter, Open Approach
        "0TB67ZZ",  # Excision of Right Ureter, Via Natural or Artificial Opening
        "0TB68ZZ",  # Excision of Right Ureter, Via Natural or Artificial Opening Endoscopic
        "0TB70ZZ",  # Excision of Left Ureter, Open Approach
        "0TB77ZZ",  # Excision of Left Ureter, Via Natural or Artificial Opening
        "0TB78ZZ",  # Excision of Left Ureter, Via Natural or Artificial Opening Endoscopic
        "0TBB0ZZ",  # Excision of Bladder, Open Approach
        "0TBB7ZZ",  # Excision of Bladder, Via Natural or Artificial Opening
        "0TBB8ZZ",  # Excision of Bladder, Via Natural or Artificial Opening Endoscopic
        "0TBC0ZZ",  # Excision of Bladder Neck, Open Approach
        "0TBC7ZZ",  # Excision of Bladder Neck, Via Natural or Artificial Opening
        "0TBC8ZZ",  # Excision of Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0TBD0ZZ",  # Excision of Urethra, Open Approach
        "0TBD7ZZ",  # Excision of Urethra, Via Natural or Artificial Opening
        "0TBD8ZZ",  # Excision of Urethra, Via Natural or Artificial Opening Endoscopic
        "0TBDXZZ",  # Excision of Urethra, External Approach
        "0TJ50ZZ",  # Inspection of Kidney, Open Approach
        "0TMB0ZZ",  # Reattachment of Bladder, Open Approach
        "0TMC0ZZ",  # Reattachment of Bladder Neck, Open Approach
        "0TN00ZZ",  # Release Right Kidney, Open Approach
        "0TN07ZZ",  # Release Right Kidney, Via Natural or Artificial Opening
        "0TN08ZZ",  # Release Right Kidney, Via Natural or Artificial Opening Endoscopic
        "0TN10ZZ",  # Release Left Kidney, Open Approach
        "0TN17ZZ",  # Release Left Kidney, Via Natural or Artificial Opening
        "0TN18ZZ",  # Release Left Kidney, Via Natural or Artificial Opening Endoscopic
        "0TN30ZZ",  # Release Right Kidney Pelvis, Open Approach
        "0TN37ZZ",  # Release Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TN38ZZ",  # Release Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TN40ZZ",  # Release Left Kidney Pelvis, Open Approach
        "0TN47ZZ",  # Release Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TN48ZZ",  # Release Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TN60ZZ",  # Release Right Ureter, Open Approach
        "0TN67ZZ",  # Release Right Ureter, Via Natural or Artificial Opening
        "0TN68ZZ",  # Release Right Ureter, Via Natural or Artificial Opening Endoscopic
        "0TN70ZZ",  # Release Left Ureter, Open Approach
        "0TN77ZZ",  # Release Left Ureter, Via Natural or Artificial Opening
        "0TN78ZZ",  # Release Left Ureter, Via Natural or Artificial Opening Endoscopic
        "0TNB0ZZ",  # Release Bladder, Open Approach
        "0TNB7ZZ",  # Release Bladder, Via Natural or Artificial Opening
        "0TNB8ZZ",  # Release Bladder, Via Natural or Artificial Opening Endoscopic
        "0TNC0ZZ",  # Release Bladder Neck, Open Approach
        "0TNC7ZZ",  # Release Bladder Neck, Via Natural or Artificial Opening
        "0TNC8ZZ",  # Release Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0TQ60ZZ",  # Repair Right Ureter, Open Approach
        "0TQ67ZZ",  # Repair Right Ureter, Via Natural or Artificial Opening
        "0TQ68ZZ",  # Repair Right Ureter, Via Natural or Artificial Opening Endoscopic
        "0TQ70ZZ",  # Repair Left Ureter, Open Approach
        "0TQ77ZZ",  # Repair Left Ureter, Via Natural or Artificial Opening
        "0TQ78ZZ",  # Repair Left Ureter, Via Natural or Artificial Opening Endoscopic
        "0TQB0ZZ",  # Repair Bladder, Open Approach
        "0TQB7ZZ",  # Repair Bladder, Via Natural or Artificial Opening
        "0TQB8ZZ",  # Repair Bladder, Via Natural or Artificial Opening Endoscopic
        "0TQC0ZZ",  # Repair Bladder Neck, Open Approach
        "0TQC7ZZ",  # Repair Bladder Neck, Via Natural or Artificial Opening
        "0TQC8ZZ",  # Repair Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0TRB07Z",  # Replacement of Bladder with Autologous Tissue Substitute, Open Approach
        "0TRB0JZ",  # Replacement of Bladder with Synthetic Substitute, Open Approach
        "0TRB0KZ",  # Replacement of Bladder with Nonautologous Tissue Substitute, Open Approach
        "0TRB77Z",  # Replacement of Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRB7JZ",  # Replacement of Bladder with Synthetic Substitute, Via Natural or Artificial Opening
        "0TRB7KZ",  # Replacement of Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRB87Z",  # Replacement of Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TRB8JZ",  # Replacement of Bladder with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TRB8KZ",  # Replacement of Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TRC07Z",  # Replacement of Bladder Neck with Autologous Tissue Substitute, Open Approach
        "0TRC0JZ",  # Replacement of Bladder Neck with Synthetic Substitute, Open Approach
        "0TRC0KZ",  # Replacement of Bladder Neck with Nonautologous Tissue Substitute, Open Approach
        "0TRC77Z",  # Replacement of Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRC7JZ",  # Replacement of Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening
        "0TRC7KZ",  # Replacement of Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRC87Z",  # Replacement of Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TRC8JZ",  # Replacement of Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TRC8KZ",  # Replacement of Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TSB0ZZ",  # Reposition Bladder, Open Approach
        "0TT00ZZ",  # Resection of Right Kidney, Open Approach
        "0TT10ZZ",  # Resection of Left Kidney, Open Approach
        "0TT20ZZ",  # Resection of Bilateral Kidneys, Open Approach
        "0TT30ZZ",  # Resection of Right Kidney Pelvis, Open Approach
        "0TT37ZZ",  # Resection of Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TT38ZZ",  # Resection of Right Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TT40ZZ",  # Resection of Left Kidney Pelvis, Open Approach
        "0TT47ZZ",  # Resection of Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TT48ZZ",  # Resection of Left Kidney Pelvis, Via Natural or Artificial Opening Endoscopic
        "0TT60ZZ",  # Resection of Right Ureter, Open Approach
        "0TT67ZZ",  # Resection of Right Ureter, Via Natural or Artificial Opening
        "0TT68ZZ",  # Resection of Right Ureter, Via Natural or Artificial Opening Endoscopic
        "0TT70ZZ",  # Resection of Left Ureter, Open Approach
        "0TT77ZZ",  # Resection of Left Ureter, Via Natural or Artificial Opening
        "0TT78ZZ",  # Resection of Left Ureter, Via Natural or Artificial Opening Endoscopic
        "0TTB0ZZ",  # Resection of Bladder, Open Approach
        "0TTB7ZZ",  # Resection of Bladder, Via Natural or Artificial Opening
        "0TTB8ZZ",  # Resection of Bladder, Via Natural or Artificial Opening Endoscopic
        "0TTC0ZZ",  # Resection of Bladder Neck, Open Approach
        "0TTC7ZZ",  # Resection of Bladder Neck, Via Natural or Artificial Opening
        "0TTC8ZZ",  # Resection of Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0TTD0ZZ",  # Resection of Urethra, Open Approach
        "0TTD7ZZ",  # Resection of Urethra, Via Natural or Artificial Opening
        "0TTD8ZZ",  # Resection of Urethra, Via Natural or Artificial Opening Endoscopic
        "0TU307Z",  # Supplement Right Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0TU30JZ",  # Supplement Right Kidney Pelvis with Synthetic Substitute, Open Approach
        "0TU30KZ",  # Supplement Right Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0TU377Z",  # Supplement Right Kidney Pelvis with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU37JZ",  # Supplement Right Kidney Pelvis with Synthetic Substitute, Via Natural or Artificial Opening
        "0TU37KZ",  # Supplement Right Kidney Pelvis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU387Z",  # Supplement Right Kidney Pelvis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU38JZ",  # Supplement Right Kidney Pelvis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU38KZ",  # Supplement Right Kidney Pelvis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU407Z",  # Supplement Left Kidney Pelvis with Autologous Tissue Substitute, Open Approach
        "0TU40JZ",  # Supplement Left Kidney Pelvis with Synthetic Substitute, Open Approach
        "0TU40KZ",  # Supplement Left Kidney Pelvis with Nonautologous Tissue Substitute, Open Approach
        "0TU477Z",  # Supplement Left Kidney Pelvis with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU47JZ",  # Supplement Left Kidney Pelvis with Synthetic Substitute, Via Natural or Artificial Opening
        "0TU47KZ",  # Supplement Left Kidney Pelvis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU487Z",  # Supplement Left Kidney Pelvis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU48JZ",  # Supplement Left Kidney Pelvis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU48KZ",  # Supplement Left Kidney Pelvis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU607Z",  # Supplement Right Ureter with Autologous Tissue Substitute, Open Approach
        "0TU60JZ",  # Supplement Right Ureter with Synthetic Substitute, Open Approach
        "0TU60KZ",  # Supplement Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0TU677Z",  # Supplement Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU67JZ",  # Supplement Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening
        "0TU67KZ",  # Supplement Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU687Z",  # Supplement Right Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU68JZ",  # Supplement Right Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU68KZ",  # Supplement Right Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU707Z",  # Supplement Left Ureter with Autologous Tissue Substitute, Open Approach
        "0TU70JZ",  # Supplement Left Ureter with Synthetic Substitute, Open Approach
        "0TU70KZ",  # Supplement Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0TU777Z",  # Supplement Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU77JZ",  # Supplement Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening
        "0TU77KZ",  # Supplement Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TU787Z",  # Supplement Left Ureter with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU78JZ",  # Supplement Left Ureter with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TU78KZ",  # Supplement Left Ureter with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUB07Z",  # Supplement Bladder with Autologous Tissue Substitute, Open Approach
        "0TUB0JZ",  # Supplement Bladder with Synthetic Substitute, Open Approach
        "0TUB0KZ",  # Supplement Bladder with Nonautologous Tissue Substitute, Open Approach
        "0TUB77Z",  # Supplement Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUB7JZ",  # Supplement Bladder with Synthetic Substitute, Via Natural or Artificial Opening
        "0TUB7KZ",  # Supplement Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUB87Z",  # Supplement Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUB8JZ",  # Supplement Bladder with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUB8KZ",  # Supplement Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUC07Z",  # Supplement Bladder Neck with Autologous Tissue Substitute, Open Approach
        "0TUC0JZ",  # Supplement Bladder Neck with Synthetic Substitute, Open Approach
        "0TUC0KZ",  # Supplement Bladder Neck with Nonautologous Tissue Substitute, Open Approach
        "0TUC77Z",  # Supplement Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUC7JZ",  # Supplement Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening
        "0TUC7KZ",  # Supplement Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUC87Z",  # Supplement Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUD07Z",  # Supplement Urethra with Autologous Tissue Substitute, Open Approach
        "0TUD0JZ",  # Supplement Urethra with Synthetic Substitute, Open Approach
        "0TUD0KZ",  # Supplement Urethra with Nonautologous Tissue Substitute, Open Approach
        "0TUD77Z",  # Supplement Urethra with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUD7JZ",  # Supplement Urethra with Synthetic Substitute, Via Natural or Artificial Opening
        "0TUD7KZ",  # Supplement Urethra with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUD87Z",  # Supplement Urethra with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUD8JZ",  # Supplement Urethra with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUD8KZ",  # Supplement Urethra with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0TUDX7Z",  # Supplement Urethra with Autologous Tissue Substitute, External Approach
        "0TVB0CZ",  # Restriction of Bladder with Extraluminal Device, Open Approach
        "0TVB0DZ",  # Restriction of Bladder with Intraluminal Device, Open Approach
        "0TVB0ZZ",  # Restriction of Bladder, Open Approach
        "0TVB7DZ",  # Restriction of Bladder with Intraluminal Device, Via Natural or Artificial Opening
        "0TVB7ZZ",  # Restriction of Bladder, Via Natural or Artificial Opening
        "0TVB8DZ",  # Restriction of Bladder with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0TVB8ZZ",  # Restriction of Bladder, Via Natural or Artificial Opening Endoscopic
        "0TVC0CZ",  # Restriction of Bladder Neck with Extraluminal Device, Open Approach
        "0TVC0DZ",  # Restriction of Bladder Neck with Intraluminal Device, Open Approach
        "0TVC0ZZ",  # Restriction of Bladder Neck, Open Approach
        "0TVC7DZ",  # Restriction of Bladder Neck with Intraluminal Device, Via Natural or Artificial Opening
        "0TVC7ZZ",  # Restriction of Bladder Neck, Via Natural or Artificial Opening
        "0TVC8DZ",  # Restriction of Bladder Neck with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0TVC8ZZ",  # Restriction of Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0VT00ZZ",  # Resection of Prostate, Open Approach
        "0VT07ZZ",  # Resection of Prostate, Via Natural or Artificial Opening
        "0VT08ZZ",  # Resection of Prostate, Via Natural or Artificial Opening Endoscopic
        "0VT30ZZ",  # Resection of Bilateral Seminal Vesicles, Open Approach
        "0WBH0ZZ",  # Excision of Retroperitoneum, Open Approach
        "0WQF0ZZ",  # Repair Abdominal Wall, Open Approach
        "0WQFXZ2",  # Repair Abdominal Wall, Stoma, External Approach
        "0WQFXZZ",  # Repair Abdominal Wall, External Approach
    }

    SNOMEDCT = {
        "108022006",  # Kidney excision (procedure)
        "108034003",  # Excision of urinary bladder (procedure)
        "11050006",  # Closure of vesicouterine fistula with hysterectomy (procedure)
        "112902005",  # Partial urinary cystectomy (procedure)
        "116033007",  # Radical nephrectomy (procedure)
        "116166009",  # Radical nephrectomy AND ipsilateral adrenalectomy (procedure)
        "119991007",  # Closure of urinary bladder (procedure)
        "12976005",  # Donor nephrectomy (procedure)
        "13246003",  # Closure of enterovesical fistula with urinary bladder resection (procedure)
        "13991004",  # Repair of ileovesical fistula (procedure)
        "15044006",  # Repair of exstrophy of urinary bladder (procedure)
        "15918009",  # Closure of vesicovaginal fistula by transvesical and vaginal approach (procedure)
        "16068008",  # Repair of vesicosigmoidovaginal fistula (procedure)
        "175905003",  # Total nephrectomy (procedure)
        "175907006",  # Nephroureterectomy (procedure)
        "175908001",  # Heminephrectomy for horseshoe kidney (procedure)
        "175911000",  # Live donor nephrectomy (procedure)
        "175912007",  # Nephroureterectomy with open lower ureterectomy (procedure)
        "175913002",  # Nephroureterectomy with pluck lower ureterectomy (procedure)
        "175916005",  # Heminephrectomy for duplex kidney (procedure)
        "175917001",  # Upper polar partial nephrectomy (procedure)
        "175918006",  # Lower polar partial nephrectomy (procedure)
        "176004009",  # Construction of continent ileal conduit (procedure)
        "176024008",  # Anastomosis of ureter to urinary bladder (procedure)
        "176025009",  # Ileal replacement of ureter (procedure)
        "176106009",  # Radical cystoprostatourethrectomy (procedure)
        "176107000",  # Radical cystourethrectomy - female (procedure)
        "176108005",  # Simple cystectomy (procedure)
        "176117005",  # Cecocystoplasty (procedure)
        "176118000",  # Ileocecocystoplasty (procedure)
        "176123000",  # Clam ileocystoplasty (procedure)
        "176132003",  # Repair of rupture of urinary bladder (procedure)
        "176144000",  # Creation of vesicovaginal fistula (procedure)
        "176165006",  # Transurethral operation to increase urinary bladder capacity (procedure)
        "176234008",  # Open resection of female urinary bladder neck (procedure)
        "176260009",  # Transvesical two stage prostatectomy (procedure)
        "176275001",  # Open resection of male urinary bladder neck (procedure)
        "176722004",  # Abdominal repair vesicovaginal fistula (procedure)
        "18008000",  # Closure of vesicovaginal fistula by vaginal approach (procedure)
        "18280002",  # Repair of vesicourethrorectal fistula (procedure)
        "1866009",  # Total excision of bilateral kidneys (procedure)
        "19149007",  # Subtotal retropubic prostatectomy (procedure)
        "21338008",  # Cystourethroplasty with bilateral ureteroneocystostomy (procedure)
        "21482000",  # Closure of enterovesical fistula with bowel resection (procedure)
        "22783008",  # Construction of anus for congenital absence with repair of urinary fistula (procedure)
        "22829001",  # Repair of vesicoperineal fistula (procedure)
        "2344008",  # Complicated cystorrhaphy (procedure)
        "236139004",  # Nephroureterectomy with cystoscopically assisted ureterectomy (procedure)
        "236140002",  # Middle partial nephrectomy (procedure)
        "236165000",  # Anastomosis of ureter to urinary bladder with psoas hitch (procedure)
        "236167008",  # Urinary undiversion (procedure)
        "236209003",  # Salvage cystoprostatourethrectomy (procedure)
        "236210008",  # Salvage cystourethrectomy - female (procedure)
        "236211007",  # Salvage cystoprostatectomy (procedure)
        "236213005",  # Partial cystectomy with excision of urachal remnant (procedure)
        "236214004",  # Primary closure of exstrophy of urinary bladder (procedure)
        "236215003",  # Single stage primary closure of exstrophy of urinary bladder (procedure)
        "236216002",  # First stage of staged closure of exstrophy of urinary bladder (procedure)
        "236219009",  # Closure of neck of urinary bladder (procedure)
        "236247008",  # Reconstruction of female urinary bladder neck (procedure)
        "236251005",  # Reconstruction of male urinary bladder neck (procedure)
        "240325001",  # Cadaver donor nephrectomy (procedure)
        "24221008",  # Repair of rectovesical fistula with colostomy (procedure)
        "243306000",  # Ureteroileostomy (procedure)
        "24677003",  # Closure of cervicovesical fistula (procedure)
        "25466008",  # Anastomosis of urinary bladder (procedure)
        "277837003",  # Detrusor myotomy (procedure)
        "2813008",  # Nephroureterocystectomy (procedure)
        "281416002",  # Second stage repair of exstrophy of urinary bladder (procedure)
        "28579000",  # Radical retropubic prostatectomy with lymph node biopsy (procedure)
        "28609005",  # Closure of vesicouterine fistula (procedure)
        "287716007",  # Urinary bladder reconstruction (procedure)
        "287718008",  # Total cystectomy and urethrectomy (procedure)
        "287720006",  # Cystoprostatectomy (procedure)
        "287722003",  # Cystectomy - wedge resection (procedure)
        "287723008",  # Excision of dome of urinary bladder (procedure)
        "288013007",  # Open excision of urinary bladder (procedure)
        "288014001",  # Reconstruction of sphincter of urinary bladder (procedure)
        "32417007",  # Paquin operation, ureteroneocystostomy (procedure)
        "33654003",  # Repair of old obstetric laceration of urinary bladder (procedure)
        "34006000",  # Hutch operation, ureteroneocystostomy (procedure)
        "361249003",  # Nephrectomy with excision of perirenal tissue (procedure)
        "361256009",  # Autoaugmentation of urinary bladder (procedure)
        "361257000",  # Clam cystoplasty (procedure)
        "36253005",  # Suprapubic prostatectomy (procedure)
        "36432003",  # Anastomosis of urinary bladder to intestine (procedure)
        "36586008",  # Bischoff operation, ureteroneocystostomy (procedure)
        "36935008",  # Closure of urethroperineovesical fistula (procedure)
        "386652006",  # Colocystoplasty (procedure)
        "387707007",  # Repair AND revision of stoma of urinary bladder (procedure)
        "39214002",  # Closure of rectovesicovaginal fistula (procedure)
        "40173000",  # Incision of urinary bladder (procedure)
        "41416003",  # Radical retropubic prostatectomy with bilateral pelvic lymphadenectomy (procedure)
        "439110003",  # Cystoscopic anastomosis of ureter to urinary bladder (procedure)
        "439235001",  # Laparoscopic radical nephrectomy (procedure)
        "439497008",  # Cystourethroscopy and excision of internal sphincter (procedure)
        "439739008",  # Laparoscopic partial excision of kidney (procedure)
        "439964001",  # Laparoscopic excision of kidney and total excision of ureter (procedure)
        "440076006",  # Cystoscopic anastomosis of ureter to urinary bladder with insertion of stent into ureter (procedure)
        "440446009",  # Laparoscopic excision of kidney and partial excision of ureter (procedure)
        "442919002",  # Adrenal sparing radical nephrectomy (procedure)
        "443869003",  # Total excision of right kidney (procedure)
        "444083005",  # Total excision of left kidney (procedure)
        "446296004",  # Laparoscopic total excision of kidney (procedure)
        "446586007",  # Laparoscopic partial excision of urinary bladder (procedure)
        "446894005",  # Laparoscopic partial excision of kidney by retroperitoneal approach (procedure)
        "446990000",  # Laparoscopic excision of ureter and kidney (procedure)
        "447307008",  # Laparoscopic total excision of urinary bladder (procedure)
        "447308003",  # Excision of urethra and urinary bladder (procedure)
        "447527008",  # Laparoscopic total excision of kidney by retroperitoneal approach (procedure)
        "447531002",  # Laparoscopic excision of ureter and kidney by retroperitoneal approach (procedure)
        "48007004",  # Augmentation of urinary bladder (procedure)
        "4895001",  # Construction of sigmoid urinary bladder (procedure)
        "49780003",  # Heminephrectomy (procedure)
        "49935007",  # Partial pelvectomy of kidney (procedure)
        "51870000",  # Excision of lesion of kidney with partial nephrectomy (procedure)
        "52969004",  # Closure of vesicocutaneous fistula (procedure)
        "533005",  # Closure of vesicocolic fistula (procedure)
        "535003",  # Closure of ureterovesicovaginal fistula (procedure)
        "54033009",  # Cystourethroplasty (procedure)
        "56256008",  # Closure of vesicocervicovaginal fistula (procedure)
        "56614001",  # Repair of fistula involving urinary bladder and intestine (procedure)
        "63016009",  # Total resection of urinary bladder (procedure)
        "63361002",  # Closure of fistula of urinary bladder (procedure)
        "64063003",  # Radical cystectomy (procedure)
        "65551008",  # Radical retropubic prostatectomy (procedure)
        "66633009",  # Nephrocystanastomosis (procedure)
        "67113002",  # Closure of vesicometrorectal fistula (procedure)
        "67212000",  # Ileocystoplasty (procedure)
        "67598001",  # Retropubic prostatectomy (procedure)
        "68960009",  # Closure of enterovesical fistula with bowel and urinary bladder resection (procedure)
        "69926006",  # Closure of vesicosigmoidal fistula (procedure)
        "71550003",  # Closure of cystostomy (procedure)
        "73380001",  # Repair of laceration of urinary bladder by suture (procedure)
        "73953002",  # Closure of rectovesical fistula (procedure)
        "73966002",  # Hemicystectomy (procedure)
        "74146001",  # Repair of urinary bladder (procedure)
        "76414006",  # Closure of urethrovesicovaginal fistula (procedure)
        "78062004",  # Closure of vaginovesical fistula (procedure)
        "80945002",  # Cystoproctostomy (procedure)
        "81232004",  # Radical cystoprostatectomy (procedure)
        "81516001",  # Partial nephrectomy (procedure)
        "83444008",  # Renal papillectomy (procedure)
        "83483005",  # Closure of urethrovesical fistula (procedure)
        "83588004",  # Ureteroneocystostomy with urinary bladder flap (procedure)
        "83809000",  # Renal calicectomy (procedure)
        "84575008",  # Urinary bladder trigonectomy (procedure)
        "85175006",  # Partial cystectomy, complicated (procedure)
        "88930008",  # Total nephrectomy in recipient (procedure)
        "89230009",  # Closure of ureterovesical fistula (procedure)
        "90648000",  # Partial cystectomy with reimplantation of ureter into urinary bladder (procedure)
        "91531008",  # One stage subtotal suprapubic prostatectomy (procedure)
    }

class ChildInfluenzaVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for an influenza vaccination for use in children 2 and younger.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent an influenza vaccination for use in children 2 and younger.

    **Exclusion Criteria:** Excludes influenza vaccines not recommended for use in children 2 and younger.
    """

    VALUE_SET_NAME = "Child Influenza Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1044"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90655",  # Influenza virus vaccine, trivalent (IIV3), split virus, preservative free, 0.25 mL dosage, for intramuscular use
        "90657",  # Influenza virus vaccine, trivalent (IIV3), split virus, 0.25 mL dosage, for intramuscular use
        "90661",  # Influenza virus vaccine, trivalent (ccIIV3), derived from cell cultures, subunit, antibiotic free, 0.5 mL dosage, for intramuscular use
        "90673",  # Influenza virus vaccine, trivalent (RIV3), derived from recombinant DNA, hemagglutinin (HA) protein only, preservative and antibiotic free, for intramuscular use
        "90674",  # Influenza virus vaccine, quadrivalent (ccIIV4), derived from cell cultures, subunit, preservative and antibiotic free, 0.5 mL dosage, for intramuscular use
        "90685",  # Influenza virus vaccine, quadrivalent (IIV4), split virus, preservative free, 0.25 mL dosage, for intramuscular use
        "90686",  # Influenza virus vaccine, quadrivalent (IIV4), split virus, preservative free, 0.5 mL dosage, for intramuscular use
        "90687",  # Influenza virus vaccine, quadrivalent (IIV4), split virus, 0.25 mL dosage, for intramuscular use
        "90688",  # Influenza virus vaccine, quadrivalent (IIV4), split virus, 0.5 mL dosage, for intramuscular use
        "90689",  # Influenza virus vaccine, quadrivalent (IIV4), inactivated, adjuvanted, preservative free, 0.25 mL dosage, for intramuscular use
        "90756",  # Influenza virus vaccine, quadrivalent (ccIIV4), derived from cell cultures, subunit, antibiotic free, 0.5 mL dosage, for intramuscular use
    }

    SNOMEDCT = {
        "86198006",  # Administration of vaccine product containing only Influenza virus antigen (procedure)
    }

class DtapVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for diphtheria, tetanus, and whooping cough (pertussis) (DTaP) vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a diphtheria, tetanus, and whooping cough (pertussis) (DTaP) vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "DTaP Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1022"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90697",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, inactivated poliovirus vaccine, Haemophilus influenzae type b PRP-OMP conjugate vaccine, and hepatitis B vaccine (DTaP-IPV-Hib-HepB), for intramuscular use
        "90698",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, Haemophilus influenzae type b, and inactivated poliovirus vaccine, (DTaP-IPV/Hib), for intramuscular use
        "90700",  # Diphtheria, tetanus toxoids, and acellular pertussis vaccine (DTaP), when administered to individuals younger than 7 years, for intramuscular use
        "90723",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, hepatitis B, and inactivated poliovirus vaccine (DTaP-HepB-IPV), for intramuscular use
    }

    SNOMEDCT = {
        "1162640003",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus and inactivated Human poliovirus antigens (procedure)
        "16290681000119103",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and inactivated whole Human poliovirus antigens (procedure)
        "310306005",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310307001",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310308006",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "312870000",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "313383003",  # Administration of fourth dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "390846000",  # Administration of booster dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "390865008",  # Administration of booster dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "399014008",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "412755006",  # Administration of first dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "412756007",  # Administration of second dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "412757003",  # Administration of third dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "412762002",  # Administration of first dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "412763007",  # Administration of second dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "412764001",  # Administration of third dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "414001002",  # Administration of vaccine product containing only five component acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "414259000",  # Administration of first dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "414620004",  # Administration of vaccine product containing only acellular Bordetella pertussis five component and Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated whole Human poliovirus antigens (procedure)
        "415507003",  # Administration of second dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "415712004",  # Administration of third dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "428251000124104",  # Administration of vaccine product containing only low dose acellular Bordetella pertussis and Clostridium tetani and low dose Corynebacterium diphtheriae antigens (procedure)
        "571571000119105",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "572561000119108",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Hepatitis B virus and inactivated whole Human poliovirus antigens (procedure)
        "770608009",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770616000",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770617009",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770618004",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "787436003",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b antigens (procedure)
        "866158005",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "866159002",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "866226006",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae antigens (procedure)
        "868273007",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868274001",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868276004",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868277008",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
    }

class HepatitisAVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a Hepatitis A vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a Hepatitis A vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hepatitis A Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1041"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90633",  # Hepatitis A vaccine (HepA), pediatric/adolescent dosage-2 dose schedule, for intramuscular use
    }

    SNOMEDCT = {
        "170378007",  # Administration of first dose of pediatric vaccine product containing only Hepatitis A virus antigen (procedure)
        "170379004",  # Administration of second dose of vaccine product containing only Hepatitis A virus antigen (procedure)
        "170380001",  # Administration of third dose of vaccine product containing only Hepatitis A virus antigen (procedure)
        "170381002",  # Administration of booster dose of vaccine product containing only Hepatitis A virus antigen (procedure)
        "170434002",  # Administration of first dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "170435001",  # Administration of second dose of vaccine product containing only Hepatitis A and B virus antigens (procedure)
        "170436000",  # Administration of third dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "170437009",  # Administration of booster dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "243789007",  # Administration of vaccine product containing only Hepatitis A virus antigen (procedure)
        "312868009",  # Administration of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "314177003",  # Administration of vaccine product containing only Hepatitis A virus and Salmonella enterica subspecies enterica serovar Typhi antigens (procedure)
        "314178008",  # Administration of first dose of vaccine product containing only Hepatitis A virus and Salmonella enterica subspecies enterica serovar Typhi antigens (procedure)
        "314179000",  # Administration of second dose of vaccine product containing only Hepatitis A virus and Salmonella enterica subspecies enterica serovar Typhi antigens (procedure)
        "394691002",  # Administration of booster dose of vaccine product containing only Hepatitis A virus and Salmonella enterica subspecies enterica serovar Typhi antigens (procedure)
        "571511000119102",  # Administration of adult vaccine product containing only Hepatitis A virus antigen (procedure)
        "871752004",  # Administration of second dose of pediatric vaccine product containing only Hepatitis A virus antigen (procedure)
        "871753009",  # Administration of third dose of pediatric vaccine product containing only Hepatitis A virus antigen (procedure)
        "871754003",  # Administration of booster dose of pediatric vaccine product containing only Hepatitis A virus antigen (procedure)
    }

class HepatitisBVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a Hepatitis B vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for a Hepatitis B vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hepatitis B Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1042"
    DEFINITION_VERSION = "20220909"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90697",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, inactivated poliovirus vaccine, Haemophilus influenzae type b PRP-OMP conjugate vaccine, and hepatitis B vaccine (DTaP-IPV-Hib-HepB), for intramuscular use
        "90723",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, hepatitis B, and inactivated poliovirus vaccine (DTaP-HepB-IPV), for intramuscular use
        "90740",  # Hepatitis B vaccine (HepB), dialysis or immunosuppressed patient dosage, 3 dose schedule, for intramuscular use
        "90744",  # Hepatitis B vaccine (HepB), pediatric/adolescent dosage, 3 dose schedule, for intramuscular use
        "90747",  # Hepatitis B vaccine (HepB), dialysis or immunosuppressed patient dosage, 4 dose schedule, for intramuscular use
        "90748",  # Hepatitis B and Haemophilus influenzae type b vaccine (Hib-HepB), for intramuscular use
    }

    SNOMEDCT = {
        "1162640003",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus and inactivated Human poliovirus antigens (procedure)
        "16584000",  # Administration of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170370000",  # Administration of first dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170371001",  # Administration of second dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170372008",  # Administration of third dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170373003",  # Administration of booster dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170374009",  # Administration of fourth dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170375005",  # Administration of fifth dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "170434002",  # Administration of first dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "170435001",  # Administration of second dose of vaccine product containing only Hepatitis A and B virus antigens (procedure)
        "170436000",  # Administration of third dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "170437009",  # Administration of booster dose of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "312868009",  # Administration of vaccine product containing only Hepatitis A and Hepatitis B virus antigens (procedure)
        "396456003",  # Administration of vaccine product containing only acellular Bordetella pertussis and Corynebacterium diphtheriae and Hepatitis B virus and inactivated whole Human poliovirus antigens (procedure)
        "416923003",  # Administration of sixth dose of vaccine product containing only Hepatitis B virus antigen (procedure)
        "572561000119108",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Hepatitis B virus and inactivated whole Human poliovirus antigens (procedure)
        "770608009",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770616000",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770617009",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "770618004",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
        "786846001",  # Administration of vaccine product containing only Haemophilus influenzae type b and Hepatitis B virus antigens (procedure)
    }

class HibVaccine3DoseScheduleAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a 3-dose haemophilus influenzae type b (Hib) vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a 3-dose haemophilus influenzae type b (Hib) vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hib Vaccine (3 dose schedule) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1084"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90647",  # Haemophilus influenzae type b vaccine (Hib), PRP-OMP conjugate, 3 dose schedule, for intramuscular use
        "90697",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, inactivated poliovirus vaccine, Haemophilus influenzae type b PRP-OMP conjugate vaccine, and hepatitis B vaccine (DTaP-IPV-Hib-HepB), for intramuscular use
        "90748",  # Hepatitis B and Haemophilus influenzae type b vaccine (Hib-HepB), for intramuscular use
    }

    SNOMEDCT = {
        "127787002",  # Administration of vaccine product containing only Haemophilus influenzae type b antigen (procedure)
        "170343007",  # Administration of first dose of vaccine product containing only Haemophilus influenzae type b antigen (procedure)
        "170344001",  # Administration of second dose of vaccine product containing only Haemophilus influenzae type b antigen (procedure)
        "170345000",  # Administration of third dose of vaccine product containing only Haemophilus influenzae type b antigen (procedure)
        "170346004",  # Administration of booster dose of vaccine product containing only Haemophilus influenzae type b antigen (procedure)
    }

class HibVaccine4DoseScheduleAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a 4-dose haemophilus influenzae type b (Hib) vaccines.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for a 4-dose haemophilus influenzae type b (Hib) vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Hib Vaccine (4 dose schedule) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1086"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90644",  # Meningococcal conjugate vaccine, serogroups C & Y and Haemophilus influenzae type b vaccine (Hib-MenCY), 4 dose schedule, when administered to children 6 weeks-18 months of age, for intramuscular use
        "90648",  # Haemophilus influenzae type b vaccine (Hib), PRP-T conjugate, 4 dose schedule, for intramuscular use
        "90698",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, Haemophilus influenzae type b, and inactivated poliovirus vaccine, (DTaP-IPV/Hib), for intramuscular use
    }

    SNOMEDCT = {
        "310306005",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310307001",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310308006",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "312870000",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "414001002",  # Administration of vaccine product containing only five component acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "414259000",  # Administration of first dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "415507003",  # Administration of second dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "415712004",  # Administration of third dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
    }

class InactivatedPolioVaccineIpvAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for an inactivated polio vaccination (IPV).

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent an inactivated polio vaccination (IPV).

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Inactivated Polio Vaccine (IPV) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1045"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90697",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, inactivated poliovirus vaccine, Haemophilus influenzae type b PRP-OMP conjugate vaccine, and hepatitis B vaccine (DTaP-IPV-Hib-HepB), for intramuscular use
        "90698",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, Haemophilus influenzae type b, and inactivated poliovirus vaccine, (DTaP-IPV/Hib), for intramuscular use
        "90713",  # Poliovirus vaccine, inactivated (IPV), for subcutaneous or intramuscular use
        "90723",  # Diphtheria, tetanus toxoids, acellular pertussis vaccine, hepatitis B, and inactivated poliovirus vaccine (DTaP-HepB-IPV), for intramuscular use
    }

    SNOMEDCT = {
        "16290681000119103",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and inactivated whole Human poliovirus antigens (procedure)
        "310306005",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310307001",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "310308006",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "312869001",  # Administration of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "312870000",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "313383003",  # Administration of fourth dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and Human poliovirus antigens (procedure)
        "390865008",  # Administration of booster dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "396456003",  # Administration of vaccine product containing only acellular Bordetella pertussis and Corynebacterium diphtheriae and Hepatitis B virus and inactivated whole Human poliovirus antigens (procedure)
        "412762002",  # Administration of first dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "412763007",  # Administration of second dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "412764001",  # Administration of third dose of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "414001002",  # Administration of vaccine product containing only five component acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "414259000",  # Administration of first dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "414619005",  # Administration of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "414620004",  # Administration of vaccine product containing only acellular Bordetella pertussis five component and Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated whole Human poliovirus antigens (procedure)
        "415507003",  # Administration of second dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "415712004",  # Administration of third dose of vaccine product containing only five component acellular Bordetella pertussis, Clostridium tetani, Corynebacterium diphtheriae, Haemophilus influenzae type b and inactivated whole Human poliovirus antigens (procedure)
        "416144004",  # Administration of third dose of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "416591003",  # Administration of first dose of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "417211006",  # Administration of first booster of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "417384007",  # Administration of second booster of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "417615007",  # Administration of second dose of vaccine product containing only Clostridium tetani and low dose Corynebacterium diphtheriae and inactivated Human poliovirus antigens (procedure)
        "572561000119108",  # Administration of vaccine product containing only acellular Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Hepatitis B virus and inactivated whole Human poliovirus antigens (procedure)
        "866186002",  # Administration of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "866227002",  # Administration of booster dose of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868266002",  # Administration of second dose of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868267006",  # Administration of first dose of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868268001",  # Administration of third dose of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868273007",  # Administration of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868274001",  # Administration of second dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868276004",  # Administration of third dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "868277008",  # Administration of first dose of vaccine product containing only Bordetella pertussis and Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
        "870670004",  # Preschool administration of vaccine product containing only Clostridium tetani and Corynebacterium diphtheriae and Human poliovirus antigens (procedure)
    }

class InfluenzaVirusLaivVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent administration of live attenuated influenza vaccine.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures for live attenuated influenza vaccination (quadrivalent and trivalent).

    **Exclusion Criteria:** Excludes concepts that represent procedures for recombinant or inactivated influenza vaccination.
    """

    VALUE_SET_NAME = "Influenza Virus LAIV Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1088"
    DEFINITION_VERSION = "20210224"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90660",  # Influenza virus vaccine, trivalent, live (LAIV3), for intranasal use
        "90672",  # Influenza virus vaccine, quadrivalent, live (LAIV4), for intranasal use
    }

class MeaslesMumpsAndRubellaMmrVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for measles, mumps and rubella (MMR) vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent measles, mumps and rubella (MMR) vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Measles, Mumps and Rubella (MMR) Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1031"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90707",  # Measles, mumps and rubella virus vaccine (MMR), live, for subcutaneous use
        "90710",  # Measles, mumps, rubella, and varicella vaccine (MMRV), live, for subcutaneous use
    }

    SNOMEDCT = {
        "170431005",  # Administration of booster dose of vaccine product containing only Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "170432003",  # Administration of pre-school booster dose of vaccine product containing only Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "170433008",  # Administration of second dose of vaccine product containing only Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "38598009",  # Administration of vaccine product containing only Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "432636005",  # Administration of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "433733003",  # Administration of second dose of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "571591000119106",  # Administration of vaccine product containing only live attenuated Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "572511000119105",  # Administration of vaccine product containing only live attenuated Measles morbillivirus and Mumps orthorubulavirus and Rubella virus and Human alphaherpesvirus 3 antigens (procedure)
        "871909005",  # Administration of first dose of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
    }

class PneumococcalConjugateVaccineAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a pneumococcal conjugate vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a pneumococcal conjugate vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Pneumococcal Conjugate Vaccine Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1046"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90670",  # Pneumococcal conjugate vaccine, 13 valent (PCV13), for intramuscular use
        "90671",  # Pneumococcal conjugate vaccine, 15 valent (PCV15), for intramuscular use
        "90677",  # Pneumococcal conjugate vaccine, 20 valent (PCV20), for intramuscular use
    }

    SNOMEDCT = {
        "1119366009",  # Administration of vaccine product containing only Streptococcus pneumoniae Danish serotype 1, 3, 4, 5, 6A, 6B, 7F, 9V, 14, 18C, 19A, 19F, and 23F capsular polysaccharide antigens (procedure)
        "1119368005",  # Administration of vaccine product containing only Streptococcus pneumoniae Danish serotype 4, 6B, 9V, 14, 18C, 19F, and 23F capsular polysaccharide antigens conjugated (procedure)
        "12866006",  # Administration of vaccine product containing only Streptococcus pneumoniae antigen (procedure)
        "1296904008",  # Administration of vaccine product containing only Streptococcus pneumoniae Danish serotype 1, 3, 4, 5, 6A, 6B, 7F, 9V, 14, 18C, 19A, 19F, and 23F capsular polysaccharide antigens conjugated (procedure)
        "394678003",  # Administration of booster dose of vaccine product containing only Streptococcus pneumoniae antigen (procedure)
        "434751000124102",  # Pneumococcal conjugate vaccination (procedure)
        "871833000",  # Subcutaneous injection of pneumococcal vaccine (procedure)
    }

class RotavirusVaccine2DoseScheduleAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a 2-dose rotavirus vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent the administration of a 2-dose rotavirus vaccine.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Rotavirus Vaccine (2 dose schedule) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1048"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90681",  # Rotavirus vaccine, human, attenuated (RV1), 2 dose schedule, live, for oral use
    }

    SNOMEDCT = {
        "434741000124104",  # Rotavirus vaccination, 2 dose schedule (procedure)
    }

class RotavirusVaccine3DoseScheduleAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a 3-dose rotavirus vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent the administration of a 3-dose rotavirus vaccine.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Rotavirus Vaccine (3 dose schedule) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1047"
    DEFINITION_VERSION = "20190316"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90680",  # Rotavirus vaccine, pentavalent (RV5), 3 dose schedule, live, for oral use
    }

    SNOMEDCT = {
        "434731000124109",  # Rotavirus vaccination, 3 dose schedule (procedure)
    }

class VaricellaZosterVaccineVzvAdministered(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a varicella zoster vaccination.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a varicella zoster vaccination.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Varicella Zoster Vaccine (VZV) Administered"
    OID = "2.16.840.1.113883.3.464.1003.110.12.1040"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90710",  # Measles, mumps, rubella, and varicella vaccine (MMRV), live, for subcutaneous use
        "90716",  # Varicella virus vaccine (VAR), live, for subcutaneous use
    }

    SNOMEDCT = {
        "425897001",  # Administration of first dose of vaccine product containing only Human alphaherpesvirus 3 antigen for chickenpox (procedure)
        "428502009",  # Administration of second dose of vaccine product containing only Human alphaherpesvirus 3 antigen for chickenpox (procedure)
        "432636005",  # Administration of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "433733003",  # Administration of second dose of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
        "572511000119105",  # Administration of vaccine product containing only live attenuated Measles morbillivirus and Mumps orthorubulavirus and Rubella virus and Human alphaherpesvirus 3 antigens (procedure)
        "737081007",  # Administration of vaccine product containing only Human alphaherpesvirus 3 antigen for chickenpox (procedure)
        "871898007",  # Administration of vaccine product containing only live attenuated Human alphaherpesvirus 3 antigen (procedure)
        "871899004",  # Administration of vaccine product containing only live attenuated Human alphaherpesvirus 3 antigen via subcutaneous route (procedure)
        "871909005",  # Administration of first dose of vaccine product containing only Human alphaherpesvirus 3 and Measles morbillivirus and Mumps orthorubulavirus and Rubella virus antigens (procedure)
    }

class AnesthesiaRequiringMonitoredCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for anesthesia requiring monitored anesthesia care (MAC)

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts for general, neuraxial, regional anesthesia or any other anesthesia that requires MAC (monitored anesthesia care) such as conscious sedation

    **Exclusion Criteria:** Excludes concepts for local nerve blocks
    """

    VALUE_SET_NAME = "Anesthesia Requiring Monitored Care"
    OID = "2.16.840.1.113762.1.4.1248.255"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "3E030FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Open Approach
        "3E033FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Percutaneous Approach
        "3E040FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Open Approach
        "3E040NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Central Vein, Open Approach
        "3E043FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Percutaneous Approach
        "3E043NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Central Vein, Percutaneous Approach
        "3E050FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Open Approach
        "3E053FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Percutaneous Approach
        "3E060FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Open Approach
        "3E060NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Central Artery, Open Approach
        "3E063FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Percutaneous Approach
        "3E063NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Central Artery, Percutaneous Approach
        "3E0R3BZ",  # Introduction of Anesthetic Agent into Spinal Canal, Percutaneous Approach
        "3E0R3NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Spinal Canal, Percutaneous Approach
        "3E0S3BZ",  # Introduction of Anesthetic Agent into Epidural Space, Percutaneous Approach
        "3E0S3NZ",  # Introduction of Analgesics, Hypnotics, Sedatives into Epidural Space, Percutaneous Approach
    }

    SNOMEDCT = {
        "112943005",  # Epidural injection of anesthetic substance, diagnostic, caudal, continuous (procedure)
        "112987001",  # Inhalation anesthesia, machine system, closed, rebreathing of primary agent (procedure)
        "180878007",  # Inhalation anesthetic using muscle relaxant (procedure)
        "180885006",  # Local anesthetic lumbar epidural block (procedure)
        "180886007",  # Local anesthetic sacral epidural block (procedure)
        "18946005",  # Epidural anesthesia (procedure)
        "231246003",  # Intravenous regional block (procedure)
        "231247007",  # Local anesthetic intravenous regional block of arm (procedure)
        "231248002",  # Local anesthetic intravenous regional block of leg (procedure)
        "231255000",  # Spinal subdural local anesthetic block (procedure)
        "231256004",  # Cervical subdural block (procedure)
        "231257008",  # Thoracic subdural block (procedure)
        "231258003",  # Lumbar subdural block (procedure)
        "231261002",  # Combined spinal/epidural local anesthetic block (procedure)
        "231262009",  # Combined spinal/epidural technique with needles at separate spaces (procedure)
        "231263004",  # Combined spinal/epidural technique - spinal needle through Tuohy needle (procedure)
        "231270004",  # Intravenous regional autonomic block (procedure)
        "241688000",  # Intravenous induction of general anesthesia (procedure)
        "241696005",  # Total intravenous anesthesia (procedure)
        "241717009",  # Inhalational analgesia (procedure)
        "24277005",  # Inhalation anesthesia, machine system, closed, circulation of primary agent (procedure)
        "266802007",  # Endotracheal anesthesia (procedure)
        "27372005",  # Regional anesthesia (procedure)
        "274507007",  # Operative general anesthesia (procedure)
        "27653002",  # Inhalation anesthesia, machine system, semi-closed, rebreathing of primary agent (procedure)
        "278868002",  # Intravenous regional sympathetic block (procedure)
        "288185006",  # Inhalation general anesthesia (procedure)
        "288186007",  # General anesthesia and muscle relaxant (procedure)
        "30050007",  # Inhalation anesthesia, machine system, semi-closed, no rebreathing of primary agent (procedure)
        "302335002",  # Local anesthetic thoracic epidural block (procedure)
        "302336001",  # Local anesthetic cervical epidural block (procedure)
        "303355006",  # Local anesthetic intravenous regional block (procedure)
        "303358008",  # Neurolytic nerve block around spinal cord meninges (procedure)
        "314271007",  # Induction of conscious sedation (procedure)
        "35597003",  # Epidural injection of anesthetic substance, diagnostic, lumbar, continuous (procedure)
        "398044000",  # Low dose epidural (procedure)
        "398239001",  # Monitored anesthesia care sedation (procedure)
        "405611007",  # Peripheral nerve plexus block (procedure)
        "405650006",  # Neuraxial nerve block (procedure)
        "426309005",  # Epidural injection of anesthetic substance, therapeutic, continuous, cervical (procedure)
        "427060008",  # Epidural injection of anesthetic substance, therapeutic, thoracic, continuous (procedure)
        "442842005",  # Oral administration of sedative for conscious sedation (procedure)
        "44812007",  # Inhalation anesthesia, machine system, closed, no rebreathing of primary agent (procedure)
        "448585004",  # Target controlled total intravenous anesthesia (procedure)
        "449126000",  # Administration of inhalation anesthetic using endotracheal tube (procedure)
        "50697003",  # Administration of general anesthetic (procedure)
        "58611004",  # Epidural injection of anesthetic substance, therapeutic, lumbar, continuous (procedure)
        "64287005",  # Inhalation anesthesia, machine system, semi-closed, circulation of primary agent and gases (procedure)
        "67716003",  # Epidural injection of anesthetic substance, therapeutic, caudal, continuous (procedure)
        "74527005",  # Injection of anesthetic substance, therapeutic, subdural, differential (procedure)
        "82776005",  # Injection of anesthetic substance, therapeutic, subdural, continuous (procedure)
        "88815003",  # Peripheral block anesthesia (procedure)
    }

class GeneralAndNeuraxialAnesthesia(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for general and/or neuraxial anesthesia to represent the occurrence of a procedure or surgery taking place

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent general and/or neuraxial anesthesia

    **Exclusion Criteria:** Excludes concepts that represent local, regional and axillary anesthesia
    """

    VALUE_SET_NAME = "General and Neuraxial Anesthesia"
    OID = "2.16.840.1.113762.1.4.1248.208"
    DEFINITION_VERSION = "20230429"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "3E030FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Open Approach
        "3E033FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Vein, Percutaneous Approach
        "3E040FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Open Approach
        "3E043FZ",  # Introduction of Intracirculatory Anesthetic into Central Vein, Percutaneous Approach
        "3E050FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Open Approach
        "3E053FZ",  # Introduction of Intracirculatory Anesthetic into Peripheral Artery, Percutaneous Approach
        "3E060FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Open Approach
        "3E063FZ",  # Introduction of Intracirculatory Anesthetic into Central Artery, Percutaneous Approach
    }

    SNOMEDCT = {
        "112943005",  # Epidural injection of anesthetic substance, diagnostic, caudal, continuous (procedure)
        "112987001",  # Inhalation anesthesia, machine system, closed, rebreathing of primary agent (procedure)
        "180878007",  # Inhalation anesthetic using muscle relaxant (procedure)
        "18946005",  # Epidural anesthesia (procedure)
        "231255000",  # Spinal subdural local anesthetic block (procedure)
        "231256004",  # Cervical subdural block (procedure)
        "231257008",  # Thoracic subdural block (procedure)
        "231258003",  # Lumbar subdural block (procedure)
        "231262009",  # Combined spinal/epidural technique with needles at separate spaces (procedure)
        "231263004",  # Combined spinal/epidural technique - spinal needle through Tuohy needle (procedure)
        "241688000",  # Intravenous induction of general anesthesia (procedure)
        "241696005",  # Total intravenous anesthesia (procedure)
        "241717009",  # Inhalational analgesia (procedure)
        "24277005",  # Inhalation anesthesia, machine system, closed, circulation of primary agent (procedure)
        "266802007",  # Endotracheal anesthesia (procedure)
        "274507007",  # Operative general anesthesia (procedure)
        "27653002",  # Inhalation anesthesia, machine system, semi-closed, rebreathing of primary agent (procedure)
        "288185006",  # Inhalation general anesthesia (procedure)
        "288186007",  # General anesthesia and muscle relaxant (procedure)
        "30050007",  # Inhalation anesthesia, machine system, semi-closed, no rebreathing of primary agent (procedure)
        "303358008",  # Neurolytic nerve block around spinal cord meninges (procedure)
        "35597003",  # Epidural injection of anesthetic substance, diagnostic, lumbar, continuous (procedure)
        "398044000",  # Low dose epidural (procedure)
        "405650006",  # Neuraxial nerve block (procedure)
        "426309005",  # Epidural injection of anesthetic substance, therapeutic, continuous, cervical (procedure)
        "427060008",  # Epidural injection of anesthetic substance, therapeutic, thoracic, continuous (procedure)
        "44812007",  # Inhalation anesthesia, machine system, closed, no rebreathing of primary agent (procedure)
        "448585004",  # Target controlled total intravenous anesthesia (procedure)
        "449126000",  # Administration of inhalation anesthetic using endotracheal tube (procedure)
        "50697003",  # Administration of general anesthetic (procedure)
        "58611004",  # Epidural injection of anesthetic substance, therapeutic, lumbar, continuous (procedure)
        "64287005",  # Inhalation anesthesia, machine system, semi-closed, circulation of primary agent and gases (procedure)
        "67716003",  # Epidural injection of anesthetic substance, therapeutic, caudal, continuous (procedure)
        "74527005",  # Injection of anesthetic substance, therapeutic, subdural, differential (procedure)
        "82776005",  # Injection of anesthetic substance, therapeutic, subdural, continuous (procedure)
    }

class HeadNeckAndThoracicSurgeriesWithHighRiskAirwayCompromise(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for selected head, neck, and thoracic surgical procedures involving significant risk of airway compromise or requiring airway protection

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent concepts for selected head, neck, and thoracic surgical procedures involving significant risk of airway compromise or requiring airway protection

    **Exclusion Criteria:** N/A
    """

    VALUE_SET_NAME = "Head, Neck, and Thoracic Surgeries with High Risk Airway Compromise"
    OID = "2.16.840.1.113762.1.4.1248.251"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "008P0ZZ",  # Division of Glossopharyngeal Nerve, Open Approach
        "008P3ZZ",  # Division of Glossopharyngeal Nerve, Percutaneous Approach
        "008P4ZZ",  # Division of Glossopharyngeal Nerve, Percutaneous Endoscopic Approach
        "008Q0ZZ",  # Division of Vagus Nerve, Open Approach
        "008Q3ZZ",  # Division of Vagus Nerve, Percutaneous Approach
        "008Q4ZZ",  # Division of Vagus Nerve, Percutaneous Endoscopic Approach
        "02RA0LZ",  # Replacement of Heart with Biologic and Synthetic Substitute, Autoregulated Electrohydraulic, Open Approach
        "02RA0MZ",  # Replacement of Heart with Synthetic Substitute, Pneumatic, Open Approach
        "02YA0Z0",  # Transplantation of Heart, Allogeneic, Open Approach
        "02YA0Z1",  # Transplantation of Heart, Syngeneic, Open Approach
        "02YA0Z2",  # Transplantation of Heart, Zooplastic, Open Approach
        "07T10ZZ",  # Resection of Right Neck Lymphatic, Open Approach
        "07T14ZZ",  # Resection of Right Neck Lymphatic, Percutaneous Endoscopic Approach
        "07T20ZZ",  # Resection of Left Neck Lymphatic, Open Approach
        "07T24ZZ",  # Resection of Left Neck Lymphatic, Percutaneous Endoscopic Approach
        "095N0ZZ",  # Destruction of Nasopharynx, Open Approach
        "095N3ZZ",  # Destruction of Nasopharynx, Percutaneous Approach
        "095N4ZZ",  # Destruction of Nasopharynx, Percutaneous Endoscopic Approach
        "095P0ZZ",  # Destruction of Accessory Sinus, Open Approach
        "095Q0ZZ",  # Destruction of Right Maxillary Sinus, Open Approach
        "095R0ZZ",  # Destruction of Left Maxillary Sinus, Open Approach
        "095S0ZZ",  # Destruction of Right Frontal Sinus, Open Approach
        "095T0ZZ",  # Destruction of Left Frontal Sinus, Open Approach
        "095U0ZZ",  # Destruction of Right Ethmoid Sinus, Open Approach
        "095V0ZZ",  # Destruction of Left Ethmoid Sinus, Open Approach
        "095W0ZZ",  # Destruction of Right Sphenoid Sinus, Open Approach
        "095X0ZZ",  # Destruction of Left Sphenoid Sinus, Open Approach
        "099N00Z",  # Drainage of Nasopharynx with Drainage Device, Open Approach
        "099N0ZZ",  # Drainage of Nasopharynx, Open Approach
        "099N30Z",  # Drainage of Nasopharynx with Drainage Device, Percutaneous Approach
        "099N40Z",  # Drainage of Nasopharynx with Drainage Device, Percutaneous Endoscopic Approach
        "099P00Z",  # Drainage of Accessory Sinus with Drainage Device, Open Approach
        "099P0ZZ",  # Drainage of Accessory Sinus, Open Approach
        "099Q00Z",  # Drainage of Right Maxillary Sinus with Drainage Device, Open Approach
        "099Q0ZZ",  # Drainage of Right Maxillary Sinus, Open Approach
        "099R00Z",  # Drainage of Left Maxillary Sinus with Drainage Device, Open Approach
        "099R0ZZ",  # Drainage of Left Maxillary Sinus, Open Approach
        "099S00Z",  # Drainage of Right Frontal Sinus with Drainage Device, Open Approach
        "099S0ZZ",  # Drainage of Right Frontal Sinus, Open Approach
        "099T00Z",  # Drainage of Left Frontal Sinus with Drainage Device, Open Approach
        "099T0ZZ",  # Drainage of Left Frontal Sinus, Open Approach
        "099U00Z",  # Drainage of Right Ethmoid Sinus with Drainage Device, Open Approach
        "099U0ZZ",  # Drainage of Right Ethmoid Sinus, Open Approach
        "099V00Z",  # Drainage of Left Ethmoid Sinus with Drainage Device, Open Approach
        "099V0ZZ",  # Drainage of Left Ethmoid Sinus, Open Approach
        "099W00Z",  # Drainage of Right Sphenoid Sinus with Drainage Device, Open Approach
        "099W0ZZ",  # Drainage of Right Sphenoid Sinus, Open Approach
        "099X00Z",  # Drainage of Left Sphenoid Sinus with Drainage Device, Open Approach
        "099X0ZZ",  # Drainage of Left Sphenoid Sinus, Open Approach
        "09BN0ZZ",  # Excision of Nasopharynx, Open Approach
        "09BP0ZZ",  # Excision of Accessory Sinus, Open Approach
        "09BQ0ZZ",  # Excision of Right Maxillary Sinus, Open Approach
        "09BR0ZZ",  # Excision of Left Maxillary Sinus, Open Approach
        "09BS0ZZ",  # Excision of Right Frontal Sinus, Open Approach
        "09BT0ZZ",  # Excision of Left Frontal Sinus, Open Approach
        "09BU0ZZ",  # Excision of Right Ethmoid Sinus, Open Approach
        "09BV0ZZ",  # Excision of Left Ethmoid Sinus, Open Approach
        "09BW0ZZ",  # Excision of Right Sphenoid Sinus, Open Approach
        "09BX0ZZ",  # Excision of Left Sphenoid Sinus, Open Approach
        "09CN0ZZ",  # Extirpation of Matter from Nasopharynx, Open Approach
        "09CN3ZZ",  # Extirpation of Matter from Nasopharynx, Percutaneous Approach
        "09CN7ZZ",  # Extirpation of Matter from Nasopharynx, Via Natural or Artificial Opening
        "09CN8ZZ",  # Extirpation of Matter from Nasopharynx, Via Natural or Artificial Opening Endoscopic
        "09CP0ZZ",  # Extirpation of Matter from Accessory Sinus, Open Approach
        "09CQ0ZZ",  # Extirpation of Matter from Right Maxillary Sinus, Open Approach
        "09CR0ZZ",  # Extirpation of Matter from Left Maxillary Sinus, Open Approach
        "09CS0ZZ",  # Extirpation of Matter from Right Frontal Sinus, Open Approach
        "09CT0ZZ",  # Extirpation of Matter from Left Frontal Sinus, Open Approach
        "09CU0ZZ",  # Extirpation of Matter from Right Ethmoid Sinus, Open Approach
        "09CV0ZZ",  # Extirpation of Matter from Left Ethmoid Sinus, Open Approach
        "09CW0ZZ",  # Extirpation of Matter from Right Sphenoid Sinus, Open Approach
        "09CX0ZZ",  # Extirpation of Matter from Left Sphenoid Sinus, Open Approach
        "09DP0ZZ",  # Extraction of Accessory Sinus, Open Approach
        "09DQ0ZZ",  # Extraction of Right Maxillary Sinus, Open Approach
        "09DR0ZZ",  # Extraction of Left Maxillary Sinus, Open Approach
        "09DS0ZZ",  # Extraction of Right Frontal Sinus, Open Approach
        "09DT0ZZ",  # Extraction of Left Frontal Sinus, Open Approach
        "09DU0ZZ",  # Extraction of Right Ethmoid Sinus, Open Approach
        "09DV0ZZ",  # Extraction of Left Ethmoid Sinus, Open Approach
        "09DW0ZZ",  # Extraction of Right Sphenoid Sinus, Open Approach
        "09DX0ZZ",  # Extraction of Left Sphenoid Sinus, Open Approach
        "09NN0ZZ",  # Release Nasopharynx, Open Approach
        "09NN3ZZ",  # Release Nasopharynx, Percutaneous Approach
        "09NN4ZZ",  # Release Nasopharynx, Percutaneous Endoscopic Approach
        "09NN7ZZ",  # Release Nasopharynx, Via Natural or Artificial Opening
        "09NN8ZZ",  # Release Nasopharynx, Via Natural or Artificial Opening Endoscopic
        "09NP0ZZ",  # Release Accessory Sinus, Open Approach
        "09NQ0ZZ",  # Release Right Maxillary Sinus, Open Approach
        "09NR0ZZ",  # Release Left Maxillary Sinus, Open Approach
        "09NS0ZZ",  # Release Right Frontal Sinus, Open Approach
        "09NT0ZZ",  # Release Left Frontal Sinus, Open Approach
        "09NU0ZZ",  # Release Right Ethmoid Sinus, Open Approach
        "09NV0ZZ",  # Release Left Ethmoid Sinus, Open Approach
        "09NW0ZZ",  # Release Right Sphenoid Sinus, Open Approach
        "09NX0ZZ",  # Release Left Sphenoid Sinus, Open Approach
        "09QN0ZZ",  # Repair Nasopharynx, Open Approach
        "09QN3ZZ",  # Repair Nasopharynx, Percutaneous Approach
        "09QN7ZZ",  # Repair Nasopharynx, Via Natural or Artificial Opening
        "09QN8ZZ",  # Repair Nasopharynx, Via Natural or Artificial Opening Endoscopic
        "09QP0ZZ",  # Repair Accessory Sinus, Open Approach
        "09QQ0ZZ",  # Repair Right Maxillary Sinus, Open Approach
        "09QR0ZZ",  # Repair Left Maxillary Sinus, Open Approach
        "09QS0ZZ",  # Repair Right Frontal Sinus, Open Approach
        "09QT0ZZ",  # Repair Left Frontal Sinus, Open Approach
        "09QU0ZZ",  # Repair Right Ethmoid Sinus, Open Approach
        "09QV0ZZ",  # Repair Left Ethmoid Sinus, Open Approach
        "09QW0ZZ",  # Repair Right Sphenoid Sinus, Open Approach
        "09QX0ZZ",  # Repair Left Sphenoid Sinus, Open Approach
        "09RN07Z",  # Replacement of Nasopharynx with Autologous Tissue Substitute, Open Approach
        "09RN0JZ",  # Replacement of Nasopharynx with Synthetic Substitute, Open Approach
        "09RN0KZ",  # Replacement of Nasopharynx with Nonautologous Tissue Substitute, Open Approach
        "09RN77Z",  # Replacement of Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "09RN7JZ",  # Replacement of Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening
        "09RN7KZ",  # Replacement of Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "09RN87Z",  # Replacement of Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "09RN8JZ",  # Replacement of Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "09RN8KZ",  # Replacement of Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "09TN0ZZ",  # Resection of Nasopharynx, Open Approach
        "09TN4ZZ",  # Resection of Nasopharynx, Percutaneous Endoscopic Approach
        "09TN7ZZ",  # Resection of Nasopharynx, Via Natural or Artificial Opening
        "09TN8ZZ",  # Resection of Nasopharynx, Via Natural or Artificial Opening Endoscopic
        "09TP0ZZ",  # Resection of Accessory Sinus, Open Approach
        "09TQ0ZZ",  # Resection of Right Maxillary Sinus, Open Approach
        "09TR0ZZ",  # Resection of Left Maxillary Sinus, Open Approach
        "09TS0ZZ",  # Resection of Right Frontal Sinus, Open Approach
        "09TT0ZZ",  # Resection of Left Frontal Sinus, Open Approach
        "09TU0ZZ",  # Resection of Right Ethmoid Sinus, Open Approach
        "09TV0ZZ",  # Resection of Left Ethmoid Sinus, Open Approach
        "09TW0ZZ",  # Resection of Right Sphenoid Sinus, Open Approach
        "09TX0ZZ",  # Resection of Left Sphenoid Sinus, Open Approach
        "09UN07Z",  # Supplement Nasopharynx with Autologous Tissue Substitute, Open Approach
        "09UN0JZ",  # Supplement Nasopharynx with Synthetic Substitute, Open Approach
        "09UN0KZ",  # Supplement Nasopharynx with Nonautologous Tissue Substitute, Open Approach
        "09UN77Z",  # Supplement Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "09UN7JZ",  # Supplement Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening
        "09UN7KZ",  # Supplement Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "09UN87Z",  # Supplement Nasopharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "09UN8JZ",  # Supplement Nasopharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "09UN8KZ",  # Supplement Nasopharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "09UP07Z",  # Supplement Accessory Sinus with Autologous Tissue Substitute, Open Approach
        "09UP0JZ",  # Supplement Accessory Sinus with Synthetic Substitute, Open Approach
        "09UP0KZ",  # Supplement Accessory Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UQ07Z",  # Supplement Right Maxillary Sinus with Autologous Tissue Substitute, Open Approach
        "09UQ0JZ",  # Supplement Right Maxillary Sinus with Synthetic Substitute, Open Approach
        "09UQ0KZ",  # Supplement Right Maxillary Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UR07Z",  # Supplement Left Maxillary Sinus with Autologous Tissue Substitute, Open Approach
        "09UR0JZ",  # Supplement Left Maxillary Sinus with Synthetic Substitute, Open Approach
        "09UR0KZ",  # Supplement Left Maxillary Sinus with Nonautologous Tissue Substitute, Open Approach
        "09US07Z",  # Supplement Right Frontal Sinus with Autologous Tissue Substitute, Open Approach
        "09US0JZ",  # Supplement Right Frontal Sinus with Synthetic Substitute, Open Approach
        "09US0KZ",  # Supplement Right Frontal Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UT07Z",  # Supplement Left Frontal Sinus with Autologous Tissue Substitute, Open Approach
        "09UT0JZ",  # Supplement Left Frontal Sinus with Synthetic Substitute, Open Approach
        "09UT0KZ",  # Supplement Left Frontal Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UU07Z",  # Supplement Right Ethmoid Sinus with Autologous Tissue Substitute, Open Approach
        "09UU0JZ",  # Supplement Right Ethmoid Sinus with Synthetic Substitute, Open Approach
        "09UU0KZ",  # Supplement Right Ethmoid Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UV07Z",  # Supplement Left Ethmoid Sinus with Autologous Tissue Substitute, Open Approach
        "09UV0JZ",  # Supplement Left Ethmoid Sinus with Synthetic Substitute, Open Approach
        "09UV0KZ",  # Supplement Left Ethmoid Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UW07Z",  # Supplement Right Sphenoid Sinus with Autologous Tissue Substitute, Open Approach
        "09UW0JZ",  # Supplement Right Sphenoid Sinus with Synthetic Substitute, Open Approach
        "09UW0KZ",  # Supplement Right Sphenoid Sinus with Nonautologous Tissue Substitute, Open Approach
        "09UX07Z",  # Supplement Left Sphenoid Sinus with Autologous Tissue Substitute, Open Approach
        "09UX0JZ",  # Supplement Left Sphenoid Sinus with Synthetic Substitute, Open Approach
        "09UX0KZ",  # Supplement Left Sphenoid Sinus with Nonautologous Tissue Substitute, Open Approach
        "0B110D6",  # Bypass Trachea to Esophagus with Intraluminal Device, Open Approach
        "0B510ZZ",  # Destruction of Trachea, Open Approach
        "0B513ZZ",  # Destruction of Trachea, Percutaneous Approach
        "0B514ZZ",  # Destruction of Trachea, Percutaneous Endoscopic Approach
        "0B517ZZ",  # Destruction of Trachea, Via Natural or Artificial Opening
        "0B518ZZ",  # Destruction of Trachea, Via Natural or Artificial Opening Endoscopic
        "0B520ZZ",  # Destruction of Carina, Open Approach
        "0B523ZZ",  # Destruction of Carina, Percutaneous Approach
        "0B524ZZ",  # Destruction of Carina, Percutaneous Endoscopic Approach
        "0B527ZZ",  # Destruction of Carina, Via Natural or Artificial Opening
        "0B528ZZ",  # Destruction of Carina, Via Natural or Artificial Opening Endoscopic
        "0B710DZ",  # Dilation of Trachea with Intraluminal Device, Open Approach
        "0B710ZZ",  # Dilation of Trachea, Open Approach
        "0B713DZ",  # Dilation of Trachea with Intraluminal Device, Percutaneous Approach
        "0B713ZZ",  # Dilation of Trachea, Percutaneous Approach
        "0B714DZ",  # Dilation of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach
        "0B714ZZ",  # Dilation of Trachea, Percutaneous Endoscopic Approach
        "0B717DZ",  # Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening
        "0B717ZZ",  # Dilation of Trachea, Via Natural or Artificial Opening
        "0B718DZ",  # Dilation of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0B718ZZ",  # Dilation of Trachea, Via Natural or Artificial Opening Endoscopic
        "0B720DZ",  # Dilation of Carina with Intraluminal Device, Open Approach
        "0B720ZZ",  # Dilation of Carina, Open Approach
        "0B723DZ",  # Dilation of Carina with Intraluminal Device, Percutaneous Approach
        "0B723ZZ",  # Dilation of Carina, Percutaneous Approach
        "0B724DZ",  # Dilation of Carina with Intraluminal Device, Percutaneous Endoscopic Approach
        "0B724ZZ",  # Dilation of Carina, Percutaneous Endoscopic Approach
        "0B727DZ",  # Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening
        "0B727ZZ",  # Dilation of Carina, Via Natural or Artificial Opening
        "0B728DZ",  # Dilation of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0B728ZZ",  # Dilation of Carina, Via Natural or Artificial Opening Endoscopic
        "0B9100Z",  # Drainage of Trachea with Drainage Device, Open Approach
        "0B910ZZ",  # Drainage of Trachea, Open Approach
        "0B9130Z",  # Drainage of Trachea with Drainage Device, Percutaneous Approach
        "0B913ZZ",  # Drainage of Trachea, Percutaneous Approach
        "0B9140Z",  # Drainage of Trachea with Drainage Device, Percutaneous Endoscopic Approach
        "0B914ZZ",  # Drainage of Trachea, Percutaneous Endoscopic Approach
        "0B9200Z",  # Drainage of Carina with Drainage Device, Open Approach
        "0B920ZZ",  # Drainage of Carina, Open Approach
        "0B9230Z",  # Drainage of Carina with Drainage Device, Percutaneous Approach
        "0B923ZZ",  # Drainage of Carina, Percutaneous Approach
        "0B9240Z",  # Drainage of Carina with Drainage Device, Percutaneous Endoscopic Approach
        "0B924ZZ",  # Drainage of Carina, Percutaneous Endoscopic Approach
        "0BB10ZZ",  # Excision of Trachea, Open Approach
        "0BB13ZZ",  # Excision of Trachea, Percutaneous Approach
        "0BB14ZZ",  # Excision of Trachea, Percutaneous Endoscopic Approach
        "0BB17ZZ",  # Excision of Trachea, Via Natural or Artificial Opening
        "0BB18ZZ",  # Excision of Trachea, Via Natural or Artificial Opening Endoscopic
        "0BB20ZZ",  # Excision of Carina, Open Approach
        "0BB23ZZ",  # Excision of Carina, Percutaneous Approach
        "0BB24ZZ",  # Excision of Carina, Percutaneous Endoscopic Approach
        "0BB27ZZ",  # Excision of Carina, Via Natural or Artificial Opening
        "0BB28ZZ",  # Excision of Carina, Via Natural or Artificial Opening Endoscopic
        "0BB30ZZ",  # Excision of Right Main Bronchus, Open Approach
        "0BB40ZZ",  # Excision of Right Upper Lobe Bronchus, Open Approach
        "0BB50ZZ",  # Excision of Right Middle Lobe Bronchus, Open Approach
        "0BB60ZZ",  # Excision of Right Lower Lobe Bronchus, Open Approach
        "0BB70ZZ",  # Excision of Left Main Bronchus, Open Approach
        "0BB80ZZ",  # Excision of Left Upper Lobe Bronchus, Open Approach
        "0BB90ZZ",  # Excision of Lingula Bronchus, Open Approach
        "0BBB0ZZ",  # Excision of Left Lower Lobe Bronchus, Open Approach
        "0BBC0ZZ",  # Excision of Right Upper Lung Lobe, Open Approach
        "0BBC4ZZ",  # Excision of Right Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BBD0ZZ",  # Excision of Right Middle Lung Lobe, Open Approach
        "0BBD4ZZ",  # Excision of Right Middle Lung Lobe, Percutaneous Endoscopic Approach
        "0BBF0ZZ",  # Excision of Right Lower Lung Lobe, Open Approach
        "0BBF4ZZ",  # Excision of Right Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BBG0ZZ",  # Excision of Left Upper Lung Lobe, Open Approach
        "0BBG4ZZ",  # Excision of Left Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BBH0ZZ",  # Excision of Lung Lingula, Open Approach
        "0BBH4ZZ",  # Excision of Lung Lingula, Percutaneous Endoscopic Approach
        "0BBJ0ZZ",  # Excision of Left Lower Lung Lobe, Open Approach
        "0BBJ4ZZ",  # Excision of Left Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BBK0ZZ",  # Excision of Right Lung, Open Approach
        "0BBK4ZZ",  # Excision of Right Lung, Percutaneous Endoscopic Approach
        "0BBL0ZZ",  # Excision of Left Lung, Open Approach
        "0BBL4ZZ",  # Excision of Left Lung, Percutaneous Endoscopic Approach
        "0BBM0ZZ",  # Excision of Bilateral Lungs, Open Approach
        "0BBM4ZZ",  # Excision of Bilateral Lungs, Percutaneous Endoscopic Approach
        "0BBT0ZZ",  # Excision of Diaphragm, Open Approach
        "0BBT4ZZ",  # Excision of Diaphragm, Percutaneous Endoscopic Approach
        "0BC10ZZ",  # Extirpation of Matter from Trachea, Open Approach
        "0BC13ZZ",  # Extirpation of Matter from Trachea, Percutaneous Approach
        "0BC14ZZ",  # Extirpation of Matter from Trachea, Percutaneous Endoscopic Approach
        "0BC20ZZ",  # Extirpation of Matter from Carina, Open Approach
        "0BC23ZZ",  # Extirpation of Matter from Carina, Percutaneous Approach
        "0BC24ZZ",  # Extirpation of Matter from Carina, Percutaneous Endoscopic Approach
        "0BF10ZZ",  # Fragmentation in Trachea, Open Approach
        "0BF13ZZ",  # Fragmentation in Trachea, Percutaneous Approach
        "0BF14ZZ",  # Fragmentation in Trachea, Percutaneous Endoscopic Approach
        "0BF17ZZ",  # Fragmentation in Trachea, Via Natural or Artificial Opening
        "0BF18ZZ",  # Fragmentation in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BF20ZZ",  # Fragmentation in Carina, Open Approach
        "0BF23ZZ",  # Fragmentation in Carina, Percutaneous Approach
        "0BF24ZZ",  # Fragmentation in Carina, Percutaneous Endoscopic Approach
        "0BF27ZZ",  # Fragmentation in Carina, Via Natural or Artificial Opening
        "0BF28ZZ",  # Fragmentation in Carina, Via Natural or Artificial Opening Endoscopic
        "0BH102Z",  # Insertion of Monitoring Device into Trachea, Open Approach
        "0BH10DZ",  # Insertion of Intraluminal Device into Trachea, Open Approach
        "0BH10YZ",  # Insertion of Other Device into Trachea, Open Approach
        "0BH13DZ",  # Insertion of Intraluminal Device into Trachea, Percutaneous Approach
        "0BH13YZ",  # Insertion of Other Device into Trachea, Percutaneous Approach
        "0BH14DZ",  # Insertion of Intraluminal Device into Trachea, Percutaneous Endoscopic Approach
        "0BH14YZ",  # Insertion of Other Device into Trachea, Percutaneous Endoscopic Approach
        "0BH172Z",  # Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening
        "0BH17DZ",  # Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening
        "0BH17YZ",  # Insertion of Other Device into Trachea, Via Natural or Artificial Opening
        "0BH182Z",  # Insertion of Monitoring Device into Trachea, Via Natural or Artificial Opening Endoscopic
        "0BH18DZ",  # Insertion of Intraluminal Device into Trachea, Via Natural or Artificial Opening Endoscopic
        "0BH18YZ",  # Insertion of Other Device into Trachea, Via Natural or Artificial Opening Endoscopic
        "0BL10CZ",  # Occlusion of Trachea with Extraluminal Device, Open Approach
        "0BL10DZ",  # Occlusion of Trachea with Intraluminal Device, Open Approach
        "0BL10ZZ",  # Occlusion of Trachea, Open Approach
        "0BL13CZ",  # Occlusion of Trachea with Extraluminal Device, Percutaneous Approach
        "0BL13DZ",  # Occlusion of Trachea with Intraluminal Device, Percutaneous Approach
        "0BL13ZZ",  # Occlusion of Trachea, Percutaneous Approach
        "0BL14CZ",  # Occlusion of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach
        "0BL14DZ",  # Occlusion of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach
        "0BL14ZZ",  # Occlusion of Trachea, Percutaneous Endoscopic Approach
        "0BL17DZ",  # Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening
        "0BL17ZZ",  # Occlusion of Trachea, Via Natural or Artificial Opening
        "0BL18DZ",  # Occlusion of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0BL18ZZ",  # Occlusion of Trachea, Via Natural or Artificial Opening Endoscopic
        "0BL20CZ",  # Occlusion of Carina with Extraluminal Device, Open Approach
        "0BL20DZ",  # Occlusion of Carina with Intraluminal Device, Open Approach
        "0BL20ZZ",  # Occlusion of Carina, Open Approach
        "0BL23CZ",  # Occlusion of Carina with Extraluminal Device, Percutaneous Approach
        "0BL23DZ",  # Occlusion of Carina with Intraluminal Device, Percutaneous Approach
        "0BL23ZZ",  # Occlusion of Carina, Percutaneous Approach
        "0BL24CZ",  # Occlusion of Carina with Extraluminal Device, Percutaneous Endoscopic Approach
        "0BL24DZ",  # Occlusion of Carina with Intraluminal Device, Percutaneous Endoscopic Approach
        "0BL24ZZ",  # Occlusion of Carina, Percutaneous Endoscopic Approach
        "0BL27DZ",  # Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening
        "0BL27ZZ",  # Occlusion of Carina, Via Natural or Artificial Opening
        "0BL28DZ",  # Occlusion of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0BL28ZZ",  # Occlusion of Carina, Via Natural or Artificial Opening Endoscopic
        "0BM10ZZ",  # Reattachment of Trachea, Open Approach
        "0BM20ZZ",  # Reattachment of Carina, Open Approach
        "0BN10ZZ",  # Release Trachea, Open Approach
        "0BN13ZZ",  # Release Trachea, Percutaneous Approach
        "0BN14ZZ",  # Release Trachea, Percutaneous Endoscopic Approach
        "0BN17ZZ",  # Release Trachea, Via Natural or Artificial Opening
        "0BN18ZZ",  # Release Trachea, Via Natural or Artificial Opening Endoscopic
        "0BN20ZZ",  # Release Carina, Open Approach
        "0BN23ZZ",  # Release Carina, Percutaneous Approach
        "0BN24ZZ",  # Release Carina, Percutaneous Endoscopic Approach
        "0BN27ZZ",  # Release Carina, Via Natural or Artificial Opening
        "0BN28ZZ",  # Release Carina, Via Natural or Artificial Opening Endoscopic
        "0BP100Z",  # Removal of Drainage Device from Trachea, Open Approach
        "0BP102Z",  # Removal of Monitoring Device from Trachea, Open Approach
        "0BP107Z",  # Removal of Autologous Tissue Substitute from Trachea, Open Approach
        "0BP10CZ",  # Removal of Extraluminal Device from Trachea, Open Approach
        "0BP10DZ",  # Removal of Intraluminal Device from Trachea, Open Approach
        "0BP10FZ",  # Removal of Tracheostomy Device from Trachea, Open Approach
        "0BP10JZ",  # Removal of Synthetic Substitute from Trachea, Open Approach
        "0BP10KZ",  # Removal of Nonautologous Tissue Substitute from Trachea, Open Approach
        "0BP130Z",  # Removal of Drainage Device from Trachea, Percutaneous Approach
        "0BP132Z",  # Removal of Monitoring Device from Trachea, Percutaneous Approach
        "0BP137Z",  # Removal of Autologous Tissue Substitute from Trachea, Percutaneous Approach
        "0BP13CZ",  # Removal of Extraluminal Device from Trachea, Percutaneous Approach
        "0BP13DZ",  # Removal of Intraluminal Device from Trachea, Percutaneous Approach
        "0BP13FZ",  # Removal of Tracheostomy Device from Trachea, Percutaneous Approach
        "0BP13JZ",  # Removal of Synthetic Substitute from Trachea, Percutaneous Approach
        "0BP13KZ",  # Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Approach
        "0BP140Z",  # Removal of Drainage Device from Trachea, Percutaneous Endoscopic Approach
        "0BP142Z",  # Removal of Monitoring Device from Trachea, Percutaneous Endoscopic Approach
        "0BP147Z",  # Removal of Autologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach
        "0BP14CZ",  # Removal of Extraluminal Device from Trachea, Percutaneous Endoscopic Approach
        "0BP14DZ",  # Removal of Intraluminal Device from Trachea, Percutaneous Endoscopic Approach
        "0BP14FZ",  # Removal of Tracheostomy Device from Trachea, Percutaneous Endoscopic Approach
        "0BP14JZ",  # Removal of Synthetic Substitute from Trachea, Percutaneous Endoscopic Approach
        "0BP14KZ",  # Removal of Nonautologous Tissue Substitute from Trachea, Percutaneous Endoscopic Approach
        "0BP170Z",  # Removal of Drainage Device from Trachea, Via Natural or Artificial Opening
        "0BP172Z",  # Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening
        "0BP177Z",  # Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening
        "0BP17CZ",  # Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening
        "0BP17DZ",  # Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening
        "0BP17JZ",  # Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening
        "0BP17KZ",  # Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening
        "0BP180Z",  # Removal of Drainage Device from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP182Z",  # Removal of Monitoring Device from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP187Z",  # Removal of Autologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP18CZ",  # Removal of Extraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP18DZ",  # Removal of Intraluminal Device from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP18JZ",  # Removal of Synthetic Substitute from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BP18KZ",  # Removal of Nonautologous Tissue Substitute from Trachea, Via Natural or Artificial Opening Endoscopic
        "0BQ10ZZ",  # Repair Trachea, Open Approach
        "0BQ13ZZ",  # Repair Trachea, Percutaneous Approach
        "0BQ14ZZ",  # Repair Trachea, Percutaneous Endoscopic Approach
        "0BQ17ZZ",  # Repair Trachea, Via Natural or Artificial Opening
        "0BQ18ZZ",  # Repair Trachea, Via Natural or Artificial Opening Endoscopic
        "0BQ20ZZ",  # Repair Carina, Open Approach
        "0BQ23ZZ",  # Repair Carina, Percutaneous Approach
        "0BQ24ZZ",  # Repair Carina, Percutaneous Endoscopic Approach
        "0BQ27ZZ",  # Repair Carina, Via Natural or Artificial Opening
        "0BQ28ZZ",  # Repair Carina, Via Natural or Artificial Opening Endoscopic
        "0BR107Z",  # Replacement of Trachea with Autologous Tissue Substitute, Open Approach
        "0BR10JZ",  # Replacement of Trachea with Synthetic Substitute, Open Approach
        "0BR10KZ",  # Replacement of Trachea with Nonautologous Tissue Substitute, Open Approach
        "0BR147Z",  # Replacement of Trachea with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BR14JZ",  # Replacement of Trachea with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0BR14KZ",  # Replacement of Trachea with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BR207Z",  # Replacement of Carina with Autologous Tissue Substitute, Open Approach
        "0BR20JZ",  # Replacement of Carina with Synthetic Substitute, Open Approach
        "0BR20KZ",  # Replacement of Carina with Nonautologous Tissue Substitute, Open Approach
        "0BR247Z",  # Replacement of Carina with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BR24JZ",  # Replacement of Carina with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0BR24KZ",  # Replacement of Carina with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BS10ZZ",  # Reposition Trachea, Open Approach
        "0BS20ZZ",  # Reposition Carina, Open Approach
        "0BT10ZZ",  # Resection of Trachea, Open Approach
        "0BT14ZZ",  # Resection of Trachea, Percutaneous Endoscopic Approach
        "0BT20ZZ",  # Resection of Carina, Open Approach
        "0BT24ZZ",  # Resection of Carina, Percutaneous Endoscopic Approach
        "0BT30ZZ",  # Resection of Right Main Bronchus, Open Approach
        "0BT34ZZ",  # Resection of Right Main Bronchus, Percutaneous Endoscopic Approach
        "0BT40ZZ",  # Resection of Right Upper Lobe Bronchus, Open Approach
        "0BT44ZZ",  # Resection of Right Upper Lobe Bronchus, Percutaneous Endoscopic Approach
        "0BT50ZZ",  # Resection of Right Middle Lobe Bronchus, Open Approach
        "0BT54ZZ",  # Resection of Right Middle Lobe Bronchus, Percutaneous Endoscopic Approach
        "0BT60ZZ",  # Resection of Right Lower Lobe Bronchus, Open Approach
        "0BT64ZZ",  # Resection of Right Lower Lobe Bronchus, Percutaneous Endoscopic Approach
        "0BT70ZZ",  # Resection of Left Main Bronchus, Open Approach
        "0BT74ZZ",  # Resection of Left Main Bronchus, Percutaneous Endoscopic Approach
        "0BT80ZZ",  # Resection of Left Upper Lobe Bronchus, Open Approach
        "0BT84ZZ",  # Resection of Left Upper Lobe Bronchus, Percutaneous Endoscopic Approach
        "0BT90ZZ",  # Resection of Lingula Bronchus, Open Approach
        "0BT94ZZ",  # Resection of Lingula Bronchus, Percutaneous Endoscopic Approach
        "0BTB0ZZ",  # Resection of Left Lower Lobe Bronchus, Open Approach
        "0BTB4ZZ",  # Resection of Left Lower Lobe Bronchus, Percutaneous Endoscopic Approach
        "0BTC0ZZ",  # Resection of Right Upper Lung Lobe, Open Approach
        "0BTC4ZZ",  # Resection of Right Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BTD0ZZ",  # Resection of Right Middle Lung Lobe, Open Approach
        "0BTD4ZZ",  # Resection of Right Middle Lung Lobe, Percutaneous Endoscopic Approach
        "0BTF0ZZ",  # Resection of Right Lower Lung Lobe, Open Approach
        "0BTF4ZZ",  # Resection of Right Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BTG0ZZ",  # Resection of Left Upper Lung Lobe, Open Approach
        "0BTG4ZZ",  # Resection of Left Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BTH0ZZ",  # Resection of Lung Lingula, Open Approach
        "0BTH4ZZ",  # Resection of Lung Lingula, Percutaneous Endoscopic Approach
        "0BTJ0ZZ",  # Resection of Left Lower Lung Lobe, Open Approach
        "0BTJ4ZZ",  # Resection of Left Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BTK0ZZ",  # Resection of Right Lung, Open Approach
        "0BTK4ZZ",  # Resection of Right Lung, Percutaneous Endoscopic Approach
        "0BTL0ZZ",  # Resection of Left Lung, Open Approach
        "0BTL4ZZ",  # Resection of Left Lung, Percutaneous Endoscopic Approach
        "0BTM0ZZ",  # Resection of Bilateral Lungs, Open Approach
        "0BTM4ZZ",  # Resection of Bilateral Lungs, Percutaneous Endoscopic Approach
        "0BTT0ZZ",  # Resection of Diaphragm, Open Approach
        "0BTT4ZZ",  # Resection of Diaphragm, Percutaneous Endoscopic Approach
        "0BU107Z",  # Supplement Trachea with Autologous Tissue Substitute, Open Approach
        "0BU10JZ",  # Supplement Trachea with Synthetic Substitute, Open Approach
        "0BU10KZ",  # Supplement Trachea with Nonautologous Tissue Substitute, Open Approach
        "0BU147Z",  # Supplement Trachea with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BU14JZ",  # Supplement Trachea with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0BU14KZ",  # Supplement Trachea with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BU187Z",  # Supplement Trachea with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0BU18JZ",  # Supplement Trachea with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0BU18KZ",  # Supplement Trachea with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0BU207Z",  # Supplement Carina with Autologous Tissue Substitute, Open Approach
        "0BU20JZ",  # Supplement Carina with Synthetic Substitute, Open Approach
        "0BU20KZ",  # Supplement Carina with Nonautologous Tissue Substitute, Open Approach
        "0BU247Z",  # Supplement Carina with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BU24JZ",  # Supplement Carina with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0BU24KZ",  # Supplement Carina with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0BU287Z",  # Supplement Carina with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0BU28JZ",  # Supplement Carina with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0BU28KZ",  # Supplement Carina with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0BV10CZ",  # Restriction of Trachea with Extraluminal Device, Open Approach
        "0BV10DZ",  # Restriction of Trachea with Intraluminal Device, Open Approach
        "0BV10ZZ",  # Restriction of Trachea, Open Approach
        "0BV13CZ",  # Restriction of Trachea with Extraluminal Device, Percutaneous Approach
        "0BV13DZ",  # Restriction of Trachea with Intraluminal Device, Percutaneous Approach
        "0BV13ZZ",  # Restriction of Trachea, Percutaneous Approach
        "0BV14CZ",  # Restriction of Trachea with Extraluminal Device, Percutaneous Endoscopic Approach
        "0BV14DZ",  # Restriction of Trachea with Intraluminal Device, Percutaneous Endoscopic Approach
        "0BV14ZZ",  # Restriction of Trachea, Percutaneous Endoscopic Approach
        "0BV17DZ",  # Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening
        "0BV17ZZ",  # Restriction of Trachea, Via Natural or Artificial Opening
        "0BV18DZ",  # Restriction of Trachea with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0BV18ZZ",  # Restriction of Trachea, Via Natural or Artificial Opening Endoscopic
        "0BV20CZ",  # Restriction of Carina with Extraluminal Device, Open Approach
        "0BV20DZ",  # Restriction of Carina with Intraluminal Device, Open Approach
        "0BV20ZZ",  # Restriction of Carina, Open Approach
        "0BV23CZ",  # Restriction of Carina with Extraluminal Device, Percutaneous Approach
        "0BV23DZ",  # Restriction of Carina with Intraluminal Device, Percutaneous Approach
        "0BV23ZZ",  # Restriction of Carina, Percutaneous Approach
        "0BV24CZ",  # Restriction of Carina with Extraluminal Device, Percutaneous Endoscopic Approach
        "0BV24DZ",  # Restriction of Carina with Intraluminal Device, Percutaneous Endoscopic Approach
        "0BV24ZZ",  # Restriction of Carina, Percutaneous Endoscopic Approach
        "0BV27DZ",  # Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening
        "0BV27ZZ",  # Restriction of Carina, Via Natural or Artificial Opening
        "0BV28DZ",  # Restriction of Carina with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0BV28ZZ",  # Restriction of Carina, Via Natural or Artificial Opening Endoscopic
        "0BW100Z",  # Revision of Drainage Device in Trachea, Open Approach
        "0BW102Z",  # Revision of Monitoring Device in Trachea, Open Approach
        "0BW107Z",  # Revision of Autologous Tissue Substitute in Trachea, Open Approach
        "0BW10CZ",  # Revision of Extraluminal Device in Trachea, Open Approach
        "0BW10DZ",  # Revision of Intraluminal Device in Trachea, Open Approach
        "0BW10JZ",  # Revision of Synthetic Substitute in Trachea, Open Approach
        "0BW10KZ",  # Revision of Nonautologous Tissue Substitute in Trachea, Open Approach
        "0BW130Z",  # Revision of Drainage Device in Trachea, Percutaneous Approach
        "0BW132Z",  # Revision of Monitoring Device in Trachea, Percutaneous Approach
        "0BW137Z",  # Revision of Autologous Tissue Substitute in Trachea, Percutaneous Approach
        "0BW13CZ",  # Revision of Extraluminal Device in Trachea, Percutaneous Approach
        "0BW13DZ",  # Revision of Intraluminal Device in Trachea, Percutaneous Approach
        "0BW13JZ",  # Revision of Synthetic Substitute in Trachea, Percutaneous Approach
        "0BW13KZ",  # Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Approach
        "0BW140Z",  # Revision of Drainage Device in Trachea, Percutaneous Endoscopic Approach
        "0BW142Z",  # Revision of Monitoring Device in Trachea, Percutaneous Endoscopic Approach
        "0BW147Z",  # Revision of Autologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach
        "0BW14CZ",  # Revision of Extraluminal Device in Trachea, Percutaneous Endoscopic Approach
        "0BW14DZ",  # Revision of Intraluminal Device in Trachea, Percutaneous Endoscopic Approach
        "0BW14JZ",  # Revision of Synthetic Substitute in Trachea, Percutaneous Endoscopic Approach
        "0BW14KZ",  # Revision of Nonautologous Tissue Substitute in Trachea, Percutaneous Endoscopic Approach
        "0BW170Z",  # Revision of Drainage Device in Trachea, Via Natural or Artificial Opening
        "0BW172Z",  # Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening
        "0BW177Z",  # Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening
        "0BW17CZ",  # Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening
        "0BW17DZ",  # Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening
        "0BW17JZ",  # Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening
        "0BW17KZ",  # Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening
        "0BW180Z",  # Revision of Drainage Device in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW182Z",  # Revision of Monitoring Device in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW187Z",  # Revision of Autologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW18CZ",  # Revision of Extraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW18DZ",  # Revision of Intraluminal Device in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW18JZ",  # Revision of Synthetic Substitute in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BW18KZ",  # Revision of Nonautologous Tissue Substitute in Trachea, Via Natural or Artificial Opening Endoscopic
        "0BYC0Z0",  # Transplantation of Right Upper Lung Lobe, Allogeneic, Open Approach
        "0BYC0Z1",  # Transplantation of Right Upper Lung Lobe, Syngeneic, Open Approach
        "0BYC0Z2",  # Transplantation of Right Upper Lung Lobe, Zooplastic, Open Approach
        "0BYD0Z0",  # Transplantation of Right Middle Lung Lobe, Allogeneic, Open Approach
        "0BYD0Z1",  # Transplantation of Right Middle Lung Lobe, Syngeneic, Open Approach
        "0BYD0Z2",  # Transplantation of Right Middle Lung Lobe, Zooplastic, Open Approach
        "0BYF0Z0",  # Transplantation of Right Lower Lung Lobe, Allogeneic, Open Approach
        "0BYF0Z1",  # Transplantation of Right Lower Lung Lobe, Syngeneic, Open Approach
        "0BYF0Z2",  # Transplantation of Right Lower Lung Lobe, Zooplastic, Open Approach
        "0BYG0Z0",  # Transplantation of Left Upper Lung Lobe, Allogeneic, Open Approach
        "0BYG0Z1",  # Transplantation of Left Upper Lung Lobe, Syngeneic, Open Approach
        "0BYG0Z2",  # Transplantation of Left Upper Lung Lobe, Zooplastic, Open Approach
        "0BYH0Z0",  # Transplantation of Lung Lingula, Allogeneic, Open Approach
        "0BYH0Z1",  # Transplantation of Lung Lingula, Syngeneic, Open Approach
        "0BYH0Z2",  # Transplantation of Lung Lingula, Zooplastic, Open Approach
        "0BYJ0Z0",  # Transplantation of Left Lower Lung Lobe, Allogeneic, Open Approach
        "0BYJ0Z1",  # Transplantation of Left Lower Lung Lobe, Syngeneic, Open Approach
        "0BYJ0Z2",  # Transplantation of Left Lower Lung Lobe, Zooplastic, Open Approach
        "0BYK0Z0",  # Transplantation of Right Lung, Allogeneic, Open Approach
        "0BYK0Z1",  # Transplantation of Right Lung, Syngeneic, Open Approach
        "0BYK0Z2",  # Transplantation of Right Lung, Zooplastic, Open Approach
        "0BYL0Z0",  # Transplantation of Left Lung, Allogeneic, Open Approach
        "0BYL0Z1",  # Transplantation of Left Lung, Syngeneic, Open Approach
        "0BYL0Z2",  # Transplantation of Left Lung, Zooplastic, Open Approach
        "0BYM0Z0",  # Transplantation of Bilateral Lungs, Allogeneic, Open Approach
        "0BYM0Z1",  # Transplantation of Bilateral Lungs, Syngeneic, Open Approach
        "0BYM0Z2",  # Transplantation of Bilateral Lungs, Zooplastic, Open Approach
        "0C520ZZ",  # Destruction of Hard Palate, Open Approach
        "0C523ZZ",  # Destruction of Hard Palate, Percutaneous Approach
        "0C52XZZ",  # Destruction of Hard Palate, External Approach
        "0C530ZZ",  # Destruction of Soft Palate, Open Approach
        "0C533ZZ",  # Destruction of Soft Palate, Percutaneous Approach
        "0C53XZZ",  # Destruction of Soft Palate, External Approach
        "0C540ZZ",  # Destruction of Buccal Mucosa, Open Approach
        "0C570ZZ",  # Destruction of Tongue, Open Approach
        "0C573ZZ",  # Destruction of Tongue, Percutaneous Approach
        "0C57XZZ",  # Destruction of Tongue, External Approach
        "0C5M0ZZ",  # Destruction of Pharynx, Open Approach
        "0C5M3ZZ",  # Destruction of Pharynx, Percutaneous Approach
        "0C5M4ZZ",  # Destruction of Pharynx, Percutaneous Endoscopic Approach
        "0C5M7ZZ",  # Destruction of Pharynx, Via Natural or Artificial Opening
        "0C5M8ZZ",  # Destruction of Pharynx, Via Natural or Artificial Opening Endoscopic
        "0C5R0ZZ",  # Destruction of Epiglottis, Open Approach
        "0C5R3ZZ",  # Destruction of Epiglottis, Percutaneous Approach
        "0C5R4ZZ",  # Destruction of Epiglottis, Percutaneous Endoscopic Approach
        "0C5R7ZZ",  # Destruction of Epiglottis, Via Natural or Artificial Opening
        "0C5R8ZZ",  # Destruction of Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0C5S0ZZ",  # Destruction of Larynx, Open Approach
        "0C5S3ZZ",  # Destruction of Larynx, Percutaneous Approach
        "0C5S4ZZ",  # Destruction of Larynx, Percutaneous Endoscopic Approach
        "0C5S7ZZ",  # Destruction of Larynx, Via Natural or Artificial Opening
        "0C5S8ZZ",  # Destruction of Larynx, Via Natural or Artificial Opening Endoscopic
        "0C5T0ZZ",  # Destruction of Right Vocal Cord, Open Approach
        "0C5T3ZZ",  # Destruction of Right Vocal Cord, Percutaneous Approach
        "0C5T4ZZ",  # Destruction of Right Vocal Cord, Percutaneous Endoscopic Approach
        "0C5T7ZZ",  # Destruction of Right Vocal Cord, Via Natural or Artificial Opening
        "0C5T8ZZ",  # Destruction of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0C5V0ZZ",  # Destruction of Left Vocal Cord, Open Approach
        "0C5V3ZZ",  # Destruction of Left Vocal Cord, Percutaneous Approach
        "0C5V4ZZ",  # Destruction of Left Vocal Cord, Percutaneous Endoscopic Approach
        "0C5V7ZZ",  # Destruction of Left Vocal Cord, Via Natural or Artificial Opening
        "0C5V8ZZ",  # Destruction of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0C7M7DZ",  # Dilation of Pharynx with Intraluminal Device, Via Natural or Artificial Opening
        "0C7M7ZZ",  # Dilation of Pharynx, Via Natural or Artificial Opening
        "0C7M8DZ",  # Dilation of Pharynx with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0C7M8ZZ",  # Dilation of Pharynx, Via Natural or Artificial Opening Endoscopic
        "0C7S0DZ",  # Dilation of Larynx with Intraluminal Device, Open Approach
        "0C7S0ZZ",  # Dilation of Larynx, Open Approach
        "0C7S3DZ",  # Dilation of Larynx with Intraluminal Device, Percutaneous Approach
        "0C7S3ZZ",  # Dilation of Larynx, Percutaneous Approach
        "0C7S4DZ",  # Dilation of Larynx with Intraluminal Device, Percutaneous Endoscopic Approach
        "0C7S4ZZ",  # Dilation of Larynx, Percutaneous Endoscopic Approach
        "0C7S7DZ",  # Dilation of Larynx with Intraluminal Device, Via Natural or Artificial Opening
        "0C7S7ZZ",  # Dilation of Larynx, Via Natural or Artificial Opening
        "0C7S8DZ",  # Dilation of Larynx with Intraluminal Device, Via Natural or Artificial Opening Endoscopic
        "0C7S8ZZ",  # Dilation of Larynx, Via Natural or Artificial Opening Endoscopic
        "0C9200Z",  # Drainage of Hard Palate with Drainage Device, Open Approach
        "0C920ZZ",  # Drainage of Hard Palate, Open Approach
        "0C9300Z",  # Drainage of Soft Palate with Drainage Device, Open Approach
        "0C930ZZ",  # Drainage of Soft Palate, Open Approach
        "0C9330Z",  # Drainage of Soft Palate with Drainage Device, Percutaneous Approach
        "0C933ZZ",  # Drainage of Soft Palate, Percutaneous Approach
        "0C93X0Z",  # Drainage of Soft Palate with Drainage Device, External Approach
        "0C93XZZ",  # Drainage of Soft Palate, External Approach
        "0C9400Z",  # Drainage of Buccal Mucosa with Drainage Device, Open Approach
        "0C940ZZ",  # Drainage of Buccal Mucosa, Open Approach
        "0C9700Z",  # Drainage of Tongue with Drainage Device, Open Approach
        "0C970ZZ",  # Drainage of Tongue, Open Approach
        "0C9730Z",  # Drainage of Tongue with Drainage Device, Percutaneous Approach
        "0C973ZZ",  # Drainage of Tongue, Percutaneous Approach
        "0C97X0Z",  # Drainage of Tongue with Drainage Device, External Approach
        "0C97XZZ",  # Drainage of Tongue, External Approach
        "0C9M00Z",  # Drainage of Pharynx with Drainage Device, Open Approach
        "0C9M0ZZ",  # Drainage of Pharynx, Open Approach
        "0C9M30Z",  # Drainage of Pharynx with Drainage Device, Percutaneous Approach
        "0C9M3ZZ",  # Drainage of Pharynx, Percutaneous Approach
        "0C9M40Z",  # Drainage of Pharynx with Drainage Device, Percutaneous Endoscopic Approach
        "0C9M4ZZ",  # Drainage of Pharynx, Percutaneous Endoscopic Approach
        "0C9R00Z",  # Drainage of Epiglottis with Drainage Device, Open Approach
        "0C9R0ZZ",  # Drainage of Epiglottis, Open Approach
        "0C9R30Z",  # Drainage of Epiglottis with Drainage Device, Percutaneous Approach
        "0C9R3ZZ",  # Drainage of Epiglottis, Percutaneous Approach
        "0C9R40Z",  # Drainage of Epiglottis with Drainage Device, Percutaneous Endoscopic Approach
        "0C9R4ZZ",  # Drainage of Epiglottis, Percutaneous Endoscopic Approach
        "0C9R70Z",  # Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening
        "0C9R7ZZ",  # Drainage of Epiglottis, Via Natural or Artificial Opening
        "0C9R80Z",  # Drainage of Epiglottis with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0C9R8ZZ",  # Drainage of Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0C9S00Z",  # Drainage of Larynx with Drainage Device, Open Approach
        "0C9S0ZZ",  # Drainage of Larynx, Open Approach
        "0C9S30Z",  # Drainage of Larynx with Drainage Device, Percutaneous Approach
        "0C9S3ZZ",  # Drainage of Larynx, Percutaneous Approach
        "0C9S40Z",  # Drainage of Larynx with Drainage Device, Percutaneous Endoscopic Approach
        "0C9S4ZZ",  # Drainage of Larynx, Percutaneous Endoscopic Approach
        "0C9T00Z",  # Drainage of Right Vocal Cord with Drainage Device, Open Approach
        "0C9T0ZZ",  # Drainage of Right Vocal Cord, Open Approach
        "0C9T30Z",  # Drainage of Right Vocal Cord with Drainage Device, Percutaneous Approach
        "0C9T3ZZ",  # Drainage of Right Vocal Cord, Percutaneous Approach
        "0C9T40Z",  # Drainage of Right Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach
        "0C9T4ZZ",  # Drainage of Right Vocal Cord, Percutaneous Endoscopic Approach
        "0C9T70Z",  # Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening
        "0C9T7ZZ",  # Drainage of Right Vocal Cord, Via Natural or Artificial Opening
        "0C9T80Z",  # Drainage of Right Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0C9T8ZZ",  # Drainage of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0C9V00Z",  # Drainage of Left Vocal Cord with Drainage Device, Open Approach
        "0C9V0ZZ",  # Drainage of Left Vocal Cord, Open Approach
        "0C9V30Z",  # Drainage of Left Vocal Cord with Drainage Device, Percutaneous Approach
        "0C9V3ZZ",  # Drainage of Left Vocal Cord, Percutaneous Approach
        "0C9V40Z",  # Drainage of Left Vocal Cord with Drainage Device, Percutaneous Endoscopic Approach
        "0C9V4ZZ",  # Drainage of Left Vocal Cord, Percutaneous Endoscopic Approach
        "0C9V70Z",  # Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening
        "0C9V7ZZ",  # Drainage of Left Vocal Cord, Via Natural or Artificial Opening
        "0C9V80Z",  # Drainage of Left Vocal Cord with Drainage Device, Via Natural or Artificial Opening Endoscopic
        "0C9V8ZZ",  # Drainage of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CB20ZZ",  # Excision of Hard Palate, Open Approach
        "0CB23ZZ",  # Excision of Hard Palate, Percutaneous Approach
        "0CB30ZZ",  # Excision of Soft Palate, Open Approach
        "0CB33ZZ",  # Excision of Soft Palate, Percutaneous Approach
        "0CB3XZZ",  # Excision of Soft Palate, External Approach
        "0CB40ZZ",  # Excision of Buccal Mucosa, Open Approach
        "0CB70ZZ",  # Excision of Tongue, Open Approach
        "0CB73ZZ",  # Excision of Tongue, Percutaneous Approach
        "0CBM0ZZ",  # Excision of Pharynx, Open Approach
        "0CBM3ZZ",  # Excision of Pharynx, Percutaneous Approach
        "0CBM4ZZ",  # Excision of Pharynx, Percutaneous Endoscopic Approach
        "0CBM7ZZ",  # Excision of Pharynx, Via Natural or Artificial Opening
        "0CBM8ZZ",  # Excision of Pharynx, Via Natural or Artificial Opening Endoscopic
        "0CBR0ZZ",  # Excision of Epiglottis, Open Approach
        "0CBR3ZZ",  # Excision of Epiglottis, Percutaneous Approach
        "0CBR4ZZ",  # Excision of Epiglottis, Percutaneous Endoscopic Approach
        "0CBR7ZZ",  # Excision of Epiglottis, Via Natural or Artificial Opening
        "0CBR8ZZ",  # Excision of Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CBS0ZZ",  # Excision of Larynx, Open Approach
        "0CBS3ZZ",  # Excision of Larynx, Percutaneous Approach
        "0CBS4ZZ",  # Excision of Larynx, Percutaneous Endoscopic Approach
        "0CBS7ZZ",  # Excision of Larynx, Via Natural or Artificial Opening
        "0CBS8ZZ",  # Excision of Larynx, Via Natural or Artificial Opening Endoscopic
        "0CBT0ZZ",  # Excision of Right Vocal Cord, Open Approach
        "0CBT3ZZ",  # Excision of Right Vocal Cord, Percutaneous Approach
        "0CBT4ZZ",  # Excision of Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CBT7ZZ",  # Excision of Right Vocal Cord, Via Natural or Artificial Opening
        "0CBT8ZZ",  # Excision of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CBV0ZZ",  # Excision of Left Vocal Cord, Open Approach
        "0CBV3ZZ",  # Excision of Left Vocal Cord, Percutaneous Approach
        "0CBV4ZZ",  # Excision of Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CBV7ZZ",  # Excision of Left Vocal Cord, Via Natural or Artificial Opening
        "0CBV8ZZ",  # Excision of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CC20ZZ",  # Extirpation of Matter from Hard Palate, Open Approach
        "0CC23ZZ",  # Extirpation of Matter from Hard Palate, Percutaneous Approach
        "0CC30ZZ",  # Extirpation of Matter from Soft Palate, Open Approach
        "0CC33ZZ",  # Extirpation of Matter from Soft Palate, Percutaneous Approach
        "0CC40ZZ",  # Extirpation of Matter from Buccal Mucosa, Open Approach
        "0CC70ZZ",  # Extirpation of Matter from Tongue, Open Approach
        "0CC73ZZ",  # Extirpation of Matter from Tongue, Percutaneous Approach
        "0CCM0ZZ",  # Extirpation of Matter from Pharynx, Open Approach
        "0CCM3ZZ",  # Extirpation of Matter from Pharynx, Percutaneous Approach
        "0CCM4ZZ",  # Extirpation of Matter from Pharynx, Percutaneous Endoscopic Approach
        "0CCR0ZZ",  # Extirpation of Matter from Epiglottis, Open Approach
        "0CCR3ZZ",  # Extirpation of Matter from Epiglottis, Percutaneous Approach
        "0CCR4ZZ",  # Extirpation of Matter from Epiglottis, Percutaneous Endoscopic Approach
        "0CCR7ZZ",  # Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening
        "0CCR8ZZ",  # Extirpation of Matter from Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CCS0ZZ",  # Extirpation of Matter from Larynx, Open Approach
        "0CCS3ZZ",  # Extirpation of Matter from Larynx, Percutaneous Approach
        "0CCS4ZZ",  # Extirpation of Matter from Larynx, Percutaneous Endoscopic Approach
        "0CCT0ZZ",  # Extirpation of Matter from Right Vocal Cord, Open Approach
        "0CCT3ZZ",  # Extirpation of Matter from Right Vocal Cord, Percutaneous Approach
        "0CCT4ZZ",  # Extirpation of Matter from Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CCT7ZZ",  # Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening
        "0CCT8ZZ",  # Extirpation of Matter from Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CCV0ZZ",  # Extirpation of Matter from Left Vocal Cord, Open Approach
        "0CCV3ZZ",  # Extirpation of Matter from Left Vocal Cord, Percutaneous Approach
        "0CCV4ZZ",  # Extirpation of Matter from Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CCV7ZZ",  # Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening
        "0CCV8ZZ",  # Extirpation of Matter from Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CDT0ZZ",  # Extraction of Right Vocal Cord, Open Approach
        "0CDT3ZZ",  # Extraction of Right Vocal Cord, Percutaneous Approach
        "0CDT4ZZ",  # Extraction of Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CDT7ZZ",  # Extraction of Right Vocal Cord, Via Natural or Artificial Opening
        "0CDT8ZZ",  # Extraction of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CDV0ZZ",  # Extraction of Left Vocal Cord, Open Approach
        "0CDV3ZZ",  # Extraction of Left Vocal Cord, Percutaneous Approach
        "0CDV4ZZ",  # Extraction of Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CDV7ZZ",  # Extraction of Left Vocal Cord, Via Natural or Artificial Opening
        "0CDV8ZZ",  # Extraction of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CM30ZZ",  # Reattachment of Soft Palate, Open Approach
        "0CM70ZZ",  # Reattachment of Tongue, Open Approach
        "0CMN0ZZ",  # Reattachment of Uvula, Open Approach
        "0CN20ZZ",  # Release Hard Palate, Open Approach
        "0CN23ZZ",  # Release Hard Palate, Percutaneous Approach
        "0CN2XZZ",  # Release Hard Palate, External Approach
        "0CN30ZZ",  # Release Soft Palate, Open Approach
        "0CN33ZZ",  # Release Soft Palate, Percutaneous Approach
        "0CN3XZZ",  # Release Soft Palate, External Approach
        "0CN40ZZ",  # Release Buccal Mucosa, Open Approach
        "0CNM0ZZ",  # Release Pharynx, Open Approach
        "0CNM3ZZ",  # Release Pharynx, Percutaneous Approach
        "0CNM4ZZ",  # Release Pharynx, Percutaneous Endoscopic Approach
        "0CNM7ZZ",  # Release Pharynx, Via Natural or Artificial Opening
        "0CNM8ZZ",  # Release Pharynx, Via Natural or Artificial Opening Endoscopic
        "0CNR0ZZ",  # Release Epiglottis, Open Approach
        "0CNR3ZZ",  # Release Epiglottis, Percutaneous Approach
        "0CNR4ZZ",  # Release Epiglottis, Percutaneous Endoscopic Approach
        "0CNR7ZZ",  # Release Epiglottis, Via Natural or Artificial Opening
        "0CNR8ZZ",  # Release Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CNS0ZZ",  # Release Larynx, Open Approach
        "0CNS3ZZ",  # Release Larynx, Percutaneous Approach
        "0CNS4ZZ",  # Release Larynx, Percutaneous Endoscopic Approach
        "0CNS7ZZ",  # Release Larynx, Via Natural or Artificial Opening
        "0CNS8ZZ",  # Release Larynx, Via Natural or Artificial Opening Endoscopic
        "0CNT0ZZ",  # Release Right Vocal Cord, Open Approach
        "0CNT3ZZ",  # Release Right Vocal Cord, Percutaneous Approach
        "0CNT4ZZ",  # Release Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CNT7ZZ",  # Release Right Vocal Cord, Via Natural or Artificial Opening
        "0CNT8ZZ",  # Release Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CNV0ZZ",  # Release Left Vocal Cord, Open Approach
        "0CNV3ZZ",  # Release Left Vocal Cord, Percutaneous Approach
        "0CNV4ZZ",  # Release Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CNV7ZZ",  # Release Left Vocal Cord, Via Natural or Artificial Opening
        "0CNV8ZZ",  # Release Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CPS00Z",  # Removal of Drainage Device from Larynx, Open Approach
        "0CPS07Z",  # Removal of Autologous Tissue Substitute from Larynx, Open Approach
        "0CPS0DZ",  # Removal of Intraluminal Device from Larynx, Open Approach
        "0CPS0JZ",  # Removal of Synthetic Substitute from Larynx, Open Approach
        "0CPS0KZ",  # Removal of Nonautologous Tissue Substitute from Larynx, Open Approach
        "0CPS0YZ",  # Removal of Other Device from Larynx, Open Approach
        "0CPS30Z",  # Removal of Drainage Device from Larynx, Percutaneous Approach
        "0CPS37Z",  # Removal of Autologous Tissue Substitute from Larynx, Percutaneous Approach
        "0CPS3DZ",  # Removal of Intraluminal Device from Larynx, Percutaneous Approach
        "0CPS3JZ",  # Removal of Synthetic Substitute from Larynx, Percutaneous Approach
        "0CPS3KZ",  # Removal of Nonautologous Tissue Substitute from Larynx, Percutaneous Approach
        "0CPS3YZ",  # Removal of Other Device from Larynx, Percutaneous Approach
        "0CPS70Z",  # Removal of Drainage Device from Larynx, Via Natural or Artificial Opening
        "0CPS77Z",  # Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening
        "0CPS7DZ",  # Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening
        "0CPS7JZ",  # Removal of Synthetic Substitute from Larynx, Via Natural or Artificial Opening
        "0CPS7KZ",  # Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening
        "0CPS7YZ",  # Removal of Other Device from Larynx, Via Natural or Artificial Opening
        "0CPS80Z",  # Removal of Drainage Device from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CPS87Z",  # Removal of Autologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CPS8DZ",  # Removal of Intraluminal Device from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CPS8JZ",  # Removal of Synthetic Substitute from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CPS8KZ",  # Removal of Nonautologous Tissue Substitute from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CPS8YZ",  # Removal of Other Device from Larynx, Via Natural or Artificial Opening Endoscopic
        "0CQ20ZZ",  # Repair Hard Palate, Open Approach
        "0CQ23ZZ",  # Repair Hard Palate, Percutaneous Approach
        "0CQ30ZZ",  # Repair Soft Palate, Open Approach
        "0CQ33ZZ",  # Repair Soft Palate, Percutaneous Approach
        "0CQ3XZZ",  # Repair Soft Palate, External Approach
        "0CQ40ZZ",  # Repair Buccal Mucosa, Open Approach
        "0CQ70ZZ",  # Repair Tongue, Open Approach
        "0CQ73ZZ",  # Repair Tongue, Percutaneous Approach
        "0CQM0ZZ",  # Repair Pharynx, Open Approach
        "0CQM3ZZ",  # Repair Pharynx, Percutaneous Approach
        "0CQM4ZZ",  # Repair Pharynx, Percutaneous Endoscopic Approach
        "0CQM7ZZ",  # Repair Pharynx, Via Natural or Artificial Opening
        "0CQM8ZZ",  # Repair Pharynx, Via Natural or Artificial Opening Endoscopic
        "0CQR0ZZ",  # Repair Epiglottis, Open Approach
        "0CQR3ZZ",  # Repair Epiglottis, Percutaneous Approach
        "0CQR4ZZ",  # Repair Epiglottis, Percutaneous Endoscopic Approach
        "0CQR7ZZ",  # Repair Epiglottis, Via Natural or Artificial Opening
        "0CQR8ZZ",  # Repair Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CQS0ZZ",  # Repair Larynx, Open Approach
        "0CQS3ZZ",  # Repair Larynx, Percutaneous Approach
        "0CQS4ZZ",  # Repair Larynx, Percutaneous Endoscopic Approach
        "0CQS7ZZ",  # Repair Larynx, Via Natural or Artificial Opening
        "0CQS8ZZ",  # Repair Larynx, Via Natural or Artificial Opening Endoscopic
        "0CQT0ZZ",  # Repair Right Vocal Cord, Open Approach
        "0CQT3ZZ",  # Repair Right Vocal Cord, Percutaneous Approach
        "0CQT4ZZ",  # Repair Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CQT7ZZ",  # Repair Right Vocal Cord, Via Natural or Artificial Opening
        "0CQT8ZZ",  # Repair Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CQV0ZZ",  # Repair Left Vocal Cord, Open Approach
        "0CQV3ZZ",  # Repair Left Vocal Cord, Percutaneous Approach
        "0CQV4ZZ",  # Repair Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CQV7ZZ",  # Repair Left Vocal Cord, Via Natural or Artificial Opening
        "0CQV8ZZ",  # Repair Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CR007Z",  # Replacement of Upper Lip with Autologous Tissue Substitute, Open Approach
        "0CR107Z",  # Replacement of Lower Lip with Autologous Tissue Substitute, Open Approach
        "0CR207Z",  # Replacement of Hard Palate with Autologous Tissue Substitute, Open Approach
        "0CR20JZ",  # Replacement of Hard Palate with Synthetic Substitute, Open Approach
        "0CR20KZ",  # Replacement of Hard Palate with Nonautologous Tissue Substitute, Open Approach
        "0CR237Z",  # Replacement of Hard Palate with Autologous Tissue Substitute, Percutaneous Approach
        "0CR23JZ",  # Replacement of Hard Palate with Synthetic Substitute, Percutaneous Approach
        "0CR23KZ",  # Replacement of Hard Palate with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CR2X7Z",  # Replacement of Hard Palate with Autologous Tissue Substitute, External Approach
        "0CR2XJZ",  # Replacement of Hard Palate with Synthetic Substitute, External Approach
        "0CR2XKZ",  # Replacement of Hard Palate with Nonautologous Tissue Substitute, External Approach
        "0CR307Z",  # Replacement of Soft Palate with Autologous Tissue Substitute, Open Approach
        "0CR30JZ",  # Replacement of Soft Palate with Synthetic Substitute, Open Approach
        "0CR30KZ",  # Replacement of Soft Palate with Nonautologous Tissue Substitute, Open Approach
        "0CR337Z",  # Replacement of Soft Palate with Autologous Tissue Substitute, Percutaneous Approach
        "0CR33JZ",  # Replacement of Soft Palate with Synthetic Substitute, Percutaneous Approach
        "0CR33KZ",  # Replacement of Soft Palate with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CR3X7Z",  # Replacement of Soft Palate with Autologous Tissue Substitute, External Approach
        "0CR3XJZ",  # Replacement of Soft Palate with Synthetic Substitute, External Approach
        "0CR3XKZ",  # Replacement of Soft Palate with Nonautologous Tissue Substitute, External Approach
        "0CR407Z",  # Replacement of Buccal Mucosa with Autologous Tissue Substitute, Open Approach
        "0CR40JZ",  # Replacement of Buccal Mucosa with Synthetic Substitute, Open Approach
        "0CR40KZ",  # Replacement of Buccal Mucosa with Nonautologous Tissue Substitute, Open Approach
        "0CR4X7Z",  # Replacement of Buccal Mucosa with Autologous Tissue Substitute, External Approach
        "0CR4XJZ",  # Replacement of Buccal Mucosa with Synthetic Substitute, External Approach
        "0CR4XKZ",  # Replacement of Buccal Mucosa with Nonautologous Tissue Substitute, External Approach
        "0CR707Z",  # Replacement of Tongue with Autologous Tissue Substitute, Open Approach
        "0CR70JZ",  # Replacement of Tongue with Synthetic Substitute, Open Approach
        "0CR70KZ",  # Replacement of Tongue with Nonautologous Tissue Substitute, Open Approach
        "0CR737Z",  # Replacement of Tongue with Autologous Tissue Substitute, Percutaneous Approach
        "0CR73JZ",  # Replacement of Tongue with Synthetic Substitute, Percutaneous Approach
        "0CR73KZ",  # Replacement of Tongue with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CR7X7Z",  # Replacement of Tongue with Autologous Tissue Substitute, External Approach
        "0CR7XJZ",  # Replacement of Tongue with Synthetic Substitute, External Approach
        "0CR7XKZ",  # Replacement of Tongue with Nonautologous Tissue Substitute, External Approach
        "0CRM07Z",  # Replacement of Pharynx with Autologous Tissue Substitute, Open Approach
        "0CRM0JZ",  # Replacement of Pharynx with Synthetic Substitute, Open Approach
        "0CRM0KZ",  # Replacement of Pharynx with Nonautologous Tissue Substitute, Open Approach
        "0CRM77Z",  # Replacement of Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRM7JZ",  # Replacement of Pharynx with Synthetic Substitute, Via Natural or Artificial Opening
        "0CRM7KZ",  # Replacement of Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRM87Z",  # Replacement of Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRM8JZ",  # Replacement of Pharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRM8KZ",  # Replacement of Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRN07Z",  # Replacement of Uvula with Autologous Tissue Substitute, Open Approach
        "0CRN0JZ",  # Replacement of Uvula with Synthetic Substitute, Open Approach
        "0CRN0KZ",  # Replacement of Uvula with Nonautologous Tissue Substitute, Open Approach
        "0CRN37Z",  # Replacement of Uvula with Autologous Tissue Substitute, Percutaneous Approach
        "0CRN3JZ",  # Replacement of Uvula with Synthetic Substitute, Percutaneous Approach
        "0CRN3KZ",  # Replacement of Uvula with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CRNX7Z",  # Replacement of Uvula with Autologous Tissue Substitute, External Approach
        "0CRNXJZ",  # Replacement of Uvula with Synthetic Substitute, External Approach
        "0CRNXKZ",  # Replacement of Uvula with Nonautologous Tissue Substitute, External Approach
        "0CRR07Z",  # Replacement of Epiglottis with Autologous Tissue Substitute, Open Approach
        "0CRR0JZ",  # Replacement of Epiglottis with Synthetic Substitute, Open Approach
        "0CRR0KZ",  # Replacement of Epiglottis with Nonautologous Tissue Substitute, Open Approach
        "0CRR77Z",  # Replacement of Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRR7JZ",  # Replacement of Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening
        "0CRR7KZ",  # Replacement of Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRR87Z",  # Replacement of Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRR8JZ",  # Replacement of Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRR8KZ",  # Replacement of Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRS07Z",  # Replacement of Larynx with Autologous Tissue Substitute, Open Approach
        "0CRS0JZ",  # Replacement of Larynx with Synthetic Substitute, Open Approach
        "0CRS0KZ",  # Replacement of Larynx with Nonautologous Tissue Substitute, Open Approach
        "0CRS77Z",  # Replacement of Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRS7JZ",  # Replacement of Larynx with Synthetic Substitute, Via Natural or Artificial Opening
        "0CRS7KZ",  # Replacement of Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRS87Z",  # Replacement of Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRS8JZ",  # Replacement of Larynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRS8KZ",  # Replacement of Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRT07Z",  # Replacement of Right Vocal Cord with Autologous Tissue Substitute, Open Approach
        "0CRT0JZ",  # Replacement of Right Vocal Cord with Synthetic Substitute, Open Approach
        "0CRT0KZ",  # Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Open Approach
        "0CRT77Z",  # Replacement of Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRT7JZ",  # Replacement of Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening
        "0CRT7KZ",  # Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRT87Z",  # Replacement of Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRT8JZ",  # Replacement of Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRT8KZ",  # Replacement of Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRV07Z",  # Replacement of Left Vocal Cord with Autologous Tissue Substitute, Open Approach
        "0CRV0JZ",  # Replacement of Left Vocal Cord with Synthetic Substitute, Open Approach
        "0CRV0KZ",  # Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Open Approach
        "0CRV77Z",  # Replacement of Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRV7JZ",  # Replacement of Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening
        "0CRV7KZ",  # Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CRV87Z",  # Replacement of Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRV8JZ",  # Replacement of Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CRV8KZ",  # Replacement of Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CS20ZZ",  # Reposition Hard Palate, Open Approach
        "0CS2XZZ",  # Reposition Hard Palate, External Approach
        "0CS30ZZ",  # Reposition Soft Palate, Open Approach
        "0CS3XZZ",  # Reposition Soft Palate, External Approach
        "0CS70ZZ",  # Reposition Tongue, Open Approach
        "0CS7XZZ",  # Reposition Tongue, External Approach
        "0CSR0ZZ",  # Reposition Epiglottis, Open Approach
        "0CSR7ZZ",  # Reposition Epiglottis, Via Natural or Artificial Opening
        "0CSR8ZZ",  # Reposition Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CSS0ZZ",  # Reposition Larynx, Open Approach
        "0CSS7ZZ",  # Reposition Larynx, Via Natural or Artificial Opening
        "0CSS8ZZ",  # Reposition Larynx, Via Natural or Artificial Opening Endoscopic
        "0CST0ZZ",  # Reposition Right Vocal Cord, Open Approach
        "0CST7ZZ",  # Reposition Right Vocal Cord, Via Natural or Artificial Opening
        "0CST8ZZ",  # Reposition Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CSV0ZZ",  # Reposition Left Vocal Cord, Open Approach
        "0CSV7ZZ",  # Reposition Left Vocal Cord, Via Natural or Artificial Opening
        "0CSV8ZZ",  # Reposition Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CT20ZZ",  # Resection of Hard Palate, Open Approach
        "0CT2XZZ",  # Resection of Hard Palate, External Approach
        "0CT30ZZ",  # Resection of Soft Palate, Open Approach
        "0CT3XZZ",  # Resection of Soft Palate, External Approach
        "0CT70ZZ",  # Resection of Tongue, Open Approach
        "0CT7XZZ",  # Resection of Tongue, External Approach
        "0CTM0ZZ",  # Resection of Pharynx, Open Approach
        "0CTM4ZZ",  # Resection of Pharynx, Percutaneous Endoscopic Approach
        "0CTM7ZZ",  # Resection of Pharynx, Via Natural or Artificial Opening
        "0CTM8ZZ",  # Resection of Pharynx, Via Natural or Artificial Opening Endoscopic
        "0CTN0ZZ",  # Resection of Uvula, Open Approach
        "0CTNXZZ",  # Resection of Uvula, External Approach
        "0CTR0ZZ",  # Resection of Epiglottis, Open Approach
        "0CTR4ZZ",  # Resection of Epiglottis, Percutaneous Endoscopic Approach
        "0CTR7ZZ",  # Resection of Epiglottis, Via Natural or Artificial Opening
        "0CTR8ZZ",  # Resection of Epiglottis, Via Natural or Artificial Opening Endoscopic
        "0CTS0ZZ",  # Resection of Larynx, Open Approach
        "0CTS4ZZ",  # Resection of Larynx, Percutaneous Endoscopic Approach
        "0CTS7ZZ",  # Resection of Larynx, Via Natural or Artificial Opening
        "0CTS8ZZ",  # Resection of Larynx, Via Natural or Artificial Opening Endoscopic
        "0CTT0ZZ",  # Resection of Right Vocal Cord, Open Approach
        "0CTT4ZZ",  # Resection of Right Vocal Cord, Percutaneous Endoscopic Approach
        "0CTT7ZZ",  # Resection of Right Vocal Cord, Via Natural or Artificial Opening
        "0CTT8ZZ",  # Resection of Right Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CTV0ZZ",  # Resection of Left Vocal Cord, Open Approach
        "0CTV4ZZ",  # Resection of Left Vocal Cord, Percutaneous Endoscopic Approach
        "0CTV7ZZ",  # Resection of Left Vocal Cord, Via Natural or Artificial Opening
        "0CTV8ZZ",  # Resection of Left Vocal Cord, Via Natural or Artificial Opening Endoscopic
        "0CU207Z",  # Supplement Hard Palate with Autologous Tissue Substitute, Open Approach
        "0CU20JZ",  # Supplement Hard Palate with Synthetic Substitute, Open Approach
        "0CU20KZ",  # Supplement Hard Palate with Nonautologous Tissue Substitute, Open Approach
        "0CU237Z",  # Supplement Hard Palate with Autologous Tissue Substitute, Percutaneous Approach
        "0CU23JZ",  # Supplement Hard Palate with Synthetic Substitute, Percutaneous Approach
        "0CU23KZ",  # Supplement Hard Palate with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CU2X7Z",  # Supplement Hard Palate with Autologous Tissue Substitute, External Approach
        "0CU2XJZ",  # Supplement Hard Palate with Synthetic Substitute, External Approach
        "0CU2XKZ",  # Supplement Hard Palate with Nonautologous Tissue Substitute, External Approach
        "0CU307Z",  # Supplement Soft Palate with Autologous Tissue Substitute, Open Approach
        "0CU30JZ",  # Supplement Soft Palate with Synthetic Substitute, Open Approach
        "0CU30KZ",  # Supplement Soft Palate with Nonautologous Tissue Substitute, Open Approach
        "0CU337Z",  # Supplement Soft Palate with Autologous Tissue Substitute, Percutaneous Approach
        "0CU33JZ",  # Supplement Soft Palate with Synthetic Substitute, Percutaneous Approach
        "0CU33KZ",  # Supplement Soft Palate with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CU3X7Z",  # Supplement Soft Palate with Autologous Tissue Substitute, External Approach
        "0CU3XJZ",  # Supplement Soft Palate with Synthetic Substitute, External Approach
        "0CU3XKZ",  # Supplement Soft Palate with Nonautologous Tissue Substitute, External Approach
        "0CU407Z",  # Supplement Buccal Mucosa with Autologous Tissue Substitute, Open Approach
        "0CU40JZ",  # Supplement Buccal Mucosa with Synthetic Substitute, Open Approach
        "0CU40KZ",  # Supplement Buccal Mucosa with Nonautologous Tissue Substitute, Open Approach
        "0CU4X7Z",  # Supplement Buccal Mucosa with Autologous Tissue Substitute, External Approach
        "0CU4XJZ",  # Supplement Buccal Mucosa with Synthetic Substitute, External Approach
        "0CU4XKZ",  # Supplement Buccal Mucosa with Nonautologous Tissue Substitute, External Approach
        "0CU707Z",  # Supplement Tongue with Autologous Tissue Substitute, Open Approach
        "0CU70JZ",  # Supplement Tongue with Synthetic Substitute, Open Approach
        "0CU70KZ",  # Supplement Tongue with Nonautologous Tissue Substitute, Open Approach
        "0CU737Z",  # Supplement Tongue with Autologous Tissue Substitute, Percutaneous Approach
        "0CU73JZ",  # Supplement Tongue with Synthetic Substitute, Percutaneous Approach
        "0CU73KZ",  # Supplement Tongue with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CU7X7Z",  # Supplement Tongue with Autologous Tissue Substitute, External Approach
        "0CU7XJZ",  # Supplement Tongue with Synthetic Substitute, External Approach
        "0CU7XKZ",  # Supplement Tongue with Nonautologous Tissue Substitute, External Approach
        "0CUM07Z",  # Supplement Pharynx with Autologous Tissue Substitute, Open Approach
        "0CUM0JZ",  # Supplement Pharynx with Synthetic Substitute, Open Approach
        "0CUM0KZ",  # Supplement Pharynx with Nonautologous Tissue Substitute, Open Approach
        "0CUM77Z",  # Supplement Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUM7JZ",  # Supplement Pharynx with Synthetic Substitute, Via Natural or Artificial Opening
        "0CUM7KZ",  # Supplement Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUM87Z",  # Supplement Pharynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUM8JZ",  # Supplement Pharynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUM8KZ",  # Supplement Pharynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUN07Z",  # Supplement Uvula with Autologous Tissue Substitute, Open Approach
        "0CUN0JZ",  # Supplement Uvula with Synthetic Substitute, Open Approach
        "0CUN0KZ",  # Supplement Uvula with Nonautologous Tissue Substitute, Open Approach
        "0CUN37Z",  # Supplement Uvula with Autologous Tissue Substitute, Percutaneous Approach
        "0CUN3JZ",  # Supplement Uvula with Synthetic Substitute, Percutaneous Approach
        "0CUN3KZ",  # Supplement Uvula with Nonautologous Tissue Substitute, Percutaneous Approach
        "0CUNX7Z",  # Supplement Uvula with Autologous Tissue Substitute, External Approach
        "0CUNXJZ",  # Supplement Uvula with Synthetic Substitute, External Approach
        "0CUNXKZ",  # Supplement Uvula with Nonautologous Tissue Substitute, External Approach
        "0CUR07Z",  # Supplement Epiglottis with Autologous Tissue Substitute, Open Approach
        "0CUR0JZ",  # Supplement Epiglottis with Synthetic Substitute, Open Approach
        "0CUR0KZ",  # Supplement Epiglottis with Nonautologous Tissue Substitute, Open Approach
        "0CUR77Z",  # Supplement Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUR7JZ",  # Supplement Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening
        "0CUR7KZ",  # Supplement Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUR87Z",  # Supplement Epiglottis with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUR8JZ",  # Supplement Epiglottis with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUR8KZ",  # Supplement Epiglottis with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUS07Z",  # Supplement Larynx with Autologous Tissue Substitute, Open Approach
        "0CUS0JZ",  # Supplement Larynx with Synthetic Substitute, Open Approach
        "0CUS0KZ",  # Supplement Larynx with Nonautologous Tissue Substitute, Open Approach
        "0CUS77Z",  # Supplement Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUS7JZ",  # Supplement Larynx with Synthetic Substitute, Via Natural or Artificial Opening
        "0CUS7KZ",  # Supplement Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUS87Z",  # Supplement Larynx with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUS8JZ",  # Supplement Larynx with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUS8KZ",  # Supplement Larynx with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUT07Z",  # Supplement Right Vocal Cord with Autologous Tissue Substitute, Open Approach
        "0CUT0JZ",  # Supplement Right Vocal Cord with Synthetic Substitute, Open Approach
        "0CUT0KZ",  # Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Open Approach
        "0CUT77Z",  # Supplement Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUT7JZ",  # Supplement Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening
        "0CUT7KZ",  # Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUT87Z",  # Supplement Right Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUT8JZ",  # Supplement Right Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUT8KZ",  # Supplement Right Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUV07Z",  # Supplement Left Vocal Cord with Autologous Tissue Substitute, Open Approach
        "0CUV0JZ",  # Supplement Left Vocal Cord with Synthetic Substitute, Open Approach
        "0CUV0KZ",  # Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Open Approach
        "0CUV77Z",  # Supplement Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUV7JZ",  # Supplement Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening
        "0CUV7KZ",  # Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0CUV87Z",  # Supplement Left Vocal Cord with Autologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUV8JZ",  # Supplement Left Vocal Cord with Synthetic Substitute, Via Natural or Artificial Opening Endoscopic
        "0CUV8KZ",  # Supplement Left Vocal Cord with Nonautologous Tissue Substitute, Via Natural or Artificial Opening Endoscopic
        "0CWS00Z",  # Revision of Drainage Device in Larynx, Open Approach
        "0CWS07Z",  # Revision of Autologous Tissue Substitute in Larynx, Open Approach
        "0CWS0DZ",  # Revision of Intraluminal Device in Larynx, Open Approach
        "0CWS0JZ",  # Revision of Synthetic Substitute in Larynx, Open Approach
        "0CWS0KZ",  # Revision of Nonautologous Tissue Substitute in Larynx, Open Approach
        "0CWS0YZ",  # Revision of Other Device in Larynx, Open Approach
        "0CWS30Z",  # Revision of Drainage Device in Larynx, Percutaneous Approach
        "0CWS37Z",  # Revision of Autologous Tissue Substitute in Larynx, Percutaneous Approach
        "0CWS3DZ",  # Revision of Intraluminal Device in Larynx, Percutaneous Approach
        "0CWS3JZ",  # Revision of Synthetic Substitute in Larynx, Percutaneous Approach
        "0CWS3KZ",  # Revision of Nonautologous Tissue Substitute in Larynx, Percutaneous Approach
        "0CWS3YZ",  # Revision of Other Device in Larynx, Percutaneous Approach
        "0CWS70Z",  # Revision of Drainage Device in Larynx, Via Natural or Artificial Opening
        "0CWS77Z",  # Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening
        "0CWS7DZ",  # Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening
        "0CWS7JZ",  # Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening
        "0CWS7KZ",  # Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening
        "0CWS7YZ",  # Revision of Other Device in Larynx, Via Natural or Artificial Opening
        "0CWS80Z",  # Revision of Drainage Device in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWS87Z",  # Revision of Autologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWS8DZ",  # Revision of Intraluminal Device in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWS8JZ",  # Revision of Synthetic Substitute in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWS8KZ",  # Revision of Nonautologous Tissue Substitute in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWS8YZ",  # Revision of Other Device in Larynx, Via Natural or Artificial Opening Endoscopic
        "0CWY00Z",  # Revision of Drainage Device in Mouth and Throat, Open Approach
        "0CWY01Z",  # Revision of Radioactive Element in Mouth and Throat, Open Approach
        "0CWY0DZ",  # Revision of Intraluminal Device in Mouth and Throat, Open Approach
        "0CWY0JZ",  # Revision of Synthetic Substitute in Mouth and Throat, Open Approach
        "0CWY0KZ",  # Revision of Nonautologous Tissue Substitute in Mouth and Throat, Open Approach
        "0CWY0YZ",  # Revision of Other Device in Mouth and Throat, Open Approach
        "0CWY30Z",  # Revision of Drainage Device in Mouth and Throat, Percutaneous Approach
        "0CWY31Z",  # Revision of Radioactive Element in Mouth and Throat, Percutaneous Approach
        "0CWY37Z",  # Revision of Autologous Tissue Substitute in Mouth and Throat, Percutaneous Approach
        "0CWY3DZ",  # Revision of Intraluminal Device in Mouth and Throat, Percutaneous Approach
        "0CWY3JZ",  # Revision of Synthetic Substitute in Mouth and Throat, Percutaneous Approach
        "0CWY3KZ",  # Revision of Nonautologous Tissue Substitute in Mouth and Throat, Percutaneous Approach
        "0CWY3YZ",  # Revision of Other Device in Mouth and Throat, Percutaneous Approach
        "0CWY70Z",  # Revision of Drainage Device in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY71Z",  # Revision of Radioactive Element in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY77Z",  # Revision of Autologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY7DZ",  # Revision of Intraluminal Device in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY7JZ",  # Revision of Synthetic Substitute in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY7KZ",  # Revision of Nonautologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY7YZ",  # Revision of Other Device in Mouth and Throat, Via Natural or Artificial Opening
        "0CWY80Z",  # Revision of Drainage Device in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY81Z",  # Revision of Radioactive Element in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY87Z",  # Revision of Autologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY8DZ",  # Revision of Intraluminal Device in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY8JZ",  # Revision of Synthetic Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY8KZ",  # Revision of Nonautologous Tissue Substitute in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CWY8YZ",  # Revision of Other Device in Mouth and Throat, Via Natural or Artificial Opening Endoscopic
        "0CX30ZZ",  # Transfer Soft Palate, Open Approach
        "0CX3XZZ",  # Transfer Soft Palate, External Approach
        "0CX40ZZ",  # Transfer Buccal Mucosa, Open Approach
        "0CX4XZZ",  # Transfer Buccal Mucosa, External Approach
        "0CX70ZZ",  # Transfer Tongue, Open Approach
        "0CX7XZZ",  # Transfer Tongue, External Approach
        "0J9100Z",  # Drainage of Face Subcutaneous Tissue and Fascia with Drainage Device, Open Approach
        "0J910ZZ",  # Drainage of Face Subcutaneous Tissue and Fascia, Open Approach
        "0JB40ZZ",  # Excision of Right Neck Subcutaneous Tissue and Fascia, Open Approach
        "0K840ZZ",  # Division of Tongue, Palate, Pharynx Muscle, Open Approach
        "0K843ZZ",  # Division of Tongue, Palate, Pharynx Muscle, Percutaneous Approach
        "0K844ZZ",  # Division of Tongue, Palate, Pharynx Muscle, Percutaneous Endoscopic Approach
        "0K847ZZ",  # Division of Tongue, Palate, Pharynx Muscle, Via Natural or Artificial Opening
        "0K848ZZ",  # Division of Tongue, Palate, Pharynx Muscle, Via Natural or Artificial Opening Endoscopic
        "0N5R0ZZ",  # Destruction of Maxilla, Open Approach
        "0N8R0ZZ",  # Division of Maxilla, Open Approach
        "0N9R00Z",  # Drainage of Maxilla with Drainage Device, Open Approach
        "0N9R0ZZ",  # Drainage of Maxilla, Open Approach
        "0NBC0ZZ",  # Excision of Sphenoid Bone, Open Approach
        "0NBF0ZZ",  # Excision of Right Ethmoid Bone, Open Approach
        "0NBG0ZZ",  # Excision of Left Ethmoid Bone, Open Approach
        "0NBK0ZZ",  # Excision of Right Palatine Bone, Open Approach
        "0NBL0ZZ",  # Excision of Left Palatine Bone, Open Approach
        "0NBR0ZZ",  # Excision of Maxilla, Open Approach
        "0NBT0ZZ",  # Excision of Right Mandible, Open Approach
        "0NBV0ZZ",  # Excision of Left Mandible, Open Approach
        "0NBX0ZZ",  # Excision of Hyoid Bone, Open Approach
        "0NBX3ZZ",  # Excision of Hyoid Bone, Percutaneous Approach
        "0NBX4ZZ",  # Excision of Hyoid Bone, Percutaneous Endoscopic Approach
        "0NCR0ZZ",  # Extirpation of Matter from Maxilla, Open Approach
        "0NDR0ZZ",  # Extraction of Maxilla, Open Approach
        "0NHR04Z",  # Insertion of Internal Fixation Device into Maxilla, Open Approach
        "0NHR05Z",  # Insertion of External Fixation Device into Maxilla, Open Approach
        "0NNC0ZZ",  # Release Sphenoid Bone, Open Approach
        "0NNF0ZZ",  # Release Right Ethmoid Bone, Open Approach
        "0NNG0ZZ",  # Release Left Ethmoid Bone, Open Approach
        "0NNK0ZZ",  # Release Right Palatine Bone, Open Approach
        "0NNL0ZZ",  # Release Left Palatine Bone, Open Approach
        "0NNR0ZZ",  # Release Maxilla, Open Approach
        "0NNX0ZZ",  # Release Hyoid Bone, Open Approach
        "0NNX3ZZ",  # Release Hyoid Bone, Percutaneous Approach
        "0NNX4ZZ",  # Release Hyoid Bone, Percutaneous Endoscopic Approach
        "0NQC0ZZ",  # Repair Sphenoid Bone, Open Approach
        "0NQF0ZZ",  # Repair Right Ethmoid Bone, Open Approach
        "0NQG0ZZ",  # Repair Left Ethmoid Bone, Open Approach
        "0NQK0ZZ",  # Repair Right Palatine Bone, Open Approach
        "0NQL0ZZ",  # Repair Left Palatine Bone, Open Approach
        "0NQR0ZZ",  # Repair Maxilla, Open Approach
        "0NQX0ZZ",  # Repair Hyoid Bone, Open Approach
        "0NQX3ZZ",  # Repair Hyoid Bone, Percutaneous Approach
        "0NQX4ZZ",  # Repair Hyoid Bone, Percutaneous Endoscopic Approach
        "0NQXXZZ",  # Repair Hyoid Bone, External Approach
        "0NRC07Z",  # Replacement of Sphenoid Bone with Autologous Tissue Substitute, Open Approach
        "0NRC0JZ",  # Replacement of Sphenoid Bone with Synthetic Substitute, Open Approach
        "0NRC0KZ",  # Replacement of Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRF07Z",  # Replacement of Right Ethmoid Bone with Autologous Tissue Substitute, Open Approach
        "0NRF0JZ",  # Replacement of Right Ethmoid Bone with Synthetic Substitute, Open Approach
        "0NRF0KZ",  # Replacement of Right Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRG07Z",  # Replacement of Left Ethmoid Bone with Autologous Tissue Substitute, Open Approach
        "0NRG0JZ",  # Replacement of Left Ethmoid Bone with Synthetic Substitute, Open Approach
        "0NRG0KZ",  # Replacement of Left Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRK07Z",  # Replacement of Right Palatine Bone with Autologous Tissue Substitute, Open Approach
        "0NRK0JZ",  # Replacement of Right Palatine Bone with Synthetic Substitute, Open Approach
        "0NRK0KZ",  # Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRK37Z",  # Replacement of Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach
        "0NRK3JZ",  # Replacement of Right Palatine Bone with Synthetic Substitute, Percutaneous Approach
        "0NRK3KZ",  # Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NRK47Z",  # Replacement of Right Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRK4JZ",  # Replacement of Right Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NRK4KZ",  # Replacement of Right Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRL07Z",  # Replacement of Left Palatine Bone with Autologous Tissue Substitute, Open Approach
        "0NRL0JZ",  # Replacement of Left Palatine Bone with Synthetic Substitute, Open Approach
        "0NRL0KZ",  # Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRL37Z",  # Replacement of Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Approach
        "0NRL3JZ",  # Replacement of Left Palatine Bone with Synthetic Substitute, Percutaneous Approach
        "0NRL3KZ",  # Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NRL47Z",  # Replacement of Left Palatine Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRL4JZ",  # Replacement of Left Palatine Bone with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NRL4KZ",  # Replacement of Left Palatine Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRR07Z",  # Replacement of Maxilla with Autologous Tissue Substitute, Open Approach
        "0NRR0JZ",  # Replacement of Maxilla with Synthetic Substitute, Open Approach
        "0NRR0KZ",  # Replacement of Maxilla with Nonautologous Tissue Substitute, Open Approach
        "0NRT07Z",  # Replacement of Right Mandible with Autologous Tissue Substitute, Open Approach
        "0NRT0JZ",  # Replacement of Right Mandible with Synthetic Substitute, Open Approach
        "0NRT0KZ",  # Replacement of Right Mandible with Nonautologous Tissue Substitute, Open Approach
        "0NRT37Z",  # Replacement of Right Mandible with Autologous Tissue Substitute, Percutaneous Approach
        "0NRT3JZ",  # Replacement of Right Mandible with Synthetic Substitute, Percutaneous Approach
        "0NRT3KZ",  # Replacement of Right Mandible with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NRT47Z",  # Replacement of Right Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRT4JZ",  # Replacement of Right Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NRT4KZ",  # Replacement of Right Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRV07Z",  # Replacement of Left Mandible with Autologous Tissue Substitute, Open Approach
        "0NRV0JZ",  # Replacement of Left Mandible with Synthetic Substitute, Open Approach
        "0NRV0KZ",  # Replacement of Left Mandible with Nonautologous Tissue Substitute, Open Approach
        "0NRV37Z",  # Replacement of Left Mandible with Autologous Tissue Substitute, Percutaneous Approach
        "0NRV3JZ",  # Replacement of Left Mandible with Synthetic Substitute, Percutaneous Approach
        "0NRV3KZ",  # Replacement of Left Mandible with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NRV47Z",  # Replacement of Left Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRV4JZ",  # Replacement of Left Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NRV4KZ",  # Replacement of Left Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRX07Z",  # Replacement of Hyoid Bone with Autologous Tissue Substitute, Open Approach
        "0NRX0JZ",  # Replacement of Hyoid Bone with Synthetic Substitute, Open Approach
        "0NRX0KZ",  # Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NRX37Z",  # Replacement of Hyoid Bone with Autologous Tissue Substitute, Percutaneous Approach
        "0NRX3JZ",  # Replacement of Hyoid Bone with Synthetic Substitute, Percutaneous Approach
        "0NRX3KZ",  # Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NRX47Z",  # Replacement of Hyoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NRX4JZ",  # Replacement of Hyoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NRX4KZ",  # Replacement of Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NSC04Z",  # Reposition Sphenoid Bone with Internal Fixation Device, Open Approach
        "0NSC0ZZ",  # Reposition Sphenoid Bone, Open Approach
        "0NSF04Z",  # Reposition Right Ethmoid Bone with Internal Fixation Device, Open Approach
        "0NSF0ZZ",  # Reposition Right Ethmoid Bone, Open Approach
        "0NSG04Z",  # Reposition Left Ethmoid Bone with Internal Fixation Device, Open Approach
        "0NSG0ZZ",  # Reposition Left Ethmoid Bone, Open Approach
        "0NSK04Z",  # Reposition Right Palatine Bone with Internal Fixation Device, Open Approach
        "0NSK0ZZ",  # Reposition Right Palatine Bone, Open Approach
        "0NSL04Z",  # Reposition Left Palatine Bone with Internal Fixation Device, Open Approach
        "0NSL0ZZ",  # Reposition Left Palatine Bone, Open Approach
        "0NSR04Z",  # Reposition Maxilla with Internal Fixation Device, Open Approach
        "0NSR05Z",  # Reposition Maxilla with External Fixation Device, Open Approach
        "0NSR0ZZ",  # Reposition Maxilla, Open Approach
        "0NSX04Z",  # Reposition Hyoid Bone with Internal Fixation Device, Open Approach
        "0NSX0ZZ",  # Reposition Hyoid Bone, Open Approach
        "0NSX34Z",  # Reposition Hyoid Bone with Internal Fixation Device, Percutaneous Approach
        "0NSX3ZZ",  # Reposition Hyoid Bone, Percutaneous Approach
        "0NSX44Z",  # Reposition Hyoid Bone with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0NSX4ZZ",  # Reposition Hyoid Bone, Percutaneous Endoscopic Approach
        "0NSXXZZ",  # Reposition Hyoid Bone, External Approach
        "0NTC0ZZ",  # Resection of Sphenoid Bone, Open Approach
        "0NTF0ZZ",  # Resection of Right Ethmoid Bone, Open Approach
        "0NTG0ZZ",  # Resection of Left Ethmoid Bone, Open Approach
        "0NTK0ZZ",  # Resection of Right Palatine Bone, Open Approach
        "0NTL0ZZ",  # Resection of Left Palatine Bone, Open Approach
        "0NTR0ZZ",  # Resection of Maxilla, Open Approach
        "0NTT0ZZ",  # Resection of Right Mandible, Open Approach
        "0NTV0ZZ",  # Resection of Left Mandible, Open Approach
        "0NTX0ZZ",  # Resection of Hyoid Bone, Open Approach
        "0NUC07Z",  # Supplement Sphenoid Bone with Autologous Tissue Substitute, Open Approach
        "0NUC0JZ",  # Supplement Sphenoid Bone with Synthetic Substitute, Open Approach
        "0NUC0KZ",  # Supplement Sphenoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUF07Z",  # Supplement Right Ethmoid Bone with Autologous Tissue Substitute, Open Approach
        "0NUF0JZ",  # Supplement Right Ethmoid Bone with Synthetic Substitute, Open Approach
        "0NUF0KZ",  # Supplement Right Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUG07Z",  # Supplement Left Ethmoid Bone with Autologous Tissue Substitute, Open Approach
        "0NUG0JZ",  # Supplement Left Ethmoid Bone with Synthetic Substitute, Open Approach
        "0NUG0KZ",  # Supplement Left Ethmoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUK07Z",  # Supplement Right Palatine Bone with Autologous Tissue Substitute, Open Approach
        "0NUK0JZ",  # Supplement Right Palatine Bone with Synthetic Substitute, Open Approach
        "0NUK0KZ",  # Supplement Right Palatine Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUL07Z",  # Supplement Left Palatine Bone with Autologous Tissue Substitute, Open Approach
        "0NUL0JZ",  # Supplement Left Palatine Bone with Synthetic Substitute, Open Approach
        "0NUL0KZ",  # Supplement Left Palatine Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUR07Z",  # Supplement Maxilla with Autologous Tissue Substitute, Open Approach
        "0NUR0JZ",  # Supplement Maxilla with Synthetic Substitute, Open Approach
        "0NUR0KZ",  # Supplement Maxilla with Nonautologous Tissue Substitute, Open Approach
        "0NUT07Z",  # Supplement Right Mandible with Autologous Tissue Substitute, Open Approach
        "0NUT0JZ",  # Supplement Right Mandible with Synthetic Substitute, Open Approach
        "0NUT0KZ",  # Supplement Right Mandible with Nonautologous Tissue Substitute, Open Approach
        "0NUT37Z",  # Supplement Right Mandible with Autologous Tissue Substitute, Percutaneous Approach
        "0NUT3JZ",  # Supplement Right Mandible with Synthetic Substitute, Percutaneous Approach
        "0NUT3KZ",  # Supplement Right Mandible with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NUT47Z",  # Supplement Right Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NUT4JZ",  # Supplement Right Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NUT4KZ",  # Supplement Right Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NUV07Z",  # Supplement Left Mandible with Autologous Tissue Substitute, Open Approach
        "0NUV0JZ",  # Supplement Left Mandible with Synthetic Substitute, Open Approach
        "0NUV0KZ",  # Supplement Left Mandible with Nonautologous Tissue Substitute, Open Approach
        "0NUV37Z",  # Supplement Left Mandible with Autologous Tissue Substitute, Percutaneous Approach
        "0NUV3JZ",  # Supplement Left Mandible with Synthetic Substitute, Percutaneous Approach
        "0NUV3KZ",  # Supplement Left Mandible with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NUV47Z",  # Supplement Left Mandible with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NUV4JZ",  # Supplement Left Mandible with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NUV4KZ",  # Supplement Left Mandible with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NUX07Z",  # Supplement Hyoid Bone with Autologous Tissue Substitute, Open Approach
        "0NUX0JZ",  # Supplement Hyoid Bone with Synthetic Substitute, Open Approach
        "0NUX0KZ",  # Supplement Hyoid Bone with Nonautologous Tissue Substitute, Open Approach
        "0NUX37Z",  # Supplement Hyoid Bone with Autologous Tissue Substitute, Percutaneous Approach
        "0NUX3JZ",  # Supplement Hyoid Bone with Synthetic Substitute, Percutaneous Approach
        "0NUX3KZ",  # Supplement Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Approach
        "0NUX47Z",  # Supplement Hyoid Bone with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0NUX4JZ",  # Supplement Hyoid Bone with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0NUX4KZ",  # Supplement Hyoid Bone with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PB20ZZ",  # Excision of 3 or More Ribs, Open Approach
        "0PB24ZZ",  # Excision of 3 or More Ribs, Percutaneous Endoscopic Approach
        "0PT20ZZ",  # Resection of 3 or More Ribs, Open Approach
        "0RGC04Z",  # Fusion of Right Temporomandibular Joint with Internal Fixation Device, Open Approach
        "0RGC07Z",  # Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Open Approach
        "0RGC0JZ",  # Fusion of Right Temporomandibular Joint with Synthetic Substitute, Open Approach
        "0RGC0KZ",  # Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Open Approach
        "0RGC34Z",  # Fusion of Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach
        "0RGC37Z",  # Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RGC3JZ",  # Fusion of Right Temporomandibular Joint with Synthetic Substitute, Percutaneous Approach
        "0RGC3KZ",  # Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RGC44Z",  # Fusion of Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RGC47Z",  # Fusion of Right Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RGC4JZ",  # Fusion of Right Temporomandibular Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RGC4KZ",  # Fusion of Right Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RGD04Z",  # Fusion of Left Temporomandibular Joint with Internal Fixation Device, Open Approach
        "0RGD07Z",  # Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Open Approach
        "0RGD0JZ",  # Fusion of Left Temporomandibular Joint with Synthetic Substitute, Open Approach
        "0RGD0KZ",  # Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Open Approach
        "0RGD34Z",  # Fusion of Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach
        "0RGD37Z",  # Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RGD3JZ",  # Fusion of Left Temporomandibular Joint with Synthetic Substitute, Percutaneous Approach
        "0RGD3KZ",  # Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RGD44Z",  # Fusion of Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RGD47Z",  # Fusion of Left Temporomandibular Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RGD4JZ",  # Fusion of Left Temporomandibular Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RGD4KZ",  # Fusion of Left Temporomandibular Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RNC0ZZ",  # Release Right Temporomandibular Joint, Open Approach
        "0RNC3ZZ",  # Release Right Temporomandibular Joint, Percutaneous Approach
        "0RNC4ZZ",  # Release Right Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RND0ZZ",  # Release Left Temporomandibular Joint, Open Approach
        "0RND3ZZ",  # Release Left Temporomandibular Joint, Percutaneous Approach
        "0RND4ZZ",  # Release Left Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RPC04Z",  # Removal of Internal Fixation Device from Right Temporomandibular Joint, Open Approach
        "0RPC34Z",  # Removal of Internal Fixation Device from Right Temporomandibular Joint, Percutaneous Approach
        "0RPC44Z",  # Removal of Internal Fixation Device from Right Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RPD04Z",  # Removal of Internal Fixation Device from Left Temporomandibular Joint, Open Approach
        "0RPD34Z",  # Removal of Internal Fixation Device from Left Temporomandibular Joint, Percutaneous Approach
        "0RPD44Z",  # Removal of Internal Fixation Device from Left Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RSC04Z",  # Reposition Right Temporomandibular Joint with Internal Fixation Device, Open Approach
        "0RSC0ZZ",  # Reposition Right Temporomandibular Joint, Open Approach
        "0RSC34Z",  # Reposition Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach
        "0RSC3ZZ",  # Reposition Right Temporomandibular Joint, Percutaneous Approach
        "0RSC44Z",  # Reposition Right Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RSC4ZZ",  # Reposition Right Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RSD04Z",  # Reposition Left Temporomandibular Joint with Internal Fixation Device, Open Approach
        "0RSD0ZZ",  # Reposition Left Temporomandibular Joint, Open Approach
        "0RSD34Z",  # Reposition Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Approach
        "0RSD3ZZ",  # Reposition Left Temporomandibular Joint, Percutaneous Approach
        "0RSD44Z",  # Reposition Left Temporomandibular Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RSD4ZZ",  # Reposition Left Temporomandibular Joint, Percutaneous Endoscopic Approach
        "0RTC0ZZ",  # Resection of Right Temporomandibular Joint, Open Approach
        "0RTD0ZZ",  # Resection of Left Temporomandibular Joint, Open Approach
        "0W9200Z",  # Drainage of Face with Drainage Device, Open Approach
        "0W920ZZ",  # Drainage of Face, Open Approach
        "0W9300Z",  # Drainage of Oral Cavity and Throat with Drainage Device, Open Approach
        "0W930ZZ",  # Drainage of Oral Cavity and Throat, Open Approach
        "0W9330Z",  # Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Approach
        "0W933ZZ",  # Drainage of Oral Cavity and Throat, Percutaneous Approach
        "0W9340Z",  # Drainage of Oral Cavity and Throat with Drainage Device, Percutaneous Endoscopic Approach
        "0W934ZZ",  # Drainage of Oral Cavity and Throat, Percutaneous Endoscopic Approach
        "0W9400Z",  # Drainage of Upper Jaw with Drainage Device, Open Approach
        "0W940ZZ",  # Drainage of Upper Jaw, Open Approach
        "0W9500Z",  # Drainage of Lower Jaw with Drainage Device, Open Approach
        "0W950ZZ",  # Drainage of Lower Jaw, Open Approach
        "0WB80ZZ",  # Excision of Chest Wall, Open Approach
        "0WB84ZZ",  # Excision of Chest Wall, Percutaneous Endoscopic Approach
        "0WBC0ZZ",  # Excision of Mediastinum, Open Approach
        "0WBC4ZZ",  # Excision of Mediastinum, Percutaneous Endoscopic Approach
        "0WC30ZZ",  # Extirpation of Matter from Oral Cavity and Throat, Open Approach
        "0WC33ZZ",  # Extirpation of Matter from Oral Cavity and Throat, Percutaneous Approach
        "0WC34ZZ",  # Extirpation of Matter from Oral Cavity and Throat, Percutaneous Endoscopic Approach
        "0WF30ZZ",  # Fragmentation in Oral Cavity and Throat, Open Approach
        "0WF33ZZ",  # Fragmentation in Oral Cavity and Throat, Percutaneous Approach
        "0WF34ZZ",  # Fragmentation in Oral Cavity and Throat, Percutaneous Endoscopic Approach
        "0WH303Z",  # Insertion of Infusion Device into Oral Cavity and Throat, Open Approach
        "0WH30YZ",  # Insertion of Other Device into Oral Cavity and Throat, Open Approach
        "0WU807Z",  # Supplement Chest Wall with Autologous Tissue Substitute, Open Approach
        "0WU80JZ",  # Supplement Chest Wall with Synthetic Substitute, Open Approach
        "0WU80KZ",  # Supplement Chest Wall with Nonautologous Tissue Substitute, Open Approach
        "0WU847Z",  # Supplement Chest Wall with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0WU84JZ",  # Supplement Chest Wall with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0WU84KZ",  # Supplement Chest Wall with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "1055303009",  # Incision of left frontal sinus via transorbital approach (procedure)
        "1055304003",  # Incision of right frontal sinus via transorbital approach (procedure)
        "107808005",  # Accessory sinus excision (procedure)
        "107812004",  # Accessory sinus destructive procedure (procedure)
        "107818000",  # Larynx manipulation (procedure)
        "107923001",  # Pharynx destructive procedure (procedure)
        "108184008",  # Diaphragm excision (procedure)
        "1119419009",  # Radical dissection of left half of neck (procedure)
        "1119420003",  # Radical dissection of right half of neck (procedure)
        "1119421004",  # Modified radical dissection of left half of neck (procedure)
        "1119422006",  # Modified radical dissection of right half of neck (procedure)
        "1119432004",  # Partial glossectomy with radical dissection of left half of neck (procedure)
        "1119433009",  # Partial glossectomy with radical dissection of right half of neck (procedure)
        "11231000087108",  # Dissection of lymph nodes of bilateral sides of central compartment of neck (procedure)
        "112716008",  # Excision of first rib (procedure)
        "112718009",  # Excision of first AND cervical rib (procedure)
        "112788002",  # Operation on accessory sinus (procedure)
        "112794005",  # Elevation of orbital bone fragments of frontal sinus (procedure)
        "112796007",  # Closure of stoma of larynx (procedure)
        "112854004",  # Fixation of tongue (procedure)
        "112855003",  # Lysis of adhesions of tongue (procedure)
        "11347004",  # Modified radical neck dissection (procedure)
        "1144496008",  # Excision of lesion of chest wall and reconstruction with prosthesis (procedure)
        "1144788004",  # Repair of bone of face (procedure)
        "1156743006",  # Percutaneous drainage of neck using computed tomography guidance (procedure)
        "1162557005",  # Excision of polyp of vocal cord (procedure)
        "118785007",  # Procedure on nasopharynx (procedure)
        "118790005",  # Procedure on vocal cord (procedure)
        "118792002",  # Procedure on trachea (procedure)
        "118816007",  # Procedure on vestibule of mouth (procedure)
        "118820006",  # Procedure on floor of mouth (procedure)
        "119000",  # Thoracoscopic partial lobectomy of lung (procedure)
        "119256007",  # Resection of tracheal carina (procedure)
        "119582004",  # Excision of neck (procedure)
        "119588000",  # Neck closure (procedure)
        "119590004",  # Neck reconstruction (procedure)
        "119592007",  # Neck repair (procedure)
        "119594008",  # Neck destructive procedure (procedure)
        "119597001",  # Neck transplantation (procedure)
        "119599003",  # Neck transposition (procedure)
        "119703009",  # Nasopharynx incision (procedure)
        "119704003",  # Nasopharynx excision (procedure)
        "119710003",  # Nasopharynx closure (procedure)
        "119713001",  # Nasopharynx reconstruction (procedure)
        "119715008",  # Nasopharynx destructive procedure (procedure)
        "119718005",  # Transplantation of tissue of nasopharynx (procedure)
        "119719002",  # Nasopharynx manipulation (procedure)
        "119723005",  # Accessory sinus closure (procedure)
        "119724004",  # Accessory sinus reconstruction (procedure)
        "119725003",  # Transplantation of tissue of accessory sinus (procedure)
        "119726002",  # Larynx closure (procedure)
        "119727006",  # Larynx reconstruction (procedure)
        "119729009",  # Transposition of tissue of larynx (procedure)
        "119737001",  # Trachea closure (procedure)
        "119739003",  # Trachea reconstruction (procedure)
        "119740001",  # Trachea destructive procedure (procedure)
        "119745006",  # Trachea manipulation (procedure)
        "119746007",  # Lung excision (procedure)
        "119867002",  # Mouth destructive procedure (procedure)
        "119868007",  # Mouth transplantation (procedure)
        "119870003",  # Mouth transposition (procedure)
        "119873001",  # Tongue closure (procedure)
        "119874007",  # Tongue reconstruction (procedure)
        "119876009",  # Pharynx closure (procedure)
        "120146008",  # Chest wall excision (procedure)
        "120153004",  # Chest wall reconstruction (procedure)
        "1202001002",  # W-plasty of skin of head (procedure)
        "1202002009",  # W-plasty of skin of neck (procedure)
        "1202033006",  # Harvest of flap (procedure)
        "1204264000",  # Reconstruction of skin using local subcutaneous pedicle island axial pattern skin flap (procedure)
        "1204266003",  # Reconstruction of skin using local subcutaneous pedicle island random pattern skin flap (procedure)
        "1217220006",  # Direct laryngoscopy with injection into vocal cord (procedure)
        "1220601000",  # Tracheoscopy using rigid bronchoscope (procedure)
        "1220606005",  # Tracheoscopy and biopsy of lesion of trachea using fiberoptic bronchoscope (procedure)
        "122867002",  # Excision of cervical rib (procedure)
        "1255915001",  # Implantation of osseointegrated implant into bone of jaw (procedure)
        "1287750007",  # Reconstruction of head using distant pedicle skin flap (procedure)
        "1287752004",  # Reconstruction of neck using distant pedicle skin flap (procedure)
        "1287753009",  # Final inset of flap of skin to head (procedure)
        "1287754003",  # Final inset of flap of skin to neck (procedure)
        "1287755002",  # Local sensory skin flap to head (procedure)
        "1287756001",  # Local sensory skin flap to neck (procedure)
        "1287757005",  # Neurovascular island sensory skin flap to head (procedure)
        "1287758000",  # Neurovascular island sensory skin flap to neck (procedure)
        "1290136005",  # Z-plasty of skin of head (procedure)
        "1290137001",  # Z-plasty of skin of neck (procedure)
        "1303763001",  # Reconstruction of head and/or neck with mucosal graft (procedure)
        "1303805000",  # Construction of stoma of trachea (procedure)
        "1329005",  # Division of congenital web of larynx (procedure)
        "13392003",  # Transplantation of larynx (procedure)
        "13475006",  # Aspiration of diverticulum of pharynx (procedure)
        "14038001",  # Autologous graft of rib cartilage to face (procedure)
        "1417002",  # Operation on face (procedure)
        "14390001000004103",  # Reconstruction of mouth using pectoralis major myocutaneous flap (procedure)
        "14545007",  # Obliteration of maxillary sinus (procedure)
        "14681002",  # Reimplantation of lung (procedure)
        "15023006",  # Thromboendarterectomy with graft of carotid artery by neck incision (procedure)
        "15292003",  # Subtotal supraglottic laryngectomy with radical neck dissection (procedure)
        "15593007",  # Cauterization of pharynx (procedure)
        "15774007",  # Maxillary sinus endoscopy with removal of mucous membrane and polyps (procedure)
        "16508004",  # Division of congenital web of pharynx (procedure)
        "16877002",  # Attachment of pedicle graft to lip (procedure)
        "169367007",  # Radiotherapy: temporary tongue implant (procedure)
        "171579009",  # Intracranial transection of glossopharyngeal nerve (IX) (procedure)
        "171580007",  # Intracranial transection of vagus nerve (X) (procedure)
        "171603003",  # Intracranial destruction of glossopharyngeal nerve (IX) (procedure)
        "171604009",  # Intracranial destruction of vagus nerve (X) (procedure)
        "171617007",  # Excision of lesion of glossopharyngeal nerve (IX) (procedure)
        "171618002",  # Excision of lesion of vagus nerve (X) (procedure)
        "171630008",  # Repair of glossopharyngeal nerve (IX) (procedure)
        "171631007",  # Repair of vagus nerve (X) (procedure)
        "171644002",  # Intracranial stereotactic neurolysis of glossopharyngeal nerve (IX) (procedure)
        "171645001",  # Intracranial stereotactic neurolysis of vagus nerve (X) (procedure)
        "171659008",  # Decompression of glossopharyngeal nerve (IX) (procedure)
        "171660003",  # Decompression of vagus nerve (X) (procedure)
        "171679000",  # Exploration of glossopharyngeal nerve (IX) (procedure)
        "171680002",  # Exploration of vagus nerve (X) (procedure)
        "17200002",  # Ventricular shunt to nasopharynx (procedure)
        "172871001",  # Reconstruction of maxilla with local flap (procedure)
        "172872008",  # Reconstruction of maxilla with distant flap (procedure)
        "172873003",  # Reconstruction of maxilla with microvascular transferred flap (procedure)
        "172880001",  # Surgical procedure on frontal sinus (procedure)
        "172886007",  # Osteoplastic frontal flap operation (procedure)
        "172888008",  # Radical frontal sinus antrostomy (procedure)
        "172892001",  # Operation on sphenoid sinus (procedure)
        "172924008",  # Functional endoscopic sinus surgery - anterior ethmoidectomy and frontal recess dissection (procedure)
        "172950007",  # Reconstruction of pharynx with distant pedicle flap (procedure)
        "172952004",  # Reconstruction of pharynx with free jejunal transfer (procedure)
        "172953009",  # Reconstruction of pharynx with stomach pull-up (procedure)
        "172954003",  # Reconstruction of pharynx with colon pull-up (procedure)
        "172958000",  # Open excision of lesion of pharynx (procedure)
        "172965008",  # Endoscopic extirpation of lesion of nasopharynx (procedure)
        "172967000",  # Endoscopic removal of foreign body from pharynx (procedure)
        "172982005",  # Hemostasis of pharynx (procedure)
        "172991009",  # Laryngofissure and cordectomy of vocal cord (procedure)
        "172996004",  # Open extirpation of lesion of larynx (procedure)
        "172997008",  # Excision of lesion of larynx using thyrotomy as approach (procedure)
        "172998003",  # Excision of lesion of larynx using lateral pharyngotomy as approach (procedure)
        "172999006",  # Open destruction of lesion of larynx (procedure)
        "173000007",  # Removal of granulation tissue from larynx (procedure)
        "173006001",  # Implantation of artificial voicebox into larynx (procedure)
        "173012006",  # Open biopsy of lesion of larynx (procedure)
        "173013001",  # Chondroplasty of larynx (procedure)
        "173018005",  # Microtherapeutic endoscopic operations on larynx (procedure)
        "173019002",  # Microtherapeutic endoscopic extirpation of lesion of larynx using laser (procedure)
        "173027006",  # Endoscopic resection of lesion of larynx (procedure)
        "173028001",  # Endoscopic destruction of lesion of larynx (procedure)
        "173029009",  # Endoscopic removal of prosthesis from larynx (procedure)
        "173050000",  # Partial excision of trachea (procedure)
        "173055005",  # Reconstruction of trachea using graft (procedure)
        "173057002",  # Reconstruction of trachea with skin flap (procedure)
        "173061008",  # Open placement of prosthesis in trachea (procedure)
        "173075001",  # Open destruction of lesion of trachea (procedure)
        "173082002",  # Rigid bronchoscopy and removal of foreign body from trachea (procedure)
        "173083007",  # Rigid bronchoscopy and snare resection of lesion of trachea (procedure)
        "173084001",  # Rigid bronchoscopic laser destruction of lesion of trachea (procedure)
        "173085000",  # Rigid bronchoscopic aspiration of trachea (procedure)
        "173086004",  # Rigid bronchoscopic lavage of trachea (procedure)
        "173094006",  # Open procedure to the carina (procedure)
        "173111001",  # Rigid bronchoscopic laser destruction of lesion below trachea (procedure)
        "173113003",  # Rigid bronchoscopic aspiration below trachea using rigid bronchoscope (procedure)
        "173117002",  # Rigid bronchoscopy and snare resection of lesion below trachea (procedure)
        "173121009",  # Bronchoscopy and biopsy below trachea using rigid bronchoscope (procedure)
        "173138008",  # Fiberoptic bronchoscopic snare resection of lesion of trachea (procedure)
        "173139000",  # Fiberoptic endoscopic laser destruction of lesion of trachea (procedure)
        "173140003",  # Fiberoptic endoscopic destruction of lesion of trachea (procedure)
        "173141004",  # Fiberoptic endoscopic aspiration of trachea (procedure)
        "173142006",  # Fiberoptic endoscopic removal of foreign body from trachea (procedure)
        "173143001",  # Fiberoptic bronchoscopic lavage of trachea (procedure)
        "173152005",  # Fiberoptic bronchoscopic snare resection of lesion below trachea (procedure)
        "173153000",  # Fiberoptic bronchoscopic laser destruction of lesion below trachea (procedure)
        "173154006",  # Fiberoptic bronchoscopic destruction of lesion below trachea (procedure)
        "173155007",  # Fiberoptic endoscopic aspiration below trachea (procedure)
        "173170008",  # Bilobectomy of lung (procedure)
        "173171007",  # Lobectomy of lung (procedure)
        "173172000",  # Excision of segment of lung (procedure)
        "173178001",  # Open excision of lesion of lung (procedure)
        "173198006",  # Open excision of lesion of mediastinum (procedure)
        "173250007",  # Reconstruction of lip with tongue flap (procedure)
        "173337009",  # Excision of dental lesion of jaw (procedure)
        "173352002",  # Tongue and palate operations (procedure)
        "173357008",  # Removal of lesion of tongue (procedure)
        "173358003",  # Excision of lesion of tongue (procedure)
        "173359006",  # Cryotherapy to lesion of tongue (procedure)
        "173360001",  # Cauterization of lesion of tongue (procedure)
        "173365006",  # Removal of foreign body from tongue (procedure)
        "173371000",  # Commissurectomy of tongue (procedure)
        "173372007",  # Release of transfixion of tongue (procedure)
        "173373002",  # Reconstruction of tongue with local flap (procedure)
        "173374008",  # Reconstruction of tongue with distant flap (procedure)
        "173375009",  # Reconstruction of tongue with microvascular transferred flap (procedure)
        "173376005",  # Freeing of adhesion of tongue with Z-plasty (procedure)
        "173394004",  # Repair of cleft soft palate with intra-velar veloplasty (procedure)
        "173401003",  # Repair of defect of palate with tongue flap (procedure)
        "173445003",  # Destruction of lesion of floor of mouth (procedure)
        "173447006",  # Cryotherapy of lesion of floor of mouth (procedure)
        "173448001",  # Cauterization of lesion of floor of mouth (procedure)
        "173454000",  # Reconstruction of mouth defect using local pedicle flap (procedure)
        "173455004",  # Reconstruction of mouth defect using distant pedicle flap (procedure)
        "173456003",  # Reconstruction of mouth defect with free myocutaneous flap (procedure)
        "173457007",  # Reconstruction of mouth defect with fasciocutaneous flap (procedure)
        "173458002",  # Reconstruction of mouth defect with free bowel flap (procedure)
        "173459005",  # Reconstruction of mouth defect with free omental flap (procedure)
        "173460000",  # Direct closure of mouth defect (procedure)
        "174802006",  # Allotransplant of heart and lung (procedure)
        "174809002",  # Heterotopic allotransplant of heart (procedure)
        "17488008",  # Irrigation by cannulation of sphenoid sinus (procedure)
        "17556002",  # Open reduction of closed depressed frontal sinus fracture (procedure)
        "176239003",  # Excision of tissue of tongue (procedure)
        "177361001",  # Distant subcutaneous pedicle flap, myocutaneous, to head or neck (procedure)
        "177372006",  # Distant subcutaneous pedicle flap, fasciocutaneous, to head or neck (procedure)
        "177392004",  # Distant subcutaneous pedicle flap, random pattern, to head or neck (procedure)
        "177423003",  # Local subcutaneous pedicle island flap, myocutaneous, to head or neck (procedure)
        "177439005",  # Local subcutaneous pedicle island flap, fasciocutaneous, to head or neck (procedure)
        "177464008",  # Local transposition flap, axial pattern, to head or neck (procedure)
        "177473000",  # Local rotation flap, random pattern, to head or neck (procedure)
        "177483001",  # Revision of flap of skin to head or neck (procedure)
        "177485008",  # Thinning of flap of skin to head or neck (procedure)
        "177486009",  # Removal of flap of skin to head or neck (procedure)
        "177540009",  # Allograft of amniotic membrane to head or neck (procedure)
        "177599009",  # Insertion of skin expander into subcutaneous tissue of head or neck (procedure)
        "177605002",  # Removal of skin expander from subcutaneous tissue of head or neck (procedure)
        "177751006",  # Partial excision of chest wall (procedure)
        "177761004",  # Reconstruction of chest wall with distant flap (procedure)
        "177789001",  # Excision of rib and open drainage of pleural cavity (procedure)
        "178283002",  # Block dissection of cervical lymph nodes (procedure)
        "178287001",  # Supraomohyoid lymph nodes neck dissection (procedure)
        "178407001",  # Intracranial osteotomy of bone of face (procedure)
        "178408006",  # Transorbital subcranial osteotomy of bone of face (procedure)
        "178422006",  # Partial maxillectomy (procedure)
        "178463000",  # Intermaxillary fixation of maxilla (procedure)
        "178468009",  # Internal fixation of bone of face with plating system (procedure)
        "178469001",  # Internal fixation of bone of face with wire (procedure)
        "178470000",  # Fixation of bone of face - external (procedure)
        "178503004",  # Intra-articular arthroplasty of temporomandibular joint (procedure)
        "178505006",  # Total prosthetic replacement of temporomandibular joint (procedure)
        "17962001",  # Radical resection of tumor of soft tissue of neck (procedure)
        "179632003",  # Closed reduction of dislocation of temporomandibular joint (procedure)
        "179835003",  # Release of webbing of neck (procedure)
        "18044005",  # Endoscopy of trachea (procedure)
        "185173003",  # Implant of inert material into vocal cords (procedure)
        "186160005",  # Implant of vocal cords (procedure)
        "18728007",  # Open reduction of open depressed frontal sinus fracture (procedure)
        "19029000",  # Closure of fistula of vocal cord (procedure)
        "19060000",  # Cauterization of vocal cords (procedure)
        "192703003",  # Transection of uvula (procedure)
        "19297001",  # Manipulation of tongue (procedure)
        "19479007",  # Reconstruction of head and neck using muscle flap (procedure)
        "20153000",  # Removal of foreign body from pharynx by pharyngotomy (procedure)
        "20632008",  # Removal of foreign body from pharynx (procedure)
        "20737007",  # Removal of keel, tantalum plate or stent from larynx (procedure)
        "21034004",  # Wedge resection of tongue (procedure)
        "21287008",  # Incision and drainage of lesion of tongue (procedure)
        "21320005",  # Manipulation of accessory sinus (procedure)
        "21400005",  # Transection of vagus nerve by abdominal approach (procedure)
        "22709009",  # Reduction of fracture of larynx (procedure)
        "22960002",  # Incision of fascial compartments of neck (procedure)
        "23054008",  # Maxillary sinus endoscopy with removal of foreign body (procedure)
        "230958000",  # Division of glossopharyngeal nerve in tonsillar fossa (procedure)
        "23136006",  # Operation on pharynx (procedure)
        "232501001",  # Reconstruction of maxilla (procedure)
        "232508007",  # Intranasal frontal recess dissection (procedure)
        "232513006",  # Obliteration of frontal sinus (procedure)
        "232532000",  # Hyoid bone expansion (procedure)
        "232533005",  # Hyoid bone suspension (procedure)
        "232551006",  # Displacement of vocal cord with biological graft (procedure)
        "232555002",  # Displacement of vocal cord by inert substance (procedure)
        "232561004",  # Excision of mucosal lesion of vocal cord (procedure)
        "232562006",  # Excision of vocal cord nodule (procedure)
        "232571002",  # Biopsy of lesion of larynx (procedure)
        "232573004",  # Removal of prosthesis from larynx (procedure)
        "232574005",  # Removal of foreign body from larynx (procedure)
        "232604000",  # Open procedure to the trachea (procedure)
        "232611001",  # Repair of traumatic rupture of the trachea (procedure)
        "232632009",  # Wedge excision of lung (procedure)
        "232633004",  # Single wedge excision of lung (procedure)
        "232634005",  # Multiple wedge excisions of lung (procedure)
        "232635006",  # Excision of segment of right upper lobe (procedure)
        "232636007",  # Excision of segment of middle lobe of right lung (procedure)
        "232637003",  # Excision of segment of right lower lobe (procedure)
        "232638008",  # Excision of segment of left upper lobe (procedure)
        "232639000",  # Excision of segment of left lower lobe (procedure)
        "232640003",  # Lobectomy with bronchoplastic procedure (procedure)
        "232641004",  # Lobectomy of lung with excision of part of adjacent lobe (procedure)
        "232642006",  # Extrapericardial pneumonectomy (procedure)
        "232643001",  # Intrapericardial pneumonectomy (procedure)
        "232644007",  # Extrapleural pneumonectomy (procedure)
        "232646009",  # Completion pneumonectomy (procedure)
        "232647000",  # Pneumonectomy with tracheobronchoplasty (procedure)
        "232648005",  # Excision of lung sequestration (procedure)
        "232657004",  # Single lung transplant (procedure)
        "232658009",  # Double lung transplant (procedure)
        "232659001",  # Double lung transplant as a block (procedure)
        "232660006",  # Bilateral sequential single lung transplant (procedure)
        "232668004",  # Tongue traction (procedure)
        "232973007",  # Allotransplant of heart (procedure)
        "23327003",  # Full thickness graft of skin to skin of lip and mouth (procedure)
        "234245003",  # Extended radical neck dissection of cervical lymph nodes (procedure)
        "234246002",  # Selective neck dissection of cervical lymph nodes (procedure)
        "234247006",  # Posterolateral lymph nodes neck dissection (procedure)
        "234248001",  # Lateral lymph nodes neck dissection (procedure)
        "234249009",  # Anterior compartment lymph nodes neck dissection (procedure)
        "234663009",  # Implantation into jaw (procedure)
        "234665002",  # Submucosal implantation into jaw (procedure)
        "234667005",  # Expose jaw implant (procedure)
        "234668000",  # Remove jaw implant (procedure)
        "234818006",  # Insertion of mouth held appliance (procedure)
        "234891004",  # Repair of cleft of hard palate (procedure)
        "234892006",  # Repair of cleft of soft palate (procedure)
        "234898005",  # Removal of lesion of floor of mouth (procedure)
        "234899002",  # Incisional biopsy of lesion of floor of mouth (procedure)
        "234906001",  # Reconstruction of mouth using flap (procedure)
        "234907005",  # Reconstruction of mouth using local pedicle flap (procedure)
        "234908000",  # Reconstruction of mouth using distant pedicle flap (procedure)
        "234909008",  # Reconstruction of mouth defect using graft (procedure)
        "234910003",  # Graft of skin to mouth (procedure)
        "234911004",  # Graft of mucosa to mouth (procedure)
        "234912006",  # Revision of repair of mouth (procedure)
        "234913001",  # Primary closure of mouth defect (procedure)
        "234917000",  # Primary repair of mouth wound (procedure)
        "234929009",  # Removal of foreign body from floor of mouth (procedure)
        "239373002",  # Enucleation of lesion of bone of face (procedure)
        "239375009",  # Marsupialization of lesion of bone of face (procedure)
        "239380000",  # Packing of wound of bone of face (procedure)
        "239382008",  # Removal of pack from bone of face (procedure)
        "239388007",  # Fixation of maxilla - external (procedure)
        "239389004",  # Fixation of bone of face - internal (procedure)
        "239392000",  # Removal of graft from bone of face (procedure)
        "239449009",  # Surgical manipulation of temporomandibular joint (procedure)
        "239532004",  # Meniscoplasty of temporomandibular joint (procedure)
        "239533009",  # Eminectomy temporomandibular joint (procedure)
        "239534003",  # Exploration of temporomandibular joint (procedure)
        "239538000",  # Plication of capsule of temporomandibular joint (procedure)
        "239539008",  # Eminence augmentation temporomandibular joint (procedure)
        "243753004",  # Provision of mouth opening appliance (procedure)
        "24447002",  # Layer closure of wounds of face and ears (procedure)
        "25321000",  # Thoracoscopic pneumonectomy (procedure)
        "261483009",  # Anastomosis of both main bronchi to end of trachea (procedure)
        "261486001",  # Anastomosis of left main bronchus to end of trachea (procedure)
        "261488000",  # Anastomosis of left main bronchus to side of trachea (procedure)
        "261490004",  # Anastomosis of right main bronchus to end of trachea (procedure)
        "261491000",  # Anastomosis of right main bronchus to side of trachea (procedure)
        "261531000",  # Excision of rib (procedure)
        "265332004",  # Repair of cleft hard palate with posteriorly based axial transposition flap (procedure)
        "265333009",  # Repair of cleft hard palate with bipedicled flaps (procedure)
        "265334003",  # Repair of cleft soft palate with Z-plasty (procedure)
        "265663008",  # Distant subcutaneous pedicle flap, axial pattern, to head or neck (procedure)
        "265670008",  # Local transposition flap, random pattern, to head or neck (procedure)
        "265703000",  # Jaw and temporomandibular joint operations (procedure)
        "265706008",  # Fixation of bone of face (procedure)
        "265707004",  # Removal of fixation from bone of face (procedure)
        "265708009",  # Division of jaw (procedure)
        "271408003",  # Open biopsy of lesion of pharynx (procedure)
        "27210007",  # Laryngopharyngoesophagectomy with radical neck dissection and synchronous thyroidectomy (procedure)
        "27271000",  # Lysis of adhesions of choanae of nasopharynx (procedure)
        "274317003",  # Laryngoscopic biopsy larynx (procedure)
        "274342007",  # Surgical biopsy of tongue (procedure)
        "274347001",  # Surgical biopsy of pharynx (procedure)
        "275189005",  # Repair of nasopharynx (procedure)
        "27926003",  # Construction of artificial larynx (procedure)
        "28228007",  # Transthoracic avulsion of vagus nerve (procedure)
        "282365006",  # Incisional biopsy of lesion of tongue (procedure)
        "282366007",  # Excisional biopsy of lesion of tongue (procedure)
        "282435000",  # Maxillectomy (procedure)
        "284063007",  # Biopsy of lesion of floor of mouth (procedure)
        "28618007",  # Radical excision of floor of mouth (procedure)
        "287430001",  # Excision of lung or bronchus (procedure)
        "287432009",  # Trachea resected and anastomosed (situation)
        "287433004",  # Reconstruction artificial larynx (procedure)
        "287437003",  # Larynx/trachea dilation (procedure)
        "287455007",  # Tongue benign neoplasm excised (situation)
        "3001009",  # Total lobectomy with bronchoplasty (procedure)
        "30120002",  # Excision of lesion of soft palate (procedure)
        "30142001",  # Destructive procedure on larynx (procedure)
        "30184009",  # Excision of lesion from frontal sinus (procedure)
        "302474008",  # Intermaxillary fixation of mandible and maxilla (procedure)
        "302476005",  # Fixation of maxilla - internal (procedure)
        "302477001",  # Osteotomy of maxilla (procedure)
        "302478006",  # Prosthetic replacement of temporomandibular joint (procedure)
        "303565004",  # Anastomosis of pharynx to esophagus (procedure)
        "303566003",  # Anastomosis of pharynx to stomach (procedure)
        "304810004",  # Operation on articular eminence of temporomandibular joint (procedure)
        "304811000",  # Operation on meniscus of temporomandibular joint (procedure)
        "307111006",  # Remove endosseous jaw implant (procedure)
        "307276000",  # Cryotherapy to mouth (procedure)
        "307277009",  # Laser surgery of mouth (procedure)
        "307660002",  # Remove blade jaw implant (procedure)
        "307822008",  # Endoscopic trachea biopsy (procedure)
        "307831008",  # Repair of cleft larynx (procedure)
        "307944004",  # Excision of mucous cyst of vocal cord (procedure)
        "308029000",  # Laryngoscopic vocal cord polypectomy (procedure)
        "308030005",  # Microlaryngoscopic vocal cord polypectomy (procedure)
        "30963003",  # Transglottic catheterization of trachea (procedure)
        "309812005",  # Extubation of trachea (procedure)
        "310216004",  # Collection of sample from mouth (procedure)
        "310657004",  # Open silastic insert to vocal cord (procedure)
        "31232000",  # Biopsy of trachea (procedure)
        "31461001",  # Transection of vagus nerve by transthoracic approach (procedure)
        "315302009",  # Reconstruction of maxilla with flap (procedure)
        "31740007",  # Arthroplasty of temporomandibular joint with autograft (procedure)
        "32368006",  # Lysis of adhesions of larynx (procedure)
        "32413006",  # Transplantation of heart (procedure)
        "32477003",  # Heart-lung transplant with recipient cardiectomy-pneumonectomy (procedure)
        "33276001",  # Drainage of fascial compartments of neck (procedure)
        "332856005",  # Aspiration of trachea with lavage (procedure)
        "333837005",  # Lavage of trachea (procedure)
        "33639009",  # Segmental lobectomy with concomitant decortication (procedure)
        "33932003",  # Replacement of artificial heart (procedure)
        "33951009",  # Excision of lesion of trachea (procedure)
        "33988007",  # Excision of lesion of vestibule of mouth with simple repair (procedure)
        "342746004",  # Tongue graft (procedure)
        "35371005",  # Lysis of adhesions of pharynx (procedure)
        "3580004",  # Removal of calculus of pharynx (procedure)
        "359556005",  # Excisional biopsy of tongue (procedure)
        "359615001",  # Partial lobectomy of lung (procedure)
        "359618004",  # Segmental lobectomy (procedure)
        "359623004",  # Total lobectomy of lung (procedure)
        "359697005",  # Phonosurgery for larynx lesion (procedure)
        "359701008",  # Excision of focal lesion of larynx (procedure)
        "360768009",  # Repair of pharynx by direct closure (procedure)
        "361247001",  # Excision of contact ulcer of vocal cord (procedure)
        "361248006",  # Excision of intubation granuloma of larynx (procedure)
        "367439007",  # Repair of trachea (procedure)
        "373824001",  # Posterior lymph nodes neck dissection (procedure)
        "3786007",  # Excision of lesion of pharynx (procedure)
        "384676001",  # Operation on mouth (procedure)
        "386668000",  # Plastic repair of mouth (procedure)
        "389059002",  # Reconstruction of pharynx (procedure)
        "391929006",  # Suture of trachea (procedure)
        "392102003",  # Endoscopy of pharynx (procedure)
        "392168001",  # Revision of stoma of trachea (procedure)
        "392169009",  # Repair of stoma of trachea (procedure)
        "392213008",  # Sphenoid sinusotomy (procedure)
        "392215001",  # Drainage of sphenoid sinus (procedure)
        "392218004",  # Puncture of sphenoid sinus (procedure)
        "39464006",  # Removal of foreign body from maxillary sinus (procedure)
        "395169002",  # Superior mediastinal lymph nodes neck dissection (procedure)
        "395199006",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve, sternocleidomastoid muscle and internal jugular vein (procedure)
        "395200009",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve and internal jugular vein (procedure)
        "395201008",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve (procedure)
        "397740001",  # Cannulation of trachea (procedure)
        "40051009",  # Repair of trachea by intrathoracic approach (procedure)
        "40203007",  # Removal of foreign body from ethmoid sinus (procedure)
        "40328001",  # Exteriorization of maxillary sinus (procedure)
        "40583006",  # Resection of trachea (procedure)
        "40902008",  # Segmental lobectomy with bronchoplasty (procedure)
        "40916003",  # Dilation of trachea (procedure)
        "41223000",  # Pharyngolaryngectomy with radical neck dissection and reconstruction (procedure)
        "41535000",  # Maxillary sinus endoscopy with removal of mucous membrane (procedure)
        "41848005",  # Direct laryngoscopy with stripping of epiglottis (procedure)
        "420005003",  # Fluoroscopy of trachea (procedure)
        "42299007",  # Fascial sling of tongue (procedure)
        "42316002",  # Excision of lesion of maxillary sinus by Caldwell-Luc approach (procedure)
        "42478006",  # Incision of tongue (procedure)
        "426308002",  # Endoscopic vocal cord medialization (procedure)
        "426723009",  # Repair of sphenoidal sinus (procedure)
        "427270000",  # Median drainage of frontal sinus (procedure)
        "428894007",  # Osteoplasty of frontal sinus (procedure)
        "429332008",  # Transplantation of single lobe of lung (procedure)
        "43071001",  # Excision of lesion of maxillary sinus (procedure)
        "430874007",  # Reduction of dislocation of temporomandibular joint (procedure)
        "431066004",  # Biopsy of vocal cord (procedure)
        "43126007",  # Reconstruction of temporomandibular joint (procedure)
        "431333009",  # Insertion of implant into soft palate (procedure)
        "431835009",  # Dilation of trachea using fluoroscopy guidance (procedure)
        "43197005",  # Destructive procedure on mouth AND lips (procedure)
        "43232006",  # Extrapleural pneumonectomy with empyemectomy (procedure)
        "432364009",  # Endoscopic dilation of trachea using fluoroscopic guidance (procedure)
        "433014002",  # Biopsy of larynx (procedure)
        "43508000",  # Excision of lesion of mediastinum (procedure)
        "43578004",  # Arthrotomy of temporomandibular joint (procedure)
        "4363008",  # Endoscopic brush biopsy of trachea (procedure)
        "43668008",  # Resection of soft palate (procedure)
        "438564001",  # Excision of mucosa of vestibule of mouth (procedure)
        "439171005",  # Laparoscopic selective transection of vagus nerve (procedure)
        "439783004",  # Division of maxilla (procedure)
        "440111005",  # Laparoscopic transection of trunk of vagus nerve (procedure)
        "440198000",  # Osteotomy of jaw (procedure)
        "440303005",  # Suture of tongue to lip for micrognathia (procedure)
        "442117004",  # Repair of temporomandibular joint (procedure)
        "44272006",  # Cordopexy of vocal cord (procedure)
        "443085007",  # Augmentation of soft tissue of face using expanded polytetrafluoroethylene implant (procedure)
        "443141008",  # Repair and augmentation of bone of face using allogeneic bone graft (procedure)
        "443184009",  # Repair and augmentation of bone of face using autogenous bone graft (procedure)
        "443185005",  # Repair and augmentation of bone of face using prosthetic implant (procedure)
        "443521008",  # Direct laryngoscopy with stripping of epiglottis using operating microscope (procedure)
        "443691008",  # Direct laryngoscopy with stripping of vocal cords using operating microscope (procedure)
        "444188003",  # Thoracoscopic wedge resection of lung (procedure)
        "44424007",  # Excision of lesion of ethmoid sinus (procedure)
        "444665003",  # Resection and anastomosis of trachea (procedure)
        "445319001",  # Insertion of vagal nerve stimulator (procedure)
        "445726003",  # Endoscopic excision of sphenoid sinus (procedure)
        "446066003",  # Destruction of glossopharyngeal nerve (procedure)
        "446256001",  # Removal of vagal nerve stimulator (procedure)
        "446799007",  # Wedge biopsy of tongue (procedure)
        "446808004",  # Total excision of maxilla and excision of nasal turbinate (procedure)
        "446854008",  # Excision of sphenoid sinus and nasal turbinate (procedure)
        "447140005",  # Excision of frontal sinus and nasal turbinate (procedure)
        "447141009",  # Endoscopic excision of ethmoid sinus (procedure)
        "447326000",  # Endoscopic incision of ethmoid sinus (procedure)
        "447327009",  # Endoscopic dilation of pharynx (procedure)
        "447328004",  # Endoscopic incision of sphenoid sinus (procedure)
        "447417004",  # Excision of ethmoid sinus and nasal turbinate (procedure)
        "447433001",  # Endoscopic excision of sphenoid sinus and nasal turbinate (procedure)
        "447599003",  # Endoscopic excision of ethmoid sinus and nasal turbinate (procedure)
        "447652004",  # Excision of nodule of larynx (procedure)
        "447741002",  # Suturing of laceration of trachea by transcervical approach (procedure)
        "447752008",  # Excision of cyst of epiglottis (procedure)
        "447794006",  # Removal of stent from larynx (procedure)
        "447984009",  # Excision of cyst of vallecula of epiglottis using laser (procedure)
        "447985005",  # Excision of lesion of larynx using laser (procedure)
        "448034002",  # Excision of nodule of vocal cord using laser (procedure)
        "448035001",  # Excision of polyp of larynx using laser (procedure)
        "448168006",  # Intracavitary brachytherapy of nasopharynx (procedure)
        "448199007",  # Centesis of temporomandibular joint (procedure)
        "448368002",  # Suturing of laceration of trachea by intrathoracic approach (procedure)
        "448369005",  # Pharyngolaryngoesophagectomy and radical neck dissection (procedure)
        "448460005",  # Excision of cyst of larynx (procedure)
        "448502003",  # Arthroscopy of temporomandibular joint (procedure)
        "448753000",  # Rigid bronchoscopic destruction of lesion of trachea (procedure)
        "448769007",  # Microtherapeutic endoscopic excision of lesion of larynx (procedure)
        "449449005",  # Excision of nodule of larynx using laser (procedure)
        "44972003",  # Endoscopy of larynx through stoma (procedure)
        "449865004",  # Division of web of larynx (procedure)
        "452031000124100",  # Transplantation of face (procedure)
        "454511000124109",  # Augmentation of maxillary sinus (procedure)
        "455251000124107",  # Thoracoscopic surgical lobectomy using video-assisted guidance (procedure)
        "45685007",  # Radical neck dissection (procedure)
        "45827003",  # Reconstruction of head and neck using fasciocutaneous flap (procedure)
        "46034007",  # Suture of laceration of pharynx (procedure)
        "4671001",  # Excision of lesion from sphenoid sinus (procedure)
        "46748004",  # Elevation of orbital bone fragments of maxillary sinus (procedure)
        "47058000",  # Heart transplant with recipient cardiectomy (procedure)
        "47298009",  # Biopsy of uvula and soft palate (procedure)
        "473275008",  # Endoscopic control of hemorrhage of pharynx (procedure)
        "47935000",  # Cordectomy of vocal cord (procedure)
        "48032002",  # Local destruction of lesion of trachea (procedure)
        "48387007",  # Incision of trachea (procedure)
        "48527009",  # Suture of pharynx (procedure)
        "49109005",  # Fascial sling of mouth (procedure)
        "49319008",  # Rhytidectomy of neck with platysmal tightening (procedure)
        "49795001",  # Total pneumonectomy (procedure)
        "50339005",  # Complete laryngectomy with radical neck dissection, synchronous thyroidectomy and synchronous tracheostomy (procedure)
        "51138002",  # Laser beam destruction of lesion of vestibule of mouth (procedure)
        "52176005",  # Aspiration of trachea, percutaneous (procedure)
        "52728000",  # Removal of foreign body of larynx by incision (procedure)
        "53580003",  # Marsupialization of cyst of larynx (procedure)
        "53736000",  # Repair of epiglottis (procedure)
        "54029002",  # Excision of lesion from floor of mouth (procedure)
        "54603001",  # Removal of intraluminal foreign body from pharynx without incision (procedure)
        "56657006",  # Coagulation of lesion of pharynx using electrical energy (procedure)
        "59085008",  # Modified radical dissection of bilateral sides of neck (procedure)
        "59355008",  # Irrigation by cannulation of maxillary sinus (procedure)
        "59378002",  # Destructive procedure of nose and nasopharynx (procedure)
        "59607002",  # Removal of foreign body from frontal sinus (procedure)
        "59687004",  # Excision of polyp of larynx (procedure)
        "609247004",  # Removal of intraluminal foreign body from trachea without incision (procedure)
        "609249001",  # Removal of intraluminal foreign body from larynx without incision (procedure)
        "609251002",  # Removal of device from neck (procedure)
        "61520005",  # Total osteoplasty of maxilla (procedure)
        "61561005",  # Incision of larynx (procedure)
        "61877000",  # Closure of internal or complex fistula of trachea (procedure)
        "62511003",  # Autotransplant of lung (procedure)
        "63820007",  # Biopsy of floor of mouth (procedure)
        "63971008",  # Dilation of larynx (procedure)
        "64145001",  # Maxillary sinus endoscopy with removal of fungus ball (procedure)
        "6511000179106",  # Microsurgery of larynx (procedure)
        "65128001",  # Revision of stoma of larynx (procedure)
        "6521000179101",  # Microsurgery of larynx using tungsten microelectrode (procedure)
        "65310005",  # Vagotomy (procedure)
        "6562000",  # Destruction of lesion of tongue (procedure)
        "65635008",  # Reconstruction of head and neck using myocutaneous flap (procedure)
        "65942005",  # Removal of impacted tooth from maxillary sinus (procedure)
        "6601000179108",  # Endoscopy of pharynx, trachea and esophagus (procedure)
        "66315005",  # Dilation of pharynx (procedure)
        "66478002",  # Total laryngectomy (procedure)
        "665001",  # Partial meniscectomy of temporomandibular joint (procedure)
        "6670003",  # Closure of external fistula of trachea (procedure)
        "66861000",  # Intubation of larynx (procedure)
        "67116005",  # Repair of trachea by cervical approach (procedure)
        "67818008",  # Operation on nasopharynx (procedure)
        "67949002",  # Maxillary sinus endoscopy with biopsy (procedure)
        "6818001",  # Excision of buccal mucosa (procedure)
        "68431006",  # Grafting of trachea (procedure)
        "68495004",  # Excision of tissue of mediastinum (procedure)
        "69394007",  # Maxillary sinus endoscopy with removal of cyst (procedure)
        "69539003",  # Direct laryngoscopy with stripping of vocal cords (procedure)
        "69572003",  # Glossopharyngeal neurotomy (procedure)
        "69809004",  # Excision of rib by cervical approach (procedure)
        "698853000",  # Plastic operation on pharynx (procedure)
        "698878001",  # Open biopsy of tongue (procedure)
        "69888009",  # Repair of pharynx (procedure)
        "698976001",  # Local myocutaneous flap to neck (procedure)
        "698978000",  # Local fasciocutaneous flap to neck (procedure)
        "698980006",  # Local flap of skin to neck (procedure)
        "69962007",  # Closure of fistula of mouth (procedure)
        "703715003",  # Plastic operation on trachea (procedure)
        "708019002",  # Displacement of vocal cord with non-biological graft (procedure)
        "708587003",  # Marsupialization of abscess of pharynx (procedure)
        "708588008",  # Marsupialization of abscess of nasopharynx (procedure)
        "708649001",  # Division of tendon of neck (procedure)
        "708737007",  # Stripping of larynx (procedure)
        "708789002",  # Suture of laceration of mouth (procedure)
        "708855009",  # Excision of lesion of buccal mucosa (procedure)
        "709083002",  # Radiofrequency ablation of soft palate (procedure)
        "709086005",  # Reconstruction of floor of mouth (procedure)
        "709087001",  # Drainage of floor of mouth (procedure)
        "709089003",  # Harvesting of buccal mucosa (procedure)
        "709092004",  # Radiofrequency ablation of root of tongue (procedure)
        "71031009",  # Excision of uvula and pharynx (procedure)
        "711501001",  # Removal of foreign body from head and neck using imaging guidance (procedure)
        "713037008",  # Replacement of larynx (procedure)
        "713499001",  # Excision of mucous membrane of inferior maxillary sinus (procedure)
        "715787006",  # Central compartment (level VI and VII) lymph node neck dissection (procedure)
        "718029003",  # Contralateral neck dissection (procedure)
        "722076003",  # Ipsilateral neck dissection (procedure)
        "7221002",  # Formation of tubed pedicle graft, head and neck (procedure)
        "72314008",  # Full thickness graft of skin to mouth (procedure)
        "726425007",  # Lobectomy of lower lobe of right lung (procedure)
        "726426008",  # Lobectomy of lower lobe of left lung (procedure)
        "726427004",  # Lobectomy of upper lobe of left lung (procedure)
        "726428009",  # Lobectomy of upper lobe of right lung (procedure)
        "726491007",  # Excision of bilateral vocal cord polyp (procedure)
        "73136008",  # Graft of skin to skin of lip and mouth (procedure)
        "735015005",  # Complete excision of left lung (procedure)
        "735016006",  # Complete excision of right lung (procedure)
        "736293004",  # Transfer of distant skin flap to neck (procedure)
        "736294005",  # Transfer of distant skin flap to head (procedure)
        "736488000",  # Closure of fistula of urethra with buccal mucosa graft (procedure)
        "73650002",  # Transposition of vocal cords (procedure)
        "736511004",  # Reconstruction of urethra with buccal mucosa graft (procedure)
        "736670004",  # Percutaneous embolization of neoplasm of face using fluoroscopic guidance with contrast (procedure)
        "736672007",  # Transcutaneous stimulation of cervical branch of vagus nerve (procedure)
        "736974006",  # Excision of middle lobe of right lung (procedure)
        "737597005",  # Allograft of skin to skin of neck (procedure)
        "738831000",  # Excision of left half of tongue (procedure)
        "738832007",  # Excision of right half of tongue (procedure)
        "739139005",  # Excision of middle and lower lobe of right lung (procedure)
        "739141006",  # Excision of middle and upper lobe of right lung (procedure)
        "741064009",  # Repair of epispadias using double faced onlay patch (procedure)
        "74253001",  # Arthroplasty of temporomandibular joint with allograft (procedure)
        "74321007",  # Maxillary sinus endoscopy with removal of polyps (procedure)
        "74966009",  # Operation on tracheobronchial tree (procedure)
        "74999006",  # Excision of thyroglossal cyst with resection of hyoid bone (procedure)
        "75369004",  # Manual reduction of open fracture of hyoid bone (procedure)
        "75935006",  # Thoracoscopic lobectomy of lung (procedure)
        "76056002",  # Radical dissection of bilateral sides of neck (procedure)
        "76315006",  # Punch resection of vocal cord (procedure)
        "7686002",  # Endoscopic removal of foreign body from larynx (procedure)
        "773292009",  # Insertion of stent into trachea using fluoroscopic guidance (procedure)
        "77464009",  # Submucous resection of vocal cord (procedure)
        "77688004",  # Cauterization of larynx (procedure)
        "78136005",  # Total osteotomy of maxilla (procedure)
        "783283000",  # Graft of skin to neck (procedure)
        "783284006",  # Full thickness autograft of skin to skin of neck (procedure)
        "783285007",  # Full thickness graft of skin to skin of neck (procedure)
        "783288009",  # Pinch graft of skin to skin of neck (procedure)
        "783293007",  # Cultured graft of skin to skin of neck (procedure)
        "783294001",  # Composite autograft of skin and soft tissues to neck (procedure)
        "783317005",  # Meshed split thickness autograft of skin to skin of neck (procedure)
        "783765009",  # Xenograft of skin to skin of neck (procedure)
        "78706005",  # Radical lymph node dissection of neck region (procedure)
        "789147006",  # Implantation of dental prosthesis into jaw (procedure)
        "78923006",  # Destruction of lesion of larynx (procedure)
        "80067004",  # Excision of lesion of vocal cord (procedure)
        "80239002",  # Biopsy of vestibule of mouth (procedure)
        "80694004",  # Repair of larynx (procedure)
        "80922008",  # Reconstruction of carina of trachea (procedure)
        "8144007",  # Removal of foreign body from trachea by incision (procedure)
        "81550005",  # Removal of foreign body from sphenoid sinus (procedure)
        "82658005",  # Incisional biopsy of larynx (procedure)
        "830081000168105",  # Repair of cartilage of larynx (procedure)
        "83356008",  # Excision of tissue of pharynx (procedure)
        "83634003",  # Excision of tumor of soft tissue of neck, deep, subfascial, intramuscular (procedure)
        "84691002",  # Removal of foreign body from maxillary sinus by Caldwell-Luc approach (procedure)
        "84840003",  # Total lobectomy of lung with concomitant decortication (procedure)
        "84848005",  # Endoscopy of tongue (procedure)
        "85154003",  # Total laryngectomy with radical neck dissection (procedure)
        "85327005",  # Transplantation of accessory sinus mucosa (procedure)
        "85357001",  # Excision of lesion of mouth (procedure)
        "85382001",  # Manipulation of joint adhesions of temporomandibular joint (procedure)
        "85910008",  # Repair of bone of accessory sinus (procedure)
        "860652003",  # Repair of hypospadias using double faced onlay patch (procedure)
        "86439006",  # Removal of foreign body from neck (procedure)
        "86800006",  # Operation on trachea (procedure)
        "87404002",  # Excision of meniscus of jaw bone (procedure)
        "87766003",  # Lysis of adhesions of trachea (procedure)
        "87829000",  # Incision and exploration of trachea (procedure)
        "88039007",  # Transplant of lung (procedure)
        "88098001",  # Cordotomy of vocal cord (procedure)
        "88517004",  # Incision of floor of mouth (procedure)
        "88623004",  # Excision of lesion of larynx (procedure)
        "88961004",  # Excision of lesion of nasopharynx (procedure)
        "90240003",  # Frontal sinusotomy with trephine (procedure)
        "90405002",  # Complete meniscectomy of temporomandibular joint (procedure)
        "90775002",  # Manual reduction of closed fracture of hyoid bone (procedure)
        "91571003",  # Bronchoscopic removal of intraluminal foreign body from trachea (procedure)
        "91596000",  # Complete excision of lung with mediastinal dissection (procedure)
        "9260006",  # Insertion of valved tube into larynx (procedure)
        "9373007",  # Stripping of vocal cords (procedure)
        "9552006",  # Transposition of tissue of tongue (procedure)
        "969009",  # Incision and exploration of larynx (procedure)
        "9774003",  # Manipulation of temporomandibular joint (procedure)
        "9811006",  # Tongue destructive procedure (procedure)
    }

class Intubation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for intubation

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent intubation

    **Exclusion Criteria:** N/A
    """

    VALUE_SET_NAME = "Intubation"
    OID = "2.16.840.1.113762.1.4.1248.179"
    DEFINITION_VERSION = "20220719"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0BH13EZ",  # Insertion of Endotracheal Airway into Trachea, Percutaneous Approach
        "0BH17EZ",  # Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening
        "0BH18EZ",  # Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening Endoscopic
    }

    SNOMEDCT = {
        "112798008",  # Insertion of endotracheal tube (procedure)
        "16883004",  # Endotracheal intubation, emergency procedure (procedure)
        "182682004",  # Emergency laryngeal intubation (procedure)
        "232674004",  # Orotracheal intubation (procedure)
        "232675003",  # Oral intubation awake (procedure)
        "232677006",  # Tracheal intubation using rigid bronchoscope (procedure)
        "232678001",  # Orotracheal fiberoptic intubation (procedure)
        "232679009",  # Nasotracheal intubation (procedure)
        "232680007",  # Nasal intubation awake (procedure)
        "232682004",  # Nasotracheal fiberoptic intubation (procedure)
        "232685002",  # Insertion of tracheostomy tube (procedure)
        "232686001",  # Insertion of tracheal T-tube (procedure)
        "304341005",  # Awake intubation (procedure)
        "418613003",  # Tracheal intubation through a laryngeal mask airway (procedure)
        "429161001",  # Insertion of endotracheal tube using laryngoscope (procedure)
    }

class MajorSurgicalProcedure(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for major surgical procedures.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes ICD-10 and SNOMED codes that represent major surgical procedures.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Major Surgical Procedure"
    OID = "2.16.840.1.113883.3.3157.4056"
    DEFINITION_VERSION = "20190216"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "00500ZZ",  # Destruction of Brain, Open Approach
        "00503Z4",  # Destruction of Brain using Stereoelectroencephalographic Radiofrequency Ablation, Percutaneous Approach
        "00510ZZ",  # Destruction of Cerebral Meninges, Open Approach
        "00520ZZ",  # Destruction of Dura Mater, Open Approach
        "00570ZZ",  # Destruction of Cerebral Hemisphere, Open Approach
        "00580ZZ",  # Destruction of Basal Ganglia, Open Approach
        "00590ZZ",  # Destruction of Thalamus, Open Approach
        "005A0ZZ",  # Destruction of Hypothalamus, Open Approach
        "005B0ZZ",  # Destruction of Pons, Open Approach
        "005C0ZZ",  # Destruction of Cerebellum, Open Approach
        "005D0ZZ",  # Destruction of Medulla Oblongata, Open Approach
        "00800ZZ",  # Division of Brain, Open Approach
        "00870ZZ",  # Division of Cerebral Hemisphere, Open Approach
        "00880ZZ",  # Division of Basal Ganglia, Open Approach
        "008Q0ZZ",  # Division of Vagus Nerve, Open Approach
        "009000Z",  # Drainage of Brain with Drainage Device, Open Approach
        "00900ZZ",  # Drainage of Brain, Open Approach
        "009100Z",  # Drainage of Cerebral Meninges with Drainage Device, Open Approach
        "00910ZZ",  # Drainage of Cerebral Meninges, Open Approach
        "009200Z",  # Drainage of Dura Mater with Drainage Device, Open Approach
        "00920ZZ",  # Drainage of Dura Mater, Open Approach
        "009300Z",  # Drainage of Intracranial Epidural Space with Drainage Device, Open Approach
        "00930ZZ",  # Drainage of Intracranial Epidural Space, Open Approach
        "009400Z",  # Drainage of Intracranial Subdural Space with Drainage Device, Open Approach
        "00940ZZ",  # Drainage of Intracranial Subdural Space, Open Approach
        "009500Z",  # Drainage of Intracranial Subarachnoid Space with Drainage Device, Open Approach
        "00950ZZ",  # Drainage of Intracranial Subarachnoid Space, Open Approach
        "00960ZZ",  # Drainage of Cerebral Ventricle, Open Approach
        "009700Z",  # Drainage of Cerebral Hemisphere with Drainage Device, Open Approach
        "00970ZZ",  # Drainage of Cerebral Hemisphere, Open Approach
        "009800Z",  # Drainage of Basal Ganglia with Drainage Device, Open Approach
        "00980ZZ",  # Drainage of Basal Ganglia, Open Approach
        "009900Z",  # Drainage of Thalamus with Drainage Device, Open Approach
        "00990ZZ",  # Drainage of Thalamus, Open Approach
        "009A00Z",  # Drainage of Hypothalamus with Drainage Device, Open Approach
        "009A0ZZ",  # Drainage of Hypothalamus, Open Approach
        "009B00Z",  # Drainage of Pons with Drainage Device, Open Approach
        "009B0ZZ",  # Drainage of Pons, Open Approach
        "009C00Z",  # Drainage of Cerebellum with Drainage Device, Open Approach
        "009C0ZZ",  # Drainage of Cerebellum, Open Approach
        "009D00Z",  # Drainage of Medulla Oblongata with Drainage Device, Open Approach
        "009D0ZZ",  # Drainage of Medulla Oblongata, Open Approach
        "00B00ZX",  # Excision of Brain, Open Approach, Diagnostic
        "00B00ZZ",  # Excision of Brain, Open Approach
        "00B03ZX",  # Excision of Brain, Percutaneous Approach, Diagnostic
        "00B03ZZ",  # Excision of Brain, Percutaneous Approach
        "00B04ZX",  # Excision of Brain, Percutaneous Endoscopic Approach, Diagnostic
        "00B04ZZ",  # Excision of Brain, Percutaneous Endoscopic Approach
        "00B10ZX",  # Excision of Cerebral Meninges, Open Approach, Diagnostic
        "00B10ZZ",  # Excision of Cerebral Meninges, Open Approach
        "00B13ZX",  # Excision of Cerebral Meninges, Percutaneous Approach, Diagnostic
        "00B13ZZ",  # Excision of Cerebral Meninges, Percutaneous Approach
        "00B14ZX",  # Excision of Cerebral Meninges, Percutaneous Endoscopic Approach, Diagnostic
        "00B14ZZ",  # Excision of Cerebral Meninges, Percutaneous Endoscopic Approach
        "00B20ZZ",  # Excision of Dura Mater, Open Approach
        "00B60ZX",  # Excision of Cerebral Ventricle, Open Approach, Diagnostic
        "00B60ZZ",  # Excision of Cerebral Ventricle, Open Approach
        "00B63ZX",  # Excision of Cerebral Ventricle, Percutaneous Approach, Diagnostic
        "00B63ZZ",  # Excision of Cerebral Ventricle, Percutaneous Approach
        "00B64ZX",  # Excision of Cerebral Ventricle, Percutaneous Endoscopic Approach, Diagnostic
        "00B64ZZ",  # Excision of Cerebral Ventricle, Percutaneous Endoscopic Approach
        "00B70ZX",  # Excision of Cerebral Hemisphere, Open Approach, Diagnostic
        "00B70ZZ",  # Excision of Cerebral Hemisphere, Open Approach
        "00B73ZX",  # Excision of Cerebral Hemisphere, Percutaneous Approach, Diagnostic
        "00B73ZZ",  # Excision of Cerebral Hemisphere, Percutaneous Approach
        "00B74ZX",  # Excision of Cerebral Hemisphere, Percutaneous Endoscopic Approach, Diagnostic
        "00B74ZZ",  # Excision of Cerebral Hemisphere, Percutaneous Endoscopic Approach
        "00B80ZZ",  # Excision of Basal Ganglia, Open Approach
        "00B90ZZ",  # Excision of Thalamus, Open Approach
        "00BA0ZZ",  # Excision of Hypothalamus, Open Approach
        "00BB0ZX",  # Excision of Pons, Open Approach, Diagnostic
        "00BB0ZZ",  # Excision of Pons, Open Approach
        "00BB3ZX",  # Excision of Pons, Percutaneous Approach, Diagnostic
        "00BB3ZZ",  # Excision of Pons, Percutaneous Approach
        "00BB4ZX",  # Excision of Pons, Percutaneous Endoscopic Approach, Diagnostic
        "00BB4ZZ",  # Excision of Pons, Percutaneous Endoscopic Approach
        "00BC0ZX",  # Excision of Cerebellum, Open Approach, Diagnostic
        "00BC0ZZ",  # Excision of Cerebellum, Open Approach
        "00BC3ZX",  # Excision of Cerebellum, Percutaneous Approach, Diagnostic
        "00BC3ZZ",  # Excision of Cerebellum, Percutaneous Approach
        "00BC4ZX",  # Excision of Cerebellum, Percutaneous Endoscopic Approach, Diagnostic
        "00BC4ZZ",  # Excision of Cerebellum, Percutaneous Endoscopic Approach
        "00BD0ZX",  # Excision of Medulla Oblongata, Open Approach, Diagnostic
        "00BD0ZZ",  # Excision of Medulla Oblongata, Open Approach
        "00BD3ZX",  # Excision of Medulla Oblongata, Percutaneous Approach, Diagnostic
        "00BD3ZZ",  # Excision of Medulla Oblongata, Percutaneous Approach
        "00BD4ZX",  # Excision of Medulla Oblongata, Percutaneous Endoscopic Approach, Diagnostic
        "00BD4ZZ",  # Excision of Medulla Oblongata, Percutaneous Endoscopic Approach
        "00C00ZZ",  # Extirpation of Matter from Brain, Open Approach
        "00C10ZZ",  # Extirpation of Matter from Cerebral Meninges, Open Approach
        "00C20ZZ",  # Extirpation of Matter from Dura Mater, Open Approach
        "00C30ZZ",  # Extirpation of Matter from Intracranial Epidural Space, Open Approach
        "00C40ZZ",  # Extirpation of Matter from Intracranial Subdural Space, Open Approach
        "00C50ZZ",  # Extirpation of Matter from Intracranial Subarachnoid Space, Open Approach
        "00C60ZZ",  # Extirpation of Matter from Cerebral Ventricle, Open Approach
        "00C70ZZ",  # Extirpation of Matter from Cerebral Hemisphere, Open Approach
        "00C80ZZ",  # Extirpation of Matter from Basal Ganglia, Open Approach
        "00C90ZZ",  # Extirpation of Matter from Thalamus, Open Approach
        "00CA0ZZ",  # Extirpation of Matter from Hypothalamus, Open Approach
        "00CB0ZZ",  # Extirpation of Matter from Pons, Open Approach
        "00CC0ZZ",  # Extirpation of Matter from Cerebellum, Open Approach
        "00CD0ZZ",  # Extirpation of Matter from Medulla Oblongata, Open Approach
        "00D10ZZ",  # Extraction of Cerebral Meninges, Open Approach
        "00D20ZZ",  # Extraction of Dura Mater, Open Approach
        "00H002Z",  # Insertion of Monitoring Device into Brain, Open Approach
        "00H003Z",  # Insertion of Infusion Device into Brain, Open Approach
        "00H602Z",  # Insertion of Monitoring Device into Cerebral Ventricle, Open Approach
        "00H603Z",  # Insertion of Infusion Device into Cerebral Ventricle, Open Approach
        "00J00ZZ",  # Inspection of Brain, Open Approach
        "00JE0ZZ",  # Inspection of Cranial Nerve, Open Approach
        "00JU0ZZ",  # Inspection of Spinal Canal, Open Approach
        "00JV0ZZ",  # Inspection of Spinal Cord, Open Approach
        "00N90ZZ",  # Release Thalamus, Open Approach
        "00NA0ZZ",  # Release Hypothalamus, Open Approach
        "00P000Z",  # Removal of Drainage Device from Brain, Open Approach
        "00P002Z",  # Removal of Monitoring Device from Brain, Open Approach
        "00P003Z",  # Removal of Infusion Device from Brain, Open Approach
        "00P007Z",  # Removal of Autologous Tissue Substitute from Brain, Open Approach
        "00P00JZ",  # Removal of Synthetic Substitute from Brain, Open Approach
        "00P00KZ",  # Removal of Nonautologous Tissue Substitute from Brain, Open Approach
        "00P600Z",  # Removal of Drainage Device from Cerebral Ventricle, Open Approach
        "00P602Z",  # Removal of Monitoring Device from Cerebral Ventricle, Open Approach
        "00P603Z",  # Removal of Infusion Device from Cerebral Ventricle, Open Approach
        "00P6X2Z",  # Removal of Monitoring Device from Cerebral Ventricle, External Approach
        "00Q90ZZ",  # Repair Thalamus, Open Approach
        "00QA0ZZ",  # Repair Hypothalamus, Open Approach
        "00T70ZZ",  # Resection of Cerebral Hemisphere, Open Approach
        "00W000Z",  # Revision of Drainage Device in Brain, Open Approach
        "00W002Z",  # Revision of Monitoring Device in Brain, Open Approach
        "00W003Z",  # Revision of Infusion Device in Brain, Open Approach
        "00W007Z",  # Revision of Autologous Tissue Substitute in Brain, Open Approach
        "00W00JZ",  # Revision of Synthetic Substitute in Brain, Open Approach
        "00W00KZ",  # Revision of Nonautologous Tissue Substitute in Brain, Open Approach
        "00W00MZ",  # Revision of Neurostimulator Lead in Brain, Open Approach
        "00W600Z",  # Revision of Drainage Device in Cerebral Ventricle, Open Approach
        "00W602Z",  # Revision of Monitoring Device in Cerebral Ventricle, Open Approach
        "00W603Z",  # Revision of Infusion Device in Cerebral Ventricle, Open Approach
        "00W60MZ",  # Revision of Neurostimulator Lead in Cerebral Ventricle, Open Approach
        "01JY0ZZ",  # Inspection of Peripheral Nerve, Open Approach
        "0210083",  # Bypass Coronary Artery, One Artery from Coronary Artery with Zooplastic Tissue, Open Approach
        "0210088",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0210089",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021008C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021008F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021008W",  # Bypass Coronary Artery, One Artery from Aorta with Zooplastic Tissue, Open Approach
        "0210093",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0210098",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0210099",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021009C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021009F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021009W",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Venous Tissue, Open Approach
        "02100A3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02100A8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02100A9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02100AC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02100AF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02100AW",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Arterial Tissue, Open Approach
        "02100J3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Synthetic Substitute, Open Approach
        "02100J8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02100J9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02100JC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Synthetic Substitute, Open Approach
        "02100JF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Synthetic Substitute, Open Approach
        "02100JW",  # Bypass Coronary Artery, One Artery from Aorta with Synthetic Substitute, Open Approach
        "02100K3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02100K8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02100K9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02100KC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02100KF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02100KW",  # Bypass Coronary Artery, One Artery from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02100Z3",  # Bypass Coronary Artery, One Artery from Coronary Artery, Open Approach
        "02100Z8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary, Open Approach
        "02100Z9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary, Open Approach
        "02100ZC",  # Bypass Coronary Artery, One Artery from Thoracic Artery, Open Approach
        "02100ZF",  # Bypass Coronary Artery, One Artery from Abdominal Artery, Open Approach
        "0210344",  # Bypass Coronary Artery, One Artery from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02103D4",  # Bypass Coronary Artery, One Artery from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0210444",  # Bypass Coronary Artery, One Artery from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0210483",  # Bypass Coronary Artery, One Artery from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210488",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210489",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048W",  # Bypass Coronary Artery, One Artery from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210493",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0210498",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0210499",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049W",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02104A3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104A8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104A9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AW",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104D4",  # Bypass Coronary Artery, One Artery from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02104J3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104J8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104J9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JW",  # Bypass Coronary Artery, One Artery from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104K3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104K8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104K9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KW",  # Bypass Coronary Artery, One Artery from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104Z3",  # Bypass Coronary Artery, One Artery from Coronary Artery, Percutaneous Endoscopic Approach
        "02104Z8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02104Z9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02104ZC",  # Bypass Coronary Artery, One Artery from Thoracic Artery, Percutaneous Endoscopic Approach
        "02104ZF",  # Bypass Coronary Artery, One Artery from Abdominal Artery, Percutaneous Endoscopic Approach
        "0211083",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0211088",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0211089",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021108C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021108F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021108W",  # Bypass Coronary Artery, Two Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0211093",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0211098",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0211099",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021109C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021109F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021109W",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02110A3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02110A8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02110A9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02110AC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02110AF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02110AW",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02110J3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02110J8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02110J9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02110JC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02110JF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02110JW",  # Bypass Coronary Artery, Two Arteries from Aorta with Synthetic Substitute, Open Approach
        "02110K3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02110K8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02110K9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02110KC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02110KF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02110KW",  # Bypass Coronary Artery, Two Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02110Z3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery, Open Approach
        "02110Z8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary, Open Approach
        "02110Z9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary, Open Approach
        "02110ZC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery, Open Approach
        "02110ZF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery, Open Approach
        "0211344",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02113D4",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0211444",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0211483",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211488",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211489",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148W",  # Bypass Coronary Artery, Two Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211493",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0211498",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0211499",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149W",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02114A3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114A8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114A9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AW",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114D4",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02114J3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114J8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114J9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JW",  # Bypass Coronary Artery, Two Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114K3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114K8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114K9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KW",  # Bypass Coronary Artery, Two Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114Z3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02114Z8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02114Z9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02114ZC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02114ZF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
        "0212083",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0212088",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0212089",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021208C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021208F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021208W",  # Bypass Coronary Artery, Three Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0212093",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0212098",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0212099",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021209C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021209F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021209W",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02120A3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02120A8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02120A9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02120AC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02120AF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02120AW",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02120J3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02120J8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02120J9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02120JC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02120JF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02120JW",  # Bypass Coronary Artery, Three Arteries from Aorta with Synthetic Substitute, Open Approach
        "02120K3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02120K8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02120K9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02120KC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02120KF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02120KW",  # Bypass Coronary Artery, Three Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02120Z3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery, Open Approach
        "02120Z8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary, Open Approach
        "02120Z9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary, Open Approach
        "02120ZC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery, Open Approach
        "02120ZF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery, Open Approach
        "0212344",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02123D4",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0212444",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0212483",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212488",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212489",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248W",  # Bypass Coronary Artery, Three Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212493",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0212498",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0212499",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249W",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02124A3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124A8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124A9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AW",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124D4",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02124J3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124J8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124J9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JW",  # Bypass Coronary Artery, Three Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124K3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124K8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124K9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KW",  # Bypass Coronary Artery, Three Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124Z3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02124Z8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02124Z9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02124ZC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02124ZF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
        "0213083",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0213088",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0213089",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021308C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021308F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021308W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0213093",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0213098",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0213099",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021309C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021309F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021309W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02130A3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02130A8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02130A9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02130AC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02130AF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02130AW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02130J3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02130J8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02130J9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02130JC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02130JF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02130JW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Synthetic Substitute, Open Approach
        "02130K3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02130K8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02130K9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02130KC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02130KF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02130KW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02130Z3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery, Open Approach
        "02130Z8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary, Open Approach
        "02130Z9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary, Open Approach
        "02130ZC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery, Open Approach
        "02130ZF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery, Open Approach
        "0213344",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02133D4",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0213444",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0213483",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213488",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213489",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213493",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0213498",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0213499",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02134A3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134A8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134A9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134D4",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02134J3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134J8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134J9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134K3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134K8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134K9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134Z3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02134Z8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02134Z9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02134ZC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02134ZF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
        "021608Q",  # Bypass Right Atrium to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021608R",  # Bypass Right Atrium to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021609P",  # Bypass Right Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach
        "021609Q",  # Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021609R",  # Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "02160AP",  # Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "02160AQ",  # Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "02160AR",  # Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "02160JP",  # Bypass Right Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach
        "02160JQ",  # Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "02160JR",  # Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "02160KP",  # Bypass Right Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach
        "02160KQ",  # Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "02160KR",  # Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "02160Z7",  # Bypass Right Atrium to Left Atrium, Open Approach
        "02160ZP",  # Bypass Right Atrium to Pulmonary Trunk, Open Approach
        "02160ZQ",  # Bypass Right Atrium to Right Pulmonary Artery, Open Approach
        "02160ZR",  # Bypass Right Atrium to Left Pulmonary Artery, Open Approach
        "021648Q",  # Bypass Right Atrium to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021648R",  # Bypass Right Atrium to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021649Q",  # Bypass Right Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021649R",  # Bypass Right Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02164AP",  # Bypass Right Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02164AQ",  # Bypass Right Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02164AR",  # Bypass Right Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02164JQ",  # Bypass Right Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02164JR",  # Bypass Right Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02164KQ",  # Bypass Right Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02164KR",  # Bypass Right Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02164ZQ",  # Bypass Right Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "02164ZR",  # Bypass Right Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021708Q",  # Bypass Left Atrium to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021708R",  # Bypass Left Atrium to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021709P",  # Bypass Left Atrium to Pulmonary Trunk with Autologous Venous Tissue, Open Approach
        "021709Q",  # Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021709R",  # Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "02170AP",  # Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "02170AQ",  # Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "02170AR",  # Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "02170AS",  # Bypass Left Atrium to Right Pulmonary Vein with Autologous Arterial Tissue, Open Approach
        "02170AT",  # Bypass Left Atrium to Left Pulmonary Vein with Autologous Arterial Tissue, Open Approach
        "02170AU",  # Bypass Left Atrium to Pulmonary Vein Confluence with Autologous Arterial Tissue, Open Approach
        "02170JP",  # Bypass Left Atrium to Pulmonary Trunk with Synthetic Substitute, Open Approach
        "02170JQ",  # Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "02170JR",  # Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "02170KP",  # Bypass Left Atrium to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach
        "02170KQ",  # Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "02170KR",  # Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "02170ZP",  # Bypass Left Atrium to Pulmonary Trunk, Open Approach
        "02170ZQ",  # Bypass Left Atrium to Right Pulmonary Artery, Open Approach
        "02170ZR",  # Bypass Left Atrium to Left Pulmonary Artery, Open Approach
        "021748Q",  # Bypass Left Atrium to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021748R",  # Bypass Left Atrium to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021749Q",  # Bypass Left Atrium to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021749R",  # Bypass Left Atrium to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02174AP",  # Bypass Left Atrium to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174AQ",  # Bypass Left Atrium to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174AR",  # Bypass Left Atrium to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174AS",  # Bypass Left Atrium to Right Pulmonary Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174AT",  # Bypass Left Atrium to Left Pulmonary Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174AU",  # Bypass Left Atrium to Pulmonary Vein Confluence with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02174JQ",  # Bypass Left Atrium to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02174JR",  # Bypass Left Atrium to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02174KQ",  # Bypass Left Atrium to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02174KR",  # Bypass Left Atrium to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02174ZQ",  # Bypass Left Atrium to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "02174ZR",  # Bypass Left Atrium to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021K08Q",  # Bypass Right Ventricle to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021K08R",  # Bypass Right Ventricle to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021K09P",  # Bypass Right Ventricle to Pulmonary Trunk with Autologous Venous Tissue, Open Approach
        "021K09Q",  # Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021K09R",  # Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021K0AP",  # Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "021K0AQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021K0AR",  # Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021K0JP",  # Bypass Right Ventricle to Pulmonary Trunk with Synthetic Substitute, Open Approach
        "021K0JQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "021K0JR",  # Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "021K0KP",  # Bypass Right Ventricle to Pulmonary Trunk with Nonautologous Tissue Substitute, Open Approach
        "021K0KQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021K0KR",  # Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021K0Z5",  # Bypass Right Ventricle to Coronary Circulation, Open Approach
        "021K0ZC",  # Bypass Right Ventricle to Thoracic Artery, Open Approach
        "021K0ZF",  # Bypass Right Ventricle to Abdominal Artery, Open Approach
        "021K0ZP",  # Bypass Right Ventricle to Pulmonary Trunk, Open Approach
        "021K0ZQ",  # Bypass Right Ventricle to Right Pulmonary Artery, Open Approach
        "021K0ZR",  # Bypass Right Ventricle to Left Pulmonary Artery, Open Approach
        "021K48Q",  # Bypass Right Ventricle to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021K48R",  # Bypass Right Ventricle to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021K49Q",  # Bypass Right Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021K49R",  # Bypass Right Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021K4AP",  # Bypass Right Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021K4AQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021K4AR",  # Bypass Right Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021K4JQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021K4JR",  # Bypass Right Ventricle to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021K4KQ",  # Bypass Right Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021K4KR",  # Bypass Right Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021K4ZC",  # Bypass Right Ventricle to Thoracic Artery, Percutaneous Endoscopic Approach
        "021K4ZF",  # Bypass Right Ventricle to Abdominal Artery, Percutaneous Endoscopic Approach
        "021K4ZQ",  # Bypass Right Ventricle to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "021K4ZR",  # Bypass Right Ventricle to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021L08Q",  # Bypass Left Ventricle to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021L08R",  # Bypass Left Ventricle to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021L09Q",  # Bypass Left Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021L09R",  # Bypass Left Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021L0AP",  # Bypass Left Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "021L0AQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021L0AR",  # Bypass Left Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021L0JQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "021L0JR",  # Bypass Left Ventricle to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "021L0KQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021L0KR",  # Bypass Left Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021L0Z5",  # Bypass Left Ventricle to Coronary Circulation, Open Approach
        "021L0ZC",  # Bypass Left Ventricle to Thoracic Artery, Open Approach
        "021L0ZF",  # Bypass Left Ventricle to Abdominal Artery, Open Approach
        "021L0ZQ",  # Bypass Left Ventricle to Right Pulmonary Artery, Open Approach
        "021L0ZR",  # Bypass Left Ventricle to Left Pulmonary Artery, Open Approach
        "021L0ZW",  # Bypass Left Ventricle to Aorta, Open Approach
        "021L48Q",  # Bypass Left Ventricle to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021L48R",  # Bypass Left Ventricle to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021L49Q",  # Bypass Left Ventricle to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021L49R",  # Bypass Left Ventricle to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021L4AP",  # Bypass Left Ventricle to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021L4AQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021L4AR",  # Bypass Left Ventricle to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021L4JQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021L4JR",  # Bypass Left Ventricle to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021L4KQ",  # Bypass Left Ventricle to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021L4KR",  # Bypass Left Ventricle to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021L4ZC",  # Bypass Left Ventricle to Thoracic Artery, Percutaneous Endoscopic Approach
        "021L4ZF",  # Bypass Left Ventricle to Abdominal Artery, Percutaneous Endoscopic Approach
        "021L4ZQ",  # Bypass Left Ventricle to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "021L4ZR",  # Bypass Left Ventricle to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021P08A",  # Bypass Pulmonary Trunk from Innominate Artery with Zooplastic Tissue, Open Approach
        "021P09A",  # Bypass Pulmonary Trunk from Innominate Artery with Autologous Venous Tissue, Open Approach
        "021P0AA",  # Bypass Pulmonary Trunk from Innominate Artery with Autologous Arterial Tissue, Open Approach
        "021P0AB",  # Bypass Pulmonary Trunk from Subclavian with Autologous Arterial Tissue, Open Approach
        "021P0AD",  # Bypass Pulmonary Trunk from Carotid with Autologous Arterial Tissue, Open Approach
        "021P0JA",  # Bypass Pulmonary Trunk from Innominate Artery with Synthetic Substitute, Open Approach
        "021P0KA",  # Bypass Pulmonary Trunk from Innominate Artery with Nonautologous Tissue Substitute, Open Approach
        "021P0ZA",  # Bypass Pulmonary Trunk from Innominate Artery with No Device, Open Approach
        "021P48A",  # Bypass Pulmonary Trunk from Innominate Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021P49A",  # Bypass Pulmonary Trunk from Innominate Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021P4AA",  # Bypass Pulmonary Trunk from Innominate Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021P4AB",  # Bypass Pulmonary Trunk from Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021P4AD",  # Bypass Pulmonary Trunk from Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021P4JA",  # Bypass Pulmonary Trunk from Innominate Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021P4KA",  # Bypass Pulmonary Trunk from Innominate Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021P4ZA",  # Bypass Pulmonary Trunk from Innominate Artery with No Device, Percutaneous Endoscopic Approach
        "021Q08A",  # Bypass Right Pulmonary Artery from Innominate Artery with Zooplastic Tissue, Open Approach
        "021Q08B",  # Bypass Right Pulmonary Artery from Subclavian with Zooplastic Tissue, Open Approach
        "021Q08D",  # Bypass Right Pulmonary Artery from Carotid with Zooplastic Tissue, Open Approach
        "021Q09A",  # Bypass Right Pulmonary Artery from Innominate Artery with Autologous Venous Tissue, Open Approach
        "021Q09B",  # Bypass Right Pulmonary Artery from Subclavian with Autologous Venous Tissue, Open Approach
        "021Q09D",  # Bypass Right Pulmonary Artery from Carotid with Autologous Venous Tissue, Open Approach
        "021Q0AA",  # Bypass Right Pulmonary Artery from Innominate Artery with Autologous Arterial Tissue, Open Approach
        "021Q0AB",  # Bypass Right Pulmonary Artery from Subclavian with Autologous Arterial Tissue, Open Approach
        "021Q0AD",  # Bypass Right Pulmonary Artery from Carotid with Autologous Arterial Tissue, Open Approach
        "021Q0JA",  # Bypass Right Pulmonary Artery from Innominate Artery with Synthetic Substitute, Open Approach
        "021Q0JB",  # Bypass Right Pulmonary Artery from Subclavian with Synthetic Substitute, Open Approach
        "021Q0JD",  # Bypass Right Pulmonary Artery from Carotid with Synthetic Substitute, Open Approach
        "021Q0KA",  # Bypass Right Pulmonary Artery from Innominate Artery with Nonautologous Tissue Substitute, Open Approach
        "021Q0KB",  # Bypass Right Pulmonary Artery from Subclavian with Nonautologous Tissue Substitute, Open Approach
        "021Q0KD",  # Bypass Right Pulmonary Artery from Carotid with Nonautologous Tissue Substitute, Open Approach
        "021Q0ZA",  # Bypass Right Pulmonary Artery from Innominate Artery with No Device, Open Approach
        "021Q0ZB",  # Bypass Right Pulmonary Artery from Subclavian with No Device, Open Approach
        "021Q0ZD",  # Bypass Right Pulmonary Artery from Carotid with No Device, Open Approach
        "021Q48A",  # Bypass Right Pulmonary Artery from Innominate Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021Q48B",  # Bypass Right Pulmonary Artery from Subclavian with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021Q48D",  # Bypass Right Pulmonary Artery from Carotid with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021Q49A",  # Bypass Right Pulmonary Artery from Innominate Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021Q49B",  # Bypass Right Pulmonary Artery from Subclavian with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021Q49D",  # Bypass Right Pulmonary Artery from Carotid with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021Q4AA",  # Bypass Right Pulmonary Artery from Innominate Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021Q4AB",  # Bypass Right Pulmonary Artery from Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021Q4AD",  # Bypass Right Pulmonary Artery from Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021Q4JA",  # Bypass Right Pulmonary Artery from Innominate Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021Q4JB",  # Bypass Right Pulmonary Artery from Subclavian with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021Q4JD",  # Bypass Right Pulmonary Artery from Carotid with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021Q4KA",  # Bypass Right Pulmonary Artery from Innominate Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021Q4KB",  # Bypass Right Pulmonary Artery from Subclavian with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021Q4KD",  # Bypass Right Pulmonary Artery from Carotid with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021Q4ZA",  # Bypass Right Pulmonary Artery from Innominate Artery with No Device, Percutaneous Endoscopic Approach
        "021Q4ZB",  # Bypass Right Pulmonary Artery from Subclavian with No Device, Percutaneous Endoscopic Approach
        "021Q4ZD",  # Bypass Right Pulmonary Artery from Carotid with No Device, Percutaneous Endoscopic Approach
        "021R08A",  # Bypass Left Pulmonary Artery from Innominate Artery with Zooplastic Tissue, Open Approach
        "021R08B",  # Bypass Left Pulmonary Artery from Subclavian with Zooplastic Tissue, Open Approach
        "021R08D",  # Bypass Left Pulmonary Artery from Carotid with Zooplastic Tissue, Open Approach
        "021R09A",  # Bypass Left Pulmonary Artery from Innominate Artery with Autologous Venous Tissue, Open Approach
        "021R09B",  # Bypass Left Pulmonary Artery from Subclavian with Autologous Venous Tissue, Open Approach
        "021R09D",  # Bypass Left Pulmonary Artery from Carotid with Autologous Venous Tissue, Open Approach
        "021R0AA",  # Bypass Left Pulmonary Artery from Innominate Artery with Autologous Arterial Tissue, Open Approach
        "021R0AB",  # Bypass Left Pulmonary Artery from Subclavian with Autologous Arterial Tissue, Open Approach
        "021R0AD",  # Bypass Left Pulmonary Artery from Carotid with Autologous Arterial Tissue, Open Approach
        "021R0JA",  # Bypass Left Pulmonary Artery from Innominate Artery with Synthetic Substitute, Open Approach
        "021R0JB",  # Bypass Left Pulmonary Artery from Subclavian with Synthetic Substitute, Open Approach
        "021R0JD",  # Bypass Left Pulmonary Artery from Carotid with Synthetic Substitute, Open Approach
        "021R0KA",  # Bypass Left Pulmonary Artery from Innominate Artery with Nonautologous Tissue Substitute, Open Approach
        "021R0KB",  # Bypass Left Pulmonary Artery from Subclavian with Nonautologous Tissue Substitute, Open Approach
        "021R0KD",  # Bypass Left Pulmonary Artery from Carotid with Nonautologous Tissue Substitute, Open Approach
        "021R0ZA",  # Bypass Left Pulmonary Artery from Innominate Artery with No Device, Open Approach
        "021R0ZB",  # Bypass Left Pulmonary Artery from Subclavian with No Device, Open Approach
        "021R0ZD",  # Bypass Left Pulmonary Artery from Carotid with No Device, Open Approach
        "021R48A",  # Bypass Left Pulmonary Artery from Innominate Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021R48B",  # Bypass Left Pulmonary Artery from Subclavian with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021R48D",  # Bypass Left Pulmonary Artery from Carotid with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021R49A",  # Bypass Left Pulmonary Artery from Innominate Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021R49B",  # Bypass Left Pulmonary Artery from Subclavian with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021R49D",  # Bypass Left Pulmonary Artery from Carotid with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021R4AA",  # Bypass Left Pulmonary Artery from Innominate Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021R4AB",  # Bypass Left Pulmonary Artery from Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021R4AD",  # Bypass Left Pulmonary Artery from Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021R4JA",  # Bypass Left Pulmonary Artery from Innominate Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021R4JB",  # Bypass Left Pulmonary Artery from Subclavian with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021R4JD",  # Bypass Left Pulmonary Artery from Carotid with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021R4KA",  # Bypass Left Pulmonary Artery from Innominate Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021R4KB",  # Bypass Left Pulmonary Artery from Subclavian with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021R4KD",  # Bypass Left Pulmonary Artery from Carotid with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021R4ZA",  # Bypass Left Pulmonary Artery from Innominate Artery with No Device, Percutaneous Endoscopic Approach
        "021R4ZB",  # Bypass Left Pulmonary Artery from Subclavian with No Device, Percutaneous Endoscopic Approach
        "021R4ZD",  # Bypass Left Pulmonary Artery from Carotid with No Device, Percutaneous Endoscopic Approach
        "021V08Q",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021V08R",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021V09Q",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021V09R",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021V0AP",  # Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "021V0AQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021V0AR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021V0AS",  # Bypass Superior Vena Cava to Right Pulmonary Vein with Autologous Arterial Tissue, Open Approach
        "021V0AT",  # Bypass Superior Vena Cava to Left Pulmonary Vein with Autologous Arterial Tissue, Open Approach
        "021V0AU",  # Bypass Superior Vena Cava to Pulmonary Vein Confluence with Autologous Arterial Tissue, Open Approach
        "021V0JQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "021V0JR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "021V0KQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021V0KR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021V0ZQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery, Open Approach
        "021V0ZR",  # Bypass Superior Vena Cava to Left Pulmonary Artery, Open Approach
        "021V48Q",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021V48R",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021V49Q",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021V49R",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021V4AP",  # Bypass Superior Vena Cava to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4AQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4AR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4AS",  # Bypass Superior Vena Cava to Right Pulmonary Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4AT",  # Bypass Superior Vena Cava to Left Pulmonary Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4AU",  # Bypass Superior Vena Cava to Pulmonary Vein Confluence with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021V4JQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021V4JR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021V4KQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021V4KR",  # Bypass Superior Vena Cava to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021V4ZQ",  # Bypass Superior Vena Cava to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "021V4ZR",  # Bypass Superior Vena Cava to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021W08F",  # Bypass Thoracic Aorta, Descending to Abdominal Artery with Zooplastic Tissue, Open Approach
        "021W08G",  # Bypass Thoracic Aorta, Descending to Axillary Artery with Zooplastic Tissue, Open Approach
        "021W08H",  # Bypass Thoracic Aorta, Descending to Brachial Artery with Zooplastic Tissue, Open Approach
        "021W08Q",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021W08R",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021W08V",  # Bypass Thoracic Aorta, Descending to Lower Extremity Artery with Zooplastic Tissue, Open Approach
        "021W09F",  # Bypass Thoracic Aorta, Descending to Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021W09G",  # Bypass Thoracic Aorta, Descending to Axillary Artery with Autologous Venous Tissue, Open Approach
        "021W09H",  # Bypass Thoracic Aorta, Descending to Brachial Artery with Autologous Venous Tissue, Open Approach
        "021W09Q",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021W09R",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021W09V",  # Bypass Thoracic Aorta, Descending to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "021W0AB",  # Bypass Thoracic Aorta, Descending to Subclavian with Autologous Arterial Tissue, Open Approach
        "021W0AD",  # Bypass Thoracic Aorta, Descending to Carotid with Autologous Arterial Tissue, Open Approach
        "021W0AF",  # Bypass Thoracic Aorta, Descending to Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "021W0AG",  # Bypass Thoracic Aorta, Descending to Axillary Artery with Autologous Arterial Tissue, Open Approach
        "021W0AH",  # Bypass Thoracic Aorta, Descending to Brachial Artery with Autologous Arterial Tissue, Open Approach
        "021W0AP",  # Bypass Thoracic Aorta, Descending to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "021W0AQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021W0AR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021W0AV",  # Bypass Thoracic Aorta, Descending to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "021W0JF",  # Bypass Thoracic Aorta, Descending to Abdominal Artery with Synthetic Substitute, Open Approach
        "021W0JG",  # Bypass Thoracic Aorta, Descending to Axillary Artery with Synthetic Substitute, Open Approach
        "021W0JH",  # Bypass Thoracic Aorta, Descending to Brachial Artery with Synthetic Substitute, Open Approach
        "021W0JQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "021W0JR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "021W0JV",  # Bypass Thoracic Aorta, Descending to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "021W0KF",  # Bypass Thoracic Aorta, Descending to Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0KG",  # Bypass Thoracic Aorta, Descending to Axillary Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0KH",  # Bypass Thoracic Aorta, Descending to Brachial Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0KQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0KR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0KV",  # Bypass Thoracic Aorta, Descending to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "021W0ZQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery, Open Approach
        "021W0ZR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery, Open Approach
        "021W48Q",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021W48R",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021W49Q",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021W49R",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021W4AB",  # Bypass Thoracic Aorta, Descending to Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021W4AD",  # Bypass Thoracic Aorta, Descending to Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021W4AP",  # Bypass Thoracic Aorta, Descending to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021W4AQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021W4AR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021W4JQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021W4JR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021W4KQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021W4KR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021W4ZQ",  # Bypass Thoracic Aorta, Descending to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "021W4ZR",  # Bypass Thoracic Aorta, Descending to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "021X08Q",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021X08R",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Zooplastic Tissue, Open Approach
        "021X09Q",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021X09R",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "021X0AB",  # Bypass Thoracic Aorta, Ascending/Arch to Subclavian with Autologous Arterial Tissue, Open Approach
        "021X0AD",  # Bypass Thoracic Aorta, Ascending/Arch to Carotid with Autologous Arterial Tissue, Open Approach
        "021X0AP",  # Bypass Thoracic Aorta, Ascending/Arch to Pulmonary Trunk with Autologous Arterial Tissue, Open Approach
        "021X0AQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021X0AR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "021X0JQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "021X0JR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "021X0KQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021X0KR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "021X0ZQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery, Open Approach
        "021X0ZR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery, Open Approach
        "021X48Q",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021X48R",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021X49Q",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021X49R",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021X4AB",  # Bypass Thoracic Aorta, Ascending/Arch to Subclavian with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021X4AD",  # Bypass Thoracic Aorta, Ascending/Arch to Carotid with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021X4AP",  # Bypass Thoracic Aorta, Ascending/Arch to Pulmonary Trunk with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021X4AQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021X4AR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "021X4JQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021X4JR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "021X4KQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021X4KR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "021X4ZQ",  # Bypass Thoracic Aorta, Ascending/Arch to Right Pulmonary Artery, Percutaneous Endoscopic Approach
        "021X4ZR",  # Bypass Thoracic Aorta, Ascending/Arch to Left Pulmonary Artery, Percutaneous Endoscopic Approach
        "02540ZZ",  # Destruction of Coronary Vein, Open Approach
        "02550ZZ",  # Destruction of Atrial Septum, Open Approach
        "02560ZZ",  # Destruction of Right Atrium, Open Approach
        "02570ZK",  # Destruction of Left Atrial Appendage, Open Approach
        "02570ZZ",  # Destruction of Left Atrium, Open Approach
        "02580ZZ",  # Destruction of Conduction Mechanism, Open Approach
        "02590ZZ",  # Destruction of Chordae Tendineae, Open Approach
        "025D0ZZ",  # Destruction of Papillary Muscle, Open Approach
        "025F0ZZ",  # Destruction of Aortic Valve, Open Approach
        "025G0ZZ",  # Destruction of Mitral Valve, Open Approach
        "025H0ZZ",  # Destruction of Pulmonary Valve, Open Approach
        "025J0ZZ",  # Destruction of Tricuspid Valve, Open Approach
        "025K0ZZ",  # Destruction of Right Ventricle, Open Approach
        "025L0ZZ",  # Destruction of Left Ventricle, Open Approach
        "025M0ZZ",  # Destruction of Ventricular Septum, Open Approach
        "027F04Z",  # Dilation of Aortic Valve with Drug-eluting Intraluminal Device, Open Approach
        "027F0DZ",  # Dilation of Aortic Valve with Intraluminal Device, Open Approach
        "027F0ZZ",  # Dilation of Aortic Valve, Open Approach
        "027G04Z",  # Dilation of Mitral Valve with Drug-eluting Intraluminal Device, Open Approach
        "027G0DZ",  # Dilation of Mitral Valve with Intraluminal Device, Open Approach
        "027G0ZZ",  # Dilation of Mitral Valve, Open Approach
        "027H04Z",  # Dilation of Pulmonary Valve with Drug-eluting Intraluminal Device, Open Approach
        "027H0DZ",  # Dilation of Pulmonary Valve with Intraluminal Device, Open Approach
        "027H0ZZ",  # Dilation of Pulmonary Valve, Open Approach
        "027J04Z",  # Dilation of Tricuspid Valve with Drug-eluting Intraluminal Device, Open Approach
        "027J0DZ",  # Dilation of Tricuspid Valve with Intraluminal Device, Open Approach
        "027J0ZZ",  # Dilation of Tricuspid Valve, Open Approach
        "027K04Z",  # Dilation of Right Ventricle with Drug-eluting Intraluminal Device, Open Approach
        "027K0DZ",  # Dilation of Right Ventricle with Intraluminal Device, Open Approach
        "027K0ZZ",  # Dilation of Right Ventricle, Open Approach
        "027R04T",  # Dilation of Ductus Arteriosus with Drug-eluting Intraluminal Device, Open Approach
        "027R0DT",  # Dilation of Ductus Arteriosus with Intraluminal Device, Open Approach
        "027R0ZT",  # Dilation of Ductus Arteriosus, Open Approach
        "02890ZZ",  # Division of Chordae Tendineae, Open Approach
        "028D0ZZ",  # Division of Papillary Muscle, Open Approach
        "02B40ZZ",  # Excision of Coronary Vein, Open Approach
        "02B50ZZ",  # Excision of Atrial Septum, Open Approach
        "02B60ZZ",  # Excision of Right Atrium, Open Approach
        "02B70ZK",  # Excision of Left Atrial Appendage, Open Approach
        "02B70ZZ",  # Excision of Left Atrium, Open Approach
        "02B80ZZ",  # Excision of Conduction Mechanism, Open Approach
        "02B90ZZ",  # Excision of Chordae Tendineae, Open Approach
        "02BD0ZZ",  # Excision of Papillary Muscle, Open Approach
        "02BF0ZZ",  # Excision of Aortic Valve, Open Approach
        "02BG0ZZ",  # Excision of Mitral Valve, Open Approach
        "02BH0ZZ",  # Excision of Pulmonary Valve, Open Approach
        "02BJ0ZZ",  # Excision of Tricuspid Valve, Open Approach
        "02BK0ZX",  # Excision of Right Ventricle, Open Approach, Diagnostic
        "02BK0ZZ",  # Excision of Right Ventricle, Open Approach
        "02BK3ZX",  # Excision of Right Ventricle, Percutaneous Approach, Diagnostic
        "02BK3ZZ",  # Excision of Right Ventricle, Percutaneous Approach
        "02BK4ZX",  # Excision of Right Ventricle, Percutaneous Endoscopic Approach, Diagnostic
        "02BK4ZZ",  # Excision of Right Ventricle, Percutaneous Endoscopic Approach
        "02BL0ZX",  # Excision of Left Ventricle, Open Approach, Diagnostic
        "02BL0ZZ",  # Excision of Left Ventricle, Open Approach
        "02BL3ZX",  # Excision of Left Ventricle, Percutaneous Approach, Diagnostic
        "02BL3ZZ",  # Excision of Left Ventricle, Percutaneous Approach
        "02BL4ZX",  # Excision of Left Ventricle, Percutaneous Endoscopic Approach, Diagnostic
        "02BL4ZZ",  # Excision of Left Ventricle, Percutaneous Endoscopic Approach
        "02BM0ZZ",  # Excision of Ventricular Septum, Open Approach
        "02BW0ZZ",  # Excision of Thoracic Aorta, Descending, Open Approach
        "02C00ZZ",  # Extirpation of Matter from Coronary Artery, One Artery, Open Approach
        "02C10ZZ",  # Extirpation of Matter from Coronary Artery, Two Arteries, Open Approach
        "02C20ZZ",  # Extirpation of Matter from Coronary Artery, Three Arteries, Open Approach
        "02C30ZZ",  # Extirpation of Matter from Coronary Artery, Four or More Arteries, Open Approach
        "02C40ZZ",  # Extirpation of Matter from Coronary Vein, Open Approach
        "02C60ZZ",  # Extirpation of Matter from Right Atrium, Open Approach
        "02C70ZZ",  # Extirpation of Matter from Left Atrium, Open Approach
        "02C80ZZ",  # Extirpation of Matter from Conduction Mechanism, Open Approach
        "02C90ZZ",  # Extirpation of Matter from Chordae Tendineae, Open Approach
        "02CD0ZZ",  # Extirpation of Matter from Papillary Muscle, Open Approach
        "02CF0ZZ",  # Extirpation of Matter from Aortic Valve, Open Approach
        "02CG0ZZ",  # Extirpation of Matter from Mitral Valve, Open Approach
        "02CH0ZZ",  # Extirpation of Matter from Pulmonary Valve, Open Approach
        "02CJ0ZZ",  # Extirpation of Matter from Tricuspid Valve, Open Approach
        "02CK0ZZ",  # Extirpation of Matter from Right Ventricle, Open Approach
        "02CL0ZZ",  # Extirpation of Matter from Left Ventricle, Open Approach
        "02CW0ZZ",  # Extirpation of Matter from Thoracic Aorta, Descending, Open Approach
        "02FN0ZZ",  # Fragmentation in Pericardium, Open Approach
        "02H402Z",  # Insertion of Monitoring Device into Coronary Vein, Open Approach
        "02H403Z",  # Insertion of Infusion Device into Coronary Vein, Open Approach
        "02H40DZ",  # Insertion of Intraluminal Device into Coronary Vein, Open Approach
        "02H602Z",  # Insertion of Monitoring Device into Right Atrium, Open Approach
        "02H603Z",  # Insertion of Infusion Device into Right Atrium, Open Approach
        "02H60DZ",  # Insertion of Intraluminal Device into Right Atrium, Open Approach
        "02H702Z",  # Insertion of Monitoring Device into Left Atrium, Open Approach
        "02H703Z",  # Insertion of Infusion Device into Left Atrium, Open Approach
        "02H70DZ",  # Insertion of Intraluminal Device into Left Atrium, Open Approach
        "02HA0QZ",  # Insertion of Implantable Heart Assist System into Heart, Open Approach
        "02HA0RS",  # Insertion of Biventricular Short-term External Heart Assist System into Heart, Open Approach
        "02HK03Z",  # Insertion of Infusion Device into Right Ventricle, Open Approach
        "02HK0DZ",  # Insertion of Intraluminal Device into Right Ventricle, Open Approach
        "02HL02Z",  # Insertion of Monitoring Device into Left Ventricle, Open Approach
        "02HL03Z",  # Insertion of Infusion Device into Left Ventricle, Open Approach
        "02HL0DZ",  # Insertion of Intraluminal Device into Left Ventricle, Open Approach
        "02HN0MZ",  # Insertion of Cardiac Lead into Pericardium, Open Approach
        "02JA0ZZ",  # Inspection of Heart, Open Approach
        "02JY0ZZ",  # Inspection of Great Vessel, Open Approach
        "02L70CK",  # Occlusion of Left Atrial Appendage with Extraluminal Device, Open Approach
        "02L70DK",  # Occlusion of Left Atrial Appendage with Intraluminal Device, Open Approach
        "02L70ZK",  # Occlusion of Left Atrial Appendage, Open Approach
        "02LR0ZT",  # Occlusion of Ductus Arteriosus, Open Approach
        "02LS0ZZ",  # Occlusion of Right Pulmonary Vein, Open Approach
        "02LT0ZZ",  # Occlusion of Left Pulmonary Vein, Open Approach
        "02N40ZZ",  # Release Coronary Vein, Open Approach
        "02N60ZZ",  # Release Right Atrium, Open Approach
        "02N70ZZ",  # Release Left Atrium, Open Approach
        "02N80ZZ",  # Release Conduction Mechanism, Open Approach
        "02N90ZZ",  # Release Chordae Tendineae, Open Approach
        "02ND0ZZ",  # Release Papillary Muscle, Open Approach
        "02NF0ZZ",  # Release Aortic Valve, Open Approach
        "02NG0ZZ",  # Release Mitral Valve, Open Approach
        "02NH0ZZ",  # Release Pulmonary Valve, Open Approach
        "02NJ0ZZ",  # Release Tricuspid Valve, Open Approach
        "02NK0ZZ",  # Release Right Ventricle, Open Approach
        "02NL0ZZ",  # Release Left Ventricle, Open Approach
        "02PA02Z",  # Removal of Monitoring Device from Heart, Open Approach
        "02PA03Z",  # Removal of Infusion Device from Heart, Open Approach
        "02PA07Z",  # Removal of Autologous Tissue Substitute from Heart, Open Approach
        "02PA08Z",  # Removal of Zooplastic Tissue from Heart, Open Approach
        "02PA0CZ",  # Removal of Extraluminal Device from Heart, Open Approach
        "02PA0DZ",  # Removal of Intraluminal Device from Heart, Open Approach
        "02PA0JZ",  # Removal of Synthetic Substitute from Heart, Open Approach
        "02PA0KZ",  # Removal of Nonautologous Tissue Substitute from Heart, Open Approach
        "02PA0QZ",  # Removal of Implantable Heart Assist System from Heart, Open Approach
        "02PA0RZ",  # Removal of Short-term External Heart Assist System from Heart, Open Approach
        "02Q00ZZ",  # Repair Coronary Artery, One Artery, Open Approach
        "02Q10ZZ",  # Repair Coronary Artery, Two Arteries, Open Approach
        "02Q20ZZ",  # Repair Coronary Artery, Three Arteries, Open Approach
        "02Q30ZZ",  # Repair Coronary Artery, Four or More Arteries, Open Approach
        "02Q40ZZ",  # Repair Coronary Vein, Open Approach
        "02Q50ZZ",  # Repair Atrial Septum, Open Approach
        "02Q60ZZ",  # Repair Right Atrium, Open Approach
        "02Q70ZZ",  # Repair Left Atrium, Open Approach
        "02Q80ZZ",  # Repair Conduction Mechanism, Open Approach
        "02Q90ZZ",  # Repair Chordae Tendineae, Open Approach
        "02QA0ZZ",  # Repair Heart, Open Approach
        "02QB0ZZ",  # Repair Right Heart, Open Approach
        "02QC0ZZ",  # Repair Left Heart, Open Approach
        "02QD0ZZ",  # Repair Papillary Muscle, Open Approach
        "02QF0ZZ",  # Repair Aortic Valve, Open Approach
        "02QG0ZZ",  # Repair Mitral Valve, Open Approach
        "02QH0ZZ",  # Repair Pulmonary Valve, Open Approach
        "02QJ0ZZ",  # Repair Tricuspid Valve, Open Approach
        "02QK0ZZ",  # Repair Right Ventricle, Open Approach
        "02QL0ZZ",  # Repair Left Ventricle, Open Approach
        "02QM0ZZ",  # Repair Ventricular Septum, Open Approach
        "02QN0ZZ",  # Repair Pericardium, Open Approach
        "02QW0ZZ",  # Repair Thoracic Aorta, Descending, Open Approach
        "02QW3ZZ",  # Repair Thoracic Aorta, Descending, Percutaneous Approach
        "02QW4ZZ",  # Repair Thoracic Aorta, Descending, Percutaneous Endoscopic Approach
        "02QX0ZZ",  # Repair Thoracic Aorta, Ascending/Arch, Open Approach
        "02QX3ZZ",  # Repair Thoracic Aorta, Ascending/Arch, Percutaneous Approach
        "02QX4ZZ",  # Repair Thoracic Aorta, Ascending/Arch, Percutaneous Endoscopic Approach
        "02R507Z",  # Replacement of Atrial Septum with Autologous Tissue Substitute, Open Approach
        "02R508Z",  # Replacement of Atrial Septum with Zooplastic Tissue, Open Approach
        "02R50JZ",  # Replacement of Atrial Septum with Synthetic Substitute, Open Approach
        "02R50KZ",  # Replacement of Atrial Septum with Nonautologous Tissue Substitute, Open Approach
        "02R607Z",  # Replacement of Right Atrium with Autologous Tissue Substitute, Open Approach
        "02R608Z",  # Replacement of Right Atrium with Zooplastic Tissue, Open Approach
        "02R60JZ",  # Replacement of Right Atrium with Synthetic Substitute, Open Approach
        "02R60KZ",  # Replacement of Right Atrium with Nonautologous Tissue Substitute, Open Approach
        "02R707Z",  # Replacement of Left Atrium with Autologous Tissue Substitute, Open Approach
        "02R708Z",  # Replacement of Left Atrium with Zooplastic Tissue, Open Approach
        "02R70JZ",  # Replacement of Left Atrium with Synthetic Substitute, Open Approach
        "02R70KZ",  # Replacement of Left Atrium with Nonautologous Tissue Substitute, Open Approach
        "02R907Z",  # Replacement of Chordae Tendineae with Autologous Tissue Substitute, Open Approach
        "02R908Z",  # Replacement of Chordae Tendineae with Zooplastic Tissue, Open Approach
        "02R90JZ",  # Replacement of Chordae Tendineae with Synthetic Substitute, Open Approach
        "02R90KZ",  # Replacement of Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach
        "02RD07Z",  # Replacement of Papillary Muscle with Autologous Tissue Substitute, Open Approach
        "02RD08Z",  # Replacement of Papillary Muscle with Zooplastic Tissue, Open Approach
        "02RD0JZ",  # Replacement of Papillary Muscle with Synthetic Substitute, Open Approach
        "02RD0KZ",  # Replacement of Papillary Muscle with Nonautologous Tissue Substitute, Open Approach
        "02RF07Z",  # Replacement of Aortic Valve with Autologous Tissue Substitute, Open Approach
        "02RF08Z",  # Replacement of Aortic Valve with Zooplastic Tissue, Open Approach
        "02RF0JZ",  # Replacement of Aortic Valve with Synthetic Substitute, Open Approach
        "02RF0KZ",  # Replacement of Aortic Valve with Nonautologous Tissue Substitute, Open Approach
        "02RG07Z",  # Replacement of Mitral Valve with Autologous Tissue Substitute, Open Approach
        "02RG08Z",  # Replacement of Mitral Valve with Zooplastic Tissue, Open Approach
        "02RG0JZ",  # Replacement of Mitral Valve with Synthetic Substitute, Open Approach
        "02RG0KZ",  # Replacement of Mitral Valve with Nonautologous Tissue Substitute, Open Approach
        "02RH07Z",  # Replacement of Pulmonary Valve with Autologous Tissue Substitute, Open Approach
        "02RH08Z",  # Replacement of Pulmonary Valve with Zooplastic Tissue, Open Approach
        "02RH0JZ",  # Replacement of Pulmonary Valve with Synthetic Substitute, Open Approach
        "02RH0KZ",  # Replacement of Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach
        "02RJ07Z",  # Replacement of Tricuspid Valve with Autologous Tissue Substitute, Open Approach
        "02RJ08Z",  # Replacement of Tricuspid Valve with Zooplastic Tissue, Open Approach
        "02RJ0JZ",  # Replacement of Tricuspid Valve with Synthetic Substitute, Open Approach
        "02RJ0KZ",  # Replacement of Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach
        "02RK07Z",  # Replacement of Right Ventricle with Autologous Tissue Substitute, Open Approach
        "02RK08Z",  # Replacement of Right Ventricle with Zooplastic Tissue, Open Approach
        "02RK0JZ",  # Replacement of Right Ventricle with Synthetic Substitute, Open Approach
        "02RK0KZ",  # Replacement of Right Ventricle with Nonautologous Tissue Substitute, Open Approach
        "02RL07Z",  # Replacement of Left Ventricle with Autologous Tissue Substitute, Open Approach
        "02RL08Z",  # Replacement of Left Ventricle with Zooplastic Tissue, Open Approach
        "02RL0JZ",  # Replacement of Left Ventricle with Synthetic Substitute, Open Approach
        "02RL0KZ",  # Replacement of Left Ventricle with Nonautologous Tissue Substitute, Open Approach
        "02RM07Z",  # Replacement of Ventricular Septum with Autologous Tissue Substitute, Open Approach
        "02RM08Z",  # Replacement of Ventricular Septum with Zooplastic Tissue, Open Approach
        "02RM0JZ",  # Replacement of Ventricular Septum with Synthetic Substitute, Open Approach
        "02RM0KZ",  # Replacement of Ventricular Septum with Nonautologous Tissue Substitute, Open Approach
        "02RN07Z",  # Replacement of Pericardium with Autologous Tissue Substitute, Open Approach
        "02RN08Z",  # Replacement of Pericardium with Zooplastic Tissue, Open Approach
        "02RN0JZ",  # Replacement of Pericardium with Synthetic Substitute, Open Approach
        "02RN0KZ",  # Replacement of Pericardium with Nonautologous Tissue Substitute, Open Approach
        "02RP0JZ",  # Replacement of Pulmonary Trunk with Synthetic Substitute, Open Approach
        "02RQ07Z",  # Replacement of Right Pulmonary Artery with Autologous Tissue Substitute, Open Approach
        "02RQ0JZ",  # Replacement of Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "02RR07Z",  # Replacement of Left Pulmonary Artery with Autologous Tissue Substitute, Open Approach
        "02RR0JZ",  # Replacement of Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "02SP0ZZ",  # Reposition Pulmonary Trunk, Open Approach
        "02SW0ZZ",  # Reposition Thoracic Aorta, Descending, Open Approach
        "02T80ZZ",  # Resection of Conduction Mechanism, Open Approach
        "02T90ZZ",  # Resection of Chordae Tendineae, Open Approach
        "02TD0ZZ",  # Resection of Papillary Muscle, Open Approach
        "02TH0ZZ",  # Resection of Pulmonary Valve, Open Approach
        "02U507Z",  # Supplement Atrial Septum with Autologous Tissue Substitute, Open Approach
        "02U508Z",  # Supplement Atrial Septum with Zooplastic Tissue, Open Approach
        "02U50JZ",  # Supplement Atrial Septum with Synthetic Substitute, Open Approach
        "02U50KZ",  # Supplement Atrial Septum with Nonautologous Tissue Substitute, Open Approach
        "02U607Z",  # Supplement Right Atrium with Autologous Tissue Substitute, Open Approach
        "02U608Z",  # Supplement Right Atrium with Zooplastic Tissue, Open Approach
        "02U60JZ",  # Supplement Right Atrium with Synthetic Substitute, Open Approach
        "02U60KZ",  # Supplement Right Atrium with Nonautologous Tissue Substitute, Open Approach
        "02U707Z",  # Supplement Left Atrium with Autologous Tissue Substitute, Open Approach
        "02U708Z",  # Supplement Left Atrium with Zooplastic Tissue, Open Approach
        "02U70JZ",  # Supplement Left Atrium with Synthetic Substitute, Open Approach
        "02U70KZ",  # Supplement Left Atrium with Nonautologous Tissue Substitute, Open Approach
        "02U907Z",  # Supplement Chordae Tendineae with Autologous Tissue Substitute, Open Approach
        "02U908Z",  # Supplement Chordae Tendineae with Zooplastic Tissue, Open Approach
        "02U90JZ",  # Supplement Chordae Tendineae with Synthetic Substitute, Open Approach
        "02U90KZ",  # Supplement Chordae Tendineae with Nonautologous Tissue Substitute, Open Approach
        "02UA07Z",  # Supplement Heart with Autologous Tissue Substitute, Open Approach
        "02UA08Z",  # Supplement Heart with Zooplastic Tissue, Open Approach
        "02UA0JZ",  # Supplement Heart with Synthetic Substitute, Open Approach
        "02UA0KZ",  # Supplement Heart with Nonautologous Tissue Substitute, Open Approach
        "02UD07Z",  # Supplement Papillary Muscle with Autologous Tissue Substitute, Open Approach
        "02UD08Z",  # Supplement Papillary Muscle with Zooplastic Tissue, Open Approach
        "02UD0JZ",  # Supplement Papillary Muscle with Synthetic Substitute, Open Approach
        "02UD0KZ",  # Supplement Papillary Muscle with Nonautologous Tissue Substitute, Open Approach
        "02UF07Z",  # Supplement Aortic Valve with Autologous Tissue Substitute, Open Approach
        "02UF08Z",  # Supplement Aortic Valve with Zooplastic Tissue, Open Approach
        "02UF0JZ",  # Supplement Aortic Valve with Synthetic Substitute, Open Approach
        "02UF0KZ",  # Supplement Aortic Valve with Nonautologous Tissue Substitute, Open Approach
        "02UG07Z",  # Supplement Mitral Valve with Autologous Tissue Substitute, Open Approach
        "02UG08Z",  # Supplement Mitral Valve with Zooplastic Tissue, Open Approach
        "02UG0JZ",  # Supplement Mitral Valve with Synthetic Substitute, Open Approach
        "02UG0KZ",  # Supplement Mitral Valve with Nonautologous Tissue Substitute, Open Approach
        "02UH07Z",  # Supplement Pulmonary Valve with Autologous Tissue Substitute, Open Approach
        "02UH08Z",  # Supplement Pulmonary Valve with Zooplastic Tissue, Open Approach
        "02UH0JZ",  # Supplement Pulmonary Valve with Synthetic Substitute, Open Approach
        "02UH0KZ",  # Supplement Pulmonary Valve with Nonautologous Tissue Substitute, Open Approach
        "02UJ07Z",  # Supplement Tricuspid Valve with Autologous Tissue Substitute, Open Approach
        "02UJ08Z",  # Supplement Tricuspid Valve with Zooplastic Tissue, Open Approach
        "02UJ0JZ",  # Supplement Tricuspid Valve with Synthetic Substitute, Open Approach
        "02UJ0KZ",  # Supplement Tricuspid Valve with Nonautologous Tissue Substitute, Open Approach
        "02UK07Z",  # Supplement Right Ventricle with Autologous Tissue Substitute, Open Approach
        "02UK08Z",  # Supplement Right Ventricle with Zooplastic Tissue, Open Approach
        "02UK0JZ",  # Supplement Right Ventricle with Synthetic Substitute, Open Approach
        "02UK0KZ",  # Supplement Right Ventricle with Nonautologous Tissue Substitute, Open Approach
        "02UL07Z",  # Supplement Left Ventricle with Autologous Tissue Substitute, Open Approach
        "02UL08Z",  # Supplement Left Ventricle with Zooplastic Tissue, Open Approach
        "02UL0JZ",  # Supplement Left Ventricle with Synthetic Substitute, Open Approach
        "02UL0KZ",  # Supplement Left Ventricle with Nonautologous Tissue Substitute, Open Approach
        "02UM07Z",  # Supplement Ventricular Septum with Autologous Tissue Substitute, Open Approach
        "02UM08Z",  # Supplement Ventricular Septum with Zooplastic Tissue, Open Approach
        "02UM0JZ",  # Supplement Ventricular Septum with Synthetic Substitute, Open Approach
        "02UM0KZ",  # Supplement Ventricular Septum with Nonautologous Tissue Substitute, Open Approach
        "02UN07Z",  # Supplement Pericardium with Autologous Tissue Substitute, Open Approach
        "02UN08Z",  # Supplement Pericardium with Zooplastic Tissue, Open Approach
        "02UN0JZ",  # Supplement Pericardium with Synthetic Substitute, Open Approach
        "02UN0KZ",  # Supplement Pericardium with Nonautologous Tissue Substitute, Open Approach
        "02VA0CZ",  # Restriction of Heart with Extraluminal Device, Open Approach
        "02VA0ZZ",  # Restriction of Heart, Open Approach
        "02VR0ZT",  # Restriction of Ductus Arteriosus, Open Approach
        "02WA02Z",  # Revision of Monitoring Device in Heart, Open Approach
        "02WA03Z",  # Revision of Infusion Device in Heart, Open Approach
        "02WA07Z",  # Revision of Autologous Tissue Substitute in Heart, Open Approach
        "02WA08Z",  # Revision of Zooplastic Tissue in Heart, Open Approach
        "02WA0CZ",  # Revision of Extraluminal Device in Heart, Open Approach
        "02WA0DZ",  # Revision of Intraluminal Device in Heart, Open Approach
        "02WA0KZ",  # Revision of Nonautologous Tissue Substitute in Heart, Open Approach
        "02WA0QZ",  # Revision of Implantable Heart Assist System in Heart, Open Approach
        "02WA0RZ",  # Revision of Short-term External Heart Assist System in Heart, Open Approach
        "02YA0Z0",  # Transplantation of Heart, Allogeneic, Open Approach
        "02YA0Z1",  # Transplantation of Heart, Syngeneic, Open Approach
        "02YA0Z2",  # Transplantation of Heart, Zooplastic, Open Approach
        "0312090",  # Bypass Innominate Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0312091",  # Bypass Innominate Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0312092",  # Bypass Innominate Artery to Bilateral Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0312093",  # Bypass Innominate Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0312094",  # Bypass Innominate Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0312095",  # Bypass Innominate Artery to Bilateral Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0312096",  # Bypass Innominate Artery to Right Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0312097",  # Bypass Innominate Artery to Left Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0312098",  # Bypass Innominate Artery to Bilateral Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0312099",  # Bypass Innominate Artery to Right Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031209B",  # Bypass Innominate Artery to Left Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031209C",  # Bypass Innominate Artery to Bilateral Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031209D",  # Bypass Innominate Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031209F",  # Bypass Innominate Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031209J",  # Bypass Innominate Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031209K",  # Bypass Innominate Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "03120A0",  # Bypass Innominate Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A1",  # Bypass Innominate Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A2",  # Bypass Innominate Artery to Bilateral Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A3",  # Bypass Innominate Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A4",  # Bypass Innominate Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A5",  # Bypass Innominate Artery to Bilateral Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03120A6",  # Bypass Innominate Artery to Right Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120A7",  # Bypass Innominate Artery to Left Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120A8",  # Bypass Innominate Artery to Bilateral Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120A9",  # Bypass Innominate Artery to Right Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120AB",  # Bypass Innominate Artery to Left Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120AC",  # Bypass Innominate Artery to Bilateral Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03120AD",  # Bypass Innominate Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03120AF",  # Bypass Innominate Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03120AJ",  # Bypass Innominate Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03120AK",  # Bypass Innominate Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03120J0",  # Bypass Innominate Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03120J1",  # Bypass Innominate Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03120J2",  # Bypass Innominate Artery to Bilateral Upper Arm Artery with Synthetic Substitute, Open Approach
        "03120J3",  # Bypass Innominate Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03120J4",  # Bypass Innominate Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03120J5",  # Bypass Innominate Artery to Bilateral Lower Arm Artery with Synthetic Substitute, Open Approach
        "03120J6",  # Bypass Innominate Artery to Right Upper Leg Artery with Synthetic Substitute, Open Approach
        "03120J7",  # Bypass Innominate Artery to Left Upper Leg Artery with Synthetic Substitute, Open Approach
        "03120J8",  # Bypass Innominate Artery to Bilateral Upper Leg Artery with Synthetic Substitute, Open Approach
        "03120J9",  # Bypass Innominate Artery to Right Lower Leg Artery with Synthetic Substitute, Open Approach
        "03120JB",  # Bypass Innominate Artery to Left Lower Leg Artery with Synthetic Substitute, Open Approach
        "03120JC",  # Bypass Innominate Artery to Bilateral Lower Leg Artery with Synthetic Substitute, Open Approach
        "03120JD",  # Bypass Innominate Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03120JF",  # Bypass Innominate Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03120JJ",  # Bypass Innominate Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "03120JK",  # Bypass Innominate Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "03120K0",  # Bypass Innominate Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K1",  # Bypass Innominate Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K2",  # Bypass Innominate Artery to Bilateral Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K3",  # Bypass Innominate Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K4",  # Bypass Innominate Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K5",  # Bypass Innominate Artery to Bilateral Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K6",  # Bypass Innominate Artery to Right Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K7",  # Bypass Innominate Artery to Left Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K8",  # Bypass Innominate Artery to Bilateral Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120K9",  # Bypass Innominate Artery to Right Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120KB",  # Bypass Innominate Artery to Left Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120KC",  # Bypass Innominate Artery to Bilateral Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03120KD",  # Bypass Innominate Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03120KF",  # Bypass Innominate Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03120KJ",  # Bypass Innominate Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03120KK",  # Bypass Innominate Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03120Z0",  # Bypass Innominate Artery to Right Upper Arm Artery, Open Approach
        "03120Z1",  # Bypass Innominate Artery to Left Upper Arm Artery, Open Approach
        "03120Z2",  # Bypass Innominate Artery to Bilateral Upper Arm Artery, Open Approach
        "03120Z3",  # Bypass Innominate Artery to Right Lower Arm Artery, Open Approach
        "03120Z4",  # Bypass Innominate Artery to Left Lower Arm Artery, Open Approach
        "03120Z5",  # Bypass Innominate Artery to Bilateral Lower Arm Artery, Open Approach
        "03120Z6",  # Bypass Innominate Artery to Right Upper Leg Artery, Open Approach
        "03120Z7",  # Bypass Innominate Artery to Left Upper Leg Artery, Open Approach
        "03120Z8",  # Bypass Innominate Artery to Bilateral Upper Leg Artery, Open Approach
        "03120Z9",  # Bypass Innominate Artery to Right Lower Leg Artery, Open Approach
        "03120ZB",  # Bypass Innominate Artery to Left Lower Leg Artery, Open Approach
        "03120ZC",  # Bypass Innominate Artery to Bilateral Lower Leg Artery, Open Approach
        "03120ZD",  # Bypass Innominate Artery to Upper Arm Vein, Open Approach
        "03120ZF",  # Bypass Innominate Artery to Lower Arm Vein, Open Approach
        "03120ZJ",  # Bypass Innominate Artery to Right Extracranial Artery, Open Approach
        "03120ZK",  # Bypass Innominate Artery to Left Extracranial Artery, Open Approach
        "0313090",  # Bypass Right Subclavian Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0313091",  # Bypass Right Subclavian Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0313092",  # Bypass Right Subclavian Artery to Bilateral Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0313093",  # Bypass Right Subclavian Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0313094",  # Bypass Right Subclavian Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0313095",  # Bypass Right Subclavian Artery to Bilateral Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0313096",  # Bypass Right Subclavian Artery to Right Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0313097",  # Bypass Right Subclavian Artery to Left Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0313098",  # Bypass Right Subclavian Artery to Bilateral Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0313099",  # Bypass Right Subclavian Artery to Right Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031309B",  # Bypass Right Subclavian Artery to Left Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031309C",  # Bypass Right Subclavian Artery to Bilateral Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031309D",  # Bypass Right Subclavian Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031309F",  # Bypass Right Subclavian Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031309J",  # Bypass Right Subclavian Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031309K",  # Bypass Right Subclavian Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031309M",  # Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "031309N",  # Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "03130A0",  # Bypass Right Subclavian Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A1",  # Bypass Right Subclavian Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A2",  # Bypass Right Subclavian Artery to Bilateral Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A3",  # Bypass Right Subclavian Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A4",  # Bypass Right Subclavian Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A5",  # Bypass Right Subclavian Artery to Bilateral Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03130A6",  # Bypass Right Subclavian Artery to Right Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130A7",  # Bypass Right Subclavian Artery to Left Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130A8",  # Bypass Right Subclavian Artery to Bilateral Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130A9",  # Bypass Right Subclavian Artery to Right Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130AB",  # Bypass Right Subclavian Artery to Left Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130AC",  # Bypass Right Subclavian Artery to Bilateral Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03130AD",  # Bypass Right Subclavian Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03130AF",  # Bypass Right Subclavian Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03130AJ",  # Bypass Right Subclavian Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03130AK",  # Bypass Right Subclavian Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03130AM",  # Bypass Right Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "03130AN",  # Bypass Right Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "03130J0",  # Bypass Right Subclavian Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03130J1",  # Bypass Right Subclavian Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03130J2",  # Bypass Right Subclavian Artery to Bilateral Upper Arm Artery with Synthetic Substitute, Open Approach
        "03130J3",  # Bypass Right Subclavian Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03130J4",  # Bypass Right Subclavian Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03130J5",  # Bypass Right Subclavian Artery to Bilateral Lower Arm Artery with Synthetic Substitute, Open Approach
        "03130J6",  # Bypass Right Subclavian Artery to Right Upper Leg Artery with Synthetic Substitute, Open Approach
        "03130J7",  # Bypass Right Subclavian Artery to Left Upper Leg Artery with Synthetic Substitute, Open Approach
        "03130J8",  # Bypass Right Subclavian Artery to Bilateral Upper Leg Artery with Synthetic Substitute, Open Approach
        "03130J9",  # Bypass Right Subclavian Artery to Right Lower Leg Artery with Synthetic Substitute, Open Approach
        "03130JB",  # Bypass Right Subclavian Artery to Left Lower Leg Artery with Synthetic Substitute, Open Approach
        "03130JC",  # Bypass Right Subclavian Artery to Bilateral Lower Leg Artery with Synthetic Substitute, Open Approach
        "03130JD",  # Bypass Right Subclavian Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03130JF",  # Bypass Right Subclavian Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03130JJ",  # Bypass Right Subclavian Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "03130JK",  # Bypass Right Subclavian Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "03130JM",  # Bypass Right Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "03130JN",  # Bypass Right Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "03130K0",  # Bypass Right Subclavian Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K1",  # Bypass Right Subclavian Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K2",  # Bypass Right Subclavian Artery to Bilateral Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K3",  # Bypass Right Subclavian Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K4",  # Bypass Right Subclavian Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K5",  # Bypass Right Subclavian Artery to Bilateral Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K6",  # Bypass Right Subclavian Artery to Right Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K7",  # Bypass Right Subclavian Artery to Left Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K8",  # Bypass Right Subclavian Artery to Bilateral Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130K9",  # Bypass Right Subclavian Artery to Right Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KB",  # Bypass Right Subclavian Artery to Left Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KC",  # Bypass Right Subclavian Artery to Bilateral Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KD",  # Bypass Right Subclavian Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03130KF",  # Bypass Right Subclavian Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03130KJ",  # Bypass Right Subclavian Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KK",  # Bypass Right Subclavian Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KM",  # Bypass Right Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "03130KN",  # Bypass Right Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "03130Z0",  # Bypass Right Subclavian Artery to Right Upper Arm Artery, Open Approach
        "03130Z1",  # Bypass Right Subclavian Artery to Left Upper Arm Artery, Open Approach
        "03130Z2",  # Bypass Right Subclavian Artery to Bilateral Upper Arm Artery, Open Approach
        "03130Z3",  # Bypass Right Subclavian Artery to Right Lower Arm Artery, Open Approach
        "03130Z4",  # Bypass Right Subclavian Artery to Left Lower Arm Artery, Open Approach
        "03130Z5",  # Bypass Right Subclavian Artery to Bilateral Lower Arm Artery, Open Approach
        "03130Z6",  # Bypass Right Subclavian Artery to Right Upper Leg Artery, Open Approach
        "03130Z7",  # Bypass Right Subclavian Artery to Left Upper Leg Artery, Open Approach
        "03130Z8",  # Bypass Right Subclavian Artery to Bilateral Upper Leg Artery, Open Approach
        "03130Z9",  # Bypass Right Subclavian Artery to Right Lower Leg Artery, Open Approach
        "03130ZB",  # Bypass Right Subclavian Artery to Left Lower Leg Artery, Open Approach
        "03130ZC",  # Bypass Right Subclavian Artery to Bilateral Lower Leg Artery, Open Approach
        "03130ZD",  # Bypass Right Subclavian Artery to Upper Arm Vein, Open Approach
        "03130ZF",  # Bypass Right Subclavian Artery to Lower Arm Vein, Open Approach
        "03130ZJ",  # Bypass Right Subclavian Artery to Right Extracranial Artery, Open Approach
        "03130ZK",  # Bypass Right Subclavian Artery to Left Extracranial Artery, Open Approach
        "03130ZM",  # Bypass Right Subclavian Artery to Right Pulmonary Artery, Open Approach
        "03130ZN",  # Bypass Right Subclavian Artery to Left Pulmonary Artery, Open Approach
        "0314090",  # Bypass Left Subclavian Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0314091",  # Bypass Left Subclavian Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0314092",  # Bypass Left Subclavian Artery to Bilateral Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0314093",  # Bypass Left Subclavian Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0314094",  # Bypass Left Subclavian Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0314095",  # Bypass Left Subclavian Artery to Bilateral Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0314096",  # Bypass Left Subclavian Artery to Right Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0314097",  # Bypass Left Subclavian Artery to Left Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0314098",  # Bypass Left Subclavian Artery to Bilateral Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0314099",  # Bypass Left Subclavian Artery to Right Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031409B",  # Bypass Left Subclavian Artery to Left Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031409C",  # Bypass Left Subclavian Artery to Bilateral Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031409D",  # Bypass Left Subclavian Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031409F",  # Bypass Left Subclavian Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031409J",  # Bypass Left Subclavian Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031409K",  # Bypass Left Subclavian Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031409M",  # Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "031409N",  # Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Venous Tissue, Open Approach
        "03140A0",  # Bypass Left Subclavian Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A1",  # Bypass Left Subclavian Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A2",  # Bypass Left Subclavian Artery to Bilateral Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A3",  # Bypass Left Subclavian Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A4",  # Bypass Left Subclavian Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A5",  # Bypass Left Subclavian Artery to Bilateral Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03140A6",  # Bypass Left Subclavian Artery to Right Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140A7",  # Bypass Left Subclavian Artery to Left Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140A8",  # Bypass Left Subclavian Artery to Bilateral Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140A9",  # Bypass Left Subclavian Artery to Right Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140AB",  # Bypass Left Subclavian Artery to Left Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140AC",  # Bypass Left Subclavian Artery to Bilateral Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03140AD",  # Bypass Left Subclavian Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03140AF",  # Bypass Left Subclavian Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03140AJ",  # Bypass Left Subclavian Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03140AK",  # Bypass Left Subclavian Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03140AM",  # Bypass Left Subclavian Artery to Right Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "03140AN",  # Bypass Left Subclavian Artery to Left Pulmonary Artery with Autologous Arterial Tissue, Open Approach
        "03140J0",  # Bypass Left Subclavian Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03140J1",  # Bypass Left Subclavian Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03140J2",  # Bypass Left Subclavian Artery to Bilateral Upper Arm Artery with Synthetic Substitute, Open Approach
        "03140J3",  # Bypass Left Subclavian Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03140J4",  # Bypass Left Subclavian Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03140J5",  # Bypass Left Subclavian Artery to Bilateral Lower Arm Artery with Synthetic Substitute, Open Approach
        "03140J6",  # Bypass Left Subclavian Artery to Right Upper Leg Artery with Synthetic Substitute, Open Approach
        "03140J7",  # Bypass Left Subclavian Artery to Left Upper Leg Artery with Synthetic Substitute, Open Approach
        "03140J8",  # Bypass Left Subclavian Artery to Bilateral Upper Leg Artery with Synthetic Substitute, Open Approach
        "03140J9",  # Bypass Left Subclavian Artery to Right Lower Leg Artery with Synthetic Substitute, Open Approach
        "03140JB",  # Bypass Left Subclavian Artery to Left Lower Leg Artery with Synthetic Substitute, Open Approach
        "03140JC",  # Bypass Left Subclavian Artery to Bilateral Lower Leg Artery with Synthetic Substitute, Open Approach
        "03140JD",  # Bypass Left Subclavian Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03140JF",  # Bypass Left Subclavian Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03140JJ",  # Bypass Left Subclavian Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "03140JK",  # Bypass Left Subclavian Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "03140JM",  # Bypass Left Subclavian Artery to Right Pulmonary Artery with Synthetic Substitute, Open Approach
        "03140JN",  # Bypass Left Subclavian Artery to Left Pulmonary Artery with Synthetic Substitute, Open Approach
        "03140K0",  # Bypass Left Subclavian Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K1",  # Bypass Left Subclavian Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K2",  # Bypass Left Subclavian Artery to Bilateral Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K3",  # Bypass Left Subclavian Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K4",  # Bypass Left Subclavian Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K5",  # Bypass Left Subclavian Artery to Bilateral Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K6",  # Bypass Left Subclavian Artery to Right Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K7",  # Bypass Left Subclavian Artery to Left Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K8",  # Bypass Left Subclavian Artery to Bilateral Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140K9",  # Bypass Left Subclavian Artery to Right Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KB",  # Bypass Left Subclavian Artery to Left Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KC",  # Bypass Left Subclavian Artery to Bilateral Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KD",  # Bypass Left Subclavian Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03140KF",  # Bypass Left Subclavian Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03140KJ",  # Bypass Left Subclavian Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KK",  # Bypass Left Subclavian Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KM",  # Bypass Left Subclavian Artery to Right Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "03140KN",  # Bypass Left Subclavian Artery to Left Pulmonary Artery with Nonautologous Tissue Substitute, Open Approach
        "03140Z0",  # Bypass Left Subclavian Artery to Right Upper Arm Artery, Open Approach
        "03140Z1",  # Bypass Left Subclavian Artery to Left Upper Arm Artery, Open Approach
        "03140Z2",  # Bypass Left Subclavian Artery to Bilateral Upper Arm Artery, Open Approach
        "03140Z3",  # Bypass Left Subclavian Artery to Right Lower Arm Artery, Open Approach
        "03140Z4",  # Bypass Left Subclavian Artery to Left Lower Arm Artery, Open Approach
        "03140Z5",  # Bypass Left Subclavian Artery to Bilateral Lower Arm Artery, Open Approach
        "03140Z6",  # Bypass Left Subclavian Artery to Right Upper Leg Artery, Open Approach
        "03140Z7",  # Bypass Left Subclavian Artery to Left Upper Leg Artery, Open Approach
        "03140Z8",  # Bypass Left Subclavian Artery to Bilateral Upper Leg Artery, Open Approach
        "03140Z9",  # Bypass Left Subclavian Artery to Right Lower Leg Artery, Open Approach
        "03140ZB",  # Bypass Left Subclavian Artery to Left Lower Leg Artery, Open Approach
        "03140ZC",  # Bypass Left Subclavian Artery to Bilateral Lower Leg Artery, Open Approach
        "03140ZD",  # Bypass Left Subclavian Artery to Upper Arm Vein, Open Approach
        "03140ZF",  # Bypass Left Subclavian Artery to Lower Arm Vein, Open Approach
        "03140ZJ",  # Bypass Left Subclavian Artery to Right Extracranial Artery, Open Approach
        "03140ZK",  # Bypass Left Subclavian Artery to Left Extracranial Artery, Open Approach
        "03140ZM",  # Bypass Left Subclavian Artery to Right Pulmonary Artery, Open Approach
        "03140ZN",  # Bypass Left Subclavian Artery to Left Pulmonary Artery, Open Approach
        "0315090",  # Bypass Right Axillary Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0315091",  # Bypass Right Axillary Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0315092",  # Bypass Right Axillary Artery to Bilateral Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0315093",  # Bypass Right Axillary Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0315094",  # Bypass Right Axillary Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0315095",  # Bypass Right Axillary Artery to Bilateral Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0315096",  # Bypass Right Axillary Artery to Right Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0315097",  # Bypass Right Axillary Artery to Left Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0315098",  # Bypass Right Axillary Artery to Bilateral Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0315099",  # Bypass Right Axillary Artery to Right Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031509B",  # Bypass Right Axillary Artery to Left Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031509C",  # Bypass Right Axillary Artery to Bilateral Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031509D",  # Bypass Right Axillary Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031509F",  # Bypass Right Axillary Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031509J",  # Bypass Right Axillary Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031509K",  # Bypass Right Axillary Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031509T",  # Bypass Right Axillary Artery to Abdominal Artery with Autologous Venous Tissue, Open Approach
        "031509V",  # Bypass Right Axillary Artery to Superior Vena Cava with Autologous Venous Tissue, Open Approach
        "03150A0",  # Bypass Right Axillary Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A1",  # Bypass Right Axillary Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A2",  # Bypass Right Axillary Artery to Bilateral Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A3",  # Bypass Right Axillary Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A4",  # Bypass Right Axillary Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A5",  # Bypass Right Axillary Artery to Bilateral Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03150A6",  # Bypass Right Axillary Artery to Right Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150A7",  # Bypass Right Axillary Artery to Left Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150A8",  # Bypass Right Axillary Artery to Bilateral Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150A9",  # Bypass Right Axillary Artery to Right Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150AB",  # Bypass Right Axillary Artery to Left Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150AC",  # Bypass Right Axillary Artery to Bilateral Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03150AD",  # Bypass Right Axillary Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03150AF",  # Bypass Right Axillary Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03150AJ",  # Bypass Right Axillary Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03150AK",  # Bypass Right Axillary Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03150AT",  # Bypass Right Axillary Artery to Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "03150AV",  # Bypass Right Axillary Artery to Superior Vena Cava with Autologous Arterial Tissue, Open Approach
        "03150J0",  # Bypass Right Axillary Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03150J1",  # Bypass Right Axillary Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03150J2",  # Bypass Right Axillary Artery to Bilateral Upper Arm Artery with Synthetic Substitute, Open Approach
        "03150J3",  # Bypass Right Axillary Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03150J4",  # Bypass Right Axillary Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03150J5",  # Bypass Right Axillary Artery to Bilateral Lower Arm Artery with Synthetic Substitute, Open Approach
        "03150J6",  # Bypass Right Axillary Artery to Right Upper Leg Artery with Synthetic Substitute, Open Approach
        "03150J7",  # Bypass Right Axillary Artery to Left Upper Leg Artery with Synthetic Substitute, Open Approach
        "03150J8",  # Bypass Right Axillary Artery to Bilateral Upper Leg Artery with Synthetic Substitute, Open Approach
        "03150J9",  # Bypass Right Axillary Artery to Right Lower Leg Artery with Synthetic Substitute, Open Approach
        "03150JB",  # Bypass Right Axillary Artery to Left Lower Leg Artery with Synthetic Substitute, Open Approach
        "03150JC",  # Bypass Right Axillary Artery to Bilateral Lower Leg Artery with Synthetic Substitute, Open Approach
        "03150JD",  # Bypass Right Axillary Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03150JF",  # Bypass Right Axillary Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03150JJ",  # Bypass Right Axillary Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "03150JK",  # Bypass Right Axillary Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "03150JT",  # Bypass Right Axillary Artery to Abdominal Artery with Synthetic Substitute, Open Approach
        "03150JV",  # Bypass Right Axillary Artery to Superior Vena Cava with Synthetic Substitute, Open Approach
        "03150K0",  # Bypass Right Axillary Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K1",  # Bypass Right Axillary Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K2",  # Bypass Right Axillary Artery to Bilateral Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K3",  # Bypass Right Axillary Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K4",  # Bypass Right Axillary Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K5",  # Bypass Right Axillary Artery to Bilateral Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K6",  # Bypass Right Axillary Artery to Right Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K7",  # Bypass Right Axillary Artery to Left Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K8",  # Bypass Right Axillary Artery to Bilateral Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150K9",  # Bypass Right Axillary Artery to Right Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KB",  # Bypass Right Axillary Artery to Left Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KC",  # Bypass Right Axillary Artery to Bilateral Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KD",  # Bypass Right Axillary Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03150KF",  # Bypass Right Axillary Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03150KJ",  # Bypass Right Axillary Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KK",  # Bypass Right Axillary Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KT",  # Bypass Right Axillary Artery to Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "03150KV",  # Bypass Right Axillary Artery to Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach
        "03150Z0",  # Bypass Right Axillary Artery to Right Upper Arm Artery, Open Approach
        "03150Z1",  # Bypass Right Axillary Artery to Left Upper Arm Artery, Open Approach
        "03150Z2",  # Bypass Right Axillary Artery to Bilateral Upper Arm Artery, Open Approach
        "03150Z3",  # Bypass Right Axillary Artery to Right Lower Arm Artery, Open Approach
        "03150Z4",  # Bypass Right Axillary Artery to Left Lower Arm Artery, Open Approach
        "03150Z5",  # Bypass Right Axillary Artery to Bilateral Lower Arm Artery, Open Approach
        "03150Z6",  # Bypass Right Axillary Artery to Right Upper Leg Artery, Open Approach
        "03150Z7",  # Bypass Right Axillary Artery to Left Upper Leg Artery, Open Approach
        "03150Z8",  # Bypass Right Axillary Artery to Bilateral Upper Leg Artery, Open Approach
        "03150Z9",  # Bypass Right Axillary Artery to Right Lower Leg Artery, Open Approach
        "03150ZB",  # Bypass Right Axillary Artery to Left Lower Leg Artery, Open Approach
        "03150ZC",  # Bypass Right Axillary Artery to Bilateral Lower Leg Artery, Open Approach
        "03150ZD",  # Bypass Right Axillary Artery to Upper Arm Vein, Open Approach
        "03150ZF",  # Bypass Right Axillary Artery to Lower Arm Vein, Open Approach
        "03150ZJ",  # Bypass Right Axillary Artery to Right Extracranial Artery, Open Approach
        "03150ZK",  # Bypass Right Axillary Artery to Left Extracranial Artery, Open Approach
        "03150ZT",  # Bypass Right Axillary Artery to Abdominal Artery, Open Approach
        "03150ZV",  # Bypass Right Axillary Artery to Superior Vena Cava, Open Approach
        "0316090",  # Bypass Left Axillary Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0316091",  # Bypass Left Axillary Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0316092",  # Bypass Left Axillary Artery to Bilateral Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0316093",  # Bypass Left Axillary Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0316094",  # Bypass Left Axillary Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0316095",  # Bypass Left Axillary Artery to Bilateral Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "0316096",  # Bypass Left Axillary Artery to Right Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0316097",  # Bypass Left Axillary Artery to Left Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0316098",  # Bypass Left Axillary Artery to Bilateral Upper Leg Artery with Autologous Venous Tissue, Open Approach
        "0316099",  # Bypass Left Axillary Artery to Right Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031609B",  # Bypass Left Axillary Artery to Left Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031609C",  # Bypass Left Axillary Artery to Bilateral Lower Leg Artery with Autologous Venous Tissue, Open Approach
        "031609D",  # Bypass Left Axillary Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031609F",  # Bypass Left Axillary Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031609J",  # Bypass Left Axillary Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031609K",  # Bypass Left Axillary Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031609T",  # Bypass Left Axillary Artery to Abdominal Artery with Autologous Venous Tissue, Open Approach
        "031609V",  # Bypass Left Axillary Artery to Superior Vena Cava with Autologous Venous Tissue, Open Approach
        "03160A0",  # Bypass Left Axillary Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A1",  # Bypass Left Axillary Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A2",  # Bypass Left Axillary Artery to Bilateral Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A3",  # Bypass Left Axillary Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A4",  # Bypass Left Axillary Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A5",  # Bypass Left Axillary Artery to Bilateral Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03160A6",  # Bypass Left Axillary Artery to Right Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160A7",  # Bypass Left Axillary Artery to Left Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160A8",  # Bypass Left Axillary Artery to Bilateral Upper Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160A9",  # Bypass Left Axillary Artery to Right Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160AB",  # Bypass Left Axillary Artery to Left Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160AC",  # Bypass Left Axillary Artery to Bilateral Lower Leg Artery with Autologous Arterial Tissue, Open Approach
        "03160AD",  # Bypass Left Axillary Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03160AF",  # Bypass Left Axillary Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03160AJ",  # Bypass Left Axillary Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03160AK",  # Bypass Left Axillary Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "03160AT",  # Bypass Left Axillary Artery to Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "03160AV",  # Bypass Left Axillary Artery to Superior Vena Cava with Autologous Arterial Tissue, Open Approach
        "03160J0",  # Bypass Left Axillary Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03160J1",  # Bypass Left Axillary Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03160J2",  # Bypass Left Axillary Artery to Bilateral Upper Arm Artery with Synthetic Substitute, Open Approach
        "03160J3",  # Bypass Left Axillary Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03160J4",  # Bypass Left Axillary Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03160J5",  # Bypass Left Axillary Artery to Bilateral Lower Arm Artery with Synthetic Substitute, Open Approach
        "03160J6",  # Bypass Left Axillary Artery to Right Upper Leg Artery with Synthetic Substitute, Open Approach
        "03160J7",  # Bypass Left Axillary Artery to Left Upper Leg Artery with Synthetic Substitute, Open Approach
        "03160J8",  # Bypass Left Axillary Artery to Bilateral Upper Leg Artery with Synthetic Substitute, Open Approach
        "03160J9",  # Bypass Left Axillary Artery to Right Lower Leg Artery with Synthetic Substitute, Open Approach
        "03160JB",  # Bypass Left Axillary Artery to Left Lower Leg Artery with Synthetic Substitute, Open Approach
        "03160JC",  # Bypass Left Axillary Artery to Bilateral Lower Leg Artery with Synthetic Substitute, Open Approach
        "03160JD",  # Bypass Left Axillary Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03160JF",  # Bypass Left Axillary Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03160JJ",  # Bypass Left Axillary Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "03160JK",  # Bypass Left Axillary Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "03160JT",  # Bypass Left Axillary Artery to Abdominal Artery with Synthetic Substitute, Open Approach
        "03160JV",  # Bypass Left Axillary Artery to Superior Vena Cava with Synthetic Substitute, Open Approach
        "03160K0",  # Bypass Left Axillary Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K1",  # Bypass Left Axillary Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K2",  # Bypass Left Axillary Artery to Bilateral Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K3",  # Bypass Left Axillary Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K4",  # Bypass Left Axillary Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K5",  # Bypass Left Axillary Artery to Bilateral Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K6",  # Bypass Left Axillary Artery to Right Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K7",  # Bypass Left Axillary Artery to Left Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K8",  # Bypass Left Axillary Artery to Bilateral Upper Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160K9",  # Bypass Left Axillary Artery to Right Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KB",  # Bypass Left Axillary Artery to Left Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KC",  # Bypass Left Axillary Artery to Bilateral Lower Leg Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KD",  # Bypass Left Axillary Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03160KF",  # Bypass Left Axillary Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03160KJ",  # Bypass Left Axillary Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KK",  # Bypass Left Axillary Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KT",  # Bypass Left Axillary Artery to Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "03160KV",  # Bypass Left Axillary Artery to Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach
        "03160Z0",  # Bypass Left Axillary Artery to Right Upper Arm Artery, Open Approach
        "03160Z1",  # Bypass Left Axillary Artery to Left Upper Arm Artery, Open Approach
        "03160Z2",  # Bypass Left Axillary Artery to Bilateral Upper Arm Artery, Open Approach
        "03160Z3",  # Bypass Left Axillary Artery to Right Lower Arm Artery, Open Approach
        "03160Z4",  # Bypass Left Axillary Artery to Left Lower Arm Artery, Open Approach
        "03160Z5",  # Bypass Left Axillary Artery to Bilateral Lower Arm Artery, Open Approach
        "03160Z6",  # Bypass Left Axillary Artery to Right Upper Leg Artery, Open Approach
        "03160Z7",  # Bypass Left Axillary Artery to Left Upper Leg Artery, Open Approach
        "03160Z8",  # Bypass Left Axillary Artery to Bilateral Upper Leg Artery, Open Approach
        "03160Z9",  # Bypass Left Axillary Artery to Right Lower Leg Artery, Open Approach
        "03160ZB",  # Bypass Left Axillary Artery to Left Lower Leg Artery, Open Approach
        "03160ZC",  # Bypass Left Axillary Artery to Bilateral Lower Leg Artery, Open Approach
        "03160ZD",  # Bypass Left Axillary Artery to Upper Arm Vein, Open Approach
        "03160ZF",  # Bypass Left Axillary Artery to Lower Arm Vein, Open Approach
        "03160ZJ",  # Bypass Left Axillary Artery to Right Extracranial Artery, Open Approach
        "03160ZK",  # Bypass Left Axillary Artery to Left Extracranial Artery, Open Approach
        "03160ZT",  # Bypass Left Axillary Artery to Abdominal Artery, Open Approach
        "03160ZV",  # Bypass Left Axillary Artery to Superior Vena Cava, Open Approach
        "0317090",  # Bypass Right Brachial Artery to Right Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0317093",  # Bypass Right Brachial Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031709D",  # Bypass Right Brachial Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031709F",  # Bypass Right Brachial Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031709V",  # Bypass Right Brachial Artery to Superior Vena Cava with Autologous Venous Tissue, Open Approach
        "03170A0",  # Bypass Right Brachial Artery to Right Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03170A3",  # Bypass Right Brachial Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03170AD",  # Bypass Right Brachial Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03170AF",  # Bypass Right Brachial Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03170AV",  # Bypass Right Brachial Artery to Superior Vena Cava with Autologous Arterial Tissue, Open Approach
        "03170J0",  # Bypass Right Brachial Artery to Right Upper Arm Artery with Synthetic Substitute, Open Approach
        "03170J3",  # Bypass Right Brachial Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03170JD",  # Bypass Right Brachial Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03170JF",  # Bypass Right Brachial Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03170JV",  # Bypass Right Brachial Artery to Superior Vena Cava with Synthetic Substitute, Open Approach
        "03170K0",  # Bypass Right Brachial Artery to Right Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03170K3",  # Bypass Right Brachial Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03170KD",  # Bypass Right Brachial Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03170KF",  # Bypass Right Brachial Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03170KV",  # Bypass Right Brachial Artery to Superior Vena Cava with Nonautologous Tissue Substitute, Open Approach
        "03170Z0",  # Bypass Right Brachial Artery to Right Upper Arm Artery, Open Approach
        "03170Z3",  # Bypass Right Brachial Artery to Right Lower Arm Artery, Open Approach
        "03170ZD",  # Bypass Right Brachial Artery to Upper Arm Vein, Open Approach
        "03170ZF",  # Bypass Right Brachial Artery to Lower Arm Vein, Open Approach
        "03170ZV",  # Bypass Right Brachial Artery to Superior Vena Cava, Open Approach
        "0318091",  # Bypass Left Brachial Artery to Left Upper Arm Artery with Autologous Venous Tissue, Open Approach
        "0318094",  # Bypass Left Brachial Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031809D",  # Bypass Left Brachial Artery to Upper Arm Vein with Autologous Venous Tissue, Open Approach
        "031809F",  # Bypass Left Brachial Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "03180A1",  # Bypass Left Brachial Artery to Left Upper Arm Artery with Autologous Arterial Tissue, Open Approach
        "03180A4",  # Bypass Left Brachial Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03180AD",  # Bypass Left Brachial Artery to Upper Arm Vein with Autologous Arterial Tissue, Open Approach
        "03180AF",  # Bypass Left Brachial Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03180J1",  # Bypass Left Brachial Artery to Left Upper Arm Artery with Synthetic Substitute, Open Approach
        "03180J4",  # Bypass Left Brachial Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "03180JD",  # Bypass Left Brachial Artery to Upper Arm Vein with Synthetic Substitute, Open Approach
        "03180JF",  # Bypass Left Brachial Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03180K1",  # Bypass Left Brachial Artery to Left Upper Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03180K4",  # Bypass Left Brachial Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03180KD",  # Bypass Left Brachial Artery to Upper Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03180KF",  # Bypass Left Brachial Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03180Z1",  # Bypass Left Brachial Artery to Left Upper Arm Artery, Open Approach
        "03180Z4",  # Bypass Left Brachial Artery to Left Lower Arm Artery, Open Approach
        "03180ZD",  # Bypass Left Brachial Artery to Upper Arm Vein, Open Approach
        "03180ZF",  # Bypass Left Brachial Artery to Lower Arm Vein, Open Approach
        "0319093",  # Bypass Right Ulnar Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031909F",  # Bypass Right Ulnar Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "03190A3",  # Bypass Right Ulnar Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "03190AF",  # Bypass Right Ulnar Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "03190J3",  # Bypass Right Ulnar Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "03190JF",  # Bypass Right Ulnar Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "03190K3",  # Bypass Right Ulnar Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "03190KF",  # Bypass Right Ulnar Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "03190Z3",  # Bypass Right Ulnar Artery to Right Lower Arm Artery, Open Approach
        "03190ZF",  # Bypass Right Ulnar Artery to Lower Arm Vein, Open Approach
        "031A094",  # Bypass Left Ulnar Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031A09F",  # Bypass Left Ulnar Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031A0A4",  # Bypass Left Ulnar Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "031A0AF",  # Bypass Left Ulnar Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "031A0J4",  # Bypass Left Ulnar Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "031A0JF",  # Bypass Left Ulnar Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "031A0K4",  # Bypass Left Ulnar Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "031A0KF",  # Bypass Left Ulnar Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "031A0Z4",  # Bypass Left Ulnar Artery to Left Lower Arm Artery, Open Approach
        "031A0ZF",  # Bypass Left Ulnar Artery to Lower Arm Vein, Open Approach
        "031B093",  # Bypass Right Radial Artery to Right Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031B09F",  # Bypass Right Radial Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031B0A3",  # Bypass Right Radial Artery to Right Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "031B0AF",  # Bypass Right Radial Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "031B0J3",  # Bypass Right Radial Artery to Right Lower Arm Artery with Synthetic Substitute, Open Approach
        "031B0JF",  # Bypass Right Radial Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "031B0K3",  # Bypass Right Radial Artery to Right Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "031B0KF",  # Bypass Right Radial Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "031B0Z3",  # Bypass Right Radial Artery to Right Lower Arm Artery, Open Approach
        "031B0ZF",  # Bypass Right Radial Artery to Lower Arm Vein, Open Approach
        "031C094",  # Bypass Left Radial Artery to Left Lower Arm Artery with Autologous Venous Tissue, Open Approach
        "031C09F",  # Bypass Left Radial Artery to Lower Arm Vein with Autologous Venous Tissue, Open Approach
        "031C0A4",  # Bypass Left Radial Artery to Left Lower Arm Artery with Autologous Arterial Tissue, Open Approach
        "031C0AF",  # Bypass Left Radial Artery to Lower Arm Vein with Autologous Arterial Tissue, Open Approach
        "031C0J4",  # Bypass Left Radial Artery to Left Lower Arm Artery with Synthetic Substitute, Open Approach
        "031C0JF",  # Bypass Left Radial Artery to Lower Arm Vein with Synthetic Substitute, Open Approach
        "031C0K4",  # Bypass Left Radial Artery to Left Lower Arm Artery with Nonautologous Tissue Substitute, Open Approach
        "031C0KF",  # Bypass Left Radial Artery to Lower Arm Vein with Nonautologous Tissue Substitute, Open Approach
        "031C0Z4",  # Bypass Left Radial Artery to Left Lower Arm Artery, Open Approach
        "031C0ZF",  # Bypass Left Radial Artery to Lower Arm Vein, Open Approach
        "031G09G",  # Bypass Intracranial Artery to Intracranial Artery with Autologous Venous Tissue, Open Approach
        "031G0AG",  # Bypass Intracranial Artery to Intracranial Artery with Autologous Arterial Tissue, Open Approach
        "031G0JG",  # Bypass Intracranial Artery to Intracranial Artery with Synthetic Substitute, Open Approach
        "031G0KG",  # Bypass Intracranial Artery to Intracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031G0ZG",  # Bypass Intracranial Artery to Intracranial Artery, Open Approach
        "031H09G",  # Bypass Right Common Carotid Artery to Intracranial Artery with Autologous Venous Tissue, Open Approach
        "031H0AG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Autologous Arterial Tissue, Open Approach
        "031H0JG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Synthetic Substitute, Open Approach
        "031H0KG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031H0ZG",  # Bypass Right Common Carotid Artery to Intracranial Artery, Open Approach
        "031J09G",  # Bypass Left Common Carotid Artery to Intracranial Artery with Autologous Venous Tissue, Open Approach
        "031J0AG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Autologous Arterial Tissue, Open Approach
        "031J0JG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Synthetic Substitute, Open Approach
        "031J0KG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031J0ZG",  # Bypass Left Common Carotid Artery to Intracranial Artery, Open Approach
        "03BG0ZX",  # Excision of Intracranial Artery, Open Approach, Diagnostic
        "03BG0ZZ",  # Excision of Intracranial Artery, Open Approach
        "03BG3ZX",  # Excision of Intracranial Artery, Percutaneous Approach, Diagnostic
        "03BG3ZZ",  # Excision of Intracranial Artery, Percutaneous Approach
        "03BG4ZX",  # Excision of Intracranial Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BG4ZZ",  # Excision of Intracranial Artery, Percutaneous Endoscopic Approach
        "03CH0ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Open Approach
        "03CH3Z7",  # Extirpation of Matter from Right Common Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CH3ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Percutaneous Approach
        "03CH4ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03CJ0ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Open Approach
        "03CJ3Z7",  # Extirpation of Matter from Left Common Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CJ3ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Percutaneous Approach
        "03CJ4ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03CK0ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Open Approach
        "03CK3Z7",  # Extirpation of Matter from Right Internal Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CK3ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Percutaneous Approach
        "03CK4ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03CL0ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Open Approach
        "03CL3Z7",  # Extirpation of Matter from Left Internal Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CL3ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Percutaneous Approach
        "03CL4ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03CM0ZZ",  # Extirpation of Matter from Right External Carotid Artery, Open Approach
        "03CM3Z7",  # Extirpation of Matter from Right External Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CM3ZZ",  # Extirpation of Matter from Right External Carotid Artery, Percutaneous Approach
        "03CM4ZZ",  # Extirpation of Matter from Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03CN0ZZ",  # Extirpation of Matter from Left External Carotid Artery, Open Approach
        "03CN3Z7",  # Extirpation of Matter from Left External Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CN3ZZ",  # Extirpation of Matter from Left External Carotid Artery, Percutaneous Approach
        "03CN4ZZ",  # Extirpation of Matter from Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03JY0ZZ",  # Inspection of Upper Artery, Open Approach
        "0410090",  # Bypass Abdominal Aorta to Abdominal Aorta with Autologous Venous Tissue, Open Approach
        "0410092",  # Bypass Abdominal Aorta to Mesenteric Artery with Autologous Venous Tissue, Open Approach
        "0410096",  # Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "0410097",  # Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "0410098",  # Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach
        "0410099",  # Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041009B",  # Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041009C",  # Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041009D",  # Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041009F",  # Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041009G",  # Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041009H",  # Bypass Abdominal Aorta to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041009J",  # Bypass Abdominal Aorta to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041009K",  # Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041009Q",  # Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041009R",  # Bypass Abdominal Aorta to Lower Artery with Autologous Venous Tissue, Open Approach
        "04100A0",  # Bypass Abdominal Aorta to Abdominal Aorta with Autologous Arterial Tissue, Open Approach
        "04100A1",  # Bypass Abdominal Aorta to Celiac Artery with Autologous Arterial Tissue, Open Approach
        "04100A2",  # Bypass Abdominal Aorta to Mesenteric Artery with Autologous Arterial Tissue, Open Approach
        "04100A6",  # Bypass Abdominal Aorta to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100A7",  # Bypass Abdominal Aorta to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100A8",  # Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "04100A9",  # Bypass Abdominal Aorta to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100AB",  # Bypass Abdominal Aorta to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100AC",  # Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "04100AD",  # Bypass Abdominal Aorta to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100AF",  # Bypass Abdominal Aorta to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "04100AG",  # Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "04100AH",  # Bypass Abdominal Aorta to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "04100AJ",  # Bypass Abdominal Aorta to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "04100AK",  # Bypass Abdominal Aorta to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "04100AQ",  # Bypass Abdominal Aorta to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "04100AR",  # Bypass Abdominal Aorta to Lower Artery with Autologous Arterial Tissue, Open Approach
        "04100J0",  # Bypass Abdominal Aorta to Abdominal Aorta with Synthetic Substitute, Open Approach
        "04100J1",  # Bypass Abdominal Aorta to Celiac Artery with Synthetic Substitute, Open Approach
        "04100J2",  # Bypass Abdominal Aorta to Mesenteric Artery with Synthetic Substitute, Open Approach
        "04100J6",  # Bypass Abdominal Aorta to Right Common Iliac Artery with Synthetic Substitute, Open Approach
        "04100J7",  # Bypass Abdominal Aorta to Left Common Iliac Artery with Synthetic Substitute, Open Approach
        "04100J8",  # Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach
        "04100J9",  # Bypass Abdominal Aorta to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "04100JB",  # Bypass Abdominal Aorta to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "04100JC",  # Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "04100JD",  # Bypass Abdominal Aorta to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "04100JF",  # Bypass Abdominal Aorta to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "04100JG",  # Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "04100JH",  # Bypass Abdominal Aorta to Right Femoral Artery with Synthetic Substitute, Open Approach
        "04100JJ",  # Bypass Abdominal Aorta to Left Femoral Artery with Synthetic Substitute, Open Approach
        "04100JK",  # Bypass Abdominal Aorta to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "04100JQ",  # Bypass Abdominal Aorta to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "04100JR",  # Bypass Abdominal Aorta to Lower Artery with Synthetic Substitute, Open Approach
        "04100K0",  # Bypass Abdominal Aorta to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach
        "04100K1",  # Bypass Abdominal Aorta to Celiac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100K2",  # Bypass Abdominal Aorta to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach
        "04100K6",  # Bypass Abdominal Aorta to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100K7",  # Bypass Abdominal Aorta to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100K8",  # Bypass Abdominal Aorta to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "04100K9",  # Bypass Abdominal Aorta to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KB",  # Bypass Abdominal Aorta to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KC",  # Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "04100KD",  # Bypass Abdominal Aorta to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KF",  # Bypass Abdominal Aorta to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KG",  # Bypass Abdominal Aorta to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "04100KH",  # Bypass Abdominal Aorta to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KJ",  # Bypass Abdominal Aorta to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KK",  # Bypass Abdominal Aorta to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "04100KQ",  # Bypass Abdominal Aorta to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "04100KR",  # Bypass Abdominal Aorta to Lower Artery with Nonautologous Tissue Substitute, Open Approach
        "04100Z0",  # Bypass Abdominal Aorta to Abdominal Aorta, Open Approach
        "04100Z1",  # Bypass Abdominal Aorta to Celiac Artery, Open Approach
        "04100Z2",  # Bypass Abdominal Aorta to Mesenteric Artery, Open Approach
        "04100Z6",  # Bypass Abdominal Aorta to Right Common Iliac Artery, Open Approach
        "04100Z7",  # Bypass Abdominal Aorta to Left Common Iliac Artery, Open Approach
        "04100Z8",  # Bypass Abdominal Aorta to Bilateral Common Iliac Arteries, Open Approach
        "04100Z9",  # Bypass Abdominal Aorta to Right Internal Iliac Artery, Open Approach
        "04100ZB",  # Bypass Abdominal Aorta to Left Internal Iliac Artery, Open Approach
        "04100ZC",  # Bypass Abdominal Aorta to Bilateral Internal Iliac Arteries, Open Approach
        "04100ZD",  # Bypass Abdominal Aorta to Right External Iliac Artery, Open Approach
        "04100ZF",  # Bypass Abdominal Aorta to Left External Iliac Artery, Open Approach
        "04100ZG",  # Bypass Abdominal Aorta to Bilateral External Iliac Arteries, Open Approach
        "04100ZH",  # Bypass Abdominal Aorta to Right Femoral Artery, Open Approach
        "04100ZJ",  # Bypass Abdominal Aorta to Left Femoral Artery, Open Approach
        "04100ZK",  # Bypass Abdominal Aorta to Bilateral Femoral Arteries, Open Approach
        "04100ZQ",  # Bypass Abdominal Aorta to Lower Extremity Artery, Open Approach
        "04100ZR",  # Bypass Abdominal Aorta to Lower Artery, Open Approach
        "0414093",  # Bypass Splenic Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach
        "0414094",  # Bypass Splenic Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach
        "0414095",  # Bypass Splenic Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach
        "041409R",  # Bypass Splenic Artery to Lower Artery with Autologous Venous Tissue, Open Approach
        "04140A3",  # Bypass Splenic Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach
        "04140A4",  # Bypass Splenic Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach
        "04140A5",  # Bypass Splenic Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach
        "04140AR",  # Bypass Splenic Artery to Lower Artery with Autologous Arterial Tissue, Open Approach
        "04140J3",  # Bypass Splenic Artery to Right Renal Artery with Synthetic Substitute, Open Approach
        "04140J4",  # Bypass Splenic Artery to Left Renal Artery with Synthetic Substitute, Open Approach
        "04140J5",  # Bypass Splenic Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach
        "04140JR",  # Bypass Splenic Artery to Lower Artery with Synthetic Substitute, Open Approach
        "04140K3",  # Bypass Splenic Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "04140K4",  # Bypass Splenic Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "04140K5",  # Bypass Splenic Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "04140KR",  # Bypass Splenic Artery to Lower Artery with Nonautologous Tissue Substitute, Open Approach
        "04140Z3",  # Bypass Splenic Artery to Right Renal Artery, Open Approach
        "04140Z4",  # Bypass Splenic Artery to Left Renal Artery, Open Approach
        "04140Z5",  # Bypass Splenic Artery to Bilateral Renal Artery, Open Approach
        "04140ZR",  # Bypass Splenic Artery to Lower Artery, Open Approach
        "041449R",  # Bypass Splenic Artery to Lower Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "04144AR",  # Bypass Splenic Artery to Lower Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "04144JR",  # Bypass Splenic Artery to Lower Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "04144KR",  # Bypass Splenic Artery to Lower Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "04144ZR",  # Bypass Splenic Artery to Lower Artery, Percutaneous Endoscopic Approach
        "041C090",  # Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Open Approach
        "041C091",  # Bypass Right Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Open Approach
        "041C092",  # Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Open Approach
        "041C093",  # Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach
        "041C094",  # Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach
        "041C095",  # Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach
        "041C096",  # Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C097",  # Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C098",  # Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041C099",  # Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C09B",  # Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C09C",  # Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041C09D",  # Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C09F",  # Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041C09G",  # Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041C09H",  # Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041C09J",  # Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041C09K",  # Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041C09Q",  # Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041C09R",  # Bypass Right Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Open Approach
        "041C0A0",  # Bypass Right Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Open Approach
        "041C0A1",  # Bypass Right Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Open Approach
        "041C0A2",  # Bypass Right Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Open Approach
        "041C0A3",  # Bypass Right Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach
        "041C0A4",  # Bypass Right Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach
        "041C0A5",  # Bypass Right Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach
        "041C0A6",  # Bypass Right Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0A7",  # Bypass Right Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0A8",  # Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041C0A9",  # Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0AB",  # Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0AC",  # Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041C0AD",  # Bypass Right Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0AF",  # Bypass Right Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041C0AG",  # Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041C0AH",  # Bypass Right Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041C0AJ",  # Bypass Right Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041C0AK",  # Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041C0AQ",  # Bypass Right Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041C0AR",  # Bypass Right Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Open Approach
        "041C0J0",  # Bypass Right Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Open Approach
        "041C0J1",  # Bypass Right Common Iliac Artery to Celiac Artery with Synthetic Substitute, Open Approach
        "041C0J2",  # Bypass Right Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Open Approach
        "041C0J3",  # Bypass Right Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Open Approach
        "041C0J4",  # Bypass Right Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Open Approach
        "041C0J5",  # Bypass Right Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach
        "041C0J6",  # Bypass Right Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Open Approach
        "041C0J7",  # Bypass Right Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Open Approach
        "041C0J8",  # Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach
        "041C0J9",  # Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041C0JB",  # Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041C0JC",  # Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041C0JD",  # Bypass Right Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041C0JF",  # Bypass Right Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041C0JG",  # Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041C0JH",  # Bypass Right Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041C0JJ",  # Bypass Right Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041C0JK",  # Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041C0JQ",  # Bypass Right Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041C0JR",  # Bypass Right Common Iliac Artery to Lower Artery with Synthetic Substitute, Open Approach
        "041C0K0",  # Bypass Right Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach
        "041C0K1",  # Bypass Right Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K2",  # Bypass Right Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K3",  # Bypass Right Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K4",  # Bypass Right Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K5",  # Bypass Right Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K6",  # Bypass Right Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K7",  # Bypass Right Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0K8",  # Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041C0K9",  # Bypass Right Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KB",  # Bypass Right Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KC",  # Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041C0KD",  # Bypass Right Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KF",  # Bypass Right Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KG",  # Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041C0KH",  # Bypass Right Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KJ",  # Bypass Right Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KK",  # Bypass Right Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041C0KQ",  # Bypass Right Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0KR",  # Bypass Right Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Open Approach
        "041C0Z0",  # Bypass Right Common Iliac Artery to Abdominal Aorta, Open Approach
        "041C0Z1",  # Bypass Right Common Iliac Artery to Celiac Artery, Open Approach
        "041C0Z2",  # Bypass Right Common Iliac Artery to Mesenteric Artery, Open Approach
        "041C0Z3",  # Bypass Right Common Iliac Artery to Right Renal Artery, Open Approach
        "041C0Z4",  # Bypass Right Common Iliac Artery to Left Renal Artery, Open Approach
        "041C0Z5",  # Bypass Right Common Iliac Artery to Bilateral Renal Artery, Open Approach
        "041C0Z6",  # Bypass Right Common Iliac Artery to Right Common Iliac Artery, Open Approach
        "041C0Z7",  # Bypass Right Common Iliac Artery to Left Common Iliac Artery, Open Approach
        "041C0Z8",  # Bypass Right Common Iliac Artery to Bilateral Common Iliac Arteries, Open Approach
        "041C0Z9",  # Bypass Right Common Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041C0ZB",  # Bypass Right Common Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041C0ZC",  # Bypass Right Common Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041C0ZD",  # Bypass Right Common Iliac Artery to Right External Iliac Artery, Open Approach
        "041C0ZF",  # Bypass Right Common Iliac Artery to Left External Iliac Artery, Open Approach
        "041C0ZG",  # Bypass Right Common Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041C0ZH",  # Bypass Right Common Iliac Artery to Right Femoral Artery, Open Approach
        "041C0ZJ",  # Bypass Right Common Iliac Artery to Left Femoral Artery, Open Approach
        "041C0ZK",  # Bypass Right Common Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041C0ZQ",  # Bypass Right Common Iliac Artery to Lower Extremity Artery, Open Approach
        "041C0ZR",  # Bypass Right Common Iliac Artery to Lower Artery, Open Approach
        "041D090",  # Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Venous Tissue, Open Approach
        "041D091",  # Bypass Left Common Iliac Artery to Celiac Artery with Autologous Venous Tissue, Open Approach
        "041D092",  # Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Venous Tissue, Open Approach
        "041D093",  # Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Venous Tissue, Open Approach
        "041D094",  # Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Venous Tissue, Open Approach
        "041D095",  # Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Venous Tissue, Open Approach
        "041D096",  # Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D097",  # Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D098",  # Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041D099",  # Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D09B",  # Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D09C",  # Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041D09D",  # Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D09F",  # Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041D09G",  # Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041D09H",  # Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041D09J",  # Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041D09K",  # Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041D09Q",  # Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041D09R",  # Bypass Left Common Iliac Artery to Lower Artery with Autologous Venous Tissue, Open Approach
        "041D0A0",  # Bypass Left Common Iliac Artery to Abdominal Aorta with Autologous Arterial Tissue, Open Approach
        "041D0A1",  # Bypass Left Common Iliac Artery to Celiac Artery with Autologous Arterial Tissue, Open Approach
        "041D0A2",  # Bypass Left Common Iliac Artery to Mesenteric Artery with Autologous Arterial Tissue, Open Approach
        "041D0A3",  # Bypass Left Common Iliac Artery to Right Renal Artery with Autologous Arterial Tissue, Open Approach
        "041D0A4",  # Bypass Left Common Iliac Artery to Left Renal Artery with Autologous Arterial Tissue, Open Approach
        "041D0A5",  # Bypass Left Common Iliac Artery to Bilateral Renal Artery with Autologous Arterial Tissue, Open Approach
        "041D0A6",  # Bypass Left Common Iliac Artery to Right Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0A7",  # Bypass Left Common Iliac Artery to Left Common Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0A8",  # Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041D0A9",  # Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0AB",  # Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0AC",  # Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041D0AD",  # Bypass Left Common Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0AF",  # Bypass Left Common Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041D0AG",  # Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041D0AH",  # Bypass Left Common Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041D0AJ",  # Bypass Left Common Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041D0AK",  # Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041D0AQ",  # Bypass Left Common Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041D0AR",  # Bypass Left Common Iliac Artery to Lower Artery with Autologous Arterial Tissue, Open Approach
        "041D0J0",  # Bypass Left Common Iliac Artery to Abdominal Aorta with Synthetic Substitute, Open Approach
        "041D0J1",  # Bypass Left Common Iliac Artery to Celiac Artery with Synthetic Substitute, Open Approach
        "041D0J2",  # Bypass Left Common Iliac Artery to Mesenteric Artery with Synthetic Substitute, Open Approach
        "041D0J3",  # Bypass Left Common Iliac Artery to Right Renal Artery with Synthetic Substitute, Open Approach
        "041D0J4",  # Bypass Left Common Iliac Artery to Left Renal Artery with Synthetic Substitute, Open Approach
        "041D0J5",  # Bypass Left Common Iliac Artery to Bilateral Renal Artery with Synthetic Substitute, Open Approach
        "041D0J6",  # Bypass Left Common Iliac Artery to Right Common Iliac Artery with Synthetic Substitute, Open Approach
        "041D0J7",  # Bypass Left Common Iliac Artery to Left Common Iliac Artery with Synthetic Substitute, Open Approach
        "041D0J8",  # Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Synthetic Substitute, Open Approach
        "041D0J9",  # Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041D0JB",  # Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041D0JC",  # Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041D0JD",  # Bypass Left Common Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041D0JF",  # Bypass Left Common Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041D0JG",  # Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041D0JH",  # Bypass Left Common Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041D0JJ",  # Bypass Left Common Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041D0JK",  # Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041D0JQ",  # Bypass Left Common Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041D0JR",  # Bypass Left Common Iliac Artery to Lower Artery with Synthetic Substitute, Open Approach
        "041D0K0",  # Bypass Left Common Iliac Artery to Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach
        "041D0K1",  # Bypass Left Common Iliac Artery to Celiac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K2",  # Bypass Left Common Iliac Artery to Mesenteric Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K3",  # Bypass Left Common Iliac Artery to Right Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K4",  # Bypass Left Common Iliac Artery to Left Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K5",  # Bypass Left Common Iliac Artery to Bilateral Renal Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K6",  # Bypass Left Common Iliac Artery to Right Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K7",  # Bypass Left Common Iliac Artery to Left Common Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0K8",  # Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041D0K9",  # Bypass Left Common Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KB",  # Bypass Left Common Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KC",  # Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041D0KD",  # Bypass Left Common Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KF",  # Bypass Left Common Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KG",  # Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041D0KH",  # Bypass Left Common Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KJ",  # Bypass Left Common Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KK",  # Bypass Left Common Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041D0KQ",  # Bypass Left Common Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0KR",  # Bypass Left Common Iliac Artery to Lower Artery with Nonautologous Tissue Substitute, Open Approach
        "041D0Z0",  # Bypass Left Common Iliac Artery to Abdominal Aorta, Open Approach
        "041D0Z1",  # Bypass Left Common Iliac Artery to Celiac Artery, Open Approach
        "041D0Z2",  # Bypass Left Common Iliac Artery to Mesenteric Artery, Open Approach
        "041D0Z3",  # Bypass Left Common Iliac Artery to Right Renal Artery, Open Approach
        "041D0Z4",  # Bypass Left Common Iliac Artery to Left Renal Artery, Open Approach
        "041D0Z5",  # Bypass Left Common Iliac Artery to Bilateral Renal Artery, Open Approach
        "041D0Z6",  # Bypass Left Common Iliac Artery to Right Common Iliac Artery, Open Approach
        "041D0Z7",  # Bypass Left Common Iliac Artery to Left Common Iliac Artery, Open Approach
        "041D0Z8",  # Bypass Left Common Iliac Artery to Bilateral Common Iliac Arteries, Open Approach
        "041D0Z9",  # Bypass Left Common Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041D0ZB",  # Bypass Left Common Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041D0ZC",  # Bypass Left Common Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041D0ZD",  # Bypass Left Common Iliac Artery to Right External Iliac Artery, Open Approach
        "041D0ZF",  # Bypass Left Common Iliac Artery to Left External Iliac Artery, Open Approach
        "041D0ZG",  # Bypass Left Common Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041D0ZH",  # Bypass Left Common Iliac Artery to Right Femoral Artery, Open Approach
        "041D0ZJ",  # Bypass Left Common Iliac Artery to Left Femoral Artery, Open Approach
        "041D0ZK",  # Bypass Left Common Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041D0ZQ",  # Bypass Left Common Iliac Artery to Lower Extremity Artery, Open Approach
        "041D0ZR",  # Bypass Left Common Iliac Artery to Lower Artery, Open Approach
        "041E099",  # Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041E09B",  # Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041E09C",  # Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041E09D",  # Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041E09F",  # Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041E09G",  # Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041E09H",  # Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041E09J",  # Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041E09K",  # Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041E09P",  # Bypass Right Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041E09Q",  # Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041E0A9",  # Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041E0AB",  # Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041E0AC",  # Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041E0AD",  # Bypass Right Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041E0AF",  # Bypass Right Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041E0AG",  # Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041E0AH",  # Bypass Right Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041E0AJ",  # Bypass Right Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041E0AK",  # Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041E0AP",  # Bypass Right Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041E0AQ",  # Bypass Right Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041E0J9",  # Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041E0JB",  # Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041E0JC",  # Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041E0JD",  # Bypass Right Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041E0JF",  # Bypass Right Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041E0JG",  # Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041E0JH",  # Bypass Right Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041E0JJ",  # Bypass Right Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041E0JK",  # Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041E0JP",  # Bypass Right Internal Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041E0JQ",  # Bypass Right Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041E0K9",  # Bypass Right Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KB",  # Bypass Right Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KC",  # Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041E0KD",  # Bypass Right Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KF",  # Bypass Right Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KG",  # Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041E0KH",  # Bypass Right Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KJ",  # Bypass Right Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KK",  # Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041E0KP",  # Bypass Right Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0KQ",  # Bypass Right Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041E0Z9",  # Bypass Right Internal Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041E0ZB",  # Bypass Right Internal Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041E0ZC",  # Bypass Right Internal Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041E0ZD",  # Bypass Right Internal Iliac Artery to Right External Iliac Artery, Open Approach
        "041E0ZF",  # Bypass Right Internal Iliac Artery to Left External Iliac Artery, Open Approach
        "041E0ZG",  # Bypass Right Internal Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041E0ZH",  # Bypass Right Internal Iliac Artery to Right Femoral Artery, Open Approach
        "041E0ZJ",  # Bypass Right Internal Iliac Artery to Left Femoral Artery, Open Approach
        "041E0ZK",  # Bypass Right Internal Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041E0ZP",  # Bypass Right Internal Iliac Artery to Foot Artery, Open Approach
        "041E0ZQ",  # Bypass Right Internal Iliac Artery to Lower Extremity Artery, Open Approach
        "041F099",  # Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041F09B",  # Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041F09C",  # Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041F09D",  # Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041F09F",  # Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041F09G",  # Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041F09H",  # Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041F09J",  # Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041F09K",  # Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041F09P",  # Bypass Left Internal Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041F09Q",  # Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041F0A9",  # Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041F0AB",  # Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041F0AC",  # Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041F0AD",  # Bypass Left Internal Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041F0AF",  # Bypass Left Internal Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041F0AG",  # Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041F0AH",  # Bypass Left Internal Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041F0AJ",  # Bypass Left Internal Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041F0AK",  # Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041F0AP",  # Bypass Left Internal Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041F0AQ",  # Bypass Left Internal Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041F0J9",  # Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041F0JB",  # Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041F0JC",  # Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041F0JD",  # Bypass Left Internal Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041F0JF",  # Bypass Left Internal Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041F0JG",  # Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041F0JH",  # Bypass Left Internal Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041F0JJ",  # Bypass Left Internal Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041F0JK",  # Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041F0JP",  # Bypass Left Internal Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041F0JQ",  # Bypass Left Internal Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041F0K9",  # Bypass Left Internal Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KB",  # Bypass Left Internal Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KC",  # Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041F0KD",  # Bypass Left Internal Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KF",  # Bypass Left Internal Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KG",  # Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041F0KH",  # Bypass Left Internal Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KJ",  # Bypass Left Internal Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KK",  # Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041F0KP",  # Bypass Left Internal Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0KQ",  # Bypass Left Internal Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041F0Z9",  # Bypass Left Internal Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041F0ZB",  # Bypass Left Internal Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041F0ZC",  # Bypass Left Internal Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041F0ZD",  # Bypass Left Internal Iliac Artery to Right External Iliac Artery, Open Approach
        "041F0ZF",  # Bypass Left Internal Iliac Artery to Left External Iliac Artery, Open Approach
        "041F0ZG",  # Bypass Left Internal Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041F0ZH",  # Bypass Left Internal Iliac Artery to Right Femoral Artery, Open Approach
        "041F0ZJ",  # Bypass Left Internal Iliac Artery to Left Femoral Artery, Open Approach
        "041F0ZK",  # Bypass Left Internal Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041F0ZP",  # Bypass Left Internal Iliac Artery to Foot Artery, Open Approach
        "041F0ZQ",  # Bypass Left Internal Iliac Artery to Lower Extremity Artery, Open Approach
        "041H099",  # Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041H09B",  # Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041H09C",  # Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041H09D",  # Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041H09F",  # Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041H09G",  # Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041H09H",  # Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041H09J",  # Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041H09K",  # Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041H09P",  # Bypass Right External Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041H09Q",  # Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041H0A9",  # Bypass Right External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041H0AB",  # Bypass Right External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041H0AC",  # Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041H0AD",  # Bypass Right External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041H0AF",  # Bypass Right External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041H0AG",  # Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041H0AH",  # Bypass Right External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041H0AJ",  # Bypass Right External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041H0AK",  # Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041H0AP",  # Bypass Right External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041H0AQ",  # Bypass Right External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041H0J9",  # Bypass Right External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041H0JB",  # Bypass Right External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041H0JC",  # Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041H0JD",  # Bypass Right External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041H0JF",  # Bypass Right External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041H0JG",  # Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041H0JH",  # Bypass Right External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041H0JJ",  # Bypass Right External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041H0JK",  # Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041H0JP",  # Bypass Right External Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041H0JQ",  # Bypass Right External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041H0K9",  # Bypass Right External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KB",  # Bypass Right External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KC",  # Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041H0KD",  # Bypass Right External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KF",  # Bypass Right External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KG",  # Bypass Right External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041H0KH",  # Bypass Right External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KJ",  # Bypass Right External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KK",  # Bypass Right External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041H0KP",  # Bypass Right External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0KQ",  # Bypass Right External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041H0Z9",  # Bypass Right External Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041H0ZB",  # Bypass Right External Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041H0ZC",  # Bypass Right External Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041H0ZD",  # Bypass Right External Iliac Artery to Right External Iliac Artery, Open Approach
        "041H0ZF",  # Bypass Right External Iliac Artery to Left External Iliac Artery, Open Approach
        "041H0ZG",  # Bypass Right External Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041H0ZH",  # Bypass Right External Iliac Artery to Right Femoral Artery, Open Approach
        "041H0ZJ",  # Bypass Right External Iliac Artery to Left Femoral Artery, Open Approach
        "041H0ZK",  # Bypass Right External Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041H0ZP",  # Bypass Right External Iliac Artery to Foot Artery, Open Approach
        "041H0ZQ",  # Bypass Right External Iliac Artery to Lower Extremity Artery, Open Approach
        "041J099",  # Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041J09B",  # Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Venous Tissue, Open Approach
        "041J09C",  # Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041J09D",  # Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041J09F",  # Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Venous Tissue, Open Approach
        "041J09G",  # Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Venous Tissue, Open Approach
        "041J09H",  # Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041J09J",  # Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041J09K",  # Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041J09P",  # Bypass Left External Iliac Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041J09Q",  # Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041J0A9",  # Bypass Left External Iliac Artery to Right Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041J0AB",  # Bypass Left External Iliac Artery to Left Internal Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041J0AC",  # Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041J0AD",  # Bypass Left External Iliac Artery to Right External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041J0AF",  # Bypass Left External Iliac Artery to Left External Iliac Artery with Autologous Arterial Tissue, Open Approach
        "041J0AG",  # Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Autologous Arterial Tissue, Open Approach
        "041J0AH",  # Bypass Left External Iliac Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041J0AJ",  # Bypass Left External Iliac Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041J0AK",  # Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041J0AP",  # Bypass Left External Iliac Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041J0AQ",  # Bypass Left External Iliac Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041J0J9",  # Bypass Left External Iliac Artery to Right Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041J0JB",  # Bypass Left External Iliac Artery to Left Internal Iliac Artery with Synthetic Substitute, Open Approach
        "041J0JC",  # Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Synthetic Substitute, Open Approach
        "041J0JD",  # Bypass Left External Iliac Artery to Right External Iliac Artery with Synthetic Substitute, Open Approach
        "041J0JF",  # Bypass Left External Iliac Artery to Left External Iliac Artery with Synthetic Substitute, Open Approach
        "041J0JG",  # Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Synthetic Substitute, Open Approach
        "041J0JH",  # Bypass Left External Iliac Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041J0JJ",  # Bypass Left External Iliac Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041J0JK",  # Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041J0JP",  # Bypass Left External Iliac Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041J0JQ",  # Bypass Left External Iliac Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041J0K9",  # Bypass Left External Iliac Artery to Right Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KB",  # Bypass Left External Iliac Artery to Left Internal Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KC",  # Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041J0KD",  # Bypass Left External Iliac Artery to Right External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KF",  # Bypass Left External Iliac Artery to Left External Iliac Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KG",  # Bypass Left External Iliac Artery to Bilateral External Iliac Arteries with Nonautologous Tissue Substitute, Open Approach
        "041J0KH",  # Bypass Left External Iliac Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KJ",  # Bypass Left External Iliac Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KK",  # Bypass Left External Iliac Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041J0KP",  # Bypass Left External Iliac Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0KQ",  # Bypass Left External Iliac Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041J0Z9",  # Bypass Left External Iliac Artery to Right Internal Iliac Artery, Open Approach
        "041J0ZB",  # Bypass Left External Iliac Artery to Left Internal Iliac Artery, Open Approach
        "041J0ZC",  # Bypass Left External Iliac Artery to Bilateral Internal Iliac Arteries, Open Approach
        "041J0ZD",  # Bypass Left External Iliac Artery to Right External Iliac Artery, Open Approach
        "041J0ZF",  # Bypass Left External Iliac Artery to Left External Iliac Artery, Open Approach
        "041J0ZG",  # Bypass Left External Iliac Artery to Bilateral External Iliac Arteries, Open Approach
        "041J0ZH",  # Bypass Left External Iliac Artery to Right Femoral Artery, Open Approach
        "041J0ZJ",  # Bypass Left External Iliac Artery to Left Femoral Artery, Open Approach
        "041J0ZK",  # Bypass Left External Iliac Artery to Bilateral Femoral Arteries, Open Approach
        "041J0ZP",  # Bypass Left External Iliac Artery to Foot Artery, Open Approach
        "041J0ZQ",  # Bypass Left External Iliac Artery to Lower Extremity Artery, Open Approach
        "041K09H",  # Bypass Right Femoral Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041K09J",  # Bypass Right Femoral Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041K09K",  # Bypass Right Femoral Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041K09L",  # Bypass Right Femoral Artery to Popliteal Artery with Autologous Venous Tissue, Open Approach
        "041K09M",  # Bypass Right Femoral Artery to Peroneal Artery with Autologous Venous Tissue, Open Approach
        "041K09N",  # Bypass Right Femoral Artery to Posterior Tibial Artery with Autologous Venous Tissue, Open Approach
        "041K09P",  # Bypass Right Femoral Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041K09Q",  # Bypass Right Femoral Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041K09S",  # Bypass Right Femoral Artery to Lower Extremity Vein with Autologous Venous Tissue, Open Approach
        "041K0AH",  # Bypass Right Femoral Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041K0AJ",  # Bypass Right Femoral Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041K0AK",  # Bypass Right Femoral Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041K0AL",  # Bypass Right Femoral Artery to Popliteal Artery with Autologous Arterial Tissue, Open Approach
        "041K0AM",  # Bypass Right Femoral Artery to Peroneal Artery with Autologous Arterial Tissue, Open Approach
        "041K0AN",  # Bypass Right Femoral Artery to Posterior Tibial Artery with Autologous Arterial Tissue, Open Approach
        "041K0AP",  # Bypass Right Femoral Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041K0AQ",  # Bypass Right Femoral Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041K0AS",  # Bypass Right Femoral Artery to Lower Extremity Vein with Autologous Arterial Tissue, Open Approach
        "041K0JH",  # Bypass Right Femoral Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041K0JJ",  # Bypass Right Femoral Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041K0JK",  # Bypass Right Femoral Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041K0JL",  # Bypass Right Femoral Artery to Popliteal Artery with Synthetic Substitute, Open Approach
        "041K0JM",  # Bypass Right Femoral Artery to Peroneal Artery with Synthetic Substitute, Open Approach
        "041K0JN",  # Bypass Right Femoral Artery to Posterior Tibial Artery with Synthetic Substitute, Open Approach
        "041K0JP",  # Bypass Right Femoral Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041K0JQ",  # Bypass Right Femoral Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041K0JS",  # Bypass Right Femoral Artery to Lower Extremity Vein with Synthetic Substitute, Open Approach
        "041K0KH",  # Bypass Right Femoral Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KJ",  # Bypass Right Femoral Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KK",  # Bypass Right Femoral Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041K0KL",  # Bypass Right Femoral Artery to Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KM",  # Bypass Right Femoral Artery to Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KN",  # Bypass Right Femoral Artery to Posterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KP",  # Bypass Right Femoral Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KQ",  # Bypass Right Femoral Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041K0KS",  # Bypass Right Femoral Artery to Lower Extremity Vein with Nonautologous Tissue Substitute, Open Approach
        "041K0ZH",  # Bypass Right Femoral Artery to Right Femoral Artery, Open Approach
        "041K0ZJ",  # Bypass Right Femoral Artery to Left Femoral Artery, Open Approach
        "041K0ZK",  # Bypass Right Femoral Artery to Bilateral Femoral Arteries, Open Approach
        "041K0ZL",  # Bypass Right Femoral Artery to Popliteal Artery, Open Approach
        "041K0ZM",  # Bypass Right Femoral Artery to Peroneal Artery, Open Approach
        "041K0ZN",  # Bypass Right Femoral Artery to Posterior Tibial Artery, Open Approach
        "041K0ZP",  # Bypass Right Femoral Artery to Foot Artery, Open Approach
        "041K0ZQ",  # Bypass Right Femoral Artery to Lower Extremity Artery, Open Approach
        "041K0ZS",  # Bypass Right Femoral Artery to Lower Extremity Vein, Open Approach
        "041L09H",  # Bypass Left Femoral Artery to Right Femoral Artery with Autologous Venous Tissue, Open Approach
        "041L09J",  # Bypass Left Femoral Artery to Left Femoral Artery with Autologous Venous Tissue, Open Approach
        "041L09K",  # Bypass Left Femoral Artery to Bilateral Femoral Arteries with Autologous Venous Tissue, Open Approach
        "041L09L",  # Bypass Left Femoral Artery to Popliteal Artery with Autologous Venous Tissue, Open Approach
        "041L09M",  # Bypass Left Femoral Artery to Peroneal Artery with Autologous Venous Tissue, Open Approach
        "041L09N",  # Bypass Left Femoral Artery to Posterior Tibial Artery with Autologous Venous Tissue, Open Approach
        "041L09P",  # Bypass Left Femoral Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041L09Q",  # Bypass Left Femoral Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041L09S",  # Bypass Left Femoral Artery to Lower Extremity Vein with Autologous Venous Tissue, Open Approach
        "041L0AH",  # Bypass Left Femoral Artery to Right Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041L0AJ",  # Bypass Left Femoral Artery to Left Femoral Artery with Autologous Arterial Tissue, Open Approach
        "041L0AK",  # Bypass Left Femoral Artery to Bilateral Femoral Arteries with Autologous Arterial Tissue, Open Approach
        "041L0AL",  # Bypass Left Femoral Artery to Popliteal Artery with Autologous Arterial Tissue, Open Approach
        "041L0AM",  # Bypass Left Femoral Artery to Peroneal Artery with Autologous Arterial Tissue, Open Approach
        "041L0AN",  # Bypass Left Femoral Artery to Posterior Tibial Artery with Autologous Arterial Tissue, Open Approach
        "041L0AP",  # Bypass Left Femoral Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041L0AQ",  # Bypass Left Femoral Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041L0AS",  # Bypass Left Femoral Artery to Lower Extremity Vein with Autologous Arterial Tissue, Open Approach
        "041L0JH",  # Bypass Left Femoral Artery to Right Femoral Artery with Synthetic Substitute, Open Approach
        "041L0JJ",  # Bypass Left Femoral Artery to Left Femoral Artery with Synthetic Substitute, Open Approach
        "041L0JK",  # Bypass Left Femoral Artery to Bilateral Femoral Arteries with Synthetic Substitute, Open Approach
        "041L0JL",  # Bypass Left Femoral Artery to Popliteal Artery with Synthetic Substitute, Open Approach
        "041L0JM",  # Bypass Left Femoral Artery to Peroneal Artery with Synthetic Substitute, Open Approach
        "041L0JN",  # Bypass Left Femoral Artery to Posterior Tibial Artery with Synthetic Substitute, Open Approach
        "041L0JP",  # Bypass Left Femoral Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041L0JQ",  # Bypass Left Femoral Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041L0JS",  # Bypass Left Femoral Artery to Lower Extremity Vein with Synthetic Substitute, Open Approach
        "041L0KH",  # Bypass Left Femoral Artery to Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KJ",  # Bypass Left Femoral Artery to Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KK",  # Bypass Left Femoral Artery to Bilateral Femoral Arteries with Nonautologous Tissue Substitute, Open Approach
        "041L0KL",  # Bypass Left Femoral Artery to Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KM",  # Bypass Left Femoral Artery to Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KN",  # Bypass Left Femoral Artery to Posterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KP",  # Bypass Left Femoral Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KQ",  # Bypass Left Femoral Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041L0KS",  # Bypass Left Femoral Artery to Lower Extremity Vein with Nonautologous Tissue Substitute, Open Approach
        "041L0ZH",  # Bypass Left Femoral Artery to Right Femoral Artery, Open Approach
        "041L0ZJ",  # Bypass Left Femoral Artery to Left Femoral Artery, Open Approach
        "041L0ZK",  # Bypass Left Femoral Artery to Bilateral Femoral Arteries, Open Approach
        "041L0ZL",  # Bypass Left Femoral Artery to Popliteal Artery, Open Approach
        "041L0ZM",  # Bypass Left Femoral Artery to Peroneal Artery, Open Approach
        "041L0ZN",  # Bypass Left Femoral Artery to Posterior Tibial Artery, Open Approach
        "041L0ZP",  # Bypass Left Femoral Artery to Foot Artery, Open Approach
        "041L0ZQ",  # Bypass Left Femoral Artery to Lower Extremity Artery, Open Approach
        "041L0ZS",  # Bypass Left Femoral Artery to Lower Extremity Vein, Open Approach
        "041M09L",  # Bypass Right Popliteal Artery to Popliteal Artery with Autologous Venous Tissue, Open Approach
        "041M09M",  # Bypass Right Popliteal Artery to Peroneal Artery with Autologous Venous Tissue, Open Approach
        "041M09P",  # Bypass Right Popliteal Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041M09Q",  # Bypass Right Popliteal Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041M09S",  # Bypass Right Popliteal Artery to Lower Extremity Vein with Autologous Venous Tissue, Open Approach
        "041M0AL",  # Bypass Right Popliteal Artery to Popliteal Artery with Autologous Arterial Tissue, Open Approach
        "041M0AM",  # Bypass Right Popliteal Artery to Peroneal Artery with Autologous Arterial Tissue, Open Approach
        "041M0AP",  # Bypass Right Popliteal Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041M0AQ",  # Bypass Right Popliteal Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041M0AS",  # Bypass Right Popliteal Artery to Lower Extremity Vein with Autologous Arterial Tissue, Open Approach
        "041M0JL",  # Bypass Right Popliteal Artery to Popliteal Artery with Synthetic Substitute, Open Approach
        "041M0JM",  # Bypass Right Popliteal Artery to Peroneal Artery with Synthetic Substitute, Open Approach
        "041M0JP",  # Bypass Right Popliteal Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041M0JQ",  # Bypass Right Popliteal Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041M0JS",  # Bypass Right Popliteal Artery to Lower Extremity Vein with Synthetic Substitute, Open Approach
        "041M0KL",  # Bypass Right Popliteal Artery to Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "041M0KM",  # Bypass Right Popliteal Artery to Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "041M0KP",  # Bypass Right Popliteal Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041M0KQ",  # Bypass Right Popliteal Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041M0KS",  # Bypass Right Popliteal Artery to Lower Extremity Vein with Nonautologous Tissue Substitute, Open Approach
        "041M0ZL",  # Bypass Right Popliteal Artery to Popliteal Artery, Open Approach
        "041M0ZM",  # Bypass Right Popliteal Artery to Peroneal Artery, Open Approach
        "041M0ZP",  # Bypass Right Popliteal Artery to Foot Artery, Open Approach
        "041M0ZQ",  # Bypass Right Popliteal Artery to Lower Extremity Artery, Open Approach
        "041M0ZS",  # Bypass Right Popliteal Artery to Lower Extremity Vein, Open Approach
        "041N09L",  # Bypass Left Popliteal Artery to Popliteal Artery with Autologous Venous Tissue, Open Approach
        "041N09M",  # Bypass Left Popliteal Artery to Peroneal Artery with Autologous Venous Tissue, Open Approach
        "041N09P",  # Bypass Left Popliteal Artery to Foot Artery with Autologous Venous Tissue, Open Approach
        "041N09Q",  # Bypass Left Popliteal Artery to Lower Extremity Artery with Autologous Venous Tissue, Open Approach
        "041N09S",  # Bypass Left Popliteal Artery to Lower Extremity Vein with Autologous Venous Tissue, Open Approach
        "041N0AL",  # Bypass Left Popliteal Artery to Popliteal Artery with Autologous Arterial Tissue, Open Approach
        "041N0AM",  # Bypass Left Popliteal Artery to Peroneal Artery with Autologous Arterial Tissue, Open Approach
        "041N0AP",  # Bypass Left Popliteal Artery to Foot Artery with Autologous Arterial Tissue, Open Approach
        "041N0AQ",  # Bypass Left Popliteal Artery to Lower Extremity Artery with Autologous Arterial Tissue, Open Approach
        "041N0AS",  # Bypass Left Popliteal Artery to Lower Extremity Vein with Autologous Arterial Tissue, Open Approach
        "041N0JL",  # Bypass Left Popliteal Artery to Popliteal Artery with Synthetic Substitute, Open Approach
        "041N0JM",  # Bypass Left Popliteal Artery to Peroneal Artery with Synthetic Substitute, Open Approach
        "041N0JP",  # Bypass Left Popliteal Artery to Foot Artery with Synthetic Substitute, Open Approach
        "041N0JQ",  # Bypass Left Popliteal Artery to Lower Extremity Artery with Synthetic Substitute, Open Approach
        "041N0JS",  # Bypass Left Popliteal Artery to Lower Extremity Vein with Synthetic Substitute, Open Approach
        "041N0KL",  # Bypass Left Popliteal Artery to Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "041N0KM",  # Bypass Left Popliteal Artery to Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "041N0KP",  # Bypass Left Popliteal Artery to Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "041N0KQ",  # Bypass Left Popliteal Artery to Lower Extremity Artery with Nonautologous Tissue Substitute, Open Approach
        "041N0KS",  # Bypass Left Popliteal Artery to Lower Extremity Vein with Nonautologous Tissue Substitute, Open Approach
        "041N0ZL",  # Bypass Left Popliteal Artery to Popliteal Artery, Open Approach
        "041N0ZM",  # Bypass Left Popliteal Artery to Peroneal Artery, Open Approach
        "041N0ZP",  # Bypass Left Popliteal Artery to Foot Artery, Open Approach
        "041N0ZQ",  # Bypass Left Popliteal Artery to Lower Extremity Artery, Open Approach
        "041N0ZS",  # Bypass Left Popliteal Artery to Lower Extremity Vein, Open Approach
        "04500ZZ",  # Destruction of Abdominal Aorta, Open Approach
        "04B00ZZ",  # Excision of Abdominal Aorta, Open Approach
        "04B10ZZ",  # Excision of Celiac Artery, Open Approach
        "04B20ZZ",  # Excision of Gastric Artery, Open Approach
        "04B30ZZ",  # Excision of Hepatic Artery, Open Approach
        "04B40ZZ",  # Excision of Splenic Artery, Open Approach
        "04B50ZZ",  # Excision of Superior Mesenteric Artery, Open Approach
        "04B60ZZ",  # Excision of Right Colic Artery, Open Approach
        "04B70ZZ",  # Excision of Left Colic Artery, Open Approach
        "04B80ZZ",  # Excision of Middle Colic Artery, Open Approach
        "04B90ZZ",  # Excision of Right Renal Artery, Open Approach
        "04BA0ZZ",  # Excision of Left Renal Artery, Open Approach
        "04BB0ZZ",  # Excision of Inferior Mesenteric Artery, Open Approach
        "04BC0ZZ",  # Excision of Right Common Iliac Artery, Open Approach
        "04BD0ZZ",  # Excision of Left Common Iliac Artery, Open Approach
        "04BE0ZZ",  # Excision of Right Internal Iliac Artery, Open Approach
        "04BF0ZZ",  # Excision of Left Internal Iliac Artery, Open Approach
        "04BH0ZZ",  # Excision of Right External Iliac Artery, Open Approach
        "04BJ0ZZ",  # Excision of Left External Iliac Artery, Open Approach
        "04C00ZZ",  # Extirpation of Matter from Abdominal Aorta, Open Approach
        "04C10ZZ",  # Extirpation of Matter from Celiac Artery, Open Approach
        "04C20ZZ",  # Extirpation of Matter from Gastric Artery, Open Approach
        "04C30ZZ",  # Extirpation of Matter from Hepatic Artery, Open Approach
        "04C40ZZ",  # Extirpation of Matter from Splenic Artery, Open Approach
        "04C50ZZ",  # Extirpation of Matter from Superior Mesenteric Artery, Open Approach
        "04C60ZZ",  # Extirpation of Matter from Right Colic Artery, Open Approach
        "04C70ZZ",  # Extirpation of Matter from Left Colic Artery, Open Approach
        "04C80ZZ",  # Extirpation of Matter from Middle Colic Artery, Open Approach
        "04C90ZZ",  # Extirpation of Matter from Right Renal Artery, Open Approach
        "04CA0ZZ",  # Extirpation of Matter from Left Renal Artery, Open Approach
        "04CB0ZZ",  # Extirpation of Matter from Inferior Mesenteric Artery, Open Approach
        "04CC0ZZ",  # Extirpation of Matter from Right Common Iliac Artery, Open Approach
        "04CD0ZZ",  # Extirpation of Matter from Left Common Iliac Artery, Open Approach
        "04CE0ZZ",  # Extirpation of Matter from Right Internal Iliac Artery, Open Approach
        "04CF0ZZ",  # Extirpation of Matter from Left Internal Iliac Artery, Open Approach
        "04CH0ZZ",  # Extirpation of Matter from Right External Iliac Artery, Open Approach
        "04CJ0ZZ",  # Extirpation of Matter from Left External Iliac Artery, Open Approach
        "04CK0ZZ",  # Extirpation of Matter from Right Femoral Artery, Open Approach
        "04CL0ZZ",  # Extirpation of Matter from Left Femoral Artery, Open Approach
        "04CM0ZZ",  # Extirpation of Matter from Right Popliteal Artery, Open Approach
        "04CN0ZZ",  # Extirpation of Matter from Left Popliteal Artery, Open Approach
        "04CP0ZZ",  # Extirpation of Matter from Right Anterior Tibial Artery, Open Approach
        "04CQ0ZZ",  # Extirpation of Matter from Left Anterior Tibial Artery, Open Approach
        "04CR0ZZ",  # Extirpation of Matter from Right Posterior Tibial Artery, Open Approach
        "04CS0ZZ",  # Extirpation of Matter from Left Posterior Tibial Artery, Open Approach
        "04CT0ZZ",  # Extirpation of Matter from Right Peroneal Artery, Open Approach
        "04CU0ZZ",  # Extirpation of Matter from Left Peroneal Artery, Open Approach
        "04CV0ZZ",  # Extirpation of Matter from Right Foot Artery, Open Approach
        "04CW0ZZ",  # Extirpation of Matter from Left Foot Artery, Open Approach
        "04CY0ZZ",  # Extirpation of Matter from Lower Artery, Open Approach
        "04JY0ZZ",  # Inspection of Lower Artery, Open Approach
        "04Q00ZZ",  # Repair Abdominal Aorta, Open Approach
        "04Q03ZZ",  # Repair Abdominal Aorta, Percutaneous Approach
        "04Q04ZZ",  # Repair Abdominal Aorta, Percutaneous Endoscopic Approach
        "04R007Z",  # Replacement of Abdominal Aorta with Autologous Tissue Substitute, Open Approach
        "04R00JZ",  # Replacement of Abdominal Aorta with Synthetic Substitute, Open Approach
        "04R00KZ",  # Replacement of Abdominal Aorta with Nonautologous Tissue Substitute, Open Approach
        "04RK07Z",  # Replacement of Right Femoral Artery with Autologous Tissue Substitute, Open Approach
        "04RK0JZ",  # Replacement of Right Femoral Artery with Synthetic Substitute, Open Approach
        "04RK0KZ",  # Replacement of Right Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "04RL07Z",  # Replacement of Left Femoral Artery with Autologous Tissue Substitute, Open Approach
        "04RL0JZ",  # Replacement of Left Femoral Artery with Synthetic Substitute, Open Approach
        "04RL0KZ",  # Replacement of Left Femoral Artery with Nonautologous Tissue Substitute, Open Approach
        "04RM07Z",  # Replacement of Right Popliteal Artery with Autologous Tissue Substitute, Open Approach
        "04RM0JZ",  # Replacement of Right Popliteal Artery with Synthetic Substitute, Open Approach
        "04RM0KZ",  # Replacement of Right Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "04RN07Z",  # Replacement of Left Popliteal Artery with Autologous Tissue Substitute, Open Approach
        "04RN0JZ",  # Replacement of Left Popliteal Artery with Synthetic Substitute, Open Approach
        "04RN0KZ",  # Replacement of Left Popliteal Artery with Nonautologous Tissue Substitute, Open Approach
        "04RP07Z",  # Replacement of Right Anterior Tibial Artery with Autologous Tissue Substitute, Open Approach
        "04RP0JZ",  # Replacement of Right Anterior Tibial Artery with Synthetic Substitute, Open Approach
        "04RP0KZ",  # Replacement of Right Anterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "04RQ07Z",  # Replacement of Left Anterior Tibial Artery with Autologous Tissue Substitute, Open Approach
        "04RQ0JZ",  # Replacement of Left Anterior Tibial Artery with Synthetic Substitute, Open Approach
        "04RQ0KZ",  # Replacement of Left Anterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "04RR07Z",  # Replacement of Right Posterior Tibial Artery with Autologous Tissue Substitute, Open Approach
        "04RR0JZ",  # Replacement of Right Posterior Tibial Artery with Synthetic Substitute, Open Approach
        "04RR0KZ",  # Replacement of Right Posterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "04RS07Z",  # Replacement of Left Posterior Tibial Artery with Autologous Tissue Substitute, Open Approach
        "04RS0JZ",  # Replacement of Left Posterior Tibial Artery with Synthetic Substitute, Open Approach
        "04RS0KZ",  # Replacement of Left Posterior Tibial Artery with Nonautologous Tissue Substitute, Open Approach
        "04RT07Z",  # Replacement of Right Peroneal Artery with Autologous Tissue Substitute, Open Approach
        "04RT0JZ",  # Replacement of Right Peroneal Artery with Synthetic Substitute, Open Approach
        "04RT0KZ",  # Replacement of Right Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "04RU07Z",  # Replacement of Left Peroneal Artery with Autologous Tissue Substitute, Open Approach
        "04RU0JZ",  # Replacement of Left Peroneal Artery with Synthetic Substitute, Open Approach
        "04RU0KZ",  # Replacement of Left Peroneal Artery with Nonautologous Tissue Substitute, Open Approach
        "04RV07Z",  # Replacement of Right Foot Artery with Autologous Tissue Substitute, Open Approach
        "04RV0JZ",  # Replacement of Right Foot Artery with Synthetic Substitute, Open Approach
        "04RV0KZ",  # Replacement of Right Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "04RW07Z",  # Replacement of Left Foot Artery with Autologous Tissue Substitute, Open Approach
        "04RW0JZ",  # Replacement of Left Foot Artery with Synthetic Substitute, Open Approach
        "04RW0KZ",  # Replacement of Left Foot Artery with Nonautologous Tissue Substitute, Open Approach
        "04RY07Z",  # Replacement of Lower Artery with Autologous Tissue Substitute, Open Approach
        "04RY0JZ",  # Replacement of Lower Artery with Synthetic Substitute, Open Approach
        "04RY0KZ",  # Replacement of Lower Artery with Nonautologous Tissue Substitute, Open Approach
        "051707Y",  # Bypass Right Axillary Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051709Y",  # Bypass Right Axillary Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "05170AY",  # Bypass Right Axillary Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "05170JY",  # Bypass Right Axillary Vein to Upper Vein with Synthetic Substitute, Open Approach
        "05170KY",  # Bypass Right Axillary Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "05170ZY",  # Bypass Right Axillary Vein to Upper Vein, Open Approach
        "051807Y",  # Bypass Left Axillary Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051809Y",  # Bypass Left Axillary Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "05180AY",  # Bypass Left Axillary Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "05180JY",  # Bypass Left Axillary Vein to Upper Vein with Synthetic Substitute, Open Approach
        "05180KY",  # Bypass Left Axillary Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "05180ZY",  # Bypass Left Axillary Vein to Upper Vein, Open Approach
        "051907Y",  # Bypass Right Brachial Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051909Y",  # Bypass Right Brachial Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "05190AY",  # Bypass Right Brachial Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "05190JY",  # Bypass Right Brachial Vein to Upper Vein with Synthetic Substitute, Open Approach
        "05190KY",  # Bypass Right Brachial Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "05190ZY",  # Bypass Right Brachial Vein to Upper Vein, Open Approach
        "051A07Y",  # Bypass Left Brachial Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051A09Y",  # Bypass Left Brachial Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051A0AY",  # Bypass Left Brachial Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051A0JY",  # Bypass Left Brachial Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051A0KY",  # Bypass Left Brachial Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051A0ZY",  # Bypass Left Brachial Vein to Upper Vein, Open Approach
        "051B07Y",  # Bypass Right Basilic Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051B09Y",  # Bypass Right Basilic Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051B0AY",  # Bypass Right Basilic Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051B0JY",  # Bypass Right Basilic Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051B0KY",  # Bypass Right Basilic Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051B0ZY",  # Bypass Right Basilic Vein to Upper Vein, Open Approach
        "051C07Y",  # Bypass Left Basilic Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051C09Y",  # Bypass Left Basilic Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051C0AY",  # Bypass Left Basilic Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051C0JY",  # Bypass Left Basilic Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051C0KY",  # Bypass Left Basilic Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051C0ZY",  # Bypass Left Basilic Vein to Upper Vein, Open Approach
        "051D07Y",  # Bypass Right Cephalic Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051D09Y",  # Bypass Right Cephalic Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051D0AY",  # Bypass Right Cephalic Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051D0JY",  # Bypass Right Cephalic Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051D0KY",  # Bypass Right Cephalic Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051D0ZY",  # Bypass Right Cephalic Vein to Upper Vein, Open Approach
        "051F07Y",  # Bypass Left Cephalic Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051F09Y",  # Bypass Left Cephalic Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051F0AY",  # Bypass Left Cephalic Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051F0JY",  # Bypass Left Cephalic Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051F0KY",  # Bypass Left Cephalic Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051F0ZY",  # Bypass Left Cephalic Vein to Upper Vein, Open Approach
        "051G07Y",  # Bypass Right Hand Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051G09Y",  # Bypass Right Hand Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051G0AY",  # Bypass Right Hand Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051G0JY",  # Bypass Right Hand Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051G0KY",  # Bypass Right Hand Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051G0ZY",  # Bypass Right Hand Vein to Upper Vein, Open Approach
        "051H07Y",  # Bypass Left Hand Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051H09Y",  # Bypass Left Hand Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051H0AY",  # Bypass Left Hand Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051H0JY",  # Bypass Left Hand Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051H0KY",  # Bypass Left Hand Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051H0ZY",  # Bypass Left Hand Vein to Upper Vein, Open Approach
        "051L07Y",  # Bypass Intracranial Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051L09Y",  # Bypass Intracranial Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051L0AY",  # Bypass Intracranial Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051L0JY",  # Bypass Intracranial Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051L0KY",  # Bypass Intracranial Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051L0ZY",  # Bypass Intracranial Vein to Upper Vein, Open Approach
        "051M07Y",  # Bypass Right Internal Jugular Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051M09Y",  # Bypass Right Internal Jugular Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051M0AY",  # Bypass Right Internal Jugular Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051M0JY",  # Bypass Right Internal Jugular Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051M0KY",  # Bypass Right Internal Jugular Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051M0ZY",  # Bypass Right Internal Jugular Vein to Upper Vein, Open Approach
        "051N07Y",  # Bypass Left Internal Jugular Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051N09Y",  # Bypass Left Internal Jugular Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051N0AY",  # Bypass Left Internal Jugular Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051N0JY",  # Bypass Left Internal Jugular Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051N0KY",  # Bypass Left Internal Jugular Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051N0ZY",  # Bypass Left Internal Jugular Vein to Upper Vein, Open Approach
        "051P07Y",  # Bypass Right External Jugular Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051P09Y",  # Bypass Right External Jugular Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051P0AY",  # Bypass Right External Jugular Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051P0JY",  # Bypass Right External Jugular Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051P0KY",  # Bypass Right External Jugular Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051P0ZY",  # Bypass Right External Jugular Vein to Upper Vein, Open Approach
        "051Q07Y",  # Bypass Left External Jugular Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051Q09Y",  # Bypass Left External Jugular Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051Q0AY",  # Bypass Left External Jugular Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051Q0JY",  # Bypass Left External Jugular Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051Q0KY",  # Bypass Left External Jugular Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051Q0ZY",  # Bypass Left External Jugular Vein to Upper Vein, Open Approach
        "051R07Y",  # Bypass Right Vertebral Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051R09Y",  # Bypass Right Vertebral Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051R0AY",  # Bypass Right Vertebral Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051R0JY",  # Bypass Right Vertebral Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051R0KY",  # Bypass Right Vertebral Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051R0ZY",  # Bypass Right Vertebral Vein to Upper Vein, Open Approach
        "051S07Y",  # Bypass Left Vertebral Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051S09Y",  # Bypass Left Vertebral Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051S0AY",  # Bypass Left Vertebral Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051S0JY",  # Bypass Left Vertebral Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051S0KY",  # Bypass Left Vertebral Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051S0ZY",  # Bypass Left Vertebral Vein to Upper Vein, Open Approach
        "051T07Y",  # Bypass Right Face Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051T09Y",  # Bypass Right Face Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051T0AY",  # Bypass Right Face Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051T0JY",  # Bypass Right Face Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051T0KY",  # Bypass Right Face Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051T0ZY",  # Bypass Right Face Vein to Upper Vein, Open Approach
        "051V07Y",  # Bypass Left Face Vein to Upper Vein with Autologous Tissue Substitute, Open Approach
        "051V09Y",  # Bypass Left Face Vein to Upper Vein with Autologous Venous Tissue, Open Approach
        "051V0AY",  # Bypass Left Face Vein to Upper Vein with Autologous Arterial Tissue, Open Approach
        "051V0JY",  # Bypass Left Face Vein to Upper Vein with Synthetic Substitute, Open Approach
        "051V0KY",  # Bypass Left Face Vein to Upper Vein with Nonautologous Tissue Substitute, Open Approach
        "051V0ZY",  # Bypass Left Face Vein to Upper Vein, Open Approach
        "05BL0ZX",  # Excision of Intracranial Vein, Open Approach, Diagnostic
        "05BL0ZZ",  # Excision of Intracranial Vein, Open Approach
        "05BL3ZX",  # Excision of Intracranial Vein, Percutaneous Approach, Diagnostic
        "05BL3ZZ",  # Excision of Intracranial Vein, Percutaneous Approach
        "05BL4ZX",  # Excision of Intracranial Vein, Percutaneous Endoscopic Approach, Diagnostic
        "05BL4ZZ",  # Excision of Intracranial Vein, Percutaneous Endoscopic Approach
        "05JY0ZZ",  # Inspection of Upper Vein, Open Approach
        "061307Y",  # Bypass Esophageal Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061309Y",  # Bypass Esophageal Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "06130AY",  # Bypass Esophageal Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "06130JY",  # Bypass Esophageal Vein to Lower Vein with Synthetic Substitute, Open Approach
        "06130KY",  # Bypass Esophageal Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "06130ZY",  # Bypass Esophageal Vein to Lower Vein, Open Approach
        "06194AY",  # Bypass Right Renal Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061B0AY",  # Bypass Left Renal Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061B4AY",  # Bypass Left Renal Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061C07Y",  # Bypass Right Common Iliac Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061C09Y",  # Bypass Right Common Iliac Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061C0AY",  # Bypass Right Common Iliac Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061C0JY",  # Bypass Right Common Iliac Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061C0KY",  # Bypass Right Common Iliac Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061C0ZY",  # Bypass Right Common Iliac Vein to Lower Vein, Open Approach
        "061C4AY",  # Bypass Right Common Iliac Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061D07Y",  # Bypass Left Common Iliac Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061D09Y",  # Bypass Left Common Iliac Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061D0AY",  # Bypass Left Common Iliac Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061D0JY",  # Bypass Left Common Iliac Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061D0KY",  # Bypass Left Common Iliac Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061D0ZY",  # Bypass Left Common Iliac Vein to Lower Vein, Open Approach
        "061D4AY",  # Bypass Left Common Iliac Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061F07Y",  # Bypass Right External Iliac Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061F09Y",  # Bypass Right External Iliac Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061F0AY",  # Bypass Right External Iliac Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061F0JY",  # Bypass Right External Iliac Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061F0KY",  # Bypass Right External Iliac Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061F0ZY",  # Bypass Right External Iliac Vein to Lower Vein, Open Approach
        "061F4AY",  # Bypass Right External Iliac Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061G07Y",  # Bypass Left External Iliac Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061G09Y",  # Bypass Left External Iliac Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061G0AY",  # Bypass Left External Iliac Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061G0JY",  # Bypass Left External Iliac Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061G0KY",  # Bypass Left External Iliac Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061G0ZY",  # Bypass Left External Iliac Vein to Lower Vein, Open Approach
        "061G4AY",  # Bypass Left External Iliac Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061H07Y",  # Bypass Right Hypogastric Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061H09Y",  # Bypass Right Hypogastric Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061H0AY",  # Bypass Right Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061H0JY",  # Bypass Right Hypogastric Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061H0KY",  # Bypass Right Hypogastric Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061H0ZY",  # Bypass Right Hypogastric Vein to Lower Vein, Open Approach
        "061H4AY",  # Bypass Right Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061J0AY",  # Bypass Left Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061J4AY",  # Bypass Left Hypogastric Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061M07Y",  # Bypass Right Femoral Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061M09Y",  # Bypass Right Femoral Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061M0AY",  # Bypass Right Femoral Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061M0JY",  # Bypass Right Femoral Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061M0KY",  # Bypass Right Femoral Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061M0ZY",  # Bypass Right Femoral Vein to Lower Vein, Open Approach
        "061M4AY",  # Bypass Right Femoral Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061N07Y",  # Bypass Left Femoral Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061N09Y",  # Bypass Left Femoral Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061N0AY",  # Bypass Left Femoral Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061N0JY",  # Bypass Left Femoral Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061N0KY",  # Bypass Left Femoral Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061N0ZY",  # Bypass Left Femoral Vein to Lower Vein, Open Approach
        "061N4AY",  # Bypass Left Femoral Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061P07Y",  # Bypass Right Saphenous Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061P09Y",  # Bypass Right Saphenous Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061P0AY",  # Bypass Right Saphenous Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061P0JY",  # Bypass Right Saphenous Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061P0KY",  # Bypass Right Saphenous Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061P0ZY",  # Bypass Right Saphenous Vein to Lower Vein, Open Approach
        "061P4AY",  # Bypass Right Saphenous Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061Q07Y",  # Bypass Left Saphenous Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061Q09Y",  # Bypass Left Saphenous Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061Q0AY",  # Bypass Left Saphenous Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061Q0JY",  # Bypass Left Saphenous Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061Q0KY",  # Bypass Left Saphenous Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061Q0ZY",  # Bypass Left Saphenous Vein to Lower Vein, Open Approach
        "061Q4AY",  # Bypass Left Saphenous Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061T07Y",  # Bypass Right Foot Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061T09Y",  # Bypass Right Foot Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061T0AY",  # Bypass Right Foot Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061T0JY",  # Bypass Right Foot Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061T0KY",  # Bypass Right Foot Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061T0ZY",  # Bypass Right Foot Vein to Lower Vein, Open Approach
        "061T4AY",  # Bypass Right Foot Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "061V07Y",  # Bypass Left Foot Vein to Lower Vein with Autologous Tissue Substitute, Open Approach
        "061V09Y",  # Bypass Left Foot Vein to Lower Vein with Autologous Venous Tissue, Open Approach
        "061V0AY",  # Bypass Left Foot Vein to Lower Vein with Autologous Arterial Tissue, Open Approach
        "061V0JY",  # Bypass Left Foot Vein to Lower Vein with Synthetic Substitute, Open Approach
        "061V0KY",  # Bypass Left Foot Vein to Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "061V0ZY",  # Bypass Left Foot Vein to Lower Vein, Open Approach
        "061V4AY",  # Bypass Left Foot Vein to Lower Vein with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "06B00ZZ",  # Excision of Inferior Vena Cava, Open Approach
        "06B10ZZ",  # Excision of Splenic Vein, Open Approach
        "06B20ZZ",  # Excision of Gastric Vein, Open Approach
        "06B40ZZ",  # Excision of Hepatic Vein, Open Approach
        "06B50ZZ",  # Excision of Superior Mesenteric Vein, Open Approach
        "06B60ZZ",  # Excision of Inferior Mesenteric Vein, Open Approach
        "06B70ZZ",  # Excision of Colic Vein, Open Approach
        "06B80ZZ",  # Excision of Portal Vein, Open Approach
        "06B90ZZ",  # Excision of Right Renal Vein, Open Approach
        "06BB0ZZ",  # Excision of Left Renal Vein, Open Approach
        "06BC0ZZ",  # Excision of Right Common Iliac Vein, Open Approach
        "06BD0ZZ",  # Excision of Left Common Iliac Vein, Open Approach
        "06BF0ZZ",  # Excision of Right External Iliac Vein, Open Approach
        "06BG0ZZ",  # Excision of Left External Iliac Vein, Open Approach
        "06BH0ZZ",  # Excision of Right Hypogastric Vein, Open Approach
        "06BJ0ZZ",  # Excision of Left Hypogastric Vein, Open Approach
        "06JY0ZZ",  # Inspection of Lower Vein, Open Approach
        "06L20ZZ",  # Occlusion of Gastric Vein, Open Approach
        "06RM07Z",  # Replacement of Right Femoral Vein with Autologous Tissue Substitute, Open Approach
        "06RM0JZ",  # Replacement of Right Femoral Vein with Synthetic Substitute, Open Approach
        "06RM0KZ",  # Replacement of Right Femoral Vein with Nonautologous Tissue Substitute, Open Approach
        "06RN07Z",  # Replacement of Left Femoral Vein with Autologous Tissue Substitute, Open Approach
        "06RN0JZ",  # Replacement of Left Femoral Vein with Synthetic Substitute, Open Approach
        "06RN0KZ",  # Replacement of Left Femoral Vein with Nonautologous Tissue Substitute, Open Approach
        "06RP07Z",  # Replacement of Right Saphenous Vein with Autologous Tissue Substitute, Open Approach
        "06RP0JZ",  # Replacement of Right Saphenous Vein with Synthetic Substitute, Open Approach
        "06RP0KZ",  # Replacement of Right Saphenous Vein with Nonautologous Tissue Substitute, Open Approach
        "06RQ07Z",  # Replacement of Left Saphenous Vein with Autologous Tissue Substitute, Open Approach
        "06RQ0JZ",  # Replacement of Left Saphenous Vein with Synthetic Substitute, Open Approach
        "06RQ0KZ",  # Replacement of Left Saphenous Vein with Nonautologous Tissue Substitute, Open Approach
        "06RT07Z",  # Replacement of Right Foot Vein with Autologous Tissue Substitute, Open Approach
        "06RT0JZ",  # Replacement of Right Foot Vein with Synthetic Substitute, Open Approach
        "06RT0KZ",  # Replacement of Right Foot Vein with Nonautologous Tissue Substitute, Open Approach
        "06RV07Z",  # Replacement of Left Foot Vein with Autologous Tissue Substitute, Open Approach
        "06RV0JZ",  # Replacement of Left Foot Vein with Synthetic Substitute, Open Approach
        "06RV0KZ",  # Replacement of Left Foot Vein with Nonautologous Tissue Substitute, Open Approach
        "06RY07Z",  # Replacement of Lower Vein with Autologous Tissue Substitute, Open Approach
        "06RY0JZ",  # Replacement of Lower Vein with Synthetic Substitute, Open Approach
        "06RY0KZ",  # Replacement of Lower Vein with Nonautologous Tissue Substitute, Open Approach
        "07JK0ZZ",  # Inspection of Thoracic Duct, Open Approach
        "07JL0ZZ",  # Inspection of Cisterna Chyli, Open Approach
        "07JM0ZZ",  # Inspection of Thymus, Open Approach
        "07JN0ZZ",  # Inspection of Lymphatic, Open Approach
        "07JN8ZZ",  # Inspection of Lymphatic, Via Natural or Artificial Opening Endoscopic
        "07JP0ZZ",  # Inspection of Spleen, Open Approach
        "07JT0ZZ",  # Inspection of Bone Marrow, Open Approach
        "07TP0ZZ",  # Resection of Spleen, Open Approach
        "0BDN0ZX",  # Extraction of Right Pleura, Open Approach, Diagnostic
        "0BDN0ZZ",  # Extraction of Right Pleura, Open Approach
        "0BDP0ZX",  # Extraction of Left Pleura, Open Approach, Diagnostic
        "0BDP0ZZ",  # Extraction of Left Pleura, Open Approach
        "0BJ00ZZ",  # Inspection of Tracheobronchial Tree, Open Approach
        "0BJ10ZZ",  # Inspection of Trachea, Open Approach
        "0BJK0ZZ",  # Inspection of Right Lung, Open Approach
        "0BJL0ZZ",  # Inspection of Left Lung, Open Approach
        "0BJQ0ZZ",  # Inspection of Pleura, Open Approach
        "0BJT0ZZ",  # Inspection of Diaphragm, Open Approach
        "0BPT00Z",  # Removal of Drainage Device from Diaphragm, Open Approach
        "0BPT02Z",  # Removal of Monitoring Device from Diaphragm, Open Approach
        "0BPT07Z",  # Removal of Autologous Tissue Substitute from Diaphragm, Open Approach
        "0BPT0JZ",  # Removal of Synthetic Substitute from Diaphragm, Open Approach
        "0BPT0KZ",  # Removal of Nonautologous Tissue Substitute from Diaphragm, Open Approach
        "0BPT0MZ",  # Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Open Approach
        "0BPT7MZ",  # Removal of Diaphragmatic Pacemaker Lead from Diaphragm, Via Natural or Artificial Opening
        "0BQT0ZZ",  # Repair Diaphragm, Open Approach
        "0BQT3ZZ",  # Repair Diaphragm, Percutaneous Approach
        "0BQT4ZZ",  # Repair Diaphragm, Percutaneous Endoscopic Approach
        "0BTC0ZZ",  # Resection of Right Upper Lung Lobe, Open Approach
        "0BTC4ZZ",  # Resection of Right Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BTD0ZZ",  # Resection of Right Middle Lung Lobe, Open Approach
        "0BTD4ZZ",  # Resection of Right Middle Lung Lobe, Percutaneous Endoscopic Approach
        "0BTF0ZZ",  # Resection of Right Lower Lung Lobe, Open Approach
        "0BTF4ZZ",  # Resection of Right Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BTG0ZZ",  # Resection of Left Upper Lung Lobe, Open Approach
        "0BTG4ZZ",  # Resection of Left Upper Lung Lobe, Percutaneous Endoscopic Approach
        "0BTH0ZZ",  # Resection of Lung Lingula, Open Approach
        "0BTH4ZZ",  # Resection of Lung Lingula, Percutaneous Endoscopic Approach
        "0BTJ0ZZ",  # Resection of Left Lower Lung Lobe, Open Approach
        "0BTJ4ZZ",  # Resection of Left Lower Lung Lobe, Percutaneous Endoscopic Approach
        "0BTK0ZZ",  # Resection of Right Lung, Open Approach
        "0BTK4ZZ",  # Resection of Right Lung, Percutaneous Endoscopic Approach
        "0BTL0ZZ",  # Resection of Left Lung, Open Approach
        "0BTL4ZZ",  # Resection of Left Lung, Percutaneous Endoscopic Approach
        "0BTM0ZZ",  # Resection of Bilateral Lungs, Open Approach
        "0BTM4ZZ",  # Resection of Bilateral Lungs, Percutaneous Endoscopic Approach
        "0BWT00Z",  # Revision of Drainage Device in Diaphragm, Open Approach
        "0BWT02Z",  # Revision of Monitoring Device in Diaphragm, Open Approach
        "0BWT07Z",  # Revision of Autologous Tissue Substitute in Diaphragm, Open Approach
        "0BWT0JZ",  # Revision of Synthetic Substitute in Diaphragm, Open Approach
        "0BWT0KZ",  # Revision of Nonautologous Tissue Substitute in Diaphragm, Open Approach
        "0BWT0MZ",  # Revision of Diaphragmatic Pacemaker Lead in Diaphragm, Open Approach
        "0BWT77Z",  # Revision of Autologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening
        "0BWT7JZ",  # Revision of Synthetic Substitute in Diaphragm, Via Natural or Artificial Opening
        "0BWT7KZ",  # Revision of Nonautologous Tissue Substitute in Diaphragm, Via Natural or Artificial Opening
        "0CBS0ZZ",  # Excision of Larynx, Open Approach
        "0CJS0ZZ",  # Inspection of Larynx, Open Approach
        "0CJY0ZZ",  # Inspection of Mouth and Throat, Open Approach
        "0CTS0ZZ",  # Resection of Larynx, Open Approach
        "0CTS4ZZ",  # Resection of Larynx, Percutaneous Endoscopic Approach
        "0CTS7ZZ",  # Resection of Larynx, Via Natural or Artificial Opening
        "0CTS8ZZ",  # Resection of Larynx, Via Natural or Artificial Opening Endoscopic
        "0D11074",  # Bypass Upper Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D11076",  # Bypass Upper Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D11079",  # Bypass Upper Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1107A",  # Bypass Upper Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1107B",  # Bypass Upper Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D110J4",  # Bypass Upper Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D110J6",  # Bypass Upper Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D110J9",  # Bypass Upper Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D110JA",  # Bypass Upper Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D110JB",  # Bypass Upper Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D110K4",  # Bypass Upper Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D110K6",  # Bypass Upper Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D110K9",  # Bypass Upper Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D110KA",  # Bypass Upper Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D110KB",  # Bypass Upper Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D110Z4",  # Bypass Upper Esophagus to Cutaneous, Open Approach
        "0D110Z6",  # Bypass Upper Esophagus to Stomach, Open Approach
        "0D110Z9",  # Bypass Upper Esophagus to Duodenum, Open Approach
        "0D110ZA",  # Bypass Upper Esophagus to Jejunum, Open Approach
        "0D110ZB",  # Bypass Upper Esophagus to Ileum, Open Approach
        "0D12074",  # Bypass Middle Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D12076",  # Bypass Middle Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D12079",  # Bypass Middle Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1207A",  # Bypass Middle Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1207B",  # Bypass Middle Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D120J4",  # Bypass Middle Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D120J6",  # Bypass Middle Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D120J9",  # Bypass Middle Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D120JA",  # Bypass Middle Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D120JB",  # Bypass Middle Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D120K4",  # Bypass Middle Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D120K6",  # Bypass Middle Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D120K9",  # Bypass Middle Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D120KA",  # Bypass Middle Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D120KB",  # Bypass Middle Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D120Z4",  # Bypass Middle Esophagus to Cutaneous, Open Approach
        "0D120Z6",  # Bypass Middle Esophagus to Stomach, Open Approach
        "0D120Z9",  # Bypass Middle Esophagus to Duodenum, Open Approach
        "0D120ZA",  # Bypass Middle Esophagus to Jejunum, Open Approach
        "0D120ZB",  # Bypass Middle Esophagus to Ileum, Open Approach
        "0D13074",  # Bypass Lower Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D13076",  # Bypass Lower Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D13079",  # Bypass Lower Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1307A",  # Bypass Lower Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1307B",  # Bypass Lower Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D130J4",  # Bypass Lower Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D130J6",  # Bypass Lower Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D130J9",  # Bypass Lower Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D130JA",  # Bypass Lower Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D130JB",  # Bypass Lower Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D130K4",  # Bypass Lower Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D130K6",  # Bypass Lower Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D130K9",  # Bypass Lower Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D130KA",  # Bypass Lower Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D130KB",  # Bypass Lower Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D130Z4",  # Bypass Lower Esophagus to Cutaneous, Open Approach
        "0D130Z6",  # Bypass Lower Esophagus to Stomach, Open Approach
        "0D130Z9",  # Bypass Lower Esophagus to Duodenum, Open Approach
        "0D130ZA",  # Bypass Lower Esophagus to Jejunum, Open Approach
        "0D130ZB",  # Bypass Lower Esophagus to Ileum, Open Approach
        "0D15074",  # Bypass Esophagus to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D15076",  # Bypass Esophagus to Stomach with Autologous Tissue Substitute, Open Approach
        "0D15079",  # Bypass Esophagus to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1507A",  # Bypass Esophagus to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1507B",  # Bypass Esophagus to Ileum with Autologous Tissue Substitute, Open Approach
        "0D150J4",  # Bypass Esophagus to Cutaneous with Synthetic Substitute, Open Approach
        "0D150J6",  # Bypass Esophagus to Stomach with Synthetic Substitute, Open Approach
        "0D150J9",  # Bypass Esophagus to Duodenum with Synthetic Substitute, Open Approach
        "0D150JA",  # Bypass Esophagus to Jejunum with Synthetic Substitute, Open Approach
        "0D150JB",  # Bypass Esophagus to Ileum with Synthetic Substitute, Open Approach
        "0D150K4",  # Bypass Esophagus to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D150K6",  # Bypass Esophagus to Stomach with Nonautologous Tissue Substitute, Open Approach
        "0D150K9",  # Bypass Esophagus to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D150KA",  # Bypass Esophagus to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D150KB",  # Bypass Esophagus to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D150Z4",  # Bypass Esophagus to Cutaneous, Open Approach
        "0D150Z6",  # Bypass Esophagus to Stomach, Open Approach
        "0D150Z9",  # Bypass Esophagus to Duodenum, Open Approach
        "0D150ZA",  # Bypass Esophagus to Jejunum, Open Approach
        "0D150ZB",  # Bypass Esophagus to Ileum, Open Approach
        "0D16079",  # Bypass Stomach to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1607A",  # Bypass Stomach to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1607B",  # Bypass Stomach to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1607L",  # Bypass Stomach to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D160J9",  # Bypass Stomach to Duodenum with Synthetic Substitute, Open Approach
        "0D160JA",  # Bypass Stomach to Jejunum with Synthetic Substitute, Open Approach
        "0D160JB",  # Bypass Stomach to Ileum with Synthetic Substitute, Open Approach
        "0D160JL",  # Bypass Stomach to Transverse Colon with Synthetic Substitute, Open Approach
        "0D160K9",  # Bypass Stomach to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D160KA",  # Bypass Stomach to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D160KB",  # Bypass Stomach to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D160KL",  # Bypass Stomach to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D160Z9",  # Bypass Stomach to Duodenum, Open Approach
        "0D160ZA",  # Bypass Stomach to Jejunum, Open Approach
        "0D160ZB",  # Bypass Stomach to Ileum, Open Approach
        "0D160ZL",  # Bypass Stomach to Transverse Colon, Open Approach
        "0D19074",  # Bypass Duodenum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D19079",  # Bypass Duodenum to Duodenum with Autologous Tissue Substitute, Open Approach
        "0D1907A",  # Bypass Duodenum to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1907B",  # Bypass Duodenum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1907L",  # Bypass Duodenum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D190J4",  # Bypass Duodenum to Cutaneous with Synthetic Substitute, Open Approach
        "0D190J9",  # Bypass Duodenum to Duodenum with Synthetic Substitute, Open Approach
        "0D190JA",  # Bypass Duodenum to Jejunum with Synthetic Substitute, Open Approach
        "0D190JB",  # Bypass Duodenum to Ileum with Synthetic Substitute, Open Approach
        "0D190JL",  # Bypass Duodenum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D190K4",  # Bypass Duodenum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D190K9",  # Bypass Duodenum to Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0D190KA",  # Bypass Duodenum to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D190KB",  # Bypass Duodenum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D190KL",  # Bypass Duodenum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D190Z4",  # Bypass Duodenum to Cutaneous, Open Approach
        "0D190Z9",  # Bypass Duodenum to Duodenum, Open Approach
        "0D190ZA",  # Bypass Duodenum to Jejunum, Open Approach
        "0D190ZB",  # Bypass Duodenum to Ileum, Open Approach
        "0D190ZL",  # Bypass Duodenum to Transverse Colon, Open Approach
        "0D1A074",  # Bypass Jejunum to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0D1A07A",  # Bypass Jejunum to Jejunum with Autologous Tissue Substitute, Open Approach
        "0D1A07B",  # Bypass Jejunum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1A07H",  # Bypass Jejunum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1A07K",  # Bypass Jejunum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07L",  # Bypass Jejunum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07M",  # Bypass Jejunum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07N",  # Bypass Jejunum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1A07P",  # Bypass Jejunum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1A07Q",  # Bypass Jejunum to Anus with Autologous Tissue Substitute, Open Approach
        "0D1A0J4",  # Bypass Jejunum to Cutaneous with Synthetic Substitute, Open Approach
        "0D1A0JA",  # Bypass Jejunum to Jejunum with Synthetic Substitute, Open Approach
        "0D1A0JB",  # Bypass Jejunum to Ileum with Synthetic Substitute, Open Approach
        "0D1A0JH",  # Bypass Jejunum to Cecum with Synthetic Substitute, Open Approach
        "0D1A0JK",  # Bypass Jejunum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1A0JL",  # Bypass Jejunum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1A0JM",  # Bypass Jejunum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1A0JN",  # Bypass Jejunum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1A0JP",  # Bypass Jejunum to Rectum with Synthetic Substitute, Open Approach
        "0D1A0JQ",  # Bypass Jejunum to Anus with Synthetic Substitute, Open Approach
        "0D1A0K4",  # Bypass Jejunum to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KA",  # Bypass Jejunum to Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KB",  # Bypass Jejunum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KH",  # Bypass Jejunum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KK",  # Bypass Jejunum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KL",  # Bypass Jejunum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KM",  # Bypass Jejunum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KN",  # Bypass Jejunum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KP",  # Bypass Jejunum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1A0KQ",  # Bypass Jejunum to Anus with Nonautologous Tissue Substitute, Open Approach
        "0D1A0Z4",  # Bypass Jejunum to Cutaneous, Open Approach
        "0D1A0ZA",  # Bypass Jejunum to Jejunum, Open Approach
        "0D1A0ZB",  # Bypass Jejunum to Ileum, Open Approach
        "0D1A0ZH",  # Bypass Jejunum to Cecum, Open Approach
        "0D1A0ZK",  # Bypass Jejunum to Ascending Colon, Open Approach
        "0D1A0ZL",  # Bypass Jejunum to Transverse Colon, Open Approach
        "0D1A0ZM",  # Bypass Jejunum to Descending Colon, Open Approach
        "0D1A0ZN",  # Bypass Jejunum to Sigmoid Colon, Open Approach
        "0D1A0ZP",  # Bypass Jejunum to Rectum, Open Approach
        "0D1A0ZQ",  # Bypass Jejunum to Anus, Open Approach
        "0D1B07B",  # Bypass Ileum to Ileum with Autologous Tissue Substitute, Open Approach
        "0D1B07H",  # Bypass Ileum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1B07K",  # Bypass Ileum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07L",  # Bypass Ileum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07M",  # Bypass Ileum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07N",  # Bypass Ileum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1B07P",  # Bypass Ileum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1B07Q",  # Bypass Ileum to Anus with Autologous Tissue Substitute, Open Approach
        "0D1B0JB",  # Bypass Ileum to Ileum with Synthetic Substitute, Open Approach
        "0D1B0JH",  # Bypass Ileum to Cecum with Synthetic Substitute, Open Approach
        "0D1B0JK",  # Bypass Ileum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1B0JL",  # Bypass Ileum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1B0JM",  # Bypass Ileum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1B0JN",  # Bypass Ileum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1B0JP",  # Bypass Ileum to Rectum with Synthetic Substitute, Open Approach
        "0D1B0JQ",  # Bypass Ileum to Anus with Synthetic Substitute, Open Approach
        "0D1B0KB",  # Bypass Ileum to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KH",  # Bypass Ileum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KK",  # Bypass Ileum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KL",  # Bypass Ileum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KM",  # Bypass Ileum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KN",  # Bypass Ileum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KP",  # Bypass Ileum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1B0KQ",  # Bypass Ileum to Anus with Nonautologous Tissue Substitute, Open Approach
        "0D1B0Z4",  # Bypass Ileum to Cutaneous, Open Approach
        "0D1B0ZB",  # Bypass Ileum to Ileum, Open Approach
        "0D1B0ZH",  # Bypass Ileum to Cecum, Open Approach
        "0D1B0ZK",  # Bypass Ileum to Ascending Colon, Open Approach
        "0D1B0ZL",  # Bypass Ileum to Transverse Colon, Open Approach
        "0D1B0ZM",  # Bypass Ileum to Descending Colon, Open Approach
        "0D1B0ZN",  # Bypass Ileum to Sigmoid Colon, Open Approach
        "0D1B0ZP",  # Bypass Ileum to Rectum, Open Approach
        "0D1B0ZQ",  # Bypass Ileum to Anus, Open Approach
        "0D1H07H",  # Bypass Cecum to Cecum with Autologous Tissue Substitute, Open Approach
        "0D1H07K",  # Bypass Cecum to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07L",  # Bypass Cecum to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07M",  # Bypass Cecum to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07N",  # Bypass Cecum to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1H07P",  # Bypass Cecum to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1H0JH",  # Bypass Cecum to Cecum with Synthetic Substitute, Open Approach
        "0D1H0JK",  # Bypass Cecum to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1H0JL",  # Bypass Cecum to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1H0JM",  # Bypass Cecum to Descending Colon with Synthetic Substitute, Open Approach
        "0D1H0JN",  # Bypass Cecum to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1H0JP",  # Bypass Cecum to Rectum with Synthetic Substitute, Open Approach
        "0D1H0KH",  # Bypass Cecum to Cecum with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KK",  # Bypass Cecum to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KL",  # Bypass Cecum to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KM",  # Bypass Cecum to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KN",  # Bypass Cecum to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1H0KP",  # Bypass Cecum to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1H0ZH",  # Bypass Cecum to Cecum, Open Approach
        "0D1H0ZK",  # Bypass Cecum to Ascending Colon, Open Approach
        "0D1H0ZL",  # Bypass Cecum to Transverse Colon, Open Approach
        "0D1H0ZM",  # Bypass Cecum to Descending Colon, Open Approach
        "0D1H0ZN",  # Bypass Cecum to Sigmoid Colon, Open Approach
        "0D1H0ZP",  # Bypass Cecum to Rectum, Open Approach
        "0D1K07K",  # Bypass Ascending Colon to Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07L",  # Bypass Ascending Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07M",  # Bypass Ascending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07N",  # Bypass Ascending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1K07P",  # Bypass Ascending Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1K0JK",  # Bypass Ascending Colon to Ascending Colon with Synthetic Substitute, Open Approach
        "0D1K0JL",  # Bypass Ascending Colon to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1K0JM",  # Bypass Ascending Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1K0JN",  # Bypass Ascending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1K0JP",  # Bypass Ascending Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1K0KK",  # Bypass Ascending Colon to Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KL",  # Bypass Ascending Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KM",  # Bypass Ascending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KN",  # Bypass Ascending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1K0KP",  # Bypass Ascending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1K0ZK",  # Bypass Ascending Colon to Ascending Colon, Open Approach
        "0D1K0ZL",  # Bypass Ascending Colon to Transverse Colon, Open Approach
        "0D1K0ZM",  # Bypass Ascending Colon to Descending Colon, Open Approach
        "0D1K0ZN",  # Bypass Ascending Colon to Sigmoid Colon, Open Approach
        "0D1K0ZP",  # Bypass Ascending Colon to Rectum, Open Approach
        "0D1L07L",  # Bypass Transverse Colon to Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07M",  # Bypass Transverse Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07N",  # Bypass Transverse Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1L07P",  # Bypass Transverse Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1L0JL",  # Bypass Transverse Colon to Transverse Colon with Synthetic Substitute, Open Approach
        "0D1L0JM",  # Bypass Transverse Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1L0JN",  # Bypass Transverse Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1L0JP",  # Bypass Transverse Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1L0KL",  # Bypass Transverse Colon to Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KM",  # Bypass Transverse Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KN",  # Bypass Transverse Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1L0KP",  # Bypass Transverse Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1L0ZL",  # Bypass Transverse Colon to Transverse Colon, Open Approach
        "0D1L0ZM",  # Bypass Transverse Colon to Descending Colon, Open Approach
        "0D1L0ZN",  # Bypass Transverse Colon to Sigmoid Colon, Open Approach
        "0D1L0ZP",  # Bypass Transverse Colon to Rectum, Open Approach
        "0D1M07M",  # Bypass Descending Colon to Descending Colon with Autologous Tissue Substitute, Open Approach
        "0D1M07N",  # Bypass Descending Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1M07P",  # Bypass Descending Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1M0JM",  # Bypass Descending Colon to Descending Colon with Synthetic Substitute, Open Approach
        "0D1M0JN",  # Bypass Descending Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1M0JP",  # Bypass Descending Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1M0KM",  # Bypass Descending Colon to Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1M0KN",  # Bypass Descending Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1M0KP",  # Bypass Descending Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1M0ZM",  # Bypass Descending Colon to Descending Colon, Open Approach
        "0D1M0ZN",  # Bypass Descending Colon to Sigmoid Colon, Open Approach
        "0D1M0ZP",  # Bypass Descending Colon to Rectum, Open Approach
        "0D1N07N",  # Bypass Sigmoid Colon to Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0D1N07P",  # Bypass Sigmoid Colon to Rectum with Autologous Tissue Substitute, Open Approach
        "0D1N0JN",  # Bypass Sigmoid Colon to Sigmoid Colon with Synthetic Substitute, Open Approach
        "0D1N0JP",  # Bypass Sigmoid Colon to Rectum with Synthetic Substitute, Open Approach
        "0D1N0KN",  # Bypass Sigmoid Colon to Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0D1N0KP",  # Bypass Sigmoid Colon to Rectum with Nonautologous Tissue Substitute, Open Approach
        "0D1N0Z4",  # Bypass Sigmoid Colon to Cutaneous, Open Approach
        "0D1N0ZN",  # Bypass Sigmoid Colon to Sigmoid Colon, Open Approach
        "0D1N0ZP",  # Bypass Sigmoid Colon to Rectum, Open Approach
        "0D590ZZ",  # Destruction of Duodenum, Open Approach
        "0D5E0ZZ",  # Destruction of Large Intestine, Open Approach
        "0D5F0ZZ",  # Destruction of Right Large Intestine, Open Approach
        "0D5G0ZZ",  # Destruction of Left Large Intestine, Open Approach
        "0D5H0ZZ",  # Destruction of Cecum, Open Approach
        "0D5K0ZZ",  # Destruction of Ascending Colon, Open Approach
        "0D5L0ZZ",  # Destruction of Transverse Colon, Open Approach
        "0D5M0ZZ",  # Destruction of Descending Colon, Open Approach
        "0D5N0ZZ",  # Destruction of Sigmoid Colon, Open Approach
        "0D710DZ",  # Dilation of Upper Esophagus with Intraluminal Device, Open Approach
        "0D710ZZ",  # Dilation of Upper Esophagus, Open Approach
        "0D720DZ",  # Dilation of Middle Esophagus with Intraluminal Device, Open Approach
        "0D720ZZ",  # Dilation of Middle Esophagus, Open Approach
        "0D730DZ",  # Dilation of Lower Esophagus with Intraluminal Device, Open Approach
        "0D730ZZ",  # Dilation of Lower Esophagus, Open Approach
        "0D740DZ",  # Dilation of Esophagogastric Junction with Intraluminal Device, Open Approach
        "0D740ZZ",  # Dilation of Esophagogastric Junction, Open Approach
        "0D750DZ",  # Dilation of Esophagus with Intraluminal Device, Open Approach
        "0D750ZZ",  # Dilation of Esophagus, Open Approach
        "0D760DZ",  # Dilation of Stomach with Intraluminal Device, Open Approach
        "0D760ZZ",  # Dilation of Stomach, Open Approach
        "0D767DZ",  # Dilation of Stomach with Intraluminal Device, Via Natural or Artificial Opening
        "0D767ZZ",  # Dilation of Stomach, Via Natural or Artificial Opening
        "0D770DZ",  # Dilation of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0D770ZZ",  # Dilation of Stomach, Pylorus, Open Approach
        "0D777DZ",  # Dilation of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0D777ZZ",  # Dilation of Stomach, Pylorus, Via Natural or Artificial Opening
        "0D780ZZ",  # Dilation of Small Intestine, Open Approach
        "0D790ZZ",  # Dilation of Duodenum, Open Approach
        "0D7A0ZZ",  # Dilation of Jejunum, Open Approach
        "0D7B0ZZ",  # Dilation of Ileum, Open Approach
        "0D7C0ZZ",  # Dilation of Ileocecal Valve, Open Approach
        "0D7E0ZZ",  # Dilation of Large Intestine, Open Approach
        "0D7F0ZZ",  # Dilation of Right Large Intestine, Open Approach
        "0D7G0ZZ",  # Dilation of Left Large Intestine, Open Approach
        "0D7H0ZZ",  # Dilation of Cecum, Open Approach
        "0D7K0ZZ",  # Dilation of Ascending Colon, Open Approach
        "0D7L0ZZ",  # Dilation of Transverse Colon, Open Approach
        "0D7M0ZZ",  # Dilation of Descending Colon, Open Approach
        "0D7N0ZZ",  # Dilation of Sigmoid Colon, Open Approach
        "0D7P0DZ",  # Dilation of Rectum with Intraluminal Device, Open Approach
        "0D7P0ZZ",  # Dilation of Rectum, Open Approach
        "0D840ZZ",  # Division of Esophagogastric Junction, Open Approach
        "0D847ZZ",  # Division of Esophagogastric Junction, Via Natural or Artificial Opening
        "0D9100Z",  # Drainage of Upper Esophagus with Drainage Device, Open Approach
        "0D910ZZ",  # Drainage of Upper Esophagus, Open Approach
        "0D917ZZ",  # Drainage of Upper Esophagus, Via Natural or Artificial Opening
        "0D9200Z",  # Drainage of Middle Esophagus with Drainage Device, Open Approach
        "0D920ZZ",  # Drainage of Middle Esophagus, Open Approach
        "0D927ZZ",  # Drainage of Middle Esophagus, Via Natural or Artificial Opening
        "0D9300Z",  # Drainage of Lower Esophagus with Drainage Device, Open Approach
        "0D930ZZ",  # Drainage of Lower Esophagus, Open Approach
        "0D937ZZ",  # Drainage of Lower Esophagus, Via Natural or Artificial Opening
        "0D9400Z",  # Drainage of Esophagogastric Junction with Drainage Device, Open Approach
        "0D940ZZ",  # Drainage of Esophagogastric Junction, Open Approach
        "0D947ZZ",  # Drainage of Esophagogastric Junction, Via Natural or Artificial Opening
        "0D9500Z",  # Drainage of Esophagus with Drainage Device, Open Approach
        "0D950ZZ",  # Drainage of Esophagus, Open Approach
        "0D9800Z",  # Drainage of Small Intestine with Drainage Device, Open Approach
        "0D980ZZ",  # Drainage of Small Intestine, Open Approach
        "0D987ZZ",  # Drainage of Small Intestine, Via Natural or Artificial Opening
        "0D9900Z",  # Drainage of Duodenum with Drainage Device, Open Approach
        "0D990ZZ",  # Drainage of Duodenum, Open Approach
        "0D997ZZ",  # Drainage of Duodenum, Via Natural or Artificial Opening
        "0D9A00Z",  # Drainage of Jejunum with Drainage Device, Open Approach
        "0D9A0ZZ",  # Drainage of Jejunum, Open Approach
        "0D9A7ZZ",  # Drainage of Jejunum, Via Natural or Artificial Opening
        "0D9B00Z",  # Drainage of Ileum with Drainage Device, Open Approach
        "0D9B0ZZ",  # Drainage of Ileum, Open Approach
        "0D9B7ZZ",  # Drainage of Ileum, Via Natural or Artificial Opening
        "0D9C00Z",  # Drainage of Ileocecal Valve with Drainage Device, Open Approach
        "0D9C0ZZ",  # Drainage of Ileocecal Valve, Open Approach
        "0D9C7ZZ",  # Drainage of Ileocecal Valve, Via Natural or Artificial Opening
        "0D9E00Z",  # Drainage of Large Intestine with Drainage Device, Open Approach
        "0D9E0ZZ",  # Drainage of Large Intestine, Open Approach
        "0D9E7ZZ",  # Drainage of Large Intestine, Via Natural or Artificial Opening
        "0D9F00Z",  # Drainage of Right Large Intestine with Drainage Device, Open Approach
        "0D9F0ZZ",  # Drainage of Right Large Intestine, Open Approach
        "0D9F7ZZ",  # Drainage of Right Large Intestine, Via Natural or Artificial Opening
        "0D9G00Z",  # Drainage of Left Large Intestine with Drainage Device, Open Approach
        "0D9G0ZZ",  # Drainage of Left Large Intestine, Open Approach
        "0D9G7ZZ",  # Drainage of Left Large Intestine, Via Natural or Artificial Opening
        "0D9H00Z",  # Drainage of Cecum with Drainage Device, Open Approach
        "0D9H0ZZ",  # Drainage of Cecum, Open Approach
        "0D9H7ZZ",  # Drainage of Cecum, Via Natural or Artificial Opening
        "0D9K00Z",  # Drainage of Ascending Colon with Drainage Device, Open Approach
        "0D9K0ZZ",  # Drainage of Ascending Colon, Open Approach
        "0D9K7ZZ",  # Drainage of Ascending Colon, Via Natural or Artificial Opening
        "0D9L00Z",  # Drainage of Transverse Colon with Drainage Device, Open Approach
        "0D9L0ZZ",  # Drainage of Transverse Colon, Open Approach
        "0D9L7ZZ",  # Drainage of Transverse Colon, Via Natural or Artificial Opening
        "0D9M00Z",  # Drainage of Descending Colon with Drainage Device, Open Approach
        "0D9M0ZZ",  # Drainage of Descending Colon, Open Approach
        "0D9M7ZZ",  # Drainage of Descending Colon, Via Natural or Artificial Opening
        "0D9N00Z",  # Drainage of Sigmoid Colon with Drainage Device, Open Approach
        "0D9N0ZZ",  # Drainage of Sigmoid Colon, Open Approach
        "0D9N7ZZ",  # Drainage of Sigmoid Colon, Via Natural or Artificial Opening
        "0D9P00Z",  # Drainage of Rectum with Drainage Device, Open Approach
        "0D9P0ZZ",  # Drainage of Rectum, Open Approach
        "0D9P7ZZ",  # Drainage of Rectum, Via Natural or Artificial Opening
        "0D9V00Z",  # Drainage of Mesentery with Drainage Device, Open Approach
        "0D9V0ZZ",  # Drainage of Mesentery, Open Approach
        "0D9W00Z",  # Drainage of Peritoneum with Drainage Device, Open Approach
        "0D9W0ZZ",  # Drainage of Peritoneum, Open Approach
        "0DB10ZZ",  # Excision of Upper Esophagus, Open Approach
        "0DB17ZZ",  # Excision of Upper Esophagus, Via Natural or Artificial Opening
        "0DB20ZZ",  # Excision of Middle Esophagus, Open Approach
        "0DB27ZZ",  # Excision of Middle Esophagus, Via Natural or Artificial Opening
        "0DB30ZZ",  # Excision of Lower Esophagus, Open Approach
        "0DB37ZZ",  # Excision of Lower Esophagus, Via Natural or Artificial Opening
        "0DB40ZZ",  # Excision of Esophagogastric Junction, Open Approach
        "0DB47ZZ",  # Excision of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DB50ZZ",  # Excision of Esophagus, Open Approach
        "0DB57ZZ",  # Excision of Esophagus, Via Natural or Artificial Opening
        "0DB60Z3",  # Excision of Stomach, Open Approach, Vertical
        "0DB60ZZ",  # Excision of Stomach, Open Approach
        "0DB67Z3",  # Excision of Stomach, Via Natural or Artificial Opening, Vertical
        "0DB67ZZ",  # Excision of Stomach, Via Natural or Artificial Opening
        "0DB80ZZ",  # Excision of Small Intestine, Open Approach
        "0DB87ZZ",  # Excision of Small Intestine, Via Natural or Artificial Opening
        "0DB90ZZ",  # Excision of Duodenum, Open Approach
        "0DB97ZZ",  # Excision of Duodenum, Via Natural or Artificial Opening
        "0DBA0ZZ",  # Excision of Jejunum, Open Approach
        "0DBA7ZZ",  # Excision of Jejunum, Via Natural or Artificial Opening
        "0DBB0ZZ",  # Excision of Ileum, Open Approach
        "0DBB7ZZ",  # Excision of Ileum, Via Natural or Artificial Opening
        "0DBC0ZZ",  # Excision of Ileocecal Valve, Open Approach
        "0DBC7ZZ",  # Excision of Ileocecal Valve, Via Natural or Artificial Opening
        "0DBE0ZZ",  # Excision of Large Intestine, Open Approach
        "0DBE4ZZ",  # Excision of Large Intestine, Percutaneous Endoscopic Approach
        "0DBE7ZZ",  # Excision of Large Intestine, Via Natural or Artificial Opening
        "0DBF0ZZ",  # Excision of Right Large Intestine, Open Approach
        "0DBF4ZZ",  # Excision of Right Large Intestine, Percutaneous Endoscopic Approach
        "0DBF7ZZ",  # Excision of Right Large Intestine, Via Natural or Artificial Opening
        "0DBG0ZZ",  # Excision of Left Large Intestine, Open Approach
        "0DBG4ZZ",  # Excision of Left Large Intestine, Percutaneous Endoscopic Approach
        "0DBG7ZZ",  # Excision of Left Large Intestine, Via Natural or Artificial Opening
        "0DBH0ZZ",  # Excision of Cecum, Open Approach
        "0DBH4ZZ",  # Excision of Cecum, Percutaneous Endoscopic Approach
        "0DBH7ZZ",  # Excision of Cecum, Via Natural or Artificial Opening
        "0DBK0ZZ",  # Excision of Ascending Colon, Open Approach
        "0DBK4ZZ",  # Excision of Ascending Colon, Percutaneous Endoscopic Approach
        "0DBK7ZZ",  # Excision of Ascending Colon, Via Natural or Artificial Opening
        "0DBL0ZZ",  # Excision of Transverse Colon, Open Approach
        "0DBL4ZZ",  # Excision of Transverse Colon, Percutaneous Endoscopic Approach
        "0DBL7ZZ",  # Excision of Transverse Colon, Via Natural or Artificial Opening
        "0DBM0ZZ",  # Excision of Descending Colon, Open Approach
        "0DBM4ZZ",  # Excision of Descending Colon, Percutaneous Endoscopic Approach
        "0DBM7ZZ",  # Excision of Descending Colon, Via Natural or Artificial Opening
        "0DBN0ZZ",  # Excision of Sigmoid Colon, Open Approach
        "0DBN4ZZ",  # Excision of Sigmoid Colon, Percutaneous Endoscopic Approach
        "0DBN7ZZ",  # Excision of Sigmoid Colon, Via Natural or Artificial Opening
        "0DBP0ZZ",  # Excision of Rectum, Open Approach
        "0DBP7ZZ",  # Excision of Rectum, Via Natural or Artificial Opening
        "0DC10ZZ",  # Extirpation of Matter from Upper Esophagus, Open Approach
        "0DC20ZZ",  # Extirpation of Matter from Middle Esophagus, Open Approach
        "0DC30ZZ",  # Extirpation of Matter from Lower Esophagus, Open Approach
        "0DC40ZZ",  # Extirpation of Matter from Esophagogastric Junction, Open Approach
        "0DC50ZZ",  # Extirpation of Matter from Esophagus, Open Approach
        "0DC80ZZ",  # Extirpation of Matter from Small Intestine, Open Approach
        "0DC90ZZ",  # Extirpation of Matter from Duodenum, Open Approach
        "0DCA0ZZ",  # Extirpation of Matter from Jejunum, Open Approach
        "0DCB0ZZ",  # Extirpation of Matter from Ileum, Open Approach
        "0DCC0ZZ",  # Extirpation of Matter from Ileocecal Valve, Open Approach
        "0DCE0ZZ",  # Extirpation of Matter from Large Intestine, Open Approach
        "0DCF0ZZ",  # Extirpation of Matter from Right Large Intestine, Open Approach
        "0DCG0ZZ",  # Extirpation of Matter from Left Large Intestine, Open Approach
        "0DCH0ZZ",  # Extirpation of Matter from Cecum, Open Approach
        "0DCK0ZZ",  # Extirpation of Matter from Ascending Colon, Open Approach
        "0DCL0ZZ",  # Extirpation of Matter from Transverse Colon, Open Approach
        "0DCM0ZZ",  # Extirpation of Matter from Descending Colon, Open Approach
        "0DCN0ZZ",  # Extirpation of Matter from Sigmoid Colon, Open Approach
        "0DCP0ZZ",  # Extirpation of Matter from Rectum, Open Approach
        "0DCV0ZZ",  # Extirpation of Matter from Mesentery, Open Approach
        "0DCW0ZZ",  # Extirpation of Matter from Peritoneum, Open Approach
        "0DF50ZZ",  # Fragmentation in Esophagus, Open Approach
        "0DF60ZZ",  # Fragmentation in Stomach, Open Approach
        "0DF80ZZ",  # Fragmentation in Small Intestine, Open Approach
        "0DF90ZZ",  # Fragmentation in Duodenum, Open Approach
        "0DFA0ZZ",  # Fragmentation in Jejunum, Open Approach
        "0DFB0ZZ",  # Fragmentation in Ileum, Open Approach
        "0DFE0ZZ",  # Fragmentation in Large Intestine, Open Approach
        "0DFF0ZZ",  # Fragmentation in Right Large Intestine, Open Approach
        "0DFG0ZZ",  # Fragmentation in Left Large Intestine, Open Approach
        "0DFH0ZZ",  # Fragmentation in Cecum, Open Approach
        "0DFK0ZZ",  # Fragmentation in Ascending Colon, Open Approach
        "0DFL0ZZ",  # Fragmentation in Transverse Colon, Open Approach
        "0DFM0ZZ",  # Fragmentation in Descending Colon, Open Approach
        "0DFN0ZZ",  # Fragmentation in Sigmoid Colon, Open Approach
        "0DH502Z",  # Insertion of Monitoring Device into Esophagus, Open Approach
        "0DH503Z",  # Insertion of Infusion Device into Esophagus, Open Approach
        "0DH572Z",  # Insertion of Monitoring Device into Esophagus, Via Natural or Artificial Opening
        "0DH573Z",  # Insertion of Infusion Device into Esophagus, Via Natural or Artificial Opening
        "0DH60DZ",  # Insertion of Intraluminal Device into Stomach, Open Approach
        "0DH67DZ",  # Insertion of Intraluminal Device into Stomach, Via Natural or Artificial Opening
        "0DH802Z",  # Insertion of Monitoring Device into Small Intestine, Open Approach
        "0DH803Z",  # Insertion of Infusion Device into Small Intestine, Open Approach
        "0DH872Z",  # Insertion of Monitoring Device into Small Intestine, Via Natural or Artificial Opening
        "0DH873Z",  # Insertion of Infusion Device into Small Intestine, Via Natural or Artificial Opening
        "0DH902Z",  # Insertion of Monitoring Device into Duodenum, Open Approach
        "0DH903Z",  # Insertion of Infusion Device into Duodenum, Open Approach
        "0DH972Z",  # Insertion of Monitoring Device into Duodenum, Via Natural or Artificial Opening
        "0DH973Z",  # Insertion of Infusion Device into Duodenum, Via Natural or Artificial Opening
        "0DHA02Z",  # Insertion of Monitoring Device into Jejunum, Open Approach
        "0DHA03Z",  # Insertion of Infusion Device into Jejunum, Open Approach
        "0DHA72Z",  # Insertion of Monitoring Device into Jejunum, Via Natural or Artificial Opening
        "0DHA73Z",  # Insertion of Infusion Device into Jejunum, Via Natural or Artificial Opening
        "0DHB02Z",  # Insertion of Monitoring Device into Ileum, Open Approach
        "0DHB03Z",  # Insertion of Infusion Device into Ileum, Open Approach
        "0DHB72Z",  # Insertion of Monitoring Device into Ileum, Via Natural or Artificial Opening
        "0DHB73Z",  # Insertion of Infusion Device into Ileum, Via Natural or Artificial Opening
        "0DJ00ZZ",  # Inspection of Upper Intestinal Tract, Open Approach
        "0DJ60ZZ",  # Inspection of Stomach, Open Approach
        "0DJD0ZZ",  # Inspection of Lower Intestinal Tract, Open Approach
        "0DJU0ZZ",  # Inspection of Omentum, Open Approach
        "0DJV0ZZ",  # Inspection of Mesentery, Open Approach
        "0DJW0ZZ",  # Inspection of Peritoneum, Open Approach
        "0DL60CZ",  # Occlusion of Stomach with Extraluminal Device, Open Approach
        "0DL60DZ",  # Occlusion of Stomach with Intraluminal Device, Open Approach
        "0DL60ZZ",  # Occlusion of Stomach, Open Approach
        "0DL67DZ",  # Occlusion of Stomach with Intraluminal Device, Via Natural or Artificial Opening
        "0DL67ZZ",  # Occlusion of Stomach, Via Natural or Artificial Opening
        "0DL70CZ",  # Occlusion of Stomach, Pylorus with Extraluminal Device, Open Approach
        "0DL70DZ",  # Occlusion of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0DL70ZZ",  # Occlusion of Stomach, Pylorus, Open Approach
        "0DL77DZ",  # Occlusion of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0DL77ZZ",  # Occlusion of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DM50ZZ",  # Reattachment of Esophagus, Open Approach
        "0DM60ZZ",  # Reattachment of Stomach, Open Approach
        "0DM80ZZ",  # Reattachment of Small Intestine, Open Approach
        "0DM90ZZ",  # Reattachment of Duodenum, Open Approach
        "0DMA0ZZ",  # Reattachment of Jejunum, Open Approach
        "0DMB0ZZ",  # Reattachment of Ileum, Open Approach
        "0DME0ZZ",  # Reattachment of Large Intestine, Open Approach
        "0DMF0ZZ",  # Reattachment of Right Large Intestine, Open Approach
        "0DMG0ZZ",  # Reattachment of Left Large Intestine, Open Approach
        "0DMH0ZZ",  # Reattachment of Cecum, Open Approach
        "0DMK0ZZ",  # Reattachment of Ascending Colon, Open Approach
        "0DML0ZZ",  # Reattachment of Transverse Colon, Open Approach
        "0DMM0ZZ",  # Reattachment of Descending Colon, Open Approach
        "0DMN0ZZ",  # Reattachment of Sigmoid Colon, Open Approach
        "0DMP0ZZ",  # Reattachment of Rectum, Open Approach
        "0DN10ZZ",  # Release Upper Esophagus, Open Approach
        "0DN17ZZ",  # Release Upper Esophagus, Via Natural or Artificial Opening
        "0DN20ZZ",  # Release Middle Esophagus, Open Approach
        "0DN27ZZ",  # Release Middle Esophagus, Via Natural or Artificial Opening
        "0DN30ZZ",  # Release Lower Esophagus, Open Approach
        "0DN37ZZ",  # Release Lower Esophagus, Via Natural or Artificial Opening
        "0DN40ZZ",  # Release Esophagogastric Junction, Open Approach
        "0DN47ZZ",  # Release Esophagogastric Junction, Via Natural or Artificial Opening
        "0DN50ZZ",  # Release Esophagus, Open Approach
        "0DN57ZZ",  # Release Esophagus, Via Natural or Artificial Opening
        "0DN60ZZ",  # Release Stomach, Open Approach
        "0DN67ZZ",  # Release Stomach, Via Natural or Artificial Opening
        "0DN70ZZ",  # Release Stomach, Pylorus, Open Approach
        "0DN77ZZ",  # Release Stomach, Pylorus, Via Natural or Artificial Opening
        "0DN80ZZ",  # Release Small Intestine, Open Approach
        "0DN90ZZ",  # Release Duodenum, Open Approach
        "0DNA0ZZ",  # Release Jejunum, Open Approach
        "0DNB0ZZ",  # Release Ileum, Open Approach
        "0DNC0ZZ",  # Release Ileocecal Valve, Open Approach
        "0DNC7ZZ",  # Release Ileocecal Valve, Via Natural or Artificial Opening
        "0DNE0ZZ",  # Release Large Intestine, Open Approach
        "0DNF0ZZ",  # Release Right Large Intestine, Open Approach
        "0DNG0ZZ",  # Release Left Large Intestine, Open Approach
        "0DNH0ZZ",  # Release Cecum, Open Approach
        "0DNJ0ZZ",  # Release Appendix, Open Approach
        "0DNK0ZZ",  # Release Ascending Colon, Open Approach
        "0DNL0ZZ",  # Release Transverse Colon, Open Approach
        "0DNM0ZZ",  # Release Descending Colon, Open Approach
        "0DNN0ZZ",  # Release Sigmoid Colon, Open Approach
        "0DNV0ZZ",  # Release Mesentery, Open Approach
        "0DNW0ZZ",  # Release Peritoneum, Open Approach
        "0DP000Z",  # Removal of Drainage Device from Upper Intestinal Tract, Open Approach
        "0DP002Z",  # Removal of Monitoring Device from Upper Intestinal Tract, Open Approach
        "0DP003Z",  # Removal of Infusion Device from Upper Intestinal Tract, Open Approach
        "0DP007Z",  # Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Open Approach
        "0DP00CZ",  # Removal of Extraluminal Device from Upper Intestinal Tract, Open Approach
        "0DP00DZ",  # Removal of Intraluminal Device from Upper Intestinal Tract, Open Approach
        "0DP00JZ",  # Removal of Synthetic Substitute from Upper Intestinal Tract, Open Approach
        "0DP00KZ",  # Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Open Approach
        "0DP00UZ",  # Removal of Feeding Device from Upper Intestinal Tract, Open Approach
        "0DP072Z",  # Removal of Monitoring Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP073Z",  # Removal of Infusion Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP077Z",  # Removal of Autologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07CZ",  # Removal of Extraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07DZ",  # Removal of Intraluminal Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07JZ",  # Removal of Synthetic Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07KZ",  # Removal of Nonautologous Tissue Substitute from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP07UZ",  # Removal of Feeding Device from Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DP501Z",  # Removal of Radioactive Element from Esophagus, Open Approach
        "0DP502Z",  # Removal of Monitoring Device from Esophagus, Open Approach
        "0DP503Z",  # Removal of Infusion Device from Esophagus, Open Approach
        "0DP50UZ",  # Removal of Feeding Device from Esophagus, Open Approach
        "0DP57DZ",  # Removal of Intraluminal Device from Esophagus, Via Natural or Artificial Opening
        "0DPD00Z",  # Removal of Drainage Device from Lower Intestinal Tract, Open Approach
        "0DPD02Z",  # Removal of Monitoring Device from Lower Intestinal Tract, Open Approach
        "0DPD03Z",  # Removal of Infusion Device from Lower Intestinal Tract, Open Approach
        "0DPD07Z",  # Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0CZ",  # Removal of Extraluminal Device from Lower Intestinal Tract, Open Approach
        "0DPD0DZ",  # Removal of Intraluminal Device from Lower Intestinal Tract, Open Approach
        "0DPD0JZ",  # Removal of Synthetic Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0KZ",  # Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Open Approach
        "0DPD0UZ",  # Removal of Feeding Device from Lower Intestinal Tract, Open Approach
        "0DPD72Z",  # Removal of Monitoring Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD73Z",  # Removal of Infusion Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD77Z",  # Removal of Autologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7CZ",  # Removal of Extraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7DZ",  # Removal of Intraluminal Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7JZ",  # Removal of Synthetic Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7KZ",  # Removal of Nonautologous Tissue Substitute from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPD7UZ",  # Removal of Feeding Device from Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DPP01Z",  # Removal of Radioactive Element from Rectum, Open Approach
        "0DPU00Z",  # Removal of Drainage Device from Omentum, Open Approach
        "0DPU01Z",  # Removal of Radioactive Element from Omentum, Open Approach
        "0DPU07Z",  # Removal of Autologous Tissue Substitute from Omentum, Open Approach
        "0DPU0JZ",  # Removal of Synthetic Substitute from Omentum, Open Approach
        "0DPU0KZ",  # Removal of Nonautologous Tissue Substitute from Omentum, Open Approach
        "0DPV00Z",  # Removal of Drainage Device from Mesentery, Open Approach
        "0DPV01Z",  # Removal of Radioactive Element from Mesentery, Open Approach
        "0DPV07Z",  # Removal of Autologous Tissue Substitute from Mesentery, Open Approach
        "0DPV0JZ",  # Removal of Synthetic Substitute from Mesentery, Open Approach
        "0DPV0KZ",  # Removal of Nonautologous Tissue Substitute from Mesentery, Open Approach
        "0DPW00Z",  # Removal of Drainage Device from Peritoneum, Open Approach
        "0DPW01Z",  # Removal of Radioactive Element from Peritoneum, Open Approach
        "0DPW07Z",  # Removal of Autologous Tissue Substitute from Peritoneum, Open Approach
        "0DPW0JZ",  # Removal of Synthetic Substitute from Peritoneum, Open Approach
        "0DPW0KZ",  # Removal of Nonautologous Tissue Substitute from Peritoneum, Open Approach
        "0DQ10ZZ",  # Repair Upper Esophagus, Open Approach
        "0DQ17ZZ",  # Repair Upper Esophagus, Via Natural or Artificial Opening
        "0DQ20ZZ",  # Repair Middle Esophagus, Open Approach
        "0DQ27ZZ",  # Repair Middle Esophagus, Via Natural or Artificial Opening
        "0DQ30ZZ",  # Repair Lower Esophagus, Open Approach
        "0DQ37ZZ",  # Repair Lower Esophagus, Via Natural or Artificial Opening
        "0DQ40ZZ",  # Repair Esophagogastric Junction, Open Approach
        "0DQ47ZZ",  # Repair Esophagogastric Junction, Via Natural or Artificial Opening
        "0DQ50ZZ",  # Repair Esophagus, Open Approach
        "0DQ57ZZ",  # Repair Esophagus, Via Natural or Artificial Opening
        "0DQ60ZZ",  # Repair Stomach, Open Approach
        "0DQ67ZZ",  # Repair Stomach, Via Natural or Artificial Opening
        "0DQ70ZZ",  # Repair Stomach, Pylorus, Open Approach
        "0DQ77ZZ",  # Repair Stomach, Pylorus, Via Natural or Artificial Opening
        "0DQ80ZZ",  # Repair Small Intestine, Open Approach
        "0DQ87ZZ",  # Repair Small Intestine, Via Natural or Artificial Opening
        "0DQ90ZZ",  # Repair Duodenum, Open Approach
        "0DQ97ZZ",  # Repair Duodenum, Via Natural or Artificial Opening
        "0DQA0ZZ",  # Repair Jejunum, Open Approach
        "0DQA7ZZ",  # Repair Jejunum, Via Natural or Artificial Opening
        "0DQB0ZZ",  # Repair Ileum, Open Approach
        "0DQB7ZZ",  # Repair Ileum, Via Natural or Artificial Opening
        "0DQC0ZZ",  # Repair Ileocecal Valve, Open Approach
        "0DQC7ZZ",  # Repair Ileocecal Valve, Via Natural or Artificial Opening
        "0DQE0ZZ",  # Repair Large Intestine, Open Approach
        "0DQE7ZZ",  # Repair Large Intestine, Via Natural or Artificial Opening
        "0DQF0ZZ",  # Repair Right Large Intestine, Open Approach
        "0DQF7ZZ",  # Repair Right Large Intestine, Via Natural or Artificial Opening
        "0DQG0ZZ",  # Repair Left Large Intestine, Open Approach
        "0DQG7ZZ",  # Repair Left Large Intestine, Via Natural or Artificial Opening
        "0DQH0ZZ",  # Repair Cecum, Open Approach
        "0DQH7ZZ",  # Repair Cecum, Via Natural or Artificial Opening
        "0DQK0ZZ",  # Repair Ascending Colon, Open Approach
        "0DQK7ZZ",  # Repair Ascending Colon, Via Natural or Artificial Opening
        "0DQL0ZZ",  # Repair Transverse Colon, Open Approach
        "0DQL7ZZ",  # Repair Transverse Colon, Via Natural or Artificial Opening
        "0DQM0ZZ",  # Repair Descending Colon, Open Approach
        "0DQM7ZZ",  # Repair Descending Colon, Via Natural or Artificial Opening
        "0DQN0ZZ",  # Repair Sigmoid Colon, Open Approach
        "0DQN7ZZ",  # Repair Sigmoid Colon, Via Natural or Artificial Opening
        "0DQP0ZZ",  # Repair Rectum, Open Approach
        "0DQP7ZZ",  # Repair Rectum, Via Natural or Artificial Opening
        "0DQQ0ZZ",  # Repair Anus, Open Approach
        "0DQQ7ZZ",  # Repair Anus, Via Natural or Artificial Opening
        "0DQV0ZZ",  # Repair Mesentery, Open Approach
        "0DQW0ZZ",  # Repair Peritoneum, Open Approach
        "0DR507Z",  # Replacement of Esophagus with Autologous Tissue Substitute, Open Approach
        "0DR50JZ",  # Replacement of Esophagus with Synthetic Substitute, Open Approach
        "0DR50KZ",  # Replacement of Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DR577Z",  # Replacement of Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DR57JZ",  # Replacement of Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DR57KZ",  # Replacement of Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DRV07Z",  # Replacement of Mesentery with Autologous Tissue Substitute, Open Approach
        "0DRV0JZ",  # Replacement of Mesentery with Synthetic Substitute, Open Approach
        "0DRV0KZ",  # Replacement of Mesentery with Nonautologous Tissue Substitute, Open Approach
        "0DRW07Z",  # Replacement of Peritoneum with Autologous Tissue Substitute, Open Approach
        "0DRW0JZ",  # Replacement of Peritoneum with Synthetic Substitute, Open Approach
        "0DRW0KZ",  # Replacement of Peritoneum with Nonautologous Tissue Substitute, Open Approach
        "0DS50ZZ",  # Reposition Esophagus, Open Approach
        "0DS57ZZ",  # Reposition Esophagus, Via Natural or Artificial Opening
        "0DS5XZZ",  # Reposition Esophagus, External Approach
        "0DS60ZZ",  # Reposition Stomach, Open Approach
        "0DS67ZZ",  # Reposition Stomach, Via Natural or Artificial Opening
        "0DS6XZZ",  # Reposition Stomach, External Approach
        "0DS90ZZ",  # Reposition Duodenum, Open Approach
        "0DS97ZZ",  # Reposition Duodenum, Via Natural or Artificial Opening
        "0DSA0ZZ",  # Reposition Jejunum, Open Approach
        "0DSA7ZZ",  # Reposition Jejunum, Via Natural or Artificial Opening
        "0DSB0ZZ",  # Reposition Ileum, Open Approach
        "0DSB7ZZ",  # Reposition Ileum, Via Natural or Artificial Opening
        "0DSH0ZZ",  # Reposition Cecum, Open Approach
        "0DSH7ZZ",  # Reposition Cecum, Via Natural or Artificial Opening
        "0DSK0ZZ",  # Reposition Ascending Colon, Open Approach
        "0DSK7ZZ",  # Reposition Ascending Colon, Via Natural or Artificial Opening
        "0DSL0ZZ",  # Reposition Transverse Colon, Open Approach
        "0DSL7ZZ",  # Reposition Transverse Colon, Via Natural or Artificial Opening
        "0DSM0ZZ",  # Reposition Descending Colon, Open Approach
        "0DSM7ZZ",  # Reposition Descending Colon, Via Natural or Artificial Opening
        "0DSN0ZZ",  # Reposition Sigmoid Colon, Open Approach
        "0DSN7ZZ",  # Reposition Sigmoid Colon, Via Natural or Artificial Opening
        "0DSP0ZZ",  # Reposition Rectum, Open Approach
        "0DSP7ZZ",  # Reposition Rectum, Via Natural or Artificial Opening
        "0DT10ZZ",  # Resection of Upper Esophagus, Open Approach
        "0DT17ZZ",  # Resection of Upper Esophagus, Via Natural or Artificial Opening
        "0DT20ZZ",  # Resection of Middle Esophagus, Open Approach
        "0DT27ZZ",  # Resection of Middle Esophagus, Via Natural or Artificial Opening
        "0DT30ZZ",  # Resection of Lower Esophagus, Open Approach
        "0DT37ZZ",  # Resection of Lower Esophagus, Via Natural or Artificial Opening
        "0DT40ZZ",  # Resection of Esophagogastric Junction, Open Approach
        "0DT47ZZ",  # Resection of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DT50ZZ",  # Resection of Esophagus, Open Approach
        "0DT57ZZ",  # Resection of Esophagus, Via Natural or Artificial Opening
        "0DT60ZZ",  # Resection of Stomach, Open Approach
        "0DT67ZZ",  # Resection of Stomach, Via Natural or Artificial Opening
        "0DT70ZZ",  # Resection of Stomach, Pylorus, Open Approach
        "0DT77ZZ",  # Resection of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DT80ZZ",  # Resection of Small Intestine, Open Approach
        "0DT87ZZ",  # Resection of Small Intestine, Via Natural or Artificial Opening
        "0DT90ZZ",  # Resection of Duodenum, Open Approach
        "0DT97ZZ",  # Resection of Duodenum, Via Natural or Artificial Opening
        "0DTA0ZZ",  # Resection of Jejunum, Open Approach
        "0DTA7ZZ",  # Resection of Jejunum, Via Natural or Artificial Opening
        "0DTB0ZZ",  # Resection of Ileum, Open Approach
        "0DTB7ZZ",  # Resection of Ileum, Via Natural or Artificial Opening
        "0DTC0ZZ",  # Resection of Ileocecal Valve, Open Approach
        "0DTC7ZZ",  # Resection of Ileocecal Valve, Via Natural or Artificial Opening
        "0DTE0ZZ",  # Resection of Large Intestine, Open Approach
        "0DTE7ZZ",  # Resection of Large Intestine, Via Natural or Artificial Opening
        "0DTF0ZZ",  # Resection of Right Large Intestine, Open Approach
        "0DTF4ZZ",  # Resection of Right Large Intestine, Percutaneous Endoscopic Approach
        "0DTF7ZZ",  # Resection of Right Large Intestine, Via Natural or Artificial Opening
        "0DTG0ZZ",  # Resection of Left Large Intestine, Open Approach
        "0DTG4ZZ",  # Resection of Left Large Intestine, Percutaneous Endoscopic Approach
        "0DTG7ZZ",  # Resection of Left Large Intestine, Via Natural or Artificial Opening
        "0DTH0ZZ",  # Resection of Cecum, Open Approach
        "0DTH4ZZ",  # Resection of Cecum, Percutaneous Endoscopic Approach
        "0DTH7ZZ",  # Resection of Cecum, Via Natural or Artificial Opening
        "0DTK0ZZ",  # Resection of Ascending Colon, Open Approach
        "0DTK4ZZ",  # Resection of Ascending Colon, Percutaneous Endoscopic Approach
        "0DTK7ZZ",  # Resection of Ascending Colon, Via Natural or Artificial Opening
        "0DTL0ZZ",  # Resection of Transverse Colon, Open Approach
        "0DTL4ZZ",  # Resection of Transverse Colon, Percutaneous Endoscopic Approach
        "0DTL7ZZ",  # Resection of Transverse Colon, Via Natural or Artificial Opening
        "0DTM0ZZ",  # Resection of Descending Colon, Open Approach
        "0DTM4ZZ",  # Resection of Descending Colon, Percutaneous Endoscopic Approach
        "0DTM7ZZ",  # Resection of Descending Colon, Via Natural or Artificial Opening
        "0DTN0ZZ",  # Resection of Sigmoid Colon, Open Approach
        "0DTN4ZZ",  # Resection of Sigmoid Colon, Percutaneous Endoscopic Approach
        "0DTN7ZZ",  # Resection of Sigmoid Colon, Via Natural or Artificial Opening
        "0DTP0ZZ",  # Resection of Rectum, Open Approach
        "0DTP7ZZ",  # Resection of Rectum, Via Natural or Artificial Opening
        "0DU107Z",  # Supplement Upper Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU10JZ",  # Supplement Upper Esophagus with Synthetic Substitute, Open Approach
        "0DU10KZ",  # Supplement Upper Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU177Z",  # Supplement Upper Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU17JZ",  # Supplement Upper Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU17KZ",  # Supplement Upper Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU207Z",  # Supplement Middle Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU20JZ",  # Supplement Middle Esophagus with Synthetic Substitute, Open Approach
        "0DU20KZ",  # Supplement Middle Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU277Z",  # Supplement Middle Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU27JZ",  # Supplement Middle Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU27KZ",  # Supplement Middle Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU307Z",  # Supplement Lower Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU30JZ",  # Supplement Lower Esophagus with Synthetic Substitute, Open Approach
        "0DU30KZ",  # Supplement Lower Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU377Z",  # Supplement Lower Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU37JZ",  # Supplement Lower Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU37KZ",  # Supplement Lower Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU407Z",  # Supplement Esophagogastric Junction with Autologous Tissue Substitute, Open Approach
        "0DU40JZ",  # Supplement Esophagogastric Junction with Synthetic Substitute, Open Approach
        "0DU40KZ",  # Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Open Approach
        "0DU477Z",  # Supplement Esophagogastric Junction with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU47JZ",  # Supplement Esophagogastric Junction with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU47KZ",  # Supplement Esophagogastric Junction with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU507Z",  # Supplement Esophagus with Autologous Tissue Substitute, Open Approach
        "0DU50JZ",  # Supplement Esophagus with Synthetic Substitute, Open Approach
        "0DU50KZ",  # Supplement Esophagus with Nonautologous Tissue Substitute, Open Approach
        "0DU577Z",  # Supplement Esophagus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU57JZ",  # Supplement Esophagus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU57KZ",  # Supplement Esophagus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU607Z",  # Supplement Stomach with Autologous Tissue Substitute, Open Approach
        "0DU60JZ",  # Supplement Stomach with Synthetic Substitute, Open Approach
        "0DU60KZ",  # Supplement Stomach with Nonautologous Tissue Substitute, Open Approach
        "0DU677Z",  # Supplement Stomach with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU67JZ",  # Supplement Stomach with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU67KZ",  # Supplement Stomach with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU707Z",  # Supplement Stomach, Pylorus with Autologous Tissue Substitute, Open Approach
        "0DU70JZ",  # Supplement Stomach, Pylorus with Synthetic Substitute, Open Approach
        "0DU70KZ",  # Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Open Approach
        "0DU777Z",  # Supplement Stomach, Pylorus with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU77JZ",  # Supplement Stomach, Pylorus with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU77KZ",  # Supplement Stomach, Pylorus with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU807Z",  # Supplement Small Intestine with Autologous Tissue Substitute, Open Approach
        "0DU80JZ",  # Supplement Small Intestine with Synthetic Substitute, Open Approach
        "0DU80KZ",  # Supplement Small Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DU877Z",  # Supplement Small Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU87JZ",  # Supplement Small Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU87KZ",  # Supplement Small Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU907Z",  # Supplement Duodenum with Autologous Tissue Substitute, Open Approach
        "0DU90JZ",  # Supplement Duodenum with Synthetic Substitute, Open Approach
        "0DU90KZ",  # Supplement Duodenum with Nonautologous Tissue Substitute, Open Approach
        "0DU977Z",  # Supplement Duodenum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DU97JZ",  # Supplement Duodenum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DU97KZ",  # Supplement Duodenum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUA07Z",  # Supplement Jejunum with Autologous Tissue Substitute, Open Approach
        "0DUA0JZ",  # Supplement Jejunum with Synthetic Substitute, Open Approach
        "0DUA0KZ",  # Supplement Jejunum with Nonautologous Tissue Substitute, Open Approach
        "0DUA77Z",  # Supplement Jejunum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUA7JZ",  # Supplement Jejunum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUA7KZ",  # Supplement Jejunum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUB07Z",  # Supplement Ileum with Autologous Tissue Substitute, Open Approach
        "0DUB0JZ",  # Supplement Ileum with Synthetic Substitute, Open Approach
        "0DUB0KZ",  # Supplement Ileum with Nonautologous Tissue Substitute, Open Approach
        "0DUB77Z",  # Supplement Ileum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUB7JZ",  # Supplement Ileum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUB7KZ",  # Supplement Ileum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUC07Z",  # Supplement Ileocecal Valve with Autologous Tissue Substitute, Open Approach
        "0DUC0JZ",  # Supplement Ileocecal Valve with Synthetic Substitute, Open Approach
        "0DUC0KZ",  # Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Open Approach
        "0DUC77Z",  # Supplement Ileocecal Valve with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUC7JZ",  # Supplement Ileocecal Valve with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUC7KZ",  # Supplement Ileocecal Valve with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUE07Z",  # Supplement Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUE0JZ",  # Supplement Large Intestine with Synthetic Substitute, Open Approach
        "0DUE0KZ",  # Supplement Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUE77Z",  # Supplement Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUE7JZ",  # Supplement Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUE7KZ",  # Supplement Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUF07Z",  # Supplement Right Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUF0JZ",  # Supplement Right Large Intestine with Synthetic Substitute, Open Approach
        "0DUF0KZ",  # Supplement Right Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUF77Z",  # Supplement Right Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUF7JZ",  # Supplement Right Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUF7KZ",  # Supplement Right Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUG07Z",  # Supplement Left Large Intestine with Autologous Tissue Substitute, Open Approach
        "0DUG0JZ",  # Supplement Left Large Intestine with Synthetic Substitute, Open Approach
        "0DUG0KZ",  # Supplement Left Large Intestine with Nonautologous Tissue Substitute, Open Approach
        "0DUG77Z",  # Supplement Left Large Intestine with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUG7JZ",  # Supplement Left Large Intestine with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUG7KZ",  # Supplement Left Large Intestine with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUH07Z",  # Supplement Cecum with Autologous Tissue Substitute, Open Approach
        "0DUH0JZ",  # Supplement Cecum with Synthetic Substitute, Open Approach
        "0DUH0KZ",  # Supplement Cecum with Nonautologous Tissue Substitute, Open Approach
        "0DUH77Z",  # Supplement Cecum with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUH7JZ",  # Supplement Cecum with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUH7KZ",  # Supplement Cecum with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUK07Z",  # Supplement Ascending Colon with Autologous Tissue Substitute, Open Approach
        "0DUK0JZ",  # Supplement Ascending Colon with Synthetic Substitute, Open Approach
        "0DUK0KZ",  # Supplement Ascending Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUK77Z",  # Supplement Ascending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUK7JZ",  # Supplement Ascending Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUK7KZ",  # Supplement Ascending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUL07Z",  # Supplement Transverse Colon with Autologous Tissue Substitute, Open Approach
        "0DUL0JZ",  # Supplement Transverse Colon with Synthetic Substitute, Open Approach
        "0DUL0KZ",  # Supplement Transverse Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUL77Z",  # Supplement Transverse Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUL7JZ",  # Supplement Transverse Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUL7KZ",  # Supplement Transverse Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUM07Z",  # Supplement Descending Colon with Autologous Tissue Substitute, Open Approach
        "0DUM0JZ",  # Supplement Descending Colon with Synthetic Substitute, Open Approach
        "0DUM0KZ",  # Supplement Descending Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUM77Z",  # Supplement Descending Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUM7JZ",  # Supplement Descending Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUM7KZ",  # Supplement Descending Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUN07Z",  # Supplement Sigmoid Colon with Autologous Tissue Substitute, Open Approach
        "0DUN0JZ",  # Supplement Sigmoid Colon with Synthetic Substitute, Open Approach
        "0DUN0KZ",  # Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Open Approach
        "0DUN77Z",  # Supplement Sigmoid Colon with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUN7JZ",  # Supplement Sigmoid Colon with Synthetic Substitute, Via Natural or Artificial Opening
        "0DUN7KZ",  # Supplement Sigmoid Colon with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0DUV07Z",  # Supplement Mesentery with Autologous Tissue Substitute, Open Approach
        "0DUV0JZ",  # Supplement Mesentery with Synthetic Substitute, Open Approach
        "0DUV0KZ",  # Supplement Mesentery with Nonautologous Tissue Substitute, Open Approach
        "0DUW07Z",  # Supplement Peritoneum with Autologous Tissue Substitute, Open Approach
        "0DUW0JZ",  # Supplement Peritoneum with Synthetic Substitute, Open Approach
        "0DUW0KZ",  # Supplement Peritoneum with Nonautologous Tissue Substitute, Open Approach
        "0DV10CZ",  # Restriction of Upper Esophagus with Extraluminal Device, Open Approach
        "0DV10DZ",  # Restriction of Upper Esophagus with Intraluminal Device, Open Approach
        "0DV10ZZ",  # Restriction of Upper Esophagus, Open Approach
        "0DV17DZ",  # Restriction of Upper Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV17ZZ",  # Restriction of Upper Esophagus, Via Natural or Artificial Opening
        "0DV20CZ",  # Restriction of Middle Esophagus with Extraluminal Device, Open Approach
        "0DV20DZ",  # Restriction of Middle Esophagus with Intraluminal Device, Open Approach
        "0DV20ZZ",  # Restriction of Middle Esophagus, Open Approach
        "0DV27DZ",  # Restriction of Middle Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV27ZZ",  # Restriction of Middle Esophagus, Via Natural or Artificial Opening
        "0DV30CZ",  # Restriction of Lower Esophagus with Extraluminal Device, Open Approach
        "0DV30DZ",  # Restriction of Lower Esophagus with Intraluminal Device, Open Approach
        "0DV30ZZ",  # Restriction of Lower Esophagus, Open Approach
        "0DV37DZ",  # Restriction of Lower Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV37ZZ",  # Restriction of Lower Esophagus, Via Natural or Artificial Opening
        "0DV40CZ",  # Restriction of Esophagogastric Junction with Extraluminal Device, Open Approach
        "0DV40DZ",  # Restriction of Esophagogastric Junction with Intraluminal Device, Open Approach
        "0DV40ZZ",  # Restriction of Esophagogastric Junction, Open Approach
        "0DV47DZ",  # Restriction of Esophagogastric Junction with Intraluminal Device, Via Natural or Artificial Opening
        "0DV47ZZ",  # Restriction of Esophagogastric Junction, Via Natural or Artificial Opening
        "0DV50CZ",  # Restriction of Esophagus with Extraluminal Device, Open Approach
        "0DV50DZ",  # Restriction of Esophagus with Intraluminal Device, Open Approach
        "0DV50ZZ",  # Restriction of Esophagus, Open Approach
        "0DV57DZ",  # Restriction of Esophagus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV57ZZ",  # Restriction of Esophagus, Via Natural or Artificial Opening
        "0DV60CZ",  # Restriction of Stomach with Extraluminal Device, Open Approach
        "0DV60DZ",  # Restriction of Stomach with Intraluminal Device, Open Approach
        "0DV60ZZ",  # Restriction of Stomach, Open Approach
        "0DV67ZZ",  # Restriction of Stomach, Via Natural or Artificial Opening
        "0DV70CZ",  # Restriction of Stomach, Pylorus with Extraluminal Device, Open Approach
        "0DV70DZ",  # Restriction of Stomach, Pylorus with Intraluminal Device, Open Approach
        "0DV70ZZ",  # Restriction of Stomach, Pylorus, Open Approach
        "0DV77DZ",  # Restriction of Stomach, Pylorus with Intraluminal Device, Via Natural or Artificial Opening
        "0DV77ZZ",  # Restriction of Stomach, Pylorus, Via Natural or Artificial Opening
        "0DV80CZ",  # Restriction of Small Intestine with Extraluminal Device, Open Approach
        "0DV80DZ",  # Restriction of Small Intestine with Intraluminal Device, Open Approach
        "0DV80ZZ",  # Restriction of Small Intestine, Open Approach
        "0DV87DZ",  # Restriction of Small Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DV87ZZ",  # Restriction of Small Intestine, Via Natural or Artificial Opening
        "0DV90CZ",  # Restriction of Duodenum with Extraluminal Device, Open Approach
        "0DV90DZ",  # Restriction of Duodenum with Intraluminal Device, Open Approach
        "0DV90ZZ",  # Restriction of Duodenum, Open Approach
        "0DV97DZ",  # Restriction of Duodenum with Intraluminal Device, Via Natural or Artificial Opening
        "0DV97ZZ",  # Restriction of Duodenum, Via Natural or Artificial Opening
        "0DVA0CZ",  # Restriction of Jejunum with Extraluminal Device, Open Approach
        "0DVA0DZ",  # Restriction of Jejunum with Intraluminal Device, Open Approach
        "0DVA0ZZ",  # Restriction of Jejunum, Open Approach
        "0DVA7DZ",  # Restriction of Jejunum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVA7ZZ",  # Restriction of Jejunum, Via Natural or Artificial Opening
        "0DVB0CZ",  # Restriction of Ileum with Extraluminal Device, Open Approach
        "0DVB0DZ",  # Restriction of Ileum with Intraluminal Device, Open Approach
        "0DVB0ZZ",  # Restriction of Ileum, Open Approach
        "0DVB7DZ",  # Restriction of Ileum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVB7ZZ",  # Restriction of Ileum, Via Natural or Artificial Opening
        "0DVC0CZ",  # Restriction of Ileocecal Valve with Extraluminal Device, Open Approach
        "0DVC0DZ",  # Restriction of Ileocecal Valve with Intraluminal Device, Open Approach
        "0DVC0ZZ",  # Restriction of Ileocecal Valve, Open Approach
        "0DVC7DZ",  # Restriction of Ileocecal Valve with Intraluminal Device, Via Natural or Artificial Opening
        "0DVC7ZZ",  # Restriction of Ileocecal Valve, Via Natural or Artificial Opening
        "0DVE0CZ",  # Restriction of Large Intestine with Extraluminal Device, Open Approach
        "0DVE0DZ",  # Restriction of Large Intestine with Intraluminal Device, Open Approach
        "0DVE0ZZ",  # Restriction of Large Intestine, Open Approach
        "0DVE7DZ",  # Restriction of Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVE7ZZ",  # Restriction of Large Intestine, Via Natural or Artificial Opening
        "0DVF0CZ",  # Restriction of Right Large Intestine with Extraluminal Device, Open Approach
        "0DVF0DZ",  # Restriction of Right Large Intestine with Intraluminal Device, Open Approach
        "0DVF0ZZ",  # Restriction of Right Large Intestine, Open Approach
        "0DVF7DZ",  # Restriction of Right Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVF7ZZ",  # Restriction of Right Large Intestine, Via Natural or Artificial Opening
        "0DVG0CZ",  # Restriction of Left Large Intestine with Extraluminal Device, Open Approach
        "0DVG0DZ",  # Restriction of Left Large Intestine with Intraluminal Device, Open Approach
        "0DVG0ZZ",  # Restriction of Left Large Intestine, Open Approach
        "0DVG7DZ",  # Restriction of Left Large Intestine with Intraluminal Device, Via Natural or Artificial Opening
        "0DVG7ZZ",  # Restriction of Left Large Intestine, Via Natural or Artificial Opening
        "0DVH0CZ",  # Restriction of Cecum with Extraluminal Device, Open Approach
        "0DVH0DZ",  # Restriction of Cecum with Intraluminal Device, Open Approach
        "0DVH0ZZ",  # Restriction of Cecum, Open Approach
        "0DVH7DZ",  # Restriction of Cecum with Intraluminal Device, Via Natural or Artificial Opening
        "0DVH7ZZ",  # Restriction of Cecum, Via Natural or Artificial Opening
        "0DVK0CZ",  # Restriction of Ascending Colon with Extraluminal Device, Open Approach
        "0DVK0DZ",  # Restriction of Ascending Colon with Intraluminal Device, Open Approach
        "0DVK0ZZ",  # Restriction of Ascending Colon, Open Approach
        "0DVK7DZ",  # Restriction of Ascending Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVK7ZZ",  # Restriction of Ascending Colon, Via Natural or Artificial Opening
        "0DVL0CZ",  # Restriction of Transverse Colon with Extraluminal Device, Open Approach
        "0DVL0DZ",  # Restriction of Transverse Colon with Intraluminal Device, Open Approach
        "0DVL0ZZ",  # Restriction of Transverse Colon, Open Approach
        "0DVL7DZ",  # Restriction of Transverse Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVL7ZZ",  # Restriction of Transverse Colon, Via Natural or Artificial Opening
        "0DVM0CZ",  # Restriction of Descending Colon with Extraluminal Device, Open Approach
        "0DVM0DZ",  # Restriction of Descending Colon with Intraluminal Device, Open Approach
        "0DVM0ZZ",  # Restriction of Descending Colon, Open Approach
        "0DVM7DZ",  # Restriction of Descending Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVM7ZZ",  # Restriction of Descending Colon, Via Natural or Artificial Opening
        "0DVN0CZ",  # Restriction of Sigmoid Colon with Extraluminal Device, Open Approach
        "0DVN0DZ",  # Restriction of Sigmoid Colon with Intraluminal Device, Open Approach
        "0DVN0ZZ",  # Restriction of Sigmoid Colon, Open Approach
        "0DVN7DZ",  # Restriction of Sigmoid Colon with Intraluminal Device, Via Natural or Artificial Opening
        "0DVN7ZZ",  # Restriction of Sigmoid Colon, Via Natural or Artificial Opening
        "0DW000Z",  # Revision of Drainage Device in Upper Intestinal Tract, Open Approach
        "0DW002Z",  # Revision of Monitoring Device in Upper Intestinal Tract, Open Approach
        "0DW003Z",  # Revision of Infusion Device in Upper Intestinal Tract, Open Approach
        "0DW007Z",  # Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Open Approach
        "0DW00CZ",  # Revision of Extraluminal Device in Upper Intestinal Tract, Open Approach
        "0DW00DZ",  # Revision of Intraluminal Device in Upper Intestinal Tract, Open Approach
        "0DW00JZ",  # Revision of Synthetic Substitute in Upper Intestinal Tract, Open Approach
        "0DW00KZ",  # Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Open Approach
        "0DW00UZ",  # Revision of Feeding Device in Upper Intestinal Tract, Open Approach
        "0DW072Z",  # Revision of Monitoring Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW073Z",  # Revision of Infusion Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW077Z",  # Revision of Autologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07CZ",  # Revision of Extraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07DZ",  # Revision of Intraluminal Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07JZ",  # Revision of Synthetic Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07KZ",  # Revision of Nonautologous Tissue Substitute in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW07UZ",  # Revision of Feeding Device in Upper Intestinal Tract, Via Natural or Artificial Opening
        "0DW57DZ",  # Revision of Intraluminal Device in Esophagus, Via Natural or Artificial Opening
        "0DWD00Z",  # Revision of Drainage Device in Lower Intestinal Tract, Open Approach
        "0DWD02Z",  # Revision of Monitoring Device in Lower Intestinal Tract, Open Approach
        "0DWD03Z",  # Revision of Infusion Device in Lower Intestinal Tract, Open Approach
        "0DWD07Z",  # Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0CZ",  # Revision of Extraluminal Device in Lower Intestinal Tract, Open Approach
        "0DWD0DZ",  # Revision of Intraluminal Device in Lower Intestinal Tract, Open Approach
        "0DWD0JZ",  # Revision of Synthetic Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0KZ",  # Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Open Approach
        "0DWD0UZ",  # Revision of Feeding Device in Lower Intestinal Tract, Open Approach
        "0DWD72Z",  # Revision of Monitoring Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD73Z",  # Revision of Infusion Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD77Z",  # Revision of Autologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7CZ",  # Revision of Extraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7DZ",  # Revision of Intraluminal Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7JZ",  # Revision of Synthetic Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7KZ",  # Revision of Nonautologous Tissue Substitute in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWD7UZ",  # Revision of Feeding Device in Lower Intestinal Tract, Via Natural or Artificial Opening
        "0DWE07Z",  # Revision of Autologous Tissue Substitute in Large Intestine, Open Approach
        "0DWE0JZ",  # Revision of Synthetic Substitute in Large Intestine, Open Approach
        "0DWE0KZ",  # Revision of Nonautologous Tissue Substitute in Large Intestine, Open Approach
        "0DWE77Z",  # Revision of Autologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWE7JZ",  # Revision of Synthetic Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWE7KZ",  # Revision of Nonautologous Tissue Substitute in Large Intestine, Via Natural or Artificial Opening
        "0DWU07Z",  # Revision of Autologous Tissue Substitute in Omentum, Open Approach
        "0DWU0JZ",  # Revision of Synthetic Substitute in Omentum, Open Approach
        "0DWU0KZ",  # Revision of Nonautologous Tissue Substitute in Omentum, Open Approach
        "0DWV07Z",  # Revision of Autologous Tissue Substitute in Mesentery, Open Approach
        "0DWV0JZ",  # Revision of Synthetic Substitute in Mesentery, Open Approach
        "0DWV0KZ",  # Revision of Nonautologous Tissue Substitute in Mesentery, Open Approach
        "0DWW07Z",  # Revision of Autologous Tissue Substitute in Peritoneum, Open Approach
        "0DWW0JZ",  # Revision of Synthetic Substitute in Peritoneum, Open Approach
        "0DWW0KZ",  # Revision of Nonautologous Tissue Substitute in Peritoneum, Open Approach
        "0DX60Z5",  # Transfer Stomach to Esophagus, Open Approach
        "0DX80Z5",  # Transfer Small Intestine to Esophagus, Open Approach
        "0DXE0Z5",  # Transfer Large Intestine to Esophagus, Open Approach
        "0DY60Z0",  # Transplantation of Stomach, Allogeneic, Open Approach
        "0DY60Z1",  # Transplantation of Stomach, Syngeneic, Open Approach
        "0DY60Z2",  # Transplantation of Stomach, Zooplastic, Open Approach
        "0F140D3",  # Bypass Gallbladder to Duodenum with Intraluminal Device, Open Approach
        "0F140D4",  # Bypass Gallbladder to Stomach with Intraluminal Device, Open Approach
        "0F140D5",  # Bypass Gallbladder to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D6",  # Bypass Gallbladder to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D7",  # Bypass Gallbladder to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F140D8",  # Bypass Gallbladder to Cystic Duct with Intraluminal Device, Open Approach
        "0F140D9",  # Bypass Gallbladder to Common Bile Duct with Intraluminal Device, Open Approach
        "0F140DB",  # Bypass Gallbladder to Small Intestine with Intraluminal Device, Open Approach
        "0F140Z3",  # Bypass Gallbladder to Duodenum, Open Approach
        "0F140Z4",  # Bypass Gallbladder to Stomach, Open Approach
        "0F140Z5",  # Bypass Gallbladder to Right Hepatic Duct, Open Approach
        "0F140Z6",  # Bypass Gallbladder to Left Hepatic Duct, Open Approach
        "0F140Z7",  # Bypass Gallbladder to Caudate Hepatic Duct, Open Approach
        "0F140Z8",  # Bypass Gallbladder to Cystic Duct, Open Approach
        "0F140Z9",  # Bypass Gallbladder to Common Bile Duct, Open Approach
        "0F140ZB",  # Bypass Gallbladder to Small Intestine, Open Approach
        "0F150D3",  # Bypass Right Hepatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F150D4",  # Bypass Right Hepatic Duct to Stomach with Intraluminal Device, Open Approach
        "0F150D5",  # Bypass Right Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D6",  # Bypass Right Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D7",  # Bypass Right Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F150D8",  # Bypass Right Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F150D9",  # Bypass Right Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F150DB",  # Bypass Right Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F150Z3",  # Bypass Right Hepatic Duct to Duodenum, Open Approach
        "0F150Z4",  # Bypass Right Hepatic Duct to Stomach, Open Approach
        "0F150Z5",  # Bypass Right Hepatic Duct to Right Hepatic Duct, Open Approach
        "0F150Z6",  # Bypass Right Hepatic Duct to Left Hepatic Duct, Open Approach
        "0F150Z7",  # Bypass Right Hepatic Duct to Caudate Hepatic Duct, Open Approach
        "0F150Z8",  # Bypass Right Hepatic Duct to Cystic Duct, Open Approach
        "0F150Z9",  # Bypass Right Hepatic Duct to Common Bile Duct, Open Approach
        "0F150ZB",  # Bypass Right Hepatic Duct to Small Intestine, Open Approach
        "0F160D3",  # Bypass Left Hepatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F160D4",  # Bypass Left Hepatic Duct to Stomach with Intraluminal Device, Open Approach
        "0F160D5",  # Bypass Left Hepatic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D6",  # Bypass Left Hepatic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D7",  # Bypass Left Hepatic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F160D8",  # Bypass Left Hepatic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F160D9",  # Bypass Left Hepatic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F160DB",  # Bypass Left Hepatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F160Z3",  # Bypass Left Hepatic Duct to Duodenum, Open Approach
        "0F160Z4",  # Bypass Left Hepatic Duct to Stomach, Open Approach
        "0F160Z5",  # Bypass Left Hepatic Duct to Right Hepatic Duct, Open Approach
        "0F160Z6",  # Bypass Left Hepatic Duct to Left Hepatic Duct, Open Approach
        "0F160Z7",  # Bypass Left Hepatic Duct to Caudate Hepatic Duct, Open Approach
        "0F160Z8",  # Bypass Left Hepatic Duct to Cystic Duct, Open Approach
        "0F160Z9",  # Bypass Left Hepatic Duct to Common Bile Duct, Open Approach
        "0F160ZB",  # Bypass Left Hepatic Duct to Small Intestine, Open Approach
        "0F180D3",  # Bypass Cystic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F180D4",  # Bypass Cystic Duct to Stomach with Intraluminal Device, Open Approach
        "0F180D5",  # Bypass Cystic Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D6",  # Bypass Cystic Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D7",  # Bypass Cystic Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F180D8",  # Bypass Cystic Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F180D9",  # Bypass Cystic Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F180DB",  # Bypass Cystic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F180Z3",  # Bypass Cystic Duct to Duodenum, Open Approach
        "0F180Z4",  # Bypass Cystic Duct to Stomach, Open Approach
        "0F180Z5",  # Bypass Cystic Duct to Right Hepatic Duct, Open Approach
        "0F180Z6",  # Bypass Cystic Duct to Left Hepatic Duct, Open Approach
        "0F180Z7",  # Bypass Cystic Duct to Caudate Hepatic Duct, Open Approach
        "0F180Z8",  # Bypass Cystic Duct to Cystic Duct, Open Approach
        "0F180Z9",  # Bypass Cystic Duct to Common Bile Duct, Open Approach
        "0F180ZB",  # Bypass Cystic Duct to Small Intestine, Open Approach
        "0F190D3",  # Bypass Common Bile Duct to Duodenum with Intraluminal Device, Open Approach
        "0F190D4",  # Bypass Common Bile Duct to Stomach with Intraluminal Device, Open Approach
        "0F190D5",  # Bypass Common Bile Duct to Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D6",  # Bypass Common Bile Duct to Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D7",  # Bypass Common Bile Duct to Caudate Hepatic Duct with Intraluminal Device, Open Approach
        "0F190D8",  # Bypass Common Bile Duct to Cystic Duct with Intraluminal Device, Open Approach
        "0F190D9",  # Bypass Common Bile Duct to Common Bile Duct with Intraluminal Device, Open Approach
        "0F190DB",  # Bypass Common Bile Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F190Z3",  # Bypass Common Bile Duct to Duodenum, Open Approach
        "0F190Z4",  # Bypass Common Bile Duct to Stomach, Open Approach
        "0F190Z5",  # Bypass Common Bile Duct to Right Hepatic Duct, Open Approach
        "0F190Z6",  # Bypass Common Bile Duct to Left Hepatic Duct, Open Approach
        "0F190Z7",  # Bypass Common Bile Duct to Caudate Hepatic Duct, Open Approach
        "0F190Z8",  # Bypass Common Bile Duct to Cystic Duct, Open Approach
        "0F190Z9",  # Bypass Common Bile Duct to Common Bile Duct, Open Approach
        "0F190ZB",  # Bypass Common Bile Duct to Small Intestine, Open Approach
        "0F1D0D3",  # Bypass Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F1D0DB",  # Bypass Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F1D0DC",  # Bypass Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach
        "0F1D0Z3",  # Bypass Pancreatic Duct to Duodenum, Open Approach
        "0F1D0ZB",  # Bypass Pancreatic Duct to Small Intestine, Open Approach
        "0F1D0ZC",  # Bypass Pancreatic Duct to Large Intestine, Open Approach
        "0F1F0D3",  # Bypass Accessory Pancreatic Duct to Duodenum with Intraluminal Device, Open Approach
        "0F1F0DB",  # Bypass Accessory Pancreatic Duct to Small Intestine with Intraluminal Device, Open Approach
        "0F1F0DC",  # Bypass Accessory Pancreatic Duct to Large Intestine with Intraluminal Device, Open Approach
        "0F1F0Z3",  # Bypass Accessory Pancreatic Duct to Duodenum, Open Approach
        "0F1F0ZB",  # Bypass Accessory Pancreatic Duct to Small Intestine, Open Approach
        "0F1F0ZC",  # Bypass Accessory Pancreatic Duct to Large Intestine, Open Approach
        "0F1G0D3",  # Bypass Pancreas to Duodenum with Intraluminal Device, Open Approach
        "0F1G0DB",  # Bypass Pancreas to Small Intestine with Intraluminal Device, Open Approach
        "0F1G0DC",  # Bypass Pancreas to Large Intestine with Intraluminal Device, Open Approach
        "0F1G0Z3",  # Bypass Pancreas to Duodenum, Open Approach
        "0F1G0ZB",  # Bypass Pancreas to Small Intestine, Open Approach
        "0F1G0ZC",  # Bypass Pancreas to Large Intestine, Open Approach
        "0F500ZZ",  # Destruction of Liver, Open Approach
        "0F510ZZ",  # Destruction of Right Lobe Liver, Open Approach
        "0F520ZZ",  # Destruction of Left Lobe Liver, Open Approach
        "0F540ZZ",  # Destruction of Gallbladder, Open Approach
        "0F550ZZ",  # Destruction of Right Hepatic Duct, Open Approach
        "0F557ZZ",  # Destruction of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F560ZZ",  # Destruction of Left Hepatic Duct, Open Approach
        "0F567ZZ",  # Destruction of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F580ZZ",  # Destruction of Cystic Duct, Open Approach
        "0F587ZZ",  # Destruction of Cystic Duct, Via Natural or Artificial Opening
        "0F590ZZ",  # Destruction of Common Bile Duct, Open Approach
        "0F597ZZ",  # Destruction of Common Bile Duct, Via Natural or Artificial Opening
        "0F5C0ZZ",  # Destruction of Ampulla of Vater, Open Approach
        "0F5C7ZZ",  # Destruction of Ampulla of Vater, Via Natural or Artificial Opening
        "0F5D0ZZ",  # Destruction of Pancreatic Duct, Open Approach
        "0F5D7ZZ",  # Destruction of Pancreatic Duct, Via Natural or Artificial Opening
        "0F5F0ZZ",  # Destruction of Accessory Pancreatic Duct, Open Approach
        "0F5F7ZZ",  # Destruction of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F5G0ZZ",  # Destruction of Pancreas, Open Approach
        "0F750DZ",  # Dilation of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0F750ZZ",  # Dilation of Right Hepatic Duct, Open Approach
        "0F757DZ",  # Dilation of Right Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F757ZZ",  # Dilation of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F760DZ",  # Dilation of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0F760ZZ",  # Dilation of Left Hepatic Duct, Open Approach
        "0F767DZ",  # Dilation of Left Hepatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F767ZZ",  # Dilation of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F780DZ",  # Dilation of Cystic Duct with Intraluminal Device, Open Approach
        "0F780ZZ",  # Dilation of Cystic Duct, Open Approach
        "0F787DZ",  # Dilation of Cystic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F787ZZ",  # Dilation of Cystic Duct, Via Natural or Artificial Opening
        "0F790DZ",  # Dilation of Common Bile Duct with Intraluminal Device, Open Approach
        "0F790ZZ",  # Dilation of Common Bile Duct, Open Approach
        "0F797DZ",  # Dilation of Common Bile Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F797ZZ",  # Dilation of Common Bile Duct, Via Natural or Artificial Opening
        "0F7C0DZ",  # Dilation of Ampulla of Vater with Intraluminal Device, Open Approach
        "0F7C0ZZ",  # Dilation of Ampulla of Vater, Open Approach
        "0F7C7DZ",  # Dilation of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0F7C7ZZ",  # Dilation of Ampulla of Vater, Via Natural or Artificial Opening
        "0F7D0DZ",  # Dilation of Pancreatic Duct with Intraluminal Device, Open Approach
        "0F7D0ZZ",  # Dilation of Pancreatic Duct, Open Approach
        "0F7D7DZ",  # Dilation of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F7D7ZZ",  # Dilation of Pancreatic Duct, Via Natural or Artificial Opening
        "0F7F0DZ",  # Dilation of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0F7F0ZZ",  # Dilation of Accessory Pancreatic Duct, Open Approach
        "0F7F7DZ",  # Dilation of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0F7F7ZZ",  # Dilation of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F8G0ZZ",  # Division of Pancreas, Open Approach
        "0F9000Z",  # Drainage of Liver with Drainage Device, Open Approach
        "0F900ZZ",  # Drainage of Liver, Open Approach
        "0F9100Z",  # Drainage of Right Lobe Liver with Drainage Device, Open Approach
        "0F910ZZ",  # Drainage of Right Lobe Liver, Open Approach
        "0F9200Z",  # Drainage of Left Lobe Liver with Drainage Device, Open Approach
        "0F920ZZ",  # Drainage of Left Lobe Liver, Open Approach
        "0F9400Z",  # Drainage of Gallbladder with Drainage Device, Open Approach
        "0F940ZX",  # Drainage of Gallbladder, Open Approach, Diagnostic
        "0F940ZZ",  # Drainage of Gallbladder, Open Approach
        "0F9500Z",  # Drainage of Right Hepatic Duct with Drainage Device, Open Approach
        "0F950ZZ",  # Drainage of Right Hepatic Duct, Open Approach
        "0F9570Z",  # Drainage of Right Hepatic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F957ZZ",  # Drainage of Right Hepatic Duct, Via Natural or Artificial Opening
        "0F9600Z",  # Drainage of Left Hepatic Duct with Drainage Device, Open Approach
        "0F960ZZ",  # Drainage of Left Hepatic Duct, Open Approach
        "0F9670Z",  # Drainage of Left Hepatic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F967ZZ",  # Drainage of Left Hepatic Duct, Via Natural or Artificial Opening
        "0F9800Z",  # Drainage of Cystic Duct with Drainage Device, Open Approach
        "0F980ZZ",  # Drainage of Cystic Duct, Open Approach
        "0F9870Z",  # Drainage of Cystic Duct with Drainage Device, Via Natural or Artificial Opening
        "0F987ZZ",  # Drainage of Cystic Duct, Via Natural or Artificial Opening
        "0F9900Z",  # Drainage of Common Bile Duct with Drainage Device, Open Approach
        "0F990ZZ",  # Drainage of Common Bile Duct, Open Approach
        "0F9C00Z",  # Drainage of Ampulla of Vater with Drainage Device, Open Approach
        "0F9C0ZZ",  # Drainage of Ampulla of Vater, Open Approach
        "0F9C7ZZ",  # Drainage of Ampulla of Vater, Via Natural or Artificial Opening
        "0F9D0ZZ",  # Drainage of Pancreatic Duct, Open Approach
        "0F9D7ZZ",  # Drainage of Pancreatic Duct, Via Natural or Artificial Opening
        "0F9F0ZZ",  # Drainage of Accessory Pancreatic Duct, Open Approach
        "0F9F7ZZ",  # Drainage of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0F9G0ZZ",  # Drainage of Pancreas, Open Approach
        "0FB00ZZ",  # Excision of Liver, Open Approach
        "0FB10ZZ",  # Excision of Right Lobe Liver, Open Approach
        "0FB20ZZ",  # Excision of Left Lobe Liver, Open Approach
        "0FB40ZX",  # Excision of Gallbladder, Open Approach, Diagnostic
        "0FB40ZZ",  # Excision of Gallbladder, Open Approach
        "0FB50ZZ",  # Excision of Right Hepatic Duct, Open Approach
        "0FB57ZZ",  # Excision of Right Hepatic Duct, Via Natural or Artificial Opening
        "0FB60ZZ",  # Excision of Left Hepatic Duct, Open Approach
        "0FB67ZZ",  # Excision of Left Hepatic Duct, Via Natural or Artificial Opening
        "0FB80ZZ",  # Excision of Cystic Duct, Open Approach
        "0FB87ZZ",  # Excision of Cystic Duct, Via Natural or Artificial Opening
        "0FB90ZZ",  # Excision of Common Bile Duct, Open Approach
        "0FB97ZZ",  # Excision of Common Bile Duct, Via Natural or Artificial Opening
        "0FBC0ZZ",  # Excision of Ampulla of Vater, Open Approach
        "0FBC7ZZ",  # Excision of Ampulla of Vater, Via Natural or Artificial Opening
        "0FBD0ZZ",  # Excision of Pancreatic Duct, Open Approach
        "0FBD7ZZ",  # Excision of Pancreatic Duct, Via Natural or Artificial Opening
        "0FBF0ZZ",  # Excision of Accessory Pancreatic Duct, Open Approach
        "0FBF7ZZ",  # Excision of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FBG0ZZ",  # Excision of Pancreas, Open Approach
        "0FC00ZZ",  # Extirpation of Matter from Liver, Open Approach
        "0FC10ZZ",  # Extirpation of Matter from Right Lobe Liver, Open Approach
        "0FC20ZZ",  # Extirpation of Matter from Left Lobe Liver, Open Approach
        "0FC40ZZ",  # Extirpation of Matter from Gallbladder, Open Approach
        "0FC50ZZ",  # Extirpation of Matter from Right Hepatic Duct, Open Approach
        "0FC60ZZ",  # Extirpation of Matter from Left Hepatic Duct, Open Approach
        "0FC80ZZ",  # Extirpation of Matter from Cystic Duct, Open Approach
        "0FC90ZZ",  # Extirpation of Matter from Common Bile Duct, Open Approach
        "0FCC0ZZ",  # Extirpation of Matter from Ampulla of Vater, Open Approach
        "0FCC7ZZ",  # Extirpation of Matter from Ampulla of Vater, Via Natural or Artificial Opening
        "0FCD0ZZ",  # Extirpation of Matter from Pancreatic Duct, Open Approach
        "0FCD7ZZ",  # Extirpation of Matter from Pancreatic Duct, Via Natural or Artificial Opening
        "0FCF0ZZ",  # Extirpation of Matter from Accessory Pancreatic Duct, Open Approach
        "0FCF7ZZ",  # Extirpation of Matter from Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FCG0ZZ",  # Extirpation of Matter from Pancreas, Open Approach
        "0FF40ZZ",  # Fragmentation in Gallbladder, Open Approach
        "0FF50ZZ",  # Fragmentation in Right Hepatic Duct, Open Approach
        "0FF57ZZ",  # Fragmentation in Right Hepatic Duct, Via Natural or Artificial Opening
        "0FF60ZZ",  # Fragmentation in Left Hepatic Duct, Open Approach
        "0FF67ZZ",  # Fragmentation in Left Hepatic Duct, Via Natural or Artificial Opening
        "0FF80ZZ",  # Fragmentation in Cystic Duct, Open Approach
        "0FF87ZZ",  # Fragmentation in Cystic Duct, Via Natural or Artificial Opening
        "0FF90ZZ",  # Fragmentation in Common Bile Duct, Open Approach
        "0FF97ZZ",  # Fragmentation in Common Bile Duct, Via Natural or Artificial Opening
        "0FFC0ZZ",  # Fragmentation in Ampulla of Vater, Open Approach
        "0FFC7ZZ",  # Fragmentation in Ampulla of Vater, Via Natural or Artificial Opening
        "0FFD0ZZ",  # Fragmentation in Pancreatic Duct, Open Approach
        "0FFD7ZZ",  # Fragmentation in Pancreatic Duct, Via Natural or Artificial Opening
        "0FFF0ZZ",  # Fragmentation in Accessory Pancreatic Duct, Open Approach
        "0FFF7ZZ",  # Fragmentation in Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FH002Z",  # Insertion of Monitoring Device into Liver, Open Approach
        "0FH102Z",  # Insertion of Monitoring Device into Right Lobe Liver, Open Approach
        "0FH202Z",  # Insertion of Monitoring Device into Left Lobe Liver, Open Approach
        "0FH402Z",  # Insertion of Monitoring Device into Gallbladder, Open Approach
        "0FH403Z",  # Insertion of Infusion Device into Gallbladder, Open Approach
        "0FH40YZ",  # Insertion of Other Device into Gallbladder, Open Approach
        "0FHB02Z",  # Insertion of Monitoring Device into Hepatobiliary Duct, Open Approach
        "0FHB0DZ",  # Insertion of Intraluminal Device into Hepatobiliary Duct, Open Approach
        "0FHB72Z",  # Insertion of Monitoring Device into Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FHB7DZ",  # Insertion of Intraluminal Device into Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FHD02Z",  # Insertion of Monitoring Device into Pancreatic Duct, Open Approach
        "0FHD0DZ",  # Insertion of Intraluminal Device into Pancreatic Duct, Open Approach
        "0FHD72Z",  # Insertion of Monitoring Device into Pancreatic Duct, Via Natural or Artificial Opening
        "0FHD7DZ",  # Insertion of Intraluminal Device into Pancreatic Duct, Via Natural or Artificial Opening
        "0FHG02Z",  # Insertion of Monitoring Device into Pancreas, Open Approach
        "0FJ00ZZ",  # Inspection of Liver, Open Approach
        "0FJ40ZZ",  # Inspection of Gallbladder, Open Approach
        "0FJB0ZZ",  # Inspection of Hepatobiliary Duct, Open Approach
        "0FJB7ZZ",  # Inspection of Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FJD0ZZ",  # Inspection of Pancreatic Duct, Open Approach
        "0FJG0ZZ",  # Inspection of Pancreas, Open Approach
        "0FL50CZ",  # Occlusion of Right Hepatic Duct with Extraluminal Device, Open Approach
        "0FL50DZ",  # Occlusion of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0FL50ZZ",  # Occlusion of Right Hepatic Duct, Open Approach
        "0FL60CZ",  # Occlusion of Left Hepatic Duct with Extraluminal Device, Open Approach
        "0FL60DZ",  # Occlusion of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0FL60ZZ",  # Occlusion of Left Hepatic Duct, Open Approach
        "0FL80CZ",  # Occlusion of Cystic Duct with Extraluminal Device, Open Approach
        "0FL80DZ",  # Occlusion of Cystic Duct with Intraluminal Device, Open Approach
        "0FL80ZZ",  # Occlusion of Cystic Duct, Open Approach
        "0FL90CZ",  # Occlusion of Common Bile Duct with Extraluminal Device, Open Approach
        "0FL90DZ",  # Occlusion of Common Bile Duct with Intraluminal Device, Open Approach
        "0FL90ZZ",  # Occlusion of Common Bile Duct, Open Approach
        "0FLC0CZ",  # Occlusion of Ampulla of Vater with Extraluminal Device, Open Approach
        "0FLC0DZ",  # Occlusion of Ampulla of Vater with Intraluminal Device, Open Approach
        "0FLC0ZZ",  # Occlusion of Ampulla of Vater, Open Approach
        "0FLC7DZ",  # Occlusion of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0FLC7ZZ",  # Occlusion of Ampulla of Vater, Via Natural or Artificial Opening
        "0FLD0CZ",  # Occlusion of Pancreatic Duct with Extraluminal Device, Open Approach
        "0FLD0DZ",  # Occlusion of Pancreatic Duct with Intraluminal Device, Open Approach
        "0FLD0ZZ",  # Occlusion of Pancreatic Duct, Open Approach
        "0FLD7DZ",  # Occlusion of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FLD7ZZ",  # Occlusion of Pancreatic Duct, Via Natural or Artificial Opening
        "0FLF0CZ",  # Occlusion of Accessory Pancreatic Duct with Extraluminal Device, Open Approach
        "0FLF0DZ",  # Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0FLF0ZZ",  # Occlusion of Accessory Pancreatic Duct, Open Approach
        "0FLF7DZ",  # Occlusion of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FLF7ZZ",  # Occlusion of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FM40ZZ",  # Reattachment of Gallbladder, Open Approach
        "0FM50ZZ",  # Reattachment of Right Hepatic Duct, Open Approach
        "0FM60ZZ",  # Reattachment of Left Hepatic Duct, Open Approach
        "0FM80ZZ",  # Reattachment of Cystic Duct, Open Approach
        "0FM90ZZ",  # Reattachment of Common Bile Duct, Open Approach
        "0FMC0ZZ",  # Reattachment of Ampulla of Vater, Open Approach
        "0FMD0ZZ",  # Reattachment of Pancreatic Duct, Open Approach
        "0FMF0ZZ",  # Reattachment of Accessory Pancreatic Duct, Open Approach
        "0FMG0ZZ",  # Reattachment of Pancreas, Open Approach
        "0FN00ZZ",  # Release Liver, Open Approach
        "0FN10ZZ",  # Release Right Lobe Liver, Open Approach
        "0FN20ZZ",  # Release Left Lobe Liver, Open Approach
        "0FN40ZZ",  # Release Gallbladder, Open Approach
        "0FN50ZZ",  # Release Right Hepatic Duct, Open Approach
        "0FN57ZZ",  # Release Right Hepatic Duct, Via Natural or Artificial Opening
        "0FN60ZZ",  # Release Left Hepatic Duct, Open Approach
        "0FN67ZZ",  # Release Left Hepatic Duct, Via Natural or Artificial Opening
        "0FN80ZZ",  # Release Cystic Duct, Open Approach
        "0FN87ZZ",  # Release Cystic Duct, Via Natural or Artificial Opening
        "0FN90ZZ",  # Release Common Bile Duct, Open Approach
        "0FN97ZZ",  # Release Common Bile Duct, Via Natural or Artificial Opening
        "0FNC0ZZ",  # Release Ampulla of Vater, Open Approach
        "0FNC7ZZ",  # Release Ampulla of Vater, Via Natural or Artificial Opening
        "0FND0ZZ",  # Release Pancreatic Duct, Open Approach
        "0FND7ZZ",  # Release Pancreatic Duct, Via Natural or Artificial Opening
        "0FNF0ZZ",  # Release Accessory Pancreatic Duct, Open Approach
        "0FNF7ZZ",  # Release Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FNG0ZZ",  # Release Pancreas, Open Approach
        "0FP000Z",  # Removal of Drainage Device from Liver, Open Approach
        "0FP002Z",  # Removal of Monitoring Device from Liver, Open Approach
        "0FP003Z",  # Removal of Infusion Device from Liver, Open Approach
        "0FP400Z",  # Removal of Drainage Device from Gallbladder, Open Approach
        "0FP402Z",  # Removal of Monitoring Device from Gallbladder, Open Approach
        "0FP403Z",  # Removal of Infusion Device from Gallbladder, Open Approach
        "0FP40DZ",  # Removal of Intraluminal Device from Gallbladder, Open Approach
        "0FP40YZ",  # Removal of Other Device from Gallbladder, Open Approach
        "0FP480Z",  # Removal of Drainage Device from Gallbladder, Via Natural or Artificial Opening Endoscopic
        "0FPB00Z",  # Removal of Drainage Device from Hepatobiliary Duct, Open Approach
        "0FPB01Z",  # Removal of Radioactive Element from Hepatobiliary Duct, Open Approach
        "0FPB02Z",  # Removal of Monitoring Device from Hepatobiliary Duct, Open Approach
        "0FPB03Z",  # Removal of Infusion Device from Hepatobiliary Duct, Open Approach
        "0FPB07Z",  # Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Open Approach
        "0FPB0CZ",  # Removal of Extraluminal Device from Hepatobiliary Duct, Open Approach
        "0FPB0DZ",  # Removal of Intraluminal Device from Hepatobiliary Duct, Open Approach
        "0FPB0JZ",  # Removal of Synthetic Substitute from Hepatobiliary Duct, Open Approach
        "0FPB0KZ",  # Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Open Approach
        "0FPB70Z",  # Removal of Drainage Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB71Z",  # Removal of Radioactive Element from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB72Z",  # Removal of Monitoring Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB73Z",  # Removal of Infusion Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB77Z",  # Removal of Autologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7CZ",  # Removal of Extraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7DZ",  # Removal of Intraluminal Device from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7JZ",  # Removal of Synthetic Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPB7KZ",  # Removal of Nonautologous Tissue Substitute from Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FPD00Z",  # Removal of Drainage Device from Pancreatic Duct, Open Approach
        "0FPD01Z",  # Removal of Radioactive Element from Pancreatic Duct, Open Approach
        "0FPD02Z",  # Removal of Monitoring Device from Pancreatic Duct, Open Approach
        "0FPD03Z",  # Removal of Infusion Device from Pancreatic Duct, Open Approach
        "0FPD07Z",  # Removal of Autologous Tissue Substitute from Pancreatic Duct, Open Approach
        "0FPD0CZ",  # Removal of Extraluminal Device from Pancreatic Duct, Open Approach
        "0FPD0DZ",  # Removal of Intraluminal Device from Pancreatic Duct, Open Approach
        "0FPD0JZ",  # Removal of Synthetic Substitute from Pancreatic Duct, Open Approach
        "0FPD0KZ",  # Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Open Approach
        "0FPD70Z",  # Removal of Drainage Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD71Z",  # Removal of Radioactive Element from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD72Z",  # Removal of Monitoring Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD73Z",  # Removal of Infusion Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD77Z",  # Removal of Autologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7CZ",  # Removal of Extraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7DZ",  # Removal of Intraluminal Device from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7JZ",  # Removal of Synthetic Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPD7KZ",  # Removal of Nonautologous Tissue Substitute from Pancreatic Duct, Via Natural or Artificial Opening
        "0FPG00Z",  # Removal of Drainage Device from Pancreas, Open Approach
        "0FPG02Z",  # Removal of Monitoring Device from Pancreas, Open Approach
        "0FPG03Z",  # Removal of Infusion Device from Pancreas, Open Approach
        "0FPG0DZ",  # Removal of Intraluminal Device from Pancreas, Open Approach
        "0FPG80Z",  # Removal of Drainage Device from Pancreas, Via Natural or Artificial Opening Endoscopic
        "0FPGXDZ",  # Removal of Intraluminal Device from Pancreas, External Approach
        "0FQ40ZZ",  # Repair Gallbladder, Open Approach
        "0FQ50ZZ",  # Repair Right Hepatic Duct, Open Approach
        "0FQ57ZZ",  # Repair Right Hepatic Duct, Via Natural or Artificial Opening
        "0FQ60ZZ",  # Repair Left Hepatic Duct, Open Approach
        "0FQ67ZZ",  # Repair Left Hepatic Duct, Via Natural or Artificial Opening
        "0FQ80ZZ",  # Repair Cystic Duct, Open Approach
        "0FQ87ZZ",  # Repair Cystic Duct, Via Natural or Artificial Opening
        "0FQ90ZZ",  # Repair Common Bile Duct, Open Approach
        "0FQ97ZZ",  # Repair Common Bile Duct, Via Natural or Artificial Opening
        "0FQC0ZZ",  # Repair Ampulla of Vater, Open Approach
        "0FQC7ZZ",  # Repair Ampulla of Vater, Via Natural or Artificial Opening
        "0FQD0ZZ",  # Repair Pancreatic Duct, Open Approach
        "0FQD7ZZ",  # Repair Pancreatic Duct, Via Natural or Artificial Opening
        "0FQF0ZZ",  # Repair Accessory Pancreatic Duct, Open Approach
        "0FQF7ZZ",  # Repair Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FQG0ZZ",  # Repair Pancreas, Open Approach
        "0FR507Z",  # Replacement of Right Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FR50JZ",  # Replacement of Right Hepatic Duct with Synthetic Substitute, Open Approach
        "0FR50KZ",  # Replacement of Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR607Z",  # Replacement of Left Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FR60JZ",  # Replacement of Left Hepatic Duct with Synthetic Substitute, Open Approach
        "0FR60KZ",  # Replacement of Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR807Z",  # Replacement of Cystic Duct with Autologous Tissue Substitute, Open Approach
        "0FR80JZ",  # Replacement of Cystic Duct with Synthetic Substitute, Open Approach
        "0FR80KZ",  # Replacement of Cystic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FR907Z",  # Replacement of Common Bile Duct with Autologous Tissue Substitute, Open Approach
        "0FR90JZ",  # Replacement of Common Bile Duct with Synthetic Substitute, Open Approach
        "0FR90KZ",  # Replacement of Common Bile Duct with Nonautologous Tissue Substitute, Open Approach
        "0FRC07Z",  # Replacement of Ampulla of Vater with Autologous Tissue Substitute, Open Approach
        "0FRC0JZ",  # Replacement of Ampulla of Vater with Synthetic Substitute, Open Approach
        "0FRC0KZ",  # Replacement of Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach
        "0FRD07Z",  # Replacement of Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FRD0JZ",  # Replacement of Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FRD0KZ",  # Replacement of Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FRF07Z",  # Replacement of Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FRF0JZ",  # Replacement of Accessory Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FRF0KZ",  # Replacement of Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FS40ZZ",  # Reposition Gallbladder, Open Approach
        "0FS50ZZ",  # Reposition Right Hepatic Duct, Open Approach
        "0FS60ZZ",  # Reposition Left Hepatic Duct, Open Approach
        "0FS80ZZ",  # Reposition Cystic Duct, Open Approach
        "0FS90ZZ",  # Reposition Common Bile Duct, Open Approach
        "0FSC0ZZ",  # Reposition Ampulla of Vater, Open Approach
        "0FSD0ZZ",  # Reposition Pancreatic Duct, Open Approach
        "0FSF0ZZ",  # Reposition Accessory Pancreatic Duct, Open Approach
        "0FT00ZZ",  # Resection of Liver, Open Approach
        "0FT10ZZ",  # Resection of Right Lobe Liver, Open Approach
        "0FT20ZZ",  # Resection of Left Lobe Liver, Open Approach
        "0FT40ZZ",  # Resection of Gallbladder, Open Approach
        "0FT50ZZ",  # Resection of Right Hepatic Duct, Open Approach
        "0FT57ZZ",  # Resection of Right Hepatic Duct, Via Natural or Artificial Opening
        "0FT60ZZ",  # Resection of Left Hepatic Duct, Open Approach
        "0FT67ZZ",  # Resection of Left Hepatic Duct, Via Natural or Artificial Opening
        "0FT80ZZ",  # Resection of Cystic Duct, Open Approach
        "0FT87ZZ",  # Resection of Cystic Duct, Via Natural or Artificial Opening
        "0FT90ZZ",  # Resection of Common Bile Duct, Open Approach
        "0FT97ZZ",  # Resection of Common Bile Duct, Via Natural or Artificial Opening
        "0FTC0ZZ",  # Resection of Ampulla of Vater, Open Approach
        "0FTC7ZZ",  # Resection of Ampulla of Vater, Via Natural or Artificial Opening
        "0FTD0ZZ",  # Resection of Pancreatic Duct, Open Approach
        "0FTD7ZZ",  # Resection of Pancreatic Duct, Via Natural or Artificial Opening
        "0FTF0ZZ",  # Resection of Accessory Pancreatic Duct, Open Approach
        "0FTF7ZZ",  # Resection of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FTG0ZZ",  # Resection of Pancreas, Open Approach
        "0FU507Z",  # Supplement Right Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FU50JZ",  # Supplement Right Hepatic Duct with Synthetic Substitute, Open Approach
        "0FU50KZ",  # Supplement Right Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU607Z",  # Supplement Left Hepatic Duct with Autologous Tissue Substitute, Open Approach
        "0FU60JZ",  # Supplement Left Hepatic Duct with Synthetic Substitute, Open Approach
        "0FU60KZ",  # Supplement Left Hepatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU807Z",  # Supplement Cystic Duct with Autologous Tissue Substitute, Open Approach
        "0FU80JZ",  # Supplement Cystic Duct with Synthetic Substitute, Open Approach
        "0FU80KZ",  # Supplement Cystic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FU907Z",  # Supplement Common Bile Duct with Autologous Tissue Substitute, Open Approach
        "0FU90JZ",  # Supplement Common Bile Duct with Synthetic Substitute, Open Approach
        "0FU90KZ",  # Supplement Common Bile Duct with Nonautologous Tissue Substitute, Open Approach
        "0FUC07Z",  # Supplement Ampulla of Vater with Autologous Tissue Substitute, Open Approach
        "0FUC0JZ",  # Supplement Ampulla of Vater with Synthetic Substitute, Open Approach
        "0FUC0KZ",  # Supplement Ampulla of Vater with Nonautologous Tissue Substitute, Open Approach
        "0FUD07Z",  # Supplement Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FUD0JZ",  # Supplement Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FUD0KZ",  # Supplement Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FUF07Z",  # Supplement Accessory Pancreatic Duct with Autologous Tissue Substitute, Open Approach
        "0FUF0JZ",  # Supplement Accessory Pancreatic Duct with Synthetic Substitute, Open Approach
        "0FUF0KZ",  # Supplement Accessory Pancreatic Duct with Nonautologous Tissue Substitute, Open Approach
        "0FV50CZ",  # Restriction of Right Hepatic Duct with Extraluminal Device, Open Approach
        "0FV50DZ",  # Restriction of Right Hepatic Duct with Intraluminal Device, Open Approach
        "0FV50ZZ",  # Restriction of Right Hepatic Duct, Open Approach
        "0FV60CZ",  # Restriction of Left Hepatic Duct with Extraluminal Device, Open Approach
        "0FV60DZ",  # Restriction of Left Hepatic Duct with Intraluminal Device, Open Approach
        "0FV60ZZ",  # Restriction of Left Hepatic Duct, Open Approach
        "0FV80CZ",  # Restriction of Cystic Duct with Extraluminal Device, Open Approach
        "0FV80DZ",  # Restriction of Cystic Duct with Intraluminal Device, Open Approach
        "0FV80ZZ",  # Restriction of Cystic Duct, Open Approach
        "0FV90CZ",  # Restriction of Common Bile Duct with Extraluminal Device, Open Approach
        "0FV90DZ",  # Restriction of Common Bile Duct with Intraluminal Device, Open Approach
        "0FV90ZZ",  # Restriction of Common Bile Duct, Open Approach
        "0FVC0CZ",  # Restriction of Ampulla of Vater with Extraluminal Device, Open Approach
        "0FVC0DZ",  # Restriction of Ampulla of Vater with Intraluminal Device, Open Approach
        "0FVC0ZZ",  # Restriction of Ampulla of Vater, Open Approach
        "0FVC7DZ",  # Restriction of Ampulla of Vater with Intraluminal Device, Via Natural or Artificial Opening
        "0FVC7ZZ",  # Restriction of Ampulla of Vater, Via Natural or Artificial Opening
        "0FVD0CZ",  # Restriction of Pancreatic Duct with Extraluminal Device, Open Approach
        "0FVD0DZ",  # Restriction of Pancreatic Duct with Intraluminal Device, Open Approach
        "0FVD0ZZ",  # Restriction of Pancreatic Duct, Open Approach
        "0FVD7DZ",  # Restriction of Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FVD7ZZ",  # Restriction of Pancreatic Duct, Via Natural or Artificial Opening
        "0FVF0CZ",  # Restriction of Accessory Pancreatic Duct with Extraluminal Device, Open Approach
        "0FVF0DZ",  # Restriction of Accessory Pancreatic Duct with Intraluminal Device, Open Approach
        "0FVF0ZZ",  # Restriction of Accessory Pancreatic Duct, Open Approach
        "0FVF7DZ",  # Restriction of Accessory Pancreatic Duct with Intraluminal Device, Via Natural or Artificial Opening
        "0FVF7ZZ",  # Restriction of Accessory Pancreatic Duct, Via Natural or Artificial Opening
        "0FW000Z",  # Revision of Drainage Device in Liver, Open Approach
        "0FW002Z",  # Revision of Monitoring Device in Liver, Open Approach
        "0FW003Z",  # Revision of Infusion Device in Liver, Open Approach
        "0FW400Z",  # Revision of Drainage Device in Gallbladder, Open Approach
        "0FW402Z",  # Revision of Monitoring Device in Gallbladder, Open Approach
        "0FW403Z",  # Revision of Infusion Device in Gallbladder, Open Approach
        "0FW40DZ",  # Revision of Intraluminal Device in Gallbladder, Open Approach
        "0FW40YZ",  # Revision of Other Device in Gallbladder, Open Approach
        "0FW480Z",  # Revision of Drainage Device in Gallbladder, Via Natural or Artificial Opening Endoscopic
        "0FWB00Z",  # Revision of Drainage Device in Hepatobiliary Duct, Open Approach
        "0FWB02Z",  # Revision of Monitoring Device in Hepatobiliary Duct, Open Approach
        "0FWB03Z",  # Revision of Infusion Device in Hepatobiliary Duct, Open Approach
        "0FWB07Z",  # Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Open Approach
        "0FWB0CZ",  # Revision of Extraluminal Device in Hepatobiliary Duct, Open Approach
        "0FWB0DZ",  # Revision of Intraluminal Device in Hepatobiliary Duct, Open Approach
        "0FWB0JZ",  # Revision of Synthetic Substitute in Hepatobiliary Duct, Open Approach
        "0FWB0KZ",  # Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Open Approach
        "0FWB70Z",  # Revision of Drainage Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB72Z",  # Revision of Monitoring Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB73Z",  # Revision of Infusion Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB77Z",  # Revision of Autologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7CZ",  # Revision of Extraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7DZ",  # Revision of Intraluminal Device in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7JZ",  # Revision of Synthetic Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWB7KZ",  # Revision of Nonautologous Tissue Substitute in Hepatobiliary Duct, Via Natural or Artificial Opening
        "0FWD00Z",  # Revision of Drainage Device in Pancreatic Duct, Open Approach
        "0FWD02Z",  # Revision of Monitoring Device in Pancreatic Duct, Open Approach
        "0FWD03Z",  # Revision of Infusion Device in Pancreatic Duct, Open Approach
        "0FWD07Z",  # Revision of Autologous Tissue Substitute in Pancreatic Duct, Open Approach
        "0FWD0CZ",  # Revision of Extraluminal Device in Pancreatic Duct, Open Approach
        "0FWD0DZ",  # Revision of Intraluminal Device in Pancreatic Duct, Open Approach
        "0FWD0JZ",  # Revision of Synthetic Substitute in Pancreatic Duct, Open Approach
        "0FWD0KZ",  # Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Open Approach
        "0FWD70Z",  # Revision of Drainage Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD72Z",  # Revision of Monitoring Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD73Z",  # Revision of Infusion Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD77Z",  # Revision of Autologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7CZ",  # Revision of Extraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7DZ",  # Revision of Intraluminal Device in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7JZ",  # Revision of Synthetic Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWD7KZ",  # Revision of Nonautologous Tissue Substitute in Pancreatic Duct, Via Natural or Artificial Opening
        "0FWG00Z",  # Revision of Drainage Device in Pancreas, Open Approach
        "0FWG02Z",  # Revision of Monitoring Device in Pancreas, Open Approach
        "0FWG03Z",  # Revision of Infusion Device in Pancreas, Open Approach
        "0FWG0DZ",  # Revision of Intraluminal Device in Pancreas, Open Approach
        "0FWG80Z",  # Revision of Drainage Device in Pancreas, Via Natural or Artificial Opening Endoscopic
        "0GC60ZZ",  # Extirpation of Matter from Left Carotid Body, Open Approach
        "0GC63ZZ",  # Extirpation of Matter from Left Carotid Body, Percutaneous Approach
        "0GC64ZZ",  # Extirpation of Matter from Left Carotid Body, Percutaneous Endoscopic Approach
        "0GC70ZZ",  # Extirpation of Matter from Right Carotid Body, Open Approach
        "0GC73ZZ",  # Extirpation of Matter from Right Carotid Body, Percutaneous Approach
        "0GC74ZZ",  # Extirpation of Matter from Right Carotid Body, Percutaneous Endoscopic Approach
        "0GC80ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Open Approach
        "0GC83ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Percutaneous Approach
        "0GC84ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GJ00ZZ",  # Inspection of Pituitary Gland, Open Approach
        "0GJ10ZZ",  # Inspection of Pineal Body, Open Approach
        "0GJ50ZZ",  # Inspection of Adrenal Gland, Open Approach
        "0GJK0ZZ",  # Inspection of Thyroid Gland, Open Approach
        "0GJR0ZZ",  # Inspection of Parathyroid Gland, Open Approach
        "0GJS0ZZ",  # Inspection of Endocrine Gland, Open Approach
        "0HRT075",  # Replacement of Right Breast using Latissimus Dorsi Myocutaneous Flap, Open Approach
        "0HRT076",  # Replacement of Right Breast using Transverse Rectus Abdominis Myocutaneous Flap, Open Approach
        "0HRT077",  # Replacement of Right Breast using Deep Inferior Epigastric Artery Perforator Flap, Open Approach
        "0HRT078",  # Replacement of Right Breast using Superficial Inferior Epigastric Artery Flap, Open Approach
        "0HRT079",  # Replacement of Right Breast using Gluteal Artery Perforator Flap, Open Approach
        "0HRT07B",  # Replacement of Right Breast using Lumbar Artery Perforator Flap, Open Approach
        "0HRU075",  # Replacement of Left Breast using Latissimus Dorsi Myocutaneous Flap, Open Approach
        "0HRU076",  # Replacement of Left Breast using Transverse Rectus Abdominis Myocutaneous Flap, Open Approach
        "0HRU077",  # Replacement of Left Breast using Deep Inferior Epigastric Artery Perforator Flap, Open Approach
        "0HRU078",  # Replacement of Left Breast using Superficial Inferior Epigastric Artery Flap, Open Approach
        "0HRU079",  # Replacement of Left Breast using Gluteal Artery Perforator Flap, Open Approach
        "0HRU07B",  # Replacement of Left Breast using Lumbar Artery Perforator Flap, Open Approach
        "0HRV075",  # Replacement of Bilateral Breast using Latissimus Dorsi Myocutaneous Flap, Open Approach
        "0HRV076",  # Replacement of Bilateral Breast using Transverse Rectus Abdominis Myocutaneous Flap, Open Approach
        "0HRV077",  # Replacement of Bilateral Breast using Deep Inferior Epigastric Artery Perforator Flap, Open Approach
        "0HRV078",  # Replacement of Bilateral Breast using Superficial Inferior Epigastric Artery Flap, Open Approach
        "0HRV079",  # Replacement of Bilateral Breast using Gluteal Artery Perforator Flap, Open Approach
        "0HRV07B",  # Replacement of Bilateral Breast using Lumbar Artery Perforator Flap, Open Approach
        "0JB80ZX",  # Excision of Abdomen Subcutaneous Tissue and Fascia, Open Approach, Diagnostic
        "0JB80ZZ",  # Excision of Abdomen Subcutaneous Tissue and Fascia, Open Approach
        "0KBK0ZX",  # Excision of Right Abdomen Muscle, Open Approach, Diagnostic
        "0KBK0ZZ",  # Excision of Right Abdomen Muscle, Open Approach
        "0KBL0ZX",  # Excision of Left Abdomen Muscle, Open Approach, Diagnostic
        "0KBL0ZZ",  # Excision of Left Abdomen Muscle, Open Approach
        "0KR507Z",  # Replacement of Right Shoulder Muscle with Autologous Tissue Substitute, Open Approach
        "0KR50JZ",  # Replacement of Right Shoulder Muscle with Synthetic Substitute, Open Approach
        "0KR50KZ",  # Replacement of Right Shoulder Muscle with Nonautologous Tissue Substitute, Open Approach
        "0KR547Z",  # Replacement of Right Shoulder Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KR54JZ",  # Replacement of Right Shoulder Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0KR54KZ",  # Replacement of Right Shoulder Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KR607Z",  # Replacement of Left Shoulder Muscle with Autologous Tissue Substitute, Open Approach
        "0KR60JZ",  # Replacement of Left Shoulder Muscle with Synthetic Substitute, Open Approach
        "0KR60KZ",  # Replacement of Left Shoulder Muscle with Nonautologous Tissue Substitute, Open Approach
        "0KR647Z",  # Replacement of Left Shoulder Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KR64JZ",  # Replacement of Left Shoulder Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0KR64KZ",  # Replacement of Left Shoulder Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KRN07Z",  # Replacement of Right Hip Muscle with Autologous Tissue Substitute, Open Approach
        "0KRN0JZ",  # Replacement of Right Hip Muscle with Synthetic Substitute, Open Approach
        "0KRN0KZ",  # Replacement of Right Hip Muscle with Nonautologous Tissue Substitute, Open Approach
        "0KRN47Z",  # Replacement of Right Hip Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KRN4JZ",  # Replacement of Right Hip Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0KRN4KZ",  # Replacement of Right Hip Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KRP07Z",  # Replacement of Left Hip Muscle with Autologous Tissue Substitute, Open Approach
        "0KRP0JZ",  # Replacement of Left Hip Muscle with Synthetic Substitute, Open Approach
        "0KRP0KZ",  # Replacement of Left Hip Muscle with Nonautologous Tissue Substitute, Open Approach
        "0KRP47Z",  # Replacement of Left Hip Muscle with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KRP4JZ",  # Replacement of Left Hip Muscle with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0KRP4KZ",  # Replacement of Left Hip Muscle with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0KXF0Z5",  # Transfer Right Trunk Muscle, Latissimus Dorsi Myocutaneous Flap, Open Approach
        "0KXF0Z7",  # Transfer Right Trunk Muscle, Deep Inferior Epigastric Artery Perforator Flap, Open Approach
        "0KXF0Z8",  # Transfer Right Trunk Muscle, Superficial Inferior Epigastric Artery Flap, Open Approach
        "0KXF0Z9",  # Transfer Right Trunk Muscle, Gluteal Artery Perforator Flap, Open Approach
        "0KXF0ZZ",  # Transfer Right Trunk Muscle, Open Approach
        "0KXF4Z5",  # Transfer Right Trunk Muscle, Latissimus Dorsi Myocutaneous Flap, Percutaneous Endoscopic Approach
        "0KXF4Z7",  # Transfer Right Trunk Muscle, Deep Inferior Epigastric Artery Perforator Flap, Percutaneous Endoscopic Approach
        "0KXF4Z8",  # Transfer Right Trunk Muscle, Superficial Inferior Epigastric Artery Flap, Percutaneous Endoscopic Approach
        "0KXF4Z9",  # Transfer Right Trunk Muscle, Gluteal Artery Perforator Flap, Percutaneous Endoscopic Approach
        "0KXG0Z5",  # Transfer Left Trunk Muscle, Latissimus Dorsi Myocutaneous Flap, Open Approach
        "0KXG0Z7",  # Transfer Left Trunk Muscle, Deep Inferior Epigastric Artery Perforator Flap, Open Approach
        "0KXG0Z8",  # Transfer Left Trunk Muscle, Superficial Inferior Epigastric Artery Flap, Open Approach
        "0KXG0Z9",  # Transfer Left Trunk Muscle, Gluteal Artery Perforator Flap, Open Approach
        "0KXG0ZZ",  # Transfer Left Trunk Muscle, Open Approach
        "0KXG4Z5",  # Transfer Left Trunk Muscle, Latissimus Dorsi Myocutaneous Flap, Percutaneous Endoscopic Approach
        "0KXG4Z7",  # Transfer Left Trunk Muscle, Deep Inferior Epigastric Artery Perforator Flap, Percutaneous Endoscopic Approach
        "0KXG4Z8",  # Transfer Left Trunk Muscle, Superficial Inferior Epigastric Artery Flap, Percutaneous Endoscopic Approach
        "0KXG4Z9",  # Transfer Left Trunk Muscle, Gluteal Artery Perforator Flap, Percutaneous Endoscopic Approach
        "0KXK0Z6",  # Transfer Right Abdomen Muscle, Transverse Rectus Abdominis Myocutaneous Flap, Open Approach
        "0KXK4Z6",  # Transfer Right Abdomen Muscle, Transverse Rectus Abdominis Myocutaneous Flap, Percutaneous Endoscopic Approach
        "0KXL0Z6",  # Transfer Left Abdomen Muscle, Transverse Rectus Abdominis Myocutaneous Flap, Open Approach
        "0KXL4Z6",  # Transfer Left Abdomen Muscle, Transverse Rectus Abdominis Myocutaneous Flap, Percutaneous Endoscopic Approach
        "0LBF0ZX",  # Excision of Right Abdomen Tendon, Open Approach, Diagnostic
        "0LBF0ZZ",  # Excision of Right Abdomen Tendon, Open Approach
        "0LBG0ZX",  # Excision of Left Abdomen Tendon, Open Approach, Diagnostic
        "0LBG0ZZ",  # Excision of Left Abdomen Tendon, Open Approach
        "0MBH0ZX",  # Excision of Right Abdomen Bursa and Ligament, Open Approach, Diagnostic
        "0MBH0ZZ",  # Excision of Right Abdomen Bursa and Ligament, Open Approach
        "0MBJ0ZX",  # Excision of Left Abdomen Bursa and Ligament, Open Approach, Diagnostic
        "0MBJ0ZZ",  # Excision of Left Abdomen Bursa and Ligament, Open Approach
        "0MR107Z",  # Replacement of Right Shoulder Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MR10JZ",  # Replacement of Right Shoulder Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MR10KZ",  # Replacement of Right Shoulder Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MR147Z",  # Replacement of Right Shoulder Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR14JZ",  # Replacement of Right Shoulder Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MR14KZ",  # Replacement of Right Shoulder Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR207Z",  # Replacement of Left Shoulder Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MR20JZ",  # Replacement of Left Shoulder Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MR20KZ",  # Replacement of Left Shoulder Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MR247Z",  # Replacement of Left Shoulder Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR24JZ",  # Replacement of Left Shoulder Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MR24KZ",  # Replacement of Left Shoulder Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR307Z",  # Replacement of Right Elbow Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MR30JZ",  # Replacement of Right Elbow Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MR30KZ",  # Replacement of Right Elbow Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MR347Z",  # Replacement of Right Elbow Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR34JZ",  # Replacement of Right Elbow Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MR34KZ",  # Replacement of Right Elbow Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR407Z",  # Replacement of Left Elbow Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MR40JZ",  # Replacement of Left Elbow Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MR40KZ",  # Replacement of Left Elbow Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MR447Z",  # Replacement of Left Elbow Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MR44JZ",  # Replacement of Left Elbow Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MR44KZ",  # Replacement of Left Elbow Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRL07Z",  # Replacement of Right Hip Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MRL0JZ",  # Replacement of Right Hip Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MRL0KZ",  # Replacement of Right Hip Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MRL47Z",  # Replacement of Right Hip Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRL4JZ",  # Replacement of Right Hip Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MRL4KZ",  # Replacement of Right Hip Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRM07Z",  # Replacement of Left Hip Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MRM0JZ",  # Replacement of Left Hip Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MRM0KZ",  # Replacement of Left Hip Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MRM47Z",  # Replacement of Left Hip Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRM4JZ",  # Replacement of Left Hip Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MRM4KZ",  # Replacement of Left Hip Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRN07Z",  # Replacement of Right Knee Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MRN0JZ",  # Replacement of Right Knee Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MRN0KZ",  # Replacement of Right Knee Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MRN47Z",  # Replacement of Right Knee Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRN4JZ",  # Replacement of Right Knee Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MRN4KZ",  # Replacement of Right Knee Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRP07Z",  # Replacement of Left Knee Bursa and Ligament with Autologous Tissue Substitute, Open Approach
        "0MRP0JZ",  # Replacement of Left Knee Bursa and Ligament with Synthetic Substitute, Open Approach
        "0MRP0KZ",  # Replacement of Left Knee Bursa and Ligament with Nonautologous Tissue Substitute, Open Approach
        "0MRP47Z",  # Replacement of Left Knee Bursa and Ligament with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0MRP4JZ",  # Replacement of Left Knee Bursa and Ligament with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0MRP4KZ",  # Replacement of Left Knee Bursa and Ligament with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0N500ZZ",  # Destruction of Skull, Open Approach
        "0N510ZZ",  # Destruction of Frontal Bone, Open Approach
        "0N530ZZ",  # Destruction of Right Parietal Bone, Open Approach
        "0N540ZZ",  # Destruction of Left Parietal Bone, Open Approach
        "0N550ZZ",  # Destruction of Right Temporal Bone, Open Approach
        "0N560ZZ",  # Destruction of Left Temporal Bone, Open Approach
        "0N570ZZ",  # Destruction of Occipital Bone, Open Approach
        "0N800ZZ",  # Division of Skull, Open Approach
        "0N810ZZ",  # Division of Frontal Bone, Open Approach
        "0N830ZZ",  # Division of Right Parietal Bone, Open Approach
        "0N840ZZ",  # Division of Left Parietal Bone, Open Approach
        "0N850ZZ",  # Division of Right Temporal Bone, Open Approach
        "0N860ZZ",  # Division of Left Temporal Bone, Open Approach
        "0N870ZZ",  # Division of Occipital Bone, Open Approach
        "0N9000Z",  # Drainage of Skull with Drainage Device, Open Approach
        "0N900ZZ",  # Drainage of Skull, Open Approach
        "0N9100Z",  # Drainage of Frontal Bone with Drainage Device, Open Approach
        "0N910ZZ",  # Drainage of Frontal Bone, Open Approach
        "0N9300Z",  # Drainage of Right Parietal Bone with Drainage Device, Open Approach
        "0N930ZZ",  # Drainage of Right Parietal Bone, Open Approach
        "0N9400Z",  # Drainage of Left Parietal Bone with Drainage Device, Open Approach
        "0N940ZZ",  # Drainage of Left Parietal Bone, Open Approach
        "0N9500Z",  # Drainage of Right Temporal Bone with Drainage Device, Open Approach
        "0N950ZZ",  # Drainage of Right Temporal Bone, Open Approach
        "0N9600Z",  # Drainage of Left Temporal Bone with Drainage Device, Open Approach
        "0N960ZZ",  # Drainage of Left Temporal Bone, Open Approach
        "0N9700Z",  # Drainage of Occipital Bone with Drainage Device, Open Approach
        "0N970ZZ",  # Drainage of Occipital Bone, Open Approach
        "0NB00ZX",  # Excision of Skull, Open Approach, Diagnostic
        "0NB00ZZ",  # Excision of Skull, Open Approach
        "0NB03ZX",  # Excision of Skull, Percutaneous Approach, Diagnostic
        "0NB03ZZ",  # Excision of Skull, Percutaneous Approach
        "0NB04ZX",  # Excision of Skull, Percutaneous Endoscopic Approach, Diagnostic
        "0NB04ZZ",  # Excision of Skull, Percutaneous Endoscopic Approach
        "0NB10ZZ",  # Excision of Frontal Bone, Open Approach
        "0NB30ZZ",  # Excision of Right Parietal Bone, Open Approach
        "0NB40ZZ",  # Excision of Left Parietal Bone, Open Approach
        "0NB50ZZ",  # Excision of Right Temporal Bone, Open Approach
        "0NB60ZZ",  # Excision of Left Temporal Bone, Open Approach
        "0NB70ZZ",  # Excision of Occipital Bone, Open Approach
        "0NBR0ZZ",  # Excision of Maxilla, Open Approach
        "0NBT0ZZ",  # Excision of Right Mandible, Open Approach
        "0NBV0ZZ",  # Excision of Left Mandible, Open Approach
        "0NC10ZZ",  # Extirpation of Matter from Frontal Bone, Open Approach
        "0NC30ZZ",  # Extirpation of Matter from Right Parietal Bone, Open Approach
        "0NC40ZZ",  # Extirpation of Matter from Left Parietal Bone, Open Approach
        "0NC50ZZ",  # Extirpation of Matter from Right Temporal Bone, Open Approach
        "0NC60ZZ",  # Extirpation of Matter from Left Temporal Bone, Open Approach
        "0NC70ZZ",  # Extirpation of Matter from Occipital Bone, Open Approach
        "0NH00MZ",  # Insertion of Bone Growth Stimulator into Skull, Open Approach
        "0NJ00ZZ",  # Inspection of Skull, Open Approach
        "0NP000Z",  # Removal of Drainage Device from Skull, Open Approach
        "0NP004Z",  # Removal of Internal Fixation Device from Skull, Open Approach
        "0NP005Z",  # Removal of External Fixation Device from Skull, Open Approach
        "0NP007Z",  # Removal of Autologous Tissue Substitute from Skull, Open Approach
        "0NP00KZ",  # Removal of Nonautologous Tissue Substitute from Skull, Open Approach
        "0NP00SZ",  # Removal of Hearing Device from Skull, Open Approach
        "0NP0X4Z",  # Removal of Internal Fixation Device from Skull, External Approach
        "0NP0XSZ",  # Removal of Hearing Device from Skull, External Approach
        "0NT10ZZ",  # Resection of Frontal Bone, Open Approach
        "0NT30ZZ",  # Resection of Right Parietal Bone, Open Approach
        "0NT40ZZ",  # Resection of Left Parietal Bone, Open Approach
        "0NT50ZZ",  # Resection of Right Temporal Bone, Open Approach
        "0NT60ZZ",  # Resection of Left Temporal Bone, Open Approach
        "0NT70ZZ",  # Resection of Occipital Bone, Open Approach
        "0NTR0ZZ",  # Resection of Maxilla, Open Approach
        "0NTT0ZZ",  # Resection of Right Mandible, Open Approach
        "0NTV0ZZ",  # Resection of Left Mandible, Open Approach
        "0NW000Z",  # Revision of Drainage Device in Skull, Open Approach
        "0NW004Z",  # Revision of Internal Fixation Device in Skull, Open Approach
        "0NW005Z",  # Revision of External Fixation Device in Skull, Open Approach
        "0NW007Z",  # Revision of Autologous Tissue Substitute in Skull, Open Approach
        "0NW00JZ",  # Revision of Synthetic Substitute in Skull, Open Approach
        "0NW00KZ",  # Revision of Nonautologous Tissue Substitute in Skull, Open Approach
        "0NW00MZ",  # Revision of Bone Growth Stimulator in Skull, Open Approach
        "0NW00NZ",  # Revision of Neurostimulator Generator in Skull, Open Approach
        "0NW00SZ",  # Revision of Hearing Device in Skull, Open Approach
        "0PB30ZX",  # Excision of Cervical Vertebra, Open Approach, Diagnostic
        "0PB30ZZ",  # Excision of Cervical Vertebra, Open Approach
        "0PB33ZX",  # Excision of Cervical Vertebra, Percutaneous Approach, Diagnostic
        "0PB33ZZ",  # Excision of Cervical Vertebra, Percutaneous Approach
        "0PB34ZX",  # Excision of Cervical Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0PB34ZZ",  # Excision of Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PB40ZX",  # Excision of Thoracic Vertebra, Open Approach, Diagnostic
        "0PB40ZZ",  # Excision of Thoracic Vertebra, Open Approach
        "0PB43ZX",  # Excision of Thoracic Vertebra, Percutaneous Approach, Diagnostic
        "0PB43ZZ",  # Excision of Thoracic Vertebra, Percutaneous Approach
        "0PB44ZX",  # Excision of Thoracic Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0PB44ZZ",  # Excision of Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PT10ZZ",  # Resection of 1 to 2 Ribs, Open Approach
        "0PT20ZZ",  # Resection of 3 or More Ribs, Open Approach
        "0QB00ZX",  # Excision of Lumbar Vertebra, Open Approach, Diagnostic
        "0QB00ZZ",  # Excision of Lumbar Vertebra, Open Approach
        "0QB03ZX",  # Excision of Lumbar Vertebra, Percutaneous Approach, Diagnostic
        "0QB03ZZ",  # Excision of Lumbar Vertebra, Percutaneous Approach
        "0QB04ZX",  # Excision of Lumbar Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0QB04ZZ",  # Excision of Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QPD0JZ",  # Removal of Synthetic Substitute from Right Patella, Open Approach
        "0QPF0JZ",  # Removal of Synthetic Substitute from Left Patella, Open Approach
        "0QRD0JZ",  # Replacement of Right Patella with Synthetic Substitute, Open Approach
        "0QRF0JZ",  # Replacement of Left Patella with Synthetic Substitute, Open Approach
        "0QUD0JZ",  # Supplement Right Patella with Synthetic Substitute, Open Approach
        "0QUF0JZ",  # Supplement Left Patella with Synthetic Substitute, Open Approach
        "0RG1070",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG1071",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG107J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG10A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG10AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG10J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG10J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG10JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG10K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG10K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG10KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG1370",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG1371",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG137J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG13JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG13KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG1470",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG1471",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG147J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG14JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG14KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG2070",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG2071",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG207J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG20A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG20AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG20J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG20J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG20JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG20K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG20K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG20KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG2370",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG2371",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG237J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG23JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG23KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG2470",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG2471",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG247J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG24JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG24KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG4070",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG4071",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG407J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG40A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG40AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG40J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG40J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG40JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG40K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG40K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG40KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG4370",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG4371",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG437J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG43JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG43KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG4470",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG4471",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG447J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG44JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG44KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG6070",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG6071",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG607J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG60A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG60AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG60J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG60J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG60JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG60K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG60K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG60KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG6370",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG6371",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG637J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG63JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG63KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG6470",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG6471",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG647J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG64JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG64KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG7070",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG7071",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG707J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG70A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG70AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG70J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG70J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG70JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG70K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG70K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG70KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG7370",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG7371",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG737J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG73JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG73KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG7470",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG7471",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG747J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG74JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG74KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG8070",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG8071",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG807J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG80A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG80AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG80J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG80J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG80JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG80K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG80K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG80KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG8370",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG8371",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG837J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG83JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG83KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG8470",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG8471",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG847J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG84JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG84KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA070",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA071",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA07J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA0A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RGA0AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RGA0J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA0J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA0JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA0K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA0K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA0KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA370",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA371",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA37J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA3JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA3KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA470",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA471",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA47J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA4JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA4KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RP10AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Open Approach
        "0RP13AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Percutaneous Approach
        "0RP14AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP40AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Open Approach
        "0RP43AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP44AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP60AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Open Approach
        "0RP63AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP64AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA0AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA3AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA4AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RRJ00Z",  # Replacement of Right Shoulder Joint with Reverse Ball and Socket Synthetic Substitute, Open Approach
        "0RRJ07Z",  # Replacement of Right Shoulder Joint with Autologous Tissue Substitute, Open Approach
        "0RRJ0J6",  # Replacement of Right Shoulder Joint with Synthetic Substitute, Humeral Surface, Open Approach
        "0RRJ0J7",  # Replacement of Right Shoulder Joint with Synthetic Substitute, Glenoid Surface, Open Approach
        "0RRJ0JZ",  # Replacement of Right Shoulder Joint with Synthetic Substitute, Open Approach
        "0RRJ0KZ",  # Replacement of Right Shoulder Joint with Nonautologous Tissue Substitute, Open Approach
        "0RRK00Z",  # Replacement of Left Shoulder Joint with Reverse Ball and Socket Synthetic Substitute, Open Approach
        "0RRK07Z",  # Replacement of Left Shoulder Joint with Autologous Tissue Substitute, Open Approach
        "0RRK0J6",  # Replacement of Left Shoulder Joint with Synthetic Substitute, Humeral Surface, Open Approach
        "0RRK0J7",  # Replacement of Left Shoulder Joint with Synthetic Substitute, Glenoid Surface, Open Approach
        "0RRK0JZ",  # Replacement of Left Shoulder Joint with Synthetic Substitute, Open Approach
        "0RRK0KZ",  # Replacement of Left Shoulder Joint with Nonautologous Tissue Substitute, Open Approach
        "0RRL07Z",  # Replacement of Right Elbow Joint with Autologous Tissue Substitute, Open Approach
        "0RRL0JZ",  # Replacement of Right Elbow Joint with Synthetic Substitute, Open Approach
        "0RRL0KZ",  # Replacement of Right Elbow Joint with Nonautologous Tissue Substitute, Open Approach
        "0RRM07Z",  # Replacement of Left Elbow Joint with Autologous Tissue Substitute, Open Approach
        "0RRM0JZ",  # Replacement of Left Elbow Joint with Synthetic Substitute, Open Approach
        "0RRM0KZ",  # Replacement of Left Elbow Joint with Nonautologous Tissue Substitute, Open Approach
        "0RW10AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Open Approach
        "0RW13AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Percutaneous Approach
        "0RW14AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW1XAZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, External Approach
        "0RW40AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Open Approach
        "0RW43AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW44AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW4XAZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, External Approach
        "0RW60AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Open Approach
        "0RW63AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW64AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW6XAZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, External Approach
        "0RWA0AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA3AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA4AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWAXAZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, External Approach
        "0RWJ0JZ",  # Revision of Synthetic Substitute in Right Shoulder Joint, Open Approach
        "0RWJ3JZ",  # Revision of Synthetic Substitute in Right Shoulder Joint, Percutaneous Approach
        "0RWJ4JZ",  # Revision of Synthetic Substitute in Right Shoulder Joint, Percutaneous Endoscopic Approach
        "0RWJXJZ",  # Revision of Synthetic Substitute in Right Shoulder Joint, External Approach
        "0RWK0JZ",  # Revision of Synthetic Substitute in Left Shoulder Joint, Open Approach
        "0RWK3JZ",  # Revision of Synthetic Substitute in Left Shoulder Joint, Percutaneous Approach
        "0RWK4JZ",  # Revision of Synthetic Substitute in Left Shoulder Joint, Percutaneous Endoscopic Approach
        "0RWKXJZ",  # Revision of Synthetic Substitute in Left Shoulder Joint, External Approach
        "0RWL0JZ",  # Revision of Synthetic Substitute in Right Elbow Joint, Open Approach
        "0RWL3JZ",  # Revision of Synthetic Substitute in Right Elbow Joint, Percutaneous Approach
        "0RWL4JZ",  # Revision of Synthetic Substitute in Right Elbow Joint, Percutaneous Endoscopic Approach
        "0RWLXJZ",  # Revision of Synthetic Substitute in Right Elbow Joint, External Approach
        "0RWM0JZ",  # Revision of Synthetic Substitute in Left Elbow Joint, Open Approach
        "0RWM3JZ",  # Revision of Synthetic Substitute in Left Elbow Joint, Percutaneous Approach
        "0RWM4JZ",  # Revision of Synthetic Substitute in Left Elbow Joint, Percutaneous Endoscopic Approach
        "0RWMXJZ",  # Revision of Synthetic Substitute in Left Elbow Joint, External Approach
        "0SG0070",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG0071",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG007J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG00A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0SG00AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0SG00J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG00J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG00JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG00K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG00K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG00KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG0370",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG0371",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG037J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG03JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG03KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG0470",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG0471",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG047J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG04JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG04KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG1070",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG1071",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG107J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG10A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0SG10AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0SG10J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG10J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG10JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG10K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG10K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG10KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG1370",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG1371",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG137J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG13JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG13KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG1470",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG1471",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG147J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG14JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG14KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SP00AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Open Approach
        "0SP03AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP04AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP909Z",  # Removal of Liner from Right Hip Joint, Open Approach
        "0SP90JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Open Approach
        "0SPB09Z",  # Removal of Liner from Left Hip Joint, Open Approach
        "0SPB0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Open Approach
        "0SPC09Z",  # Removal of Liner from Right Knee Joint, Open Approach
        "0SPC0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPD09Z",  # Removal of Liner from Left Knee Joint, Open Approach
        "0SPD0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Open Approach
        "0SQ90ZZ",  # Repair Right Hip Joint, Open Approach
        "0SQ9XZZ",  # Repair Right Hip Joint, External Approach
        "0SQB0ZZ",  # Repair Left Hip Joint, Open Approach
        "0SQBXZZ",  # Repair Left Hip Joint, External Approach
        "0SR9019",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SR901A",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SR901Z",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SR9029",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR902A",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR902Z",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SR9039",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SR903A",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SR903Z",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SR9049",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR904A",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR904Z",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SR9069",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR906A",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR906Z",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SR907Z",  # Replacement of Right Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SR90EZ",  # Replacement of Right Hip Joint with Articulating Spacer, Open Approach
        "0SR90J9",  # Replacement of Right Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SR90JA",  # Replacement of Right Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SR90JZ",  # Replacement of Right Hip Joint with Synthetic Substitute, Open Approach
        "0SR90KZ",  # Replacement of Right Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRA009",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRA00A",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRA00Z",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRA019",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRA01A",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRA01Z",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRA039",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRA03A",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRA03Z",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRA07Z",  # Replacement of Right Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRA0J9",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRA0JA",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRA0JZ",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRA0KZ",  # Replacement of Right Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRB019",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRB01A",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRB01Z",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SRB029",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB02A",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB02Z",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SRB039",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRB03A",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRB03Z",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SRB049",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB04A",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB04Z",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SRB069",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB06A",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB06Z",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRB07Z",  # Replacement of Left Hip Joint with Autologous Tissue Substitute, Open Approach
        "0SRB0EZ",  # Replacement of Left Hip Joint with Articulating Spacer, Open Approach
        "0SRB0J9",  # Replacement of Left Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRB0JA",  # Replacement of Left Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRB0JZ",  # Replacement of Left Hip Joint with Synthetic Substitute, Open Approach
        "0SRB0KZ",  # Replacement of Left Hip Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRC069",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRC06A",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRC06Z",  # Replacement of Right Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRC07Z",  # Replacement of Right Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SRC0EZ",  # Replacement of Right Knee Joint with Articulating Spacer, Open Approach
        "0SRC0J9",  # Replacement of Right Knee Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRC0JA",  # Replacement of Right Knee Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRC0JZ",  # Replacement of Right Knee Joint with Synthetic Substitute, Open Approach
        "0SRC0KZ",  # Replacement of Right Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRC0L9",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRC0LA",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRC0LZ",  # Replacement of Right Knee Joint with Medial Unicondylar Synthetic Substitute, Open Approach
        "0SRC0M9",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRC0MA",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRC0MZ",  # Replacement of Right Knee Joint with Lateral Unicondylar Synthetic Substitute, Open Approach
        "0SRC0N9",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Cemented, Open Approach
        "0SRC0NA",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Uncemented, Open Approach
        "0SRC0NZ",  # Replacement of Right Knee Joint with Patellofemoral Synthetic Substitute, Open Approach
        "0SRD069",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRD06A",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRD06Z",  # Replacement of Left Knee Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRD07Z",  # Replacement of Left Knee Joint with Autologous Tissue Substitute, Open Approach
        "0SRD0EZ",  # Replacement of Left Knee Joint with Articulating Spacer, Open Approach
        "0SRD0J9",  # Replacement of Left Knee Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRD0JA",  # Replacement of Left Knee Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRD0JZ",  # Replacement of Left Knee Joint with Synthetic Substitute, Open Approach
        "0SRD0KZ",  # Replacement of Left Knee Joint with Nonautologous Tissue Substitute, Open Approach
        "0SRD0L9",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRD0LA",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRD0LZ",  # Replacement of Left Knee Joint with Medial Unicondylar Synthetic Substitute, Open Approach
        "0SRD0M9",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Cemented, Open Approach
        "0SRD0MA",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Uncemented, Open Approach
        "0SRD0MZ",  # Replacement of Left Knee Joint with Lateral Unicondylar Synthetic Substitute, Open Approach
        "0SRD0N9",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Cemented, Open Approach
        "0SRD0NA",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Uncemented, Open Approach
        "0SRD0NZ",  # Replacement of Left Knee Joint with Patellofemoral Synthetic Substitute, Open Approach
        "0SRE009",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRE00A",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRE00Z",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRE019",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRE01A",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRE01Z",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRE039",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRE03A",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRE03Z",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRE07Z",  # Replacement of Left Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRE0J9",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRE0JA",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRE0JZ",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRE0KZ",  # Replacement of Left Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRR019",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRR01A",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRR01Z",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRR039",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRR03A",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRR03Z",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRR07Z",  # Replacement of Right Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRR0J9",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRR0JA",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRR0JZ",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRR0KZ",  # Replacement of Right Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRS019",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRS01A",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRS01Z",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRS039",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRS03A",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRS03Z",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRS07Z",  # Replacement of Left Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRS0J9",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRS0JA",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRS0JZ",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRS0KZ",  # Replacement of Left Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRT07Z",  # Replacement of Right Knee Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRT0J9",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRT0JA",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRT0JZ",  # Replacement of Right Knee Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRT0KZ",  # Replacement of Right Knee Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRU07Z",  # Replacement of Left Knee Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRU0J9",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRU0JA",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRU0JZ",  # Replacement of Left Knee Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRU0KZ",  # Replacement of Left Knee Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRV07Z",  # Replacement of Right Knee Joint, Tibial Surface with Autologous Tissue Substitute, Open Approach
        "0SRV0J9",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRV0JA",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRV0JZ",  # Replacement of Right Knee Joint, Tibial Surface with Synthetic Substitute, Open Approach
        "0SRV0KZ",  # Replacement of Right Knee Joint, Tibial Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRW07Z",  # Replacement of Left Knee Joint, Tibial Surface with Autologous Tissue Substitute, Open Approach
        "0SRW0J9",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRW0JA",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRW0JZ",  # Replacement of Left Knee Joint, Tibial Surface with Synthetic Substitute, Open Approach
        "0SRW0KZ",  # Replacement of Left Knee Joint, Tibial Surface with Nonautologous Tissue Substitute, Open Approach
        "0SU909Z",  # Supplement Right Hip Joint with Liner, Open Approach
        "0SU90BZ",  # Supplement Right Hip Joint with Resurfacing Device, Open Approach
        "0SUA09Z",  # Supplement Right Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUA0BZ",  # Supplement Right Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUB09Z",  # Supplement Left Hip Joint with Liner, Open Approach
        "0SUB0BZ",  # Supplement Left Hip Joint with Resurfacing Device, Open Approach
        "0SUC09C",  # Supplement Right Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUC0JZ",  # Supplement Right Knee Joint with Synthetic Substitute, Open Approach
        "0SUD09C",  # Supplement Left Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUD0JZ",  # Supplement Left Knee Joint with Synthetic Substitute, Open Approach
        "0SUE09Z",  # Supplement Left Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUE0BZ",  # Supplement Left Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUR09Z",  # Supplement Right Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUR0BZ",  # Supplement Right Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SUS09Z",  # Supplement Left Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUS0BZ",  # Supplement Left Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SUT09Z",  # Supplement Right Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUU09Z",  # Supplement Left Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUV09Z",  # Supplement Right Knee Joint, Tibial Surface with Liner, Open Approach
        "0SUW09Z",  # Supplement Left Knee Joint, Tibial Surface with Liner, Open Approach
        "0SW00AZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, Open Approach
        "0SW03AZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW04AZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW0XAZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, External Approach
        "0SW90JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Open Approach
        "0SW93JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Percutaneous Approach
        "0SW94JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Percutaneous Endoscopic Approach
        "0SW9XJZ",  # Revision of Synthetic Substitute in Right Hip Joint, External Approach
        "0SWA0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Open Approach
        "0SWA3JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SWA4JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SWAXJZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, External Approach
        "0SWB0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Open Approach
        "0SWB3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Percutaneous Approach
        "0SWB4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Percutaneous Endoscopic Approach
        "0SWBXJZ",  # Revision of Synthetic Substitute in Left Hip Joint, External Approach
        "0SWC0JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Open Approach
        "0SWC0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Open Approach
        "0SWC3JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Percutaneous Approach
        "0SWC3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Percutaneous Approach
        "0SWC4JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SWC4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Percutaneous Endoscopic Approach
        "0SWCXJC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, External Approach
        "0SWCXJZ",  # Revision of Synthetic Substitute in Right Knee Joint, External Approach
        "0SWD0JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Open Approach
        "0SWD0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Open Approach
        "0SWD3JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Percutaneous Approach
        "0SWD3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Percutaneous Approach
        "0SWD4JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SWD4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Percutaneous Endoscopic Approach
        "0SWDXJC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, External Approach
        "0SWDXJZ",  # Revision of Synthetic Substitute in Left Knee Joint, External Approach
        "0SWE0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Open Approach
        "0SWE3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SWE4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SWEXJZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, External Approach
        "0SWR0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Open Approach
        "0SWR3JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Percutaneous Approach
        "0SWR4JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWRXJZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, External Approach
        "0SWS0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Open Approach
        "0SWS3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Percutaneous Approach
        "0SWS4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWSXJZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, External Approach
        "0SWT0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Open Approach
        "0SWT3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Percutaneous Approach
        "0SWT4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWTXJZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, External Approach
        "0SWU0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Open Approach
        "0SWU3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Percutaneous Approach
        "0SWU4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWUXJZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, External Approach
        "0SWV0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Open Approach
        "0SWV3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Percutaneous Approach
        "0SWV4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
        "0SWVXJZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, External Approach
        "0SWW0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Open Approach
        "0SWW3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Percutaneous Approach
        "0SWW4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
        "0SWWXJZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, External Approach
        "0T1307B",  # Bypass Right Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach
        "0T130JB",  # Bypass Right Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach
        "0T130KB",  # Bypass Right Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T130ZB",  # Bypass Right Kidney Pelvis to Bladder, Open Approach
        "0T1407B",  # Bypass Left Kidney Pelvis to Bladder with Autologous Tissue Substitute, Open Approach
        "0T140JB",  # Bypass Left Kidney Pelvis to Bladder with Synthetic Substitute, Open Approach
        "0T140KB",  # Bypass Left Kidney Pelvis to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T140ZB",  # Bypass Left Kidney Pelvis to Bladder, Open Approach
        "0T16076",  # Bypass Right Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T16077",  # Bypass Right Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T16078",  # Bypass Right Ureter to Colon with Autologous Tissue Substitute, Open Approach
        "0T1607A",  # Bypass Right Ureter to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1607B",  # Bypass Right Ureter to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1607C",  # Bypass Right Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T160J6",  # Bypass Right Ureter to Right Ureter with Synthetic Substitute, Open Approach
        "0T160J7",  # Bypass Right Ureter to Left Ureter with Synthetic Substitute, Open Approach
        "0T160J8",  # Bypass Right Ureter to Colon with Synthetic Substitute, Open Approach
        "0T160JA",  # Bypass Right Ureter to Ileum with Synthetic Substitute, Open Approach
        "0T160JB",  # Bypass Right Ureter to Bladder with Synthetic Substitute, Open Approach
        "0T160JC",  # Bypass Right Ureter to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T160K6",  # Bypass Right Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T160K7",  # Bypass Right Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T160K8",  # Bypass Right Ureter to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T160KA",  # Bypass Right Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T160KB",  # Bypass Right Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T160KC",  # Bypass Right Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T160Z6",  # Bypass Right Ureter to Right Ureter, Open Approach
        "0T160Z7",  # Bypass Right Ureter to Left Ureter, Open Approach
        "0T160Z8",  # Bypass Right Ureter to Colon, Open Approach
        "0T160ZA",  # Bypass Right Ureter to Ileum, Open Approach
        "0T160ZB",  # Bypass Right Ureter to Bladder, Open Approach
        "0T160ZC",  # Bypass Right Ureter to Ileocutaneous, Open Approach
        "0T17076",  # Bypass Left Ureter to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T17077",  # Bypass Left Ureter to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T17078",  # Bypass Left Ureter to Colon with Autologous Tissue Substitute, Open Approach
        "0T1707A",  # Bypass Left Ureter to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1707B",  # Bypass Left Ureter to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1707C",  # Bypass Left Ureter to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T170J6",  # Bypass Left Ureter to Right Ureter with Synthetic Substitute, Open Approach
        "0T170J7",  # Bypass Left Ureter to Left Ureter with Synthetic Substitute, Open Approach
        "0T170J8",  # Bypass Left Ureter to Colon with Synthetic Substitute, Open Approach
        "0T170JA",  # Bypass Left Ureter to Ileum with Synthetic Substitute, Open Approach
        "0T170JB",  # Bypass Left Ureter to Bladder with Synthetic Substitute, Open Approach
        "0T170JC",  # Bypass Left Ureter to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T170K6",  # Bypass Left Ureter to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T170K7",  # Bypass Left Ureter to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T170K8",  # Bypass Left Ureter to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T170KA",  # Bypass Left Ureter to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T170KB",  # Bypass Left Ureter to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T170KC",  # Bypass Left Ureter to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T170Z6",  # Bypass Left Ureter to Right Ureter, Open Approach
        "0T170Z7",  # Bypass Left Ureter to Left Ureter, Open Approach
        "0T170Z8",  # Bypass Left Ureter to Colon, Open Approach
        "0T170ZA",  # Bypass Left Ureter to Ileum, Open Approach
        "0T170ZB",  # Bypass Left Ureter to Bladder, Open Approach
        "0T170ZC",  # Bypass Left Ureter to Ileocutaneous, Open Approach
        "0T18076",  # Bypass Bilateral Ureters to Right Ureter with Autologous Tissue Substitute, Open Approach
        "0T18077",  # Bypass Bilateral Ureters to Left Ureter with Autologous Tissue Substitute, Open Approach
        "0T18078",  # Bypass Bilateral Ureters to Colon with Autologous Tissue Substitute, Open Approach
        "0T1807A",  # Bypass Bilateral Ureters to Ileum with Autologous Tissue Substitute, Open Approach
        "0T1807B",  # Bypass Bilateral Ureters to Bladder with Autologous Tissue Substitute, Open Approach
        "0T1807C",  # Bypass Bilateral Ureters to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T180J6",  # Bypass Bilateral Ureters to Right Ureter with Synthetic Substitute, Open Approach
        "0T180J7",  # Bypass Bilateral Ureters to Left Ureter with Synthetic Substitute, Open Approach
        "0T180J8",  # Bypass Bilateral Ureters to Colon with Synthetic Substitute, Open Approach
        "0T180JA",  # Bypass Bilateral Ureters to Ileum with Synthetic Substitute, Open Approach
        "0T180JB",  # Bypass Bilateral Ureters to Bladder with Synthetic Substitute, Open Approach
        "0T180JC",  # Bypass Bilateral Ureters to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T180K6",  # Bypass Bilateral Ureters to Right Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T180K7",  # Bypass Bilateral Ureters to Left Ureter with Nonautologous Tissue Substitute, Open Approach
        "0T180K8",  # Bypass Bilateral Ureters to Colon with Nonautologous Tissue Substitute, Open Approach
        "0T180KA",  # Bypass Bilateral Ureters to Ileum with Nonautologous Tissue Substitute, Open Approach
        "0T180KB",  # Bypass Bilateral Ureters to Bladder with Nonautologous Tissue Substitute, Open Approach
        "0T180KC",  # Bypass Bilateral Ureters to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T180Z6",  # Bypass Bilateral Ureters to Right Ureter, Open Approach
        "0T180Z7",  # Bypass Bilateral Ureters to Left Ureter, Open Approach
        "0T180Z8",  # Bypass Bilateral Ureters to Colon, Open Approach
        "0T180ZA",  # Bypass Bilateral Ureters to Ileum, Open Approach
        "0T180ZB",  # Bypass Bilateral Ureters to Bladder, Open Approach
        "0T180ZC",  # Bypass Bilateral Ureters to Ileocutaneous, Open Approach
        "0T1B079",  # Bypass Bladder to Colocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B07C",  # Bypass Bladder to Ileocutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B07D",  # Bypass Bladder to Cutaneous with Autologous Tissue Substitute, Open Approach
        "0T1B0J9",  # Bypass Bladder to Colocutaneous with Synthetic Substitute, Open Approach
        "0T1B0JC",  # Bypass Bladder to Ileocutaneous with Synthetic Substitute, Open Approach
        "0T1B0JD",  # Bypass Bladder to Cutaneous with Synthetic Substitute, Open Approach
        "0T1B0K9",  # Bypass Bladder to Colocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0KC",  # Bypass Bladder to Ileocutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0KD",  # Bypass Bladder to Cutaneous with Nonautologous Tissue Substitute, Open Approach
        "0T1B0Z9",  # Bypass Bladder to Colocutaneous, Open Approach
        "0T1B0ZC",  # Bypass Bladder to Ileocutaneous, Open Approach
        "0T7B0DZ",  # Dilation of Bladder with Intraluminal Device, Open Approach
        "0T7B0ZZ",  # Dilation of Bladder, Open Approach
        "0T7B7DZ",  # Dilation of Bladder with Intraluminal Device, Via Natural or Artificial Opening
        "0T7B7ZZ",  # Dilation of Bladder, Via Natural or Artificial Opening
        "0T9300Z",  # Drainage of Right Kidney Pelvis with Drainage Device, Open Approach
        "0T930ZZ",  # Drainage of Right Kidney Pelvis, Open Approach
        "0T9400Z",  # Drainage of Left Kidney Pelvis with Drainage Device, Open Approach
        "0T940ZZ",  # Drainage of Left Kidney Pelvis, Open Approach
        "0TB00ZZ",  # Excision of Right Kidney, Open Approach
        "0TB07ZZ",  # Excision of Right Kidney, Via Natural or Artificial Opening
        "0TB10ZZ",  # Excision of Left Kidney, Open Approach
        "0TB17ZZ",  # Excision of Left Kidney, Via Natural or Artificial Opening
        "0TB30ZZ",  # Excision of Right Kidney Pelvis, Open Approach
        "0TB37ZZ",  # Excision of Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TB40ZZ",  # Excision of Left Kidney Pelvis, Open Approach
        "0TB47ZZ",  # Excision of Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TBB0ZZ",  # Excision of Bladder, Open Approach
        "0TBC0ZZ",  # Excision of Bladder Neck, Open Approach
        "0TJ50ZZ",  # Inspection of Kidney, Open Approach
        "0TJ90ZZ",  # Inspection of Ureter, Open Approach
        "0TJB0ZZ",  # Inspection of Bladder, Open Approach
        "0TJD0ZZ",  # Inspection of Urethra, Open Approach
        "0TMB0ZZ",  # Reattachment of Bladder, Open Approach
        "0TMC0ZZ",  # Reattachment of Bladder Neck, Open Approach
        "0TN00ZZ",  # Release Right Kidney, Open Approach
        "0TN07ZZ",  # Release Right Kidney, Via Natural or Artificial Opening
        "0TN10ZZ",  # Release Left Kidney, Open Approach
        "0TN17ZZ",  # Release Left Kidney, Via Natural or Artificial Opening
        "0TN30ZZ",  # Release Right Kidney Pelvis, Open Approach
        "0TN37ZZ",  # Release Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TN40ZZ",  # Release Left Kidney Pelvis, Open Approach
        "0TN47ZZ",  # Release Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TN60ZZ",  # Release Right Ureter, Open Approach
        "0TN70ZZ",  # Release Left Ureter, Open Approach
        "0TQ60ZZ",  # Repair Right Ureter, Open Approach
        "0TQ67ZZ",  # Repair Right Ureter, Via Natural or Artificial Opening
        "0TQ70ZZ",  # Repair Left Ureter, Open Approach
        "0TQ77ZZ",  # Repair Left Ureter, Via Natural or Artificial Opening
        "0TQB0ZZ",  # Repair Bladder, Open Approach
        "0TQB7ZZ",  # Repair Bladder, Via Natural or Artificial Opening
        "0TQC0ZZ",  # Repair Bladder Neck, Open Approach
        "0TQC7ZZ",  # Repair Bladder Neck, Via Natural or Artificial Opening
        "0TRB07Z",  # Replacement of Bladder with Autologous Tissue Substitute, Open Approach
        "0TRB0JZ",  # Replacement of Bladder with Synthetic Substitute, Open Approach
        "0TRB0KZ",  # Replacement of Bladder with Nonautologous Tissue Substitute, Open Approach
        "0TRB77Z",  # Replacement of Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRB7JZ",  # Replacement of Bladder with Synthetic Substitute, Via Natural or Artificial Opening
        "0TRB7KZ",  # Replacement of Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRC07Z",  # Replacement of Bladder Neck with Autologous Tissue Substitute, Open Approach
        "0TRC0JZ",  # Replacement of Bladder Neck with Synthetic Substitute, Open Approach
        "0TRC0KZ",  # Replacement of Bladder Neck with Nonautologous Tissue Substitute, Open Approach
        "0TRC77Z",  # Replacement of Bladder Neck with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TRC7JZ",  # Replacement of Bladder Neck with Synthetic Substitute, Via Natural or Artificial Opening
        "0TRC7KZ",  # Replacement of Bladder Neck with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TSB0ZZ",  # Reposition Bladder, Open Approach
        "0TT00ZZ",  # Resection of Right Kidney, Open Approach
        "0TT10ZZ",  # Resection of Left Kidney, Open Approach
        "0TT20ZZ",  # Resection of Bilateral Kidneys, Open Approach
        "0TT30ZZ",  # Resection of Right Kidney Pelvis, Open Approach
        "0TT37ZZ",  # Resection of Right Kidney Pelvis, Via Natural or Artificial Opening
        "0TT40ZZ",  # Resection of Left Kidney Pelvis, Open Approach
        "0TT47ZZ",  # Resection of Left Kidney Pelvis, Via Natural or Artificial Opening
        "0TTB0ZZ",  # Resection of Bladder, Open Approach
        "0TTB4ZZ",  # Resection of Bladder, Percutaneous Endoscopic Approach
        "0TTB7ZZ",  # Resection of Bladder, Via Natural or Artificial Opening
        "0TTB8ZZ",  # Resection of Bladder, Via Natural or Artificial Opening Endoscopic
        "0TTC0ZZ",  # Resection of Bladder Neck, Open Approach
        "0TTC4ZZ",  # Resection of Bladder Neck, Percutaneous Endoscopic Approach
        "0TTC7ZZ",  # Resection of Bladder Neck, Via Natural or Artificial Opening
        "0TTC8ZZ",  # Resection of Bladder Neck, Via Natural or Artificial Opening Endoscopic
        "0TTD0ZZ",  # Resection of Urethra, Open Approach
        "0TTD7ZZ",  # Resection of Urethra, Via Natural or Artificial Opening
        "0TUB07Z",  # Supplement Bladder with Autologous Tissue Substitute, Open Approach
        "0TUB0JZ",  # Supplement Bladder with Synthetic Substitute, Open Approach
        "0TUB0KZ",  # Supplement Bladder with Nonautologous Tissue Substitute, Open Approach
        "0TUB77Z",  # Supplement Bladder with Autologous Tissue Substitute, Via Natural or Artificial Opening
        "0TUB7JZ",  # Supplement Bladder with Synthetic Substitute, Via Natural or Artificial Opening
        "0TUB7KZ",  # Supplement Bladder with Nonautologous Tissue Substitute, Via Natural or Artificial Opening
        "0TVB0CZ",  # Restriction of Bladder with Extraluminal Device, Open Approach
        "0TVB0DZ",  # Restriction of Bladder with Intraluminal Device, Open Approach
        "0TVB0ZZ",  # Restriction of Bladder, Open Approach
        "0TVB7DZ",  # Restriction of Bladder with Intraluminal Device, Via Natural or Artificial Opening
        "0TVB7ZZ",  # Restriction of Bladder, Via Natural or Artificial Opening
        "0TVC0CZ",  # Restriction of Bladder Neck with Extraluminal Device, Open Approach
        "0TVC0DZ",  # Restriction of Bladder Neck with Intraluminal Device, Open Approach
        "0TVC0ZZ",  # Restriction of Bladder Neck, Open Approach
        "0TVC7DZ",  # Restriction of Bladder Neck with Intraluminal Device, Via Natural or Artificial Opening
        "0TVC7ZZ",  # Restriction of Bladder Neck, Via Natural or Artificial Opening
        "0U500ZZ",  # Destruction of Right Ovary, Open Approach
        "0U510ZZ",  # Destruction of Left Ovary, Open Approach
        "0U520ZZ",  # Destruction of Bilateral Ovaries, Open Approach
        "0U550ZZ",  # Destruction of Right Fallopian Tube, Open Approach
        "0U560ZZ",  # Destruction of Left Fallopian Tube, Open Approach
        "0U570ZZ",  # Destruction of Bilateral Fallopian Tubes, Open Approach
        "0U590ZZ",  # Destruction of Uterus, Open Approach
        "0U5B0ZZ",  # Destruction of Endometrium, Open Approach
        "0U800ZZ",  # Division of Right Ovary, Open Approach
        "0U810ZZ",  # Division of Left Ovary, Open Approach
        "0U820ZZ",  # Division of Bilateral Ovaries, Open Approach
        "0UB00ZZ",  # Excision of Right Ovary, Open Approach
        "0UB07ZZ",  # Excision of Right Ovary, Via Natural or Artificial Opening
        "0UB10ZZ",  # Excision of Left Ovary, Open Approach
        "0UB17ZZ",  # Excision of Left Ovary, Via Natural or Artificial Opening
        "0UB20ZZ",  # Excision of Bilateral Ovaries, Open Approach
        "0UB27ZZ",  # Excision of Bilateral Ovaries, Via Natural or Artificial Opening
        "0UB50ZZ",  # Excision of Right Fallopian Tube, Open Approach
        "0UB60ZZ",  # Excision of Left Fallopian Tube, Open Approach
        "0UB70ZZ",  # Excision of Bilateral Fallopian Tubes, Open Approach
        "0UJ30ZZ",  # Inspection of Ovary, Open Approach
        "0UJ80ZZ",  # Inspection of Fallopian Tube, Open Approach
        "0UJD0ZZ",  # Inspection of Uterus and Cervix, Open Approach
        "0UJM0ZZ",  # Inspection of Vulva, Open Approach
        "0UQM0ZZ",  # Repair Vulva, Open Approach
        "0UT00ZZ",  # Resection of Right Ovary, Open Approach
        "0UT04ZZ",  # Resection of Right Ovary, Percutaneous Endoscopic Approach
        "0UT07ZZ",  # Resection of Right Ovary, Via Natural or Artificial Opening
        "0UT08ZZ",  # Resection of Right Ovary, Via Natural or Artificial Opening Endoscopic
        "0UT0FZZ",  # Resection of Right Ovary, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT10ZZ",  # Resection of Left Ovary, Open Approach
        "0UT14ZZ",  # Resection of Left Ovary, Percutaneous Endoscopic Approach
        "0UT17ZZ",  # Resection of Left Ovary, Via Natural or Artificial Opening
        "0UT18ZZ",  # Resection of Left Ovary, Via Natural or Artificial Opening Endoscopic
        "0UT1FZZ",  # Resection of Left Ovary, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT20ZZ",  # Resection of Bilateral Ovaries, Open Approach
        "0UT24ZZ",  # Resection of Bilateral Ovaries, Percutaneous Endoscopic Approach
        "0UT27ZZ",  # Resection of Bilateral Ovaries, Via Natural or Artificial Opening
        "0UT28ZZ",  # Resection of Bilateral Ovaries, Via Natural or Artificial Opening Endoscopic
        "0UT2FZZ",  # Resection of Bilateral Ovaries, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT40ZZ",  # Resection of Uterine Supporting Structure, Open Approach
        "0UT47ZZ",  # Resection of Uterine Supporting Structure, Via Natural or Artificial Opening
        "0UT50ZZ",  # Resection of Right Fallopian Tube, Open Approach
        "0UT54ZZ",  # Resection of Right Fallopian Tube, Percutaneous Endoscopic Approach
        "0UT57ZZ",  # Resection of Right Fallopian Tube, Via Natural or Artificial Opening
        "0UT58ZZ",  # Resection of Right Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UT5FZZ",  # Resection of Right Fallopian Tube, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT60ZZ",  # Resection of Left Fallopian Tube, Open Approach
        "0UT64ZZ",  # Resection of Left Fallopian Tube, Percutaneous Endoscopic Approach
        "0UT67ZZ",  # Resection of Left Fallopian Tube, Via Natural or Artificial Opening
        "0UT68ZZ",  # Resection of Left Fallopian Tube, Via Natural or Artificial Opening Endoscopic
        "0UT6FZZ",  # Resection of Left Fallopian Tube, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT70ZZ",  # Resection of Bilateral Fallopian Tubes, Open Approach
        "0UT74ZZ",  # Resection of Bilateral Fallopian Tubes, Percutaneous Endoscopic Approach
        "0UT77ZZ",  # Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening
        "0UT78ZZ",  # Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening Endoscopic
        "0UT7FZZ",  # Resection of Bilateral Fallopian Tubes, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT90ZL",  # Resection of Uterus, Supracervical, Open Approach
        "0UT90ZZ",  # Resection of Uterus, Open Approach
        "0UT94ZL",  # Resection of Uterus, Supracervical, Percutaneous Endoscopic Approach
        "0UT94ZZ",  # Resection of Uterus, Percutaneous Endoscopic Approach
        "0UT97ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening
        "0UT97ZZ",  # Resection of Uterus, Via Natural or Artificial Opening
        "0UT98ZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening Endoscopic
        "0UT98ZZ",  # Resection of Uterus, Via Natural or Artificial Opening Endoscopic
        "0UT9FZL",  # Resection of Uterus, Supracervical, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UT9FZZ",  # Resection of Uterus, Via Natural or Artificial Opening With Percutaneous Endoscopic Assistance
        "0UTC0ZZ",  # Resection of Cervix, Open Approach
        "0UTC7ZZ",  # Resection of Cervix, Via Natural or Artificial Opening
        "0VBC0ZX",  # Excision of Bilateral Testes, Open Approach, Diagnostic
        "0VBC0ZZ",  # Excision of Bilateral Testes, Open Approach
        "0VBC3ZZ",  # Excision of Bilateral Testes, Percutaneous Approach
        "0VBC4ZZ",  # Excision of Bilateral Testes, Percutaneous Endoscopic Approach
        "0VJD0ZZ",  # Inspection of Testis, Open Approach
        "0VJR0ZZ",  # Inspection of Vas Deferens, Open Approach
        "0VJS0ZZ",  # Inspection of Penis, Open Approach
        "0VT00ZZ",  # Resection of Prostate, Open Approach
        "0VT04ZZ",  # Resection of Prostate, Percutaneous Endoscopic Approach
        "0VT07ZZ",  # Resection of Prostate, Via Natural or Artificial Opening
        "0VT08ZZ",  # Resection of Prostate, Via Natural or Artificial Opening Endoscopic
        "0VT30ZZ",  # Resection of Bilateral Seminal Vesicles, Open Approach
        "0VTC0ZZ",  # Resection of Bilateral Testes, Open Approach
        "0VTC4ZZ",  # Resection of Bilateral Testes, Percutaneous Endoscopic Approach
        "0W1G0JY",  # Bypass Peritoneal Cavity to Lower Vein with Synthetic Substitute, Open Approach
        "0W3G0ZZ",  # Control Bleeding in Peritoneal Cavity, Open Approach
        "0W3H0ZZ",  # Control Bleeding in Retroperitoneum, Open Approach
        "0W3P0ZZ",  # Control Bleeding in Gastrointestinal Tract, Open Approach
        "0W9100Z",  # Drainage of Cranial Cavity with Drainage Device, Open Approach
        "0W910ZZ",  # Drainage of Cranial Cavity, Open Approach
        "0W9G00Z",  # Drainage of Peritoneal Cavity with Drainage Device, Open Approach
        "0W9G0ZZ",  # Drainage of Peritoneal Cavity, Open Approach
        "0W9J00Z",  # Drainage of Pelvic Cavity with Drainage Device, Open Approach
        "0W9J0ZZ",  # Drainage of Pelvic Cavity, Open Approach
        "0WBF0ZZ",  # Excision of Abdominal Wall, Open Approach
        "0WBF3ZZ",  # Excision of Abdominal Wall, Percutaneous Approach
        "0WBF4ZZ",  # Excision of Abdominal Wall, Percutaneous Endoscopic Approach
        "0WBH0ZZ",  # Excision of Retroperitoneum, Open Approach
        "0WC10ZZ",  # Extirpation of Matter from Cranial Cavity, Open Approach
        "0WCG0ZZ",  # Extirpation of Matter from Peritoneal Cavity, Open Approach
        "0WCJ0ZZ",  # Extirpation of Matter from Pelvic Cavity, Open Approach
        "0WCP0ZZ",  # Extirpation of Matter from Gastrointestinal Tract, Open Approach
        "0WCR0ZZ",  # Extirpation of Matter from Genitourinary Tract, Open Approach
        "0WFD0ZZ",  # Fragmentation in Pericardial Cavity, Open Approach
        "0WFDXZZ",  # Fragmentation in Pericardial Cavity, External Approach
        "0WFG0ZZ",  # Fragmentation in Peritoneal Cavity, Open Approach
        "0WH10YZ",  # Insertion of Other Device into Cranial Cavity, Open Approach
        "0WHG03Z",  # Insertion of Infusion Device into Peritoneal Cavity, Open Approach
        "0WHG0YZ",  # Insertion of Other Device into Peritoneal Cavity, Open Approach
        "0WHH03Z",  # Insertion of Infusion Device into Retroperitoneum, Open Approach
        "0WHH0YZ",  # Insertion of Other Device into Retroperitoneum, Open Approach
        "0WHJ03Z",  # Insertion of Infusion Device into Pelvic Cavity, Open Approach
        "0WHJ0YZ",  # Insertion of Other Device into Pelvic Cavity, Open Approach
        "0WHP03Z",  # Insertion of Infusion Device into Gastrointestinal Tract, Open Approach
        "0WJ00ZZ",  # Inspection of Head, Open Approach
        "0WJ10ZZ",  # Inspection of Cranial Cavity, Open Approach
        "0WJ30ZZ",  # Inspection of Oral Cavity and Throat, Open Approach
        "0WJ40ZZ",  # Inspection of Upper Jaw, Open Approach
        "0WJ50ZZ",  # Inspection of Lower Jaw, Open Approach
        "0WJ60ZZ",  # Inspection of Neck, Open Approach
        "0WJ80ZZ",  # Inspection of Chest Wall, Open Approach
        "0WJ90ZZ",  # Inspection of Right Pleural Cavity, Open Approach
        "0WJB0ZZ",  # Inspection of Left Pleural Cavity, Open Approach
        "0WJC0ZZ",  # Inspection of Mediastinum, Open Approach
        "0WJD0ZZ",  # Inspection of Pericardial Cavity, Open Approach
        "0WJF0ZZ",  # Inspection of Abdominal Wall, Open Approach
        "0WJG0ZZ",  # Inspection of Peritoneal Cavity, Open Approach
        "0WJH0ZZ",  # Inspection of Retroperitoneum, Open Approach
        "0WJJ0ZZ",  # Inspection of Pelvic Cavity, Open Approach
        "0WJM0ZZ",  # Inspection of Male Perineum, Open Approach
        "0WJN0ZZ",  # Inspection of Female Perineum, Open Approach
        "0WJP0ZZ",  # Inspection of Gastrointestinal Tract, Open Approach
        "0WJQ0ZZ",  # Inspection of Respiratory Tract, Open Approach
        "0WJR0ZZ",  # Inspection of Genitourinary Tract, Open Approach
        "0WMF0ZZ",  # Reattachment of Abdominal Wall, Open Approach
        "0WP100Z",  # Removal of Drainage Device from Cranial Cavity, Open Approach
        "0WP101Z",  # Removal of Radioactive Element from Cranial Cavity, Open Approach
        "0WP10JZ",  # Removal of Synthetic Substitute from Cranial Cavity, Open Approach
        "0WP10YZ",  # Removal of Other Device from Cranial Cavity, Open Approach
        "0WPG00Z",  # Removal of Drainage Device from Peritoneal Cavity, Open Approach
        "0WPG01Z",  # Removal of Radioactive Element from Peritoneal Cavity, Open Approach
        "0WPG03Z",  # Removal of Infusion Device from Peritoneal Cavity, Open Approach
        "0WPG0JZ",  # Removal of Synthetic Substitute from Peritoneal Cavity, Open Approach
        "0WPG0YZ",  # Removal of Other Device from Peritoneal Cavity, Open Approach
        "0WPH00Z",  # Removal of Drainage Device from Retroperitoneum, Open Approach
        "0WPH01Z",  # Removal of Radioactive Element from Retroperitoneum, Open Approach
        "0WPH03Z",  # Removal of Infusion Device from Retroperitoneum, Open Approach
        "0WPH0YZ",  # Removal of Other Device from Retroperitoneum, Open Approach
        "0WPP01Z",  # Removal of Radioactive Element from Gastrointestinal Tract, Open Approach
        "0WPP03Z",  # Removal of Infusion Device from Gastrointestinal Tract, Open Approach
        "0WPP0YZ",  # Removal of Other Device from Gastrointestinal Tract, Open Approach
        "0WQF0ZZ",  # Repair Abdominal Wall, Open Approach
        "0WQN0ZZ",  # Repair Female Perineum, Open Approach
        "0WQN3ZZ",  # Repair Female Perineum, Percutaneous Approach
        "0WQN4ZZ",  # Repair Female Perineum, Percutaneous Endoscopic Approach
        "0WW100Z",  # Revision of Drainage Device in Cranial Cavity, Open Approach
        "0WW101Z",  # Revision of Radioactive Element in Cranial Cavity, Open Approach
        "0WW103Z",  # Revision of Infusion Device in Cranial Cavity, Open Approach
        "0WW10JZ",  # Revision of Synthetic Substitute in Cranial Cavity, Open Approach
        "0WW10YZ",  # Revision of Other Device in Cranial Cavity, Open Approach
        "0WWG00Z",  # Revision of Drainage Device in Peritoneal Cavity, Open Approach
        "0WWG01Z",  # Revision of Radioactive Element in Peritoneal Cavity, Open Approach
        "0WWG03Z",  # Revision of Infusion Device in Peritoneal Cavity, Open Approach
        "0WWG0JZ",  # Revision of Synthetic Substitute in Peritoneal Cavity, Open Approach
        "0WWG0YZ",  # Revision of Other Device in Peritoneal Cavity, Open Approach
        "0WWH00Z",  # Revision of Drainage Device in Retroperitoneum, Open Approach
        "0WWH01Z",  # Revision of Radioactive Element in Retroperitoneum, Open Approach
        "0WWH03Z",  # Revision of Infusion Device in Retroperitoneum, Open Approach
        "0WWH0YZ",  # Revision of Other Device in Retroperitoneum, Open Approach
        "0WWJ00Z",  # Revision of Drainage Device in Pelvic Cavity, Open Approach
        "0WWJ01Z",  # Revision of Radioactive Element in Pelvic Cavity, Open Approach
        "0WWJ03Z",  # Revision of Infusion Device in Pelvic Cavity, Open Approach
        "0WWJ0JZ",  # Revision of Synthetic Substitute in Pelvic Cavity, Open Approach
        "0WWJ0YZ",  # Revision of Other Device in Pelvic Cavity, Open Approach
        "0WWP01Z",  # Revision of Radioactive Element in Gastrointestinal Tract, Open Approach
        "0WWP03Z",  # Revision of Infusion Device in Gastrointestinal Tract, Open Approach
        "0WWP0YZ",  # Revision of Other Device in Gastrointestinal Tract, Open Approach
        "0X600ZZ",  # Detachment at Right Forequarter, Open Approach
        "0X610ZZ",  # Detachment at Left Forequarter, Open Approach
        "0X620ZZ",  # Detachment at Right Shoulder Region, Open Approach
        "0X630ZZ",  # Detachment at Left Shoulder Region, Open Approach
        "0X680Z1",  # Detachment at Right Upper Arm, High, Open Approach
        "0X680Z2",  # Detachment at Right Upper Arm, Mid, Open Approach
        "0X680Z3",  # Detachment at Right Upper Arm, Low, Open Approach
        "0X690Z1",  # Detachment at Left Upper Arm, High, Open Approach
        "0X690Z2",  # Detachment at Left Upper Arm, Mid, Open Approach
        "0X690Z3",  # Detachment at Left Upper Arm, Low, Open Approach
        "0X6B0ZZ",  # Detachment at Right Elbow Region, Open Approach
        "0X6C0ZZ",  # Detachment at Left Elbow Region, Open Approach
        "0X6D0Z1",  # Detachment at Right Lower Arm, High, Open Approach
        "0X6D0Z2",  # Detachment at Right Lower Arm, Mid, Open Approach
        "0X6D0Z3",  # Detachment at Right Lower Arm, Low, Open Approach
        "0X6F0Z1",  # Detachment at Left Lower Arm, High, Open Approach
        "0X6F0Z2",  # Detachment at Left Lower Arm, Mid, Open Approach
        "0X6F0Z3",  # Detachment at Left Lower Arm, Low, Open Approach
        "0X6J0Z0",  # Detachment at Right Hand, Complete, Open Approach
        "0X6K0Z0",  # Detachment at Left Hand, Complete, Open Approach
        "0Y620ZZ",  # Detachment at Right Hindquarter, Open Approach
        "0Y630ZZ",  # Detachment at Left Hindquarter, Open Approach
        "0Y640ZZ",  # Detachment at Bilateral Hindquarter, Open Approach
        "0Y670ZZ",  # Detachment at Right Femoral Region, Open Approach
        "0Y680ZZ",  # Detachment at Left Femoral Region, Open Approach
        "0Y6C0Z1",  # Detachment at Right Upper Leg, High, Open Approach
        "0Y6C0Z2",  # Detachment at Right Upper Leg, Mid, Open Approach
        "0Y6C0Z3",  # Detachment at Right Upper Leg, Low, Open Approach
        "0Y6D0Z1",  # Detachment at Left Upper Leg, High, Open Approach
        "0Y6D0Z2",  # Detachment at Left Upper Leg, Mid, Open Approach
        "0Y6D0Z3",  # Detachment at Left Upper Leg, Low, Open Approach
        "0Y6F0ZZ",  # Detachment at Right Knee Region, Open Approach
        "0Y6G0ZZ",  # Detachment at Left Knee Region, Open Approach
        "0Y6H0Z1",  # Detachment at Right Lower Leg, High, Open Approach
        "0Y6H0Z2",  # Detachment at Right Lower Leg, Mid, Open Approach
        "0Y6H0Z3",  # Detachment at Right Lower Leg, Low, Open Approach
        "0Y6J0Z1",  # Detachment at Left Lower Leg, High, Open Approach
        "0Y6J0Z2",  # Detachment at Left Lower Leg, Mid, Open Approach
        "0Y6J0Z3",  # Detachment at Left Lower Leg, Low, Open Approach
        "0Y6M0Z0",  # Detachment at Right Foot, Complete, Open Approach
        "0Y6N0Z0",  # Detachment at Left Foot, Complete, Open Approach
        "0YJ50ZZ",  # Inspection of Right Inguinal Region, Open Approach
        "0YJ60ZZ",  # Inspection of Left Inguinal Region, Open Approach
        "0YJ70ZZ",  # Inspection of Right Femoral Region, Open Approach
        "0YJ80ZZ",  # Inspection of Left Femoral Region, Open Approach
        "0YJA0ZZ",  # Inspection of Bilateral Inguinal Region, Open Approach
        "0YJE0ZZ",  # Inspection of Bilateral Femoral Region, Open Approach
        "10J00ZZ",  # Inspection of Products of Conception, Open Approach
        "10J10ZZ",  # Inspection of Products of Conception, Retained, Open Approach
        "10J20ZZ",  # Inspection of Products of Conception, Ectopic, Open Approach
        "10T20ZZ",  # Resection of Products of Conception, Ectopic, Open Approach
        "10T27ZZ",  # Resection of Products of Conception, Ectopic, Via Natural or Artificial Opening
        "5A02116",  # Assistance with Cardiac Output using Other Pump, Intermittent
        "5A02216",  # Assistance with Cardiac Output using Other Pump, Continuous
    }

    SNOMEDCT = {
        "10002003",  # Resection of stomach fundus (procedure)
        "10088002",  # Revision of jejunoileal bypass (procedure)
        "1010191006",  # Bisection of left ovary (procedure)
        "1010192004",  # Bisection of right ovary (procedure)
        "1010291002",  # Anastomosis of left internal thoracic artery to left anterior descending coronary artery (procedure)
        "1010433004",  # Wedge resection of right ovary (procedure)
        "1010434005",  # Wedge resection of left ovary (procedure)
        "1010436007",  # Destruction of left fallopian tube (procedure)
        "1010437003",  # Destruction of right fallopian tube (procedure)
        "1010570006",  # Sequential anastomosis of left internal thoracic artery from second obtuse marginal branch of circumflex coronary artery to left anterior descending coronary artery (procedure)
        "1010571005",  # Sequential anastomosis of left internal thoracic artery from first obtuse marginal branch of circumflex coronary artery to left anterior descending coronary artery (procedure)
        "1010572003",  # Sequential anastomosis of left internal thoracic artery from ramus intermedius branch of left coronary artery to left anterior descending coronary artery (procedure)
        "1010688007",  # Anastomosis of left internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "10107006",  # Duodenoduodenostomy (procedure)
        "1010714005",  # Excision of left uterine adnexa (procedure)
        "1010715006",  # Excision of right uterine adnexa (procedure)
        "1017203005",  # Anastomosis of left internal thoracic artery to intermediate artery (procedure)
        "1017205003",  # Anastomosis of left internal thoracic artery to obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1017216005",  # Laparoscopic total excision of right kidney (procedure)
        "1017217001",  # Laparoscopic total excision of left kidney (procedure)
        "1017218006",  # Laparoscopic total excision of left kidney via retroperitoneal approach (procedure)
        "1017219003",  # Laparoscopic total excision of right kidney via retroperitoneal approach (procedure)
        "1017227007",  # Sequential anastomosis of left internal thoracic artery from second diagonal branch of anterior descending branch of left coronary artery to left anterior descending artery (procedure)
        "1017228002",  # Sequential anastomosis of left internal thoracic artery from first diagonal branch of anterior descending branch of left coronary artery to left anterior descending artery (procedure)
        "1017229005",  # Sequential anastomosis of left internal thoracic artery from proximal portion of anterior descending branch of left coronary artery to mid portion of anterior descending branch of left coronary artery (procedure)
        "1017232008",  # Sequential anastomosis of left internal thoracic artery from proximal portion of anterior descending branch of left coronary artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "1017233003",  # Sequential anastomosis of left internal thoracic artery from mid portion of anterior descending branch of left coronary artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "10190003",  # Aortocoronary bypass of four or more coronary arteries (procedure)
        "10220003",  # Subclavian-axillary artery bypass graft with vein (procedure)
        "10254002",  # Excision of diverticulum of small intestine (procedure)
        "10259007",  # Proctopexy (procedure)
        "10272008",  # Closure of fistula of diaphragm (procedure)
        "10326007",  # Coronary artery bypass with autogenous graft, three grafts (procedure)
        "10359005",  # Revision of obstructed valve in cerebrospinal fluid shunt system (procedure)
        "103727006",  # Vestibular nerve section (procedure)
        "10388006",  # Manipulation of tissue of diaphragm (procedure)
        "1054002",  # Incision of sphincter of Oddi (procedure)
        "10541000087107",  # Excision of left hemidiaphragm (procedure)
        "10591000087100",  # Excision of right hemidiaphragm (procedure)
        "10594004",  # Excision of hydrocele from bilateral sides of scrotum (procedure)
        "10615008",  # Atrial aneurysmectomy (procedure)
        "10671000087103",  # Ligation of right internal iliac artery (procedure)
        "10683007",  # Destruction of lesion of heart (procedure)
        "1071001",  # Proximal splenorenal anastomosis (procedure)
        "10751000087103",  # Ligation of left internal iliac artery (procedure)
        "107936001",  # Operative procedure on small intestine (procedure)
        "107938000",  # Small intestine excision (procedure)
        "107943007",  # Small intestine manipulation (procedure)
        "107944001",  # Large intestine excision (procedure)
        "10795000",  # Ventriculocaval shunt with valve (procedure)
        "107963000",  # Excision of liver (procedure)
        "108022006",  # Kidney excision (procedure)
        "108025008",  # Kidney destructive procedure (procedure)
        "108034003",  # Excision of urinary bladder (procedure)
        "108184008",  # Diaphragm excision (procedure)
        "1084005",  # Excision of perinephric cyst (procedure)
        "10869008",  # Hemigastrectomy with vagotomy (procedure)
        "10870009",  # Repair of atrial septal defect in total repair of total anomalous pulmonary venous connection (procedure)
        "10871008",  # Segmental osteotomy of maxilla (procedure)
        "10985002",  # Gallbladder to intestine anastomosis (procedure)
        "110470001",  # Amputation above-knee, mid-thigh (procedure)
        "11050006",  # Closure of vesicouterine fistula with hysterectomy (procedure)
        "11127003",  # Esophagogastrostomy, antesternal or antethoracic (procedure)
        "11171000087101",  # Stripping of right hemidiaphragm (procedure)
        "11181000087104",  # Stripping of left hemidiaphragm (procedure)
        "1119317007",  # Laparoscopic excision of part of left kidney (procedure)
        "1119318002",  # Laparoscopic excision of part of right kidney (procedure)
        "1119319005",  # Laparoscopic excision of part of left kidney via retroperitoneal approach (procedure)
        "1119320004",  # Laparoscopic excision of part of right kidney via retroperitoneal approach (procedure)
        "1119419009",  # Radical dissection of left half of neck (procedure)
        "1119420003",  # Radical dissection of right half of neck (procedure)
        "1119421004",  # Modified radical dissection of left half of neck (procedure)
        "1119422006",  # Modified radical dissection of right half of neck (procedure)
        "1119427000",  # Open transposition of left subclavian artery to carotid artery with endovascular repair of descending thoracic aorta by neck incision (procedure)
        "1119428005",  # Open transposition of right subclavian artery to carotid artery with endovascular repair of descending thoracic aorta by neck incision (procedure)
        "1119432004",  # Partial glossectomy with radical dissection of left half of neck (procedure)
        "1119433009",  # Partial glossectomy with radical dissection of right half of neck (procedure)
        "11228000",  # Dieffenbach operation for hip disarticulation (procedure)
        "11236009",  # Near-total pancreatectomy with preservation of duodenum (procedure)
        "11242008",  # Aneurysmectomy with graft replacement of thoracic artery (procedure)
        "112742008",  # Osteotomy of clavicle with internal fixation (procedure)
        "112809000",  # Division of chordae tendineae (procedure)
        "112811009",  # Closure of atrial septal defect (procedure)
        "112812002",  # Division of common wall between posterior left atrium and coronary sinus with roofing of resultant defect and patch graft (procedure)
        "112813007",  # Reimplantation of pulmonary artery for hemitruncus repair (procedure)
        "112814001",  # Closed heart valvotomy of aortic valve (procedure)
        "112815000",  # Implantation of heart valve (procedure)
        "112817008",  # Resuture of heart valve prosthesis, poppet (procedure)
        "112818003",  # Division of descending aorta (procedure)
        "112819006",  # Arteriectomy with graft replacement of thoracoabdominal aorta (procedure)
        "112821001",  # Thrombectomy with catheter of aortoiliac artery by leg incision (procedure)
        "112823003",  # Creation of aorta-carotid-brachial vascular bypass (procedure)
        "112825005",  # Intracranial arteriectomy with anastomosis (procedure)
        "112826006",  # Excision of intracranial artery (procedure)
        "112827002",  # Excision of lower limb artery (procedure)
        "112828007",  # Femoral-popliteal artery bypass graft (procedure)
        "112860004",  # Hemigastrectomy by abdominal approach (procedure)
        "112864008",  # Incision and exploration of jejunum (procedure)
        "112868006",  # Ileostomy transplantation to new site (procedure)
        "112871003",  # Anastomosis of large intestine to large intestine (procedure)
        "112872005",  # Suture of large intestine (procedure)
        "112873000",  # Closure of perineosigmoidal fistula (procedure)
        "112877004",  # Frickman operation, abdominal proctopexy (procedure)
        "112883001",  # Removal of foreign body from liver (procedure)
        "112884007",  # Cholecystoduodenostomy (procedure)
        "112888005",  # Incision and exploration of pineal gland (procedure)
        "112892003",  # Radical excision of iliac lymph nodes (procedure)
        "112896000",  # Replacement of mechanical kidney (procedure)
        "112902005",  # Partial urinary cystectomy (procedure)
        "112903000",  # Excision of perivesical tissue (procedure)
        "112910006",  # Excision of entire bilateral undescended testes (procedure)
        "112918004",  # Vaginal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "112929000",  # Exploratory craniotomy, supratentorial (procedure)
        "112939006",  # Ventricular shunt to head or neck structure (procedure)
        "112972006",  # Mastoidectomy, cortical, conservative (procedure)
        "112977000",  # Repair of diaphragmatic hernia (procedure)
        "112981000",  # Incision and drainage of subhepatic space (procedure)
        "11346008",  # Thrombectomy with catheter of vena cava by abdominal and leg incision (procedure)
        "11347004",  # Modified radical neck dissection (procedure)
        "11377005",  # Direct laryngoscopy with excision of tumor (procedure)
        "1144644003",  # Anastomosis of right internal thoracic artery to anterior descending branch of left coronary artery (procedure)
        "1144645002",  # Anastomosis of right internal thoracic artery to obtuse marginal branch of circumflex branch of coronary artery (procedure)
        "1144646001",  # Anastomosis of right internal thoracic artery to marginal branch of right coronary artery (procedure)
        "1144647005",  # Anastomosis of right internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1144889004",  # Anastomosis of right internal thoracic artery to intermediate artery (procedure)
        "1144890008",  # Anastomosis of right internal thoracic artery to posterior descending artery (procedure)
        "1145198009",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1145199001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to anterior descending branch of left coronary artery (procedure)
        "1145200003",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to obtuse marginal branch of circumflex branch of coronary artery (procedure)
        "1148601000000109",  # Percutaneous microwave ablation of lesion of left kidney using computed tomography guidance (procedure)
        "1148611000000106",  # Percutaneous microwave ablation of lesion of right kidney using computed tomography guidance (procedure)
        "1148690000",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to intermediate artery (procedure)
        "1148691001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to posterior descending artery (procedure)
        "1148706009",  # Transsphenoidal partial excision of neoplasm of pituitary gland (procedure)
        "1148707000",  # Transsphenoidal total excision of neoplasm of pituitary gland (procedure)
        "1148721000000106",  # Percutaneous microwave ablation of lesion of bilateral kidneys using computed tomography guidance (procedure)
        "1149059007",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to marginal branch of right coronary artery (procedure)
        "1149060002",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1149127003",  # Endoscopic endonasal transsphenoidal total excision of neoplasm of pituitary gland (procedure)
        "1149128008",  # Endoscopic endonasal transsphenoidal partial excision of neoplasm of pituitary gland (procedure)
        "11516009",  # Aneurysmectomy with graft replacement of abdominal aorta (procedure)
        "11519002",  # Closure of bronchoesophageal fistula (procedure)
        "1155686006",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to intermediate artery (procedure)
        "1155687002",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1155885007",  # Repair of aortic valve (procedure)
        "1156983007",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1156984001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to anterior descending branch of left coronary artery (procedure)
        "11574002",  # Creation of endorectal ileal H-pouch with anastomosis to anus (procedure)
        "116026007",  # Total jejunectomy (procedure)
        "116027003",  # Partial jejunectomy (procedure)
        "116028008",  # Salpingo-oophorectomy (procedure)
        "116029000",  # Wedge biopsy of kidney (procedure)
        "116033007",  # Radical nephrectomy (procedure)
        "116140006",  # Total hysterectomy (procedure)
        "116141005",  # Abdominal hysterectomy (procedure)
        "116142003",  # Radical hysterectomy (procedure)
        "116143008",  # Total abdominal hysterectomy (procedure)
        "116144002",  # Total abdominal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "116145001",  # Radical vaginectomy (procedure)
        "116164007",  # Laryngotracheal resection (procedure)
        "116165008",  # Bronchial sleeve resection with pulmonary resection (procedure)
        "116166009",  # Radical nephrectomy AND ipsilateral adrenalectomy (procedure)
        "116167000",  # Excision of lesion of ovary in fragments (procedure)
        "116171002",  # Radical inguinal orchiectomy (procedure)
        "116172009",  # Incisional biopsy of ovary (procedure)
        "116173004",  # Wedge resection of ovary (procedure)
        "116174005",  # Wedge resection biopsy of ovary (procedure)
        "116175006",  # Proximal subtotal gastrectomy (procedure)
        "116215005",  # Proximal subtotal pancreatectomy (procedure)
        "116216006",  # Incisional biopsy of lesion of heart (procedure)
        "116217002",  # Excisional biopsy of lesion of heart (procedure)
        "116226004",  # Excisional biopsy of lesion of esophagus (procedure)
        "116228003",  # Excisional biopsy of lesion of stomach (procedure)
        "116229006",  # Incisional biopsy of large intestine (procedure)
        "116236007",  # Excisional biopsy of lesion of large intestine (procedure)
        "116240003",  # Incisional biopsy of ampulla of Vater (procedure)
        "116241004",  # Pancreaticoduodenectomy (procedure)
        "116242006",  # Pylorus-sparing pancreaticoduodenectomy (procedure)
        "116243001",  # Incisional biopsy of kidney (procedure)
        "1162442001",  # Laparoscopic excision of cyst of liver (procedure)
        "116248005",  # Excisional biopsy of lesion of fallopian tube (procedure)
        "1162481001",  # Laparoscopic cystogastrostomy of pancreas (procedure)
        "11626008",  # Miles operation, complete proctectomy (procedure)
        "1163003",  # Ureteroenterostomy (procedure)
        "1163245005",  # Laparoscopic excision of leiomyoma of uterus using robotic assistance (procedure)
        "1163275000",  # Laparoscopic radical amputation of cervix uteri (procedure)
        "1163432009",  # Adenomyomectomy (procedure)
        "116360008",  # Arterial bypass graft (procedure)
        "116361007",  # Creation of venovenous bypass (procedure)
        "11682008",  # Jejunoileostomy (procedure)
        "11717003",  # Wedge osteotomy of clavicle (procedure)
        "11718008",  # Excision of thoracic artery (procedure)
        "11748006",  # Repair of atrial septal defect with prosthesis by open heart technique (procedure)
        "1179198005",  # Endoscopic excision of medial wall of maxillary antrum (procedure)
        "11809004",  # Operation on cecum (procedure)
        "118156007",  # Gastrointestinal tract reconstruction (procedure)
        "1184598006",  # Open reconstruction of articular cartilage of knee joint (procedure)
        "1184599003",  # Reconstruction of articular cartilage of knee joint (procedure)
        "1184601001",  # Revision of transcatheter aortic valve implantation (procedure)
        "1186912002",  # Laparoscopic low anterior resection of rectum with construction of colostomy (procedure)
        "1186993008",  # Laparoscopic preperitoneal onlay mesh repair of incisional hernia of anterior abdominal wall (procedure)
        "1187086002",  # Transaxillary endoscopic diverticulectomy of esophagus with myotomy using robotic assistance (procedure)
        "1187630003",  # Subtotal hysterectomy via vaginal approach (procedure)
        "11878008",  # Excision of lesion of uterosacral ligament (procedure)
        "119000",  # Thoracoscopic partial lobectomy of lung (procedure)
        "119288004",  # Excision of ampulla of Vater (procedure)
        "11933006",  # Repair of atrial heart septum in total repair of tetralogy of Fallot (procedure)
        "1193692007",  # Prostatectomy using robotic assistance (procedure)
        "11954004",  # Repair of liver (procedure)
        "119560006",  # Urinary diversion procedure (procedure)
        "119564002",  # Internal mammary-coronary artery bypass graft (procedure)
        "119565001",  # Coronary artery bypass graft, anastomosis of artery of thorax to coronary artery (procedure)
        "119566000",  # Creation of cerebrospinal fluid shunt (procedure)
        "119746007",  # Lung excision (procedure)
        "1197529002",  # Transurethral en bloc excision of neoplasm of urinary bladder (procedure)
        "1197604004",  # Laparoscopic radiofrequency ablation of neoplasm of kidney (procedure)
        "119765004",  # Heart closure (procedure)
        "119766003",  # Heart reconstruction (procedure)
        "119768002",  # Heart valve closure (procedure)
        "119769005",  # Heart valve reconstruction (procedure)
        "119772003",  # Coronary artery closure (procedure)
        "119773008",  # Coronary artery reconstruction (procedure)
        "119776000",  # Aorta closure (procedure)
        "119777009",  # Aorta reconstruction (procedure)
        "119781009",  # Neck artery excision (procedure)
        "119782002",  # Head artery excision (procedure)
        "119807004",  # Thoracic artery closure (procedure)
        "119808009",  # Abdominal artery closure (procedure)
        "119809001",  # Thoracic artery repair (procedure)
        "119810006",  # Abdominal artery repair (procedure)
        "119811005",  # Abdominal artery transposition (procedure)
        "119812003",  # Thoracic artery transposition (procedure)
        "119815001",  # Artery of extremity reconstruction (procedure)
        "119823004",  # Pericardium closure (procedure)
        "119824005",  # Pericardium reconstruction (procedure)
        "119879002",  # Esophagus closure (procedure)
        "119880004",  # Esophagus reconstruction (procedure)
        "119887001",  # Small intestine closure (procedure)
        "119888006",  # Small intestine reconstruction (procedure)
        "119892004",  # Large intestine closure (procedure)
        "119893009",  # Large intestine reconstruction (procedure)
        "119901004",  # Rectum closure (procedure)
        "119902006",  # Anus reconstruction (procedure)
        "119903001",  # Rectum reconstruction (procedure)
        "119913009",  # Liver closure (procedure)
        "119914003",  # Liver reconstruction (procedure)
        "119936000",  # Pancreas closure (procedure)
        "119937009",  # Pancreas reconstruction (procedure)
        "119982008",  # Kidney closure (procedure)
        "119983003",  # Kidney reconstruction (procedure)
        "119984009",  # Kidney transposition (procedure)
        "120001005",  # Testis excision (procedure)
        "120038005",  # Cervix excision (procedure)
        "120053002",  # Fallopian tube excision (procedure)
        "120059003",  # Ovary closure (procedure)
        "120061007",  # Ovary reconstruction (procedure)
        "120074001",  # Brain closure (procedure)
        "120075000",  # Brain reconstruction (procedure)
        "120076004",  # Transplantation of brain tissue (procedure)
        "120145007",  # Chest wall incision (procedure)
        "120167008",  # Diaphragm closure (procedure)
        "120168003",  # Diaphragm transposition (procedure)
        "1201734006",  # Total replacement of knee with impaction bone graft (procedure)
        "1201737004",  # Le Fort I incision of maxilla with incision of maxilla into multiple segments (procedure)
        "120175002",  # Excision from retroperitoneum (procedure)
        "120185001",  # Retroperitoneum closure (procedure)
        "1201858000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch of circumflex coronary artery and left anterior descending coronary artery (procedure)
        "120187009",  # Retroperitoneum reconstruction (procedure)
        "120191004",  # Retroperitoneum destructive procedure (procedure)
        "1201979001",  # Excision of lesion of brain (procedure)
        "1202019006",  # Craniectomy of base of skull and excision of midline neoplasm of brain (procedure)
        "120215003",  # Removal from artery of thorax (procedure)
        "120216002",  # Removal from artery of abdomen (procedure)
        "12038002",  # Incision of endocardium (procedure)
        "1204284001",  # Repair of mitral valve using robotic assistance (procedure)
        "1204285000",  # Removal of leadless implantable cardiac pacemaker (procedure)
        "1204377006",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery and distal portion of anterior descending branch of left coronary artery (procedure)
        "1204378001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery and mid portion of anterior descending branch of left coronary artery (procedure)
        "12072005",  # Maxillectomy with orbital exenteration (procedure)
        "1208430005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to mid and distal portion of anterior descending branch of left coronary artery (procedure)
        "1208673009",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and second obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1208675002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery and anterior descending branch of left coronary artery (procedure)
        "1208676001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery to anterior descending branch of left coronary artery (procedure)
        "1208760004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of left anterior descending coronary artery to left anterior descending coronary artery (procedure)
        "1208761000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second diagonal branch of left anterior descending coronary artery to left anterior descending coronary artery (procedure)
        "1208916007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and first obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1208917003",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and third diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208919000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and second diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208940004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and first diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208942007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery and third marginal circumflex coronary artery (procedure)
        "1208943002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery and first marginal circumflex coronary artery (procedure)
        "1209060000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and third diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209061001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and second diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209062008",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second and third diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209115004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and third obtuse marginal branches of circumflex branch of left coronary artery (procedure)
        "1209117007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and second obtuse marginal branches of circumflex branch of left coronary artery (procedure)
        "1209119005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1209120004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209172009",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to left posterior lateral branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209174005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to third obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209180002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to third obtuse marginal branch and left posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1209181003",  # Excision arthroplasty of joint of knee with insertion of cement spacer (procedure)
        "12092003",  # Repair of rectal prolapse (procedure)
        "1209203006",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch and left posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1209204000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second and third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1209205004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "12102007",  # Inversion of diverticulum of large intestine (procedure)
        "12144001",  # Ligation of inferior vena cava (procedure)
        "1216987008",  # Choledochoduodenostomy using endoscopic ultrasonography guidance (procedure)
        "1217005008",  # Bioprosthetic transcatheter aortic valve implantation (procedure)
        "1217137007",  # Partial replacement of joint of right shoulder (procedure)
        "1217139005",  # Partial replacement of joint of left hip with bipolar prosthesis (procedure)
        "1217140007",  # Partial replacement of joint of right hip with bipolar prosthesis (procedure)
        "1217141006",  # Partial replacement of joint of bilateral hips with bipolar prosthesis (procedure)
        "1217153005",  # Anastomosis of left internal thoracic artery to third diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217155003",  # Anastomosis of left internal thoracic artery to second obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1217157006",  # Anastomosis of left internal thoracic artery to second diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217158001",  # Anastomosis of left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery (procedure)
        "1217161000",  # Anastomosis of left internal thoracic artery to mid portion of anterior descending branch of left coronary artery (procedure)
        "1217162007",  # Anastomosis of left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1217165009",  # Anastomosis of left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217166005",  # Anastomosis of left internal thoracic artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "1217168006",  # Anastomosis of left internal thoracic artery to third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1217259002",  # Percutaneous cryoablation of lesion of adrenal gland (procedure)
        "1217319002",  # Laparoscopic excision of adrenal gland using robotic assistance (procedure)
        "1217395007",  # Mediastinoscopy and thoracoscopy with ventilation of lung (procedure)
        "1217398009",  # Biopsy of lesion of pancreas using laparoscopic ultrasonography guidance (procedure)
        "1217421008",  # Biopsy of lesion of liver using laparoscopic ultrasonography guidance (procedure)
        "1217701008",  # Biopsy of pericardium using ultrasonographic guidance (procedure)
        "1217702001",  # Total replacement of knee joint using cone (procedure)
        "1220544006",  # Open excision of jejunum (procedure)
        "1220546008",  # Laparoscopic excision of jejunum (procedure)
        "1220548009",  # Open excision of ileum (procedure)
        "1220549001",  # Laparoscopic excision of ileum (procedure)
        "1220622006",  # Total replacement of knee joint using knee stem (procedure)
        "1222566006",  # Total replacement of knee joint with bone allograft (procedure)
        "1222567002",  # Total replacement of knee joint with bone autograft (procedure)
        "1222568007",  # Total replacement of knee joint using modular metal augmentation (procedure)
        "1222569004",  # Total replacement of knee joint using rotating hinge prosthesis (procedure)
        "1222570003",  # Total knee replacement using valgus-varus constrained prosthesis (procedure)
        "122487007",  # Division of patent ductus arteriosus (procedure)
        "12253004",  # Insertion of graft of aorta with cardiopulmonary bypass (procedure)
        "1227005",  # Incision of subvalvular tissue for discrete subvalvular aortic stenosis (procedure)
        "1230048008",  # Total replacement of hip joint using metal augmentation (procedure)
        "1230401003",  # Ligation of ileocolic artery (procedure)
        "1230404006",  # Closure of atrial septal defect using robotic assistance (procedure)
        "1231171000",  # Incision of maxilla with insertion and adjustment of fixed rapid maxillary expansion appliance (procedure)
        "1231314009",  # Cystogastrostomy of pancreas using endoscopic ultrasonography guidance (procedure)
        "1231338008",  # Ligation of middle colic artery (procedure)
        "1231339000",  # Ligation of left colic artery (procedure)
        "1231340003",  # Ligation of right colic artery (procedure)
        "1231398006",  # Prosthetic unicompartmental arthroplasty of knee using robotic assistance (procedure)
        "1231410001",  # Total replacement of hip joint using impaction bone graft (procedure)
        "1231411002",  # Total replacement of hip joint using mesh (procedure)
        "1231412009",  # Total replacement of hip joint using ring (procedure)
        "1231413004",  # Total replacement of hip joint using allograft of bone (procedure)
        "1231414005",  # Total replacement of hip joint using autogenous bone graft (procedure)
        "1231415006",  # Total replacement of hip joint using cage (procedure)
        "1231417003",  # Enlargement of aortic root using anterior approach (procedure)
        "1231418008",  # Enlargement of aortic root using posterior approach (procedure)
        "1231421005",  # Resection of non-ruptured aneurysm of carotid artery with interposition graft replacement (procedure)
        "1231449003",  # Transcatheter repair of tricuspid valve (procedure)
        "1231463009",  # Excision of leukoplakia of vocal cord (procedure)
        "1231496006",  # Implantation of meniscus prosthesis into knee joint (procedure)
        "12317003",  # Ligation of superior longitudinal sinus (procedure)
        "1231726001",  # Transapical transcatheter implantation of bioprosthetic mitral valve prosthesis into mitral annuloplasty ring using fluoroscopic and transoesophageal echocardiographic guidance (procedure)
        "1231734007",  # Excision of breast (procedure)
        "1231783006",  # Percutaneous closure of prosthetic cardiac paravalvular leak using device (procedure)
        "1231785004",  # Carotid artery to axillary artery bypass (procedure)
        "1231787007",  # Endovascular repair of thoracic aorta using fenestrated stent graft (procedure)
        "1231788002",  # Balloon atrial septostomy with implantation of atrial flow regulator (procedure)
        "1231832001",  # Excision of endometriotic cyst of retroperitoneum (procedure)
        "12324002",  # Exploratory laparotomy with biopsy (procedure)
        "12330002",  # Complete parathyroidectomy (procedure)
        "1234785000",  # Replacement of cardiac resynchronization therapy defibrillator pulse generator (procedure)
        "1234788003",  # Replacement of leadless cardiac pacemaker (procedure)
        "12363009",  # Complete repair of tetralogy of Fallot with closure of previous shunt (procedure)
        "1236755008",  # Implantation of cardiac resynchronization defibrillator system (procedure)
        "1236821000",  # Removal of tumor thrombus associated with renal cell carcinoma from inferior vena cava by open approach (procedure)
        "1236848004",  # Excision of cyst of cranium (procedure)
        "1236939005",  # Bypass of left common carotid artery to right common carotid artery using prosthetic graft (procedure)
        "1236959009",  # Excision of cyst of liver (procedure)
        "1237589003",  # Transcatheter repair of leaflet of tricuspid valve using transesophageal echocardiography and fluoroscopic guidance (procedure)
        "12378002",  # Incision and drainage of perigastric space (procedure)
        "12398007",  # Excision of endometrial synechiae (procedure)
        "1240409003",  # Implantation of prosthesis into right glenohumeral joint (procedure)
        "1240410008",  # Implantation of prosthesis into left glenohumeral joint (procedure)
        "1240411007",  # Implantation of prosthesis into right knee joint (procedure)
        "1240412000",  # Implantation of prosthesis into left knee joint (procedure)
        "12445005",  # Repair of ventricular septal defect with tissue graft (procedure)
        "12483006",  # Osteotomy of clavicle (procedure)
        "1251509000",  # Repair of joint of right knee (procedure)
        "1251510005",  # Repair of joint of left knee (procedure)
        "1255087004",  # Endovascular repair of dissection of aorta using endovascular stent graft (procedure)
        "1255088009",  # Repair of dissection of aorta using stent graft (procedure)
        "1255141008",  # Transcatheter annuloplasty of tricuspid valve using transesophageal echocardiography and fluoroscopy guidance (procedure)
        "1255199009",  # Percutaneous endoscopic insertion of jejunostomy tube (procedure)
        "1255424002",  # Laparoscopic excision of part of liver using robotic assistance (procedure)
        "1255425001",  # Percutaneous cryoablation of lesion of liver (procedure)
        "1255431003",  # Laparoscopic subtotal excision of distal pancreas (procedure)
        "1255432005",  # Subtotal excision of distal pancreas using robotic assistance (procedure)
        "1255819006",  # Serial transverse enteroplasty (procedure)
        "1255832001",  # Laparoscopic anorectoplasty (procedure)
        "1255923004",  # Central pancreatectomy (procedure)
        "1255931009",  # Endoscopic argon plasma coagulation of pancreas (procedure)
        "1255940008",  # Anastomosis of hepatic duct to jejunum using robotic assistance (procedure)
        "1256034006",  # Esophagogastroduodenoscopy and closure of fistula of duodenum (procedure)
        "1256038009",  # Open creation of jejunostomy (procedure)
        "1256040004",  # Percutaneous endoscopic creation of jejunostomy (procedure)
        "1256087006",  # Laparoscopic reversal of Hartmann procedure (procedure)
        "1256090000",  # Percutaneous irreversible electroporation ablation of neoplasm of liver using imaging guidance (procedure)
        "1258864009",  # Laparoscopic irreversible electroporation ablation of neoplasm of liver (procedure)
        "1258865005",  # Open irreversible electroporation of neoplasm of liver (procedure)
        "1258871004",  # Transanal total mesorectal excision (procedure)
        "1259868000",  # Endovascular repair of aneurysm of aorta using endovascular stent graft and stent graft anchoring system (procedure)
        "1259910000",  # Endovascular repair of aneurysm of thoracic aorta using endovascular stent graft and stent graft anchoring system (procedure)
        "1259982004",  # Anchoring of aortic endovascular stent graft to vessel wall using endovascular stent graft anchoring system (procedure)
        "1259983009",  # Anchoring of thoracic aortic endovascular stent graft to vessel wall using endovascular stent graft anchoring system (procedure)
        "1260291002",  # Central pancreatectomy using robotic assistance (procedure)
        "1260293004",  # Pancreaticoduodenectomy using robotic assistance (procedure)
        "1260402001",  # Laparoscopic excision of entire pyloric portion of stomach (procedure)
        "1260408002",  # Segmental excision of duodenum (procedure)
        "1260433004",  # Subtotal gastrectomy using robotic assistance (procedure)
        "1260434005",  # Total gastrectomy using robotic assistance (procedure)
        "12605005",  # Incision of retroperitoneum (procedure)
        "12617007",  # Resection of lower limb artery with replacement by graft (procedure)
        "1263398006",  # Proctoscopic control of rectal bleeding using electrocoagulation (procedure)
        "1263412009",  # Excision of entire pyloric portion of stomach (procedure)
        "1263487006",  # Subtotal gastrectomy with abdominal lymphadenectomy using robotic assistance (procedure)
        "1263731004",  # Implantation of custom-made extraluminal aortic root mesh-sleeve (procedure)
        "1263735008",  # Endoscopic microsurgical excision of rectum by transanal approach (procedure)
        "1263754008",  # Percutaneous gastrostomy using imaging guidance (procedure)
        "1264200003",  # Excision of lesion of rectum by transanal minimal invasive surgery (procedure)
        "1264335004",  # Gastrojejunostomy and insertion of stent using endoscopic ultrasonography guidance (procedure)
        "1264439009",  # Anastomosis of gallbladder to duodenum and insertion of stent using endoscopic ultrasonography guidance (procedure)
        "12654003",  # Closure of foramen ovale with prosthesis by closed heart technique (procedure)
        "12663001",  # Amputation below-knee conversion into above-knee amputation (procedure)
        "12664007",  # Excision with graft replacement of thoracic artery (procedure)
        "12666009",  # Cholecystocolostomy (procedure)
        "12672009",  # Proctolysis (procedure)
        "1268321007",  # Excision of endometriosis of pelvis (procedure)
        "1268346007",  # Laparoscopic dissection of para-aortic lymph node group (procedure)
        "1268399001",  # Cystogastrostomy of pancreas and insertion of stent using endoscopic ultrasonography guidance (procedure)
        "1268402000",  # Choledochoduodenostomy and insertion of biliary stent using endoscopic ultrasonography guidance (procedure)
        "1268524003",  # Surgical procedure on aneurysm of right cardiac ventricle (procedure)
        "1268525002",  # Excision of aneurysm of right cardiac ventricle (procedure)
        "1268980002",  # Simple mastectomy of right breast and prophylactic mastectomy of left breast (procedure)
        "1269061009",  # Simple mastectomy of left breast and prophylactic mastectomy of right breast (procedure)
        "1269448002",  # Amputation of bilateral arms through humerus (procedure)
        "1269449005",  # Disarticulation of left ankle joint (procedure)
        "1269450005",  # Disarticulation of ankle joint (procedure)
        "1269466007",  # Disarticulation of bilateral ankle joints (procedure)
        "12718005",  # Excision of aberrant renal artery (procedure)
        "12739003",  # Revision of ileostomy, complicated (procedure)
        "127598004",  # Repair of joint capsule of knee (procedure)
        "12780005",  # Indirect laryngoscopy with removal of lesion (procedure)
        "1279570000",  # Closure of defect of atrioventricular septum with surgical patch and repair of cleft of mitral valve (procedure)
        "1279571001",  # Closure of defect of atrioventricular septum with surgical patch and repair of cleft of tricuspid valve (procedure)
        "1279572008",  # Repair of defect of atrioventricular septum with prosthesis (procedure)
        "1279575005",  # Repair of defect of atrioventricular septum with tissue graft (procedure)
        "1279870005",  # Repair of mitral valve using prosthetic chordae tendinae using transesophageal echocardiography guidance (procedure)
        "1280009009",  # Ligation of brachiocephalic artery (procedure)
        "12827003",  # Excision of part of rectum (procedure)
        "128397006",  # Operative procedure on intracranial vein (procedure)
        "128401003",  # Anastomosis of duodenum (procedure)
        "128406008",  # Anastomosis of jejunum (procedure)
        "1285098007",  # Disarticulation of bilateral knee joints (procedure)
        "128531006",  # Islet cell transplant (procedure)
        "1285546002",  # Partial replacement of joint of bilateral shoulders (procedure)
        "12863003",  # Closure of rectouterine fistula (procedure)
        "1287441006",  # Uncemented reverse polarity total prosthetic arthroplasty of shoulder joint (procedure)
        "1287541008",  # Coagulation of brain tissue (procedure)
        "1287561002",  # Coronary artery bypass using robotic assistance (procedure)
        "1287654009",  # Transcatheter replacement of mitral valve bioprosthesis (procedure)
        "1287655005",  # Percutaneous repair of mitral valve (procedure)
        "1287736002",  # Operation on iliac artery (procedure)
        "1287739009",  # Operation on femoral artery (procedure)
        "1287777003",  # Angioplasty of transverse sinus of brain with insertion of stent (procedure)
        "1287782005",  # Total excision of urinary bladder and creation of ileal conduit (procedure)
        "1287897002",  # Radical amputation of cervix uteri (procedure)
        "1287903006",  # Operation on cerebral artery (procedure)
        "1287945008",  # Prosthetic hybrid total knee replacement (procedure)
        "1287977006",  # Laparoscopic radical cystectomy (procedure)
        "1287995009",  # Radiofrequency ablation of neoplasm of pancreatic duct (procedure)
        "1288011001",  # Endoluminal gastroplication for gastroesophageal reflux disease (procedure)
        "1288015005",  # Decompressive craniectomy (procedure)
        "1288030001",  # Laparoscopic ileal replacement of ureter (procedure)
        "128952006",  # Catheterization of both left and right heart with graft (procedure)
        "1289993008",  # Laparoscopic radical excision of kidney by retroperitoneal approach (procedure)
        "1290095005",  # Reverse polarity cemented total prosthetic replacement of shoulder joint (procedure)
        "1290230006",  # Transluminal operation on cerebral artery (procedure)
        "1290534001",  # Hysteroscopic morcellation of uterine fibroid (procedure)
        "129212008",  # Grafting procedure of heart (procedure)
        "129243001",  # Closed heart valvotomy of tricuspid valve (procedure)
        "1292979002",  # Repair of congenital hernia of diaphragm (procedure)
        "1293269004",  # Biopsy of transplanted kidney (procedure)
        "12935001",  # Ligation of intracranial vein (procedure)
        "12951009",  # Implantation of electrode into cardiac atrium, replacement (procedure)
        "12959006",  # Creation of subcutaneous tunnel with esophageal anastomosis (procedure)
        "1296604005",  # Drainage of cerebral epidural space by trephination (procedure)
        "1296606007",  # Drainage of cerebral subdural space by trephination (procedure)
        "1296607003",  # Drainage of cerebral subarachnoid space by trephination (procedure)
        "1296608008",  # Drainage of cerebral subarachnoid space by incision (procedure)
        "1296626006",  # Division of tract of brain (procedure)
        "1296901000",  # Insertion of tunneled peritoneal drain into peritoneal cavity using fluoroscopic guidance (procedure)
        "1297096009",  # Arterial bypass graft of lower limb artery using arterial graft (procedure)
        "1297100005",  # Arterial bypass graft of thoracic artery using arterial graft (procedure)
        "1297175004",  # Arterial bypass graft of upper limb artery using arterial graft (procedure)
        "12976005",  # Donor nephrectomy (procedure)
        "12986006",  # Excision of Pancoast tumor of lung (procedure)
        "1299000",  # Excision of appendiceal stump (procedure)
        "1303413007",  # Laparoscopic sigmoid colectomy (procedure)
        "1303414001",  # Laparoscopic vaginal hysterectomy (procedure)
        "1303628001",  # Excision of cyst of vocal cord (procedure)
        "1303772009",  # Aortic valve reconstruction with glutaraldehyde-treated autologous pericardium (procedure)
        "1303993002",  # Esophagogastroduodenoscopy with full thickness resection of esophagus (procedure)
        "1303994008",  # Esophagogastroduodenoscopy with repair of duodenum using suture (procedure)
        "1303995009",  # Esophagogastroduodenoscopy and repair of small intestine using suture (procedure)
        "1304002009",  # Esophagogastroduodenoscopy and full thickness resection of duodenum (procedure)
        "1304004005",  # Esophagogastroduodenoscopy and repair of esophagus using suture (procedure)
        "1304005006",  # Esophagogastroduodenoscopy and full thickness resection of small intestine (procedure)
        "1304006007",  # Esophagogastroduodenoscopy and full thickness resection of stomach (procedure)
        "1304042004",  # Colonoscopy and suturing of rectum (procedure)
        "1304043009",  # Colonoscopy and suturing of colon (procedure)
        "1304044003",  # Colonoscopy and suturing of small intestine (procedure)
        "1304045002",  # Colonoscopy and full thickness resection of rectum (procedure)
        "1304049008",  # Colonoscopy and full thickness resection of colon (procedure)
        "1304050008",  # Colonoscopy and full thickness resection of small intestine (procedure)
        "1304053005",  # Sigmoidoscopy and full thickness resection of rectum (procedure)
        "1304055003",  # Sigmoidoscopy and suturing of colon (procedure)
        "1304056002",  # Sigmoidoscopy and suturing of rectum (procedure)
        "1304057006",  # Sigmoidoscopy and full thickness resection of colon (procedure)
        "1304071003",  # Endoscopic vaginal natural orifice transluminal hysterectomy (procedure)
        "1304149006",  # Focal resurfacing of periarticular bone of knee joint with implantation of prosthesis (procedure)
        "1304236006",  # Total prosthetic replacement of hip joint via supercapsular approach using percutaneous assistance (procedure)
        "1306752006",  # Trapping of aneurysm of cerebral artery (procedure)
        "1306775004",  # Electrocoagulation of aneurysm of cerebral artery (procedure)
        "13119002",  # Operation on adrenal gland (procedure)
        "13137001",  # Operation on pineal gland (procedure)
        "13246003",  # Closure of enterovesical fistula with urinary bladder resection (procedure)
        "13254001",  # Abdominal hysterectomy with colpo-urethrocystopexy, Marshall-Marchetti-Krantz type (procedure)
        "13275009",  # Excision of lesion of thoracic aorta with end-to-end anastomosis (procedure)
        "1328002",  # Ilioiliac shunt (procedure)
        "13316004",  # Implantation of pulmonary valve with tissue graft (procedure)
        "1331906009",  # Insertion of peritoneal drain into peritoneal cavity using fluoroscopic guidance (procedure)
        "1332006009",  # Wedge osteotomy of rib (procedure)
        "1332084009",  # Revision of cerebral ventriculopleural shunt (procedure)
        "1332236008",  # Implantation of cardiac resynchronization defibrillator system using fluoroscopic guidance (procedure)
        "1332525002",  # Cardiac ablation for atrial fibrillation (procedure)
        "1332526001",  # Clipping of aneurysm of cerebral artery (procedure)
        "1332561007",  # Balloon dilation of nephrostomy using fluoroscopic guidance (procedure)
        "1332562000",  # Balloon dilation of nephrostomy using ultrasonographic guidance (procedure)
        "13327002",  # Partial colectomy by transanal approach (procedure)
        "1334018004",  # Percutaneous mechanical thrombectomy of left renal artery using fluoroscopic guidance with contrast (procedure)
        "1334019007",  # Percutaneous mechanical thrombectomy of right renal artery using fluoroscopic guidance with contrast (procedure)
        "1334020001",  # Percutaneous mechanical thrombectomy of bilateral renal arteries using fluoroscopic guidance with contrast (procedure)
        "1334027003",  # Percutaneous radiofrequency ablation of left renal artery using fluoroscopic guidance (procedure)
        "1334028008",  # Percutaneous radiofrequency ablation of right renal artery using fluoroscopic guidance (procedure)
        "1334029000",  # Percutaneous radiofrequency ablation of bilateral renal arteries using fluoroscopic guidance (procedure)
        "13341006",  # Reduction of volvulus by laparotomy (procedure)
        "13347005",  # Destruction of intra-abdominal tumor, extensive (procedure)
        "1335721004",  # Construction of permanent stoma of stomach with insertion of gastrostomy tube (procedure)
        "13358001",  # Complete parotidectomy (procedure)
        "1336165006",  # Incision of kidney and drainage of kidney using tube (procedure)
        "1336241002",  # Excision of bilateral mastoid bones (procedure)
        "1337002",  # Colosigmoidostomy (procedure)
        "1337008004",  # Trapping of aneurysm of intracranial artery (procedure)
        "1338047000",  # Removal of hip joint prosthesis for periprosthetic joint infection (procedure)
        "1338057004",  # Thoracoscopic excision of lobe of lung using robotic assistance (procedure)
        "133870002",  # Repair of ileorectal fistula (procedure)
        "133871003",  # Cecosigmoidostomy (procedure)
        "13391000087100",  # Left nephrostomy using ultrasonographic guidance (procedure)
        "13401000087102",  # Right nephrostomy using ultrasonographic guidance (procedure)
        "1340181009",  # Laparoscopic duodenal switch (procedure)
        "1340182002",  # Laparoscopic duodenojejunal bypass (procedure)
        "1340187008",  # High intensity focused ultrasound ablation of subthalamus using magnetic resonance imaging guidance (procedure)
        "13403003",  # Pharyngoesophageal repair (procedure)
        "1342337003",  # Laparoscopic radical prostatectomy (procedure)
        "13431007",  # Prefrontal lobotomy (procedure)
        "13433005",  # Excision of lesion of brain by transtemporal approach (procedure)
        "13532002",  # Excision of lesion of thoracic aorta with interposition graft replacement (procedure)
        "13575008",  # Femoroperoneal vascular bypass (procedure)
        "13580001000004101",  # Excision of tumor of liver (procedure)
        "13623009",  # Pyeloureterovesical anastomosis (procedure)
        "13642008",  # Transection of aberrant renal vessels (procedure)
        "13662000",  # Anastomosis of pulmonary-subclavian artery by Blalock-Taussig operation (procedure)
        "13666002",  # Leukotomy (procedure)
        "13721004",  # Operation on chordae tendineae (procedure)
        "13819004",  # Division of laryngeal nerve (procedure)
        "13830001",  # Incision of pericardium (procedure)
        "13893002",  # Closure of appendiceal fistula (procedure)
        "1390003",  # Laparoscopic-assisted sigmoid colectomy (procedure)
        "13963007",  # Repair of coronary arteriovenous chamber fistula (procedure)
        "1398005",  # Direct thrombectomy of iliac vein by leg incision (procedure)
        "13991004",  # Repair of ileovesical fistula (procedure)
        "13995008",  # Amputation of arm through shoulder (procedure)
        "14072009",  # Total esophagectomy (procedure)
        "1411001",  # Operation on papillary muscle of heart (procedure)
        "14117005",  # Insertion of Crosby-Cooney button into abdomen (procedure)
        "14135001",  # Labyrinthectomy with mastoidectomy (procedure)
        "14201006",  # Coronary angioplasty by open chest approach (procedure)
        "14247003",  # Cerebral hemispherectomy (procedure)
        "14257002",  # Repair of hypoplastic left heart syndrome (procedure)
        "14259004",  # Endarterectomy of descending aorta (procedure)
        "14296008",  # Excision of common wall between posterior left atrium and coronary sinus with roofing of resultant defect with patch graft (procedure)
        "14303006",  # Ligation of ascending aorta (procedure)
        "14323007",  # Aortocoronary bypass of three coronary arteries (procedure)
        "14335006",  # Closure of esophageal fistula by thoracic approach (procedure)
        "14346009",  # Labyrinthotomy with cryosurgery, with mastoidectomy (procedure)
        "14352005",  # Incision and drainage of pseudocyst of pancreas (procedure)
        "14403007",  # Ileotransversostomy (procedure)
        "14412009",  # Laparoscopy with fulguration of lesion (procedure)
        "14520001000004109",  # Laparoscopic excision of mass of liver (procedure)
        "14554005",  # Cryotherapy of brain (procedure)
        "1457004",  # Truncal vagotomy with pyloroplasty and gastrostomy (procedure)
        "14572004",  # Repair of ovary (procedure)
        "14599002",  # Esophagojejunostomy by abdominal approach (procedure)
        "14622006",  # Esophagoplasty by cervical approach (procedure)
        "14653008",  # Repair of aneurysm with graft of mesenteric artery (procedure)
        "14687003",  # Choledochoenterostomy (procedure)
        "14690009",  # Anastomosis of large intestine to rectum (procedure)
        "14752007",  # Axillary-axillary artery bypass graft (procedure)
        "14776004",  # Diverticulectomy of kidney (procedure)
        "15007006",  # Thrombectomy of arterial graft (procedure)
        "1501006",  # Altemeier operation, perineal rectal pull-through (procedure)
        "15023006",  # Thromboendarterectomy with graft of carotid artery by neck incision (procedure)
        "15050001",  # Laparoscopic biopsy of liver (procedure)
        "15051002",  # Repair of pancreas (procedure)
        "15067008",  # Nephropexy (procedure)
        "15101009",  # Percutaneous transluminal angioplasty of visceral artery (procedure)
        "15163009",  # Total arthroplasty of hip, low friction (procedure)
        "15213007",  # Truncal vagotomy (procedure)
        "15257006",  # Lobectomy of liver with partial excision of adjacent lobes (procedure)
        "15292003",  # Subtotal supraglottic laryngectomy with radical neck dissection (procedure)
        "15309003",  # Coating of aneurysm of brain (procedure)
        "15365007",  # Excision of hematocele of tunica vaginalis (procedure)
        "15440009",  # Disarticulation (procedure)
        "15457008",  # Closure of cecosigmoidal fistula (procedure)
        "15491007",  # Open excisional biopsy of urinary bladder (procedure)
        "15532000",  # Release of celiac artery axis (procedure)
        "15648005",  # Banding of pulmonary artery (procedure)
        "15669007",  # Splenorenal bypass graft (procedure)
        "1576000",  # Closure of uteroenteric fistula (procedure)
        "1583007",  # Tractotomy of mesencephalon (procedure)
        "15853003",  # Thromboendarterectomy with graft of femoral and popliteal arteries (procedure)
        "15855005",  # Transposition of retroperitoneal tissue (procedure)
        "15905001",  # Embolectomy with catheter of celiac artery by abdominal incision (procedure)
        "15918009",  # Closure of vesicovaginal fistula by transvesical and vaginal approach (procedure)
        "15944009",  # Arthrotomy for excision of torn cartilage of sternoclavicular joint (procedure)
        "15946006",  # Correction of coarctation of aorta with graft replacement (procedure)
        "16068008",  # Repair of vesicosigmoidovaginal fistula (procedure)
        "16079003",  # Sauer-Bacon operation, abdominoperineal resection (procedure)
        "16156002",  # Drainage of pseudocyst of pancreas by anastomosis (procedure)
        "16180003",  # Laryngotomy with removal of tumor (procedure)
        "16224002",  # Closure of hepatic duct fistula (procedure)
        "16280008",  # Closure of gastric fistula (procedure)
        "16334008",  # Take-down of systemic-pulmonary artery anastomosis (procedure)
        "16337001",  # Complicated pyelotomy (procedure)
        "1638004",  # Cineplastic amputation with cineplastic prosthesis of extremity (procedure)
        "1645004",  # Transduodenal sphincterotomy (procedure)
        "16550151000119102",  # Percutaneous cryoablation of neoplasm of right kidney using computed tomography guidance (procedure)
        "16550201000119105",  # Percutaneous cryoablation of neoplasm of left kidney using computed tomography guidance (procedure)
        "16558481000119100",  # Percutaneous cryoablation of lesion of liver using computed tomography guidance (procedure)
        "16564004",  # Hartmann operation, rectal resection (procedure)
        "16631961000119101",  # Endovascular insertion of inferior vena cava filter using imaging guidance (procedure)
        "16781006",  # Removal of cardiac pacemaker from epicardium with replacement of atrial and/or ventricular leads (procedure)
        "16866008",  # Larrey shoulder disarticulation (procedure)
        "16881002",  # Obliteration of caliceal diverticulum (procedure)
        "1689004",  # Local excision of ovary (procedure)
        "16900000",  # Replacement of ventricular shunt (procedure)
        "16903003",  # Thromboendarterectomy with graft, combined aortoiliofemoral (procedure)
        "169360009",  # Radiotherapy: seeds into hypophysis (procedure)
        "169361008",  # Radiotherapy: seeds into brain (procedure)
        "169365004",  # Radiotherapy: temporary brain implant (procedure)
        "17041004",  # Incision of appendix (procedure)
        "17049002",  # Diverticulectomy of esophagus by thoracic approach (procedure)
        "17073005",  # Aortocoronary artery bypass graft with vein graft (procedure)
        "17086001",  # Modified radical mastectomy, bilateral (procedure)
        "171373006",  # Excision of perirectal tissue (procedure)
        "171442008",  # Major excision of brain tissue (procedure)
        "171443003",  # Total lobectomy of brain (procedure)
        "171447002",  # Excision of lesion of frontal lobe of brain (procedure)
        "171448007",  # Excision of lesion of temporal lobe of brain (procedure)
        "171449004",  # Excision of lesion of parietal lobe of brain (procedure)
        "171450004",  # Excision of lesion of occipital lobe of brain (procedure)
        "171451000",  # Excision of lesion of cerebellum (procedure)
        "171454008",  # Stereotactic ablation of brain tissue (procedure)
        "171455009",  # Stereotactic leucotomy (procedure)
        "171457001",  # Stereotactic ablation of thalamus tissue (procedure)
        "171458006",  # Stereotactic ablation of globus pallidus tissue (procedure)
        "171459003",  # Stereotactic ablation of brainstem tissue (procedure)
        "171462000",  # Open biopsy of lesion of brain tissue (procedure)
        "171463005",  # Open biopsy of lesion of frontal lobe of brain (procedure)
        "171464004",  # Open biopsy of lesion of temporal lobe of brain (procedure)
        "171465003",  # Open biopsy of lesion of parietal lobe of brain (procedure)
        "171466002",  # Open biopsy of lesion of occipital lobe of brain (procedure)
        "171467006",  # Open biopsy of lesion of cerebellum (procedure)
        "171468001",  # Open biopsy of lesion of brainstem (procedure)
        "171479006",  # Open division of brain tissue (procedure)
        "171496007",  # Removal of neurostimulator electrode from brain tissue (procedure)
        "171509005",  # Brain cerebrospinal fluid pathway operation (procedure)
        "171510000",  # Creation of connection from ventricle of brain (procedure)
        "171514009",  # Creation of subcutaneous cerebrospinal fluid reservoir (procedure)
        "171540003",  # Endoscopic extirpation of lesion of ventricle of brain (procedure)
        "171553000",  # Operation on subarachnoid space of brain (procedure)
        "171570008",  # Intracranial transection of cranial nerve (procedure)
        "171571007",  # Intracranial transection of optic nerve (II) (procedure)
        "171572000",  # Intracranial transection of oculomotor nerve (III) (procedure)
        "171573005",  # Intracranial transection of trochlear nerve (IV) (procedure)
        "171574004",  # Intracranial transection of abducens nerve (VI) (procedure)
        "171575003",  # Intracranial transection of trigeminal nerve (V) (procedure)
        "171577006",  # Intracranial transection of facial nerve (VII) (procedure)
        "171578001",  # Intracranial transection of acoustic nerve (VIII) (procedure)
        "171579009",  # Intracranial transection of glossopharyngeal nerve (IX) (procedure)
        "171580007",  # Intracranial transection of vagus nerve (X) (procedure)
        "171581006",  # Intracranial transection of accessory nerve (XI) (procedure)
        "171582004",  # Intracranial transection of hypoglossal nerve (XII) (procedure)
        "171697008",  # Extirpation of lesion of falx cerebri (procedure)
        "171702004",  # Repair of meningoencephalocele (procedure)
        "171756006",  # Creation of syringoperitoneal shunt (procedure)
        "171757002",  # Creation of lumboperitoneal shunt (procedure)
        "171797004",  # Total sacrifice of peripheral nerve (procedure)
        "171944000",  # Transethmoidal hypophysectomy (procedure)
        "171946003",  # Transseptal hypophysectomy (procedure)
        "171947007",  # Transcranial hypophysectomy (procedure)
        "171948002",  # Transsphenoidal hypophysectomy (procedure)
        "171957008",  # Excision of lesion of pituitary gland (procedure)
        "171959006",  # Decompression of pituitary gland (procedure)
        "171961002",  # Operation on pituitary stalk (procedure)
        "171964005",  # Excision of pineal gland (procedure)
        "17197003",  # Closure of cervicosigmoidal fistula (procedure)
        "171995003",  # Total parathyroidectomy and parathyroid tissue transplantation (procedure)
        "17200002",  # Ventricular shunt to nasopharynx (procedure)
        "172021003",  # Excision of bilateral adrenal glands with transposition of adrenal tissue (procedure)
        "172026008",  # Operation on aberrant adrenal tissue (procedure)
        "172027004",  # Excision of lesion of aberrant adrenal tissue (procedure)
        "172028009",  # Exploration of aberrant adrenal tissue (procedure)
        "17203000",  # Total hepatectomy (procedure)
        "172043006",  # Simple mastectomy (procedure)
        "172133006",  # Partial excision of bony orbit (procedure)
        "172275004",  # Excision of lesion of perirectal tissue (procedure)
        "17228008",  # Advancement of leaflet of heart valve (procedure)
        "17238003",  # Right ventricular resection for infundibular stenosis with commissurotomy (procedure)
        "172649009",  # Simple mastoidectomy (procedure)
        "172661001",  # Total petrosectomy (procedure)
        "172662008",  # Partial petrosectomy (procedure)
        "172797001",  # Septovomerine osteotomy (procedure)
        "172921000",  # Uncinectomy (procedure)
        "172922007",  # Uncinectomy and excision of bulla ethmoidalis (procedure)
        "172938001",  # Pharyngolaryngoesophagectomy (procedure)
        "172991009",  # Laryngofissure and cordectomy of vocal cord (procedure)
        "172997008",  # Excision of lesion of larynx using thyrotomy as approach (procedure)
        "172998003",  # Excision of lesion of larynx using lateral pharyngotomy as approach (procedure)
        "173005002",  # Excision of laryngeal stenosis and insertion of stent (procedure)
        "173027006",  # Endoscopic resection of lesion of larynx (procedure)
        "17309004",  # Excision of lesion of thoracoabdominal aorta with interposition graft replacement (procedure)
        "173170008",  # Bilobectomy of lung (procedure)
        "173171007",  # Lobectomy of lung (procedure)
        "173172000",  # Excision of segment of lung (procedure)
        "173178001",  # Open excision of lesion of lung (procedure)
        "173354001",  # Commando radical glossectomy (procedure)
        "17339009",  # Arteriectomy with anastomosis of abdominal artery (procedure)
        "173415001",  # Palatal osteotomy (procedure)
        "173458002",  # Reconstruction of mouth defect with free bowel flap (procedure)
        "173486005",  # Radical excision of parotid gland (procedure)
        "173560002",  # Esophagogastrectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173566008",  # Total esophagectomy and interposition of microvascularly attached colon (procedure)
        "173571001",  # Partial esophagectomy and end-to-end anastomosis of esophagus (procedure)
        "173572008",  # Partial esophagectomy and interposition of microvascularly attached jejunum (procedure)
        "173573003",  # Partial esophagectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173576006",  # Partial esophagectomy and interposition of microvascularly attached colon (procedure)
        "173586007",  # Bypass of esophagus by anastomosis of esophagus to esophagus (procedure)
        "173587003",  # Bypass of esophagus using stomach (procedure)
        "173588008",  # Bypass of esophagus by interposition of microvascularly attached jejunum (procedure)
        "173590009",  # Bypass of esophagus by interposition of microvascularly attached colon (procedure)
        "173600008",  # Conversion from previous direct anastomosis of esophagus (procedure)
        "173605003",  # Repair of rupture of esophagus (procedure)
        "173636006",  # Fiberoptic esophagoscopy and snare resection (procedure)
        "173649001",  # Enlargement of stoma of small intestine (procedure)
        "173657003",  # Rigid esophagoscopy and snare resection (procedure)
        "173685001",  # Repair of esophageal hiatus using abdominal approach (procedure)
        "173687009",  # Boerema repair of hiatus hernia (procedure)
        "173714001",  # Total gastrectomy and excision of surrounding tissue (procedure)
        "173715000",  # Total gastrectomy and interposition of jejunum (procedure)
        "173716004",  # Total gastrectomy and anastomosis of esophagus to transposed jejunum (procedure)
        "173720000",  # Partial gastrectomy and anastomosis of stomach to transposed jejunum (procedure)
        "173722008",  # Maki's pylorus-preserving gastrectomy (procedure)
        "173734000",  # Plastic operations on stomach (procedure)
        "173740007",  # Bypass of stomach by anastomosis of esophagus to duodenum (procedure)
        "173747005",  # Roux-en-Y gastrojejunostomy (procedure)
        "173748000",  # Bypass of stomach by anastomosis of stomach to transposed jejunum (procedure)
        "173750008",  # Conversion to anastomosis of stomach to transposed jejunum (procedure)
        "173751007",  # Conversion from previous anastomosis of stomach to transposed jejunum (procedure)
        "173794002",  # Pyloromyotomy and wedge resection (procedure)
        "173848007",  # Total excision of duodenum (procedure)
        "173849004",  # Partial excision of duodenum (procedure)
        "173852007",  # Open extirpation of lesion of duodenum (procedure)
        "173856005",  # Bypass of duodenum (procedure)
        "173857001",  # Bypass of duodenum by anastomosis of stomach to jejunum (procedure)
        "173858006",  # Bypass of duodenum by anastomosis of duodenum to duodenum (procedure)
        "173860008",  # Anastomosis of duodenum to colon (procedure)
        "173864004",  # Open biopsy of lesion of duodenum (procedure)
        "173866002",  # Open insertion of tubal prosthesis into duodenum (procedure)
        "173868001",  # Mobilization of duodenum (procedure)
        "173887007",  # Duodenal ulcer operation (procedure)
        "173889005",  # Closure of perforated duodenal ulcer (procedure)
        "173895006",  # Total jejunectomy and anastomosis of stomach to ileum (procedure)
        "173896007",  # Total jejunectomy and anastomosis of duodenum to ileum (procedure)
        "173897003",  # Total jejunectomy and anastomosis of duodenum to colon (procedure)
        "173898008",  # Partial jejunectomy and anastomosis of jejunum to ileum (procedure)
        "173899000",  # Partial jejunectomy and anastomosis of duodenum to colon (procedure)
        "173902002",  # Extirpation of lesion of jejunum (procedure)
        "173903007",  # Excision of lesion of jejunum (procedure)
        "173908003",  # Refashioning of jejunostomy (procedure)
        "173911002",  # Bypass of jejunum (procedure)
        "173913004",  # Bypass of jejunum by anastomosis of jejunum to ileum (procedure)
        "173922003",  # Open biopsy of lesion of jejunum (procedure)
        "173923008",  # Closure of perforation of jejunum (procedure)
        "173928004",  # Endoscopic extirpation of lesion of jejunum (procedure)
        "173941000",  # Transposition of jejunum (procedure)
        "173947001",  # Ileectomy and anastomosis of stomach to ileum (procedure)
        "173948006",  # Ileectomy and anastomosis of duodenum to ileum (procedure)
        "173949003",  # Ileectomy and anastomosis of ileum to ileum (procedure)
        "173950003",  # Ileectomy and anastomosis of ileum to colon (procedure)
        "173953001",  # Open extirpation of lesion of ileum (procedure)
        "173958005",  # Bypass of ileum (procedure)
        "173959002",  # Bypass of ileum by anastomosis of jejunum to ileum (procedure)
        "173960007",  # Bypass of ileum by anastomosis of ileum to ileum (procedure)
        "173961006",  # Bypass of ileum by anastomosis of ileum to cecum (procedure)
        "173962004",  # Ileotransverse colonic bypass (procedure)
        "173967005",  # Anastomosis of ileum to cecum (procedure)
        "173973006",  # Revision of anastomosis of ileum (procedure)
        "173974000",  # Closure of anastomosis of ileum (procedure)
        "173977007",  # Creation of ileostomy (procedure)
        "173980008",  # Creation of temporary ileostomy (procedure)
        "173985003",  # Refashioning of ileostomy (procedure)
        "173986002",  # Repair of prolapse of ileostomy (procedure)
        "173991001",  # Open reduction of intussusception of ileum (procedure)
        "173992008",  # Open relief of strangulation of ileum (procedure)
        "173994009",  # Plication of ileum (procedure)
        "173999004",  # Open biopsy of lesion of ileum (procedure)
        "174000002",  # Strictureplasty of ileum (procedure)
        "174002005",  # Closure of perforation of ileum (procedure)
        "174003000",  # Exclusion of segment of ileum (procedure)
        "174005007",  # Oversewing of small intestine (procedure)
        "174010006",  # Endoscopic extirpation of lesion of ileum (procedure)
        "174018004",  # Enterotomy and endoscopic examination of ileum (procedure)
        "174036004",  # Emergency appendectomy (procedure)
        "174039006",  # Emergency excision of normal appendix (procedure)
        "174041007",  # Laparoscopic emergency appendectomy (procedure)
        "174045003",  # Interval appendectomy (procedure)
        "174059005",  # Excision of colon and rectum (procedure)
        "174071004",  # Extended right hemicolectomy (procedure)
        "174072006",  # Extended right hemicolectomy and end-to-end anastomosis (procedure)
        "174073001",  # Extended right hemicolectomy and anastomosis of ileum to colon (procedure)
        "174080004",  # Right hemicolectomy and end to end anastomosis of ileum to colon (procedure)
        "174081000",  # Right hemicolectomy and side-to-side anastomosis of ileum to transverse colon (procedure)
        "174086005",  # Transverse colectomy and end-to-end anastomosis (procedure)
        "174087001",  # Transverse colectomy and anastomosis of ileum to colon (procedure)
        "174093009",  # Left hemicolectomy and end to end anastomosis of colon to rectum (procedure)
        "174094003",  # Left hemicolectomy and end-to-end anastomosis of colon to colon (procedure)
        "174100000",  # Sigmoid colectomy and end-to-end anastomosis of ileum to rectum (procedure)
        "174101001",  # Sigmoid colectomy and anastomosis of colon to rectum (procedure)
        "174116003",  # Extirpation of lesion of colon (procedure)
        "174121000",  # Bypass of colon (procedure)
        "174123002",  # Bypass of colon by anastomosis of ileum to colon (procedure)
        "174124008",  # Bypass of colon by anastomosis of cecum to sigmoid colon (procedure)
        "174125009",  # Bypass of colon by anastomosis of transverse colon to sigmoid colon (procedure)
        "174126005",  # Bypass of colon by anastomosis of transverse colon to rectum (procedure)
        "174132000",  # Refashioning of cecostomy (procedure)
        "174149007",  # Open reduction of intussusception of colon (procedure)
        "174150007",  # Open reduction of volvulus of cecum (procedure)
        "174151006",  # Open reduction of volvulus of sigmoid colon (procedure)
        "174153009",  # Open relief of strangulation of colon (procedure)
        "174162006",  # Open biopsy of lesion of colon (procedure)
        "174164007",  # Enterorrhaphy of colon (procedure)
        "174165008",  # Open removal of foreign body from colon (procedure)
        "174171002",  # Fiberoptic endoscopic snare resection of lesion of colon (procedure)
        "174179000",  # Fiberoptic endoscopic dilation of colon (procedure)
        "174190005",  # Endoscopic snare resection of lesion of lower bowel using fiberoptic sigmoidoscope (procedure)
        "174197008",  # Endoscopic dilation of lower bowel using fiberoptic sigmoidoscope (procedure)
        "174207007",  # Endoscopic snare resection of lesion of sigmoid colon using rigid sigmoidoscope (procedure)
        "174215005",  # Endoscopic dilation of sigmoid colon using rigid sigmoidoscope (procedure)
        "174240003",  # Proctectomy and anastomosis of colon to anus (procedure)
        "174241004",  # Anterior resection of rectum and anastomosis of colon to rectum using staples (procedure)
        "174247000",  # Open excision of lesion of rectum (procedure)
        "174269007",  # Operation on rectum via anal sphincter (procedure)
        "174270008",  # Transsphincteric excision of mucosa of rectum (procedure)
        "174273005",  # Transsphincteric anastomosis of colon to anus (procedure)
        "174278001",  # Rectosigmoidectomy and peranal anastomosis (procedure)
        "174286001",  # Perineal operations for rectal prolapse (procedure)
        "174288000",  # Perianal plication of levator ani muscles and anal sphincters (procedure)
        "174312005",  # Reversal of Hartmann's procedure (procedure)
        "174320007",  # Proctoscopic extirpation of lesion of rectum (procedure)
        "174321006",  # Proctoscopic excision of lesion of rectum (procedure)
        "174349004",  # Reanastomosis of rectum to anal canal for correction of congenital atresia of rectum (procedure)
        "17437005",  # Gas endarterectomy of thoracic artery (procedure)
        "174416009",  # Closure of bowel fistula (procedure)
        "174425003",  # Orthotopic liver transplant (procedure)
        "174426002",  # Heterotopic liver transplant (procedure)
        "174427006",  # Replacement of previous liver transplant (procedure)
        "174430004",  # Resection of segment of liver (procedure)
        "174431000",  # Wedge resection of liver (procedure)
        "174432007",  # Marsupialization of liver lesion (procedure)
        "174439003",  # Removal of lacerated fragment of liver (procedure)
        "174444005",  # Open removal of calculus from liver (procedure)
        "174445006",  # Open wedge biopsy of lesion of liver (procedure)
        "174449000",  # Open devascularization of liver (procedure)
        "174457002",  # Laparoscopic removal of calculus from liver (procedure)
        "174473008",  # Percutaneous removal of calculus from liver (procedure)
        "174494006",  # Total cholecystectomy and excision of surrounding tissue (procedure)
        "174495007",  # Total cholecystectomy and exploration of common bile duct (procedure)
        "174507008",  # Closure of fistula of gallbladder (procedure)
        "174542007",  # Excision of ampulla of Vater and replantation of common bile duct into duodenum (procedure)
        "174543002",  # Partial excision of bile duct and anastomosis of bile duct to duodenum (procedure)
        "174545009",  # Partial excision of bile duct and anastomosis of bile duct to jejunum (procedure)
        "174561006",  # Anastomosis main bile duct to duodenum (procedure)
        "174562004",  # Anastomosis of common bile duct to transposed jejunum (procedure)
        "174588001",  # Plastic repair of sphincter of Oddi using duodenal approach (procedure)
        "174589009",  # Sphincteroplasty of bile duct and pancreatic duct using duodenal approach (procedure)
        "174595005",  # Revision of stoma of small intestine (procedure)
        "174596006",  # Sphincterotomy of bile duct and pancreatic duct using duodenal approach (procedure)
        "174602009",  # Transduodenal excision of ampulla of Vater (procedure)
        "174613007",  # Endoscopic incision of sphincter of Oddi (procedure)
        "174615000",  # Endoscopic sphincterotomy of sphincter of Oddi and insertion of tubal prosthesis into bile duct (procedure)
        "174619006",  # Endoscopic sphincterotomy of accessory ampulla of Vater (procedure)
        "174639005",  # Endoscopic retrograde dilation of pancreatic duct (procedure)
        "174691005",  # Transplantation of pancreas and duodenum (procedure)
        "174692003",  # Whole organ pancreatic transplant (procedure)
        "174693008",  # Transplantation of tail of pancreas (procedure)
        "174694002",  # Renewal of transplanted pancreatic tissue (procedure)
        "174697009",  # Total pancreatectomy and excision of surrounding tissue (procedure)
        "174699007",  # Excision of transplanted pancreas (procedure)
        "174702000",  # Proximal pancreatectomy (procedure)
        "174710004",  # Left pancreatectomy and drainage of pancreatic duct (procedure)
        "174712007",  # Excision of tail of pancreas and drainage of pancreatic duct (procedure)
        "174717001",  # Extirpation of lesion of pancreas (procedure)
        "174718006",  # Excision of lesion of islet of Langerhans (procedure)
        "174720009",  # Destruction of lesion of pancreas (procedure)
        "174726003",  # Revision of anastomosis of pancreatic duct (procedure)
        "174727007",  # Closure of anastomosis of pancreatic duct (procedure)
        "17473003",  # Cecotomy (procedure)
        "174733003",  # Open insertion of pancreatic duct stent (procedure)
        "174734009",  # Open dilation of pancreatic duct (procedure)
        "174735005",  # Repair of pancreatic duct (procedure)
        "174740002",  # Open cystogastrostomy of pancreas (procedure)
        "174741003",  # Roux-en-Y drainage of pancreatic cyst (procedure)
        "174746008",  # Division of annular pancreas (procedure)
        "174757003",  # Open biopsy of lesion of pancreas (procedure)
        "174758008",  # Suture of pancreas (procedure)
        "174776001",  # Total splenectomy (procedure)
        "174778000",  # Total splenectomy and reimplantation of fragments (procedure)
        "17478007",  # Closure of pancreatic fistula (procedure)
        "174802006",  # Allotransplant of heart and lung (procedure)
        "174808005",  # Xenotransplant of heart (procedure)
        "174809002",  # Heterotopic allotransplant of heart (procedure)
        "17481002",  # Major thoracotomy with removal of fibrin deposit (procedure)
        "174815002",  # Correction of tetralogy of Fallot using valved right ventricular outflow conduit (procedure)
        "174822005",  # Atrial inversion operation using atrial wall (procedure)
        "174826008",  # Arterial switch operation (procedure)
        "174836000",  # Repair of defect of atrioventricular septum (procedure)
        "174837009",  # Closure of defect of atrioventricular septum using dual prosthetic patches (procedure)
        "174839007",  # Closure of defect of atrioventricular septum using tissue graft (procedure)
        "174841008",  # Revision of closure of defect of atrioventricular septum (procedure)
        "174844000",  # Closure of defect of interatrial septum using prosthetic patch (procedure)
        "174845004",  # Closure of defect of interatrial septum using pericardial patch (procedure)
        "174848002",  # Revision of closure of defect of interatrial septum (procedure)
        "174851009",  # Closure of defect of interventricular septum using pericardial patch (procedure)
        "174854001",  # Revision of closure of defect of interventricular septum (procedure)
        "174866007",  # Open enlargement of atrial septal defect (procedure)
        "174870004",  # Closed operations on septum of heart (procedure)
        "174880000",  # Creation of valved cardiac conduit (procedure)
        "174881001",  # Creation of valved conduit between atrium and ventricle of heart (procedure)
        "174882008",  # Creation of valved conduit between right atrium and pulmonary artery (procedure)
        "174883003",  # Creation of valved conduit between right ventricle of heart and pulmonary artery (procedure)
        "174884009",  # Creation of valved conduit between left ventricle of heart and aorta (procedure)
        "174898001",  # Refashioning of atrium (procedure)
        "174911007",  # Revascularization of wall of heart (procedure)
        "174916002",  # Valves of heart and adjacent structures operations (procedure)
        "174918001",  # Homograft mitral valve replacement (procedure)
        "174919009",  # Xenograft mitral valve replacement (procedure)
        "174927000",  # Homograft aortic valve replacement (procedure)
        "174928005",  # Xenograft aortic valve replacement (procedure)
        "174929002",  # Mechanical prosthetic aortic valve replacement (procedure)
        "174935002",  # Homograft tricuspid valve replacement (procedure)
        "174936001",  # Xenograft tricuspid valve replacement (procedure)
        "174937005",  # Mechanical prosthetic tricuspid valve replacement (procedure)
        "174939008",  # Repair of Ebstein's anomaly of tricuspid valve (procedure)
        "174944001",  # Homograft pulmonary valve replacement (procedure)
        "174945000",  # Xenograft pulmonary valve replacement (procedure)
        "174946004",  # Mechanical prosthetic pulmonary valve replacement (procedure)
        "174959007",  # Revision of plastic repair of valve of heart (procedure)
        "174960002",  # Revision of plastic repair of mitral valve (procedure)
        "174961003",  # Revision of plastic repair of aortic valve (procedure)
        "174962005",  # Revision of plastic repair of tricuspid valve (procedure)
        "174963000",  # Revision of plastic repair of pulmonary valve (procedure)
        "174966008",  # Open cardiac valvotomy (procedure)
        "174967004",  # Open mitral valvotomy (procedure)
        "174968009",  # Open aortic valvotomy (procedure)
        "174969001",  # Open tricuspid valvotomy (procedure)
        "174970000",  # Open pulmonary valvotomy (procedure)
        "174973003",  # Closed cardiac valvotomy (procedure)
        "174980001",  # Annuloplasty of mitral valve (procedure)
        "174983004",  # Excision of vegetations of valve of heart (procedure)
        "174987003",  # Percutaneous transluminal mitral valvotomy (procedure)
        "174988008",  # Percutaneous transluminal aortic valvotomy (procedure)
        "174989000",  # Percutaneous transluminal tricuspid valvotomy (procedure)
        "174990009",  # Percutaneous transluminal pulmonary valvotomy (procedure)
        "174994000",  # Removal of obstruction from structure adjacent to valve of heart (procedure)
        "174995004",  # Infundibulectomy of heart using patch (procedure)
        "174997007",  # Operations on the left ventricular outflow tract (procedure)
        "175021005",  # Allograft bypass of coronary artery (procedure)
        "175029007",  # Prosthetic bypass of coronary artery (procedure)
        "175036008",  # Revision of bypass for coronary artery (procedure)
        "175037004",  # Revision of bypass for one coronary artery (procedure)
        "175038009",  # Revision of bypass for two coronary arteries (procedure)
        "175039001",  # Revision of bypass for three coronary arteries (procedure)
        "175040004",  # Revision of bypass for four or more coronary arteries (procedure)
        "175042007",  # Revision of implantation of thoracic artery into heart (procedure)
        "175047001",  # Double implantation of mammary arteries into coronary arteries (procedure)
        "175048006",  # Single anastomosis of mammary artery to left anterior descending coronary artery (procedure)
        "175050003",  # Single implantation of mammary artery into coronary artery (procedure)
        "175061006",  # Transection of coronary artery muscle bridge (procedure)
        "175071008",  # Percutaneous transluminal coronary thrombolysis using streptokinase (procedure)
        "175085003",  # Replacement of coronary arteries using multiple methods (procedure)
        "175094009",  # Open operations on conducting system of heart (procedure)
        "175095005",  # Open ablation of atrioventricular node (procedure)
        "175096006",  # Epicardial excision of rhythmogenic focus (procedure)
        "175097002",  # Endocardial excision of rhythmogenic focus (procedure)
        "175098007",  # Open division of accessory pathway within heart (procedure)
        "175117000",  # Transluminal removal of heart assist system (procedure)
        "175125003",  # Percutaneous transluminal ablation of atrioventricular node (procedure)
        "175140001",  # Complete removal of implanted cardiac pacemaker system (procedure)
        "175189006",  # Pericardial biopsy (procedure)
        "175211006",  # Open correction of patent ductus arteriosus (procedure)
        "175212004",  # Ligation of ductus arteriosus (procedure)
        "175214003",  # Revision of correction of patent ductus arteriosus (procedure)
        "175218000",  # Transluminal closure of ductus arteriosus with prosthesis (procedure)
        "175221003",  # Construction of central aortopulmonary interposition shunt (procedure)
        "175223000",  # Creation of shunt from ascending aorta to right pulmonary artery using interposition tube prosthesis (procedure)
        "175224006",  # Creation of shunt from ascending aorta to left pulmonary artery using interposition tube prosthesis (procedure)
        "175228009",  # Construction of aortopulmonary window (procedure)
        "175236000",  # Creation of shunt from subclavian artery to pulmonary artery using interposition tube prosthesis (procedure)
        "175237009",  # Creation of shunt from right subclavian artery to right pulmonary artery using interposition tube prosthesis (procedure)
        "175238004",  # Creation of shunt from left subclavian artery to left pulmonary artery using interposition tube prosthesis (procedure)
        "175249005",  # Anastomosis of vena cava to pulmonary artery (procedure)
        "175253007",  # Repair of pulmonary artery (procedure)
        "175254001",  # Repair of pulmonary artery using prosthesis (procedure)
        "175260001",  # Adjustment of pulmonary artery band (procedure)
        "175261002",  # Removal of band from pulmonary artery (procedure)
        "175262009",  # Open pulmonary embolectomy (procedure)
        "175266007",  # Transluminal operations on pulmonary artery (procedure)
        "175268008",  # Percutaneous embolectomy of pulmonary artery (procedure)
        "175275009",  # Extra-anatomic bypass of aorta (procedure)
        "175276005",  # Emergency bypass of aorta by anastomosis of axillary artery to femoral artery (procedure)
        "175279003",  # Axillo-unifemoral polytetrafluoroethylene bypass graft (procedure)
        "175282008",  # Repair of aortic aneurysm using bifurcation graft (procedure)
        "175283003",  # Emergency replacement of aneurysmal bifurcation of aorta by anastomosis of aorta to femoral artery (procedure)
        "175284009",  # Replacement of aneurysmal bifurcation of aorta by anastomosis of aorta to femoral artery (procedure)
        "175290008",  # Emergency bypass of bifurcation of aorta by anastomosis of aorta to femoral artery (procedure)
        "175292000",  # Emergency bypass of bifurcation of aorta by anastomosis of aorta to iliac artery (procedure)
        "175297006",  # Emergency replacement of aneurysmal segment of aorta (procedure)
        "175298001",  # Emergency replacement of aneurysmal segment of ascending aorta by anastomosis of aorta to aorta (procedure)
        "175299009",  # Emergency replacement of aneurysmal segment of thoracic aorta by anastomosis of aorta to aorta (procedure)
        "175300001",  # Emergency replacement of aneurysmal segment of suprarenal abdominal aorta by anastomosis of aorta to aorta (procedure)
        "175301002",  # Emergency replacement of aneurysmal segment of infrarenal abdominal aorta by anastomosis of aorta to aorta (procedure)
        "175331008",  # Revision of prosthesis of thoracic aorta (procedure)
        "175332001",  # Revision of prosthesis of bifurcation of aorta (procedure)
        "175334000",  # Removal of prosthesis from aorta (procedure)
        "175337007",  # Plastic repair of aorta (procedure)
        "175338002",  # Plastic repair of aorta and end-to-end anastomosis of aorta (procedure)
        "175339005",  # Repair of aorta with subclavian flap (procedure)
        "175340007",  # Patch repair of aorta (procedure)
        "175342004",  # Revision of plastic repair of aorta (procedure)
        "175346001",  # Endarterectomy of aorta and patch repair of aorta (procedure)
        "175348000",  # Open embolectomy of bifurcation of aorta (procedure)
        "175354004",  # Percutaneous transluminal balloon angioplasty of aorta (procedure)
        "175356002",  # Percutaneous transluminal embolectomy of bifurcation of aorta (procedure)
        "175365009",  # Intracranial bypass to carotid artery (procedure)
        "175367001",  # Endarterectomy of carotid artery and patch repair of carotid artery (procedure)
        "175385008",  # Excision of aneurysm of cerebral artery (procedure)
        "175386009",  # Excision of aneurysm of circle of Willis (procedure)
        "175387000",  # Clipping of aneurysm of circle of Willis (procedure)
        "175395001",  # Reconstruction of cerebral artery (procedure)
        "175396000",  # Reconstruction of circle of Willis (procedure)
        "175397009",  # Anastomosis of cerebral artery (procedure)
        "175398004",  # Anastomosis of circle of Willis (procedure)
        "175399007",  # Open embolectomy of cerebral artery (procedure)
        "175400000",  # Open embolectomy of circle of Willis (procedure)
        "175418002",  # Endarterectomy of vertebral artery (procedure)
        "175419005",  # Endarterectomy of subclavian artery and patch repair of subclavian artery (procedure)
        "175425009",  # Endarterectomy of vertebral artery and patch repair of vertebral artery (procedure)
        "175429003",  # Reconstruction of axillary artery with vein graft (procedure)
        "175430008",  # Reconstruction of brachial artery with vein graft (procedure)
        "175445001",  # Ligation of axillary artery (procedure)
        "175472009",  # Branches of abdominal aorta operations (procedure)
        "175474005",  # Reconstruction of renal artery (procedure)
        "175475006",  # Plastic repair of renal artery and end-to-end anastomosis of renal artery (procedure)
        "175476007",  # Reimplantation of renal artery (procedure)
        "175477003",  # Endarterectomy of renal artery (procedure)
        "175482005",  # Open embolectomy of renal artery (procedure)
        "175483000",  # Ligation of renal artery (procedure)
        "175485007",  # Operation on aneurysm of renal artery (procedure)
        "175489001",  # Percutaneous transluminal embolectomy of renal artery (procedure)
        "175557007",  # Repair of stoma of small intestine (procedure)
        "175562008",  # Emergency replacement of aneurysmal iliac artery by anastomosis of iliac artery to femoral artery (procedure)
        "175564009",  # Emergency replacement of aneurysmal iliac artery by anastomosis of femoral artery to femoral artery (procedure)
        "175566006",  # Emergency replacement of aneurysmal common iliac artery by anastomosis of aorta to common iliac artery (procedure)
        "175567002",  # Emergency replacement of aneurysmal iliac artery by anastomosis of aorta to external iliac artery (procedure)
        "175568007",  # Emergency replacement of aneurysmal artery of leg by anastomosis of aorta to common femoral artery (procedure)
        "175569004",  # Emergency replacement of aneurysmal artery of leg by anastomosis of aorta to superficial femoral artery (procedure)
        "175570003",  # Emergency replacement of aneurysmal iliac artery by anastomosis of iliac artery to iliac artery (procedure)
        "175594003",  # Iliofemoral prosthetic crossover graft (procedure)
        "175612005",  # Percutaneous transluminal angioplasty of iliac artery (procedure)
        "175613000",  # Percutaneous transluminal embolectomy of iliac artery (procedure)
        "175615007",  # Insertion of iliac artery stent (procedure)
        "175685004",  # Femorofemoral prosthetic crossover graft (procedure)
        "175693004",  # Profundoplasty of femoral artery and patch repair of deep femoral artery (procedure)
        "175697003",  # Reconstruction of femoral artery with vein graft (procedure)
        "175698008",  # Reconstruction of popliteal artery with vein graft (procedure)
        "17571000087103",  # Angioplasty of left iliac artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "175730007",  # Revision of reconstruction involving aorta (procedure)
        "175731006",  # Revision of reconstruction involving iliac artery (procedure)
        "175791008",  # Connection of vena cava or branch of vena cava (procedure)
        "175793006",  # Mesocaval shunt (procedure)
        "175798002",  # Plication of vena cava (procedure)
        "175803006",  # Bypass operation for priapism (procedure)
        "17581000087101",  # Angioplasty of right iliac artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "175843001",  # Open removal of thrombus from vein (procedure)
        "175844007",  # Open thrombectomy of vein of upper limb (procedure)
        "175845008",  # Open thrombectomy of vein of lower limb (procedure)
        "175853000",  # Removal of central venous line (procedure)
        "175891000",  # Prosthetic graft thrombectomy (procedure)
        "175898006",  # Kidney operation (procedure)
        "175899003",  # Autotransplantation of kidney (procedure)
        "175901007",  # Live donor renal transplant (procedure)
        "175902000",  # Cadaveric renal transplant (procedure)
        "175905003",  # Total nephrectomy (procedure)
        "175907006",  # Nephroureterectomy (procedure)
        "175908001",  # Heminephrectomy for horseshoe kidney (procedure)
        "17591000087104",  # Angioplasty of left renal artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "175911000",  # Live donor nephrectomy (procedure)
        "175912007",  # Nephroureterectomy with open lower ureterectomy (procedure)
        "175913002",  # Nephroureterectomy with pluck lower ureterectomy (procedure)
        "175916005",  # Heminephrectomy for duplex kidney (procedure)
        "175917001",  # Upper polar partial nephrectomy (procedure)
        "175918006",  # Lower polar partial nephrectomy (procedure)
        "175921008",  # Open removal or destruction of renal lesion (procedure)
        "175922001",  # Open deroofing of renal cyst (procedure)
        "175923006",  # Open excision of renal lesion (procedure)
        "175924000",  # Open destruction of renal lesion (procedure)
        "175925004",  # Rovsing's operation (procedure)
        "175930000",  # Revision of pyeloplasty (procedure)
        "175931001",  # Plication of kidney (procedure)
        "175933003",  # Anderson-Hynes pyeloplasty (procedure)
        "175939004",  # Therapeutic open aspiration of renal cyst (procedure)
        "175958008",  # Endoscopic extirpation of lesion of kidney (procedure)
        "176003",  # Anastomosis of rectum (procedure)
        "176004009",  # Construction of continent ileal conduit (procedure)
        "176006006",  # Construction of colonic conduit (procedure)
        "176008007",  # Insertion of subcutaneous urinary diversion stent (procedure)
        "17601000087107",  # Angioplasty of right renal artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "176106009",  # Radical cystoprostatourethrectomy (procedure)
        "176107000",  # Radical cystourethrectomy - female (procedure)
        "176108005",  # Simple cystectomy (procedure)
        "176123000",  # Clam ileocystoplasty (procedure)
        "176143006",  # Open excision of lesion of urinary bladder (procedure)
        "176155000",  # Rigid cystoscopy and transurethral resection of lesion of urinary bladder (procedure)
        "176158003",  # Flexible cystoscopic excision of lesion of urinary bladder (procedure)
        "176234008",  # Open resection of female urinary bladder neck (procedure)
        "176246007",  # Transurethral excision of female urinary bladder neck (procedure)
        "176261008",  # Radical prostatectomy without pelvic node excision (procedure)
        "176262001",  # Radical prostatectomy with pelvic node sampling (procedure)
        "176263006",  # Radical prostatectomy with pelvic lymphadenectomy (procedure)
        "176275001",  # Open resection of male urinary bladder neck (procedure)
        "176415006",  # Subcapsular excision of bilateral testes (procedure)
        "176416007",  # Total excision of bilateral testes (procedure)
        "176417003",  # Total inguinal excision of bilateral testes (procedure)
        "176418008",  # Total transcrotal excision of bilateral testes (procedure)
        "176423008",  # Excision of ectopic testis (procedure)
        "176430002",  # Excision of lesion of testis (procedure)
        "176524000",  # Total amputation of penis (procedure)
        "176563007",  # Distal shunt for priapism (procedure)
        "176564001",  # Proximal shunt for priapism (procedure)
        "176597000",  # Total vulvectomy (procedure)
        "176697007",  # Repair of vaginal prolapse and amputation of cervix uteri (procedure)
        "17671008",  # Laparoscopic-assisted anterior resection of rectum with colostomy (procedure)
        "176722004",  # Abdominal repair vesicovaginal fistula (procedure)
        "176795006",  # Subtotal abdominal hysterectomy (procedure)
        "176873000",  # Transcervical resection endometrium (procedure)
        "176895001",  # Vaginal excision of lesion of uterus (procedure)
        "176901003",  # Excision of bilateral adnexa of uterus (procedure)
        "176914002",  # Right salpingo-oophorectomy (procedure)
        "176915001",  # Left salpingo-oophorectomy (procedure)
        "176916000",  # Right salpingectomy (procedure)
        "176917009",  # Left salpingectomy (procedure)
        "176928008",  # Excision of ectopic ovarian pregnancy (procedure)
        "1770009",  # Mitral valvotomy (procedure)
        "177015007",  # Marsupialization of lesion of ovary (procedure)
        "177022004",  # Replantation of ovary (procedure)
        "177023009",  # Suture ruptured corpus luteum (procedure)
        "177030003",  # Open biopsy of lesion of ovary (procedure)
        "177032006",  # Oophorotomy and drainage of abscess (procedure)
        "177046006",  # Endoscopic extirpation of lesion of ovary (procedure)
        "177047002",  # Endoscopic freeing of adhesions of ovary (procedure)
        "177055009",  # Excision of lesion of broad ligament of uterus (procedure)
        "177220003",  # Immediate repair of obstetric laceration of vagina and floor of pelvis (procedure)
        "17737009",  # Femoral-femoral artery bypass graft with vein (procedure)
        "17744000",  # Subtotal hysterectomy after cesarean delivery (procedure)
        "17751000087102",  # Angioplasty of bilateral iliac arteries and insertion of bilateral iliac artery stents using fluoroscopic guidance with contrast (procedure)
        "177654009",  # Escharotomy of burnt skin of chest (procedure)
        "177765008",  # Opening of chest (procedure)
        "177766009",  # Exploratory median sternotomy (procedure)
        "177768005",  # Reopening of chest and re-exploration of intra-abdominal operation site and surgical arrest of postoperative bleeding (procedure)
        "177784006",  # Decortication of lung (procedure)
        "177835007",  # Repair of rupture of the diaphragm (procedure)
        "177836008",  # Repair of traumatic rupture of the diaphragm (procedure)
        "177837004",  # Repair of postoperative rupture of the diaphragm (procedure)
        "177846005",  # Destruction of lesion of diaphragm (procedure)
        "177940003",  # Open extirpation of lesion of peritoneum (procedure)
        "177956008",  # Reopening of abdomen and re-exploration of intra-abdominal operation site and surgical arrest of postoperative bleeding (procedure)
        "178054006",  # Radical palmar fasciectomy (procedure)
        "178373004",  # Exploratory open craniotomy (procedure)
        "178375006",  # Exploratory burrhole of cranium (procedure)
        "178376007",  # Reopening of cranium and re-exploration of intracranial operation site and surgical arrest of postoperative bleeding (procedure)
        "178387004",  # Excision of base of skull in anterior cranial fossa (procedure)
        "178388009",  # Excision of base of skull in middle cranial fossa (procedure)
        "178389001",  # Excision of base of skull in posterior cranial fossa (procedure)
        "178390005",  # Strip craniectomy (procedure)
        "178391009",  # Cranial morcellization (procedure)
        "178421004",  # Total maxillectomy (procedure)
        "178422006",  # Partial maxillectomy (procedure)
        "178423001",  # Extended maxillectomy (procedure)
        "178477002",  # Le Fort III osteotomy (procedure)
        "178478007",  # Low level osteotomy of maxilla (procedure)
        "178479004",  # Osteotomy of alveolar segment of maxilla (procedure)
        "178481002",  # Osteotomy of mandible and advancement of mandible (procedure)
        "178482009",  # Osteotomy of mandible and retrusion of mandible (procedure)
        "178483004",  # Osteotomy of alveolar segment of mandible (procedure)
        "178484005",  # Premaxillary osteotomy (procedure)
        "178485006",  # Sagittal split mandibular osteotomy (procedure)
        "178486007",  # Vertical sub-sigmoid mandibular osteotomy (procedure)
        "17881005",  # Cauterization of choroid plexus (procedure)
        "17882003",  # Direct laryngoscopy with arytenoidectomy (procedure)
        "179294005",  # Conversion to cemented total hip replacement (procedure)
        "179305003",  # Conversion to uncemented total hip replacement (procedure)
        "179320009",  # Conversion to cemented hemiarthroplasty of hip (procedure)
        "179327007",  # Conversion to uncemented hemiarthroplasty of hip (procedure)
        "179345007",  # Conversion to cemented total knee replacement (procedure)
        "179352009",  # Conversion to uncemented total knee replacement (procedure)
        "179353004",  # Revision of uncemented total prosthetic replacement of knee joint (procedure)
        "179373009",  # Open repair medial meniscus (procedure)
        "179374003",  # Open repair lateral meniscus (procedure)
        "179402001",  # Cemented unicompartmental knee replacement (procedure)
        "179404000",  # Conversion to cemented unicompartmental knee replacement (procedure)
        "179407007",  # Uncemented unicompartmental knee replacement (procedure)
        "179409005",  # Conversion to uncemented unicompartmental knee replacement (procedure)
        "179412008",  # Hybrid unicompartmental knee replacement (procedure)
        "179414009",  # Conversion to hybrid unicompartmental knee replacement (procedure)
        "179415005",  # Revision hybrid unicompartmental knee replacement (procedure)
        "179422002",  # Conversion to cemented hemiarthroplasty of shoulder (procedure)
        "179429006",  # Conversion to uncemented hemiarthroplasty of shoulder (procedure)
        "179442006",  # Cemented prosthetic total shoulder replacement (procedure)
        "179444007",  # Conversion to cemented total prosthetic shoulder replacement (procedure)
        "179447000",  # Uncemented prosthetic total arthroplasty of shoulder joint (procedure)
        "179449002",  # Conversion to uncemented total shoulder replacement (procedure)
        "179452005",  # Prosthetic hybrid total shoulder replacement (procedure)
        "17954009",  # Replacement of transvenous atrial pacemaker electrode leads (procedure)
        "17962001",  # Radical resection of tumor of soft tissue of neck (procedure)
        "17988003",  # Partial ileectomy (procedure)
        "179959009",  # Prosthetic hybrid total elbow replacement (procedure)
        "180000003",  # Autotransplantation of adrenal medulla to caudate nucleus of brain (procedure)
        "180011006",  # Above elbow amputation (procedure)
        "180015002",  # Amputation of hand, thumb or finger (procedure)
        "180030006",  # Amputation of the foot (procedure)
        "18008000",  # Closure of vesicovaginal fistula by vaginal approach (procedure)
        "18027006",  # Transplantation of liver (procedure)
        "18047003",  # Closure of small intestine fistula (procedure)
        "18078004",  # Excision of lesion of ascending aorta (procedure)
        "18133004",  # Child operation, radical subtotal pancreatectomy (procedure)
        "18160006",  # Excision of lesion of thoracic artery (procedure)
        "18223002",  # Mastoidotomy (procedure)
        "182622002",  # Incisional biopsy of brain (procedure)
        "18280002",  # Repair of vesicourethrorectal fistula (procedure)
        "18286008",  # Catheter ablation of tissue of heart (procedure)
        "183579007",  # Open excisional biopsy of brain (procedure)
        "1836002",  # Excision of diverticulum of ventricle of heart (procedure)
        "18412003",  # Laparotomy for staging of ovarian cancer (procedure)
        "18455004",  # Closure of bronchovisceral fistula (procedure)
        "18522006",  # Donor cardiectomy-pneumonectomy with preparation and maintenance of homograft (procedure)
        "18561003",  # Closure of enterocolic fistula (procedure)
        "185876009",  # Enteroplasty (procedure)
        "18623006",  # Vascular subclavian-axillary bypass (procedure)
        "1866009",  # Total excision of bilateral kidneys (procedure)
        "186648008",  # Repair of intestine (procedure)
        "18674003",  # Creation of carotid-vertebral artery shunt (procedure)
        "1868005",  # Removal of foreign body from brain (procedure)
        "18692006",  # Bypass gastroenterostomy (procedure)
        "1871002",  # Repair of aneurysm of coronary artery (procedure)
        "18878003",  # Hepatic portoenterostomy (procedure)
        "18887007",  # Excision with graft replacement of abdominal artery (procedure)
        "189009",  # Excision of lesion of artery (procedure)
        "18918008",  # Direct thrombectomy of subclavian vein by arm incision (procedure)
        "18932005",  # Rastelli operation in repair of transposition of great vessels (procedure)
        "19063003",  # Arthroplasty of knee (procedure)
        "19080004",  # Closure of abdominothoracic fistula (procedure)
        "19235006",  # Tractotomy of brain (procedure)
        "19262003",  # Repair of thoracic aortic aneurysm (procedure)
        "193289008",  # Dilation of pancreatic sphincter (procedure)
        "19463006",  # Transplantation of retroperitoneal tissue (procedure)
        "195115001",  # Duodenocholedochotomy (procedure)
        "195969003",  # Choledochoduodenostomy (procedure)
        "19606007",  # Endarterectomy of aortic arch (procedure)
        "19612002",  # Hepatocholangiocystoduodenostomy (procedure)
        "196281000",  # Excisional biopsy of heart (procedure)
        "19639005",  # Sigmoidopexy (procedure)
        "19725004",  # Correction of esophageal atresia (procedure)
        "197537000",  # Subarachnoid-ureteral spinal shunt with valve (procedure)
        "197889000",  # Laminectomy with excision of intraspinal lesion (procedure)
        "1983001",  # Total urethrectomy including cystostomy in female (procedure)
        "198323007",  # Subarachnoid-ureteral spinal shunt (procedure)
        "19953008",  # Thromboendarterectomy with graft of abdominal aorta (procedure)
        "19979004",  # Excision of lesion of atrium (procedure)
        "20101008",  # Anastomosis of renal pelvis (procedure)
        "20119004",  # Ureterocecostomy (procedure)
        "20122002",  # Laryngeal ventriculocordectomy (procedure)
        "20142006",  # Excision of accessory or ectopic lung tissue (procedure)
        "20157004",  # Incision of pineal gland (procedure)
        "20176002",  # Binnie operation, hepatopexy (procedure)
        "20241005",  # Closure of gastroduodenostomy (procedure)
        "20246000",  # Fixation of intestine (procedure)
        "20294001",  # Excision of lesion of artery of lower limb (procedure)
        "20454000",  # Aortic valve commissurotomy with inflow occlusion (procedure)
        "20460000",  # Closure of gastroenterocolic fistula (procedure)
        "20492004",  # Thrombectomy with catheter of femoropopliteal artery by leg incision (procedure)
        "20500004",  # Percutaneous transluminal iliac artery balloon angioplasty (procedure)
        "20504008",  # Enterostomy, delayed opening (procedure)
        "20534003",  # Aortopopliteal vascular bypass (procedure)
        "20702009",  # Intracranial phlebectomy with graft replacement (procedure)
        "20738002",  # Endarterectomy of coronary artery (procedure)
        "20767001",  # Shunt of portal vein to vena cava (procedure)
        "2079001",  # Reimplantation of anomalous pulmonary artery (procedure)
        "2080003",  # Angiectomy with anastomosis of lower limb artery (procedure)
        "20874001",  # Complete excision of anus (procedure)
        "208841009",  # Frazier operation (procedure)
        "20912009",  # Craniotomy for decompression of skull fracture (procedure)
        "20934006",  # Gas endarterectomy with patch graft (procedure)
        "209846007",  # Trigeminal rhizotomy (procedure)
        "21057005",  # Manipulation of coronary artery (procedure)
        "210707000",  # Chopart type procedure (procedure)
        "21089001",  # Laminectomy for implantation of neurostimulator electrodes, subdural (procedure)
        "21118000",  # Destructive procedure on pancreas (procedure)
        "211570009",  # Amputation through foot (procedure)
        "211601002",  # Excisional biopsy of artery (procedure)
        "21228001",  # Lysis of adhesions of ovary (procedure)
        "212442006",  # Lisfranc foot amputation (procedure)
        "21245006",  # Tube cecostomy (procedure)
        "21260009",  # Gas endarterectomy of head and neck artery (procedure)
        "213041003",  # Hey foot amputation (procedure)
        "21344007",  # Vascular bypass of renal artery with graft (procedure)
        "2137005",  # Excision of pericardial tumor (procedure)
        "21400005",  # Transection of vagus nerve by abdominal approach (procedure)
        "214046003",  # Hindfoot amputation (procedure)
        "214084009",  # Excisional biopsy of skull (procedure)
        "214173000",  # Excision of lesion of uterine ligament (procedure)
        "21482000",  # Closure of enterovesical fistula with bowel resection (procedure)
        "21484004",  # Popliteal-tibial artery bypass graft with vein (procedure)
        "21516005",  # Osteotomy of mandibular ramus (procedure)
        "21526003",  # Repair of atrial septal defect with tissue graft (procedure)
        "21538007",  # Distal subtotal gastrectomy with vagotomy (procedure)
        "21551002",  # Division of brain tissue (procedure)
        "21574003",  # Repair of aneurysm of abdominal aorta with graft (procedure)
        "2166008",  # Bicuspidization of aortic valve (procedure)
        "21710002",  # Removal of thrombus from intracranial artery (procedure)
        "21756005",  # Closure of rectal fistula (procedure)
        "21789005",  # Gas endarterectomy of coronary artery (procedure)
        "21832001",  # Suture of laceration of diaphragm (procedure)
        "21841006",  # Thrombectomy with catheter of mesenteric artery by abdominal incision (procedure)
        "21849008",  # Triple valve replacement (procedure)
        "21904006",  # Nephrostomy with tube drainage (procedure)
        "2193006",  # Repair of ruptured aneurysm with graft of celiac artery (procedure)
        "21991006",  # Belsey operation, esophagogastric sphincter (procedure)
        "22082007",  # Aneurysmectomy with anastomosis of abdominal vein (procedure)
        "22094003",  # Linear proctotomy (procedure)
        "22216006",  # Closure of cholecystoduodenal fistula (procedure)
        "22276002",  # Thrombectomy of abdominal vein (procedure)
        "22324003",  # Jejunectomy, resection of a segment or all of jejunum (procedure)
        "2234009",  # Proctosigmoidopexy (procedure)
        "22418005",  # Simple mastectomy of bilateral breasts (procedure)
        "2250001",  # Resection of ascending aorta with anastomosis (procedure)
        "225092005",  # Permanent percutaneous nephrostomy (procedure)
        "225785008",  # Temporary percutaneous nephrostomy (procedure)
        "22656002",  # Closure of ureterorectal fistula (procedure)
        "22699007",  # Partial salpingo-oophorectomy (procedure)
        "22783008",  # Construction of anus for congenital absence with repair of urinary fistula (procedure)
        "22800003",  # Removal of thrombus from arteriovenous shunt or cannula (procedure)
        "22829001",  # Repair of vesicoperineal fistula (procedure)
        "22833008",  # Division of aorta (procedure)
        "22860001",  # Partial submucous laryngectomy (procedure)
        "22870004",  # Repair of sinus of Valsalva fistula with cardiopulmonary bypass (procedure)
        "22953008",  # Repair of heart (procedure)
        "229577006",  # Replacement of permanent transvenous electrodes (procedure)
        "22997007",  # Repair of esophagus (procedure)
        "23075000",  # Creation of vascular bypass (procedure)
        "230810008",  # Excision of tumor of brain meninges (procedure)
        "230816002",  # Creation of subdural-peritoneal shunt (procedure)
        "230818001",  # Creation of subdural-pleural shunt (procedure)
        "230825008",  # Renewal of subdural-peritoneal shunt (procedure)
        "230829002",  # Removal of subdural-peritoneal shunt (procedure)
        "230835002",  # Repair of encephalocele (procedure)
        "230837005",  # Focal excision of brain tissue tumor (procedure)
        "230838000",  # Focal excision of brain abscess (procedure)
        "230839008",  # Focal excision of brain hematoma (procedure)
        "230840005",  # Focal excision of brain cyst (procedure)
        "230841009",  # Focal excision of swollen brain (procedure)
        "230859007",  # Destruction of brain tumor (procedure)
        "230868009",  # Removal of radioactive implant from brain tissue (procedure)
        "230872008",  # Torkildsen ventriculocisternostomy (procedure)
        "230873003",  # Revision of more than one component of ventricular shunt (procedure)
        "230874009",  # Renewal of proximal catheter of ventricular shunt (procedure)
        "230875005",  # Renewal of distal catheter of ventricular shunt (procedure)
        "230878007",  # Removal of ventriculoperitoneal shunt (procedure)
        "230879004",  # Removal of shunt tubing from ventricle of brain (procedure)
        "230890009",  # Excision of lesion of ventricle of brain (procedure)
        "230891008",  # Destruction of choroid plexus (procedure)
        "230893006",  # Removal of foreign body from ventricle of brain (procedure)
        "230896003",  # Intracranial vascular operation (procedure)
        "230897007",  # Surgical procedure on intracranial aneurysm (procedure)
        "230898002",  # Clipping of aneurysm of intracranial artery (procedure)
        "230899005",  # Ligation of intracranial aneurysm (procedure)
        "230900000",  # Trapping of intracranial aneurysm (procedure)
        "230901001",  # Reinforcing of wall of intracranial aneurysm (procedure)
        "230903003",  # Ligation of intracranial vessel (procedure)
        "230904009",  # Excision of lesion of intracranial vessel (procedure)
        "230905005",  # Excision of intracranial arteriovenous malformation (procedure)
        "230906006",  # Excision of intracranial venous malformation (procedure)
        "230907002",  # Excision of intracranial cavernous hemangioma (procedure)
        "230908007",  # Destruction of lesion of intracranial vessel (procedure)
        "230915004",  # Endovascular obliteration of lesion of intracranial vasculature (procedure)
        "230916003",  # Endovascular obliteration of intracranial aneurysm (procedure)
        "230917007",  # Endovascular obliteration of intracranial arteriovenous malformation (procedure)
        "230918002",  # Obliteration of intracranial arteriovenous fistula (procedure)
        "230919005",  # Anastomosis of intracranial artery (procedure)
        "230920004",  # Intracranial to intracranial arterial anastomosis (procedure)
        "230921000",  # Revascularization of brain tissue (procedure)
        "230922007",  # Angioplasty of intracranial artery (procedure)
        "230923002",  # Percutaneous balloon angioplasty of intracranial artery (procedure)
        "230932000",  # Laser recanalization of intracranial vessel (procedure)
        "230933005",  # Laser recanalization of intracranial artery (procedure)
        "230934004",  # Thrombolysis of intracranial vessel (procedure)
        "230935003",  # Percutaneous thrombolysis of intracranial artery (procedure)
        "230936002",  # Percutaneous thrombolysis of intracranial vein (procedure)
        "230944002",  # Renewal of intracranial pressure transducer (procedure)
        "230947009",  # Renewal of intracranial pressure catheter (procedure)
        "230958000",  # Division of glossopharyngeal nerve in tonsillar fossa (procedure)
        "230985007",  # Excision of tumor of spinal cord meninges (procedure)
        "231009006",  # Excision of tumor of spinal cord tissue (procedure)
        "231049003",  # Creation of connection from spinal subarachnoid space (procedure)
        "231050003",  # Creation of thecopleural shunt (procedure)
        "231067005",  # Transplantation of tissue into brain (procedure)
        "231627005",  # Total excision of lacrimal gland (procedure)
        "231792001",  # Ventriculolumbar shunt with valve (procedure)
        "232206007",  # Excision of lesion of internal auditory canal (procedure)
        "232487002",  # Transantral ligation of maxillary artery (procedure)
        "232523002",  # Division of styloid process in tonsillar fossa (procedure)
        "232524008",  # Partial pharyngolaryngectomy (procedure)
        "232544002",  # Near-total laryngectomy (procedure)
        "232546000",  # Total pharyngolaryngectomy (procedure)
        "232553009",  # Arytenoidectomy (procedure)
        "232560003",  # Excision of lesion of vocal fold cover (procedure)
        "232561004",  # Excision of mucosal lesion of vocal cord (procedure)
        "232562006",  # Excision of vocal cord nodule (procedure)
        "232564007",  # Excision of lesion of vocal ligament and Reinke's space (procedure)
        "232565008",  # Excision of sulcus vocalis (procedure)
        "232566009",  # Excision of mucosal bridge of vocal fold (procedure)
        "232567000",  # Excision of epidermoid cyst of vocal fold (procedure)
        "232609005",  # Shunt of cerebral ventricle to lumbar site (procedure)
        "232627008",  # Open repair of acquired bronchoesophageal fistula (procedure)
        "232632009",  # Wedge excision of lung (procedure)
        "232633004",  # Single wedge excision of lung (procedure)
        "232634005",  # Multiple wedge excisions of lung (procedure)
        "232635006",  # Excision of segment of right upper lobe (procedure)
        "232636007",  # Excision of segment of middle lobe of right lung (procedure)
        "232637003",  # Excision of segment of right lower lobe (procedure)
        "232638008",  # Excision of segment of left upper lobe (procedure)
        "232639000",  # Excision of segment of left lower lobe (procedure)
        "232640003",  # Lobectomy with bronchoplastic procedure (procedure)
        "232641004",  # Lobectomy of lung with excision of part of adjacent lobe (procedure)
        "232642006",  # Extrapericardial pneumonectomy (procedure)
        "232643001",  # Intrapericardial pneumonectomy (procedure)
        "232644007",  # Extrapleural pneumonectomy (procedure)
        "232645008",  # Pleuropneumonectomy (procedure)
        "232646009",  # Completion pneumonectomy (procedure)
        "232647000",  # Pneumonectomy with tracheobronchoplasty (procedure)
        "232648005",  # Excision of lung sequestration (procedure)
        "232651003",  # Excision of bulla of lung (procedure)
        "232657004",  # Single lung transplant (procedure)
        "232658009",  # Double lung transplant (procedure)
        "232659001",  # Double lung transplant as a block (procedure)
        "232660006",  # Bilateral sequential single lung transplant (procedure)
        "232717009",  # Coronary artery bypass grafting (procedure)
        "232719007",  # Coronary artery bypass graft x 1 (procedure)
        "232720001",  # Coronary artery bypass grafts x 2 (procedure)
        "232721002",  # Coronary artery bypass grafts x 3 (procedure)
        "232722009",  # Coronary artery bypass grafts x 4 (procedure)
        "232723004",  # Coronary artery bypass grafts x 5 (procedure)
        "232724005",  # Coronary artery bypass grafts greater than 5 (procedure)
        "232731009",  # Percutaneous thrombolysis of coronary artery (procedure)
        "232733007",  # Exploration of coronary artery with arteriotomy (procedure)
        "232735000",  # Operation for coronary artery fistula (procedure)
        "232736004",  # Open operation for coronary artery fistula (procedure)
        "232738003",  # Ligation of coronary artery (procedure)
        "232740008",  # Mitral valve operation (procedure)
        "232744004",  # Mitral annuloplasty using ring (procedure)
        "232745003",  # Plication annuloplasty of mitral valve (procedure)
        "232746002",  # Mitral annulus enlargement (procedure)
        "232747006",  # Operation on mitral papillary muscle (procedure)
        "232748001",  # Shortening operation on mitral papillary muscle (procedure)
        "232749009",  # Lengthening operation on mitral papillary muscle (procedure)
        "232750009",  # Reimplantation of mitral papillary muscle (procedure)
        "232751008",  # Operation on mitral chordae tendinae (procedure)
        "232752001",  # Repair of rupture of mitral chordae tendinae (procedure)
        "232753006",  # Prosthetic replacement of mitral chordae tendinae (procedure)
        "232754000",  # Fenestration operation on mitral chordae tendinae (procedure)
        "232755004",  # Shortening plasty to mitral chordae tendinae (procedure)
        "232756003",  # Operation on mitral leaflet (procedure)
        "232757007",  # Repair of defect of mitral leaflet (procedure)
        "232758002",  # Resection of mitral leaflet (procedure)
        "232759005",  # Excision of lesion from mitral leaflet (procedure)
        "232760000",  # Closure of true mitral cleft (procedure)
        "232761001",  # Excision of vegetations from mitral valve (procedure)
        "232762008",  # Excision of vegetations from implanted mitral valve (procedure)
        "232763003",  # Removal of thrombus from mitral valve (procedure)
        "232764009",  # Removal of thrombus from implanted mitral valve (procedure)
        "232765005",  # Excision of supramitral valve ring (procedure)
        "232766006",  # Decalcification of mitral valve (procedure)
        "232767002",  # Decalcification of implanted mitral valve (procedure)
        "232768007",  # Closure of mitral valve (procedure)
        "232770003",  # Balloon dilation of implanted mitral valve (procedure)
        "232771004",  # Transluminal radiofrequency mitral valvotomy (procedure)
        "232772006",  # Mitral valvectomy (procedure)
        "232773001",  # Bjork-Shiley prosthetic replacement of mitral valve (procedure)
        "232774007",  # Operation on implanted mitral valve (procedure)
        "232775008",  # Rotation of implanted mitral valve (procedure)
        "232776009",  # Repair of implanted mitral paravalvular leak (procedure)
        "232777000",  # Replacement of implanted mitral valve (procedure)
        "232778005",  # Tricuspid valve operation (procedure)
        "232782007",  # Tricuspid annuloplasty using ring (procedure)
        "232783002",  # Plication annuloplasty of tricuspid valve (procedure)
        "232784008",  # Bicuspidization of tricuspid valve (procedure)
        "232785009",  # Operation on tricuspid papillary muscle (procedure)
        "232786005",  # Shortening operation on tricuspid papillary muscle (procedure)
        "232787001",  # Lengthening operation on tricuspid papillary muscle (procedure)
        "232788006",  # Reimplantation of tricuspid papillary muscle (procedure)
        "232789003",  # Operation on tricuspid chordae tendinae (procedure)
        "232790007",  # Repair of rupture of tricuspid chordae tendinae (procedure)
        "232791006",  # Prosthetic replacement of tricuspid chordae tendinae (procedure)
        "232792004",  # Division/resection of tricuspid chordae tendinae (procedure)
        "232793009",  # Fenestration operation on tricuspid chordae tendinae (procedure)
        "232794003",  # Shortening plasty to tricuspid chordae tendinae (procedure)
        "232795002",  # Operation on tricuspid valve leaflet (procedure)
        "232796001",  # Repair of defect of tricuspid leaflet (procedure)
        "232797005",  # Resection of tricuspid valve leaflet (procedure)
        "232799008",  # Excision of lesion from tricuspid leaflet (procedure)
        "232800007",  # Closure of true tricuspid cleft (procedure)
        "232801006",  # Excision of vegetations from tricuspid valve (procedure)
        "232802004",  # Excision of vegetations from implanted tricuspid valve (procedure)
        "232803009",  # Removal of thrombus from tricuspid valve (procedure)
        "232804003",  # Removal of thrombus from implanted tricuspid valve (procedure)
        "232806001",  # Decalcification of tricuspid valve (procedure)
        "232807005",  # Decalcification of implanted tricuspid valve (procedure)
        "232808000",  # Closure of tricuspid valve (procedure)
        "232809008",  # Balloon tricuspid valvotomy (procedure)
        "232810003",  # Balloon dilation of implanted tricuspid valve (procedure)
        "232811004",  # Tricuspid valvectomy (procedure)
        "232812006",  # Operation on implanted tricuspid valve (procedure)
        "232813001",  # Rotation of implanted tricuspid valve (procedure)
        "232814007",  # Repair of implanted tricuspid paravalvular leak (procedure)
        "232815008",  # Replacement of implanted tricuspid valve (procedure)
        "232816009",  # Aortic valve and adjacent structure operations (procedure)
        "232820008",  # Annuloplasty of aortic valve (procedure)
        "232821007",  # Patch enlargement of aortic valve annulus (procedure)
        "232822000",  # Operation on aortic valve cusp (procedure)
        "232823005",  # Plication of aortic valve cusp (procedure)
        "232825003",  # Aortic valve cusp resection and repair with circumferential annuloplasty (procedure)
        "232826002",  # Excision of lesion from aortic valve cusp (procedure)
        "232827006",  # Repair of aortic valve cusp (procedure)
        "232828001",  # Excision of vegetations from aortic valve (procedure)
        "232829009",  # Excision of vegetations from implanted aortic valve (procedure)
        "232830004",  # Removal of thrombus from aortic valve (procedure)
        "232831000",  # Removal of thrombus from implanted aortic valve (procedure)
        "232832007",  # Decalcification of aortic valve (procedure)
        "232833002",  # Decalcification of implanted aortic valve (procedure)
        "232834008",  # Balloon aortic valvotomy (procedure)
        "232835009",  # Balloon dilation of implanted aortic valve (procedure)
        "232836005",  # Transluminal laser aortic valvotomy (procedure)
        "232837001",  # Transluminal radiofrequency aortic valvotomy (procedure)
        "232838006",  # Autograft aortic valve replacement (procedure)
        "232839003",  # Operation on implanted aortic valve (procedure)
        "232840001",  # Rotation of implanted aortic valve (procedure)
        "232841002",  # Repair of implanted aortic paravalvular leak (procedure)
        "232842009",  # Replacement of implanted aortic valve (procedure)
        "232843004",  # Composite aortic root operation (procedure)
        "232845006",  # Aortic valve resuspension and replacement of ascending aorta (procedure)
        "232846007",  # Aortic valve replacement and replacement of ascending aorta (procedure)
        "232847003",  # Aortic valve replacement and aortoplasty (procedure)
        "232848008",  # Aortic valve replacement and patch aortoplasty (procedure)
        "232849000",  # Aortic valve replacement and plication of ascending aorta (procedure)
        "232850000",  # Aortoventriculoplasty (procedure)
        "232851001",  # Konno aortoventriculoplasty (procedure)
        "232853003",  # Placement of left ventricular outflow tract transannular patch (procedure)
        "232854009",  # Pulmonary valve operation (procedure)
        "232857002",  # Excision of vegetations from pulmonary valve (procedure)
        "232858007",  # Excision of vegetations from implanted pulmonary valve (procedure)
        "232859004",  # Operation on pulmonary valve cusp (procedure)
        "232860009",  # Resection of pulmonary valve cusp (procedure)
        "232861008",  # Repair of pulmonary valve cusp (procedure)
        "232862001",  # Excision of lesion from pulmonary valve cusp (procedure)
        "232863006",  # Removal of thrombus from pulmonary valve (procedure)
        "232864000",  # Removal of thrombus from implanted pulmonary valve (procedure)
        "232865004",  # Decalcification of pulmonary valve (procedure)
        "232866003",  # Decalcification of implanted pulmonary valve (procedure)
        "232867007",  # Closure of pulmonary valve (procedure)
        "232868002",  # Pulmonary valvotomy (procedure)
        "232869005",  # Balloon dilation of implanted pulmonary valve (procedure)
        "232870006",  # Transluminal laser pulmonary valvotomy (procedure)
        "232871005",  # Transluminal radiofrequency pulmonary valvotomy (procedure)
        "232872003",  # Pulmonary valvectomy (procedure)
        "232873008",  # Open pulmonary valvectomy (procedure)
        "232874002",  # Closed pulmonary valvectomy (procedure)
        "232875001",  # Operation on implanted pulmonary valve (procedure)
        "232876000",  # Rotation of implanted pulmonary valve (procedure)
        "232877009",  # Repair of implanted pulmonary paravalvular leak (procedure)
        "232878004",  # Replacement of implanted pulmonary valve (procedure)
        "232879007",  # Atrioventricular (non-mitral, non-tricuspid) valve operation (procedure)
        "232882002",  # Repair of atrioventricular valve (procedure)
        "232883007",  # Repair of Ebstein's anomaly of atrioventricular valve (procedure)
        "232884001",  # Annuloplasty of atrioventricular valve (procedure)
        "232885000",  # Atrioventricular valve annuloplasty using ring (procedure)
        "232886004",  # Plication annuloplasty of atrioventricular valve (procedure)
        "232887008",  # Operation on atrioventricular papillary muscle (procedure)
        "232888003",  # Shortening operation on atrioventricular valve papillary muscle (procedure)
        "232889006",  # Lengthening operation on atrioventricular valve papillary muscle (procedure)
        "232890002",  # Reimplantation of atrioventricular valve papillary muscle (procedure)
        "232891003",  # Operation on atrioventricular valve chordae tendinae (procedure)
        "232892005",  # Repair of rupture of atrioventricular valve chordae tendinae (procedure)
        "232893000",  # Prosthetic replacement of atrioventricular valve chordae tendinae (procedure)
        "232894006",  # Division/resection of atrioventricular valve chordae tendinae (procedure)
        "232895007",  # Fenestration operation to atrioventricular valve chordae tendinae (procedure)
        "232896008",  # Shortening plasty to atrioventricular valve chordae tendinae (procedure)
        "232897004",  # Operation on atrioventricular valve leaflet (procedure)
        "232898009",  # Repair of defect of atrioventricular valve leaflet (procedure)
        "232899001",  # Resection of atrioventricular valve leaflet (procedure)
        "232900006",  # Excision of lesion from atrioventricular valve leaflet (procedure)
        "232901005",  # Closure of true atrioventricular valve cleft (procedure)
        "232902003",  # Operation to atrioventricular valve bridging leaflet (procedure)
        "232903008",  # Suturing together of atrioventricular valve bridging leaflets (procedure)
        "232904002",  # Division of atrioventricular valve bridging leaflet (procedure)
        "232905001",  # Liberation of superior bridging leaflet using patch (procedure)
        "232906000",  # Excision of vegetations from atrioventricular valve (procedure)
        "232907009",  # Excision of vegetations from implanted atrioventricular valve (procedure)
        "232908004",  # Removal of thrombus from atrioventricular valve (procedure)
        "232909007",  # Removal of thrombus from implanted atrioventricular valve (procedure)
        "232910002",  # Decalcification of atrioventricular valve (procedure)
        "232911003",  # Decalcification of implanted atrioventricular valve (procedure)
        "232912005",  # Closure of atrioventricular valve (procedure)
        "232913000",  # Atrioventricular valvotomy (procedure)
        "232915007",  # Open atrioventricular valvotomy (procedure)
        "232916008",  # Closed atrioventricular valvotomy (procedure)
        "232917004",  # Balloon atrioventricular valvotomy (procedure)
        "232918009",  # Balloon dilation of implanted atrioventricular valve (procedure)
        "232919001",  # Atrioventricular valvectomy (procedure)
        "232920007",  # Atrioventricular (non-mitral non-tricuspid) valve replacement (procedure)
        "232921006",  # Homograft atrioventricular valve replacement (procedure)
        "232922004",  # Xenograft atrioventricular valve replacement (procedure)
        "232924003",  # Operation on implanted atrioventricular valve (procedure)
        "232925002",  # Rotation of implanted atrioventricular valve (procedure)
        "232926001",  # Repair of implanted atrioventricular paravalvular leak (procedure)
        "232927005",  # Replacement of implanted atrioventricular valve (procedure)
        "232928000",  # Truncal valve operation (procedure)
        "232931004",  # Repair of truncal valve (procedure)
        "232932006",  # Annuloplasty of truncal valve (procedure)
        "232933001",  # Operation on truncal valve cusp (procedure)
        "232934007",  # Repair of truncal valve cusp (procedure)
        "232935008",  # Resection of truncal valve cusp (procedure)
        "232936009",  # Excision of lesion from truncal valve cusp (procedure)
        "232937000",  # Excision of vegetations from truncal valve (procedure)
        "232938005",  # Excision of vegetations from implanted truncal valve (procedure)
        "232939002",  # Removal of thrombus from truncal valve (procedure)
        "232940000",  # Removal of thrombus from implanted truncal valve (procedure)
        "232941001",  # Decalcification of truncal valve (procedure)
        "232942008",  # Decalcification of implanted truncal valve (procedure)
        "232943003",  # Valvotomy of truncal valve (procedure)
        "232944009",  # Closed truncal valvotomy (procedure)
        "232945005",  # Open truncal valvotomy (procedure)
        "232946006",  # Balloon truncal valvotomy (procedure)
        "232947002",  # Balloon dilation of implanted truncal valve (procedure)
        "232948007",  # Replacement of truncal valve (procedure)
        "232949004",  # Homograft truncal valve replacement (procedure)
        "232950004",  # Xenograft truncal valve replacement (procedure)
        "232951000",  # Mechanical prosthetic truncal valve replacement (procedure)
        "232952007",  # Operation on implanted truncal valve (procedure)
        "232953002",  # Rotation of implanted truncal valve (procedure)
        "232954008",  # Repair of implanted truncal paravalvular leak (procedure)
        "232955009",  # Replacement of implanted truncal valve (procedure)
        "232956005",  # Truncal root replacement (procedure)
        "232960008",  # Transluminal removal of aortic balloon pump (procedure)
        "232964004",  # Transluminal removal of intraventricular pump (procedure)
        "232970005",  # Cardiomyoplasty operation (procedure)
        "232972002",  # Removal of heart-lung block from donor (procedure)
        "232973007",  # Allotransplant of heart (procedure)
        "232974001",  # Orthotopic allotransplant of heart (procedure)
        "232975000",  # Removal of heart from donor (procedure)
        "232976004",  # Operation on the great veins and coronary sinus (procedure)
        "232977008",  # Relief of anastomotic stricture between pulmonary venous confluence and left atrium (procedure)
        "232985004",  # Operation on systemic great vein (procedure)
        "232986003",  # Operation on superior vena cava (procedure)
        "232987007",  # Repair of superior vena cava (procedure)
        "232988002",  # Angioplasty of superior vena cava (procedure)
        "232989005",  # Insertion of stent in superior vena cava (procedure)
        "232990001",  # Ligation of superior vena cava (procedure)
        "232991002",  # Ligation of left superior vena cava (procedure)
        "232992009",  # Division of superior vena cava (procedure)
        "232993004",  # Diversion of superior vena cava to systemic venous atrium (procedure)
        "232994005",  # Operation on inferior vena cava (procedure)
        "232995006",  # Repair of inferior vena cava (procedure)
        "232996007",  # Angioplasty of inferior vena cava (procedure)
        "232997003",  # Insertion of stent in inferior vena cava (procedure)
        "232998008",  # Division of inferior vena cava (procedure)
        "233004008",  # Coronary sinus operation (procedure)
        "233006005",  # Repair of unroofed coronary sinus to create tunnel (procedure)
        "233007001",  # Closure of coronary sinus atrial septal defect (procedure)
        "233008006",  # Relief of coronary sinus obstruction (procedure)
        "233009003",  # Atrial operation (procedure)
        "233010008",  # Right atrial operation (procedure)
        "233011007",  # Excision of right atrial lesion (procedure)
        "233013005",  # Excision of right atrial myxoma (procedure)
        "233014004",  # Excision of right atrial vegetations (procedure)
        "233015003",  # Removal of right atrial thrombus (procedure)
        "233016002",  # Open removal of foreign body from right atrium (procedure)
        "233017006",  # Suture of right atrium (procedure)
        "233018001",  # Plication of right atrial wall (procedure)
        "233019009",  # Eustachian valve operation (procedure)
        "233020003",  # Construction of conduit - right atrium to right ventricle (procedure)
        "233021004",  # Construction of conduit from right atrium to rudimentary ventricle (procedure)
        "233022006",  # Construction of conduit - right atrium to pulmonary trunk (procedure)
        "233024007",  # Left atrial operation (procedure)
        "233025008",  # Excision of left atrial lesion (procedure)
        "233026009",  # Excision of left atrial myxoma (procedure)
        "233027000",  # Excision of left atrial vegetations (procedure)
        "233028005",  # Removal of left atrial thrombus (procedure)
        "233029002",  # Open removal of foreign body from left atrium (procedure)
        "233030007",  # Suture of left atrium (procedure)
        "233031006",  # Plication of left atrial wall (procedure)
        "233032004",  # Plication of left atrial appendage (procedure)
        "233033009",  # Repair of cor triatrium (procedure)
        "233034003",  # Construction of conduit - left atrium to left ventricle (procedure)
        "233036001",  # Atrial septation operation (procedure)
        "233038000",  # Operation to open an interatrial communication (procedure)
        "233040005",  # Atrial fenestration operation (procedure)
        "233041009",  # Open atrial fenestration (procedure)
        "233042002",  # Closed atrial fenestration (procedure)
        "233043007",  # Atrial septostomy operation (procedure)
        "233044001",  # Balloon atrial septostomy by pull-back (procedure)
        "233045000",  # Static balloon atrial septostomy (procedure)
        "233046004",  # Construction of adjustable interatrial communication (procedure)
        "233047008",  # Repair of atrioventricular septal defect: ventricular component (procedure)
        "233048003",  # Repair of atrioventricular septal defect with common valve orifice (procedure)
        "233049006",  # Repair of atrioventricular septal defect associated with Fallot's tetralogy (procedure)
        "233050006",  # Ventricular operation (procedure)
        "233051005",  # Right ventricular operation (procedure)
        "233052003",  # Open right ventricular biopsy (procedure)
        "233053008",  # Excision of right ventricular lesion (procedure)
        "233054002",  # Excision of right ventricular vegetations (procedure)
        "233055001",  # Open removal of foreign body from right ventricle (procedure)
        "233056000",  # Suture of right ventricle (procedure)
        "233057009",  # Resection of right ventricular muscle (procedure)
        "233058004",  # Open resection of right ventricular muscle bands (procedure)
        "233059007",  # Reinsertion of heart valve, prosthetic (procedure)
        "233060002",  # Transluminal laser resection of right ventricular muscle (procedure)
        "233061003",  # Resection of septomarginal trabeculation (procedure)
        "233062005",  # Right ventricular endocardectomy (procedure)
        "233063000",  # Removal of right ventricular thrombus (procedure)
        "233064006",  # Operation on the right ventricular outflow tract (procedure)
        "233065007",  # Right ventricular outflow tract muscle resection (procedure)
        "233066008",  # Right ventricular outflow tract myotomy (procedure)
        "233067004",  # Balloon dilation of right ventricular outflow tract (procedure)
        "233068009",  # Transluminal right ventricular outflow tract laser operation (procedure)
        "233069001",  # Patch enlargement of right ventricular outflow tract (procedure)
        "233071001",  # Construction of conduit - right ventricle to pulmonary trunk (procedure)
        "233072008",  # Construction of conduit - right ventricle to right pulmonary artery (procedure)
        "233073003",  # Construction of conduit - right ventricle to left pulmonary artery (procedure)
        "233074009",  # Construction of conduit - right ventricle to both pulmonary arteries (procedure)
        "233075005",  # Construction of conduit - right ventricle to aorta (procedure)
        "233076006",  # Left ventricular operation (procedure)
        "233077002",  # Open left ventricular biopsy (procedure)
        "233079004",  # Excision of left ventricular lesion (procedure)
        "233080001",  # Excision of left ventricular vegetations (procedure)
        "233081002",  # Removal of left ventricular thrombus (procedure)
        "233082009",  # Open removal of foreign body from left ventricle (procedure)
        "233083004",  # Suture of left ventricle (procedure)
        "233084005",  # Open resection of left ventricular muscle (procedure)
        "233085006",  # Transluminal laser resection of left ventricular muscle (procedure)
        "233086007",  # Left ventricular endocardectomy (procedure)
        "233087003",  # Surgical procedure on aneurysm of left cardiac ventricle (procedure)
        "233088008",  # Left ventricular aneurysm plication (procedure)
        "233089000",  # Excision of aneurysm of left cardiac ventricle (procedure)
        "233090009",  # Left ventricular outflow tract muscle resection (procedure)
        "233091008",  # Left ventricular outflow tract myotomy (procedure)
        "233092001",  # Balloon dilation of left ventricular outflow tract (procedure)
        "233093006",  # Transluminal left ventricular outflow tract laser operation (procedure)
        "233094000",  # Resection of subaortic fibromuscular shelf (procedure)
        "233095004",  # Construction of conduit - apex of left ventricle to descending aorta (procedure)
        "233096003",  # Construction of conduit - left ventricle to ascending aorta (procedure)
        "233097007",  # Construction of conduit - left ventricle to pulmonary trunk (procedure)
        "233098002",  # Operation on the interventricular septum (procedure)
        "233099005",  # Closure of ventricular septal defect by direct suture (procedure)
        "233100002",  # Transluminal closure of ventricular septal defect using prosthesis (procedure)
        "233101003",  # Operation on ventricular septal patch (procedure)
        "233102005",  # Removal of patch from ventricular septum (procedure)
        "233103000",  # Fenestration of patch of ventricular septum (procedure)
        "233104006",  # Enlargement of restrictive ventricular septal defect (procedure)
        "233105007",  # Resection of subarterial outlet septum (procedure)
        "233106008",  # Ventricular septation operation (procedure)
        "233108009",  # Partial ventricular septation operation (procedure)
        "233109001",  # Complete ventricular septation operation (procedure)
        "233110006",  # Completion of ventricular septation operation (procedure)
        "233111005",  # Operations for Fallot-type conditions (procedure)
        "233112003",  # Repair of Fallot-type double outlet right ventricle (procedure)
        "233114002",  # Operation for major aortopulmonary collateral arteries (procedure)
        "233116000",  # Banding of major aortopulmonary collateral artery (procedure)
        "233118004",  # Placement of stent in major aortopulmonary collateral artery (procedure)
        "233120001",  # Unifocalization operation to an artery (procedure)
        "233123004",  # Intraventricular operation for transposition or double outlet ventricle (procedure)
        "233124005",  # Construction of intraventricular left ventricle to aorta tunnel (procedure)
        "233125006",  # Construction of intraventricular left ventricle to aorta spiral tunnel (procedure)
        "233126007",  # Repair of double outlet right ventricle (procedure)
        "233127003",  # REV operation (procedure)
        "233128008",  # Repair of double outlet left ventricle (procedure)
        "233129000",  # Repair of double outlet left ventricle with right ventricle to pulmonary trunk conduit and ventricular septal defect closure (procedure)
        "233130005",  # Repair of double outlet left ventricle with direct right ventricle to pulmonary trunk anastomosis and ventricular septal defect closure (procedure)
        "233131009",  # Repair of double outlet left ventricle with intraventricular right ventricle to pulmonary trunk tunnel (procedure)
        "233134001",  # Damus-Stansel-Kaye operation (procedure)
        "233136004",  # Total cavopulmonary connection (procedure)
        "233137008",  # Completion of total cavopulmonary connection (procedure)
        "233139006",  # Norwood type operation (procedure)
        "233140008",  # Direct aorta to pulmonary artery anastomosis with fenestrated baffle (procedure)
        "233141007",  # Placement of stent in ductus arteriosus and banding of both pulmonary arteries (procedure)
        "233142000",  # Cardiac conduit operation (procedure)
        "233143005",  # Construction of a cardiac conduit (procedure)
        "233144004",  # Construction of conduit - pulmonary trunk to right pulmonary artery (procedure)
        "233145003",  # Construction of conduit - pulmonary trunk to left pulmonary artery (procedure)
        "233146002",  # Construction of conduit - right pulmonary artery to left pulmonary artery (procedure)
        "233153006",  # Resection of conduit aneurysm (procedure)
        "233155004",  # Transection operation for arrhythmia (procedure)
        "233156003",  # Excision operation for arrhythmia (procedure)
        "233157007",  # Exclusion operation for arrhythmia (procedure)
        "233158002",  # Ligation operation for arrhythmia (procedure)
        "233159005",  # Ablation operation for arrhythmia (procedure)
        "233160000",  # Ablation operation for arrhythmia by diathermy (procedure)
        "233161001",  # Cryoablation operation for arrhythmia (procedure)
        "233162008",  # Laser ablation operation for arrhythmia (procedure)
        "233163003",  # Radiofrequency ablation operation for arrhythmia (procedure)
        "233164009",  # Atrioventricular disconnection operation (procedure)
        "233165005",  # Partial atrioventricular disconnection operation (procedure)
        "233166006",  # Complete atrioventricular disconnection operation (procedure)
        "233167002",  # Maze procedure for atrial arrhythmia (procedure)
        "233168007",  # Corridor operation (procedure)
        "233170003",  # Implantation of automatic cardiac defibrillator (procedure)
        "233187001",  # Removal of implanted cardiac pacemaker system or components (procedure)
        "233188006",  # Removal of cardiac pacing electrode (procedure)
        "233189003",  # Percutaneous insertion of pericardial drainage tube (procedure)
        "233191006",  # Pericardiotomy and drainage of effusion (procedure)
        "233192004",  # Open decompression of cardiac tamponade (procedure)
        "233193009",  # Pericardial shunt operation (procedure)
        "233194003",  # Pericardiopleural shunt operation (procedure)
        "233195002",  # Pericardioperitoneal shunt operation (procedure)
        "233196001",  # Repair of common arterial trunk with right ventricle to pulmonary trunk direct anastomosis (procedure)
        "233198000",  # Excision of ductus arteriosus (procedure)
        "233199008",  # Closure of ductus arteriosus with clip (procedure)
        "233201005",  # Transpulmonary endovascular oversewing of ductus arteriosus (procedure)
        "233202003",  # Balloon dilation of ductus arteriosus (procedure)
        "233205001",  # Excision of vegetations from ductus arteriosus (procedure)
        "233206000",  # Resection of aneurysm of ductus arteriosus (procedure)
        "233207009",  # Operation on the pulmonary trunk and arteries (procedure)
        "233208004",  # Pulmonary trunk and artery banding operation (procedure)
        "233209007",  # Reimplantation of pulmonary artery (procedure)
        "233210002",  # Transection and oversewing of pulmonary trunk (procedure)
        "233211003",  # Resection and end-to-end anastomosis of pulmonary trunk (procedure)
        "233213000",  # Reconstruction of pulmonary artery (procedure)
        "233214006",  # Angioplasty of pulmonary artery (procedure)
        "233215007",  # Placement of stent in pulmonary artery (procedure)
        "233216008",  # Excision of vegetations from pulmonary trunk (procedure)
        "233217004",  # Operation for pulmonary embolism (procedure)
        "233218009",  # Transluminal fragmentation of pulmonary embolus (procedure)
        "233222004",  # Operation on the sinuses of Valsalva (procedure)
        "233223009",  # Closure of an aortic tunnel (procedure)
        "233224003",  # Central aortopulmonary shunt operation (procedure)
        "233225002",  # Disconnection of pulmonary trunk and reanastomosis of distal end to ascending aorta (procedure)
        "233226001",  # Subclavian to pulmonary artery shunt operation (procedure)
        "233227005",  # Modified Blalock-Taussig shunt (procedure)
        "233228000",  # Classical Glenn shunt (procedure)
        "233230003",  # Hemi-Fontan operation (procedure)
        "233231004",  # Division of superior vena cava with end-to-side anastomoses of both ends to the right pulmonary artery with isolation from right atrium (procedure)
        "233233001",  # Operation on systemic to pulmonary artery shunt (procedure)
        "233234007",  # Application of band to systemic to pulmonary artery shunt (procedure)
        "233235008",  # Removal of band from systemic to pulmonary artery shunt (procedure)
        "233236009",  # Adjustment of systemic to pulmonary artery shunt (procedure)
        "233238005",  # Resection of systemic to pulmonary shunt aneurysm (procedure)
        "233243003",  # Transluminal operations on suprarenal artery (procedure)
        "233244009",  # Transluminal operations on superior mesenteric artery (procedure)
        "233245005",  # Transluminal operations on inferior mesenteric artery (procedure)
        "233246006",  # Transluminal operations on celiac artery (procedure)
        "233257001",  # Transluminal operations on circle of Willis (procedure)
        "233263005",  # Angioplasty of aorta (procedure)
        "233264004",  # Angioplasty of thoracic aorta (procedure)
        "233265003",  # Percutaneous balloon angioplasty of thoracic aorta (procedure)
        "233266002",  # Angioplasty of abdominal aorta (procedure)
        "233267006",  # Percutaneous balloon angioplasty of abdominal aorta (procedure)
        "233268001",  # Angioplasty of hepatic artery (procedure)
        "233269009",  # Percutaneous balloon angioplasty of hepatic artery (procedure)
        "233270005",  # Angioplasty of celiac artery (procedure)
        "233271009",  # Percutaneous balloon angioplasty of celiac artery (procedure)
        "233272002",  # Angioplasty of superior mesenteric artery (procedure)
        "233273007",  # Percutaneous balloon angioplasty of superior mesenteric artery (procedure)
        "233275000",  # Angioplasty of common iliac artery (procedure)
        "233276004",  # Percutaneous balloon angioplasty of common iliac artery (procedure)
        "233277008",  # Angioplasty of internal iliac artery (procedure)
        "233278003",  # Percutaneous balloon angioplasty of internal iliac artery (procedure)
        "233279006",  # Angioplasty of external iliac artery (procedure)
        "233280009",  # Percutaneous balloon angioplasty of external iliac artery (procedure)
        "233296007",  # Endarterectomy of internal carotid artery (procedure)
        "233297003",  # Endarterectomy of external carotid artery (procedure)
        "233298008",  # Endarterectomy of common carotid artery (procedure)
        "233300008",  # Endarterectomy of abdominal aorta (procedure)
        "233301007",  # Endarterectomy of hepatic artery (procedure)
        "233302000",  # Endarterectomy of celiac artery (procedure)
        "233303005",  # Endarterectomy of superior mesenteric artery (procedure)
        "233304004",  # Endarterectomy of common iliac artery (procedure)
        "233305003",  # Endarterectomy of internal iliac artery (procedure)
        "233306002",  # Endarterectomy of external iliac artery (procedure)
        "233336006",  # Percutaneous thrombolysis of common iliac artery (procedure)
        "233337002",  # Percutaneous thrombolysis of external iliac artery (procedure)
        "233344006",  # Polyethylene terephthalate graft operations on aorta (procedure)
        "233345007",  # Reconstruction of common iliac artery (procedure)
        "233346008",  # Reconstruction of suprarenal artery (procedure)
        "233347004",  # Reconstruction of superior mesenteric artery (procedure)
        "233348009",  # Reconstruction of inferior mesenteric artery (procedure)
        "233349001",  # Reconstruction of celiac artery (procedure)
        "233350001",  # Reconstruction of brachial artery (procedure)
        "233351002",  # Reconstruction of axillary artery (procedure)
        "233370007",  # Repair of aneurysm of aorta (procedure)
        "233371006",  # Anastomosis of intracranial vasculature (procedure)
        "233372004",  # Bypass graft of the descending thoracic aorta (procedure)
        "233373009",  # Aortic bifurcation bypass graft (procedure)
        "233374003",  # Y graft of abdominal aortic aneurysm - emergency (procedure)
        "233375002",  # Straight graft of the abdominal aorta (procedure)
        "233376001",  # Repair of aortic aneurysm using straight graft (procedure)
        "233377005",  # Tube graft abdominal aortic aneurysm - emergency (procedure)
        "233378000",  # Extra-anatomical bypass graft (procedure)
        "233379008",  # Axillobifemoral bypass graft (procedure)
        "233380006",  # Femorofemoral crossover bypass graft (procedure)
        "233381005",  # Iliofemoral crossover graft (procedure)
        "233383008",  # Femorodistal bypass (procedure)
        "233384002",  # Popliteal-distal bypass (procedure)
        "233385001",  # Interposition arterial graft (procedure)
        "233386000",  # Arterial jump graft (procedure)
        "233388004",  # Replacement of aneurysmal iliac artery by anastomosis (procedure)
        "233393001",  # Ligation of internal iliac artery (procedure)
        "233394007",  # Ligation of anterior division of internal iliac artery (procedure)
        "233397000",  # Division of subclavian artery (procedure)
        "233398005",  # Excision/biopsy of artery (procedure)
        "233399002",  # Extended resection of thoracic aorta (procedure)
        "233400009",  # Radically extended resection of thoracic aorta (procedure)
        "233406003",  # Insertion of thoracic aorta stent (procedure)
        "233407007",  # Insertion of abdominal aorta stent (procedure)
        "233408002",  # Insertion common iliac artery stent (procedure)
        "233409005",  # Insertion of external iliac artery stent (procedure)
        "233422002",  # Thrombectomy of intracranial vein (procedure)
        "233423007",  # Thrombectomy of iliofemoral vein (procedure)
        "233439000",  # Removal of inferior vena caval filter (procedure)
        "233444007",  # Portosystemic shunt operation (procedure)
        "233445008",  # Transjugular intrahepatic portosystemic shunt (procedure)
        "233446009",  # Creation of Eck side-to-side portocaval shunt (procedure)
        "233447000",  # Mesoatrial shunt (procedure)
        "233448005",  # Femorofemoral venous crossover shunt (procedure)
        "233469007",  # Creation brachiocephalic fistula (procedure)
        "233470008",  # Creation of saphenous vein shunt (procedure)
        "233471007",  # Creation of external arteriovenous shunt (procedure)
        "233472000",  # Creation of Scribner shunt (procedure)
        "233569005",  # Single period of cardiopulmonary bypass (procedure)
        "233570006",  # Multiple periods of cardiopulmonary bypass (procedure)
        "233571005",  # Apicoaortic shunt procedure (procedure)
        "23363003",  # Transposition of large intestine (procedure)
        "233912002",  # Replacement of heart valve poppet, prosthetic (procedure)
        "234064002",  # Marsupialization of cerebral lesion (procedure)
        "234245003",  # Extended radical neck dissection of cervical lymph nodes (procedure)
        "23425005",  # Amputation of arm through elbow (procedure)
        "234322007",  # Splenic cystojejunostomy (procedure)
        "234359008",  # Suture of small intestine, single site (procedure)
        "234660007",  # Transeptal alveolotomy (procedure)
        "23478009",  # Intra-abdominal manipulation of small intestine (procedure)
        "234889007",  # Radical glossectomy (procedure)
        "234915008",  # Marsupialization of lesion of brain (procedure)
        "235152003",  # Enterorrhaphy of small intestine (procedure)
        "235160002",  # McKeown esophagectomy (procedure)
        "235161003",  # Ivor Lewis subtotal esophagectomy (procedure)
        "235162005",  # Trans-hiatal esophagectomy (procedure)
        "235163000",  # Partial esophagectomy with supra-aortic reconstruction (procedure)
        "235164006",  # Partial esophagectomy with subaortic reconstruction (procedure)
        "235165007",  # Partial esophagectomy and total gastrectomy (procedure)
        "235166008",  # Partial segmental esophagectomy (procedure)
        "235168009",  # Direct reconstruction of esophagus (procedure)
        "235169001",  # Gastric pull-up (procedure)
        "235170000",  # Reconstruction of esophagus - gastric tube (procedure)
        "235171001",  # Jejunal interposition (procedure)
        "235173003",  # Primary repair of esophageal atresia (procedure)
        "235174009",  # Secondary repair for esophageal atresia (procedure)
        "235175005",  # Closure of recurrent tracheoesophageal fistula (procedure)
        "235176006",  # Bypass of esophagus using colon (procedure)
        "235177002",  # Creation of cervical tube esophagostomy (procedure)
        "235178007",  # Creation of transthoracic tube esophagostomy (procedure)
        "235179004",  # Creation of end esophagostomy (procedure)
        "235193006",  # V-Y gastroplasty (procedure)
        "235203005",  # Snare resection of esophageal lesion (procedure)
        "235214008",  # Billroth I partial gastrectomy - Horsley modification (procedure)
        "235215009",  # Billroth I partial gastrectomy - von Haberer-Finney modification (procedure)
        "235216005",  # Billroth I partial gastrectomy - von Haberer modification (procedure)
        "235217001",  # Billroth I partial gastrectomy - Schoemaker modification (procedure)
        "235218006",  # Billroth II partial gastrectomy - Moynihan modification (procedure)
        "235219003",  # Billroth II partial gastrectomy - Balfour modification (procedure)
        "235225004",  # Heineke-Mikulicz pyloroplasty (procedure)
        "235226003",  # Finney pyloroplasty (procedure)
        "235229005",  # Truncal vagotomy with drainage (procedure)
        "235230000",  # Highly selective vagotomy with drainage (procedure)
        "235253004",  # Excision of cyst of duodenum (procedure)
        "235254005",  # Excision of duodenal atresia (procedure)
        "235255006",  # Excision of duodenal atresia with tapering (procedure)
        "235256007",  # Duodenostomy operation (procedure)
        "235259000",  # Closure of perforated duodenum (procedure)
        "235260005",  # Closure of perforated duodenum using omental patch (procedure)
        "235271006",  # Application of hemoclips onto duodenal lesion (procedure)
        "235281005",  # Anterior gastrojejunostomy (procedure)
        "235282003",  # Anastomosis of stomach to ileum (procedure)
        "235284002",  # Creation of percutaneous jejunostomy (procedure)
        "235285001",  # Loop transposition of jejunum (procedure)
        "235286000",  # Roux-en-Y transposition of jejunum (procedure)
        "235287009",  # Transmesenteric plication (procedure)
        "235289007",  # Enterotomy and washout (procedure)
        "235291004",  # Bishop-Koop anastomosis (procedure)
        "235292006",  # Santulli enterostomy (procedure)
        "235296009",  # Correction of malrotation of intestine (procedure)
        "235304001",  # Closure of perforated small intestine (procedure)
        "235305000",  # Interposition of small intestine (procedure)
        "235306004",  # Ileal transposition (procedure)
        "235307008",  # Bypass of small intestine (procedure)
        "235312009",  # Removal of prosthesis from small intestine (procedure)
        "235313004",  # Non-emergency appendectomy (procedure)
        "235314005",  # Inversion appendectomy (procedure)
        "235316007",  # Malone's operation (procedure)
        "235318008",  # Construction of open stoma of cecum (procedure)
        "235320006",  # Transverse taeniamyotomy (procedure)
        "235325001",  # Extended left hemicolectomy (procedure)
        "235332005",  # Creation of ileal pouch (procedure)
        "235333000",  # Oversewing of distal bowel (procedure)
        "235334006",  # Oversewing of rectal stump (procedure)
        "235337004",  # Dilation of stricture - large intestine (procedure)
        "235338009",  # Balloon dilation of stricture - large intestine (procedure)
        "235342007",  # Endoscopic extirpation of lesion of rectum using fiberoptic sigmoidoscope (procedure)
        "235343002",  # Colonoscopy and extirpation of lesion of colon (procedure)
        "235344008",  # Fiberoptic endoscopic extirpation of lesion of colon (procedure)
        "235354007",  # Removal of prosthesis from large intestine (procedure)
        "235355008",  # Closure of perforated large intestine (procedure)
        "235360007",  # Dunphy repair for rectal prolapse (procedure)
        "235364003",  # Total proctectomy (procedure)
        "235365002",  # Swenson procedure (procedure)
        "235366001",  # Duhamel procedure (procedure)
        "235367005",  # Duhamel procedure with Lester Martin modification (procedure)
        "235374000",  # Posterior sagittal anorectoplasty (procedure)
        "235375004",  # V-Y anoplasty (procedure)
        "235376003",  # Anterior fixation of rectum (procedure)
        "235377007",  # Posterior fixation of rectum (procedure)
        "235378002",  # Mesh rectopexy (procedure)
        "235379005",  # Lateral ligament suspension (procedure)
        "235386002",  # Repair of rectourinary fistula (procedure)
        "235387006",  # Repair of prostatorectal fistula (procedure)
        "235404002",  # Removal of stomal bridge (procedure)
        "235405001",  # Removal of colostomy bridge (procedure)
        "235406000",  # Removal of ileostomy bridge (procedure)
        "235407009",  # Gastrointestinal tract anastomosis - intestine (procedure)
        "235408004",  # Anastomosis of jejunum to colon (procedure)
        "235409007",  # Ileocolic anastomosis (procedure)
        "235460008",  # Marsupialization of liver abscess (procedure)
        "235462000",  # Live donor partial hepatectomy (procedure)
        "235463005",  # Extended hepatic resection (procedure)
        "235466002",  # Supraradical pancreatectomy (procedure)
        "235467006",  # Duodenal preserving pancreatic head resection (procedure)
        "235468001",  # Distal pancreatic tail resection (procedure)
        "235469009",  # 30 to 50 percent distal pancreatectomy (procedure)
        "235470005",  # 50 to 70 percent distal pancreatectomy (procedure)
        "235472002",  # Local excision of lesion of pancreas (procedure)
        "235476004",  # Operation on pancreatic duct (procedure)
        "235477008",  # Pancreaticojejunostomy (procedure)
        "235479006",  # Proximal pancreaticojejunostomy (procedure)
        "235480009",  # Caudal pancreaticojejunostomy (procedure)
        "235481008",  # Lateral pancreaticojejunostomy (procedure)
        "235485004",  # Pancreatic duct prosthesis operations (procedure)
        "235489005",  # Incision of pancreatic duct (procedure)
        "235490001",  # Suturing of pancreatic duct (procedure)
        "235491002",  # Ligation of pancreatic duct (procedure)
        "235493004",  # Dilation of pancreatic duct (procedure)
        "235498008",  # Operation for pancreatic fistula (procedure)
        "235499000",  # Excision of pancreatic fistula (procedure)
        "235501008",  # Occlusion of pancreatic fistula (procedure)
        "235509005",  # Triple bypass of pancreas (procedure)
        "235510000",  # Stapling of pancreas (procedure)
        "235511001",  # Banding of pancreas (procedure)
        "235512008",  # Direct measurement of pancreatic parenchymal pressure (procedure)
        "235530001",  # Closure of choledochoduodenostomy (procedure)
        "235531002",  # Closure of choledochoenterostomy (procedure)
        "235537003",  # Closure of hepatico-duodenostomy (procedure)
        "235538008",  # Closure of hepatico-enterostomy (procedure)
        "235580007",  # Operations on duodenal papilla (procedure)
        "235581006",  # Sphincteroplasty of duodenal papilla (procedure)
        "235582004",  # Sphincterotomy of duodenal papilla (procedure)
        "235583009",  # Endoscopic retrograde cholangiopancreatography sphincterotomy of sphincter of Oddi and calculus removal (procedure)
        "235584003",  # Sphincterotomy of accessory papilla (procedure)
        "235586001",  # Pancreatic duct septotomy (procedure)
        "235588000",  # Excision of duodenal papilla (procedure)
        "236138007",  # Xenograft renal transplant (procedure)
        "236139004",  # Nephroureterectomy with cystoscopically assisted ureterectomy (procedure)
        "236140002",  # Middle partial nephrectomy (procedure)
        "236142005",  # Open repair of kidney or renal pelvis (procedure)
        "236145007",  # Open incision or drainage of kidney or renal pelvis (procedure)
        "236147004",  # Open nephrostomy (procedure)
        "236148009",  # Open pyelostomy (procedure)
        "236149001",  # Incision of renal capsule (procedure)
        "236154005",  # Nephroscopic resection of lesion in renal pelvis (procedure)
        "236155006",  # Percutaneous endoluminal balloon rupture of stenosis of pelviureteric junction (procedure)
        "23617000",  # Repair of ruptured aneurysm of abdominal aorta with graft (procedure)
        "236183008",  # Ureteroscopic pyelolysis (procedure)
        "236200004",  # Transurethral resection of ureteric orifice (procedure)
        "236202007",  # Construction of jejunal conduit (procedure)
        "236203002",  # Construction of continent urinary reservoir (procedure)
        "236205009",  # Revision of colonic conduit (procedure)
        "236209003",  # Salvage cystoprostatourethrectomy (procedure)
        "236210008",  # Salvage cystourethrectomy - female (procedure)
        "236211007",  # Salvage cystoprostatectomy (procedure)
        "236213005",  # Partial cystectomy with excision of urachal remnant (procedure)
        "23625003",  # Aneurysmectomy with anastomosis of thoracic artery (procedure)
        "236253008",  # Transurethral excision of male urinary bladder neck (procedure)
        "236334001",  # Total orchidectomy (procedure)
        "236335000",  # Subcapsular orchidectomy (procedure)
        "236336004",  # Excision of strangulated testis (procedure)
        "236501004",  # Pelvic floor repair (procedure)
        "23676000",  # Partial excision of pituitary gland by transsphenoidal approach (procedure)
        "236783000",  # Terminal ileum bypass (procedure)
        "236826000",  # Radical total vulvectomy (procedure)
        "236875004",  # Laser excision of lesion of cervix (procedure)
        "236876003",  # Wedge biopsy of cervix (procedure)
        "236886002",  # Hysterectomy (procedure)
        "236887006",  # Laparoscopic hysterectomy (procedure)
        "236888001",  # Laparoscopic total hysterectomy (procedure)
        "236889009",  # Laparoscopic supracervical hysterectomy (procedure)
        "236891001",  # Laparoscopic radical hysterectomy (procedure)
        "236897002",  # Endometrial resection (procedure)
        "236901008",  # Endoscopic myomectomy (procedure)
        "236902001",  # Vaginal myomectomy (procedure)
        "236906003",  # Excision of rudimentary uterine horn (procedure)
        "236911001",  # Excision of tubo-ovarian mass (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "236988000",  # Elective cesarean hysterectomy (procedure)
        "237367009",  # Total mastectomy and division of pectoralis minor muscle (procedure)
        "237368004",  # Total mastectomy and excision of part of pectoral muscles and chest wall (procedure)
        "237400005",  # Microdochectomy (procedure)
        "237478002",  # Sublabial hypophysectomy (procedure)
        "237479005",  # Selective transsphenoidal pituitary adenomectomy (procedure)
        "23748004",  # Thrombectomy of ascending aorta (procedure)
        "237483005",  # Operation on lesion of pituitary gland (procedure)
        "237485003",  # Chiasmopexy (procedure)
        "237588004",  # Perirectal fistulectomy (procedure)
        "23763001",  # Tubo-ovarian lysis of adhesions (procedure)
        "23770001",  # Closure of uteroureteric fistula (procedure)
        "23797008",  # Esophagojejunostomy (procedure)
        "238189001",  # Keel repair of incisional hernia (procedure)
        "238199006",  # Repair of perineal hernia (procedure)
        "238200009",  # Repair of incisional perineal hernia (procedure)
        "238236007",  # Excision of vitelline fistula (procedure)
        "238240003",  # Excision of vitelline sinus (procedure)
        "238255007",  # Second look laparotomy (procedure)
        "238277006",  # Peritoneal cyst marsupialization (procedure)
        "238278001",  # Peritoneal cystojejunostomy (procedure)
        "238282004",  # Peritoneal pseudocyst marsupialization (procedure)
        "238283009",  # Peritoneal pseudocyst cystojejunostomy (procedure)
        "238286001",  # Insertion of peritoneal drain (procedure)
        "238293002",  # Renewal of abdominal pack (procedure)
        "238298006",  # Fistulectomy of rectum (procedure)
        "238325002",  # Mediastinoscopy and lymph node sampling (procedure)
        "238326001",  # Mediastinoscopy and aspiration of lesion (procedure)
        "238348007",  # Open pleuroperitoneal shunt procedure (procedure)
        "238358006",  # Reconstruction of diaphragm with mesh (procedure)
        "23908002",  # Closure of pleuropericardial fistula (procedure)
        "239298003",  # Zygomatic osteotomy (procedure)
        "239299006",  # Le Fort I osteotomy (procedure)
        "239300003",  # Le Fort II osteotomy (procedure)
        "239301004",  # Maxillary anterior segmental osteotomy (procedure)
        "239302006",  # Maxillary posterior segmental osteotomy (procedure)
        "239303001",  # Maxillary multisegmental osteotomy (procedure)
        "239335001",  # Sutural craniectomy (procedure)
        "239338004",  # Hemimaxillectomy (procedure)
        "239349005",  # Excision of lesion of maxilla (procedure)
        "239377001",  # Removal of sequestra from maxillary wound (procedure)
        "239407004",  # Inverted L mandibular osteotomy (procedure)
        "239408009",  # Subcondylar mandibular osteotomy (procedure)
        "239409001",  # Mandibular angle osteotomy (procedure)
        "239419007",  # Repair of meniscus (procedure)
        "239469004",  # Prosthetic hemiarthroplasty of the shoulder (procedure)
        "239501000",  # Prosthetic unicompartmental arthroplasty of knee (procedure)
        "239502007",  # Prosthetic arthroplasty of the patellofemoral joint (procedure)
        "239533009",  # Eminectomy temporomandibular joint (procedure)
        "239582009",  # Amputation of finger through carpometacarpal joint (procedure)
        "239591008",  # Midfoot amputation (procedure)
        "239592001",  # Forefoot amputation (procedure)
        "23968004",  # Excision of colon (procedure)
        "23970008",  # Enlargement of atrial septal defect in repair of total anomalous pulmonary venous connection (procedure)
        "23984003",  # Plication of inferior vena cava (procedure)
        "24026006",  # Intra-abdominal venous shunt (procedure)
        "240294007",  # Open operation for fetal congenital diaphragmatic hernia (procedure)
        "240325001",  # Cadaver donor nephrectomy (procedure)
        "240326000",  # Removal of liver from donor (procedure)
        "240327009",  # Removal of liver from cadaver donor (procedure)
        "240328004",  # Removal of pancreatic tissue from donor (procedure)
        "24068006",  # Hysterectomy for removal of hydatidiform mole (procedure)
        "240690005",  # Anastomosis of intrahepatic duct to jejunum (procedure)
        "240919008",  # Balloon cardiac valvotomy (procedure)
        "240920002",  # Laser cardiac valvotomy (procedure)
        "240921003",  # Radiofrequency cardiac valvotomy (procedure)
        "240927004",  # Embolectomy of aortoiliac segment (procedure)
        "240928009",  # Embolectomy of iliac artery (procedure)
        "240932003",  # Percutaneous thrombolysis of celiac artery (procedure)
        "240933008",  # Percutaneous thrombolysis of superior mesenteric artery (procedure)
        "240934002",  # Percutaneous thrombolysis of renal artery (procedure)
        "240935001",  # Percutaneous thrombolysis of hepatic artery (procedure)
        "240936000",  # Percutaneous thrombolysis of internal iliac artery (procedure)
        "24132004",  # Resection of pericardium for chronic constrictive pericarditis (procedure)
        "241560003",  # Portoenterostomy (procedure)
        "24221008",  # Repair of rectovesical fistula with colostomy (procedure)
        "242519008",  # Kasai procedure (procedure)
        "24262002",  # Thrombectomy with catheter of radial artery by arm incision (procedure)
        "24293001",  # Excision of cervical stump by abdominal approach (procedure)
        "243009",  # Replacement of cerebral ventricular tube (procedure)
        "243129003",  # Ventriculomastoid anastomosis with valve (procedure)
        "243306000",  # Ureteroileostomy (procedure)
        "243374001",  # Ventricular shunt to mastoid (procedure)
        "24355008",  # Thromboendarterectomy of coronary artery by open chest approach (procedure)
        "243775002",  # Construction of cutaneous ureteroileostomy (procedure)
        "24394009",  # Percutaneous balloon valvuloplasty of heart (procedure)
        "24429008",  # Revision of nephrostomy (procedure)
        "24431004",  # Esophagoduodenostomy with complete gastrectomy (procedure)
        "244608001",  # Formation of pericardial baffle, interatrial (procedure)
        "24506003",  # Esophagogastrectomy (procedure)
        "245667003",  # Paul operation, temporary ileostomy (procedure)
        "24622007",  # Closure of sigmoidovaginal fistula (procedure)
        "24636002",  # Closure of colovaginal fistula (procedure)
        "246525005",  # Insertion of atrial septal umbrella (procedure)
        "24677003",  # Closure of cervicovesical fistula (procedure)
        "24728001",  # Incision of myocardium (procedure)
        "247394006",  # Insertion of King-Mills umbrella device for closure of septal defect (procedure)
        "247501005",  # Creation of Brooke ileostomy (procedure)
        "248352005",  # Permanent continent ileostomy (procedure)
        "24883002",  # Biliopancreatic bypass to ileum with partial gastrectomy (procedure)
        "24902006",  # Closure of intestinovaginal fistula (procedure)
        "249210007",  # Creation of continent ileostomy (procedure)
        "25022002",  # Closure of enteroenteric fistula (procedure)
        "25048005",  # Cingulotomy of brain by percutaneous radiofrequency (procedure)
        "251000087100",  # Left nephrostomy using fluoroscopic guidance (procedure)
        "251096000",  # Unroofing of cyst of kidney (procedure)
        "25196007",  # Proctectomy by abdominoperineal resection with colostomy, two stages (procedure)
        "252053007",  # Excision of cyst of kidney (procedure)
        "25236004",  # Replacement of tricuspid valve (procedure)
        "25240008",  # Cerebral ventriculotomy (procedure)
        "252917007",  # Decortication of kidney (procedure)
        "25321000",  # Thoracoscopic pneumonectomy (procedure)
        "25328006",  # Creation of arterial bypass with nonautogenous graft (procedure)
        "25344002",  # Fimbriectomy of fallopian tube (procedure)
        "25353009",  # Craniotomy (procedure)
        "253766005",  # Renal capsulectomy (procedure)
        "25406005",  # Radical excision of bursa of synovia of flexors of wrist (procedure)
        "2552005",  # Incision of cerebral subarachnoid space (procedure)
        "25558001",  # Removal of thrombus from thoracic artery (procedure)
        "255682001",  # Cutting of trigeminal nerve (procedure)
        "25600005",  # Ventricular shunt to cervical subarachnoid space (procedure)
        "256541003",  # Division of trigeminal nerve at foramen ovale (procedure)
        "25679005",  # Cardiolysis (procedure)
        "25736000",  # Excision of coarctation of aorta with end-to-end anastomosis (procedure)
        "257909005",  # Repair of congenital diaphragmatic hernia using muscle flap (procedure)
        "25799009",  # Excision of lesion of perivesical tissue (procedure)
        "25811000",  # Total excision of bilateral fallopian tubes (procedure)
        "25871006",  # Cricothyroidectomy (procedure)
        "25927006",  # Incision of thoracic artery (procedure)
        "2598006",  # Open heart surgery (procedure)
        "260584009",  # Mediastinoscopy without biopsy (procedure)
        "260643004",  # Middle cranial fossa craniectomy (procedure)
        "260644005",  # Posterior cranial fossa craniectomy (procedure)
        "260645006",  # Minilaparotomy (procedure)
        "260647003",  # Total laminectomy (procedure)
        "260648008",  # Hemilaminectomy (procedure)
        "260657002",  # Cervical collar incision with sternotomy extension (procedure)
        "260659004",  # Partial sternotomy (procedure)
        "260660009",  # Total sternotomy (procedure)
        "26076006",  # Thromboendarterectomy of thoracic artery (procedure)
        "261000087102",  # Right nephrostomy using fluoroscopic guidance (procedure)
        "261169004",  # Posterolateral thoracotomy (procedure)
        "261410000",  # Direct anastomosis of pulmonary venous confluence to left atrium (procedure)
        "261440003",  # Open - laryngofissure (procedure)
        "261479009",  # Mediastinoscopy (procedure)
        "261485002",  # Anastomosis of pulmonary venous confluence to left atrium via connecting vein (procedure)
        "261492007",  # Anterolateral thoracotomy (procedure)
        "261502004",  # Choledochojejunostomy to Roux loop (procedure)
        "261515008",  # Creation of right to left atrial tunnel with roofing patch (procedure)
        "261538006",  # Open intracoronary repair of coronary artery fistula (procedure)
        "261540001",  # Laminotomy (procedure)
        "261559004",  # Urinary reservoir bowel segment revision (procedure)
        "261563006",  # Sleeve pulmonary artery resection (procedure)
        "261566003",  # Tangential pulmonary artery resection (procedure)
        "261585000",  # Forward sliding genioplasty (procedure)
        "26163003",  # Plication of ventricle of heart (procedure)
        "261677001",  # Repair of incisional hernia using anterior sheath overlap (procedure)
        "261756009",  # Coronary interposition technique (procedure)
        "261766001",  # Construction of divided colostomy (procedure)
        "26187001",  # Kader operation, temporary gastrostomy (procedure)
        "26212005",  # Replacement of aortic valve (procedure)
        "262457001",  # Cervical mediastinoscopy (procedure)
        "26294005",  # Radical prostatectomy (procedure)
        "26337002",  # Exploratory thoracotomy (procedure)
        "26390003",  # Total colectomy (procedure)
        "26411001",  # Decortication of lung and parietal pleurectomy (procedure)
        "2644002",  # Cauterization of liver (procedure)
        "26447000",  # Revision of cutaneous ureteroileostomy (procedure)
        "26452005",  # Total gastrectomy (procedure)
        "26495009",  # Creation of shunt left-to-right, systemic to pulmonary circulation (procedure)
        "264969004",  # Laparotomy and removal of foreign body from abdominal cavity (procedure)
        "264970003",  # Reopening of abdomen (procedure)
        "264971004",  # Endoscopic division of adhesions of peritoneum (procedure)
        "264982007",  # Total excision of bursa (procedure)
        "265056007",  # Vaginal hysterectomy (procedure)
        "265059000",  # Open myomectomy (procedure)
        "265065000",  # Endoscopic excision of lesion of uterus (procedure)
        "265106004",  # Insertion of Austin-Moore prosthesis (procedure)
        "265157000",  # Cemented prosthetic total arthroplasty of hip joint (procedure)
        "265160007",  # Uncemented prosthetic total arthroplasty of hip joint (procedure)
        "265170009",  # Cemented prosthetic total arthroplasty of knee joint (procedure)
        "265172001",  # Uncemented total prosthetic arthroplasty of knee joint (procedure)
        "265184004",  # Cemented prosthetic hemiarthroplasty of shoulder joint (procedure)
        "265185003",  # Uncemented prosthetic hemiarthroplasty of shoulder joint (procedure)
        "265218003",  # Creation of ventriculocisternostomy (procedure)
        "265305009",  # Exenteration of mastoid disease (procedure)
        "265306005",  # Cortical mastoidectomy (procedure)
        "265340005",  # Partial esophagectomy and proximal gastrectomy (procedure)
        "265341009",  # Total esophagectomy and interposition of microvascularly attached jejunum (procedure)
        "265344001",  # Bypass of esophagus (procedure)
        "265345000",  # Creation of tube esophagostomy (procedure)
        "265356000",  # Repair of esophageal hiatus using thoracic approach (procedure)
        "265365007",  # Bypass of stomach by anastomosis of stomach to duodenum (procedure)
        "265375005",  # Correction of malrotation of duodenum (procedure)
        "265376006",  # Endoscopic extirpation of lesion of duodenum (procedure)
        "265379004",  # Creation of jejunostomy (procedure)
        "265380001",  # Jejunojejunal bypass (procedure)
        "265381002",  # Bypass of jejunum by anastomosis of jejunum to colon (procedure)
        "265384005",  # Creation of defunctioning ileostomy (procedure)
        "265402006",  # Endoscopic extirpation of lesion of colon (procedure)
        "265404007",  # Endoscopic extirpation of lesion of lower bowel using fiberoptic sigmoidoscope (procedure)
        "265407000",  # Endoscopic extirpation of lesion of sigmoid colon using rigid sigmoidoscope (procedure)
        "265411006",  # Partial excision of rectum and sigmoid colon for prolapse (procedure)
        "265412004",  # Rectosigmoidectomy and rectopexy (procedure)
        "265414003",  # Abdominoperineal resection of rectum (procedure)
        "265415002",  # Open extirpation of lesion of rectum (procedure)
        "265417005",  # Posterior fixation of rectum using prosthetic material (procedure)
        "265418000",  # Fixation of rectum using fascia lata (procedure)
        "265420002",  # Sponge rectopexy (procedure)
        "265421003",  # Abdominal repair of levator ani muscles (procedure)
        "265425007",  # Excision of lesion of rectum by transanal approach (procedure)
        "265439007",  # Transluminal operations on blood vessel of liver (procedure)
        "265448002",  # Anastomosis of hepatic duct to transposed jejunum and insertion of tubal prosthesis (procedure)
        "265455000",  # Endoscopic sphincterotomy of sphincter of Oddi and removal of calculus (procedure)
        "265458003",  # Pancreaticoduodenectomy and excision of surrounding tissue (procedure)
        "265459006",  # Pancreaticoduodenectomy with distal gastrectomy (procedure)
        "265461002",  # 95 percent distal pancreatectomy (procedure)
        "265462009",  # Roux-en-Y pancreaticojejunostomy (procedure)
        "265466007",  # Heart wall, septum and chamber operations (procedure)
        "265467003",  # Atrial inversion operation (procedure)
        "265472007",  # Plastic repair of mitral valve (procedure)
        "265473002",  # Plastic repair of aortic valve (procedure)
        "265474008",  # Plastic repair of tricuspid valve (procedure)
        "265475009",  # Plastic repair of pulmonary valve (procedure)
        "265480000",  # Annuloplasty of tricuspid valve (procedure)
        "265481001",  # Double anastomosis of mammary arteries to coronary arteries (procedure)
        "265492000",  # Emergency replacement of aneurysmal bifurcation of aorta by anastomosis of aorta to iliac artery (procedure)
        "265493005",  # Replacement of aneurysmal bifurcation of aorta by anastomosis of aorta to iliac artery (procedure)
        "265495003",  # Bypass of bifurcation of aorta by anastomosis of aorta to iliac artery (procedure)
        "265499009",  # Transluminal operations on aorta (procedure)
        "265506006",  # Translocation of branch of renal artery (procedure)
        "265507002",  # Transluminal operations on renal artery (procedure)
        "265512001",  # Replacement of aneurysmal iliac artery (procedure)
        "265516003",  # Reconstruction of iliac artery (procedure)
        "265517007",  # Endarterectomy of iliac artery and patch repair of iliac artery (procedure)
        "265521000",  # Open embolectomy of iliac artery (procedure)
        "265522007",  # Transluminal operations on iliac artery (procedure)
        "265528006",  # Reconstruction of femoral artery or popliteal artery (procedure)
        "265530008",  # Transluminal operations on femoral artery or popliteal artery (procedure)
        "265540006",  # Palma operation (procedure)
        "265550007",  # Transplant nephrectomy (procedure)
        "265551006",  # Open repair of kidney (procedure)
        "265554003",  # Percutaneous pyelolysis (procedure)
        "265556001",  # Percutaneous needle puncture operation on kidney or renal pelvis (procedure)
        "26565004",  # Polya operation, gastrectomy (procedure)
        "265708009",  # Division of jaw (procedure)
        "265736004",  # Through knee amputation (procedure)
        "265737008",  # Disarticulation of the tarsal bones (procedure)
        "265738003",  # Disarticulation of the tarsometatarsal joints (procedure)
        "265739006",  # Amputation through metatarsal bones (procedure)
        "265859004",  # Patent ductus arteriosus repair (procedure)
        "26619008",  # Closure of ileosigmoidal fistula (procedure)
        "266810008",  # Replacement of IV endocardial electrode (procedure)
        "26732005",  # Esophagogastrostomy by abdominal approach (procedure)
        "26897008",  # Excision of lesion of tunica vaginalis (procedure)
        "26925005",  # Transverse colectomy (procedure)
        "26934000",  # Closure of uterovaginal fistula (procedure)
        "26942004",  # Destructive procedure of coronary artery (procedure)
        "26977004",  # Repair of mitral valve with prosthesis (procedure)
        "26978009",  # Myotomy of intestine (procedure)
        "26993008",  # Abdominal esophagostomy (procedure)
        "27010001",  # Partial excision of large intestine (procedure)
        "27041000",  # Cholecystoileostomy (procedure)
        "27099004",  # Repair of retroperitoneal hernia (procedure)
        "27137001",  # Aneurysmectomy of abdominal artery (procedure)
        "271411002",  # Transluminal balloon atrial septostomy (procedure)
        "27147003",  # Filipuncture of cerebral aneurysm (procedure)
        "271489009",  # Open endoscopic operations on jejunum (procedure)
        "271490000",  # Open endoscopic operation on colon (procedure)
        "27150000",  # Total ethmoidectomy by extranasal approach (procedure)
        "27181009",  # Removal of foreign body from brain by incision (procedure)
        "27185000",  # Vaginal hysterectomy with partial colpectomy and repair of enterocele (procedure)
        "271996001",  # Excision of duodenal web (procedure)
        "27210007",  # Laryngopharyngoesophagectomy with radical neck dissection and synchronous thyroidectomy (procedure)
        "27227004",  # Excision of descending aorta (procedure)
        "27280000",  # Liver transplant with recipient hepatectomy (procedure)
        "27290008",  # Osteotomy of body of mandible (procedure)
        "27303004",  # Re-entry operation of aorta (procedure)
        "2737001",  # Exploratory craniotomy, infratentorial (procedure)
        "273990004",  # Coagulation of brain tissue using electrical energy (procedure)
        "273991000",  # Lobotomy of brain (procedure)
        "274020000",  # Epiglottic lesion excision (procedure)
        "274022008",  # Repair of coarctation of aorta (procedure)
        "274027002",  # Bowel stoma operation (procedure)
        "274037007",  # Transurethral excision of papilloma of urinary bladder (procedure)
        "27428006",  # Aortic valve commissurotomy with cardiopulmonary bypass (procedure)
        "274348006",  # Surgical biopsy of intestine (procedure)
        "274351004",  # Surgical biopsy of colon (procedure)
        "274352006",  # Surgical biopsy of rectum (procedure)
        "274354007",  # Surgical biopsy of pancreas (procedure)
        "274355008",  # Surgical biopsy of liver (procedure)
        "274358005",  # Surgical biopsy of kidney (procedure)
        "274365002",  # Surgical biopsy of ovary (procedure)
        "27443008",  # Removal of transvenous electrodes (procedure)
        "27449007",  # Removal of embolus from intracranial artery (procedure)
        "2745006",  # Omentopexy (procedure)
        "274957008",  # Radical mastectomy including axillary lymph nodes (procedure)
        "274959006",  # Morison omentopexy (procedure)
        "274960001",  # Talma omentopexy (procedure)
        "27500002",  # Aortoiliac to popliteal vascular bypass (procedure)
        "275007007",  # Ivor Lewis partial esophagogastrectomy (procedure)
        "275011001",  # Ripstein fixation of rectum using polytetrafluoroethylene mesh (procedure)
        "275014009",  # Soave endorectal pull-through operation for Hirschsprung's disease (procedure)
        "275017002",  # Duhamel removal of colorectal septum (procedure)
        "275019004",  # Roux-en-Y cholecystojejunostomy (procedure)
        "275028003",  # Heart atria and ventricle operations (procedure)
        "275030001",  # Ball valve mitral valve replacement (procedure)
        "275031002",  # Disc valve mitral valve replacement (procedure)
        "275032009",  # Single leaflet mitral valve replacement (procedure)
        "275033004",  # Bileaflet mitral valve replacement (procedure)
        "275037003",  # Repair of aortic aneurysm using Y graft (procedure)
        "275038008",  # Polyethylene terephthalate aortofemoral Y graft (procedure)
        "275039000",  # Polyethylene terephthalate aortoiliac Y graft (procedure)
        "275040003",  # Emergency femorofemoral prosthetic crossover graft (procedure)
        "275041004",  # Endarterectomy and patch repair of common iliac artery (procedure)
        "275045008",  # Transluminal operations on common iliac artery (procedure)
        "275051003",  # Reconstruction of popliteal artery (procedure)
        "275052005",  # Reconstruction of deep femoral artery (procedure)
        "275053000",  # Reconstruction of common femoral artery (procedure)
        "275054006",  # Reconstruction of femoral artery (procedure)
        "275095009",  # Syme through ankle amputation (procedure)
        "275145005",  # Reconstruction of superficial femoral artery (procedure)
        "275162000",  # Hofmeister valved gastrectomy (procedure)
        "275194005",  # Bancroft partial gastrectomy and esophagogastrostomy (procedure)
        "275199000",  # Ball valve aortic valve replacement (procedure)
        "275200002",  # Disc valve aortic valve replacement (procedure)
        "275201003",  # Single leaflet aortic valve replacement (procedure)
        "275202005",  # Bileaflet aortic valve replacement (procedure)
        "275203000",  # Ball valve tricuspid valve replacement (procedure)
        "275204006",  # Disc valve tricuspid valve replacement (procedure)
        "275205007",  # Single leaflet tricuspid valve replacement (procedure)
        "275206008",  # Bileaflet tricuspid valve replacement (procedure)
        "275207004",  # Ball valve pulmonary valve replacement (procedure)
        "275209001",  # Disc valve pulmonary valve replacement (procedure)
        "275210006",  # Single leaflet pulmonary valve replacement (procedure)
        "275211005",  # Bileaflet pulmonary valve replacement (procedure)
        "275215001",  # Left internal mammary artery single anastomosis (procedure)
        "275216000",  # Right internal mammary artery single anastomosis (procedure)
        "275217009",  # Ligation of sinus of Valsalva fistula (procedure)
        "275227003",  # Myocardial revascularization (procedure)
        "275230005",  # Repair of duodenum (procedure)
        "275232002",  # Benchekroun construction of continent urinary reservoir (procedure)
        "275254000",  # Open embolectomy of common iliac artery (procedure)
        "27560000",  # Incision of abdominal artery (procedure)
        "27613000",  # Ventriculopleural shunt with valve (procedure)
        "276178005",  # Uncemented prosthetic hemiarthroplasty of hip joint (procedure)
        "276185009",  # Repair of ileostomy (procedure)
        "276186005",  # Repair of rectum (procedure)
        "276189003",  # Excision of broad ligament (procedure)
        "276190007",  # Ileocolic resection (procedure)
        "27651000",  # Incision and drainage of extraperitoneal abscess (procedure)
        "27672002",  # Intraoperative transluminal angioplasty of renal artery (procedure)
        "276893007",  # Percutaneous balloon angioplasty of superior vena cava (procedure)
        "276894001",  # Percutaneous balloon angioplasty of inferior vena cava (procedure)
        "276921001",  # Percutaneous endarterectomy of intracranial artery (procedure)
        "276925005",  # Percutaneous endarterectomy of abdominal aorta (procedure)
        "276926006",  # Percutaneous endarterectomy of renal artery (procedure)
        "276927002",  # Percutaneous endarterectomy of hepatic artery (procedure)
        "276928007",  # Percutaneous endarterectomy of celiac artery (procedure)
        "276929004",  # Percutaneous endarterectomy of superior mesenteric artery (procedure)
        "276930009",  # Percutaneous endarterectomy of common iliac artery (procedure)
        "276931008",  # Percutaneous endarterectomy of internal iliac artery (procedure)
        "276932001",  # Percutaneous endarterectomy of external iliac artery (procedure)
        "276949008",  # Percutaneous endarterectomy of internal carotid artery (procedure)
        "276950008",  # Percutaneous endarterectomy of external carotid artery (procedure)
        "276951007",  # Percutaneous endarterectomy of common carotid artery (procedure)
        "276977004",  # Percutaneous venous thrombectomy (procedure)
        "276978009",  # Percutaneous embolectomy of intracranial artery (procedure)
        "276981004",  # Percutaneous thrombectomy of intracranial vein (procedure)
        "276982006",  # Percutaneous thrombectomy of iliofemoral vein zone (procedure)
        "277065002",  # C mandibular osteotomy (procedure)
        "27711007",  # Destructive procedure on heart (procedure)
        "277182002",  # Resection of pulmonary artery (procedure)
        "277192005",  # Coronary artery graft placement (procedure)
        "277451006",  # Part organ pancreatic transplant (procedure)
        "27748003",  # Arthroplasty of knee, condyle and plateau, lateral compartment (procedure)
        "27755001",  # Operation on vessels of heart (procedure)
        "27779004",  # Sternotomy (procedure)
        "277992006",  # Percutaneous removal of foreign body from intracranial artery (procedure)
        "277994007",  # Percutaneous removal of foreign body from coronary artery (procedure)
        "277997000",  # Percutaneous removal of foreign body from pulmonary artery (procedure)
        "277998005",  # Percutaneous removal of foreign body from intracranial vein (procedure)
        "27842006",  # Radical excision of lymph nodes (procedure)
        "278704006",  # Iliac fossa muscle cutting incision (procedure)
        "27892007",  # Closure of fecal fistula (procedure)
        "27944006",  # Removal of electronic heart device pulse generator (procedure)
        "28009009",  # Liver transplant without recipient hepatectomy (procedure)
        "280390001",  # Excisional biopsy of hypophysis (procedure)
        "280392009",  # Shunt procedure into peritoneum (procedure)
        "281187007",  # Open atrial septectomy with inflow occlusion (procedure)
        "281216000",  # Excisional biopsy of pericardium (procedure)
        "2813008",  # Nephroureterocystectomy (procedure)
        "281413005",  # Duodenoplasty (procedure)
        "281590006",  # Exteriorization of brain ventricular shunt (procedure)
        "281627009",  # Transposition of adrenal tissue (procedure)
        "28178002",  # Hepatopexy (procedure)
        "28197002",  # Axillary-femoral artery bypass graft with vein (procedure)
        "282191000",  # Laminotomy for decompression and exploration (procedure)
        "282435000",  # Maxillectomy (procedure)
        "282568009",  # Ring annuloplasty (procedure)
        "282570000",  # Enlargement annuloplasty (procedure)
        "28264008",  # Amygdalotomy (procedure)
        "28301000",  # Manchester-Fothergill operation on uterus (procedure)
        "283165009",  # Decompression laminectomy (procedure)
        "28369001",  # Spinal pleurothecal shunt with valve (procedure)
        "284088004",  # Gill operation (procedure)
        "284348003",  # Excision of rejected transplanted kidney (procedure)
        "28467003",  # Intrathoracic esophagoileostomy (procedure)
        "2851008",  # Arthrotomy for synovectomy of sternoclavicular joint (procedure)
        "28523002",  # Excision of hydrocele of processus vaginalis testis (procedure)
        "2857007",  # Complete excision of pineal gland (procedure)
        "28579000",  # Radical retropubic prostatectomy with lymph node biopsy (procedure)
        "285865009",  # Removal of electrodes of brain with synchronous replacement (procedure)
        "28609005",  # Closure of vesicouterine fistula (procedure)
        "28658009",  # Plication of diaphragm for hernia repair by thoracoabdominal approach (procedure)
        "287205001",  # Brain decortication (procedure)
        "287206000",  # Cranial venous sinus ligation (procedure)
        "287208004",  # Operation on thalamus (procedure)
        "287218009",  # Cranial ventriculostomy (procedure)
        "287277008",  # Indirect heart revascularization (procedure)
        "287278003",  # Excision of cyst of pericardium (procedure)
        "287279006",  # Heart - aortic branch implant (procedure)
        "287286003",  # Pericardial decortication (procedure)
        "287289005",  # Ruptured heart aneurysm repair (procedure)
        "287290001",  # Heart suturing (procedure)
        "287291002",  # Heart ligation (procedure)
        "287292009",  # Infarct myocardiectomy (procedure)
        "287294005",  # Atrial appendage excision (procedure)
        "287310004",  # Lung tumor excision (procedure)
        "287312007",  # Heart valve replacement - graft (procedure)
        "287315009",  # Suture of diaphragm (procedure)
        "287334009",  # Esophagus to antesternal anastomosis (procedure)
        "287347004",  # Arterial bypass using vein graft (procedure)
        "28738003",  # Ligation of thoracic artery (procedure)
        "287440003",  # Radical laryngectomy (procedure)
        "287558007",  # Surgical biopsy of diaphragm (procedure)
        "287559004",  # Surgical biopsy of small intestine (procedure)
        "287644000",  # Supramalleolar ankle amputation (procedure)
        "287647007",  # Disarticulation at wrist (procedure)
        "287707008",  # Transurethral excision of neck of urinary bladder (procedure)
        "287708003",  # Transurethral excision of urinary bladder (procedure)
        "287718008",  # Total cystectomy and urethrectomy (procedure)
        "287720006",  # Cystoprostatectomy (procedure)
        "287722003",  # Cystectomy - wedge resection (procedure)
        "287723008",  # Excision of dome of urinary bladder (procedure)
        "287726000",  # Open excision of neoplasm of urinary bladder (procedure)
        "287728004",  # Open excision of diverticulum of urinary bladder (procedure)
        "287730002",  # Kidney - local excision (procedure)
        "287736008",  # Nephrotomy with drainage (procedure)
        "287737004",  # Ureter implanted into rectum (procedure)
        "287783005",  # Bowel stoma repair (procedure)
        "287784004",  # Local excision of rectum (procedure)
        "287788001",  # Bowel stoma closure (procedure)
        "287793003",  # Intestine bypass shunt (procedure)
        "287806006",  # External closure - rectal fistula (procedure)
        "287807002",  # Internal closure - rectal fistula (procedure)
        "287809004",  # Total gastroduodenectomy (procedure)
        "287810009",  # Gastric anastomosis revision (procedure)
        "287815004",  # Excision of excess colostomy mucosa (procedure)
        "287816003",  # Radical gastrectomy (procedure)
        "287818002",  # Partial gastroduodenectomy (procedure)
        "287819005",  # Pylorectomy (procedure)
        "287821000",  # Cardiectomy (procedure)
        "287824008",  # Enterotomy for exploration (procedure)
        "287827001",  # Excision of redundant mucosa from ileostomy (procedure)
        "287828006",  # Duodenal ulcer excision (procedure)
        "287829003",  # Enterotomy for drainage (procedure)
        "287832000",  # Stomach ulcer excision (procedure)
        "287833005",  # Stomach diverticulum excision (procedure)
        "287834004",  # Gastroenterostomy - no gastrectomy (procedure)
        "287846000",  # Radical pancreaticoduodenectomy (procedure)
        "287847009",  # Partial pancreatectomy (procedure)
        "287848004",  # Pancreas local excision (procedure)
        "287852004",  # Removal of cerebral ventricular valve (procedure)
        "287853009",  # Laparotomy and division of peritoneal adhesions (procedure)
        "287859008",  # Sigmoid myotomy (procedure)
        "287924009",  # Excision of cervix stump (procedure)
        "287931008",  # Fallopian tube cornual resection (procedure)
        "287935004",  # Partial excision of bilateral fallopian tubes (procedure)
        "287945002",  # Resection of bilateral fallopian tubes (procedure)
        "287966005",  # Radical vulvectomy (procedure)
        "287992003",  # Bowel malrotation manipulation (procedure)
        "288001002",  # Pyelotomy and drainage (procedure)
        "288002009",  # Pyelotomy and exploration (procedure)
        "288007003",  # Renal anastomosis (procedure)
        "288011009",  # Transurethral electroresection of urinary bladder (procedure)
        "288012002",  # Transurethral excision of ulcer of urinary bladder (procedure)
        "288013007",  # Open excision of urinary bladder (procedure)
        "288017008",  # Retroperitoneal tissue operations (procedure)
        "288018003",  # Retroperitoneal drainage operation (procedure)
        "288020000",  # Retroperitoneal adhesions freeing (procedure)
        "288042004",  # Hysterectomy and fetus removal (procedure)
        "288043009",  # Hysterectomy in pregnancy (procedure)
        "288184005",  # Removal of IV endocardial electrode (procedure)
        "28851001",  # Esophagojejunoplasty (procedure)
        "28856006",  # Repair of ruptured aneurysm of abdominal aorta involving iliac vessels with graft (procedure)
        "288846002",  # Cerebrospinal fluid shunt removal (procedure)
        "28886004",  # Closure of ventricular septal defect with infundibular resection and patch (procedure)
        "28890002",  # Repair of traumatic diaphragmatic hernia, chronic (procedure)
        "289060008",  # Anterior resection of rectal prolapse (procedure)
        "289184002",  # Removal of tumor from inferior vena cava (procedure)
        "28981008",  # Closure of esophagostomy by cervical approach (procedure)
        "290017007",  # Excision of anterior mucosal prolapse of rectum (procedure)
        "29063004",  # Incision of cerebrum (procedure)
        "29065006",  # Sigmoidotomy (procedure)
        "29075009",  # Femoral-anterior tibial artery bypass graft with vein (procedure)
        "29114001",  # Incision and drainage of cranial sinus (procedure)
        "29126006",  # Incision of cortical adhesions of brain (procedure)
        "29205000",  # Proctopexy for prolapse by perineal approach (procedure)
        "29222003",  # Posterior mitral commissurotomy (procedure)
        "29224002",  # Operative endoscopy of small intestine (procedure)
        "29255008",  # Iliofemoral bypass graft with vein (procedure)
        "2945004",  # Suprapubic excision of diverticulum of urinary bladder (procedure)
        "29453004",  # Jejunotomy (procedure)
        "29454005",  # Surgical occlusion of aorta (procedure)
        "29474003",  # Take-down of systemic-pulmonary artery anastomosis in total repair of tetralogy of Fallot (procedure)
        "294743008",  # Closure of ureterovisceral fistula (procedure)
        "29507009",  # Transplantation of heart valve (procedure)
        "29529008",  # Supracervical hysterectomy with removal of both tubes and ovaries (procedure)
        "2960001",  # Closure of fistula of uterine cervix (procedure)
        "29630005",  # Operation on pancreas (procedure)
        "29656003",  # Radical mastoidectomy, modified (procedure)
        "29672006",  # Partial oophorectomy (procedure)
        "2968008",  # Craniectomy with treatment of penetrating wound of brain (procedure)
        "29690007",  # Excision of lesion of artery of upper limb (procedure)
        "2977001",  # Thrombectomy of lower limb vein (procedure)
        "298049006",  # Upper thigh amputation (procedure)
        "298050006",  # Lower thigh amputation (procedure)
        "29819009",  # Aortocoronary bypass of one coronary artery (procedure)
        "29827000",  # Excision of bilateral fallopian tubes and ovaries (procedure)
        "29830007",  # Lysis of cortical adhesions of brain (procedure)
        "29858004",  # Excision of artery (procedure)
        "298799004",  # Pancreaticojejunostomy, side-to-side anastomosis (procedure)
        "299673006",  # Blalock Hanlon operation (procedure)
        "299681007",  # Construction of transverse loop colostomy (procedure)
        "299682000",  # Construction of sigmoid loop colostomy (procedure)
        "29978008",  # Destructive procedure of liver (procedure)
        "3001009",  # Total lobectomy with bronchoplasty (procedure)
        "30036002",  # Excision of lesion of mastoid bone (procedure)
        "30038001",  # Ureteropyelostomy (procedure)
        "300897007",  # Total salpingectomy (procedure)
        "301079005",  # Embolectomy of renal artery (procedure)
        "30123000",  # Repair of atrial septal defect with prosthesis by closed heart technique (procedure)
        "301426005",  # Excision of base of skull (procedure)
        "30160001",  # Vaginal hysterectomy with total colpectomy and repair of enterocele (procedure)
        "30161002",  # Incision and exploration of duodenum (procedure)
        "30162009",  # Destructive procedure of pericardium (procedure)
        "301698008",  # Snare resection of lesion - large intestine (procedure)
        "301699000",  # Snare resection of lesion of colon (procedure)
        "301700004",  # Snare resection of lesion of rectum (procedure)
        "301704008",  # Roux-en-Y hepaticojejunostomy (procedure)
        "301884003",  # Removal of aortic cross clamp (procedure)
        "30195003",  # Excision of lesion of cerebral cortex (procedure)
        "30215008",  # Closure of esophageal fistula (procedure)
        "302190000",  # Abdominal hysterectomy and right salpingo-oophorectomy (procedure)
        "302191001",  # Abdominal hysterectomy and left salpingo-oophorectomy (procedure)
        "302326009",  # Revision of proximal catheter of ventricular shunt (procedure)
        "302329002",  # Revision of distal catheter of ventricular shunt (procedure)
        "302340005",  # Excision of bilateral adrenal glands (procedure)
        "302354002",  # Small intestinal strictureplasty (procedure)
        "302366008",  # Excision of intra-abdominal testis (procedure)
        "302376006",  # Right oophorectomy (procedure)
        "302377002",  # Left oophorectomy (procedure)
        "302379004",  # Excision biopsy of ovary (procedure)
        "302448002",  # Previous chest wall incision - reopening (procedure)
        "302477001",  # Osteotomy of maxilla (procedure)
        "302621009",  # Closed tricuspid commissurotomy (procedure)
        "30273008",  # Repair of neonatal diaphragmatic hernia (procedure)
        "303167002",  # Patch repair of celiac artery (procedure)
        "303169004",  # Patch repair of superior mesenteric artery (procedure)
        "303272002",  # Jatene procedure (procedure)
        "303401008",  # Parks panproctocolectomy, anastomosis of ileum to anus and creation of pouch (procedure)
        "303439000",  # Division of pectoralis minor (procedure)
        "303453000",  # Orbital osteotomy (procedure)
        "303565004",  # Anastomosis of pharynx to esophagus (procedure)
        "303572003",  # Total elbow joint replacement prosthesis (physical object)
        "303583007",  # Colon and cecum operations (procedure)
        "303584001",  # Sphincter-saving proctectomy (procedure)
        "303621003",  # Total pharyngectomy (procedure)
        "30368004",  # Excision of artery of extremity (procedure)
        "303876007",  # Meniscal reattachment (procedure)
        "303877003",  # Meniscal suture (procedure)
        "30398006",  # Renal calicotomy (procedure)
        "30402002",  # Ripstein operation, repair of prolapsed rectum (procedure)
        "30405000",  # Incision of periureteral tissue (procedure)
        "304060006",  # Anastomosis of esophagus to transposed jejunum (procedure)
        "304389001",  # Axillo-unifemoral bypass graft (procedure)
        "30456009",  # Open heart valvuloplasty without replacement of valve (procedure)
        "304583001",  # Extended right hemicolectomy and ileostomy (procedure)
        "304584007",  # Colonic pouch operations (procedure)
        "304585008",  # Creation of colonic pouch (procedure)
        "304586009",  # Revision of colonic pouch (procedure)
        "304587000",  # Excision of colonic pouch (procedure)
        "304588005",  # Trial dissection of rectum (procedure)
        "304834009",  # Removal of ventriculoatrial shunt (procedure)
        "304837002",  # Removal of ventriculopleural shunt (procedure)
        "304848009",  # Callosotomy (procedure)
        "304849001",  # Subpial transection of cortex (procedure)
        "304918005",  # Removal of subcutaneous cerebrospinal fluid reservoir (procedure)
        "304926002",  # Open atrial septostomy (procedure)
        "30562002",  # Ligation of gastric artery (procedure)
        "305621000",  # Bypass of duodenum by anastomosis of duodenum to jejunum (procedure)
        "30582003",  # Duodenostomy (procedure)
        "305928009",  # Open atrial septectomy (procedure)
        "305969000",  # Schroeder operation for cervical myomectomy (procedure)
        "3060007",  # Forequarter amputation, right (procedure)
        "30613005",  # Aneurysmectomy with anastomosis (procedure)
        "30670000",  # Anastomosis of thoracic artery to coronary artery, double (procedure)
        "306812009",  # Thrombectomy of vascular graft (procedure)
        "306966000",  # Excision of myoma of cervix uteri (procedure)
        "307009004",  # Wedge resection of rectum (procedure)
        "307010009",  # Extended right hemihepatectomy (procedure)
        "307011008",  # Caudate lobectomy of liver (procedure)
        "30706000",  # Incision of small intestine (procedure)
        "307183003",  # Creation of ileo-anal pouch (procedure)
        "307185005",  # Creation of ileo-anal W-shaped pouch (procedure)
        "307193005",  # Resection of terminal ileum (procedure)
        "307195003",  # Laparoscopic gastroenterostomy (procedure)
        "307196002",  # Excision of cyst of pancreas (procedure)
        "307241002",  # Dilation of colonic stricture (procedure)
        "307242009",  # Dilation of colonic anastomotic stricture (procedure)
        "307244005",  # Dilation of rectal stricture (procedure)
        "307245006",  # Dilation of rectal anastomotic stricture (procedure)
        "307260009",  # Suture of duodenal ulcer (procedure)
        "307261008",  # Closure of perforated bowel ulcer (procedure)
        "307262001",  # Extraperitoneal incision (procedure)
        "307281009",  # Transection of vagus (procedure)
        "307303006",  # Billroth I gastric antrectomy (procedure)
        "307304000",  # Polya gastric antrectomy (procedure)
        "307330001",  # Esophagectomy and interposition reconstruction (procedure)
        "307331002",  # Esophagectomy and jejunal interposition (procedure)
        "307332009",  # Esophagectomy and colonic interposition (procedure)
        "307364002",  # Repair of jejunum (procedure)
        "307367009",  # Repair of small intestine (procedure)
        "307368004",  # Repair of ileum (procedure)
        "307507006",  # Excision jejunocolic fistula (procedure)
        "307508001",  # Excision jejunorectal fistula (procedure)
        "307509009",  # Excision of ileorectal fistula (procedure)
        "307510004",  # Oversewing of bleeding duodenal ulcer (procedure)
        "307511000",  # Under-running of bleeding duodenal ulcer (procedure)
        "307546008",  # Operation on brain ventricle (procedure)
        "307555006",  # Enterotomy and removal of gallstone (procedure)
        "307560005",  # Closure of intestinal-vascular graft fistula (procedure)
        "307561009",  # Ileectomy and ileostomy (procedure)
        "307563007",  # Extracranial transection of accessory nerve (procedure)
        "307564001",  # Extracranial transection of trigeminal nerve (procedure)
        "307568003",  # Excision of neck of pancreas (procedure)
        "307569006",  # Partial excision of head of pancreas (procedure)
        "307581005",  # Laparoscopic interval appendectomy (procedure)
        "307583008",  # Excision of cecum (procedure)
        "307584002",  # Cecoplasty (procedure)
        "307585001",  # Percutaneous cecostomy (procedure)
        "307586000",  # Laparoscopic transverse colectomy (procedure)
        "307615003",  # Extended left hemihepatectomy (procedure)
        "307616002",  # Excision of liver metastasis (procedure)
        "307619009",  # Laparoscopic biopsy of liver lesion (procedure)
        "307620003",  # Marsupialization of liver cyst (procedure)
        "307621004",  # Laparoscopic marsupialization of liver cyst (procedure)
        "307626009",  # Destruction of liver metastasis (procedure)
        "307627000",  # Cryotherapy of liver metastases (procedure)
        "307638000",  # Anastomosis of ascending colon to rectum (procedure)
        "307639008",  # Anastomosis of transverse colon to rectum (procedure)
        "307640005",  # Anastomosis of descending colon to rectum (procedure)
        "307642002",  # Anastomosis of ascending to sigmoid colon (procedure)
        "307643007",  # Anastomosis of transverse to sigmoid colon (procedure)
        "307644001",  # Transverse colectomy and ileostomy (procedure)
        "307645000",  # Transverse colectomy and colostomy (procedure)
        "307648003",  # Sigmoid colectomy and ileostomy (procedure)
        "307654002",  # Subtotal colectomy with cecorectal anastomosis (procedure)
        "307655001",  # Anastomosis of cecum to rectum (procedure)
        "307656000",  # Subtotal colectomy with ileosigmoid anastomosis (procedure)
        "307657009",  # Left hemicolectomy and colostomy (procedure)
        "307658004",  # Sigmoid colectomy and colostomy (procedure)
        "307659007",  # Subtotal colectomy with ileorectal anastomosis (procedure)
        "307666008",  # Total colectomy and ileostomy (procedure)
        "307667004",  # Total colectomy, ileostomy and rectal mucous fistula (procedure)
        "307668009",  # Construction of rectal mucous fistula (procedure)
        "307669001",  # Total colectomy, ileostomy and closure of rectal stump (procedure)
        "307709007",  # Repair of perforated colon (procedure)
        "307710002",  # Ligation of abdominal aorta (procedure)
        "307771009",  # Radical abdominal hysterectomy (procedure)
        "307799000",  # Cardiac valvotomy (procedure)
        "307800001",  # Endarterectomy of iliac artery (procedure)
        "3078002",  # Embolectomy with catheter of renal artery by abdominal incision (procedure)
        "307814001",  # Cemented arthroplasty of hip joint (procedure)
        "307815000",  # Uncemented prosthetic arthroplasty of hip joint (procedure)
        "307817008",  # Cemented arthroplasty of knee joint (procedure)
        "307819006",  # Uncemented prosthetic arthroplasty of knee joint (procedure)
        "307823003",  # Heart wall operations (procedure)
        "307944004",  # Excision of mucous cyst of vocal cord (procedure)
        "307983002",  # Removal of normal pituitary gland (procedure)
        "307987001",  # Surgical biopsy of adrenal gland (procedure)
        "308022009",  # Excision of cholesteatoma of mastoid (procedure)
        "30803004",  # Printen and Mason operation, high gastric bypass (procedure)
        "30845002",  # Creation of porto-splenic shunt (procedure)
        "308662007",  # Porcine mitral valve replacement (procedure)
        "308663002",  # Porcine aortic valve replacement (procedure)
        "308664008",  # Porcine tricuspid valve replacement (procedure)
        "308665009",  # Biological pulmonary valve replacement (procedure)
        "308666005",  # Porcine pulmonary valve replacement (procedure)
        "308667001",  # Biological atrioventricular valve replacement (procedure)
        "308668006",  # Ball valve atrioventricular valve replacement (procedure)
        "308669003",  # Disc valve atrioventricular valve replacement (procedure)
        "308670002",  # Single leaflet atrioventricular valve replacement (procedure)
        "308671003",  # Bileaflet atrioventricular valve replacement (procedure)
        "308672005",  # Biological truncal valve replacement (procedure)
        "308673000",  # Ball valve truncal valve replacement (procedure)
        "308674006",  # Disc valve truncal valve replacement (procedure)
        "308675007",  # Single leaflet truncal valve replacement (procedure)
        "308676008",  # Bileaflet truncal valve replacement (procedure)
        "308677004",  # Resection of coarctation (procedure)
        "308678009",  # Resection of segmental aortic hypoplasia (procedure)
        "308679001",  # Resection of interrupted aortic arch (procedure)
        "308681004",  # Prosthetic arthroplasty of shoulder (procedure)
        "308696000",  # Coarctation angioplasty (procedure)
        "3090000",  # Aneurysmectomy with graft replacement of lower limb artery (procedure)
        "309458002",  # Cemented Thompson hemiarthroplasty of hip joint (procedure)
        "309876004",  # Endocervical excision (procedure)
        "309879006",  # Abdominal hysterocolpectomy (procedure)
        "309880009",  # Vaginal hysterocolpectomy (procedure)
        "310223003",  # Repair of congenital diaphragmatic hernia using sutures (procedure)
        "310224009",  # Repair of congenital diaphragmatic hernia using patch (procedure)
        "310242004",  # Patch repair of inferior mesenteric artery (procedure)
        "310395004",  # Patching of mitral valve (procedure)
        "310396003",  # Suturing of mitral valve (procedure)
        "310397007",  # Patching of atrioventricular valve (procedure)
        "310398002",  # Suturing of atrioventricular valve (procedure)
        "310400003",  # Patching of coronary sinus atrial septal defect (procedure)
        "310412004",  # Ligation of aortopulmonary window (procedure)
        "310415002",  # Suturing of sinus of Valsalva fistula (procedure)
        "310416001",  # Patching of sinus of Valsalva fistula (procedure)
        "310420002",  # Patch repair of traumatic rupture of diaphragm (procedure)
        "310421003",  # Patch repair of postoperative rupture of diaphragm (procedure)
        "310442001",  # Closure of atrial septal defect using sutures (procedure)
        "310443006",  # Closure of atrial septal defect using patch (procedure)
        "310444000",  # Closure of patent foramen ovale using sutures (procedure)
        "310445004",  # Closure of patent foramen ovale using patch (procedure)
        "310470004",  # Suture of defect of atrioventricular valve leaflet (procedure)
        "310471000",  # Patching of defect of atrioventricular valve leaflet (procedure)
        "310472007",  # Suturing of defect of mitral leaflet (procedure)
        "310473002",  # Patching of defect of mitral leaflet (procedure)
        "310474008",  # Suturing of defect of tricuspid valve leaflet (procedure)
        "310475009",  # Patching of defect of tricuspid valve leaflet (procedure)
        "310476005",  # Suturing of tricuspid valve (procedure)
        "310477001",  # Patching of tricuspid valve (procedure)
        "310478006",  # Patch repair of pulmonary artery (procedure)
        "310490008",  # Division of coronary arteriovenous fistula (procedure)
        "310615005",  # Patching of left ventricle (procedure)
        "310622002",  # Patch repair of iliac artery (procedure)
        "310623007",  # Patch repair of common iliac artery (procedure)
        "310625000",  # Patch repair of adrenal artery (procedure)
        "31078004",  # Closure of esophagocutaneous fistula (procedure)
        "310803009",  # Interposition of microvascularly attached colon (procedure)
        "310804003",  # Interposition of microvascularly attached jejunum (procedure)
        "31118007",  # Intracranial phlebectomy (procedure)
        "31130001",  # Total abdominal colectomy with proctectomy and ileostomy (procedure)
        "31158009",  # Enterectomy with double-barrel enterostomy (procedure)
        "31191000",  # Excision of septum of uterus (procedure)
        "312226000",  # Open insertion of iliac artery stent (procedure)
        "312227009",  # Percutaneous transluminal insertion of iliac artery stent (procedure)
        "31223002",  # Total urethrectomy including cystostomy in male (procedure)
        "312302004",  # Plication of ascending aorta (procedure)
        "312311004",  # Replacement of ascending aorta (procedure)
        "312316009",  # Direct anastomosis of right ventricle to pulmonary trunk (procedure)
        "312332007",  # Operation on the atrial septum (procedure)
        "312340001",  # Temporary percutaneous endoscopic gastrostomy (procedure)
        "312341002",  # Permanent percutaneous endoscopic gastrostomy (procedure)
        "312406002",  # Anterior resection of rectum with stapled anastomosis of sigmoid colon to anus with creation of J pouch (procedure)
        "312407006",  # Anastomosis of sigmoid colon to anal canal (procedure)
        "312497000",  # Salvage abdominoperineal resection of rectum (procedure)
        "312687001",  # Femoro-anterior tibial bypass graft (procedure)
        "312688006",  # Femoropopliteal prosthetic bypass graft (procedure)
        "312692004",  # Mesenteric artery bypass graft (procedure)
        "312693009",  # Aorta to inferior mesenteric artery bypass graft (procedure)
        "312694003",  # Excision of necrotic patch of small intestine (procedure)
        "312695002",  # Excision of anal fissure and island flap closure (procedure)
        "312734005",  # Femoroposterior tibial reversed vein bypass graft (procedure)
        "312735006",  # Femoroperoneal reversed vein bypass graft (procedure)
        "312736007",  # Ilioanterior tibial bypass graft (procedure)
        "313030004",  # Donor renal transplantation (procedure)
        "313039003",  # Solid organ transplant (procedure)
        "313270001",  # Percutaneous insertion of nephrostomy tube (procedure)
        "314118002",  # Laparoscopic total splenectomy (procedure)
        "31413008",  # Operative procedure on coronary artery (procedure)
        "31415001",  # Atrial septostomy by blade method (procedure)
        "314491003",  # Prosthetic hybrid total replacement of hip joint (procedure)
        "314511002",  # Left hepatic trisegmentectomy (procedure)
        "314587006",  # Trial dissection of liver mass (procedure)
        "314592008",  # Low anterior resection of rectum (procedure)
        "314593003",  # Posterior gastrojejunostomy (procedure)
        "314594009",  # Creation of loop ileostomy (procedure)
        "314610005",  # Transanal submucosal excision of rectal tumor (procedure)
        "31461001",  # Transection of vagus nerve by transthoracic approach (procedure)
        "314617008",  # Chemoembolization of liver metastases (procedure)
        "314636003",  # Insertion of reversed intestinal segment (procedure)
        "314637007",  # Insertion of reversed onlay intestinal graft (procedure)
        "314638002",  # Excision of reversed intestinal segment (procedure)
        "314639005",  # Excision of reversed onlay intestinal graft (procedure)
        "31484008",  # Resection of left ventricle of heart (procedure)
        "315059002",  # Laparoscopic salpingectomy (procedure)
        "315060007",  # Open salpingectomy (procedure)
        "31521004",  # Resection of intestine for interposition (procedure)
        "315285005",  # Ligation of hepatic artery (procedure)
        "315319006",  # Anastomosis of intestine to anus (procedure)
        "315321001",  # Right hemicolectomy and anastomosis of ileum to colon (procedure)
        "315322008",  # Transverse colectomy with anastomosis (procedure)
        "315324009",  # Left hemicolectomy with anastomosis (procedure)
        "315325005",  # Left hemicolectomy with stoma (procedure)
        "315326006",  # Sigmoid colectomy with anastomosis (procedure)
        "315327002",  # Sigmoid colectomy with stoma (procedure)
        "315328007",  # Anterior resection of rectum with anastomosis (procedure)
        "315343009",  # Percutaneous thrombolysis of iliac artery (procedure)
        "315362007",  # Repair of thoracoabdominal aortic aneurysm (procedure)
        "31573003",  # Anastomosis of carotid-subclavian artery (procedure)
        "31736003",  # Construction of corpora cavernosa-corpus spongiosum shunt (procedure)
        "31738002",  # Jejunocecostomy for obesity (procedure)
        "31750008",  # Division of superior laryngeal nerve (procedure)
        "31778009",  # Replacement of transvenous ventricular pacemaker electrode leads (procedure)
        "31844009",  # Ligation of intracranial artery (procedure)
        "31855009",  # Axillary-brachial artery vascular bypass (procedure)
        "31904001",  # Adrenalectomy (procedure)
        "31937004",  # Embolectomy of descending aorta (procedure)
        "31938009",  # Repair of ruptured aneurysm with graft of iliac artery (procedure)
        "31977007",  # Aneurysmectomy of lower limb artery (procedure)
        "32040001000004102",  # Excision of mass of cerebrum (procedure)
        "32044008",  # Partial colectomy by abdominal approach (procedure)
        "32091001",  # Repair of large intestine (procedure)
        "32159004",  # Osteotomy of mandible by Gigli saw (procedure)
        "32238004",  # Empyemectomy (procedure)
        "32252002",  # Arteriectomy with graft replacement of thoracic aorta (procedure)
        "32325005",  # Inversion of diverticulum of small intestine (procedure)
        "32327002",  # Excision of lesion of stomach (procedure)
        "32413006",  # Transplantation of heart (procedure)
        "32477003",  # Heart-lung transplant with recipient cardiectomy-pneumonectomy (procedure)
        "32483000",  # Angiectomy with anastomosis of thoracic artery (procedure)
        "32581000",  # Partial hip replacement by cup with acetabuloplasty (procedure)
        "32606000",  # Marsupialization of cyst of brain (procedure)
        "32632006",  # Excision of artery of head and neck (procedure)
        "32678003",  # Total ostectomy for graft of mandible (procedure)
        "32687007",  # Anastomosis of extracranial-intracranial arteries (procedure)
        "32733009",  # Ventriculocisternostomy, third ventricle, stereotactic method (procedure)
        "32899003",  # Resection of subvalvular tissue for discrete subvalvular aortic stenosis (procedure)
        "32907006",  # Operation on aorta (procedure)
        "32912007",  # Thrombolysis of coronary artery by intravenous infusion (procedure)
        "32918006",  # Anastomosis of esophagus by intrathoracic approach (procedure)
        "32956007",  # Transplantation of tissue of rectum (procedure)
        "32998005",  # Latzko operation on vagina (procedure)
        "331000087101",  # Angioplasty of left renal artery using fluoroscopic guidance with contrast (procedure)
        "33131006",  # Closure of nephrocutaneous fistula (procedure)
        "33149006",  # Pancreatectomy (procedure)
        "33179004",  # Suture of jejunum (procedure)
        "33327009",  # Condylotomy of mandible (procedure)
        "33349003",  # Creation of carotid-subclavian shunt (procedure)
        "33422004",  # Removal of blood clot from kidney by incision (procedure)
        "33424003",  # Radical excision of periaortic lymph nodes (procedure)
        "33479006",  # Distal subtotal pancreatectomy with splenectomy and pancreaticojejunostomy (procedure)
        "33507007",  # Excision of malignant tumor of rectum by transanal approach (procedure)
        "33582004",  # Revision of pyelointestinal anastomosis (procedure)
        "33639009",  # Segmental lobectomy with concomitant decortication (procedure)
        "33649007",  # Fixation of large intestine to abdominal wall (procedure)
        "336863008",  # Excision of cyst of lung (procedure)
        "33755005",  # Lysis of adhesions of kidney (procedure)
        "33756006",  # Partial adrenalectomy (procedure)
        "33759004",  # Suture of cecum (procedure)
        "337877003",  # Thoracotomy for excision of emphysematous bleb of lung (procedure)
        "33808009",  # Closure of hepatopleural fistula (procedure)
        "33932003",  # Replacement of artificial heart (procedure)
        "34044009",  # Repair of papillary muscle of heart (procedure)
        "34068001",  # Heart valve replacement (procedure)
        "340922009",  # Partial hip replacement by cup (procedure)
        "340926007",  # Incision of heart (procedure)
        "34097003",  # Craniectomy for excision of meningioma, infratentorial (procedure)
        "341000087107",  # Angioplasty of right renal artery using fluoroscopic guidance with contrast (procedure)
        "34130000",  # Cholecystocecostomy (procedure)
        "341943002",  # Atrioventriculostomy (procedure)
        "34214004",  # Creation of aorta-subclavian-carotid vascular bypass (procedure)
        "34220003",  # Sequestrectomy of bone of skull (procedure)
        "34233009",  # Repair of aneurysm with graft of innominate artery by thoracic incision (procedure)
        "34249006",  # Repair of kidney (procedure)
        "342960005",  # Ventriculoatrial anastomosis with valve (procedure)
        "34339006",  # Popliteal-tibial artery vascular bypass (procedure)
        "34414001",  # Incision of rectum (procedure)
        "34444003",  # Repair of ostium secundum defect (procedure)
        "344780002",  # Reimplantation of pancreatic tissue (procedure)
        "344990004",  # Balloon angioplasty of renal artery (procedure)
        "34518000",  # Anastomosis of artery with bypass graft (procedure)
        "34564005",  # Creation of shunt of ascending aorta to pulmonary artery (procedure)
        "345797001",  # Autotransplantation of pancreatic tissue (procedure)
        "346007006",  # Angioplasty of renal artery (procedure)
        "34645006",  # Excision of lesion of choroid plexus (procedure)
        "34646007",  # Pericardiectomy (procedure)
        "3479000",  # Removal of heart assist system with replacement (procedure)
        "34815006",  # Transplantation of adrenal gland tissue to cerebral ventricle (procedure)
        "34853001",  # Operation on intestine (procedure)
        "34876003",  # Repair of aneurysm with graft of iliac artery (procedure)
        "34905004",  # Transplant of ovary (procedure)
        "34947000",  # Operation to establish drainage of ventricle (procedure)
        "34969007",  # Operation on retroperitoneum (procedure)
        "3498003",  # Total excision of pituitary gland by transsphenoidal approach (procedure)
        "34984003",  # Incisional biopsy of rectum (procedure)
        "35030006",  # Operation on heart valve adjacent structure (procedure)
        "35052002",  # Thrombectomy and repair of venous graft (procedure)
        "35086006",  # Repair of foramen ovale with tissue graft (procedure)
        "35129004",  # Intrathoracic esophageal anastomosis with interposition of colon (procedure)
        "35139005",  # Removal of foreign body of kidney by incision (procedure)
        "3515001",  # Replacement of electronic heart device, pulse generator (procedure)
        "35157006",  # Incision of kidney (procedure)
        "35350000",  # Implantation of aortic valve prosthesis or synthetic device (procedure)
        "35404004",  # Creation of arterial bypass with synthetic graft (procedure)
        "3546002",  # Aortocoronary artery bypass graft with saphenous vein graft (procedure)
        "35535008",  # Transverse colostomy (procedure)
        "355610001",  # Anastomosis of esophagus to small bowel (procedure)
        "35603000",  # Subarachnoid-peritoneal spinal shunt (procedure)
        "3562008",  # Rodney Smith operation, radical subtotal pancreatectomy (procedure)
        "35634001",  # Craniectomy with excision of bone lesion of skull (procedure)
        "356386000",  # Division of artery of abdomen (procedure)
        "35642000",  # Repair of supracristal defect of heart (procedure)
        "35649009",  # Laryngeal ventriculectomy (procedure)
        "35653006",  # Endarterectomy of ascending aorta (procedure)
        "35753000",  # Closure of perineorectal fistula (procedure)
        "357575008",  # Repair of aortic valve without replacement (procedure)
        "357579002",  # Anastomosis of esophagus to intestinal segment (procedure)
        "35823001",  # Catheter ablation of lesion of heart (procedure)
        "35830007",  # Aneurysmectomy with graft replacement of upper limb artery (procedure)
        "358571002",  # Open heart valvuloplasty of aortic valve without replacement (procedure)
        "358575006",  # Gastroduodenostomy (procedure)
        "35887003",  # Excision of lesion of ovary (procedure)
        "35888008",  # Endarterectomy of head and neck artery (procedure)
        "35892001",  # Oversewing of perforated duodenal ulcer (procedure)
        "35915004",  # Bicuspidization of heart valve (procedure)
        "359528000",  # Repair of tricuspid valve without replacement (procedure)
        "359529008",  # Jaboulay operation, gastroduodenostomy (procedure)
        "359544009",  # Enterotomy of small bowel for biopsy (procedure)
        "359548007",  # Open biopsy of small intestine (procedure)
        "35955002",  # Radical vaginal hysterectomy (procedure)
        "359562000",  # Duodenotomy for biopsy (procedure)
        "359564004",  # Open biopsy of duodenum (procedure)
        "359567006",  # Proximal subtotal pancreatectomy with pancreaticoduodenectomy and pancreatic jejunostomy (procedure)
        "359571009",  # Right colectomy (procedure)
        "359572002",  # Enterectomy with anastomosis (procedure)
        "359574001",  # Closure of ulcer of duodenum (procedure)
        "359575000",  # Gastrectomy with jejunal transposition (procedure)
        "359579006",  # Partial gastrectomy with jejunal transposition (procedure)
        "359581008",  # Jejunoileostomy bypass shunt for obesity (procedure)
        "359584000",  # Fixation of duodenum to abdominal wall (procedure)
        "359585004",  # Ileojejunal bypass (procedure)
        "359586003",  # Laminectomy with spinal fusion (procedure)
        "359587007",  # Resection of cecum and terminal ileum (procedure)
        "359588002",  # Fixation of duodenum (procedure)
        "359591002",  # Suture of laceration of duodenum (procedure)
        "359594005",  # Duodenorrhaphy (procedure)
        "359596007",  # Fixation of jejunum to abdominal wall (procedure)
        "359597003",  # Single internal mammary-coronary artery bypass (procedure)
        "359599000",  # Jejunopexy (procedure)
        "359601003",  # Coronary artery bypass with autogenous graft of internal mammary artery, single graft (procedure)
        "359603000",  # Repair of jejunostomy (procedure)
        "359605007",  # Fixation of ileum to abdominal wall (procedure)
        "359607004",  # Ileopexy (procedure)
        "359615001",  # Partial lobectomy of lung (procedure)
        "359618004",  # Segmental lobectomy (procedure)
        "359623004",  # Total lobectomy of lung (procedure)
        "359672006",  # Median sternotomy (procedure)
        "359675008",  # Stacke operation (procedure)
        "359679002",  # Schwartze operation (procedure)
        "359684008",  # Simple mastoidectomy, complete (procedure)
        "359699008",  # Whitehead operation, radical glossectomy (procedure)
        "359701008",  # Excision of focal lesion of larynx (procedure)
        "359703006",  # Total glossectomy (procedure)
        "359708002",  # Radical amputation of penis (procedure)
        "35971000087108",  # Excision of left mastoid (procedure)
        "359713003",  # Total hypophysectomy (procedure)
        "359728003",  # Radical mastectomy including pectoral muscles and axillary lymph nodes (procedure)
        "359731002",  # Urban operation, extended radical mastectomy (procedure)
        "359734005",  # Halsted mastectomy (procedure)
        "359740003",  # Extended radical mastectomy (procedure)
        "359860002",  # Colotomy for biopsy (procedure)
        "359863000",  # Open biopsy of colon (procedure)
        "359882009",  # Thyroidectomy with laryngectomy (procedure)
        "359884005",  # Radical laryngopharyngectomy with synchronous thyroidectomy (procedure)
        "359905005",  # Repair of abdominal fistula (procedure)
        "359908007",  # Repair of paraesophageal diaphragmatic hernia (procedure)
        "359911008",  # Repair of parahiatal diaphragmatic hernia (procedure)
        "359936002",  # Hellstom operation for transplantation of aberrant renal artery (procedure)
        "359938001",  # Reimplantation of aberrant renal artery (procedure)
        "359945001",  # Incision of ascending aorta (procedure)
        "359946000",  # Repair of current obstetric laceration of rectum and sphincter ani (procedure)
        "359949007",  # Incision of aortic arch (procedure)
        "359955002",  # Incision of descending thoracic aorta (procedure)
        "359958000",  # Ventriculocisternal shunt with valve (procedure)
        "359970001",  # Resection of pericardium for removal of adhesions (procedure)
        "359971002",  # Ward-Mayo operation for vaginal hysterectomy (procedure)
        "359973004",  # Excision of pericardial adhesions (procedure)
        "359974005",  # Tuffier operation for vaginal hysterectomy (procedure)
        "359977003",  # Mayo operation for vaginal hysterectomy (procedure)
        "359980002",  # Vaginal panhysterectomy (procedure)
        "359983000",  # Heaney operation for vaginal hysterectomy (procedure)
        "36006002",  # Axillary-femoral-femoral artery bypass graft with vein (procedure)
        "36011000087108",  # Excision of right mastoid (procedure)
        "36033003",  # Resection of aortic valve for subvalvular stenosis (procedure)
        "36036006",  # Direct thrombectomy of axillary vein by arm incision (procedure)
        "360363002",  # Ureterocolostomy (procedure)
        "360789000",  # Local excision of renal parenchymal tumor (procedure)
        "360797007",  # Enucleation of renal tumor (procedure)
        "36092006",  # Nephrocolopexy (procedure)
        "361152000",  # Revision of stoma of large intestine (procedure)
        "361153005",  # Repair of stoma of large intestine (procedure)
        "361222003",  # Wertheim-Meigs abdominal hysterectomy (procedure)
        "361223008",  # Wertheim operation (procedure)
        "361247001",  # Excision of contact ulcer of vocal cord (procedure)
        "361248006",  # Excision of intubation granuloma of larynx (procedure)
        "361249003",  # Nephrectomy with excision of perirenal tissue (procedure)
        "36192008",  # Total abdominal colectomy with ileoproctostomy (procedure)
        "36268006",  # Closure of abdominouterine fistula (procedure)
        "36286009",  # Creation of aortorenal shunt (procedure)
        "363687006",  # Endoscopic procedure (procedure)
        "363742001",  # Pyloroplasty (procedure)
        "36384005",  # Excision of lesion of uterine cervix (procedure)
        "36432003",  # Anastomosis of urinary bladder to intestine (procedure)
        "36482008",  # Incision and exploration of intestine (procedure)
        "36496002",  # Ureterocaliceal anastomosis (procedure)
        "3654008",  # Lingulectomy of lung (procedure)
        "367365007",  # Anastomosis main bile duct to small intestine (procedure)
        "367405008",  # Excision of tissue of diaphragm (procedure)
        "367436000",  # Permanent ileostomy (procedure)
        "367453007",  # Partial resection of intestine with anastomosis (procedure)
        "367531007",  # Total adrenalectomy (procedure)
        "36788006",  # Cecoplication (procedure)
        "3690001",  # Shunt of left subclavian to descending aorta by Blalock-Park operation (procedure)
        "36910002",  # Craniectomy (procedure)
        "36928000",  # Operation on pituitary gland (procedure)
        "36935008",  # Closure of urethroperineovesical fistula (procedure)
        "36936009",  # Major thoracotomy with cardiac massage (procedure)
        "36984000",  # Closure of ileal fistula (procedure)
        "37011005",  # Direct thrombectomy of iliac vein by abdominal and leg incision (procedure)
        "37026007",  # Closed excisional biopsy of brain (procedure)
        "37034001",  # Ligation of arteriovenous fistula of coronary artery (procedure)
        "37119005",  # Double valve replacement with valvuloplasty of one valve (procedure)
        "37149007",  # Destruction of retroperitoneal cysts (procedure)
        "37153009",  # Implantation of heart valve with tissue graft (procedure)
        "37179000",  # Esophagoduodenal anastomosis with interposition of small bowel (procedure)
        "37184006",  # Enucleation of cyst of liver (procedure)
        "37199006",  # Thoracic endarterectomy (procedure)
        "373360004",  # Angioplasty of branches of abdominal aorta (procedure)
        "37359003",  # Aneurysmectomy with anastomosis of head and neck artery (procedure)
        "37364004",  # Plication of annulus of heart valve (procedure)
        "37380009",  # Destruction of cerebral lesion (procedure)
        "37402004",  # Incision and drainage of abscess of appendix (procedure)
        "37403009",  # Sphincterotomy of sphincter of Oddi by endoscopic retrograde cholangiopancreatography (procedure)
        "37412006",  # Excision of large intestine for interposition (procedure)
        "37441002",  # Removal of electronic heart device (procedure)
        "37499001",  # Thrombectomy of abdominal artery (procedure)
        "37576003",  # Endoscopic excision of lesion of esophagus (procedure)
        "37577007",  # Open heart valvuloplasty of pulmonary valve without replacement (procedure)
        "3758004",  # Thrombectomy with catheter of subclavian artery by neck incision (procedure)
        "37592008",  # Excision of lesion of aortic arch (procedure)
        "37593003",  # Total ileectomy (procedure)
        "37828009",  # Suture of small intestine with colostomy (procedure)
        "37851009",  # Radical perineal prostatectomy with bilateral pelvic lymphadenectomy (procedure)
        "37884009",  # Repair of artery of thorax and abdomen (procedure)
        "37897006",  # Removal of intra-aortic balloon with repair of aorta (procedure)
        "3796003",  # Excision of redundant mucosa from jejunostomy (procedure)
        "38014004",  # Disarticulation through wrist, reamputation (procedure)
        "38072008",  # Perirenal lysis of adhesions (procedure)
        "38075005",  # Clipping of inferior vena cava, complete (procedure)
        "38162008",  # Disarticulation of hip (procedure)
        "38179002",  # Excisional biopsy of pituitary gland by transfrontal approach (procedure)
        "38192004",  # Incision of ovary (procedure)
        "38208004",  # Removal of cardiac pacemaker and replacement with single-chamber cardiac device (procedure)
        "38210002",  # Ligation of venous connection between anomalous vein and superior vena cava (procedure)
        "38213000",  # Closure of intestinal cutaneous fistula (procedure)
        "38249004",  # Repair with vein graft of intra-abdominal artery (procedure)
        "38260008",  # Thromboendarterectomy of abdominal artery (procedure)
        "38303005",  # Incision of skin of breast (procedure)
        "38315009",  # Repair of pacemaker with replacement of pulse generator (procedure)
        "38432009",  # Complex hepatorrhaphy (procedure)
        "38447000",  # Clipping of inferior vena cava, partial (procedure)
        "384641003",  # Repair of mitral valve (procedure)
        "384642005",  # Mitral valvuloplasty (procedure)
        "384643000",  # Repair of tricuspid valve (procedure)
        "384648009",  # Repair of atresia of pulmonary valve with ventricular septal defect (procedure)
        "384677005",  # Reimplantation of coronary artery (procedure)
        "384678000",  # Transposition of coronary artery (procedure)
        "384680006",  # Patch enlargement of coronary artery (procedure)
        "384681005",  # Patch repair of coronary artery (procedure)
        "384683008",  # Replacement of cardiac pacemaker device with dual-chamber device (procedure)
        "384684002",  # Removal of cardiac pacemaker with replacement by single-chamber device, rate-responsive (procedure)
        "384723003",  # Radical mastectomy (procedure)
        "38497001",  # Enterolithotomy (procedure)
        "38502004",  # Intracranial aneurysmectomy with anastomosis (procedure)
        "38512006",  # Resection of cerebral meninges (procedure)
        "38610007",  # Pulmonary bypass operation (procedure)
        "3861005",  # Aneurysmectomy with anastomosis of lower limb artery (procedure)
        "386196007",  # Excision of diverticulum from large intestine (procedure)
        "386197003",  # Excision of diverticula of colon (procedure)
        "38629001",  # Operative procedure on the arteries of the thorax and abdomen (procedure)
        "386551007",  # Open excisional biopsy of esophagus (procedure)
        "386553005",  # Excision of esophageal lesion (procedure)
        "386561000",  # Fetal craniotomy (procedure)
        "386578009",  # Creation of split ileostomy (procedure)
        "386634006",  # Excision of bilateral testes (procedure)
        "386649003",  # Partial hip replacement by prosthesis (procedure)
        "38665006",  # Rastelli operation for repair of pulmonary artery atresia (procedure)
        "386670009",  # Total ostectomy (procedure)
        "386671008",  # Abdominal cavity operation (procedure)
        "386695005",  # Repair of stoma of esophagus (procedure)
        "386697002",  # Repair AND revision of stoma of esophagus (procedure)
        "386698007",  # Revision of esophagostomy (procedure)
        "386727004",  # Percutaneous transluminal cardiac valvotomy (procedure)
        "386749005",  # Repair of pulmonary valve (procedure)
        "386791007",  # Cystoscopy and transurethral resection of neoplasm of urinary bladder (procedure)
        "386792000",  # Transurethral excision of neoplasm of urinary bladder (procedure)
        "386793005",  # Cystoscopy and transurethral resection of lesion of urinary bladder (procedure)
        "386814008",  # Excision of aneurysm of sinus of Valsalva (procedure)
        "386815009",  # Repair of aneurysm of sinus of Valsalva (procedure)
        "386828002",  # Creation of defunctioning colostomy (procedure)
        "386829005",  # Temporary colostomy (procedure)
        "38737004",  # Transposition of tissue of rectum (procedure)
        "387606008",  # Construction of end colostomy (procedure)
        "387626007",  # Amputation of cervix (procedure)
        "387631009",  # Partial supraglottic laryngectomy (procedure)
        "38764003",  # Embolectomy of aorta (procedure)
        "387642000",  # Repair of mitral valve without replacement (procedure)
        "387643005",  # Partial hysterectomy (procedure)
        "387644004",  # Supracervical hysterectomy (procedure)
        "387646002",  # Uterine fundectomy (procedure)
        "387664009",  # Incision and exploration of kidney (procedure)
        "387683002",  # Transcarpal amputation (procedure)
        "387684008",  # Transmetacarpal amputation (procedure)
        "38772001",  # Open condylotomy of mandible (procedure)
        "387731002",  # Excision of lamina of vertebra (procedure)
        "387739000",  # Enucleation of ovarian cyst (procedure)
        "387740003",  # Removal of ovarian cyst (procedure)
        "387741004",  # Excision of cyst of ovary (procedure)
        "38782000",  # Insertion of Thomas shunt (procedure)
        "387824007",  # Incision and drainage of subdiaphragmatic abscess (procedure)
        "387826009",  # Incision and drainage of subphrenic space (procedure)
        "387860002",  # Excision rectal mucosal prolapse (procedure)
        "387861003",  # Excision of rectal mucosa (procedure)
        "38805005",  # Transplantation of pituitary gland (procedure)
        "388230004",  # Cystourethroscopy with resection of neck of urinary bladder (procedure)
        "38863001",  # Gas endarterectomy (procedure)
        "388997005",  # Axillary-femoral arterial bypass (procedure)
        "389007008",  # Superficial axillofemoral vascular bypass (procedure)
        "38971003",  # Esophagoileostomy, antesternal or antethoracic, with interposition of small bowel (procedure)
        "3907006",  # Incision and drainage of retroperitoneal abscess (procedure)
        "390781007",  # Insertion of auditory implant to brainstem (procedure)
        "39083009",  # Thoracic esophagostomy (procedure)
        "390849007",  # Excision of fimbrial cyst (procedure)
        "3915009",  # Excision of artery of thorax and abdomen (procedure)
        "39166002",  # Closure of duodenal fistula (procedure)
        "391935006",  # Pericardiolysis (procedure)
        "39202005",  # Coronary artery bypass with autogenous graft, four grafts (procedure)
        "392139002",  # Incision of aorta (procedure)
        "39214002",  # Closure of rectovesicovaginal fistula (procedure)
        "392237008",  # Total arthroplasty of knee, geomedic or polycentric (procedure)
        "392238003",  # Prosthetic arthroplasty of knee joint (procedure)
        "39267002",  # Pericardiorrhaphy (procedure)
        "39309004",  # Hemilaryngectomy (procedure)
        "39312001",  # Destructive procedure of aorta (procedure)
        "39316003",  # Operation on sigmoid (procedure)
        "39410008",  # Resuture of cardiac septum prosthesis (procedure)
        "395136002",  # Total mesorectal excision (procedure)
        "395137006",  # Ligation of inferior mesenteric artery (procedure)
        "395138001",  # Flush ligation of inferior mesenteric artery (procedure)
        "395139009",  # Ligation of inferior mesenteric artery with preservation of ascending left colic artery (procedure)
        "395140006",  # Primary intestinal anastomosis (procedure)
        "395151006",  # Multi-layer hand sewn intestinal anastomosis (procedure)
        "395152004",  # Single layer hand sewn intestinal anastomosis (procedure)
        "395199006",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve, sternocleidomastoid muscle and internal jugular vein (procedure)
        "395200009",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve and internal jugular vein (procedure)
        "395201008",  # Modified radical neck dissection of cervical lymph nodes, with preservation of accessory nerve (procedure)
        "395218007",  # Implantation of internal cardiac defibrillator (procedure)
        "39536003",  # Duodenotomy for foreign body removal (procedure)
        "3957003",  # Arteriectomy of thoracoabdominal aorta (procedure)
        "395702000",  # Patey total mastectomy (procedure)
        "395721008",  # Removal of electronic stimulator of brain with synchronous replacement (procedure)
        "395722001",  # Repair of atresia of pulmonary valve (procedure)
        "395723006",  # Operation for transposition of great vessels (procedure)
        "39620006",  # Witzel operation, temporary gastrostomy (procedure)
        "396222007",  # Strip craniectomy with opening of cranial suture (procedure)
        "39655008",  # Esophagoplasty by thoracic approach (procedure)
        "39667005",  # Amygdalohippocampotomy (procedure)
        "396692006",  # Radical orchiectomy (procedure)
        "396767001",  # Amputation of arm through wrist (procedure)
        "397028007",  # Removal of thrombus from vein (procedure)
        "397046001",  # Thrombectomy of artery (procedure)
        "397108006",  # Esophagostomy operation (procedure)
        "397117006",  # Amputation of lower limb (procedure)
        "397159007",  # Exteriorization of intestine (procedure)
        "397160002",  # Paul-Mikulicz operation (procedure)
        "397161003",  # Rankin operation, exteriorization of intestine (procedure)
        "397162005",  # Mikulicz operation, exteriorization of intestine, first stage (procedure)
        "397163000",  # Callander's amputation (procedure)
        "397164006",  # Batch-Spittler-McFaddin amputation (procedure)
        "397166008",  # Gritti-Stokes amputation (procedure)
        "397167004",  # Mazet amputation (procedure)
        "397168009",  # S.P. Rogers amputation (procedure)
        "39717005",  # Aortic valvotomy (procedure)
        "397204004",  # Cecostomy operations (procedure)
        "397218006",  # Amputation of ankle (procedure)
        "397219003",  # Guyon amputation (procedure)
        "397220009",  # Amputation by disarticulation of hindfoot (procedure)
        "397238007",  # Suture of liver (procedure)
        "397239004",  # Repair of liver laceration (procedure)
        "39724006",  # Anastomosis of internal mammary artery to coronary artery, double vessel (procedure)
        "397241003",  # Simple hepatorrhaphy (procedure)
        "397242005",  # Suture of liver laceration (procedure)
        "397243000",  # Suture of renal laceration (procedure)
        "397244006",  # Repair of renal laceration (procedure)
        "397913009",  # Percutaneous insertion of inferior vena cava filter (procedure)
        "3980006",  # Subtotal resection of esophagus (procedure)
        "398010007",  # Prosthetic arthroplasty of hip (procedure)
        "398135004",  # Exteriorization of cecum (procedure)
        "398182006",  # Insertion of inferior vena caval filter (procedure)
        "398235007",  # Construction of stoma of cecum (procedure)
        "398243002",  # Repair of rectocele (procedure)
        "398246005",  # Anastomosis of pancreatic duct to stomach (procedure)
        "398261006",  # Pancreas to stomach anastomosis (procedure)
        "39827007",  # Ventricular shunt to intracerebral site (procedure)
        "398291001",  # Replacement of pyelostomy tube (procedure)
        "39834009",  # Nephrostomy (procedure)
        "398740003",  # Colostomy (procedure)
        "398801000",  # Exteriorization of colon (procedure)
        "39887009",  # Thrombectomy with catheter of carotid artery by neck incision (procedure)
        "399052009",  # Aortic balloon pump operation (procedure)
        "399175004",  # Intra-aortic balloon catheter operation (procedure)
        "399733007",  # Excision of retroperitoneal lymph node (procedure)
        "39991003",  # Transection of infraorbital nerve (procedure)
        "400053006",  # Amputation of limb (procedure)
        "400136002",  # Amputation of upper limb (procedure)
        "40053007",  # Closure of biliary tract fistula (procedure)
        "40059006",  # Creation of aortofemoral shunt (procedure)
        "40081001",  # Excision of lesion of descending aorta (procedure)
        "400988008",  # High anterior resection of rectum (procedure)
        "401004",  # Distal subtotal pancreatectomy (procedure)
        "401107009",  # Excision of cyst of male hydatid of Morgagni (procedure)
        "40134000",  # Open biopsy of kidney (procedure)
        "40234006",  # Hemigastrectomy (procedure)
        "40250003",  # Interatrial transposition of venous return (procedure)
        "40267005",  # Intraaortic balloon pump removal (procedure)
        "4028006",  # Epiplopexy (procedure)
        "40298007",  # Arteriectomy with graft replacement by interposition (procedure)
        "404201002",  # Ligation of aneurysm of iliac artery (procedure)
        "404215000",  # Femoral-distal bypass with posterior tibial artery (procedure)
        "4044002",  # Permanent colostomy (procedure)
        "4045001",  # Drainage of cerebral ventricle by incision (procedure)
        "405299001",  # Femoro-posterior tibial graft for repair of aneurysm using vein (procedure)
        "405300009",  # Femoro-posterior tibial graft for repair of aneurysm using in-situ vein (procedure)
        "405305004",  # Abdominal extra-anatomic arterial bypass (procedure)
        "405309005",  # Axillobifemoral bypass graft with prosthesis (procedure)
        "405311001",  # Axillobiprofunda bypass graft (procedure)
        "405312008",  # Axillobiprofunda bypass using prosthesis (procedure)
        "405315005",  # Thrombectomy of common femoral vein (procedure)
        "405316006",  # Axilloprofunda bypass using prosthesis (procedure)
        "405317002",  # Thrombectomy of superficial femoral vein (procedure)
        "405318007",  # Removal of thrombus from profunda femoris vein (procedure)
        "405332009",  # Sapheno-femoral crossover bypass graft (procedure)
        "405333004",  # Femorofemoral crossover arterial bypass with vein (procedure)
        "405346009",  # Thrombolysis of iliac artery (procedure)
        "405347000",  # Thrombolysis of aorta (procedure)
        "405370008",  # Removal of thrombus from vascular prosthesis of upper limb (procedure)
        "405371007",  # Removal of thrombus from vascular prosthesis of lower limb (procedure)
        "405373005",  # Abdominal vascular prosthesis thrombectomy (procedure)
        "405380007",  # Thrombectomy of tibioperoneal artery (procedure)
        "405383009",  # Embolectomy of external iliac artery (procedure)
        "405395009",  # Endarterectomy and angioplasty of abdominal visceral artery (procedure)
        "405396005",  # Endarterectomy and angioplasty of iliac artery (procedure)
        "405397001",  # Endarterectomy and angioplasty of aorta (procedure)
        "405398006",  # Endarterectomy and angioplasty of aorta with vein (procedure)
        "405399003",  # Endarterectomy and angioplasty of aorta with prosthesis (procedure)
        "405403007",  # Endarterectomy and angioplasty of renal artery (procedure)
        "405404001",  # Endarterectomy and angioplasty of renal artery with vein (procedure)
        "405405000",  # Endarterectomy and angioplasty of renal artery with prosthesis (procedure)
        "405406004",  # Endarterectomy and angioplasty of iliac artery with vein (procedure)
        "405407008",  # Endarterectomy and angioplasty of internal carotid artery (procedure)
        "405408003",  # Endarterectomy and angioplasty of internal carotid artery with prosthesis (procedure)
        "405409006",  # Endarterectomy and angioplasty of internal carotid artery with vein (procedure)
        "405411002",  # Endarterectomy and angioplasty of external carotid artery (procedure)
        "405412009",  # Endarterectomy of internal carotid artery with eversion and end-to-end anastomosis (procedure)
        "405418008",  # Angioplasty of aorta with prosthesis (procedure)
        "40543003",  # Repair of diaphragmatic hiatal hernia by thoracoabdominal approach (procedure)
        "405441000",  # Transtarsal amputation (procedure)
        "405443002",  # Ray amputation of foot (procedure)
        "405455008",  # Endarterectomy and angioplasty of Iliac artery with prosthesis (procedure)
        "405457000",  # Femoro-posterior tibial artery bypass (procedure)
        "405458005",  # Femoro-anterior tibial artery bypass (procedure)
        "405459002",  # Femoro-tibio-peroneal artery bypass (procedure)
        "405460007",  # Femoro-tibio-peroneal artery bypass with vein (procedure)
        "405461006",  # Femoro-tibio-peroneal artery bypass with prosthesis (procedure)
        "405462004",  # Femoro-distal popliteal artery bypass (procedure)
        "405463009",  # Femoral artery bypass (procedure)
        "405464003",  # Femoro-distal popliteal artery bypass with vein (procedure)
        "405465002",  # Femoro-distal popliteal artery bypass with prosthesis (procedure)
        "405466001",  # Femoro-proximal popliteal artery bypass (procedure)
        "405467005",  # Femoro-proximal popliteal artery bypass with vein (procedure)
        "405468000",  # Femoro-proximal popliteal artery bypass with prosthesis (procedure)
        "405469008",  # Arterial bypass of lower limb artery (procedure)
        "405470009",  # Femoro-tibio-peroneal graft for repair of aneurysm using in-situ vein (procedure)
        "405471008",  # Femoro-tibio-peroneal graft for repair of aneurysm (procedure)
        "405472001",  # Femoro-tibio-peroneal graft for repair of aneurysm with prosthesis (procedure)
        "405473006",  # Ilio-profunda artery bypass (procedure)
        "405474000",  # Ilio-profunda artery bypass with prosthesis (procedure)
        "405475004",  # Ilio-femoral artery bypass with prosthesis (procedure)
        "405476003",  # Ilio-profunda artery bypass with vein (procedure)
        "405477007",  # Iliac artery bypass (procedure)
        "405478002",  # Ilio-popliteal artery bypass (procedure)
        "405479005",  # Ilio-popliteal artery bypass with prosthesis (procedure)
        "405480008",  # Ilio-popliteal artery bypass with vein (procedure)
        "405481007",  # Aorto-profunda arterial bypass (procedure)
        "405482000",  # Aorto-femoral arterial bypass (procedure)
        "405483005",  # Aorto-femoral arterial bypass with prosthesis (procedure)
        "405484004",  # Aorto-profunda arterial bypass with prosthesis (procedure)
        "405485003",  # Bypass of bifurcation of aorta by anastomosis of aorta to profunda femoris artery (procedure)
        "405486002",  # Aorto-profunda bifurcation bypass with prosthesis (procedure)
        "405487006",  # Aorto-iliac bypass with prosthesis (procedure)
        "405488001",  # Ilio-iliac artery bypass with prosthesis (procedure)
        "405489009",  # Bypass of bifurcation of aorta by anastomosis of aorta to iliac artery with prosthesis (procedure)
        "405504004",  # Femoro-tibio-peroneal artery bypass for repair of aneurysm using vein (procedure)
        "405505003",  # Femoro-distal popliteal graft for repair of aneurysm using vein (procedure)
        "405506002",  # Femoro-distal popliteal graft for repair of aneurysm using prosthesis (procedure)
        "405507006",  # Femoro-distal popliteal graft for repair of aneurysm (procedure)
        "405508001",  # Femoro-proximal popliteal artery bypass for repair of aneurysm with vein (procedure)
        "405509009",  # Ilio-distal popliteal graft for repair of aneurysm using vein (procedure)
        "405510004",  # Ilio-distal popliteal graft for repair of aneurysm with prosthesis (procedure)
        "405511000",  # Ilio-distal popliteal graft for repair of aneurysm (procedure)
        "405512007",  # Ilio-profunda artery bypass for repair of aneurysm with vein (procedure)
        "405513002",  # Ilio-profunda artery bypass for repair of aneurysm with prosthesis (procedure)
        "405514008",  # Femoro-distal popliteal graft for repair of aneurysm with in-situ vein (procedure)
        "405515009",  # Femoro-proximal popliteal artery bypass for repair of aneurysm with in-situ vein (procedure)
        "405517001",  # Ilio-proximal popliteal graft for repair of aneurysm using vein (procedure)
        "405518006",  # Ilio-proximal popliteal graft for repair of aneurysm with prosthesis (procedure)
        "405519003",  # Ilio-proximal popliteal graft for repair of aneurysm (procedure)
        "405520009",  # Aorto-profunda graft for repair of aneurysm (procedure)
        "405521008",  # Aorto-profunda bifurcation graft for repair of aneurysm (procedure)
        "405522001",  # Aorto-femoral bifurcation graft for repair of aneurysm (procedure)
        "405523006",  # Ilio-profunda graft for repair of aneurysm (procedure)
        "405524000",  # Aorto-Iliac graft for repair of aneurysm (procedure)
        "405525004",  # Repair of aneurysm of abdominal aorta (procedure)
        "405526003",  # Femoro-peroneal artery bypass (procedure)
        "405527007",  # Femoro-peroneal artery bypass with in-situ vein (procedure)
        "405528002",  # Ilio-femoral bypass for repair of aneurysm with prosthesis (procedure)
        "405529005",  # Aorto-iliac bifurcation graft for repair of aneurysm (procedure)
        "405530000",  # Femoro-peroneal artery bypass with prosthesis (procedure)
        "405593001",  # Angioplasty of external iliac artery with prosthesis (procedure)
        "405596009",  # Thrombolysis of iliac artery via femoral artery catheter (procedure)
        "405597000",  # Ilio-distal popliteal graft (procedure)
        "405598005",  # Aortocoronary artery bypass graft with two vein grafts (procedure)
        "405599002",  # Aortocoronary artery bypass graft with three vein grafts (procedure)
        "40576004",  # Removal of pleurothecal shunt (procedure)
        "405768001",  # Autotransplantation of heart (procedure)
        "40580009",  # Perirenal operation (procedure)
        "406505007",  # Modified radical mastectomy (procedure)
        "40658002",  # Amputation of forearm through radius AND ulna (procedure)
        "40667002",  # Fixation of small intestine (procedure)
        "4068005",  # Removal of ventricular reservoir with synchronous replacement (procedure)
        "40801000",  # Exploration of pelvic cavity by laparotomy (procedure)
        "40809003",  # Closure of ureterosigmoidal fistula (procedure)
        "40858004",  # Incision of cranial nerve (procedure)
        "408621000119100",  # Removal of tunneled central venous catheter with subcutaneous port using fluoroscopic guidance with contrast (procedure)
        "4090005",  # Replacement of transvenous atrial and ventricular pacemaker electrode leads (procedure)
        "40902008",  # Segmental lobectomy with bronchoplasty (procedure)
        "409710009",  # Cardiorrhaphy (procedure)
        "40995000",  # Pancreaticocystojejunostomy (procedure)
        "41004001",  # Incision and exploration of pituitary gland (procedure)
        "41058005",  # Salpingectomy for tubal ectopic pregnancy by abdominal approach (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "41141002",  # Operative procedure on large intestine (procedure)
        "41161008",  # Creation of aortoiliac shunt (procedure)
        "4119008",  # Intracranial phlebectomy with anastomosis (procedure)
        "41200003",  # Excision of rectal procidentia with anastomosis by perineal approach (procedure)
        "41223000",  # Pharyngolaryngectomy with radical neck dissection and reconstruction (procedure)
        "412711000",  # Repair of aneurysm of hepatic artery (procedure)
        "41276005",  # Creation of endorectal ileal S-pouch with anastomosis to anus (procedure)
        "413144006",  # Abdominal hysterectomy with conservation of ovaries (procedure)
        "413145007",  # Vaginal hysterectomy with conservation of ovaries (procedure)
        "413181005",  # Laparoscopic repair of hiatus hernia (procedure)
        "413257009",  # Pancreatic sphincterotomy by endoscopic retrograde cholangiopancreatography (procedure)
        "41330009",  # Laparoscopic-assisted left colectomy (procedure)
        "41339005",  # Coronary angioplasty (procedure)
        "41393008",  # Excision of cyst of fallopian tube (procedure)
        "414088005",  # Emergency coronary artery bypass graft (procedure)
        "414158004",  # Excisional biopsy of pancreas (procedure)
        "41416003",  # Radical retropubic prostatectomy with bilateral pelvic lymphadenectomy (procedure)
        "41424008",  # Detorsion of kidney (procedure)
        "41436002",  # Plication of intestine (procedure)
        "414512008",  # Insertion of renal artery stent (procedure)
        "414575003",  # Laparoscopic total abdominal hysterectomy and bilateral salpingo-oophorectomy (procedure)
        "415072000",  # Percutaneous thromboendarterectomy (procedure)
        "41536004",  # Revision of anastomosis of small intestine (procedure)
        "41566006",  # Excision of cervical stump by vaginal approach (procedure)
        "41616008",  # Removal of embolus from thoracic artery (procedure)
        "416708003",  # Lateral temporal bone resection (procedure)
        "417972004",  # Angioplasty of hepatic artery using fluoroscopic guidance with contrast (procedure)
        "418066004",  # Angioplasty of superior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "418132005",  # Angioplasty of abdominal aorta and femoral artery using fluoroscopic guidance with contrast (procedure)
        "41817002",  # Closure of patent foramen ovale (procedure)
        "418269003",  # Angioplasty of iliac artery using fluoroscopic guidance with contrast (procedure)
        "418461002",  # Angioplasty of coronary artery using fluoroscopic guidance with contrast (procedure)
        "418485005",  # Fluoroscopic venography of superior vena cava with contrast and insertion of stent (procedure)
        "418486006",  # Angioplasty of mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "418491007",  # Fluoroscopic angiography with contrast and thrombolysis of intracranial artery (procedure)
        "418790007",  # Angioplasty of inferior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "418824004",  # Off-pump coronary artery bypass (procedure)
        "418982001",  # Fluoroscopic angiography of coronary artery with contrast and insertion of stent (procedure)
        "419104005",  # Angioplasty of cerebral artery using fluoroscopic guidance with contrast (procedure)
        "419132001",  # Minimally invasive direct coronary artery bypass (procedure)
        "419224002",  # Angioplasty of renal transplant using fluoroscopic guidance with contrast (procedure)
        "41925004",  # Ventriculocisternostomy, third ventricle (procedure)
        "419524003",  # Angioplasty of aorta using fluoroscopic guidance with contrast (procedure)
        "419526001",  # Fluoroscopic angiography of aorta with contrast and insertion of stent (procedure)
        "419539002",  # Fluoroscopic angiography of iliac artery with contrast and insertion of stent (procedure)
        "419740003",  # Fluoroscopic angiography of abdominal aorta with contrast and insertion of stent (procedure)
        "419801000",  # Angioplasty of abdominal artery using fluoroscopic guidance with contrast (procedure)
        "419827000",  # Angioplasty of renal artery using fluoroscopic guidance with contrast (procedure)
        "41995008",  # Embolectomy with catheter of mesenteric artery by abdominal incision (procedure)
        "419987004",  # Angioplasty of celiac artery using fluoroscopic guidance with contrast (procedure)
        "420085001",  # Angioplasty of splenic artery using fluoroscopic guidance with contrast (procedure)
        "42010004",  # Uterine myomectomy (procedure)
        "420128004",  # Fluoroscopic angiography of thoracic aorta with contrast and insertion of stent (procedure)
        "42022001",  # Repair of ruptured aneurysm with graft of renal artery (procedure)
        "42050001",  # Cricoidectomy (procedure)
        "42150003",  # Suture repair of great vessels with cardiopulmonary bypass (procedure)
        "42191001",  # Excisional biopsy of brain (procedure)
        "42227002",  # Closure of rectovaginal fistula by transanal approach (procedure)
        "42246001",  # Hepatojejunostomy (procedure)
        "42262007",  # Prosthetic total arthroplasty of shoulder joint (procedure)
        "42268006",  # Ileosigmoidostomy (procedure)
        "422940006",  # Removal of pulmonary artery catheter (procedure)
        "42301000",  # Temporary tube ileostomy (procedure)
        "42332004",  # Fistulization of appendix (procedure)
        "42349006",  # Excision of cyst of broad ligament (procedure)
        "424606007",  # Removal of peripherally inserted central catheter (procedure)
        "42471000",  # Thrombectomy with catheter of popliteal-tibio-peroneal artery by leg incision (procedure)
        "42481001",  # Excision of coronary artery (procedure)
        "42486006",  # Implantation of mitral valve with tissue graft (procedure)
        "4252008",  # Aneurysmectomy with graft replacement by interposition (procedure)
        "42527001",  # Endoscopic excision of lesion of stomach (procedure)
        "42532000",  # Infundibulectomy of ventricle in total repair of tetralogy of Fallot (procedure)
        "425435003",  # Implantation of cardioverter defibrillator with one electrode lead (procedure)
        "425437006",  # Ligation of pulmonary artery (procedure)
        "425486008",  # Transpetrous excision of lesion of jugular foramen (procedure)
        "425494001",  # Fiberoptic endoscopic submucosal resection of lesion of stomach (procedure)
        "425577000",  # Extracorporeal shockwave lithotripsy of calculus of pancreas (procedure)
        "425603008",  # Open operation on both pulmonary arteries (procedure)
        "425616008",  # Transplantation of ileum (procedure)
        "425632006",  # Fiberoptic endoscopic extirpation of lesion of stomach (procedure)
        "425654008",  # Repair of univentricular heart (procedure)
        "425712000",  # Percutaneous transluminal angioplasty of blood vessel of liver (procedure)
        "425751005",  # Endoscopic cryoablation of lesion of kidney (procedure)
        "425779004",  # Endoscopic transurethral resection of ureteric orifice (procedure)
        "425851003",  # Fiberoptic endoscopic submucosal resection of lesion of colon (procedure)
        "425895009",  # Repair of coronary sinus abnormality (procedure)
        "425900008",  # Removal of filter in vena cava (procedure)
        "425904004",  # Closure of connection of stomach to transposed jejunum (procedure)
        "42593007",  # Thrombectomy of aorta (procedure)
        "425948009",  # Percutaneous angioplasty of pulmonary artery (procedure)
        "425979008",  # Percutaneous transluminal balloon angioplasty of pulmonary artery (procedure)
        "425989007",  # Prosthetic hybrid total arthroplasty of shoulder joint using cemented glenoid component (procedure)
        "426039009",  # Percutaneous transluminal insertion of stent into renal artery (procedure)
        "426059008",  # Percutaneous transluminal thrombolysis of blood vessel of liver (procedure)
        "426066009",  # Endoscopic submucosal resection of lesion of lower bowel using fiberoptic sigmoidoscope (procedure)
        "426067000",  # Open removal of cardiac thrombus (procedure)
        "426080002",  # Repair of left ventricular aneurysm (procedure)
        "426104007",  # Repair of rupture of coronary artery (procedure)
        "426105008",  # Fiberoptic endoscopic submucosal resection of lesion of esophagus (procedure)
        "426143006",  # Percutaneous chemical ablation of lesion of pancreas (procedure)
        "426185005",  # Open removal of cardiac vegetations (procedure)
        "42624001",  # Correction of malrotation by lysis of duodenal bands (procedure)
        "426281009",  # Excision of lesion of jugular foramen (procedure)
        "426305004",  # Removal of atrial catheter (procedure)
        "426321009",  # Primary decompression of posterior fossa and upper cervical spinal cord and instrumentation (procedure)
        "426340003",  # Creation of graft fistula for dialysis (procedure)
        "426356008",  # Orthotopic transplantation of whole liver (procedure)
        "426410001",  # Removal of ventricular assist device (procedure)
        "426436008",  # Repair of pulmonary arterial sling (procedure)
        "42645008",  # Closure of gastroesophageal fistula (procedure)
        "426456009",  # Medial maxillectomy (procedure)
        "426463009",  # Allotransplantation of ileum (procedure)
        "426506002",  # Creation of valved conduit between left ventricle of heart and pulmonary artery (procedure)
        "426541003",  # Atrial septectomy (procedure)
        "426568003",  # Open implantation of cardiac ventricular assist device (procedure)
        "426617006",  # Fiberoptic endoscopic insertion of expanding metal stent into colon (procedure)
        "426618001",  # Prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "426697007",  # Percutaneous radiofrequency ablation of lesion of kidney (procedure)
        "426699005",  # Endoscopic submucosal resection of lesion of sigmoid colon using rigid sigmoidoscope (procedure)
        "426713000",  # Percutaneous chemical ablation of lesion of liver (procedure)
        "426736009",  # Patch angioplasty of renal artery (procedure)
        "426738005",  # Duodenal switch (procedure)
        "426791002",  # Repair of anomalous caval vein connection (procedure)
        "426805006",  # Creation of lumbar subcutaneous shunt (procedure)
        "426812002",  # Radial optic neurotomy (procedure)
        "426869003",  # Repair of interrupted aortic arch (procedure)
        "426876008",  # Repair of right ventricular aneurysm (procedure)
        "426900002",  # Pulmonary thromboendarterectomy (procedure)
        "426902005",  # Combined atrial switch and arterial switch operation (procedure)
        "426904006",  # Prosthetic hybrid total replacement of hip joint using cemented acetabular component (procedure)
        "426911005",  # Repair of double chambered right ventricle (procedure)
        "426937008",  # Open thrombectomy of renal vein (procedure)
        "426978005",  # Endoscopic de-roofing of multiple cysts of kidney (procedure)
        "426989003",  # Excision of multiple lesions of liver (procedure)
        "42699003",  # Incision of brain (procedure)
        "427012005",  # Percutaneous transluminal insertion of stent into cerebral artery (procedure)
        "427037000",  # Complete callosotomy (procedure)
        "42704006",  # Endoscopic destruction of lesion of pancreatic duct (procedure)
        "427074001",  # Laparoscopic sleeve gastrectomy (procedure)
        "427092009",  # Excision of lesion of vena cava (procedure)
        "427107006",  # Excision of accessory uterus (procedure)
        "427109009",  # Repair of arteriovenous malformation of coronary artery (procedure)
        "427138007",  # Repair of anomalous pulmonary artery origin from ascending aorta (procedure)
        "427173009",  # Endoscopic endoluminal balloon rupture of stenosis of pelviureteric junction of kidney (procedure)
        "427175002",  # Removal of electrocardiography loop recorder (procedure)
        "427188004",  # Repair of double outlet right ventricle with intraventricular tunnel (procedure)
        "427193001",  # Closure of prosthetic shunt to pulmonary artery from subclavian artery (procedure)
        "427198005",  # Open removal of ventricular assist device (procedure)
        "427231006",  # Percutaneous transluminal insertion of stent into aorta (procedure)
        "427262005",  # Incision of tissue in atria (procedure)
        "427265007",  # Closure of aorto-left ventricular tunnel (procedure)
        "427280001",  # Repair of traumatic injury of heart (procedure)
        "427318004",  # Repair of double outlet ventricle (procedure)
        "427421001",  # Fiberoptic endoscopic endoluminal plication of gastro-esophageal junction (procedure)
        "427486009",  # Bypass of carotid artery by anastomosis of superficial temporal artery to middle cerebral artery (procedure)
        "427503001",  # Percutaneous transluminal insertion of stent into pulmonary artery (procedure)
        "427506009",  # Excision of ileoanal pouch (procedure)
        "427508005",  # Implantation of cardioverter defibrillator with two electrode leads (procedure)
        "427560001",  # Prosthetic hybrid replacement of shoulder joint using cemented humeral component (procedure)
        "427585005",  # Biventricular repair of hypoplastic left heart syndrome (procedure)
        "427599009",  # Excision of lesion of tissue of brain stem (procedure)
        "42767002",  # Repair of heart assist system (procedure)
        "427700000",  # Operation on blood vessel of liver (procedure)
        "427731007",  # Replacement of aortic root using mechanical prosthesis (procedure)
        "427752004",  # Excision of lamina of lumbar vertebra for decompression of spinal cord (procedure)
        "427862004",  # Percutaneous transluminal balloon angioplasty with insertion of stent into renal artery (procedure)
        "427886002",  # Total cavopulmonary connection with lateral atrial tunnel (procedure)
        "427929006",  # Thermal ablation of multiple lesions of liver (procedure)
        "427987005",  # Subepithelial cordectomy (procedure)
        "427992007",  # Removal of thrombus of arteriovenous fistula (procedure)
        "427997001",  # Subtotal excision of colon and rectum and creation of colonic pouch and anastomosis of colon to anus (procedure)
        "428050002",  # Percutaneous transluminal insertion of stent into ductus arteriosus (procedure)
        "428057004",  # Insertion of stent into aorta (procedure)
        "428062003",  # Percutaneous transluminal balloon angioplasty with insertion of stent into aorta (procedure)
        "428115007",  # Thermal ablation of single lesion of liver (procedure)
        "428179006",  # Replacement of aortic root (procedure)
        "428182001",  # Replacement of aortic root using homograft (procedure)
        "428280004",  # Insertion of stent into ductus arteriosus (procedure)
        "428290007",  # Percutaneous transluminal ablation of wall of atrium (procedure)
        "42837001",  # Excision of aortic arch (procedure)
        "428388009",  # Percutaneous transluminal insertion of peripherally inserted central catheter (procedure)
        "428391009",  # Percutaneous transluminal ablation of congenital malformation of heart (procedure)
        "428411007",  # Aortopulmonary reconstruction with systemic-to-pulmonary arterial shunt (procedure)
        "428423001",  # Insertion of stent into cerebral artery (procedure)
        "42843004",  # Laparoscopic adrenalectomy (procedure)
        "428501002",  # Replacement of aortic root using pulmonary valve autograft with right ventricle to pulmonary artery valved conduit (procedure)
        "428515000",  # Correction of congenital deformity of heart (procedure)
        "428551007",  # Transmuscular cordectomy (procedure)
        "428553005",  # Excision of muscle of left ventricular outflow tract (procedure)
        "428568006",  # Division of aberrant aortic arch (procedure)
        "428581004",  # Percutaneous transluminal ablation of accessory pathway (procedure)
        "428585008",  # Percutaneous transluminal insertion of stent graft into hepatic artery (procedure)
        "428602002",  # Unifocalization of pulmonary artery (procedure)
        "428621005",  # Transluminal occlusion of left internal mammary artery side branch (procedure)
        "428625001",  # Replacement of cardioverter defibrillator (procedure)
        "428652006",  # Transcervical resection of leiomyoma of uterus (procedure)
        "428663009",  # Ablation of atrioventricular node (procedure)
        "428679005",  # Partial excision of left ventricle of heart (procedure)
        "428759006",  # Closure of multiple ventricular septal defects (procedure)
        "428809009",  # Endovascular repair of aorto-uniiliac aneurysm using endovascular stent graft (procedure)
        "428818006",  # Closure of defect of interventricular septum using intraoperative transluminal prosthesis (procedure)
        "428840002",  # Percutaneous endoscopic sigmoidostomy (procedure)
        "428893001",  # Replacement of valved cardiac conduit (procedure)
        "428918005",  # Total cordectomy (procedure)
        "428953003",  # Percutaneous transluminal thrombectomy of blood vessel of liver (procedure)
        "428998009",  # Plastic repair of aorta with tube graft (procedure)
        "42904002",  # Repair of rectal laceration (procedure)
        "42905001",  # Suture repair of aorta with cardiopulmonary bypass (procedure)
        "429064006",  # Implantation of cardiac resynchronization therapy implantable pacemaker (procedure)
        "42909007",  # Radical excision of bursa of synovia of extensors of wrist (procedure)
        "429113008",  # Percutaneous transluminal closure of patent oval foramen using prosthesis (procedure)
        "429122009",  # Percutaneous transluminal balloon angioplasty with insertion of stent into pulmonary artery (procedure)
        "429167002",  # Endovascular repair of infrarenal abdominal aortic aneurysm using endovascular stent graft (procedure)
        "429199008",  # Subligamental cordectomy (procedure)
        "429211003",  # Maze procedure for atrial fibrillation (procedure)
        "429248007",  # Percutaneous transluminal replacement of pulmonary valve (procedure)
        "429269009",  # Aortoventriculoplasty with pulmonary valve autograft (procedure)
        "429270005",  # Takedown of anastomosis from aorta to pulmonary artery (procedure)
        "429314008",  # Partial callosotomy (procedure)
        "429332008",  # Transplantation of single lobe of lung (procedure)
        "429359000",  # Revision of plastic repair of truncal valve (procedure)
        "429377005",  # Excision of lamina of lumbar vertebra (procedure)
        "429394007",  # Division of anomalous aortic arch (procedure)
        "429401008",  # Image guided balloon dilation of colorectal stricture (procedure)
        "429510003",  # Multiple subpial transections of cerebral cortex (procedure)
        "429542009",  # Implantation of intravenous cardiac resynchronization therapy implantable pacemaker (procedure)
        "429575001",  # Construction of left ventricle to aorta tunnel with right ventricle to pulmonary trunk direct anastomosis (procedure)
        "429582002",  # Open enlargement of defect of interventricular septum (procedure)
        "429595007",  # Resection of neoplasm of heart (procedure)
        "429609002",  # Lung volume reduction surgery (procedure)
        "429616001",  # Aortopulmonary reconstruction with right ventricle to pulmonary arterial valveless conduit (procedure)
        "429620002",  # Construction of left ventricle to aorta tunnel with right ventricle to pulmonary artery valved conduit (procedure)
        "429679003",  # Endovascular repair of thoracic aortic aneurysm using endovascular stent graft (procedure)
        "429706004",  # Endovascular repair of suprarenal aortic aneurysm using endovascular stent graft (procedure)
        "429761006",  # Percutaneous transluminal closure of defect of interatrial septum using prosthesis (procedure)
        "429788001",  # Transjugular intrahepatic thrombectomy of portal vein (procedure)
        "429794009",  # Fluoroscopic angiography with contrast and thrombolysis of aorta and femoral artery (procedure)
        "429805005",  # Fluoroscopic angiography of abdominal aorta with contrast and insertion of stent graft (procedure)
        "430286000",  # Percutaneous fenestration of aorta using fluoroscopic guidance with contrast (procedure)
        "430385003",  # Incision and drainage of abscess of chest (procedure)
        "4304000",  # Excision of cusp of tricuspid valve (procedure)
        "430457001",  # Incisional biopsy of heart (procedure)
        "43047008",  # Lysis of adhesions of liver (procedure)
        "43053008",  # Creation of intrathoracic vascular shunt or bypass (procedure)
        "430614000",  # Fluoroscopic venography of hepatic vein with contrast and insertion of stent (procedure)
        "430688009",  # Prosthetic arthroplasty of left hip (procedure)
        "430694001",  # Prosthetic arthroplasty of right hip (procedure)
        "43075005",  # Partial resection of colon (procedure)
        "430798009",  # Transanal resection of rectum and anastomosis using staples (procedure)
        "43082009",  # Closure of gastrojejunal fistula (procedure)
        "431185003",  # Replacement of central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431211009",  # Percutaneous transluminal angioplasty of internal iliac artery using fluoroscopic guidance with contrast (procedure)
        "431220000",  # Cardiac septostomy using fluoroscopic guidance (procedure)
        "431229004",  # Percutaneous transluminal cutting balloon angioplasty of thoracic aorta using fluoroscopic guidance with contrast (procedure)
        "431243009",  # Removal of central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431253005",  # Percutaneous transluminal angioplasty of renal vein using fluoroscopic guidance with contrast (procedure)
        "431254004",  # Percutaneous mechanical thrombectomy of mesenteric vein using fluoroscopic guidance with contrast (procedure)
        "431255003",  # Percutaneous mechanical thrombectomy of artery of upper limb using fluoroscopic guidance with contrast (procedure)
        "431302007",  # Percutaneous thrombolysis of renal artery using fluoroscopic guidance (procedure)
        "431303002",  # Percutaneous mechanical thrombectomy of vein of upper limb using fluoroscopic guidance with contrast (procedure)
        "431305009",  # Percutaneous insertion of stent graft into bypass of external iliac artery to internal iliac artery using fluoroscopic guidance with contrast (procedure)
        "431316002",  # Laparoscopic subtotal hysterectomy (procedure)
        "431339008",  # Mechanical prosthetic mitral valve replacement (procedure)
        "431400003",  # Jejunostomy using fluoroscopic guidance (procedure)
        "431410007",  # Nephrostomy using fluoroscopic guidance (procedure)
        "431426008",  # Percutaneous mechanical thrombectomy of vein of lower limb using fluoroscopic guidance with contrast (procedure)
        "431427004",  # Percutaneous mechanical thrombectomy of vascular graft of lower limb using fluoroscopic guidance with contrast (procedure)
        "431433008",  # Percutaneous transluminal angioplasty of inferior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "431434002",  # Percutaneous transluminal angioplasty of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "431441000124105",  # Ventricular shunt to subgaleal space (procedure)
        "431457008",  # Fluoroscopic angiography of celiac artery using contrast with insertion of drug eluting stent (procedure)
        "431470004",  # Cecostomy using fluoroscopic guidance (procedure)
        "431477001",  # Intraoperative fluoroscopic angiography of abdominal aorta and iliac artery with contrast with insertion of unilateral aorto-iliac stent graft (procedure)
        "431483003",  # Percutaneous transluminal cutting balloon angioplasty of hepatic artery using fluoroscopic guidance with contrast (procedure)
        "431484009",  # Percutaneous transluminal angioplasty of renal artery using fluoroscopic guidance with contrast (procedure)
        "431486006",  # Percutaneous aspiration thrombectomy of graft of lower limb using fluoroscopic guidance with contrast (procedure)
        "431487002",  # Percutaneous aspiration thrombectomy of graft of upper limb using fluoroscopic guidance with contrast (procedure)
        "43151008",  # Splenorenal bypass graft with vein (procedure)
        "431514000",  # Cardiac septostomy using ultrasound guidance (procedure)
        "431516003",  # Fluoroscopic venography of superior vena cava using contrast with insertion of stent graft (procedure)
        "431525009",  # Percutaneous transluminal angioplasty of abdominal aorta using fluoroscopic guidance with contrast (procedure)
        "431528006",  # Percutaneous thrombolysis of superior vena cava using fluoroscopic guidance (procedure)
        "431530008",  # Percutaneous brachytherapy of hepatic artery using fluoroscopic guidance with contrast (procedure)
        "431536002",  # Percutaneous transluminal angioplasty of external iliac artery using fluoroscopic guidance with contrast (procedure)
        "431538001",  # Percutaneous aspiration thrombectomy of artery of upper limb using fluoroscopy guidance with contrast (procedure)
        "431592002",  # Insertion of permanent superior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "431595000",  # Percutaneous transluminal angioplasty of superior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "431596004",  # Percutaneous mechanical thrombectomy of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "431600009",  # Fluoroscopic angiography of internal iliac artery with contrast with insertion of stent (procedure)
        "431602001",  # Fluoroscopic venography of hepatic vein with contrast and insertion of stent graft (procedure)
        "431650002",  # Percutaneous replacement of temporary filter of superior vena cava using fluoroscopic guidance with contrast (procedure)
        "431655007",  # Fibrin sheath stripping from central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431661005",  # Percutaneous mechanical thrombectomy of superior vena cava using fluoroscopic guidance with contrast (procedure)
        "431663008",  # Percutaneous mechanical thrombectomy of pulmonary vein using fluoroscopic guidance with contrast (procedure)
        "431667009",  # Intracoronary brachytherapy using fluoroscopic guidance with contrast (procedure)
        "431670008",  # Replacement of jejunostomy tube using fluoroscopic guidance (procedure)
        "431678001",  # Removal of tunneled central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431702005",  # Removal of cardiac pacemaker lead using fluoroscopic guidance (procedure)
        "431712003",  # Fluoroscopic angiography of common iliac artery using contrast with insertion of drug eluting stent (procedure)
        "431713008",  # Fluoroscopic angiography of renal artery with contrast and insertion of drug eluting stent (procedure)
        "431715001",  # Fluoroscopic angiography of external iliac artery using contrast with insertion of stent (procedure)
        "431716000",  # Fluoroscopic angiography of splenic artery using contrast with insertion of stent (procedure)
        "431733007",  # Fluoroscopic angiography of inferior mesenteric artery using contrast with insertion of drug eluting stent (procedure)
        "431751008",  # Percutaneous transluminal angioplasty of thoracic aorta using fluoroscopic guidance with contrast (procedure)
        "431753006",  # Percutaneous mechanical thrombectomy of mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "431754000",  # Fluoroscopic angiography of aorta using contrast with insertion of hybrid stent graft (procedure)
        "431755004",  # Fluoroscopic venography of inferior vena cava using contrast with insertion of stent graft (procedure)
        "431764009",  # Guide wire recanalization of central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431783002",  # Replacement of tunneled central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431795002",  # Percutaneous mechanical thrombectomy of hepatic vein using fluoroscopic guidance with contrast (procedure)
        "431800007",  # Fluoroscopic angiography of abdominal aorta using contrast with insertion of fenestrated aortounilateral iliac stent graft (procedure)
        "431802004",  # Percutaneous aspiration of thrombus using fluoroscopic guidance (procedure)
        "431846007",  # Replacement of permanent cardiac pacemaker using fluoroscopic guidance (procedure)
        "431863006",  # Fluoroscopic angiography of celiac artery with contrast and insertion of stent (procedure)
        "431872003",  # Fluoroscopic angiography of inferior mesenteric artery using contrast with insertion of stent (procedure)
        "431879007",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of branched unilateral aortoiliac stent graft (procedure)
        "431893000",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of stent graft (procedure)
        "431930003",  # Percutaneous transluminal cutting balloon angioplasty of brachiocephalic artery using fluoroscopic guidance with contrast (procedure)
        "431931004",  # Percutaneous transluminal angioplasty of splenic artery using fluoroscopic guidance with contrast (procedure)
        "431933001",  # Percutaneous thrombolysis of central venous catheter using fluoroscopic guidance with contrast (procedure)
        "431939002",  # Percutaneous aspiration thrombectomy of artery of lower limb using fluoroscopy guidance with contrast (procedure)
        "43196001",  # Ileocolotomy (procedure)
        "432030003",  # Fluoroscopic angiography of abdominal aorta with contrast with insertion of drug eluting stent (procedure)
        "432035008",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast and insertion of stent graft (procedure)
        "432040000",  # Percutaneous transluminal cutting balloon angioplasty of hepatic vein using fluoroscopic guidance with contrast (procedure)
        "432046006",  # Percutaneous aspiration thrombectomy of mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "432052007",  # Percutaneous replacement of temporary filter of inferior vena cava using fluoroscopic guidance with contrast (procedure)
        "432067006",  # Fluoroscopic angiography of pulmonary artery using contrast with insertion of stent (procedure)
        "432069009",  # Fluoroscopic venography of renal vein using contrast with insertion of stent (procedure)
        "432074001",  # Removal of pulmonary artery catheter using fluoroscopic guidance with contrast (procedure)
        "432079006",  # Removal of inferior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "432114008",  # Closure of atrial septal defect using fluoroscopic guidance (procedure)
        "432117001",  # Insertion of temporary superior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "432226009",  # Ablation of lesion of liver using computed tomography guidance (procedure)
        "432258004",  # Excision of neoplasm of central nervous system (procedure)
        "432274009",  # Dilation of colon and rectum using fluoroscopic guidance (procedure)
        "43232006",  # Extrapleural pneumonectomy with empyemectomy (procedure)
        "432394003",  # Repair of mitral valve using fluoroscopic guidance (procedure)
        "432396001",  # Percutaneous pyeloplasty using fluoroscopic guidance (procedure)
        "432397005",  # Retrograde percutaneous pyeloplasty using fluoroscopic guidance (procedure)
        "432401002",  # Insertion of temporary inferior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "432422008",  # Ablation of cyst of kidney using ultrasound guidance (procedure)
        "432423003",  # Fluoroscopic angiography of common iliac artery using contrast with insertion of stent (procedure)
        "432424009",  # Fluoroscopic angiography of renal artery with contrast and insertion of stent (procedure)
        "432427002",  # Fluoroscopic angiography of superior mesenteric artery using contrast with insertion of drug eluting stent (procedure)
        "432545004",  # Cardiac radiofrequency ablation using ultrasound guidance (procedure)
        "432558003",  # Fluoroscopic angiography of abdominal aorta with insertion of branched bifurcated stent graft with contrast (procedure)
        "432584009",  # Fluoroscopic angiography of hepatic artery using contrast with insertion of drug eluting stent (procedure)
        "432585005",  # Fluoroscopic angiography of hepatic artery using contrast with insertion of stent (procedure)
        "432590008",  # Fluoroscopic angiography of thoracic aorta with contrast with insertion of drug eluting stent (procedure)
        "43259003",  # Insertion of graft of great vessels with cardiopulmonary bypass (procedure)
        "432632007",  # Repair of heart valve using fluoroscopic guidance (procedure)
        "432639003",  # Percutaneous transluminal cutting balloon angioplasty of inferior vena cava using fluoroscopic guidance with contrast (procedure)
        "432641002",  # Percutaneous transluminal cutting balloon angioplasty of superior vena cava using fluoroscopic guidance with contrast (procedure)
        "432649000",  # Lithotripsy of pancreas using fluoroscopic guidance (procedure)
        "432670006",  # Closure of patent foramen ovale using fluoroscopic guidance (procedure)
        "432683007",  # Percutaneous transluminal angioplasty of brachiocephalic artery using fluoroscopic guidance with contrast (procedure)
        "432684001",  # Percutaneous transluminal angioplasty of celiac artery using fluoroscopic guidance with contrast (procedure)
        "432687008",  # Percutaneous mechanical thrombectomy of vascular graft of upper limb using fluoroscopic guidance with contrast (procedure)
        "432790005",  # Fluoroscopic angiography of abdominal aorta with insertion of bifurcated stent graft with contrast (procedure)
        "432852006",  # Replacement of cardiac defibrillator using fluoroscopic guidance (procedure)
        "432868000",  # Percutaneous transluminal angioplasty of common iliac artery using fluoroscopic guidance with contrast (procedure)
        "43288002",  # Excision of lesion of pineal gland (procedure)
        "432941004",  # Percutaneous thrombolysis of hepatic vein using fluoroscopic guidance (procedure)
        "432942006",  # Percutaneous mechanical thrombectomy of inferior vena cava using fluoroscopic guidance with contrast (procedure)
        "432946009",  # Percutaneous mechanical thrombectomy of artery of lower limb using fluoroscopic guidance with contrast (procedure)
        "432947000",  # Percutaneous thrombectomy using fluoroscopic guidance (procedure)
        "432948005",  # Percutaneous thrombolysis of inferior vena cava using fluoroscopic guidance (procedure)
        "433024005",  # Gastrojejunostomy using fluoroscopic guidance (procedure)
        "433025006",  # Fluoroscopic angiography of superior mesenteric artery using contrast with insertion of stent (procedure)
        "433029000",  # Nephrostomy using computed tomography guidance (procedure)
        "433030005",  # Replacement of pulmonary artery catheter using fluoroscopic guidance with contrast (procedure)
        "433039006",  # Fluoroscopic angiography of abdominal aorta and iliac artery using contrast with insertion of unilateral aorto-iliac stent graft (procedure)
        "433040008",  # Percutaneous transluminal angioplasty of hepatic artery using fluoroscopic guidance with contrast (procedure)
        "433041007",  # Fluoroscopic angiography of abdominal aorta with contrast and insertion of branched unilateral aorto-iliac stent graft (procedure)
        "433062008",  # Removal of superior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "433112001",  # Percutaneous mechanical thrombectomy of portal vein using fluoroscopic guidance with contrast (procedure)
        "433113006",  # Percutaneous thrombolysis of mesenteric artery using fluoroscopic guidance (procedure)
        "43317001",  # Suture of aorta (procedure)
        "433172009",  # Cardiac ablation using fluoroscopy guidance (procedure)
        "433173004",  # Insertion of permanent inferior vena cava filter using fluoroscopic guidance with contrast (procedure)
        "433199006",  # Fluoroscopic angiography of brachiocephalic artery using contrast with insertion of drug eluting stent (procedure)
        "433200009",  # Fluoroscopic angiography of brachiocephalic artery using contrast with insertion of stent (procedure)
        "433240003",  # Percutaneous thrombolysis of pulmonary artery using fluoroscopic guidance (procedure)
        "433256008",  # Percutaneous transluminal cutting balloon angioplasty of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "43344006",  # Hofmeister operation, gastrectomy (procedure)
        "433589009",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of bifurcated stent graft (procedure)
        "433595005",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast and insertion of branched tapered stent graft (procedure)
        "433596006",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of fenestrated stent graft (procedure)
        "433598007",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of branched bifurcated stent graft (procedure)
        "4336007",  # Lloyd-Davies operation, abdominoperineal resection (procedure)
        "433633006",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of fenestrated bifurcated stent graft (procedure)
        "433634000",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast with insertion of tapered stent graft (procedure)
        "433638002",  # Percutaneous transluminal cutting balloon angioplasty of inferior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "433662007",  # Percutaneous transluminal cutting balloon angioplasty of external iliac artery using fluoroscopic guidance with contrast (procedure)
        "433663002",  # Percutaneous transluminal cutting balloon angioplasty of renal vein using fluoroscopic guidance with contrast (procedure)
        "433682006",  # Fluoroscopic venography of renal vein using contrast with insertion of stent graft (procedure)
        "433688005",  # Percutaneous thrombolysis of transplanted renal artery using fluoroscopic guidance (procedure)
        "433689002",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast with insertion of branched stent graft (procedure)
        "433691005",  # Fluoroscopic angiography of splenic artery using contrast with insertion of drug eluting stent (procedure)
        "433694002",  # Percutaneous transluminal cutting balloon angioplasty of celiac artery using fluoroscopic guidance with contrast (procedure)
        "43371006",  # Aneurysmectomy of ventricle of heart (procedure)
        "433713002",  # Fluoroscopic angiography of internal iliac artery using contrast with insertion of drug eluting stent (procedure)
        "433714008",  # Removal of temporary cardiac pacemaker lead using fluoroscopic guidance (procedure)
        "433736006",  # Fluoroscopic angiography of superior mesenteric artery using contrast with insertion of stent graft (procedure)
        "433762002",  # Fluoroscopic angiography of external iliac artery using contrast with insertion of stent graft (procedure)
        "433765000",  # Fluoroscopic angiography of thoracic aorta using contrast with insertion of branched tapered stent graft (procedure)
        "433766004",  # Fluoroscopic angiography of thoracic aorta using contrast with insertion of branched tube stent graft (procedure)
        "433768003",  # Fluoroscopic angiography of hepatic artery using contrast with insertion of stent graft (procedure)
        "433771006",  # Fluoroscopic angiography of internal iliac artery using contrast with insertion of stent graft (procedure)
        "433777005",  # Fluoroscopic angiography of inferior mesenteric artery using contrast with insertion of stent graft (procedure)
        "433780006",  # Fluoroscopic angiography of thoracic aorta with contrast with insertion of tapered stent graft (procedure)
        "433781005",  # Fluoroscopic angiography of thoracic aorta with contrast with insertion of tube stent graft (procedure)
        "433796009",  # Intraoperative fluoroscopic angiography of abdominal aorta with contrast with insertion of fenestrated unilateral aorto-iliac stent graft (procedure)
        "433802006",  # Fluoroscopic angiography of transplanted renal artery using contrast with insertion of stent (procedure)
        "433806009",  # Fluoroscopic angiography of abdominal aorta using contrast with insertion of fenestrated bifurcated stent graft (procedure)
        "433816001",  # Percutaneous transluminal cutting balloon angioplasty of common iliac artery using fluoroscopic guidance with contrast (procedure)
        "433857008",  # Fluoroscopic angiography of pulmonary artery using contrast with insertion of stent graft (procedure)
        "433866007",  # Laser ablation of lesion of liver using computed tomography guidance (procedure)
        "433876005",  # Ethanol ablation of lesion of liver using computed tomography guidance (procedure)
        "434026002",  # Fluoroscopic angiography of common iliac artery using contrast with insertion of stent graft (procedure)
        "434120005",  # Fluoroscopic angiography of splenic artery using contrast with insertion of stent graft (procedure)
        "434121009",  # Fluoroscopic angiography of thoracic aorta using contrast with insertion of fenestrated tube stent graft (procedure)
        "434138008",  # Fluoroscopic angiography of renal artery with contrast and insertion of stent graft (procedure)
        "434157004",  # Percutaneous transluminal cutting balloon angioplasty of renal artery using fluoroscopic guidance with contrast (procedure)
        "434160006",  # Fluoroscopic angiography of external iliac artery using contrast with insertion of drug eluting stent (procedure)
        "434245001",  # Percutaneous transluminal cutting balloon angioplasty of splenic artery using fluoroscopic guidance with contrast (procedure)
        "434246000",  # Percutaneous transluminal cutting balloon angioplasty of superior mesenteric artery using fluoroscopic guidance with contrast (procedure)
        "434297008",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast with insertion of fenestrated tapered stent graft (procedure)
        "434310007",  # Fluoroscopic angiography of brachiocephalic artery using contrast with insertion of stent graft (procedure)
        "434359006",  # Percutaneous transluminal cutting balloon angioplasty of abdominal aorta using fluoroscopic guidance with contrast (procedure)
        "434376005",  # Fluoroscopic angiography of abdominal aorta using contrast with insertion of fenestrated tube stent graft (procedure)
        "434377001",  # Fluoroscopic angiography of abdominal aorta using contrast with insertion of tube stent graft (procedure)
        "434432002",  # Intraoperative fluoroscopic angiography of thoracic aorta with contrast with insertion of fenestrated stent graft (procedure)
        "434436004",  # Percutaneous transluminal cutting balloon angioplasty of internal iliac artery using fluoroscopic guidance with contrast (procedure)
        "43450002",  # Total thymectomy (procedure)
        "43642001",  # Incision and exploration of retroperitoneum (procedure)
        "43654001",  # Correction of coarctation of aorta with anastomosis (procedure)
        "43685005",  # Repair of heart septum with tissue graft (procedure)
        "43750003",  # Endoscopic excision of lesion of duodenum (procedure)
        "43762004",  # Repair of ruptured aneurysm with graft of hepatic artery (procedure)
        "437746009",  # Percutaneous transluminal septal myocardial ablation (procedure)
        "43791001",  # Vaginal hysterectomy with total colpectomy (procedure)
        "43805005",  # Resection of thoracic artery with replacement (procedure)
        "43810009",  # Removal of thrombus (procedure)
        "438215008",  # Harvesting of part of small intestine from living donor (procedure)
        "438338008",  # Total gastrectomy with extended lymphadenectomy (procedure)
        "438341004",  # Laparoscopic insertion of peritoneal dialysis catheter (procedure)
        "438361005",  # Endovascular repair of abdominal aortic aneurysm with insertion of visceral extension prosthesis (procedure)
        "438362003",  # Excision of lamina of vertebra with transection of spinal accessory nerve (procedure)
        "438364002",  # Replacement of components of artificial heart (procedure)
        "438369007",  # Incision of bone of cranium and transection of tentorium cerebelli (procedure)
        "438371007",  # Excision of bone of suboccipital cranium and lamina of cervical vertebra for decompression of medulla and spinal cord (procedure)
        "438477007",  # Ventricular restoration procedure of heart using prosthetic patch graft (procedure)
        "438478002",  # Open circular amputation of forearm through radius and ulna (procedure)
        "438529005",  # Ablation of liver using magnetic resonance imaging guidance (procedure)
        "438562002",  # Excision of patella and reconstruction of patellofemoral joint for recurrent dislocation (procedure)
        "438563007",  # Division of clavicle (procedure)
        "438572004",  # Excision of lamina of thoracic vertebra and excision of intradural spinal lesion (procedure)
        "438580006",  # Grafting to aorta (procedure)
        "438614004",  # Excision of bone of cranium and excision of neoplasm of cerebellopontine angle (procedure)
        "438617006",  # Vestibular nerve section by transcranial approach (procedure)
        "438624007",  # Excision of lamina of cervical vertebra with transection of dentate ligament (procedure)
        "43868005",  # Creation of endorectal ileal J-pouch with anastomosis to anus (procedure)
        "4387005",  # Valvuloplasty of pulmonary valve in total repair of tetralogy of Fallot (procedure)
        "438769009",  # Unifocalization of pulmonary artery with cardiopulmonary bypass (procedure)
        "438781008",  # Reconstruction of vena cava (procedure)
        "4388000",  # Closure of splenocolic fistula (procedure)
        "438837007",  # Suspension and fixation of aorta for tracheal decompression (procedure)
        "438887000",  # Fibrin sheath stripping of tunneled central venous catheter using fluoroscopic guidance with contrast (procedure)
        "438945003",  # Endovascular insertion of stent into aortobiiliac aneurysm (procedure)
        "438999008",  # Percutaneous thrombolysis of cerebral venous sinus using fluoroscopic guidance (procedure)
        "439008003",  # Transfer of free jejunum with microvascular anastomosis (procedure)
        "439019000",  # Bypass of subclavian artery to brachial artery using vein graft (procedure)
        "439020006",  # Bypass of subclavian artery to vertebral artery using graft (procedure)
        "43907003",  # Pancreas to intestine anastomosis (procedure)
        "439071008",  # Complete removal of allograft of intestine (procedure)
        "439083005",  # Ablation of kidney using magnetic resonance imaging guidance (procedure)
        "439092008",  # Percutaneous transluminal insertion of stent into inferior vena cava using fluoroscopic guidance with contrast (procedure)
        "439093003",  # Percutaneous transluminal insertion of stent into superior vena cava using fluoroscopic guidance with contrast (procedure)
        "439095005",  # Replacement of peripherally inserted central catheter using fluoroscopic guidance with contrast (procedure)
        "439118005",  # Closed division of angle of mandible (procedure)
        "439171005",  # Laparoscopic selective transection of vagus nerve (procedure)
        "439234002",  # Open division of angle of mandible (procedure)
        "439235001",  # Laparoscopic radical nephrectomy (procedure)
        "439309000",  # Thoracoscopy with partial pulmonary decortication (procedure)
        "439331008",  # Repair of wound of heart with cardiopulmonary bypass (procedure)
        "439355003",  # Replacement of tunneled centrally inserted central venous catheter (procedure)
        "439368006",  # Laparoscopic cryoablation of neoplasm of liver (procedure)
        "439388005",  # Grafting to aorta with shunt bypass (procedure)
        "439397009",  # Distal revascularization and interval ligation (procedure)
        "439409004",  # Thoracoscopy with total pericardiectomy (procedure)
        "439414000",  # Replacement of peripherally inserted central venous catheter (procedure)
        "439435003",  # Thoracoscopy with excision of pericardial lesion (procedure)
        "439447009",  # Reimplantation of visceral artery into infrarenal aortic prosthesis (procedure)
        "439467000",  # Percutaneous transluminal balloon angioplasty of cerebral artery (procedure)
        "439493007",  # Incision of bone of cranium with elevation of bone flap and excision of epileptogenic focus (procedure)
        "439497008",  # Cystourethroscopy and excision of internal sphincter (procedure)
        "439499006",  # Excision of bone of cranium and excision of infratentorial neoplasm of brain (procedure)
        "439524005",  # Dissection within cavernous sinus for obliteration of carotid artery aneurysm (procedure)
        "439529000",  # Endovascular repair of aneurysm of iliac artery (procedure)
        "439536004",  # Percutaneous angiography of cerebral artery and insertion of drug eluting stent using fluoroscopic guidance with contrast (procedure)
        "439541007",  # Percutaneous thrombectomy of cerebral artery using fluoroscopic guidance with contrast (procedure)
        "439572006",  # Bypass of axillary artery to brachial artery using vein graft (procedure)
        "439587001",  # Harvesting of small intestine from cadaver donor (procedure)
        "439625002",  # Percutaneous mechanical thrombectomy of cerebral venous sinus using fluoroscopic guidance with contrast (procedure)
        "439700004",  # Percutaneous transluminal angioplasty of cerebral venous sinus using fluoroscopic guidance with contrast (procedure)
        "439703002",  # Percutaneous insertion of stent into cerebral venous sinus using fluoroscopic guidance with contrast (procedure)
        "439707001",  # Transmyocardial closure of ventricular septal defect using catheter with implant (procedure)
        "439712000",  # Cholecystectomy with exploration of common bile duct and choledochoenterostomy (procedure)
        "439714004",  # Transmyocardial closure of ventricular septal defect using catheter with implant and cardiopulmonary bypass (procedure)
        "439739008",  # Laparoscopic partial excision of kidney (procedure)
        "439769001",  # Excision of lamina of cervical vertebra and excision of intradural spinal lesion (procedure)
        "439773003",  # Endovascular repair of aneurysm with transcatheter placement of wireless physiologic sensor (procedure)
        "439783004",  # Division of maxilla (procedure)
        "439808002",  # Laparoscopic ablation of lesion of kidney (procedure)
        "439817002",  # Thoracoscopy with total pulmonary decortication and intrapleural pneumonolysis (procedure)
        "439825000",  # Laparoscopic insertion of permanent intraperitoneal catheter (procedure)
        "439826004",  # Laparoscopic reconstruction of ureteropelvic junction (procedure)
        "439844007",  # Reduction and reconstruction of craniomegalic skull with incision of bone of cranium (procedure)
        "439878001",  # Completion gastrectomy (procedure)
        "439917001",  # Replacement of nontunneled centrally inserted central venous catheter (procedure)
        "439918006",  # Replacement of tunneled centrally inserted central venous catheter with subcutaneous pump (procedure)
        "439944006",  # Insertion of proximal extension prosthesis for repair of descending thoracic aorta (procedure)
        "439964001",  # Laparoscopic excision of kidney and total excision of ureter (procedure)
        "439969006",  # Grafting to great vessel of thorax with shunt bypass (procedure)
        "440000004",  # Delayed insertion of distal extension prosthesis of descending thoracic aorta (procedure)
        "440008006",  # Excision of lamina of lumbar vertebra and excision of intradural spinal lesion (procedure)
        "440032004",  # Laparoscopic fixation of prolapse of rectum (procedure)
        "440033009",  # Laparoscopic fixation of prolapse of rectum with resection of sigmoid (procedure)
        "440041009",  # Incision of peritoneum with removal of gastric stimulation electrodes from lesser curvature of stomach (procedure)
        "440062005",  # Excision of bone of cranium with excision of abscess of brain (procedure)
        "440070000",  # Repair of ileoanal pouch (procedure)
        "440078007",  # Laparoscopic construction of gastric stoma (procedure)
        "440104006",  # Insertion of peripancreatic drain with gastrostomy (procedure)
        "44011001",  # Transection of mental nerve (procedure)
        "440111005",  # Laparoscopic transection of trunk of vagus nerve (procedure)
        "440147006",  # Incision of bone of cranium and excision of neoplasm of pituitary gland using intracranial approach (procedure)
        "440148001",  # Partial excision of uterus for removal of interstitial ectopic pregnancy (procedure)
        "440189003",  # Incision of peritoneum with replacement of gastric stimulation electrodes from lesser curvature of stomach (procedure)
        "440198000",  # Osteotomy of jaw (procedure)
        "440211003",  # Excision of lamina of vertebra with release of tethered lumbar spinal cord (procedure)
        "440221006",  # Bypass of carotid artery to brachial artery using vein graft (procedure)
        "440235008",  # Percutaneous thrombolysis of cerebral artery using fluoroscopic guidance (procedure)
        "440333003",  # Percutaneous chemical angioplasty of cerebral artery using fluoroscopic guidance with contrast (procedure)
        "440383008",  # Radical amputation of cervix with bilateral total pelvic lymphadenectomy and paraaortic lymph node biopsy (procedure)
        "440446009",  # Laparoscopic excision of kidney and partial excision of ureter (procedure)
        "440448005",  # Insertion of stent graft into iliac artery using fluoroscopic guidance with contrast (procedure)
        "440459001",  # Removal of peripherally inserted central catheter using fluoroscopic guidance with contrast (procedure)
        "440499003",  # Radiofrequency ablation of lesion of kidney using ultrasound guidance (procedure)
        "440549000",  # Insertion of peripancreatic drain with jejunostomy (procedure)
        "440563006",  # Division of mandible (procedure)
        "440581009",  # Laparoscopic anastomosis of gallbladder to intestine (procedure)
        "440619008",  # Ablation of ventricular arrhythmogenic focus with cardiopulmonary bypass (procedure)
        "440649005",  # Cystourethroscopy with percutaneous insertion of nephrostomy tube (procedure)
        "440715006",  # Stapled transanal resection of rectum (procedure)
        "44076002",  # Esophagoenteric anastomosis, antesternal or antethoracic (procedure)
        "44119001",  # Anastomosis of small intestine to small intestine (procedure)
        "44148007",  # Revision of enterostomy (procedure)
        "441597006",  # Insertion of depth electroencephalogram electrode into brain for electroencephalography (procedure)
        "441619002",  # Repair of obstetric laceration of perineum and anal sphincter and mucosa of rectum (procedure)
        "441631000",  # Percutaneous transluminal angioplasty of inferior vena cava using fluoroscopic guidance with contrast (procedure)
        "441676000",  # Occlusion of patent ductus arteriosus using embolization coil (procedure)
        "441751006",  # Transplantation of intestine (procedure)
        "441754003",  # Percutaneous transhepatic transplantation of pancreatic islet cell using fluoroscopic guidance (procedure)
        "441763001",  # Endoscopic exteriorization of third ventricle (procedure)
        "441820006",  # Laparoscopic-assisted vaginal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "441873006",  # Percutaneous replacement of aortic valve using fluoroscopic guidance (procedure)
        "441895005",  # Replacement of aortic valve with tissue graft (procedure)
        "44192009",  # Closure of cholecystogastroenteric fistula (procedure)
        "441959001",  # Anastomosis of pulmonary artery to subclavian artery (procedure)
        "44198008",  # Complicated pyeloplasty (procedure)
        "442058009",  # Revisional hemilaminectomy for decompression of lumbar spine (procedure)
        "442087005",  # Closure of ventricular septal defect using fluoroscopic guidance (procedure)
        "442102003",  # Cemented prosthetic hybrid arthroplasty of knee joint (procedure)
        "442118009",  # Primary hemilaminectomy for decompression of lumbar spine (procedure)
        "442123009",  # Sano procedure (procedure)
        "442188002",  # Percutaneous transluminal valvotomy and valvuloplasty of pulmonary valve structure (procedure)
        "442189005",  # Excision of hydrocele of tunica vaginalis (procedure)
        "442249009",  # Implantation of cardioverter defibrillator with three electrode leads (procedure)
        "442251008",  # Percutaneous transluminal balloon dilation of interposition tube prosthesis between pulmonary artery and subclavian artery (procedure)
        "442278005",  # Anastomosis of superior vena cava to pulmonary artery (procedure)
        "442285009",  # Excision of intestinal structure (procedure)
        "442328006",  # Percutaneous transluminal ablation of septum of heart using chemical substance (procedure)
        "442399006",  # Circumferential panniculectomy (procedure)
        "442442002",  # Division of mitral valve chordae tendineae (procedure)
        "442456000",  # Closure of atrial septal defect using septal occluder device (procedure)
        "442525005",  # Percutaneous replacement of pulmonary valve using fluoroscopic guidance (procedure)
        "442656009",  # Conversion to cemented prosthetic hybrid arthroplasty of knee joint (procedure)
        "44267002",  # Insertion of drainage tube into kidney pelvis (procedure)
        "442737009",  # Appendicovesicostomy (procedure)
        "442806002",  # Excision of rectum with anastomosis of colon to anus and creation of colonic reservoir (procedure)
        "442907005",  # Surgical removal of single chamber pacing cardioverter defibrillator electrode by transthoracic approach (procedure)
        "442919002",  # Adrenal sparing radical nephrectomy (procedure)
        "443025009",  # Reamputation of lower leg through tibia and fibula (procedure)
        "443102006",  # Excision of entire pericardium (procedure)
        "443112004",  # Repair of partial anomalous pulmonary venous connection by direct anomalous vein anastomosis to left atrium (procedure)
        "443176005",  # Repair of partial anomalous pulmonary venous connection by unroofing of coronary sinus with patch closure of interatrial communication (procedure)
        "443186006",  # Reconstruction of ramus of mandible by L osteotomy with grafting of bone (procedure)
        "443194004",  # Reconstruction of ramus of mandible by horizontal osteotomy with grafting of bone (procedure)
        "443202004",  # Le Fort I osteotomy of maxilla into one segment with grafting of bone (procedure)
        "443211004",  # Excision of part of pericardium (procedure)
        "443221007",  # Reconstruction of ramus of mandible by sagittal split osteotomy with internal fixation (procedure)
        "443331002",  # Le Fort I osteotomy of maxilla into three segments with grafting of bone (procedure)
        "443332009",  # Le Fort I osteotomy of maxilla into two segments with grafting of bone (procedure)
        "443358009",  # Surgical removal of transvenous electrode of dual chamber pacing cardioverter defibrillator pulse generator by transthoracic approach (procedure)
        "443391000",  # Repair of total anomalous pulmonary venous connection by insertion of interatrial baffle with reconnection of superior caval vein to right atrial appendage (procedure)
        "443407008",  # Revision of anastomosis of intestine (procedure)
        "443434006",  # Replacement of dual chamber pulse generator (procedure)
        "443435007",  # Total replacement of right hip joint (procedure)
        "443463009",  # Percutaneous transluminal insertion of stent into superior mesenteric artery (procedure)
        "443468000",  # Reconstruction of ramus of mandible by vertical osteotomy with grafting of bone (procedure)
        "443470009",  # Reconstruction of ramus of mandible by C osteotomy with grafting of bone (procedure)
        "443477007",  # Surgical removal of transvenous electrode of dual chamber pacing cardioverter defibrillator pulse generator by transvenous approach (procedure)
        "443480008",  # Laparoscopic excision of part of colon (procedure)
        "44349007",  # Operation on jejunum (procedure)
        "443509009",  # Sliding osteotomy of chin with single piece implant (procedure)
        "443531001",  # Percutaneous insertion of stent into coarctation of aorta using fluoroscopic guidance with contrast (procedure)
        "443532008",  # Percutaneous insertion of stent into right ventricular outflow tract using fluoroscopic guidance (procedure)
        "443577005",  # Percutaneous insertion of stent into aneurysm of cerebrum using fluoroscopic guidance with contrast (procedure)
        "443624007",  # Percutaneous insertion of pulmonary valve using fluoroscopic guidance (procedure)
        "443647008",  # Revision of percutaneous transluminal coil embolization of cerebral aneurysm using fluoroscopic guidance with contrast (procedure)
        "443665007",  # Transanal disk excision of rectum (procedure)
        "443676006",  # Surgical removal of transvenous electrode of single chamber pacing cardioverter defibrillator pulse generator by transthoracic approach (procedure)
        "443681002",  # Total replacement of left knee joint (procedure)
        "443682009",  # Total replacement of right knee joint (procedure)
        "443706000",  # Repair of partial anomalous pulmonary venous connection by direct anomalous vein anastomosis to left atrial appendage (procedure)
        "44378008",  # Partial colectomy with anastomosis (procedure)
        "443791002",  # Excision of cyst of maxilla with partial excision of maxilla by extraoral approach (procedure)
        "443829004",  # Percutaneous transluminal balloon angioplasty of coarctation of aorta with insertion of stent (procedure)
        "443852000",  # Replacement of temporary transvenous electrode of dual chamber pulse generator (procedure)
        "443869003",  # Total excision of right kidney (procedure)
        "443870002",  # Laparoscopy of rectum (procedure)
        "443886009",  # Endovascular repair of aneurysm of iliac artery with graft using radiologic guidance (procedure)
        "443887000",  # Grafting of aortic valve prosthesis by transapical approach (procedure)
        "443906008",  # Bypass of stomach with short limb Roux-en-Y gastroenterostomy (procedure)
        "443908009",  # Excision of part of rectum by perineal approach (procedure)
        "443909001",  # Excision of part of colon with mobilization of splenic flexure (procedure)
        "443916000",  # Grafting of aortic valve prosthesis by transluminal approach (procedure)
        "443918004",  # Local excision of malignant neoplasm of stomach (procedure)
        "443935000",  # Excision of ruptured appendix by open approach (procedure)
        "443958002",  # Surgical removal of transvenous electrode of single chamber pacing cardioverter defibrillator pulse generator by transvenous approach (procedure)
        "443959005",  # Direct laryngoscopy with excision of neoplasm using operating microscope (procedure)
        "443989003",  # Bidirectional Glenn shunt procedure of left superior vena cava (procedure)
        "444001009",  # Right Glenn shunt procedure (procedure)
        "444034006",  # Bidirectional Glenn shunt procedure of right superior vena cava (procedure)
        "444068000",  # Thoracoscopic excision of neoplasm of pericardium (procedure)
        "444080008",  # Thoracoscopic excision of cyst of pericardium (procedure)
        "444083005",  # Total excision of left kidney (procedure)
        "444141002",  # Thoracoscopic excision of mass of pericardium (procedure)
        "444165004",  # Excision of part of colon and excision of terminal ileum with ileocolic anastomosis (procedure)
        "444176000",  # Repair of total anomalous pulmonary venous connection by unroofing and closure of coronary sinus and interatrial communication (procedure)
        "444178004",  # Left Glenn shunt procedure (procedure)
        "444188003",  # Thoracoscopic wedge resection of lung (procedure)
        "4443002",  # Decortication of ovary (procedure)
        "444348000",  # Augmentation of chin using sliding osteotomy and interpositional autograft (procedure)
        "444463001",  # Replacement of bilateral knee joints (procedure)
        "444566006",  # Replacement of cardiac pacemaker (procedure)
        "444588006",  # Endovascular repair of abdominal aortic aneurysm (procedure)
        "444593009",  # Marsupialization of cyst of pancreas (procedure)
        "444601005",  # Removal of extracranial ventricular shunt (procedure)
        "444624003",  # Laparoscopic wedge resection of ovary (procedure)
        "444688002",  # Suspension and fixation of rectum to wall of vagina for prolapse of rectum (procedure)
        "44476006",  # Closure of fistula of kidney (procedure)
        "444798002",  # Repair of partial anomalous pulmonary venous connection by baffle redirection to left atrium with superior caval vein anastomosis to right atrial appendage (procedure)
        "444812008",  # Repair of aortic valve using fluoroscopic guidance (procedure)
        "444825000",  # Repair of partial anomalous pulmonary venous connection by direct anomalous vein anastomosis to right atrium and insertion of baffle to direct pulmonary venous flow to left atrium (procedure)
        "444866000",  # Laparoscopic marsupialization of cyst of ovary (procedure)
        "44496003",  # Repair with vein graft of intrathoracic artery with bypass (procedure)
        "445089003",  # Closure of patent ductus arteriosus using fluoroscopic guidance with contrast (procedure)
        "445168005",  # Laparoscopic incision and drainage of cyst of ovary (procedure)
        "445370005",  # Repair of partial anomalous pulmonary venous connection by baffle redirection to left atrium with attachment of systemic vein to atrial appendage (procedure)
        "445420008",  # Replacement of extracranial ventricular shunt (procedure)
        "445429009",  # Excision of lamina of cervical vertebra (procedure)
        "445746008",  # Craniotomy and removal of hematoma from extradural space (procedure)
        "445831005",  # Partial excision of stomach and jejunal transposition and excision of intestine (procedure)
        "445838004",  # Laparoscopic excision of lesion of stomach by intragastric approach (procedure)
        "445884009",  # Wedge resection of colon (procedure)
        "445891007",  # Suturing of laceration of small intestine (procedure)
        "445892000",  # Sigmoidoscopic excision of lesion of large intestine (procedure)
        "445912000",  # Excision of fallopian tube and surgical removal of ectopic pregnancy (procedure)
        "445917006",  # Laparoscopic excision of retroperitoneal lymph node (procedure)
        "445942002",  # Excision of temporal lobe (procedure)
        "445971001",  # Laparotomy and incision of small intestine (procedure)
        "446103006",  # Partial excision of lesion of brain (procedure)
        "446109005",  # Simple mastectomy with excision of axillary lymph nodes (procedure)
        "446126004",  # Laparoscopic fixation of kidney (procedure)
        "446170008",  # Colonoscopic excision of lesion of large intestine (procedure)
        "44619000",  # Wedge resection of bilateral ovaries (procedure)
        "44620006",  # Endarterectomy of intracranial artery (procedure)
        "446222007",  # Hand-assisted laparoscopic left colectomy (procedure)
        "446223002",  # Hand-assisted laparoscopic right colectomy (procedure)
        "446228006",  # Hand-assisted laparoscopic extended left colectomy (procedure)
        "446240006",  # Excision of medial part of maxilla and nasal turbinate (procedure)
        "446296004",  # Laparoscopic total excision of kidney (procedure)
        "446307000",  # Total excision of meningioma (procedure)
        "446309002",  # Partial excision of meningioma (procedure)
        "446318000",  # Incision of cyst of ovary (procedure)
        "446342001",  # Excision of neoplasm from spinal extradural space (procedure)
        "446420001",  # Simple mastectomy with complete axillary lymphadenectomy (procedure)
        "44642007",  # Partial colectomy with colostomy and creation of mucofistula (procedure)
        "446440008",  # Curettage of lesion of brain (procedure)
        "446446002",  # Total abdominal hysterectomy and removal of vaginal cuff (procedure)
        "446481000",  # Excision of testis by inguinal approach (procedure)
        "446510008",  # Excision of dentigerous cyst of maxilla (procedure)
        "446526009",  # Debulking of neoplasm of ovary (procedure)
        "446528005",  # Incision and drainage of cyst of ovary (procedure)
        "446544001",  # Laparoscopic closure of fistula of ureter (procedure)
        "446570000",  # Craniectomy by retromastoid approach (procedure)
        "446571001",  # Craniotomy of posterior fossa (procedure)
        "446576006",  # Surgical procedure on vascular structure of adrenal gland (procedure)
        "446585006",  # Laparoscopic excision of diverticulum of urinary bladder (procedure)
        "446586007",  # Laparoscopic partial excision of urinary bladder (procedure)
        "446598002",  # Laparoscopic salpingo-oophorectomy (procedure)
        "44662004",  # Femoral-femoral artery vascular bypass (procedure)
        "446624000",  # Removal of cardiac monitoring implant using fluoroscopic guidance (procedure)
        "446650001",  # Excision of mucosa of stomach (procedure)
        "446652009",  # Laparotomy and decompression of intestine (procedure)
        "446677005",  # Laparoscopic excision of adrenal gland by retroperitoneal approach (procedure)
        "446679008",  # Total laparoscopic excision of uterus by abdominal approach (procedure)
        "446683008",  # Craniotomy and decompression of brain (procedure)
        "446693001",  # Excision of cyst of vallecula of epiglottis (procedure)
        "446705006",  # Trephination of cranium and drainage of chronic hematoma from subdural space (procedure)
        "446715000",  # Laparoscopic total excision of ureter (procedure)
        "446726006",  # Ligation of varix of duodenum (procedure)
        "446733006",  # Suturing of laceration of esophagus by transabdominal approach (procedure)
        "446734000",  # Suturing of laceration of esophagus by transthoracic approach (procedure)
        "446741006",  # Endoscopic ligation of varix of small intestine (procedure)
        "446744003",  # Endoscopy of upper gastrointestinal tract and ligation of varix of duodenum (procedure)
        "446746001",  # Excision of mucosa of duodenum (procedure)
        "446804002",  # Hysteroscopic excision of leiomyoma of uterus (procedure)
        "446808004",  # Total excision of maxilla and excision of nasal turbinate (procedure)
        "446816008",  # Craniotomy and evacuation of traumatic hematoma (procedure)
        "446823009",  # Excision of lesion of internal urethral orifice (procedure)
        "446830003",  # Laparoscopic wedge resection of lesion of stomach (procedure)
        "446844009",  # Laparoscopic decortication of cyst of kidney (procedure)
        "446848007",  # Craniotomy and evacuation of blood clot (procedure)
        "446850004",  # Trephination of cranium and external drainage of effusion from subdural space (procedure)
        "446865003",  # Craniotomy and removal of foreign body (procedure)
        "446868001",  # Excision of meningioma by supratentorial approach (procedure)
        "446891002",  # Laparoscopic lysis of perirenal adhesions (procedure)
        "446892009",  # Laparoscopic lysis of periureteral adhesions (procedure)
        "446894005",  # Laparoscopic partial excision of kidney by retroperitoneal approach (procedure)
        "446942007",  # Laparoscopic incision and drainage of cyst of ovary with conversion to laparotomy (procedure)
        "446959002",  # Laparoscopic incision of fallopian tube with conversion to laparotomy (procedure)
        "446960007",  # Laparoscopic salpingo-oophorectomy with conversion to laparotomy (procedure)
        "446990000",  # Laparoscopic excision of ureter and kidney (procedure)
        "447028007",  # Incision of petrous pyramid air cells (procedure)
        "447038002",  # Ligation of blood vessel of cerebral meninges (procedure)
        "447067001",  # Partial excision of neoplasm of spinal cord (procedure)
        "447076008",  # Laparoscopic excision of fimbrial cyst of fallopian tube (procedure)
        "447077004",  # Laparoscopic lysis of adhesions of ovary and fallopian tube (procedure)
        "447107001",  # Craniotomy and excision of neoplasm of brain (procedure)
        "447115003",  # Total excision of lesion of brain tissue (procedure)
        "447124007",  # Marsupialization of lesion of kidney (procedure)
        "447135002",  # Simple mastectomy with axillary lymph node sampling (procedure)
        "447156000",  # Trephination of cranium and drainage of empyema of subdural space (procedure)
        "447166008",  # Hand-assisted laparoscopic extended right colectomy (procedure)
        "447173003",  # Excision of contiguous neoplasm of extradural and intradural space of spine (procedure)
        "447174009",  # Excision of neoplasm of intradural extramedullary spine (procedure)
        "447237002",  # Hysteroscopic excision of uterus and supporting structures (procedure)
        "447281005",  # Video assisted thoracoscopic excision of esophagus (procedure)
        "447307008",  # Laparoscopic total excision of urinary bladder (procedure)
        "447308003",  # Excision of urethra and urinary bladder (procedure)
        "447362004",  # Excision of meningioma by infratentorial approach (procedure)
        "447363009",  # Craniotomy by retromastoid approach (procedure)
        "447380008",  # Dissection of retroperitoneum (procedure)
        "447423009",  # Laparoscopic total excision of ureter by retroperitoneal approach (procedure)
        "447469009",  # Partial excision of meningioma by infratentorial approach (procedure)
        "447480009",  # Laparoscopic decortication of cyst of kidney by retroperitoneal approach (procedure)
        "447494005",  # Laparoscopic lysis of perirenal adhesions by retroperitoneal approach (procedure)
        "447495006",  # Laparoscopic lysis of periureteral adhesions by retroperitoneal approach (procedure)
        "44751009",  # Total abdominal colectomy with proctectomy and continent ileostomy (procedure)
        "447517002",  # Video assisted thoracoscopic excision of esophagus by abdominal approach (procedure)
        "447518007",  # Partial excision of meningioma by supratentorial approach (procedure)
        "447527008",  # Laparoscopic total excision of kidney by retroperitoneal approach (procedure)
        "447529006",  # Total excision of meningioma by infratentorial approach (procedure)
        "447531002",  # Laparoscopic excision of ureter and kidney by retroperitoneal approach (procedure)
        "447532009",  # Laparoscopic fixation of kidney by retroperitoneal approach (procedure)
        "447582004",  # Laparoscopic closure of fistula of ureter by retroperitoneal approach (procedure)
        "447594008",  # Total excision of meningioma by supratentorial approach (procedure)
        "447647009",  # Suturing of intrathoracic artery (procedure)
        "447652004",  # Excision of nodule of larynx (procedure)
        "447743004",  # Percutaneous transluminal angioplasty of superior mesenteric artery (procedure)
        "447746007",  # Endarterectomy and patch repair of inferior mesenteric artery (procedure)
        "447748008",  # Primary repair of defect of atrioventricular septum (procedure)
        "447752008",  # Excision of cyst of epiglottis (procedure)
        "44777001",  # Creation of conduit between right ventricle and pulmonary artery (procedure)
        "447771005",  # Abdominal hysterectomy and excision of periuterine tissue (procedure)
        "447786004",  # Creation of anastomosis from ascending aorta to main pulmonary artery (procedure)
        "447787008",  # Conversion from previous anastomosis of stomach to jejunum (procedure)
        "447788003",  # Creation of anastomosis from left subclavian artery to left pulmonary artery (procedure)
        "447842000",  # Replacement of enterostomy device of large intestine (procedure)
        "447844004",  # Wedge excision of cervix uteri and suturing of surgical wound (procedure)
        "447878005",  # Creation of anastomosis from aorta to pulmonary artery (procedure)
        "447879002",  # Creation of anastomosis from ascending aorta to right pulmonary artery (procedure)
        "447880004",  # Exploration and ligation of aneurysm of cerebral artery (procedure)
        "447884008",  # Creation of conduit between right ventricle of heart and vena cava (procedure)
        "447887001",  # Closure of defect of atrioventricular septum using prosthetic patch (procedure)
        "447892004",  # Reversal of duodenal switch (procedure)
        "447908005",  # Bypass of common iliac artery by anastomosis of aorta to common iliac artery (procedure)
        "447911006",  # Segmental osteotomy of anterior mandible (procedure)
        "447920002",  # Bypass of iliac artery by anastomosis of iliac artery to iliac artery (procedure)
        "447921003",  # Bypass of femoral artery by anastomosis of femoral artery to tibial artery using prosthesis (procedure)
        "44793005",  # Excision of lesion of fallopian tube (procedure)
        "447936000",  # Excision of intrahepatic bile duct (procedure)
        "44794004",  # Incision of atrium of heart (procedure)
        "447946003",  # Right hemicolectomy and ileostomy (procedure)
        "447950005",  # Creation of anastomosis from ascending aorta to left pulmonary artery (procedure)
        "447954001",  # Radiofrequency ablation of lesion of heart (procedure)
        "447958003",  # Creation of anastomosis from right subclavian artery to right pulmonary artery (procedure)
        "447976005",  # Removal of cardiac biventricular permanent pacemaker using fluoroscopic guidance (procedure)
        "447981001",  # Allotransplantation of islets of Langerhans (procedure)
        "447982008",  # Low anterior resection of rectum with colostomy (procedure)
        "447983003",  # Excision of Reinke's edema (procedure)
        "447984009",  # Excision of cyst of vallecula of epiglottis using laser (procedure)
        "447985005",  # Excision of lesion of larynx using laser (procedure)
        "448008005",  # Bypass of segment of thoracic aorta by anastomosis of aorta to aorta (procedure)
        "448009002",  # Bypass of iliac artery by anastomosis of aorta to external iliac artery (procedure)
        "448010007",  # Bypass of artery of lower extremity by anastomosis of aorta to superficial femoral artery (procedure)
        "448012004",  # High intensity focused ultrasound ablation of lesion of liver (procedure)
        "448013009",  # Incision of pancreatic sphincter (procedure)
        "448019008",  # Creation of conduit between atrium and ventricle of heart (procedure)
        "448020002",  # Creation of shunt from aorta to pulmonary artery using interposition tube prosthesis (procedure)
        "448031005",  # Debulking of neoplasm of abdominal cavity (procedure)
        "448034002",  # Excision of nodule of vocal cord using laser (procedure)
        "448036000",  # Bypass of popliteal artery by anastomosis of popliteal artery to popliteal artery using prosthesis (procedure)
        "448042001",  # Percutaneous transluminal ablation of conducting system of heart (procedure)
        "448043006",  # Primary closure of defect of interventricular septum (procedure)
        "448050005",  # Colectomy and side to side anastomosis of ileum to colon (procedure)
        "448088007",  # Bypass of artery of lower extremity by anastomosis of aorta to common femoral artery (procedure)
        "448109000",  # Osteotomy and advancement of mandible and maxilla (procedure)
        "448122008",  # Bypass of segment of abdominal aorta by anastomosis of aorta to aorta (procedure)
        "448123003",  # Bypass of popliteal artery by anastomosis of popliteal artery to tibial artery using prosthesis (procedure)
        "448143009",  # Anterior resection of rectum and total excision of mesorectum (procedure)
        "448156002",  # Excision of rectum by transanal approach (procedure)
        "448166005",  # Laparoscopic ligation of inferior mesenteric artery (procedure)
        "448171003",  # Bypass of bifurcation of aorta by anastomosis of aorta to femoral artery (procedure)
        "448187000",  # Craniotomy and clipping of intracranial aneurysm (procedure)
        "448190006",  # Partial excision of esophagus and anastomosis of esophagus to jejunum (procedure)
        "448195001",  # Closure of defect of septum of heart using pericardial patch (procedure)
        "448198004",  # Removal of cardiac biventricular implantable cardioverter defibrillator using fluoroscopic guidance (procedure)
        "448268000",  # Excision of aneurysm of splenic artery (procedure)
        "448322000",  # Bypass of popliteal artery by anastomosis of popliteal artery to peroneal artery using prosthesis (procedure)
        "448323005",  # Bypass of femoral artery by anastomosis of femoral artery to peroneal artery using vein graft (procedure)
        "448324004",  # Bypass of popliteal artery by anastomosis of popliteal artery to popliteal artery using vein graft (procedure)
        "448340001",  # Reopening of chest and reexploration of intraabdominal operation site (procedure)
        "448341002",  # Endarterectomy and patch repair of suprarenal artery (procedure)
        "448342009",  # Endovascular fenestration of dissection of aorta (procedure)
        "448352008",  # Sphincteroplasty of pancreatic duct using duodenal approach (procedure)
        "448369005",  # Pharyngolaryngoesophagectomy and radical neck dissection (procedure)
        "448444006",  # Incision of intracranial vascular structure (procedure)
        "448460005",  # Excision of cyst of larynx (procedure)
        "448474003",  # Low anterior resection of rectum and total excision of mesorectum (procedure)
        "448529008",  # Bypass of femoral artery by anastomosis of femoral artery to tibial artery using vein graft (procedure)
        "448534007",  # Endarterectomy and patch repair of celiac artery (procedure)
        "448539002",  # Vaginal hysterectomy and excision of periuterine tissue (procedure)
        "448540000",  # Laparoscopic microwave ablation of lesion of liver (procedure)
        "448596007",  # Excision of rectal stump (procedure)
        "448653006",  # Anterior resection of rectum and total mesorectal excision with colostomy (procedure)
        "448658002",  # Wedge osteotomy of sternum (procedure)
        "448685009",  # Percutaneous transluminal angioplasty of celiac artery (procedure)
        "448699008",  # Thoracotomy and removal of cardiac pacemaker electrodes (procedure)
        "448754006",  # Percutaneous transluminal angioplasty of inferior mesenteric artery (procedure)
        "448769007",  # Microtherapeutic endoscopic excision of lesion of larynx (procedure)
        "448779009",  # Occlusion of systemic to pulmonary artery shunt using transluminal embolic device (procedure)
        "448785002",  # Placement of stent at site of coarctation of aorta (procedure)
        "448797001",  # Bidirectional superior cavopulmonary anastomosis (procedure)
        "448798006",  # Percutaneous transluminal angioplasty of adrenal artery (procedure)
        "448829002",  # Suturing of laceration of rectum (procedure)
        "448840005",  # Repair of supravalvar aorta using patch (procedure)
        "448841009",  # Replacement of cardiac biventricular implantable cardioverter defibrillator using fluoroscopic guidance (procedure)
        "448869001",  # Replacement of cardiac biventricular permanent pacemaker using fluoroscopic guidance (procedure)
        "448939009",  # Endarterectomy of inferior mesenteric artery (procedure)
        "448941005",  # Reopening of abdomen and reexploration of intraabdominal operation site (procedure)
        "448949007",  # Placement of stent in systemic to pulmonary artery shunt (procedure)
        "448957005",  # Closure of defect of septum of heart using prosthetic patch (procedure)
        "449033003",  # Balloon dilation of cardiac conduit (procedure)
        "449039004",  # Conversion to total prosthetic replacement of hip joint (procedure)
        "449041003",  # Replacement of cardiac conduit (procedure)
        "449046008",  # Posterior segmental osteotomy of mandible (procedure)
        "449095008",  # Excision of exostosis of maxilla (procedure)
        "449103007",  # Ligation of common hepatic artery (procedure)
        "449113004",  # Anastomosis of superior vena cava to pulmonary artery and reconstruction of right ventricular outlfow tract (procedure)
        "449145009",  # Suturing of intraabdominal artery (procedure)
        "449146005",  # Replacement of root of aorta with reimplantation of coronary arteries using button technique (procedure)
        "449227002",  # Balloon dilation of systemic to pulmonary artery shunt (procedure)
        "449266005",  # Excision of intracranial vascular structure (procedure)
        "449338000",  # Replacement of aortic root using pulmonary valve autograft and replacement of pulmonary valve (procedure)
        "449389000",  # Placement of stent in cardiac conduit (procedure)
        "449395004",  # Endoscopic retrograde cholangiopancreatography and incision of duodenal papilla (procedure)
        "449403001",  # Balloon dilation of recoarctation of aorta (procedure)
        "449404007",  # Cryosurgery of lesion of liver (procedure)
        "449432003",  # Excision of Reinke's edema using laser (procedure)
        "449449005",  # Excision of nodule of larynx using laser (procedure)
        "449453007",  # Bypass of segment of suprarenal abdominal aorta by anastomosis of aorta to aorta (procedure)
        "449454001",  # Bypass of segment of ascending aorta by anastomosis of aorta to aorta (procedure)
        "449530008",  # Bypass of segment of infrarenal abdominal aorta by anastomosis of aorta to aorta (procedure)
        "449549007",  # Bilateral bidirectional superior cavopulmonary anastomoses (procedure)
        "449550007",  # Central aortopulmonary anastomosis (procedure)
        "449611000124108",  # Repair of aortic arch (procedure)
        "450330005",  # Removal of tunneled central venous catheter (procedure)
        "450460004",  # Nephrostomy using ultrasound guidance (procedure)
        "450461000",  # Subtotal excision of colon and creation of colonic pouch (procedure)
        "450462007",  # Subtotal excision of colon and rectum and creation of colonic pouch (procedure)
        "450481004",  # Partial excision of esophagus and interposition of colon (procedure)
        "450482006",  # Bypass of esophagus by interposition of jejunum (procedure)
        "450487000",  # Laparoscopic excision of ovary (procedure)
        "450501004",  # Laser recanalization of intestine (procedure)
        "450503001",  # Obliteration of aneurysm of cerebral artery (procedure)
        "450504007",  # Obliteration of aneurysm of circle of Willis (procedure)
        "450513009",  # Endarterectomy of suprarenal artery (procedure)
        "450559006",  # Laparoscopic excision of leiomyoma of uterus (procedure)
        "450560001",  # Laparoscopic removal of ovarian cyst (procedure)
        "450563004",  # Laparoscopic excision of ectopic ovarian pregnancy (procedure)
        "450564005",  # Ligation of superior mesenteric artery (procedure)
        "450565006",  # Ligation of celiac artery (procedure)
        "450567003",  # Conversion to hybrid total knee replacement (procedure)
        "450578006",  # Removal of foreign body from rectouterine pouch (procedure)
        "450592000",  # Excision of lesion of cardiac wall (procedure)
        "450635009",  # Open biopsy of lesion of peritoneal cavity (procedure)
        "450645006",  # Open embolectomy of celiac artery (procedure)
        "450646007",  # Open embolectomy of inferior mesenteric artery (procedure)
        "450647003",  # Open embolectomy of superior mesenteric artery (procedure)
        "450648008",  # Open embolectomy of suprarenal artery (procedure)
        "450669005",  # Excision of adnexa of uterus (procedure)
        "450682002",  # Bypass of segment of aorta (procedure)
        "450683007",  # Bypass of axillary artery (procedure)
        "450684001",  # Bypass of brachial artery (procedure)
        "450685000",  # Bypass of celiac artery (procedure)
        "450686004",  # Bypass of subclavian artery (procedure)
        "450687008",  # Bypass of superior mesenteric artery (procedure)
        "450688003",  # Bypass of suprarenal artery (procedure)
        "450689006",  # Bypass of vertebral artery (procedure)
        "450690002",  # Bypass of carotid artery (procedure)
        "450691003",  # Bypass of inferior mesenteric artery (procedure)
        "450692005",  # Open excision of lesion of uterus (procedure)
        "450703003",  # Repair of iliac artery (procedure)
        "450704009",  # Repair of common iliac artery (procedure)
        "450707002",  # Repair of cardiac wall (procedure)
        "450783000",  # Primary closure of defect of septum of heart (procedure)
        "450794001",  # Revision of closure of defect of septum of heart (procedure)
        "450815006",  # Revision of prosthesis of abdominal aorta (procedure)
        "450816007",  # Revision of transplantation of heart (procedure)
        "450821005",  # Replacement of pulse generator of implantable cardioverter defibrillator using fluoroscopic guidance (procedure)
        "450826000",  # Surgical procedure on internal thoracic artery by transluminal approach (procedure)
        "450938008",  # Commissurotomy of heart valve (procedure)
        "451025008",  # Open commissurotomy of mitral valve (procedure)
        "451026009",  # Open commissurotomy of tricuspid valve (procedure)
        "451027000",  # Commissurotomy of atrioventricular valve (procedure)
        "451028005",  # Commissurotomy of mitral valve (procedure)
        "451029002",  # Commissurotomy of tricuspid valve (procedure)
        "451040005",  # Excision of cerebral hemisphere (procedure)
        "4520009",  # Anastomosis of esophagus, antesternal or antethoracic, with insertion of prosthesis (procedure)
        "45220009",  # Posterior resection of rectum (procedure)
        "45228002",  # Duodenoileostomy (procedure)
        "452351000124109",  # Amputation of right arm through wrist (procedure)
        "452361000124106",  # Amputation of left arm through wrist (procedure)
        "452391000124103",  # Disarticulation of right hip (procedure)
        "452401000124101",  # Disarticulation of left hip (procedure)
        "45257003",  # Destructive procedure of heart valve (procedure)
        "45314005",  # Closure of cholecystogastric fistula (procedure)
        "45322003",  # Decompression of brain (procedure)
        "453381000124108",  # Disarticulation of right knee (procedure)
        "453391000124106",  # Disarticulation of left knee (procedure)
        "45464004",  # Excision of lesion of ampulla of Vater (procedure)
        "45523007",  # Partial laterovertical laryngectomy (procedure)
        "455251000124107",  # Thoracoscopic surgical lobectomy using video-assisted guidance (procedure)
        "455271000124102",  # Total resection of visible brain tumor (procedure)
        "45533004",  # Repair of ruptured aneurysm with graft of mesenteric artery (procedure)
        "4558008",  # Anterior resection of rectum (procedure)
        "456004",  # Total abdominal colectomy with ileostomy (procedure)
        "45657005",  # Transmetacarpal amputation, reamputation (procedure)
        "45669002",  # Cardio-omentopexy (procedure)
        "45685007",  # Radical neck dissection (procedure)
        "456903003",  # Extended radical mastectomy of bilateral breasts (procedure)
        "45692002",  # Anastomosis of esophagus, antesternal or antethoracic, with interposition of jejunal loop (procedure)
        "4570007",  # Chemopallidectomy (procedure)
        "45720008",  # Implantation of electrode into cardiac ventricle, replacement (procedure)
        "45753003",  # Cholecystenterostomy with gastroenterostomy (procedure)
        "45782002",  # Thrombectomy with catheter of axillary artery by arm incision (procedure)
        "458111000124107",  # Nerve-sparing retroperitoneal lymph node dissection (procedure)
        "45842000",  # Repair of retroperitoneal tissue (procedure)
        "45858002",  # Amputation of forearm through radius AND ulna, reamputation (procedure)
        "45874007",  # Aneurysmectomy with graft replacement of thoracic aorta (procedure)
        "45900003",  # Esophagectomy (procedure)
        "459006",  # Closed condylotomy of mandible (procedure)
        "45902006",  # Salpingo-oophororrhaphy (procedure)
        "45918000",  # Repair of pericardium (procedure)
        "45921003",  # Removal of cardiac pacemaker electrodes with replacement (procedure)
        "459341000124100",  # Endovascular renal denervation (procedure)
        "459691000124101",  # Transcatheter mitral valve repair (procedure)
        "459831000124101",  # Removal of left knee joint prosthesis for periprosthetic joint infection (procedure)
        "459841000124106",  # Removal of right knee joint prosthesis for periprosthetic joint infection (procedure)
        "459851000124108",  # Removal of left shoulder joint prosthesis for periprosthetic joint infection (procedure)
        "459861000124105",  # Removal of right shoulder joint prosthesis for periprosthetic joint infection (procedure)
        "459871000124103",  # Removal of left hip joint prosthesis for periprosthetic joint infection (procedure)
        "459881000124100",  # Removal of right hip joint prosthesis for periprosthetic joint infection (procedure)
        "46013000",  # Excision of lesion of abdominal aorta with interposition graft replacement (procedure)
        "46028000",  # Amputation of hand (procedure)
        "46032006",  # Division of recurrent laryngeal nerve (procedure)
        "46070005",  # Revision of colostomy (procedure)
        "46082001",  # Lateral orbitotomy (procedure)
        "461331000124101",  # Left atrial appendage closure (procedure)
        "46196009",  # Surgical construction of arteriovenous shunt (procedure)
        "46205009",  # Pyelotomy with tube drainage (procedure)
        "46226009",  # Cervicectomy with synchronous colporrhaphy (procedure)
        "46227000",  # Fenestration of aortic aneurysm (procedure)
        "46295004",  # Proctectomy by abdominoperineal resection with colostomy, one stage (procedure)
        "46342000",  # Removal of electrodes from brain (procedure)
        "46381000",  # Truncal vagotomy with pyloroplasty (procedure)
        "46412002",  # Apicectomy lung (procedure)
        "46437007",  # Repair of ostium secundum defect with prosthesis by closed heart technique (procedure)
        "46444003",  # Gas endarterectomy of lower limb artery (procedure)
        "46489002",  # Excision of lesion of ventricle of heart (procedure)
        "46569000",  # Laparoscopic-assisted abdominoperineal resection (procedure)
        "46599009",  # Closure of fistula of common bile duct (procedure)
        "46645006",  # Replacement of electroencephalographic receiver in brain (procedure)
        "46647003",  # Excision of lesion of perirenal tissue (procedure)
        "46790005",  # Orbitotomy with bone flap by lateral approach, Kroenlein, with drainage (procedure)
        "46816007",  # Intracranial aneurysmectomy (procedure)
        "46852006",  # Excision of falciform ligament (procedure)
        "46897002",  # Percutaneous balloon valvuloplasty of pulmonary valve (procedure)
        "46936007",  # Hemigastrectomy by thoracic approach (procedure)
        "47020004",  # Ventriculoperitoneal shunt (procedure)
        "47041005",  # Incision and exploration of pancreas (procedure)
        "47045001",  # Excision of lesion of round ligament (procedure)
        "47058000",  # Heart transplant with recipient cardiectomy (procedure)
        "47074005",  # Ventriculocholecystostomy (procedure)
        "47110007",  # Complete sialoadenectomy (procedure)
        "47195003",  # Repair of ostium primum defect (procedure)
        "472191000119101",  # Thrombolysis of cerebral artery by intravenous infusion (procedure)
        "47302001",  # Ileotomy (procedure)
        "473216000",  # Suture plication of artery for control of duodenal ulcer hemorrhage (procedure)
        "473229000",  # Catheter ablation of arrhythmogenic focus (procedure)
        "473239006",  # Excision of bilateral ovaries for malignant neoplasm of ovary (procedure)
        "473276009",  # Suture plication of artery for control of rectal hemorrhage (procedure)
        "473277000",  # Laparoscopic excision of hydrosalpinx (procedure)
        "47353000",  # Reopening of laparotomy site (procedure)
        "474008",  # Esophagoenteric anastomosis, intrathoracic (procedure)
        "47432005",  # Implantation of heart valve prosthesis or synthetic device (procedure)
        "47454007",  # Ovarian denervation (procedure)
        "47478002",  # Laminectomy for implantation of neurostimulator electrodes, spinal cord (procedure)
        "47485003",  # Creation of aortomesenteric shunt (procedure)
        "47490000",  # Revision of pyelostomy (procedure)
        "47547004",  # Culp-Scardino operation, ureteral flap pyeloplasty (procedure)
        "47568003",  # Removal of ventricular catheter with synchronous replacement (procedure)
        "47575002",  # Femorotibial vascular bypass, anterior or posterior (procedure)
        "47596009",  # Cup arthroplasty of hip with use of methyl methacrylate (procedure)
        "47610008",  # Costotomy (procedure)
        "47696003",  # Anastomosis of intestine, large-to-anus (procedure)
        "47705001",  # Incision of rectal stricture (procedure)
        "47712005",  # Total ureterectomy of ectopic ureter by combined approach (procedure)
        "47751000",  # Angiectomy with graft replacement by interposition (procedure)
        "47765003",  # Closure of pleuroperitoneal fistula (procedure)
        "47789005",  # Excision of lesion of pancreas (procedure)
        "47800001",  # Repair of coronary arteriocardiac chamber fistula (procedure)
        "47934001",  # Plastic repair with reconstruction of cardia of stomach (procedure)
        "47935000",  # Cordectomy of vocal cord (procedure)
        "4811009",  # Reconstruction of diaphragm (procedure)
        "48195000",  # Thrombectomy with catheter of innominate artery by thoracic incision (procedure)
        "48222002",  # Closure of rectovaginal fistula by vaginal approach (procedure)
        "48268004",  # Closed atrial septectomy (procedure)
        "4829004",  # Excision of small intestine for interposition (procedure)
        "48307009",  # Closure of rectourethral fistula (procedure)
        "48431000",  # Anastomosis of thoracic artery to coronary artery, single (procedure)
        "4862007",  # Revision of anastomosis of large intestine (procedure)
        "48630009",  # Incision and exploration of rectum (procedure)
        "48710004",  # Thrombectomy with catheter of aortoiliac artery by abdominal incision (procedure)
        "48765000",  # Creation of arteriovenous fistula by autologous graft (procedure)
        "48775002",  # Repair of obstetric laceration of pelvic floor (procedure)
        "48783008",  # Choledochopancreatostomy (procedure)
        "48835001",  # Esophagogastric fundoplasty (procedure)
        "48855000",  # Incision and exploration of ileum (procedure)
        "48859006",  # Creation of esophagogastric sphincteric competence (procedure)
        "4891005",  # Proctostomy (procedure)
        "48920003",  # Hepatotomy with packing (procedure)
        "48939008",  # Destruction of retroperitoneal tumor (procedure)
        "48941009",  # Creation of lumbosubarachnoid shunt with valve (procedure)
        "49187005",  # Endarterectomy of aorta (procedure)
        "49195009",  # Cranioplasty with synchronous repair of encephalocele (procedure)
        "49209004",  # Subtotal gastrectomy (procedure)
        "49245001",  # Bypass gastrojejunostomy (procedure)
        "49273004",  # Laparoscopy with removal of adnexal structures (procedure)
        "4930005",  # Repair of thoracogastric fistula (procedure)
        "49326008",  # Creation of conduit between left ventricle and aorta (procedure)
        "49438003",  # Appendectomy with drainage (procedure)
        "49440008",  # Revision of colostomy, complicated (procedure)
        "49493009",  # Resection of cornua of bilateral fallopian tubes (procedure)
        "49552003",  # Arteriectomy with anastomosis of head and neck artery (procedure)
        "49586007",  # Appendicocecostomy (procedure)
        "49624000",  # Transfer of iliopsoas to femoral neck (procedure)
        "49676001",  # Closed mitral valve commissurotomy (procedure)
        "49683008",  # Replacement of small intestine tube (procedure)
        "4974007",  # Operation on liver (procedure)
        "49754004",  # Manipulation of heart valve (procedure)
        "49780003",  # Heminephrectomy (procedure)
        "49795001",  # Total pneumonectomy (procedure)
        "49826001",  # Highly selective vagotomy with pyloroplasty (procedure)
        "49924002",  # Perineal colostomy (procedure)
        "49935007",  # Partial pelvectomy of kidney (procedure)
        "50012009",  # Cecopexy (procedure)
        "50075004",  # Repair of aneurysm of abdominal aorta involving iliac vessels with graft (procedure)
        "50078002",  # Anastomosis of common pulmonary trunk and left atrium, posterior wall (procedure)
        "50086002",  # Thrombectomy with catheter of renal artery by abdominal incision (procedure)
        "50128001",  # Grafting of mediastinal fat to myocardium (procedure)
        "50216001",  # Replacement of mitral valve with tissue graft (procedure)
        "50225007",  # Removal of heart assist system (procedure)
        "50339005",  # Complete laryngectomy with radical neck dissection, synchronous thyroidectomy and synchronous tracheostomy (procedure)
        "50419005",  # Aneurysmectomy of descending aorta with anastomosis (procedure)
        "50434004",  # Excision of lesion of aorta (procedure)
        "50445001",  # Mastoidectomy (procedure)
        "50447009",  # Aortic-superior mesenteric vascular bypass (procedure)
        "50466009",  # Repair of aneurysm with graft of celiac artery (procedure)
        "50487004",  # Thrombectomy with catheter of vena cava by leg incision (procedure)
        "50558002",  # Division of intracranial artery (procedure)
        "5057003",  # Radical orbitomaxillectomy (procedure)
        "50736001",  # Division of aortic arch (procedure)
        "50774009",  # Laparoscopic-assisted right colectomy (procedure)
        "50807007",  # Femoral-popliteal artery bypass graft with vein (procedure)
        "50887005",  # Enlargement of pre-existing foramen ovale (procedure)
        "50965000",  # Destructive procedure on pineal gland (procedure)
        "51018000",  # Iliofemoral shunt (procedure)
        "510831000124107",  # Disarticulation of right ankle (procedure)
        "51087002",  # Repair of ostium primum defect with tissue graft (procedure)
        "51113007",  # Lysis of adhesions of appendix (procedure)
        "5119000",  # Epiglottidectomy (procedure)
        "51201009",  # Correction of transposition of great vessels (procedure)
        "51233000",  # Sigmoidostomy (procedure)
        "51279005",  # Enlargement of foramen ovale in repair of total anomalous pulmonary venous connection (procedure)
        "51346007",  # Ileectomy (procedure)
        "51382002",  # Creation of carotid-carotid shunt (procedure)
        "51396008",  # Anastomosis of small intestine to rectal stump (procedure)
        "51452009",  # Closure of pharyngoesophageal fistula (procedure)
        "51582001",  # Transposition of tissue of diaphragm (procedure)
        "51607004",  # Total ureterectomy (procedure)
        "5165002",  # Division of thoracic artery (procedure)
        "51684008",  # Radical subtotal resection of pancreas (procedure)
        "51748005",  # Insertion of drainage tube into kidney (procedure)
        "5176003",  # Thromboendarterectomy with graft of renal artery (procedure)
        "51870000",  # Excision of lesion of kidney with partial nephrectomy (procedure)
        "51884005",  # Division of trabeculae carneae cordis (procedure)
        "51977003",  # Repair of rectourethral fistula with colostomy (procedure)
        "51993007",  # Aneurysmectomy of aortic arch (procedure)
        "51994001",  # Excision of benign tumor of rectum by transanal approach (procedure)
        "52005005",  # Thromboendarterectomy with graft, combined aortoiliac (procedure)
        "52155000",  # Thrombectomy with catheter of subclavian vein by neck incision (procedure)
        "52247003",  # Repair of aortic valve with tissue graft (procedure)
        "52314009",  # Extended simple mastectomy of bilateral breasts (procedure)
        "52395009",  # Ligation of splenic artery (procedure)
        "52428004",  # Creation of subarachnoid/subdural-pleural shunt (procedure)
        "52592006",  # Stewart operation, renal plication with pyeloplasty (procedure)
        "52667004",  # Sigmoidosigmoidostomy (procedure)
        "5267001",  # Ligation of adrenal artery (procedure)
        "52672008",  # Excision of lesion of abdominal aorta with end-to-end anastomosis (procedure)
        "52699005",  # Hypophysectomy (procedure)
        "52712005",  # Open excision of lesion of endometrium (procedure)
        "52734007",  # Total replacement of hip (procedure)
        "52785004",  # Destruction of lesion of kidney (procedure)
        "52808008",  # Removal of neuropacemaker of brain (procedure)
        "52821001",  # Sequestrectomy of bone of nose (procedure)
        "52917009",  # Repair of diaphragmatic hernia by abdominal approach (procedure)
        "52930005",  # Closure of ventricular septal defect (procedure)
        "52961001",  # Construction of corpora cavernosa-saphenous vein shunt (procedure)
        "52969004",  # Closure of vesicocutaneous fistula (procedure)
        "53003006",  # Resection of splenic flexure of colon (procedure)
        "53059001",  # Replacement of mitral valve (procedure)
        "53087005",  # Intrathoracic esophagocolostomy (procedure)
        "531007",  # Open pulmonary valve commissurotomy with inflow occlusion (procedure)
        "53122004",  # Peroneal artery bypass graft with vein (procedure)
        "53133001",  # Cecocoloplicopexy (procedure)
        "5316002",  # Closed osteotomy of mandibular ramus (procedure)
        "5317006",  # Radical amputation of penis with bilateral pelvic lymphadenectomy (procedure)
        "53266006",  # Operation on diaphragm (procedure)
        "533005",  # Closure of vesicocolic fistula (procedure)
        "53344002",  # Repair of ostium primum defect with prosthesis (procedure)
        "53380002",  # Removal of calculus of perirenal tissue (procedure)
        "53386008",  # Arthroplasty of knee, tibial plateau (procedure)
        "53423001",  # Lobectomy of brain (procedure)
        "53442002",  # Excision of stomach structure (procedure)
        "535003",  # Closure of ureterovesicovaginal fistula (procedure)
        "53514001",  # Ultrasonic fragmentation of urinary stone through percutaneous nephrostomy (procedure)
        "53791007",  # Destruction of endometriotic cyst of retroperitoneum (procedure)
        "53816005",  # Incision and exploration of perirenal tissue (procedure)
        "53828002",  # Ligation of major artery of abdomen (procedure)
        "5384005",  # Excision of part of frontal cortex (procedure)
        "53879008",  # Creation of aortoceliac shunt (procedure)
        "53937001",  # Distal splenorenal anastomosis (procedure)
        "53941002",  # Closure of septal fenestration of heart (procedure)
        "54109001",  # Repair of ostium secundum defect with tissue graft (procedure)
        "54130005",  # Hysteroscopy with resection of intrauterine septum (procedure)
        "5415002",  # Destruction of lesion of liver (procedure)
        "54153006",  # Incision of liver (procedure)
        "54154000",  # Resection of sphincter of Oddi (procedure)
        "54164009",  # Rectosigmoidectomy (procedure)
        "54261007",  # Excision of uterus and supporting structures (procedure)
        "543008",  # Choledochostomy with transduodenal sphincteroplasty (procedure)
        "54315009",  # Revision of prosthesis of heart valve (procedure)
        "54357003",  # Partial excision of small intestine (procedure)
        "54490004",  # Vaginal hysterectomy with repair of enterocele (procedure)
        "54629001",  # Permanent magnetic colostomy (procedure)
        "54636000",  # Cardiomyopexy (procedure)
        "54747001",  # Multiple segmental resections of large intestine (procedure)
        "54750003",  # Pyelorrhaphy (procedure)
        "54817007",  # Arteriovenous shunt for renal dialysis by external cannula (procedure)
        "54854003",  # Creation of conduit of right ventricle and pulmonary artery in repair of pulmonary artery atresia (procedure)
        "549007",  # Incision of intracranial vein (procedure)
        "54956002",  # Gastrostomy (procedure)
        "55192007",  # Ligation of coronary sinus (procedure)
        "55230008",  # Implantation of mammary artery into ventricle (procedure)
        "55253009",  # Replacement of tricuspid valve with tissue graft (procedure)
        "55278008",  # Repair of mitral valve with tissue graft (procedure)
        "55286008",  # Janeway operation, permanent gastrostomy (procedure)
        "55312004",  # Aneurysmectomy with anastomosis of lower limb vein (procedure)
        "55377001",  # Embolectomy with catheter of innominate artery by arm incision (procedure)
        "55407004",  # Excision of cusp of heart valve (procedure)
        "55415001",  # Culp-Deweerd operation, spiral flap pyeloplasty (procedure)
        "55506005",  # Single valve replacement with valvuloplasty of two valves (procedure)
        "5551006",  # Direct laryngoscopy with arytenoidectomy with operating microscope (procedure)
        "55517006",  # Esophagogastric fundoplasty with fundic patch (procedure)
        "55586007",  # Anastomosis of esophagus, antesternal or antethoracic, with insertion of rubber tube (procedure)
        "55588008",  # Excision of abdominal artery (procedure)
        "55645007",  # Closure of reno-intestinal fistula (procedure)
        "55648009",  # Endoscopic dilation of pancreatic duct (procedure)
        "55698005",  # Valvulotomy of pulmonary valve in total repair of tetralogy of Fallot (procedure)
        "55703004",  # Ligation of atrium of heart (procedure)
        "5571002",  # Excision of lesion of aorta with end-to-end anastomosis (procedure)
        "5572009",  # Incision of kidney pelvis (procedure)
        "55743009",  # Division of nerve of adrenal gland (procedure)
        "55966001",  # Rankin operation, complete proctectomy (procedure)
        "56036004",  # Osteotomy of mandible (procedure)
        "56078005",  # Endoscopic excision of lesion of sphincter of Oddi (procedure)
        "56157005",  # Bicuspidization of mitral valve (procedure)
        "56206003",  # Hemimaxillectomy with bone graft or with prosthesis (procedure)
        "56256008",  # Closure of vesicocervicovaginal fistula (procedure)
        "56283009",  # Transplantation of small intestine (procedure)
        "56349004",  # Aorto-myocardial shunt (procedure)
        "56373007",  # Excision of lesion of artery of head and neck (procedure)
        "56463006",  # Diverticulectomy of esophagus with myotomy (procedure)
        "56569008",  # Excision of lesion of cranium (procedure)
        "56614001",  # Repair of fistula involving urinary bladder and intestine (procedure)
        "56674006",  # Closure of ureteral fistula (procedure)
        "56782003",  # Direct thrombectomy of subclavian vein by neck incision (procedure)
        "56784002",  # Removal of automatic implantable cardioverter/defibrillator pads and electrodes (procedure)
        "56801004",  # Percutaneous transluminal angioplasty of renal artery (procedure)
        "56816001",  # Insertion of Davidson button into abdomen (procedure)
        "56834002",  # Repair of aneurysm of abdominal aorta involving visceral vessels with graft (procedure)
        "5687005",  # Removal of foreign body from ovary (procedure)
        "56875009",  # Repair of aneurysm with graft of hepatic artery (procedure)
        "56959007",  # Operation on globus pallidus (procedure)
        "56970000",  # Central shunt with prosthetic graft (procedure)
        "5722009",  # Surgical reanastomosis of colon (procedure)
        "57236003",  # Closure of cholecystenteric fistula (procedure)
        "5726007",  # Removal of epicardial electrodes (procedure)
        "57262007",  # Manipulation of pineal gland (procedure)
        "57297009",  # Implantation of electrode into cardiac atrium and ventricle, replacement (procedure)
        "57363000",  # Transplantation of aberrant renal vein (procedure)
        "57365007",  # Excision of lesion of aorta with interposition graft replacement (procedure)
        "57416001",  # Segmental osteotomy of mandible (procedure)
        "5745003",  # Excision ampulla of Vater with reimplantation of common duct (procedure)
        "57462009",  # Replacement of tube or enterostomy device of large intestine (procedure)
        "57545001",  # Ligation of major artery of chest (procedure)
        "57589001",  # Arthroplasty, acetabular and proximal femoral prosthetic replacement, with allograft (procedure)
        "57598003",  # Ventricular shunt to bone marrow (procedure)
        "57639000",  # Excision of lesion of urinary bladder (procedure)
        "57655007",  # Ureterosigmoidostomy (procedure)
        "57721002",  # Valvulotomy of tricuspid valve (procedure)
        "57822000",  # Incision of intracranial artery (procedure)
        "57881009",  # Ventricular shunt to cisterna magna (procedure)
        "5789003",  # Lysis of adhesions of intestines (procedure)
        "58036009",  # Partial laryngectomy via anterovertical approach (procedure)
        "58090001",  # Pancreaticocystoenterostomy (procedure)
        "5809008",  # Selective vagotomy with pyloroplasty and gastrostomy (procedure)
        "58157003",  # Anastomosis of renal vein and splenic vein (procedure)
        "58204008",  # Reconstruction of annulus of cardiac valve (procedure)
        "58211007",  # Removal of electronic heart device battery (procedure)
        "58337008",  # Transorbital leukotomy (procedure)
        "58380004",  # Pericardiostomy (procedure)
        "58506009",  # Embolectomy of ascending aorta (procedure)
        "58515002",  # Cryoleucotomy (procedure)
        "58622000",  # Intraoperative transluminal angioplasty of visceral artery (procedure)
        "58661008",  # Closure of fistula of uterus (procedure)
        "58665004",  # Revision of corrective procedure on heart (procedure)
        "58685000",  # Closure of cholecystocolic fistula (procedure)
        "58818008",  # Excision of lesion from abdominal artery (procedure)
        "58902001",  # Ilioiliac bypass graft with vein (procedure)
        "59012002",  # Carotid-subclavian artery bypass graft with vein (procedure)
        "59085008",  # Modified radical dissection of bilateral sides of neck (procedure)
        "59117006",  # Implantation of aortic valve with tissue graft (procedure)
        "59304001",  # Open biopsy of pancreas (procedure)
        "59411003",  # Destruction of tissue of diaphragm (procedure)
        "59444009",  # Bifemoral bypass graft with vein (procedure)
        "59461007",  # Direct repair of intra-abdominal artery (procedure)
        "59567009",  # Imbrication of diaphragm (procedure)
        "59656001",  # Incision of pancreas (procedure)
        "59657005",  # Intestinal bypass for morbid obesity (procedure)
        "59701006",  # Closure of ureterocervical fistula (procedure)
        "59710003",  # Simple orchiectomy with placement of testicular prosthesis by scrotal approach (procedure)
        "59793000",  # Amputation of arm through humerus, reamputation (procedure)
        "59893009",  # Repair of tricuspid valve with tissue graft (procedure)
        "59896001",  # Dilation of rectum (procedure)
        "59912004",  # Pancreaticoduodenostomy (procedure)
        "60041003",  # Laryngopharyngectomy (procedure)
        "60055006",  # Closure of cholecystojejunal fistula (procedure)
        "60158005",  # Bisection of bilateral ovaries (procedure)
        "60159002",  # Implantation of Swanson prosthesis of knee (procedure)
        "60194009",  # Distal subtotal pancreatectomy with splenectomy (procedure)
        "6025007",  # Laparoscopic appendectomy (procedure)
        "60283001",  # Closure of aortoduodenal fistula (procedure)
        "6029001",  # Augmentation of outflow tract of pulmonary valve (procedure)
        "60362005",  # Suture of sigmoid (procedure)
        "60385006",  # Thrombectomy with catheter of ulnar artery by arm incision (procedure)
        "60440009",  # Arthroplasty of knee, femoral condyles with debridement and partial synovectomy (procedure)
        "60465006",  # Permanent gastrostomy (procedure)
        "60501001",  # Creation of ventriculo-jugular shunt (procedure)
        "60510009",  # Creation of subarachnoid/subdural-jugular shunt (procedure)
        "60572009",  # Vascular subclavian-subclavian bypass (procedure)
        "60581003",  # Intrathoracic esophageal anastomosis with interposition of small bowel (procedure)
        "60606003",  # Duodenocholedochotomy with transduodenal choledocholithotomy (procedure)
        "60869005",  # Division of ascending aorta (procedure)
        "608797008",  # Open repair of inguinal hernia (procedure)
        "608799006",  # Open repair of obstructed inguinal hernia (procedure)
        "608801009",  # Open repair of strangulated inguinal hernia (procedure)
        "608805000",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele (procedure)
        "608806004",  # Laparoscopic assisted vaginal hysterectomy with repair of rectocele (procedure)
        "608807008",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "608873006",  # Open repair of thoracoabdominal aortic aneurysm (procedure)
        "608885003",  # Laparoscopic left oophorectomy (procedure)
        "608887006",  # Laparoscopic right oophorectomy (procedure)
        "608888001",  # Laparoscopic left salpingo-oophorectomy (procedure)
        "608889009",  # Laparoscopic right salpingo-oophorectomy (procedure)
        "608890000",  # Laparoscopic excision of bilateral ovarian cysts (procedure)
        "608891001",  # Laparoscopic excision of cyst of left ovary (procedure)
        "608892008",  # Laparoscopic excision of cyst of right ovary (procedure)
        "608893003",  # Laparoscopic repair of ovary and fallopian tube (procedure)
        "60892005",  # Jejunojejunostomy (procedure)
        "609159007",  # Percutaneous transluminal angioplasty of superior vena cava using fluoroscopic guidance with contrast (procedure)
        "609161003",  # Percutaneous transluminal insertion of fenestrated stent graft into aorta (procedure)
        "609162005",  # Percutaneous transluminal insertion of branched stent graft into aorta (procedure)
        "609163000",  # Percutaneous transluminal insertion of stent into hepatic artery (procedure)
        "609168009",  # Percutaneous cryoablation of neoplasm of kidney using computed tomography guidance (procedure)
        "609169001",  # Percutaneous cryoablation of neoplasm of liver using computed tomography guidance (procedure)
        "609189000",  # Endoscopic destruction of tissue of pancreatic duct (procedure)
        "609192001",  # Endoscopic excision of tissue of pancreatic duct (procedure)
        "609194000",  # Endoscopic surgical procedure on duodenum using laser (procedure)
        "609196003",  # Endoscopic surgical procedure on rectum using laser (procedure)
        "609197007",  # Endoscopic surgical procedure on colon using laser (procedure)
        "609230000",  # Laparoscopic bilateral salpingo-oophorectomy (procedure)
        "609232008",  # Percutaneous microwave ablation of lesion of liver (procedure)
        "609233003",  # Percutaneous radiofrequency ablation of epicardium (procedure)
        "609234009",  # Percutaneous transluminal closure of defect of cardiac septum (procedure)
        "609236006",  # Percutaneous cardiopulmonary bypass support (procedure)
        "609238007",  # Percutaneous endopyelotomy (procedure)
        "609269006",  # Open repair of obstructed inguinal hernia with prosthesis (procedure)
        "609270007",  # Open repair of recurrent paraumbilical hernia with prosthesis (procedure)
        "609272004",  # Open repair of strangulated inguinal hernia with prosthesis (procedure)
        "609274003",  # Laparoscopic bilateral salpingo-oophorectomy with conversion to laparotomy (procedure)
        "609275002",  # Laparoscopic bilateral oophorectomy (procedure)
        "609279008",  # Endoscopic excision of tissue of large intestine (procedure)
        "609280006",  # Endoscopic excision of tissue of esophagus (procedure)
        "609281005",  # Endoscopic excision of tissue of stomach (procedure)
        "609293001",  # Laparoscopic left oophorectomy after prior right oophorectomy (procedure)
        "609294007",  # Laparoscopic right oophorectomy after prior left oophorectomy (procedure)
        "609295008",  # Laparoscopic left salpingo-oophorectomy after prior right salpingo-oophorectomy (procedure)
        "609296009",  # Laparoscopic right salpingo-oophorectomy after prior left salpingo-oophorectomy (procedure)
        "60954008",  # Cervical esophagostomy (procedure)
        "609588000",  # Prosthetic total arthroplasty of knee joint (procedure)
        "60985004",  # Implantation of cardiac dual-chamber device, replacement (procedure)
        "610941000124104",  # Dissection and segmentation of fissure of sylvius (procedure)
        "61125008",  # Aortofemoral-popliteal artery in-situ vein bypass graft (procedure)
        "61151005",  # Cholecystopancreatostomy (procedure)
        "61166008",  # Aneurysmectomy of ascending aorta (procedure)
        "61174009",  # Transposition of ovary (procedure)
        "61192001",  # Closure of aorticopulmonary fistula or fenestration (procedure)
        "61218006",  # Excision of Meckel's diverticulum (procedure)
        "61236006",  # Aortocoronary artery bypass graft repeated (situation)
        "61326000",  # Hepatectomy, total right lobectomy (procedure)
        "61327009",  # Puncture and drainage of craniopharyngioma (procedure)
        "61392008",  # Reconstruction of pelvic floor (procedure)
        "61404002",  # Oophoropexy (procedure)
        "6146001",  # Repair of heart septum with prosthesis (procedure)
        "61473007",  # Detorsion of ovary (procedure)
        "61477008",  # Incision and exploration of liver (procedure)
        "61522002",  # Atrial septectomy by blade method (procedure)
        "61524001",  # Resection of exteriorized segment of small intestine (procedure)
        "61535006",  # Transplantation of spleen (procedure)
        "61600000",  # Take-down of jejunoileal bypass (procedure)
        "6166005",  # Nephropyeloureterostomy (procedure)
        "61667000",  # Highly selective vagotomy (procedure)
        "61735009",  # Hepatoduodenostomy (procedure)
        "61893009",  # Laparoscopic treatment of ectopic pregnancy with oophorectomy (procedure)
        "6190006",  # Colopexy (procedure)
        "61965006",  # Incision of cranial sinus (procedure)
        "62069000",  # Delayed opening colostomy (procedure)
        "62118003",  # Repair of ventricular septal defect with prosthesis in total repair of tetralogy of Fallot (procedure)
        "62149008",  # Labyrinthotomy, osseous (procedure)
        "62157006",  # Closure of ureterovaginal fistula (procedure)
        "6221005",  # Suture of colon (procedure)
        "62399001",  # Autotransplantation of ovary (procedure)
        "62400008",  # Division of isthmus of horseshoe kidney (procedure)
        "62438007",  # Transplantation of pancreas (procedure)
        "62446008",  # Destructive procedure on brain (procedure)
        "62460005",  # Repair of inguinal hernia with orchiectomy and prosthetic replacement (procedure)
        "62461000052109",  # Second transurethral resection of urinary bladder (procedure)
        "62511003",  # Autotransplant of lung (procedure)
        "62634001",  # Aneurysmectomy with anastomosis of abdominal artery (procedure)
        "62665005",  # Abdominal aorta-iliac artery polyethylene terephthalate bypass graft (procedure)
        "62692004",  # Partial salpingectomy (procedure)
        "62726004",  # Fetal clavicotomy (procedure)
        "62807001",  # Incision and exploration of colon (procedure)
        "62820003",  # Correction of tetralogy of Fallot, one-stage (procedure)
        "62858004",  # Operation on pericardium (procedure)
        "62937006",  # Feeding enterostomy of jejunum (procedure)
        "62990005",  # Repair of chordae tendineae (procedure)
        "63003009",  # Removal of foreign body from large intestine by incision (procedure)
        "63016009",  # Total resection of urinary bladder (procedure)
        "63077009",  # Implantation of blood vessels into myocardium (procedure)
        "63186001",  # Radical mastoidectomy (procedure)
        "63187005",  # Myomectomy of broad ligament (procedure)
        "63296004",  # Excision of aorta (procedure)
        "63361002",  # Closure of fistula of urinary bladder (procedure)
        "6337001",  # Ligation of aortic arch (procedure)
        "63372007",  # Direct thrombectomy of iliac vein by abdominal incision (procedure)
        "63377001",  # Open core needle biopsy of liver (procedure)
        "63499008",  # Repair of tracheoesophageal fistula by cervical approach (procedure)
        "63516002",  # Vaginal hysterectomy with colpo-urethrocystopexy, Pereyra type (procedure)
        "63560002",  # Direct thrombectomy of vena cava by abdominal incision (procedure)
        "63586007",  # Thrombectomy with catheter of celiac artery by abdominal incision (procedure)
        "63596003",  # Laparoscopic treatment of ectopic pregnancy with salpingectomy (procedure)
        "63697000",  # Cardiopulmonary bypass operation (procedure)
        "63699002",  # Renoportal anastomosis (procedure)
        "63731008",  # Creation of aortocarotid shunt (procedure)
        "63816008",  # Hepatectomy, total left lobectomy (procedure)
        "63933000",  # Ventriculostomy (procedure)
        "63970009",  # Resection of upper limb artery with anastomosis (procedure)
        "64063003",  # Radical cystectomy (procedure)
        "640961000124109",  # MAKOplasty of knee joint (procedure)
        "64118002",  # Repair of aneurysm with graft of splenic artery (procedure)
        "64119005",  # Marsupialization of cyst of kidney (procedure)
        "64122007",  # Intracranial gas endarterectomy (procedure)
        "64129003",  # Excision of lesion of thoracoabdominal aorta with end-to-end anastomosis (procedure)
        "64210000",  # Division of papillary muscle of heart (procedure)
        "64215005",  # Construction of mucous fistula (procedure)
        "64227008",  # Thrombectomy with catheter of iliac vein by leg incision (procedure)
        "64289008",  # Excision of tumor of stomach (procedure)
        "64295009",  # Arthroplasty of knee, femoral condyles (procedure)
        "6433003",  # Closed heart valvotomy of mitral valve (procedure)
        "64372002",  # Common hepatic-common iliac-renal vascular bypass (procedure)
        "64608008",  # Excision of ascending aorta (procedure)
        "64622008",  # Thrombectomy with catheter of femoropopliteal vein by leg incision (procedure)
        "64656002",  # Ventriculomyocardiotomy (procedure)
        "64670002",  # Embolectomy with catheter of aortoiliac artery by leg incision (procedure)
        "647000",  # Excision of cervix by conization using electrical energy (procedure)
        "6471000179103",  # Transplantation of kidney and pancreas (procedure)
        "64744006",  # Open biopsy of jejunum (procedure)
        "64784002",  # Thrombectomy and repair of arterial graft (procedure)
        "64844000",  # Transection of cranial nerve (procedure)
        "64861002",  # Talma-Morison operation, omentopexy (procedure)
        "64887002",  # Operation on ovary (procedure)
        "64915003",  # Operation on heart (procedure)
        "64928005",  # Closure of rectovaginal fistula (procedure)
        "64933009",  # Thromboendarterectomy with graft of iliac artery (procedure)
        "64961003",  # Excision of cusp of mitral valve (procedure)
        "6501000179109",  # Laparoscopic biopsy of transplanted kidney (procedure)
        "65023008",  # Arthroplasty of patella (procedure)
        "65239007",  # Endometrectomy of urinary bladder (procedure)
        "65291007",  # Total resection of large intestine (procedure)
        "65310005",  # Vagotomy (procedure)
        "65347005",  # Donor cardiectomy with preparation and maintenance of homograft (procedure)
        "6536009",  # Implantation of tricuspid valve with tissue graft (procedure)
        "65468005",  # Hemilaminectomy for decompression and exploration (procedure)
        "65542000",  # Excision of uterosacral ligament (procedure)
        "65551008",  # Radical retropubic prostatectomy (procedure)
        "6561000179108",  # Open nephrectomy from live donor (procedure)
        "6567006",  # Pulmonary valve commissurotomy by transvenous balloon method (procedure)
        "65725008",  # Creation of lumbar subarachnoid-pleural shunt with laminectomy (procedure)
        "65745000",  # Partial hypophysectomy (procedure)
        "65788008",  # Closure of gastrocolic fistula (procedure)
        "65815005",  # Aneurysmectomy with anastomosis of upper limb artery (procedure)
        "65861005",  # Excision of aneurysm of coronary artery (procedure)
        "65886001",  # Excision of petrous apex cells (procedure)
        "6591000179103",  # Laparoscopic biopsy of transplanted pancreas (procedure)
        "65954005",  # Hepaticocystoduodenostomy (procedure)
        "65960005",  # Closure of gastrojejunocolic fistula (procedure)
        "65975002",  # Thrombectomy of head and neck artery (procedure)
        "66170004",  # Resection of descending aorta with anastomosis (procedure)
        "66245006",  # Radical vulvectomy with inguinofemoral lymphadenectomy (procedure)
        "66343001",  # Excision of lesion of small intestine (procedure)
        "66369007",  # Aneurysmectomy of head and neck artery (procedure)
        "6639006",  # Replacement of obstructed valve in shunt system (procedure)
        "66399003",  # Take-down of cerebral ventricular shunt (procedure)
        "66478002",  # Total laryngectomy (procedure)
        "66545007",  # Repair of tracheoesophageal fistula by thoracic approach (procedure)
        "66605009",  # Cardiopneumopexy (procedure)
        "6661001",  # King-Steelquist hindquarter operation (procedure)
        "66626009",  # Forequarter amputation, left (procedure)
        "66633009",  # Nephrocystanastomosis (procedure)
        "66681002",  # Subdural-peritoneal shunt with valve (procedure)
        "66706000",  # Repair of pituitary gland (procedure)
        "66740000",  # Coloplication (procedure)
        "66786003",  # Closure of foramen ovale with prosthesis by open heart technique (procedure)
        "66896000",  # Esophagoileostomy with interposition of small bowel (procedure)
        "66943005",  # Ureterectomy with urinary bladder cuff (procedure)
        "66951008",  # Carotid endarterectomy (procedure)
        "66979002",  # Excision of upper limb artery (procedure)
        "67018009",  # Cineplastic amputation of upper extremity (procedure)
        "6704000",  # Repair of parasternal diaphragmatic hernia (procedure)
        "67113002",  # Closure of vesicometrorectal fistula (procedure)
        "67119003",  # Intracranial aneurysmectomy with graft replacement (procedure)
        "67160005",  # Empyemectomy with lobectomy (procedure)
        "67166004",  # Aortocoronary artery bypass graft (procedure)
        "67199002",  # Exploration of adrenal gland with biopsy by transabdominal approach (procedure)
        "67212000",  # Ileocystoplasty (procedure)
        "6726004",  # Incision and drainage of appendiceal abscess by transabdominal approach (procedure)
        "67358002",  # Thrombectomy of lower limb artery (procedure)
        "67389003",  # Diverticulectomy of duodenum (procedure)
        "67399008",  # Ligation of anomalous coronary artery (procedure)
        "67402009",  # Partial lobectomy of brain (procedure)
        "67460004",  # Hypophyseal infundibulectomy (procedure)
        "67476008",  # Repair of inguinal hernia with orchiectomy (procedure)
        "67512004",  # Epicardiectomy (procedure)
        "67562009",  # Heterotransplant of pancreas (procedure)
        "67590008",  # Excision of trabeculae carneae cordis (procedure)
        "6759001",  # Esophagojejunostomy by thoracic approach (procedure)
        "67635009",  # Arteriectomy of abdominal aorta (procedure)
        "67706006",  # Pancreatolithotomy (procedure)
        "67749007",  # Total excision of pituitary gland by transfrontal approach (procedure)
        "6776002",  # Auricular aneurysmectomy (procedure)
        "67800005",  # Acoustic neurotomy (procedure)
        "67860007",  # Salpingo-oophorotomy (procedure)
        "67864003",  # Trephination of cranium (procedure)
        "67914009",  # Division of hypophyseal stalk (procedure)
        "68065001",  # Jejunostomy, delayed opening (procedure)
        "68082004",  # Thrombectomy with catheter of iliac vein by abdominal and leg incision (procedure)
        "68087005",  # Aneurysmectomy with graft replacement of abdominal artery (procedure)
        "680991000124106",  # Disarticulation of left ankle (procedure)
        "68111003",  # Cardioschisis (procedure)
        "68188002",  # Incision of large intestine (procedure)
        "68234001",  # Repair of ovary and fallopian tube (procedure)
        "68247006",  # Suture of ovary (procedure)
        "68255004",  # Open heart valvuloplasty of tricuspid valve without replacement (procedure)
        "68257007",  # Repair of ruptured aneurysm with graft of splenic artery (procedure)
        "68287001",  # Closed atrial septostomy (procedure)
        "68316003",  # Transplantation of pineal gland (procedure)
        "68324008",  # Thromboendarterectomy with graft of femoral, popliteal and tibioperoneal arteries (procedure)
        "6833009",  # Closed osteotomy of mandibular angle (procedure)
        "68342003",  # Ramstedt operation, pyloromyotomy with wedge resection (procedure)
        "68348004",  # Operation on rectum and perirectal tissue (procedure)
        "6846004",  # Incision of pituitary gland (procedure)
        "68645009",  # Embolectomy with catheter of aortoiliac artery by abdominal incision (procedure)
        "68731005",  # Decortication of cerebral meninges (procedure)
        "68746003",  # Stereotactic operation on thalamus (procedure)
        "68800008",  # Creation of conduit of right ventricle and pulmonary artery in repair of transposition of great vessels (procedure)
        "68860005",  # Percutaneous valvuloplasty (procedure)
        "68882003",  # Replacement of electronic heart device, myocardial electrode (procedure)
        "68904003",  # Local destruction of renal tissue (procedure)
        "68911004",  # Duodenojejunostomy (procedure)
        "68960009",  # Closure of enterovesical fistula with bowel and urinary bladder resection (procedure)
        "69036001",  # Total pancreaticoduodenectomy (procedure)
        "69051005",  # Repair of pulmonary heart valve with prosthesis in total repair of tetralogy of Fallot (procedure)
        "69058004",  # Conversion of rectal position (procedure)
        "69086007",  # Curettage of brain (procedure)
        "69191006",  # Incision of intestine (procedure)
        "69220007",  # Repair of atrium (procedure)
        "69455007",  # Repair of diaphragm (procedure)
        "69483009",  # Shunt of cerebral ventricle to extracranial site (procedure)
        "69489008",  # Thromboendarterectomy with graft of subclavian artery by neck incision (procedure)
        "69538006",  # Excision of diverticulum of ampulla of Vater (procedure)
        "69572003",  # Glossopharyngeal neurotomy (procedure)
        "69579007",  # Closed transventricular pulmonary valve commissurotomy (procedure)
        "69628009",  # Resection of alveolar process and palate, en bloc (procedure)
        "69685005",  # Repair of tricuspid valve with prosthesis (procedure)
        "69727009",  # Cochlear device implantation with mastoidectomy (procedure)
        "69762004",  # Excision of lesion of kidney (procedure)
        "69788001",  # Tractotomy of medulla oblongata (procedure)
        "698413006",  # Endoscopic electrohydraulic lithotripsy of duodenal bezoar (procedure)
        "698496004",  # Patch repair of perforated duodenal ulcer (procedure)
        "698642008",  # Percutaneous closure of patent ductus arteriosus (procedure)
        "698643003",  # Balloon dilation of coarctation of aorta (procedure)
        "69882005",  # Thrombectomy with catheter of femoropopliteal vein by abdominal and leg incision (procedure)
        "698869007",  # Laparoscopic bilateral total nephrectomy (procedure)
        "698973009",  # Creation of programmable cystoperitoneal shunt of brain (procedure)
        "698984002",  # Femoral-popliteal artery bypass using autogenous vein graft (procedure)
        "698986000",  # Femoro-distal popliteal artery bypass with autogenous vein graft (procedure)
        "698992006",  # Emergency repair of ruptured infrarenal abdominal aortic aneurysm (procedure)
        "698994007",  # Emergency repair of ruptured suprarenal abdominal aortic aneurysm (procedure)
        "698995008",  # Elective repair of infrarenal abdominal aortic aneurysm (procedure)
        "698996009",  # Elective repair of suprarenal abdominal aortic aneurysm (procedure)
        "699078008",  # Repair of atrioventricular septal defect with atrial and ventricular components (procedure)
        "699114003",  # Endovascular repair of abdominal aortic aneurysm using fluoroscopic guidance with contrast (procedure)
        "699115002",  # Endovascular repair of thoracic aortic aneurysm using fluoroscopic guidance with contrast (procedure)
        "699123000",  # Removal of single-chamber cardiac pacemaker with replacement by dual-chamber cardiac pacemaker (procedure)
        "699131005",  # Incision of bone of cranium and repair of intracranial aneurysm using intracranial approach (procedure)
        "699139007",  # Femoro-dorsalis pedis artery bypass (procedure)
        "699140009",  # Creation of aorto-common carotid artery shunt (procedure)
        "699141008",  # Bypass of artery of lower limb through obturator canal (procedure)
        "699156004",  # Excision arthroplasty of joint of knee (procedure)
        "699198001",  # Conversion to total prosthetic replacement of shoulder joint (procedure)
        "69926006",  # Closure of vesicosigmoidal fistula (procedure)
        "699279007",  # Replacement of aortic arch (procedure)
        "699280005",  # Repair of aortic root (procedure)
        "699347000",  # Mechanical heart valve replacement (procedure)
        "699352005",  # Repair of ascending aorta (procedure)
        "699385002",  # Repair of dissection of proximal thoracic aorta (procedure)
        "699386001",  # Repair of dissection of distal thoracic aorta (procedure)
        "699527007",  # Reimplantation of adrenal tissue (procedure)
        "699558008",  # Laparoscopic ureteroileostomy (procedure)
        "699570007",  # Laparoscopic ureteroileostomy with urinary diversion (procedure)
        "699667004",  # Endovascular repair of abdominal aortic aneurysm using endovascular stent graft (procedure)
        "699721006",  # Laparoscopic construction of cutaneous ureteroileostomy (procedure)
        "699726001",  # Pial synangiosis (procedure)
        "699727005",  # Laparoscopic reconstruction of ureteropelvic junction by retroperitoneal approach (procedure)
        "699729008",  # Encephalo-duro-arterio-synangiosis (procedure)
        "699730003",  # Encephalo-duro-arterio-myo-synangiosis (procedure)
        "699789005",  # Vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "699796007",  # Creation of cystoperitoneal shunt (procedure)
        "70015005",  # Incision of space of Retzius (procedure)
        "70018007",  # Incision of adrenal gland (procedure)
        "700196009",  # Extracorporeal resection of liver (procedure)
        "70031002",  # Oophorostomy (procedure)
        "700418003",  # Hand-assisted laparoscopic repair of aneurysm of abdominal aorta (procedure)
        "700499007",  # Implantation of extra-articular partial load absorber device to knee joint (procedure)
        "701000087108",  # Bilateral nephrostomy using fluoroscopic guidance (procedure)
        "70110002",  # Plication of small intestine (procedure)
        "70131004",  # Intraoperative transluminal angioplasty of iliac artery (procedure)
        "70172002",  # Ileoanal anastomosis (procedure)
        "70243005",  # Anastomosis of colon to rectum (procedure)
        "702621000",  # Thrombolysis of intracranial vessel using fluoroscopic guidance with contrast (procedure)
        "702625009",  # Excision of muscle of left ventricle interventricular septum (procedure)
        "70263003",  # Closure of perforation of esophagus (procedure)
        "703518004",  # Rastelli creation of conduit between right ventricle and pulmonary artery (procedure)
        "703519007",  # Repair of double outlet right ventricle with anastomosis of left ventricle to aorta via ventricular septal defect and right ventricle to pulmonary trunk conduit (procedure)
        "70377003",  # Closure of gastrojejunostomy (procedure)
        "704115002",  # Revision of cardiac pacemaker electrode using fluoroscopic guidance (procedure)
        "704140009",  # Reimplantation of celiac artery (procedure)
        "704142001",  # Reimplantation of inferior mesenteric artery (procedure)
        "705015009",  # Endovascular repair of aneurysm of bifurcation of aorta using endovascular stent graft (procedure)
        "70536003",  # Transplant of kidney (procedure)
        "70553005",  # Fistulization for ileus (procedure)
        "70586009",  # Operation on brain (procedure)
        "70589002",  # Removal of epicardial electrodes with replacement of epicardial lead (procedure)
        "70627009",  # Myocardial resection (procedure)
        "70676003",  # Esophagogastrostomy (procedure)
        "707813002",  # Percutaneous transluminal balloon dilation of cardiac conduit (procedure)
        "707832008",  # Percutaneous transluminal ablation of atrial wall for atrial flutter (procedure)
        "707835005",  # Hemicraniotomy (procedure)
        "708589000",  # Laparoscopic resection of segment of liver (procedure)
        "708590009",  # Laparoscopic wedge resection of liver (procedure)
        "708627007",  # Laparoscopic excision of small intestine (procedure)
        "708629005",  # Laparoscopic bypass gastrojejunostomy (procedure)
        "708631001",  # Laparoscopic-assisted low anterior resection of rectum (procedure)
        "708641003",  # Thoracoscopic repair of congenital diaphragmatic hernia (procedure)
        "708642005",  # Laparoscopic fixation of rectum using robotic assistance (procedure)
        "708643000",  # Laparoscopic proximal subtotal gastrectomy using robotic assistance (procedure)
        "708645007",  # Laparoscopic closure of ulcer of duodenum using patch (procedure)
        "708646008",  # Laparotomy with biopsy of peritoneum (procedure)
        "708647004",  # Laparotomy with biopsy of omentum (procedure)
        "708648009",  # Laparotomy with biopsy of pancreas (procedure)
        "708729004",  # Extensive debulking of neoplasm of abdominal cavity (procedure)
        "708730009",  # Open incision of liver (procedure)
        "708743009",  # Thoracoscopic biopsy of pericardium (procedure)
        "708747005",  # Laparoscopic biopsy of ovary (procedure)
        "708816008",  # Laparoscopic bilateral salpingo-oophorectomy using robotic assistance (procedure)
        "708818009",  # Laparoscopic wedge resection of liver using robotic assistance (procedure)
        "708860008",  # Percutaneous transluminal angioplasty of superior vena cava (procedure)
        "708875000",  # Percutaneous radiofrequency ablation of lesion of adrenal gland (procedure)
        "708876004",  # Laparoscopic appendectomy using robotic assistance (procedure)
        "708877008",  # Laparoscopic total hysterectomy using robotic assistance (procedure)
        "708878003",  # Laparoscopic radical hysterectomy using robotic assistance (procedure)
        "708884000",  # Excision of emphysematous bleb of lung (procedure)
        "708888002",  # Laparoscopic para-aortic lymph node dissection using robotic assistance (procedure)
        "708889005",  # Laparoscopic segmentectomy of liver using robotic assistance (procedure)
        "708908003",  # Laparoscopic live donor nephrectomy (procedure)
        "708913004",  # Laparoscopic abdominoperineal resection using robotic assistance (procedure)
        "708918008",  # Laparoscopic pyeloplasty using robotic assistance (procedure)
        "708919000",  # Laparoscopic radical prostatectomy using robotic assistance (procedure)
        "708920006",  # Laparoscopic anterior resection of rectum using robotic assistance (procedure)
        "708927009",  # Laparoscopic excision of diverticulum of urinary bladder using robotic assistance (procedure)
        "708928004",  # Laparoscopic radical cystectomy using robotic assistance (procedure)
        "708929007",  # Laparoscopic partial excision of kidney using robotic assistance (procedure)
        "708966001",  # Percutaneous removal of cardiac pacemaker electrode (procedure)
        "708968000",  # Endovascular thrombectomy of abdominal vein (procedure)
        "708969008",  # Endovascular thrombectomy of inferior vena cava (procedure)
        "708984004",  # Laparoscopic highly selective vagotomy (procedure)
        "708985003",  # Laparoscopic vaginal hysterectomy using robotic assistance (procedure)
        "708987006",  # Laparoscopic total gastrectomy (procedure)
        "708993003",  # Removal of foreign body from peritoneal cavity (procedure)
        "709020001",  # Open microwave ablation of lesion of liver (procedure)
        "709061001",  # Cervical esophagogastrostomy using retrosternal route (procedure)
        "7091000179107",  # Incision of thoracic wall and diaphragm (procedure)
        "709112001",  # Creation of pleuroperitoneal shunt (procedure)
        "709113006",  # Opening of chest and insertion of pleural tube drain (procedure)
        "709192002",  # Local excision of lesion of duodenum (procedure)
        "709217002",  # Excision of lamina of thoracic vertebra (procedure)
        "709421007",  # Colonoscopy and dilation of stricture of colon (procedure)
        "709425003",  # Thoracoscopic truncal vagotomy (procedure)
        "709857008",  # Repair of aneurysm of aorta using fluoroscopic guidance with contrast (procedure)
        "709860001",  # Angioplasty of superior vena cava using fluoroscopic guidance with contrast (procedure)
        "709861002",  # Angioplasty of abdominal aorta using fluoroscopic guidance with contrast (procedure)
        "71000002",  # Thrombectomy with catheter of subclavian vein by arm incision (procedure)
        "710006",  # Thromboendarterectomy with graft of mesenteric artery (procedure)
        "71007004",  # Thrombectomy of venous graft (procedure)
        "710074006",  # Angioplasty of abdominal aorta, iliac artery and femoral artery using fluoroscopic guidance with contrast (procedure)
        "710163000",  # Fluoroscopic angiography of artery of abdomen with contrast and insertion of stent (procedure)
        "71071004",  # Smith operation, open osteotomy of mandible (procedure)
        "710788003",  # Biopsy of pericardium by subxiphoid approach (procedure)
        "71102008",  # Revision of gastric anastomosis with jejunal interposition (procedure)
        "711166001",  # Removal of implantable cardiac pacemaker (procedure)
        "711202009",  # Conversion to prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "711204005",  # Conversion to prosthetic hybrid total replacement of shoulder joint using cemented humeral component (procedure)
        "711205006",  # Conversion to prosthetic hybrid total replacement of shoulder joint using cemented glenoid component (procedure)
        "711209000",  # Insertion of stent into intracranial vascular structure using fluoroscopic guidance with contrast (procedure)
        "711221001",  # Angioplasty of iliac artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "711229004",  # Laparoscopic distal subtotal gastrectomy (procedure)
        "711363000",  # Angioplasty of renal artery and insertion of stent using fluoroscopic guidance with contrast (procedure)
        "711369001",  # Stripping of diaphragm (procedure)
        "71140006",  # Release of median arcuate ligament (procedure)
        "711411006",  # Allotransplantation of kidney from beating heart cadaver (procedure)
        "711413009",  # Allotransplantation of kidney from non-beating heart cadaver (procedure)
        "711415002",  # Video assisted thoracoscopy for major lung resection (procedure)
        "71145001",  # Suture of esophagus (procedure)
        "71198003",  # Aorto-iliac thromboendarterectomy (procedure)
        "71212008",  # Manipulation of adrenal gland (procedure)
        "71213003",  # Arthrotomy of sternoclavicular joint for infection with drainage (procedure)
        "71221009",  # Transfer of iliopsoas to greater trochanter (procedure)
        "71225000",  # Excision of uterine ligament (procedure)
        "712538004",  # Radiofrequency ablation of renal cyst using fluoroscopic guidance (procedure)
        "712539007",  # Radiofrequency ablation of cyst of liver using fluoroscopic guidance (procedure)
        "71273005",  # Partial hepatectomy (procedure)
        "712815006",  # Endoscopic closure of fistula of small intestine (procedure)
        "712880008",  # Excision of proximal stomach using laparoscope (procedure)
        "712983009",  # Laparoscopic esophagectomy and gastric mobilization (procedure)
        "712992007",  # Laparoscopic low anterior resection of rectum using robotic assistance (procedure)
        "712995009",  # Esophagogastroduodenoscopy and closure of fistula of esophagus (procedure)
        "712997001",  # Esophagogastroduodenoscopy and closure of fistula of stomach (procedure)
        "712998006",  # Laparoscopic radical nephrectomy using robotic assistance (procedure)
        "713000002",  # Endoscopic submucosal dissection of rectum using sigmoidoscope (procedure)
        "713044004",  # Removal of cardiac pacemaker using fluoroscopic guidance (procedure)
        "713046002",  # Open reduction of intussusception of intestine (procedure)
        "713077000",  # Removal of automatic cardiac defibrillator using fluoroscopic guidance (procedure)
        "71308004",  # Thrombectomy of pulmonary artery (procedure)
        "713154003",  # Endoscopic submucosal dissection of rectum using colonoscope (procedure)
        "713157005",  # Laparoscopic resection of uterine cornua (procedure)
        "713160003",  # Excision of exostosis of skull bone (procedure)
        "713165008",  # Laparoscopic total colectomy with ileo-rectal anastomosis (procedure)
        "713170001",  # Percutaneous transluminal balloon angioplasty with insertion of stent into iliac artery (procedure)
        "713174005",  # Laparoscopic excision of pelvic endometriosis (procedure)
        "713285002",  # Prophylactic oophorectomy (procedure)
        "71335008",  # Thrombectomy with catheter of axillary vein by arm incision (procedure)
        "71346006",  # Augmentation of outflow tract in total repair of tetralogy of Fallot (procedure)
        "713519000",  # Partial lobectomy of liver (procedure)
        "713541000",  # Removal of entire genital organ (procedure)
        "713592008",  # Retrograde cutting balloon incision of renal pelvis using fluoroscopic guidance (procedure)
        "713617008",  # Percutaneous transluminal revascularization of chronic total occlusion of coronary artery using fluoroscopic guidance with contrast (procedure)
        "713685008",  # Reconstruction of joint of knee with external fixation (procedure)
        "713686009",  # Partial hip replacement with bipolar prosthesis (procedure)
        "713687000",  # Total replacement of joint of knee with constrained condylar prosthesis (procedure)
        "713689002",  # Repair of coronary artery (procedure)
        "71382001",  # Creation of cutaneoperitoneal fistula (procedure)
        "714031000",  # Single port laparoscopic nephrectomy (procedure)
        "71492005",  # Fenestration of cardiac valve (procedure)
        "7151000179104",  # Excision of benign neoplasm of esophagus (procedure)
        "71523008",  # Repair of ostium secundum defect with prosthesis by open heart technique (procedure)
        "71543002",  # Fenestration of aorta (procedure)
        "715761006",  # Reopening of chest and re-exploration of intrathoracic operation site (procedure)
        "715765002",  # Resection of ascending thoracic aorta using prosthetic graft with cardiopulmonary bypass (procedure)
        "715805005",  # Laparoscopic sub-segmentectomy of liver (procedure)
        "715996000",  # Insertion of drain into pericardium using ultrasonographic guidance (procedure)
        "716558001",  # Insertion of tunneled peritoneal drain into peritoneal cavity using ultrasonographic guidance (procedure)
        "716748002",  # Percutaneous irreversible electroporation ablation of neoplasm of kidney using computed tomography guidance (procedure)
        "716756004",  # Percutaneous irreversible electroporation ablation of neoplasm of liver using computed tomography guidance (procedure)
        "71722001",  # Thalamotomy (procedure)
        "71743000",  # Single valve replacement with commissurotomy of two valves (procedure)
        "7177008",  # Horizontal osteotomy of mandible (procedure)
        "717764002",  # Cardiac septal myectomy (procedure)
        "71827004",  # Choledochojejunostomy (procedure)
        "718288003",  # Percutaneous transluminal angioplasty of trunk of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "718289006",  # Percutaneous transluminal angioplasty of banding of trunk of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "718314009",  # Insertion of stent into cystogastric fistula for drainage of pancreas using fluoroscopic guidance (procedure)
        "718683004",  # Percutaneous transluminal angioplasty of banding of pulmonary artery using fluoroscopic guidance with contrast (procedure)
        "719261002",  # Repair of perineal hernia using surgical mesh (procedure)
        "719262009",  # Repair of perineal hernia using surgical suture (procedure)
        "71947008",  # Homotransplant of pancreas (procedure)
        "719919003",  # Balloon dilation of colon using fluoroscopic guidance (procedure)
        "720583008",  # Transcatheter implantation of mitral valve using fluoroscopic guidance (procedure)
        "721056006",  # Amputation of left lower limb above knee (procedure)
        "721059004",  # Amputation of right lower limb above knee (procedure)
        "721060009",  # Amputation of left leg through tibia and fibula (procedure)
        "72106008",  # Excision of lesion of liver (procedure)
        "721061008",  # Amputation of right leg through tibia and fibula (procedure)
        "7216000",  # Aneurysmectomy with graft replacement of head and neck artery (procedure)
        "72173001",  # Thrombectomy with catheter of innominate artery by neck incision (procedure)
        "7218004",  # Thrombectomy with catheter of vena cava by abdominal incision (procedure)
        "72193009",  # Cardiosplenopexy (procedure)
        "722022003",  # Hysteroscopic bipolar electrosurgical excision of uterine fibroid (procedure)
        "722274008",  # Excision of neoplasm of kidney (procedure)
        "72261007",  # Resection of rectum with pelvic exenteration (procedure)
        "72283006",  # Suture of ileum (procedure)
        "723069003",  # Associating liver partition and portal vein ligation for staged hepatectomy (procedure)
        "723251002",  # Hemiarthroplasty of left shoulder (procedure)
        "723311002",  # Amputation of left foot (procedure)
        "723312009",  # Amputation of right foot (procedure)
        "723314005",  # Amputation of left upper extremity (procedure)
        "723315006",  # Amputation of right upper extremity (procedure)
        "723689001",  # Amputation of left and right leg through tibia and fibula (procedure)
        "723818004",  # Percutaneous endopyelotomy using fluoroscopic guidance (procedure)
        "723840008",  # Amputation of bilateral lower limbs above knee (procedure)
        "72386000",  # Closure of jejunal fistula (procedure)
        "72388004",  # Radical perineal prostatectomy with lymph node biopsy (procedure)
        "72396009",  # Thromboendarterectomy of upper limb artery (procedure)
        "7240008",  # Total laryngectomy with partial pharyngectomy and synchronous tracheostomy (procedure)
        "724198005",  # Amputation of left forearm through radius and ulna (procedure)
        "724199002",  # Amputation of right forearm through radius and ulna (procedure)
        "724200004",  # Amputation of left arm through humerus (procedure)
        "724201000",  # Amputation of right arm through humerus (procedure)
        "725368002",  # Repair of dissection of abdominal aorta (procedure)
        "725370006",  # Fenestration of dissection of thoracic aorta (procedure)
        "72540001",  # Destructive procedure of abdominal artery (procedure)
        "725466004",  # Irreversible electroporation ablation of neoplasm of pancreas (procedure)
        "72583007",  # Repair of diaphragmatic hernia by thoracic approach (procedure)
        "726011000",  # Placement of stent in coronary artery bypass graft (procedure)
        "726108003",  # Total excision of left parotid gland (procedure)
        "726109006",  # Total excision of right parotid gland (procedure)
        "726418000",  # Prosthetic unicompartmental arthroplasty of left knee (procedure)
        "726419008",  # Prosthetic unicompartmental arthroplasty of right knee (procedure)
        "726425007",  # Lobectomy of lower lobe of right lung (procedure)
        "726426008",  # Lobectomy of lower lobe of left lung (procedure)
        "726427004",  # Lobectomy of upper lobe of left lung (procedure)
        "726428009",  # Lobectomy of upper lobe of right lung (procedure)
        "726429001",  # Radical mastectomy of left breast (procedure)
        "726430006",  # Radical mastectomy of right breast (procedure)
        "726436000",  # Modified radical mastectomy of right breast (procedure)
        "726437009",  # Modified radical mastectomy of left breast (procedure)
        "726488007",  # Excision of left adrenal gland (procedure)
        "726489004",  # Excision of right adrenal gland (procedure)
        "726547009",  # Removal of transhepatic central venous catheter using fluoroscopic guidance with contrast (procedure)
        "726550007",  # Percutaneous transluminal angioplasty of hepatic vein using fluoroscopic guidance with contrast (procedure)
        "726555002",  # Percutaneous angioplasty of transplanted hepatic vein using fluoroscopic guidance with contrast (procedure)
        "72672007",  # Thromboendarterectomy with graft of iliofemoral artery (procedure)
        "72741000",  # Creation of lumbar subarachnoid-peritoneal shunt with laminectomy (procedure)
        "72749003",  # Closure of ureterocutaneous fistula (procedure)
        "72791001",  # Laryngectomy (procedure)
        "72816003",  # Excision of lesion of epicardium (procedure)
        "72912002",  # Revision of cerebral ventricular shunt (procedure)
        "72915000",  # Dilation of colon (procedure)
        "72971003",  # Gastrojejunostomy with vagotomy (procedure)
        "72988005",  # Direct thrombectomy of femoropopliteal vein by leg incision (procedure)
        "73006002",  # Cranial decompression, posterior fossa (procedure)
        "73019003",  # Incision of hematoma of space of Retzius (procedure)
        "73021008",  # Pyeloileocutaneous anastomosis (procedure)
        "73202009",  # Aortofemoral to popliteal vascular bypass (procedure)
        "732212008",  # Amputation of bilateral lower limbs (procedure)
        "732213003",  # Amputation of right lower limb (procedure)
        "732214009",  # Amputation of left lower limb (procedure)
        "73269003",  # Complete excision of vagina (procedure)
        "733426006",  # Prosthetic total arthroplasty of left shoulder (procedure)
        "733429004",  # Prosthetic total arthroplasty of right shoulder (procedure)
        "73354002",  # Thrombectomy with catheter of brachial artery by arm incision (procedure)
        "733591007",  # Reverse prosthetic total arthroplasty of left shoulder (procedure)
        "733592000",  # Reverse prosthetic total arthroplasty of right shoulder (procedure)
        "73470004",  # Hinging of mitral valve (procedure)
        "735015005",  # Complete excision of left lung (procedure)
        "735016006",  # Complete excision of right lung (procedure)
        "735027008",  # Excision of left testis (procedure)
        "735028003",  # Excision of right testis (procedure)
        "73509004",  # Partial colectomy with skin level colostomy (procedure)
        "735224005",  # Prosthetic unicompartmental arthroplasty of bilateral knees (procedure)
        "735243005",  # Creation of ventriculosubgaleal shunt (procedure)
        "73536002",  # Myotomy of colon (procedure)
        "735396004",  # Open biopsy of adrenal gland (procedure)
        "73543008",  # Open biopsy of large intestine (procedure)
        "73544002",  # Operation on heart valve (procedure)
        "73554003",  # Cardioplasty of stomach and esophagus (procedure)
        "73570001",  # Dupuytren shoulder disarticulation (procedure)
        "736132002",  # Radiofrequency rhizotomy of trigeminal nerve (procedure)
        "736295006",  # Total pharyngectomy with reconstruction using free flap (procedure)
        "7365008",  # Ligation of abdominal artery (procedure)
        "736612004",  # Open rhizotomy of trigeminal nerve (procedure)
        "736613009",  # Excision of lesion of base of skull with reconstruction (procedure)
        "736614003",  # Open biopsy of brain (procedure)
        "736616001",  # Local destruction of lesion of kidney (procedure)
        "736617005",  # Local excision of lesion of kidney (procedure)
        "736700005",  # Revision of internal cardiac defibrillator lead using fluoroscopic guidance (procedure)
        "736702002",  # Revision of biventricular permanent pacemaker lead using fluoroscopic guidance (procedure)
        "736751004",  # Total excision of lactiferous duct of left breast (procedure)
        "736752006",  # Total excision of lactiferous duct of right breast (procedure)
        "736758005",  # Laparoscopic excision of lateral segment of left lobe of liver (procedure)
        "736778002",  # Reconstruction of breast using myocutaneous flap with microvascular anastomosis (procedure)
        "736842003",  # Closure of enterovesicovaginal fistula (procedure)
        "73693002",  # Division of intracranial vein (procedure)
        "736962007",  # Bypass of four or more coronary arteries with prosthesis (procedure)
        "736963002",  # Bypass of one coronary artery with prosthesis (procedure)
        "736964008",  # Bypass of three coronary arteries with prosthesis (procedure)
        "736965009",  # Bypass of two coronary arteries with prosthesis (procedure)
        "736966005",  # Aortocoronary artery bypass of four or more coronary arteries with saphenous vein graft (procedure)
        "736967001",  # Aortocoronary artery bypass of one coronary artery with saphenous vein graft (procedure)
        "736968006",  # Aortocoronary artery bypass of three coronary arteries with saphenous vein graft (procedure)
        "736969003",  # Aortocoronary artery bypass of two coronary arteries with saphenous vein graft (procedure)
        "736970002",  # Allograft bypass of four or more coronary arteries (procedure)
        "736971003",  # Allograft bypass of one coronary artery (procedure)
        "736972005",  # Allograft bypass of three coronary arteries (procedure)
        "736973000",  # Allograft bypass of two coronary arteries (procedure)
        "736974006",  # Excision of middle lobe of right lung (procedure)
        "737011002",  # Revision of cardiac biventricular implantable cardioverter defibrillator lead using fluoroscopic guidance (procedure)
        "737085003",  # Percutaneous insertion of bioresorbable stent into coronary artery using fluoroscopic guidance (procedure)
        "737099004",  # Abdominal hysterectomy with sacrocolpopexy using mesh (procedure)
        "73759005",  # Duodenectomy (procedure)
        "737891000",  # Reimplantation of superior mesenteric artery (procedure)
        "7383000",  # Anastomosis of intrathoracic artery (procedure)
        "738551006",  # Right colectomy with stoma (procedure)
        "738552004",  # Partial resection of colon with stoma (procedure)
        "7388009",  # Closure of arterial graft-enteric fistula (procedure)
        "73886004",  # Roux-en-Y cholecystojejunostomy with jejunojejunostomy (procedure)
        "73910002",  # Incision and exploration of subdiaphragmatic space (procedure)
        "739128007",  # Total excision of left submandibular gland (procedure)
        "739129004",  # Total excision of right submandibular gland (procedure)
        "739130009",  # Total excision of right sublingual gland (procedure)
        "739131008",  # Total excision of left sublingual gland (procedure)
        "739139005",  # Excision of middle and lower lobe of right lung (procedure)
        "739141006",  # Excision of middle and upper lobe of right lung (procedure)
        "73921006",  # Repair of rectal prolapse by abdominal approach (procedure)
        "73953002",  # Closure of rectovesical fistula (procedure)
        "73966002",  # Hemicystectomy (procedure)
        "739669004",  # Supracervical hysterectomy with left salpingo-oophorectomy (procedure)
        "739670003",  # Supracervical hysterectomy with right salpingo-oophorectomy (procedure)
        "739671004",  # Total hysterectomy with left oophorectomy (procedure)
        "739672006",  # Total hysterectomy with right oophorectomy (procedure)
        "739673001",  # Total hysterectomy with left salpingo-oophorectomy (procedure)
        "739674007",  # Total hysterectomy with right salpingo-oophorectomy (procedure)
        "73978003",  # Multiple segmental resections of small intestine (procedure)
        "73996007",  # Proctopexy combined with sigmoid resection by abdominal approach (procedure)
        "74041008",  # Repair of tricuspid atresia (procedure)
        "740514001",  # Total hysterectomy with right salpingectomy (procedure)
        "740515000",  # Total hysterectomy with left salpingectomy (procedure)
        "741009001",  # Simple mastectomy of left breast (procedure)
        "741010006",  # Simple mastectomy of right breast (procedure)
        "7414004",  # Intraoperative transluminal aortic angioplasty (procedure)
        "74180003",  # Hepatectomy, trisegmentectomy (procedure)
        "74239001",  # Popliteal-peroneal artery bypass graft with vein (procedure)
        "74284004",  # Diverticulectomy of esophagus (procedure)
        "74371005",  # Coronary artery bypass with autogenous graft, two grafts (procedure)
        "74406002",  # Open heart valvuloplasty of mitral valve without replacement (procedure)
        "74531004",  # Partial bilateral adrenalectomy (procedure)
        "74539002",  # Intra-abdominal manipulation of large intestine (procedure)
        "74556005",  # Thromboendarterectomy (procedure)
        "74625006",  # Repair of supracristal defect with prosthesis (procedure)
        "74738008",  # Skin level cecostomy (procedure)
        "74770008",  # Exploratory laparotomy (procedure)
        "7495004",  # Stromeyer-Little operation hepatotomy (procedure)
        "74971002",  # Operation on rectum (procedure)
        "75078003",  # Repair of ventricular septal defect in total repair of tetralogy of Fallot (procedure)
        "751000087109",  # Angioplasty of bilateral renal arteries using fluoroscopic guidance with contrast (procedure)
        "75108001",  # Suture of rectum (procedure)
        "75113002",  # Ligation of adrenal vein (procedure)
        "75146008",  # Thromboendarterectomy with graft of innominate artery by thoracic incision (procedure)
        "75172004",  # Fistulization of cerebral subarachnoid space (procedure)
        "75240005",  # Cerebral ventricular shunt to venous system (procedure)
        "75312003",  # Partial colectomy with ileostomy and creation of mucofistula (procedure)
        "75389003",  # Cystogastrostomy of pancreas (procedure)
        "754000",  # Total urethrectomy (procedure)
        "75404005",  # Epiploectomy (procedure)
        "75409000",  # Implantation of outflow tract prosthesis in total repair of tetralogy of Fallot (procedure)
        "75420000",  # Excision of redundant mucosa from duodenostomy (procedure)
        "75479001",  # Transplantation of tissue of diaphragm (procedure)
        "75533008",  # Aneurysmectomy with graft replacement of thoracoabdominal aorta (procedure)
        "75596002",  # Destructive procedure on adrenal gland (procedure)
        "75601006",  # Excision of choroid plexus (procedure)
        "75626002",  # Pallidoamygdalotomy (procedure)
        "75835007",  # Laparoscopic-assisted vaginal hysterectomy (procedure)
        "75894009",  # Delorme operation, proctopexy (procedure)
        "75905004",  # Sigmoidoproctostomy (procedure)
        "75908002",  # Puncture and drainage of pituitary gland (procedure)
        "75916006",  # Arterial intra-abdominal bypass (procedure)
        "75924001",  # Urinary cystotomy for excision of bladder diverticulum (procedure)
        "75935006",  # Thoracoscopic lobectomy of lung (procedure)
        "75942006",  # Total correction of transposition of great vessels (procedure)
        "76017008",  # Gordon-Taylor hindquarter operation (procedure)
        "76025005",  # Correction of ventricular septal defect (procedure)
        "76056002",  # Radical dissection of bilateral sides of neck (procedure)
        "76062007",  # Duhamel operation, abdominoperineal pull-through (procedure)
        "76077004",  # Total pancreatectomy with transplantation (procedure)
        "76081004",  # Craniectomy for osteomyelitis (procedure)
        "76192001",  # Mikulicz operation, resection of large intestine, second stage (procedure)
        "762049002",  # Reimplantation of suprarenal artery (procedure)
        "762055007",  # Conversion to hybrid prosthetic replacement of shoulder joint using cemented component (procedure)
        "762225006",  # Endovascular insertion of stent into aortic bifurcation (procedure)
        "762625001",  # Vaginal hysterectomy and pelvic floor repair (procedure)
        "762809009",  # Partial left salpingectomy (procedure)
        "762810004",  # Partial right salpingectomy (procedure)
        "76315006",  # Punch resection of vocal cord (procedure)
        "763343001",  # Intraoperative insertion of cardiac pacemaker (procedure)
        "763471008",  # Fluoroscopic venography of renal vein using contrast with insertion of metallic stent graft (procedure)
        "76372001",  # Embolectomy of abdominal artery (procedure)
        "763725002",  # Percutaneous transluminal angioplasty of coronary artery using drug eluting balloon catheter (procedure)
        "76414006",  # Closure of urethrovesicovaginal fistula (procedure)
        "7642007",  # Endarterectomy of abdominal artery (procedure)
        "764386009",  # Removal of joint prosthesis for periprosthetic joint infection (procedure)
        "76443003",  # Total esophagectomy with gastropharyngostomy (procedure)
        "764677008",  # Selective internal radiotherapy of liver using yttrium (90-Y) labeled microspheres (procedure)
        "76468001",  # Radical mastectomy of bilateral breasts (procedure)
        "764738000",  # Percutaneous irreversible electroporation ablation of neoplasm of pancreas using computed tomography guidance (procedure)
        "764919003",  # Open repair of ventral hernia (procedure)
        "764920009",  # Open repair of obstructed ventral hernia (procedure)
        "764921008",  # Open repair of obstructed ventral hernia with prosthesis (procedure)
        "764922001",  # Open repair of strangulated ventral hernia (procedure)
        "764923006",  # Open repair of strangulated ventral hernia with prosthesis (procedure)
        "764931001",  # Revision of one component of total prosthetic replacement of knee joint not using cement (procedure)
        "764932008",  # Revision of one component of total prosthetic replacement of knee joint using cement (procedure)
        "76504000",  # Embolectomy of aortic arch (procedure)
        "765472003",  # Excision of left kidney (procedure)
        "765473008",  # Excision of right kidney (procedure)
        "765474002",  # Percutaneous cryotherapy of neoplasm of left kidney (procedure)
        "765475001",  # Percutaneous cryotherapy of neoplasm of right kidney (procedure)
        "765478004",  # Allotransplantation of left kidney (procedure)
        "765479007",  # Allotransplantation of right kidney (procedure)
        "76580007",  # Peripheral arterial bypass (procedure)
        "766744003",  # Incision and exploration of adrenal gland (procedure)
        "76681003",  # Thrombectomy with catheter of innominate artery by arm incision (procedure)
        "767335007",  # Excision of entire left testis by inguinal approach (procedure)
        "767336008",  # Excision of entire right testis by inguinal approach (procedure)
        "767337004",  # Excision of entire left testis and entire left spermatic cord (procedure)
        "767339001",  # Excision of entire right testis and entire right spermatic cord (procedure)
        "767341000",  # Excision of entire left testis by scrotal approach (procedure)
        "767342007",  # Excision of entire right testis by scrotal approach (procedure)
        "767343002",  # Excision of entire left testis (procedure)
        "767344008",  # Excision of entire right testis (procedure)
        "767345009",  # Left subcapsular orchidectomy (procedure)
        "767346005",  # Right subcapsular orchidectomy (procedure)
        "767347001",  # Excision of entire left undescended testis (procedure)
        "767348006",  # Excision of entire right undescended testis (procedure)
        "767610009",  # Total hysterectomy via vaginal approach (procedure)
        "767611008",  # Total abdominal hysterectomy using intrafascial technique (procedure)
        "767612001",  # Total hysterectomy via vaginal approach using intrafascial technique (procedure)
        "767751001",  # Laparoscopic adenomyomectomy (procedure)
        "76876009",  # Excision of bilateral ovaries (procedure)
        "768879004",  # Revision of hybrid prosthetic replacement of hip joint using cemented acetabular component (procedure)
        "76961002",  # Lobotomy and tractotomy (procedure)
        "76967003",  # Ileopancreatostomy (procedure)
        "76987002",  # Repair of aorta (procedure)
        "77018005",  # Excision of lesion of intracranial artery (procedure)
        "770606008",  # Total replacement of left hip joint (procedure)
        "77066003",  # Excision of hydatid of Morgagni in male (procedure)
        "7709002",  # Transfer of finger to opposite hand with amputation (procedure)
        "77100009",  # Reconstruction of ureteropelvic junction (procedure)
        "77109005",  # Direct closure of sinus venosus with anomalous pulmonary venous drainage (procedure)
        "771348000",  # Repair of supravalvar aortic stenosis (procedure)
        "771453009",  # Repair of abdominal aortic aneurysm with insertion of stent (procedure)
        "771557007",  # Replacement of cardiac biventricular implantable cardioverter defibrillator (procedure)
        "771580004",  # Excision of lesion of mucous membrane of lower bowel using fiberoptic sigmoidoscope (procedure)
        "77163008",  # Endarterectomy of aorta with patch graft (procedure)
        "77166000",  # Percutaneous balloon valvuloplasty of aortic valve (procedure)
        "771682001",  # Open repair of spigelian hernia using synthetic mesh (procedure)
        "771683006",  # Open repair of spigelian hernia using biological mesh (procedure)
        "771684000",  # Open repair of spigelian hernia using sutures (procedure)
        "771687007",  # Open repair of recurrent spigelian hernia using synthetic mesh (procedure)
        "771688002",  # Open repair of recurrent spigelian hernia using biological mesh (procedure)
        "771713005",  # Open repair of inguinal hernia using sutures (procedure)
        "771714004",  # Open repair of inguinal hernia using synthetic mesh (procedure)
        "771715003",  # Open repair of inguinal hernia using biological mesh (procedure)
        "771716002",  # Open repair of recurrent inguinal hernia using synthetic mesh (procedure)
        "771717006",  # Open repair of recurrent inguinal hernia using sutures (procedure)
        "771718001",  # Open repair of epigastric hernia using synthetic mesh (procedure)
        "771719009",  # Open repair of epigastric hernia using sutures (procedure)
        "771720003",  # Open repair of epigastric hernia using biological mesh (procedure)
        "771721004",  # Open repair of recurrent epigastric hernia using synthetic mesh (procedure)
        "771722006",  # Open repair of recurrent epigastric hernia using biological mesh (procedure)
        "77181006",  # Implantation of mitral valve prosthesis or synthetic device (procedure)
        "772044006",  # Amputation of entire lower limb and ipsilateral hemipelvis (procedure)
        "772057003",  # Amputation of entire left lower limb and ipsilateral hemipelvis (procedure)
        "772058008",  # Amputation of entire right lower limb and ipsilateral hemipelvis (procedure)
        "772099002",  # Repair of incisional hernia of abdominal wall (procedure)
        "772100005",  # Laparoscopic repair of incisional hernia of abdominal wall (procedure)
        "772101009",  # Repair of recurrent incisional hernia of abdominal wall (procedure)
        "77241005",  # Creation of ventriculo-auricular shunt (procedure)
        "773127003",  # Laparoscopic augmentation of lower esophageal sphincter using magnetic titanium ring (procedure)
        "773128008",  # Endoscopic snare resection of lesion of colon using full thickness resection device (procedure)
        "773173001",  # Angioplasty of thoracic artery using fluoroscopic guidance with contrast (procedure)
        "773254008",  # Laparoscopic repair of incisional hernia of anterior abdominal wall using biological mesh with anterior component separation (procedure)
        "773255009",  # Laparoscopic repair of recurrent incisional hernia of anterior abdominal wall using biological mesh with anterior component separation (procedure)
        "773256005",  # Laparoscopic repair of incisional hernia of anterior abdominal wall using synthetic mesh with anterior component separation (procedure)
        "773257001",  # Laparoscopic repair of recurrent incisional hernia of anterior abdominal wall using synthetic mesh with anterior component separation (procedure)
        "77326008",  # Removal of pacemaker electrode leads without replacement (procedure)
        "773285004",  # Open repair of incisional hernia of anterior abdominal wall using synthetic mesh with anterior component separation (procedure)
        "773286003",  # Open repair of incisional hernia of anterior abdominal wall using biological mesh with anterior component separation (procedure)
        "773287007",  # Open repair of recurrent incisional hernia of anterior abdominal wall using synthetic mesh with anterior component separation (procedure)
        "773288002",  # Open repair of recurrent incisional hernia of anterior abdominal wall using biological mesh with anterior component separation (procedure)
        "773311000",  # Open repair of incisional hernia of anterior abdominal wall using synthetic mesh with posterior component separation (procedure)
        "773312007",  # Open repair of recurrent incisional hernia of anterior abdominal wall using synthetic mesh with posterior component separation (procedure)
        "773313002",  # Open repair of incisional hernia of anterior abdominal wall using biological mesh with posterior component separation (procedure)
        "773314008",  # Open repair of recurrent incisional hernia of anterior abdominal wall using biological mesh with posterior component separation (procedure)
        "773407007",  # Open repair of recurrent incisional hernia of anterior abdominal wall using biological mesh with component separation (procedure)
        "77371000",  # Operation on cardiac septum (procedure)
        "773819005",  # Amputation of right forefoot (procedure)
        "773821000",  # Amputation of left forefoot (procedure)
        "773996000",  # Transcatheter aortic valve implantation (procedure)
        "774013003",  # High intensity focused ultrasound ablation of lesion of liver using magnetic resonance imaging guidance (procedure)
        "774014009",  # High intensity focused ultrasound ablation of lesion of kidney using magnetic resonance imaging guidance (procedure)
        "77442000",  # Removal of subarachnoid-peritoneal shunt (procedure)
        "77445003",  # Thromboendarterectomy with graft of common femoral artery (procedure)
        "77464009",  # Submucous resection of vocal cord (procedure)
        "77539008",  # Pancreaticocystoduodenostomy (procedure)
        "77570000",  # Major thoracotomy with removal of intrapleural foreign body (procedure)
        "77596003",  # Ileostomy, delayed opening (procedure)
        "77615009",  # Resection of aorta with anastomosis (procedure)
        "77648009",  # Excision of lesion of lung (procedure)
        "77651002",  # Destruction of intra-abdominal cysts (procedure)
        "77697000",  # Cryothalamectomy (procedure)
        "77871009",  # Total osteotomy of mandible (procedure)
        "77878003",  # Von Kraske operation, proctectomy (procedure)
        "77902002",  # Vaginal hysterectomy with partial colpectomy (procedure)
        "77959002",  # Suture of large intestine with colostomy (procedure)
        "78017003",  # Filleting of pancreas (procedure)
        "78033000",  # Lysis of periureteral adhesions (procedure)
        "78062004",  # Closure of vaginovesical fistula (procedure)
        "7813002",  # Esophagojejunostomy with interposition of small bowel (procedure)
        "78133002",  # Endoscopic excision of lesion of large intestine (procedure)
        "78136005",  # Total osteotomy of maxilla (procedure)
        "782391003",  # Revision of decompressive excision of lamina of lumbar vertebra (procedure)
        "7824000",  # Enlargement of pre-existing atrial septal defect (procedure)
        "782655004",  # Laparoscopic transplant of kidney using robotic assistance (procedure)
        "782657007",  # High intensity focused ultrasound ablation of thalamus using magnetic resonance imaging guidance (procedure)
        "78268008",  # Operation on trabeculae carneae cordis (procedure)
        "78283003",  # Wide excision of malignant lesion of cervical esophagus (procedure)
        "782836001",  # Laparoscopic ventral fixation of internal prolapse of rectum using mesh (procedure)
        "782837005",  # Laparoscopic ventral fixation of internal prolapse of rectum using biological mesh (procedure)
        "782838000",  # Laparoscopic ventral fixation of internal prolapse of rectum using synthetic mesh (procedure)
        "78297006",  # Transection of supraorbital nerve (procedure)
        "7833003",  # Endoscopic excision of lesion of pancreatic duct (procedure)
        "78418002",  # Operation on heart and pericardium (procedure)
        "784191009",  # Classic intrafascial supracervical hysterectomy (procedure)
        "785850002",  # Reverse prosthetic total arthroplasty of shoulder (procedure)
        "78585009",  # Transurethral electroresection of neck of urinary bladder (procedure)
        "78646006",  # Ileoentectropy (procedure)
        "78692009",  # Ligation of aorta (procedure)
        "787108001",  # Laparoscopic restorative proctocolectomy with ileal pouch anal anastomosis (procedure)
        "787109009",  # Excision of entire colon and entire rectum (procedure)
        "787159000",  # Amputation of left midfoot (procedure)
        "787161009",  # Amputation of right midfoot (procedure)
        "787162002",  # Percutaneous pulmonary valve implantation (procedure)
        "787399008",  # Amputation of bilateral feet (procedure)
        "787874000",  # Laparoscopic total colectomy (procedure)
        "787875004",  # Laparoscopic total colectomy and loop ileostomy (procedure)
        "787876003",  # Laparoscopic total colectomy and ileostomy (procedure)
        "78788008",  # Excision of cusp of aortic valve (procedure)
        "787938008",  # Partial resection of small bowel using robotic assistance (procedure)
        "78824001",  # Repair of traumatic diaphragmatic hernia, acute (procedure)
        "78861005",  # Donor hepatectomy with preparation and maintenance of homograft (procedure)
        "78878001",  # Radical vulvectomy with inguinofemoral, iliac and pelvic lymphadenectomy (procedure)
        "789003",  # Cranial decompression, subtemporal, supratentorial (procedure)
        "78905003",  # Removal of foreign body from pancreas (procedure)
        "78925004",  # Ileoileostomy (procedure)
        "789781007",  # Ureterorenoscopy with fragmentation and removal of calculus of kidney (procedure)
        "79025009",  # Thrombectomy of aortic arch (procedure)
        "7908000",  # Operation on ileum (procedure)
        "79093007",  # Craniotomy with treatment of penetrating wound of brain (procedure)
        "79095000",  # Complete excision of organ (procedure)
        "79105008",  # Removal of electronic heart device, myocardial electrode (procedure)
        "79170004",  # Ileoduodenotomy (procedure)
        "79182008",  # Posterior tibial artery bypass graft with vein (procedure)
        "79345008",  # Excision of maxillary torus palatinus (procedure)
        "7937001",  # Transplantation of adrenal gland (procedure)
        "79373005",  # Transposition procedure of small intestine (procedure)
        "79433000",  # Closure of colon fistula (procedure)
        "79507006",  # Carotid-vertebral artery bypass graft with vein (procedure)
        "79553004",  # Resection of abdominal aorta with replacement (procedure)
        "79632004",  # Disarticulation of toe (procedure)
        "79733001",  # Amputation above-knee (procedure)
        "79746006",  # Single valve replacement with commissurotomy of another valve (procedure)
        "79764004",  # Excision of lesion of rectum (procedure)
        "79765003",  # Diverticulectomy of esophagus by cervical approach (procedure)
        "79827002",  # Arteriovenous anastomosis for renal dialysis (procedure)
        "79896000",  # Manipulation of pericardium (procedure)
        "79917009",  # Implantation of pulmonary valve prosthesis or synthetic device (procedure)
        "80028005",  # Closure of choledochoduodenal fistula (procedure)
        "80067004",  # Excision of lesion of vocal cord (procedure)
        "80070000",  # Transfer of iliopsoas muscle (procedure)
        "80099005",  # Manipulation of pituitary gland (procedure)
        "80100002",  # Extradural transection of cranial nerve (procedure)
        "80102005",  # Creation of external-internal carotid bypass (procedure)
        "80146002",  # Excision of appendix (procedure)
        "80150009",  # Embolectomy with catheter of innominate artery by neck incision (procedure)
        "8020008",  # Arthroplasty of shoulder with synthetic joint prosthesis (procedure)
        "80206000",  # Orbitotomy with bone flap by lateral approach, Kroenlein, with decompression (procedure)
        "80215007",  # Grafting of pericardium (procedure)
        "80285004",  # Hill-Allison operation, diaphragmatic hernia repair (procedure)
        "80294005",  # Total abdominal colectomy with rectal mucosectomy and ileoanal anastomosis (procedure)
        "80319003",  # Cholecystojejunostomy (procedure)
        "80331001",  # Incision of coronary artery (procedure)
        "80345007",  # Beck-Jianu operation, permanent gastrostomy (procedure)
        "80362002",  # Excisional biopsy of pineal gland (procedure)
        "80377005",  # Aneurysmectomy of heart (procedure)
        "80443001",  # Excision of lesion of diaphragm (procedure)
        "80486001",  # Arthrotomy with open meniscus repair (procedure)
        "80513001",  # Partial laryngectomy (procedure)
        "80629001",  # Marsupialization of ovarian cyst (procedure)
        "80636000",  # Repair of heart atrial septum in total repair of truncus arteriosus (procedure)
        "80637009",  # Isolation of ileal loop (procedure)
        "80685005",  # Double valve replacement with commissurotomy of one valve (procedure)
        "8069005",  # Implantation of tricuspid valve prosthesis or synthetic device (procedure)
        "807005",  # Excision of brain (procedure)
        "8072003",  # Replacement of cardiac pacemaker with single chamber pacemaker not specified as rate responsive (procedure)
        "80733000",  # Closure of ventricular septal defect with patch (procedure)
        "80779005",  # Repair of brain (procedure)
        "80925005",  # Removal of cerebral ventricular reservoir, Ommaya or Rickham (procedure)
        "80945002",  # Cystoproctostomy (procedure)
        "80989002",  # Excision of lesion of pericardium (procedure)
        "80999007",  # Tube pericardiostomy (procedure)
        "8102001",  # Aneurysmectomy of aorta (procedure)
        "81092009",  # Transsacral rectosigmoidectomy (procedure)
        "81232004",  # Radical cystoprostatectomy (procedure)
        "81266008",  # Heart revascularization (procedure)
        "81338003",  # Partial excision of pineal gland (procedure)
        "81398000",  # Excision of chemodectoma of pineal gland (procedure)
        "81516001",  # Partial nephrectomy (procedure)
        "8155007",  # Excision of basal ganglion (procedure)
        "81569000",  # Selective vagotomy with pyloroplasty (procedure)
        "81697004",  # Division of inferior laryngeal nerve (procedure)
        "81714000",  # Ileorectostomy (procedure)
        "81731007",  # Excision of lesion of adrenal gland (procedure)
        "8174004",  # Roux-en-Y cholecystenterostomy (procedure)
        "81751006",  # Excision of intra-abdominal mass (procedure)
        "81762006",  # Double valve replacement (procedure)
        "81788009",  # Repair of aneurysm with graft of renal artery (procedure)
        "81849003",  # Pallidoansotomy (procedure)
        "81913003",  # Plication of diaphragm (procedure)
        "8194008",  # Closure of pyelocutaneous fistula (procedure)
        "819811000000101",  # Percutaneous microwave ablation of lesion of kidney using computed tomography guidance (procedure)
        "82068004",  # Closure of vaginoileal fistula (procedure)
        "8215004",  # Lisfranc shoulder disarticulation (procedure)
        "82158006",  # Petrous apicectomy including radical mastoidectomy (procedure)
        "82208009",  # Creation of spinal shunt with valve (procedure)
        "82213008",  # Aneurysmectomy of aorta with anastomosis (procedure)
        "8223002",  # Labyrinthectomy, osseous (procedure)
        "82247006",  # Coronary artery bypass with autogenous graft, five grafts (procedure)
        "82293009",  # Repair of esophageal stricture (procedure)
        "822991000",  # Plastic repair of aortic arch with tube graft (procedure)
        "82367000",  # Aneurysmectomy of ascending aorta with anastomosis (procedure)
        "82417006",  # Thrombectomy with catheter of subclavian artery by thoracic incision (procedure)
        "82421004",  # Excisional biopsy of pituitary gland by transsphenoidal approach (procedure)
        "82467004",  # Thromboendarterectomy of lower limb artery (procedure)
        "82487003",  # Subclavian-vertebral artery bypass graft with vein (procedure)
        "82531008",  # Open biopsy of liver (procedure)
        "82596000",  # Axillary-axillary artery bypass graft with vein (procedure)
        "82611002",  # Creation of aortosubclavian shunt (procedure)
        "82619000",  # Left colectomy (procedure)
        "82692008",  # Dilation of rectal stricture under nonlocal anesthesia (procedure)
        "82730006",  # Incidental excision of appendix (procedure)
        "82744000",  # Esophagogastrostomy by thoracic approach (procedure)
        "82762009",  # Ventricular shunt to circulatory system (procedure)
        "82764005",  # Operation on duodenum (procedure)
        "82839004",  # Panniculectomy (procedure)
        "82874003",  # Operation on colon (procedure)
        "82900007",  # Partial colectomy with coloproctostomy (procedure)
        "82917000",  # Reopening of craniotomy site (procedure)
        "82944005",  # Direct thrombectomy of vena cava by abdominal and leg incision (procedure)
        "830042005",  # Endovascular insertion of stent into intracranial vascular structure (procedure)
        "83101004",  # Laryngoesophagectomy (procedure)
        "83152002",  # Oophorectomy (procedure)
        "83187002",  # Closed heart valvotomy of pulmonary valve (procedure)
        "8324006",  # Incision and drainage of rectal abscess (procedure)
        "83340003",  # Excision of lesion of intracranial vein (procedure)
        "83371007",  # Proximal subtotal gastrectomy by abdominal approach (procedure)
        "83444008",  # Renal papillectomy (procedure)
        "8345004",  # Wedge biopsy of liver (procedure)
        "83467005",  # Repair of supracristal defect with tissue graft (procedure)
        "83471008",  # Laminectomy for implantation of neurostimulator electrodes, endodural (procedure)
        "83478002",  # Incision and exploration of kidney pelvis (procedure)
        "83483005",  # Closure of urethrovesical fistula (procedure)
        "83518006",  # Radical amputation of penis with bilateral inguinofemoral lymphadenectomy (procedure)
        "83538007",  # Aneurysmectomy of aortic arch with anastomosis (procedure)
        "83574003",  # Sorondo-Ferre hindquarter operation (procedure)
        "83599002",  # Incision of duodenum (procedure)
        "83613005",  # Cardiopericardiopexy (procedure)
        "836435007",  # Simple mastectomy of right breast using robotic assistance (procedure)
        "836436008",  # Simple mastectomy of bilateral breasts using robotic assistance (procedure)
        "836437004",  # Simple mastectomy of left breast using robotic assistance (procedure)
        "83662005",  # Thromboendarterectomy with graft of subclavian artery by thoracic incision (procedure)
        "83789005",  # Destruction of intra-abdominal tumor (procedure)
        "83809000",  # Renal calicectomy (procedure)
        "8382008",  # Creation of pulmonary-innominate artery shunt (procedure)
        "83857006",  # Gastroduodenectomy (procedure)
        "8388007",  # Partial horizontal laryngectomy (procedure)
        "83948001",  # Repair of spinal pseudomeningocele with laminectomy (procedure)
        "83985009",  # Resection of stomach with gastrojejunal anastomosis (procedure)
        "84042007",  # Cystourethroscopy with resection of external sphincter of urinary bladder (procedure)
        "84064003",  # Hysteroscopy with removal of submucous leiomyomata (procedure)
        "84125003",  # Removal of foreign body from pericardium (procedure)
        "84126002",  # Closure of vulvorectal fistula (procedure)
        "84179007",  # Thrombectomy of descending aorta (procedure)
        "84257008",  # Open biopsy of ileum (procedure)
        "84262009",  # Scott operation, intestinal bypass for obesity (procedure)
        "84343009",  # Partial antero-latero-vertical laryngectomy (procedure)
        "84354004",  # Closure of tracheoesophageal fistula (procedure)
        "8444004",  # Removal of gallstones from liver (procedure)
        "84456001",  # Saucerization of rectum (procedure)
        "84510006",  # Temporary gastrostomy (procedure)
        "84539006",  # Forequarter amputation (procedure)
        "84552001",  # Resection of aortic arch with anastomosis (procedure)
        "84575008",  # Urinary bladder trigonectomy (procedure)
        "84594003",  # Gas endarterectomy of aorta (procedure)
        "84604002",  # Sigmoid colectomy (procedure)
        "84824001",  # Pemberton operation, rectal prolapse repair (procedure)
        "84840003",  # Total lobectomy of lung with concomitant decortication (procedure)
        "84949001",  # Removal of spinal thecal shunt (procedure)
        "84952009",  # Resection of colon for interposition (procedure)
        "85041007",  # Formation of conduit of right ventricle and pulmonary artery in repair of pulmonary artery atresia in truncus arteriosus (procedure)
        "85088006",  # Bypass of carotid artery to carotid artery using vein graft (procedure)
        "85154003",  # Total laryngectomy with radical neck dissection (procedure)
        "85162006",  # Embolectomy with catheter of innominate artery by thoracic incision (procedure)
        "85175006",  # Partial cystectomy, complicated (procedure)
        "85195000",  # Removal of foreign body of renal pelvis by incision (procedure)
        "85217002",  # Distal subtotal gastrectomy (procedure)
        "85290004",  # Thrombectomy with catheter of iliac vein by abdominal incision (procedure)
        "85435009",  # Suture of intestine (procedure)
        "85448008",  # Thromboendarterectomy with graft of vertebral artery by neck incision (procedure)
        "8550006",  # Partial bilateral salpingectomy for sterilization (procedure)
        "85543002",  # Plication of superior vena cava (procedure)
        "85664008",  # Thromboendarterectomy with graft of celiac artery (procedure)
        "85704004",  # Open osteotomy of mandibular ramus (procedure)
        "85735005",  # Repair of heart and pericardium (procedure)
        "85827004",  # Total ethmoidectomy by intranasal approach (procedure)
        "85830006",  # Repair of heart valve (procedure)
        "85849005",  # Exteriorization of large intestine (procedure)
        "85855000",  # Partial bilateral salpingectomy for sterilization by endoscopy (procedure)
        "858579005",  # Excision of entire colon, entire rectum and entire anal canal (procedure)
        "85946004",  # Lobectomy of liver (procedure)
        "85963000",  # Replacement of cerebral ventricular valve (procedure)
        "86032007",  # Enlargement of intestinal stoma (procedure)
        "86046007",  # Arthrotomy of sternoclavicular joint for infection with exploration (procedure)
        "860602007",  # Postpartum excision of uterus (procedure)
        "86127008",  # Excision of hydatid cyst of liver (procedure)
        "8613002",  # Operation on appendix (procedure)
        "86146009",  # Partial excision of pituitary gland by transfrontal approach (procedure)
        "86250007",  # Cineplastic amputation (procedure)
        "86291009",  # Repair of adrenal gland (procedure)
        "86369000",  # Hepatotomy with drainage (procedure)
        "8645007",  # Aorta-coronary sinus shunt (procedure)
        "86462008",  # Aneurysmectomy of upper limb artery (procedure)
        "86477000",  # Total hysterectomy with removal of both tubes and ovaries (procedure)
        "86481000",  # Laparotomy (procedure)
        "86642005",  # Creation of shunt of descending aorta to pulmonary artery (procedure)
        "86676001",  # Incision of Rathke's pouch (procedure)
        "86730009",  # Excision of lesion of heart (procedure)
        "86789002",  # Abdominoperineal pull-through procedure (procedure)
        "868227005",  # Coronary artery bypass grafting using radial artery graft (procedure)
        "868228000",  # Coronary artery bypass grafting using gastroepiploic artery graft (procedure)
        "868230003",  # Coronary artery bypass grafting using free right internal thoracic artery graft (procedure)
        "868231004",  # Coronary artery bypass grafting using free left internal thoracic artery graft (procedure)
        "86868001",  # Aortoiliofemoral vascular bypass (procedure)
        "86881008",  # Revision of heart procedure (procedure)
        "869261000000104",  # Percutaneous mechanical thrombectomy of renal artery using fluoroscopic guidance with contrast (procedure)
        "869331000000103",  # Percutaneous radiofrequency ablation of renal artery using fluoroscopic guidance (procedure)
        "86971008",  # Closure of ventricular septal defect with pulmonary valvotomy and patch (procedure)
        "870252007",  # Ventricular tachycardia ablation (procedure)
        "870257001",  # Stereoelectroencephalography (procedure)
        "870378000",  # Endoscopic sleeve gastroplasty (procedure)
        "870390003",  # Dissection of fissure of Sylvius (procedure)
        "87050008",  # Arteriectomy with graft replacement of head and neck artery (procedure)
        "87055003",  # Excision of intracardiac tumor with cardiopulmonary bypass (procedure)
        "87060004",  # Pallidotomy (procedure)
        "870623000",  # Closure of rectovaginal fistula using plug (procedure)
        "870743002",  # Anastomosis of right internal thoracic artery to coronary artery (procedure)
        "870744008",  # Anastomosis of left internal thoracic artery to coronary artery (procedure)
        "870745009",  # Therapeutic transcatheter insertion of interatrial shunt (procedure)
        "87102007",  # Excision of round ligament (procedure)
        "87123001",  # Excision of heart valve (procedure)
        "871496000",  # Sequential anastomosis of left internal thoracic artery to coronary artery (procedure)
        "871497009",  # Sequential anastomosis of right internal thoracic artery to coronary artery (procedure)
        "871498004",  # Sequential anastomosis of free right internal thoracic artery to coronary artery (procedure)
        "87150006",  # Enterostomy (procedure)
        "87180001",  # Cerebral ventricular shunt to urinary system (procedure)
        "87192001",  # Pallidectomy (procedure)
        "87230000",  # Take-down of anastomosis of intestine (procedure)
        "87260005",  # Closure of esophageal fistula by cervical approach (procedure)
        "87279008",  # Excision of lesion of colon (procedure)
        "87289007",  # Anastomosis of small-to-large intestine (procedure)
        "87299002",  # Resection of temporal bone by external approach (procedure)
        "87308003",  # Gas endarterectomy of upper limb artery (procedure)
        "87366001",  # Total elbow replacement (procedure)
        "87375004",  # Radical bilateral orchiectomy (procedure)
        "87562003",  # Boyd operation for hip disarticulation (procedure)
        "87577006",  # Percutaneous transluminal renal artery balloon angioplasty (procedure)
        "87594008",  # Intra-abdominal manipulation of intestine (procedure)
        "8760004",  # Thrombectomy with catheter of subclavian artery by arm incision (procedure)
        "87604009",  # Sleeve resection of stomach (procedure)
        "87635003",  # Enteroentectropy (procedure)
        "87659008",  # Incision and drainage of duodenum (procedure)
        "87677003",  # Resection of rectum (procedure)
        "8773000",  # Transplantation of large intestine (procedure)
        "87742009",  # Gas endarterectomy of abdominal artery (procedure)
        "8777004",  # Incision and exploration of large intestine (procedure)
        "87795007",  # Complete transurethral resection of prostate, including control of postoperative bleeding (procedure)
        "8782006",  # Radical perineal prostatectomy (procedure)
        "87917005",  # Exploratory cardiotomy (procedure)
        "87922005",  # Suture of pelvic floor (procedure)
        "880070008",  # Repair of double inlet ventricle (procedure)
        "88019008",  # Subclavian-subclavian artery bypass graft with vein (procedure)
        "88039007",  # Transplant of lung (procedure)
        "88045004",  # Replacement of pulmonary valve (procedure)
        "88056004",  # Arthroplasty of knee with constrained prosthesis (procedure)
        "88065006",  # Intracranial arteriectomy with graft replacement (procedure)
        "8811003",  # Closure of hepatothoracic fistula (procedure)
        "88144003",  # Removal of ectopic interstitial uterine pregnancy requiring total hysterectomy (procedure)
        "88218008",  # Excision of lesion of uterus (procedure)
        "8831004",  # Removal of coronary artery thrombus (procedure)
        "88312006",  # Amputation of leg through tibia and fibula (procedure)
        "88314007",  # Wookey excision of esophagus with radical neck dissection (procedure)
        "88354009",  # Removal of foreign body from intestine by incision (procedure)
        "88421008",  # Closure of rectovaginal fistula by abdominal approach (procedure)
        "88553002",  # Removal of cardioverter/defibrillator pulse generator without replacement (procedure)
        "88623004",  # Excision of lesion of larynx (procedure)
        "88713003",  # Transection of lingual nerve (procedure)
        "88734005",  # Renal pyelostomy (procedure)
        "8876004",  # Aortocoronary artery bypass graft with prosthesis (procedure)
        "88821004",  # Vascular bypass of renal artery (procedure)
        "88849006",  # Esophagesophagostomy (procedure)
        "88851005",  # Cerebral ventricular to thoracic cavity shunt (procedure)
        "88859007",  # Angiectomy with graft replacement of upper limb artery (procedure)
        "88888008",  # Excision of lesion of duodenum (procedure)
        "88930008",  # Total nephrectomy in recipient (procedure)
        "890177005",  # Repair of biventricular atrioventricular connection (procedure)
        "8902002",  # Simple pyeloplasty (procedure)
        "8903007",  # Excision of omphalomesenteric duct (procedure)
        "890305007",  # Reverse prosthetic total arthroplasty of bilateral shoulders (procedure)
        "8906004",  # Cecocolostomy (procedure)
        "89230009",  # Closure of ureterovesical fistula (procedure)
        "89297009",  # Replacement of temporary transvenous pacemaker system (procedure)
        "89300004",  # Incision and exploration of small intestine (procedure)
        "89339008",  # Partial colectomy with coloproctostomy and colostomy (procedure)
        "894153007",  # Repair of congenital malformation of heart valve (procedure)
        "89441008",  # Jejunocolostomy for obesity (procedure)
        "8951004",  # Excision of lesion of retroperitoneum (procedure)
        "89522007",  # Colocolostomy (procedure)
        "895468008",  # Prosthetic total arthroplasty of bilateral shoulders (procedure)
        "895470004",  # Amputation of bilateral upper limbs (procedure)
        "895471000",  # Amputation of bilateral forearms through radius and ulna (procedure)
        "895473002",  # Amputation of bilateral hands (procedure)
        "895522007",  # Amputation of left hand (procedure)
        "895523002",  # Amputation of right hand (procedure)
        "89562006",  # Abrasion of epicardial surface (procedure)
        "89587004",  # Removal of foreign body from abdominal cavity (procedure)
        "89610000",  # Suture of kidney (procedure)
        "89642006",  # Anterior resection of rectum with colostomy (procedure)
        "89726007",  # Drainage of ovary by incision (procedure)
        "897702006",  # Laparoscopic extended left hemihepatectomy (procedure)
        "897703001",  # Laparoscopic extended right hemihepatectomy (procedure)
        "897705008",  # Laparoscopic trisegmentectomy of left lobe of liver (procedure)
        "897706009",  # Laparoscopic trisegmentectomy of right lobe of liver (procedure)
        "897714003",  # Laparoscopic excision of entire caudate lobe of liver (procedure)
        "8979005",  # Creation of peritoneal-venous shunt (procedure)
        "898089006",  # Excision of Couinaud hepatic segment V and Couinaud hepatic segment VIII (procedure)
        "898090002",  # Laparoscopic excision of Couinaud hepatic segment V and Couinaud hepatic segment VIII (procedure)
        "89814007",  # Repair of ventricular septal defect with prosthesis (procedure)
        "898177006",  # Excision of Couinaud hepatic segment VI and Couinaud hepatic segment VII (procedure)
        "898178001",  # Laparoscopic excision of Couinaud hepatic segment VI and Couinaud hepatic segment VII (procedure)
        "898179009",  # Excision of Couinaud hepatic segment IV and Couinaud hepatic segment V and Couinaud hepatic segment VIII (procedure)
        "898180007",  # Laparoscopic excision of Couinaud hepatic segment IV and Couinaud hepatic segment V and Couinaud hepatic segment VIII (procedure)
        "89946006",  # Pulmonary vein to atrium shunt construction (procedure)
        "89951000",  # U-tube hepaticoenterostomy (procedure)
        "90021002",  # Esophagoduodenostomy (procedure)
        "90033007",  # Ligation of middle meningeal artery (procedure)
        "90061001",  # Ligation of descending aorta (procedure)
        "90063003",  # Pyloric antrectomy (procedure)
        "90090007",  # Anastomosis of pancreatic duct (procedure)
        "90161004",  # Arteriectomy with anastomosis (procedure)
        "90205004",  # Cardiac revascularization with bypass anastomosis (procedure)
        "90252005",  # Open osteotomy of mandibular angle (procedure)
        "90287005",  # Thalamectomy (procedure)
        "90297001",  # Closure of thoracointestinal fistula (procedure)
        "90333001",  # Direct thrombectomy of vena cava by leg incision (procedure)
        "90410003",  # Wedge resection of stomach (procedure)
        "90451001",  # Neurotomy of lacrimal branch nerve (procedure)
        "90476000",  # Creation of subclavian-vertebral artery shunt (procedure)
        "90481009",  # Thromboendarterectomy with graft of deep profunda femoral artery (procedure)
        "90487008",  # Aortocoronary bypass of two coronary arteries (procedure)
        "90499001",  # Thromboendarterectomy with graft of axillary-brachial artery (procedure)
        "90524002",  # Closure of esophagopleurocutaneous fistula (procedure)
        "90527009",  # Laparoscopic-assisted anterior resection of rectum (procedure)
        "90543002",  # Replacement of electronic heart device, transvenous electrode (procedure)
        "90648000",  # Partial cystectomy with reimplantation of ureter into urinary bladder (procedure)
        "90788004",  # Revision of stoma of rectum (procedure)
        "90827004",  # Arthroplasty of knee, condyle and plateau, medial compartment (procedure)
        "90856004",  # Total excision of small intestine (procedure)
        "90989000",  # Arteriectomy of thoracic aorta (procedure)
        "9102008",  # Proximal subtotal gastrectomy by thoracic approach (procedure)
        "911581000000102",  # Fluoroscopic angiography of transplanted hepatic artery with contrast with insertion of stent (procedure)
        "911601000000106",  # Percutaneous thrombolysis of hepatic artery using fluoroscopic guidance (procedure)
        "911611000000108",  # Percutaneous thrombolysis of transplanted hepatic artery using fluoroscopic guidance (procedure)
        "91165003",  # Subapical osteotomy of mandible (procedure)
        "911651000000107",  # Percutaneous transluminal angioplasty of transplanted hepatic artery using fluoroscopic guidance with contrast (procedure)
        "91176004",  # Direct thrombectomy of femoropopliteal vein by abdominal and leg incision (procedure)
        "9121000",  # Excision of lesion of large intestine (procedure)
        "91219007",  # Correction of atresia of rectum (procedure)
        "91270003",  # Arytenoidectomy by external approach (procedure)
        "91292004",  # Repair of ventricular septum in total repair of total anomalous pulmonary venous connection (procedure)
        "91516004",  # Distal subtotal pancreatectomy with pancreaticojejunostomy (procedure)
        "91592003",  # Closure of fistula of sinus of Valsalva (procedure)
        "91596000",  # Complete excision of lung with mediastinal dissection (procedure)
        "91617003",  # Incision and exploration of spleen (procedure)
        "91621005",  # Total gastrectomy with intestinal interposition (procedure)
        "91625001",  # Selective vagotomy (procedure)
        "91643002",  # Cerebral ventricular shunt to ureter (procedure)
        "91679005",  # Plication of jejunum (procedure)
        "9175006",  # Parietal cell vagotomy with pyloroplasty and gastrostomy (procedure)
        "9215005",  # Colotomy (procedure)
        "9221009",  # Surgical treatment of septic abortion (procedure)
        "9274004",  # Removal of intra-aortic balloon with repair of aorta with graft (procedure)
        "9280007",  # Proctoplasty for stenosis (procedure)
        "9292007",  # Esophageal hiatus hernia repair (procedure)
        "935441000168106",  # Excision of retroperitoneal lymph node group (procedure)
        "9372002",  # Drainage of cranial sinus by trephination (procedure)
        "9409008",  # Incisional biopsy of pancreas (procedure)
        "9429009",  # Plastic repair with reconstruction of stomach (procedure)
        "9451001",  # Removal of thrombus of bovine graft (procedure)
        "9524002",  # Total pancreatectomy (procedure)
        "9580008",  # Chemothalamectomy (procedure)
        "9666005",  # Esophagocolic anastomosis, antesternal or antethoracic (procedure)
        "9735005",  # Repair of colon (procedure)
        "9835001",  # Removal of cerebral ventricular catheter (procedure)
        "9860001",  # Aneurysmectomy of descending aorta (procedure)
        "9905009",  # Loop colostomy (procedure)
        "9993006",  # Incision of diaphragm (procedure)
    }

class MechanicalVentilation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for respiratory mechanical ventilation

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent respiratory mechanical ventilation, both invasive and non-invasive

    **Exclusion Criteria:** Excludes concepts that represent CPAP (continuous positive airway pressure)
    """

    VALUE_SET_NAME = "Mechanical Ventilation"
    OID = "2.16.840.1.113762.1.4.1248.107"
    DEFINITION_VERSION = "20220507"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "5A1935Z",  # Respiratory Ventilation, Less than 24 Consecutive Hours
        "5A1945Z",  # Respiratory Ventilation, 24-96 Consecutive Hours
        "5A1955Z",  # Respiratory Ventilation, Greater than 96 Consecutive Hours
    }

    SNOMEDCT = {
        "1258985005",  # Invasive mechanical ventilation (regime/therapy)
        "243141005",  # Mechanically assisted spontaneous ventilation (regime/therapy)
    }

class NonInvasiveOxygenTherapyByNasalCannulaOrMask(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for noninvasive oxygen therapy and oxygen administered by nasal cannula or mask

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent noninvasive oxygen therapy and oxygen administered by nasal cannula or mask and not associated with mechanical ventilation

    **Exclusion Criteria:** Invasive oxygen therapies associated with mechanical ventilation
    """

    VALUE_SET_NAME = "Non Invasive Oxygen Therapy by Nasal Cannula or Mask"
    OID = "2.16.840.1.113762.1.4.1248.209"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    SNOMEDCT = {
        "1259025002",  # Heated and humidified high flow oxygen therapy using nasal cannula (procedure)
        "243136002",  # Short-term oxygen therapy (procedure)
        "243137006",  # Long-term oxygen therapy (procedure)
        "304577004",  # Humidified oxygen therapy (procedure)
        "31253009",  # Application of tracheostomy mask using jet humidifier (procedure)
        "315041000",  # High concentration oxygen therapy (procedure)
        "371907003",  # Oxygen administration by nasal cannula (procedure)
        "371908008",  # Oxygen administration by mask (procedure)
        "429253002",  # Oxygen administration by Venturi mask (procedure)
        "57485005",  # Oxygen therapy (procedure)
        "71786000",  # Intranasal oxygen therapy (procedure)
        "870533002",  # Heated and humidified high flow oxygen therapy (procedure)
    }

class TracheostomyProcedures(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for tracheostomy procedures

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent tracheostomy procedures

    **Exclusion Criteria:** N/A
    """

    VALUE_SET_NAME = "Tracheostomy Procedures"
    OID = "2.16.840.1.113762.1.4.1248.181"
    DEFINITION_VERSION = "20220719"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0B110F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach
        "0B110Z4",  # Bypass Trachea to Cutaneous, Open Approach
        "0B113F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach
        "0B113Z4",  # Bypass Trachea to Cutaneous, Percutaneous Approach
        "0B114F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach
        "0B114Z4",  # Bypass Trachea to Cutaneous, Percutaneous Endoscopic Approach
        "0BW10FZ",  # Revision of Tracheostomy Device in Trachea, Open Approach
        "0BW13FZ",  # Revision of Tracheostomy Device in Trachea, Percutaneous Approach
        "0BW14FZ",  # Revision of Tracheostomy Device in Trachea, Percutaneous Endoscopic Approach
        "0WB6XZ2",  # Excision of Neck, Stoma, External Approach
    }

    SNOMEDCT = {
        "232615005",  # Exploration of tracheostomy (procedure)
        "274969000",  # Anterior tracheostomy (procedure)
        "274970004",  # Lateral tracheostomy (procedure)
        "307007002",  # Percutaneous tracheostomy (procedure)
        "397930006",  # Percutaneous dilational tracheostomy (procedure)
        "50339005",  # Complete laryngectomy with radical neck dissection, synchronous thyroidectomy and synchronous tracheostomy (procedure)
        "52536006",  # Mediastinal tracheostomy (procedure)
        "55622001",  # Tracheostomy, emergency procedure by transtracheal approach (procedure)
        "7240008",  # Total laryngectomy with partial pharyngectomy and synchronous tracheostomy (procedure)
    }

class HysterectomyWithNoResidualCervix(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for hysterectomies that include removal of the patient's cervix.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent total or radical hysterectomies.

    **Exclusion Criteria:** Excludes concepts that represent partial hysterectomies and hysterectomies that leave the patient's cervix intact.
    """

    VALUE_SET_NAME = "Hysterectomy with No Residual Cervix"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1014"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "58293",  # Vaginal hysterectomy, for uterus greater than 250 g; with colpo-urethrocystopexy (Marshall-Marchetti-Krantz type, Pereyra type) with or without endoscopic control
        "59135",  # Surgical treatment of ectopic pregnancy; interstitial, uterine pregnancy requiring total hysterectomy
        "57530",  # Trachelectomy (cervicectomy), amputation of cervix (separate procedure)
        "57531",  # Radical trachelectomy, with bilateral total pelvic lymphadenectomy and para-aortic lymph node sampling biopsy, with or without removal of tube(s), with or without removal of ovary(s)
        "57540",  # Excision of cervical stump, abdominal approach
        "57545",  # Excision of cervical stump, abdominal approach; with pelvic floor repair
        "57550",  # Excision of cervical stump, vaginal approach
        "57555",  # Excision of cervical stump, vaginal approach; with anterior and/or posterior repair
        "57556",  # Excision of cervical stump, vaginal approach; with repair of enterocele
        "58150",  # Total abdominal hysterectomy (corpus and cervix), with or without removal of tube(s), with or without removal of ovary(s)
        "58152",  # Total abdominal hysterectomy (corpus and cervix), with or without removal of tube(s), with or without removal of ovary(s); with colpo-urethrocystopexy (eg, Marshall-Marchetti-Krantz, Burch)
        "58200",  # Total abdominal hysterectomy, including partial vaginectomy, with para-aortic and pelvic lymph node sampling, with or without removal of tube(s), with or without removal of ovary(s)
        "58210",  # Radical abdominal hysterectomy, with bilateral total pelvic lymphadenectomy and para-aortic lymph node sampling (biopsy), with or without removal of tube(s), with or without removal of ovary(s)
        "58240",  # Pelvic exenteration for gynecologic malignancy, with total abdominal hysterectomy or cervicectomy, with or without removal of tube(s), with or without removal of ovary(s), with removal of bladder and ureteral transplantations, and/or abdominoperineal resection of rectum and colon and colostomy, or any combination thereof
        "58260",  # Vaginal hysterectomy, for uterus 250 g or less
        "58262",  # Vaginal hysterectomy, for uterus 250 g or less; with removal of tube(s), and/or ovary(s)
        "58263",  # Vaginal hysterectomy, for uterus 250 g or less; with removal of tube(s), and/or ovary(s), with repair of enterocele
        "58267",  # Vaginal hysterectomy, for uterus 250 g or less; with colpo-urethrocystopexy (Marshall-Marchetti-Krantz type, Pereyra type) with or without endoscopic control
        "58270",  # Vaginal hysterectomy, for uterus 250 g or less; with repair of enterocele
        "58275",  # Vaginal hysterectomy, with total or partial vaginectomy
        "58280",  # Vaginal hysterectomy, with total or partial vaginectomy; with repair of enterocele
        "58285",  # Vaginal hysterectomy, radical (Schauta type operation)
        "58290",  # Vaginal hysterectomy, for uterus greater than 250 g
        "58291",  # Vaginal hysterectomy, for uterus greater than 250 g; with removal of tube(s) and/or ovary(s)
        "58292",  # Vaginal hysterectomy, for uterus greater than 250 g; with removal of tube(s) and/or ovary(s), with repair of enterocele
        "58294",  # Vaginal hysterectomy, for uterus greater than 250 g; with repair of enterocele
        "58548",  # Laparoscopy, surgical, with radical hysterectomy, with bilateral total pelvic lymphadenectomy and para-aortic lymph node sampling (biopsy), with removal of tube(s) and ovary(s), if performed
        "58550",  # Laparoscopy, surgical, with vaginal hysterectomy, for uterus 250 g or less
        "58552",  # Laparoscopy, surgical, with vaginal hysterectomy, for uterus 250 g or less; with removal of tube(s) and/or ovary(s)
        "58553",  # Laparoscopy, surgical, with vaginal hysterectomy, for uterus greater than 250 g
        "58554",  # Laparoscopy, surgical, with vaginal hysterectomy, for uterus greater than 250 g; with removal of tube(s) and/or ovary(s)
        "58570",  # Laparoscopy, surgical, with total hysterectomy, for uterus 250 g or less
        "58571",  # Laparoscopy, surgical, with total hysterectomy, for uterus 250 g or less; with removal of tube(s) and/or ovary(s)
        "58572",  # Laparoscopy, surgical, with total hysterectomy, for uterus greater than 250 g
        "58573",  # Laparoscopy, surgical, with total hysterectomy, for uterus greater than 250 g; with removal of tube(s) and/or ovary(s)
        "58575",  # Laparoscopy, surgical, total hysterectomy for resection of malignancy (tumor debulking), with omentectomy including salpingo-oophorectomy, unilateral or bilateral, when performed
        "58951",  # Resection (initial) of ovarian, tubal or primary peritoneal malignancy with bilateral salpingo-oophorectomy and omentectomy; with total abdominal hysterectomy, pelvic and limited para-aortic lymphadenectomy
        "58953",  # Bilateral salpingo-oophorectomy with omentectomy, total abdominal hysterectomy and radical dissection for debulking
        "58954",  # Bilateral salpingo-oophorectomy with omentectomy, total abdominal hysterectomy and radical dissection for debulking; with pelvic lymphadenectomy and limited para-aortic lymphadenectomy
        "58956",  # Bilateral salpingo-oophorectomy with total omentectomy, total abdominal hysterectomy for malignancy
    }

    ICD10PCS = {
        "0UTC0ZZ",  # Resection of Cervix, Open Approach
        "0UTC4ZZ",  # Resection of Cervix, Percutaneous Endoscopic Approach
        "0UTC7ZZ",  # Resection of Cervix, Via Natural or Artificial Opening
        "0UTC8ZZ",  # Resection of Cervix, Via Natural or Artificial Opening Endoscopic
    }

    SNOMEDCT = {
        "82418001",  # Manchester operation on uterus (procedure)
        "116140006",  # Total hysterectomy (procedure)
        "116142003",  # Radical hysterectomy (procedure)
        "116143008",  # Total abdominal hysterectomy (procedure)
        "116144002",  # Total abdominal hysterectomy with bilateral salpingo-oophorectomy (procedure)
        "1163275000",  # Laparoscopic radical amputation of cervix uteri (procedure)
        "1287897002",  # Radical amputation of cervix uteri (procedure)
        "176697007",  # Repair of vaginal prolapse and amputation of cervix uteri (procedure)
        "236888001",  # Laparoscopic total hysterectomy (procedure)
        "236891001",  # Laparoscopic radical hysterectomy (procedure)
        "24293001",  # Excision of cervical stump by abdominal approach (procedure)
        "287924009",  # Excision of cervix stump (procedure)
        "307771009",  # Radical abdominal hysterectomy (procedure)
        "35955002",  # Radical vaginal hysterectomy (procedure)
        "361222003",  # Wertheim-Meigs abdominal hysterectomy (procedure)
        "361223008",  # Wertheim operation (procedure)
        "387626007",  # Amputation of cervix (procedure)
        "414575003",  # Laparoscopic total abdominal hysterectomy and bilateral salpingo-oophorectomy (procedure)
        "41566006",  # Excision of cervical stump by vaginal approach (procedure)
        "440383008",  # Radical amputation of cervix with bilateral total pelvic lymphadenectomy and paraaortic lymph node biopsy (procedure)
        "446446002",  # Total abdominal hysterectomy and removal of vaginal cuff (procedure)
        "446679008",  # Total laparoscopic excision of uterus by abdominal approach (procedure)
        "46226009",  # Cervicectomy with synchronous colporrhaphy (procedure)
        "608805000",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele (procedure)
        "608806004",  # Laparoscopic assisted vaginal hysterectomy with repair of rectocele (procedure)
        "608807008",  # Laparoscopic assisted vaginal hysterectomy with repair of cystocele and rectocele (procedure)
        "708877008",  # Laparoscopic total hysterectomy using robotic assistance (procedure)
        "708878003",  # Laparoscopic radical hysterectomy using robotic assistance (procedure)
        "739671004",  # Total hysterectomy with left oophorectomy (procedure)
        "739672006",  # Total hysterectomy with right oophorectomy (procedure)
        "739673001",  # Total hysterectomy with left salpingo-oophorectomy (procedure)
        "739674007",  # Total hysterectomy with right salpingo-oophorectomy (procedure)
        "740514001",  # Total hysterectomy with right salpingectomy (procedure)
        "740515000",  # Total hysterectomy with left salpingectomy (procedure)
        "767610009",  # Total hysterectomy via vaginal approach (procedure)
        "767611008",  # Total abdominal hysterectomy using intrafascial technique (procedure)
        "767612001",  # Total hysterectomy via vaginal approach using intrafascial technique (procedure)
        "86477000",  # Total hysterectomy with removal of both tubes and ovaries (procedure)
        "88144003",  # Removal of ectopic interstitial uterine pregnancy requiring total hysterectomy (procedure)
    }

class BilateralMastectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for bilateral mastectomy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for bilateral mastectomy.

    **Exclusion Criteria:** Excludes concepts that represent procedures for a unilateral or unspecified mastectomy.
    """

    VALUE_SET_NAME = "Bilateral Mastectomy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1005"
    DEFINITION_VERSION = "20240105"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0HTV0ZZ",  # Resection of Bilateral Breast, Open Approach
    }

    SNOMEDCT = {
        "1268980002",  # Simple mastectomy of right breast and prophylactic mastectomy of left breast (procedure)
        "1269061009",  # Simple mastectomy of left breast and prophylactic mastectomy of right breast (procedure)
        "1279986002",  # Excision of bilateral nipples (procedure)
        "14693006",  # Subcutaneous mammectomy of bilateral breasts (procedure)
        "14714006",  # Excision of bilateral breasts with excision of bilateral regional lymph nodes (procedure)
        "17086001",  # Modified radical mastectomy, bilateral (procedure)
        "22418005",  # Simple mastectomy of bilateral breasts (procedure)
        "27865001",  # Excision of bilateral breasts (procedure)
        "456903003",  # Extended radical mastectomy of bilateral breasts (procedure)
        "52314009",  # Extended simple mastectomy of bilateral breasts (procedure)
        "60633004",  # Subcutaneous mammectomy of bilateral breasts with synchronous implants (procedure)
        "726636007",  # Prophylactic bilateral mastectomy (procedure)
        "76468001",  # Radical mastectomy of bilateral breasts (procedure)
        "836436008",  # Simple mastectomy of bilateral breasts using robotic assistance (procedure)
        "870629001",  # Excision of bilateral breasts for female to male transsexual (procedure)
    }

class UnilateralMastectomyLeft(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for unilateral mastectomy of the left breast.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for a unilateral mastectomy of the left breast.

    **Exclusion Criteria:** Excludes concepts that represent procedures for right or bilateral, or unspecified mastectomy.
    """

    VALUE_SET_NAME = "Unilateral Mastectomy Left"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1133"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0HTU0ZZ",  # Resection of Left Breast, Open Approach
    }

    SNOMEDCT = {
        "428571003",  # Excision of left breast (procedure)
        "451211000124109",  # Prophylactic mastectomy of left breast (procedure)
        "726429001",  # Radical mastectomy of left breast (procedure)
        "726435001",  # Subcutaneous mastectomy of left breast (procedure)
        "726437009",  # Modified radical mastectomy of left breast (procedure)
        "741009001",  # Simple mastectomy of left breast (procedure)
        "741018004",  # Subcutaneous mastectomy of left breast with prosthetic implant (procedure)
        "836437004",  # Simple mastectomy of left breast using robotic assistance (procedure)
    }

class UnilateralMastectomyRight(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for unilateral mastectomy of the right breast.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for unilateral mastectomy of the right breast.

    **Exclusion Criteria:** Excludes concepts that represent procedures for left or bilateral, or unspecified mastectomy.
    """

    VALUE_SET_NAME = "Unilateral Mastectomy Right"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1134"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0HTT0ZZ",  # Resection of Right Breast, Open Approach
    }

    SNOMEDCT = {
        "429400009",  # Excision of right breast (procedure)
        "451201000124106",  # Prophylactic mastectomy of right breast (procedure)
        "726430006",  # Radical mastectomy of right breast (procedure)
        "726434002",  # Subcutaneous mastectomy of right breast (procedure)
        "726436000",  # Modified radical mastectomy of right breast (procedure)
        "741010006",  # Simple mastectomy of right breast (procedure)
        "741019007",  # Subcutaneous mastectomy of right breast with prosthetic implant (procedure)
        "836435007",  # Simple mastectomy of right breast using robotic assistance (procedure)
    }

class ProstateCancerTreatment(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for prostate cancer treatments.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for interstitial prostate brachytherapy, external beam radiotherapy to the prostate, and radical prostatectomy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Prostate Cancer Treatment"
    OID = "2.16.840.1.113883.3.526.3.398"
    DEFINITION_VERSION = "20250205"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "55810",  # Prostatectomy, perineal radical
        "55812",  # Prostatectomy, perineal radical; with lymph node biopsy(s) (limited pelvic lymphadenectomy)
        "55815",  # Prostatectomy, perineal radical; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric and obturator nodes
        "55840",  # Prostatectomy, retropubic radical, with or without nerve sparing
        "55842",  # Prostatectomy, retropubic radical, with or without nerve sparing; with lymph node biopsy(s) (limited pelvic lymphadenectomy)
        "55845",  # Prostatectomy, retropubic radical, with or without nerve sparing; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric, and obturator nodes
        "55866",  # Laparoscopy, surgical prostatectomy, retropubic radical, including nerve sparing, includes robotic assistance, when performed
        "55875",  # Transperineal placement of needles or catheters into prostate for interstitial radioelement application, with or without cystoscopy
        "55880",  # Ablation of malignant prostate tissue, transrectal, with high intensity-focused ultrasound (HIFU), including ultrasound guidance
        "77373",  # Stereotactic body radiation therapy, treatment delivery, per fraction to 1 or more lesions, including image guidance, entire course not to exceed 5 fractions
        "77385",  # Intensity modulated radiation treatment delivery (IMRT), includes guidance and tracking, when performed; simple
        "77386",  # Intensity modulated radiation treatment delivery (IMRT), includes guidance and tracking, when performed; complex
        "77401",  # Radiation treatment delivery, superficial and/or ortho voltage, per day
        "77402",  # Radiation treatment delivery, >=1 MeV; simple
        "77407",  # Radiation treatment delivery, >=1 MeV; intermediate
        "77412",  # Radiation treatment delivery, >=1 MeV; complex
        "77423",  # High energy neutron radiation treatment delivery, 1 or more isocenter(s) with coplanar or non-coplanar geometry with blocking and/or wedge, and/or compensator(s)
        "77427",  # Radiation treatment management, 5 treatments
        "77435",  # Stereotactic body radiation therapy, treatment management, per treatment course, to 1 or more lesions, including image guidance, entire course not to exceed 5 fractions
        "77520",  # Proton treatment delivery; simple, without compensation
        "77522",  # Proton treatment delivery; simple, with compensation
        "77523",  # Proton treatment delivery; intermediate
        "77525",  # Proton treatment delivery; complex
        "77772",  # Remote afterloading high dose rate radionuclide interstitial or intracavitary brachytherapy, includes basic dosimetry, when performed; over 12 channels
        "77778",  # Interstitial radiation source application, complex, includes supervision, handling, loading of radiation source, when performed
    }

    HCPCSLEVELII = {
        "G6003",  # Radiation treatment delivery, single treatment area,single port or parallel opposed ports, simple blocks or no blocks: up to 5 mev
        "G6004",  # Radiation treatment delivery, single treatment area,single port or parallel opposed ports, simple blocks or no blocks: 6-10 mev
        "G6005",  # Radiation treatment delivery, single treatment area,single port or parallel opposed ports, simple blocks or no blocks: 11-19 mev
        "G6006",  # Radiation treatment delivery, single treatment area,single port or parallel opposed ports, simple blocks or no blocks: 20 mev or greater
        "G6007",  # Radiation treatment delivery, 2 separate treatment areas, 3 or more ports on a single treatment area, use of multiple blocks: up to 5 mev
        "G6008",  # Radiation treatment delivery, 2 separate treatment areas, 3 or more ports on a single treatment area, use of multiple blocks: 6-10 mev
        "G6009",  # Radiation treatment delivery, 2 separate treatment areas, 3 or more ports on a single treatment area, use of multiple blocks: 11-19 mev
        "G6010",  # Radiation treatment delivery, 2 separate treatment areas, 3 or more ports on a single treatment area, use of multiple blocks: 20 mev or greater
        "G6011",  # Radiation treatment delivery,3 or more separate treatment areas, custom blocking, tangential ports, wedges, rotational beam, compensators, electron beam; up to 5 mev
        "G6012",  # Radiation treatment delivery,3 or more separate treatment areas, custom blocking, tangential ports, wedges, rotational beam, compensators, electron beam; 6-10 mev
        "G6013",  # Radiation treatment delivery,3 or more separate treatment areas, custom blocking, tangential ports, wedges, rotational beam, compensators, electron beam; 11-19 mev
        "G6014",  # Radiation treatment delivery,3 or more separate treatment areas, custom blocking, tangential ports, wedges, rotational beam, compensators, electron beam; 20 mev or greater
        "G6015",  # Intensity modulated treatment delivery, single or multiple fields/arcs,via narrow spatially and temporally modulated beams, binary, dynamic mlc, per treatment session
        "G6016",  # Compensator-based beam modulation treatment delivery of inverse planned treatment using 3 or more high resolution (milled or cast) compensator, convergent beam modulated fields, per treatment session
    }

    SNOMEDCT = {
        "113120007",  # Interstitial brachytherapy (procedure)
        "169359004",  # Internal radiotherapy - permanent seeds (procedure)
        "176261008",  # Radical prostatectomy without pelvic node excision (procedure)
        "176262001",  # Radical prostatectomy with pelvic node sampling (procedure)
        "176263006",  # Radical prostatectomy with pelvic lymphadenectomy (procedure)
        "228677009",  # Cesium 137 brachytherapy (procedure)
        "228681009",  # Cobalt 60 brachytherapy (procedure)
        "228682002",  # Cobalt 60 brachytherapy manual loading (procedure)
        "228684001",  # Iridium 192 brachytherapy (procedure)
        "228688003",  # Californium 252 brachytherapy (procedure)
        "228689006",  # Gold 198 brachytherapy (procedure)
        "228690002",  # Iodine 125 brachytherapy (procedure)
        "228691003",  # Phosphorus 32 brachytherapy (procedure)
        "228692005",  # Radon 222 brachytherapy (procedure)
        "228693000",  # Ruthenium 106 brachytherapy (procedure)
        "228694006",  # Strontium 90 brachytherapy (procedure)
        "228695007",  # Tantalum 182 brachytherapy (procedure)
        "228696008",  # Yttrium 90 brachytherapy (procedure)
        "228697004",  # Yttrium 90 therapy (procedure)
        "228698009",  # Iodine 131 therapy (procedure)
        "228699001",  # Phosphorus 32 therapy (procedure)
        "228701001",  # Strontium 89 therapy (procedure)
        "228702008",  # Gold 198 therapy (procedure)
        "24242005",  # Cobalt-60 therapy (procedure)
        "26294005",  # Radical prostatectomy (procedure)
        "271291003",  # Temporary implant radiotherapy (procedure)
        "28579000",  # Radical retropubic prostatectomy with lymph node biopsy (procedure)
        "312243002",  # Cesium 137 brachytherapy remote afterloading (procedure)
        "312244008",  # Cobalt 60 brachytherapy remote afterloading (procedure)
        "36253005",  # Suprapubic prostatectomy (procedure)
        "37851009",  # Radical perineal prostatectomy with bilateral pelvic lymphadenectomy (procedure)
        "38915000",  # Radionuclide therapy for gland ablation (procedure)
        "394902000",  # High dose brachytherapy (procedure)
        "394918006",  # Iodine seed radiotherapy (procedure)
        "399180008",  # Interstitial radioactive colloid therapy (procedure)
        "41371003",  # Perineal prostatectomy (procedure)
        "41416003",  # Radical retropubic prostatectomy with bilateral pelvic lymphadenectomy (procedure)
        "440093006",  # Insertion of radioactive implant into interstitial tissue using ultrasound guidance (procedure)
        "440094000",  # Insertion of radioactive implant into interstitial tissue of prostate with transperineal insertion of catheter (procedure)
        "62867004",  # External beam radiation therapy with radioactive cesium (procedure)
        "65551008",  # Radical retropubic prostatectomy (procedure)
        "67598001",  # Retropubic prostatectomy (procedure)
        "72388004",  # Radical perineal prostatectomy with lymph node biopsy (procedure)
        "77613002",  # External beam radiation therapy with iodine-125 (procedure)
        "84755001",  # Radiation therapy treatment management (procedure)
        "8782006",  # Radical perineal prostatectomy (procedure)
        "90470006",  # Prostatectomy (procedure)
    }

class SalvageTherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for salvage therapy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for salvage therapy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Salvage Therapy"
    OID = "2.16.840.1.113883.3.526.3.399"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "51597",  # Pelvic exenteration, complete, for vesical, prostatic or urethral malignancy, with removal of bladder and ureteral transplantations, with or without hysterectomy and/or abdominoperineal resection of rectum and colon and colostomy, or any combination thereof
        "55860",  # Exposure of prostate, any approach, for insertion of radioactive substance
        "55862",  # Exposure of prostate, any approach, for insertion of radioactive substance; with lymph node biopsy(s) (limited pelvic lymphadenectomy)
        "55865",  # Exposure of prostate, any approach, for insertion of radioactive substance; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric and obturator nodes
    }

    SNOMEDCT = {
        "236209003",  # Salvage cystoprostatourethrectomy (procedure)
        "236211007",  # Salvage cystoprostatectomy (procedure)
    }

class Colonoscopy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a colonoscopy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a screening or diagnostic colonoscopy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Colonoscopy"
    OID = "2.16.840.1.113883.3.464.1003.108.12.1020"
    DEFINITION_VERSION = "20171219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "44388",  # Colonoscopy through stoma; diagnostic, including collection of specimen(s) by brushing or washing, when performed (separate procedure)
        "44389",  # Colonoscopy through stoma; with biopsy, single or multiple
        "44390",  # Colonoscopy through stoma; with removal of foreign body(s)
        "44391",  # Colonoscopy through stoma; with control of bleeding, any method
        "44392",  # Colonoscopy through stoma; with removal of tumor(s), polyp(s), or other lesion(s) by hot biopsy forceps
        "44394",  # Colonoscopy through stoma; with removal of tumor(s), polyp(s), or other lesion(s) by snare technique
        "44401",  # Colonoscopy through stoma; with ablation of tumor(s), polyp(s), or other lesion(s) (includes pre-and post-dilation and guide wire passage, when performed)
        "44402",  # Colonoscopy through stoma; with endoscopic stent placement (including pre- and post-dilation and guide wire passage, when performed)
        "44403",  # Colonoscopy through stoma; with endoscopic mucosal resection
        "44404",  # Colonoscopy through stoma; with directed submucosal injection(s), any substance
        "44405",  # Colonoscopy through stoma; with transendoscopic balloon dilation
        "44406",  # Colonoscopy through stoma; with endoscopic ultrasound examination, limited to the sigmoid, descending, transverse, or ascending colon and cecum and adjacent structures
        "44407",  # Colonoscopy through stoma; with transendoscopic ultrasound guided intramural or transmural fine needle aspiration/biopsy(s), includes endoscopic ultrasound examination limited to the sigmoid, descending, transverse, or ascending colon and cecum and adjacent structures
        "44408",  # Colonoscopy through stoma; with decompression (for pathologic distention) (eg, volvulus, megacolon), including placement of decompression tube, when performed
        "45378",  # Colonoscopy, flexible; diagnostic, including collection of specimen(s) by brushing or washing, when performed (separate procedure)
        "45379",  # Colonoscopy, flexible; with removal of foreign body(s)
        "45380",  # Colonoscopy, flexible; with biopsy, single or multiple
        "45381",  # Colonoscopy, flexible; with directed submucosal injection(s), any substance
        "45382",  # Colonoscopy, flexible; with control of bleeding, any method
        "45384",  # Colonoscopy, flexible; with removal of tumor(s), polyp(s), or other lesion(s) by hot biopsy forceps
        "45385",  # Colonoscopy, flexible; with removal of tumor(s), polyp(s), or other lesion(s) by snare technique
        "45386",  # Colonoscopy, flexible; with transendoscopic balloon dilation
        "45388",  # Colonoscopy, flexible; with ablation of tumor(s), polyp(s), or other lesion(s) (includes pre- and post-dilation and guide wire passage, when performed)
        "45389",  # Colonoscopy, flexible; with endoscopic stent placement (includes pre- and post-dilation and guide wire passage, when performed)
        "45390",  # Colonoscopy, flexible; with endoscopic mucosal resection
        "45391",  # Colonoscopy, flexible; with endoscopic ultrasound examination limited to the rectum, sigmoid, descending, transverse, or ascending colon and cecum, and adjacent structures
        "45392",  # Colonoscopy, flexible; with transendoscopic ultrasound guided intramural or transmural fine needle aspiration/biopsy(s), includes endoscopic ultrasound examination limited to the rectum, sigmoid, descending, transverse, or ascending colon and cecum, and adjacent structures
        "45393",  # Colonoscopy, flexible; with decompression (for pathologic distention) (eg, volvulus, megacolon), including placement of decompression tube, when performed
        "45398",  # Colonoscopy, flexible; with band ligation(s) (eg, hemorrhoids)
    }

    HCPCSLEVELII = {
        "G0105",  # Colorectal cancer screening; colonoscopy on individual at high risk
        "G0121",  # Colorectal cancer screening; colonoscopy on individual not meeting criteria for high risk
    }

    SNOMEDCT = {
        "174185007",  # Diagnostic fiberoptic endoscopic examination of colon and biopsy of lesion of colon (procedure)
        "25732003",  # Fiberoptic colonoscopy with biopsy (procedure)
        "367535003",  # Fiberoptic colonoscopy (procedure)
        "1209098000",  # Fiberoptic colonoscopy with biopsy of lesion of colon (procedure)
        "1217313001",  # Biopsy of colon using endoscopic ultrasonography guidance (procedure)
        "12350003",  # Colonoscopy with rigid sigmoidoscope through colotomy (procedure)
        "1304042004",  # Colonoscopy and suturing of rectum (procedure)
        "1304043009",  # Colonoscopy and suturing of colon (procedure)
        "1304044003",  # Colonoscopy and suturing of small intestine (procedure)
        "1304045002",  # Colonoscopy and full thickness resection of rectum (procedure)
        "1304049008",  # Colonoscopy and full thickness resection of colon (procedure)
        "1304050008",  # Colonoscopy and full thickness resection of small intestine (procedure)
        "174158000",  # Open colonoscopy (procedure)
        "174173004",  # Fiberoptic endoscopic laser destruction of lesion of colon (procedure)
        "174179000",  # Fiberoptic endoscopic dilation of colon (procedure)
        "235150006",  # Total colonoscopy (procedure)
        "302052009",  # Endoscopic biopsy of lesion of colon (procedure)
        "443998000",  # Colonoscopy through colostomy with endoscopic biopsy of colon (procedure)
        "444783004",  # Screening colonoscopy (procedure)
        "446521004",  # Colonoscopy and excision of mucosa of colon (procedure)
        "446745002",  # Colonoscopy and biopsy of colon (procedure)
        "447021001",  # Colonoscopy and tattooing (procedure)
        "48021000087103",  # Colonoscopy using cecal retroflexion technique (procedure)
        "48031000087101",  # Colonoscopy using rectal retroflexion technique (procedure)
        "609197007",  # Endoscopic surgical procedure on colon using laser (procedure)
        "709421007",  # Colonoscopy and dilation of stricture of colon (procedure)
        "710293001",  # Colonoscopy using fluoroscopic guidance (procedure)
        "711307001",  # Colonoscopy using plain X-ray guidance (procedure)
        "73761001",  # Colonoscopy (procedure)
        "771568007",  # Fiberoptic endoscopic excision of lesion of colonic mucous membrane (procedure)
        "789778002",  # Colonoscopy and fecal microbiota transplantation (procedure)
        "8180007",  # Fiberoptic colonoscopy through colostomy (procedure)
    }

class FlexibleSigmoidoscopy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a flexible sigmoidoscopy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a flexible sigmoidoscopy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Flexible Sigmoidoscopy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1010"
    DEFINITION_VERSION = "20171219"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "45330",  # Sigmoidoscopy, flexible; diagnostic, including collection of specimen(s) by brushing or washing, when performed (separate procedure)
        "45331",  # Sigmoidoscopy, flexible; with biopsy, single or multiple
        "45332",  # Sigmoidoscopy, flexible; with removal of foreign body(s)
        "45333",  # Sigmoidoscopy, flexible; with removal of tumor(s), polyp(s), or other lesion(s) by hot biopsy forceps
        "45334",  # Sigmoidoscopy, flexible; with control of bleeding, any method
        "45335",  # Sigmoidoscopy, flexible; with directed submucosal injection(s), any substance
        "45337",  # Sigmoidoscopy, flexible; with decompression (for pathologic distention) (eg, volvulus, megacolon), including placement of decompression tube, when performed
        "45338",  # Sigmoidoscopy, flexible; with removal of tumor(s), polyp(s), or other lesion(s) by snare technique
        "45340",  # Sigmoidoscopy, flexible; with transendoscopic balloon dilation
        "45341",  # Sigmoidoscopy, flexible; with endoscopic ultrasound examination
        "45342",  # Sigmoidoscopy, flexible; with transendoscopic ultrasound guided intramural or transmural fine needle aspiration/biopsy(s)
        "45346",  # Sigmoidoscopy, flexible; with ablation of tumor(s), polyp(s), or other lesion(s) (includes pre- and post-dilation and guide wire passage, when performed)
        "45347",  # Sigmoidoscopy, flexible; with placement of endoscopic stent (includes pre- and post-dilation and guide wire passage, when performed)
        "45349",  # Sigmoidoscopy, flexible; with endoscopic mucosal resection
        "45350",  # Sigmoidoscopy, flexible; with band ligation(s) (eg, hemorrhoids)
    }

    HCPCSLEVELII = {
        "G0104",  # Colorectal cancer screening; flexible sigmoidoscopy
    }

    SNOMEDCT = {
        "1217117008",  # Endoscopic examination of lower bowel and sampling for bacterial overgrowth (procedure)
        "396226005",  # Flexible fiberoptic sigmoidoscopy with biopsy (procedure)
        "44441009",  # Flexible fiberoptic sigmoidoscopy (procedure)
    }

class TotalColectomy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a total colectomy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a total colectomy.

    **Exclusion Criteria:** Excludes concepts that represent partial colectomies.
    """

    VALUE_SET_NAME = "Total Colectomy"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1019"
    DEFINITION_VERSION = "20240112"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "44152",  # Colectomy, total, abdominal, without proctectomy; with rectal mucosectomy, ileoanal anastomosis, with or without loop ileostomy
        "44153",  # Colectomy, total, abdominal, without proctectomy; with rectal mucosectomy, ileoanal anastomosis, creation of ileal reservoir (S or J), with or without loop ileostomy
        "44150",  # Colectomy, total, abdominal, without proctectomy; with ileostomy or ileoproctostomy
        "44151",  # Colectomy, total, abdominal, without proctectomy; with continent ileostomy
        "44155",  # Colectomy, total, abdominal, with proctectomy; with ileostomy
        "44156",  # Colectomy, total, abdominal, with proctectomy; with continent ileostomy
        "44157",  # Colectomy, total, abdominal, with proctectomy; with ileoanal anastomosis, includes loop ileostomy, and rectal mucosectomy, when performed
        "44158",  # Colectomy, total, abdominal, with proctectomy; with ileoanal anastomosis, creation of ileal reservoir (S or J), includes loop ileostomy, and rectal mucosectomy, when performed
        "44210",  # Laparoscopy, surgical; colectomy, total, abdominal, without proctectomy, with ileostomy or ileoproctostomy
        "44211",  # Laparoscopy, surgical; colectomy, total, abdominal, with proctectomy, with ileoanal anastomosis, creation of ileal reservoir (S or J), with loop ileostomy, includes rectal mucosectomy, when performed
        "44212",  # Laparoscopy, surgical; colectomy, total, abdominal, with proctectomy, with ileostomy
    }

    ICD10PCS = {
        "0DTE0ZZ",  # Resection of Large Intestine, Open Approach
        "0DTE4ZZ",  # Resection of Large Intestine, Percutaneous Endoscopic Approach
        "0DTE7ZZ",  # Resection of Large Intestine, Via Natural or Artificial Opening
        "0DTE8ZZ",  # Resection of Large Intestine, Via Natural or Artificial Opening Endoscopic
    }

    SNOMEDCT = {
        "26390003",  # Total colectomy (procedure)
        "303401008",  # Parks panproctocolectomy, anastomosis of ileum to anus and creation of pouch (procedure)
        "307666008",  # Total colectomy and ileostomy (procedure)
        "307667004",  # Total colectomy, ileostomy and rectal mucous fistula (procedure)
        "307669001",  # Total colectomy, ileostomy and closure of rectal stump (procedure)
        "31130001",  # Total abdominal colectomy with proctectomy and ileostomy (procedure)
        "36192008",  # Total abdominal colectomy with ileoproctostomy (procedure)
        "44751009",  # Total abdominal colectomy with proctectomy and continent ileostomy (procedure)
        "456004",  # Total abdominal colectomy with ileostomy (procedure)
        "713165008",  # Laparoscopic total colectomy with ileo-rectal anastomosis (procedure)
        "787108001",  # Laparoscopic restorative proctocolectomy with ileal pouch anal anastomosis (procedure)
        "787109009",  # Excision of entire colon and entire rectum (procedure)
        "787874000",  # Laparoscopic total colectomy (procedure)
        "787875004",  # Laparoscopic total colectomy and loop ileostomy (procedure)
        "787876003",  # Laparoscopic total colectomy and ileostomy (procedure)
        "80294005",  # Total abdominal colectomy with rectal mucosectomy and ileoanal anastomosis (procedure)
        "858579005",  # Excision of entire colon, entire rectum and entire anal canal (procedure)
    }

class CataractSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for cataract surgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure of cataract surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cataract Surgery"
    OID = "2.16.840.1.113883.3.526.3.1411"
    DEFINITION_VERSION = "20210210"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "66840",  # Removal of lens material; aspiration technique, 1 or more stages
        "66850",  # Removal of lens material; phacofragmentation technique (mechanical or ultrasonic) (eg, phacoemulsification), with aspiration
        "66852",  # Removal of lens material; pars plana approach, with or without vitrectomy
        "66920",  # Removal of lens material; intracapsular
        "66930",  # Removal of lens material; intracapsular, for dislocated lens
        "66940",  # Removal of lens material; extracapsular (other than 66840, 66850, 66852)
        "66982",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1-stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification), complex, requiring devices or techniques not generally used in routine cataract surgery (eg, iris expansion device, suture support for intraocular lens, or primary posterior capsulorrhexis) or performed on patients in the amblyogenic developmental stage; without endoscopic cyclophotocoagulation
        "66983",  # Intracapsular cataract extraction with insertion of intraocular lens prosthesis (1 stage procedure)
        "66984",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1 stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification); without endoscopic cyclophotocoagulation
        "66987",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1-stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification), complex, requiring devices or techniques not generally used in routine cataract surgery (eg, iris expansion device, suture support for intraocular lens, or primary posterior capsulorrhexis) or performed on patients in the amblyogenic developmental stage; with endoscopic cyclophotocoagulation
        "66988",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1 stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification); with endoscopic cyclophotocoagulation
        "66989",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1-stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification), complex, requiring devices or techniques not generally used in routine cataract surgery (eg, iris expansion device, suture support for intraocular lens, or primary posterior capsulorrhexis) or performed on patients in the amblyogenic developmental stage; with insertion of intraocular (eg, trabecular meshwork, supraciliary, suprachoroidal) anterior segment aqueous drainage device, without extraocular reservoir, internal approach, one or more
        "66991",  # Extracapsular cataract removal with insertion of intraocular lens prosthesis (1 stage procedure), manual or mechanical technique (eg, irrigation and aspiration or phacoemulsification); with insertion of intraocular (eg, trabecular meshwork, supraciliary, suprachoroidal) anterior segment aqueous drainage device, without extraocular reservoir, internal approach, one or more
    }

    SNOMEDCT = {
        "10178000",  # Extracapsular extraction of cataract by inferior temporal route (procedure)
        "110473004",  # Cataract surgery (procedure)
        "112963003",  # Aspiration of cataract (procedure)
        "112964009",  # Extraction of cataract by rotoextraction with aspiration (procedure)
        "12163000",  # Needling of secondary cataract (procedure)
        "1231296008",  # Extraction of cataract of bilateral eyes (procedure)
        "1231299001",  # Extraction of cataract of right eye (procedure)
        "1231300009",  # Extraction of cataract of left eye (procedure)
        "1236928007",  # Surgery of cataract of bilateral eyes (procedure)
        "172523009",  # Intracapsular extraction of lens (procedure)
        "231744001",  # Cataract extraction, insertion of intraocular lens and trabeculectomy (procedure)
        "308694002",  # Extracapsular cataract extraction and insertion of intraocular lens (procedure)
        "308695001",  # Intracapsular cataract extraction and insertion of intraocular lens (procedure)
        "313999004",  # Small incision phacoemulsification of cataract and insertion of intraocular lens (procedure)
        "31705006",  # Extraction of cataract by rotoextraction with aspiration by posterior route (procedure)
        "335636001",  # Extraction of primary membranous cataract by phacofragmentation (procedure)
        "336651000",  # Extraction of primary membranous cataract by mechanical fragmentation (procedure)
        "35717002",  # Discission of congenital cataract (procedure)
        "361191005",  # Extraction of cataract by ultrasonic phacofragmentation (procedure)
        "3700004",  # Erysiphake extraction of cataract by intracapsular approach (procedure)
        "385468004",  # Cataract extraction and implantation of intraocular lens (procedure)
        "38621000087109",  # Extraction of cataract and implantation of intraocular lens in left eye (procedure)
        "38631000087106",  # Extraction of cataract and implantation of intraocular lens in right eye (procedure)
        "39243005",  # Mechanical fragmentation of cataract with extraction by posterior route (procedure)
        "397544007",  # Complicated cataract surgery (procedure)
        "404628003",  # Intracapsular cataract extraction (procedure)
        "415089008",  # Phacoemulsification of cataract with intraocular lens implantation (procedure)
        "417493007",  # Small incision manual extracapsular cataract extraction (procedure)
        "418430006",  # Bimanual phacoemulsification of cataract with intraocular lens implantation (procedure)
        "419767009",  # Bimanual microincisional phacoemulsification of cataract with intraocular lens implantation (procedure)
        "420260004",  # Bimanual phacoemulsification (procedure)
        "420526005",  # Coaxial phacoemulsfication (procedure)
        "424945000",  # Torsional phacoemulsification (procedure)
        "446548003",  # Extraction of cataract and trabeculectomy (procedure)
        "46309001",  # Extraction of primary membranous cataract by excision (procedure)
        "46426006",  # Cryoextraction of cataract by inferior temporal route (procedure)
        "46562009",  # Incisional discission of secondary membranous cataract and anterior hyaloid (procedure)
        "50538003",  # Discission of membranous cataract, primary (procedure)
        "5130002",  # Extraction of primary membranous cataract by discission (procedure)
        "51839008",  # Extraction of primary membranous cataract by needling (procedure)
        "54885007",  # Extraction of cataract (procedure)
        "65812008",  # Removal of secondary membranous cataract with iridectomy (procedure)
        "67760003",  # Extracapsular extraction of cataract by rotoextraction with aspiration (procedure)
        "69360005",  # Extracapsular extraction of cataract by mechanical fragmentation with aspiration by posterior route (procedure)
        "74490003",  # Mechanical fragmentation of primary membranous cataract (procedure)
        "75814005",  # Erysiphake extraction of cataract by inferior temporal route (procedure)
        "79611007",  # Extracapsular extraction of cataract by curette evacuation (procedure)
        "82155009",  # Extracapsular extraction of cataract by emulsification with aspiration (procedure)
        "84149000",  # Aspiration of cataract by phacoemulsification (procedure)
        "85622008",  # Cryoextraction of cataract by intracapsular approach (procedure)
        "860930005",  # Surgery of cataract of left eye (procedure)
        "860934001",  # Surgery of cataract of right eye (procedure)
        "88282000",  # Extracapsular extraction of cataract by mechanical fragmentation with aspiration (procedure)
        "89153001",  # Extracapsular extraction of cataract by rotoextraction with aspiration by posterior route (procedure)
        "9137006",  # Discission of membranous cataract, secondary (procedure)
    }

class HeartTransplant(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for heart transplant.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent heart transplant.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Heart Transplant"
    OID = "2.16.840.1.113762.1.4.1178.33"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33935",  # Heart-lung transplant with recipient cardiectomy-pneumonectomy
        "33945",  # Heart transplant, with or without recipient cardiectomy
    }

    SNOMEDCT = {
        "174803001",  # Revision of transplantation of heart and lung (procedure)
        "32413006",  # Transplantation of heart (procedure)
        "32477003",  # Heart-lung transplant with recipient cardiectomy-pneumonectomy (procedure)
        "450816007",  # Revision of transplantation of heart (procedure)
        "47058000",  # Heart transplant with recipient cardiectomy (procedure)
    }

class LeftVentricularAssistDevicePlacement(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for left ventricular assist device placement.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures for left ventricular and biventricular device placement.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Left Ventricular Assist Device Placement"
    OID = "2.16.840.1.113762.1.4.1178.61"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33976",  # Insertion of ventricular assist device; extracorporeal, biventricular
        "33981",  # Replacement of extracorporeal ventricular assist device, single or biventricular, pump(s), single or each pump
        "33990",  # Insertion of ventricular assist device, percutaneous, including radiological supervision and interpretation; left heart, arterial access only
        "33991",  # Insertion of ventricular assist device, percutaneous, including radiological supervision and interpretation; left heart, both arterial and venous access, with transseptal puncture
        "33993",  # Repositioning of percutaneous right or left heart ventricular assist device with imaging guidance at separate and distinct session from insertion
    }

    SNOMEDCT = {
        "232967006",  # Implantation of left cardiac ventricular assist device (procedure)
        "232968001",  # Implantation of cardiac biventricular assist device (procedure)
        "700151002",  # Temporary implantation of left cardiac ventricular assist device (procedure)
    }

class SubstanceUseDisorderLongActingMedicationAdministration(ValueSet):
    """
    **Clinical Focus:** The purpose of this Grouping value set is to represent concepts for long acting substance use disorder medication assisted treatment, usually administered as part of an opioid treatment program.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes interventions where a long acting substance use disorder treatment medication is administered (e.g., buprenorphine implant, buprenorphine injection or naltrexone injection).

    **Exclusion Criteria:** Excludes administration of short acting oral and sublingual substance use disorder treatment medication.
    """

    VALUE_SET_NAME = "Substance Use Disorder Long Acting Medication Administration"
    OID = "2.16.840.1.113883.3.464.1003.1156"
    DEFINITION_VERSION = "20211102"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    HCPCSLEVELII = {
        "G2070",  # Medication assisted treatment, buprenorphine (implant insertion); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "G2072",  # Medication assisted treatment, buprenorphine (implant insertion and removal); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "J0570",  # Buprenorphine implant, 74.2 mg
        "G2069",  # Medication assisted treatment, buprenorphine (injectable) administered on a monthly basis; bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "G2073",  # Medication assisted treatment, naltrexone; weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "H0020",  # Alcohol and/or drug services; methadone administration and/or service (provision of the drug by a licensed program)
        "H0033",  # Oral medication administration, direct observation
        "J0571",  # Buprenorphine, oral, 1 mg
        "J0572",  # Buprenorphine/naloxone, oral, less than or equal to 3 mg buprenorphine
        "J0573",  # Buprenorphine/naloxone, oral, greater than 3 mg, but less than or equal to 6 mg buprenorphine
        "J0574",  # Buprenorphine/naloxone, oral, greater than 6 mg, but less than or equal to 10 mg buprenorphine
        "J0575",  # Buprenorphine/naloxone, oral, greater than 10 mg buprenorphine
        "J0577",  # Injection, buprenorphine extended-release (brixadi), less than or equal to 7 days of therapy
        "J0578",  # Injection, buprenorphine extended release (brixadi), greater than 7 days and up to 28 days of therapy
        "J2315",  # Injection, naltrexone, depot form, 1 mg
        "Q9991",  # Injection, buprenorphine extended-release (sublocade), less than or equal to 100 mg
        "Q9992",  # Injection, buprenorphine extended-release (sublocade), greater than 100 mg
        "S0109",  # Methadone, oral, 5 mg
    }

class SubstanceUseDisorderShortActingMedicationAdministration(ValueSet):
    """
    **Clinical Focus:** The purpose of this Grouping Value Set is to represent concepts for short acting substance use disorder medication assisted treatment, usually administered as part of an opioid treatment program.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes interventions where a short acting substance use disorder treatment medication is administered (oral and sublingual naltrexone, buprenorphine and buprenorphine).

    **Exclusion Criteria:** Excludes administration of long acting substance use disorder treatment medications (implants, injections).
    """

    VALUE_SET_NAME = "Substance Use Disorder Short Acting Medication Administration"
    OID = "2.16.840.1.113883.3.464.1003.1157"
    DEFINITION_VERSION = "20211102"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    HCPCSLEVELII = {
        "G2067",  # Medication assisted treatment, methadone; weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing, if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "G2068",  # Medication assisted treatment, buprenorphine (oral); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        "G2078",  # Take-home supply of methadone; up to 7 additional day supply (provision of the services by a medicare-enrolled opioid treatment program); list separately in addition to code for primary procedure
        "G2079",  # Take-home supply of buprenorphine (oral); up to 7 additional day supply (provision of the services by a medicare-enrolled opioid treatment program); list separately in addition to code for primary procedure
        "H0006",  # Alcohol and/or drug services; case management
        "H0020",  # Alcohol and/or drug services; methadone administration and/or service (provision of the drug by a licensed program)
        "H0028",  # Alcohol and/or drug prevention problem identification and referral service (e.g., student assistance and employee assistance programs), does not include assessment
        "H0033",  # Oral medication administration, direct observation
    }

    SNOMEDCT = {
        "1254709001",  # Referral to substance use disorder treatment program (procedure)
        "310653000",  # Drug addiction therapy using methadone (regime/therapy)
    }

class CardiacPacer(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for cardiac pacer procedures.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent procedures related to cardiac pacer.

    **Exclusion Criteria:** Excludes procedures related to the removal of cardiac pacer.
    """

    VALUE_SET_NAME = "Cardiac Pacer"
    OID = "2.16.840.1.113762.1.4.1178.53"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33206",  # Insertion of new or replacement of permanent pacemaker with transvenous electrode(s); atrial
        "33207",  # Insertion of new or replacement of permanent pacemaker with transvenous electrode(s); ventricular
        "33208",  # Insertion of new or replacement of permanent pacemaker with transvenous electrode(s); atrial and ventricular
        "33210",  # Insertion or replacement of temporary transvenous single chamber cardiac electrode or pacemaker catheter (separate procedure)
        "33211",  # Insertion or replacement of temporary transvenous dual chamber pacing electrodes (separate procedure)
        "33212",  # Insertion of pacemaker pulse generator only; with existing single lead
        "33213",  # Insertion of pacemaker pulse generator only; with existing dual leads
        "33214",  # Upgrade of implanted pacemaker system, conversion of single chamber system to dual chamber system (includes removal of previously placed pulse generator, testing of existing lead, insertion of new lead, insertion of new pulse generator)
        "33216",  # Insertion of a single transvenous electrode, permanent pacemaker or implantable defibrillator
        "33217",  # Insertion of 2 transvenous electrodes, permanent pacemaker or implantable defibrillator
        "33221",  # Insertion of pacemaker pulse generator only; with existing multiple leads
        "33224",  # Insertion of pacing electrode, cardiac venous system, for left ventricular pacing, with attachment to previously placed pacemaker or implantable defibrillator pulse generator (including revision of pocket, removal, insertion, and/or replacement of existing generator)
        "33225",  # Insertion of pacing electrode, cardiac venous system, for left ventricular pacing, at time of insertion of implantable defibrillator or pacemaker pulse generator (eg, for upgrade to dual chamber system) (List separately in addition to code for primary procedure)
        "33274",  # Transcatheter insertion or replacement of permanent leadless pacemaker, right ventricular, including imaging guidance (eg, fluoroscopy, venous ultrasound, ventriculography, femoral venography) and device evaluation (eg, interrogation or programming), when performed
    }

    SNOMEDCT = {
        "1231779006",  # Implantation of leadless cardiac pacemaker (procedure)
        "1234788003",  # Replacement of leadless cardiac pacemaker (procedure)
        "1287918007",  # Repair of cardiac pacemaker pocket (procedure)
        "175135009",  # Introduction of cardiac pacemaker system via vein (procedure)
        "175142009",  # Implantation of permanent intravenous cardiac pacemaker (procedure)
        "175143004",  # Implantation of intravenous fixed-rate cardiac pacemaker (procedure)
        "175144005",  # Implantation of intravenous triggered cardiac pacemaker (procedure)
        "20453006",  # Relocation of cardiac pacemaker pocket to new site in subcutaneous tissue (procedure)
        "230588002",  # Reposition of cardiac pacemaker pocket (procedure)
        "233174007",  # Cardiac pacemaker procedure (procedure)
        "233175008",  # Temporary cardiac pacemaker procedure (procedure)
        "233182007",  # Permanent cardiac pacemaker procedure (procedure)
        "233183002",  # Insertion of permanent epicardial cardiac pacemaker system (procedure)
        "23325006",  # Repair of cardiac pacemaker (procedure)
        "23999003",  # Implantation of rate-responsive cardiac single-chamber device (procedure)
        "25267002",  # Insertion of intracardiac pacemaker (procedure)
        "265482008",  # Implantation of emergency intravenous cardiac pacemaker (procedure)
        "307280005",  # Implantation of cardiac pacemaker (procedure)
        "308805008",  # Reimplantation of cardiac pacemaker electrode (procedure)
        "309405007",  # Implantation of simple one wire intravenous cardiac pacemaker (procedure)
        "309407004",  # Implantation of complex one wire intravenous cardiac pacemaker (procedure)
        "309408009",  # Implantation of complex two wire intravenous cardiac pacemaker (procedure)
        "309471004",  # Implantation of temporary intravenous cardiac pacemaker (procedure)
        "310582005",  # Implantation of intravenous dual chamber permanent cardiac pacemaker (procedure)
        "384683008",  # Replacement of cardiac pacemaker device with dual-chamber device (procedure)
        "429064006",  # Implantation of cardiac resynchronization therapy implantable pacemaker (procedure)
        "429528001",  # Implantation of intravenous single chamber cardiac pacemaker system (procedure)
        "429542009",  # Implantation of intravenous cardiac resynchronization therapy implantable pacemaker (procedure)
        "430294007",  # Reposition of permanent cardiac pacemaker using fluoroscopic guidance (procedure)
        "431846007",  # Replacement of permanent cardiac pacemaker using fluoroscopic guidance (procedure)
        "432113002",  # Insertion of temporary cardiac pacemaker using fluoroscopic guidance (procedure)
        "443753002",  # Insertion of single chamber cardiac pacemaker pulse generator (procedure)
        "444179007",  # Insertion of dual chamber cardiac pacemaker pulse generator (procedure)
        "444566006",  # Replacement of cardiac pacemaker (procedure)
        "448242007",  # Repositioning of cardiac pacemaker lead using fluoroscopic guidance (procedure)
        "448841009",  # Replacement of cardiac biventricular implantable cardioverter defibrillator using fluoroscopic guidance (procedure)
        "448869001",  # Replacement of cardiac biventricular permanent pacemaker using fluoroscopic guidance (procedure)
        "449397007",  # Insertion of permanent cardiac pacemaker pulse generator and electrode (procedure)
        "450820006",  # Replacement of pulse generator of permanent cardiac pacemaker using fluoroscopic guidance (procedure)
        "58863009",  # Initial implantation of cardiac dual-chamber device (procedure)
        "65219008",  # Subcutaneous implantation of cardiac pacemaker (procedure)
        "699125007",  # Insertion of programmable cardiac pacemaker (procedure)
        "699135001",  # Implantation of cardiac defibrillator lead (procedure)
        "703073006",  # Cardiac pacemaker procedure using fluoroscopic guidance (procedure)
        "704115002",  # Revision of cardiac pacemaker electrode using fluoroscopic guidance (procedure)
        "736700005",  # Revision of internal cardiac defibrillator lead using fluoroscopic guidance (procedure)
        "737011002",  # Revision of cardiac biventricular implantable cardioverter defibrillator lead using fluoroscopic guidance (procedure)
        "771557007",  # Replacement of cardiac biventricular implantable cardioverter defibrillator (procedure)
        "77453006",  # Revision of permanent cardiac pacemaker device (procedure)
        "8072003",  # Replacement of cardiac pacemaker with single chamber pacemaker not specified as rate responsive (procedure)
    }

class CardiacSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of procedures for cardiac surgery.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for cardiac surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cardiac Surgery"
    OID = "2.16.840.1.113883.3.526.3.371"
    DEFINITION_VERSION = "20210218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33140",  # Transmyocardial laser revascularization, by thoracotomy; (separate procedure)
        "33510",  # Coronary artery bypass, vein only; single coronary venous graft
        "33511",  # Coronary artery bypass, vein only; 2 coronary venous grafts
        "33512",  # Coronary artery bypass, vein only; 3 coronary venous grafts
        "33513",  # Coronary artery bypass, vein only; 4 coronary venous grafts
        "33514",  # Coronary artery bypass, vein only; 5 coronary venous grafts
        "33516",  # Coronary artery bypass, vein only; 6 or more coronary venous grafts
        "33522",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 5 venous grafts (List separately in addition to code for primary procedure)
        "33523",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 6 or more venous grafts (List separately in addition to code for primary procedure)
        "33533",  # Coronary artery bypass, using arterial graft(s); single arterial graft
        "33534",  # Coronary artery bypass, using arterial graft(s); 2 coronary arterial grafts
        "33535",  # Coronary artery bypass, using arterial graft(s); 3 coronary arterial grafts
        "33536",  # Coronary artery bypass, using arterial graft(s); 4 or more coronary arterial grafts
        "92920",  # Percutaneous transluminal coronary angioplasty; single major coronary artery or branch
        "92924",  # Percutaneous transluminal coronary atherectomy, with coronary angioplasty when performed; single major coronary artery or branch
        "92928",  # Percutaneous transcatheter placement of intracoronary stent(s), with coronary angioplasty when performed; single major coronary artery or branch
        "92933",  # Percutaneous transluminal coronary atherectomy, with intracoronary stent, with coronary angioplasty when performed; single major coronary artery or branch
        "92937",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of intracoronary stent, atherectomy and angioplasty, including distal protection when performed; single vessel
        "92941",  # Percutaneous transluminal revascularization of acute total/subtotal occlusion during acute myocardial infarction, coronary artery or coronary artery bypass graft, any combination of intracoronary stent, atherectomy and angioplasty, including aspiration thrombectomy when performed, single vessel
        "92943",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of intracoronary stent, atherectomy and angioplasty; single vessel
    }

    SNOMEDCT = {
        "1010291002",  # Anastomosis of left internal thoracic artery to left anterior descending coronary artery (procedure)
        "1010570006",  # Sequential anastomosis of left internal thoracic artery from second obtuse marginal branch of circumflex coronary artery to left anterior descending coronary artery (procedure)
        "1010571005",  # Sequential anastomosis of left internal thoracic artery from first obtuse marginal branch of circumflex coronary artery to left anterior descending coronary artery (procedure)
        "1010572003",  # Sequential anastomosis of left internal thoracic artery from ramus intermedius branch of left coronary artery to left anterior descending coronary artery (procedure)
        "1010688007",  # Anastomosis of left internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1017203005",  # Anastomosis of left internal thoracic artery to intermediate artery (procedure)
        "1017205003",  # Anastomosis of left internal thoracic artery to obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1017227007",  # Sequential anastomosis of left internal thoracic artery from second diagonal branch of anterior descending branch of left coronary artery to left anterior descending artery (procedure)
        "1017228002",  # Sequential anastomosis of left internal thoracic artery from first diagonal branch of anterior descending branch of left coronary artery to left anterior descending artery (procedure)
        "1017229005",  # Sequential anastomosis of left internal thoracic artery from proximal portion of anterior descending branch of left coronary artery to mid portion of anterior descending branch of left coronary artery (procedure)
        "1017232008",  # Sequential anastomosis of left internal thoracic artery from proximal portion of anterior descending branch of left coronary artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "1017233003",  # Sequential anastomosis of left internal thoracic artery from mid portion of anterior descending branch of left coronary artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "10190003",  # Aortocoronary bypass of four or more coronary arteries (procedure)
        "10326007",  # Coronary artery bypass with autogenous graft, three grafts (procedure)
        "1144644003",  # Anastomosis of right internal thoracic artery to anterior descending branch of left coronary artery (procedure)
        "1144645002",  # Anastomosis of right internal thoracic artery to obtuse marginal branch of circumflex branch of coronary artery (procedure)
        "1144646001",  # Anastomosis of right internal thoracic artery to marginal branch of right coronary artery (procedure)
        "1144647005",  # Anastomosis of right internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1144889004",  # Anastomosis of right internal thoracic artery to intermediate artery (procedure)
        "1144890008",  # Anastomosis of right internal thoracic artery to posterior descending artery (procedure)
        "1145198009",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1145199001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to anterior descending branch of left coronary artery (procedure)
        "1145200003",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to obtuse marginal branch of circumflex branch of coronary artery (procedure)
        "1148690000",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to intermediate artery (procedure)
        "1148691001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to posterior descending artery (procedure)
        "1149059007",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to marginal branch of right coronary artery (procedure)
        "1149060002",  # Coronary artery bypass grafting using free right internal thoracic artery graft from aorta to posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1155686006",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to intermediate artery (procedure)
        "1155687002",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1156983007",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1156984001",  # Coronary artery bypass grafting using free right internal thoracic artery graft from left internal thoracic artery to anterior descending branch of left coronary artery (procedure)
        "119564002",  # Internal mammary-coronary artery bypass graft (procedure)
        "119565001",  # Coronary artery bypass graft, anastomosis of artery of thorax to coronary artery (procedure)
        "1201858000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch of circumflex coronary artery and left anterior descending coronary artery (procedure)
        "1204377006",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery and distal portion of anterior descending branch of left coronary artery (procedure)
        "1204378001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery and mid portion of anterior descending branch of left coronary artery (procedure)
        "1208430005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to mid and distal portion of anterior descending branch of left coronary artery (procedure)
        "1208673009",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and second obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1208675002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery and anterior descending branch of left coronary artery (procedure)
        "1208676001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery to anterior descending branch of left coronary artery (procedure)
        "1208760004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of left anterior descending coronary artery to left anterior descending coronary artery (procedure)
        "1208761000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second diagonal branch of left anterior descending coronary artery to left anterior descending coronary artery (procedure)
        "1208916007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and first obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1208917003",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and third diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208919000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and second diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208940004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and first diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1208942007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery and third marginal circumflex coronary artery (procedure)
        "1208943002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery and first marginal circumflex coronary artery (procedure)
        "1209060000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and third diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209061001",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and second diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209062008",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second and third diagonal branches of anterior descending branch of left coronary artery (procedure)
        "1209115004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and third obtuse marginal branches of circumflex branch of left coronary artery (procedure)
        "1209117007",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first and second obtuse marginal branches of circumflex branch of left coronary artery (procedure)
        "1209119005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to intermediate artery and third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1209120004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209172009",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to left posterior lateral branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209174005",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to third obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1209180002",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to third obtuse marginal branch and left posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1209203006",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch and left posterior lateral branch of circumflex branch of left coronary artery (procedure)
        "1209204000",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second and third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1209205004",  # Sequential anastomosis of free right internal thoracic artery from left internal thoracic artery to second obtuse marginal branch of circumflex branch of left coronary artery and posterior descending coronary artery (procedure)
        "1217153005",  # Anastomosis of left internal thoracic artery to third diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217155003",  # Anastomosis of left internal thoracic artery to second obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1217157006",  # Anastomosis of left internal thoracic artery to second diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217158001",  # Anastomosis of left internal thoracic artery to proximal portion of anterior descending branch of left coronary artery (procedure)
        "1217161000",  # Anastomosis of left internal thoracic artery to mid portion of anterior descending branch of left coronary artery (procedure)
        "1217162007",  # Anastomosis of left internal thoracic artery to first obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1217165009",  # Anastomosis of left internal thoracic artery to first diagonal branch of anterior descending branch of left coronary artery (procedure)
        "1217166005",  # Anastomosis of left internal thoracic artery to distal portion of anterior descending branch of left coronary artery (procedure)
        "1217168006",  # Anastomosis of left internal thoracic artery to third obtuse marginal branch of circumflex branch of left coronary artery (procedure)
        "1287561002",  # Coronary artery bypass using robotic assistance (procedure)
        "14323007",  # Aortocoronary bypass of three coronary arteries (procedure)
        "15256002",  # Transmyocardial revascularization by laser technique (procedure)
        "174911007",  # Revascularization of wall of heart (procedure)
        "175021005",  # Allograft bypass of coronary artery (procedure)
        "175029007",  # Prosthetic bypass of coronary artery (procedure)
        "175036008",  # Revision of bypass for coronary artery (procedure)
        "175037004",  # Revision of bypass for one coronary artery (procedure)
        "175038009",  # Revision of bypass for two coronary arteries (procedure)
        "175039001",  # Revision of bypass for three coronary arteries (procedure)
        "175040004",  # Revision of bypass for four or more coronary arteries (procedure)
        "175041000",  # Revision of connection of thoracic artery to coronary artery (procedure)
        "175047001",  # Double implantation of mammary arteries into coronary arteries (procedure)
        "175048006",  # Single anastomosis of mammary artery to left anterior descending coronary artery (procedure)
        "175050003",  # Single implantation of mammary artery into coronary artery (procedure)
        "232717009",  # Coronary artery bypass grafting (procedure)
        "232719007",  # Coronary artery bypass graft x 1 (procedure)
        "232720001",  # Coronary artery bypass grafts x 2 (procedure)
        "232721002",  # Coronary artery bypass grafts x 3 (procedure)
        "232722009",  # Coronary artery bypass grafts x 4 (procedure)
        "232723004",  # Coronary artery bypass grafts x 5 (procedure)
        "232724005",  # Coronary artery bypass grafts greater than 5 (procedure)
        "265481001",  # Double anastomosis of mammary arteries to coronary arteries (procedure)
        "275215001",  # Left internal mammary artery single anastomosis (procedure)
        "275216000",  # Right internal mammary artery single anastomosis (procedure)
        "275227003",  # Myocardial revascularization (procedure)
        "287277008",  # Indirect heart revascularization (procedure)
        "30670000",  # Anastomosis of thoracic artery to coronary artery, double (procedure)
        "3546002",  # Aortocoronary artery bypass graft with saphenous vein graft (procedure)
        "359597003",  # Single internal mammary-coronary artery bypass (procedure)
        "359601003",  # Coronary artery bypass with autogenous graft of internal mammary artery, single graft (procedure)
        "39202005",  # Coronary artery bypass with autogenous graft, four grafts (procedure)
        "39724006",  # Anastomosis of internal mammary artery to coronary artery, double vessel (procedure)
        "414088005",  # Emergency coronary artery bypass graft (procedure)
        "418824004",  # Off-pump coronary artery bypass (procedure)
        "419132001",  # Minimally invasive direct coronary artery bypass (procedure)
        "48431000",  # Anastomosis of thoracic artery to coronary artery, single (procedure)
        "67166004",  # Aortocoronary artery bypass graft (procedure)
        "736962007",  # Bypass of four or more coronary arteries with prosthesis (procedure)
        "736963002",  # Bypass of one coronary artery with prosthesis (procedure)
        "736964008",  # Bypass of three coronary arteries with prosthesis (procedure)
        "736965009",  # Bypass of two coronary arteries with prosthesis (procedure)
        "736966005",  # Aortocoronary artery bypass of four or more coronary arteries with saphenous vein graft (procedure)
        "736967001",  # Aortocoronary artery bypass of one coronary artery with saphenous vein graft (procedure)
        "736968006",  # Aortocoronary artery bypass of three coronary arteries with saphenous vein graft (procedure)
        "736969003",  # Aortocoronary artery bypass of two coronary arteries with saphenous vein graft (procedure)
        "736970002",  # Allograft bypass of four or more coronary arteries (procedure)
        "736971003",  # Allograft bypass of one coronary artery (procedure)
        "736972005",  # Allograft bypass of three coronary arteries (procedure)
        "736973000",  # Allograft bypass of two coronary arteries (procedure)
        "74371005",  # Coronary artery bypass with autogenous graft, two grafts (procedure)
        "81266008",  # Heart revascularization (procedure)
        "82247006",  # Coronary artery bypass with autogenous graft, five grafts (procedure)
        "868227005",  # Coronary artery bypass grafting using radial artery graft (procedure)
        "868228000",  # Coronary artery bypass grafting using gastroepiploic artery graft (procedure)
        "868230003",  # Coronary artery bypass grafting using free right internal thoracic artery graft (procedure)
        "868231004",  # Coronary artery bypass grafting using free left internal thoracic artery graft (procedure)
        "871496000",  # Sequential anastomosis of left internal thoracic artery to coronary artery (procedure)
        "871497009",  # Sequential anastomosis of right internal thoracic artery to coronary artery (procedure)
        "871498004",  # Sequential anastomosis of free right internal thoracic artery to coronary artery (procedure)
        "90205004",  # Cardiac revascularization with bypass anastomosis (procedure)
        "90487008",  # Aortocoronary bypass of two coronary arteries (procedure)
    }

class ProceduresUsedToIndicateSexualActivity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent procedures indicating history of sexual activity

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for pregnancy or postpartum or contraceptive use.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Procedures Used to Indicate Sexual Activity"
    OID = "2.16.840.1.113883.3.464.1003.111.12.1017"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "11976",  # Removal, implantable contraceptive capsules
        "57022",  # Incision and drainage of vaginal hematoma; obstetrical/postpartum
        "57170",  # Diaphragm or cervical cap fitting with instructions
        "58300",  # Insertion of intrauterine device (IUD)
        "58301",  # Removal of intrauterine device (IUD)
        "58600",  # Ligation or transection of fallopian tube(s), abdominal or vaginal approach, unilateral or bilateral
        "58605",  # Ligation or transection of fallopian tube(s), abdominal or vaginal approach, postpartum, unilateral or bilateral, during same hospitalization (separate procedure)
        "58615",  # Occlusion of fallopian tube(s) by device (eg, band, clip, Falope ring) vaginal or suprapubic approach
        "58970",  # Follicle puncture for oocyte retrieval, any method
        "58974",  # Embryo transfer, intrauterine
        "58976",  # Gamete, zygote, or embryo intrafallopian transfer, any method
        "59000",  # Amniocentesis; diagnostic
        "59001",  # Amniocentesis; therapeutic amniotic fluid reduction (includes ultrasound guidance)
        "59012",  # Cordocentesis (intrauterine), any method
        "59015",  # Chorionic villus sampling, any method
        "59020",  # Fetal contraction stress test
        "59025",  # Fetal non-stress test
        "59030",  # Fetal scalp blood sampling
        "59050",  # Fetal monitoring during labor by consulting physician (ie, non-attending physician) with written report; supervision and interpretation
        "59051",  # Fetal monitoring during labor by consulting physician (ie, non-attending physician) with written report; interpretation only
        "59070",  # Transabdominal amnioinfusion, including ultrasound guidance
        "59072",  # Fetal umbilical cord occlusion, including ultrasound guidance
        "59074",  # Fetal fluid drainage (eg, vesicocentesis, thoracocentesis, paracentesis), including ultrasound guidance
        "59076",  # Fetal shunt placement, including ultrasound guidance
        "59100",  # Hysterotomy, abdominal (eg, for hydatidiform mole, abortion)
        "59120",  # Surgical treatment of ectopic pregnancy; tubal or ovarian, requiring salpingectomy and/or oophorectomy, abdominal or vaginal approach
        "59121",  # Surgical treatment of ectopic pregnancy; tubal or ovarian, without salpingectomy and/or oophorectomy
        "59130",  # Surgical treatment of ectopic pregnancy; abdominal pregnancy
        "59136",  # Surgical treatment of ectopic pregnancy; interstitial, uterine pregnancy with partial resection of uterus
        "59140",  # Surgical treatment of ectopic pregnancy; cervical, with evacuation
        "59150",  # Laparoscopic treatment of ectopic pregnancy; without salpingectomy and/or oophorectomy
        "59151",  # Laparoscopic treatment of ectopic pregnancy; with salpingectomy and/or oophorectomy
        "59160",  # Curettage, postpartum
        "59200",  # Insertion of cervical dilator (eg, laminaria, prostaglandin) (separate procedure)
        "59300",  # Episiotomy or vaginal repair, by other than attending
        "59320",  # Cerclage of cervix, during pregnancy; vaginal
        "59325",  # Cerclage of cervix, during pregnancy; abdominal
        "59350",  # Hysterorrhaphy of ruptured uterus
        "59400",  # Routine obstetric care including antepartum care, vaginal delivery (with or without episiotomy, and/or forceps) and postpartum care
        "59409",  # Vaginal delivery only (with or without episiotomy and/or forceps)
        "59410",  # Vaginal delivery only (with or without episiotomy and/or forceps); including postpartum care
        "59412",  # External cephalic version, with or without tocolysis
        "59414",  # Delivery of placenta (separate procedure)
        "59425",  # Antepartum care only; 4-6 visits
        "59426",  # Antepartum care only; 7 or more visits
        "59430",  # Postpartum care only (separate procedure)
        "59510",  # Routine obstetric care including antepartum care, cesarean delivery, and postpartum care
        "59514",  # Cesarean delivery only
        "59515",  # Cesarean delivery only; including postpartum care
        "59525",  # Subtotal or total hysterectomy after cesarean delivery (List separately in addition to code for primary procedure)
        "59610",  # Routine obstetric care including antepartum care, vaginal delivery (with or without episiotomy, and/or forceps) and postpartum care, after previous cesarean delivery
        "59612",  # Vaginal delivery only, after previous cesarean delivery (with or without episiotomy and/or forceps)
        "59614",  # Vaginal delivery only, after previous cesarean delivery (with or without episiotomy and/or forceps); including postpartum care
        "59618",  # Routine obstetric care including antepartum care, cesarean delivery, and postpartum care, following attempted vaginal delivery after previous cesarean delivery
        "59620",  # Cesarean delivery only, following attempted vaginal delivery after previous cesarean delivery
        "59622",  # Cesarean delivery only, following attempted vaginal delivery after previous cesarean delivery; including postpartum care
        "59812",  # Treatment of incomplete abortion, any trimester, completed surgically
        "59820",  # Treatment of missed abortion, completed surgically; first trimester
        "59821",  # Treatment of missed abortion, completed surgically; second trimester
        "59830",  # Treatment of septic abortion, completed surgically
        "59840",  # Induced abortion, by dilation and curettage
        "59841",  # Induced abortion, by dilation and evacuation
        "59850",  # Induced abortion, by 1 or more intra-amniotic injections (amniocentesis-injections), including hospital admission and visits, delivery of fetus and secundines
        "59851",  # Induced abortion, by 1 or more intra-amniotic injections (amniocentesis-injections), including hospital admission and visits, delivery of fetus and secundines; with dilation and curettage and/or evacuation
        "59852",  # Induced abortion, by 1 or more intra-amniotic injections (amniocentesis-injections), including hospital admission and visits, delivery of fetus and secundines; with hysterotomy (failed intra-amniotic injection)
        "59855",  # Induced abortion, by 1 or more vaginal suppositories (eg, prostaglandin) with or without cervical dilation (eg, laminaria), including hospital admission and visits, delivery of fetus and secundines
        "59856",  # Induced abortion, by 1 or more vaginal suppositories (eg, prostaglandin) with or without cervical dilation (eg, laminaria), including hospital admission and visits, delivery of fetus and secundines; with dilation and curettage and/or evacuation
        "59857",  # Induced abortion, by 1 or more vaginal suppositories (eg, prostaglandin) with or without cervical dilation (eg, laminaria), including hospital admission and visits, delivery of fetus and secundines; with hysterotomy (failed medical evacuation)
        "59866",  # Multifetal pregnancy reduction(s) (MPR)
        "59870",  # Uterine evacuation and curettage for hydatidiform mole
        "59871",  # Removal of cerclage suture under anesthesia (other than local)
        "59897",  # Unlisted fetal invasive procedure, including ultrasound guidance, when performed
        "59898",  # Unlisted laparoscopy procedure, maternity care and delivery
        "59899",  # Unlisted procedure, maternity care and delivery
    }

    HCPCSLEVELII = {
        "S0199",  # Medically induced abortion by oral ingestion of medication including all associated services and supplies (e.g., patient counseling, office visits, confirmation of pregnancy by hcg, ultrasound to confirm duration of pregnancy, ultrasound to confirm completion of abortion) except drugs
        "S4981",  # Insertion of levonorgestrel-releasing intrauterine system
        "S8055",  # Ultrasound guidance for multifetal pregnancy reduction(s), technical component (only to be used when the physician doing the reduction procedure does not perform the ultrasound, guidance is included in the cpt code for multifetal pregnancy reduction - 59866)
    }

    SNOMEDCT = {
        "386276009",  # Electronic fetal monitoring: antepartum (regime/therapy)
        "386277000",  # Electronic fetal monitoring: intrapartum (regime/therapy)
        "10455003",  # Removal of ectopic cervical pregnancy by evacuation (procedure)
        "10745001",  # Delivery of transverse presentation (procedure)
        "10763001",  # Therapeutic abortion by insertion of laminaria (procedure)
        "1141750000",  # Physiological delivery of placenta (procedure)
        "11466000",  # Cesarean section (procedure)
        "1263416007",  # Removal of intrauterine contraceptive device (procedure)
        "133875007",  # Injection of prostaglandin (procedure)
        "14119008",  # Braxton Hicks obstetrical version with extraction (procedure)
        "15413009",  # High forceps delivery with episiotomy (procedure)
        "16819009",  # Delivery of face presentation (procedure)
        "169553002",  # Insertion of subcutaneous contraceptive (procedure)
        "176749004",  # Removal of intrauterine contraceptive device from pouch of Douglas (procedure)
        "176833006",  # Dilation of cervix uteri and curettage of uterus for removal of missed abortion (procedure)
        "176839005",  # Removal of displaced intrauterine contraceptive device (procedure)
        "176850008",  # Intra-amniotic prostaglandin instillation (procedure)
        "176852000",  # Extra-amniotic prostaglandin instillation (procedure)
        "176854004",  # Insertion of prostaglandin abortifacient suppository (procedure)
        "176928008",  # Excision of ectopic ovarian pregnancy (procedure)
        "176929000",  # Excision of ruptured ectopic tubal pregnancy (procedure)
        "176996001",  # Endoscopic intrafallopian transfer of gamete (procedure)
        "177037000",  # Oocyte recovery (procedure)
        "177042008",  # Transvaginal oocyte recovery (procedure)
        "177091002",  # Fetoscopic sampling of fetal blood (procedure)
        "177100008",  # Percutaneous fetal procedure (procedure)
        "177101007",  # Percutaneous insertion of vesicoamniotic shunt (procedure)
        "177102000",  # Percutaneous insertion of pleuroamniotic shunt (procedure)
        "177106002",  # Percutaneous biopsy of fetus (procedure)
        "177107006",  # Percutaneous sampling of chorionic villus (procedure)
        "177112007",  # Therapeutic drainage of amniotic fluid (procedure)
        "177122001",  # External version of breech (procedure)
        "177136006",  # Prostaglandin induction of labor (procedure)
        "177141003",  # Elective cesarean section (procedure)
        "177142005",  # Elective upper segment cesarean section (procedure)
        "177143000",  # Elective lower segment cesarean section (procedure)
        "177152009",  # Breech extraction delivery with version (procedure)
        "177157003",  # Spontaneous breech delivery (procedure)
        "177158008",  # Assisted breech delivery (procedure)
        "177161009",  # Forceps cephalic delivery (procedure)
        "177162002",  # High forceps cephalic delivery with rotation (procedure)
        "177164001",  # Midforceps cephalic delivery with rotation (procedure)
        "177167008",  # Barton forceps cephalic delivery with rotation (procedure)
        "177168003",  # DeLee forceps cephalic delivery with rotation (procedure)
        "177170007",  # Piper forceps delivery (procedure)
        "177173009",  # High vacuum delivery (procedure)
        "177174003",  # Low vacuum delivery (procedure)
        "177175002",  # Vacuum delivery before full dilation of cervix (procedure)
        "177176001",  # Trial of vacuum delivery (procedure)
        "177179008",  # Cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177180006",  # Manipulative cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177181005",  # Non-manipulative cephalic vaginal delivery with abnormal presentation of head at delivery without instrument (procedure)
        "177184002",  # Normal delivery procedure (procedure)
        "177185001",  # Water birth delivery (procedure)
        "177222006",  # Repair of episiotomy (procedure)
        "17744000",  # Subtotal hysterectomy after cesarean delivery (procedure)
        "17860005",  # Low forceps delivery with episiotomy (procedure)
        "18302006",  # Therapeutic abortion by hysterotomy (procedure)
        "18489003",  # Surgical treatment of missed abortion of first trimester (procedure)
        "18625004",  # Low forceps delivery (procedure)
        "19390001",  # Partial breech delivery with forceps to aftercoming head (procedure)
        "215192006",  # Uterine evacuation and curettage of hydatidiform mole (procedure)
        "216209009",  # Removal of hydatidiform mole (procedure)
        "225251006",  # Fitting of cervical cap (procedure)
        "225256001",  # Removal of cervical cap (procedure)
        "225257005",  # Insertion of cervical cap (procedure)
        "22633006",  # Vaginal delivery, medical personnel present (procedure)
        "233560009",  # Percutaneous intraperitoneal fetal blood transfusion (procedure)
        "233561008",  # Percutaneous intravascular fetal blood transfusion (procedure)
        "236892008",  # Removal of displaced intrauterine contraceptive device from peritoneal cavity (procedure)
        "236894009",  # Frozen embryo transfer (procedure)
        "236912008",  # Gamete intrafallopian transfer (procedure)
        "236913003",  # Fallopian replacement of egg with delayed insemination (procedure)
        "236914009",  # Zygote intrafallopian transfer (procedure)
        "236915005",  # Tubal embryo transfer (procedure)
        "236952005",  # Therapeutic drainage of amniotic fluid by single aspiration (procedure)
        "236953000",  # Therapeutic drainage of amniotic fluid by indwelling catheter (procedure)
        "236954006",  # Diagnostic amniocentesis with fluid replacement (procedure)
        "236955007",  # Hyperalimentation of fetus through amniotic cavity (procedure)
        "236956008",  # Amnioinfusion (procedure)
        "236966000",  # Cervical ripening with Prostaglandin E2 (procedure)
        "236974004",  # Instrumental delivery (procedure)
        "236975003",  # Nonrotational forceps delivery (procedure)
        "236976002",  # Outlet forceps delivery (procedure)
        "236977006",  # Forceps delivery, face to pubes (procedure)
        "236978001",  # Forceps delivery to the aftercoming head (procedure)
        "236980007",  # Groin traction at breech delivery (procedure)
        "236981006",  # Lovset's maneuver (procedure)
        "236982004",  # Delivery of the after coming head (procedure)
        "236983009",  # Burns Marshall maneuver (procedure)
        "236984003",  # Mauriceau Smellie Veit maneuver (procedure)
        "236985002",  # Emergency lower segment cesarean section (procedure)
        "236986001",  # Emergency upper segment cesarean section (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "236988000",  # Elective cesarean hysterectomy (procedure)
        "236989008",  # Abdominal delivery for shoulder dystocia (procedure)
        "236990004",  # Postmortem cesarean section (procedure)
        "236992007",  # Right mediolateral episiotomy (procedure)
        "236993002",  # Vacuum dilation of cervix (procedure)
        "236994008",  # Placental delivery procedure (procedure)
        "237017007",  # Injection of prostaglandin into myometrium (procedure)
        "237025009",  # Repair of right mediolateral episiotomy (procedure)
        "237311001",  # Breech delivery (procedure)
        "240278000",  # External cephalic version (procedure)
        "240284002",  # Multifetal pregnancy reduction (procedure)
        "240285001",  # Percutaneous insertion of fetal shunt (procedure)
        "240286000",  # Percutaneous insertion of ventriculoamniotic shunt (procedure)
        "240288004",  # Percutaneous insertion of peritoneal-amniotic shunt (procedure)
        "240289007",  # Percutaneous aspiration of fetal lesion (procedure)
        "240290003",  # Percutaneous aspiration of fetal ascites (procedure)
        "240291004",  # Percutaneous aspiration of fetal pleural effusion (procedure)
        "240292006",  # Percutaneous aspiration of fetal pericardial effusion (procedure)
        "24068006",  # Hysterectomy for removal of hydatidiform mole (procedure)
        "243773009",  # Fetal blood sampling (procedure)
        "243774003",  # Transvaginal fetal blood sampling (procedure)
        "25296001",  # Delivery by Scanzoni maneuver (procedure)
        "25828002",  # Mid forceps delivery with episiotomy (procedure)
        "259863001",  # Removal of Shirodkar cerclage from cervix (procedure)
        "26313002",  # Delivery by vacuum extraction with episiotomy (procedure)
        "265082003",  # Laparoscopic oocyte recovery (procedure)
        "265635006",  # Diagnostic aspiration of amniotic cavity (procedure)
        "265639000",  # Midforceps delivery without rotation (procedure)
        "265642006",  # Obstetric monitoring (regime/therapy)
        "26578004",  # Hysterotomy with removal of hydatidiform mole (procedure)
        "274042004",  # Transvaginal removal of coil (procedure)
        "274130007",  # Emergency cesarean section (procedure)
        "274973002",  # Dilation of cervix uteri and curettage for termination of pregnancy (procedure)
        "275168001",  # Neville-Barnes forceps delivery (procedure)
        "275169009",  # Simpson's forceps delivery (procedure)
        "275222009",  # Transcervical sampling of chorionic villus (procedure)
        "275223004",  # Transabdominal chorionic villus biopsy (procedure)
        "275811000",  # Diaphragm fit (procedure)
        "28107008",  # Wright's obstetrical version (procedure)
        "28136008",  # Removal of intraluminal foreign body from uterus without incision (procedure)
        "281568006",  # Fetal heart monitoring (regime/therapy)
        "281570002",  # Fetal heart monitoring in labor (regime/therapy)
        "28379004",  # Dilation and curettage of uterus after abortion (procedure)
        "285409006",  # Medical termination of pregnancy (procedure)
        "285416007",  # Resuture of episiotomy (procedure)
        "285417003",  # Resuture of episiotomy dehiscence (procedure)
        "28542003",  # Wigand-Martin maneuver (procedure)
        "285434006",  # Insertion of abortifacient suppository (procedure)
        "287954004",  # Intrauterine exchange transfusion (procedure)
        "288042004",  # Hysterectomy and fetus removal (procedure)
        "288045002",  # Injection of substance into amniotic cavity for termination of pregnancy (procedure)
        "288193006",  # Supervision - normal delivery (procedure)
        "288194000",  # Routine episiotomy and repair (procedure)
        "28860009",  # Prague maneuver (procedure)
        "29613008",  # Delivery by double application of forceps (procedure)
        "301806003",  # Insertion of subcutaneous contraceptive in arm (procedure)
        "301807007",  # Removal of subcutaneous contraceptive (procedure)
        "302375005",  # Operative termination of pregnancy (procedure)
        "302382009",  # Breech extraction with internal podalic version (procedure)
        "302383004",  # Forceps delivery (procedure)
        "302384005",  # Controlled cord traction of placenta (procedure)
        "303720006",  # Sampling of chorionic villus (procedure)
        "30476003",  # Barton's forceps delivery (procedure)
        "306727001",  # Breech presentation, delivery, no version (procedure)
        "309877008",  # Intrapartum cardiotochogram monitoring (regime/therapy)
        "315308008",  # Dilatation of cervix for delivery (procedure)
        "33807004",  # Internal and combined version with extraction (procedure)
        "359940006",  # Partial breech extraction (procedure)
        "359943008",  # Partial breech delivery (procedure)
        "36708009",  # External fetal monitor surveillance (regime/therapy)
        "384729004",  # Delivery of vertex presentation (procedure)
        "384730009",  # Delivery of cephalic presentation (procedure)
        "38479009",  # Frank breech delivery (procedure)
        "386639001",  # Termination of pregnancy (procedure)
        "386641000",  # Therapeutic abortion procedure (procedure)
        "387615001",  # Removal of ectopic pregnancy from fallopian tube (procedure)
        "387616000",  # Fimbrial extraction of tubal pregnancy (procedure)
        "387673001",  # Nonstress test (regime/therapy)
        "387678005",  # External obstetrical version (procedure)
        "387709005",  # Removal of ectopic fetus from abdominal cavity (procedure)
        "387710000",  # Removal of extrauterine ectopic fetus (procedure)
        "391896006",  # Therapeutic abortion by aspiration curettage (procedure)
        "391897002",  # Aspiration curettage of uterus for termination of pregnancy (procedure)
        "391998006",  # Dilation and curettage of uterus after delivery (procedure)
        "392000009",  # Hysterotomy for retained placenta (procedure)
        "398221005",  # Fetal pH monitoring (regime/therapy)
        "40219000",  # Delivery by Malstrom's extraction with episiotomy (procedure)
        "40704000",  # Wright's obstetrical version with extraction (procedure)
        "40792007",  # Kristeller maneuver (procedure)
        "408804006",  # Continuous fetal heart monitoring during labor (regime/therapy)
        "408805007",  # Intermittent fetal heart monitoring during labor (regime/therapy)
        "408806008",  # Fetal heart monitoring using hand held Doppler (regime/therapy)
        "408807004",  # Fetal heart monitoring using fetal scalp electrode (regime/therapy)
        "408808009",  # Fetal heart monitoring using ultrasound transducer (regime/therapy)
        "408819007",  # Delivery of placenta by maternal effort (procedure)
        "408840008",  # Extra-amniotic termination of pregnancy (procedure)
        "408845003",  # Fetal heart monitoring with Pinard stethoscope (regime/therapy)
        "41058005",  # Salpingectomy for tubal ectopic pregnancy by abdominal approach (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "416055001",  # Total breech extraction (procedure)
        "417121007",  # Breech extraction (procedure)
        "41780001",  # Removal of external fetal monitor (procedure)
        "425707001",  # Serial drainage of amniotic fluid (procedure)
        "425939004",  # Endoscopic serial drainage of amniotic fluid (procedure)
        "426287008",  # Amniocentesis for possible neural tube defect (procedure)
        "426737000",  # Endoscopic serial drainage of amniotic fluid for twin-twin transfusion syndrome (procedure)
        "427230007",  # Amniocentesis for possible chromosomal abnormality (procedure)
        "427320001",  # Serial drainage of amniotic fluid for twin-twin transfusion syndrome (procedure)
        "429613009",  # Percutaneous insertion of tracheal plug for congenital diaphragmatic hernia of fetus (procedure)
        "429737004",  # Percutaneous laser ablation of lesion of fetus (procedure)
        "43096003",  # Insertion of laminaria into vagina (procedure)
        "433153009",  # Chorionic villus sampling using obstetric ultrasound guidance (procedure)
        "440148001",  # Partial excision of uterus for removal of interstitial ectopic pregnancy (procedure)
        "440668008",  # Replacement of implantable contraceptive capsule (procedure)
        "442490009",  # Removal of etonogestrel implant (procedure)
        "443005005",  # Diagnostic amniocentesis using ultrasound guidance (procedure)
        "445865006",  # Sampling of fetal blood using ultrasound guidance (procedure)
        "445912000",  # Excision of fallopian tube and surgical removal of ectopic pregnancy (procedure)
        "446135006",  # Amnioinfusion using ultrasound guidance (procedure)
        "446341008",  # Sampling of fetal blood and biopsy of fetus (procedure)
        "447214008",  # Ligation of umbilical cord of fetus (procedure)
        "447771005",  # Abdominal hysterectomy and excision of periuterine tissue (procedure)
        "447972007",  # Medical termination of pregnancy using prostaglandin (procedure)
        "448543003",  # Transmyometrial transfer of embryo to uterus (procedure)
        "4504004",  # Potter's obstetrical version with extraction (procedure)
        "450483001",  # Cesarean section through inverted T shaped incision of uterus (procedure)
        "450484007",  # Cesarean section through J shaped incision of uterus (procedure)
        "450563004",  # Laparoscopic excision of ectopic ovarian pregnancy (procedure)
        "450798003",  # Delivery of occipitoposterior presentation (procedure)
        "450836008",  # Removal of etonogestrel radiopaque contraceptive implant (procedure)
        "45460008",  # Intrauterine transfusion (procedure)
        "45718005",  # Vaginal delivery with forceps including postpartum care (procedure)
        "46681009",  # Cerclage of cervix during pregnancy by abdominal approach (procedure)
        "46706006",  # Replacement of contraceptive intrauterine device (procedure)
        "472837007",  # Insertion of hormone releasing intrauterine contraceptive device (procedure)
        "472838002",  # Removal of hormone releasing intrauterine contraceptive device (procedure)
        "480571000119102",  # Doppler ultrasound velocimetry of umbilical artery of fetus (procedure)
        "48204000",  # Spontaneous unassisted delivery, medical personnel present (procedure)
        "5048009",  # External cephalic version with tocolysis (procedure)
        "5191001",  # Surgical treatment of missed abortion of second trimester (procedure)
        "52660002",  # Induced abortion following intra-amniotic injection with hysterotomy (procedure)
        "54973000",  # Total breech delivery with forceps to aftercoming head (procedure)
        "5556001",  # Manually assisted spontaneous delivery (procedure)
        "56620000",  # Delivery of placenta following delivery of infant outside of hospital (procedure)
        "57271003",  # Extraperitoneal cesarean section (procedure)
        "58391006",  # Diaphragm fitting with education (procedure)
        "58705005",  # Bracht maneuver (procedure)
        "61586001",  # Delivery by vacuum extraction (procedure)
        "61893009",  # Laparoscopic treatment of ectopic pregnancy with oophorectomy (procedure)
        "62508004",  # Mid forceps delivery (procedure)
        "62688006",  # Surgical treatment of spontaneous abortion of any trimester (procedure)
        "63407004",  # Episioproctotomy (procedure)
        "63596003",  # Laparoscopic treatment of ectopic pregnancy with salpingectomy (procedure)
        "65200003",  # Insertion of intrauterine contraceptive device (procedure)
        "65243006",  # Delivery by midwife (procedure)
        "65378009",  # Delayed repair of episiotomy (procedure)
        "6708002",  # Intrauterine cordocentesis (procedure)
        "69422002",  # Trial forceps delivery (procedure)
        "69621003",  # Surgical treatment of missed abortion of third trimester (procedure)
        "699999008",  # Obstetrical version with extraction (procedure)
        "700000006",  # Vaginal delivery of fetus (procedure)
        "700041001",  # Induced termination of pregnancy under unsafe conditions (procedure)
        "709004006",  # Emergency lower segment cesarean section with inverted T incision (procedure)
        "71166009",  # Forceps delivery with rotation of fetal head (procedure)
        "713117007",  # Monitoring fetal development (regime/therapy)
        "713120004",  # Counting fetal movement (regime/therapy)
        "714812005",  # Induced termination of pregnancy (procedure)
        "716100006",  # Doppler ultrasonography of vascular structure of fetal head (procedure)
        "72492007",  # Footling breech delivery (procedure)
        "725927001",  # Pronuclear transfer mitochondrial replacement therapy (procedure)
        "726556001",  # Maternal spindle transfer mitochondrial replacement therapy (procedure)
        "732970000",  # Controlled ovarian stimulation (procedure)
        "73341009",  # Removal of ectopic fetus from ovary without oophorectomy (procedure)
        "734275002",  # Delivery by outlet vacuum extraction (procedure)
        "734276001",  # Delivery by mid-vacuum extraction (procedure)
        "736018001",  # Elective upper segment cesarean section with bilateral tubal ligation (procedure)
        "736020003",  # Emergency upper segment cesarean section with bilateral tubal ligation (procedure)
        "736026009",  # Elective lower segment cesarean section with bilateral tubal ligation (procedure)
        "736118004",  # Emergency lower segment cesarean section with bilateral tubal ligation (procedure)
        "737099004",  # Abdominal hysterectomy with sacrocolpopexy using mesh (procedure)
        "75456002",  # Embryo transfer (procedure)
        "75928003",  # Pinard maneuver (procedure)
        "79575002",  # Removal of intrauterine pack (procedure)
        "81130000",  # Removal of intraligamentous ectopic pregnancy (procedure)
        "81855008",  # Internal fetal monitoring during labor (regime/therapy)
        "82688001",  # Removal of ectopic fetus (procedure)
        "83506002",  # Uterine contraction monitor removal (procedure)
        "84195007",  # Classical cesarean section (procedure)
        "84275009",  # Obstetrical hysterotomy (procedure)
        "85179000",  # Insertion of laminaria into cervix (procedure)
        "85403009",  # Delivery, medical personnel present (procedure)
        "85548006",  # Episiotomy (procedure)
        "87431004",  # Dilation of cervical canal (procedure)
        "87663001",  # External fetal monitor surveillance during multiple pregnancy (regime/therapy)
        "88144003",  # Removal of ectopic interstitial uterine pregnancy requiring total hysterectomy (procedure)
        "88362001",  # Removal of ectopic fetus from fallopian tube without salpingectomy (procedure)
        "89053004",  # Vaginal cesarean section (procedure)
        "89346004",  # Delivery by Kielland rotation (procedure)
        "89849000",  # High forceps delivery (procedure)
        "90438006",  # Delivery by Malstrom's extraction (procedure)
        "90442009",  # Cerclage of cervix during pregnancy by vaginal approach (procedure)
        "9221009",  # Surgical treatment of septic abortion (procedure)
        "9497004",  # Removal of laminaria from uterus (procedure)
        "9724000",  # Repair of current obstetric laceration of uterus (procedure)
    }

class ChemotherapyAdministration(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for the procedure of chemotherapy administration.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for chemotherapy administration.

    **Exclusion Criteria:** Excludes concepts that describe a procedure for topical chemotherapy.
    """

    VALUE_SET_NAME = "Chemotherapy Administration"
    OID = "2.16.840.1.113883.3.526.3.1027"
    DEFINITION_VERSION = "20240131"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "50391",  # Instillation(s) of therapeutic agent into renal pelvis and/or ureter through established nephrostomy, pyelostomy or ureterostomy tube (eg, anticarcinogenic or antifungal agent)
        "51720",  # Bladder instillation of anticarcinogenic agent (including retention time)
        "96401",  # Chemotherapy administration, subcutaneous or intramuscular; non-hormonal anti-neoplastic
        "96405",  # Chemotherapy administration; intralesional, up to and including 7 lesions
        "96406",  # Chemotherapy administration; intralesional, more than 7 lesions
        "96409",  # Chemotherapy administration; intravenous, push technique, single or initial substance/drug
        "96413",  # Chemotherapy administration, intravenous infusion technique; up to 1 hour, single or initial substance/drug
        "96416",  # Chemotherapy administration, intravenous infusion technique; initiation of prolonged chemotherapy infusion (more than 8 hours), requiring use of a portable or implantable pump
        "96420",  # Chemotherapy administration, intra-arterial; push technique
        "96422",  # Chemotherapy administration, intra-arterial; infusion technique, up to 1 hour
        "96425",  # Chemotherapy administration, intra-arterial; infusion technique, initiation of prolonged infusion (more than 8 hours), requiring the use of a portable or implantable pump
        "96440",  # Chemotherapy administration into pleural cavity, requiring and including thoracentesis
        "96446",  # Chemotherapy administration into the peritoneal cavity via implanted port or catheter
        "96450",  # Chemotherapy administration, into CNS (eg, intrathecal), requiring and including spinal puncture
        "96521",  # Refilling and maintenance of portable pump
        "96522",  # Refilling and maintenance of implantable pump or reservoir for drug delivery, systemic (eg, intravenous, intra-arterial)
        "96523",  # Irrigation of implanted venous access device for drug delivery systems
        "96542",  # Chemotherapy injection, subarachnoid or intraventricular via subcutaneous reservoir, single or multiple agents
        "96549",  # Unlisted chemotherapy procedure
    }

    HCPCSLEVELII = {
        "G0498",  # Chemotherapy administration, intravenous infusion technique; initiation of infusion in the office/clinic setting using office/clinic pump/supplies, with continuation of the infusion in the community setting (e.g., home, domiciliary, rest home or assisted living) using a portable pump provided by the office/clinic, includes follow up office/clinic visit at the conclusion of the infusion
    }

    SNOMEDCT = {
        "1141994004",  # Rituximab, cyclophosphamide, etoposide, vincristine, prednisone antineoplastic chemotherapy regimen (regime/therapy)
        "1172516002",  # High-dose chemotherapy with stem cell transplant (procedure)
        "12149006",  # Chemotherapy administration, subcutaneous, with local anesthesia (procedure)
        "1254741007",  # Intensification antineoplastic therapy regimen (regime/therapy)
        "1254742000",  # Delayed intensification antineoplastic therapy regimen (regime/therapy)
        "1255831008",  # Chemotherapy for malignant neoplastic disease using targeted agent (procedure)
        "1255834000",  # Administration of antineoplastic chemotherapy into peritoneal cavity (procedure)
        "1255904006",  # Administration of palliative antineoplastic agent (procedure)
        "1259200004",  # Neoadjuvant antineoplastic chemotherapy (procedure)
        "1290226008",  # Administration of antineoplastic agent via intravenous route (procedure)
        "1290228009",  # Administration of antineoplastic agent via subcutaneous route (procedure)
        "169396008",  # Radiomimetic chemotherapy (procedure)
        "169397004",  # Radiochemotherapy: local infiltration (procedure)
        "169398009",  # Radiochemotherapy: to stop spread (procedure)
        "169399001",  # Radiochemotherapy via oral route (procedure)
        "169400008",  # Radiochemotherapy via intravenous route (procedure)
        "169401007",  # Radiochemotherapy: for immunosuppression (procedure)
        "169402000",  # Radiochemotherapy: local artery (procedure)
        "169403005",  # Radiochemotherapy: into cavity (procedure)
        "171765004",  # Intrathecal injection of chemotherapeutic agent (procedure)
        "230857009",  # Injection of chemotherapeutic substance into subcutaneous reservoir (procedure)
        "265761001",  # Intramuscular chemotherapy (procedure)
        "266719004",  # Oral chemotherapy (procedure)
        "29391003",  # Chemotherapy administration into peritoneal cavity requiring paracentesis (procedure)
        "309698005",  # Continuous infusion of chemotherapy (procedure)
        "315601005",  # Ambulatory chemotherapy (procedure)
        "31652009",  # Intracavitary chemotherapy for malignant neoplasm (procedure)
        "363688001",  # Administration of antineoplastic agent (procedure)
        "367336001",  # Chemotherapy (procedure)
        "38216008",  # Infusion chemotherapy for malignant neoplasm (procedure)
        "394894008",  # Pre-operative chemotherapy (procedure)
        "394895009",  # Postoperative chemotherapy (procedure)
        "394934009",  # Combined preoperative chemotherapy and radiotherapy (procedure)
        "394935005",  # Combined postoperative chemotherapy and radiotherapy (procedure)
        "399042005",  # Chemotherapy cycle (procedure)
        "4114003",  # Parenteral chemotherapy for malignant neoplasm (procedure)
        "420965003",  # Administration of prophylactic antineoplastic agent (procedure)
        "450827009",  # Induction chemotherapy (procedure)
        "455281000124104",  # Dose-dense methotrexate-vinblastine-doxorubicin-cisplatin chemotherapy regimen (regime/therapy)
        "455291000124101",  # Dose-adjusted etoposide-prednisolone-oncovin-cyclophosphamide-hydroxydaunorubicin (regime/therapy)
        "461391000124102",  # Folinic acid-fluorouracil-oxaliplatin regimen (regime/therapy)
        "51534007",  # Oral chemotherapy for malignant neoplasm (procedure)
        "6872008",  # Perfusion chemotherapy for malignant neoplasm (procedure)
        "700496000",  # Postoperative administration of antineoplastic agent into peritoneal cavity (procedure)
        "703423002",  # Combined chemotherapy and radiation therapy (procedure)
        "70446004",  # Chemotherapy administration into pleural cavity requiring thoracentesis (procedure)
        "716872004",  # Antineoplastic chemotherapy regimen (regime/therapy)
        "720379009",  # Hepatic arterial infusion chemotherapy (procedure)
        "722472009",  # Rituximab, ifosfamide, carboplatin and etoposide chemotherapy regimen (regime/therapy)
        "722482005",  # Doxorubicin, bleomycin, vinblastine and dacarbazine chemotherapy regimen (regime/therapy)
        "722491009",  # Rituximab, cyclophosphamide, doxorubicin, vincristine and prednisone chemotherapy regimen (regime/therapy)
        "77738002",  # Local chemotherapy for malignant neoplasm (procedure)
        "816151001",  # Administration of consolidation antineoplastic agent (procedure)
        "830155008",  # Administration of hyperthermic antineoplastic agent into peritoneal cavity (procedure)
        "830156009",  # Administration of hyperthermic antineoplastic agent into thoracic cavity (procedure)
        "86036005",  # Intravenous chemotherapy administration by push technique (procedure)
        "870249004",  # Folinic acid, fluorouracil and irinotecan antineoplastic chemotherapy regimen (regime/therapy)
        "870250004",  # Fluorouracil, leucovorin, irinotecan and oxaliplatin antineoplastic chemotherapy regimen (regime/therapy)
        "87776000",  # Intra-arterial infusion of antineoplastic agent (procedure)
        "890093004",  # Administration of multiple antineoplastic agents (procedure)
        "897713009",  # Antineoplastic chemoimmunotherapy (regime/therapy)
    }

class DialysisServices(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for dialysis services.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent dialysis services.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Dialysis Services"
    OID = "2.16.840.1.113883.3.464.1003.109.12.1013"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "90935",  # Hemodialysis procedure with single evaluation by a physician or other qualified health care professional
        "90937",  # Hemodialysis procedure requiring repeated evaluation(s) with or without substantial revision of dialysis prescription
        "90945",  # Dialysis procedure other than hemodialysis (eg, peritoneal dialysis, hemofiltration, or other continuous renal replacement therapies), with single evaluation by a physician or other qualified health care professional
        "90947",  # Dialysis procedure other than hemodialysis (eg, peritoneal dialysis, hemofiltration, or other continuous renal replacement therapies) requiring repeated evaluations by a physician or other qualified health care professional, with or without substantial revision of dialysis prescription
        "90997",  # Hemoperfusion (eg, with activated charcoal or resin)
        "90999",  # Unlisted dialysis procedure, inpatient or outpatient
        "99512",  # Home visit for hemodialysis
    }

    HCPCSLEVELII = {
        "G0257",  # Unscheduled or emergency dialysis treatment for an esrd patient in a hospital outpatient department that is not certified as an esrd facility
        "S9339",  # Home therapy; peritoneal dialysis, administrative services, professional pharmacy services, care coordination and all necessary supplies and equipment (drugs and nursing visits coded separately), per diem
    }

    SNOMEDCT = {
        "108241001",  # Dialysis procedure (procedure)
        "10848006",  # Removal of peritoneal dialysis catheter (procedure)
        "11932001",  # Stabilizing hemodialysis (procedure)
        "14684005",  # Peritoneal dialysis excluding cannulation (procedure)
        "180273006",  # Removal of chronic ambulatory peritoneal dialysis catheter (procedure)
        "225230008",  # Chronic peritoneal dialysis (procedure)
        "225231007",  # Stab peritoneal dialysis (procedure)
        "233575001",  # Intermittent hemodialysis (procedure)
        "233576000",  # Intermittent hemodialysis with sequential ultrafiltration (procedure)
        "233577009",  # Intermittent hemodialysis with continuous ultrafiltration (procedure)
        "233578004",  # Continuous hemodialysis (procedure)
        "233579007",  # Continuous arteriovenous hemodialysis (procedure)
        "233580005",  # Continuous venovenous hemodialysis (procedure)
        "233581009",  # Hemofiltration (procedure)
        "233582002",  # Intermittent hemofiltration (procedure)
        "233583007",  # Continuous hemofiltration (procedure)
        "233584001",  # Continuous arteriovenous hemofiltration (procedure)
        "233585000",  # Continuous venovenous hemofiltration (procedure)
        "233586004",  # Hemodiafiltration (procedure)
        "233587008",  # Intermittent hemodiafiltration (procedure)
        "233588003",  # Continuous hemodiafiltration (procedure)
        "233589006",  # Continuous arteriovenous hemodiafiltration (procedure)
        "233590002",  # Continuous venovenous hemodiafiltration (procedure)
        "238316008",  # Aspiration of chronic ambulatory peritoneal dialysis catheter (procedure)
        "238317004",  # Flushing of chronic ambulatory peritoneal dialysis catheter (procedure)
        "238318009",  # Continuous ambulatory peritoneal dialysis (procedure)
        "238319001",  # Continuous cycling peritoneal dialysis (procedure)
        "238321006",  # Intermittent peritoneal dialysis (procedure)
        "238322004",  # Tidal peritoneal dialysis (procedure)
        "238323009",  # Night-time intermittent peritoneal dialysis (procedure)
        "265764009",  # Renal dialysis (procedure)
        "288182009",  # Extracorporeal kidney (procedure)
        "302497006",  # Hemodialysis (procedure)
        "34897002",  # Hemodialysis, maintenance in hospital (procedure)
        "427053002",  # Extracorporeal albumin hemodialysis (procedure)
        "428648006",  # Automated peritoneal dialysis (procedure)
        "439278006",  # Measurement of static venous pressure in hemodialysis vascular access (procedure)
        "439976001",  # Measurement of recirculation in hemodialysis vascular access (procedure)
        "57274006",  # Initial hemodialysis (procedure)
        "676002",  # Peritoneal dialysis including cannulation (procedure)
        "67970008",  # Hemodialysis, maintenance at home (procedure)
        "68341005",  # Hemodialysis, supervision at home (procedure)
        "71192002",  # Peritoneal dialysis (procedure)
        "714749008",  # Continuous renal replacement therapy (procedure)
    }

class KidneyTransplant(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a kidney transplant.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent kidney transplants.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Kidney Transplant"
    OID = "2.16.840.1.113883.3.464.1003.109.12.1012"
    DEFINITION_VERSION = "20190315"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "50360",  # Renal allotransplantation, implantation of graft; without recipient nephrectomy
        "50365",  # Renal allotransplantation, implantation of graft; with recipient nephrectomy
        "50380",  # Renal autotransplantation, reimplantation of kidney
    }

    HCPCSLEVELII = {
        "S2065",  # Simultaneous pancreas kidney transplantation
    }

    ICD10PCS = {
        "0TY00Z0",  # Transplantation of Right Kidney, Allogeneic, Open Approach
        "0TY00Z1",  # Transplantation of Right Kidney, Syngeneic, Open Approach
        "0TY00Z2",  # Transplantation of Right Kidney, Zooplastic, Open Approach
        "0TY10Z0",  # Transplantation of Left Kidney, Allogeneic, Open Approach
        "0TY10Z1",  # Transplantation of Left Kidney, Syngeneic, Open Approach
        "0TY10Z2",  # Transplantation of Left Kidney, Zooplastic, Open Approach
    }

    SNOMEDCT = {
        "52213001",  # Renal homotransplantation excluding donor and recipient nephrectomy (procedure)
        "175899003",  # Autotransplantation of kidney (procedure)
        "175901007",  # Live donor renal transplant (procedure)
        "175902000",  # Cadaveric renal transplant (procedure)
        "236138007",  # Xenograft renal transplant (procedure)
        "313030004",  # Donor renal transplantation (procedure)
        "6471000179103",  # Transplantation of kidney and pancreas (procedure)
        "70536003",  # Transplant of kidney (procedure)
        "711411006",  # Allotransplantation of kidney from beating heart cadaver (procedure)
        "711413009",  # Allotransplantation of kidney from non-beating heart cadaver (procedure)
        "765478004",  # Allotransplantation of left kidney (procedure)
        "765479007",  # Allotransplantation of right kidney (procedure)
        "782655004",  # Laparoscopic transplant of kidney using robotic assistance (procedure)
    }

class CesareanBirth(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to group concepts of procedures for cesarean section.

    **Data Element Scope:** This value set may use a model element related to procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure of a cesarean section.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Cesarean Birth"
    OID = "2.16.840.1.113883.3.117.1.7.1.282"
    DEFINITION_VERSION = "20210611"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "10D00Z0",  # Extraction of Products of Conception, High, Open Approach
        "10D00Z1",  # Extraction of Products of Conception, Low, Open Approach
        "10D00Z2",  # Extraction of Products of Conception, Extraperitoneal, Open Approach
    }

    SNOMEDCT = {
        "11466000",  # Cesarean section (procedure)
        "1197219005",  # Elective lower segment cesarean section through inverted T shaped incision of uterus (procedure)
        "177141003",  # Elective cesarean section (procedure)
        "177142005",  # Elective upper segment cesarean section (procedure)
        "177143000",  # Elective lower segment cesarean section (procedure)
        "236985002",  # Emergency lower segment cesarean section (procedure)
        "236986001",  # Emergency upper segment cesarean section (procedure)
        "236987005",  # Emergency cesarean hysterectomy (procedure)
        "236988000",  # Elective cesarean hysterectomy (procedure)
        "236989008",  # Abdominal delivery for shoulder dystocia (procedure)
        "274130007",  # Emergency cesarean section (procedure)
        "41059002",  # Cesarean hysterectomy (procedure)
        "450483001",  # Cesarean section through inverted T shaped incision of uterus (procedure)
        "450484007",  # Cesarean section through J shaped incision of uterus (procedure)
        "57271003",  # Extraperitoneal cesarean section (procedure)
        "709004006",  # Emergency lower segment cesarean section with inverted T incision (procedure)
        "736018001",  # Elective upper segment cesarean section with bilateral tubal ligation (procedure)
        "736020003",  # Emergency upper segment cesarean section with bilateral tubal ligation (procedure)
        "736026009",  # Elective lower segment cesarean section with bilateral tubal ligation (procedure)
        "736118004",  # Emergency lower segment cesarean section with bilateral tubal ligation (procedure)
        "788180009",  # Lower uterine segment cesarean section (procedure)
        "84195007",  # Classical cesarean section (procedure)
        "89053004",  # Vaginal cesarean section (procedure)
    }

class CabgSurgeries(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of coronary artery bypass graft (CABG) procedures.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for coronary artery bypass graft (CABG) surgery.

    **Exclusion Criteria:** Excludes concepts that represent a coronary artery bypass graft (CABG) performed using a scope.
    """

    VALUE_SET_NAME = "CABG Surgeries"
    OID = "2.16.840.1.113883.3.666.5.694"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0210083",  # Bypass Coronary Artery, One Artery from Coronary Artery with Zooplastic Tissue, Open Approach
        "0210088",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0210089",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021008C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021008F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021008W",  # Bypass Coronary Artery, One Artery from Aorta with Zooplastic Tissue, Open Approach
        "0210093",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0210098",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0210099",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021009C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021009F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021009W",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Venous Tissue, Open Approach
        "02100A3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02100A8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02100A9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02100AC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02100AF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02100AW",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Arterial Tissue, Open Approach
        "02100J3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Synthetic Substitute, Open Approach
        "02100J8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02100J9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02100JC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Synthetic Substitute, Open Approach
        "02100JF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Synthetic Substitute, Open Approach
        "02100JW",  # Bypass Coronary Artery, One Artery from Aorta with Synthetic Substitute, Open Approach
        "02100K3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02100K8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02100K9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02100KC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02100KF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02100KW",  # Bypass Coronary Artery, One Artery from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02100Z3",  # Bypass Coronary Artery, One Artery from Coronary Artery, Open Approach
        "02100Z8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary, Open Approach
        "02100Z9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary, Open Approach
        "02100ZC",  # Bypass Coronary Artery, One Artery from Thoracic Artery, Open Approach
        "02100ZF",  # Bypass Coronary Artery, One Artery from Abdominal Artery, Open Approach
        "0210344",  # Bypass Coronary Artery, One Artery from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02103D4",  # Bypass Coronary Artery, One Artery from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0210444",  # Bypass Coronary Artery, One Artery from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0210483",  # Bypass Coronary Artery, One Artery from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210488",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210489",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021048W",  # Bypass Coronary Artery, One Artery from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0210493",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0210498",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0210499",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049C",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049F",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021049W",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02104A3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104A8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104A9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104AW",  # Bypass Coronary Artery, One Artery from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02104D4",  # Bypass Coronary Artery, One Artery from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02104J3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104J8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104J9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104JW",  # Bypass Coronary Artery, One Artery from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02104K3",  # Bypass Coronary Artery, One Artery from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104K8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104K9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KC",  # Bypass Coronary Artery, One Artery from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KF",  # Bypass Coronary Artery, One Artery from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104KW",  # Bypass Coronary Artery, One Artery from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02104Z3",  # Bypass Coronary Artery, One Artery from Coronary Artery, Percutaneous Endoscopic Approach
        "02104Z8",  # Bypass Coronary Artery, One Artery from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02104Z9",  # Bypass Coronary Artery, One Artery from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02104ZC",  # Bypass Coronary Artery, One Artery from Thoracic Artery, Percutaneous Endoscopic Approach
        "02104ZF",  # Bypass Coronary Artery, One Artery from Abdominal Artery, Percutaneous Endoscopic Approach
        "0211083",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0211088",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0211089",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021108C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021108F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021108W",  # Bypass Coronary Artery, Two Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0211093",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0211098",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0211099",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021109C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021109F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021109W",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02110A3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02110A8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02110A9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02110AC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02110AF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02110AW",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02110J3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02110J8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02110J9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02110JC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02110JF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02110JW",  # Bypass Coronary Artery, Two Arteries from Aorta with Synthetic Substitute, Open Approach
        "02110K3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02110K8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02110K9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02110KC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02110KF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02110KW",  # Bypass Coronary Artery, Two Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02110Z3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery, Open Approach
        "02110Z8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary, Open Approach
        "02110Z9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary, Open Approach
        "02110ZC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery, Open Approach
        "02110ZF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery, Open Approach
        "0211344",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02113D4",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0211444",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0211483",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211488",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211489",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021148W",  # Bypass Coronary Artery, Two Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0211493",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0211498",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0211499",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149C",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149F",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021149W",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02114A3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114A8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114A9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114AW",  # Bypass Coronary Artery, Two Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02114D4",  # Bypass Coronary Artery, Two Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02114J3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114J8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114J9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114JW",  # Bypass Coronary Artery, Two Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02114K3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114K8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114K9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114KW",  # Bypass Coronary Artery, Two Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02114Z3",  # Bypass Coronary Artery, Two Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02114Z8",  # Bypass Coronary Artery, Two Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02114Z9",  # Bypass Coronary Artery, Two Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02114ZC",  # Bypass Coronary Artery, Two Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02114ZF",  # Bypass Coronary Artery, Two Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
        "0212083",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0212088",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0212089",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021208C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021208F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021208W",  # Bypass Coronary Artery, Three Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0212093",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0212098",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0212099",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021209C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021209F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021209W",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02120A3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02120A8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02120A9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02120AC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02120AF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02120AW",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02120J3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02120J8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02120J9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02120JC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02120JF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02120JW",  # Bypass Coronary Artery, Three Arteries from Aorta with Synthetic Substitute, Open Approach
        "02120K3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02120K8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02120K9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02120KC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02120KF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02120KW",  # Bypass Coronary Artery, Three Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02120Z3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery, Open Approach
        "02120Z8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary, Open Approach
        "02120Z9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary, Open Approach
        "02120ZC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery, Open Approach
        "02120ZF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery, Open Approach
        "0212344",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02123D4",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0212444",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0212483",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212488",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212489",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021248W",  # Bypass Coronary Artery, Three Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0212493",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0212498",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0212499",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249C",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249F",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021249W",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02124A3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124A8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124A9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124AW",  # Bypass Coronary Artery, Three Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02124D4",  # Bypass Coronary Artery, Three Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02124J3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124J8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124J9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124JW",  # Bypass Coronary Artery, Three Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02124K3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124K8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124K9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124KW",  # Bypass Coronary Artery, Three Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02124Z3",  # Bypass Coronary Artery, Three Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02124Z8",  # Bypass Coronary Artery, Three Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02124Z9",  # Bypass Coronary Artery, Three Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02124ZC",  # Bypass Coronary Artery, Three Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02124ZF",  # Bypass Coronary Artery, Three Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
        "0213083",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Zooplastic Tissue, Open Approach
        "0213088",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Zooplastic Tissue, Open Approach
        "0213089",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Zooplastic Tissue, Open Approach
        "021308C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Zooplastic Tissue, Open Approach
        "021308F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Zooplastic Tissue, Open Approach
        "021308W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Zooplastic Tissue, Open Approach
        "0213093",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Venous Tissue, Open Approach
        "0213098",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Venous Tissue, Open Approach
        "0213099",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Venous Tissue, Open Approach
        "021309C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Venous Tissue, Open Approach
        "021309F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Venous Tissue, Open Approach
        "021309W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Venous Tissue, Open Approach
        "02130A3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Arterial Tissue, Open Approach
        "02130A8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02130A9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Arterial Tissue, Open Approach
        "02130AC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Arterial Tissue, Open Approach
        "02130AF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Arterial Tissue, Open Approach
        "02130AW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Arterial Tissue, Open Approach
        "02130J3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Synthetic Substitute, Open Approach
        "02130J8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Synthetic Substitute, Open Approach
        "02130J9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Synthetic Substitute, Open Approach
        "02130JC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Synthetic Substitute, Open Approach
        "02130JF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Synthetic Substitute, Open Approach
        "02130JW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Synthetic Substitute, Open Approach
        "02130K3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Nonautologous Tissue Substitute, Open Approach
        "02130K8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02130K9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Open Approach
        "02130KC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Open Approach
        "02130KF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Open Approach
        "02130KW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Nonautologous Tissue Substitute, Open Approach
        "02130Z3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery, Open Approach
        "02130Z8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary, Open Approach
        "02130Z9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary, Open Approach
        "02130ZC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery, Open Approach
        "02130ZF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery, Open Approach
        "0213344",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Approach
        "02133D4",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Intraluminal Device, Percutaneous Approach
        "0213444",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0213483",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213488",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213489",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "021348W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Zooplastic Tissue, Percutaneous Endoscopic Approach
        "0213493",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0213498",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "0213499",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349C",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349F",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "021349W",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Venous Tissue, Percutaneous Endoscopic Approach
        "02134A3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134A8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134A9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134AW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Autologous Arterial Tissue, Percutaneous Endoscopic Approach
        "02134D4",  # Bypass Coronary Artery, Four or More Arteries from Coronary Vein with Intraluminal Device, Percutaneous Endoscopic Approach
        "02134J3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134J8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134J9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134JW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Synthetic Substitute, Percutaneous Endoscopic Approach
        "02134K3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134K8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134K9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134KW",  # Bypass Coronary Artery, Four or More Arteries from Aorta with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "02134Z3",  # Bypass Coronary Artery, Four or More Arteries from Coronary Artery, Percutaneous Endoscopic Approach
        "02134Z8",  # Bypass Coronary Artery, Four or More Arteries from Right Internal Mammary, Percutaneous Endoscopic Approach
        "02134Z9",  # Bypass Coronary Artery, Four or More Arteries from Left Internal Mammary, Percutaneous Endoscopic Approach
        "02134ZC",  # Bypass Coronary Artery, Four or More Arteries from Thoracic Artery, Percutaneous Endoscopic Approach
        "02134ZF",  # Bypass Coronary Artery, Four or More Arteries from Abdominal Artery, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "10190003",  # Aortocoronary bypass of four or more coronary arteries (procedure)
        "10326007",  # Coronary artery bypass with autogenous graft, three grafts (procedure)
        "119564002",  # Internal mammary-coronary artery bypass graft (procedure)
        "119565001",  # Coronary artery bypass graft, anastomosis of artery of thorax to coronary artery (procedure)
        "1287561002",  # Coronary artery bypass using robotic assistance (procedure)
        "14323007",  # Aortocoronary bypass of three coronary arteries (procedure)
        "17073005",  # Aortocoronary artery bypass graft with vein graft (procedure)
        "175021005",  # Allograft bypass of coronary artery (procedure)
        "175029007",  # Prosthetic bypass of coronary artery (procedure)
        "175036008",  # Revision of bypass for coronary artery (procedure)
        "175037004",  # Revision of bypass for one coronary artery (procedure)
        "175038009",  # Revision of bypass for two coronary arteries (procedure)
        "175039001",  # Revision of bypass for three coronary arteries (procedure)
        "175040004",  # Revision of bypass for four or more coronary arteries (procedure)
        "175066001",  # Percutaneous transluminal balloon angioplasty of bypass graft of coronary artery (procedure)
        "232717009",  # Coronary artery bypass grafting (procedure)
        "232719007",  # Coronary artery bypass graft x 1 (procedure)
        "232720001",  # Coronary artery bypass grafts x 2 (procedure)
        "232721002",  # Coronary artery bypass grafts x 3 (procedure)
        "232722009",  # Coronary artery bypass grafts x 4 (procedure)
        "232723004",  # Coronary artery bypass grafts x 5 (procedure)
        "232724005",  # Coronary artery bypass grafts greater than 5 (procedure)
        "252427007",  # Angiography of coronary artery bypass graft (procedure)
        "29819009",  # Aortocoronary bypass of one coronary artery (procedure)
        "3546002",  # Aortocoronary artery bypass graft with saphenous vein graft (procedure)
        "359597003",  # Single internal mammary-coronary artery bypass (procedure)
        "359601003",  # Coronary artery bypass with autogenous graft of internal mammary artery, single graft (procedure)
        "39202005",  # Coronary artery bypass with autogenous graft, four grafts (procedure)
        "39724006",  # Anastomosis of internal mammary artery to coronary artery, double vessel (procedure)
        "405598005",  # Aortocoronary artery bypass graft with two vein grafts (procedure)
        "405599002",  # Aortocoronary artery bypass graft with three vein grafts (procedure)
        "414088005",  # Emergency coronary artery bypass graft (procedure)
        "419132001",  # Minimally invasive direct coronary artery bypass (procedure)
        "438530000",  # Magnetic resonance angiography of coronary artery bypass graft (procedure)
        "440332008",  # Fluoroscopic angiography of left ventricle and coronary artery bypass graft with contrast (procedure)
        "450506009",  # Computed tomography angiography of coronary artery bypass graft with contrast (procedure)
        "67166004",  # Aortocoronary artery bypass graft (procedure)
        "736970002",  # Allograft bypass of four or more coronary arteries (procedure)
        "736971003",  # Allograft bypass of one coronary artery (procedure)
        "736972005",  # Allograft bypass of three coronary arteries (procedure)
        "736973000",  # Allograft bypass of two coronary arteries (procedure)
        "74371005",  # Coronary artery bypass with autogenous graft, two grafts (procedure)
        "82247006",  # Coronary artery bypass with autogenous graft, five grafts (procedure)
        "8876004",  # Aortocoronary artery bypass graft with prosthesis (procedure)
        "90487008",  # Aortocoronary bypass of two coronary arteries (procedure)
    }

class CabgOrPciProcedure(ValueSet):
    """
    **Clinical Focus:** CABG and PCI procedures

    **Data Element Scope:** CABG and PCI procedures

    **Inclusion Criteria:** N/A

    **Exclusion Criteria:** None
    """

    VALUE_SET_NAME = "CABG or PCI Procedure"
    OID = "2.16.840.1.113762.1.4.1138.566"
    DEFINITION_VERSION = "20180818"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33510",  # Coronary artery bypass, vein only; single coronary venous graft
        "33511",  # Coronary artery bypass, vein only; 2 coronary venous grafts
        "33512",  # Coronary artery bypass, vein only; 3 coronary venous grafts
        "33513",  # Coronary artery bypass, vein only; 4 coronary venous grafts
        "33514",  # Coronary artery bypass, vein only; 5 coronary venous grafts
        "33516",  # Coronary artery bypass, vein only; 6 or more coronary venous grafts
        "33517",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); single vein graft (List separately in addition to code for primary procedure)
        "33518",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 2 venous grafts (List separately in addition to code for primary procedure)
        "33519",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 3 venous grafts (List separately in addition to code for primary procedure)
        "33521",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 4 venous grafts (List separately in addition to code for primary procedure)
        "33522",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 5 venous grafts (List separately in addition to code for primary procedure)
        "33523",  # Coronary artery bypass, using venous graft(s) and arterial graft(s); 6 or more venous grafts (List separately in addition to code for primary procedure)
        "33533",  # Coronary artery bypass, using arterial graft(s); single arterial graft
        "33534",  # Coronary artery bypass, using arterial graft(s); 2 coronary arterial grafts
        "33535",  # Coronary artery bypass, using arterial graft(s); 3 coronary arterial grafts
        "33536",  # Coronary artery bypass, using arterial graft(s); 4 or more coronary arterial grafts
        "92920",  # Percutaneous transluminal coronary angioplasty; single major coronary artery or branch
        "92924",  # Percutaneous transluminal coronary atherectomy, with coronary angioplasty when performed; single major coronary artery or branch
        "92928",  # Percutaneous transcatheter placement of intracoronary stent(s), with coronary angioplasty when performed; single major coronary artery or branch
        "92933",  # Percutaneous transluminal coronary atherectomy, with intracoronary stent, with coronary angioplasty when performed; single major coronary artery or branch
        "92937",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of intracoronary stent, atherectomy and angioplasty, including distal protection when performed; single vessel
        "92941",  # Percutaneous transluminal revascularization of acute total/subtotal occlusion during acute myocardial infarction, coronary artery or coronary artery bypass graft, any combination of intracoronary stent, atherectomy and angioplasty, including aspiration thrombectomy when performed, single vessel
        "92943",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of intracoronary stent, atherectomy and angioplasty; single vessel
        "92972",  # Percutaneous transluminal coronary lithotripsy (List separately in addition to code for primary procedure)
    }

    HCPCSLEVELII = {
        "C9600",  # Percutaneous transcatheter placement of drug eluting intracoronary stent(s), with coronary angioplasty when performed; single major coronary artery or branch
        "C9601",  # Percutaneous transcatheter placement of drug-eluting intracoronary stent(s), with coronary angioplasty when performed; each additional branch of a major coronary artery (list separately in addition to code for primary procedure)
        "C9602",  # Percutaneous transluminal coronary atherectomy, with drug eluting intracoronary stent, with coronary angioplasty when performed; single major coronary artery or branch
        "C9603",  # Percutaneous transluminal coronary atherectomy, with drug-eluting intracoronary stent, with coronary angioplasty when performed; each additional branch of a major coronary artery (list separately in addition to code for primary procedure)
        "C9604",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of drug-eluting intracoronary stent, atherectomy and angioplasty, including distal protection when performed; single vessel
        "C9605",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of drug-eluting intracoronary stent, atherectomy and angioplasty, including distal protection when performed; each additional branch subtended by the bypass graft (list separately in addition to code for primary procedure)
        "C9606",  # Percutaneous transluminal revascularization of acute total/subtotal occlusion during acute myocardial infarction, coronary artery or coronary artery bypass graft, any combination of drug-eluting intracoronary stent, atherectomy and angioplasty, including aspiration thrombectomy when performed, single vessel
        "C9607",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of drug-eluting intracoronary stent, atherectomy and angioplasty; single vessel
        "C9608",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of drug-eluting intracoronary stent, atherectomy and angioplasty; each additional coronary artery, coronary artery branch, or bypass graft (list separately in addition to code for primary procedure)
        "S2205",  # Minimally invasive direct coronary artery bypass surgery involving mini-thoracotomy or mini-sternotomy surgery, performed under direct vision; using arterial graft(s), single coronary arterial graft
        "S2206",  # Minimally invasive direct coronary artery bypass surgery involving mini-thoracotomy or mini-sternotomy surgery, performed under direct vision; using arterial graft(s), two coronary arterial grafts
        "S2207",  # Minimally invasive direct coronary artery bypass surgery involving mini-thoracotomy or mini-sternotomy surgery, performed under direct vision; using venous graft only, single coronary venous graft
        "S2208",  # Minimally invasive direct coronary artery bypass surgery involving mini-thoracotomy or mini-sternotomy surgery, performed under direct vision; using single arterial and venous graft(s), single venous graft
        "S2209",  # Minimally invasive direct coronary artery bypass surgery involving mini-thoracotomy or mini-sternotomy surgery, performed under direct vision; using two arterial grafts and single venous graft
    }

class CarotidIntervention(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures for carotid surgical intervention.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure carotid intervention surgery.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Carotid Intervention"
    OID = "2.16.840.1.113883.3.117.1.7.1.204"
    DEFINITION_VERSION = "20240208"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "031H09G",  # Bypass Right Common Carotid Artery to Intracranial Artery with Autologous Venous Tissue, Open Approach
        "031H09J",  # Bypass Right Common Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031H09K",  # Bypass Right Common Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031H09Y",  # Bypass Right Common Carotid Artery to Upper Artery with Autologous Venous Tissue, Open Approach
        "031H0AG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Autologous Arterial Tissue, Open Approach
        "031H0AJ",  # Bypass Right Common Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031H0AK",  # Bypass Right Common Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031H0AY",  # Bypass Right Common Carotid Artery to Upper Artery with Autologous Arterial Tissue, Open Approach
        "031H0JG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Synthetic Substitute, Open Approach
        "031H0JJ",  # Bypass Right Common Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031H0JK",  # Bypass Right Common Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031H0JY",  # Bypass Right Common Carotid Artery to Upper Artery with Synthetic Substitute, Open Approach
        "031H0KG",  # Bypass Right Common Carotid Artery to Intracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031H0KJ",  # Bypass Right Common Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031H0KK",  # Bypass Right Common Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031H0KY",  # Bypass Right Common Carotid Artery to Upper Artery with Nonautologous Tissue Substitute, Open Approach
        "031H0ZG",  # Bypass Right Common Carotid Artery to Intracranial Artery, Open Approach
        "031H0ZJ",  # Bypass Right Common Carotid Artery to Right Extracranial Artery, Open Approach
        "031H0ZK",  # Bypass Right Common Carotid Artery to Left Extracranial Artery, Open Approach
        "031H0ZY",  # Bypass Right Common Carotid Artery to Upper Artery, Open Approach
        "031J09G",  # Bypass Left Common Carotid Artery to Intracranial Artery with Autologous Venous Tissue, Open Approach
        "031J09J",  # Bypass Left Common Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031J09K",  # Bypass Left Common Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031J09Y",  # Bypass Left Common Carotid Artery to Upper Artery with Autologous Venous Tissue, Open Approach
        "031J0AG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Autologous Arterial Tissue, Open Approach
        "031J0AJ",  # Bypass Left Common Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031J0AK",  # Bypass Left Common Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031J0AY",  # Bypass Left Common Carotid Artery to Upper Artery with Autologous Arterial Tissue, Open Approach
        "031J0JG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Synthetic Substitute, Open Approach
        "031J0JJ",  # Bypass Left Common Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031J0JK",  # Bypass Left Common Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031J0JY",  # Bypass Left Common Carotid Artery to Upper Artery with Synthetic Substitute, Open Approach
        "031J0KG",  # Bypass Left Common Carotid Artery to Intracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031J0KJ",  # Bypass Left Common Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031J0KK",  # Bypass Left Common Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031J0KY",  # Bypass Left Common Carotid Artery to Upper Artery with Nonautologous Tissue Substitute, Open Approach
        "031J0ZG",  # Bypass Left Common Carotid Artery to Intracranial Artery, Open Approach
        "031J0ZJ",  # Bypass Left Common Carotid Artery to Right Extracranial Artery, Open Approach
        "031J0ZK",  # Bypass Left Common Carotid Artery to Left Extracranial Artery, Open Approach
        "031J0ZY",  # Bypass Left Common Carotid Artery to Upper Artery, Open Approach
        "031K09J",  # Bypass Right Internal Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031K09K",  # Bypass Right Internal Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031K0AJ",  # Bypass Right Internal Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031K0AK",  # Bypass Right Internal Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031K0JJ",  # Bypass Right Internal Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031K0JK",  # Bypass Right Internal Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031K0KJ",  # Bypass Right Internal Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031K0KK",  # Bypass Right Internal Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031K0ZJ",  # Bypass Right Internal Carotid Artery to Right Extracranial Artery, Open Approach
        "031K0ZK",  # Bypass Right Internal Carotid Artery to Left Extracranial Artery, Open Approach
        "031L09J",  # Bypass Left Internal Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031L09K",  # Bypass Left Internal Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031L0AJ",  # Bypass Left Internal Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031L0AK",  # Bypass Left Internal Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031L0JJ",  # Bypass Left Internal Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031L0JK",  # Bypass Left Internal Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031L0KJ",  # Bypass Left Internal Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031L0KK",  # Bypass Left Internal Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031L0ZJ",  # Bypass Left Internal Carotid Artery to Right Extracranial Artery, Open Approach
        "031L0ZK",  # Bypass Left Internal Carotid Artery to Left Extracranial Artery, Open Approach
        "031M09J",  # Bypass Right External Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031M09K",  # Bypass Right External Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031M0AJ",  # Bypass Right External Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031M0AK",  # Bypass Right External Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031M0JJ",  # Bypass Right External Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031M0JK",  # Bypass Right External Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031M0KJ",  # Bypass Right External Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031M0KK",  # Bypass Right External Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031M0ZJ",  # Bypass Right External Carotid Artery to Right Extracranial Artery, Open Approach
        "031M0ZK",  # Bypass Right External Carotid Artery to Left Extracranial Artery, Open Approach
        "031N09J",  # Bypass Left External Carotid Artery to Right Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031N09K",  # Bypass Left External Carotid Artery to Left Extracranial Artery with Autologous Venous Tissue, Open Approach
        "031N0AJ",  # Bypass Left External Carotid Artery to Right Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031N0AK",  # Bypass Left External Carotid Artery to Left Extracranial Artery with Autologous Arterial Tissue, Open Approach
        "031N0JJ",  # Bypass Left External Carotid Artery to Right Extracranial Artery with Synthetic Substitute, Open Approach
        "031N0JK",  # Bypass Left External Carotid Artery to Left Extracranial Artery with Synthetic Substitute, Open Approach
        "031N0KJ",  # Bypass Left External Carotid Artery to Right Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031N0KK",  # Bypass Left External Carotid Artery to Left Extracranial Artery with Nonautologous Tissue Substitute, Open Approach
        "031N0ZJ",  # Bypass Left External Carotid Artery to Right Extracranial Artery, Open Approach
        "031N0ZK",  # Bypass Left External Carotid Artery to Left Extracranial Artery, Open Approach
        "035H0ZZ",  # Destruction of Right Common Carotid Artery, Open Approach
        "035H3ZZ",  # Destruction of Right Common Carotid Artery, Percutaneous Approach
        "035H4ZZ",  # Destruction of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "035J0ZZ",  # Destruction of Left Common Carotid Artery, Open Approach
        "035J3ZZ",  # Destruction of Left Common Carotid Artery, Percutaneous Approach
        "035J4ZZ",  # Destruction of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "035K0ZZ",  # Destruction of Right Internal Carotid Artery, Open Approach
        "035K3ZZ",  # Destruction of Right Internal Carotid Artery, Percutaneous Approach
        "035K4ZZ",  # Destruction of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "035L0ZZ",  # Destruction of Left Internal Carotid Artery, Open Approach
        "035L3ZZ",  # Destruction of Left Internal Carotid Artery, Percutaneous Approach
        "035L4ZZ",  # Destruction of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "035M0ZZ",  # Destruction of Right External Carotid Artery, Open Approach
        "035M3ZZ",  # Destruction of Right External Carotid Artery, Percutaneous Approach
        "035M4ZZ",  # Destruction of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "035N0ZZ",  # Destruction of Left External Carotid Artery, Open Approach
        "035N3ZZ",  # Destruction of Left External Carotid Artery, Percutaneous Approach
        "035N4ZZ",  # Destruction of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "037H04Z",  # Dilation of Right Common Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037H05Z",  # Dilation of Right Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037H06Z",  # Dilation of Right Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037H07Z",  # Dilation of Right Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037H0DZ",  # Dilation of Right Common Carotid Artery with Intraluminal Device, Open Approach
        "037H0EZ",  # Dilation of Right Common Carotid Artery with Two Intraluminal Devices, Open Approach
        "037H0FZ",  # Dilation of Right Common Carotid Artery with Three Intraluminal Devices, Open Approach
        "037H0GZ",  # Dilation of Right Common Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037H0ZZ",  # Dilation of Right Common Carotid Artery, Open Approach
        "037H34Z",  # Dilation of Right Common Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037H35Z",  # Dilation of Right Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037H36Z",  # Dilation of Right Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037H37Z",  # Dilation of Right Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037H3DZ",  # Dilation of Right Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037H3EZ",  # Dilation of Right Common Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037H3FZ",  # Dilation of Right Common Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037H3GZ",  # Dilation of Right Common Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037H3ZZ",  # Dilation of Right Common Carotid Artery, Percutaneous Approach
        "037H44Z",  # Dilation of Right Common Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037H45Z",  # Dilation of Right Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H46Z",  # Dilation of Right Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H47Z",  # Dilation of Right Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H4DZ",  # Dilation of Right Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037H4EZ",  # Dilation of Right Common Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H4FZ",  # Dilation of Right Common Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H4GZ",  # Dilation of Right Common Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037H4ZZ",  # Dilation of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "037J04Z",  # Dilation of Left Common Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037J05Z",  # Dilation of Left Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037J06Z",  # Dilation of Left Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037J07Z",  # Dilation of Left Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037J0DZ",  # Dilation of Left Common Carotid Artery with Intraluminal Device, Open Approach
        "037J0EZ",  # Dilation of Left Common Carotid Artery with Two Intraluminal Devices, Open Approach
        "037J0FZ",  # Dilation of Left Common Carotid Artery with Three Intraluminal Devices, Open Approach
        "037J0GZ",  # Dilation of Left Common Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037J0ZZ",  # Dilation of Left Common Carotid Artery, Open Approach
        "037J34Z",  # Dilation of Left Common Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037J35Z",  # Dilation of Left Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037J36Z",  # Dilation of Left Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037J37Z",  # Dilation of Left Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037J3DZ",  # Dilation of Left Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037J3EZ",  # Dilation of Left Common Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037J3FZ",  # Dilation of Left Common Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037J3GZ",  # Dilation of Left Common Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037J3ZZ",  # Dilation of Left Common Carotid Artery, Percutaneous Approach
        "037J44Z",  # Dilation of Left Common Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037J45Z",  # Dilation of Left Common Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J46Z",  # Dilation of Left Common Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J47Z",  # Dilation of Left Common Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J4DZ",  # Dilation of Left Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037J4EZ",  # Dilation of Left Common Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J4FZ",  # Dilation of Left Common Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J4GZ",  # Dilation of Left Common Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037J4ZZ",  # Dilation of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "037K04Z",  # Dilation of Right Internal Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037K05Z",  # Dilation of Right Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037K06Z",  # Dilation of Right Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037K07Z",  # Dilation of Right Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037K0DZ",  # Dilation of Right Internal Carotid Artery with Intraluminal Device, Open Approach
        "037K0EZ",  # Dilation of Right Internal Carotid Artery with Two Intraluminal Devices, Open Approach
        "037K0FZ",  # Dilation of Right Internal Carotid Artery with Three Intraluminal Devices, Open Approach
        "037K0GZ",  # Dilation of Right Internal Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037K0ZZ",  # Dilation of Right Internal Carotid Artery, Open Approach
        "037K34Z",  # Dilation of Right Internal Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037K35Z",  # Dilation of Right Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037K36Z",  # Dilation of Right Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037K37Z",  # Dilation of Right Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037K3DZ",  # Dilation of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037K3EZ",  # Dilation of Right Internal Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037K3FZ",  # Dilation of Right Internal Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037K3GZ",  # Dilation of Right Internal Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037K3ZZ",  # Dilation of Right Internal Carotid Artery, Percutaneous Approach
        "037K44Z",  # Dilation of Right Internal Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037K45Z",  # Dilation of Right Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K46Z",  # Dilation of Right Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K47Z",  # Dilation of Right Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K4DZ",  # Dilation of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037K4EZ",  # Dilation of Right Internal Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K4FZ",  # Dilation of Right Internal Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K4GZ",  # Dilation of Right Internal Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037K4ZZ",  # Dilation of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "037L04Z",  # Dilation of Left Internal Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037L05Z",  # Dilation of Left Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037L06Z",  # Dilation of Left Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037L07Z",  # Dilation of Left Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037L0DZ",  # Dilation of Left Internal Carotid Artery with Intraluminal Device, Open Approach
        "037L0EZ",  # Dilation of Left Internal Carotid Artery with Two Intraluminal Devices, Open Approach
        "037L0FZ",  # Dilation of Left Internal Carotid Artery with Three Intraluminal Devices, Open Approach
        "037L0GZ",  # Dilation of Left Internal Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037L0ZZ",  # Dilation of Left Internal Carotid Artery, Open Approach
        "037L34Z",  # Dilation of Left Internal Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037L35Z",  # Dilation of Left Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037L36Z",  # Dilation of Left Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037L37Z",  # Dilation of Left Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037L3DZ",  # Dilation of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037L3EZ",  # Dilation of Left Internal Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037L3FZ",  # Dilation of Left Internal Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037L3GZ",  # Dilation of Left Internal Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037L3ZZ",  # Dilation of Left Internal Carotid Artery, Percutaneous Approach
        "037L44Z",  # Dilation of Left Internal Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037L45Z",  # Dilation of Left Internal Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L46Z",  # Dilation of Left Internal Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L47Z",  # Dilation of Left Internal Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L4DZ",  # Dilation of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037L4EZ",  # Dilation of Left Internal Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L4FZ",  # Dilation of Left Internal Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L4GZ",  # Dilation of Left Internal Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037L4ZZ",  # Dilation of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "037M04Z",  # Dilation of Right External Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037M05Z",  # Dilation of Right External Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037M06Z",  # Dilation of Right External Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037M07Z",  # Dilation of Right External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037M0DZ",  # Dilation of Right External Carotid Artery with Intraluminal Device, Open Approach
        "037M0EZ",  # Dilation of Right External Carotid Artery with Two Intraluminal Devices, Open Approach
        "037M0FZ",  # Dilation of Right External Carotid Artery with Three Intraluminal Devices, Open Approach
        "037M0GZ",  # Dilation of Right External Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037M0ZZ",  # Dilation of Right External Carotid Artery, Open Approach
        "037M34Z",  # Dilation of Right External Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037M35Z",  # Dilation of Right External Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037M36Z",  # Dilation of Right External Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037M37Z",  # Dilation of Right External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037M3DZ",  # Dilation of Right External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037M3EZ",  # Dilation of Right External Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037M3FZ",  # Dilation of Right External Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037M3GZ",  # Dilation of Right External Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037M3ZZ",  # Dilation of Right External Carotid Artery, Percutaneous Approach
        "037M44Z",  # Dilation of Right External Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037M45Z",  # Dilation of Right External Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M46Z",  # Dilation of Right External Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M47Z",  # Dilation of Right External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M4DZ",  # Dilation of Right External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037M4EZ",  # Dilation of Right External Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M4FZ",  # Dilation of Right External Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M4GZ",  # Dilation of Right External Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037M4ZZ",  # Dilation of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "037N04Z",  # Dilation of Left External Carotid Artery with Drug-eluting Intraluminal Device, Open Approach
        "037N05Z",  # Dilation of Left External Carotid Artery with Two Drug-eluting Intraluminal Devices, Open Approach
        "037N06Z",  # Dilation of Left External Carotid Artery with Three Drug-eluting Intraluminal Devices, Open Approach
        "037N07Z",  # Dilation of Left External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Open Approach
        "037N0DZ",  # Dilation of Left External Carotid Artery with Intraluminal Device, Open Approach
        "037N0EZ",  # Dilation of Left External Carotid Artery with Two Intraluminal Devices, Open Approach
        "037N0FZ",  # Dilation of Left External Carotid Artery with Three Intraluminal Devices, Open Approach
        "037N0GZ",  # Dilation of Left External Carotid Artery with Four or More Intraluminal Devices, Open Approach
        "037N0ZZ",  # Dilation of Left External Carotid Artery, Open Approach
        "037N34Z",  # Dilation of Left External Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "037N35Z",  # Dilation of Left External Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037N36Z",  # Dilation of Left External Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037N37Z",  # Dilation of Left External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "037N3DZ",  # Dilation of Left External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "037N3EZ",  # Dilation of Left External Carotid Artery with Two Intraluminal Devices, Percutaneous Approach
        "037N3FZ",  # Dilation of Left External Carotid Artery with Three Intraluminal Devices, Percutaneous Approach
        "037N3GZ",  # Dilation of Left External Carotid Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "037N3ZZ",  # Dilation of Left External Carotid Artery, Percutaneous Approach
        "037N44Z",  # Dilation of Left External Carotid Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "037N45Z",  # Dilation of Left External Carotid Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N46Z",  # Dilation of Left External Carotid Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N47Z",  # Dilation of Left External Carotid Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N4DZ",  # Dilation of Left External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "037N4EZ",  # Dilation of Left External Carotid Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N4FZ",  # Dilation of Left External Carotid Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N4GZ",  # Dilation of Left External Carotid Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "037N4ZZ",  # Dilation of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "039H00Z",  # Drainage of Right Common Carotid Artery with Drainage Device, Open Approach
        "039H0ZX",  # Drainage of Right Common Carotid Artery, Open Approach, Diagnostic
        "039H0ZZ",  # Drainage of Right Common Carotid Artery, Open Approach
        "039H30Z",  # Drainage of Right Common Carotid Artery with Drainage Device, Percutaneous Approach
        "039H3ZX",  # Drainage of Right Common Carotid Artery, Percutaneous Approach, Diagnostic
        "039H3ZZ",  # Drainage of Right Common Carotid Artery, Percutaneous Approach
        "039H40Z",  # Drainage of Right Common Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039H4ZX",  # Drainage of Right Common Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039H4ZZ",  # Drainage of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "039J00Z",  # Drainage of Left Common Carotid Artery with Drainage Device, Open Approach
        "039J0ZX",  # Drainage of Left Common Carotid Artery, Open Approach, Diagnostic
        "039J0ZZ",  # Drainage of Left Common Carotid Artery, Open Approach
        "039J30Z",  # Drainage of Left Common Carotid Artery with Drainage Device, Percutaneous Approach
        "039J3ZX",  # Drainage of Left Common Carotid Artery, Percutaneous Approach, Diagnostic
        "039J3ZZ",  # Drainage of Left Common Carotid Artery, Percutaneous Approach
        "039J40Z",  # Drainage of Left Common Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039J4ZX",  # Drainage of Left Common Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039J4ZZ",  # Drainage of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "039K00Z",  # Drainage of Right Internal Carotid Artery with Drainage Device, Open Approach
        "039K0ZX",  # Drainage of Right Internal Carotid Artery, Open Approach, Diagnostic
        "039K0ZZ",  # Drainage of Right Internal Carotid Artery, Open Approach
        "039K30Z",  # Drainage of Right Internal Carotid Artery with Drainage Device, Percutaneous Approach
        "039K3ZX",  # Drainage of Right Internal Carotid Artery, Percutaneous Approach, Diagnostic
        "039K3ZZ",  # Drainage of Right Internal Carotid Artery, Percutaneous Approach
        "039K40Z",  # Drainage of Right Internal Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039K4ZX",  # Drainage of Right Internal Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039K4ZZ",  # Drainage of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "039L00Z",  # Drainage of Left Internal Carotid Artery with Drainage Device, Open Approach
        "039L0ZX",  # Drainage of Left Internal Carotid Artery, Open Approach, Diagnostic
        "039L0ZZ",  # Drainage of Left Internal Carotid Artery, Open Approach
        "039L30Z",  # Drainage of Left Internal Carotid Artery with Drainage Device, Percutaneous Approach
        "039L3ZX",  # Drainage of Left Internal Carotid Artery, Percutaneous Approach, Diagnostic
        "039L3ZZ",  # Drainage of Left Internal Carotid Artery, Percutaneous Approach
        "039L40Z",  # Drainage of Left Internal Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039L4ZX",  # Drainage of Left Internal Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039L4ZZ",  # Drainage of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "039M00Z",  # Drainage of Right External Carotid Artery with Drainage Device, Open Approach
        "039M0ZX",  # Drainage of Right External Carotid Artery, Open Approach, Diagnostic
        "039M0ZZ",  # Drainage of Right External Carotid Artery, Open Approach
        "039M30Z",  # Drainage of Right External Carotid Artery with Drainage Device, Percutaneous Approach
        "039M3ZX",  # Drainage of Right External Carotid Artery, Percutaneous Approach, Diagnostic
        "039M3ZZ",  # Drainage of Right External Carotid Artery, Percutaneous Approach
        "039M40Z",  # Drainage of Right External Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039M4ZX",  # Drainage of Right External Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039M4ZZ",  # Drainage of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "039N00Z",  # Drainage of Left External Carotid Artery with Drainage Device, Open Approach
        "039N0ZX",  # Drainage of Left External Carotid Artery, Open Approach, Diagnostic
        "039N0ZZ",  # Drainage of Left External Carotid Artery, Open Approach
        "039N30Z",  # Drainage of Left External Carotid Artery with Drainage Device, Percutaneous Approach
        "039N3ZX",  # Drainage of Left External Carotid Artery, Percutaneous Approach, Diagnostic
        "039N3ZZ",  # Drainage of Left External Carotid Artery, Percutaneous Approach
        "039N40Z",  # Drainage of Left External Carotid Artery with Drainage Device, Percutaneous Endoscopic Approach
        "039N4ZX",  # Drainage of Left External Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "039N4ZZ",  # Drainage of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03BH0ZX",  # Excision of Right Common Carotid Artery, Open Approach, Diagnostic
        "03BH0ZZ",  # Excision of Right Common Carotid Artery, Open Approach
        "03BH3ZX",  # Excision of Right Common Carotid Artery, Percutaneous Approach, Diagnostic
        "03BH3ZZ",  # Excision of Right Common Carotid Artery, Percutaneous Approach
        "03BH4ZX",  # Excision of Right Common Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BH4ZZ",  # Excision of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03BJ0ZX",  # Excision of Left Common Carotid Artery, Open Approach, Diagnostic
        "03BJ0ZZ",  # Excision of Left Common Carotid Artery, Open Approach
        "03BJ3ZX",  # Excision of Left Common Carotid Artery, Percutaneous Approach, Diagnostic
        "03BJ3ZZ",  # Excision of Left Common Carotid Artery, Percutaneous Approach
        "03BJ4ZX",  # Excision of Left Common Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BJ4ZZ",  # Excision of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03BK0ZX",  # Excision of Right Internal Carotid Artery, Open Approach, Diagnostic
        "03BK0ZZ",  # Excision of Right Internal Carotid Artery, Open Approach
        "03BK3ZX",  # Excision of Right Internal Carotid Artery, Percutaneous Approach, Diagnostic
        "03BK3ZZ",  # Excision of Right Internal Carotid Artery, Percutaneous Approach
        "03BK4ZX",  # Excision of Right Internal Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BK4ZZ",  # Excision of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03BL0ZX",  # Excision of Left Internal Carotid Artery, Open Approach, Diagnostic
        "03BL0ZZ",  # Excision of Left Internal Carotid Artery, Open Approach
        "03BL3ZX",  # Excision of Left Internal Carotid Artery, Percutaneous Approach, Diagnostic
        "03BL3ZZ",  # Excision of Left Internal Carotid Artery, Percutaneous Approach
        "03BL4ZX",  # Excision of Left Internal Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BL4ZZ",  # Excision of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03BM0ZX",  # Excision of Right External Carotid Artery, Open Approach, Diagnostic
        "03BM0ZZ",  # Excision of Right External Carotid Artery, Open Approach
        "03BM3ZX",  # Excision of Right External Carotid Artery, Percutaneous Approach, Diagnostic
        "03BM3ZZ",  # Excision of Right External Carotid Artery, Percutaneous Approach
        "03BM4ZX",  # Excision of Right External Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BM4ZZ",  # Excision of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03BN0ZX",  # Excision of Left External Carotid Artery, Open Approach, Diagnostic
        "03BN0ZZ",  # Excision of Left External Carotid Artery, Open Approach
        "03BN3ZX",  # Excision of Left External Carotid Artery, Percutaneous Approach, Diagnostic
        "03BN3ZZ",  # Excision of Left External Carotid Artery, Percutaneous Approach
        "03BN4ZX",  # Excision of Left External Carotid Artery, Percutaneous Endoscopic Approach, Diagnostic
        "03BN4ZZ",  # Excision of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03CH0ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Open Approach
        "03CH3Z7",  # Extirpation of Matter from Right Common Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CH3ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Percutaneous Approach
        "03CH4ZZ",  # Extirpation of Matter from Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03CJ0ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Open Approach
        "03CJ3Z7",  # Extirpation of Matter from Left Common Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CJ3ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Percutaneous Approach
        "03CJ4ZZ",  # Extirpation of Matter from Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03CK0ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Open Approach
        "03CK3Z7",  # Extirpation of Matter from Right Internal Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CK3ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Percutaneous Approach
        "03CK4ZZ",  # Extirpation of Matter from Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03CL0ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Open Approach
        "03CL3Z7",  # Extirpation of Matter from Left Internal Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CL3ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Percutaneous Approach
        "03CL4ZZ",  # Extirpation of Matter from Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03CM0ZZ",  # Extirpation of Matter from Right External Carotid Artery, Open Approach
        "03CM3Z7",  # Extirpation of Matter from Right External Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CM3ZZ",  # Extirpation of Matter from Right External Carotid Artery, Percutaneous Approach
        "03CM4ZZ",  # Extirpation of Matter from Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03CN0ZZ",  # Extirpation of Matter from Left External Carotid Artery, Open Approach
        "03CN3Z7",  # Extirpation of Matter from Left External Carotid Artery using Stent Retriever, Percutaneous Approach
        "03CN3ZZ",  # Extirpation of Matter from Left External Carotid Artery, Percutaneous Approach
        "03CN4ZZ",  # Extirpation of Matter from Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03HH03Z",  # Insertion of Infusion Device into Right Common Carotid Artery, Open Approach
        "03HH0DZ",  # Insertion of Intraluminal Device into Right Common Carotid Artery, Open Approach
        "03HH33Z",  # Insertion of Infusion Device into Right Common Carotid Artery, Percutaneous Approach
        "03HH3DZ",  # Insertion of Intraluminal Device into Right Common Carotid Artery, Percutaneous Approach
        "03HH43Z",  # Insertion of Infusion Device into Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03HH4DZ",  # Insertion of Intraluminal Device into Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03HJ03Z",  # Insertion of Infusion Device into Left Common Carotid Artery, Open Approach
        "03HJ0DZ",  # Insertion of Intraluminal Device into Left Common Carotid Artery, Open Approach
        "03HJ33Z",  # Insertion of Infusion Device into Left Common Carotid Artery, Percutaneous Approach
        "03HJ3DZ",  # Insertion of Intraluminal Device into Left Common Carotid Artery, Percutaneous Approach
        "03HJ43Z",  # Insertion of Infusion Device into Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03HJ4DZ",  # Insertion of Intraluminal Device into Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03HK03Z",  # Insertion of Infusion Device into Right Internal Carotid Artery, Open Approach
        "03HK0DZ",  # Insertion of Intraluminal Device into Right Internal Carotid Artery, Open Approach
        "03HK0MZ",  # Insertion of Stimulator Lead into Right Internal Carotid Artery, Open Approach
        "03HK33Z",  # Insertion of Infusion Device into Right Internal Carotid Artery, Percutaneous Approach
        "03HK3DZ",  # Insertion of Intraluminal Device into Right Internal Carotid Artery, Percutaneous Approach
        "03HK3MZ",  # Insertion of Stimulator Lead into Right Internal Carotid Artery, Percutaneous Approach
        "03HK43Z",  # Insertion of Infusion Device into Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HK4DZ",  # Insertion of Intraluminal Device into Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HK4MZ",  # Insertion of Stimulator Lead into Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HL03Z",  # Insertion of Infusion Device into Left Internal Carotid Artery, Open Approach
        "03HL0DZ",  # Insertion of Intraluminal Device into Left Internal Carotid Artery, Open Approach
        "03HL0MZ",  # Insertion of Stimulator Lead into Left Internal Carotid Artery, Open Approach
        "03HL33Z",  # Insertion of Infusion Device into Left Internal Carotid Artery, Percutaneous Approach
        "03HL3DZ",  # Insertion of Intraluminal Device into Left Internal Carotid Artery, Percutaneous Approach
        "03HL3MZ",  # Insertion of Stimulator Lead into Left Internal Carotid Artery, Percutaneous Approach
        "03HL43Z",  # Insertion of Infusion Device into Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HL4DZ",  # Insertion of Intraluminal Device into Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HL4MZ",  # Insertion of Stimulator Lead into Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03HM03Z",  # Insertion of Infusion Device into Right External Carotid Artery, Open Approach
        "03HM0DZ",  # Insertion of Intraluminal Device into Right External Carotid Artery, Open Approach
        "03HM33Z",  # Insertion of Infusion Device into Right External Carotid Artery, Percutaneous Approach
        "03HM3DZ",  # Insertion of Intraluminal Device into Right External Carotid Artery, Percutaneous Approach
        "03HM43Z",  # Insertion of Infusion Device into Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03HM4DZ",  # Insertion of Intraluminal Device into Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03HN03Z",  # Insertion of Infusion Device into Left External Carotid Artery, Open Approach
        "03HN0DZ",  # Insertion of Intraluminal Device into Left External Carotid Artery, Open Approach
        "03HN33Z",  # Insertion of Infusion Device into Left External Carotid Artery, Percutaneous Approach
        "03HN3DZ",  # Insertion of Intraluminal Device into Left External Carotid Artery, Percutaneous Approach
        "03HN43Z",  # Insertion of Infusion Device into Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03HN4DZ",  # Insertion of Intraluminal Device into Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03LH0BZ",  # Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LH0CZ",  # Occlusion of Right Common Carotid Artery with Extraluminal Device, Open Approach
        "03LH0DZ",  # Occlusion of Right Common Carotid Artery with Intraluminal Device, Open Approach
        "03LH0ZZ",  # Occlusion of Right Common Carotid Artery, Open Approach
        "03LH3BZ",  # Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LH3CZ",  # Occlusion of Right Common Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LH3DZ",  # Occlusion of Right Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LH3ZZ",  # Occlusion of Right Common Carotid Artery, Percutaneous Approach
        "03LH4BZ",  # Occlusion of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LH4CZ",  # Occlusion of Right Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LH4DZ",  # Occlusion of Right Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LH4ZZ",  # Occlusion of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03LJ0BZ",  # Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LJ0CZ",  # Occlusion of Left Common Carotid Artery with Extraluminal Device, Open Approach
        "03LJ0DZ",  # Occlusion of Left Common Carotid Artery with Intraluminal Device, Open Approach
        "03LJ0ZZ",  # Occlusion of Left Common Carotid Artery, Open Approach
        "03LJ3BZ",  # Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LJ3CZ",  # Occlusion of Left Common Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LJ3DZ",  # Occlusion of Left Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LJ3ZZ",  # Occlusion of Left Common Carotid Artery, Percutaneous Approach
        "03LJ4BZ",  # Occlusion of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LJ4CZ",  # Occlusion of Left Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LJ4DZ",  # Occlusion of Left Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LJ4ZZ",  # Occlusion of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03LK0BZ",  # Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LK0CZ",  # Occlusion of Right Internal Carotid Artery with Extraluminal Device, Open Approach
        "03LK0DZ",  # Occlusion of Right Internal Carotid Artery with Intraluminal Device, Open Approach
        "03LK0ZZ",  # Occlusion of Right Internal Carotid Artery, Open Approach
        "03LK3BZ",  # Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LK3CZ",  # Occlusion of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LK3DZ",  # Occlusion of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LK3ZZ",  # Occlusion of Right Internal Carotid Artery, Percutaneous Approach
        "03LK4BZ",  # Occlusion of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LK4CZ",  # Occlusion of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LK4DZ",  # Occlusion of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LK4ZZ",  # Occlusion of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03LL0BZ",  # Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LL0CZ",  # Occlusion of Left Internal Carotid Artery with Extraluminal Device, Open Approach
        "03LL0DZ",  # Occlusion of Left Internal Carotid Artery with Intraluminal Device, Open Approach
        "03LL0ZZ",  # Occlusion of Left Internal Carotid Artery, Open Approach
        "03LL3BZ",  # Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LL3CZ",  # Occlusion of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LL3DZ",  # Occlusion of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LL3ZZ",  # Occlusion of Left Internal Carotid Artery, Percutaneous Approach
        "03LL4BZ",  # Occlusion of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LL4CZ",  # Occlusion of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LL4DZ",  # Occlusion of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LL4ZZ",  # Occlusion of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03LM0BZ",  # Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LM0CZ",  # Occlusion of Right External Carotid Artery with Extraluminal Device, Open Approach
        "03LM0DZ",  # Occlusion of Right External Carotid Artery with Intraluminal Device, Open Approach
        "03LM0ZZ",  # Occlusion of Right External Carotid Artery, Open Approach
        "03LM3BZ",  # Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LM3CZ",  # Occlusion of Right External Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LM3DZ",  # Occlusion of Right External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LM3ZZ",  # Occlusion of Right External Carotid Artery, Percutaneous Approach
        "03LM4BZ",  # Occlusion of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LM4CZ",  # Occlusion of Right External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LM4DZ",  # Occlusion of Right External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LM4ZZ",  # Occlusion of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03LN0BZ",  # Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03LN0CZ",  # Occlusion of Left External Carotid Artery with Extraluminal Device, Open Approach
        "03LN0DZ",  # Occlusion of Left External Carotid Artery with Intraluminal Device, Open Approach
        "03LN0ZZ",  # Occlusion of Left External Carotid Artery, Open Approach
        "03LN3BZ",  # Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03LN3CZ",  # Occlusion of Left External Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03LN3DZ",  # Occlusion of Left External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03LN3ZZ",  # Occlusion of Left External Carotid Artery, Percutaneous Approach
        "03LN4BZ",  # Occlusion of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03LN4CZ",  # Occlusion of Left External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03LN4DZ",  # Occlusion of Left External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03LN4ZZ",  # Occlusion of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03NH0ZZ",  # Release Right Common Carotid Artery, Open Approach
        "03NH3ZZ",  # Release Right Common Carotid Artery, Percutaneous Approach
        "03NH4ZZ",  # Release Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03NJ0ZZ",  # Release Left Common Carotid Artery, Open Approach
        "03NJ3ZZ",  # Release Left Common Carotid Artery, Percutaneous Approach
        "03NJ4ZZ",  # Release Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03NK0ZZ",  # Release Right Internal Carotid Artery, Open Approach
        "03NK3ZZ",  # Release Right Internal Carotid Artery, Percutaneous Approach
        "03NK4ZZ",  # Release Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03NL0ZZ",  # Release Left Internal Carotid Artery, Open Approach
        "03NL3ZZ",  # Release Left Internal Carotid Artery, Percutaneous Approach
        "03NL4ZZ",  # Release Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03NM0ZZ",  # Release Right External Carotid Artery, Open Approach
        "03NM3ZZ",  # Release Right External Carotid Artery, Percutaneous Approach
        "03NM4ZZ",  # Release Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03NN0ZZ",  # Release Left External Carotid Artery, Open Approach
        "03NN3ZZ",  # Release Left External Carotid Artery, Percutaneous Approach
        "03NN4ZZ",  # Release Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03QH0ZZ",  # Repair Right Common Carotid Artery, Open Approach
        "03QH3ZZ",  # Repair Right Common Carotid Artery, Percutaneous Approach
        "03QH4ZZ",  # Repair Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03QJ0ZZ",  # Repair Left Common Carotid Artery, Open Approach
        "03QJ3ZZ",  # Repair Left Common Carotid Artery, Percutaneous Approach
        "03QJ4ZZ",  # Repair Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03QK0ZZ",  # Repair Right Internal Carotid Artery, Open Approach
        "03QK3ZZ",  # Repair Right Internal Carotid Artery, Percutaneous Approach
        "03QK4ZZ",  # Repair Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03QL0ZZ",  # Repair Left Internal Carotid Artery, Open Approach
        "03QL3ZZ",  # Repair Left Internal Carotid Artery, Percutaneous Approach
        "03QL4ZZ",  # Repair Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03QM0ZZ",  # Repair Right External Carotid Artery, Open Approach
        "03QM3ZZ",  # Repair Right External Carotid Artery, Percutaneous Approach
        "03QM4ZZ",  # Repair Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03QN0ZZ",  # Repair Left External Carotid Artery, Open Approach
        "03QN3ZZ",  # Repair Left External Carotid Artery, Percutaneous Approach
        "03QN4ZZ",  # Repair Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03RH07Z",  # Replacement of Right Common Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RH0JZ",  # Replacement of Right Common Carotid Artery with Synthetic Substitute, Open Approach
        "03RH0KZ",  # Replacement of Right Common Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RH47Z",  # Replacement of Right Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RH4JZ",  # Replacement of Right Common Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RH4KZ",  # Replacement of Right Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RJ07Z",  # Replacement of Left Common Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RJ0JZ",  # Replacement of Left Common Carotid Artery with Synthetic Substitute, Open Approach
        "03RJ0KZ",  # Replacement of Left Common Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RJ47Z",  # Replacement of Left Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RJ4JZ",  # Replacement of Left Common Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RJ4KZ",  # Replacement of Left Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RK07Z",  # Replacement of Right Internal Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RK0JZ",  # Replacement of Right Internal Carotid Artery with Synthetic Substitute, Open Approach
        "03RK0KZ",  # Replacement of Right Internal Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RK47Z",  # Replacement of Right Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RK4JZ",  # Replacement of Right Internal Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RK4KZ",  # Replacement of Right Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RL07Z",  # Replacement of Left Internal Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RL0JZ",  # Replacement of Left Internal Carotid Artery with Synthetic Substitute, Open Approach
        "03RL0KZ",  # Replacement of Left Internal Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RL47Z",  # Replacement of Left Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RL4JZ",  # Replacement of Left Internal Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RL4KZ",  # Replacement of Left Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RM07Z",  # Replacement of Right External Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RM0JZ",  # Replacement of Right External Carotid Artery with Synthetic Substitute, Open Approach
        "03RM0KZ",  # Replacement of Right External Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RM47Z",  # Replacement of Right External Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RM4JZ",  # Replacement of Right External Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RM4KZ",  # Replacement of Right External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RN07Z",  # Replacement of Left External Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03RN0JZ",  # Replacement of Left External Carotid Artery with Synthetic Substitute, Open Approach
        "03RN0KZ",  # Replacement of Left External Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03RN47Z",  # Replacement of Left External Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03RN4JZ",  # Replacement of Left External Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03RN4KZ",  # Replacement of Left External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03SH0ZZ",  # Reposition Right Common Carotid Artery, Open Approach
        "03SH3ZZ",  # Reposition Right Common Carotid Artery, Percutaneous Approach
        "03SH4ZZ",  # Reposition Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03SJ0ZZ",  # Reposition Left Common Carotid Artery, Open Approach
        "03SJ3ZZ",  # Reposition Left Common Carotid Artery, Percutaneous Approach
        "03SJ4ZZ",  # Reposition Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03SK0ZZ",  # Reposition Right Internal Carotid Artery, Open Approach
        "03SK3ZZ",  # Reposition Right Internal Carotid Artery, Percutaneous Approach
        "03SK4ZZ",  # Reposition Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03SL0ZZ",  # Reposition Left Internal Carotid Artery, Open Approach
        "03SL3ZZ",  # Reposition Left Internal Carotid Artery, Percutaneous Approach
        "03SL4ZZ",  # Reposition Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03SM0ZZ",  # Reposition Right External Carotid Artery, Open Approach
        "03SM3ZZ",  # Reposition Right External Carotid Artery, Percutaneous Approach
        "03SM4ZZ",  # Reposition Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03SN0ZZ",  # Reposition Left External Carotid Artery, Open Approach
        "03SN3ZZ",  # Reposition Left External Carotid Artery, Percutaneous Approach
        "03SN4ZZ",  # Reposition Left External Carotid Artery, Percutaneous Endoscopic Approach
        "03UH07Z",  # Supplement Right Common Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UH0JZ",  # Supplement Right Common Carotid Artery with Synthetic Substitute, Open Approach
        "03UH0KZ",  # Supplement Right Common Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UH37Z",  # Supplement Right Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UH3JZ",  # Supplement Right Common Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UH3KZ",  # Supplement Right Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UH47Z",  # Supplement Right Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UH4JZ",  # Supplement Right Common Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UH4KZ",  # Supplement Right Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UJ07Z",  # Supplement Left Common Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UJ0JZ",  # Supplement Left Common Carotid Artery with Synthetic Substitute, Open Approach
        "03UJ0KZ",  # Supplement Left Common Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UJ37Z",  # Supplement Left Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UJ3JZ",  # Supplement Left Common Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UJ3KZ",  # Supplement Left Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UJ47Z",  # Supplement Left Common Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UJ4JZ",  # Supplement Left Common Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UJ4KZ",  # Supplement Left Common Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UK07Z",  # Supplement Right Internal Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UK0JZ",  # Supplement Right Internal Carotid Artery with Synthetic Substitute, Open Approach
        "03UK0KZ",  # Supplement Right Internal Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UK37Z",  # Supplement Right Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UK3JZ",  # Supplement Right Internal Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UK3KZ",  # Supplement Right Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UK47Z",  # Supplement Right Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UK4JZ",  # Supplement Right Internal Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UK4KZ",  # Supplement Right Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UL07Z",  # Supplement Left Internal Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UL0JZ",  # Supplement Left Internal Carotid Artery with Synthetic Substitute, Open Approach
        "03UL0KZ",  # Supplement Left Internal Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UL37Z",  # Supplement Left Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UL3JZ",  # Supplement Left Internal Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UL3KZ",  # Supplement Left Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UL47Z",  # Supplement Left Internal Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UL4JZ",  # Supplement Left Internal Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UL4KZ",  # Supplement Left Internal Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UM07Z",  # Supplement Right External Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UM0JZ",  # Supplement Right External Carotid Artery with Synthetic Substitute, Open Approach
        "03UM0KZ",  # Supplement Right External Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UM37Z",  # Supplement Right External Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UM3JZ",  # Supplement Right External Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UM3KZ",  # Supplement Right External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UM47Z",  # Supplement Right External Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UM4JZ",  # Supplement Right External Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UM4KZ",  # Supplement Right External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UN07Z",  # Supplement Left External Carotid Artery with Autologous Tissue Substitute, Open Approach
        "03UN0JZ",  # Supplement Left External Carotid Artery with Synthetic Substitute, Open Approach
        "03UN0KZ",  # Supplement Left External Carotid Artery with Nonautologous Tissue Substitute, Open Approach
        "03UN37Z",  # Supplement Left External Carotid Artery with Autologous Tissue Substitute, Percutaneous Approach
        "03UN3JZ",  # Supplement Left External Carotid Artery with Synthetic Substitute, Percutaneous Approach
        "03UN3KZ",  # Supplement Left External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Approach
        "03UN47Z",  # Supplement Left External Carotid Artery with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03UN4JZ",  # Supplement Left External Carotid Artery with Synthetic Substitute, Percutaneous Endoscopic Approach
        "03UN4KZ",  # Supplement Left External Carotid Artery with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "03VH0BZ",  # Restriction of Right Common Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VH0CZ",  # Restriction of Right Common Carotid Artery with Extraluminal Device, Open Approach
        "03VH0DZ",  # Restriction of Right Common Carotid Artery with Intraluminal Device, Open Approach
        "03VH0ZZ",  # Restriction of Right Common Carotid Artery, Open Approach
        "03VH3BZ",  # Restriction of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VH3CZ",  # Restriction of Right Common Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VH3DZ",  # Restriction of Right Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VH3ZZ",  # Restriction of Right Common Carotid Artery, Percutaneous Approach
        "03VH4BZ",  # Restriction of Right Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VH4CZ",  # Restriction of Right Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VH4DZ",  # Restriction of Right Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VH4ZZ",  # Restriction of Right Common Carotid Artery, Percutaneous Endoscopic Approach
        "03VJ0BZ",  # Restriction of Left Common Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VJ0CZ",  # Restriction of Left Common Carotid Artery with Extraluminal Device, Open Approach
        "03VJ0DZ",  # Restriction of Left Common Carotid Artery with Intraluminal Device, Open Approach
        "03VJ0ZZ",  # Restriction of Left Common Carotid Artery, Open Approach
        "03VJ3BZ",  # Restriction of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VJ3CZ",  # Restriction of Left Common Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VJ3DZ",  # Restriction of Left Common Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VJ3ZZ",  # Restriction of Left Common Carotid Artery, Percutaneous Approach
        "03VJ4BZ",  # Restriction of Left Common Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VJ4CZ",  # Restriction of Left Common Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VJ4DZ",  # Restriction of Left Common Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VJ4ZZ",  # Restriction of Left Common Carotid Artery, Percutaneous Endoscopic Approach
        "03VK0BZ",  # Restriction of Right Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VK0CZ",  # Restriction of Right Internal Carotid Artery with Extraluminal Device, Open Approach
        "03VK0DZ",  # Restriction of Right Internal Carotid Artery with Intraluminal Device, Open Approach
        "03VK0ZZ",  # Restriction of Right Internal Carotid Artery, Open Approach
        "03VK3BZ",  # Restriction of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VK3CZ",  # Restriction of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VK3DZ",  # Restriction of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VK3ZZ",  # Restriction of Right Internal Carotid Artery, Percutaneous Approach
        "03VK4BZ",  # Restriction of Right Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VK4CZ",  # Restriction of Right Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VK4DZ",  # Restriction of Right Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VK4ZZ",  # Restriction of Right Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03VL0BZ",  # Restriction of Left Internal Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VL0CZ",  # Restriction of Left Internal Carotid Artery with Extraluminal Device, Open Approach
        "03VL0DZ",  # Restriction of Left Internal Carotid Artery with Intraluminal Device, Open Approach
        "03VL0ZZ",  # Restriction of Left Internal Carotid Artery, Open Approach
        "03VL3BZ",  # Restriction of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VL3CZ",  # Restriction of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VL3DZ",  # Restriction of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VL3ZZ",  # Restriction of Left Internal Carotid Artery, Percutaneous Approach
        "03VL4BZ",  # Restriction of Left Internal Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VL4CZ",  # Restriction of Left Internal Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VL4DZ",  # Restriction of Left Internal Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VL4ZZ",  # Restriction of Left Internal Carotid Artery, Percutaneous Endoscopic Approach
        "03VM0BZ",  # Restriction of Right External Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VM0CZ",  # Restriction of Right External Carotid Artery with Extraluminal Device, Open Approach
        "03VM0DZ",  # Restriction of Right External Carotid Artery with Intraluminal Device, Open Approach
        "03VM0ZZ",  # Restriction of Right External Carotid Artery, Open Approach
        "03VM3BZ",  # Restriction of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VM3CZ",  # Restriction of Right External Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VM3DZ",  # Restriction of Right External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VM3ZZ",  # Restriction of Right External Carotid Artery, Percutaneous Approach
        "03VM4BZ",  # Restriction of Right External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VM4CZ",  # Restriction of Right External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VM4DZ",  # Restriction of Right External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VM4ZZ",  # Restriction of Right External Carotid Artery, Percutaneous Endoscopic Approach
        "03VN0BZ",  # Restriction of Left External Carotid Artery with Bioactive Intraluminal Device, Open Approach
        "03VN0CZ",  # Restriction of Left External Carotid Artery with Extraluminal Device, Open Approach
        "03VN0DZ",  # Restriction of Left External Carotid Artery with Intraluminal Device, Open Approach
        "03VN0ZZ",  # Restriction of Left External Carotid Artery, Open Approach
        "03VN3BZ",  # Restriction of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Approach
        "03VN3CZ",  # Restriction of Left External Carotid Artery with Extraluminal Device, Percutaneous Approach
        "03VN3DZ",  # Restriction of Left External Carotid Artery with Intraluminal Device, Percutaneous Approach
        "03VN3ZZ",  # Restriction of Left External Carotid Artery, Percutaneous Approach
        "03VN4BZ",  # Restriction of Left External Carotid Artery with Bioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "03VN4CZ",  # Restriction of Left External Carotid Artery with Extraluminal Device, Percutaneous Endoscopic Approach
        "03VN4DZ",  # Restriction of Left External Carotid Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "03VN4ZZ",  # Restriction of Left External Carotid Artery, Percutaneous Endoscopic Approach
        "0G560ZZ",  # Destruction of Left Carotid Body, Open Approach
        "0G563ZZ",  # Destruction of Left Carotid Body, Percutaneous Approach
        "0G564ZZ",  # Destruction of Left Carotid Body, Percutaneous Endoscopic Approach
        "0G570ZZ",  # Destruction of Right Carotid Body, Open Approach
        "0G573ZZ",  # Destruction of Right Carotid Body, Percutaneous Approach
        "0G574ZZ",  # Destruction of Right Carotid Body, Percutaneous Endoscopic Approach
        "0G580ZZ",  # Destruction of Bilateral Carotid Bodies, Open Approach
        "0G583ZZ",  # Destruction of Bilateral Carotid Bodies, Percutaneous Approach
        "0G584ZZ",  # Destruction of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0G9600Z",  # Drainage of Left Carotid Body with Drainage Device, Open Approach
        "0G960ZX",  # Drainage of Left Carotid Body, Open Approach, Diagnostic
        "0G960ZZ",  # Drainage of Left Carotid Body, Open Approach
        "0G9630Z",  # Drainage of Left Carotid Body with Drainage Device, Percutaneous Approach
        "0G963ZX",  # Drainage of Left Carotid Body, Percutaneous Approach, Diagnostic
        "0G963ZZ",  # Drainage of Left Carotid Body, Percutaneous Approach
        "0G9640Z",  # Drainage of Left Carotid Body with Drainage Device, Percutaneous Endoscopic Approach
        "0G964ZX",  # Drainage of Left Carotid Body, Percutaneous Endoscopic Approach, Diagnostic
        "0G964ZZ",  # Drainage of Left Carotid Body, Percutaneous Endoscopic Approach
        "0G9700Z",  # Drainage of Right Carotid Body with Drainage Device, Open Approach
        "0G970ZX",  # Drainage of Right Carotid Body, Open Approach, Diagnostic
        "0G970ZZ",  # Drainage of Right Carotid Body, Open Approach
        "0G9730Z",  # Drainage of Right Carotid Body with Drainage Device, Percutaneous Approach
        "0G973ZX",  # Drainage of Right Carotid Body, Percutaneous Approach, Diagnostic
        "0G973ZZ",  # Drainage of Right Carotid Body, Percutaneous Approach
        "0G9740Z",  # Drainage of Right Carotid Body with Drainage Device, Percutaneous Endoscopic Approach
        "0G974ZX",  # Drainage of Right Carotid Body, Percutaneous Endoscopic Approach, Diagnostic
        "0G974ZZ",  # Drainage of Right Carotid Body, Percutaneous Endoscopic Approach
        "0G9800Z",  # Drainage of Bilateral Carotid Bodies with Drainage Device, Open Approach
        "0G980ZX",  # Drainage of Bilateral Carotid Bodies, Open Approach, Diagnostic
        "0G980ZZ",  # Drainage of Bilateral Carotid Bodies, Open Approach
        "0G9830Z",  # Drainage of Bilateral Carotid Bodies with Drainage Device, Percutaneous Approach
        "0G983ZX",  # Drainage of Bilateral Carotid Bodies, Percutaneous Approach, Diagnostic
        "0G983ZZ",  # Drainage of Bilateral Carotid Bodies, Percutaneous Approach
        "0G9840Z",  # Drainage of Bilateral Carotid Bodies with Drainage Device, Percutaneous Endoscopic Approach
        "0G984ZX",  # Drainage of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach, Diagnostic
        "0G984ZZ",  # Drainage of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GB60ZX",  # Excision of Left Carotid Body, Open Approach, Diagnostic
        "0GB60ZZ",  # Excision of Left Carotid Body, Open Approach
        "0GB63ZX",  # Excision of Left Carotid Body, Percutaneous Approach, Diagnostic
        "0GB63ZZ",  # Excision of Left Carotid Body, Percutaneous Approach
        "0GB64ZX",  # Excision of Left Carotid Body, Percutaneous Endoscopic Approach, Diagnostic
        "0GB64ZZ",  # Excision of Left Carotid Body, Percutaneous Endoscopic Approach
        "0GB70ZX",  # Excision of Right Carotid Body, Open Approach, Diagnostic
        "0GB70ZZ",  # Excision of Right Carotid Body, Open Approach
        "0GB73ZX",  # Excision of Right Carotid Body, Percutaneous Approach, Diagnostic
        "0GB73ZZ",  # Excision of Right Carotid Body, Percutaneous Approach
        "0GB74ZX",  # Excision of Right Carotid Body, Percutaneous Endoscopic Approach, Diagnostic
        "0GB74ZZ",  # Excision of Right Carotid Body, Percutaneous Endoscopic Approach
        "0GB80ZX",  # Excision of Bilateral Carotid Bodies, Open Approach, Diagnostic
        "0GB80ZZ",  # Excision of Bilateral Carotid Bodies, Open Approach
        "0GB83ZX",  # Excision of Bilateral Carotid Bodies, Percutaneous Approach, Diagnostic
        "0GB83ZZ",  # Excision of Bilateral Carotid Bodies, Percutaneous Approach
        "0GB84ZX",  # Excision of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach, Diagnostic
        "0GB84ZZ",  # Excision of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GC60ZZ",  # Extirpation of Matter from Left Carotid Body, Open Approach
        "0GC63ZZ",  # Extirpation of Matter from Left Carotid Body, Percutaneous Approach
        "0GC64ZZ",  # Extirpation of Matter from Left Carotid Body, Percutaneous Endoscopic Approach
        "0GC70ZZ",  # Extirpation of Matter from Right Carotid Body, Open Approach
        "0GC73ZZ",  # Extirpation of Matter from Right Carotid Body, Percutaneous Approach
        "0GC74ZZ",  # Extirpation of Matter from Right Carotid Body, Percutaneous Endoscopic Approach
        "0GC80ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Open Approach
        "0GC83ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Percutaneous Approach
        "0GC84ZZ",  # Extirpation of Matter from Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GN60ZZ",  # Release Left Carotid Body, Open Approach
        "0GN63ZZ",  # Release Left Carotid Body, Percutaneous Approach
        "0GN64ZZ",  # Release Left Carotid Body, Percutaneous Endoscopic Approach
        "0GN70ZZ",  # Release Right Carotid Body, Open Approach
        "0GN73ZZ",  # Release Right Carotid Body, Percutaneous Approach
        "0GN74ZZ",  # Release Right Carotid Body, Percutaneous Endoscopic Approach
        "0GN80ZZ",  # Release Bilateral Carotid Bodies, Open Approach
        "0GN83ZZ",  # Release Bilateral Carotid Bodies, Percutaneous Approach
        "0GN84ZZ",  # Release Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GQ60ZZ",  # Repair Left Carotid Body, Open Approach
        "0GQ63ZZ",  # Repair Left Carotid Body, Percutaneous Approach
        "0GQ64ZZ",  # Repair Left Carotid Body, Percutaneous Endoscopic Approach
        "0GQ70ZZ",  # Repair Right Carotid Body, Open Approach
        "0GQ73ZZ",  # Repair Right Carotid Body, Percutaneous Approach
        "0GQ74ZZ",  # Repair Right Carotid Body, Percutaneous Endoscopic Approach
        "0GQ80ZZ",  # Repair Bilateral Carotid Bodies, Open Approach
        "0GQ83ZZ",  # Repair Bilateral Carotid Bodies, Percutaneous Approach
        "0GQ84ZZ",  # Repair Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "0GT60ZZ",  # Resection of Left Carotid Body, Open Approach
        "0GT64ZZ",  # Resection of Left Carotid Body, Percutaneous Endoscopic Approach
        "0GT70ZZ",  # Resection of Right Carotid Body, Open Approach
        "0GT74ZZ",  # Resection of Right Carotid Body, Percutaneous Endoscopic Approach
        "0GT80ZZ",  # Resection of Bilateral Carotid Bodies, Open Approach
        "0GT84ZZ",  # Resection of Bilateral Carotid Bodies, Percutaneous Endoscopic Approach
        "B3060ZZ",  # Plain Radiography of Right Internal Carotid Artery using High Osmolar Contrast
        "B3061ZZ",  # Plain Radiography of Right Internal Carotid Artery using Low Osmolar Contrast
        "B306YZZ",  # Plain Radiography of Right Internal Carotid Artery using Other Contrast
        "B3070ZZ",  # Plain Radiography of Left Internal Carotid Artery using High Osmolar Contrast
        "B3071ZZ",  # Plain Radiography of Left Internal Carotid Artery using Low Osmolar Contrast
        "B307YZZ",  # Plain Radiography of Left Internal Carotid Artery using Other Contrast
        "B3080ZZ",  # Plain Radiography of Bilateral Internal Carotid Arteries using High Osmolar Contrast
        "B3081ZZ",  # Plain Radiography of Bilateral Internal Carotid Arteries using Low Osmolar Contrast
        "B308YZZ",  # Plain Radiography of Bilateral Internal Carotid Arteries using Other Contrast
        "B3160ZZ",  # Fluoroscopy of Right Internal Carotid Artery using High Osmolar Contrast
        "B3161ZZ",  # Fluoroscopy of Right Internal Carotid Artery using Low Osmolar Contrast
        "B316YZZ",  # Fluoroscopy of Right Internal Carotid Artery using Other Contrast
        "B3170ZZ",  # Fluoroscopy of Left Internal Carotid Artery using High Osmolar Contrast
        "B3171ZZ",  # Fluoroscopy of Left Internal Carotid Artery using Low Osmolar Contrast
        "B317YZZ",  # Fluoroscopy of Left Internal Carotid Artery using Other Contrast
        "B3180ZZ",  # Fluoroscopy of Bilateral Internal Carotid Arteries using High Osmolar Contrast
        "B3181ZZ",  # Fluoroscopy of Bilateral Internal Carotid Arteries using Low Osmolar Contrast
        "B318YZZ",  # Fluoroscopy of Bilateral Internal Carotid Arteries using Other Contrast
    }

    SNOMEDCT = {
        "112823003",  # Creation of aorta-carotid-brachial vascular bypass (procedure)
        "15023006",  # Thromboendarterectomy with graft of carotid artery by neck incision (procedure)
        "175363002",  # Reconstruction of carotid artery (procedure)
        "175364008",  # Replacement of carotid artery using graft (procedure)
        "175365009",  # Intracranial bypass to carotid artery (procedure)
        "175367001",  # Endarterectomy of carotid artery and patch repair of carotid artery (procedure)
        "175373000",  # Ligation of carotid artery (procedure)
        "175374006",  # Open embolectomy of carotid artery (procedure)
        "175376008",  # Operation on aneurysm of carotid artery (procedure)
        "175379001",  # Transluminal operations on carotid artery (procedure)
        "175380003",  # Percutaneous transluminal angioplasty of carotid artery (procedure)
        "175398004",  # Anastomosis of circle of Willis (procedure)
        "18674003",  # Creation of carotid-vertebral artery shunt (procedure)
        "22928005",  # Ligation of internal carotid artery (procedure)
        "233259003",  # Angioplasty of external carotid artery (procedure)
        "233260008",  # Percutaneous balloon angioplasty of extracranial carotid artery (procedure)
        "233296007",  # Endarterectomy of internal carotid artery (procedure)
        "233297003",  # Endarterectomy of external carotid artery (procedure)
        "233298008",  # Endarterectomy of common carotid artery (procedure)
        "233405004",  # Insertion of carotid artery stent (procedure)
        "241219006",  # Angiography of internal carotid artery (procedure)
        "276949008",  # Percutaneous endarterectomy of internal carotid artery (procedure)
        "276950008",  # Percutaneous endarterectomy of external carotid artery (procedure)
        "276951007",  # Percutaneous endarterectomy of common carotid artery (procedure)
        "287606009",  # Selective angiography of carotid artery (procedure)
        "302053004",  # Embolectomy of carotid artery (procedure)
        "303161001",  # Patch repair of carotid artery (procedure)
        "31573003",  # Anastomosis of carotid-subclavian artery (procedure)
        "34214004",  # Creation of aorta-subclavian-carotid vascular bypass (procedure)
        "39887009",  # Thrombectomy with catheter of carotid artery by neck incision (procedure)
        "405326004",  # Angioplasty of internal carotid artery (procedure)
        "405379009",  # Repair of internal carotid artery (procedure)
        "405407008",  # Endarterectomy and angioplasty of internal carotid artery (procedure)
        "405408003",  # Endarterectomy and angioplasty of internal carotid artery with prosthesis (procedure)
        "405409006",  # Endarterectomy and angioplasty of internal carotid artery with vein (procedure)
        "405411002",  # Endarterectomy and angioplasty of external carotid artery (procedure)
        "405412009",  # Endarterectomy of internal carotid artery with eversion and end-to-end anastomosis (procedure)
        "405415006",  # Angioplasty of internal carotid artery with vein (procedure)
        "417884003",  # Angioplasty of external carotid artery using fluoroscopic guidance with contrast (procedure)
        "418405008",  # Fluoroscopic angiography of carotid artery with contrast and insertion of stent (procedure)
        "418838006",  # Fluoroscopic angiography of internal carotid artery with contrast (procedure)
        "419014003",  # Angioplasty of internal carotid artery using fluoroscopic guidance with contrast (procedure)
        "420026003",  # Angioplasty of common carotid artery using fluoroscopic guidance with contrast (procedure)
        "420046008",  # Angioplasty of carotid artery using fluoroscopic guidance with contrast (procedure)
        "420171008",  # Fluoroscopic angiography of carotid artery with contrast (procedure)
        "425611003",  # Percutaneous transluminal insertion of stent into carotid artery (procedure)
        "427486009",  # Bypass of carotid artery by anastomosis of superficial temporal artery to middle cerebral artery (procedure)
        "428802000",  # Endovascular repair of carotid artery (procedure)
        "429287007",  # Angioplasty of carotid artery (procedure)
        "431515004",  # Fluoroscopic angiography of external carotid artery using contrast with insertion of stent (procedure)
        "431519005",  # Fluoroscopic angiography of common carotid artery using contrast with insertion of stent (procedure)
        "431535003",  # Percutaneous transluminal angioplasty of external carotid artery using fluoroscopic guidance with contrast (procedure)
        "431659001",  # Percutaneous transluminal angioplasty of common carotid artery using fluoroscopic guidance with contrast (procedure)
        "432039002",  # Percutaneous transluminal angioplasty of internal carotid artery using fluoroscopic guidance with contrast (procedure)
        "432785007",  # Fluoroscopic angiography of common carotid artery using contrast with insertion of drug eluting stent (procedure)
        "433056003",  # Fluoroscopic angiography of internal carotid artery using contrast with insertion of stent (procedure)
        "433061001",  # Fluoroscopic intravenous digital subtraction angiography of carotid artery with contrast (procedure)
        "433591001",  # Fluoroscopic angiography of common carotid artery using contrast with insertion of stent graft (procedure)
        "433683001",  # Fluoroscopic angiography of internal carotid artery using contrast with insertion of drug eluting stent (procedure)
        "433690006",  # Fluoroscopic angiography of external carotid artery using contrast with insertion of drug eluting stent (procedure)
        "433711000",  # Percutaneous transluminal cutting balloon angioplasty of external carotid artery using fluoroscopic guidance with contrast (procedure)
        "433734009",  # Percutaneous transluminal cutting balloon angioplasty of internal carotid artery using fluoroscopic guidance with contrast (procedure)
        "434159001",  # Fluoroscopic angiography of external carotid artery using contrast with insertion of stent graft (procedure)
        "434378006",  # Fluoroscopic angiography of internal carotid artery using contrast with insertion of stent graft (procedure)
        "434433007",  # Percutaneous transluminal cutting balloon angioplasty of common carotid artery using fluoroscopic guidance with contrast (procedure)
        "43628009",  # Insertion of needle into carotid artery (procedure)
        "438615003",  # Procedure on carotid artery using imaging guidance (procedure)
        "440221006",  # Bypass of carotid artery to brachial artery using vein graft (procedure)
        "440453000",  # Fluoroscopic angiography of aortic arch and carotid artery with contrast (procedure)
        "440518005",  # Fluoroscopic angiography of carotid artery by direct puncture with contrast (procedure)
        "449242004",  # Clipping of carotid artery by cervical approach (procedure)
        "46912008",  # Ligation of external carotid artery for nasal hemorrhage (procedure)
        "51382002",  # Creation of carotid-carotid shunt (procedure)
        "53412000",  # Ligation of common carotid artery (procedure)
        "59012002",  # Carotid-subclavian artery bypass graft with vein (procedure)
        "59109003",  # Ligation of external carotid artery (procedure)
        "66951008",  # Carotid endarterectomy (procedure)
        "74720005",  # Percutaneous transluminal catheter embolectomy of carotid artery (procedure)
        "79507006",  # Carotid-vertebral artery bypass graft with vein (procedure)
        "80102005",  # Creation of external-internal carotid bypass (procedure)
        "80104006",  # Exteriorization of carotid artery (procedure)
        "87314005",  # Exploration of carotid artery (procedure)
        "90931006",  # Introduction of catheter into carotid artery (procedure)
        "9339002",  # Perfusion of carotid artery (procedure)
    }

class Pci(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent procedures used to define percutaneous coronary intervention (PCI).

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Procedure, Performed. The intent of this data element is to identify patients who had percutaneous coronary intervention (PCI) during an episode of acute myocardial infarction.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying patients receiving percutaneous coronary intervention (PCI). This is a grouping of ICD-10-PCS and SNOMED CT codes.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "PCI"
    OID = "2.16.840.1.113762.1.4.1045.67"
    DEFINITION_VERSION = "20170504"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "0270346",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027034Z",  # Dilation of Coronary Artery, One Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0270356",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027035Z",  # Dilation of Coronary Artery, One Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0270366",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027036Z",  # Dilation of Coronary Artery, One Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0270376",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027037Z",  # Dilation of Coronary Artery, One Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02703D6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02703DZ",  # Dilation of Coronary Artery, One Artery with Intraluminal Device, Percutaneous Approach
        "02703E6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02703EZ",  # Dilation of Coronary Artery, One Artery with Two Intraluminal Devices, Percutaneous Approach
        "02703F6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02703FZ",  # Dilation of Coronary Artery, One Artery with Three Intraluminal Devices, Percutaneous Approach
        "02703G6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02703GZ",  # Dilation of Coronary Artery, One Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "02703T6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02703TZ",  # Dilation of Coronary Artery, One Artery with Radioactive Intraluminal Device, Percutaneous Approach
        "02703Z6",  # Dilation of Coronary Artery, One Artery, Bifurcation, Percutaneous Approach
        "02703ZZ",  # Dilation of Coronary Artery, One Artery, Percutaneous Approach
        "0270446",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027044Z",  # Dilation of Coronary Artery, One Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0270456",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027045Z",  # Dilation of Coronary Artery, One Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0270466",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027046Z",  # Dilation of Coronary Artery, One Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0270476",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027047Z",  # Dilation of Coronary Artery, One Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704D6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02704DZ",  # Dilation of Coronary Artery, One Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "02704E6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704EZ",  # Dilation of Coronary Artery, One Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704F6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704FZ",  # Dilation of Coronary Artery, One Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704G6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704GZ",  # Dilation of Coronary Artery, One Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704T6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02704TZ",  # Dilation of Coronary Artery, One Artery with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02704Z6",  # Dilation of Coronary Artery, One Artery, Bifurcation, Percutaneous Endoscopic Approach
        "02704ZZ",  # Dilation of Coronary Artery, One Artery, Percutaneous Endoscopic Approach
        "0271346",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027134Z",  # Dilation of Coronary Artery, Two Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0271356",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027135Z",  # Dilation of Coronary Artery, Two Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0271366",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027136Z",  # Dilation of Coronary Artery, Two Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0271376",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027137Z",  # Dilation of Coronary Artery, Two Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02713D6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02713DZ",  # Dilation of Coronary Artery, Two Arteries with Intraluminal Device, Percutaneous Approach
        "02713E6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02713EZ",  # Dilation of Coronary Artery, Two Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02713F6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02713FZ",  # Dilation of Coronary Artery, Two Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02713G6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02713GZ",  # Dilation of Coronary Artery, Two Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02713T6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02713TZ",  # Dilation of Coronary Artery, Two Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02713Z6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, Percutaneous Approach
        "02713ZZ",  # Dilation of Coronary Artery, Two Arteries, Percutaneous Approach
        "0271446",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027144Z",  # Dilation of Coronary Artery, Two Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0271456",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027145Z",  # Dilation of Coronary Artery, Two Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0271466",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027146Z",  # Dilation of Coronary Artery, Two Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0271476",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027147Z",  # Dilation of Coronary Artery, Two Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714D6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02714DZ",  # Dilation of Coronary Artery, Two Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02714E6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714EZ",  # Dilation of Coronary Artery, Two Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714F6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714FZ",  # Dilation of Coronary Artery, Two Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714G6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714GZ",  # Dilation of Coronary Artery, Two Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714T6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02714TZ",  # Dilation of Coronary Artery, Two Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02714Z6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02714ZZ",  # Dilation of Coronary Artery, Two Arteries, Percutaneous Endoscopic Approach
        "0272346",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027234Z",  # Dilation of Coronary Artery, Three Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0272356",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027235Z",  # Dilation of Coronary Artery, Three Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0272366",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027236Z",  # Dilation of Coronary Artery, Three Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0272376",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027237Z",  # Dilation of Coronary Artery, Three Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02723D6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02723DZ",  # Dilation of Coronary Artery, Three Arteries with Intraluminal Device, Percutaneous Approach
        "02723E6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02723EZ",  # Dilation of Coronary Artery, Three Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02723F6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02723FZ",  # Dilation of Coronary Artery, Three Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02723G6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02723GZ",  # Dilation of Coronary Artery, Three Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02723T6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02723TZ",  # Dilation of Coronary Artery, Three Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02723Z6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, Percutaneous Approach
        "02723ZZ",  # Dilation of Coronary Artery, Three Arteries, Percutaneous Approach
        "0272446",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027244Z",  # Dilation of Coronary Artery, Three Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0272456",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027245Z",  # Dilation of Coronary Artery, Three Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0272466",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027246Z",  # Dilation of Coronary Artery, Three Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0272476",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027247Z",  # Dilation of Coronary Artery, Three Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724D6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02724DZ",  # Dilation of Coronary Artery, Three Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02724E6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724EZ",  # Dilation of Coronary Artery, Three Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724F6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724FZ",  # Dilation of Coronary Artery, Three Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724G6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724GZ",  # Dilation of Coronary Artery, Three Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724T6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02724TZ",  # Dilation of Coronary Artery, Three Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02724Z6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02724ZZ",  # Dilation of Coronary Artery, Three Arteries, Percutaneous Endoscopic Approach
        "0273346",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027334Z",  # Dilation of Coronary Artery, Four or More Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0273356",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027335Z",  # Dilation of Coronary Artery, Four or More Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0273366",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027336Z",  # Dilation of Coronary Artery, Four or More Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0273376",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027337Z",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02733D6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02733DZ",  # Dilation of Coronary Artery, Four or More Arteries with Intraluminal Device, Percutaneous Approach
        "02733E6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02733EZ",  # Dilation of Coronary Artery, Four or More Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02733F6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02733FZ",  # Dilation of Coronary Artery, Four or More Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02733G6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02733GZ",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02733T6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02733TZ",  # Dilation of Coronary Artery, Four or More Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02733Z6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Approach
        "02733ZZ",  # Dilation of Coronary Artery, Four or More Arteries, Percutaneous Approach
        "0273446",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027344Z",  # Dilation of Coronary Artery, Four or More Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0273456",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027345Z",  # Dilation of Coronary Artery, Four or More Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0273466",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027346Z",  # Dilation of Coronary Artery, Four or More Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0273476",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027347Z",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734D6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02734DZ",  # Dilation of Coronary Artery, Four or More Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02734E6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734EZ",  # Dilation of Coronary Artery, Four or More Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734F6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734FZ",  # Dilation of Coronary Artery, Four or More Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734G6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734GZ",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734T6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02734TZ",  # Dilation of Coronary Artery, Four or More Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02734Z6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02734ZZ",  # Dilation of Coronary Artery, Four or More Arteries, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "11101003",  # Percutaneous transluminal coronary angioplasty (procedure)
        "175066001",  # Percutaneous transluminal balloon angioplasty of bypass graft of coronary artery (procedure)
        "36969009",  # Placement of stent in coronary artery (procedure)
        "397193006",  # Percutaneous transluminal coronary angioplasty by rotoablation (procedure)
        "397431004",  # Percutaneous transluminal coronary angioplasty with rotoablation, single vessel (procedure)
        "414089002",  # Emergency percutaneous coronary intervention (procedure)
        "415070008",  # Percutaneous coronary intervention (procedure)
        "428488008",  # Placement of stent in anterior descending branch of left coronary artery (procedure)
        "429499003",  # Placement of stent in circumflex branch of left coronary artery (procedure)
        "429639007",  # Percutaneous transluminal balloon angioplasty with insertion of stent into coronary artery (procedure)
        "429809004",  # Percutaneous transluminal angioplasty of coronary artery using fluoroscopic guidance with contrast (procedure)
        "609153008",  # Percutaneous insertion of drug eluting stent into coronary artery using fluoroscopic guidance with contrast (procedure)
        "609154002",  # Percutaneous transluminal insertion of metal stent into coronary artery using fluoroscopic guidance with contrast (procedure)
        "68466008",  # Removal of coronary artery obstruction by percutaneous transluminal balloon, single vessel (procedure)
        "698740005",  # Percutaneous transluminal atherectomy of coronary artery by rotary cutter using fluoroscopic guidance with contrast (procedure)
        "707828002",  # Percutaneous transluminal cutting balloon angioplasty of coronary artery (procedure)
        "737085003",  # Percutaneous insertion of bioresorbable stent into coronary artery using fluoroscopic guidance (procedure)
        "85053006",  # Percutaneous transluminal coronary angioplasty, multiple vessels (procedure)
    }

class PartialArthroplastyOfHip(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for partial arthroplasty of the hip.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a procedure for partial arthroplasty of the hip.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Partial Arthroplasty of Hip"
    OID = "2.16.840.1.113883.3.464.1003.1184"
    DEFINITION_VERSION = "20230722"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "27125",  # Hemiarthroplasty, hip, partial (eg, femoral stem prosthesis, bipolar arthroplasty)
    }

    ICD10PCS = {
        "0SRA009",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRA00A",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRA00Z",  # Replacement of Right Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRA019",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRA01A",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRA01Z",  # Replacement of Right Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRA039",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRA03A",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRA03Z",  # Replacement of Right Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRA07Z",  # Replacement of Right Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRA0J9",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRA0JA",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRA0JZ",  # Replacement of Right Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRA0KZ",  # Replacement of Right Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRE009",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRE00A",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRE00Z",  # Replacement of Left Hip Joint, Acetabular Surface with Polyethylene Synthetic Substitute, Open Approach
        "0SRE019",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRE01A",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRE01Z",  # Replacement of Left Hip Joint, Acetabular Surface with Metal Synthetic Substitute, Open Approach
        "0SRE039",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRE03A",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRE03Z",  # Replacement of Left Hip Joint, Acetabular Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRE07Z",  # Replacement of Left Hip Joint, Acetabular Surface with Autologous Tissue Substitute, Open Approach
        "0SRE0J9",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRE0JA",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRE0JZ",  # Replacement of Left Hip Joint, Acetabular Surface with Synthetic Substitute, Open Approach
        "0SRE0KZ",  # Replacement of Left Hip Joint, Acetabular Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRR019",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRR01A",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRR01Z",  # Replacement of Right Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRR039",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRR03A",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRR03Z",  # Replacement of Right Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRR07Z",  # Replacement of Right Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRR0J9",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRR0JA",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRR0JZ",  # Replacement of Right Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRR0KZ",  # Replacement of Right Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
        "0SRS019",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRS01A",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRS01Z",  # Replacement of Left Hip Joint, Femoral Surface with Metal Synthetic Substitute, Open Approach
        "0SRS039",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRS03A",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRS03Z",  # Replacement of Left Hip Joint, Femoral Surface with Ceramic Synthetic Substitute, Open Approach
        "0SRS07Z",  # Replacement of Left Hip Joint, Femoral Surface with Autologous Tissue Substitute, Open Approach
        "0SRS0J9",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Cemented, Open Approach
        "0SRS0JA",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Uncemented, Open Approach
        "0SRS0JZ",  # Replacement of Left Hip Joint, Femoral Surface with Synthetic Substitute, Open Approach
        "0SRS0KZ",  # Replacement of Left Hip Joint, Femoral Surface with Nonautologous Tissue Substitute, Open Approach
    }

    SNOMEDCT = {
        "1217139005",  # Partial replacement of joint of left hip with bipolar prosthesis (procedure)
        "1217140007",  # Partial replacement of joint of right hip with bipolar prosthesis (procedure)
        "1217141006",  # Partial replacement of joint of bilateral hips with bipolar prosthesis (procedure)
        "265106004",  # Insertion of Austin-Moore prosthesis (procedure)
        "276178005",  # Uncemented prosthetic hemiarthroplasty of hip joint (procedure)
        "309458002",  # Cemented Thompson hemiarthroplasty of hip joint (procedure)
        "32581000",  # Partial hip replacement by cup with acetabuloplasty (procedure)
        "340922009",  # Partial hip replacement by cup (procedure)
        "386649003",  # Partial hip replacement by prosthesis (procedure)
        "69625007",  # Replacement of femoral head by prosthesis (procedure)
        "713686009",  # Partial hip replacement with bipolar prosthesis (procedure)
    }

class PrimaryThaProcedure(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of primary total hip arthroplasty (THA).

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent primary total hip arthroplasty (THA).

    **Exclusion Criteria:** Excludes concepts that represent revision or partial hip arthroplasty.
    """

    VALUE_SET_NAME = "Primary THA Procedure"
    OID = "2.16.840.1.113883.3.464.1003.198.12.1006"
    DEFINITION_VERSION = "20230831"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "27130",  # Arthroplasty, acetabular and proximal femoral prosthetic replacement (total hip arthroplasty), with or without autograft or allograft
        "27132",  # Conversion of previous hip surgery to total hip arthroplasty, with or without autograft or allograft
    }

    ICD10PCS = {
        "0SR9019",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SR901A",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SR901Z",  # Replacement of Right Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SR9029",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR902A",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR902Z",  # Replacement of Right Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SR9039",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SR903A",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SR903Z",  # Replacement of Right Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SR9049",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR904A",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR904Z",  # Replacement of Right Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SR9069",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SR906A",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SR906Z",  # Replacement of Right Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SR90J9",  # Replacement of Right Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SR90JA",  # Replacement of Right Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SR90JZ",  # Replacement of Right Hip Joint with Synthetic Substitute, Open Approach
        "0SRB019",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Cemented, Open Approach
        "0SRB01A",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Uncemented, Open Approach
        "0SRB01Z",  # Replacement of Left Hip Joint with Metal Synthetic Substitute, Open Approach
        "0SRB029",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB02A",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB02Z",  # Replacement of Left Hip Joint with Metal on Polyethylene Synthetic Substitute, Open Approach
        "0SRB039",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Cemented, Open Approach
        "0SRB03A",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Uncemented, Open Approach
        "0SRB03Z",  # Replacement of Left Hip Joint with Ceramic Synthetic Substitute, Open Approach
        "0SRB049",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB04A",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB04Z",  # Replacement of Left Hip Joint with Ceramic on Polyethylene Synthetic Substitute, Open Approach
        "0SRB069",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Cemented, Open Approach
        "0SRB06A",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Uncemented, Open Approach
        "0SRB06Z",  # Replacement of Left Hip Joint with Oxidized Zirconium on Polyethylene Synthetic Substitute, Open Approach
        "0SRB0J9",  # Replacement of Left Hip Joint with Synthetic Substitute, Cemented, Open Approach
        "0SRB0JA",  # Replacement of Left Hip Joint with Synthetic Substitute, Uncemented, Open Approach
        "0SRB0JZ",  # Replacement of Left Hip Joint with Synthetic Substitute, Open Approach
    }

    SNOMEDCT = {
        "1230048008",  # Total replacement of hip joint using metal augmentation (procedure)
        "1231410001",  # Total replacement of hip joint using impaction bone graft (procedure)
        "1231411002",  # Total replacement of hip joint using mesh (procedure)
        "1231412009",  # Total replacement of hip joint using ring (procedure)
        "1231413004",  # Total replacement of hip joint using allograft of bone (procedure)
        "1231414005",  # Total replacement of hip joint using autogenous bone graft (procedure)
        "1231415006",  # Total replacement of hip joint using cage (procedure)
        "1304236006",  # Total prosthetic replacement of hip joint via supercapsular approach using percutaneous assistance (procedure)
        "15163009",  # Total arthroplasty of hip, low friction (procedure)
        "265157000",  # Cemented prosthetic total arthroplasty of hip joint (procedure)
        "265160007",  # Uncemented prosthetic total arthroplasty of hip joint (procedure)
        "307814001",  # Cemented arthroplasty of hip joint (procedure)
        "314491003",  # Prosthetic hybrid total replacement of hip joint (procedure)
        "426618001",  # Prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "426904006",  # Prosthetic hybrid total replacement of hip joint using cemented acetabular component (procedure)
        "443435007",  # Total replacement of right hip joint (procedure)
        "52734007",  # Total replacement of hip (procedure)
        "57589001",  # Arthroplasty, acetabular and proximal femoral prosthetic replacement, with allograft (procedure)
        "770606008",  # Total replacement of left hip joint (procedure)
    }

class RemovalRevisionAndSupplementProceduresOfTheLowerBodyAndSpine(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for removal, revision and supplement procedures of the lower body and spine.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent a removal, revision or supplement procedure on the lower body and spine.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Removal, Revision and Supplement Procedures of the Lower Body and Spine"
    OID = "2.16.840.1.113883.3.464.1003.1189"
    DEFINITION_VERSION = "20230722"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "27134",  # Revision of total hip arthroplasty; both components, with or without autograft or allograft
        "27137",  # Revision of total hip arthroplasty; acetabular component only, with or without autograft or allograft
        "27138",  # Revision of total hip arthroplasty; femoral component only, with or without allograft
    }

    HCPCSLEVELII = {
        "S2118",  # Metal-on-metal total hip resurfacing, including acetabular and femoral components
    }

    ICD10PCS = {
        "0QP604Z",  # Removal of Internal Fixation Device from Right Upper Femur, Open Approach
        "0QP605Z",  # Removal of External Fixation Device from Right Upper Femur, Open Approach
        "0QP607Z",  # Removal of Autologous Tissue Substitute from Right Upper Femur, Open Approach
        "0QP60JZ",  # Removal of Synthetic Substitute from Right Upper Femur, Open Approach
        "0QP60KZ",  # Removal of Nonautologous Tissue Substitute from Right Upper Femur, Open Approach
        "0QP634Z",  # Removal of Internal Fixation Device from Right Upper Femur, Percutaneous Approach
        "0QP635Z",  # Removal of External Fixation Device from Right Upper Femur, Percutaneous Approach
        "0QP637Z",  # Removal of Autologous Tissue Substitute from Right Upper Femur, Percutaneous Approach
        "0QP63JZ",  # Removal of Synthetic Substitute from Right Upper Femur, Percutaneous Approach
        "0QP63KZ",  # Removal of Nonautologous Tissue Substitute from Right Upper Femur, Percutaneous Approach
        "0QP644Z",  # Removal of Internal Fixation Device from Right Upper Femur, Percutaneous Endoscopic Approach
        "0QP645Z",  # Removal of External Fixation Device from Right Upper Femur, Percutaneous Endoscopic Approach
        "0QP647Z",  # Removal of Autologous Tissue Substitute from Right Upper Femur, Percutaneous Endoscopic Approach
        "0QP64JZ",  # Removal of Synthetic Substitute from Right Upper Femur, Percutaneous Endoscopic Approach
        "0QP64KZ",  # Removal of Nonautologous Tissue Substitute from Right Upper Femur, Percutaneous Endoscopic Approach
        "0QP704Z",  # Removal of Internal Fixation Device from Left Upper Femur, Open Approach
        "0QP705Z",  # Removal of External Fixation Device from Left Upper Femur, Open Approach
        "0QP707Z",  # Removal of Autologous Tissue Substitute from Left Upper Femur, Open Approach
        "0QP70JZ",  # Removal of Synthetic Substitute from Left Upper Femur, Open Approach
        "0QP70KZ",  # Removal of Nonautologous Tissue Substitute from Left Upper Femur, Open Approach
        "0QP734Z",  # Removal of Internal Fixation Device from Left Upper Femur, Percutaneous Approach
        "0QP735Z",  # Removal of External Fixation Device from Left Upper Femur, Percutaneous Approach
        "0QP737Z",  # Removal of Autologous Tissue Substitute from Left Upper Femur, Percutaneous Approach
        "0QP73JZ",  # Removal of Synthetic Substitute from Left Upper Femur, Percutaneous Approach
        "0QP73KZ",  # Removal of Nonautologous Tissue Substitute from Left Upper Femur, Percutaneous Approach
        "0QP744Z",  # Removal of Internal Fixation Device from Left Upper Femur, Percutaneous Endoscopic Approach
        "0QP745Z",  # Removal of External Fixation Device from Left Upper Femur, Percutaneous Endoscopic Approach
        "0QP747Z",  # Removal of Autologous Tissue Substitute from Left Upper Femur, Percutaneous Endoscopic Approach
        "0QP74JZ",  # Removal of Synthetic Substitute from Left Upper Femur, Percutaneous Endoscopic Approach
        "0QP74KZ",  # Removal of Nonautologous Tissue Substitute from Left Upper Femur, Percutaneous Endoscopic Approach
        "0QP804Z",  # Removal of Internal Fixation Device from Right Femoral Shaft, Open Approach
        "0QP805Z",  # Removal of External Fixation Device from Right Femoral Shaft, Open Approach
        "0QP807Z",  # Removal of Autologous Tissue Substitute from Right Femoral Shaft, Open Approach
        "0QP80JZ",  # Removal of Synthetic Substitute from Right Femoral Shaft, Open Approach
        "0QP80KZ",  # Removal of Nonautologous Tissue Substitute from Right Femoral Shaft, Open Approach
        "0QP834Z",  # Removal of Internal Fixation Device from Right Femoral Shaft, Percutaneous Approach
        "0QP835Z",  # Removal of External Fixation Device from Right Femoral Shaft, Percutaneous Approach
        "0QP837Z",  # Removal of Autologous Tissue Substitute from Right Femoral Shaft, Percutaneous Approach
        "0QP83JZ",  # Removal of Synthetic Substitute from Right Femoral Shaft, Percutaneous Approach
        "0QP83KZ",  # Removal of Nonautologous Tissue Substitute from Right Femoral Shaft, Percutaneous Approach
        "0QP844Z",  # Removal of Internal Fixation Device from Right Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP845Z",  # Removal of External Fixation Device from Right Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP847Z",  # Removal of Autologous Tissue Substitute from Right Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP84JZ",  # Removal of Synthetic Substitute from Right Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP84KZ",  # Removal of Nonautologous Tissue Substitute from Right Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP904Z",  # Removal of Internal Fixation Device from Left Femoral Shaft, Open Approach
        "0QP905Z",  # Removal of External Fixation Device from Left Femoral Shaft, Open Approach
        "0QP907Z",  # Removal of Autologous Tissue Substitute from Left Femoral Shaft, Open Approach
        "0QP90JZ",  # Removal of Synthetic Substitute from Left Femoral Shaft, Open Approach
        "0QP90KZ",  # Removal of Nonautologous Tissue Substitute from Left Femoral Shaft, Open Approach
        "0QP934Z",  # Removal of Internal Fixation Device from Left Femoral Shaft, Percutaneous Approach
        "0QP935Z",  # Removal of External Fixation Device from Left Femoral Shaft, Percutaneous Approach
        "0QP937Z",  # Removal of Autologous Tissue Substitute from Left Femoral Shaft, Percutaneous Approach
        "0QP93JZ",  # Removal of Synthetic Substitute from Left Femoral Shaft, Percutaneous Approach
        "0QP93KZ",  # Removal of Nonautologous Tissue Substitute from Left Femoral Shaft, Percutaneous Approach
        "0QP944Z",  # Removal of Internal Fixation Device from Left Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP945Z",  # Removal of External Fixation Device from Left Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP947Z",  # Removal of Autologous Tissue Substitute from Left Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP94JZ",  # Removal of Synthetic Substitute from Left Femoral Shaft, Percutaneous Endoscopic Approach
        "0QP94KZ",  # Removal of Nonautologous Tissue Substitute from Left Femoral Shaft, Percutaneous Endoscopic Approach
        "0QPB04Z",  # Removal of Internal Fixation Device from Right Lower Femur, Open Approach
        "0QPB05Z",  # Removal of External Fixation Device from Right Lower Femur, Open Approach
        "0QPB07Z",  # Removal of Autologous Tissue Substitute from Right Lower Femur, Open Approach
        "0QPB0JZ",  # Removal of Synthetic Substitute from Right Lower Femur, Open Approach
        "0QPB0KZ",  # Removal of Nonautologous Tissue Substitute from Right Lower Femur, Open Approach
        "0QPB34Z",  # Removal of Internal Fixation Device from Right Lower Femur, Percutaneous Approach
        "0QPB35Z",  # Removal of External Fixation Device from Right Lower Femur, Percutaneous Approach
        "0QPB37Z",  # Removal of Autologous Tissue Substitute from Right Lower Femur, Percutaneous Approach
        "0QPB3JZ",  # Removal of Synthetic Substitute from Right Lower Femur, Percutaneous Approach
        "0QPB3KZ",  # Removal of Nonautologous Tissue Substitute from Right Lower Femur, Percutaneous Approach
        "0QPB44Z",  # Removal of Internal Fixation Device from Right Lower Femur, Percutaneous Endoscopic Approach
        "0QPB45Z",  # Removal of External Fixation Device from Right Lower Femur, Percutaneous Endoscopic Approach
        "0QPB47Z",  # Removal of Autologous Tissue Substitute from Right Lower Femur, Percutaneous Endoscopic Approach
        "0QPB4JZ",  # Removal of Synthetic Substitute from Right Lower Femur, Percutaneous Endoscopic Approach
        "0QPB4KZ",  # Removal of Nonautologous Tissue Substitute from Right Lower Femur, Percutaneous Endoscopic Approach
        "0QPC04Z",  # Removal of Internal Fixation Device from Left Lower Femur, Open Approach
        "0QPC05Z",  # Removal of External Fixation Device from Left Lower Femur, Open Approach
        "0QPC07Z",  # Removal of Autologous Tissue Substitute from Left Lower Femur, Open Approach
        "0QPC0JZ",  # Removal of Synthetic Substitute from Left Lower Femur, Open Approach
        "0QPC0KZ",  # Removal of Nonautologous Tissue Substitute from Left Lower Femur, Open Approach
        "0QPC34Z",  # Removal of Internal Fixation Device from Left Lower Femur, Percutaneous Approach
        "0QPC35Z",  # Removal of External Fixation Device from Left Lower Femur, Percutaneous Approach
        "0QPC37Z",  # Removal of Autologous Tissue Substitute from Left Lower Femur, Percutaneous Approach
        "0QPC3JZ",  # Removal of Synthetic Substitute from Left Lower Femur, Percutaneous Approach
        "0QPC3KZ",  # Removal of Nonautologous Tissue Substitute from Left Lower Femur, Percutaneous Approach
        "0QPC44Z",  # Removal of Internal Fixation Device from Left Lower Femur, Percutaneous Endoscopic Approach
        "0QPC45Z",  # Removal of External Fixation Device from Left Lower Femur, Percutaneous Endoscopic Approach
        "0QPC47Z",  # Removal of Autologous Tissue Substitute from Left Lower Femur, Percutaneous Endoscopic Approach
        "0QPC4JZ",  # Removal of Synthetic Substitute from Left Lower Femur, Percutaneous Endoscopic Approach
        "0QPC4KZ",  # Removal of Nonautologous Tissue Substitute from Left Lower Femur, Percutaneous Endoscopic Approach
        "0QPD04Z",  # Removal of Internal Fixation Device from Right Patella, Open Approach
        "0QPD05Z",  # Removal of External Fixation Device from Right Patella, Open Approach
        "0QPD07Z",  # Removal of Autologous Tissue Substitute from Right Patella, Open Approach
        "0QPD0JZ",  # Removal of Synthetic Substitute from Right Patella, Open Approach
        "0QPD0KZ",  # Removal of Nonautologous Tissue Substitute from Right Patella, Open Approach
        "0QPD34Z",  # Removal of Internal Fixation Device from Right Patella, Percutaneous Approach
        "0QPD35Z",  # Removal of External Fixation Device from Right Patella, Percutaneous Approach
        "0QPD37Z",  # Removal of Autologous Tissue Substitute from Right Patella, Percutaneous Approach
        "0QPD3JZ",  # Removal of Synthetic Substitute from Right Patella, Percutaneous Approach
        "0QPD3KZ",  # Removal of Nonautologous Tissue Substitute from Right Patella, Percutaneous Approach
        "0QPD44Z",  # Removal of Internal Fixation Device from Right Patella, Percutaneous Endoscopic Approach
        "0QPD45Z",  # Removal of External Fixation Device from Right Patella, Percutaneous Endoscopic Approach
        "0QPD47Z",  # Removal of Autologous Tissue Substitute from Right Patella, Percutaneous Endoscopic Approach
        "0QPD4JZ",  # Removal of Synthetic Substitute from Right Patella, Percutaneous Endoscopic Approach
        "0QPD4KZ",  # Removal of Nonautologous Tissue Substitute from Right Patella, Percutaneous Endoscopic Approach
        "0QPF04Z",  # Removal of Internal Fixation Device from Left Patella, Open Approach
        "0QPF05Z",  # Removal of External Fixation Device from Left Patella, Open Approach
        "0QPF07Z",  # Removal of Autologous Tissue Substitute from Left Patella, Open Approach
        "0QPF0JZ",  # Removal of Synthetic Substitute from Left Patella, Open Approach
        "0QPF0KZ",  # Removal of Nonautologous Tissue Substitute from Left Patella, Open Approach
        "0QPF34Z",  # Removal of Internal Fixation Device from Left Patella, Percutaneous Approach
        "0QPF35Z",  # Removal of External Fixation Device from Left Patella, Percutaneous Approach
        "0QPF37Z",  # Removal of Autologous Tissue Substitute from Left Patella, Percutaneous Approach
        "0QPF3JZ",  # Removal of Synthetic Substitute from Left Patella, Percutaneous Approach
        "0QPF3KZ",  # Removal of Nonautologous Tissue Substitute from Left Patella, Percutaneous Approach
        "0QPF44Z",  # Removal of Internal Fixation Device from Left Patella, Percutaneous Endoscopic Approach
        "0QPF45Z",  # Removal of External Fixation Device from Left Patella, Percutaneous Endoscopic Approach
        "0QPF47Z",  # Removal of Autologous Tissue Substitute from Left Patella, Percutaneous Endoscopic Approach
        "0QPF4JZ",  # Removal of Synthetic Substitute from Left Patella, Percutaneous Endoscopic Approach
        "0QPF4KZ",  # Removal of Nonautologous Tissue Substitute from Left Patella, Percutaneous Endoscopic Approach
        "0QPG04Z",  # Removal of Internal Fixation Device from Right Tibia, Open Approach
        "0QPG05Z",  # Removal of External Fixation Device from Right Tibia, Open Approach
        "0QPG07Z",  # Removal of Autologous Tissue Substitute from Right Tibia, Open Approach
        "0QPG0JZ",  # Removal of Synthetic Substitute from Right Tibia, Open Approach
        "0QPG0KZ",  # Removal of Nonautologous Tissue Substitute from Right Tibia, Open Approach
        "0QPG34Z",  # Removal of Internal Fixation Device from Right Tibia, Percutaneous Approach
        "0QPG35Z",  # Removal of External Fixation Device from Right Tibia, Percutaneous Approach
        "0QPG37Z",  # Removal of Autologous Tissue Substitute from Right Tibia, Percutaneous Approach
        "0QPG3JZ",  # Removal of Synthetic Substitute from Right Tibia, Percutaneous Approach
        "0QPG3KZ",  # Removal of Nonautologous Tissue Substitute from Right Tibia, Percutaneous Approach
        "0QPG44Z",  # Removal of Internal Fixation Device from Right Tibia, Percutaneous Endoscopic Approach
        "0QPG45Z",  # Removal of External Fixation Device from Right Tibia, Percutaneous Endoscopic Approach
        "0QPG47Z",  # Removal of Autologous Tissue Substitute from Right Tibia, Percutaneous Endoscopic Approach
        "0QPG4JZ",  # Removal of Synthetic Substitute from Right Tibia, Percutaneous Endoscopic Approach
        "0QPG4KZ",  # Removal of Nonautologous Tissue Substitute from Right Tibia, Percutaneous Endoscopic Approach
        "0QPH04Z",  # Removal of Internal Fixation Device from Left Tibia, Open Approach
        "0QPH05Z",  # Removal of External Fixation Device from Left Tibia, Open Approach
        "0QPH07Z",  # Removal of Autologous Tissue Substitute from Left Tibia, Open Approach
        "0QPH0JZ",  # Removal of Synthetic Substitute from Left Tibia, Open Approach
        "0QPH0KZ",  # Removal of Nonautologous Tissue Substitute from Left Tibia, Open Approach
        "0QPH34Z",  # Removal of Internal Fixation Device from Left Tibia, Percutaneous Approach
        "0QPH35Z",  # Removal of External Fixation Device from Left Tibia, Percutaneous Approach
        "0QPH37Z",  # Removal of Autologous Tissue Substitute from Left Tibia, Percutaneous Approach
        "0QPH3JZ",  # Removal of Synthetic Substitute from Left Tibia, Percutaneous Approach
        "0QPH3KZ",  # Removal of Nonautologous Tissue Substitute from Left Tibia, Percutaneous Approach
        "0QPH44Z",  # Removal of Internal Fixation Device from Left Tibia, Percutaneous Endoscopic Approach
        "0QPH45Z",  # Removal of External Fixation Device from Left Tibia, Percutaneous Endoscopic Approach
        "0QPH47Z",  # Removal of Autologous Tissue Substitute from Left Tibia, Percutaneous Endoscopic Approach
        "0QPH4JZ",  # Removal of Synthetic Substitute from Left Tibia, Percutaneous Endoscopic Approach
        "0QPH4KZ",  # Removal of Nonautologous Tissue Substitute from Left Tibia, Percutaneous Endoscopic Approach
        "0QPJ04Z",  # Removal of Internal Fixation Device from Right Fibula, Open Approach
        "0QPJ05Z",  # Removal of External Fixation Device from Right Fibula, Open Approach
        "0QPJ07Z",  # Removal of Autologous Tissue Substitute from Right Fibula, Open Approach
        "0QPJ0JZ",  # Removal of Synthetic Substitute from Right Fibula, Open Approach
        "0QPJ0KZ",  # Removal of Nonautologous Tissue Substitute from Right Fibula, Open Approach
        "0QPJ34Z",  # Removal of Internal Fixation Device from Right Fibula, Percutaneous Approach
        "0QPJ35Z",  # Removal of External Fixation Device from Right Fibula, Percutaneous Approach
        "0QPJ37Z",  # Removal of Autologous Tissue Substitute from Right Fibula, Percutaneous Approach
        "0QPJ3JZ",  # Removal of Synthetic Substitute from Right Fibula, Percutaneous Approach
        "0QPJ3KZ",  # Removal of Nonautologous Tissue Substitute from Right Fibula, Percutaneous Approach
        "0QPJ44Z",  # Removal of Internal Fixation Device from Right Fibula, Percutaneous Endoscopic Approach
        "0QPJ45Z",  # Removal of External Fixation Device from Right Fibula, Percutaneous Endoscopic Approach
        "0QPJ47Z",  # Removal of Autologous Tissue Substitute from Right Fibula, Percutaneous Endoscopic Approach
        "0QPJ4JZ",  # Removal of Synthetic Substitute from Right Fibula, Percutaneous Endoscopic Approach
        "0QPJ4KZ",  # Removal of Nonautologous Tissue Substitute from Right Fibula, Percutaneous Endoscopic Approach
        "0QPK04Z",  # Removal of Internal Fixation Device from Left Fibula, Open Approach
        "0QPK05Z",  # Removal of External Fixation Device from Left Fibula, Open Approach
        "0QPK07Z",  # Removal of Autologous Tissue Substitute from Left Fibula, Open Approach
        "0QPK0JZ",  # Removal of Synthetic Substitute from Left Fibula, Open Approach
        "0QPK0KZ",  # Removal of Nonautologous Tissue Substitute from Left Fibula, Open Approach
        "0QPK34Z",  # Removal of Internal Fixation Device from Left Fibula, Percutaneous Approach
        "0QPK35Z",  # Removal of External Fixation Device from Left Fibula, Percutaneous Approach
        "0QPK37Z",  # Removal of Autologous Tissue Substitute from Left Fibula, Percutaneous Approach
        "0QPK3JZ",  # Removal of Synthetic Substitute from Left Fibula, Percutaneous Approach
        "0QPK3KZ",  # Removal of Nonautologous Tissue Substitute from Left Fibula, Percutaneous Approach
        "0QPK44Z",  # Removal of Internal Fixation Device from Left Fibula, Percutaneous Endoscopic Approach
        "0QPK45Z",  # Removal of External Fixation Device from Left Fibula, Percutaneous Endoscopic Approach
        "0QPK47Z",  # Removal of Autologous Tissue Substitute from Left Fibula, Percutaneous Endoscopic Approach
        "0QPK4JZ",  # Removal of Synthetic Substitute from Left Fibula, Percutaneous Endoscopic Approach
        "0QPK4KZ",  # Removal of Nonautologous Tissue Substitute from Left Fibula, Percutaneous Endoscopic Approach
        "0QUD0JZ",  # Supplement Right Patella with Synthetic Substitute, Open Approach
        "0QUD3JZ",  # Supplement Right Patella with Synthetic Substitute, Percutaneous Approach
        "0QUD4JZ",  # Supplement Right Patella with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QUF0JZ",  # Supplement Left Patella with Synthetic Substitute, Open Approach
        "0QUF3JZ",  # Supplement Left Patella with Synthetic Substitute, Percutaneous Approach
        "0QUF4JZ",  # Supplement Left Patella with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RP00JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Open Approach
        "0RP03JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Percutaneous Approach
        "0RP04JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP10JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Open Approach
        "0RP13JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Percutaneous Approach
        "0RP14JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP30JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Open Approach
        "0RP33JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Percutaneous Approach
        "0RP34JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP40JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Open Approach
        "0RP43JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP44JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP50JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Open Approach
        "0RP53JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP54JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP60JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Open Approach
        "0RP63JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP64JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP90JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Open Approach
        "0RP93JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP94JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPA0JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA3JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA4JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPB0JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB3JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB4JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP00JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Open Approach
        "0SP03JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP04JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP20JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Open Approach
        "0SP23JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP24JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP30JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Open Approach
        "0SP33JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Percutaneous Approach
        "0SP34JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP40JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Open Approach
        "0SP43JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Percutaneous Approach
        "0SP44JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP50JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Open Approach
        "0SP53JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Percutaneous Approach
        "0SP54JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP60JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Open Approach
        "0SP63JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Percutaneous Approach
        "0SP64JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP70JZ",  # Removal of Synthetic Substitute from Right Sacroiliac Joint, Open Approach
        "0SP73JZ",  # Removal of Synthetic Substitute from Right Sacroiliac Joint, Percutaneous Approach
        "0SP74JZ",  # Removal of Synthetic Substitute from Right Sacroiliac Joint, Percutaneous Endoscopic Approach
        "0SP80JZ",  # Removal of Synthetic Substitute from Left Sacroiliac Joint, Open Approach
        "0SP83JZ",  # Removal of Synthetic Substitute from Left Sacroiliac Joint, Percutaneous Approach
        "0SP84JZ",  # Removal of Synthetic Substitute from Left Sacroiliac Joint, Percutaneous Endoscopic Approach
        "0SP904Z",  # Removal of Internal Fixation Device from Right Hip Joint, Open Approach
        "0SP905Z",  # Removal of External Fixation Device from Right Hip Joint, Open Approach
        "0SP908Z",  # Removal of Spacer from Right Hip Joint, Open Approach
        "0SP909Z",  # Removal of Liner from Right Hip Joint, Open Approach
        "0SP90BZ",  # Removal of Resurfacing Device from Right Hip Joint, Open Approach
        "0SP90EZ",  # Removal of Articulating Spacer from Right Hip Joint, Open Approach
        "0SP90JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Open Approach
        "0SP934Z",  # Removal of Internal Fixation Device from Right Hip Joint, Percutaneous Approach
        "0SP935Z",  # Removal of External Fixation Device from Right Hip Joint, Percutaneous Approach
        "0SP93JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Percutaneous Approach
        "0SP944Z",  # Removal of Internal Fixation Device from Right Hip Joint, Percutaneous Endoscopic Approach
        "0SP945Z",  # Removal of External Fixation Device from Right Hip Joint, Percutaneous Endoscopic Approach
        "0SP94JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Percutaneous Endoscopic Approach
        "0SP9X4Z",  # Removal of Internal Fixation Device from Right Hip Joint, External Approach
        "0SP9X5Z",  # Removal of External Fixation Device from Right Hip Joint, External Approach
        "0SPA0JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Acetabular Surface, Open Approach
        "0SPA3JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SPA4JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SPB04Z",  # Removal of Internal Fixation Device from Left Hip Joint, Open Approach
        "0SPB05Z",  # Removal of External Fixation Device from Left Hip Joint, Open Approach
        "0SPB08Z",  # Removal of Spacer from Left Hip Joint, Open Approach
        "0SPB09Z",  # Removal of Liner from Left Hip Joint, Open Approach
        "0SPB0BZ",  # Removal of Resurfacing Device from Left Hip Joint, Open Approach
        "0SPB0EZ",  # Removal of Articulating Spacer from Left Hip Joint, Open Approach
        "0SPB0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Open Approach
        "0SPB34Z",  # Removal of Internal Fixation Device from Left Hip Joint, Percutaneous Approach
        "0SPB35Z",  # Removal of External Fixation Device from Left Hip Joint, Percutaneous Approach
        "0SPB3JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Percutaneous Approach
        "0SPB44Z",  # Removal of Internal Fixation Device from Left Hip Joint, Percutaneous Endoscopic Approach
        "0SPB45Z",  # Removal of External Fixation Device from Left Hip Joint, Percutaneous Endoscopic Approach
        "0SPB4JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Percutaneous Endoscopic Approach
        "0SPBX4Z",  # Removal of Internal Fixation Device from Left Hip Joint, External Approach
        "0SPBX5Z",  # Removal of External Fixation Device from Left Hip Joint, External Approach
        "0SPC04Z",  # Removal of Internal Fixation Device from Right Knee Joint, Open Approach
        "0SPC05Z",  # Removal of External Fixation Device from Right Knee Joint, Open Approach
        "0SPC08Z",  # Removal of Spacer from Right Knee Joint, Open Approach
        "0SPC09Z",  # Removal of Liner from Right Knee Joint, Open Approach
        "0SPC0EZ",  # Removal of Articulating Spacer from Right Knee Joint, Open Approach
        "0SPC0JC",  # Removal of Synthetic Substitute from Right Knee Joint, Patellar Surface, Open Approach
        "0SPC0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPC0LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPC0MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPC0NZ",  # Removal of Patellofemoral Synthetic Substitute from Right Knee Joint, Open Approach
        "0SPC34Z",  # Removal of Internal Fixation Device from Right Knee Joint, Percutaneous Approach
        "0SPC35Z",  # Removal of External Fixation Device from Right Knee Joint, Percutaneous Approach
        "0SPC3JC",  # Removal of Synthetic Substitute from Right Knee Joint, Patellar Surface, Percutaneous Approach
        "0SPC3JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Percutaneous Approach
        "0SPC3LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Right Knee Joint, Percutaneous Approach
        "0SPC3MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Right Knee Joint, Percutaneous Approach
        "0SPC3NZ",  # Removal of Patellofemoral Synthetic Substitute from Right Knee Joint, Percutaneous Approach
        "0SPC44Z",  # Removal of Internal Fixation Device from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPC45Z",  # Removal of External Fixation Device from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPC4JC",  # Removal of Synthetic Substitute from Right Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SPC4JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPC4LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPC4MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPC4NZ",  # Removal of Patellofemoral Synthetic Substitute from Right Knee Joint, Percutaneous Endoscopic Approach
        "0SPCX4Z",  # Removal of Internal Fixation Device from Right Knee Joint, External Approach
        "0SPCX5Z",  # Removal of External Fixation Device from Right Knee Joint, External Approach
        "0SPD04Z",  # Removal of Internal Fixation Device from Left Knee Joint, Open Approach
        "0SPD05Z",  # Removal of External Fixation Device from Left Knee Joint, Open Approach
        "0SPD08Z",  # Removal of Spacer from Left Knee Joint, Open Approach
        "0SPD09Z",  # Removal of Liner from Left Knee Joint, Open Approach
        "0SPD0EZ",  # Removal of Articulating Spacer from Left Knee Joint, Open Approach
        "0SPD0JC",  # Removal of Synthetic Substitute from Left Knee Joint, Patellar Surface, Open Approach
        "0SPD0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Open Approach
        "0SPD0LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Left Knee Joint, Open Approach
        "0SPD0MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Left Knee Joint, Open Approach
        "0SPD0NZ",  # Removal of Patellofemoral Synthetic Substitute from Left Knee Joint, Open Approach
        "0SPD34Z",  # Removal of Internal Fixation Device from Left Knee Joint, Percutaneous Approach
        "0SPD35Z",  # Removal of External Fixation Device from Left Knee Joint, Percutaneous Approach
        "0SPD3JC",  # Removal of Synthetic Substitute from Left Knee Joint, Patellar Surface, Percutaneous Approach
        "0SPD3JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Percutaneous Approach
        "0SPD3LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Left Knee Joint, Percutaneous Approach
        "0SPD3MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Left Knee Joint, Percutaneous Approach
        "0SPD3NZ",  # Removal of Patellofemoral Synthetic Substitute from Left Knee Joint, Percutaneous Approach
        "0SPD44Z",  # Removal of Internal Fixation Device from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPD45Z",  # Removal of External Fixation Device from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPD4JC",  # Removal of Synthetic Substitute from Left Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SPD4JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPD4LZ",  # Removal of Medial Unicondylar Synthetic Substitute from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPD4MZ",  # Removal of Lateral Unicondylar Synthetic Substitute from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPD4NZ",  # Removal of Patellofemoral Synthetic Substitute from Left Knee Joint, Percutaneous Endoscopic Approach
        "0SPDX4Z",  # Removal of Internal Fixation Device from Left Knee Joint, External Approach
        "0SPDX5Z",  # Removal of External Fixation Device from Left Knee Joint, External Approach
        "0SPE0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Acetabular Surface, Open Approach
        "0SPE3JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SPE4JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SPR0JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Femoral Surface, Open Approach
        "0SPR3JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Femoral Surface, Percutaneous Approach
        "0SPR4JZ",  # Removal of Synthetic Substitute from Right Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SPS0JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Femoral Surface, Open Approach
        "0SPS3JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Femoral Surface, Percutaneous Approach
        "0SPS4JZ",  # Removal of Synthetic Substitute from Left Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SPT0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Femoral Surface, Open Approach
        "0SPT3JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Femoral Surface, Percutaneous Approach
        "0SPT4JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SPU0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Femoral Surface, Open Approach
        "0SPU3JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Femoral Surface, Percutaneous Approach
        "0SPU4JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SPV0JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Tibial Surface, Open Approach
        "0SPV3JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Tibial Surface, Percutaneous Approach
        "0SPV4JZ",  # Removal of Synthetic Substitute from Right Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
        "0SPW0JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Tibial Surface, Open Approach
        "0SPW3JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Tibial Surface, Percutaneous Approach
        "0SPW4JZ",  # Removal of Synthetic Substitute from Left Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
        "0SR90EZ",  # Replacement of Right Hip Joint with Articulating Spacer, Open Approach
        "0SRB0EZ",  # Replacement of Left Hip Joint with Articulating Spacer, Open Approach
        "0SRC0EZ",  # Replacement of Right Knee Joint with Articulating Spacer, Open Approach
        "0SRD0EZ",  # Replacement of Left Knee Joint with Articulating Spacer, Open Approach
        "0SU909Z",  # Supplement Right Hip Joint with Liner, Open Approach
        "0SU90BZ",  # Supplement Right Hip Joint with Resurfacing Device, Open Approach
        "0SUA09Z",  # Supplement Right Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUA0BZ",  # Supplement Right Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUB09Z",  # Supplement Left Hip Joint with Liner, Open Approach
        "0SUB0BZ",  # Supplement Left Hip Joint with Resurfacing Device, Open Approach
        "0SUC09C",  # Supplement Right Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUC0JZ",  # Supplement Right Knee Joint with Synthetic Substitute, Open Approach
        "0SUC4JZ",  # Supplement Right Knee Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SUD09C",  # Supplement Left Knee Joint with Liner, Patellar Surface, Open Approach
        "0SUD0JZ",  # Supplement Left Knee Joint with Synthetic Substitute, Open Approach
        "0SUD4JZ",  # Supplement Left Knee Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SUE09Z",  # Supplement Left Hip Joint, Acetabular Surface with Liner, Open Approach
        "0SUE0BZ",  # Supplement Left Hip Joint, Acetabular Surface with Resurfacing Device, Open Approach
        "0SUR09Z",  # Supplement Right Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUR0BZ",  # Supplement Right Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SUS09Z",  # Supplement Left Hip Joint, Femoral Surface with Liner, Open Approach
        "0SUS0BZ",  # Supplement Left Hip Joint, Femoral Surface with Resurfacing Device, Open Approach
        "0SUT09Z",  # Supplement Right Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUU09Z",  # Supplement Left Knee Joint, Femoral Surface with Liner, Open Approach
        "0SUV09Z",  # Supplement Right Knee Joint, Tibial Surface with Liner, Open Approach
        "0SUW09Z",  # Supplement Left Knee Joint, Tibial Surface with Liner, Open Approach
        "0SW90JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Open Approach
        "0SW93JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Percutaneous Approach
        "0SW94JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Percutaneous Endoscopic Approach
        "0SWA0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Open Approach
        "0SWA3JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SWA4JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SWB0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Open Approach
        "0SWB3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Percutaneous Approach
        "0SWB4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Percutaneous Endoscopic Approach
        "0SWC0JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Open Approach
        "0SWC0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Open Approach
        "0SWC3JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Percutaneous Approach
        "0SWC3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Percutaneous Approach
        "0SWC4JC",  # Revision of Synthetic Substitute in Right Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SWC4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Percutaneous Endoscopic Approach
        "0SWD0JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Open Approach
        "0SWD0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Open Approach
        "0SWD3JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Percutaneous Approach
        "0SWD3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Percutaneous Approach
        "0SWD4JC",  # Revision of Synthetic Substitute in Left Knee Joint, Patellar Surface, Percutaneous Endoscopic Approach
        "0SWD4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Percutaneous Endoscopic Approach
        "0SWE0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Open Approach
        "0SWE3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Percutaneous Approach
        "0SWE4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Acetabular Surface, Percutaneous Endoscopic Approach
        "0SWF0JZ",  # Revision of Synthetic Substitute in Right Ankle Joint, Open Approach
        "0SWF3JZ",  # Revision of Synthetic Substitute in Right Ankle Joint, Percutaneous Approach
        "0SWF4JZ",  # Revision of Synthetic Substitute in Right Ankle Joint, Percutaneous Endoscopic Approach
        "0SWG0JZ",  # Revision of Synthetic Substitute in Left Ankle Joint, Open Approach
        "0SWG3JZ",  # Revision of Synthetic Substitute in Left Ankle Joint, Percutaneous Approach
        "0SWG4JZ",  # Revision of Synthetic Substitute in Left Ankle Joint, Percutaneous Endoscopic Approach
        "0SWH0JZ",  # Revision of Synthetic Substitute in Right Tarsal Joint, Open Approach
        "0SWH3JZ",  # Revision of Synthetic Substitute in Right Tarsal Joint, Percutaneous Approach
        "0SWH4JZ",  # Revision of Synthetic Substitute in Right Tarsal Joint, Percutaneous Endoscopic Approach
        "0SWJ0JZ",  # Revision of Synthetic Substitute in Left Tarsal Joint, Open Approach
        "0SWJ3JZ",  # Revision of Synthetic Substitute in Left Tarsal Joint, Percutaneous Approach
        "0SWJ4JZ",  # Revision of Synthetic Substitute in Left Tarsal Joint, Percutaneous Endoscopic Approach
        "0SWK0JZ",  # Revision of Synthetic Substitute in Right Tarsometatarsal Joint, Open Approach
        "0SWK3JZ",  # Revision of Synthetic Substitute in Right Tarsometatarsal Joint, Percutaneous Approach
        "0SWK4JZ",  # Revision of Synthetic Substitute in Right Tarsometatarsal Joint, Percutaneous Endoscopic Approach
        "0SWL0JZ",  # Revision of Synthetic Substitute in Left Tarsometatarsal Joint, Open Approach
        "0SWL3JZ",  # Revision of Synthetic Substitute in Left Tarsometatarsal Joint, Percutaneous Approach
        "0SWL4JZ",  # Revision of Synthetic Substitute in Left Tarsometatarsal Joint, Percutaneous Endoscopic Approach
        "0SWM0JZ",  # Revision of Synthetic Substitute in Right Metatarsal-Phalangeal Joint, Open Approach
        "0SWM3JZ",  # Revision of Synthetic Substitute in Right Metatarsal-Phalangeal Joint, Percutaneous Approach
        "0SWM4JZ",  # Revision of Synthetic Substitute in Right Metatarsal-Phalangeal Joint, Percutaneous Endoscopic Approach
        "0SWN0JZ",  # Revision of Synthetic Substitute in Left Metatarsal-Phalangeal Joint, Open Approach
        "0SWN3JZ",  # Revision of Synthetic Substitute in Left Metatarsal-Phalangeal Joint, Percutaneous Approach
        "0SWN4JZ",  # Revision of Synthetic Substitute in Left Metatarsal-Phalangeal Joint, Percutaneous Endoscopic Approach
        "0SWP0JZ",  # Revision of Synthetic Substitute in Right Toe Phalangeal Joint, Open Approach
        "0SWP3JZ",  # Revision of Synthetic Substitute in Right Toe Phalangeal Joint, Percutaneous Approach
        "0SWP4JZ",  # Revision of Synthetic Substitute in Right Toe Phalangeal Joint, Percutaneous Endoscopic Approach
        "0SWQ0JZ",  # Revision of Synthetic Substitute in Left Toe Phalangeal Joint, Open Approach
        "0SWQ3JZ",  # Revision of Synthetic Substitute in Left Toe Phalangeal Joint, Percutaneous Approach
        "0SWQ4JZ",  # Revision of Synthetic Substitute in Left Toe Phalangeal Joint, Percutaneous Endoscopic Approach
        "0SWR0JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Open Approach
        "0SWR3JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Percutaneous Approach
        "0SWR4JZ",  # Revision of Synthetic Substitute in Right Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWS0JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Open Approach
        "0SWS3JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Percutaneous Approach
        "0SWS4JZ",  # Revision of Synthetic Substitute in Left Hip Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWT0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Open Approach
        "0SWT3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Percutaneous Approach
        "0SWT4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWU0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Open Approach
        "0SWU3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Percutaneous Approach
        "0SWU4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Femoral Surface, Percutaneous Endoscopic Approach
        "0SWV0JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Open Approach
        "0SWV3JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Percutaneous Approach
        "0SWV4JZ",  # Revision of Synthetic Substitute in Right Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
        "0SWW0JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Open Approach
        "0SWW3JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Percutaneous Approach
        "0SWW4JZ",  # Revision of Synthetic Substitute in Left Knee Joint, Tibial Surface, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "112727005",  # Revision of prosthetic replacement of hip joint (procedure)
        "112728000",  # Total revision of hip replacement with use of methyl methacrylate (procedure)
        "1179075000",  # Colonna arthroplasty (procedure)
        "1179136006",  # Excision of exostosis of tibia (procedure)
        "119614000",  # Hip joint reconstruction (procedure)
        "1197485009",  # Revision of total replacement of knee joint with bone allograft (procedure)
        "1197492004",  # Revision of total replacement of knee joint with bone autograft (procedure)
        "1197495002",  # Revision of total replacement of knee joint with impaction bone grafting (procedure)
        "1201748006",  # Revision of total replacement of knee joint with isolated tibial polyethylene insert exchange (procedure)
        "1201749003",  # Revision of total replacement of knee joint using rotating hinge prosthesis (procedure)
        "1201751004",  # Revision of total replacement of knee joint using valgus-varus constrained prosthesis (procedure)
        "1201752006",  # Revision of total replacement of knee joint using modular metal augmentation (procedure)
        "1217703006",  # Revision of total replacement of knee joint using cone (procedure)
        "1220621004",  # Revision of total replacement of knee joint using stem (procedure)
        "1222612005",  # Revision of total replacement of hip joint using mesh (procedure)
        "1222787003",  # Revision of total replacement of hip joint using cup-cage (procedure)
        "1230047003",  # Revision of total replacement of hip joint using metal augmentation (procedure)
        "1230048008",  # Total replacement of hip joint using metal augmentation (procedure)
        "1230060009",  # Revision of total replacement of hip joint using acetabular liner exchange (procedure)
        "1230110000",  # Revision of total replacement of hip joint using cage (procedure)
        "1230209004",  # Revision of total replacement of hip joint with allograft of bone (procedure)
        "1231408003",  # Revision of total replacement of hip joint with autograft of bone (procedure)
        "1231409006",  # Revision of total replacement of hip joint using impaction bone graft (procedure)
        "1260154000",  # Open reconstruction of acetabular labrum (procedure)
        "1260155004",  # Arthroscopic reconstruction of acetabular labrum (procedure)
        "1287712001",  # Revision of prosthetic total arthroplasty of bilateral knee joints (procedure)
        "1290058005",  # Conversion from cemented hybrid prosthetic replacement of hip joint (procedure)
        "1290059002",  # Conversion from total prosthetic replacement of hip joint (procedure)
        "1290160009",  # Revision of total prosthetic replacement of knee joint (procedure)
        "1290163006",  # Revision of prosthetic replacement of ankle joint (procedure)
        "1290165004",  # Revision of total prosthetic replacement of ankle joint (procedure)
        "16117008",  # Revision of total knee arthroplasty, all components (procedure)
        "179294005",  # Conversion to cemented total hip replacement (procedure)
        "179295006",  # Revision of cemented total prosthetic replacement of hip joint (procedure)
        "179305003",  # Conversion to uncemented total hip replacement (procedure)
        "179306002",  # Revision of uncemented total prosthetic replacement of hip joint (procedure)
        "179320009",  # Conversion to cemented hemiarthroplasty of hip (procedure)
        "179321008",  # Revision cemented hemiarthroplasty of hip (procedure)
        "179327007",  # Conversion to uncemented hemiarthroplasty of hip (procedure)
        "179328002",  # Revision uncemented hemiarthroplasty of hip (procedure)
        "179345007",  # Conversion to cemented total knee replacement (procedure)
        "179346008",  # Revision of cemented total prosthetic replacement of knee joint (procedure)
        "179352009",  # Conversion to uncemented total knee replacement (procedure)
        "179353004",  # Revision of uncemented total prosthetic replacement of knee joint (procedure)
        "179404000",  # Conversion to cemented unicompartmental knee replacement (procedure)
        "179405004",  # Revision of cemented unicompartmental prosthetic replacement of knee joint (procedure)
        "179406003",  # Conversion from cemented unicompartmental knee replacement (procedure)
        "179409005",  # Conversion to uncemented unicompartmental knee replacement (procedure)
        "179410000",  # Revision uncemented unicompartmental knee replacement (procedure)
        "179411001",  # Conversion from uncemented unicompartmental knee replacement (procedure)
        "179414009",  # Conversion to hybrid unicompartmental knee replacement (procedure)
        "179415005",  # Revision hybrid unicompartmental knee replacement (procedure)
        "179416006",  # Conversion from hybrid unicompartmental knee replacement (procedure)
        "179968006",  # Conversion to cemented total ankle replacement (procedure)
        "179969003",  # Revision of cemented total prosthetic replacement of ankle joint (procedure)
        "179970002",  # Conversion from cemented total ankle replacement (procedure)
        "179973000",  # Conversion to uncemented total replacement ankle joint (procedure)
        "179974006",  # Revision of uncemented total prosthetic replacement of ankle joint (procedure)
        "179975007",  # Conversion from uncemented total replacement ankle joint (procedure)
        "179978009",  # Conversion to hybrid total replacement ankle joint (procedure)
        "179979001",  # Revision of hybrid total prosthetic replacement of ankle joint (procedure)
        "179980003",  # Conversion from hybrid total replacement ankle joint (procedure)
        "180117008",  # Correction of congenital deformity of lower limb (procedure)
        "19954002",  # Reconstruction of hip with use of methyl methacrylate (procedure)
        "25769000",  # Removal of internal fixation device from patella (procedure)
        "26166006",  # Removal of internal fixation device from pelvis (procedure)
        "265159002",  # Conversion from cemented total hip replacement (procedure)
        "265161006",  # Conversion from uncemented total hip replacement (procedure)
        "265166001",  # Conversion from cemented hemiarthroplasty of hip (procedure)
        "265167005",  # Conversion from uncemented hemiarthroplasty of hip (procedure)
        "265171008",  # Conversion from cemented total knee replacement (procedure)
        "265173006",  # Conversion from uncemented total knee replacement (procedure)
        "280460009",  # Revision of hip arthroplasty (procedure)
        "280462001",  # Revision of knee arthroplasty (procedure)
        "280470006",  # Revision of ankle arthroplasty (procedure)
        "29712008",  # Revision of prosthetic replacement of knee joint (procedure)
        "309852007",  # Removal of previous uncemented total prosthetic replacement of knee joint (procedure)
        "309853002",  # Removal of previous uncemented total prosthetic replacement of hip joint (procedure)
        "34025002",  # Removal of internal fixation device of femur (procedure)
        "39466008",  # Acetabular augmentation (procedure)
        "405421005",  # Revision of disarticulation of hip (procedure)
        "405449003",  # Revision of above knee amputation (procedure)
        "413111000",  # Removal of exostosis from tibia (procedure)
        "416450002",  # Removal of sacral nerve stimulator (procedure)
        "428622003",  # Revision of hybrid total prosthetic replacement of hip joint using cemented femoral component (procedure)
        "430435008",  # Cementoplasty of acetabulum using computed tomography guidance (procedure)
        "432431008",  # Cementoplasty of acetabulum using fluoroscopic guidance (procedure)
        "438363008",  # Removal of cervical total intervertebral disc prosthesis using anterior approach (procedure)
        "439898006",  # Removal of lumbar total intervertebral disc prosthesis using anterior approach (procedure)
        "440424001",  # Removal of halo device of femur (procedure)
        "44167006",  # Shelf operation, arthroplasty of hip (procedure)
        "442579001",  # Revision of hybrid prosthetic replacement of knee joint using cement (procedure)
        "442656009",  # Conversion to cemented prosthetic hybrid arthroplasty of knee joint (procedure)
        "444886001",  # Removal of cemented total prosthetic replacement of knee joint (procedure)
        "445091006",  # Revision of hybrid knee replacement (procedure)
        "447736005",  # Removal of total hip prosthesis (procedure)
        "447992000",  # Conversion from hybrid total prosthetic hip replacement (procedure)
        "448409009",  # Removal of partial hip prosthesis (procedure)
        "449039004",  # Conversion to total prosthetic replacement of hip joint (procedure)
        "450530006",  # Conversion from hybrid total knee replacement (procedure)
        "450567003",  # Conversion to hybrid total knee replacement (procedure)
        "450814005",  # Revision of hybrid total knee replacement (procedure)
        "45538008",  # Revision of total hip arthroplasty, both components, with autograft (procedure)
        "459831000124101",  # Removal of left knee joint prosthesis for periprosthetic joint infection (procedure)
        "459841000124106",  # Removal of right knee joint prosthesis for periprosthetic joint infection (procedure)
        "459871000124103",  # Removal of left hip joint prosthesis for periprosthetic joint infection (procedure)
        "459881000124100",  # Removal of right hip joint prosthesis for periprosthetic joint infection (procedure)
        "60538001",  # Removal of internal fixation device of tibia (procedure)
        "608878002",  # Removal of implant from femur (procedure)
        "608879005",  # Removal of implant from fibula (procedure)
        "608880008",  # Removal of implant from tibia and fibula (procedure)
        "608883005",  # Removal of implant from pelvic bone (procedure)
        "608884004",  # Removal of implant from vertebra (procedure)
        "609131006",  # Revision of internal fixation of fibula (procedure)
        "609132004",  # Revision of internal fixation of tibia (procedure)
        "609134003",  # Revision of internal fixation of tibia and fibula (procedure)
        "609135002",  # Revision of internal fixation of patella (procedure)
        "609136001",  # Revision of internal fixation of metatarsal bone (procedure)
        "609137005",  # Revision of internal fixation of tarsal bone (procedure)
        "609138000",  # Revision of internal fixation of tarsal and metatarsal bones (procedure)
        "609139008",  # Revision of internal fixation of calcaneus (procedure)
        "609140005",  # Revision of internal fixation of pelvic bone (procedure)
        "609141009",  # Revision of internal fixation of wall of acetabulum (procedure)
        "609142002",  # Revision of internal fixation of bone of anterior ring of pelvis (procedure)
        "609144001",  # Revision of internal fixation of bone of posterior ring of pelvis (procedure)
        "609145000",  # Revision of internal fixation of iliac spine of pelvis (procedure)
        "609217001",  # Revision of below knee amputation stump (procedure)
        "609277005",  # Removal of implant from tibia (procedure)
        "62402000",  # Revision of total hip replacement (procedure)
        "708871009",  # Removal of locking nail device from tibia (procedure)
        "708893004",  # Removal of bone screw device from femur (procedure)
        "708997002",  # Removal of Kirschner wire internal bone fixator from hip (procedure)
        "709189001",  # Revision of internal fixation of talus bone (procedure)
        "711202009",  # Conversion to prosthetic hybrid total replacement of hip joint using cemented femoral component (procedure)
        "713087001",  # Removal of bone screw internal fixation device from ankle (procedure)
        "713522003",  # Total hip resurfacing arthroplasty (procedure)
        "735194005",  # Revision of right total hip arthroplasty (procedure)
        "735196007",  # Revision of left total hip arthroplasty (procedure)
        "735261001",  # Revision of right total prosthetic knee arthroplasty (procedure)
        "735262008",  # Revision of left total prosthetic knee arthroplasty (procedure)
        "764931001",  # Revision of one component of total prosthetic replacement of knee joint not using cement (procedure)
        "764932008",  # Revision of one component of total prosthetic replacement of knee joint using cement (procedure)
        "768498005",  # Revision of one component of total prosthetic replacement of hip joint not using cement (procedure)
        "768847008",  # Revision of one component of total prosthetic replacement of hip joint using cement (procedure)
        "768879004",  # Revision of hybrid prosthetic replacement of hip joint using cemented acetabular component (procedure)
        "77253009",  # Removal of hip prosthesis, complicated, with methyl methacrylate (procedure)
        "78786007",  # Removal of internal fixation device from phalanges of foot (procedure)
        "7989006",  # Removal of prosthesis of joint of hip (procedure)
        "8271002",  # Revision of total hip arthroplasty, both components, with allograft (procedure)
        "86310007",  # Removal of internal fixation device of fibula (procedure)
        "86782006",  # Removal of internal fixation device of vertebra (procedure)
    }

class CystectomyForUrologyCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of cystectomy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that identify a procedure of cystectomy.

    **Exclusion Criteria:** Excludes concepts that identify a surgery for the bladder other than cystectomy.
    """

    VALUE_SET_NAME = "Cystectomy for Urology Care"
    OID = "2.16.840.1.113762.1.4.1248.358"
    DEFINITION_VERSION = "20250312"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "51550",  # Cystectomy, partial; simple
        "51555",  # Cystectomy, partial; complicated (eg, postradiation, previous surgery, difficult location)
        "51565",  # Cystectomy, partial, with reimplantation of ureter(s) into bladder (ureteroneocystostomy)
        "51570",  # Cystectomy, complete; (separate procedure)
        "51575",  # Cystectomy, complete; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric, and obturator nodes
        "51585",  # Cystectomy, complete, with ureterosigmoidostomy or ureterocutaneous transplantations; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric, and obturator nodes
        "51590",  # Cystectomy, complete, with ureteroileal conduit or sigmoid bladder, including intestine anastomosis
        "51595",  # Cystectomy, complete, with ureteroileal conduit or sigmoid bladder, including intestine anastomosis; with bilateral pelvic lymphadenectomy, including external iliac, hypogastric, and obturator nodes
    }

    SNOMEDCT = {
        "112902005",  # Partial urinary cystectomy (procedure)
        "1287977006",  # Laparoscopic radical cystectomy (procedure)
        "176106009",  # Radical cystoprostatourethrectomy (procedure)
        "176107000",  # Radical cystourethrectomy - female (procedure)
        "176108005",  # Simple cystectomy (procedure)
        "236209003",  # Salvage cystoprostatourethrectomy (procedure)
        "236210008",  # Salvage cystourethrectomy - female (procedure)
        "236213005",  # Partial cystectomy with excision of urachal remnant (procedure)
        "287718008",  # Total cystectomy and urethrectomy (procedure)
        "287722003",  # Cystectomy - wedge resection (procedure)
        "287723008",  # Excision of dome of urinary bladder (procedure)
        "64063003",  # Radical cystectomy (procedure)
        "708928004",  # Laparoscopic radical cystectomy using robotic assistance (procedure)
        "85175006",  # Partial cystectomy, complicated (procedure)
        "90648000",  # Partial cystectomy with reimplantation of ureter into urinary bladder (procedure)
    }

class AtrialAblation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures for atrial ablation.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that identify a procedure for atrial ablation.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Atrial Ablation"
    OID = "2.16.840.1.113883.3.117.1.7.1.203"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "02560ZZ",  # Destruction of Right Atrium, Open Approach
        "02563ZZ",  # Destruction of Right Atrium, Percutaneous Approach
        "02564ZZ",  # Destruction of Right Atrium, Percutaneous Endoscopic Approach
        "02580ZZ",  # Destruction of Conduction Mechanism, Open Approach
        "02583ZF",  # Destruction of Conduction Mechanism using Irreversible Electroporation, Percutaneous Approach
        "02583ZZ",  # Destruction of Conduction Mechanism, Percutaneous Approach
        "02584ZZ",  # Destruction of Conduction Mechanism, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "1332525002",  # Cardiac ablation for atrial fibrillation (procedure)
        "175095005",  # Open ablation of atrioventricular node (procedure)
        "175125003",  # Percutaneous transluminal ablation of atrioventricular node (procedure)
        "233159005",  # Ablation operation for arrhythmia (procedure)
        "233160000",  # Ablation operation for arrhythmia by diathermy (procedure)
        "233161001",  # Cryoablation operation for arrhythmia (procedure)
        "233162008",  # Laser ablation operation for arrhythmia (procedure)
        "233163003",  # Radiofrequency ablation operation for arrhythmia (procedure)
        "428290007",  # Percutaneous transluminal ablation of wall of atrium (procedure)
        "428663009",  # Ablation of atrioventricular node (procedure)
        "429211003",  # Maze procedure for atrial fibrillation (procedure)
        "440619008",  # Ablation of ventricular arrhythmogenic focus with cardiopulmonary bypass (procedure)
        "473229000",  # Catheter ablation of arrhythmogenic focus (procedure)
        "707832008",  # Percutaneous transluminal ablation of atrial wall for atrial flutter (procedure)
        "870252007",  # Ventricular tachycardia ablation (procedure)
    }

class IntravenousOrIntraarterialThrombolyticTpaTherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures of intravenous or intra-arterial thrombolytic (t-PA) therapy.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that identify a procedure for intravenous or intra-arterial thrombolytic (t-PA) therapy.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intravenous or Intraarterial Thrombolytic tPA Therapy"
    OID = "2.16.840.1.113762.1.4.1045.21"
    DEFINITION_VERSION = "20210220"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "3E03017",  # Introduction of Other Thrombolytic into Peripheral Vein, Open Approach
        "3E03317",  # Introduction of Other Thrombolytic into Peripheral Vein, Percutaneous Approach
        "3E04017",  # Introduction of Other Thrombolytic into Central Vein, Open Approach
        "3E04317",  # Introduction of Other Thrombolytic into Central Vein, Percutaneous Approach
        "3E05017",  # Introduction of Other Thrombolytic into Peripheral Artery, Open Approach
        "3E05317",  # Introduction of Other Thrombolytic into Peripheral Artery, Percutaneous Approach
        "3E06017",  # Introduction of Other Thrombolytic into Central Artery, Open Approach
        "3E06317",  # Introduction of Other Thrombolytic into Central Artery, Percutaneous Approach
    }

    SNOMEDCT = {
        "307521008",  # Intravenous infusion of thrombolytic (procedure)
        "69245005",  # Intra-arterial infusion of thrombolytic agent (procedure)
    }

class FluorideVarnishApplicationForChildren(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for fluoride varnish applications.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes concepts that represent fluoride varnish applications.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Fluoride Varnish Application for Children"
    OID = "2.16.840.1.113883.3.464.1003.125.12.1002"
    DEFINITION_VERSION = "20210928"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CDT = {
        "D1206",  # topical application of fluoride varnish
        "D1208",  # topical application of fluoride - excluding varnish
    }

    SNOMEDCT = {
        "234723000",  # Topical application of fluoride - tooth (procedure)
        "313042009",  # Application of dental fluoride varnish (procedure)
        "35889000",  # Topical application of fluoride excluding prophylaxis, child (procedure)
        "70468009",  # Topical application of fluoride including prophylaxis, child (procedure)
    }

class HighRiskProceduresForAki(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for procedures that puts patients at risk for developing acute kidney injury (AKI)

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts related to extracorporeal membrane oxygenation (ECMO), intra-aortic balloon pump (IABP), resuscitative endovascular balloon occlusion of the aorta (REBOA), and nephrectomy

    **Exclusion Criteria:** N/A
    """

    VALUE_SET_NAME = "High Risk Procedures for AKI"
    OID = "2.16.840.1.113762.1.4.1248.19"
    DEFINITION_VERSION = "20230205"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "02LW3DJ",  # Occlusion of Thoracic Aorta, Descending with Intraluminal Device, Temporary, Percutaneous Approach
        "04L03DJ",  # Occlusion of Abdominal Aorta with Intraluminal Device, Temporary, Percutaneous Approach
        "0TB00ZZ",  # Excision of Right Kidney, Open Approach
        "0TB03ZZ",  # Excision of Right Kidney, Percutaneous Approach
        "0TB04ZZ",  # Excision of Right Kidney, Percutaneous Endoscopic Approach
        "0TB07ZZ",  # Excision of Right Kidney, Via Natural or Artificial Opening
        "0TB08ZZ",  # Excision of Right Kidney, Via Natural or Artificial Opening Endoscopic
        "0TB10ZZ",  # Excision of Left Kidney, Open Approach
        "0TB13ZZ",  # Excision of Left Kidney, Percutaneous Approach
        "0TB14ZZ",  # Excision of Left Kidney, Percutaneous Endoscopic Approach
        "0TB17ZZ",  # Excision of Left Kidney, Via Natural or Artificial Opening
        "0TB18ZZ",  # Excision of Left Kidney, Via Natural or Artificial Opening Endoscopic
        "0TT00ZZ",  # Resection of Right Kidney, Open Approach
        "0TT04ZZ",  # Resection of Right Kidney, Percutaneous Endoscopic Approach
        "0TT10ZZ",  # Resection of Left Kidney, Open Approach
        "0TT14ZZ",  # Resection of Left Kidney, Percutaneous Endoscopic Approach
        "0TT20ZZ",  # Resection of Bilateral Kidneys, Open Approach
        "0TT24ZZ",  # Resection of Bilateral Kidneys, Percutaneous Endoscopic Approach
        "5A02110",  # Assistance with Cardiac Output using Balloon Pump, Intermittent
        "5A02210",  # Assistance with Cardiac Output using Balloon Pump, Continuous
        "5A1522F",  # Extracorporeal Oxygenation, Membrane, Central
        "5A1522G",  # Extracorporeal Oxygenation, Membrane, Peripheral Veno-arterial
        "5A1522H",  # Extracorporeal Oxygenation, Membrane, Peripheral Veno-venous
        "5A15A2F",  # Extracorporeal Oxygenation, Membrane, Central, Intraoperative
        "5A15A2G",  # Extracorporeal Oxygenation, Membrane, Peripheral Veno-arterial, Intraoperative
        "5A15A2H",  # Extracorporeal Oxygenation, Membrane, Peripheral Veno-venous, Intraoperative
    }

    SNOMEDCT = {
        "1017216005",  # Laparoscopic total excision of right kidney (procedure)
        "1017217001",  # Laparoscopic total excision of left kidney (procedure)
        "108022006",  # Kidney excision (procedure)
        "116033007",  # Radical nephrectomy (procedure)
        "116166009",  # Radical nephrectomy AND ipsilateral adrenalectomy (procedure)
        "12976005",  # Donor nephrectomy (procedure)
        "16965008",  # Insertion of intra-aortic balloon counterpulsation (procedure)
        "175113001",  # Transluminal insertion of aortic balloon pump (procedure)
        "175905003",  # Total nephrectomy (procedure)
        "175911000",  # Live donor nephrectomy (procedure)
        "1866009",  # Total excision of bilateral kidneys (procedure)
        "232969009",  # Cardiac support using extracorporeal membrane oxygenation circuitry (procedure)
        "233572003",  # Extracorporeal gas exchange (procedure)
        "233573008",  # Extracorporeal membrane oxygenation (procedure)
        "233574002",  # Extracorporeal carbon dioxide removal (procedure)
        "265550007",  # Transplant nephrectomy (procedure)
        "361249003",  # Nephrectomy with excision of perirenal tissue (procedure)
        "399217008",  # Cardioassist by aortic balloon pump (procedure)
        "432375008",  # Insertion of intra-aortic balloon pump using fluoroscopic guidance with contrast (procedure)
        "439235001",  # Laparoscopic radical nephrectomy (procedure)
        "439964001",  # Laparoscopic excision of kidney and total excision of ureter (procedure)
        "440446009",  # Laparoscopic excision of kidney and partial excision of ureter (procedure)
        "442919002",  # Adrenal sparing radical nephrectomy (procedure)
        "444083005",  # Total excision of left kidney (procedure)
        "446296004",  # Laparoscopic total excision of kidney (procedure)
        "6561000179108",  # Open nephrectomy from live donor (procedure)
        "698869007",  # Laparoscopic bilateral total nephrectomy (procedure)
        "708908003",  # Laparoscopic live donor nephrectomy (procedure)
        "712998006",  # Laparoscopic radical nephrectomy using robotic assistance (procedure)
        "714031000",  # Single port laparoscopic nephrectomy (procedure)
        "765472003",  # Excision of left kidney (procedure)
        "765473008",  # Excision of right kidney (procedure)
        "786451004",  # Venoarterial extracorporeal membrane oxygenation (procedure)
        "786452006",  # Arteriovenous extracorporeal membrane oxygenation (procedure)
        "786453001",  # Venovenous extracorporeal membrane oxygenation (procedure)
        "83676006",  # Percutaneous insertion of intra-aortic balloon catheter (procedure)
        "88930008",  # Total nephrectomy in recipient (procedure)
    }

class HospitalBasedDialysisServices(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to identify concepts that represent hospital based dialysis procedures indicative of acute dialysis

    **Data Element Scope:** This value set may use a model element related to Procedure

    **Inclusion Criteria:** Includes concepts that represent hospital based dialysis procedures

    **Exclusion Criteria:** Excludes concepts that represent outpatient or home based dialysis procedures
    """

    VALUE_SET_NAME = "Hospital Based Dialysis Services"
    OID = "2.16.840.1.113762.1.4.1248.199"
    DEFINITION_VERSION = "20230205"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "5A1D70Z",  # Performance of Urinary Filtration, Intermittent, Less than 6 Hours Per Day
        "5A1D80Z",  # Performance of Urinary Filtration, Prolonged Intermittent, 6-18 hours Per Day
        "5A1D90Z",  # Performance of Urinary Filtration, Continuous, Greater than 18 hours Per Day
    }

    SNOMEDCT = {
        "11932001",  # Stabilizing hemodialysis (procedure)
        "14684005",  # Peritoneal dialysis excluding cannulation (procedure)
        "225231007",  # Stab peritoneal dialysis (procedure)
        "233575001",  # Intermittent hemodialysis (procedure)
        "233576000",  # Intermittent hemodialysis with sequential ultrafiltration (procedure)
        "233577009",  # Intermittent hemodialysis with continuous ultrafiltration (procedure)
        "233578004",  # Continuous hemodialysis (procedure)
        "233579007",  # Continuous arteriovenous hemodialysis (procedure)
        "233580005",  # Continuous venovenous hemodialysis (procedure)
        "233581009",  # Hemofiltration (procedure)
        "233582002",  # Intermittent hemofiltration (procedure)
        "233583007",  # Continuous hemofiltration (procedure)
        "233584001",  # Continuous arteriovenous hemofiltration (procedure)
        "233585000",  # Continuous venovenous hemofiltration (procedure)
        "233586004",  # Hemodiafiltration (procedure)
        "233587008",  # Intermittent hemodiafiltration (procedure)
        "233588003",  # Continuous hemodiafiltration (procedure)
        "233589006",  # Continuous arteriovenous hemodiafiltration (procedure)
        "233590002",  # Continuous venovenous hemodiafiltration (procedure)
        "238318009",  # Continuous ambulatory peritoneal dialysis (procedure)
        "238319001",  # Continuous cycling peritoneal dialysis (procedure)
        "238321006",  # Intermittent peritoneal dialysis (procedure)
        "238322004",  # Tidal peritoneal dialysis (procedure)
        "238323009",  # Night-time intermittent peritoneal dialysis (procedure)
        "265764009",  # Renal dialysis (procedure)
        "288182009",  # Extracorporeal kidney (procedure)
        "302497006",  # Hemodialysis (procedure)
        "341939001",  # Extracorporeal hemofiltration (procedure)
        "427053002",  # Extracorporeal albumin hemodialysis (procedure)
        "428648006",  # Automated peritoneal dialysis (procedure)
        "48781000175105",  # Nocturnal dialysis (procedure)
        "57274006",  # Initial hemodialysis (procedure)
        "676002",  # Peritoneal dialysis including cannulation (procedure)
        "708932005",  # Emergency hemodialysis (procedure)
        "708933000",  # Emergency hemodiafiltration (procedure)
        "71192002",  # Peritoneal dialysis (procedure)
        "714749008",  # Continuous renal replacement therapy (procedure)
        "715743002",  # Emergency hemofiltration (procedure)
        "895382009",  # Prolonged intermittent renal replacement therapy (procedure)
    }

class EndotrachealIntubation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent procedures used to define endotracheal intubation.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Procedure. The intent of this data element is to identify patients who received endotracheal intubation as a medical procedure.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying patients who had endotracheal intubation. This is a grouping of ICD-10-PCS and SNOMED CT codes.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Endotracheal Intubation"
    OID = "2.16.840.1.113762.1.4.1045.69"
    DEFINITION_VERSION = "20250212"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "31500",  # Intubation, endotracheal, emergency procedure
        "94002",  # Ventilation assist and management, initiation of pressure or volume preset ventilators for assisted or controlled breathing; hospital inpatient/observation, initial day
        "94003",  # Ventilation assist and management, initiation of pressure or volume preset ventilators for assisted or controlled breathing; hospital inpatient/observation, each subsequent day
    }

    ICD10PCS = {
        "0B110F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Open Approach
        "0B113F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Approach
        "0B114F4",  # Bypass Trachea to Cutaneous with Tracheostomy Device, Percutaneous Endoscopic Approach
        "0BH13EZ",  # Insertion of Endotracheal Airway into Trachea, Percutaneous Approach
        "0BH17EZ",  # Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening
        "0BH18EZ",  # Insertion of Endotracheal Airway into Trachea, Via Natural or Artificial Opening Endoscopic
        "5A1935Z",  # Respiratory Ventilation, Less than 24 Consecutive Hours
        "5A1945Z",  # Respiratory Ventilation, 24-96 Consecutive Hours
        "5A1955Z",  # Respiratory Ventilation, Greater than 96 Consecutive Hours
    }

    SNOMEDCT = {
        "112798008",  # Insertion of endotracheal tube (procedure)
        "16883004",  # Endotracheal intubation, emergency procedure (procedure)
        "182682004",  # Emergency laryngeal intubation (procedure)
        "232674004",  # Orotracheal intubation (procedure)
        "232675003",  # Oral intubation awake (procedure)
        "232677006",  # Tracheal intubation using rigid bronchoscope (procedure)
        "232678001",  # Orotracheal fiberoptic intubation (procedure)
        "232679009",  # Nasotracheal intubation (procedure)
        "232680007",  # Nasal intubation awake (procedure)
        "232682004",  # Nasotracheal fiberoptic intubation (procedure)
        "232685002",  # Insertion of tracheostomy tube (procedure)
        "232686001",  # Insertion of tracheal T-tube (procedure)
        "243141005",  # Mechanically assisted spontaneous ventilation (regime/therapy)
        "304341005",  # Awake intubation (procedure)
        "418613003",  # Tracheal intubation through a laryngeal mask airway (procedure)
        "429161001",  # Insertion of endotracheal tube using laryngoscope (procedure)
    }

class IntracranialOrIntraspinalSurgery(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for either intracranial or intraspinal surgeries.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes ICD-10 and SNOMED codes that represent procedures for intracranial or intraspinal surgeries.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Intracranial or Intraspinal surgery"
    OID = "2.16.840.1.113762.1.4.1170.2"
    DEFINITION_VERSION = "20220218"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    ICD10PCS = {
        "00500ZZ",  # Destruction of Brain, Open Approach
        "00510ZZ",  # Destruction of Cerebral Meninges, Open Approach
        "00520ZZ",  # Destruction of Dura Mater, Open Approach
        "00560ZZ",  # Destruction of Cerebral Ventricle, Open Approach
        "00570ZZ",  # Destruction of Cerebral Hemisphere, Open Approach
        "00580ZZ",  # Destruction of Basal Ganglia, Open Approach
        "00590ZZ",  # Destruction of Thalamus, Open Approach
        "005A0ZZ",  # Destruction of Hypothalamus, Open Approach
        "005B0ZZ",  # Destruction of Pons, Open Approach
        "005C0ZZ",  # Destruction of Cerebellum, Open Approach
        "005D0ZZ",  # Destruction of Medulla Oblongata, Open Approach
        "00800ZZ",  # Division of Brain, Open Approach
        "00870ZZ",  # Division of Cerebral Hemisphere, Open Approach
        "00880ZZ",  # Division of Basal Ganglia, Open Approach
        "009000Z",  # Drainage of Brain with Drainage Device, Open Approach
        "00900ZZ",  # Drainage of Brain, Open Approach
        "009100Z",  # Drainage of Cerebral Meninges with Drainage Device, Open Approach
        "00910ZZ",  # Drainage of Cerebral Meninges, Open Approach
        "009200Z",  # Drainage of Dura Mater with Drainage Device, Open Approach
        "00920ZZ",  # Drainage of Dura Mater, Open Approach
        "009300Z",  # Drainage of Intracranial Epidural Space with Drainage Device, Open Approach
        "00930ZZ",  # Drainage of Intracranial Epidural Space, Open Approach
        "009400Z",  # Drainage of Intracranial Subdural Space with Drainage Device, Open Approach
        "00940ZZ",  # Drainage of Intracranial Subdural Space, Open Approach
        "009500Z",  # Drainage of Intracranial Subarachnoid Space with Drainage Device, Open Approach
        "00950ZZ",  # Drainage of Intracranial Subarachnoid Space, Open Approach
        "009600Z",  # Drainage of Cerebral Ventricle with Drainage Device, Open Approach
        "00960ZZ",  # Drainage of Cerebral Ventricle, Open Approach
        "009700Z",  # Drainage of Cerebral Hemisphere with Drainage Device, Open Approach
        "00970ZZ",  # Drainage of Cerebral Hemisphere, Open Approach
        "009800Z",  # Drainage of Basal Ganglia with Drainage Device, Open Approach
        "00980ZZ",  # Drainage of Basal Ganglia, Open Approach
        "009900Z",  # Drainage of Thalamus with Drainage Device, Open Approach
        "00990ZZ",  # Drainage of Thalamus, Open Approach
        "009A00Z",  # Drainage of Hypothalamus with Drainage Device, Open Approach
        "009A0ZZ",  # Drainage of Hypothalamus, Open Approach
        "009B00Z",  # Drainage of Pons with Drainage Device, Open Approach
        "009B0ZZ",  # Drainage of Pons, Open Approach
        "009C00Z",  # Drainage of Cerebellum with Drainage Device, Open Approach
        "009C0ZZ",  # Drainage of Cerebellum, Open Approach
        "009D00Z",  # Drainage of Medulla Oblongata with Drainage Device, Open Approach
        "009D0ZZ",  # Drainage of Medulla Oblongata, Open Approach
        "00B00ZZ",  # Excision of Brain, Open Approach
        "00B10ZZ",  # Excision of Cerebral Meninges, Open Approach
        "00B20ZZ",  # Excision of Dura Mater, Open Approach
        "00B60ZZ",  # Excision of Cerebral Ventricle, Open Approach
        "00B70ZZ",  # Excision of Cerebral Hemisphere, Open Approach
        "00B80ZZ",  # Excision of Basal Ganglia, Open Approach
        "00B90ZZ",  # Excision of Thalamus, Open Approach
        "00BA0ZZ",  # Excision of Hypothalamus, Open Approach
        "00BB0ZZ",  # Excision of Pons, Open Approach
        "00BC0ZZ",  # Excision of Cerebellum, Open Approach
        "00BD0ZZ",  # Excision of Medulla Oblongata, Open Approach
        "00C00ZZ",  # Extirpation of Matter from Brain, Open Approach
        "00C10ZZ",  # Extirpation of Matter from Cerebral Meninges, Open Approach
        "00C20ZZ",  # Extirpation of Matter from Dura Mater, Open Approach
        "00C30ZZ",  # Extirpation of Matter from Intracranial Epidural Space, Open Approach
        "00C40ZZ",  # Extirpation of Matter from Intracranial Subdural Space, Open Approach
        "00C50ZZ",  # Extirpation of Matter from Intracranial Subarachnoid Space, Open Approach
        "00C60ZZ",  # Extirpation of Matter from Cerebral Ventricle, Open Approach
        "00C70ZZ",  # Extirpation of Matter from Cerebral Hemisphere, Open Approach
        "00C80ZZ",  # Extirpation of Matter from Basal Ganglia, Open Approach
        "00C90ZZ",  # Extirpation of Matter from Thalamus, Open Approach
        "00CA0ZZ",  # Extirpation of Matter from Hypothalamus, Open Approach
        "00CB0ZZ",  # Extirpation of Matter from Pons, Open Approach
        "00CC0ZZ",  # Extirpation of Matter from Cerebellum, Open Approach
        "00CD0ZZ",  # Extirpation of Matter from Medulla Oblongata, Open Approach
        "00D10ZZ",  # Extraction of Cerebral Meninges, Open Approach
        "00D20ZZ",  # Extraction of Dura Mater, Open Approach
        "00H002Z",  # Insertion of Monitoring Device into Brain, Open Approach
        "00H003Z",  # Insertion of Infusion Device into Brain, Open Approach
        "00H602Z",  # Insertion of Monitoring Device into Cerebral Ventricle, Open Approach
        "00H603Z",  # Insertion of Infusion Device into Cerebral Ventricle, Open Approach
        "00J00ZZ",  # Inspection of Brain, Open Approach
        "00N90ZZ",  # Release Thalamus, Open Approach
        "00NA0ZZ",  # Release Hypothalamus, Open Approach
        "00P000Z",  # Removal of Drainage Device from Brain, Open Approach
        "00P002Z",  # Removal of Monitoring Device from Brain, Open Approach
        "00P003Z",  # Removal of Infusion Device from Brain, Open Approach
        "00P007Z",  # Removal of Autologous Tissue Substitute from Brain, Open Approach
        "00P00JZ",  # Removal of Synthetic Substitute from Brain, Open Approach
        "00P00KZ",  # Removal of Nonautologous Tissue Substitute from Brain, Open Approach
        "00P600Z",  # Removal of Drainage Device from Cerebral Ventricle, Open Approach
        "00P602Z",  # Removal of Monitoring Device from Cerebral Ventricle, Open Approach
        "00P603Z",  # Removal of Infusion Device from Cerebral Ventricle, Open Approach
        "00P6X2Z",  # Removal of Monitoring Device from Cerebral Ventricle, External Approach
        "00Q90ZZ",  # Repair Thalamus, Open Approach
        "00QA0ZZ",  # Repair Hypothalamus, Open Approach
        "00T70ZZ",  # Resection of Cerebral Hemisphere, Open Approach
        "00W000Z",  # Revision of Drainage Device in Brain, Open Approach
        "00W002Z",  # Revision of Monitoring Device in Brain, Open Approach
        "00W003Z",  # Revision of Infusion Device in Brain, Open Approach
        "00W007Z",  # Revision of Autologous Tissue Substitute in Brain, Open Approach
        "00W00JZ",  # Revision of Synthetic Substitute in Brain, Open Approach
        "00W00KZ",  # Revision of Nonautologous Tissue Substitute in Brain, Open Approach
        "00W00MZ",  # Revision of Neurostimulator Lead in Brain, Open Approach
        "00W600Z",  # Revision of Drainage Device in Cerebral Ventricle, Open Approach
        "00W602Z",  # Revision of Monitoring Device in Cerebral Ventricle, Open Approach
        "00W603Z",  # Revision of Infusion Device in Cerebral Ventricle, Open Approach
        "00W60MZ",  # Revision of Neurostimulator Lead in Cerebral Ventricle, Open Approach
        "0N9000Z",  # Drainage of Skull with Drainage Device, Open Approach
        "0N900ZZ",  # Drainage of Skull, Open Approach
        "0N9100Z",  # Drainage of Frontal Bone with Drainage Device, Open Approach
        "0N910ZZ",  # Drainage of Frontal Bone, Open Approach
        "0N9300Z",  # Drainage of Right Parietal Bone with Drainage Device, Open Approach
        "0N930ZZ",  # Drainage of Right Parietal Bone, Open Approach
        "0N9400Z",  # Drainage of Left Parietal Bone with Drainage Device, Open Approach
        "0N940ZZ",  # Drainage of Left Parietal Bone, Open Approach
        "0N9500Z",  # Drainage of Right Temporal Bone with Drainage Device, Open Approach
        "0N950ZZ",  # Drainage of Right Temporal Bone, Open Approach
        "0N9600Z",  # Drainage of Left Temporal Bone with Drainage Device, Open Approach
        "0N960ZZ",  # Drainage of Left Temporal Bone, Open Approach
        "0N9700Z",  # Drainage of Occipital Bone with Drainage Device, Open Approach
        "0N970ZZ",  # Drainage of Occipital Bone, Open Approach
        "0P530ZZ",  # Destruction of Cervical Vertebra, Open Approach
        "0P533ZZ",  # Destruction of Cervical Vertebra, Percutaneous Approach
        "0P534ZZ",  # Destruction of Cervical Vertebra, Percutaneous Endoscopic Approach
        "0P540ZZ",  # Destruction of Thoracic Vertebra, Open Approach
        "0P543ZZ",  # Destruction of Thoracic Vertebra, Percutaneous Approach
        "0P544ZZ",  # Destruction of Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0P830ZZ",  # Division of Cervical Vertebra, Open Approach
        "0P833ZZ",  # Division of Cervical Vertebra, Percutaneous Approach
        "0P834ZZ",  # Division of Cervical Vertebra, Percutaneous Endoscopic Approach
        "0P840ZZ",  # Division of Thoracic Vertebra, Open Approach
        "0P843ZZ",  # Division of Thoracic Vertebra, Percutaneous Approach
        "0P844ZZ",  # Division of Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0P9300Z",  # Drainage of Cervical Vertebra with Drainage Device, Open Approach
        "0P930ZX",  # Drainage of Cervical Vertebra, Open Approach, Diagnostic
        "0P930ZZ",  # Drainage of Cervical Vertebra, Open Approach
        "0P9330Z",  # Drainage of Cervical Vertebra with Drainage Device, Percutaneous Approach
        "0P933ZX",  # Drainage of Cervical Vertebra, Percutaneous Approach, Diagnostic
        "0P933ZZ",  # Drainage of Cervical Vertebra, Percutaneous Approach
        "0P9340Z",  # Drainage of Cervical Vertebra with Drainage Device, Percutaneous Endoscopic Approach
        "0P934ZX",  # Drainage of Cervical Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0P934ZZ",  # Drainage of Cervical Vertebra, Percutaneous Endoscopic Approach
        "0P9400Z",  # Drainage of Thoracic Vertebra with Drainage Device, Open Approach
        "0P940ZX",  # Drainage of Thoracic Vertebra, Open Approach, Diagnostic
        "0P940ZZ",  # Drainage of Thoracic Vertebra, Open Approach
        "0P9430Z",  # Drainage of Thoracic Vertebra with Drainage Device, Percutaneous Approach
        "0P943ZX",  # Drainage of Thoracic Vertebra, Percutaneous Approach, Diagnostic
        "0P943ZZ",  # Drainage of Thoracic Vertebra, Percutaneous Approach
        "0P9440Z",  # Drainage of Thoracic Vertebra with Drainage Device, Percutaneous Endoscopic Approach
        "0P944ZX",  # Drainage of Thoracic Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0P944ZZ",  # Drainage of Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PB30ZX",  # Excision of Cervical Vertebra, Open Approach, Diagnostic
        "0PB30ZZ",  # Excision of Cervical Vertebra, Open Approach
        "0PB33ZX",  # Excision of Cervical Vertebra, Percutaneous Approach, Diagnostic
        "0PB33ZZ",  # Excision of Cervical Vertebra, Percutaneous Approach
        "0PB34ZX",  # Excision of Cervical Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0PB34ZZ",  # Excision of Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PB40ZX",  # Excision of Thoracic Vertebra, Open Approach, Diagnostic
        "0PB40ZZ",  # Excision of Thoracic Vertebra, Open Approach
        "0PB43ZX",  # Excision of Thoracic Vertebra, Percutaneous Approach, Diagnostic
        "0PB43ZZ",  # Excision of Thoracic Vertebra, Percutaneous Approach
        "0PB44ZX",  # Excision of Thoracic Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0PB44ZZ",  # Excision of Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PC30ZZ",  # Extirpation of Matter from Cervical Vertebra, Open Approach
        "0PC33ZZ",  # Extirpation of Matter from Cervical Vertebra, Percutaneous Approach
        "0PC34ZZ",  # Extirpation of Matter from Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PC40ZZ",  # Extirpation of Matter from Thoracic Vertebra, Open Approach
        "0PC43ZZ",  # Extirpation of Matter from Thoracic Vertebra, Percutaneous Approach
        "0PC44ZZ",  # Extirpation of Matter from Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PD30ZZ",  # Extraction of Cervical Vertebra, Open Approach
        "0PD40ZZ",  # Extraction of Thoracic Vertebra, Open Approach
        "0PH304Z",  # Insertion of Internal Fixation Device into Cervical Vertebra, Open Approach
        "0PH334Z",  # Insertion of Internal Fixation Device into Cervical Vertebra, Percutaneous Approach
        "0PH344Z",  # Insertion of Internal Fixation Device into Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PH404Z",  # Insertion of Internal Fixation Device into Thoracic Vertebra, Open Approach
        "0PH434Z",  # Insertion of Internal Fixation Device into Thoracic Vertebra, Percutaneous Approach
        "0PH444Z",  # Insertion of Internal Fixation Device into Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PN30ZZ",  # Release Cervical Vertebra, Open Approach
        "0PN33ZZ",  # Release Cervical Vertebra, Percutaneous Approach
        "0PN34ZZ",  # Release Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PN40ZZ",  # Release Thoracic Vertebra, Open Approach
        "0PN43ZZ",  # Release Thoracic Vertebra, Percutaneous Approach
        "0PN44ZZ",  # Release Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PP304Z",  # Removal of Internal Fixation Device from Cervical Vertebra, Open Approach
        "0PP307Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebra, Open Approach
        "0PP30JZ",  # Removal of Synthetic Substitute from Cervical Vertebra, Open Approach
        "0PP30KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebra, Open Approach
        "0PP334Z",  # Removal of Internal Fixation Device from Cervical Vertebra, Percutaneous Approach
        "0PP337Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebra, Percutaneous Approach
        "0PP33JZ",  # Removal of Synthetic Substitute from Cervical Vertebra, Percutaneous Approach
        "0PP33KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebra, Percutaneous Approach
        "0PP344Z",  # Removal of Internal Fixation Device from Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PP347Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PP34JZ",  # Removal of Synthetic Substitute from Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PP34KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PP3X4Z",  # Removal of Internal Fixation Device from Cervical Vertebra, External Approach
        "0PP404Z",  # Removal of Internal Fixation Device from Thoracic Vertebra, Open Approach
        "0PP407Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebra, Open Approach
        "0PP40JZ",  # Removal of Synthetic Substitute from Thoracic Vertebra, Open Approach
        "0PP40KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebra, Open Approach
        "0PP434Z",  # Removal of Internal Fixation Device from Thoracic Vertebra, Percutaneous Approach
        "0PP437Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebra, Percutaneous Approach
        "0PP43JZ",  # Removal of Synthetic Substitute from Thoracic Vertebra, Percutaneous Approach
        "0PP43KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebra, Percutaneous Approach
        "0PP444Z",  # Removal of Internal Fixation Device from Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PP447Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PP44JZ",  # Removal of Synthetic Substitute from Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PP44KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PP4X4Z",  # Removal of Internal Fixation Device from Thoracic Vertebra, External Approach
        "0PQ30ZZ",  # Repair Cervical Vertebra, Open Approach
        "0PQ33ZZ",  # Repair Cervical Vertebra, Percutaneous Approach
        "0PQ34ZZ",  # Repair Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PQ3XZZ",  # Repair Cervical Vertebra, External Approach
        "0PQ40ZZ",  # Repair Thoracic Vertebra, Open Approach
        "0PQ43ZZ",  # Repair Thoracic Vertebra, Percutaneous Approach
        "0PQ44ZZ",  # Repair Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PQ4XZZ",  # Repair Thoracic Vertebra, External Approach
        "0PR307Z",  # Replacement of Cervical Vertebra with Autologous Tissue Substitute, Open Approach
        "0PR30JZ",  # Replacement of Cervical Vertebra with Synthetic Substitute, Open Approach
        "0PR30KZ",  # Replacement of Cervical Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0PR337Z",  # Replacement of Cervical Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0PR33JZ",  # Replacement of Cervical Vertebra with Synthetic Substitute, Percutaneous Approach
        "0PR33KZ",  # Replacement of Cervical Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0PR347Z",  # Replacement of Cervical Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PR34JZ",  # Replacement of Cervical Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0PR34KZ",  # Replacement of Cervical Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PR407Z",  # Replacement of Thoracic Vertebra with Autologous Tissue Substitute, Open Approach
        "0PR40JZ",  # Replacement of Thoracic Vertebra with Synthetic Substitute, Open Approach
        "0PR40KZ",  # Replacement of Thoracic Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0PR437Z",  # Replacement of Thoracic Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0PR43JZ",  # Replacement of Thoracic Vertebra with Synthetic Substitute, Percutaneous Approach
        "0PR43KZ",  # Replacement of Thoracic Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0PR447Z",  # Replacement of Thoracic Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PR44JZ",  # Replacement of Thoracic Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0PR44KZ",  # Replacement of Thoracic Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PS304Z",  # Reposition Cervical Vertebra with Internal Fixation Device, Open Approach
        "0PS30ZZ",  # Reposition Cervical Vertebra, Open Approach
        "0PS334Z",  # Reposition Cervical Vertebra with Internal Fixation Device, Percutaneous Approach
        "0PS33ZZ",  # Reposition Cervical Vertebra, Percutaneous Approach
        "0PS344Z",  # Reposition Cervical Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0PS34ZZ",  # Reposition Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PS3XZZ",  # Reposition Cervical Vertebra, External Approach
        "0PS403Z",  # Reposition Thoracic Vertebra with Spinal Stabilization Device, Vertebral Body Tether, Open Approach
        "0PS404Z",  # Reposition Thoracic Vertebra with Internal Fixation Device, Open Approach
        "0PS40ZZ",  # Reposition Thoracic Vertebra, Open Approach
        "0PS434Z",  # Reposition Thoracic Vertebra with Internal Fixation Device, Percutaneous Approach
        "0PS43ZZ",  # Reposition Thoracic Vertebra, Percutaneous Approach
        "0PS443Z",  # Reposition Thoracic Vertebra with Spinal Stabilization Device, Vertebral Body Tether, Percutaneous Endoscopic Approach
        "0PS444Z",  # Reposition Thoracic Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0PS44ZZ",  # Reposition Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PS4XZZ",  # Reposition Thoracic Vertebra, External Approach
        "0PU307Z",  # Supplement Cervical Vertebra with Autologous Tissue Substitute, Open Approach
        "0PU30JZ",  # Supplement Cervical Vertebra with Synthetic Substitute, Open Approach
        "0PU30KZ",  # Supplement Cervical Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0PU337Z",  # Supplement Cervical Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0PU33JZ",  # Supplement Cervical Vertebra with Synthetic Substitute, Percutaneous Approach
        "0PU33KZ",  # Supplement Cervical Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0PU347Z",  # Supplement Cervical Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PU34JZ",  # Supplement Cervical Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0PU34KZ",  # Supplement Cervical Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PU407Z",  # Supplement Thoracic Vertebra with Autologous Tissue Substitute, Open Approach
        "0PU40JZ",  # Supplement Thoracic Vertebra with Synthetic Substitute, Open Approach
        "0PU40KZ",  # Supplement Thoracic Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0PU437Z",  # Supplement Thoracic Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0PU43JZ",  # Supplement Thoracic Vertebra with Synthetic Substitute, Percutaneous Approach
        "0PU43KZ",  # Supplement Thoracic Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0PU447Z",  # Supplement Thoracic Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PU44JZ",  # Supplement Thoracic Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0PU44KZ",  # Supplement Thoracic Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0PW304Z",  # Revision of Internal Fixation Device in Cervical Vertebra, Open Approach
        "0PW307Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebra, Open Approach
        "0PW30JZ",  # Revision of Synthetic Substitute in Cervical Vertebra, Open Approach
        "0PW30KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebra, Open Approach
        "0PW334Z",  # Revision of Internal Fixation Device in Cervical Vertebra, Percutaneous Approach
        "0PW337Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebra, Percutaneous Approach
        "0PW33JZ",  # Revision of Synthetic Substitute in Cervical Vertebra, Percutaneous Approach
        "0PW33KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebra, Percutaneous Approach
        "0PW344Z",  # Revision of Internal Fixation Device in Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PW347Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PW34JZ",  # Revision of Synthetic Substitute in Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PW34KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebra, Percutaneous Endoscopic Approach
        "0PW3X4Z",  # Revision of Internal Fixation Device in Cervical Vertebra, External Approach
        "0PW3X7Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebra, External Approach
        "0PW3XJZ",  # Revision of Synthetic Substitute in Cervical Vertebra, External Approach
        "0PW3XKZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebra, External Approach
        "0PW404Z",  # Revision of Internal Fixation Device in Thoracic Vertebra, Open Approach
        "0PW407Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebra, Open Approach
        "0PW40JZ",  # Revision of Synthetic Substitute in Thoracic Vertebra, Open Approach
        "0PW40KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebra, Open Approach
        "0PW434Z",  # Revision of Internal Fixation Device in Thoracic Vertebra, Percutaneous Approach
        "0PW437Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebra, Percutaneous Approach
        "0PW43JZ",  # Revision of Synthetic Substitute in Thoracic Vertebra, Percutaneous Approach
        "0PW43KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebra, Percutaneous Approach
        "0PW444Z",  # Revision of Internal Fixation Device in Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PW447Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PW44JZ",  # Revision of Synthetic Substitute in Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PW44KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebra, Percutaneous Endoscopic Approach
        "0PW4X4Z",  # Revision of Internal Fixation Device in Thoracic Vertebra, External Approach
        "0PW4X7Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebra, External Approach
        "0PW4XJZ",  # Revision of Synthetic Substitute in Thoracic Vertebra, External Approach
        "0PW4XKZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebra, External Approach
        "0Q500ZZ",  # Destruction of Lumbar Vertebra, Open Approach
        "0Q503ZZ",  # Destruction of Lumbar Vertebra, Percutaneous Approach
        "0Q504ZZ",  # Destruction of Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0Q510ZZ",  # Destruction of Sacrum, Open Approach
        "0Q513ZZ",  # Destruction of Sacrum, Percutaneous Approach
        "0Q514ZZ",  # Destruction of Sacrum, Percutaneous Endoscopic Approach
        "0Q5S0ZZ",  # Destruction of Coccyx, Open Approach
        "0Q5S3ZZ",  # Destruction of Coccyx, Percutaneous Approach
        "0Q5S4ZZ",  # Destruction of Coccyx, Percutaneous Endoscopic Approach
        "0Q800ZZ",  # Division of Lumbar Vertebra, Open Approach
        "0Q803ZZ",  # Division of Lumbar Vertebra, Percutaneous Approach
        "0Q804ZZ",  # Division of Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0Q810ZZ",  # Division of Sacrum, Open Approach
        "0Q813ZZ",  # Division of Sacrum, Percutaneous Approach
        "0Q814ZZ",  # Division of Sacrum, Percutaneous Endoscopic Approach
        "0Q8S0ZZ",  # Division of Coccyx, Open Approach
        "0Q8S3ZZ",  # Division of Coccyx, Percutaneous Approach
        "0Q8S4ZZ",  # Division of Coccyx, Percutaneous Endoscopic Approach
        "0Q9000Z",  # Drainage of Lumbar Vertebra with Drainage Device, Open Approach
        "0Q900ZX",  # Drainage of Lumbar Vertebra, Open Approach, Diagnostic
        "0Q900ZZ",  # Drainage of Lumbar Vertebra, Open Approach
        "0Q9030Z",  # Drainage of Lumbar Vertebra with Drainage Device, Percutaneous Approach
        "0Q903ZX",  # Drainage of Lumbar Vertebra, Percutaneous Approach, Diagnostic
        "0Q903ZZ",  # Drainage of Lumbar Vertebra, Percutaneous Approach
        "0Q9040Z",  # Drainage of Lumbar Vertebra with Drainage Device, Percutaneous Endoscopic Approach
        "0Q904ZX",  # Drainage of Lumbar Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0Q904ZZ",  # Drainage of Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0Q9100Z",  # Drainage of Sacrum with Drainage Device, Open Approach
        "0Q910ZX",  # Drainage of Sacrum, Open Approach, Diagnostic
        "0Q910ZZ",  # Drainage of Sacrum, Open Approach
        "0Q9130Z",  # Drainage of Sacrum with Drainage Device, Percutaneous Approach
        "0Q913ZX",  # Drainage of Sacrum, Percutaneous Approach, Diagnostic
        "0Q913ZZ",  # Drainage of Sacrum, Percutaneous Approach
        "0Q9140Z",  # Drainage of Sacrum with Drainage Device, Percutaneous Endoscopic Approach
        "0Q914ZX",  # Drainage of Sacrum, Percutaneous Endoscopic Approach, Diagnostic
        "0Q914ZZ",  # Drainage of Sacrum, Percutaneous Endoscopic Approach
        "0Q9S00Z",  # Drainage of Coccyx with Drainage Device, Open Approach
        "0Q9S0ZX",  # Drainage of Coccyx, Open Approach, Diagnostic
        "0Q9S0ZZ",  # Drainage of Coccyx, Open Approach
        "0Q9S30Z",  # Drainage of Coccyx with Drainage Device, Percutaneous Approach
        "0Q9S3ZX",  # Drainage of Coccyx, Percutaneous Approach, Diagnostic
        "0Q9S3ZZ",  # Drainage of Coccyx, Percutaneous Approach
        "0Q9S40Z",  # Drainage of Coccyx with Drainage Device, Percutaneous Endoscopic Approach
        "0Q9S4ZX",  # Drainage of Coccyx, Percutaneous Endoscopic Approach, Diagnostic
        "0Q9S4ZZ",  # Drainage of Coccyx, Percutaneous Endoscopic Approach
        "0QB00ZX",  # Excision of Lumbar Vertebra, Open Approach, Diagnostic
        "0QB00ZZ",  # Excision of Lumbar Vertebra, Open Approach
        "0QB03ZX",  # Excision of Lumbar Vertebra, Percutaneous Approach, Diagnostic
        "0QB03ZZ",  # Excision of Lumbar Vertebra, Percutaneous Approach
        "0QB04ZX",  # Excision of Lumbar Vertebra, Percutaneous Endoscopic Approach, Diagnostic
        "0QB04ZZ",  # Excision of Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QB10ZX",  # Excision of Sacrum, Open Approach, Diagnostic
        "0QB10ZZ",  # Excision of Sacrum, Open Approach
        "0QB13ZX",  # Excision of Sacrum, Percutaneous Approach, Diagnostic
        "0QB13ZZ",  # Excision of Sacrum, Percutaneous Approach
        "0QB14ZX",  # Excision of Sacrum, Percutaneous Endoscopic Approach, Diagnostic
        "0QB14ZZ",  # Excision of Sacrum, Percutaneous Endoscopic Approach
        "0QBS0ZX",  # Excision of Coccyx, Open Approach, Diagnostic
        "0QBS0ZZ",  # Excision of Coccyx, Open Approach
        "0QBS3ZX",  # Excision of Coccyx, Percutaneous Approach, Diagnostic
        "0QBS3ZZ",  # Excision of Coccyx, Percutaneous Approach
        "0QBS4ZX",  # Excision of Coccyx, Percutaneous Endoscopic Approach, Diagnostic
        "0QBS4ZZ",  # Excision of Coccyx, Percutaneous Endoscopic Approach
        "0QC00ZZ",  # Extirpation of Matter from Lumbar Vertebra, Open Approach
        "0QC03ZZ",  # Extirpation of Matter from Lumbar Vertebra, Percutaneous Approach
        "0QC04ZZ",  # Extirpation of Matter from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QC10ZZ",  # Extirpation of Matter from Sacrum, Open Approach
        "0QC13ZZ",  # Extirpation of Matter from Sacrum, Percutaneous Approach
        "0QC14ZZ",  # Extirpation of Matter from Sacrum, Percutaneous Endoscopic Approach
        "0QCS0ZZ",  # Extirpation of Matter from Coccyx, Open Approach
        "0QCS3ZZ",  # Extirpation of Matter from Coccyx, Percutaneous Approach
        "0QCS4ZZ",  # Extirpation of Matter from Coccyx, Percutaneous Endoscopic Approach
        "0QD00ZZ",  # Extraction of Lumbar Vertebra, Open Approach
        "0QD10ZZ",  # Extraction of Sacrum, Open Approach
        "0QDS0ZZ",  # Extraction of Coccyx, Open Approach
        "0QH004Z",  # Insertion of Internal Fixation Device into Lumbar Vertebra, Open Approach
        "0QH005Z",  # Insertion of External Fixation Device into Lumbar Vertebra, Open Approach
        "0QH034Z",  # Insertion of Internal Fixation Device into Lumbar Vertebra, Percutaneous Approach
        "0QH035Z",  # Insertion of External Fixation Device into Lumbar Vertebra, Percutaneous Approach
        "0QH044Z",  # Insertion of Internal Fixation Device into Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QH045Z",  # Insertion of External Fixation Device into Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QH104Z",  # Insertion of Internal Fixation Device into Sacrum, Open Approach
        "0QH105Z",  # Insertion of External Fixation Device into Sacrum, Open Approach
        "0QH134Z",  # Insertion of Internal Fixation Device into Sacrum, Percutaneous Approach
        "0QH135Z",  # Insertion of External Fixation Device into Sacrum, Percutaneous Approach
        "0QH144Z",  # Insertion of Internal Fixation Device into Sacrum, Percutaneous Endoscopic Approach
        "0QH145Z",  # Insertion of External Fixation Device into Sacrum, Percutaneous Endoscopic Approach
        "0QHS04Z",  # Insertion of Internal Fixation Device into Coccyx, Open Approach
        "0QHS05Z",  # Insertion of External Fixation Device into Coccyx, Open Approach
        "0QHS34Z",  # Insertion of Internal Fixation Device into Coccyx, Percutaneous Approach
        "0QHS35Z",  # Insertion of External Fixation Device into Coccyx, Percutaneous Approach
        "0QHS44Z",  # Insertion of Internal Fixation Device into Coccyx, Percutaneous Endoscopic Approach
        "0QHS45Z",  # Insertion of External Fixation Device into Coccyx, Percutaneous Endoscopic Approach
        "0QN00ZZ",  # Release Lumbar Vertebra, Open Approach
        "0QN03ZZ",  # Release Lumbar Vertebra, Percutaneous Approach
        "0QN04ZZ",  # Release Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QN10ZZ",  # Release Sacrum, Open Approach
        "0QN13ZZ",  # Release Sacrum, Percutaneous Approach
        "0QN14ZZ",  # Release Sacrum, Percutaneous Endoscopic Approach
        "0QNS0ZZ",  # Release Coccyx, Open Approach
        "0QNS3ZZ",  # Release Coccyx, Percutaneous Approach
        "0QNS4ZZ",  # Release Coccyx, Percutaneous Endoscopic Approach
        "0QP004Z",  # Removal of Internal Fixation Device from Lumbar Vertebra, Open Approach
        "0QP005Z",  # Removal of External Fixation Device from Lumbar Vertebra, Open Approach
        "0QP007Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebra, Open Approach
        "0QP00JZ",  # Removal of Synthetic Substitute from Lumbar Vertebra, Open Approach
        "0QP00KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebra, Open Approach
        "0QP034Z",  # Removal of Internal Fixation Device from Lumbar Vertebra, Percutaneous Approach
        "0QP035Z",  # Removal of External Fixation Device from Lumbar Vertebra, Percutaneous Approach
        "0QP037Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebra, Percutaneous Approach
        "0QP03JZ",  # Removal of Synthetic Substitute from Lumbar Vertebra, Percutaneous Approach
        "0QP03KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebra, Percutaneous Approach
        "0QP044Z",  # Removal of Internal Fixation Device from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QP045Z",  # Removal of External Fixation Device from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QP047Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QP04JZ",  # Removal of Synthetic Substitute from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QP04KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QP0X4Z",  # Removal of Internal Fixation Device from Lumbar Vertebra, External Approach
        "0QP0X5Z",  # Removal of External Fixation Device from Lumbar Vertebra, External Approach
        "0QP104Z",  # Removal of Internal Fixation Device from Sacrum, Open Approach
        "0QP105Z",  # Removal of External Fixation Device from Sacrum, Open Approach
        "0QP107Z",  # Removal of Autologous Tissue Substitute from Sacrum, Open Approach
        "0QP10JZ",  # Removal of Synthetic Substitute from Sacrum, Open Approach
        "0QP10KZ",  # Removal of Nonautologous Tissue Substitute from Sacrum, Open Approach
        "0QP134Z",  # Removal of Internal Fixation Device from Sacrum, Percutaneous Approach
        "0QP135Z",  # Removal of External Fixation Device from Sacrum, Percutaneous Approach
        "0QP137Z",  # Removal of Autologous Tissue Substitute from Sacrum, Percutaneous Approach
        "0QP13JZ",  # Removal of Synthetic Substitute from Sacrum, Percutaneous Approach
        "0QP13KZ",  # Removal of Nonautologous Tissue Substitute from Sacrum, Percutaneous Approach
        "0QP144Z",  # Removal of Internal Fixation Device from Sacrum, Percutaneous Endoscopic Approach
        "0QP145Z",  # Removal of External Fixation Device from Sacrum, Percutaneous Endoscopic Approach
        "0QP147Z",  # Removal of Autologous Tissue Substitute from Sacrum, Percutaneous Endoscopic Approach
        "0QP14JZ",  # Removal of Synthetic Substitute from Sacrum, Percutaneous Endoscopic Approach
        "0QP14KZ",  # Removal of Nonautologous Tissue Substitute from Sacrum, Percutaneous Endoscopic Approach
        "0QP1X4Z",  # Removal of Internal Fixation Device from Sacrum, External Approach
        "0QP1X5Z",  # Removal of External Fixation Device from Sacrum, External Approach
        "0QPS04Z",  # Removal of Internal Fixation Device from Coccyx, Open Approach
        "0QPS05Z",  # Removal of External Fixation Device from Coccyx, Open Approach
        "0QPS07Z",  # Removal of Autologous Tissue Substitute from Coccyx, Open Approach
        "0QPS0JZ",  # Removal of Synthetic Substitute from Coccyx, Open Approach
        "0QPS0KZ",  # Removal of Nonautologous Tissue Substitute from Coccyx, Open Approach
        "0QPS34Z",  # Removal of Internal Fixation Device from Coccyx, Percutaneous Approach
        "0QPS35Z",  # Removal of External Fixation Device from Coccyx, Percutaneous Approach
        "0QPS37Z",  # Removal of Autologous Tissue Substitute from Coccyx, Percutaneous Approach
        "0QPS3JZ",  # Removal of Synthetic Substitute from Coccyx, Percutaneous Approach
        "0QPS3KZ",  # Removal of Nonautologous Tissue Substitute from Coccyx, Percutaneous Approach
        "0QPS44Z",  # Removal of Internal Fixation Device from Coccyx, Percutaneous Endoscopic Approach
        "0QPS45Z",  # Removal of External Fixation Device from Coccyx, Percutaneous Endoscopic Approach
        "0QPS47Z",  # Removal of Autologous Tissue Substitute from Coccyx, Percutaneous Endoscopic Approach
        "0QPS4JZ",  # Removal of Synthetic Substitute from Coccyx, Percutaneous Endoscopic Approach
        "0QPS4KZ",  # Removal of Nonautologous Tissue Substitute from Coccyx, Percutaneous Endoscopic Approach
        "0QPSX4Z",  # Removal of Internal Fixation Device from Coccyx, External Approach
        "0QPSX5Z",  # Removal of External Fixation Device from Coccyx, External Approach
        "0QQ00ZZ",  # Repair Lumbar Vertebra, Open Approach
        "0QQ03ZZ",  # Repair Lumbar Vertebra, Percutaneous Approach
        "0QQ04ZZ",  # Repair Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QQ0XZZ",  # Repair Lumbar Vertebra, External Approach
        "0QQ10ZZ",  # Repair Sacrum, Open Approach
        "0QQ13ZZ",  # Repair Sacrum, Percutaneous Approach
        "0QQ14ZZ",  # Repair Sacrum, Percutaneous Endoscopic Approach
        "0QQ1XZZ",  # Repair Sacrum, External Approach
        "0QQS0ZZ",  # Repair Coccyx, Open Approach
        "0QQS3ZZ",  # Repair Coccyx, Percutaneous Approach
        "0QQS4ZZ",  # Repair Coccyx, Percutaneous Endoscopic Approach
        "0QQSXZZ",  # Repair Coccyx, External Approach
        "0QR007Z",  # Replacement of Lumbar Vertebra with Autologous Tissue Substitute, Open Approach
        "0QR00JZ",  # Replacement of Lumbar Vertebra with Synthetic Substitute, Open Approach
        "0QR00KZ",  # Replacement of Lumbar Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0QR037Z",  # Replacement of Lumbar Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0QR03JZ",  # Replacement of Lumbar Vertebra with Synthetic Substitute, Percutaneous Approach
        "0QR03KZ",  # Replacement of Lumbar Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QR047Z",  # Replacement of Lumbar Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QR04JZ",  # Replacement of Lumbar Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QR04KZ",  # Replacement of Lumbar Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QR107Z",  # Replacement of Sacrum with Autologous Tissue Substitute, Open Approach
        "0QR10JZ",  # Replacement of Sacrum with Synthetic Substitute, Open Approach
        "0QR10KZ",  # Replacement of Sacrum with Nonautologous Tissue Substitute, Open Approach
        "0QR137Z",  # Replacement of Sacrum with Autologous Tissue Substitute, Percutaneous Approach
        "0QR13JZ",  # Replacement of Sacrum with Synthetic Substitute, Percutaneous Approach
        "0QR13KZ",  # Replacement of Sacrum with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QR147Z",  # Replacement of Sacrum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QR14JZ",  # Replacement of Sacrum with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QR14KZ",  # Replacement of Sacrum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QRS07Z",  # Replacement of Coccyx with Autologous Tissue Substitute, Open Approach
        "0QRS0JZ",  # Replacement of Coccyx with Synthetic Substitute, Open Approach
        "0QRS0KZ",  # Replacement of Coccyx with Nonautologous Tissue Substitute, Open Approach
        "0QRS37Z",  # Replacement of Coccyx with Autologous Tissue Substitute, Percutaneous Approach
        "0QRS3JZ",  # Replacement of Coccyx with Synthetic Substitute, Percutaneous Approach
        "0QRS3KZ",  # Replacement of Coccyx with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QRS47Z",  # Replacement of Coccyx with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QRS4JZ",  # Replacement of Coccyx with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QRS4KZ",  # Replacement of Coccyx with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QS003Z",  # Reposition Lumbar Vertebra with Spinal Stabilization Device, Vertebral Body Tether, Open Approach
        "0QS004Z",  # Reposition Lumbar Vertebra with Internal Fixation Device, Open Approach
        "0QS00ZZ",  # Reposition Lumbar Vertebra, Open Approach
        "0QS034Z",  # Reposition Lumbar Vertebra with Internal Fixation Device, Percutaneous Approach
        "0QS03ZZ",  # Reposition Lumbar Vertebra, Percutaneous Approach
        "0QS043Z",  # Reposition Lumbar Vertebra with Spinal Stabilization Device, Vertebral Body Tether, Percutaneous Endoscopic Approach
        "0QS044Z",  # Reposition Lumbar Vertebra with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0QS04ZZ",  # Reposition Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QS0XZZ",  # Reposition Lumbar Vertebra, External Approach
        "0QS104Z",  # Reposition Sacrum with Internal Fixation Device, Open Approach
        "0QS10ZZ",  # Reposition Sacrum, Open Approach
        "0QS134Z",  # Reposition Sacrum with Internal Fixation Device, Percutaneous Approach
        "0QS13ZZ",  # Reposition Sacrum, Percutaneous Approach
        "0QS144Z",  # Reposition Sacrum with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0QS14ZZ",  # Reposition Sacrum, Percutaneous Endoscopic Approach
        "0QS1XZZ",  # Reposition Sacrum, External Approach
        "0QSS04Z",  # Reposition Coccyx with Internal Fixation Device, Open Approach
        "0QSS0ZZ",  # Reposition Coccyx, Open Approach
        "0QSS34Z",  # Reposition Coccyx with Internal Fixation Device, Percutaneous Approach
        "0QSS3ZZ",  # Reposition Coccyx, Percutaneous Approach
        "0QSS44Z",  # Reposition Coccyx with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0QSS4ZZ",  # Reposition Coccyx, Percutaneous Endoscopic Approach
        "0QSSXZZ",  # Reposition Coccyx, External Approach
        "0QTS0ZZ",  # Resection of Coccyx, Open Approach
        "0QU007Z",  # Supplement Lumbar Vertebra with Autologous Tissue Substitute, Open Approach
        "0QU00JZ",  # Supplement Lumbar Vertebra with Synthetic Substitute, Open Approach
        "0QU00KZ",  # Supplement Lumbar Vertebra with Nonautologous Tissue Substitute, Open Approach
        "0QU037Z",  # Supplement Lumbar Vertebra with Autologous Tissue Substitute, Percutaneous Approach
        "0QU03JZ",  # Supplement Lumbar Vertebra with Synthetic Substitute, Percutaneous Approach
        "0QU03KZ",  # Supplement Lumbar Vertebra with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QU047Z",  # Supplement Lumbar Vertebra with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QU04JZ",  # Supplement Lumbar Vertebra with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QU04KZ",  # Supplement Lumbar Vertebra with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QU107Z",  # Supplement Sacrum with Autologous Tissue Substitute, Open Approach
        "0QU10JZ",  # Supplement Sacrum with Synthetic Substitute, Open Approach
        "0QU10KZ",  # Supplement Sacrum with Nonautologous Tissue Substitute, Open Approach
        "0QU137Z",  # Supplement Sacrum with Autologous Tissue Substitute, Percutaneous Approach
        "0QU13JZ",  # Supplement Sacrum with Synthetic Substitute, Percutaneous Approach
        "0QU13KZ",  # Supplement Sacrum with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QU147Z",  # Supplement Sacrum with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QU14JZ",  # Supplement Sacrum with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QU14KZ",  # Supplement Sacrum with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QUS07Z",  # Supplement Coccyx with Autologous Tissue Substitute, Open Approach
        "0QUS0JZ",  # Supplement Coccyx with Synthetic Substitute, Open Approach
        "0QUS0KZ",  # Supplement Coccyx with Nonautologous Tissue Substitute, Open Approach
        "0QUS37Z",  # Supplement Coccyx with Autologous Tissue Substitute, Percutaneous Approach
        "0QUS3JZ",  # Supplement Coccyx with Synthetic Substitute, Percutaneous Approach
        "0QUS3KZ",  # Supplement Coccyx with Nonautologous Tissue Substitute, Percutaneous Approach
        "0QUS47Z",  # Supplement Coccyx with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QUS4JZ",  # Supplement Coccyx with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0QUS4KZ",  # Supplement Coccyx with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0QW004Z",  # Revision of Internal Fixation Device in Lumbar Vertebra, Open Approach
        "0QW007Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebra, Open Approach
        "0QW00JZ",  # Revision of Synthetic Substitute in Lumbar Vertebra, Open Approach
        "0QW00KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebra, Open Approach
        "0QW034Z",  # Revision of Internal Fixation Device in Lumbar Vertebra, Percutaneous Approach
        "0QW037Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebra, Percutaneous Approach
        "0QW03JZ",  # Revision of Synthetic Substitute in Lumbar Vertebra, Percutaneous Approach
        "0QW03KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebra, Percutaneous Approach
        "0QW044Z",  # Revision of Internal Fixation Device in Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QW047Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QW04JZ",  # Revision of Synthetic Substitute in Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QW04KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebra, Percutaneous Endoscopic Approach
        "0QW0X4Z",  # Revision of Internal Fixation Device in Lumbar Vertebra, External Approach
        "0QW0X7Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebra, External Approach
        "0QW0XJZ",  # Revision of Synthetic Substitute in Lumbar Vertebra, External Approach
        "0QW0XKZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebra, External Approach
        "0QW104Z",  # Revision of Internal Fixation Device in Sacrum, Open Approach
        "0QW107Z",  # Revision of Autologous Tissue Substitute in Sacrum, Open Approach
        "0QW10JZ",  # Revision of Synthetic Substitute in Sacrum, Open Approach
        "0QW10KZ",  # Revision of Nonautologous Tissue Substitute in Sacrum, Open Approach
        "0QW134Z",  # Revision of Internal Fixation Device in Sacrum, Percutaneous Approach
        "0QW137Z",  # Revision of Autologous Tissue Substitute in Sacrum, Percutaneous Approach
        "0QW13JZ",  # Revision of Synthetic Substitute in Sacrum, Percutaneous Approach
        "0QW13KZ",  # Revision of Nonautologous Tissue Substitute in Sacrum, Percutaneous Approach
        "0QW144Z",  # Revision of Internal Fixation Device in Sacrum, Percutaneous Endoscopic Approach
        "0QW147Z",  # Revision of Autologous Tissue Substitute in Sacrum, Percutaneous Endoscopic Approach
        "0QW14JZ",  # Revision of Synthetic Substitute in Sacrum, Percutaneous Endoscopic Approach
        "0QW14KZ",  # Revision of Nonautologous Tissue Substitute in Sacrum, Percutaneous Endoscopic Approach
        "0QW1X4Z",  # Revision of Internal Fixation Device in Sacrum, External Approach
        "0QW1X7Z",  # Revision of Autologous Tissue Substitute in Sacrum, External Approach
        "0QW1XJZ",  # Revision of Synthetic Substitute in Sacrum, External Approach
        "0QW1XKZ",  # Revision of Nonautologous Tissue Substitute in Sacrum, External Approach
        "0QWS04Z",  # Revision of Internal Fixation Device in Coccyx, Open Approach
        "0QWS07Z",  # Revision of Autologous Tissue Substitute in Coccyx, Open Approach
        "0QWS0JZ",  # Revision of Synthetic Substitute in Coccyx, Open Approach
        "0QWS0KZ",  # Revision of Nonautologous Tissue Substitute in Coccyx, Open Approach
        "0QWS34Z",  # Revision of Internal Fixation Device in Coccyx, Percutaneous Approach
        "0QWS37Z",  # Revision of Autologous Tissue Substitute in Coccyx, Percutaneous Approach
        "0QWS3JZ",  # Revision of Synthetic Substitute in Coccyx, Percutaneous Approach
        "0QWS3KZ",  # Revision of Nonautologous Tissue Substitute in Coccyx, Percutaneous Approach
        "0QWS44Z",  # Revision of Internal Fixation Device in Coccyx, Percutaneous Endoscopic Approach
        "0QWS47Z",  # Revision of Autologous Tissue Substitute in Coccyx, Percutaneous Endoscopic Approach
        "0QWS4JZ",  # Revision of Synthetic Substitute in Coccyx, Percutaneous Endoscopic Approach
        "0QWS4KZ",  # Revision of Nonautologous Tissue Substitute in Coccyx, Percutaneous Endoscopic Approach
        "0QWSX4Z",  # Revision of Internal Fixation Device in Coccyx, External Approach
        "0QWSX7Z",  # Revision of Autologous Tissue Substitute in Coccyx, External Approach
        "0QWSXJZ",  # Revision of Synthetic Substitute in Coccyx, External Approach
        "0QWSXKZ",  # Revision of Nonautologous Tissue Substitute in Coccyx, External Approach
        "0R500ZZ",  # Destruction of Occipital-cervical Joint, Open Approach
        "0R503ZZ",  # Destruction of Occipital-cervical Joint, Percutaneous Approach
        "0R504ZZ",  # Destruction of Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0R510ZZ",  # Destruction of Cervical Vertebral Joint, Open Approach
        "0R513ZZ",  # Destruction of Cervical Vertebral Joint, Percutaneous Approach
        "0R514ZZ",  # Destruction of Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0R530ZZ",  # Destruction of Cervical Vertebral Disc, Open Approach
        "0R533ZZ",  # Destruction of Cervical Vertebral Disc, Percutaneous Approach
        "0R534ZZ",  # Destruction of Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0R540ZZ",  # Destruction of Cervicothoracic Vertebral Joint, Open Approach
        "0R543ZZ",  # Destruction of Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0R544ZZ",  # Destruction of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0R550ZZ",  # Destruction of Cervicothoracic Vertebral Disc, Open Approach
        "0R553ZZ",  # Destruction of Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0R554ZZ",  # Destruction of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0R560ZZ",  # Destruction of Thoracic Vertebral Joint, Open Approach
        "0R563ZZ",  # Destruction of Thoracic Vertebral Joint, Percutaneous Approach
        "0R564ZZ",  # Destruction of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0R590ZZ",  # Destruction of Thoracic Vertebral Disc, Open Approach
        "0R593ZZ",  # Destruction of Thoracic Vertebral Disc, Percutaneous Approach
        "0R594ZZ",  # Destruction of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0R5A0ZZ",  # Destruction of Thoracolumbar Vertebral Joint, Open Approach
        "0R5A3ZZ",  # Destruction of Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0R5A4ZZ",  # Destruction of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0R5B0ZZ",  # Destruction of Thoracolumbar Vertebral Disc, Open Approach
        "0R5B3ZZ",  # Destruction of Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0R5B4ZZ",  # Destruction of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0R9000Z",  # Drainage of Occipital-cervical Joint with Drainage Device, Open Approach
        "0R900ZX",  # Drainage of Occipital-cervical Joint, Open Approach, Diagnostic
        "0R900ZZ",  # Drainage of Occipital-cervical Joint, Open Approach
        "0R9030Z",  # Drainage of Occipital-cervical Joint with Drainage Device, Percutaneous Approach
        "0R903ZX",  # Drainage of Occipital-cervical Joint, Percutaneous Approach, Diagnostic
        "0R903ZZ",  # Drainage of Occipital-cervical Joint, Percutaneous Approach
        "0R9040Z",  # Drainage of Occipital-cervical Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0R904ZX",  # Drainage of Occipital-cervical Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0R904ZZ",  # Drainage of Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0R9100Z",  # Drainage of Cervical Vertebral Joint with Drainage Device, Open Approach
        "0R910ZX",  # Drainage of Cervical Vertebral Joint, Open Approach, Diagnostic
        "0R910ZZ",  # Drainage of Cervical Vertebral Joint, Open Approach
        "0R9130Z",  # Drainage of Cervical Vertebral Joint with Drainage Device, Percutaneous Approach
        "0R913ZX",  # Drainage of Cervical Vertebral Joint, Percutaneous Approach, Diagnostic
        "0R913ZZ",  # Drainage of Cervical Vertebral Joint, Percutaneous Approach
        "0R9140Z",  # Drainage of Cervical Vertebral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0R914ZX",  # Drainage of Cervical Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0R914ZZ",  # Drainage of Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0R9300Z",  # Drainage of Cervical Vertebral Disc with Drainage Device, Open Approach
        "0R930ZX",  # Drainage of Cervical Vertebral Disc, Open Approach, Diagnostic
        "0R930ZZ",  # Drainage of Cervical Vertebral Disc, Open Approach
        "0R9330Z",  # Drainage of Cervical Vertebral Disc with Drainage Device, Percutaneous Approach
        "0R933ZX",  # Drainage of Cervical Vertebral Disc, Percutaneous Approach, Diagnostic
        "0R933ZZ",  # Drainage of Cervical Vertebral Disc, Percutaneous Approach
        "0R9340Z",  # Drainage of Cervical Vertebral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0R934ZX",  # Drainage of Cervical Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0R934ZZ",  # Drainage of Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0R9400Z",  # Drainage of Cervicothoracic Vertebral Joint with Drainage Device, Open Approach
        "0R940ZX",  # Drainage of Cervicothoracic Vertebral Joint, Open Approach, Diagnostic
        "0R940ZZ",  # Drainage of Cervicothoracic Vertebral Joint, Open Approach
        "0R9430Z",  # Drainage of Cervicothoracic Vertebral Joint with Drainage Device, Percutaneous Approach
        "0R943ZX",  # Drainage of Cervicothoracic Vertebral Joint, Percutaneous Approach, Diagnostic
        "0R943ZZ",  # Drainage of Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0R9440Z",  # Drainage of Cervicothoracic Vertebral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0R944ZX",  # Drainage of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0R944ZZ",  # Drainage of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0R9500Z",  # Drainage of Cervicothoracic Vertebral Disc with Drainage Device, Open Approach
        "0R950ZX",  # Drainage of Cervicothoracic Vertebral Disc, Open Approach, Diagnostic
        "0R950ZZ",  # Drainage of Cervicothoracic Vertebral Disc, Open Approach
        "0R9530Z",  # Drainage of Cervicothoracic Vertebral Disc with Drainage Device, Percutaneous Approach
        "0R953ZX",  # Drainage of Cervicothoracic Vertebral Disc, Percutaneous Approach, Diagnostic
        "0R953ZZ",  # Drainage of Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0R9540Z",  # Drainage of Cervicothoracic Vertebral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0R954ZX",  # Drainage of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0R954ZZ",  # Drainage of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0R9600Z",  # Drainage of Thoracic Vertebral Joint with Drainage Device, Open Approach
        "0R960ZX",  # Drainage of Thoracic Vertebral Joint, Open Approach, Diagnostic
        "0R960ZZ",  # Drainage of Thoracic Vertebral Joint, Open Approach
        "0R9630Z",  # Drainage of Thoracic Vertebral Joint with Drainage Device, Percutaneous Approach
        "0R963ZX",  # Drainage of Thoracic Vertebral Joint, Percutaneous Approach, Diagnostic
        "0R963ZZ",  # Drainage of Thoracic Vertebral Joint, Percutaneous Approach
        "0R9640Z",  # Drainage of Thoracic Vertebral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0R964ZX",  # Drainage of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0R964ZZ",  # Drainage of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0R9900Z",  # Drainage of Thoracic Vertebral Disc with Drainage Device, Open Approach
        "0R990ZX",  # Drainage of Thoracic Vertebral Disc, Open Approach, Diagnostic
        "0R990ZZ",  # Drainage of Thoracic Vertebral Disc, Open Approach
        "0R9930Z",  # Drainage of Thoracic Vertebral Disc with Drainage Device, Percutaneous Approach
        "0R993ZX",  # Drainage of Thoracic Vertebral Disc, Percutaneous Approach, Diagnostic
        "0R993ZZ",  # Drainage of Thoracic Vertebral Disc, Percutaneous Approach
        "0R9940Z",  # Drainage of Thoracic Vertebral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0R994ZX",  # Drainage of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0R994ZZ",  # Drainage of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0R9A00Z",  # Drainage of Thoracolumbar Vertebral Joint with Drainage Device, Open Approach
        "0R9A0ZX",  # Drainage of Thoracolumbar Vertebral Joint, Open Approach, Diagnostic
        "0R9A0ZZ",  # Drainage of Thoracolumbar Vertebral Joint, Open Approach
        "0R9A30Z",  # Drainage of Thoracolumbar Vertebral Joint with Drainage Device, Percutaneous Approach
        "0R9A3ZX",  # Drainage of Thoracolumbar Vertebral Joint, Percutaneous Approach, Diagnostic
        "0R9A3ZZ",  # Drainage of Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0R9A40Z",  # Drainage of Thoracolumbar Vertebral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0R9A4ZX",  # Drainage of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0R9A4ZZ",  # Drainage of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0R9B00Z",  # Drainage of Thoracolumbar Vertebral Disc with Drainage Device, Open Approach
        "0R9B0ZX",  # Drainage of Thoracolumbar Vertebral Disc, Open Approach, Diagnostic
        "0R9B0ZZ",  # Drainage of Thoracolumbar Vertebral Disc, Open Approach
        "0R9B30Z",  # Drainage of Thoracolumbar Vertebral Disc with Drainage Device, Percutaneous Approach
        "0R9B3ZX",  # Drainage of Thoracolumbar Vertebral Disc, Percutaneous Approach, Diagnostic
        "0R9B3ZZ",  # Drainage of Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0R9B40Z",  # Drainage of Thoracolumbar Vertebral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0R9B4ZX",  # Drainage of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0R9B4ZZ",  # Drainage of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RB00ZX",  # Excision of Occipital-cervical Joint, Open Approach, Diagnostic
        "0RB00ZZ",  # Excision of Occipital-cervical Joint, Open Approach
        "0RB03ZX",  # Excision of Occipital-cervical Joint, Percutaneous Approach, Diagnostic
        "0RB03ZZ",  # Excision of Occipital-cervical Joint, Percutaneous Approach
        "0RB04ZX",  # Excision of Occipital-cervical Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0RB04ZZ",  # Excision of Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RB10ZX",  # Excision of Cervical Vertebral Joint, Open Approach, Diagnostic
        "0RB10ZZ",  # Excision of Cervical Vertebral Joint, Open Approach
        "0RB13ZX",  # Excision of Cervical Vertebral Joint, Percutaneous Approach, Diagnostic
        "0RB13ZZ",  # Excision of Cervical Vertebral Joint, Percutaneous Approach
        "0RB14ZX",  # Excision of Cervical Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0RB14ZZ",  # Excision of Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RB30ZX",  # Excision of Cervical Vertebral Disc, Open Approach, Diagnostic
        "0RB30ZZ",  # Excision of Cervical Vertebral Disc, Open Approach
        "0RB33ZX",  # Excision of Cervical Vertebral Disc, Percutaneous Approach, Diagnostic
        "0RB33ZZ",  # Excision of Cervical Vertebral Disc, Percutaneous Approach
        "0RB34ZX",  # Excision of Cervical Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0RB34ZZ",  # Excision of Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RB40ZX",  # Excision of Cervicothoracic Vertebral Joint, Open Approach, Diagnostic
        "0RB40ZZ",  # Excision of Cervicothoracic Vertebral Joint, Open Approach
        "0RB43ZX",  # Excision of Cervicothoracic Vertebral Joint, Percutaneous Approach, Diagnostic
        "0RB43ZZ",  # Excision of Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RB44ZX",  # Excision of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0RB44ZZ",  # Excision of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RB50ZX",  # Excision of Cervicothoracic Vertebral Disc, Open Approach, Diagnostic
        "0RB50ZZ",  # Excision of Cervicothoracic Vertebral Disc, Open Approach
        "0RB53ZX",  # Excision of Cervicothoracic Vertebral Disc, Percutaneous Approach, Diagnostic
        "0RB53ZZ",  # Excision of Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RB54ZX",  # Excision of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0RB54ZZ",  # Excision of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RB60ZX",  # Excision of Thoracic Vertebral Joint, Open Approach, Diagnostic
        "0RB60ZZ",  # Excision of Thoracic Vertebral Joint, Open Approach
        "0RB63ZX",  # Excision of Thoracic Vertebral Joint, Percutaneous Approach, Diagnostic
        "0RB63ZZ",  # Excision of Thoracic Vertebral Joint, Percutaneous Approach
        "0RB64ZX",  # Excision of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0RB64ZZ",  # Excision of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RB90ZX",  # Excision of Thoracic Vertebral Disc, Open Approach, Diagnostic
        "0RB90ZZ",  # Excision of Thoracic Vertebral Disc, Open Approach
        "0RB93ZX",  # Excision of Thoracic Vertebral Disc, Percutaneous Approach, Diagnostic
        "0RB93ZZ",  # Excision of Thoracic Vertebral Disc, Percutaneous Approach
        "0RB94ZX",  # Excision of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0RB94ZZ",  # Excision of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RBA0ZX",  # Excision of Thoracolumbar Vertebral Joint, Open Approach, Diagnostic
        "0RBA0ZZ",  # Excision of Thoracolumbar Vertebral Joint, Open Approach
        "0RBA3ZX",  # Excision of Thoracolumbar Vertebral Joint, Percutaneous Approach, Diagnostic
        "0RBA3ZZ",  # Excision of Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RBA4ZX",  # Excision of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0RBA4ZZ",  # Excision of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RBB0ZX",  # Excision of Thoracolumbar Vertebral Disc, Open Approach, Diagnostic
        "0RBB0ZZ",  # Excision of Thoracolumbar Vertebral Disc, Open Approach
        "0RBB3ZX",  # Excision of Thoracolumbar Vertebral Disc, Percutaneous Approach, Diagnostic
        "0RBB3ZZ",  # Excision of Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RBB4ZX",  # Excision of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0RBB4ZZ",  # Excision of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RC00ZZ",  # Extirpation of Matter from Occipital-cervical Joint, Open Approach
        "0RC03ZZ",  # Extirpation of Matter from Occipital-cervical Joint, Percutaneous Approach
        "0RC04ZZ",  # Extirpation of Matter from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RC10ZZ",  # Extirpation of Matter from Cervical Vertebral Joint, Open Approach
        "0RC13ZZ",  # Extirpation of Matter from Cervical Vertebral Joint, Percutaneous Approach
        "0RC14ZZ",  # Extirpation of Matter from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RC30ZZ",  # Extirpation of Matter from Cervical Vertebral Disc, Open Approach
        "0RC33ZZ",  # Extirpation of Matter from Cervical Vertebral Disc, Percutaneous Approach
        "0RC34ZZ",  # Extirpation of Matter from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RC40ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Joint, Open Approach
        "0RC43ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RC44ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RC50ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Disc, Open Approach
        "0RC53ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RC54ZZ",  # Extirpation of Matter from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RC60ZZ",  # Extirpation of Matter from Thoracic Vertebral Joint, Open Approach
        "0RC63ZZ",  # Extirpation of Matter from Thoracic Vertebral Joint, Percutaneous Approach
        "0RC64ZZ",  # Extirpation of Matter from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RC90ZZ",  # Extirpation of Matter from Thoracic Vertebral Disc, Open Approach
        "0RC93ZZ",  # Extirpation of Matter from Thoracic Vertebral Disc, Percutaneous Approach
        "0RC94ZZ",  # Extirpation of Matter from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RCA0ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Joint, Open Approach
        "0RCA3ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RCA4ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RCB0ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Disc, Open Approach
        "0RCB3ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RCB4ZZ",  # Extirpation of Matter from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RG0070",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG0071",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG007J",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG00A0",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG00AJ",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG00J0",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG00J1",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG00JJ",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG00K0",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG00K1",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG00KJ",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG0370",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG0371",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG037J",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG03A0",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG03AJ",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG03J0",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG03J1",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG03JJ",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG03K0",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG03K1",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG03KJ",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG0470",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG0471",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG047J",  # Fusion of Occipital-cervical Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04A0",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04AJ",  # Fusion of Occipital-cervical Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04J0",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04J1",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG04JJ",  # Fusion of Occipital-cervical Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04K0",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG04K1",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG04KJ",  # Fusion of Occipital-cervical Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG1070",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG1071",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG107J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG10A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG10AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG10J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG10J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG10JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG10K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG10K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG10KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG1370",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG1371",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG137J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG13JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG13K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG13K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG13KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG1470",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG1471",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG147J",  # Fusion of Cervical Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14A0",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14AJ",  # Fusion of Cervical Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14J0",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14J1",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG14JJ",  # Fusion of Cervical Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14K0",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG14K1",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG14KJ",  # Fusion of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG2070",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG2071",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG207J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG20A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG20AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG20J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG20J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG20JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG20K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG20K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG20KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG2370",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG2371",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG237J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG23JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG23K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG23K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG23KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG2470",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG2471",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG247J",  # Fusion of 2 or more Cervical Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24A0",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24AJ",  # Fusion of 2 or more Cervical Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24J0",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24J1",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG24JJ",  # Fusion of 2 or more Cervical Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24K0",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG24K1",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG24KJ",  # Fusion of 2 or more Cervical Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG4070",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG4071",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG407J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG40A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG40AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG40J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG40J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG40JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG40K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG40K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG40KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG4370",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG4371",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG437J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG43JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG43K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG43K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG43KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG4470",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG4471",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG447J",  # Fusion of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44A0",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44AJ",  # Fusion of Cervicothoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44J0",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44J1",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG44JJ",  # Fusion of Cervicothoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44K0",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG44K1",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG44KJ",  # Fusion of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG6070",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG6071",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG607J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG60A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG60AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG60J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG60J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG60JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG60K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG60K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG60KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG6370",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG6371",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG637J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG63JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG63K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG63K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG63KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG6470",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG6471",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG647J",  # Fusion of Thoracic Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64A0",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64AJ",  # Fusion of Thoracic Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64J0",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64J1",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG64JJ",  # Fusion of Thoracic Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64K0",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG64K1",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG64KJ",  # Fusion of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG7070",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG7071",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG707J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG70A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG70AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG70J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG70J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG70JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG70K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG70K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG70KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG7370",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG7371",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG737J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG73JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG73K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG73K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG73KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG7470",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG7471",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG747J",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74A0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74AJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74J0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74J1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG74JJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74K0",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG74K1",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG74KJ",  # Fusion of 2 to 7 Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG8070",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG8071",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG807J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG80A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RG80AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RG80J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG80J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG80JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG80K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RG80K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RG80KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RG8370",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG8371",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG837J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG83JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG83K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RG83K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RG83KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RG8470",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG8471",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG847J",  # Fusion of 8 or more Thoracic Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84A0",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84AJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84J0",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84J1",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG84JJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84K0",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RG84K1",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RG84KJ",  # Fusion of 8 or more Thoracic Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA070",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA071",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA07J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA0A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0RGA0AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0RGA0J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA0J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA0JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA0K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0RGA0K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0RGA0KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0RGA370",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA371",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA37J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA3JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0RGA3K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0RGA3KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0RGA470",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA471",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA47J",  # Fusion of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4A0",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4AJ",  # Fusion of Thoracolumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4J0",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4J1",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA4JJ",  # Fusion of Thoracolumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4K0",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RGA4K1",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0RGA4KJ",  # Fusion of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0RH003Z",  # Insertion of Infusion Device into Occipital-cervical Joint, Open Approach
        "0RH004Z",  # Insertion of Internal Fixation Device into Occipital-cervical Joint, Open Approach
        "0RH008Z",  # Insertion of Spacer into Occipital-cervical Joint, Open Approach
        "0RH00BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Open Approach
        "0RH00CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Open Approach
        "0RH00DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Open Approach
        "0RH033Z",  # Insertion of Infusion Device into Occipital-cervical Joint, Percutaneous Approach
        "0RH034Z",  # Insertion of Internal Fixation Device into Occipital-cervical Joint, Percutaneous Approach
        "0RH038Z",  # Insertion of Spacer into Occipital-cervical Joint, Percutaneous Approach
        "0RH03BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach
        "0RH03CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach
        "0RH03DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Approach
        "0RH043Z",  # Insertion of Infusion Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH044Z",  # Insertion of Internal Fixation Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH048Z",  # Insertion of Spacer into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH04BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH04CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH04DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RH103Z",  # Insertion of Infusion Device into Cervical Vertebral Joint, Open Approach
        "0RH104Z",  # Insertion of Internal Fixation Device into Cervical Vertebral Joint, Open Approach
        "0RH108Z",  # Insertion of Spacer into Cervical Vertebral Joint, Open Approach
        "0RH10BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach
        "0RH10CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach
        "0RH10DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Open Approach
        "0RH133Z",  # Insertion of Infusion Device into Cervical Vertebral Joint, Percutaneous Approach
        "0RH134Z",  # Insertion of Internal Fixation Device into Cervical Vertebral Joint, Percutaneous Approach
        "0RH138Z",  # Insertion of Spacer into Cervical Vertebral Joint, Percutaneous Approach
        "0RH13BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach
        "0RH13CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach
        "0RH13DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Approach
        "0RH143Z",  # Insertion of Infusion Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH144Z",  # Insertion of Internal Fixation Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH148Z",  # Insertion of Spacer into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH14BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH14CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH14DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH303Z",  # Insertion of Infusion Device into Cervical Vertebral Disc, Open Approach
        "0RH333Z",  # Insertion of Infusion Device into Cervical Vertebral Disc, Percutaneous Approach
        "0RH343Z",  # Insertion of Infusion Device into Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RH403Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Joint, Open Approach
        "0RH404Z",  # Insertion of Internal Fixation Device into Cervicothoracic Vertebral Joint, Open Approach
        "0RH408Z",  # Insertion of Spacer into Cervicothoracic Vertebral Joint, Open Approach
        "0RH40BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach
        "0RH40CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach
        "0RH40DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Open Approach
        "0RH433Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH434Z",  # Insertion of Internal Fixation Device into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH438Z",  # Insertion of Spacer into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH43BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH43CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH43DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RH443Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH444Z",  # Insertion of Internal Fixation Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH448Z",  # Insertion of Spacer into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH44BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH44CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH44DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH503Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Disc, Open Approach
        "0RH533Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RH543Z",  # Insertion of Infusion Device into Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RH603Z",  # Insertion of Infusion Device into Thoracic Vertebral Joint, Open Approach
        "0RH604Z",  # Insertion of Internal Fixation Device into Thoracic Vertebral Joint, Open Approach
        "0RH608Z",  # Insertion of Spacer into Thoracic Vertebral Joint, Open Approach
        "0RH60BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach
        "0RH60CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach
        "0RH60DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Open Approach
        "0RH633Z",  # Insertion of Infusion Device into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH634Z",  # Insertion of Internal Fixation Device into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH638Z",  # Insertion of Spacer into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH63BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH63CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH63DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Approach
        "0RH643Z",  # Insertion of Infusion Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH644Z",  # Insertion of Internal Fixation Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH648Z",  # Insertion of Spacer into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH64BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH64CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH64DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RH903Z",  # Insertion of Infusion Device into Thoracic Vertebral Disc, Open Approach
        "0RH933Z",  # Insertion of Infusion Device into Thoracic Vertebral Disc, Percutaneous Approach
        "0RH943Z",  # Insertion of Infusion Device into Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RHA03Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA04Z",  # Insertion of Internal Fixation Device into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA08Z",  # Insertion of Spacer into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA0BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA0CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA0DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Open Approach
        "0RHA33Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA34Z",  # Insertion of Internal Fixation Device into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA38Z",  # Insertion of Spacer into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA3BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA3CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA3DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RHA43Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHA44Z",  # Insertion of Internal Fixation Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHA48Z",  # Insertion of Spacer into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHA4BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHA4CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHA4DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RHB03Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Disc, Open Approach
        "0RHB33Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RHB43Z",  # Insertion of Infusion Device into Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RJ00ZZ",  # Inspection of Occipital-cervical Joint, Open Approach
        "0RJ03ZZ",  # Inspection of Occipital-cervical Joint, Percutaneous Approach
        "0RJ04ZZ",  # Inspection of Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RJ0XZZ",  # Inspection of Occipital-cervical Joint, External Approach
        "0RJ10ZZ",  # Inspection of Cervical Vertebral Joint, Open Approach
        "0RJ13ZZ",  # Inspection of Cervical Vertebral Joint, Percutaneous Approach
        "0RJ14ZZ",  # Inspection of Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RJ1XZZ",  # Inspection of Cervical Vertebral Joint, External Approach
        "0RJ30ZZ",  # Inspection of Cervical Vertebral Disc, Open Approach
        "0RJ33ZZ",  # Inspection of Cervical Vertebral Disc, Percutaneous Approach
        "0RJ34ZZ",  # Inspection of Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RJ3XZZ",  # Inspection of Cervical Vertebral Disc, External Approach
        "0RJ40ZZ",  # Inspection of Cervicothoracic Vertebral Joint, Open Approach
        "0RJ43ZZ",  # Inspection of Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RJ44ZZ",  # Inspection of Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RJ4XZZ",  # Inspection of Cervicothoracic Vertebral Joint, External Approach
        "0RJ50ZZ",  # Inspection of Cervicothoracic Vertebral Disc, Open Approach
        "0RJ53ZZ",  # Inspection of Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RJ54ZZ",  # Inspection of Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RJ5XZZ",  # Inspection of Cervicothoracic Vertebral Disc, External Approach
        "0RJ60ZZ",  # Inspection of Thoracic Vertebral Joint, Open Approach
        "0RJ63ZZ",  # Inspection of Thoracic Vertebral Joint, Percutaneous Approach
        "0RJ64ZZ",  # Inspection of Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RJ6XZZ",  # Inspection of Thoracic Vertebral Joint, External Approach
        "0RJ90ZZ",  # Inspection of Thoracic Vertebral Disc, Open Approach
        "0RJ93ZZ",  # Inspection of Thoracic Vertebral Disc, Percutaneous Approach
        "0RJ94ZZ",  # Inspection of Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RJ9XZZ",  # Inspection of Thoracic Vertebral Disc, External Approach
        "0RJA0ZZ",  # Inspection of Thoracolumbar Vertebral Joint, Open Approach
        "0RJA3ZZ",  # Inspection of Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RJA4ZZ",  # Inspection of Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RJAXZZ",  # Inspection of Thoracolumbar Vertebral Joint, External Approach
        "0RJB0ZZ",  # Inspection of Thoracolumbar Vertebral Disc, Open Approach
        "0RJB3ZZ",  # Inspection of Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RJB4ZZ",  # Inspection of Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RJBXZZ",  # Inspection of Thoracolumbar Vertebral Disc, External Approach
        "0RN00ZZ",  # Release Occipital-cervical Joint, Open Approach
        "0RN03ZZ",  # Release Occipital-cervical Joint, Percutaneous Approach
        "0RN04ZZ",  # Release Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RN0XZZ",  # Release Occipital-cervical Joint, External Approach
        "0RN10ZZ",  # Release Cervical Vertebral Joint, Open Approach
        "0RN13ZZ",  # Release Cervical Vertebral Joint, Percutaneous Approach
        "0RN14ZZ",  # Release Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RN1XZZ",  # Release Cervical Vertebral Joint, External Approach
        "0RN30ZZ",  # Release Cervical Vertebral Disc, Open Approach
        "0RN33ZZ",  # Release Cervical Vertebral Disc, Percutaneous Approach
        "0RN34ZZ",  # Release Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RN3XZZ",  # Release Cervical Vertebral Disc, External Approach
        "0RN40ZZ",  # Release Cervicothoracic Vertebral Joint, Open Approach
        "0RN43ZZ",  # Release Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RN44ZZ",  # Release Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RN4XZZ",  # Release Cervicothoracic Vertebral Joint, External Approach
        "0RN50ZZ",  # Release Cervicothoracic Vertebral Disc, Open Approach
        "0RN53ZZ",  # Release Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RN54ZZ",  # Release Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RN5XZZ",  # Release Cervicothoracic Vertebral Disc, External Approach
        "0RN60ZZ",  # Release Thoracic Vertebral Joint, Open Approach
        "0RN63ZZ",  # Release Thoracic Vertebral Joint, Percutaneous Approach
        "0RN64ZZ",  # Release Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RN6XZZ",  # Release Thoracic Vertebral Joint, External Approach
        "0RN90ZZ",  # Release Thoracic Vertebral Disc, Open Approach
        "0RN93ZZ",  # Release Thoracic Vertebral Disc, Percutaneous Approach
        "0RN94ZZ",  # Release Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RN9XZZ",  # Release Thoracic Vertebral Disc, External Approach
        "0RNA0ZZ",  # Release Thoracolumbar Vertebral Joint, Open Approach
        "0RNA3ZZ",  # Release Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RNA4ZZ",  # Release Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RNAXZZ",  # Release Thoracolumbar Vertebral Joint, External Approach
        "0RNB0ZZ",  # Release Thoracolumbar Vertebral Disc, Open Approach
        "0RNB3ZZ",  # Release Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RNB4ZZ",  # Release Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RNBXZZ",  # Release Thoracolumbar Vertebral Disc, External Approach
        "0RP000Z",  # Removal of Drainage Device from Occipital-cervical Joint, Open Approach
        "0RP003Z",  # Removal of Infusion Device from Occipital-cervical Joint, Open Approach
        "0RP004Z",  # Removal of Internal Fixation Device from Occipital-cervical Joint, Open Approach
        "0RP007Z",  # Removal of Autologous Tissue Substitute from Occipital-cervical Joint, Open Approach
        "0RP008Z",  # Removal of Spacer from Occipital-cervical Joint, Open Approach
        "0RP00AZ",  # Removal of Interbody Fusion Device from Occipital-cervical Joint, Open Approach
        "0RP00JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Open Approach
        "0RP00KZ",  # Removal of Nonautologous Tissue Substitute from Occipital-cervical Joint, Open Approach
        "0RP030Z",  # Removal of Drainage Device from Occipital-cervical Joint, Percutaneous Approach
        "0RP033Z",  # Removal of Infusion Device from Occipital-cervical Joint, Percutaneous Approach
        "0RP034Z",  # Removal of Internal Fixation Device from Occipital-cervical Joint, Percutaneous Approach
        "0RP037Z",  # Removal of Autologous Tissue Substitute from Occipital-cervical Joint, Percutaneous Approach
        "0RP038Z",  # Removal of Spacer from Occipital-cervical Joint, Percutaneous Approach
        "0RP03AZ",  # Removal of Interbody Fusion Device from Occipital-cervical Joint, Percutaneous Approach
        "0RP03JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Percutaneous Approach
        "0RP03KZ",  # Removal of Nonautologous Tissue Substitute from Occipital-cervical Joint, Percutaneous Approach
        "0RP040Z",  # Removal of Drainage Device from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP043Z",  # Removal of Infusion Device from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP044Z",  # Removal of Internal Fixation Device from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP047Z",  # Removal of Autologous Tissue Substitute from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP048Z",  # Removal of Spacer from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP04AZ",  # Removal of Interbody Fusion Device from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP04JZ",  # Removal of Synthetic Substitute from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP04KZ",  # Removal of Nonautologous Tissue Substitute from Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RP0X0Z",  # Removal of Drainage Device from Occipital-cervical Joint, External Approach
        "0RP0X3Z",  # Removal of Infusion Device from Occipital-cervical Joint, External Approach
        "0RP0X4Z",  # Removal of Internal Fixation Device from Occipital-cervical Joint, External Approach
        "0RP100Z",  # Removal of Drainage Device from Cervical Vertebral Joint, Open Approach
        "0RP103Z",  # Removal of Infusion Device from Cervical Vertebral Joint, Open Approach
        "0RP104Z",  # Removal of Internal Fixation Device from Cervical Vertebral Joint, Open Approach
        "0RP107Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Joint, Open Approach
        "0RP108Z",  # Removal of Spacer from Cervical Vertebral Joint, Open Approach
        "0RP10AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Open Approach
        "0RP10JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Open Approach
        "0RP10KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Joint, Open Approach
        "0RP130Z",  # Removal of Drainage Device from Cervical Vertebral Joint, Percutaneous Approach
        "0RP133Z",  # Removal of Infusion Device from Cervical Vertebral Joint, Percutaneous Approach
        "0RP134Z",  # Removal of Internal Fixation Device from Cervical Vertebral Joint, Percutaneous Approach
        "0RP137Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Joint, Percutaneous Approach
        "0RP138Z",  # Removal of Spacer from Cervical Vertebral Joint, Percutaneous Approach
        "0RP13AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Percutaneous Approach
        "0RP13JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Percutaneous Approach
        "0RP13KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Joint, Percutaneous Approach
        "0RP140Z",  # Removal of Drainage Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP143Z",  # Removal of Infusion Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP144Z",  # Removal of Internal Fixation Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP147Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP148Z",  # Removal of Spacer from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP14AZ",  # Removal of Interbody Fusion Device from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP14JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP14KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP1X0Z",  # Removal of Drainage Device from Cervical Vertebral Joint, External Approach
        "0RP1X3Z",  # Removal of Infusion Device from Cervical Vertebral Joint, External Approach
        "0RP1X4Z",  # Removal of Internal Fixation Device from Cervical Vertebral Joint, External Approach
        "0RP300Z",  # Removal of Drainage Device from Cervical Vertebral Disc, Open Approach
        "0RP303Z",  # Removal of Infusion Device from Cervical Vertebral Disc, Open Approach
        "0RP307Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Disc, Open Approach
        "0RP30JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Open Approach
        "0RP30KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Disc, Open Approach
        "0RP330Z",  # Removal of Drainage Device from Cervical Vertebral Disc, Percutaneous Approach
        "0RP333Z",  # Removal of Infusion Device from Cervical Vertebral Disc, Percutaneous Approach
        "0RP337Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Disc, Percutaneous Approach
        "0RP33JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Percutaneous Approach
        "0RP33KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Disc, Percutaneous Approach
        "0RP340Z",  # Removal of Drainage Device from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP343Z",  # Removal of Infusion Device from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP347Z",  # Removal of Autologous Tissue Substitute from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP34JZ",  # Removal of Synthetic Substitute from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP34KZ",  # Removal of Nonautologous Tissue Substitute from Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP3X0Z",  # Removal of Drainage Device from Cervical Vertebral Disc, External Approach
        "0RP3X3Z",  # Removal of Infusion Device from Cervical Vertebral Disc, External Approach
        "0RP400Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Joint, Open Approach
        "0RP403Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Joint, Open Approach
        "0RP404Z",  # Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Open Approach
        "0RP407Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Joint, Open Approach
        "0RP408Z",  # Removal of Spacer from Cervicothoracic Vertebral Joint, Open Approach
        "0RP40AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Open Approach
        "0RP40JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Open Approach
        "0RP40KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Joint, Open Approach
        "0RP430Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP433Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP434Z",  # Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP437Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP438Z",  # Removal of Spacer from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP43AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP43JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP43KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RP440Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP443Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP444Z",  # Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP447Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP448Z",  # Removal of Spacer from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP44AZ",  # Removal of Interbody Fusion Device from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP44JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP44KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP4X0Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Joint, External Approach
        "0RP4X3Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Joint, External Approach
        "0RP4X4Z",  # Removal of Internal Fixation Device from Cervicothoracic Vertebral Joint, External Approach
        "0RP500Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Disc, Open Approach
        "0RP503Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Disc, Open Approach
        "0RP507Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Disc, Open Approach
        "0RP50JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Open Approach
        "0RP50KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Disc, Open Approach
        "0RP530Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP533Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP537Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP53JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP53KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RP540Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP543Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP547Z",  # Removal of Autologous Tissue Substitute from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP54JZ",  # Removal of Synthetic Substitute from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP54KZ",  # Removal of Nonautologous Tissue Substitute from Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP5X0Z",  # Removal of Drainage Device from Cervicothoracic Vertebral Disc, External Approach
        "0RP5X3Z",  # Removal of Infusion Device from Cervicothoracic Vertebral Disc, External Approach
        "0RP600Z",  # Removal of Drainage Device from Thoracic Vertebral Joint, Open Approach
        "0RP603Z",  # Removal of Infusion Device from Thoracic Vertebral Joint, Open Approach
        "0RP604Z",  # Removal of Internal Fixation Device from Thoracic Vertebral Joint, Open Approach
        "0RP607Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Joint, Open Approach
        "0RP608Z",  # Removal of Spacer from Thoracic Vertebral Joint, Open Approach
        "0RP60AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Open Approach
        "0RP60JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Open Approach
        "0RP60KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Joint, Open Approach
        "0RP630Z",  # Removal of Drainage Device from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP633Z",  # Removal of Infusion Device from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP634Z",  # Removal of Internal Fixation Device from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP637Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP638Z",  # Removal of Spacer from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP63AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP63JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP63KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Joint, Percutaneous Approach
        "0RP640Z",  # Removal of Drainage Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP643Z",  # Removal of Infusion Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP644Z",  # Removal of Internal Fixation Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP647Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP648Z",  # Removal of Spacer from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP64AZ",  # Removal of Interbody Fusion Device from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP64JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP64KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RP6X0Z",  # Removal of Drainage Device from Thoracic Vertebral Joint, External Approach
        "0RP6X3Z",  # Removal of Infusion Device from Thoracic Vertebral Joint, External Approach
        "0RP6X4Z",  # Removal of Internal Fixation Device from Thoracic Vertebral Joint, External Approach
        "0RP900Z",  # Removal of Drainage Device from Thoracic Vertebral Disc, Open Approach
        "0RP903Z",  # Removal of Infusion Device from Thoracic Vertebral Disc, Open Approach
        "0RP907Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Disc, Open Approach
        "0RP90JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Open Approach
        "0RP90KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Disc, Open Approach
        "0RP930Z",  # Removal of Drainage Device from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP933Z",  # Removal of Infusion Device from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP937Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP93JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP93KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Disc, Percutaneous Approach
        "0RP940Z",  # Removal of Drainage Device from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP943Z",  # Removal of Infusion Device from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP947Z",  # Removal of Autologous Tissue Substitute from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP94JZ",  # Removal of Synthetic Substitute from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP94KZ",  # Removal of Nonautologous Tissue Substitute from Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RP9X0Z",  # Removal of Drainage Device from Thoracic Vertebral Disc, External Approach
        "0RP9X3Z",  # Removal of Infusion Device from Thoracic Vertebral Disc, External Approach
        "0RPA00Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA03Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA04Z",  # Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA07Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA08Z",  # Removal of Spacer from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA0AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA0JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA0KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Joint, Open Approach
        "0RPA30Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA33Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA34Z",  # Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA37Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA38Z",  # Removal of Spacer from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA3AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA3JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA3KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RPA40Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA43Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA44Z",  # Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA47Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA48Z",  # Removal of Spacer from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA4AZ",  # Removal of Interbody Fusion Device from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA4JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPA4KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RPAX0Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Joint, External Approach
        "0RPAX3Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Joint, External Approach
        "0RPAX4Z",  # Removal of Internal Fixation Device from Thoracolumbar Vertebral Joint, External Approach
        "0RPB00Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB03Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB07Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB0JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB0KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Disc, Open Approach
        "0RPB30Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB33Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB37Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB3JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB3KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RPB40Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPB43Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPB47Z",  # Removal of Autologous Tissue Substitute from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPB4JZ",  # Removal of Synthetic Substitute from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPB4KZ",  # Removal of Nonautologous Tissue Substitute from Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RPBX0Z",  # Removal of Drainage Device from Thoracolumbar Vertebral Disc, External Approach
        "0RPBX3Z",  # Removal of Infusion Device from Thoracolumbar Vertebral Disc, External Approach
        "0RQ00ZZ",  # Repair Occipital-cervical Joint, Open Approach
        "0RQ03ZZ",  # Repair Occipital-cervical Joint, Percutaneous Approach
        "0RQ04ZZ",  # Repair Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RQ0XZZ",  # Repair Occipital-cervical Joint, External Approach
        "0RQ10ZZ",  # Repair Cervical Vertebral Joint, Open Approach
        "0RQ13ZZ",  # Repair Cervical Vertebral Joint, Percutaneous Approach
        "0RQ14ZZ",  # Repair Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RQ1XZZ",  # Repair Cervical Vertebral Joint, External Approach
        "0RQ30ZZ",  # Repair Cervical Vertebral Disc, Open Approach
        "0RQ33ZZ",  # Repair Cervical Vertebral Disc, Percutaneous Approach
        "0RQ34ZZ",  # Repair Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RQ3XZZ",  # Repair Cervical Vertebral Disc, External Approach
        "0RQ40ZZ",  # Repair Cervicothoracic Vertebral Joint, Open Approach
        "0RQ43ZZ",  # Repair Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RQ44ZZ",  # Repair Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RQ4XZZ",  # Repair Cervicothoracic Vertebral Joint, External Approach
        "0RQ50ZZ",  # Repair Cervicothoracic Vertebral Disc, Open Approach
        "0RQ53ZZ",  # Repair Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RQ54ZZ",  # Repair Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RQ5XZZ",  # Repair Cervicothoracic Vertebral Disc, External Approach
        "0RQ60ZZ",  # Repair Thoracic Vertebral Joint, Open Approach
        "0RQ63ZZ",  # Repair Thoracic Vertebral Joint, Percutaneous Approach
        "0RQ64ZZ",  # Repair Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RQ6XZZ",  # Repair Thoracic Vertebral Joint, External Approach
        "0RQ90ZZ",  # Repair Thoracic Vertebral Disc, Open Approach
        "0RQ93ZZ",  # Repair Thoracic Vertebral Disc, Percutaneous Approach
        "0RQ94ZZ",  # Repair Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RQ9XZZ",  # Repair Thoracic Vertebral Disc, External Approach
        "0RQA0ZZ",  # Repair Thoracolumbar Vertebral Joint, Open Approach
        "0RQA3ZZ",  # Repair Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RQA4ZZ",  # Repair Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RQAXZZ",  # Repair Thoracolumbar Vertebral Joint, External Approach
        "0RQB0ZZ",  # Repair Thoracolumbar Vertebral Disc, Open Approach
        "0RQB3ZZ",  # Repair Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RQB4ZZ",  # Repair Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RQBXZZ",  # Repair Thoracolumbar Vertebral Disc, External Approach
        "0RR007Z",  # Replacement of Occipital-cervical Joint with Autologous Tissue Substitute, Open Approach
        "0RR00JZ",  # Replacement of Occipital-cervical Joint with Synthetic Substitute, Open Approach
        "0RR00KZ",  # Replacement of Occipital-cervical Joint with Nonautologous Tissue Substitute, Open Approach
        "0RR107Z",  # Replacement of Cervical Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RR10JZ",  # Replacement of Cervical Vertebral Joint with Synthetic Substitute, Open Approach
        "0RR10KZ",  # Replacement of Cervical Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RR307Z",  # Replacement of Cervical Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RR30JZ",  # Replacement of Cervical Vertebral Disc with Synthetic Substitute, Open Approach
        "0RR30KZ",  # Replacement of Cervical Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RR407Z",  # Replacement of Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RR40JZ",  # Replacement of Cervicothoracic Vertebral Joint with Synthetic Substitute, Open Approach
        "0RR40KZ",  # Replacement of Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RR507Z",  # Replacement of Cervicothoracic Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RR50JZ",  # Replacement of Cervicothoracic Vertebral Disc with Synthetic Substitute, Open Approach
        "0RR50KZ",  # Replacement of Cervicothoracic Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RR607Z",  # Replacement of Thoracic Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RR60JZ",  # Replacement of Thoracic Vertebral Joint with Synthetic Substitute, Open Approach
        "0RR60KZ",  # Replacement of Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RR907Z",  # Replacement of Thoracic Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RR90JZ",  # Replacement of Thoracic Vertebral Disc with Synthetic Substitute, Open Approach
        "0RR90KZ",  # Replacement of Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RRA07Z",  # Replacement of Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RRA0JZ",  # Replacement of Thoracolumbar Vertebral Joint with Synthetic Substitute, Open Approach
        "0RRA0KZ",  # Replacement of Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RRB07Z",  # Replacement of Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RRB0JZ",  # Replacement of Thoracolumbar Vertebral Disc with Synthetic Substitute, Open Approach
        "0RRB0KZ",  # Replacement of Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RS004Z",  # Reposition Occipital-cervical Joint with Internal Fixation Device, Open Approach
        "0RS00ZZ",  # Reposition Occipital-cervical Joint, Open Approach
        "0RS034Z",  # Reposition Occipital-cervical Joint with Internal Fixation Device, Percutaneous Approach
        "0RS03ZZ",  # Reposition Occipital-cervical Joint, Percutaneous Approach
        "0RS044Z",  # Reposition Occipital-cervical Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RS04ZZ",  # Reposition Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RS0X4Z",  # Reposition Occipital-cervical Joint with Internal Fixation Device, External Approach
        "0RS0XZZ",  # Reposition Occipital-cervical Joint, External Approach
        "0RS104Z",  # Reposition Cervical Vertebral Joint with Internal Fixation Device, Open Approach
        "0RS10ZZ",  # Reposition Cervical Vertebral Joint, Open Approach
        "0RS134Z",  # Reposition Cervical Vertebral Joint with Internal Fixation Device, Percutaneous Approach
        "0RS13ZZ",  # Reposition Cervical Vertebral Joint, Percutaneous Approach
        "0RS144Z",  # Reposition Cervical Vertebral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RS14ZZ",  # Reposition Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RS1X4Z",  # Reposition Cervical Vertebral Joint with Internal Fixation Device, External Approach
        "0RS1XZZ",  # Reposition Cervical Vertebral Joint, External Approach
        "0RS404Z",  # Reposition Cervicothoracic Vertebral Joint with Internal Fixation Device, Open Approach
        "0RS40ZZ",  # Reposition Cervicothoracic Vertebral Joint, Open Approach
        "0RS434Z",  # Reposition Cervicothoracic Vertebral Joint with Internal Fixation Device, Percutaneous Approach
        "0RS43ZZ",  # Reposition Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RS444Z",  # Reposition Cervicothoracic Vertebral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RS44ZZ",  # Reposition Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RS4X4Z",  # Reposition Cervicothoracic Vertebral Joint with Internal Fixation Device, External Approach
        "0RS4XZZ",  # Reposition Cervicothoracic Vertebral Joint, External Approach
        "0RS604Z",  # Reposition Thoracic Vertebral Joint with Internal Fixation Device, Open Approach
        "0RS60ZZ",  # Reposition Thoracic Vertebral Joint, Open Approach
        "0RS634Z",  # Reposition Thoracic Vertebral Joint with Internal Fixation Device, Percutaneous Approach
        "0RS63ZZ",  # Reposition Thoracic Vertebral Joint, Percutaneous Approach
        "0RS644Z",  # Reposition Thoracic Vertebral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RS64ZZ",  # Reposition Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RS6X4Z",  # Reposition Thoracic Vertebral Joint with Internal Fixation Device, External Approach
        "0RS6XZZ",  # Reposition Thoracic Vertebral Joint, External Approach
        "0RSA04Z",  # Reposition Thoracolumbar Vertebral Joint with Internal Fixation Device, Open Approach
        "0RSA0ZZ",  # Reposition Thoracolumbar Vertebral Joint, Open Approach
        "0RSA34Z",  # Reposition Thoracolumbar Vertebral Joint with Internal Fixation Device, Percutaneous Approach
        "0RSA3ZZ",  # Reposition Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RSA44Z",  # Reposition Thoracolumbar Vertebral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0RSA4ZZ",  # Reposition Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RSAX4Z",  # Reposition Thoracolumbar Vertebral Joint with Internal Fixation Device, External Approach
        "0RSAXZZ",  # Reposition Thoracolumbar Vertebral Joint, External Approach
        "0RT30ZZ",  # Resection of Cervical Vertebral Disc, Open Approach
        "0RT40ZZ",  # Resection of Cervicothoracic Vertebral Joint, Open Approach
        "0RT50ZZ",  # Resection of Cervicothoracic Vertebral Disc, Open Approach
        "0RT90ZZ",  # Resection of Thoracic Vertebral Disc, Open Approach
        "0RTB0ZZ",  # Resection of Thoracolumbar Vertebral Disc, Open Approach
        "0RU007Z",  # Supplement Occipital-cervical Joint with Autologous Tissue Substitute, Open Approach
        "0RU00JZ",  # Supplement Occipital-cervical Joint with Synthetic Substitute, Open Approach
        "0RU00KZ",  # Supplement Occipital-cervical Joint with Nonautologous Tissue Substitute, Open Approach
        "0RU037Z",  # Supplement Occipital-cervical Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RU03JZ",  # Supplement Occipital-cervical Joint with Synthetic Substitute, Percutaneous Approach
        "0RU03KZ",  # Supplement Occipital-cervical Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU047Z",  # Supplement Occipital-cervical Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU04JZ",  # Supplement Occipital-cervical Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU04KZ",  # Supplement Occipital-cervical Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU107Z",  # Supplement Cervical Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RU10JZ",  # Supplement Cervical Vertebral Joint with Synthetic Substitute, Open Approach
        "0RU10KZ",  # Supplement Cervical Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RU137Z",  # Supplement Cervical Vertebral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RU13JZ",  # Supplement Cervical Vertebral Joint with Synthetic Substitute, Percutaneous Approach
        "0RU13KZ",  # Supplement Cervical Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU147Z",  # Supplement Cervical Vertebral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU14JZ",  # Supplement Cervical Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU14KZ",  # Supplement Cervical Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU307Z",  # Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RU30JZ",  # Supplement Cervical Vertebral Disc with Synthetic Substitute, Open Approach
        "0RU30KZ",  # Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RU337Z",  # Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0RU33JZ",  # Supplement Cervical Vertebral Disc with Synthetic Substitute, Percutaneous Approach
        "0RU33KZ",  # Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU347Z",  # Supplement Cervical Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU34JZ",  # Supplement Cervical Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU34KZ",  # Supplement Cervical Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU407Z",  # Supplement Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RU40JZ",  # Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Open Approach
        "0RU40KZ",  # Supplement Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RU437Z",  # Supplement Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RU43JZ",  # Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Percutaneous Approach
        "0RU43KZ",  # Supplement Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU447Z",  # Supplement Cervicothoracic Vertebral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU44JZ",  # Supplement Cervicothoracic Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU44KZ",  # Supplement Cervicothoracic Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU507Z",  # Supplement Cervicothoracic Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RU50JZ",  # Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Open Approach
        "0RU50KZ",  # Supplement Cervicothoracic Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RU537Z",  # Supplement Cervicothoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0RU53JZ",  # Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Percutaneous Approach
        "0RU53KZ",  # Supplement Cervicothoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU547Z",  # Supplement Cervicothoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU54JZ",  # Supplement Cervicothoracic Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU54KZ",  # Supplement Cervicothoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU607Z",  # Supplement Thoracic Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RU60JZ",  # Supplement Thoracic Vertebral Joint with Synthetic Substitute, Open Approach
        "0RU60KZ",  # Supplement Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RU637Z",  # Supplement Thoracic Vertebral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RU63JZ",  # Supplement Thoracic Vertebral Joint with Synthetic Substitute, Percutaneous Approach
        "0RU63KZ",  # Supplement Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU647Z",  # Supplement Thoracic Vertebral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU64JZ",  # Supplement Thoracic Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU64KZ",  # Supplement Thoracic Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU907Z",  # Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RU90JZ",  # Supplement Thoracic Vertebral Disc with Synthetic Substitute, Open Approach
        "0RU90KZ",  # Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RU937Z",  # Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0RU93JZ",  # Supplement Thoracic Vertebral Disc with Synthetic Substitute, Percutaneous Approach
        "0RU93KZ",  # Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RU947Z",  # Supplement Thoracic Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RU94JZ",  # Supplement Thoracic Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RU94KZ",  # Supplement Thoracic Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RUA07Z",  # Supplement Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0RUA0JZ",  # Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Open Approach
        "0RUA0KZ",  # Supplement Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0RUA37Z",  # Supplement Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0RUA3JZ",  # Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Percutaneous Approach
        "0RUA3KZ",  # Supplement Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RUA47Z",  # Supplement Thoracolumbar Vertebral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RUA4JZ",  # Supplement Thoracolumbar Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RUA4KZ",  # Supplement Thoracolumbar Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RUB07Z",  # Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0RUB0JZ",  # Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Open Approach
        "0RUB0KZ",  # Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0RUB37Z",  # Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0RUB3JZ",  # Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Percutaneous Approach
        "0RUB3KZ",  # Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0RUB47Z",  # Supplement Thoracolumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RUB4JZ",  # Supplement Thoracolumbar Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0RUB4KZ",  # Supplement Thoracolumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0RW000Z",  # Revision of Drainage Device in Occipital-cervical Joint, Open Approach
        "0RW003Z",  # Revision of Infusion Device in Occipital-cervical Joint, Open Approach
        "0RW004Z",  # Revision of Internal Fixation Device in Occipital-cervical Joint, Open Approach
        "0RW007Z",  # Revision of Autologous Tissue Substitute in Occipital-cervical Joint, Open Approach
        "0RW008Z",  # Revision of Spacer in Occipital-cervical Joint, Open Approach
        "0RW00AZ",  # Revision of Interbody Fusion Device in Occipital-cervical Joint, Open Approach
        "0RW00JZ",  # Revision of Synthetic Substitute in Occipital-cervical Joint, Open Approach
        "0RW00KZ",  # Revision of Nonautologous Tissue Substitute in Occipital-cervical Joint, Open Approach
        "0RW030Z",  # Revision of Drainage Device in Occipital-cervical Joint, Percutaneous Approach
        "0RW033Z",  # Revision of Infusion Device in Occipital-cervical Joint, Percutaneous Approach
        "0RW034Z",  # Revision of Internal Fixation Device in Occipital-cervical Joint, Percutaneous Approach
        "0RW037Z",  # Revision of Autologous Tissue Substitute in Occipital-cervical Joint, Percutaneous Approach
        "0RW038Z",  # Revision of Spacer in Occipital-cervical Joint, Percutaneous Approach
        "0RW03AZ",  # Revision of Interbody Fusion Device in Occipital-cervical Joint, Percutaneous Approach
        "0RW03JZ",  # Revision of Synthetic Substitute in Occipital-cervical Joint, Percutaneous Approach
        "0RW03KZ",  # Revision of Nonautologous Tissue Substitute in Occipital-cervical Joint, Percutaneous Approach
        "0RW040Z",  # Revision of Drainage Device in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW043Z",  # Revision of Infusion Device in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW044Z",  # Revision of Internal Fixation Device in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW047Z",  # Revision of Autologous Tissue Substitute in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW048Z",  # Revision of Spacer in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW04AZ",  # Revision of Interbody Fusion Device in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW04JZ",  # Revision of Synthetic Substitute in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW04KZ",  # Revision of Nonautologous Tissue Substitute in Occipital-cervical Joint, Percutaneous Endoscopic Approach
        "0RW0X0Z",  # Revision of Drainage Device in Occipital-cervical Joint, External Approach
        "0RW0X3Z",  # Revision of Infusion Device in Occipital-cervical Joint, External Approach
        "0RW0X4Z",  # Revision of Internal Fixation Device in Occipital-cervical Joint, External Approach
        "0RW0X7Z",  # Revision of Autologous Tissue Substitute in Occipital-cervical Joint, External Approach
        "0RW0X8Z",  # Revision of Spacer in Occipital-cervical Joint, External Approach
        "0RW0XAZ",  # Revision of Interbody Fusion Device in Occipital-cervical Joint, External Approach
        "0RW0XJZ",  # Revision of Synthetic Substitute in Occipital-cervical Joint, External Approach
        "0RW0XKZ",  # Revision of Nonautologous Tissue Substitute in Occipital-cervical Joint, External Approach
        "0RW100Z",  # Revision of Drainage Device in Cervical Vertebral Joint, Open Approach
        "0RW103Z",  # Revision of Infusion Device in Cervical Vertebral Joint, Open Approach
        "0RW104Z",  # Revision of Internal Fixation Device in Cervical Vertebral Joint, Open Approach
        "0RW107Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Joint, Open Approach
        "0RW108Z",  # Revision of Spacer in Cervical Vertebral Joint, Open Approach
        "0RW10AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Open Approach
        "0RW10JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Joint, Open Approach
        "0RW10KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Joint, Open Approach
        "0RW130Z",  # Revision of Drainage Device in Cervical Vertebral Joint, Percutaneous Approach
        "0RW133Z",  # Revision of Infusion Device in Cervical Vertebral Joint, Percutaneous Approach
        "0RW134Z",  # Revision of Internal Fixation Device in Cervical Vertebral Joint, Percutaneous Approach
        "0RW137Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Joint, Percutaneous Approach
        "0RW138Z",  # Revision of Spacer in Cervical Vertebral Joint, Percutaneous Approach
        "0RW13AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Percutaneous Approach
        "0RW13JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Joint, Percutaneous Approach
        "0RW13KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Joint, Percutaneous Approach
        "0RW140Z",  # Revision of Drainage Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW143Z",  # Revision of Infusion Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW144Z",  # Revision of Internal Fixation Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW147Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW148Z",  # Revision of Spacer in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW14AZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW14JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW14KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW1X0Z",  # Revision of Drainage Device in Cervical Vertebral Joint, External Approach
        "0RW1X3Z",  # Revision of Infusion Device in Cervical Vertebral Joint, External Approach
        "0RW1X4Z",  # Revision of Internal Fixation Device in Cervical Vertebral Joint, External Approach
        "0RW1X7Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Joint, External Approach
        "0RW1X8Z",  # Revision of Spacer in Cervical Vertebral Joint, External Approach
        "0RW1XAZ",  # Revision of Interbody Fusion Device in Cervical Vertebral Joint, External Approach
        "0RW1XJZ",  # Revision of Synthetic Substitute in Cervical Vertebral Joint, External Approach
        "0RW1XKZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Joint, External Approach
        "0RW300Z",  # Revision of Drainage Device in Cervical Vertebral Disc, Open Approach
        "0RW303Z",  # Revision of Infusion Device in Cervical Vertebral Disc, Open Approach
        "0RW307Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Disc, Open Approach
        "0RW30JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Disc, Open Approach
        "0RW30KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Disc, Open Approach
        "0RW330Z",  # Revision of Drainage Device in Cervical Vertebral Disc, Percutaneous Approach
        "0RW333Z",  # Revision of Infusion Device in Cervical Vertebral Disc, Percutaneous Approach
        "0RW337Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Disc, Percutaneous Approach
        "0RW33JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Disc, Percutaneous Approach
        "0RW33KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Disc, Percutaneous Approach
        "0RW340Z",  # Revision of Drainage Device in Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW343Z",  # Revision of Infusion Device in Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW347Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW34JZ",  # Revision of Synthetic Substitute in Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW34KZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW3X0Z",  # Revision of Drainage Device in Cervical Vertebral Disc, External Approach
        "0RW3X3Z",  # Revision of Infusion Device in Cervical Vertebral Disc, External Approach
        "0RW3X7Z",  # Revision of Autologous Tissue Substitute in Cervical Vertebral Disc, External Approach
        "0RW3XJZ",  # Revision of Synthetic Substitute in Cervical Vertebral Disc, External Approach
        "0RW3XKZ",  # Revision of Nonautologous Tissue Substitute in Cervical Vertebral Disc, External Approach
        "0RW400Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Joint, Open Approach
        "0RW403Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Joint, Open Approach
        "0RW404Z",  # Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Open Approach
        "0RW407Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Joint, Open Approach
        "0RW408Z",  # Revision of Spacer in Cervicothoracic Vertebral Joint, Open Approach
        "0RW40AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Open Approach
        "0RW40JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Open Approach
        "0RW40KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Joint, Open Approach
        "0RW430Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW433Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW434Z",  # Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW437Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW438Z",  # Revision of Spacer in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW43AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW43JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW43KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Joint, Percutaneous Approach
        "0RW440Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW443Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW444Z",  # Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW447Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW448Z",  # Revision of Spacer in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW44AZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW44JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW44KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW4X0Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Joint, External Approach
        "0RW4X3Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Joint, External Approach
        "0RW4X4Z",  # Revision of Internal Fixation Device in Cervicothoracic Vertebral Joint, External Approach
        "0RW4X7Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Joint, External Approach
        "0RW4X8Z",  # Revision of Spacer in Cervicothoracic Vertebral Joint, External Approach
        "0RW4XAZ",  # Revision of Interbody Fusion Device in Cervicothoracic Vertebral Joint, External Approach
        "0RW4XJZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Joint, External Approach
        "0RW4XKZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Joint, External Approach
        "0RW500Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Disc, Open Approach
        "0RW503Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Disc, Open Approach
        "0RW507Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Disc, Open Approach
        "0RW50JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Open Approach
        "0RW50KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Disc, Open Approach
        "0RW530Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RW533Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RW537Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RW53JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RW53KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Disc, Percutaneous Approach
        "0RW540Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW543Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW547Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW54JZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW54KZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW5X0Z",  # Revision of Drainage Device in Cervicothoracic Vertebral Disc, External Approach
        "0RW5X3Z",  # Revision of Infusion Device in Cervicothoracic Vertebral Disc, External Approach
        "0RW5X7Z",  # Revision of Autologous Tissue Substitute in Cervicothoracic Vertebral Disc, External Approach
        "0RW5XJZ",  # Revision of Synthetic Substitute in Cervicothoracic Vertebral Disc, External Approach
        "0RW5XKZ",  # Revision of Nonautologous Tissue Substitute in Cervicothoracic Vertebral Disc, External Approach
        "0RW600Z",  # Revision of Drainage Device in Thoracic Vertebral Joint, Open Approach
        "0RW603Z",  # Revision of Infusion Device in Thoracic Vertebral Joint, Open Approach
        "0RW604Z",  # Revision of Internal Fixation Device in Thoracic Vertebral Joint, Open Approach
        "0RW607Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Joint, Open Approach
        "0RW608Z",  # Revision of Spacer in Thoracic Vertebral Joint, Open Approach
        "0RW60AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Open Approach
        "0RW60JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Joint, Open Approach
        "0RW60KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Joint, Open Approach
        "0RW630Z",  # Revision of Drainage Device in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW633Z",  # Revision of Infusion Device in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW634Z",  # Revision of Internal Fixation Device in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW637Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW638Z",  # Revision of Spacer in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW63AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW63JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW63KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Joint, Percutaneous Approach
        "0RW640Z",  # Revision of Drainage Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW643Z",  # Revision of Infusion Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW644Z",  # Revision of Internal Fixation Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW647Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW648Z",  # Revision of Spacer in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW64AZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW64JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW64KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Joint, Percutaneous Endoscopic Approach
        "0RW6X0Z",  # Revision of Drainage Device in Thoracic Vertebral Joint, External Approach
        "0RW6X3Z",  # Revision of Infusion Device in Thoracic Vertebral Joint, External Approach
        "0RW6X4Z",  # Revision of Internal Fixation Device in Thoracic Vertebral Joint, External Approach
        "0RW6X7Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Joint, External Approach
        "0RW6X8Z",  # Revision of Spacer in Thoracic Vertebral Joint, External Approach
        "0RW6XAZ",  # Revision of Interbody Fusion Device in Thoracic Vertebral Joint, External Approach
        "0RW6XJZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Joint, External Approach
        "0RW6XKZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Joint, External Approach
        "0RW900Z",  # Revision of Drainage Device in Thoracic Vertebral Disc, Open Approach
        "0RW903Z",  # Revision of Infusion Device in Thoracic Vertebral Disc, Open Approach
        "0RW907Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Disc, Open Approach
        "0RW90JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Disc, Open Approach
        "0RW90KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Disc, Open Approach
        "0RW930Z",  # Revision of Drainage Device in Thoracic Vertebral Disc, Percutaneous Approach
        "0RW933Z",  # Revision of Infusion Device in Thoracic Vertebral Disc, Percutaneous Approach
        "0RW937Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Disc, Percutaneous Approach
        "0RW93JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Disc, Percutaneous Approach
        "0RW93KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Disc, Percutaneous Approach
        "0RW940Z",  # Revision of Drainage Device in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW943Z",  # Revision of Infusion Device in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW947Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW94JZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW94KZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Disc, Percutaneous Endoscopic Approach
        "0RW9X0Z",  # Revision of Drainage Device in Thoracic Vertebral Disc, External Approach
        "0RW9X3Z",  # Revision of Infusion Device in Thoracic Vertebral Disc, External Approach
        "0RW9X7Z",  # Revision of Autologous Tissue Substitute in Thoracic Vertebral Disc, External Approach
        "0RW9XJZ",  # Revision of Synthetic Substitute in Thoracic Vertebral Disc, External Approach
        "0RW9XKZ",  # Revision of Nonautologous Tissue Substitute in Thoracic Vertebral Disc, External Approach
        "0RWA00Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA03Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA04Z",  # Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA07Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA08Z",  # Revision of Spacer in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA0AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA0JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA0KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Joint, Open Approach
        "0RWA30Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA33Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA34Z",  # Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA37Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA38Z",  # Revision of Spacer in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA3AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA3JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA3KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Joint, Percutaneous Approach
        "0RWA40Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA43Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA44Z",  # Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA47Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA48Z",  # Revision of Spacer in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA4AZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA4JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWA4KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0RWAX0Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Joint, External Approach
        "0RWAX3Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Joint, External Approach
        "0RWAX4Z",  # Revision of Internal Fixation Device in Thoracolumbar Vertebral Joint, External Approach
        "0RWAX7Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Joint, External Approach
        "0RWAX8Z",  # Revision of Spacer in Thoracolumbar Vertebral Joint, External Approach
        "0RWAXAZ",  # Revision of Interbody Fusion Device in Thoracolumbar Vertebral Joint, External Approach
        "0RWAXJZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Joint, External Approach
        "0RWAXKZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Joint, External Approach
        "0RWB00Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Disc, Open Approach
        "0RWB03Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Disc, Open Approach
        "0RWB07Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Disc, Open Approach
        "0RWB0JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Open Approach
        "0RWB0KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Disc, Open Approach
        "0RWB30Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RWB33Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RWB37Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RWB3JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RWB3KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Disc, Percutaneous Approach
        "0RWB40Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RWB43Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RWB47Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RWB4JZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RWB4KZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0RWBX0Z",  # Revision of Drainage Device in Thoracolumbar Vertebral Disc, External Approach
        "0RWBX3Z",  # Revision of Infusion Device in Thoracolumbar Vertebral Disc, External Approach
        "0RWBX7Z",  # Revision of Autologous Tissue Substitute in Thoracolumbar Vertebral Disc, External Approach
        "0RWBXJZ",  # Revision of Synthetic Substitute in Thoracolumbar Vertebral Disc, External Approach
        "0RWBXKZ",  # Revision of Nonautologous Tissue Substitute in Thoracolumbar Vertebral Disc, External Approach
        "0S500ZZ",  # Destruction of Lumbar Vertebral Joint, Open Approach
        "0S503ZZ",  # Destruction of Lumbar Vertebral Joint, Percutaneous Approach
        "0S504ZZ",  # Destruction of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0S520ZZ",  # Destruction of Lumbar Vertebral Disc, Open Approach
        "0S523ZZ",  # Destruction of Lumbar Vertebral Disc, Percutaneous Approach
        "0S524ZZ",  # Destruction of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0S530ZZ",  # Destruction of Lumbosacral Joint, Open Approach
        "0S533ZZ",  # Destruction of Lumbosacral Joint, Percutaneous Approach
        "0S534ZZ",  # Destruction of Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0S540ZZ",  # Destruction of Lumbosacral Disc, Open Approach
        "0S543ZZ",  # Destruction of Lumbosacral Disc, Percutaneous Approach
        "0S544ZZ",  # Destruction of Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0S550ZZ",  # Destruction of Sacrococcygeal Joint, Open Approach
        "0S553ZZ",  # Destruction of Sacrococcygeal Joint, Percutaneous Approach
        "0S554ZZ",  # Destruction of Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0S560ZZ",  # Destruction of Coccygeal Joint, Open Approach
        "0S563ZZ",  # Destruction of Coccygeal Joint, Percutaneous Approach
        "0S564ZZ",  # Destruction of Coccygeal Joint, Percutaneous Endoscopic Approach
        "0S9000Z",  # Drainage of Lumbar Vertebral Joint with Drainage Device, Open Approach
        "0S900ZX",  # Drainage of Lumbar Vertebral Joint, Open Approach, Diagnostic
        "0S900ZZ",  # Drainage of Lumbar Vertebral Joint, Open Approach
        "0S9030Z",  # Drainage of Lumbar Vertebral Joint with Drainage Device, Percutaneous Approach
        "0S903ZX",  # Drainage of Lumbar Vertebral Joint, Percutaneous Approach, Diagnostic
        "0S903ZZ",  # Drainage of Lumbar Vertebral Joint, Percutaneous Approach
        "0S9040Z",  # Drainage of Lumbar Vertebral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0S904ZX",  # Drainage of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0S904ZZ",  # Drainage of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0S9200Z",  # Drainage of Lumbar Vertebral Disc with Drainage Device, Open Approach
        "0S920ZX",  # Drainage of Lumbar Vertebral Disc, Open Approach, Diagnostic
        "0S920ZZ",  # Drainage of Lumbar Vertebral Disc, Open Approach
        "0S9230Z",  # Drainage of Lumbar Vertebral Disc with Drainage Device, Percutaneous Approach
        "0S923ZX",  # Drainage of Lumbar Vertebral Disc, Percutaneous Approach, Diagnostic
        "0S923ZZ",  # Drainage of Lumbar Vertebral Disc, Percutaneous Approach
        "0S9240Z",  # Drainage of Lumbar Vertebral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0S924ZX",  # Drainage of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0S924ZZ",  # Drainage of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0S9300Z",  # Drainage of Lumbosacral Joint with Drainage Device, Open Approach
        "0S930ZX",  # Drainage of Lumbosacral Joint, Open Approach, Diagnostic
        "0S930ZZ",  # Drainage of Lumbosacral Joint, Open Approach
        "0S9330Z",  # Drainage of Lumbosacral Joint with Drainage Device, Percutaneous Approach
        "0S933ZX",  # Drainage of Lumbosacral Joint, Percutaneous Approach, Diagnostic
        "0S933ZZ",  # Drainage of Lumbosacral Joint, Percutaneous Approach
        "0S9340Z",  # Drainage of Lumbosacral Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0S934ZX",  # Drainage of Lumbosacral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0S934ZZ",  # Drainage of Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0S9400Z",  # Drainage of Lumbosacral Disc with Drainage Device, Open Approach
        "0S940ZX",  # Drainage of Lumbosacral Disc, Open Approach, Diagnostic
        "0S940ZZ",  # Drainage of Lumbosacral Disc, Open Approach
        "0S9430Z",  # Drainage of Lumbosacral Disc with Drainage Device, Percutaneous Approach
        "0S943ZX",  # Drainage of Lumbosacral Disc, Percutaneous Approach, Diagnostic
        "0S943ZZ",  # Drainage of Lumbosacral Disc, Percutaneous Approach
        "0S9440Z",  # Drainage of Lumbosacral Disc with Drainage Device, Percutaneous Endoscopic Approach
        "0S944ZX",  # Drainage of Lumbosacral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0S944ZZ",  # Drainage of Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0S9500Z",  # Drainage of Sacrococcygeal Joint with Drainage Device, Open Approach
        "0S950ZX",  # Drainage of Sacrococcygeal Joint, Open Approach, Diagnostic
        "0S950ZZ",  # Drainage of Sacrococcygeal Joint, Open Approach
        "0S9530Z",  # Drainage of Sacrococcygeal Joint with Drainage Device, Percutaneous Approach
        "0S953ZX",  # Drainage of Sacrococcygeal Joint, Percutaneous Approach, Diagnostic
        "0S953ZZ",  # Drainage of Sacrococcygeal Joint, Percutaneous Approach
        "0S9540Z",  # Drainage of Sacrococcygeal Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0S954ZX",  # Drainage of Sacrococcygeal Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0S954ZZ",  # Drainage of Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0S9600Z",  # Drainage of Coccygeal Joint with Drainage Device, Open Approach
        "0S960ZX",  # Drainage of Coccygeal Joint, Open Approach, Diagnostic
        "0S960ZZ",  # Drainage of Coccygeal Joint, Open Approach
        "0S9630Z",  # Drainage of Coccygeal Joint with Drainage Device, Percutaneous Approach
        "0S963ZX",  # Drainage of Coccygeal Joint, Percutaneous Approach, Diagnostic
        "0S963ZZ",  # Drainage of Coccygeal Joint, Percutaneous Approach
        "0S9640Z",  # Drainage of Coccygeal Joint with Drainage Device, Percutaneous Endoscopic Approach
        "0S964ZX",  # Drainage of Coccygeal Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0S964ZZ",  # Drainage of Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SB00ZX",  # Excision of Lumbar Vertebral Joint, Open Approach, Diagnostic
        "0SB00ZZ",  # Excision of Lumbar Vertebral Joint, Open Approach
        "0SB03ZX",  # Excision of Lumbar Vertebral Joint, Percutaneous Approach, Diagnostic
        "0SB03ZZ",  # Excision of Lumbar Vertebral Joint, Percutaneous Approach
        "0SB04ZX",  # Excision of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0SB04ZZ",  # Excision of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SB20ZX",  # Excision of Lumbar Vertebral Disc, Open Approach, Diagnostic
        "0SB20ZZ",  # Excision of Lumbar Vertebral Disc, Open Approach
        "0SB23ZX",  # Excision of Lumbar Vertebral Disc, Percutaneous Approach, Diagnostic
        "0SB23ZZ",  # Excision of Lumbar Vertebral Disc, Percutaneous Approach
        "0SB24ZX",  # Excision of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0SB24ZZ",  # Excision of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SB30ZX",  # Excision of Lumbosacral Joint, Open Approach, Diagnostic
        "0SB30ZZ",  # Excision of Lumbosacral Joint, Open Approach
        "0SB33ZX",  # Excision of Lumbosacral Joint, Percutaneous Approach, Diagnostic
        "0SB33ZZ",  # Excision of Lumbosacral Joint, Percutaneous Approach
        "0SB34ZX",  # Excision of Lumbosacral Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0SB34ZZ",  # Excision of Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SB40ZX",  # Excision of Lumbosacral Disc, Open Approach, Diagnostic
        "0SB40ZZ",  # Excision of Lumbosacral Disc, Open Approach
        "0SB43ZX",  # Excision of Lumbosacral Disc, Percutaneous Approach, Diagnostic
        "0SB43ZZ",  # Excision of Lumbosacral Disc, Percutaneous Approach
        "0SB44ZX",  # Excision of Lumbosacral Disc, Percutaneous Endoscopic Approach, Diagnostic
        "0SB44ZZ",  # Excision of Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SB50ZX",  # Excision of Sacrococcygeal Joint, Open Approach, Diagnostic
        "0SB50ZZ",  # Excision of Sacrococcygeal Joint, Open Approach
        "0SB53ZX",  # Excision of Sacrococcygeal Joint, Percutaneous Approach, Diagnostic
        "0SB53ZZ",  # Excision of Sacrococcygeal Joint, Percutaneous Approach
        "0SB54ZX",  # Excision of Sacrococcygeal Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0SB54ZZ",  # Excision of Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SB60ZX",  # Excision of Coccygeal Joint, Open Approach, Diagnostic
        "0SB60ZZ",  # Excision of Coccygeal Joint, Open Approach
        "0SB63ZX",  # Excision of Coccygeal Joint, Percutaneous Approach, Diagnostic
        "0SB63ZZ",  # Excision of Coccygeal Joint, Percutaneous Approach
        "0SB64ZX",  # Excision of Coccygeal Joint, Percutaneous Endoscopic Approach, Diagnostic
        "0SB64ZZ",  # Excision of Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SC00ZZ",  # Extirpation of Matter from Lumbar Vertebral Joint, Open Approach
        "0SC03ZZ",  # Extirpation of Matter from Lumbar Vertebral Joint, Percutaneous Approach
        "0SC04ZZ",  # Extirpation of Matter from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SC20ZZ",  # Extirpation of Matter from Lumbar Vertebral Disc, Open Approach
        "0SC23ZZ",  # Extirpation of Matter from Lumbar Vertebral Disc, Percutaneous Approach
        "0SC24ZZ",  # Extirpation of Matter from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SC30ZZ",  # Extirpation of Matter from Lumbosacral Joint, Open Approach
        "0SC33ZZ",  # Extirpation of Matter from Lumbosacral Joint, Percutaneous Approach
        "0SC34ZZ",  # Extirpation of Matter from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SC40ZZ",  # Extirpation of Matter from Lumbosacral Disc, Open Approach
        "0SC43ZZ",  # Extirpation of Matter from Lumbosacral Disc, Percutaneous Approach
        "0SC44ZZ",  # Extirpation of Matter from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SC50ZZ",  # Extirpation of Matter from Sacrococcygeal Joint, Open Approach
        "0SC53ZZ",  # Extirpation of Matter from Sacrococcygeal Joint, Percutaneous Approach
        "0SC54ZZ",  # Extirpation of Matter from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SC60ZZ",  # Extirpation of Matter from Coccygeal Joint, Open Approach
        "0SC63ZZ",  # Extirpation of Matter from Coccygeal Joint, Percutaneous Approach
        "0SC64ZZ",  # Extirpation of Matter from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SG0070",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG0071",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG007J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG00A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0SG00AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0SG00J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG00J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG00JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG00K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG00K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG00KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG0370",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG0371",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG037J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG03JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG03K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG03K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG03KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG0470",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG0471",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG047J",  # Fusion of Lumbar Vertebral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04A0",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04AJ",  # Fusion of Lumbar Vertebral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04J0",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04J1",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG04JJ",  # Fusion of Lumbar Vertebral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04K0",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG04K1",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG04KJ",  # Fusion of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG1070",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG1071",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG107J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG10A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0SG10AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0SG10J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG10J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG10JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG10K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG10K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG10KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG1370",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG1371",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG137J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG13JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG13K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG13K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG13KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG1470",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG1471",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG147J",  # Fusion of 2 or more Lumbar Vertebral Joints with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14A0",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14AJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14J0",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14J1",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG14JJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14K0",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG14K1",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG14KJ",  # Fusion of 2 or more Lumbar Vertebral Joints with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG3070",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG3071",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG307J",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG30A0",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Open Approach
        "0SG30AJ",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Open Approach
        "0SG30J0",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG30J1",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG30JJ",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG30K0",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Open Approach
        "0SG30K1",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Open Approach
        "0SG30KJ",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Open Approach
        "0SG3370",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG3371",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG337J",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG33A0",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG33AJ",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG33J0",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG33J1",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG33JJ",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG33K0",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Approach
        "0SG33K1",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Approach
        "0SG33KJ",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Approach
        "0SG3470",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG3471",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG347J",  # Fusion of Lumbosacral Joint with Autologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34A0",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34AJ",  # Fusion of Lumbosacral Joint with Interbody Fusion Device, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34J0",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34J1",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG34JJ",  # Fusion of Lumbosacral Joint with Synthetic Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34K0",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Anterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG34K1",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Posterior Column, Percutaneous Endoscopic Approach
        "0SG34KJ",  # Fusion of Lumbosacral Joint with Nonautologous Tissue Substitute, Posterior Approach, Anterior Column, Percutaneous Endoscopic Approach
        "0SG504Z",  # Fusion of Sacrococcygeal Joint with Internal Fixation Device, Open Approach
        "0SG507Z",  # Fusion of Sacrococcygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SG50JZ",  # Fusion of Sacrococcygeal Joint with Synthetic Substitute, Open Approach
        "0SG50KZ",  # Fusion of Sacrococcygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SG534Z",  # Fusion of Sacrococcygeal Joint with Internal Fixation Device, Percutaneous Approach
        "0SG537Z",  # Fusion of Sacrococcygeal Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SG53JZ",  # Fusion of Sacrococcygeal Joint with Synthetic Substitute, Percutaneous Approach
        "0SG53KZ",  # Fusion of Sacrococcygeal Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SG544Z",  # Fusion of Sacrococcygeal Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SG547Z",  # Fusion of Sacrococcygeal Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SG54JZ",  # Fusion of Sacrococcygeal Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SG54KZ",  # Fusion of Sacrococcygeal Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SG604Z",  # Fusion of Coccygeal Joint with Internal Fixation Device, Open Approach
        "0SG607Z",  # Fusion of Coccygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SG60JZ",  # Fusion of Coccygeal Joint with Synthetic Substitute, Open Approach
        "0SG60KZ",  # Fusion of Coccygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SG634Z",  # Fusion of Coccygeal Joint with Internal Fixation Device, Percutaneous Approach
        "0SG637Z",  # Fusion of Coccygeal Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SG63JZ",  # Fusion of Coccygeal Joint with Synthetic Substitute, Percutaneous Approach
        "0SG63KZ",  # Fusion of Coccygeal Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SG644Z",  # Fusion of Coccygeal Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SG647Z",  # Fusion of Coccygeal Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SG64JZ",  # Fusion of Coccygeal Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SG64KZ",  # Fusion of Coccygeal Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SH003Z",  # Insertion of Infusion Device into Lumbar Vertebral Joint, Open Approach
        "0SH004Z",  # Insertion of Internal Fixation Device into Lumbar Vertebral Joint, Open Approach
        "0SH008Z",  # Insertion of Spacer into Lumbar Vertebral Joint, Open Approach
        "0SH00BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach
        "0SH00CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach
        "0SH00DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Open Approach
        "0SH033Z",  # Insertion of Infusion Device into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH034Z",  # Insertion of Internal Fixation Device into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH038Z",  # Insertion of Spacer into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH03BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH03CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH03DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Approach
        "0SH043Z",  # Insertion of Infusion Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH044Z",  # Insertion of Internal Fixation Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH048Z",  # Insertion of Spacer into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH04BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH04CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH04DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SH203Z",  # Insertion of Infusion Device into Lumbar Vertebral Disc, Open Approach
        "0SH208Z",  # Insertion of Spacer into Lumbar Vertebral Disc, Open Approach
        "0SH233Z",  # Insertion of Infusion Device into Lumbar Vertebral Disc, Percutaneous Approach
        "0SH238Z",  # Insertion of Spacer into Lumbar Vertebral Disc, Percutaneous Approach
        "0SH243Z",  # Insertion of Infusion Device into Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SH248Z",  # Insertion of Spacer into Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SH303Z",  # Insertion of Infusion Device into Lumbosacral Joint, Open Approach
        "0SH304Z",  # Insertion of Internal Fixation Device into Lumbosacral Joint, Open Approach
        "0SH308Z",  # Insertion of Spacer into Lumbosacral Joint, Open Approach
        "0SH30BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Open Approach
        "0SH30CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Open Approach
        "0SH30DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Open Approach
        "0SH333Z",  # Insertion of Infusion Device into Lumbosacral Joint, Percutaneous Approach
        "0SH334Z",  # Insertion of Internal Fixation Device into Lumbosacral Joint, Percutaneous Approach
        "0SH338Z",  # Insertion of Spacer into Lumbosacral Joint, Percutaneous Approach
        "0SH33BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach
        "0SH33CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach
        "0SH33DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Approach
        "0SH343Z",  # Insertion of Infusion Device into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH344Z",  # Insertion of Internal Fixation Device into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH348Z",  # Insertion of Spacer into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH34BZ",  # Insertion of Interspinous Process Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH34CZ",  # Insertion of Pedicle-Based Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH34DZ",  # Insertion of Facet Replacement Spinal Stabilization Device into Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SH403Z",  # Insertion of Infusion Device into Lumbosacral Disc, Open Approach
        "0SH408Z",  # Insertion of Spacer into Lumbosacral Disc, Open Approach
        "0SH433Z",  # Insertion of Infusion Device into Lumbosacral Disc, Percutaneous Approach
        "0SH438Z",  # Insertion of Spacer into Lumbosacral Disc, Percutaneous Approach
        "0SH443Z",  # Insertion of Infusion Device into Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SH448Z",  # Insertion of Spacer into Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SH503Z",  # Insertion of Infusion Device into Sacrococcygeal Joint, Open Approach
        "0SH504Z",  # Insertion of Internal Fixation Device into Sacrococcygeal Joint, Open Approach
        "0SH508Z",  # Insertion of Spacer into Sacrococcygeal Joint, Open Approach
        "0SH533Z",  # Insertion of Infusion Device into Sacrococcygeal Joint, Percutaneous Approach
        "0SH534Z",  # Insertion of Internal Fixation Device into Sacrococcygeal Joint, Percutaneous Approach
        "0SH538Z",  # Insertion of Spacer into Sacrococcygeal Joint, Percutaneous Approach
        "0SH543Z",  # Insertion of Infusion Device into Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SH544Z",  # Insertion of Internal Fixation Device into Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SH548Z",  # Insertion of Spacer into Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SH603Z",  # Insertion of Infusion Device into Coccygeal Joint, Open Approach
        "0SH604Z",  # Insertion of Internal Fixation Device into Coccygeal Joint, Open Approach
        "0SH608Z",  # Insertion of Spacer into Coccygeal Joint, Open Approach
        "0SH633Z",  # Insertion of Infusion Device into Coccygeal Joint, Percutaneous Approach
        "0SH634Z",  # Insertion of Internal Fixation Device into Coccygeal Joint, Percutaneous Approach
        "0SH638Z",  # Insertion of Spacer into Coccygeal Joint, Percutaneous Approach
        "0SH643Z",  # Insertion of Infusion Device into Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SH644Z",  # Insertion of Internal Fixation Device into Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SH648Z",  # Insertion of Spacer into Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SJ00ZZ",  # Inspection of Lumbar Vertebral Joint, Open Approach
        "0SJ03ZZ",  # Inspection of Lumbar Vertebral Joint, Percutaneous Approach
        "0SJ04ZZ",  # Inspection of Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SJ0XZZ",  # Inspection of Lumbar Vertebral Joint, External Approach
        "0SJ20ZZ",  # Inspection of Lumbar Vertebral Disc, Open Approach
        "0SJ23ZZ",  # Inspection of Lumbar Vertebral Disc, Percutaneous Approach
        "0SJ24ZZ",  # Inspection of Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SJ2XZZ",  # Inspection of Lumbar Vertebral Disc, External Approach
        "0SJ30ZZ",  # Inspection of Lumbosacral Joint, Open Approach
        "0SJ33ZZ",  # Inspection of Lumbosacral Joint, Percutaneous Approach
        "0SJ34ZZ",  # Inspection of Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SJ3XZZ",  # Inspection of Lumbosacral Joint, External Approach
        "0SJ40ZZ",  # Inspection of Lumbosacral Disc, Open Approach
        "0SJ43ZZ",  # Inspection of Lumbosacral Disc, Percutaneous Approach
        "0SJ44ZZ",  # Inspection of Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SJ4XZZ",  # Inspection of Lumbosacral Disc, External Approach
        "0SJ50ZZ",  # Inspection of Sacrococcygeal Joint, Open Approach
        "0SJ53ZZ",  # Inspection of Sacrococcygeal Joint, Percutaneous Approach
        "0SJ54ZZ",  # Inspection of Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SJ5XZZ",  # Inspection of Sacrococcygeal Joint, External Approach
        "0SJ60ZZ",  # Inspection of Coccygeal Joint, Open Approach
        "0SJ63ZZ",  # Inspection of Coccygeal Joint, Percutaneous Approach
        "0SJ64ZZ",  # Inspection of Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SJ6XZZ",  # Inspection of Coccygeal Joint, External Approach
        "0SN00ZZ",  # Release Lumbar Vertebral Joint, Open Approach
        "0SN03ZZ",  # Release Lumbar Vertebral Joint, Percutaneous Approach
        "0SN04ZZ",  # Release Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SN0XZZ",  # Release Lumbar Vertebral Joint, External Approach
        "0SN20ZZ",  # Release Lumbar Vertebral Disc, Open Approach
        "0SN23ZZ",  # Release Lumbar Vertebral Disc, Percutaneous Approach
        "0SN24ZZ",  # Release Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SN2XZZ",  # Release Lumbar Vertebral Disc, External Approach
        "0SN30ZZ",  # Release Lumbosacral Joint, Open Approach
        "0SN33ZZ",  # Release Lumbosacral Joint, Percutaneous Approach
        "0SN34ZZ",  # Release Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SN3XZZ",  # Release Lumbosacral Joint, External Approach
        "0SN40ZZ",  # Release Lumbosacral Disc, Open Approach
        "0SN43ZZ",  # Release Lumbosacral Disc, Percutaneous Approach
        "0SN44ZZ",  # Release Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SN4XZZ",  # Release Lumbosacral Disc, External Approach
        "0SN50ZZ",  # Release Sacrococcygeal Joint, Open Approach
        "0SN53ZZ",  # Release Sacrococcygeal Joint, Percutaneous Approach
        "0SN54ZZ",  # Release Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SN5XZZ",  # Release Sacrococcygeal Joint, External Approach
        "0SN60ZZ",  # Release Coccygeal Joint, Open Approach
        "0SN63ZZ",  # Release Coccygeal Joint, Percutaneous Approach
        "0SN64ZZ",  # Release Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SN6XZZ",  # Release Coccygeal Joint, External Approach
        "0SP000Z",  # Removal of Drainage Device from Lumbar Vertebral Joint, Open Approach
        "0SP003Z",  # Removal of Infusion Device from Lumbar Vertebral Joint, Open Approach
        "0SP004Z",  # Removal of Internal Fixation Device from Lumbar Vertebral Joint, Open Approach
        "0SP007Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Joint, Open Approach
        "0SP008Z",  # Removal of Spacer from Lumbar Vertebral Joint, Open Approach
        "0SP00AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Open Approach
        "0SP00JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Open Approach
        "0SP00KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Joint, Open Approach
        "0SP030Z",  # Removal of Drainage Device from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP033Z",  # Removal of Infusion Device from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP034Z",  # Removal of Internal Fixation Device from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP037Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP038Z",  # Removal of Spacer from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP03AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP03JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP03KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Joint, Percutaneous Approach
        "0SP040Z",  # Removal of Drainage Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP043Z",  # Removal of Infusion Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP044Z",  # Removal of Internal Fixation Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP047Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP048Z",  # Removal of Spacer from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP04AZ",  # Removal of Interbody Fusion Device from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP04JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP04KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SP0X0Z",  # Removal of Drainage Device from Lumbar Vertebral Joint, External Approach
        "0SP0X3Z",  # Removal of Infusion Device from Lumbar Vertebral Joint, External Approach
        "0SP0X4Z",  # Removal of Internal Fixation Device from Lumbar Vertebral Joint, External Approach
        "0SP200Z",  # Removal of Drainage Device from Lumbar Vertebral Disc, Open Approach
        "0SP203Z",  # Removal of Infusion Device from Lumbar Vertebral Disc, Open Approach
        "0SP207Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Disc, Open Approach
        "0SP20JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Open Approach
        "0SP20KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Disc, Open Approach
        "0SP230Z",  # Removal of Drainage Device from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP233Z",  # Removal of Infusion Device from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP237Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP23JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP23KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Disc, Percutaneous Approach
        "0SP240Z",  # Removal of Drainage Device from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP243Z",  # Removal of Infusion Device from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP247Z",  # Removal of Autologous Tissue Substitute from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP24JZ",  # Removal of Synthetic Substitute from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP24KZ",  # Removal of Nonautologous Tissue Substitute from Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SP2X0Z",  # Removal of Drainage Device from Lumbar Vertebral Disc, External Approach
        "0SP2X3Z",  # Removal of Infusion Device from Lumbar Vertebral Disc, External Approach
        "0SP300Z",  # Removal of Drainage Device from Lumbosacral Joint, Open Approach
        "0SP303Z",  # Removal of Infusion Device from Lumbosacral Joint, Open Approach
        "0SP304Z",  # Removal of Internal Fixation Device from Lumbosacral Joint, Open Approach
        "0SP307Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Joint, Open Approach
        "0SP308Z",  # Removal of Spacer from Lumbosacral Joint, Open Approach
        "0SP30AZ",  # Removal of Interbody Fusion Device from Lumbosacral Joint, Open Approach
        "0SP30JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Open Approach
        "0SP30KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Joint, Open Approach
        "0SP330Z",  # Removal of Drainage Device from Lumbosacral Joint, Percutaneous Approach
        "0SP333Z",  # Removal of Infusion Device from Lumbosacral Joint, Percutaneous Approach
        "0SP334Z",  # Removal of Internal Fixation Device from Lumbosacral Joint, Percutaneous Approach
        "0SP337Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Joint, Percutaneous Approach
        "0SP338Z",  # Removal of Spacer from Lumbosacral Joint, Percutaneous Approach
        "0SP33AZ",  # Removal of Interbody Fusion Device from Lumbosacral Joint, Percutaneous Approach
        "0SP33JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Percutaneous Approach
        "0SP33KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Joint, Percutaneous Approach
        "0SP340Z",  # Removal of Drainage Device from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP343Z",  # Removal of Infusion Device from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP344Z",  # Removal of Internal Fixation Device from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP347Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP348Z",  # Removal of Spacer from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP34AZ",  # Removal of Interbody Fusion Device from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP34JZ",  # Removal of Synthetic Substitute from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP34KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SP3X0Z",  # Removal of Drainage Device from Lumbosacral Joint, External Approach
        "0SP3X3Z",  # Removal of Infusion Device from Lumbosacral Joint, External Approach
        "0SP3X4Z",  # Removal of Internal Fixation Device from Lumbosacral Joint, External Approach
        "0SP400Z",  # Removal of Drainage Device from Lumbosacral Disc, Open Approach
        "0SP403Z",  # Removal of Infusion Device from Lumbosacral Disc, Open Approach
        "0SP407Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Disc, Open Approach
        "0SP40JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Open Approach
        "0SP40KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Disc, Open Approach
        "0SP430Z",  # Removal of Drainage Device from Lumbosacral Disc, Percutaneous Approach
        "0SP433Z",  # Removal of Infusion Device from Lumbosacral Disc, Percutaneous Approach
        "0SP437Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Disc, Percutaneous Approach
        "0SP43JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Percutaneous Approach
        "0SP43KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Disc, Percutaneous Approach
        "0SP440Z",  # Removal of Drainage Device from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP443Z",  # Removal of Infusion Device from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP447Z",  # Removal of Autologous Tissue Substitute from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP44JZ",  # Removal of Synthetic Substitute from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP44KZ",  # Removal of Nonautologous Tissue Substitute from Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SP4X0Z",  # Removal of Drainage Device from Lumbosacral Disc, External Approach
        "0SP4X3Z",  # Removal of Infusion Device from Lumbosacral Disc, External Approach
        "0SP500Z",  # Removal of Drainage Device from Sacrococcygeal Joint, Open Approach
        "0SP503Z",  # Removal of Infusion Device from Sacrococcygeal Joint, Open Approach
        "0SP504Z",  # Removal of Internal Fixation Device from Sacrococcygeal Joint, Open Approach
        "0SP507Z",  # Removal of Autologous Tissue Substitute from Sacrococcygeal Joint, Open Approach
        "0SP508Z",  # Removal of Spacer from Sacrococcygeal Joint, Open Approach
        "0SP50JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Open Approach
        "0SP50KZ",  # Removal of Nonautologous Tissue Substitute from Sacrococcygeal Joint, Open Approach
        "0SP530Z",  # Removal of Drainage Device from Sacrococcygeal Joint, Percutaneous Approach
        "0SP533Z",  # Removal of Infusion Device from Sacrococcygeal Joint, Percutaneous Approach
        "0SP534Z",  # Removal of Internal Fixation Device from Sacrococcygeal Joint, Percutaneous Approach
        "0SP537Z",  # Removal of Autologous Tissue Substitute from Sacrococcygeal Joint, Percutaneous Approach
        "0SP538Z",  # Removal of Spacer from Sacrococcygeal Joint, Percutaneous Approach
        "0SP53JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Percutaneous Approach
        "0SP53KZ",  # Removal of Nonautologous Tissue Substitute from Sacrococcygeal Joint, Percutaneous Approach
        "0SP540Z",  # Removal of Drainage Device from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP543Z",  # Removal of Infusion Device from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP544Z",  # Removal of Internal Fixation Device from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP547Z",  # Removal of Autologous Tissue Substitute from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP548Z",  # Removal of Spacer from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP54JZ",  # Removal of Synthetic Substitute from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP54KZ",  # Removal of Nonautologous Tissue Substitute from Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SP5X0Z",  # Removal of Drainage Device from Sacrococcygeal Joint, External Approach
        "0SP5X3Z",  # Removal of Infusion Device from Sacrococcygeal Joint, External Approach
        "0SP5X4Z",  # Removal of Internal Fixation Device from Sacrococcygeal Joint, External Approach
        "0SP600Z",  # Removal of Drainage Device from Coccygeal Joint, Open Approach
        "0SP603Z",  # Removal of Infusion Device from Coccygeal Joint, Open Approach
        "0SP604Z",  # Removal of Internal Fixation Device from Coccygeal Joint, Open Approach
        "0SP607Z",  # Removal of Autologous Tissue Substitute from Coccygeal Joint, Open Approach
        "0SP608Z",  # Removal of Spacer from Coccygeal Joint, Open Approach
        "0SP60JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Open Approach
        "0SP60KZ",  # Removal of Nonautologous Tissue Substitute from Coccygeal Joint, Open Approach
        "0SP630Z",  # Removal of Drainage Device from Coccygeal Joint, Percutaneous Approach
        "0SP633Z",  # Removal of Infusion Device from Coccygeal Joint, Percutaneous Approach
        "0SP634Z",  # Removal of Internal Fixation Device from Coccygeal Joint, Percutaneous Approach
        "0SP637Z",  # Removal of Autologous Tissue Substitute from Coccygeal Joint, Percutaneous Approach
        "0SP638Z",  # Removal of Spacer from Coccygeal Joint, Percutaneous Approach
        "0SP63JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Percutaneous Approach
        "0SP63KZ",  # Removal of Nonautologous Tissue Substitute from Coccygeal Joint, Percutaneous Approach
        "0SP640Z",  # Removal of Drainage Device from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP643Z",  # Removal of Infusion Device from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP644Z",  # Removal of Internal Fixation Device from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP647Z",  # Removal of Autologous Tissue Substitute from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP648Z",  # Removal of Spacer from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP64JZ",  # Removal of Synthetic Substitute from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP64KZ",  # Removal of Nonautologous Tissue Substitute from Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SP6X0Z",  # Removal of Drainage Device from Coccygeal Joint, External Approach
        "0SP6X3Z",  # Removal of Infusion Device from Coccygeal Joint, External Approach
        "0SP6X4Z",  # Removal of Internal Fixation Device from Coccygeal Joint, External Approach
        "0SQ00ZZ",  # Repair Lumbar Vertebral Joint, Open Approach
        "0SQ03ZZ",  # Repair Lumbar Vertebral Joint, Percutaneous Approach
        "0SQ04ZZ",  # Repair Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SQ0XZZ",  # Repair Lumbar Vertebral Joint, External Approach
        "0SQ20ZZ",  # Repair Lumbar Vertebral Disc, Open Approach
        "0SQ23ZZ",  # Repair Lumbar Vertebral Disc, Percutaneous Approach
        "0SQ24ZZ",  # Repair Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SQ2XZZ",  # Repair Lumbar Vertebral Disc, External Approach
        "0SQ30ZZ",  # Repair Lumbosacral Joint, Open Approach
        "0SQ33ZZ",  # Repair Lumbosacral Joint, Percutaneous Approach
        "0SQ34ZZ",  # Repair Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SQ3XZZ",  # Repair Lumbosacral Joint, External Approach
        "0SQ40ZZ",  # Repair Lumbosacral Disc, Open Approach
        "0SQ43ZZ",  # Repair Lumbosacral Disc, Percutaneous Approach
        "0SQ44ZZ",  # Repair Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SQ4XZZ",  # Repair Lumbosacral Disc, External Approach
        "0SQ50ZZ",  # Repair Sacrococcygeal Joint, Open Approach
        "0SQ53ZZ",  # Repair Sacrococcygeal Joint, Percutaneous Approach
        "0SQ54ZZ",  # Repair Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SQ5XZZ",  # Repair Sacrococcygeal Joint, External Approach
        "0SQ60ZZ",  # Repair Coccygeal Joint, Open Approach
        "0SQ63ZZ",  # Repair Coccygeal Joint, Percutaneous Approach
        "0SQ64ZZ",  # Repair Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SQ6XZZ",  # Repair Coccygeal Joint, External Approach
        "0SR007Z",  # Replacement of Lumbar Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0SR00JZ",  # Replacement of Lumbar Vertebral Joint with Synthetic Substitute, Open Approach
        "0SR00KZ",  # Replacement of Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0SR207Z",  # Replacement of Lumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0SR20JZ",  # Replacement of Lumbar Vertebral Disc with Synthetic Substitute, Open Approach
        "0SR20KZ",  # Replacement of Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0SR307Z",  # Replacement of Lumbosacral Joint with Autologous Tissue Substitute, Open Approach
        "0SR30JZ",  # Replacement of Lumbosacral Joint with Synthetic Substitute, Open Approach
        "0SR30KZ",  # Replacement of Lumbosacral Joint with Nonautologous Tissue Substitute, Open Approach
        "0SR407Z",  # Replacement of Lumbosacral Disc with Autologous Tissue Substitute, Open Approach
        "0SR40JZ",  # Replacement of Lumbosacral Disc with Synthetic Substitute, Open Approach
        "0SR40KZ",  # Replacement of Lumbosacral Disc with Nonautologous Tissue Substitute, Open Approach
        "0SR507Z",  # Replacement of Sacrococcygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SR50JZ",  # Replacement of Sacrococcygeal Joint with Synthetic Substitute, Open Approach
        "0SR50KZ",  # Replacement of Sacrococcygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SR607Z",  # Replacement of Coccygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SR60JZ",  # Replacement of Coccygeal Joint with Synthetic Substitute, Open Approach
        "0SR60KZ",  # Replacement of Coccygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SS004Z",  # Reposition Lumbar Vertebral Joint with Internal Fixation Device, Open Approach
        "0SS00ZZ",  # Reposition Lumbar Vertebral Joint, Open Approach
        "0SS034Z",  # Reposition Lumbar Vertebral Joint with Internal Fixation Device, Percutaneous Approach
        "0SS03ZZ",  # Reposition Lumbar Vertebral Joint, Percutaneous Approach
        "0SS044Z",  # Reposition Lumbar Vertebral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SS04ZZ",  # Reposition Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SS0X4Z",  # Reposition Lumbar Vertebral Joint with Internal Fixation Device, External Approach
        "0SS0XZZ",  # Reposition Lumbar Vertebral Joint, External Approach
        "0SS304Z",  # Reposition Lumbosacral Joint with Internal Fixation Device, Open Approach
        "0SS30ZZ",  # Reposition Lumbosacral Joint, Open Approach
        "0SS334Z",  # Reposition Lumbosacral Joint with Internal Fixation Device, Percutaneous Approach
        "0SS33ZZ",  # Reposition Lumbosacral Joint, Percutaneous Approach
        "0SS344Z",  # Reposition Lumbosacral Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SS34ZZ",  # Reposition Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SS3X4Z",  # Reposition Lumbosacral Joint with Internal Fixation Device, External Approach
        "0SS3XZZ",  # Reposition Lumbosacral Joint, External Approach
        "0SS504Z",  # Reposition Sacrococcygeal Joint with Internal Fixation Device, Open Approach
        "0SS50ZZ",  # Reposition Sacrococcygeal Joint, Open Approach
        "0SS534Z",  # Reposition Sacrococcygeal Joint with Internal Fixation Device, Percutaneous Approach
        "0SS53ZZ",  # Reposition Sacrococcygeal Joint, Percutaneous Approach
        "0SS544Z",  # Reposition Sacrococcygeal Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SS54ZZ",  # Reposition Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SS5X4Z",  # Reposition Sacrococcygeal Joint with Internal Fixation Device, External Approach
        "0SS5XZZ",  # Reposition Sacrococcygeal Joint, External Approach
        "0SS604Z",  # Reposition Coccygeal Joint with Internal Fixation Device, Open Approach
        "0SS60ZZ",  # Reposition Coccygeal Joint, Open Approach
        "0SS634Z",  # Reposition Coccygeal Joint with Internal Fixation Device, Percutaneous Approach
        "0SS63ZZ",  # Reposition Coccygeal Joint, Percutaneous Approach
        "0SS644Z",  # Reposition Coccygeal Joint with Internal Fixation Device, Percutaneous Endoscopic Approach
        "0SS64ZZ",  # Reposition Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SS6X4Z",  # Reposition Coccygeal Joint with Internal Fixation Device, External Approach
        "0SS6XZZ",  # Reposition Coccygeal Joint, External Approach
        "0ST20ZZ",  # Resection of Lumbar Vertebral Disc, Open Approach
        "0ST40ZZ",  # Resection of Lumbosacral Disc, Open Approach
        "0ST50ZZ",  # Resection of Sacrococcygeal Joint, Open Approach
        "0ST60ZZ",  # Resection of Coccygeal Joint, Open Approach
        "0SU007Z",  # Supplement Lumbar Vertebral Joint with Autologous Tissue Substitute, Open Approach
        "0SU00JZ",  # Supplement Lumbar Vertebral Joint with Synthetic Substitute, Open Approach
        "0SU00KZ",  # Supplement Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Open Approach
        "0SU037Z",  # Supplement Lumbar Vertebral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SU03JZ",  # Supplement Lumbar Vertebral Joint with Synthetic Substitute, Percutaneous Approach
        "0SU03KZ",  # Supplement Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU047Z",  # Supplement Lumbar Vertebral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU04JZ",  # Supplement Lumbar Vertebral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU04KZ",  # Supplement Lumbar Vertebral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU207Z",  # Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Open Approach
        "0SU20JZ",  # Supplement Lumbar Vertebral Disc with Synthetic Substitute, Open Approach
        "0SU20KZ",  # Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Open Approach
        "0SU237Z",  # Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0SU23JZ",  # Supplement Lumbar Vertebral Disc with Synthetic Substitute, Percutaneous Approach
        "0SU23KZ",  # Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU247Z",  # Supplement Lumbar Vertebral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU24JZ",  # Supplement Lumbar Vertebral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU24KZ",  # Supplement Lumbar Vertebral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU307Z",  # Supplement Lumbosacral Joint with Autologous Tissue Substitute, Open Approach
        "0SU30JZ",  # Supplement Lumbosacral Joint with Synthetic Substitute, Open Approach
        "0SU30KZ",  # Supplement Lumbosacral Joint with Nonautologous Tissue Substitute, Open Approach
        "0SU337Z",  # Supplement Lumbosacral Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SU33JZ",  # Supplement Lumbosacral Joint with Synthetic Substitute, Percutaneous Approach
        "0SU33KZ",  # Supplement Lumbosacral Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU347Z",  # Supplement Lumbosacral Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU34JZ",  # Supplement Lumbosacral Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU34KZ",  # Supplement Lumbosacral Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU407Z",  # Supplement Lumbosacral Disc with Autologous Tissue Substitute, Open Approach
        "0SU40JZ",  # Supplement Lumbosacral Disc with Synthetic Substitute, Open Approach
        "0SU40KZ",  # Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Open Approach
        "0SU437Z",  # Supplement Lumbosacral Disc with Autologous Tissue Substitute, Percutaneous Approach
        "0SU43JZ",  # Supplement Lumbosacral Disc with Synthetic Substitute, Percutaneous Approach
        "0SU43KZ",  # Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU447Z",  # Supplement Lumbosacral Disc with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU44JZ",  # Supplement Lumbosacral Disc with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU44KZ",  # Supplement Lumbosacral Disc with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU507Z",  # Supplement Sacrococcygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SU50JZ",  # Supplement Sacrococcygeal Joint with Synthetic Substitute, Open Approach
        "0SU50KZ",  # Supplement Sacrococcygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SU537Z",  # Supplement Sacrococcygeal Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SU53JZ",  # Supplement Sacrococcygeal Joint with Synthetic Substitute, Percutaneous Approach
        "0SU53KZ",  # Supplement Sacrococcygeal Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU547Z",  # Supplement Sacrococcygeal Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU54JZ",  # Supplement Sacrococcygeal Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU54KZ",  # Supplement Sacrococcygeal Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU607Z",  # Supplement Coccygeal Joint with Autologous Tissue Substitute, Open Approach
        "0SU60JZ",  # Supplement Coccygeal Joint with Synthetic Substitute, Open Approach
        "0SU60KZ",  # Supplement Coccygeal Joint with Nonautologous Tissue Substitute, Open Approach
        "0SU637Z",  # Supplement Coccygeal Joint with Autologous Tissue Substitute, Percutaneous Approach
        "0SU63JZ",  # Supplement Coccygeal Joint with Synthetic Substitute, Percutaneous Approach
        "0SU63KZ",  # Supplement Coccygeal Joint with Nonautologous Tissue Substitute, Percutaneous Approach
        "0SU647Z",  # Supplement Coccygeal Joint with Autologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SU64JZ",  # Supplement Coccygeal Joint with Synthetic Substitute, Percutaneous Endoscopic Approach
        "0SU64KZ",  # Supplement Coccygeal Joint with Nonautologous Tissue Substitute, Percutaneous Endoscopic Approach
        "0SW000Z",  # Revision of Drainage Device in Lumbar Vertebral Joint, Open Approach
        "0SW003Z",  # Revision of Infusion Device in Lumbar Vertebral Joint, Open Approach
        "0SW004Z",  # Revision of Internal Fixation Device in Lumbar Vertebral Joint, Open Approach
        "0SW007Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Joint, Open Approach
        "0SW008Z",  # Revision of Spacer in Lumbar Vertebral Joint, Open Approach
        "0SW00JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Joint, Open Approach
        "0SW00KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Joint, Open Approach
        "0SW030Z",  # Revision of Drainage Device in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW033Z",  # Revision of Infusion Device in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW034Z",  # Revision of Internal Fixation Device in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW037Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW038Z",  # Revision of Spacer in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW03AZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW03JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW03KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Joint, Percutaneous Approach
        "0SW040Z",  # Revision of Drainage Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW043Z",  # Revision of Infusion Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW044Z",  # Revision of Internal Fixation Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW047Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW048Z",  # Revision of Spacer in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW04AZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW04JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW04KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Joint, Percutaneous Endoscopic Approach
        "0SW0X0Z",  # Revision of Drainage Device in Lumbar Vertebral Joint, External Approach
        "0SW0X3Z",  # Revision of Infusion Device in Lumbar Vertebral Joint, External Approach
        "0SW0X4Z",  # Revision of Internal Fixation Device in Lumbar Vertebral Joint, External Approach
        "0SW0X7Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Joint, External Approach
        "0SW0X8Z",  # Revision of Spacer in Lumbar Vertebral Joint, External Approach
        "0SW0XAZ",  # Revision of Interbody Fusion Device in Lumbar Vertebral Joint, External Approach
        "0SW0XJZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Joint, External Approach
        "0SW0XKZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Joint, External Approach
        "0SW200Z",  # Revision of Drainage Device in Lumbar Vertebral Disc, Open Approach
        "0SW203Z",  # Revision of Infusion Device in Lumbar Vertebral Disc, Open Approach
        "0SW207Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Disc, Open Approach
        "0SW20JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Disc, Open Approach
        "0SW20KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Disc, Open Approach
        "0SW230Z",  # Revision of Drainage Device in Lumbar Vertebral Disc, Percutaneous Approach
        "0SW233Z",  # Revision of Infusion Device in Lumbar Vertebral Disc, Percutaneous Approach
        "0SW237Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Disc, Percutaneous Approach
        "0SW23JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Disc, Percutaneous Approach
        "0SW23KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Disc, Percutaneous Approach
        "0SW240Z",  # Revision of Drainage Device in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SW243Z",  # Revision of Infusion Device in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SW247Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SW24JZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SW24KZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Disc, Percutaneous Endoscopic Approach
        "0SW2X0Z",  # Revision of Drainage Device in Lumbar Vertebral Disc, External Approach
        "0SW2X3Z",  # Revision of Infusion Device in Lumbar Vertebral Disc, External Approach
        "0SW2X7Z",  # Revision of Autologous Tissue Substitute in Lumbar Vertebral Disc, External Approach
        "0SW2XJZ",  # Revision of Synthetic Substitute in Lumbar Vertebral Disc, External Approach
        "0SW2XKZ",  # Revision of Nonautologous Tissue Substitute in Lumbar Vertebral Disc, External Approach
        "0SW300Z",  # Revision of Drainage Device in Lumbosacral Joint, Open Approach
        "0SW303Z",  # Revision of Infusion Device in Lumbosacral Joint, Open Approach
        "0SW304Z",  # Revision of Internal Fixation Device in Lumbosacral Joint, Open Approach
        "0SW307Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Joint, Open Approach
        "0SW308Z",  # Revision of Spacer in Lumbosacral Joint, Open Approach
        "0SW30AZ",  # Revision of Interbody Fusion Device in Lumbosacral Joint, Open Approach
        "0SW30JZ",  # Revision of Synthetic Substitute in Lumbosacral Joint, Open Approach
        "0SW30KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Joint, Open Approach
        "0SW330Z",  # Revision of Drainage Device in Lumbosacral Joint, Percutaneous Approach
        "0SW333Z",  # Revision of Infusion Device in Lumbosacral Joint, Percutaneous Approach
        "0SW334Z",  # Revision of Internal Fixation Device in Lumbosacral Joint, Percutaneous Approach
        "0SW337Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Joint, Percutaneous Approach
        "0SW338Z",  # Revision of Spacer in Lumbosacral Joint, Percutaneous Approach
        "0SW33AZ",  # Revision of Interbody Fusion Device in Lumbosacral Joint, Percutaneous Approach
        "0SW33JZ",  # Revision of Synthetic Substitute in Lumbosacral Joint, Percutaneous Approach
        "0SW33KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Joint, Percutaneous Approach
        "0SW340Z",  # Revision of Drainage Device in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW343Z",  # Revision of Infusion Device in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW344Z",  # Revision of Internal Fixation Device in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW347Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW348Z",  # Revision of Spacer in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW34AZ",  # Revision of Interbody Fusion Device in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW34JZ",  # Revision of Synthetic Substitute in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW34KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Joint, Percutaneous Endoscopic Approach
        "0SW3X0Z",  # Revision of Drainage Device in Lumbosacral Joint, External Approach
        "0SW3X3Z",  # Revision of Infusion Device in Lumbosacral Joint, External Approach
        "0SW3X4Z",  # Revision of Internal Fixation Device in Lumbosacral Joint, External Approach
        "0SW3X7Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Joint, External Approach
        "0SW3X8Z",  # Revision of Spacer in Lumbosacral Joint, External Approach
        "0SW3XAZ",  # Revision of Interbody Fusion Device in Lumbosacral Joint, External Approach
        "0SW3XJZ",  # Revision of Synthetic Substitute in Lumbosacral Joint, External Approach
        "0SW3XKZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Joint, External Approach
        "0SW400Z",  # Revision of Drainage Device in Lumbosacral Disc, Open Approach
        "0SW403Z",  # Revision of Infusion Device in Lumbosacral Disc, Open Approach
        "0SW407Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Disc, Open Approach
        "0SW40JZ",  # Revision of Synthetic Substitute in Lumbosacral Disc, Open Approach
        "0SW40KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Disc, Open Approach
        "0SW430Z",  # Revision of Drainage Device in Lumbosacral Disc, Percutaneous Approach
        "0SW433Z",  # Revision of Infusion Device in Lumbosacral Disc, Percutaneous Approach
        "0SW437Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Disc, Percutaneous Approach
        "0SW43JZ",  # Revision of Synthetic Substitute in Lumbosacral Disc, Percutaneous Approach
        "0SW43KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Disc, Percutaneous Approach
        "0SW440Z",  # Revision of Drainage Device in Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SW443Z",  # Revision of Infusion Device in Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SW447Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SW44JZ",  # Revision of Synthetic Substitute in Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SW44KZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Disc, Percutaneous Endoscopic Approach
        "0SW4X0Z",  # Revision of Drainage Device in Lumbosacral Disc, External Approach
        "0SW4X3Z",  # Revision of Infusion Device in Lumbosacral Disc, External Approach
        "0SW4X7Z",  # Revision of Autologous Tissue Substitute in Lumbosacral Disc, External Approach
        "0SW4XJZ",  # Revision of Synthetic Substitute in Lumbosacral Disc, External Approach
        "0SW4XKZ",  # Revision of Nonautologous Tissue Substitute in Lumbosacral Disc, External Approach
        "0SW500Z",  # Revision of Drainage Device in Sacrococcygeal Joint, Open Approach
        "0SW503Z",  # Revision of Infusion Device in Sacrococcygeal Joint, Open Approach
        "0SW504Z",  # Revision of Internal Fixation Device in Sacrococcygeal Joint, Open Approach
        "0SW507Z",  # Revision of Autologous Tissue Substitute in Sacrococcygeal Joint, Open Approach
        "0SW508Z",  # Revision of Spacer in Sacrococcygeal Joint, Open Approach
        "0SW50JZ",  # Revision of Synthetic Substitute in Sacrococcygeal Joint, Open Approach
        "0SW50KZ",  # Revision of Nonautologous Tissue Substitute in Sacrococcygeal Joint, Open Approach
        "0SW530Z",  # Revision of Drainage Device in Sacrococcygeal Joint, Percutaneous Approach
        "0SW533Z",  # Revision of Infusion Device in Sacrococcygeal Joint, Percutaneous Approach
        "0SW534Z",  # Revision of Internal Fixation Device in Sacrococcygeal Joint, Percutaneous Approach
        "0SW537Z",  # Revision of Autologous Tissue Substitute in Sacrococcygeal Joint, Percutaneous Approach
        "0SW538Z",  # Revision of Spacer in Sacrococcygeal Joint, Percutaneous Approach
        "0SW53JZ",  # Revision of Synthetic Substitute in Sacrococcygeal Joint, Percutaneous Approach
        "0SW53KZ",  # Revision of Nonautologous Tissue Substitute in Sacrococcygeal Joint, Percutaneous Approach
        "0SW540Z",  # Revision of Drainage Device in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW543Z",  # Revision of Infusion Device in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW544Z",  # Revision of Internal Fixation Device in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW547Z",  # Revision of Autologous Tissue Substitute in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW548Z",  # Revision of Spacer in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW54JZ",  # Revision of Synthetic Substitute in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW54KZ",  # Revision of Nonautologous Tissue Substitute in Sacrococcygeal Joint, Percutaneous Endoscopic Approach
        "0SW5X0Z",  # Revision of Drainage Device in Sacrococcygeal Joint, External Approach
        "0SW5X3Z",  # Revision of Infusion Device in Sacrococcygeal Joint, External Approach
        "0SW5X4Z",  # Revision of Internal Fixation Device in Sacrococcygeal Joint, External Approach
        "0SW5X7Z",  # Revision of Autologous Tissue Substitute in Sacrococcygeal Joint, External Approach
        "0SW5X8Z",  # Revision of Spacer in Sacrococcygeal Joint, External Approach
        "0SW5XJZ",  # Revision of Synthetic Substitute in Sacrococcygeal Joint, External Approach
        "0SW5XKZ",  # Revision of Nonautologous Tissue Substitute in Sacrococcygeal Joint, External Approach
        "0SW600Z",  # Revision of Drainage Device in Coccygeal Joint, Open Approach
        "0SW603Z",  # Revision of Infusion Device in Coccygeal Joint, Open Approach
        "0SW604Z",  # Revision of Internal Fixation Device in Coccygeal Joint, Open Approach
        "0SW607Z",  # Revision of Autologous Tissue Substitute in Coccygeal Joint, Open Approach
        "0SW608Z",  # Revision of Spacer in Coccygeal Joint, Open Approach
        "0SW60JZ",  # Revision of Synthetic Substitute in Coccygeal Joint, Open Approach
        "0SW60KZ",  # Revision of Nonautologous Tissue Substitute in Coccygeal Joint, Open Approach
        "0SW630Z",  # Revision of Drainage Device in Coccygeal Joint, Percutaneous Approach
        "0SW633Z",  # Revision of Infusion Device in Coccygeal Joint, Percutaneous Approach
        "0SW634Z",  # Revision of Internal Fixation Device in Coccygeal Joint, Percutaneous Approach
        "0SW637Z",  # Revision of Autologous Tissue Substitute in Coccygeal Joint, Percutaneous Approach
        "0SW638Z",  # Revision of Spacer in Coccygeal Joint, Percutaneous Approach
        "0SW63JZ",  # Revision of Synthetic Substitute in Coccygeal Joint, Percutaneous Approach
        "0SW63KZ",  # Revision of Nonautologous Tissue Substitute in Coccygeal Joint, Percutaneous Approach
        "0SW640Z",  # Revision of Drainage Device in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW643Z",  # Revision of Infusion Device in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW644Z",  # Revision of Internal Fixation Device in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW647Z",  # Revision of Autologous Tissue Substitute in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW648Z",  # Revision of Spacer in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW64JZ",  # Revision of Synthetic Substitute in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW64KZ",  # Revision of Nonautologous Tissue Substitute in Coccygeal Joint, Percutaneous Endoscopic Approach
        "0SW6X0Z",  # Revision of Drainage Device in Coccygeal Joint, External Approach
        "0SW6X3Z",  # Revision of Infusion Device in Coccygeal Joint, External Approach
        "0SW6X4Z",  # Revision of Internal Fixation Device in Coccygeal Joint, External Approach
        "0SW6X7Z",  # Revision of Autologous Tissue Substitute in Coccygeal Joint, External Approach
        "0SW6X8Z",  # Revision of Spacer in Coccygeal Joint, External Approach
        "0SW6XJZ",  # Revision of Synthetic Substitute in Coccygeal Joint, External Approach
        "0SW6XKZ",  # Revision of Nonautologous Tissue Substitute in Coccygeal Joint, External Approach
        "0W9100Z",  # Drainage of Cranial Cavity with Drainage Device, Open Approach
        "0W910ZZ",  # Drainage of Cranial Cavity, Open Approach
        "0WC10ZZ",  # Extirpation of Matter from Cranial Cavity, Open Approach
        "0WH10YZ",  # Insertion of Other Device into Cranial Cavity, Open Approach
        "0WJ10ZZ",  # Inspection of Cranial Cavity, Open Approach
        "0WP100Z",  # Removal of Drainage Device from Cranial Cavity, Open Approach
        "0WP101Z",  # Removal of Radioactive Element from Cranial Cavity, Open Approach
        "0WP10JZ",  # Removal of Synthetic Substitute from Cranial Cavity, Open Approach
        "0WP10YZ",  # Removal of Other Device from Cranial Cavity, Open Approach
        "0WW100Z",  # Revision of Drainage Device in Cranial Cavity, Open Approach
        "0WW101Z",  # Revision of Radioactive Element in Cranial Cavity, Open Approach
        "0WW103Z",  # Revision of Infusion Device in Cranial Cavity, Open Approach
        "0WW10JZ",  # Revision of Synthetic Substitute in Cranial Cavity, Open Approach
        "0WW10YZ",  # Revision of Other Device in Cranial Cavity, Open Approach
    }

    SNOMEDCT = {
        "10458001",  # Evacuation of intracerebral hematoma (procedure)
        "112929000",  # Exploratory craniotomy, supratentorial (procedure)
        "1201979001",  # Excision of lesion of brain (procedure)
        "1287541008",  # Coagulation of brain tissue (procedure)
        "1296603004",  # Drainage of cerebral epidural space by incision (procedure)
        "1296604005",  # Drainage of cerebral epidural space by trephination (procedure)
        "1296605006",  # Drainage of cerebral subdural space by incision (procedure)
        "1296606007",  # Drainage of cerebral subdural space by trephination (procedure)
        "1296607003",  # Drainage of cerebral subarachnoid space by trephination (procedure)
        "1296608008",  # Drainage of cerebral subarachnoid space by incision (procedure)
        "1296626006",  # Division of tract of brain (procedure)
        "13431007",  # Prefrontal lobotomy (procedure)
        "13433005",  # Excision of lesion of brain by transtemporal approach (procedure)
        "13666002",  # Leukotomy (procedure)
        "14247003",  # Cerebral hemispherectomy (procedure)
        "14554005",  # Cryotherapy of brain (procedure)
        "1583007",  # Tractotomy of mesencephalon (procedure)
        "171442008",  # Major excision of brain tissue (procedure)
        "171443003",  # Total lobectomy of brain (procedure)
        "171447002",  # Excision of lesion of frontal lobe of brain (procedure)
        "171448007",  # Excision of lesion of temporal lobe of brain (procedure)
        "171449004",  # Excision of lesion of parietal lobe of brain (procedure)
        "171450004",  # Excision of lesion of occipital lobe of brain (procedure)
        "171451000",  # Excision of lesion of cerebellum (procedure)
        "171455009",  # Stereotactic leucotomy (procedure)
        "171471009",  # Drainage of lesion of brain tissue (procedure)
        "171472002",  # Drainage of abscess of brain tissue (procedure)
        "171473007",  # Evacuation of hematoma from temporal lobe of brain (procedure)
        "171474001",  # Evacuation of hematoma from cerebellum (procedure)
        "171479006",  # Open division of brain tissue (procedure)
        "171495006",  # Maintenance of neurostimulator in brain (procedure)
        "171540003",  # Endoscopic extirpation of lesion of ventricle of brain (procedure)
        "171554006",  # Drainage of subarachnoid space of brain (procedure)
        "171692002",  # Removal of lesion of brain meninges (procedure)
        "171693007",  # Extirpation of lesion of meninges of brain cortex (procedure)
        "171694001",  # Extirpation of lesion of meninges of sphenoidal ridge of cranium (procedure)
        "171695000",  # Extirpation of lesion of meninges of subfrontal region of brain (procedure)
        "171696004",  # Extirpation of lesion of meninges of parasagittal region of brain (procedure)
        "171697008",  # Extirpation of lesion of falx cerebri (procedure)
        "171698003",  # Extirpation of lesion of tentorium cerebelli (procedure)
        "171722000",  # Evacuation of intracranial subdural abscess (procedure)
        "178373004",  # Exploratory open craniotomy (procedure)
        "178375006",  # Exploratory burrhole of cranium (procedure)
        "178376007",  # Reopening of cranium and re-exploration of intracranial operation site and surgical arrest of postoperative bleeding (procedure)
        "178391009",  # Cranial morcellization (procedure)
        "1868005",  # Removal of foreign body from brain (procedure)
        "1995001",  # Stripping of cerebral meninges (procedure)
        "21551002",  # Division of brain tissue (procedure)
        "22480005",  # Intracranial microdissection (procedure)
        "230810008",  # Excision of tumor of brain meninges (procedure)
        "230811007",  # Excision of cyst of brain meninges (procedure)
        "230814004",  # Removal of intracranial subdural tumor (procedure)
        "230837005",  # Focal excision of brain tissue tumor (procedure)
        "230838000",  # Focal excision of brain abscess (procedure)
        "230839008",  # Focal excision of brain hematoma (procedure)
        "230840005",  # Focal excision of brain cyst (procedure)
        "230841009",  # Focal excision of swollen brain (procedure)
        "230859007",  # Destruction of brain tumor (procedure)
        "230886007",  # Exploration of ventricle of brain (procedure)
        "230888008",  # Open exploration of ventricle of brain (procedure)
        "230890009",  # Excision of lesion of ventricle of brain (procedure)
        "230893006",  # Removal of foreign body from ventricle of brain (procedure)
        "234064002",  # Marsupialization of cerebral lesion (procedure)
        "234915008",  # Marsupialization of lesion of brain (procedure)
        "25353009",  # Craniotomy (procedure)
        "2552005",  # Incision of cerebral subarachnoid space (procedure)
        "260643004",  # Middle cranial fossa craniectomy (procedure)
        "260644005",  # Posterior cranial fossa craniectomy (procedure)
        "26096003",  # Stripping of cerebral subdural membrane (procedure)
        "27181009",  # Removal of foreign body from brain by incision (procedure)
        "27190002",  # Excision of lesion of cerebral meninges (procedure)
        "2737001",  # Exploratory craniotomy, infratentorial (procedure)
        "273990004",  # Coagulation of brain tissue using electrical energy (procedure)
        "273991000",  # Lobotomy of brain (procedure)
        "28264008",  # Amygdalotomy (procedure)
        "287205001",  # Brain decortication (procedure)
        "287208004",  # Operation on thalamus (procedure)
        "29063004",  # Incision of cerebrum (procedure)
        "29114001",  # Incision and drainage of cranial sinus (procedure)
        "2968008",  # Craniectomy with treatment of penetrating wound of brain (procedure)
        "30195003",  # Excision of lesion of cerebral cortex (procedure)
        "304848009",  # Callosotomy (procedure)
        "304849001",  # Subpial transection of cortex (procedure)
        "307299001",  # Removal of lesion from intracranial subdural space (procedure)
        "31746001",  # Debridement of cerebral meninges (procedure)
        "32606000",  # Marsupialization of cyst of brain (procedure)
        "34097003",  # Craniectomy for excision of meningioma, infratentorial (procedure)
        "36910002",  # Craniectomy (procedure)
        "37380009",  # Destruction of cerebral lesion (procedure)
        "38512006",  # Resection of cerebral meninges (procedure)
        "39667005",  # Amygdalohippocampotomy (procedure)
        "39704007",  # Exploration of brain (procedure)
        "4045001",  # Drainage of cerebral ventricle by incision (procedure)
        "42699003",  # Incision of brain (procedure)
        "427037000",  # Complete callosotomy (procedure)
        "427599009",  # Excision of lesion of tissue of brain stem (procedure)
        "429314008",  # Partial callosotomy (procedure)
        "429510003",  # Multiple subpial transections of cerebral cortex (procedure)
        "438369007",  # Incision of bone of cranium and transection of tentorium cerebelli (procedure)
        "438614004",  # Excision of bone of cranium and excision of neoplasm of cerebellopontine angle (procedure)
        "439493007",  # Incision of bone of cranium with elevation of bone flap and excision of epileptogenic focus (procedure)
        "439499006",  # Excision of bone of cranium and excision of infratentorial neoplasm of brain (procedure)
        "440062005",  # Excision of bone of cranium with excision of abscess of brain (procedure)
        "441832005",  # Extirpation of lesion of meninges of base of skull (procedure)
        "442616007",  # Extirpation of lesion of meninges of clivus of skull (procedure)
        "445746008",  # Craniotomy and removal of hematoma from extradural space (procedure)
        "44589002",  # Debridement of brain (procedure)
        "445926009",  # Excision of intradural lesion of base of middle cranial fossa (procedure)
        "445942002",  # Excision of temporal lobe (procedure)
        "446103006",  # Partial excision of lesion of brain (procedure)
        "446115005",  # Elevation of depressed fracture of skull and debridement of brain (procedure)
        "446209004",  # Excision of intradural lesion of base of anterior cranial fossa (procedure)
        "446210009",  # Excision of intradural lesion of base of posterior cranial fossa (procedure)
        "446440008",  # Curettage of lesion of brain (procedure)
        "446570000",  # Craniectomy by retromastoid approach (procedure)
        "446571001",  # Craniotomy of posterior fossa (procedure)
        "446683008",  # Craniotomy and decompression of brain (procedure)
        "446705006",  # Trephination of cranium and drainage of chronic hematoma from subdural space (procedure)
        "446816008",  # Craniotomy and evacuation of traumatic hematoma (procedure)
        "446848007",  # Craniotomy and evacuation of blood clot (procedure)
        "446850004",  # Trephination of cranium and external drainage of effusion from subdural space (procedure)
        "446865003",  # Craniotomy and removal of foreign body (procedure)
        "446868001",  # Excision of meningioma by supratentorial approach (procedure)
        "447107001",  # Craniotomy and excision of neoplasm of brain (procedure)
        "447115003",  # Total excision of lesion of brain tissue (procedure)
        "447156000",  # Trephination of cranium and drainage of empyema of subdural space (procedure)
        "447362004",  # Excision of meningioma by infratentorial approach (procedure)
        "447363009",  # Craniotomy by retromastoid approach (procedure)
        "447469009",  # Partial excision of meningioma by infratentorial approach (procedure)
        "447518007",  # Partial excision of meningioma by supratentorial approach (procedure)
        "447529006",  # Total excision of meningioma by infratentorial approach (procedure)
        "447594008",  # Total excision of meningioma by supratentorial approach (procedure)
        "45322003",  # Decompression of brain (procedure)
        "4570007",  # Chemopallidectomy (procedure)
        "53423001",  # Lobectomy of brain (procedure)
        "5384005",  # Excision of part of frontal cortex (procedure)
        "54872009",  # Destruction of lesion of cerebral meninges (procedure)
        "54891009",  # Removal of foreign body from cerebral meninges (procedure)
        "56143008",  # Division of cerebral meninges (procedure)
        "56959007",  # Operation on globus pallidus (procedure)
        "58337008",  # Transorbital leukotomy (procedure)
        "58515002",  # Cryoleucotomy (procedure)
        "59696004",  # Curettage of cerebral meninges (procedure)
        "59712006",  # Evacuation of subdural hematoma (procedure)
        "61327009",  # Puncture and drainage of craniopharyngioma (procedure)
        "61965006",  # Incision of cranial sinus (procedure)
        "67402009",  # Partial lobectomy of brain (procedure)
        "67864003",  # Trephination of cranium (procedure)
        "68346000",  # Drainage of cerebrum (procedure)
        "68731005",  # Decortication of cerebral meninges (procedure)
        "69086007",  # Curettage of brain (procedure)
        "69788001",  # Tractotomy of medulla oblongata (procedure)
        "71722001",  # Thalamotomy (procedure)
        "73006002",  # Cranial decompression, posterior fossa (procedure)
        "75626002",  # Pallidoamygdalotomy (procedure)
        "76961002",  # Lobotomy and tractotomy (procedure)
        "77697000",  # Cryothalamectomy (procedure)
        "789003",  # Cranial decompression, subtemporal, supratentorial (procedure)
        "79093007",  # Craniotomy with treatment of penetrating wound of brain (procedure)
        "807005",  # Excision of brain (procedure)
        "81259007",  # Incision of cerebral meninges (procedure)
        "8155007",  # Excision of basal ganglion (procedure)
        "81849003",  # Pallidoansotomy (procedure)
        "82678002",  # Incision of cerebral epidural space (procedure)
        "82917000",  # Reopening of craniotomy site (procedure)
        "87060004",  # Pallidotomy (procedure)
        "87192001",  # Pallidectomy (procedure)
        "90287005",  # Thalamectomy (procedure)
        "91652006",  # Incision of cerebral subdural space (procedure)
        "9257004",  # Revision of intracranial neurostimulator receiver (procedure)
        "9372002",  # Drainage of cranial sinus by trephination (procedure)
        "9580008",  # Chemothalamectomy (procedure)
        "9888007",  # Excision of intracranial lesion (procedure)
    }

class MechanicalCirculatoryAssistDevice(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for mechanical circulatory assist devices.

    **Data Element Scope:** This value set may use a model element related to Procedure.

    **Inclusion Criteria:** Includes CPT, ICD10PCS and SNOMEDCT codes that represent procedures associated with mechanical circulatory devices used to support heart function.

    **Exclusion Criteria:** No exclusions.
    """

    VALUE_SET_NAME = "Mechanical Circulatory Assist Device"
    OID = "2.16.840.1.113883.3.3157.4052"
    DEFINITION_VERSION = "20250211"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "33967",  # Insertion of intra-aortic balloon assist device, percutaneous
        "33968",  # Removal of intra-aortic balloon assist device, percutaneous
        "33970",  # Insertion of intra-aortic balloon assist device through the femoral artery, open approach
        "33971",  # Removal of intra-aortic balloon assist device including repair of femoral artery, with or without graft
        "33973",  # Insertion of intra-aortic balloon assist device through the ascending aorta
        "33974",  # Removal of intra-aortic balloon assist device from the ascending aorta, including repair of the ascending aorta, with or without graft
        "33975",  # Insertion of ventricular assist device; extracorporeal, single ventricle
        "33976",  # Insertion of ventricular assist device; extracorporeal, biventricular
        "33977",  # Removal of ventricular assist device; extracorporeal, single ventricle
        "33978",  # Removal of ventricular assist device; extracorporeal, biventricular
        "33979",  # Insertion of ventricular assist device, implantable intracorporeal, single ventricle
        "33980",  # Removal of ventricular assist device, implantable intracorporeal, single ventricle
        "33981",  # Replacement of extracorporeal ventricular assist device, single or biventricular, pump(s), single or each pump
        "33982",  # Replacement of ventricular assist device pump(s); implantable intracorporeal, single ventricle, without cardiopulmonary bypass
        "33983",  # Replacement of ventricular assist device pump(s); implantable intracorporeal, single ventricle, with cardiopulmonary bypass
        "33990",  # Insertion of ventricular assist device, percutaneous, including radiological supervision and interpretation; left heart, arterial access only
        "33991",  # Insertion of ventricular assist device, percutaneous, including radiological supervision and interpretation; left heart, both arterial and venous access, with transseptal puncture
        "33992",  # Removal of percutaneous left heart ventricular assist device, arterial or arterial and venous cannula(s), at separate and distinct session from insertion
        "33993",  # Repositioning of percutaneous right or left heart ventricular assist device with imaging guidance at separate and distinct session from insertion
        "33995",  # Insertion of ventricular assist device, percutaneous, including radiological supervision and interpretation; right heart, venous access only
        "33997",  # Removal of percutaneous right heart ventricular assist device, venous cannula, at separate and distinct session from insertion
    }

    ICD10PCS = {
        "02HA0QZ",  # Insertion of Implantable Heart Assist System into Heart, Open Approach
        "02HA0RJ",  # Insertion of Short-term External Heart Assist System into Heart, Intraoperative, Open Approach
        "02HA0RS",  # Insertion of Biventricular Short-term External Heart Assist System into Heart, Open Approach
        "02HA0RZ",  # Insertion of Short-term External Heart Assist System into Heart, Open Approach
        "02HA3QZ",  # Insertion of Implantable Heart Assist System into Heart, Percutaneous Approach
        "02HA3RJ",  # Insertion of Short-term External Heart Assist System into Heart, Intraoperative, Percutaneous Approach
        "02HA3RS",  # Insertion of Biventricular Short-term External Heart Assist System into Heart, Percutaneous Approach
        "02HA3RZ",  # Insertion of Short-term External Heart Assist System into Heart, Percutaneous Approach
        "02HA4QZ",  # Insertion of Implantable Heart Assist System into Heart, Percutaneous Endoscopic Approach
        "02HA4RJ",  # Insertion of Short-term External Heart Assist System into Heart, Intraoperative, Percutaneous Endoscopic Approach
        "02HA4RS",  # Insertion of Biventricular Short-term External Heart Assist System into Heart, Percutaneous Endoscopic Approach
        "02HA4RZ",  # Insertion of Short-term External Heart Assist System into Heart, Percutaneous Endoscopic Approach
        "02HK0YZ",  # Insertion of Other Device into Right Ventricle, Open Approach
        "02HK3YZ",  # Insertion of Other Device into Right Ventricle, Percutaneous Approach
        "02HK4YZ",  # Insertion of Other Device into Right Ventricle, Percutaneous Endoscopic Approach
        "02HL0YZ",  # Insertion of Other Device into Left Ventricle, Open Approach
        "02HL3YZ",  # Insertion of Other Device into Left Ventricle, Percutaneous Approach
        "02HL4YZ",  # Insertion of Other Device into Left Ventricle, Percutaneous Endoscopic Approach
        "02HW3RZ",  # Insertion of Short-term External Heart Assist System into Thoracic Aorta, Descending, Percutaneous Approach
        "02PA0QZ",  # Removal of Implantable Heart Assist System from Heart, Open Approach
        "02PA0RS",  # Removal of Biventricular Short-term External Heart Assist System from Heart, Open Approach
        "02PA0RZ",  # Removal of Short-term External Heart Assist System from Heart, Open Approach
        "02PA3QZ",  # Removal of Implantable Heart Assist System from Heart, Percutaneous Approach
        "02PA3RS",  # Removal of Biventricular Short-term External Heart Assist System from Heart, Percutaneous Approach
        "02PA3RZ",  # Removal of Short-term External Heart Assist System from Heart, Percutaneous Approach
        "02PA4QZ",  # Removal of Implantable Heart Assist System from Heart, Percutaneous Endoscopic Approach
        "02PA4RS",  # Removal of Biventricular Short-term External Heart Assist System from Heart, Percutaneous Endoscopic Approach
        "02PA4RZ",  # Removal of Short-term External Heart Assist System from Heart, Percutaneous Endoscopic Approach
        "02WA0RS",  # Revision of Biventricular Short-term External Heart Assist System in Heart, Open Approach
        "02WA0RZ",  # Revision of Short-term External Heart Assist System in Heart, Open Approach
        "5A02110",  # Assistance with Cardiac Output using Balloon Pump, Intermittent
        "5A02115",  # Assistance with Cardiac Output using Pulsatile Compression, Intermittent
        "5A02116",  # Assistance with Cardiac Output using Other Pump, Intermittent
        "5A0211D",  # Assistance with Cardiac Output using Impeller Pump, Intermittent
        "5A02210",  # Assistance with Cardiac Output using Balloon Pump, Continuous
        "5A02215",  # Assistance with Cardiac Output using Pulsatile Compression, Continuous
        "5A02216",  # Assistance with Cardiac Output using Other Pump, Continuous
        "5A0221D",  # Assistance with Cardiac Output using Impeller Pump, Continuous
        "5A0222C",  # Assistance with Cardiac Oxygenation, Supersaturated
    }

    SNOMEDCT = {
        "16965008",  # Insertion of intra-aortic balloon counterpulsation (procedure)
        "175113001",  # Transluminal insertion of aortic balloon pump (procedure)
        "232965003",  # Implantation of cardiac ventricular assist device (procedure)
        "232966002",  # Implantation of right cardiac ventricular assist device (procedure)
        "232967006",  # Implantation of left cardiac ventricular assist device (procedure)
        "232968001",  # Implantation of cardiac biventricular assist device (procedure)
        "399052009",  # Aortic balloon pump operation (procedure)
        "399217008",  # Cardioassist by aortic balloon pump (procedure)
        "426410001",  # Removal of ventricular assist device (procedure)
        "426568003",  # Open implantation of cardiac ventricular assist device (procedure)
        "427198005",  # Open removal of ventricular assist device (procedure)
        "432375008",  # Insertion of intra-aortic balloon pump using fluoroscopic guidance with contrast (procedure)
        "459251000124109",  # Left ventricular assist device for destination therapy (regime/therapy)
        "700151002",  # Temporary implantation of left cardiac ventricular assist device (procedure)
        "87701006",  # Intraaortic balloon pump weaning (procedure)
    }

class PercutaneousCoronaryIntervention(ValueSet):
    """
    **Clinical Focus:** This comprehensive set of codes is intended to capture a patient who undergoes a PCI procedure.

    **Data Element Scope:** The intent of this data element is to identify patients with a PCI procedure.

    **Inclusion Criteria:** Includes HCPCS/CPT/SNOMED-CT/ICD-10-PCS codes that indicate PCI procedures.

    **Exclusion Criteria:** Excludes HCPCS/CPT/SNOMED-CT/ICD-10-PCS codes that do not indicate PCI procedures.
    """

    VALUE_SET_NAME = "Percutaneous Coronary Intervention"
    OID = "2.16.840.1.113883.3.3157.2000.5"
    DEFINITION_VERSION = "20180523"
    EXPANSION_VERSION = "eCQM Update 2025-05-08"

    CPT = {
        "0913T",  # Percutaneous transcatheter therapeutic drug delivery by intracoronary drug-delivery balloon (eg, drug-coated, drug-eluting), including mechanical dilation by nondrug-delivery balloon angioplasty, endoluminal imaging using intravascular ultrasound (IVUS) or optical coherence tomography (OCT) when performed, imaging supervision, interpretation, and report, single major coronary artery or branch
        "0914T",  # Percutaneous transcatheter therapeutic drug delivery by intracoronary drug-delivery balloon (eg, drug-coated, drug-eluting) performed on a separate target lesion from the target lesion treated with balloon angioplasty, coronary stent placement or coronary atherectomy, including mechanical dilation by nondrug-delivery balloon angioplasty, endoluminal imaging using intravascular ultrasound (IVUS) or optical coherence tomography (OCT) when performed, imaging supervision, interpretation, and report, single major coronary artery or branch (List separately in addition to code for percutaneous coronary stent or atherectomy intervention)
        "92920",  # Percutaneous transluminal coronary angioplasty; single major coronary artery or branch
        "92924",  # Percutaneous transluminal coronary atherectomy, with coronary angioplasty when performed; single major coronary artery or branch
        "92928",  # Percutaneous transcatheter placement of intracoronary stent(s), with coronary angioplasty when performed; single major coronary artery or branch
        "92933",  # Percutaneous transluminal coronary atherectomy, with intracoronary stent, with coronary angioplasty when performed; single major coronary artery or branch
        "92937",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of intracoronary stent, atherectomy and angioplasty, including distal protection when performed; single vessel
        "92943",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of intracoronary stent, atherectomy and angioplasty; single vessel
    }

    HCPCSLEVELII = {
        "C9600",  # Percutaneous transcatheter placement of drug eluting intracoronary stent(s), with coronary angioplasty when performed; single major coronary artery or branch
        "C9602",  # Percutaneous transluminal coronary atherectomy, with drug eluting intracoronary stent, with coronary angioplasty when performed; single major coronary artery or branch
        "C9604",  # Percutaneous transluminal revascularization of or through coronary artery bypass graft (internal mammary, free arterial, venous), any combination of drug-eluting intracoronary stent, atherectomy and angioplasty, including distal protection when performed; single vessel
        "C9607",  # Percutaneous transluminal revascularization of chronic total occlusion, coronary artery, coronary artery branch, or coronary artery bypass graft, any combination of drug-eluting intracoronary stent, atherectomy and angioplasty; single vessel
    }

    ICD10PCS = {
        "0270346",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027034Z",  # Dilation of Coronary Artery, One Artery with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0270356",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027035Z",  # Dilation of Coronary Artery, One Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0270366",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027036Z",  # Dilation of Coronary Artery, One Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0270376",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027037Z",  # Dilation of Coronary Artery, One Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02703D6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02703DZ",  # Dilation of Coronary Artery, One Artery with Intraluminal Device, Percutaneous Approach
        "02703E6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02703EZ",  # Dilation of Coronary Artery, One Artery with Two Intraluminal Devices, Percutaneous Approach
        "02703F6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02703FZ",  # Dilation of Coronary Artery, One Artery with Three Intraluminal Devices, Percutaneous Approach
        "02703G6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02703GZ",  # Dilation of Coronary Artery, One Artery with Four or More Intraluminal Devices, Percutaneous Approach
        "02703T6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02703TZ",  # Dilation of Coronary Artery, One Artery with Radioactive Intraluminal Device, Percutaneous Approach
        "02703Z6",  # Dilation of Coronary Artery, One Artery, Bifurcation, Percutaneous Approach
        "02703ZZ",  # Dilation of Coronary Artery, One Artery, Percutaneous Approach
        "0270446",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027044Z",  # Dilation of Coronary Artery, One Artery with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0270456",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027045Z",  # Dilation of Coronary Artery, One Artery with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0270466",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027046Z",  # Dilation of Coronary Artery, One Artery with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0270476",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027047Z",  # Dilation of Coronary Artery, One Artery with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704D6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02704DZ",  # Dilation of Coronary Artery, One Artery with Intraluminal Device, Percutaneous Endoscopic Approach
        "02704E6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704EZ",  # Dilation of Coronary Artery, One Artery with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704F6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704FZ",  # Dilation of Coronary Artery, One Artery with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704G6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704GZ",  # Dilation of Coronary Artery, One Artery with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02704T6",  # Dilation of Coronary Artery, One Artery, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02704TZ",  # Dilation of Coronary Artery, One Artery with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02704Z6",  # Dilation of Coronary Artery, One Artery, Bifurcation, Percutaneous Endoscopic Approach
        "02704ZZ",  # Dilation of Coronary Artery, One Artery, Percutaneous Endoscopic Approach
        "0271346",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027134Z",  # Dilation of Coronary Artery, Two Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0271356",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027135Z",  # Dilation of Coronary Artery, Two Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0271366",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027136Z",  # Dilation of Coronary Artery, Two Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0271376",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027137Z",  # Dilation of Coronary Artery, Two Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02713D6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02713DZ",  # Dilation of Coronary Artery, Two Arteries with Intraluminal Device, Percutaneous Approach
        "02713E6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02713EZ",  # Dilation of Coronary Artery, Two Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02713F6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02713FZ",  # Dilation of Coronary Artery, Two Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02713G6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02713GZ",  # Dilation of Coronary Artery, Two Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02713T6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02713TZ",  # Dilation of Coronary Artery, Two Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02713Z6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, Percutaneous Approach
        "02713ZZ",  # Dilation of Coronary Artery, Two Arteries, Percutaneous Approach
        "0271446",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027144Z",  # Dilation of Coronary Artery, Two Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0271456",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027145Z",  # Dilation of Coronary Artery, Two Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0271466",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027146Z",  # Dilation of Coronary Artery, Two Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0271476",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027147Z",  # Dilation of Coronary Artery, Two Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714D6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02714DZ",  # Dilation of Coronary Artery, Two Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02714E6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714EZ",  # Dilation of Coronary Artery, Two Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714F6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714FZ",  # Dilation of Coronary Artery, Two Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714G6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714GZ",  # Dilation of Coronary Artery, Two Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02714T6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02714TZ",  # Dilation of Coronary Artery, Two Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02714Z6",  # Dilation of Coronary Artery, Two Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02714ZZ",  # Dilation of Coronary Artery, Two Arteries, Percutaneous Endoscopic Approach
        "0272346",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027234Z",  # Dilation of Coronary Artery, Three Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0272356",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027235Z",  # Dilation of Coronary Artery, Three Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0272366",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027236Z",  # Dilation of Coronary Artery, Three Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0272376",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027237Z",  # Dilation of Coronary Artery, Three Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02723D6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02723DZ",  # Dilation of Coronary Artery, Three Arteries with Intraluminal Device, Percutaneous Approach
        "02723E6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02723EZ",  # Dilation of Coronary Artery, Three Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02723F6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02723FZ",  # Dilation of Coronary Artery, Three Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02723G6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02723GZ",  # Dilation of Coronary Artery, Three Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02723T6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02723TZ",  # Dilation of Coronary Artery, Three Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02723Z6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, Percutaneous Approach
        "02723ZZ",  # Dilation of Coronary Artery, Three Arteries, Percutaneous Approach
        "0272446",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027244Z",  # Dilation of Coronary Artery, Three Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0272456",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027245Z",  # Dilation of Coronary Artery, Three Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0272466",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027246Z",  # Dilation of Coronary Artery, Three Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0272476",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027247Z",  # Dilation of Coronary Artery, Three Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724D6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02724DZ",  # Dilation of Coronary Artery, Three Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02724E6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724EZ",  # Dilation of Coronary Artery, Three Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724F6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724FZ",  # Dilation of Coronary Artery, Three Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724G6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724GZ",  # Dilation of Coronary Artery, Three Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02724T6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02724TZ",  # Dilation of Coronary Artery, Three Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02724Z6",  # Dilation of Coronary Artery, Three Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02724ZZ",  # Dilation of Coronary Artery, Three Arteries, Percutaneous Endoscopic Approach
        "0273346",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Approach
        "027334Z",  # Dilation of Coronary Artery, Four or More Arteries with Drug-eluting Intraluminal Device, Percutaneous Approach
        "0273356",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027335Z",  # Dilation of Coronary Artery, Four or More Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0273366",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027336Z",  # Dilation of Coronary Artery, Four or More Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Approach
        "0273376",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "027337Z",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Approach
        "02733D6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Intraluminal Device, Percutaneous Approach
        "02733DZ",  # Dilation of Coronary Artery, Four or More Arteries with Intraluminal Device, Percutaneous Approach
        "02733E6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Approach
        "02733EZ",  # Dilation of Coronary Artery, Four or More Arteries with Two Intraluminal Devices, Percutaneous Approach
        "02733F6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Approach
        "02733FZ",  # Dilation of Coronary Artery, Four or More Arteries with Three Intraluminal Devices, Percutaneous Approach
        "02733G6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Approach
        "02733GZ",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Intraluminal Devices, Percutaneous Approach
        "02733T6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Approach
        "02733TZ",  # Dilation of Coronary Artery, Four or More Arteries with Radioactive Intraluminal Device, Percutaneous Approach
        "02733Z6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Approach
        "02733ZZ",  # Dilation of Coronary Artery, Four or More Arteries, Percutaneous Approach
        "0273446",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "027344Z",  # Dilation of Coronary Artery, Four or More Arteries with Drug-eluting Intraluminal Device, Percutaneous Endoscopic Approach
        "0273456",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027345Z",  # Dilation of Coronary Artery, Four or More Arteries with Two Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0273466",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027346Z",  # Dilation of Coronary Artery, Four or More Arteries with Three Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "0273476",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "027347Z",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Drug-eluting Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734D6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Intraluminal Device, Percutaneous Endoscopic Approach
        "02734DZ",  # Dilation of Coronary Artery, Four or More Arteries with Intraluminal Device, Percutaneous Endoscopic Approach
        "02734E6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734EZ",  # Dilation of Coronary Artery, Four or More Arteries with Two Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734F6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734FZ",  # Dilation of Coronary Artery, Four or More Arteries with Three Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734G6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734GZ",  # Dilation of Coronary Artery, Four or More Arteries with Four or More Intraluminal Devices, Percutaneous Endoscopic Approach
        "02734T6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02734TZ",  # Dilation of Coronary Artery, Four or More Arteries with Radioactive Intraluminal Device, Percutaneous Endoscopic Approach
        "02734Z6",  # Dilation of Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02734ZZ",  # Dilation of Coronary Artery, Four or More Arteries, Percutaneous Endoscopic Approach
        "02C03Z6",  # Extirpation of Matter from Coronary Artery, One Artery, Bifurcation, Percutaneous Approach
        "02C03ZZ",  # Extirpation of Matter from Coronary Artery, One Artery, Percutaneous Approach
        "02C04Z6",  # Extirpation of Matter from Coronary Artery, One Artery, Bifurcation, Percutaneous Endoscopic Approach
        "02C04ZZ",  # Extirpation of Matter from Coronary Artery, One Artery, Percutaneous Endoscopic Approach
        "02C13Z6",  # Extirpation of Matter from Coronary Artery, Two Arteries, Bifurcation, Percutaneous Approach
        "02C13ZZ",  # Extirpation of Matter from Coronary Artery, Two Arteries, Percutaneous Approach
        "02C14Z6",  # Extirpation of Matter from Coronary Artery, Two Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02C14ZZ",  # Extirpation of Matter from Coronary Artery, Two Arteries, Percutaneous Endoscopic Approach
        "02C23Z6",  # Extirpation of Matter from Coronary Artery, Three Arteries, Bifurcation, Percutaneous Approach
        "02C23ZZ",  # Extirpation of Matter from Coronary Artery, Three Arteries, Percutaneous Approach
        "02C24Z6",  # Extirpation of Matter from Coronary Artery, Three Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02C24ZZ",  # Extirpation of Matter from Coronary Artery, Three Arteries, Percutaneous Endoscopic Approach
        "02C33Z6",  # Extirpation of Matter from Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Approach
        "02C33ZZ",  # Extirpation of Matter from Coronary Artery, Four or More Arteries, Percutaneous Approach
        "02C34Z6",  # Extirpation of Matter from Coronary Artery, Four or More Arteries, Bifurcation, Percutaneous Endoscopic Approach
        "02C34ZZ",  # Extirpation of Matter from Coronary Artery, Four or More Arteries, Percutaneous Endoscopic Approach
        "02C43ZZ",  # Extirpation of Matter from Coronary Vein, Percutaneous Approach
        "02C44ZZ",  # Extirpation of Matter from Coronary Vein, Percutaneous Endoscopic Approach
    }

    SNOMEDCT = {
        "11101003",  # Percutaneous transluminal coronary angioplasty (procedure)
        "1222571004",  # Percutaneous transluminal insertion of bare metal stent into coronary artery using fluoroscopic guidance with contrast (procedure)
        "1222673007",  # Percutaneous transluminal insertion of drug eluting stent into left main coronary artery using fluoroscopic guidance with contrast (procedure)
        "1222674001",  # Percutaneous transluminal balloon angioplasty of left main coronary artery (procedure)
        "15256002",  # Transmyocardial revascularization by laser technique (procedure)
        "175066001",  # Percutaneous transluminal balloon angioplasty of bypass graft of coronary artery (procedure)
        "232726007",  # Percutaneous endarterectomy of coronary artery (procedure)
        "232727003",  # Percutaneous directional coronary atherectomy (procedure)
        "232728008",  # Percutaneous low speed rotational coronary atherectomy (procedure)
        "232729000",  # Percutaneous high speed rotational coronary atherectomy (procedure)
        "23322009",  # Removal of coronary artery obstruction, percutaneous transluminal, multiple vessels (procedure)
        "24088005",  # Coronary artery atherectomy (procedure)
        "29843007",  # Coronary atherectomy by laser (procedure)
        "36969009",  # Placement of stent in coronary artery (procedure)
        "397193006",  # Percutaneous transluminal coronary angioplasty by rotoablation (procedure)
        "397194000",  # Percutaneous rotational coronary endarterectomy (procedure)
        "397431004",  # Percutaneous transluminal coronary angioplasty with rotoablation, single vessel (procedure)
        "410023005",  # Directional atherectomy (procedure)
        "415069007",  # Percutaneous atherectomy of artery (procedure)
        "415070008",  # Percutaneous coronary intervention (procedure)
        "428488008",  # Placement of stent in anterior descending branch of left coronary artery (procedure)
        "429499003",  # Placement of stent in circumflex branch of left coronary artery (procedure)
        "429639007",  # Percutaneous transluminal balloon angioplasty with insertion of stent into coronary artery (procedure)
        "429809004",  # Percutaneous transluminal angioplasty of coronary artery using fluoroscopic guidance with contrast (procedure)
        "431759005",  # Percutaneous transluminal atherectomy using fluoroscopic guidance (procedure)
        "6026008",  # Removal of coronary artery obstruction by percutaneous transluminal balloon with thrombolytic agent (procedure)
        "609152003",  # Percutaneous insertion of stent into aneurysm of coronary artery using fluoroscopic guidance with contrast (procedure)
        "609153008",  # Percutaneous insertion of drug eluting stent into coronary artery using fluoroscopic guidance with contrast (procedure)
        "609154002",  # Percutaneous transluminal insertion of metal stent into coronary artery using fluoroscopic guidance with contrast (procedure)
        "68466008",  # Removal of coronary artery obstruction by percutaneous transluminal balloon, single vessel (procedure)
        "698740005",  # Percutaneous transluminal atherectomy of coronary artery by rotary cutter using fluoroscopic guidance with contrast (procedure)
        "707828002",  # Percutaneous transluminal cutting balloon angioplasty of coronary artery (procedure)
        "709981002",  # Coronary artery atherectomy using fluoroscopic guidance with contrast (procedure)
        "75761004",  # Infusion of intra-arterial thrombolytic agent with percutaneous transluminal coronary angioplasty (procedure)
        "80762004",  # Infusion of intra-arterial thrombolytic agent with percutaneous transluminal coronary angioplasty, multiple vessels (procedure)
        "85053006",  # Percutaneous transluminal coronary angioplasty, multiple vessels (procedure)
        "91338001",  # Infusion of intra-arterial thrombolytic agent with percutaneous transluminal coronary angioplasty, single vessel (procedure)
    }

__exports__ = (
    "BloodTransfusion",
    "ConversionOfCardiacRhythm",
    "DeliveryProcedures",
    "Hysterectomy",
    "SevereMaternalMorbidityProcedures",
    "Tracheostomy",
    "Ventilation",
    "ApplicationOfGraduatedCompressionStockings",
    "ApplicationOfIntermittentPneumaticCompressionDevices",
    "ApplicationOfVenousFootPumps",
    "GeneralSurgery",
    "GeneralOrNeuraxialAnesthesia",
    "GynecologicalSurgery",
    "HipFractureSurgery",
    "HipReplacementSurgery",
    "IntracranialNeurosurgery",
    "KneeReplacementSurgery",
    "UrologicalSurgery",
    "ChildInfluenzaVaccineAdministered",
    "DtapVaccineAdministered",
    "HepatitisAVaccineAdministered",
    "HepatitisBVaccineAdministered",
    "HibVaccine3DoseScheduleAdministered",
    "HibVaccine4DoseScheduleAdministered",
    "InactivatedPolioVaccineIpvAdministered",
    "InfluenzaVirusLaivVaccineAdministered",
    "MeaslesMumpsAndRubellaMmrVaccineAdministered",
    "PneumococcalConjugateVaccineAdministered",
    "RotavirusVaccine2DoseScheduleAdministered",
    "RotavirusVaccine3DoseScheduleAdministered",
    "VaricellaZosterVaccineVzvAdministered",
    "AnesthesiaRequiringMonitoredCare",
    "GeneralAndNeuraxialAnesthesia",
    "HeadNeckAndThoracicSurgeriesWithHighRiskAirwayCompromise",
    "Intubation",
    "MajorSurgicalProcedure",
    "MechanicalVentilation",
    "NonInvasiveOxygenTherapyByNasalCannulaOrMask",
    "TracheostomyProcedures",
    "HysterectomyWithNoResidualCervix",
    "BilateralMastectomy",
    "UnilateralMastectomyLeft",
    "UnilateralMastectomyRight",
    "ProstateCancerTreatment",
    "SalvageTherapy",
    "Colonoscopy",
    "FlexibleSigmoidoscopy",
    "TotalColectomy",
    "CataractSurgery",
    "HeartTransplant",
    "LeftVentricularAssistDevicePlacement",
    "SubstanceUseDisorderLongActingMedicationAdministration",
    "SubstanceUseDisorderShortActingMedicationAdministration",
    "CardiacPacer",
    "CardiacSurgery",
    "ProceduresUsedToIndicateSexualActivity",
    "ChemotherapyAdministration",
    "DialysisServices",
    "KidneyTransplant",
    "CesareanBirth",
    "CabgSurgeries",
    "CabgOrPciProcedure",
    "CarotidIntervention",
    "Pci",
    "PartialArthroplastyOfHip",
    "PrimaryThaProcedure",
    "RemovalRevisionAndSupplementProceduresOfTheLowerBodyAndSpine",
    "CystectomyForUrologyCare",
    "AtrialAblation",
    "IntravenousOrIntraarterialThrombolyticTpaTherapy",
    "FluorideVarnishApplicationForChildren",
    "HighRiskProceduresForAki",
    "HospitalBasedDialysisServices",
    "EndotrachealIntubation",
    "IntracranialOrIntraspinalSurgery",
    "MechanicalCirculatoryAssistDevice",
    "PercutaneousCoronaryIntervention",
)
